import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription.ActiveType
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterHandler
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPinHandler
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.ConfigurationIssue
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.GenericIssue
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.ModuleConfiguration
import org.slf4j.LoggerFactory

class ActiveLevelCheck {
	def static logger = LoggerFactory.getLogger(ActiveLevelCheck.class)


	private static boolean isValid(boolean pinInv, boolean pinType, boolean responseLevel) {
		(responseLevel == pinType) ^ pinInv
		//(!pinType) ^ responseLevel ^ (!pinInv)
	}

	private static ActiveType getActualLevel(ActiveType activeType, IParameterHandler parameter) {
		if (parameter!=null)
			return ActiveType.fromString(parameter.getValue().orElseThrow())
		return activeType
	}

	static Boolean checkInternal(IConfigurationDocument document, ModuleConfiguration<?> module, IPortHandler<?> port, ActiveType activeType, IParameterHandler parameter, boolean report) {
		if (!port.isConnected()) {
			logger.warn("{} not connected", port.getAbsoluteConnectorName());
			return null; // do not say anything
		}

		def pin = port.singleConnectedPort
		def pinDesc = pin.connectorDescription

		if (!(pin instanceof IPinHandler)) {
			logger.info("{} is not connected to a pin, not validating (it's a {})", port.getAbsoluteConnectorName(), pin.class);
			return null;
		}


		if (pinDesc.activeType == ActiveType.Unknown) {
			logger.warn("Update the TargetDescription for {} to enable level validation!", document.target.targetDescription);
			return null;
		}

		def reqLevel = getActualLevel(activeType, parameter)
		boolean requiredType =  reqLevel == ActiveType.High
		boolean actualType = pinDesc.activeType == ActiveType.High
		boolean pinInv = pin.inverted

		boolean res = isValid(pinInv, actualType, requiredType);

		if (!res && report) {

			logger.error("Pin '{}' is Active-{}, inverted={} but {}'s level is {}", pin.getPortName(), pinDesc.activeType, pinInv, port.getAbsoluteConnectorName(), reqLevel);
		}
		return res;
	}

	private static Boolean checkLevelInternal(IConfigurationDocument document, ModuleConfiguration<?> module, IPortHandler<?> port, ActiveType activeType, IParameterHandler parameter, boolean report) {
		Boolean v = checkInternal(document, module, port, activeType, parameter, true)
		if (v == null)
			return null;

		if (!v) {

			if (report) {

				println "reporting issue"
				document.issueSink.report(new GenericIssue(module, "configured for another level than what is connected",
						"Level validation", ConfigurationIssue.IssueType.Error,
						{
							def conn = port.getSingleConnectedPort()
							"${module.prettyString}.$port.portName: ${getActualLevel(activeType, parameter)}, ${conn.portName}: ${conn.connectorDescription.activeType} ${conn.inverted?' (inverted)':''}".toString()
						},
						{
							Boolean v2 = checkInternal(document, module, port, activeType, parameter, false)
							return (v2==null || v2) ? ConfigurationIssue.Actuality.Resolved : ConfigurationIssue.Actuality.Current
						}))
			}
			return false;
		}

		logger.info("Level Config seems to be OK!");
		return true;
	}


	static Boolean checkStaticLevel(IConfigurationDocument document, ModuleConfiguration<?> module, IPortHandler<?> port, ActiveType activeType, boolean report) {
		return checkLevelInternal(document, module, port, activeType, null, report)
	}

	static Boolean checkConfigurableLevel(IConfigurationDocument document, IModuleConfiguration<?> module, IPortHandler<?> port, IParameterHandler parameter, boolean report) {
		return checkLevelInternal(document, module, port, null, parameter, report)
	}
}