/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument
import org.slf4j.Logger
import org.slf4j.LoggerFactory

class ClockUtils {

	private static final Logger log = LoggerFactory.getLogger("groovy.ClockUtils")

	static def clk_xilinx(IConfigurationDocument document){
		return document.findByType("clk_xilinx");
	}

	static def clk_simple(IConfigurationDocument document){
		return document.findByType("clk_simple");
	}

	static def clk_user(IConfigurationDocument document){
		return document.findByType("clk_user");
	}

	static def clk_xilinx_mmcm(IConfigurationDocument document){
		return document.findByType("clk_xilinx_mmcm");
	}


	static def noOfClocksFound(IConfigurationDocument document){
		return clk_xilinx(document).size() + clk_simple(document).size() + clk_user(document).size() + clk_xilinx_mmcm(document).size()
	}

	static def getSingleClockModule(IConfigurationDocument document){
		if (noOfClocksFound(document) == 1) {

			if (!clk_xilinx(document).empty) {
				return clk_xilinx(document)
			}

			if (!clk_simple(document).empty) {
				return clk_simple(document)
			}

			if (!clk_user(document).empty) {
				return clk_user(document)
			}

			if (!clk_xilinx_mmcm(document).empty) {
				return clk_xilinx_mmcm(document)
			}
		}

		else {
			log.error("Didn't find exactly one clock module")
		}

		return null
	}

	static def addClock(IConfigurationDocument document) {
		def clk
		if (document.getTarget().getToolchain().toString() == "ise" || document.getTarget().getToolchain().toString() == "xilinx" || document.getTarget().getToolchain().toString() == "vivado") {
			if (document.getTarget().getDeviceName().startsWith("XC7"))
				clk = document + 'clk_xilinx_mmcm'
			else
				clk = document + 'clk_xilinx'
		} else {
			log.error("No clock for toolchain " + document.getTarget().getToolchain() + " found")
		}
		return clk
	}

	static def addClockIfNoneExisting(IConfigurationDocument document) {
		if (noOfClocksFound(document) == 0) {
			return addClock(document)
		} else {
			log.info("Clocks already exist, no need to add.")
		}

		return getSingleClockModule(document)
	}



	static def findClockPort(IConfigurationDocument document) {

		if (noOfClocksFound(document) == 1) {

			if (!clk_xilinx(document).empty) {
				return clk_xilinx(document)[0].ports["clkfx"]
			}

			if (!clk_simple(document).empty) {
				return clk_simple(document)[0].ports["clkfx"]
			}

			if (!clk_user(document).empty) {
				return clk_user(document)[0].ports["clk_sdv"]
			}

			if (!clk_xilinx_mmcm(document).empty) {
				return clk_xilinx_mmcm(document)[0].ports["clk0"]
			}

		}
		return null;
	}

	static def findLockedPort(IConfigurationDocument document) {

		if (noOfClocksFound(document) == 1) {
			return getSingleClockModule(document)[0].ports["locked"];

		}
	}

}
