/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IIntegerParameterHandler
import org.slf4j.Logger
import org.slf4j.LoggerFactory

class CoreInterconnectUtils {

	private static final Logger log = LoggerFactory.getLogger("groovy.CoreInterconnectUtils")

	static def CORE_CONNECTOR_BUS_NAME = "connector"
	static def DMA_SIZE_PARAM = "DMA_SIZE";

	static boolean propagateDMASizeCoreConnector(IModuleConfiguration<?> module) {
		return propagateDMASize(module, CORE_CONNECTOR_BUS_NAME)
	}

	static boolean propagateDMASize(IModuleConfiguration<?> module, connectorName) {


		def paramHandler = module.getParameterHandler(IIntegerParameterHandler.class, module.findParameterDescription(DMA_SIZE_PARAM).get())
		def myDMASize = paramHandler.getValue().get().toInteger().intValue()

		if (module.getBus(connectorName).streamOppositeConnectors().findFirst().isPresent()) {
			IModuleConfiguration<?> oppositeConnector = module.getBus(connectorName).streamOppositeConnectors().findFirst().get().getSourceElement()
			def oppositeParamHandler = oppositeConnector.getParameterHandler(IIntegerParameterHandler.class, oppositeConnector.findParameterDescription(DMA_SIZE_PARAM).get())
			def oppositeDMASize = oppositeParamHandler.getValue().get().toInteger().intValue()
			if (myDMASize != oppositeDMASize) {
				oppositeParamHandler.setValue(paramHandler.getValue().get().toInteger().intValue())
			}
			return true
		} else {
			log.error("Can not set opposite parameter $DMA_SIZE_PARAM since core-connector-bus is unconnected")
		}
		return false
	}

	static boolean verifyDMASizeMatchesCoreConnector(IModuleConfiguration<?> module) {
		return verifyDMASizeMatches(module, CORE_CONNECTOR_BUS_NAME)
	}

	static boolean verifyDMASizeMatches(IModuleConfiguration<?> module, String connectorName) {
		def paramHandler = module.getParameterHandler(IIntegerParameterHandler.class, module.findParameterDescription(DMA_SIZE_PARAM).get())
		def myDMASize = paramHandler.getValue().get().toInteger().intValue()

		if (module.getBus(connectorName).streamOppositeConnectors().findFirst().isPresent()) {
			IModuleConfiguration<?> oppositeConnector = module.getBus(connectorName).streamOppositeConnectors().findFirst().get().getSourceElement()
			if ( oppositeConnector.findParameterDescription(DMA_SIZE_PARAM).isPresent() ) {
				def oppositeParamHandler = oppositeConnector.getParameterHandler(IIntegerParameterHandler.class, oppositeConnector.findParameterDescription(DMA_SIZE_PARAM).get())
				def oppositeDMASize = oppositeParamHandler.getValue().get().toInteger().intValue()
				if (myDMASize == oppositeDMASize) {
					return true
				}
			} else {
				log.warn("Can not validate opposite parameter $DMA_SIZE_PARAM since opponent doesnt have such a value.")
			}
		} else {
			log.error("Can not validate opposite parameter $DMA_SIZE_PARAM since core-connector-bus is unconnected")
		}
		return false
	}
}
