/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
// (c) fpga4fun.com & KNJN LLC 2013




// mostly copy-pasted from http://www.fpga4fun.com/HDMI.html
// added reset, pixelStop, frameStart


////////////////////////////////////////////////////////////////////////

`default_nettype none
`timescale 1ns / 1ps


module HDMIsimple
#
(
	//parameter NEW_FRAME_LEAD  = 0,
	//parameter NEW_ROW_LEAD    = 0,
	//parameter PIX_ENABLE_LEAD = 0,
	//parameter BOARDER_LEAD    = 0  by reducing the START_X and equivalently the END_X value you get the needed lead or leg for your moduls
	parameter SCALE_BIT_WIDTH	= 4,
	parameter DELAY_FOR_HARDWARE_SIGNALS = 1 // this module already include a delay of one, if you want more you have to use a number bigger than 0 -> (=1 -> delay of 2)
)
(
	input  wire pixclk,      // 25MHz
	input  wire clk_TMDS,    // 250MHz
	
	input  wire reset,
	input  wire [7:0] r,g,b,
	
	input  wire [9:0] START_X, // START_X =(pixelID-1) of [0:799] -> so if pixelID=0 then START_X=799, must include already the scale!
	input  wire [9:0] START_Y, // START_Y =(pixelID-1) of [0:524] -> so if pixelID=0 then START_X=524, must include already the scale!
	input  wire [9:0] END_X,   // same as above, START_X AND END_X must not be the same!, must include already the scale!
	input  wire [9:0] END_Y,   // same as above, must include already the scale!
	input  wire [SCALE_BIT_WIDTH-1:0] SCALE,   // 0)noScale, 1)2xScale, 2)3xScale	
		
	output reg   newFrame,	// arrives on clock before the meant pixel, so has a lead of 1
	output reg   newRow,	// arrives on clock before the meant pixel, so has a lead of 1
	output wire  newPixel,	// used for scale
	output wire  boarder,	// arrives on clock before the meant pixel, so has a lead of 1. is also 1 for the offscreen values
	
	output wire [2:0] TMDSp, TMDSn,
	output wire TMDSp_clock, TMDSn_clock 

);


////////////////////////////////////////////////////////////////////////
reg [9:0] CounterX, CounterY;
reg hSync, vSync, DrawArea;

reg [SCALE_BIT_WIDTH-1:0] rowCounter;
reg [SCALE_BIT_WIDTH-1:0] pixelCounter;

reg imgX, imgY;
assign boarder  = !(imgX & imgY);

reg borderLag;

// for SCALE==0, every pixel is a new one
// for SCALE==1, every second pixel is a new one -> 2times scale
// for sCALE==2, every third pixel isa new one ...
assign newPixel = (!boarder & (pixelCounter == SCALE));

always @(posedge pixclk)
begin

    if(reset)
    begin
     
		newFrame <= 0;
		newRow   <= 0;
		
		CounterX <= 0;
		CounterY <= 0;
		hSync    <= 0;
		vSync    <= 0;
		DrawArea <= 0;
		
		rowCounter   <= 0;
		pixelCounter <= 0;
		
		imgX     <= 0;
		imgY     <= 0;
		
		borderLag <= 0;
    
    end
    else
    begin
    
		borderLag <= boarder;
              
        DrawArea <= (CounterX<640) && (CounterY<480);        
        CounterX <= (CounterX==799) ? 0 : CounterX+1;       
        if(CounterX==799) CounterY <= (CounterY==524) ? 0 : CounterY+1;       
        hSync <= (CounterX>=656) && (CounterX<752);
        vSync <= (CounterY>=490) && (CounterY<492);

		// newRow and in display image)
		// border signal and new row rise at same tme
		// to determine the new row is of the display image and not
		// in the border, you look again at the borderLag
		if (CounterX == END_X && !borderLag)
		begin
			
			if(rowCounter != SCALE)
			begin
				rowCounter <= rowCounter +1;
				newRow		<= 0;
			end
			else
			begin
				rowCounter  <= 0;
				newRow 		<= 1;
			end
		end
		else
		begin
			newRow <= 0;
		end
		
	
		// newPixel
		if(! boarder)
		begin
		
			if(pixelCounter != SCALE)
				pixelCounter <= pixelCounter+1;
			else
				pixelCounter <= 0;
		
		end
		else
			pixelCounter <= 0;
	
	
		// border
		if(CounterX == START_X)
			imgX <= 1;
		else if (CounterX == END_X)
			imgX <= 0;
			
		if(CounterY == START_Y && CounterX == (799-DELAY_FOR_HARDWARE_SIGNALS))
			imgY <= 1;
		else if (CounterY == END_Y && CounterX == (799-DELAY_FOR_HARDWARE_SIGNALS))
			imgY <= 0;
   
   
    
		// newFrame
		if(CounterX == END_X && CounterY == END_Y)
			newFrame <= 1;
		else
			newFrame <= 0;
			
    
    end

end

//always @(posedge pixclk) DrawArea <= (CounterX<640) && (CounterY<480);

//always @(posedge pixclk) CounterX <= (CounterX==799) ? 0 : CounterX+1;
//always @(posedge pixclk) if(CounterX==799) CounterY <= (CounterY==524) ? 0 : CounterY+1;

//always @(posedge pixclk) hSync <= (CounterX>=656) && (CounterX<752);
//always @(posedge pixclk) vSync <= (CounterY>=490) && (CounterY<492);

//always @(posedge pixclk) DrawArea <= (CounterX<640) && (CounterY<480);

//always @(posedge pixclk) CounterX <= (CounterX==799) ? 0 : CounterX+1;
//always @(posedge pixclk) if(CounterX==799) CounterY <= (CounterY==524) ? 0 : CounterY+1;

//always @(posedge pixclk) hSync <= (CounterX>=656) && (CounterX<752);
//always @(posedge pixclk) vSync <= (CounterY>=490) && (CounterY<492);



////////////////
//wire [7:0] W = {8{CounterX[7:0]==CounterY[7:0]}};
//wire [7:0] A = {8{CounterX[7:5]==3'h2 && CounterY[7:5]==3'h2}};

//reg [7:0] red, green, blue;
//always @(posedge pixclk) red    <= r;//(CounterX[0] && CounterY[0]) ? 'd255 : (!CounterX[0] && !CounterY[0]) ? 'd255 : 'd0;//({CounterX[5:0] & {6{CounterY[4:3]==~CounterX[4:3]}}, 2'b00} | W) & ~A;
//always @(posedge pixclk) green  <= g;//(CounterX[0] && CounterY[0]) ? 'd255 : (!CounterX[0] && !CounterY[0]) ? 'd255 : 'd0;//(CounterX[7:0] & {8{CounterY[6]}} | W) & ~A;
//always @(posedge pixclk) blue   <= b;//(CounterX[0] && CounterY[0]) ? 'd255 : (!CounterX[0] && !CounterY[0]) ? 'd255 : 'd0;//CounterY[7:0] | W | A;

////////////////////////////////////////////////////////////////////////
wire [9:0] TMDS_red, TMDS_green, TMDS_blue;
//TMDS_encoder encode_R(.clk(pixclk), .VD(red  ), .CD(2'b00)        , .VDE(DrawArea), .TMDS(TMDS_red));
//TMDS_encoder encode_G(.clk(pixclk), .VD(green), .CD(2'b00)        , .VDE(DrawArea), .TMDS(TMDS_green));
//TMDS_encoder encode_B(.clk(pixclk), .VD(blue ), .CD({vSync,hSync}), .VDE(DrawArea), .TMDS(TMDS_blue));
TMDS_encoder encode_R(.clk(pixclk), .VD(r), .CD(2'b00)        , .VDE(DrawArea), .TMDS(TMDS_red));
TMDS_encoder encode_G(.clk(pixclk), .VD(g), .CD(2'b00)        , .VDE(DrawArea), .TMDS(TMDS_green));
TMDS_encoder encode_B(.clk(pixclk), .VD(b), .CD({vSync,hSync}), .VDE(DrawArea), .TMDS(TMDS_blue));

////////////////////////////////////////////////////////////////////////
//wire clk_TMDS, DCM_TMDS_CLKFX;  // 25MHz x 10 = 250MHz
//DCM_SP #(.CLKFX_MULTIPLY(10)) DCM_TMDS_inst(.CLKIN(pixclk), .CLKFX(DCM_TMDS_CLKFX), .RST(1'b0));
//BUFG BUFG_TMDSp(.I(DCM_TMDS_CLKFX), .O(clk_TMDS));

//wire clk_TMDS;
//clk_wiz_0 clockMultiply(.clk_in1(pixclk),.clk_out1(clk_TMDS), .reset(0));


////////////////////////////////////////////////////////////////////////
reg [3:0] TMDS_mod10=0;  // modulus 10 counter
reg [9:0] TMDS_shift_red=0, TMDS_shift_green=0, TMDS_shift_blue=0;
reg TMDS_shift_load=0;
always @(posedge clk_TMDS) TMDS_shift_load <= (TMDS_mod10==4'd9);

always @(posedge clk_TMDS)
begin
	TMDS_shift_red   <= TMDS_shift_load ? TMDS_red   : TMDS_shift_red  [9:1];
	TMDS_shift_green <= TMDS_shift_load ? TMDS_green : TMDS_shift_green[9:1];
	TMDS_shift_blue  <= TMDS_shift_load ? TMDS_blue  : TMDS_shift_blue [9:1];	
	TMDS_mod10 <= (TMDS_mod10==4'd9) ? 4'd0 : TMDS_mod10+4'd1;
end

OBUFDS OBUFDS_red  (.I(TMDS_shift_red  [0]), .O(TMDSp[2]), .OB(TMDSn[2]));
OBUFDS OBUFDS_green(.I(TMDS_shift_green[0]), .O(TMDSp[1]), .OB(TMDSn[1]));
OBUFDS OBUFDS_blue (.I(TMDS_shift_blue [0]), .O(TMDSp[0]), .OB(TMDSn[0]));
OBUFDS OBUFDS_clock(.I(pixclk), .O(TMDSp_clock), .OB(TMDSn_clock));
endmodule


////////////////////////////////////////////////////////////////////////
module TMDS_encoder
(
	input wire clk,
	input wire[7:0] VD,  // video data (red, green or blue)
	input wire[1:0] CD,  // control data
	input wire VDE,  // video data enable, to choose between CD (when VDE=0) and VD (when VDE=1)
	output reg [9:0] TMDS = 0
);

wire [3:0] Nb1s = VD[0] + VD[1] + VD[2] + VD[3] + VD[4] + VD[5] + VD[6] + VD[7];
wire XNOR = (Nb1s>4'd4) || (Nb1s==4'd4 && VD[0]==1'b0);
wire [8:0] q_m = {~XNOR, q_m[6:0] ^ VD[7:1] ^ {7{XNOR}}, VD[0]};

reg [3:0] balance_acc = 0;
wire [3:0] balance = q_m[0] + q_m[1] + q_m[2] + q_m[3] + q_m[4] + q_m[5] + q_m[6] + q_m[7] - 4'd4;
wire balance_sign_eq = (balance[3] == balance_acc[3]);
wire invert_q_m = (balance==0 || balance_acc==0) ? ~q_m[8] : balance_sign_eq;
wire [3:0] balance_acc_inc = balance - ({q_m[8] ^ ~balance_sign_eq} & ~(balance==0 || balance_acc==0));
wire [3:0] balance_acc_new = invert_q_m ? balance_acc-balance_acc_inc : balance_acc+balance_acc_inc;
wire [9:0] TMDS_data = {invert_q_m, q_m[8], q_m[7:0] ^ {8{invert_q_m}}};
wire [9:0] TMDS_code = CD[1] ? (CD[0] ? 10'b1010101011 : 10'b0101010100) : (CD[0] ? 10'b0010101011 : 10'b1101010100);

always @(posedge clk) TMDS <= VDE ? TMDS_data : TMDS_code;
always @(posedge clk) balance_acc <= VDE ? balance_acc_new : 4'h0;
endmodule


////////////////////////////////////////////////////////////////////////
