/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`default_nettype none
`timescale 1ns / 1ps

//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 09.11.2017 16:46:54
// Design Name: 
// Module Name: videoI420
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module VideoI420
#
(
    parameter dummy     = 0,
    parameter BASE_ADR  = 10'h0,
    parameter SCALE_BIT_WIDTH = 4
)
(
	// peri-bus
	input  wire reset,
	input  wire clk_peri,
	input  wire access_peri,
	input  wire wr_peri,
	input  wire [ 9:0] addr_peri,
	input  wire [17:0] do_peri,
	output reg  [17:0] di_peri,

	
	// second-data-bus
	output wire							clk2,
	output wire							reset2,
	output wire		[9:0]				mem_addr_block,
	output wire		[7:0]				mem_addr_high,
	input  wire		[17:0]				mem_di,
	output wire							mem_wr_low,
	output wire							mem_wr_high,
	output wire							mem_access,
	output wire		[17:0]				mem_do,
	input  wire							mem_busy,


	// hdmi signals
    input  wire pix_clk,       //  25MHz - HDMIsimple
	input  wire pix_clk3x,	   //  75MHz - reading memory, 3 times for yuv to rgb conversion
	input  wire tmds_clk,      // 250MHz - HDMIsimple
	
	output wire [2:0] TMDSp,   // - HDMIsimple
	output wire [2:0] TMDSn,   // - HDMIsimple
	output wire TMDSp_clk,     // - HDMIsimple
	output wire TMDSn_clk      // - HDMIsimple
	
);


//assign clk2 = pix_clk;
assign reset2 = 0;

// second-data-bus

assign reset2 = 0;

//assign mem_wr_low = 0;
//assign mem_wr_high = 0;
//assign mem_access = 1;
//assign mem_do = 0;

// ...
// =====================================================================



	
reg peri_inst_enable;
reg peri_inst_nextBufID;

wire pixl_inst_enable;
wire pixl_inst_nextBufID;

SHIFT_DFF #(.LATENCY(1)) sft_1 (.clk(pix_clk), .rst(reset), .dataIn(peri_inst_enable),    .dataOut(pixl_inst_enable));
SHIFT_DFF #(.LATENCY(1)) sft_2 (.clk(pix_clk), .rst(reset), .dataIn(peri_inst_nextBufID), .dataOut(pixl_inst_nextBufID));

reg [17:0] yJump;

reg [17:0] pY0;

reg [17:0] pY1;

reg [17:0] uvJump;
reg [17:0] pU0;
reg [17:0] pU1;
reg [17:0] pV0;
reg [17:0] pV1;

reg [9:0] startX;
reg [9:0] startY;
reg [9:0] endX;
reg [9:0] endY;	
reg [SCALE_BIT_WIDTH-1:0] scale;

reg pixl_out_curr_bufID;
reg pixl_out_next_bufID;

wire peri_out_curr_bufID;
wire peri_out_next_bufID;

SHIFT_DFF #(.LATENCY(1)) sft_3 (.clk(clk_peri), .rst(reset), .dataIn(pixl_out_curr_bufID), .dataOut(peri_out_curr_bufID));
SHIFT_DFF #(.LATENCY(1)) sft_4 (.clk(clk_peri), .rst(reset), .dataIn(pixl_inst_nextBufID), .dataOut(peri_out_next_bufID));


// peri block
// =====================================================================


// with 1 or 0 on address 0, you start or stop the videoplayer
// with 1 or 0 on address 1, you choose the second or first buffer(pY1,pU1,pV1, or pY0, pU0, pV0)
// 



always @(posedge clk_peri)
begin	
	
	if(reset)
	begin
	
		peri_inst_enable	<= 0;
		peri_inst_nextBufID	<= 0;
				
	end
	else
	begin
			
		if( access_peri && (addr_peri[9:4] == BASE_ADR[9:4]) )
		begin
			
			if(wr_peri)
			begin
			
				case(addr_peri[3:0])
				4'd0  : peri_inst_enable 	<= do_peri[0];
				4'd1  : peri_inst_nextBufID	<= do_peri[0];					
				4'd2  : yJump				<= do_peri;
				4'd3  : pY0					<= do_peri;
				4'd4  : pY1					<= do_peri;
				4'd5  : uvJump				<= do_peri;
				4'd6  : pU0					<= do_peri;
				4'd7  : pU1					<= do_peri;
				4'd8  : pV0					<= do_peri;
				4'd9  : pV1					<= do_peri;
				4'd10 : startX[9:0]			<= do_peri[9:0];
				4'd11 : startY[9:0]			<= do_peri[9:0];
				4'd12 : endX[9:0]			<= do_peri[9:0];
				4'd13 : endY[9:0]			<= do_peri[9:0];
				4'd14 : scale[SCALE_BIT_WIDTH-1:0]			<= do_peri[SCALE_BIT_WIDTH-1:0];
				default : $display("ERROR: VideoI420 defaultcase.");
				endcase
				
				di_peri <= 0;
				
			end
			else
			begin
			
				if(addr_peri[3:0] == 4'b0000)
					di_peri <= { 15'b0, peri_out_curr_bufID, peri_out_next_bufID, peri_inst_enable };
				else
					di_peri <= 0;
			
			end
		end
		else
		begin
			
			di_peri <= 0;
		
		end
	end
	
end

// =====================================================================

reg [1:0] state;
localparam IDLE = 2'd0;
localparam WAIT = 2'd1;
localparam SHOW = 2'd2;



reg [7:0] r, g, b;

reg [17:0] yAddr;	// always on mem_addr_high, mem_addr_block
reg [18:0] uAddr;	// use addition bit to realize the 4:2:0 ratio in vp8
reg [18:0] vAddr;

reg [17:0] yRowAddress;
reg [18:0] uRowAddress;
reg [18:0] vRowAddress;

//assign  mem_addr_high[7:0] = {1'b0,yAddr[17:11]};
//assign mem_addr_block[9:0] = yAddr[10:1];


wire newFrame;
wire newRow;
wire newPixel;

wire boarder;
wire boarderDelayed;
wire boarderDelayed2;

SHIFT_DFF #(.LATENCY(1)) sft_5 (.clk(pix_clk), .rst(reset), .dataIn(boarder), .dataOut(boarderDelayed));
SHIFT_DFF #(.LATENCY(2)) sft_11 (.clk(pix_clk), .rst(reset), .dataIn(boarder), .dataOut(boarderDelayed2));



//reg highByte;

wire [7:0] red, green, blue;

//always @(posedge pix_clk)


always @(*)
begin
	
	if(boarderDelayed2)
	begin
			// draw red border around the video
			r = 0;
			g = 0;
			b = 0;
	end
	else
	begin	
	
		if(pixl_inst_enable)
		begin
		
			// display the video
			r = red;
			g = green;
			b = blue;
		
		end
		else
		begin
			
			// fill content with gray if no video is displayed
			r = 100;
			g = 100;
			b = 100;
		
		end
	end

end


reg secRowTime;

always @(posedge pix_clk)
begin
	
	//highByte <= !yAddr[0];
	
	if(reset)
	begin
		
		state <= IDLE;		
		secRowTime <= 0;
	end
	else
	begin
		if(pixl_inst_enable)
		begin
			
			
			case(state)
			
			IDLE: 
					begin
					pixl_out_curr_bufID <= pixl_inst_nextBufID;	//? same line below
					
					if(newFrame)
					begin
					
						yAddr				<= pixl_inst_nextBufID ? pY1 : pY0;
						yRowAddress			<= pixl_inst_nextBufID ? pY1 : pY0;
						
						uAddr[18:1]			<= pixl_inst_nextBufID ? pU1[17:0] : pU0[17:0];
						uAddr[0]			<= 0;					
						uRowAddress[18:1]	<= pixl_inst_nextBufID ? pU1[17:0] : pU0[17:0];
						uRowAddress[0]		<= 0;
						
						vAddr[18:1]			<= pixl_inst_nextBufID ? pV1[17:0] : pV0[17:0];	
						vAddr[0]			<= 0;								
						vRowAddress[18:1]	<= pixl_inst_nextBufID ? pV1[17:0] : pV0[17:0];
						vRowAddress[0]		<= 0;
						
						secRowTime			<= 0;
						state				<= SHOW;
											
					end
					end
						
			SHOW:

					if(newFrame)
					begin
						pixl_out_curr_bufID <= pixl_inst_nextBufID;
						yAddr				<= pixl_inst_nextBufID ? pY1 : pY0;
						yRowAddress			<= pixl_inst_nextBufID ? pY1 : pY0;
						
						uAddr[18:1]			<= pixl_inst_nextBufID ? pU1[17:0] : pU0[17:0];
						uAddr[0]			<= 0;
						uRowAddress[18:1]	<= pixl_inst_nextBufID ? pU1[17:0] : pU0[17:0];
						uRowAddress[0]		<= 0;
						
						vAddr[18:1]			<= pixl_inst_nextBufID ? pV1[17:0] : pV0[17:0];	
						vAddr[0]			<= 0;	
						vRowAddress[18:1]	<= pixl_inst_nextBufID ? pV1[17:0] : pV0[17:0];
						vRowAddress[0]		<= 0;
						secRowTime			<= 0;
						
						
						
						state				<= SHOW;	// ignore newRow, as long you are in border section
					end
					else if(newRow)
					begin
						
						secRowTime			<= ! secRowTime;
						
						yAddr				<= yAddr + yJump;
						yRowAddress			<= yAddr + yJump;
						
						if(secRowTime)
						begin
							
							secRowTime <= !secRowTime;
							
							uAddr[18:1]			<= uAddr[18:1] + uvJump;
							uAddr[0]			<= 0;
							uRowAddress[18:1]	<= uAddr[18:1] + uvJump;
							uRowAddress[0]		<= 0;
						
							vAddr[18:1]			<= vAddr[18:1] + uvJump;
							vAddr[0]			<= 0;
							vRowAddress[18:1]	<= vAddr[18:1] + uvJump;
							vRowAddress[0]		<= 0;
						end
						else
						begin
							uAddr			<= uRowAddress;
							vAddr			<= vRowAddress;
						end
						
						
					end
					else if(newPixel)
					begin
						yAddr	<= yAddr+1;
						uAddr	<= uAddr+1;
						vAddr	<= vAddr+1;
						//state		<= SHOW;
					end
					else if(boarder)	// you got border but no newRow, so reset the addr to row address
					begin
						yAddr	<= yRowAddress;
						uAddr	<= uRowAddress;
						vAddr	<= vRowAddress;
						
						//state		<= SHOW;
					end
					else	// no border but also no newPixel, therefore keep the current pixel at the output
					begin
						yAddr	<= yAddr;
						uAddr	<= uAddr;
						vAddr	<= vAddr;
					end
			endcase
			
		end
		else
		begin
		
			state <= IDLE;
		
		end
	end
end


// I DO NOT KNOW, why you have to invert the highByte value, it seems, that in spartanMC the
// address higher byte is the lower byte on the select signal, FTS...
YUV2RGB yuv2rgb ( .clk(pix_clk3x), .reset(reset), .start(!boarder), .Yaddr({1'b0,yAddr[17:1]}), .YhighByte(!yAddr[0]), .Uaddr({1'b0,uAddr[18:2]}), .UhighByte(!uAddr[1]), .Vaddr({1'b0,vAddr[18:2]}), .VhighByte(!vAddr[1]),
					.r(red), .g(green), .b(blue),
				  .clk2(clk2), .mem_addr_block(mem_addr_block), .mem_addr_high(mem_addr_high), .mem_di(mem_di), .mem_wr_low(mem_wr_low), .mem_wr_high(mem_wr_high), .mem_access(mem_access), .mem_do(mem_do), .mem_busy(mem_busy));
	


wire [0:9] START_X, START_Y, END_X, END_Y;
wire [SCALE_BIT_WIDTH-1:0] SCALE;


SHIFT_DFF #(.LATENCY(1), .BIT_WIDTH(10) ) sft_6 (.clk(pix_clk), .rst(reset), .dataIn(startX), .dataOut(START_X));
SHIFT_DFF #(.LATENCY(1), .BIT_WIDTH(10) ) sft_7 (.clk(pix_clk), .rst(reset), .dataIn(startY), .dataOut(START_Y));
SHIFT_DFF #(.LATENCY(1), .BIT_WIDTH(10) ) sft_8 (.clk(pix_clk), .rst(reset), .dataIn(endX),   .dataOut(END_X));
SHIFT_DFF #(.LATENCY(1), .BIT_WIDTH(10) ) sft_9 (.clk(pix_clk), .rst(reset), .dataIn(endY),   .dataOut(END_Y));
SHIFT_DFF #(.LATENCY(1), .BIT_WIDTH(SCALE_BIT_WIDTH) ) sft_10 (.clk(pix_clk), .rst(reset), .dataIn(scale),   .dataOut(SCALE));


HDMIsimple hdmi ( .pixclk(pix_clk), .clk_TMDS(tmds_clk), .reset(reset), .r(r), .g(g), .b(b), .START_X(START_X), .START_Y(START_Y), .END_X(END_X), .END_Y(END_Y), .SCALE(SCALE), .newFrame(newFrame), .newRow(newRow), .newPixel(newPixel), .boarder(boarder), .TMDSp(TMDSp), .TMDSn(TMDSn), .TMDSp_clock(TMDSp_clk), .TMDSn_clock(TMDSn_clk));


endmodule









