/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ps
`default_nettype none

module YUV2RGB
/*#
(

)*/
(
	input  wire clk,			
	input  wire reset,
	
	input  wire start,
	input  wire [17:0] Yaddr,
	input  wire YhighByte,
	input  wire [17:0] Uaddr,
	input  wire UhighByte,
	input  wire [17:0] Vaddr,
	input  wire VhighByte,

	//output  reg enable,
	output  reg [7:0] r,
	output  reg [7:0] g,
	output  reg [7:0] b,
	
	// second-data-bus
	output wire							clk2,
	//output wire						reset2,
	output reg		[9:0]				mem_addr_block,
	output reg		[7:0]				mem_addr_high,
	input  wire		[17:0]				mem_di,
	output wire							mem_wr_low,
	output wire							mem_wr_high,
	output wire							mem_access,
	output wire		[17:0]				mem_do,
	input  wire							mem_busy
	
);

assign clk2 = clk;

assign mem_wr_low = 0;
assign mem_wr_high = 0;
assign mem_access = 1;
assign mem_do = 0;

//======================================================================

/*
assign  mem_addr_high[7:0]  = Yaddr[17:10];
assign  mem_addr_block[9:0] = Yaddr[9:0];

reg hb;

reg [2:0] c;

always@(posedge clk)
begin
	
	if(start || c==2)
		c <= 0;
	else
		c <= c+1;
		
		
	if(c==1)
	begin
	
	if(!YhighByte)
	begin
		r <= mem_di[7:0];
		g <= mem_di[7:0];
		b <= mem_di[7:0];
	end
	else
	begin
		r <= mem_di[16:9];
		g <= mem_di[16:9];
		b <= mem_di[16:9];
	end
	
	end
	else
end
*/

reg [1:0] readState;
parameter sY = 2'd0;
parameter sU = 2'd1;
parameter sV = 2'd2;

reg [7:0] vYin;
reg [7:0] vUin;
reg [7:0] vVin;


reg VhighByteSaved; // save this value, as it would will be needed in 4.clock, but would be overwritten by the input at this time


always@(*)
begin

	case(readState)
	sY :
			begin
			mem_addr_high[7:0]		= Yaddr[17:10];
			mem_addr_block[9:0]		= Yaddr[9:0];			
			vVin[7:0]				= VhighByteSaved ? mem_di[16:9] : mem_di[7:0];
			end
			
	sU :	
			begin
			mem_addr_high[7:0]		= Uaddr[17:10];
			mem_addr_block[9:0]		= Uaddr[9:0];
			vYin[7:0] 				= YhighByte ? mem_di[16:9] : mem_di[7:0];
			end
			
	sV :	
			begin
			mem_addr_high[7:0]		= Vaddr[17:10];
			mem_addr_block[9:0]		= Vaddr[9:0];
			vUin[7:0] 				= UhighByte ? mem_di[16:9] : mem_di[7:0];
			end
			
			
	default : $display("ERROR: YUV2RGB defaultcase.");
	endcase

end

reg secRound;

always@(posedge clk)
begin

	if(reset)
	begin
		readState <= sY;
		secRound  <= 0;
	end
	else
	begin

	VhighByteSaved <= VhighByte;


	case(readState)
	sY :	if(start || secRound)		readState <= sU; 
	sU :								readState <= sV; 
	sV : begin
			readState <= sY;
			secRound <= ! secRound;
		 end

	default : $display("ERROR: YUV2RGB defaultcase.");
	endcase

	end

end




// calculating the output
//======================================================================

// floating point calculations

// R = 1.164(Y-16)                   + 1.596 * (V-128);
// G = 1.164(Y-16) - 0.391 * (U-128) - 0.813 * (V-128); 
// B = 1.164(Y-16) + 2.018 * (U-128);

// to fixpoint
// taken from https://stackoverflow.com/questions/17088118/decode-from-vp8-video-frame-to-rgb




// R = clamp8((298*(Y-16)                 + 409 * (V-128) + 128) >> 8);
// G = clamp8((298*(Y-16) - 100 * (U-128) - 208 * (V-128) + 128) >> 8); 
// B = clamp8((298*(Y-16) + 516 * (U-128)                 + 128) >> 8);




// max value = 298*(255-16)+409*(255-128)+128			= 123293
// max value = 298*(255-16)-100*(0-128)-208*(0-128)+128 = 110774
// max value = 298*(255-16)+516*(255-128)+128			= 136882 -> 0b100001011010110010 (2^18)

// min value = 298*(0-16)+409*(0-128)+128			      = -56992
// min value = 298*(0-16)-100*(255-128)-208*(255-128)+128 = -43756
// min value = 298*(0-16)+516*(0-128)+128			      = -70688

// => you need a signed value with total 19 bit (incl. signed bit)!

//reg signed [17:0] p1,p2,
reg signed [17:0] p1,p2,p3,p4,p5,p6;

//reg signed [18:0] p1,p2,p3,p4,p5,p6;

wire signed [18:0] red, green, blue;



assign red   = (p3+p6)>>>8;
assign green = (p2+p5)>>>8;
assign blue  = (p6+p4)>>>8;
//assign blue  = (p4)>>>8;

always@(posedge clk)
begin

	
	case(readState)
	sY :
		begin
			// 3) here you get the sV from memory
			p1 <= p1+128;		// 298*(Y-16) + 128; (128 is for rounding, will be shifted by 8 so its equal to 0.5)
			p3 <= (vVin-128);	// (V-128) 
			p2 <= p2*-100;		// -100*(U-128)
			p4 <= p2* 516;		//  516*(U-128)
			
		end
	
	
	sU :
		begin
			// 1) here you get the sY from memory
			p1 <= (vYin - 16);	// (Y-16)
			// 4) 
			//p4 <= (p1+p4); // equal to blue
			p2 <= (p1+p2); // equal to first part of green
			
			p3 <= p3* 409;
			p5 <= p3*-208;
			p6 <= p1;
		end
		
		
	sV :
		begin
			// 2)  here you get the sU from memory
			p1 <= p1 *298;		// 298*(Y-16)
			p2 <= (vUin-128);	// (U-128)
			
			// 5)
			//r[7:0] <= (  red < 0) ? 0 : ( (red   > 255) ? 255 : red);
			//g[7:0] <= (green < 0) ? 0 : ( (green > 255) ? 255 : green);
			//b[7:0] <= (blue  < 0) ? 0 : ( (blue  > 255) ? 255 : blue);
			
			r[7:0] <= (  red[18]   ) ? {8{1'b0}} : ( red   > 255 ? {8{1'b1}} : red);
			g[7:0] <= (  green[18] ) ? {8{1'b0}} : ( green > 255 ? {8{1'b1}} : green);
			b[7:0] <= (  blue[18]  ) ? {8{1'b0}} : ( blue  > 255 ? {8{1'b1}} : blue);
		end
		
	default : $display("ERROR: YUV2RGB defaultcase.");
	endcase

end













endmodule
