/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`timescale	1ns / 10ps

//////////////////////////////////////////////////////////////////////////////////
//Module:		AXI-Master (mirror ctrl registers)
//Author:		Tobias Schladt
//Version:		23.11.2015
//Description:	Registers to mirror content of the DMA memory's control registers.
//////////////////////////////////////////////////////////////////////////////////

module mirror_ctrl_regs(
	input wire clk,
	input wire reset,
	//interface to arbiter
	output reg [15:0] valid_reg_0,
	output reg [15:0] valid_reg_1,
	output reg [15:0] valid_reg_2,
	output reg [15:0] valid_reg_3,
	input wire [3:0] transaction_finished,
	//interface to SpartanMC peripheral bus
	input wire [17:0] do_peri,
	input wire [9:0] addr_peri,
	input wire we_dma
	);
	
	parameter SPMC_W_CTRL_ADDR = 0;	//SpMC address of write control register 0
	parameter SPMC_R_CTRL_ADDR = 2;	//SpMC address of read control register 0
	parameter VALID_BIT_POS		= 10;	//position of valid bit
	
	//detect set of valid bit	
	wire [3:0] set_valid;
	assign set_valid[0] = (we_dma && (addr_peri == SPMC_W_CTRL_ADDR) && (do_peri[VALID_BIT_POS] == 1));
	assign set_valid[1] = (we_dma && (addr_peri == (SPMC_W_CTRL_ADDR+1)) && (do_peri[VALID_BIT_POS] == 1));
	assign set_valid[2] = (we_dma && (addr_peri == SPMC_R_CTRL_ADDR) && (do_peri[VALID_BIT_POS] == 1));
	assign set_valid[3] = (we_dma && (addr_peri == (SPMC_R_CTRL_ADDR+1)) && (do_peri[VALID_BIT_POS] == 1));
		
	//logic to set and reset mirror control registers
	always@(posedge clk) begin
		if(transaction_finished[0] || reset) valid_reg_0 <= 0;
		else if(set_valid[0]) valid_reg_0 <= do_peri[15:0];
	end
	
	always@(posedge clk) begin
		if(transaction_finished[1] || reset) valid_reg_1 <= 0;
		else if(set_valid[1]) valid_reg_1 <= do_peri[15:0];
	end
	
	always@(posedge clk) begin
		if(transaction_finished[2] || reset) valid_reg_2 <= 0;
		else if(set_valid[2]) valid_reg_2 <= do_peri[15:0];
	end
	
	always@(posedge clk) begin
		if(transaction_finished[3] || reset) valid_reg_3 <= 0;
		else if(set_valid[3]) valid_reg_3 <= do_peri[15:0];
	end
	
endmodule
