/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`timescale 1ns / 10ps

module coherence #(
	parameter RESET_LEVEL = "LOW_ACTIVE",
	parameter N = 2 // number of connected caches
) (
	input  wire                clk,
	input  wire                reset,
	input  wire                last_i,
	output wire                last_o,
	input  wire [N-1:0]        request,
	output wire [N-1:0]        grant,

	input  wire [17:0]         addr_i,
	output wire [17:0]         addr_o,
	input  wire                addr_valid_i,
	output wire                addr_valid_o,
	input  wire [17:0]         data_i,
	output wire [17:0]         data_o,
	input  wire                hit_i,
	output wire                hit_o,
	input  wire [1:0]          type_i, // read or write request?
	output wire [1:0]          type_o,
	input  wire                hold_i,
	output wire                hold_o
);

wire reset_active_high;

generate
if (RESET_LEVEL == "LOW_ACTIVE") begin
	assign reset_active_high = ~reset;
end else begin
	assign reset_active_high = reset;
end

if (N > 1) begin

assign last_o = last_i;
assign addr_o = addr_i;
assign addr_valid_o = addr_valid_i;
assign data_o = data_i;
assign hit_o  = hit_i;
assign type_o = type_i;
assign hold_o = hold_i;

concentrator_arbiter #(
	.NUMBER_OF_SLAVES(N)
) arbiter (
	.clk(clk),
	.reset(reset_active_high),
	.last_value_write_sel(last_i),
	.req(request),
	.grant(grant)
);

end else begin

assign last_o = 1'b0;
assign addr_o = 18'b0;
assign addr_valid_o = 1'b0;
assign data_o = 18'b0;
assign hit_o  = 1'b0;
assign type_o = 2'b0;

assign grant = 1'b1;

end
endgenerate

endmodule
