/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`timescale 1ns / 10ps

module dp_ram #(
	parameter WIDTH = 18,
	parameter ADDRWIDTH = 4,
	parameter SIZE = 4,
	parameter NUM_COL = 1,
	parameter COL_WIDTH = WIDTH / NUM_COL
) (
	input wire                 clk,
	input wire                 enA,
	input wire                 enB,
	input wire [NUM_COL-1:0]   weA,
	input wire                 weB,
	input wire [ADDRWIDTH-1:0] addrA,
	input wire [ADDRWIDTH-1:0] addrB,
	input wire [WIDTH-1:0]     diA,
	input wire [WIDTH-1:0]     diB,
	output reg [WIDTH-1:0]     doA,
	output reg [WIDTH-1:0]     doB
);

reg [WIDTH-1:0] ram [SIZE-1:0];

generate
integer ram_index;
initial begin
	for (ram_index = 0; ram_index < SIZE; ram_index = ram_index+1)
		ram[ram_index] = {(WIDTH){1'b0}};
end
endgenerate

generate
genvar i;
// Port A: write-first, optional with byte-wide write
if (NUM_COL == 1) begin // without byte wide write
	always @(posedge clk) begin
		if(weA) begin
			ram[addrA] <= diA;
			doA <= diA;
		end else
			doA <= ram[addrA];
	end
end else begin // byte wide write
	for (i = 0; i < NUM_COL; i = i+1) begin : portA
		always @(posedge clk) begin
			if(enA)
				if(weA[i]) begin
					ram[addrA][(i+1)*COL_WIDTH-1:i*COL_WIDTH] <= diA[(i+1)*COL_WIDTH-1:i*COL_WIDTH];
					doA[(i+1)*COL_WIDTH-1:i*COL_WIDTH] <= diA[(i+1)*COL_WIDTH-1:i*COL_WIDTH];
				end else begin
					doA[(i+1)*COL_WIDTH-1:i*COL_WIDTH] <= ram[addrA][(i+1)*COL_WIDTH-1:i*COL_WIDTH];
				end
		end
	end
end
endgenerate

always @(posedge clk) begin
	// Port B: read first
	if(enB) begin
		if(weB) begin
			ram[addrB] <= diB;
		end
		doB <= ram[addrB];
	end
end

endmodule
