/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


////////////////////////////////////////////////////////////////////////////////
// Create Date:   18:33:45 06/07/2006
// Design Name:   CAN_btl
// Module Name:   can_testbench.v
// Project Name:  CAN-Controller
////////////////////////////////////////////////////////////////////////////////

`default_nettype none
`include "dma/can/can_testbench_defines.vh" 

module can_testbench;
    
reg						clk;
reg						reset;

//Ausgang zum CAN-Bus
reg						can_rx;
reg					   can_tx;

//an SpartanMC ir-Controller
wire					   exception_ir;
wire         				 tx_successful_ir;
wire					   rx_successful_ir;
wire  [18*15-1:0]	data_out;	//Datenbus

//an/von DMA-Blockram
wire		[9:0]		     dma_adr_A;
wire					   dma_we_A;
wire		[17:0]	     dma_dataI_A;
//reg		[17:0]	     dma_dataO_A; 

//von SpartanMC
reg						wr_peri;		//Write Enable vom SpartanMC
reg						activ_peri; //aktiviere Peripherieregister

reg		[17:0]				data_in;		//Datenbus
reg		[9:0]		     adr;		   //Adressbus

reg               transmitting;
reg               start_TB;

parameter CYCLE = 30;
parameter TBIT = 120;

	
// Intanziiere Unit Under Test (UUT) CAN_top
	can_top uut (
	   .can_clk          (clk               ),
      .reset             (reset             ),

      //Ausgang zum CAN-Bus
      .can_tx            (can_tx            ),
      .can_rx            (can_rx            ),

      //an SpartanMC ir-Controller
      .exception_ir      (exception_ir      ),
      .tx_successful_ir  (tx_successful_ir  ),
      .rx_successful_ir  (rx_successful_ir  ),


      //an/von DMA-Blockram
      .addr_high         (dma_adr_A         ),
      .store_access          (dma_we_A          ),
//      .dma_dataI_A       (dma_dataI_A       ),
//      .dma_dataO_A       (dma_dataO_A       ), 

      //von SpartanMC
      .wr_peri           (wr_peri           ),//Write Enable vom SpartanMC
      .access_peri       (activ_peri        ),//aktiviere Peripherieregister

      .di_peri           ({data_in,dma_dataI_A}          ),//Datenbus
      .do_peri           (data_out          ),//Datenbus
      .addr_peri         (adr               )	//Adressbus
	);

   //Rückkoppeln von tx
   always @ (posedge clk)
   begin
   if(transmitting) can_rx<=can_tx;
   end	  

	//Takt
	initial clk = 1'b0;
	always #(CYCLE/2) clk = ~clk;
	
	//Reset
	initial 
	begin
	    reset = 1'b0; #(1*CYCLE)
	    reset = 1'b1; #(3*CYCLE)
	    reset = 1'b0;
	end
	
	//reg [17:0] bustiming;
	
	initial 
	begin 
	//bustiming <= {`CAN_BRP,`CAN_SJW,`CAN_TIME_SEGMENT1,`CAN_TIME_SEGMENT2,`CAN_SAMPLE_MODE,0,0};	
	#200 set_peri_reg(10'd0,18'b0001_1000_0000_0000_00);//workreg irq_en, config_mode
	#200 set_peri_reg(10'd1,18'b1000_0000_0000_0000_00);//config_reg ext_mode
	#200 set_peri_reg(10'd2,18'b0001_0011_0110_0110_00);//bustiming irq_en, config_mode
	#200 set_peri_reg(10'd6,18'b1111_1111_0100_1101_01);//ac 1
	#200 set_peri_reg(10'd7,18'b1111_1110_0001_1111_11);//am 1
	#200 set_peri_reg(10'd8,18'b0000_0000_0000_0000_00);//ac 2
	#200 set_peri_reg(10'd9,18'b1111_1111_1111_1111_11);//am 2
	#200 set_peri_reg(10'd9,18'b0000_0000_0000_0000_00);//acf adr_select
	#200 set_peri_reg(10'd6,18'b1010_1010_1010_1111_10);//ac 1
	#200 set_peri_reg(10'd7,18'b1111_1111_1111_1111_10);//am 1
	#200 set_peri_reg(10'd8,18'b0000_0000_0000_0000_00);//ac 2
	#200 set_peri_reg(10'd9,18'b1111_1111_1111_1111_00);//am 2
	#200 set_peri_reg(10'd9,18'b1000_1000_0000_0000_00);//acf adr_select
	//Startsignal für Testbench
	#200 start_TB = 1;
	   
   end
	
	
//-HilfsTasks---------------------------------------------------------------------	
task set_peri_reg;
  input [9:0]    addr;
  input [17:0]   data;
  
  begin
     activ_peri = 1;
     #100
     adr = addr;
     #100
     data_in = data;
     #100
     wr_peri = 1;
     #1000
     wr_peri = 0;
     activ_peri = 0;
  end
 endtask
//------------------------------------------------------------------------------- 

//-HilfsTasks---------------------------------------------------------------------	
task set_bus_bit;
  input bit;
  //integer cnt;
  
  begin
    #1 can_rx=bit;
    repeat ((`CAN_TIME_SEGMENT1 + `CAN_TIME_SEGMENT2 + 3)*((`CAN_BRP+1)*2)) @ (posedge clk);
  end
endtask
//------------------------------------------------------------------------------- 
	
/// testbench.main --------------------------------------------------------------
initial
begin
  wait(start_TB);

  #10;
  repeat (1000) @ (posedge clk);
  
  // Ausschalten reset mode
  //reset_mode = ~`CAN_RESET_MODE;
  
  //repeat (BRP) @ (posedge clk);   // At least BRP clocks needed before bus goes to dominant level. Otherwise 1 quant difference is possible
                                  // This difference is resynchronized later.

  // Nach beenden des Reset Mode senden von 11x Bus-Free
  repeat (11) set_bus_bit(1);

//test_listen_only_mode  // Test AUS
//test_reset_mode        // Test AUS
//overload_test          // Test AUS
//error_passiv_test      // Test AUS      
//bus_off_test           // Test AUS
//stuff_error            // Test AUS
//crc_error              // Test AUS
//ack_error              // Test AUS
//form_error             // Test AUS
//bit_error              // Test AUS
//send_extendedframe     // Test AUS
//empfang_extendedframe  // Test AUS
//empfang_basicframe;    // Test AUS
//sende_basicframe;      // Test AN
//test_basic_mode        // Test AUS

  $display("CAN Testbench Ende !");
  $stop;
end
//-------------------------------------------------------------------------------
	
/*task empfang_basicframe;    // Testbench sendet eine Basicframe
begin
	begin
	   tx_ide    = 0; //Basicframe
	   tx_rtr    = 0; //Dataframe
      tx_dlc    = 4'b0100;//4 Byte lang
	   basicID   = 11'b0101_0101_010;
	   extID     = 18'b0000_0000_0000_0000_00;
	   tx_data_0 =  8'b0000_1111;
	   tx_data_1 =  8'b0101_0101;
	   tx_data_2 =  8'b0000_1111;
	   tx_data_3 =  8'b0101_0101;
	   tx_data_4 =  8'b0000_0000;
	   tx_data_5 =  8'b0000_0000;
	   tx_data_6 =  8'b0000_0000;
	   tx_data_7 =  8'b0000_0000;
	  
	   //tx = 1;
   end  
	
	begin

   //tx_bypassed = 0;    // When this signal is on, tx is not looped back to the rx.
    
    #931;
    repeat (1)
    begin
       set_bus_bit(0);  // SOF
       set_bus_bit(0);  // ID
       set_bus_bit(1);  // ID
       set_bus_bit(0);  // ID
       set_bus_bit(1);  // ID
       set_bus_bit(0);  // ID
       set_bus_bit(1);  // ID
       set_bus_bit(0);  // ID
       set_bus_bit(1);  // ID
       set_bus_bit(0);  // ID
       set_bus_bit(1);  // ID
       set_bus_bit(0);  // ID
       set_bus_bit(1);  // RTR
       set_bus_bit(0);  // IDE
       set_bus_bit(0);  // r0
       set_bus_bit(0);  // DLC
       set_bus_bit(1);  // DLC
       set_bus_bit(1);  // DLC
       set_bus_bit(1);  // DLC
       
       repeat (11) 
       begin
         set_bus_bit(0); //MSG  
         set_bus_bit(0); //MSG  
         set_bus_bit(0); //MSG  
         set_bus_bit(0); //MSG
         set_bus_bit(0); //MSG
         set_bus_bit(1); //MSG stuff
       end
       set_bus_bit(1); //MSG
       
       set_bus_bit(1);  // CRC 1
       set_bus_bit(0);  // CRC
       set_bus_bit(0);  // CRC
       set_bus_bit(0);  // CRC 
       set_bus_bit(1);  // CRC 
       set_bus_bit(1);  // CRC
       set_bus_bit(0);  // CRC
       set_bus_bit(0);  // CRC
       set_bus_bit(1);  // CRC  
       set_bus_bit(1);  // CRC 
       set_bus_bit(0);  // CRC
       set_bus_bit(1);  // CRC
       set_bus_bit(1);  // CRC
       set_bus_bit(1);  // CRC 
       set_bus_bit(1);  // CRC 15
       set_bus_bit(1);  // CRC DELIM
       set_bus_bit(0);  // ACK
       set_bus_bit(1);  // ACK DELIM
       set_bus_bit(1);  // EOF
       set_bus_bit(1);  // EOF
       set_bus_bit(1);  // EOF
       set_bus_bit(1);  // EOF
       set_bus_bit(1);  // EOF
       set_bus_bit(1);  // EOF
       set_bus_bit(1);  // EOF
       set_bus_bit(1);  // INTER
       set_bus_bit(1);  // INTER
       set_bus_bit(1);  // INTER
       #400;
       end // repeat
   end    
end
endtask   //  empfang_basicframe---------------------------------------------------------------------

task sende_basicframe;    // Controller sendet eine Basicframe mit vorhergehenden empfang + Abitrierungsverlust 
begin
	begin
	    //Senden vorbereiten
	   tx_ide    = 0; //Basicframe
	   tx_rtr    = 0; //Dataframe
      tx_dlc    = 4'b0100;//4 Byte lang
	   basicID   = 11'b0101_0101_011;
	   extID     = 18'b0000_0000_0000_0000_00;
	   tx_data_0 =  8'b0000_1111;
	   tx_data_1 =  8'b0101_0101;
	   tx_data_2 =  8'b0000_1111;
	   tx_data_3 =  8'b0101_0101;
	   tx_data_4 =  8'b0000_0000;
	   tx_data_5 =  8'b0000_0000;
	   tx_data_6 =  8'b0000_0000;
	   tx_data_7 =  8'b0000_0000;
	   tx_request = 1;
	   #5000;
	    
	   //Kommando Register
	   overload_request = 0;   
	end
	
	begin
    fork
      begin
        #931;
        repeat (1)
        begin
          transmitting = 1;
          set_bus_bit(0);  // SOF
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          transmitting = 0;
          set_bus_bit(0);  // ID arbi. verlust

          set_bus_bit(1);  // RTR
          set_bus_bit(0);  // IDE
          set_bus_bit(0);  // r0
          set_bus_bit(0);  // DLC
          set_bus_bit(1);  // DLC
          set_bus_bit(1);  // DLC
          set_bus_bit(1);  // DLC
          
          repeat (11) 
          begin
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG
            set_bus_bit(0); //MSG
            set_bus_bit(1); //MSG stuff
          end
          set_bus_bit(1); //MSG
        
          set_bus_bit(1);  // CRC 1
          set_bus_bit(0);  // CRC
          set_bus_bit(0);  // CRC
          set_bus_bit(0);  // CRC 
          set_bus_bit(1);  // CRC 
          set_bus_bit(1);  // CRC
          set_bus_bit(0);  // CRC
          set_bus_bit(0);  // CRC
          set_bus_bit(1);  // CRC  
          set_bus_bit(1);  // CRC 
          set_bus_bit(0);  // CRC
          set_bus_bit(1);  // CRC
          set_bus_bit(1);  // CRC
          set_bus_bit(1);  // CRC 
          set_bus_bit(1);  // CRC 15
          set_bus_bit(1);  // CRC DELIM
          set_bus_bit(0);  // ACK
          set_bus_bit(1);  // ACK DELIM
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          transmitting =1;
          set_bus_bit(1);  // INTER
          set_bus_bit(1);  // INTER
          set_bus_bit(1);  // INTER     
        #400;
        end // repeat
      end 
      
      begin //gleichzeitiges Senden (künstliches verlängern der Testzeit) 
         #1000000;
      end  
    join
    
   end
end
endtask   //  senden_basicframe------------------------------------------------------------------------------------

task test_listen_only_mode;  // setzen des Controllers in Listen-Only-Mode
begin
end
endtask //  test_listen_only_mode----------------------------------------------------------------------------------

task test_reset_mode;  // setzen des Controllers in Reset-Mode
begin
end
endtask  //  test_reset_mode---------------------------------------------------------------------------------------

task overload_test;  // Erzeugen eines Overloadframes
begin
end
endtask //  overload_test------------------------------------------------------------------------------------------

task error_passiv_test;    //Setzt den Controller nach 128 Error in Eror-Passiv-Mode 
begin
end
endtask //  error_passiv_test--------------------------------------------------------------------------------------
     
task bus_off_test;   //Setzt den Controller nach 256 Error in Bus-OFF-Mode
begin
end
endtask//   bus_off_test-------------------------------------------------------------------------------------------

task stuff_error;   // Erzeugt ein Stuffing Error
begin
    begin
	    //Senden vorbereiten
	   tx_ide    = 0; //Basicframe
	   tx_rtr    = 0; //Dataframe
      tx_dlc    = 4'b0100;//4 Byte lang
	   basicID   = 11'b0101_0101_011;
	   extID     = 18'b0000_0000_0000_0000_00;
	   tx_data_0 =  8'b0000_1111;
	   tx_data_1 =  8'b0101_0101;
	   tx_data_2 =  8'b0000_1111;
	   tx_data_3 =  8'b0101_0101;
	   tx_data_4 =  8'b0000_0000;
	   tx_data_5 =  8'b0000_0000;
	   tx_data_6 =  8'b0000_0000;
	   tx_data_7 =  8'b0000_0000;
	   tx_request = 0;
	   #5000;
	    
	   //Kommando Register
	   overload_request = 0;   
	end
	
	begin
        #931;
        repeat (1)
        begin
          set_bus_bit(0);  // SOF
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID arbi. verlust

          set_bus_bit(1);  // RTR
          set_bus_bit(0);  // IDE
          set_bus_bit(0);  // r0
          set_bus_bit(0);  // DLC
          set_bus_bit(1);  // DLC
          set_bus_bit(1);  // DLC
          set_bus_bit(1);  // DLC
          
          repeat (11) 
          begin
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG
            set_bus_bit(0); //MSG Stuff Error
          end
        end // repeat
      end  
end
endtask //   stuff_error-------------------------------------------------------------------------------------------

task crc_error;   // Erzeugt ein CRC Error
begin
    begin
	    //Senden vorbereiten
	   tx_ide    = 0; //Basicframe
	   tx_rtr    = 0; //Dataframe
      tx_dlc    = 4'b0100;//4 Byte lang
	   basicID   = 11'b0101_0101_011;
	   extID     = 18'b0000_0000_0000_0000_00;
	   tx_data_0 =  8'b0000_1111;
	   tx_data_1 =  8'b0101_0101;
	   tx_data_2 =  8'b0000_1111;
	   tx_data_3 =  8'b0101_0101;
	   tx_data_4 =  8'b0000_0000;
	   tx_data_5 =  8'b0000_0000;
	   tx_data_6 =  8'b0000_0000;
	   tx_data_7 =  8'b0000_0000;
	   tx_request = 1;
	   #5000;
	    
	   //Kommando Register
	   overload_request = 0;   
	end
	
	begin
        #931;
        repeat (1)
        begin
          set_bus_bit(0);  // SOF
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID arbi. verlust

          set_bus_bit(1);  // RTR
          set_bus_bit(0);  // IDE
          set_bus_bit(0);  // r0
          set_bus_bit(0);  // DLC
          set_bus_bit(1);  // DLC
          set_bus_bit(1);  // DLC
          set_bus_bit(1);  // DLC
          
          repeat (11) 
          begin
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG
            set_bus_bit(0); //MSG
            set_bus_bit(1); //MSG stuff
          end
          set_bus_bit(1); //MSG
        
          set_bus_bit(1);  // CRC 1
          set_bus_bit(0);  // CRC
          set_bus_bit(0);  // CRC
          set_bus_bit(0);  // CRC 
          set_bus_bit(1);  // CRC 
          set_bus_bit(1);  // CRC
          set_bus_bit(0);  // CRC
          set_bus_bit(1);  // CRC CRC Error
          set_bus_bit(1);  // CRC  
          set_bus_bit(1);  // CRC 
          set_bus_bit(0);  // CRC
          set_bus_bit(1);  // CRC
          set_bus_bit(1);  // CRC
          set_bus_bit(1);  // CRC 
          set_bus_bit(1);  // CRC 15
          set_bus_bit(1);  // CRC DELIM
          set_bus_bit(0);  // ACK
          set_bus_bit(1);  // ACK DELIM
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // INTER
          set_bus_bit(1);  // INTER
          set_bus_bit(1);  // INTER     
        #400;
        end // repeat
   end
end
endtask //   crc_error-------------------------------------------------------------------------------------------

task ack_error;   // Erzeugt ein Acknowledge Error
begin
	    //Senden vorbereiten
	   tx_ide    = 0; //Basicframe
	   tx_rtr    = 0; //Dataframe
      tx_dlc    = 4'b0100;//4 Byte lang
	   basicID   = 11'b0101_0101_011;
	   extID     = 18'b0000_0000_0000_0000_00;
	   tx_data_0 =  8'b0000_1111;
	   tx_data_1 =  8'b0101_0101;
	   tx_data_2 =  8'b0000_1111;
	   tx_data_3 =  8'b0101_0101;
	   tx_data_4 =  8'b0000_0000;
	   tx_data_5 =  8'b0000_0000;
	   tx_data_6 =  8'b0000_0000;
	   tx_data_7 =  8'b0000_0000;
	   tx_request = 1;
	   transmitting = 1;
	   #5000;
	    
	   //Kommando Register
	   overload_request = 0; 
	   #1000000;  
end
endtask //   ack_error-------------------------------------------------------------------------------------------

task form_error;   // Erzeugt ein Form Error
begin
    begin
	    //Senden vorbereiten
	   tx_ide    = 0; //Basicframe
	   tx_rtr    = 0; //Dataframe
      tx_dlc    = 4'b0100;//4 Byte lang
	   basicID   = 11'b0101_0101_011;
	   extID     = 18'b0000_0000_0000_0000_00;
	   tx_data_0 =  8'b0000_1111;
	   tx_data_1 =  8'b0101_0101;
	   tx_data_2 =  8'b0000_1111;
	   tx_data_3 =  8'b0101_0101;
	   tx_data_4 =  8'b0000_0000;
	   tx_data_5 =  8'b0000_0000;
	   tx_data_6 =  8'b0000_0000;
	   tx_data_7 =  8'b0000_0000;
	   tx_request = 1;
	   #5000;
	    
	   //Kommando Register
	   overload_request = 0;   
	end
	
	begin
        #931;
        repeat (1)
        begin
          set_bus_bit(0);  // SOF
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID
          set_bus_bit(1);  // ID
          set_bus_bit(0);  // ID arbi. verlust

          set_bus_bit(1);  // RTR
          set_bus_bit(0);  // IDE
          set_bus_bit(0);  // r0
          set_bus_bit(0);  // DLC
          set_bus_bit(1);  // DLC
          set_bus_bit(1);  // DLC
          set_bus_bit(1);  // DLC
          
          repeat (11) 
          begin
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG  
            set_bus_bit(0); //MSG
            set_bus_bit(0); //MSG
            set_bus_bit(1); //MSG stuff
          end
          set_bus_bit(1); //MSG
        
          set_bus_bit(1);  // CRC 1
          set_bus_bit(0);  // CRC
          set_bus_bit(0);  // CRC
          set_bus_bit(0);  // CRC 
          set_bus_bit(1);  // CRC 
          set_bus_bit(1);  // CRC
          set_bus_bit(0);  // CRC
          set_bus_bit(1);  // CRC CRC Error
          set_bus_bit(1);  // CRC  
          set_bus_bit(1);  // CRC 
          set_bus_bit(0);  // CRC
          set_bus_bit(1);  // CRC
          set_bus_bit(1);  // CRC
          set_bus_bit(1);  // CRC 
          set_bus_bit(1);  // CRC 15
          set_bus_bit(1);  // CRC DELIM
          set_bus_bit(0);  // ACK
          set_bus_bit(1);  // ACK DELIM
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(1);  // EOF
          set_bus_bit(0);  // INTER Form Error
          set_bus_bit(1);  // INTER
          set_bus_bit(1);  // INTER     
        #400;
        end // repeat
   end
end
endtask //   form_error-------------------------------------------------------------------------------------------

task bit_error;   // Erzeugt ein Bit Error
begin
	    //Senden vorbereiten
	   tx_ide    = 0; //Basicframe
	   tx_rtr    = 0; //Dataframe
      tx_dlc    = 4'b0100;//4 Byte lang
	   basicID   = 11'b0101_0101_011;
	   extID     = 18'b0000_0000_0000_0000_00;
	   tx_data_0 =  8'b0000_1111;
	   tx_data_1 =  8'b0101_0101;
	   tx_data_2 =  8'b0000_1111;
	   tx_data_3 =  8'b0101_0101;
	   tx_data_4 =  8'b0000_0000;
	   tx_data_5 =  8'b0000_0000;
	   tx_data_6 =  8'b0000_0000;
	   tx_data_7 =  8'b0000_0000;
	   tx_request = 1;
	   transmitting = 0; // Erzeugt ein Bit-Error im ersten Bit beim Senden
	   #5000;
	    
	   //Kommando Register
	   overload_request = 0; 
	   #1000000;  
end
endtask //   bit_error-------------------------------------------------------------------------------------------

task send_extendedframe;     // Controller sendet ein Extended Frame
begin
end
endtask //   send_extendedframe----------------------------------------------------------------------------------

task empfang_extendedframe;  // Testbench sendet ein Extended Frame 
begin
end
endtask //   empfang_extendedframe-------------------------------------------------------------------------------    

task test_basic_mode;    // Cotroller ist im Basic-Mode und kann keine Extended Frames mehr empfangen
begin
end
endtask //  test_basic_mode--------------------------------------------------------------------------------------
*/
endmodule
