/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`timescale	1 ns / 10 ps

module testbench	(
			);


wire		dp;
wire		dn;
pullup(dp);
pulldown(dn);

reg		rst_phy;
wire		clk_phy;
wire		TxReady;
reg		TxValid;
reg	[7:0]	DataOut;
wire	[7:0]	DataIn;
reg	[7:0]	DataRecived;
wire		RxValid, RxActive,	RxError;
wire	[1:0]	LineState;

reg	[7:0]	packets [1<<6:0];
reg	[6:0]	index;
reg	[7:0]	size;
reg	[7:0]	start;
reg	[7:0]	wait_time;
reg	[7:0]	wait_loop;

reg		clk;
reg		reset;
reg		interrupt;
reg	[7:0]	char;
wire	[6:0]	leds_lowactiv;
reg		tx = 1;
reg	[7:0]	serial_data[2];

initial
begin
	tx = 1'b1;
	reset=1'b1;
	interrupt=1'b1;
	#200;
	reset=1'b0;#475;
	reset=1'b1;
	#15000
	interrupt=1'b0;
	#150
	interrupt=1'b1;

	serial_data[0] = 8'h47;
	serial_data[1] = 8'h0d;
end

always
begin
	clk=1'b0;#20;
	clk=1'b1;#20;
end

always @(leds_lowactiv)
	case (~leds_lowactiv)
		7'h00	: char = " ";
		7'h08	: char = "_";
		7'h3f	: char = "0"; 
		7'h06	: char = "1";
		7'h5b	: char = "2";
		7'h4f	: char = "3";
		7'h66	: char = "4";
		7'h6d	: char = "5";
		7'h7d	: char = "6";  
		7'h07	: char = "7";
		7'h7F	: char = "8";
		7'h6F	: char = "9";

		7'h77	: char = "A";
		7'h7c	: char = "b";
		7'h39	: char = "C";
		7'h5e	: char = "d";
		7'h79	: char = "E";
		7'h71	: char = "F";
		7'h76	: char = "H";
		7'h10	: char = "i";
		7'h1e	: char = "J";
		7'h38	: char = "L";
		7'h54	: char = "n";
		7'h5c	: char = "o";
		7'h73	: char = "P";
		7'h50	: char = "r";
		7'h6d	: char = "S";
		7'h78	: char = "t";
		7'h1c	: char = "u";
		default	: char = "_";
	endcase

reg [31:0] i,j; //Zaehlvariablen fuer schleifen

initial
begin
	#50000;
	for (j=0; j < 2; j=j+1)
	begin
		tx = 1'b0; #8681;	// Startbit
		for (i=0; i < 8; i=i+1)
		begin
			tx = ~ serial_data[j][0];
			serial_data[j] = serial_data[j] >> 1;
			#8681;
		end
		tx = 1'b0; #17362;	// 2 Stopbits 
		tx = 1'b1; #10000;	// kleine Pause
	end
end


// USB Datengenerierung

initial
begin
	TxValid = 1'b0;
	DataOut = 8'h00;
	DataRecived = 8'h00;	
	rst_phy = 1'b0; #4000;
	rst_phy = 1'b1; #2000;

	$readmemh ("transfers.txt", packets);
	index = 0;
	
	while (packets[index]!=0)
	begin
		size = packets[index]; index = index +1;
		wait_time = packets[index]; index = index + 1;
		start = index;
	
		for( index = index ;index <= (start + size - 1); index = index + 1)
		begin
			DataOut = packets[index];
			TxValid = 1'b1;
			wait (TxReady);
			wait (!TxReady);
		end
		TxValid = 1'b0;
		for( wait_loop = 0; wait_loop <=wait_time; wait_loop = wait_loop + 1)
			#100;
	end
end

always @(posedge RxValid)
	if (RxActive == 1'b1)
		DataRecived <= DataIn;

system sys	(
			.clk(			clk		),
			.clk_i(			clk_phy		),
			.reset_lowactiv(	reset		),
			.interrupt_lowactiv(	interrupt	),
			.leds_lowactiv(		leds_lowactiv	),
			.rx(			tx		),
			.tx(					),
			.dp(			dp		),
			.dn(			dn		)
		);

usb_phy	phy 	(	.clk(			clk_phy		),
			.rst(			rst_phy		),
			.phy_tx_mode(		1'b1		),
			.usb_rst(				),
			.dp(			dp		),
			.dn(			dn		),
			.DataOut_i(		DataOut		),
			.TxValid_i(		TxValid		),
			.TxReady_o(		TxReady		),
			.DataIn_o(		DataIn		),
			.RxValid_o(		RxValid		),
			.RxActive_o(		RxActive	),
			.RxError_o(		RxError		),
			.LineState_o(		LineState	)
		);

endmodule
