/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  ACF                                                        ////
////  Dient der Umsetzung einer Filterregel                      ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none
// synopsys translate_off
`include "can/src/timescale.vh"
// synopsys translate_on

module can_acf(
	can_clk,
	reset,
	
	/* Mode register */
	config_mode,
	//acceptance_filter_mode,
	extended_mode,

	ac_b_id_0,
	am_b_id_0,
	ac_e_id_0,
	am_e_id_0,
	ac_rtr1_0,
	am_rtr1_0,
	ac_rtr2_0,
	am_rtr2_0,
	ac_data_0,
	am_data_0,

	acf_adr,
//	acf_code,
	we_acf,		// ist Synchron zu clk_peri
	clear_we_acf,

	idx,
	data0,
	ide,
	rtr1,
	rtr2,
	no_byte0,

	//id_ok		//Ausgabe bei akzeptierten Telegramm
	match_basic,
	match_std,
	match_ext
	);

parameter	tp		= 1;
parameter	ACF_CODE	= 0;

input wire			can_clk;
input wire			reset;
//Input von can_bsp
input wire	[28:0]		idx;
input wire	[3:0]		data0;
input wire			ide;
input wire			rtr1;
input wire			rtr2;
input wire			no_byte0;

//Kommandoregister
input wire			config_mode;
//input wire			acceptance_filter_mode;
input wire			extended_mode;

input wire	[10:0]		ac_b_id_0;
input wire	[10:0]		am_b_id_0;
input wire	[17:0]		ac_e_id_0;
input wire	[17:0]		am_e_id_0;
input wire			ac_rtr1_0;
input wire			am_rtr1_0;
input wire			ac_rtr2_0;
input wire			am_rtr2_0;
input wire	[3:0]		ac_data_0;
input wire	[3:0]		am_data_0;

input wire	[4:0]		acf_adr;
//input wire	[4:0]		acf_code;
input wire			we_acf;		// ist Synchron zu clk_peri
output reg			clear_we_acf;

reg	[10:0]		ac_b_id;
reg	[10:0]		am_b_id;
reg	[17:0]		ac_e_id;
reg	[17:0]		am_e_id;
reg		      	ac_rtr1;
reg			am_rtr1;
reg			ac_rtr2;
reg			am_rtr2;
reg	[3:0]		ac_data;
reg	[3:0]		am_data;

//Ausabe an Can_bsp
output wire			match_basic; //für Basicmode
output wire			match_std; //Standardframe im Extended Mode
output wire			match_ext; //Extendedframe im Extended Mode

//reg			id_ok;



// Basic Mode --> prüft ID für Standardframe (11-bit ID).
assign match_basic =   ((idx[0]  == ac_b_id[0] | am_b_id[0] ) &
			(idx[1]  == ac_b_id[1] | am_b_id[1] ) &
			(idx[2]  == ac_b_id[2] | am_b_id[2] ) &
			(idx[3]  == ac_b_id[3] | am_b_id[3] ) &
			(idx[4]  == ac_b_id[4] | am_b_id[4] ) &
			(idx[5]  == ac_b_id[5] | am_b_id[5] ) &
			(idx[6]  == ac_b_id[6] | am_b_id[6] ) &
			(idx[7]  == ac_b_id[7] | am_b_id[7] ) &
			(idx[8]  == ac_b_id[8] | am_b_id[8] ) &
			(idx[9]  == ac_b_id[9] | am_b_id[9] ) &
			(idx[10] == ac_b_id[10]| am_b_id[10]) &
			ide == 1'b0//sonst ist Frame kein Basicfarme und wird ignoriert
		 );
								
// Extended Mode --> prüft ID für Standardframe + rtr1 + Datenbyte0high  (11-bit ID).
assign match_std =     ((idx[0]  == ac_b_id[0] | am_b_id[0] ) &
			(idx[1]  == ac_b_id[1] | am_b_id[1] ) &
			(idx[2]  == ac_b_id[2] | am_b_id[2] ) &
			(idx[3]  == ac_b_id[3] | am_b_id[3] ) &
			(idx[4]  == ac_b_id[4] | am_b_id[4] ) &
			(idx[5]  == ac_b_id[5] | am_b_id[5] ) &
			(idx[6]  == ac_b_id[6] | am_b_id[6] ) &
			(idx[7]  == ac_b_id[7] | am_b_id[7] ) &
			(idx[8]  == ac_b_id[8] | am_b_id[8] ) &
			(idx[9]  == ac_b_id[9] | am_b_id[9] ) &
			(idx[10] == ac_b_id[10]| am_b_id[10]) &
								
			(rtr1   == ac_rtr1	 | am_rtr1		) &
								
			(data0[0]  == ac_data[0] | am_data[0] | no_byte0) &
			(data0[1]  == ac_data[1] | am_data[1] | no_byte0) &
			(data0[2]  == ac_data[2] | am_data[2] | no_byte0) &
			(data0[3]  == ac_data[3] | am_data[3] | no_byte0) 
		 );	

// Extended Mode --> prüft ID für Extendedframe + rtr2 + Datenbyte0high  (29-bit ID).
assign match_ext =     ((idx[18] == ac_b_id[0] | am_b_id[0] ) &
			(idx[19] == ac_b_id[1] | am_b_id[1] ) &
			(idx[20] == ac_b_id[2] | am_b_id[2] ) &
			(idx[21] == ac_b_id[3] | am_b_id[3] ) &
			(idx[22] == ac_b_id[4] | am_b_id[4] ) &
			(idx[23] == ac_b_id[5] | am_b_id[5] ) &
			(idx[24] == ac_b_id[6] | am_b_id[6] ) &
			(idx[25] == ac_b_id[7] | am_b_id[7] ) &
			(idx[26] == ac_b_id[8] | am_b_id[8] ) &
			(idx[27] == ac_b_id[9] | am_b_id[9] ) &
			(idx[28] == ac_b_id[10]| am_b_id[10]) &
								
			(idx[0]  == ac_e_id[0] | am_e_id[0] ) &
			(idx[1]  == ac_e_id[1] | am_e_id[1] ) &
			(idx[2]  == ac_e_id[2] | am_e_id[2] ) &
			(idx[3]  == ac_e_id[3] | am_e_id[3] ) &
			(idx[4]  == ac_e_id[4] | am_e_id[4] ) &
			(idx[5]  == ac_e_id[5] | am_e_id[5] ) &
			(idx[6]  == ac_e_id[6] | am_e_id[6] ) &
			(idx[7]  == ac_e_id[7] | am_e_id[7] ) &
			(idx[8]  == ac_e_id[8] | am_e_id[8] ) &
			(idx[9]  == ac_e_id[9] | am_e_id[9] ) &
			(idx[10] == ac_e_id[10]| am_e_id[10]) &
			(idx[11] == ac_e_id[11]| am_e_id[11]) &
			(idx[12] == ac_e_id[12]| am_e_id[12]) &
			(idx[13] == ac_e_id[13]| am_e_id[13]) &
			(idx[14] == ac_e_id[14]| am_e_id[14]) &
			(idx[15] == ac_e_id[15]| am_e_id[15]) &
			(idx[16] == ac_e_id[16]| am_e_id[16]) &
			(idx[17] == ac_e_id[17]| am_e_id[17]) &
								
			(rtr2   == ac_rtr2	 | am_rtr2		) &
								
			(data0[0]  == ac_data[0] | am_data[0] | no_byte0) &
			(data0[1]  == ac_data[1] | am_data[1] | no_byte0) &
			(data0[2]  == ac_data[2] | am_data[2] | no_byte0) &
			(data0[3]  == ac_data[3] | am_data[3] | no_byte0) 
		 );	
								 

// DEMUX für Filtereingänge
always @ (posedge can_clk or posedge reset)
    begin
	if (reset)
	begin
	     ac_b_id <=#tp 11'd0;
	     am_b_id <=#tp 11'h7FF;//Ausschalten der Maske
	     ac_e_id <=#tp 18'd0;
	     am_e_id <=#tp 18'h3FFFF;
	     ac_rtr1 <=#tp 1'd0;
	     am_rtr1 <=#tp 1'd1;
	     ac_rtr2 <=#tp 1'd0;
	     am_rtr2 <=#tp 1'd1;
	     ac_data <=#tp 4'd0;
	     am_data <=#tp 4'd15;
	     clear_we_acf <= 1'b0;
	end
	else if (config_mode)
        begin
	    if ((acf_adr == ACF_CODE) & we_acf)	// ist Synchron zu clk_peri
	    begin
		if (extended_mode)
		begin
		ac_b_id <=#tp ac_b_id_0;
		am_b_id <=#tp am_b_id_0;
		ac_e_id <=#tp ac_e_id_0;
		am_e_id <=#tp am_e_id_0;
		ac_rtr1 <=#tp ac_rtr1_0;
		am_rtr1 <=#tp am_rtr1_0;
		ac_rtr2 <=#tp ac_rtr2_0;
		am_rtr2 <=#tp am_rtr2_0;
		ac_data <=#tp ac_data_0;
		am_data <=#tp am_data_0;
		end
		else //basic_mode
		begin
		ac_b_id <=#tp ac_b_id_0;
		am_b_id <=#tp am_b_id_0;
		ac_rtr1 <=#tp ac_rtr1_0;
		am_rtr1 <=#tp am_rtr1_0;
		ac_data <=#tp ac_data_0;
		am_data <=#tp am_data_0;
		end
	     clear_we_acf <= 1'b1;
	    end
	    else
	     clear_we_acf <= 1'b0;
	end
    end
	
endmodule
