/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Blockram ACCESS                                            ////
////  Diese Modul schreibt und liest den DMA Puffer              ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none

module can_blockram_access
	(
	clk_peri,
	reset,

	//Von can_rx_buffer und can_tx_buffer
	write_rx_buffer,
	tx_start_adr,
	read_tx_buffer,
	rx_start_adr,
	row_ready0,
	row_ready8,

	//Signale von can_crc
	calculated_crc,

	//Ausgabe an can_rx_buffer und can_tx_buffer
	ready,
	//Ausgabe an can_tx_buffer
	retry_tx,

	//Von can_acf
	match,

	//Ausgabe an das Blockram 
	dma_adr_A,
	dma_we_A,
	dma_dataI_A,

	dma_dataO_A, //zum lesen der Mailbox

	//Input von can_bsp
	id,
	ide,
	rtr1,
	rtr2,
	data_length,
	rx_data0,
	rx_data1,
	rx_data2,
	rx_data3,
	rx_data4,
	rx_data5,
	rx_data6,
	rx_data7,

	//Ausgabe an can_bsp
	tx_ide,
	tx_rtr,
	tx_dlc,
	tx_basicID,
	tx_extID, 
	tx_data0,
	tx_data1,
	tx_data2,
	tx_data3,
	tx_data4,
	tx_data5,
	tx_data6,
	tx_data7

	);
parameter TX_MAILBOX	= 18; //1..18
parameter RX_MAILBOX	= 18; //1..18
parameter ACF_CHAINS	= 16; //1..18
parameter BASE_ADR	= 10'h3f0;

input wire				clk_peri;
input wire				reset;

//Von can_tx_buffer und can_rx_buffer
input wire				write_rx_buffer;
input wire	[9:0]			tx_start_adr;
input wire				read_tx_buffer;
input wire	[9:0]			rx_start_adr;
//input wire				write_error_capture_code; //feste adr
//input wire				write_transmitter_flag; //feste adr
//input wire				write_receiver_flag; //feste adr

//Von can_acf
input wire	[ACF_CHAINS-1:0]	match;

//Signale von can_crc
input wire	[14:0]			calculated_crc;

//Ausgabe an can_rx_buffer und can_tx_buffer
output wire				ready;
output wire				row_ready0;
output wire				row_ready8;
//Ausgabe an can_tx_buffer
output reg	[5:0]			retry_tx;

//Ausgabe an das Blockram 
output reg	[9:0]			dma_adr_A;
output reg					dma_we_A;
output reg	[17:0]			dma_dataI_A;

input wire	[17:0]			dma_dataO_A; //zum lesen der Mailbox

//input wire von can_bsp
input wire	[28:0]			id;
input wire				ide;
input wire				rtr1;
input wire				rtr2;
input wire	[3:0]			data_length;
input wire	[7:0]			rx_data0;
input wire	[7:0]			rx_data1;
input wire	[7:0]			rx_data2;
input wire	[7:0]			rx_data3;
input wire	[7:0]			rx_data4;
input wire	[7:0]			rx_data5;
input wire	[7:0]			rx_data6;
input wire	[7:0]			rx_data7;

//Ausgabe an can_bsp
output reg				tx_ide;
output reg				tx_rtr;
output reg	[3:0]			tx_dlc;
output reg	[10:0]			tx_basicID;
output reg	[17:0]			tx_extID; 
output reg	[7:0]			tx_data0;
output reg	[7:0]			tx_data1;
output reg	[7:0]			tx_data2;
output reg	[7:0]			tx_data3;
output reg	[7:0]			tx_data4;
output reg	[7:0]			tx_data5;
output reg	[7:0]			tx_data6;
output reg	[7:0]			tx_data7;



reg	[3:0]			row_ready;


assign	row_ready0	= (row_ready==4'h0)? 1'b1 : 1'b0;
assign	row_ready8	= (row_ready==4'h8)? 1'b1 : 1'b0;
assign	ready		= (read_tx_buffer || write_rx_buffer) & (row_ready8);

always @ (posedge clk_peri or posedge reset)
begin
if (reset)
	row_ready	<= 4'h0;
else if (((read_tx_buffer & (~write_rx_buffer)) | write_rx_buffer) & (row_ready8))
	row_ready	<= 4'h0;
else if (((read_tx_buffer & (~write_rx_buffer)) | write_rx_buffer))
	row_ready	<= row_ready + 4'h1;
else 
	row_ready	<= 4'h0;
end

//wire not_rst;
//wire not_clk_peri;

//assign not_rst = ~reset;
//assign not_clk_peri  = ~clk_peri;

always @ (posedge clk_peri or posedge reset)
begin
if (reset)
begin
	dma_adr_A	<= 10'h0;
	dma_dataI_A	<= 18'h0;
	tx_ide		<= 1'b0;
	tx_rtr		<= 1'b0;
	tx_dlc		<= 4'b0;
	tx_basicID	<= 11'b0;
	tx_extID	<= 18'b0; 
	tx_data0	<= 8'b0;
	tx_data1	<= 8'b0;
	tx_data2	<= 8'b0;
	tx_data3	<= 8'b0;
	tx_data4	<= 8'b0;
	tx_data5	<= 8'b0;
	tx_data6	<= 8'b0;
	tx_data7	<= 8'b0;
	retry_tx	<= 6'b1;
	dma_we_A	<= 1'b0;
end
else
	begin
	case (row_ready)
	 4'h0:
		 begin
			 if	(read_tx_buffer & (~write_rx_buffer))
			 begin //tx lesen
				dma_adr_A		<= tx_start_adr;
				//todo dma_enable	 = 0
			 end
			 else if (write_rx_buffer)
			 begin //rx schreiben
			 	dma_adr_A		<= rx_start_adr;
				//todo dma_enable	 = 0
				dma_we_A		<= 1'b1;
				dma_dataI_A[15:8]	<= rx_data0;
				dma_dataI_A[7:0]	<= rx_data1;
			 end
		 end
	4'h1:
		begin
			if	(read_tx_buffer)//legt Prioritaet fest read befor write
			begin //tx lesen
				//todo setze dma_enable
				dma_adr_A		<= dma_adr_A + 4'd1;
			end
			else if (write_rx_buffer)
			begin //rx schreiben
				//todo setze dma_enable
				dma_adr_A		<= dma_adr_A + 4'd1;
				dma_dataI_A[15:8]	<= rx_data2;
				dma_dataI_A[7:0]	<= rx_data3;
			end
		end
	4'h2:
		begin
			if	(read_tx_buffer)
			begin //tx lesen
				tx_data0		<= dma_dataO_A[15:8];
	        		tx_data1		<= dma_dataO_A[7:0];	
				dma_adr_A		<= dma_adr_A + 4'd1;		
			end
			else if (write_rx_buffer)
			begin //rx schreiben
				dma_adr_A		<= dma_adr_A + 4'd1;	
				dma_dataI_A[15:8]	<= rx_data4;
				dma_dataI_A[7:0]	<= rx_data5;
			end
		end
	4'h3:
		begin
			if	(read_tx_buffer)
			begin //tx lesen
				tx_data2		<= dma_dataO_A[15:8];
				tx_data3		<= dma_dataO_A[7:0];	
				dma_adr_A		<= dma_adr_A + 4'd1;
			end
			else if (write_rx_buffer)
			begin //rx schreiben
				dma_adr_A		<= dma_adr_A + 4'd1;	
				dma_dataI_A[15:8]	<= rx_data6;
				dma_dataI_A[7:0]	<= rx_data7;
			end
		end
	4'h4:
		begin
			if	(read_tx_buffer)
			begin //tx lesen
				tx_data4		<= dma_dataO_A[15:8];
				tx_data5		<= dma_dataO_A[7:0];	
				dma_adr_A		<= dma_adr_A + 4'd1;	
			end
			else if (write_rx_buffer)
			begin //rx schreiben
				dma_adr_A		<= dma_adr_A + 4'd1;	
				if (ide)//ext Frame
				   dma_dataI_A[17:0]	<= id[17:0]; //todo funzt nicht im basicmode
				else//basic Frame
				   dma_dataI_A		<= 18'd0;
			end
		end
	4'h5:
		begin
			if	(read_tx_buffer)
			begin //tx lesen
				tx_data6		<= dma_dataO_A[15:8];
				tx_data7		<= dma_dataO_A[7:0];	
				dma_adr_A		<= dma_adr_A + 4'd1;	
			end
			else if (write_rx_buffer)
			begin //rx schreiben
				dma_adr_A		<= dma_adr_A + 4'd1;	
				dma_dataI_A[16]		<= ide;
				if (ide)//ext Frame
				begin
				   dma_dataI_A[10:0]	<= id[28:18];
				   dma_dataI_A[17]	<= rtr2;	
				end
				else//basic Frame
				begin
				   dma_dataI_A[10:0]	<= id[10:0];
				   dma_dataI_A[17]	<= rtr1;
				end
				dma_dataI_A[15:12]	<= data_length;
				dma_dataI_A[11]		<= 1'b0;	
			end
		end
	4'h6:
		begin
			if	(read_tx_buffer)
			begin //tx lesen
				tx_extID		<= dma_dataO_A[17:0];
				dma_adr_A		<= dma_adr_A + 4'd1;	
			end
			else if (write_rx_buffer)
			begin //rx schreiben
			   dma_adr_A			<= dma_adr_A + 4'd1;	
			   dma_dataI_A[ACF_CHAINS-1:0]	<= match;	
			end
		end
	4'h7:
		begin
			if	(read_tx_buffer)
			begin //tx lesen
				tx_basicID		<= dma_dataO_A[10:0];
				tx_ide			<= dma_dataO_A[16];
				tx_rtr			<= dma_dataO_A[17];
				tx_dlc			<= dma_dataO_A[15:12];	
				dma_adr_A		<= dma_adr_A + 4'd1;	
			end
			else if (write_rx_buffer)
			begin //rx schreiben
				dma_adr_A		<= dma_adr_A + 4'd1;
				dma_dataI_A[17:15]	<= 3'd0;
				dma_dataI_A[14:0]	<= calculated_crc[14:0];	// CRC zur Kontrolle puffern
			end
		end
	4'h8:
		begin
			if	(read_tx_buffer)
			begin //tx lesen
				retry_tx		<= dma_dataO_A[5:0]+1'b1;
				dma_adr_A		<= dma_adr_A + 4'd1;	
			end
			else if (write_rx_buffer)
			begin //rx schreiben
				dma_adr_A		<= dma_adr_A + 4'd1;
				dma_dataI_A		<= 18'd0;	
				dma_we_A		<= 1'b0;
			end
		end
	endcase
	end
end

endmodule
