/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  BSP                                                        ////
////  Kern des CAN Controllers. Sendet und Empfaengt Telegramme  ////
////  Bus und speichert diese in Zwischenpuffer.                 ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none
// synopsys translate_off
`include "can/src/timescale.vh"
// synopsys translate_on

module can_bsp(
	can_clk,
	reset,
	
	id_ok,

	//Signale von can_btl
	sample_strobe_q,
	clk_fsm,
	sample_bit,
	tx_strobe,
	hard_sync,
	
	//Signale von can_emu
	node_error_passive,
	node_error_active,
	node_bus_off,
	error_strobe,
	bus_free,
	
	//Ausgabe an can_btl
	rx_idle,
	transmitting,
	transmitter,
	go_rx_inter,
	rx_inter,
	tx_next,
	go_overload_frame,
	go_error_frame,
	go_tx,
	send_ack,
	
	//Ausgabe an can_bsl
	tx,
	go_rx_id1,
	go_rx_crc_lim,
	rx_crc_lim_stuffing_disable,
	
	//Mode Register
	config_mode,
	listen_only_mode,
	self_test_mode,
	extended_mode,
	
	//Signale von can_crc
	calculated_crc,
	dlc0_crc,
	set_dlc0_crc,
	
	//Ausgabe an CRC-Modul
	crc,
	crc_strobe,
	crc_enable_stb,
	go_crc_enable,
	
	//Ausgabe an Akzeptanzfiltermodul
	idy,
	no_byte0,
//	data0,
	rtr1,
	rtr2,
	ide,
	
	//Ausgabe an emu
	bit_error,
	form_error,
	ack_error,
	error_capture_code_segment,
	error_direction,
	rule5,
	rule3_exception1_2,
	tx_successful,
	error_cnt1,
	error_frame,
	arbitration_field,
	go_rx_ack_lim,
	arbitration_lost,
	error_flag_over,
	error_frame_lim,
	delayed_dominant_cnt,
	
	//Kommando Register
	overload_request,
	tx_request,
	abort_tx,
	
	//Temp Senden
	tx_ide,
	tx_rtr,
	tx_dlc,
	tx_basicID,
	tx_extID,
	tx_data0,
	tx_data1,
	tx_data2,
	tx_data3,
	tx_data4,
	tx_data5,
	tx_data6,
	tx_data7,
	
	//Signale von can_acf
	match_basic,
	match_std,
	match_ext,
	acf_en,
	
	//an can_blockram_access
	rx_data0,
	rx_data1,
	rx_data2,
	rx_data3,
	rx_data4,
	rx_data5,
	rx_data6,
	rx_data7,
	
	//Signal an can_blockram_access
	match,
	data_length,
	row_ready8,
	
	rx_successful,
	receiving,

// Zustands Signale der FSM zur Fehlersuche
//	rx_idle,		// ist schon als OUTPUT declariert.
	rx_id1,
	rx_rtr1,
	rx_ide,
	rx_id2,
	rx_rtr2,
	rx_r1,
	rx_r0,
	rx_dlc,
	rx_data,
	rx_crc,
	rx_crc_lim,
	rx_ack,
	rx_ack_lim,
	rx_eof,
//	rx_inter,		// ist schon als OUTPUT declariert.
//	error_frame,		// ist schon als OUTPUT declariert.
	error_frame_limz,
	overload_frame,
	overload_frame_limz

	);

parameter TX_MAILBOX	= 18; //1..18
parameter RX_MAILBOX	= 18; //1..18
parameter ACF_CHAINS	= 18; //1..18
parameter BASE_ADR	= 6'h8;

//parameter MAILBOX = 2;
parameter tp = 1;

input wire				can_clk;
input wire				reset;
//Signale von btl
input wire				sample_strobe_q;
input wire				clk_fsm;
input wire				sample_bit;
input wire				tx_strobe;
input wire				hard_sync;
//Kommandoregister - Moderegister
input wire				config_mode;
input wire				listen_only_mode;
input wire				self_test_mode;
input wire				extended_mode;
input wire				overload_request;
input wire				tx_request;
input wire				abort_tx;
//Signale von emu
input wire				node_error_passive;
input wire				node_error_active;
input wire				node_bus_off;
input wire				error_strobe;
input wire				bus_free;
	
//zum Senden temporäre Lösung
input wire				tx_ide;
input wire				tx_rtr;
input wire	[3:0]			tx_dlc;
input wire	[10:0]			tx_basicID;
input wire	[17:0]			tx_extID; 
input wire	[7:0]			tx_data0;
input wire	[7:0]			tx_data1;
input wire	[7:0]			tx_data2;
input wire	[7:0]			tx_data3;
input wire	[7:0]			tx_data4;
input wire	[7:0]			tx_data5;
input wire	[7:0]			tx_data6;
input wire	[7:0]			tx_data7;
	
//Signale von acf
input wire	[(ACF_CHAINS-1):0]	match_basic;
input wire	[(ACF_CHAINS-1):0]	match_std;
input wire	[(ACF_CHAINS-1):0]	match_ext;
input wire	[(ACF_CHAINS-1):0]	acf_en;
	
//Signale von can_blockramaccess
input wire				row_ready8;
	
//Signale an emu
output wire				bit_error;
output wire				form_error;
output wire				ack_error;
output wire	[4:0]			error_capture_code_segment;
output wire				error_direction;
output wire				tx_successful;
output wire				rule5;
output reg				rule3_exception1_2;
output reg	[2:0] 			error_cnt1;
output wire      			error_frame;
output wire      			arbitration_field;
output wire      			go_rx_ack_lim;
output reg      			arbitration_lost;
output wire      			error_flag_over;
output wire     			error_frame_lim;
output reg	[2:0] 			delayed_dominant_cnt;

//Signale von crc Modul
input wire	[14:0]			calculated_crc;
//Signale an crc Modul
output reg	[14:0]			crc;
output reg	[14:0]			dlc0_crc;
output reg				set_dlc0_crc;
output reg				crc_strobe;
output wire				crc_enable_stb;
output wire				go_crc_enable;
//Signale an btl
output wire				rx_idle;
output wire				rx_inter;
output wire				go_rx_inter;
output reg				tx_next;
output wire				go_overload_frame;
output wire				go_error_frame;
output wire				go_tx;
output wire				send_ack;
output reg				transmitting;
output reg				transmitter;
//Signale an can_acf
output wire				no_byte0;
output reg	[28:0]		idy; //Speichert den Identifier zur Übergabe an den Akzeptanzfilter
output reg				rtr1;
output reg				rtr2;
output reg				ide;
	
//Signale an bsl
output reg				tx;
output wire				go_rx_crc_lim;
output wire				go_rx_id1;
output wire				rx_crc_lim_stuffing_disable;
	
//acf Signal
output reg				id_ok;
	
//Signal an can_blockram_access
output reg	[ACF_CHAINS-1:0]	match;

//an can_blockram_access
output reg	[3:0]			data_length;
output reg	[7:0]			rx_data0;
output reg	[7:0]			rx_data1;
output reg	[7:0]			rx_data2;
output reg	[7:0]			rx_data3;
output reg	[7:0]			rx_data4;
output reg	[7:0]			rx_data5;
output reg	[7:0]			rx_data6;
output reg	[7:0]			rx_data7;
	
output wire				rx_successful;
output wire				receiving;

// Zustands Signale der FSM zur Fehlersuche
//output wire				rx_idle;	// ist schon als OUTPUT declariert.
output wire				rx_id1;
output wire				rx_rtr1;
output wire				rx_ide;
output wire				rx_id2;
output wire				rx_rtr2;
output wire				rx_r1;
output wire				rx_r0;
output wire				rx_dlc;
output wire				rx_data;
output wire				rx_crc;
output wire				rx_crc_lim;
output wire				rx_ack;
output wire				rx_ack_lim;
output wire				rx_eof;
//output wire				rx_inter;	// ist schon als OUTPUT declariert.
//output wire				error_frame;	// ist schon als OUTPUT declariert.
output wire				error_frame_limz;
output wire				overload_frame;
output wire				overload_frame_limz;


//Sendedaten
reg	[6:0]			bit_cnt;
reg	[6:0]			tx_pointer;
reg				tx_bit;
//reg				tx_ide;
//reg				tx_rtr;
//reg	[3:0]			tx_dlc;
reg	[5:0]			tx_dlc_tmp;
	
	
//Puffer Register
reg	[7:0]			tmp_data;
//reg	[7:0]			tmp_fifo [0:7]; //8 Byte temp. Nachrichtenpuffer
reg				write_data_to_tmp_fifo;

//reg	[7:0]			tx_data_0;
//reg	[7:0]			tx_data_1;
//reg	[7:0]			tx_data_2;
//reg	[7:0]			tx_data_3;
//reg	[7:0]			tx_data_4;
//reg	[7:0]			tx_data_5;
//reg	[7:0]			tx_data_6;
//reg	[7:0]			tx_data_7;

//reg	[10:0]			basicID; //zu sendende ID
//reg	[17:0]			extID;   //zu sendende ID

reg	[2:0]			byte_cnt;

wire				mem_dlc0_crc;
reg	[2:0]			eof_cnt;
	
//Arbitration und Hilfsregister
reg	[4:0]			arbitration_cnt;
reg				arbitration_lost_capture;
reg				arbitration_blocked;
reg				tx_state;
reg				need_to_tx;//ist erst bei einem dominaten Bit auf den Bus gesetzt (in inter oder nach idle)
reg				suspend;//Zustand wenn in Errorpassivmode nach inter
reg				suspend_cnt;
reg				suspend_cnt_enable;
	
reg 				finish_msg;
//reg				abort_tx;
//reg 				errorframe_ended;
reg	[2:0]			overload_cnt1;
reg	[2:0]			overload_cnt2;
reg				overload_frame_blocked;
reg				enable_overload_cnt2;
reg				overload_request_cnt;
reg	[2:0]			error_cnt2;
reg				enable_error_cnt2;

reg				crc_enable;
	
reg				rule3_exception1_1;
	
reg				sample_strobe_d;	// Verzoegerter sample_strobe_q
reg				sample_strobe_e;	// Verzoegerter sample_strobe_q
reg				crc_stb_en;		// Verzoegerter sample_strobe_q
wire				sample_strobe_l;	// Verlaengerter sample_strobe_q
	
//Register fuer BTL bzw. BSL
	
wire				go_rx_idle;
wire				go_rx_rtr1;
wire				go_rx_ide;
wire				go_rx_id2;
wire				go_rx_rtr2;
wire				go_rx_r1;
wire				go_rx_r0;
wire				go_rx_dlc;
wire				go_rx_data;
wire				go_rx_crc1;
wire				go_rx_crc2;
wire				go_rx_ack;
wire				go_rx_eof;
reg				error_frame_limv;
wire				overload_frame_lim;
reg				overload_frame_limv;
wire				go_early_tx;
	
wire				reset_tx_pointer;
//wire	[5:0]			lim_dat_basic;
wire	[5:0]			lim_data;
	
wire				last_bit_of_inter;

//Invertierte Sendedaten
wire	[7:0]			inv_tx_data_0;
wire	[7:0]			inv_tx_data_1;
wire	[7:0]			inv_tx_data_2;
wire	[7:0]			inv_tx_data_3;
wire	[7:0]			inv_tx_data_4;
wire	[7:0]			inv_tx_data_5;
wire	[7:0]			inv_tx_data_6;
wire	[7:0]			inv_tx_data_7;
wire	[3:0]			inv_tx_dlc;
wire	[10:0]			inv_basicID;
wire	[17:0]			inv_extID;
wire	[14:0]			inv_calculated_crc;
	
//Telegramteile zum Senden
wire	[63:0]			data_chain;
wire 	[38:0]			extended_chain;
wire	[18:0]			basic_chain;		
	
wire 				bit_error_exc1;
wire 				bit_error_exc2;
wire 				bit_error_exc3;
wire 				bit_error_exc4;
wire 				bit_error_exc5;
wire 				bit_error_exc6;
	
wire	[7:0]		data_length_in_bit;
	
wire				rtr_frame_rx;
wire				rtr_frame_tx;
wire				early_bit_of_inter;
wire				set_arbitration_lost;
wire				overload_flag_over;
wire				rst_crc_enable;

/**
 * Definition der Zustaende des Rx Automaten
 */
parameter [19:0]
		RX_IDLE		=	20'b00000000000000000001,
		RX_ID1		=	20'b00000000000000000010,
		RX_RTR1		=	20'b00000000000000000100,
		RX_IDE		=	20'b00000000000000001000,
		RX_ID2		=	20'b00000000000000010000,
		RX_RTR2		=	20'b00000000000000100000,
		RX_R1		=	20'b00000000000001000000,
		RX_R0		=	20'b00000000000010000000,
		RX_DLC		=	20'b00000000000100000000,
		RX_DATA		=	20'b00000000001000000000,
		RX_CRC		=	20'b00000000010000000000,
		RX_CRC_LIM	=	20'b00000000100000000000,
		RX_ACK		=	20'b00000001000000000000,
		RX_ACK_LIM	=	20'b00000010000000000000,
		RX_EOF		=	20'b00000100000000000000,
		RX_INTERM	=	20'b00001000000000000000,
		RX_ERROR	=	20'b00010000000000000000,
		RX_ERROR_LIM	=	20'b00100000000000000000,
		RX_OVERL	=	20'b01000000000000000000,
		RX_OVERL_LIM	=	20'b10000000000000000000;


reg	[19:0]			state, 
				next_state;

assign crc_enable_stb	  = sample_strobe_q & crc_enable;	// CRC-clk

assign rtr_frame_rx	  = (/*receiving &*/ ((rtr1 & ~ide) | (rtr2 & ide)));
assign rtr_frame_tx	  = (transmitter & tx_rtr);
	
assign data_length_in_bit = data_length<<3;
	
assign go_rx_idle	  = /*sample_strobe_q &*/ (sample_bit & last_bit_of_inter | bus_free & (~node_bus_off));
assign go_rx_id1	  = sample_strobe_l & ((~sample_bit) & (rx_idle | last_bit_of_inter));
assign go_rx_rtr1 	  = sample_strobe_l & (rx_id1 & (bit_cnt[3:0] == 4'd10)); //bzw. SSR
assign go_rx_ide 	  = sample_strobe_q & (rx_rtr1);
assign go_rx_id2 	  = sample_strobe_q & (rx_ide & sample_bit);
assign go_rx_rtr2 	  = sample_strobe_l & (rx_id2  & (bit_cnt[4:0] == 5'd17));
assign go_rx_r1 	  = sample_strobe_q & (rx_rtr2);
assign go_rx_r0 	  = sample_strobe_q & ((rx_ide  & (~sample_bit) | rx_r1)); //bzw. kein ide bit
assign go_rx_dlc 	  = sample_strobe_q & (rx_r0);
assign go_rx_data	  = sample_strobe_q & (rx_dlc & (bit_cnt[1:0] == 2'd3) & ~rtr_frame_rx);
assign go_rx_crc1 	  = sample_strobe_q & (rx_dlc & (bit_cnt[1:0] == 2'd3) &  rtr_frame_rx);
assign mem_dlc0_crc	  = sample_strobe_q & (rx_data & (data_length==0));
assign go_rx_crc2 	  = sample_strobe_q & (rx_data & ((bit_cnt[6:0] == (data_length_in_bit - 1'b1)) | (data_length==0)));
assign go_rx_crc_lim	  = sample_strobe_q & (rx_crc & (bit_cnt[3:0] == 4'd14));
assign go_rx_ack 	  = sample_strobe_q & (rx_crc_lim);
assign go_rx_ack_lim	  = sample_strobe_q & (rx_ack);
assign go_rx_eof 	  = sample_strobe_q & (rx_ack_lim);
assign go_rx_inter 	  = sample_strobe_q & (((rx_eof & (eof_cnt == 3'd6)) | error_frame_limz | overload_frame_limz)
					    & (~overload_request)); //inter muss um 8 bit laenger werden im falle von node_error_passiv 
assign rx_crc_lim_stuffing_disable = sample_strobe_q & (rx_crc & (bit_cnt[3:0] == 4'd13));
	
assign go_error_frame 	  = error_strobe;
assign error_frame_lim 	  = sample_strobe_q & (error_cnt2 == 3'd7) /* & tx_strobe */;
	
assign go_overload_frame  = (sample_strobe_q & ((~sample_bit) | overload_request) & (rx_eof & (~transmitter)
					     & (eof_cnt == 3'd6) | error_frame_lim | overload_frame_lim)	 |	//Regel 1 und Regel 3
			     sample_strobe_q & (~sample_bit) & rx_inter & (bit_cnt[1:0] < 2'd2)			 |	//Regel 2
			     sample_strobe_q & (~sample_bit) & ((error_cnt2 == 3'd7) | (overload_cnt2 == 3'd7)))	//Regel 3
					     & (~overload_frame_blocked);
assign overload_frame_lim = (overload_cnt2 == 3'd7) & tx_strobe;
	
assign last_bit_of_inter  = rx_inter & (bit_cnt[1:0] == 2'd2);//
assign early_bit_of_inter = rx_inter & (bit_cnt[1:0] != 2'd0);//wird gebraucht da sonst go_tx zu spät kommt (bzw. kurz vorm senden wieder weggenommen wird
assign tx_successful 	  = transmitter & go_rx_inter & (~go_error_frame) & (~error_frame_lim) & (~overload_frame_lim)
					& (~arbitration_lost);//Erfolgreiches Versenden eines Frames
assign rx_successful	  = receiving & go_rx_inter & (~go_error_frame) & (~error_frame_lim)
				      & (~overload_frame_lim); //& (~arbitration_lost); //receiving und arbit_lost bei empfangsversuch ist unmoeglich 
assign receiving	  = (~transmitting | arbitration_field | self_test_mode) & (~rx_idle) & (~rx_inter);


// Verzoegern von error_frame_lim und overload_frame_lim
always @ (posedge can_clk or posedge reset)
begin
	if (reset)
		begin
		error_frame_limv	<= 1'b0;
		overload_frame_limv	<= 1'b0;
		end
	else
		begin
		error_frame_limv	<= error_frame_lim;
		overload_frame_limv	<= overload_frame_lim;
		end
end

	/*Zaehlen der Bits pro Telegrammabschnitt bit_cnt*/
	always @ (posedge can_clk or posedge reset)
	begin
		if (reset)
			bit_cnt <= 7'd0;
			// sample_strobe_q ist in den go_xxx Signalen bereits mit & verknuepft!)
		else if ((go_rx_id1 | go_rx_id2 | go_rx_dlc | go_rx_data | go_rx_crc1 | go_rx_crc2&(data_length>0) |
					go_rx_ack | go_rx_eof | go_rx_inter | go_error_frame | go_overload_frame))
			bit_cnt <= 7'd0;
		else if (sample_strobe_q /*& (~transmitter)*/)
			bit_cnt <= bit_cnt + 1'b1;
		//else
		//	bit_cnt <= 1'b0;
	end
	
	/*Zählen von eof bits eof_cnt*/
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 eof_cnt <= 3'd0;
	  else if (sample_strobe_q)
		 begin
			if (go_rx_inter | go_error_frame | go_overload_frame)
			  eof_cnt <=#tp 3'd0;
			else if (rx_eof)
			  eof_cnt <=#tp eof_cnt + 1'b1;
		 end
	end

//--- alte Rx FSM Start ------------------------------------------------------------------------
	/*Zustand rx_inter (Interframespace)*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_inter <= 1'b0;
	else if (go_rx_idle | go_rx_id1 | go_overload_frame | go_error_frame)
		rx_inter <=#tp 1'b0;
	else if (go_rx_inter)
		rx_inter <=#tp 1'b1;
	end */
	
	/*Zustand rx_idle*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_idle <= 1'b0;
	else if (go_rx_id1 | go_error_frame)
		rx_idle <=#tp 1'b0;
	else if (go_rx_idle)
		rx_idle <=#tp 1'b1;
	end */
	
	/*Zustand rx_id1*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_id1 <= 1'b0;
	else if (go_rx_rtr1 | go_error_frame)
		rx_id1 <=#tp 1'b0;
	else if (go_rx_id1)
		rx_id1 <=#tp 1'b1;
	end */
	
	/*Zustand rx_rtr1*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_rtr1 <= 1'b0;
	else if (go_rx_ide | go_error_frame)
		rx_rtr1 <=#tp 1'b0;
	else if (go_rx_rtr1)
		rx_rtr1 <=#tp 1'b1;
	end */

	/*Zustand rx_ide*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_ide <= 1'b0;
	else if (go_rx_r0 | go_rx_id2 | go_error_frame)
		rx_ide <=#tp 1'b0;
	else if (go_rx_ide)
		rx_ide <=#tp 1'b1;
	end */

	/*Zustand rx_id2*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_id2 <= 1'b0;
	else if (go_rx_rtr2 | go_error_frame)
		rx_id2 <=#tp 1'b0;
	else if (go_rx_id2)
		rx_id2 <=#tp 1'b1;
	end */

	/*Zustand rx_rtr2*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_rtr2 <= 1'b0;
	else if (go_rx_r1 | go_error_frame)
		rx_rtr2 <=#tp 1'b0;
	else if (go_rx_rtr2)
		rx_rtr2 <=#tp 1'b1;
	end */

	/*Zustand rx_r1*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_r1 <= 1'b0;
	else if (go_rx_r0 | go_error_frame)
		rx_r1 <=#tp 1'b0;
	else if (go_rx_r1)
		rx_r1 <=#tp 1'b1;
	end */

	/*Zustand rx_r0*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_r0 <= 1'b0;
	else if (go_rx_dlc | go_error_frame)
		rx_r0 <=#tp 1'b0;
	else if (go_rx_r0)
		rx_r0 <=#tp 1'b1;
	end */

	/*Zustand rx_dlc*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_dlc <= 1'b0;
	else if (go_rx_data | go_rx_crc | go_error_frame)
		rx_dlc <=#tp 1'b0;
	else if (go_rx_dlc)
		rx_dlc <=#tp 1'b1;
	end */

	/*Zustand rx_data vorbereiten*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		go_rx_data <= 1'b0;
	else if (go_rx_data)
		go_rx_data <=#tp 1'b1;
	else
		go_rx_data <=#tp 1'b0;
	end */

	/*Zustand rx_data*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_datar <= 1'b0;
	else if (go_rx_crc | go_error_frame)
		rx_datar <=#tp 1'b0;
	else if (go_rx_data)
		rx_datar <=#tp 1'b1;
	end

	assign rx_data = go_rx_data | rx_datar; */

	/*Zustand rx_crc*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_crc <= 1'b0;
	else if (go_rx_crc_lim | go_error_frame)
		rx_crc <=#tp 1'b0;
	else if (go_rx_crc)
		rx_crc <=#tp 1'b1;
	end */

	/*Zustand rx_crc_lim*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_crc_lim <= 1'b0;
	else if (go_rx_ack | go_error_frame)
		rx_crc_lim <=#tp 1'b0;
	else if (go_rx_crc_lim)
		rx_crc_lim <=#tp 1'b1;
	end */

	/*Zustand rx_ack*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_ack <= 1'b0;
	else if (go_rx_ack_lim | go_error_frame)
		rx_ack <=#tp 1'b0;
	else if (go_rx_ack)
		rx_ack <=#tp 1'b1;
	end */

	/*Zustand rx_ack_lim*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_ack_lim <= 1'b0;
	else if (go_rx_eof | go_error_frame)
		rx_ack_lim <=#tp 1'b0;
	else if (go_rx_ack_lim)
		rx_ack_lim <=#tp 1'b1;
	end */

	/*Zustand rx_eof*/
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		rx_eof <= 1'b0;
	else if (go_rx_inter | go_error_frame | go_overload_frame)
		rx_eof <=#tp 1'b0;
	else if (go_rx_eof)
		rx_eof <=#tp 1'b1;
	end */
//--- alte Rx FSM Ende --------------------------------------------------------------------------------

	// Verzögern von sample_strobe_q
	// crc_stb_en 2 Takte verzoegern um bei DLC = 0 keinen weiteren Impuls mehr zu erzeugen
	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		begin
		crc_stb_en	<= 1'h0;
		sample_strobe_e	<= 1'h0;
		sample_strobe_d	<= 1'h0;
		end
	else if (sample_strobe_q)
		begin
		sample_strobe_d <= sample_strobe_q;
		sample_strobe_e <= sample_strobe_q & crc_enable;
		end
	else
		begin
		crc_stb_en	<= sample_strobe_e;
		sample_strobe_e	<= 1'h0;
		sample_strobe_d <= 1'd0;
		end
	end
	// Verlaengerung von sample_strobe_q
	assign	sample_strobe_l	= sample_strobe_q | sample_strobe_d;
	
	//ID Register befüllen (linksshift)
	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		idy <= 29'h0;
	else if (sample_strobe_q & (rx_id1 | rx_id2) & ((~tx_state & receiving) | self_test_mode))
		idy <=#tp {idy[27:0], sample_bit}; //schrittweises Befüllen von links
	else if (sample_strobe_q & rx_id1 & tx_state & ~self_test_mode)
		idy <=#tp {18'b0, tx_basicID};
	else if (sample_strobe_q & rx_id2 & tx_state & ~self_test_mode)
		idy <=#tp {idy[10:0], tx_extID};
	end
	
	
	// rtr1 bit
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 rtr1 <= 1'b0;
	  else if (sample_strobe_q & rx_rtr1 & (~tx_state | self_test_mode))
		 rtr1 <=#tp sample_bit;
	  else if (sample_strobe_q & rx_rtr1 & tx_state & ~self_test_mode)
		 rtr1 <=#tp tx_rtr;
	  else if (rx_idle)
		 rtr1 <= 1'b0;
	end
	
	
	// rtr2 bit
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 rtr2 <= 1'b0;
	  else if (sample_strobe_q & rx_rtr2 & (~tx_state | self_test_mode))
		 rtr2 <=#tp sample_bit;
	  else if (sample_strobe_q & rx_rtr2 & tx_state & ~self_test_mode)
		 rtr2 <=#tp tx_rtr;
	  else if (rx_idle)
		 rtr2 <= 1'b0;
	end
	
	
	// ide bit
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 ide <= 1'b0;
	  else if (sample_strobe_q & rx_ide & (~tx_state | self_test_mode))
		 ide <=#tp sample_bit;
	  else if (sample_strobe_q & rx_ide & tx_state & ~self_test_mode)
		 ide <=#tp tx_ide;
	  else if (rx_idle)
		 ide <= 1'b0;
	end
	
	
	// Data length
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		begin
		 data_length	<= 4'b0;
		end
	  else if (sample_strobe_q & rx_dlc & (~tx_state | self_test_mode))
		begin
		 data_length	<=#tp {data_length[2:0], sample_bit};
		end
	  else if (sample_strobe_q & rx_dlc & tx_state & ~self_test_mode)
		begin
		 data_length	<=#tp tx_dlc;
		end
	end
	
	// Data length
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		begin
		 dlc0_crc	<= 0;
		 set_dlc0_crc	<= 0;
		end
	  else if (mem_dlc0_crc | crc_stb_en)
		begin
		 dlc0_crc	<=#tp calculated_crc;
		 set_dlc0_crc	<= mem_dlc0_crc;
		end
	  else if (rx_crc)
		 set_dlc0_crc	<= 0;
	end

	
	// Schreiben der Daten in ein Byte-großes Temp-Register
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 tmp_data <= 8'h0;
	  else if (sample_strobe_q & rx_crc & (bit_cnt > 1))	// wechsel zu crc bei laenge 0
		 tmp_data <= 8'h0;
	  else if (sample_strobe_q & rx_data & (~tx_state | self_test_mode))
		 tmp_data <=#tp {tmp_data[6:0], sample_bit};
	end
	
	// Schreiben von data0 fuer ACF
/*	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 data0 <= 4'h0;
	  else if (sample_strobe_q & rx_data & bit_cnt < 4'd4 & (~tx_state | self_test_mode))
		 data0 <=#tp {data0[2:0], sample_bit};
	end  */

//	assign	data0	= rx_data0[7:4];	// high rx_data0 fuer ACF an data0 vom ACF anlegen
	
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 write_data_to_tmp_fifo <= 1'b0;
	  else if (sample_strobe_q & rx_data & (&bit_cnt[2:0]) & (~tx_state | self_test_mode))
		 write_data_to_tmp_fifo <=#tp 1'b1;
	  else
		 write_data_to_tmp_fifo <=#tp 1'b0;
	end
	
	
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 byte_cnt <= 3'h0;
	  else if (write_data_to_tmp_fifo)
		 byte_cnt <=#tp byte_cnt + 1'b1;
	  else if (sample_strobe_q & (go_rx_crc1 | go_rx_crc2&(data_length>0))) 
		 byte_cnt <=#tp 3'h0;
	end
	
	reg	[2:0]	data_byte_cnt;

wire	rst;
assign	rst	= reset | row_ready8;

	always @ (posedge can_clk or posedge rst)
	begin
	  if (rst)
	  begin
		rx_data0 <= 8'h0;
		rx_data1 <= 8'h0;
		rx_data2 <= 8'h0;
		rx_data3 <= 8'h0;
		rx_data4 <= 8'h0;
		rx_data5 <= 8'h0;
		rx_data6 <= 8'h0;
		rx_data7 <= 8'h0; 
	  end
	  else if (write_data_to_tmp_fifo)
	  begin
		case(data_byte_cnt)
		3'd0:rx_data0 <= tmp_data;
		3'd1:rx_data1 <= tmp_data;
		3'd2:rx_data2 <= tmp_data;
		3'd3:rx_data3 <= tmp_data;
		3'd4:rx_data4 <= tmp_data;
		3'd5:rx_data5 <= tmp_data;
		3'd6:rx_data6 <= tmp_data;
		3'd7:rx_data7 <= tmp_data;
		endcase
		//tmp_fifo[byte_cnt] <=#tp tmp_data;//eigentliche schreiben in tmp_fifo
	  end
	end
	
	always @ (posedge can_clk or posedge reset)
	begin
		if (reset)
			data_byte_cnt <= 3'd0;
//		else if (write_data_to_tmp_fifo & (data_byte_cnt == 3'd7))
		else if ((write_data_to_tmp_fifo & (data_byte_cnt == data_length-1)) | rx_idle)
			data_byte_cnt <= 3'd0;
		else if (write_data_to_tmp_fifo)
			data_byte_cnt <= data_byte_cnt + 1'b1;	  
	end
	
	// schreiben des CRC-Registers 
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 crc <= 15'h0;
	  else if (sample_strobe_q & rx_r0)		// Einlesen beginnt bei data_length==0 schon im rx_data
		 crc <= 15'h0;
	  else if (sample_strobe_q & (rx_crc | (rx_data & (bit_cnt==0))))
		 crc <=#tp {crc[13:0], sample_bit};	// linksschift fuellen
	end
	
	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		crc_strobe <= 1'b0;
	else if (sample_strobe_q & rx_crc_lim)
		crc_strobe <= 1'b1;
	else
		crc_strobe <= 1'b0;
	end

//Senden------------------------------------------------------------------------------------------------------------

	can_ibo i_can_ibo
	(
	//von can_bsp
	.tx_data_0		(tx_data0		),
	.tx_data_1		(tx_data1		),
	.tx_data_2		(tx_data2		),
	.tx_data_3		(tx_data3		),
	.tx_data_4		(tx_data4		),
	.tx_data_5		(tx_data5		),
	.tx_data_6		(tx_data6		),
	.tx_data_7		(tx_data7		),
	.tx_dlc			(tx_dlc			),
	.tx_basicID		(tx_basicID		),
	.tx_extID		(tx_extID		),
	.calculated_crc		(dlc0_crc		),
	//an can_bsp
	.inv_tx_data_0		(inv_tx_data_0		),
	.inv_tx_data_1		(inv_tx_data_1		),
	.inv_tx_data_2		(inv_tx_data_2		),
	.inv_tx_data_3		(inv_tx_data_3		),
	.inv_tx_data_4		(inv_tx_data_4		),
	.inv_tx_data_5		(inv_tx_data_5		),
	.inv_tx_data_6		(inv_tx_data_6		),
	.inv_tx_data_7		(inv_tx_data_7		),
	.inv_tx_dlc		(inv_tx_dlc		),
	.inv_basicID		(inv_basicID		),
	.inv_extID		(inv_extID		),
	.inv_calculated_crc	(inv_calculated_crc	)
	);

	always @ (posedge can_clk or posedge reset)
	begin
		if (reset)
			tx_dlc_tmp <= 6'h0;
		else if (tx_dlc[3:0] == 4'h0)
			tx_dlc_tmp <= 6'h1;
		else 
			tx_dlc_tmp <=#tp {tx_dlc[2:0], 3'h0};	//tx_dlc * 8
	end
	
assign basic_chain	= {inv_tx_dlc, 2'h0, tx_rtr, inv_basicID, 1'b0};
assign extended_chain	= {inv_tx_dlc, 2'h0, tx_rtr, inv_extID, 1'b1, 1'b1, inv_basicID, 1'b0};
assign data_chain 	= {inv_tx_data_7, inv_tx_data_6, inv_tx_data_5, inv_tx_data_4, inv_tx_data_3, inv_tx_data_2, inv_tx_data_1, inv_tx_data_0};
//Ermitteln der Datenteillaenge in Bit zum zurcksetzen von tx_pointer
assign lim_data		= tx_dlc[3] ? 6'h3f : ((tx_dlc_tmp) - 1'b1);	//x8-1 da tx_pointer von 0 zaehlt
//assign lim_dat_basic  = tx_dlc[3] ? 6'h3f : ((tx_dlc_tmp[2:0] <<3) - 1'b1);
//assign extended_frame	= tx_dlc[3] ? 6'h3f : ((tx_dlc_tmp[2:0] <<3) - 1'b1);
	
//zuruecksetzen des tx_pointer
assign reset_tx_pointer	= (tx_strobe & (~rx_data) & tx_ide & tx_pointer == 6'd38	)|// arbitration + control fuer extended Frame
			(tx_strobe & (~rx_data) & (~tx_ide) & tx_pointer == 6'd18	)|// arbitration + control fuer basic Frame
			(tx_strobe & rx_data & (tx_dlc>0) & (tx_pointer == lim_data)	)|// Daten 
		//	(tx_strobe & rx_data & (~extended_mode) & tx_pointer == lim_dat	)|// Daten 
			go_rx_crc1 | (go_rx_crc2 & (tx_dlc>0)) | go_error_frame		 |// Daten werden beendet (nur bei tx_dlc > 0)
			(tx_strobe & (~rx_data) & rx_crc & tx_pointer == 6'd14		)|// crc
			(go_rx_idle							)|// Ende des Telegramms
			(config_mode							)|
			(overload_frame							)|
			(error_frame							);

	
	always @ (extended_mode or rx_data or tx_pointer or rx_crc or inv_calculated_crc or
		 tx_ide or inv_tx_data_0 or extended_chain or data_chain or basic_chain or
		 finish_msg or tx_dlc)
	begin
	  if (extended_mode)
		 begin
			if (rx_data & (tx_dlc > 0))  // Daten versenden
			  tx_bit <= data_chain[tx_pointer];
			else if (rx_crc | (rx_data & (tx_dlc == 0)))
			  tx_bit <= inv_calculated_crc[tx_pointer];
			else if (finish_msg)
			  tx_bit <= 1'b1;
			else // Arbitrierungsfeld und Kontrollfeld (Basic oder Extended Frame)
			  begin
				 if (tx_ide) // Extended Frame
					tx_bit <= extended_chain[tx_pointer];
				 else
					tx_bit <= basic_chain[tx_pointer];
			  end
		 end
	  else //Basic Mode
		 begin
			if (rx_data & (tx_dlc > 0))  // Daten versenden
			  tx_bit <= data_chain[tx_pointer];
			else if (rx_crc | (rx_data & (tx_dlc == 0)))
			  tx_bit <= inv_calculated_crc[tx_pointer];
			else if (finish_msg)
			  tx_bit <= 1'b1;
			else // Arbitrierungsfeld und Kontrollfeld (nur Basic Frame)
			  tx_bit <= basic_chain[tx_pointer];
		 end
	end
	
	//inkrementieren des tx_pointer
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 tx_pointer <= 6'h0;
	  else if (reset_tx_pointer)
		 tx_pointer <=#tp 6'h0;
	  else if (go_early_tx | (tx_strobe & (tx_state | go_tx)))
		 tx_pointer <=#tp tx_pointer + 1'b1;
	end
	
	// Zustand tx
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 tx_state <= 1'b0;
	  else if (config_mode | go_rx_inter | error_frame_limz /* | arbitration_lost */)
		 tx_state <=#tp 1'b0;
	  else if (go_tx)
		 tx_state <=#tp 1'b1;
	end
	
	//erzeugen einer Sendeanfrage
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 need_to_tx <= 1'b0;
	  else if (tx_successful | config_mode | (abort_tx & (~transmitting)))
		 need_to_tx <=#tp 1'h0;
	  else if (tx_request & sample_strobe_q)
		 need_to_tx <=#tp 1'b1;
	end

assign go_early_tx = (~listen_only_mode) & need_to_tx & (~tx_state) & (~suspend | (suspend_cnt == 3'h7)) & sample_strobe_q & (~sample_bit) & (rx_idle | last_bit_of_inter);
assign go_tx       = (~listen_only_mode) & need_to_tx & (~tx_state) & (~suspend | (sample_strobe_q & (suspend_cnt == 3'h7))) & (go_early_tx | rx_idle);
	
	//Suspend Transmission setzen
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 suspend <= 1'b0;
	  else if (config_mode | (sample_strobe_q & (suspend_cnt == 3'h7)))
		 suspend <=#tp 1'b0;
	  else if (early_bit_of_inter & transmitter & node_error_passive)
		 suspend <=#tp 1'b1;
	end
	
	//Suspend Transmission Zähler
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 suspend_cnt_enable <= 1'b0;
	  else if (config_mode | (sample_strobe_q & (suspend_cnt == 3'h7)))
		 suspend_cnt_enable <=#tp 1'b0;
	  else if (suspend & sample_strobe_q & last_bit_of_inter)
		 suspend_cnt_enable <=#tp 1'b1;
	end	
	
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 suspend_cnt <= 3'h0;
	  else if (config_mode | (sample_strobe_q & (suspend_cnt == 3'h7)))
		 suspend_cnt <=#tp 3'h0;
	  else if (suspend_cnt_enable & sample_strobe_q)
		 suspend_cnt <=#tp suspend_cnt + 1'b1;
	end
	
	
	//Signalisiert das Senden eines Telegramms
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 transmitter <= 1'b0;
	  else if (go_tx)
		 transmitter <=#tp 1'b1;
	  else if (config_mode | go_rx_idle | suspend & go_rx_id1)
		 transmitter <=#tp 1'b0;
	end

	//Signalisiert generelles Senden (message, error frame oder overload frame). (Verbitet hardsync.)
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 transmitting <= 1'b0;
	  else if (go_error_frame | go_overload_frame | go_tx | send_ack)
		 transmitting <=#tp 1'b1;
	  else if (config_mode | go_rx_idle | (go_rx_id1 & (~tx_state)) | (go_rx_eof & (~tx_state))| (arbitration_lost & tx_state))
		 transmitting <=#tp 1'b0;
	end

	//Signalisiert Ende des Telegramms ab CRC-Lim. werden bis zum Bus-Idle nur noch rezessive Bits gesendet.
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 finish_msg <= 1'b0;
	  else if (go_rx_idle | go_rx_id1 | error_frame | config_mode)
		 finish_msg <=#tp 1'b0;
	  else if (go_rx_crc_lim)
		 finish_msg <=#tp 1'b1;
	end

assign arbitration_field	= rx_id1 | rx_rtr1 | rx_ide | rx_id2 | rx_rtr2;
assign set_arbitration_lost	= arbitration_lost & (~arbitration_blocked);
	
	//Sigalisiert den verlust der Arbitrierung
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 arbitration_lost <= 1'b0;
	  else if (go_rx_idle | error_frame_lim)//je nach implementierung des error moduls
		 arbitration_lost <=#tp 1'b0;
	  else if (transmitter & sample_strobe_q & tx & ~sample_bit & arbitration_field)//tx=1 aber rx=0
		 arbitration_lost <=#tp 1'b1;
	end

	/*always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 arbitration_field_d <=#tp 1'b0;
	  else if (sample_point)
		 arbitration_field_d <=#tp arbitration_field;
	end*/

	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 arbitration_cnt <= 5'h0;
	  else if (sample_strobe_q)
		 if (arbitration_field)
			arbitration_cnt <=#tp arbitration_cnt + 1'b1;
		 else
			arbitration_cnt <=#tp 5'h0;
	end

	//Beinhaltet die ID-Stelle an der die Arbitrierung verloren ging (zur Diagnose)
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 arbitration_lost_capture <= 5'h0;
	  else if (set_arbitration_lost)
		 arbitration_lost_capture <=#tp arbitration_cnt;
	end

	//Blockiert die Arbitrierung nach ausloesen eines Arbitration_lost_IRQ
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 arbitration_blocked <= 1'b0;
	  else if (|arbitration_lost_capture)
		 arbitration_blocked <=#tp 1'b0;
	  else if (set_arbitration_lost)
		 arbitration_blocked <=#tp 1'b1;
	end

	// Einleitung zum Senden des Overloadframes (alte Rx FSM)
	/* Rx Zustand OVERLOAD */
/*	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 overload_frame <= 1'b0;
	  else if (overload_frame_lim | go_error_frame)
		 overload_frame <=#tp 1'b0;
	  else if (go_overload_frame)
		 overload_frame <=#tp 1'b1;
	end */

	//Zaehlt die Bits im Overloadframe
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 overload_cnt1 <= 3'd0;
	  else if (overload_frame_lim | go_error_frame | go_overload_frame)
		 overload_cnt1 <=#tp 3'd0;
	  else if (overload_frame & tx_strobe & (overload_cnt1 < 3'd7))
		 overload_cnt1 <=#tp overload_cnt1 + 1'b1;
	end

//Signalisiert das Ende des ersten Overloadframes (im 7. Bit)
assign overload_flag_over = sample_strobe_q & (overload_cnt1 == 3'd7) & (~enable_overload_cnt2);

	//Ermoeglicht zur Steigenden Bitflanke des sample_bit den overload_cnt2
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 enable_overload_cnt2 <= 1'b0;
	  else if (overload_frame_lim | go_error_frame | go_overload_frame)
		 enable_overload_cnt2 <=#tp 1'b0;
	  else if (overload_frame & (overload_flag_over & sample_bit))
		 enable_overload_cnt2 <=#tp 1'b1;
	end

	//zaehlt die Overloadbits nach dem Overloadframe (max. 6)
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 overload_cnt2 <= 3'd0;
	  else if (overload_frame_lim | go_error_frame | go_overload_frame)
		 overload_cnt2 <=#tp 3'd0;
	  else if (enable_overload_cnt2 & tx_strobe)
		/* Rx Zustand OVERLOAD_LIM */
		 overload_cnt2 <=#tp overload_cnt2 + 1'b1;
	end

	//zaehlt die Overload Anfragen waehrend des Overloadframes
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 overload_request_cnt <= 2'b0;
	  else if (go_error_frame | go_rx_id1)
		 overload_request_cnt <=#tp 2'b0;
	  else if (overload_request & overload_frame)
		 overload_request_cnt <=#tp overload_request_cnt + 1'b1;
	end

	//Nach 12 Bit bzw zwei aufeinanderfolgenden Anfragen wird blockiert
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 overload_frame_blocked <= 1'b0;
	  else if (go_error_frame | go_rx_id1)
		 overload_frame_blocked <=#tp 1'b0;
	  else if (overload_request & overload_frame & overload_request_cnt == 2'h2)   // nach zwei aufeinanderfolgenden Overloadanfragen
		 overload_frame_blocked <=#tp 1'b1;
	end

//Senden des Acknowledgebits
assign send_ack = (~tx_state | self_test_mode) & rx_ack & (~go_error_frame) & (~listen_only_mode);


	//Senden der verschiedenen Framearten
	always @ (config_mode or node_bus_off or tx_state or go_tx or tx_bit or
		send_ack or go_overload_frame or overload_frame or overload_cnt1 or
		go_error_frame or error_frame or error_cnt1 or node_error_passive)
	begin
	  if (config_mode | node_bus_off)				// Reset or node_bus_off
		 tx_next = 1'b1;
	  else
		 begin
			if (go_error_frame | error_frame)		// Transmitting error frame
			  begin
				 if (error_cnt1 < 3'd6)
					begin
					  if (node_error_passive | tx_state)
						 tx_next = 1'b1;	// Error im passive Mode oder beim Senden
					  else
						 tx_next = 1'b0;
					end
				 else
					tx_next = 1'b1;
			  end
			else if (go_overload_frame | overload_frame)	// Transmitting overload frame
			  begin
				 if (overload_cnt1 < 3'd6)//
					tx_next = 1'b0;
				 else
					tx_next = 1'b1;
			  end
			else if (send_ack)				// Acknowledge
			  tx_next = 1'b0;
			else if (go_tx | tx_state)			// Transmitting message
				tx_next = tx_bit;
			else
			  tx_next = 1'b1;
		 end
	end
	
	//Eigentliches Senden der Nachricht
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 tx <= 1'b1;
	  else if (config_mode | listen_only_mode)
		 tx <= 1'b1;
	  else if (tx_strobe)
		 tx <=#tp tx_next;
	end
	
//Error Ausgabe----------------------------------------------------------------------------
//Form Error
assign form_error	= /*sample_strobe_q &*/ ((rx_crc_lim    & (~sample_bit)                 ) |//dominates Bit in CRC_lim
			  ( rx_ack_lim			  & (~sample_bit)                 ) |//dominates Bit in Ack_lim
			  ((eof_cnt < 3'd6)& rx_eof	  & (~sample_bit)& (~transmitter) ) |//dominates Bit in EoF als Reciever
			  ( rx_eof			  & (~sample_bit)&   transmitter  ));//dominates Bit in EoF als Transmitter
//Acknowledge Error
assign ack_error	= rx_ack /*& sample_strobe_q*/ & sample_bit & tx_state;//Ack.-slot bleibt auf 1
//Bit Error
assign bit_error	= (tx_state | error_frame | overload_frame | rx_ack) /*& sample_strobe_q*/ & (tx != sample_bit) & (~bit_error_exc1) & (~bit_error_exc2) & (~bit_error_exc3) & (~bit_error_exc4) & (~bit_error_exc5) & (~bit_error_exc6) & (~config_mode);
assign bit_error_exc1	= tx_state & arbitration_field & tx; //Error im Arbitrationfeld wird ignoriert
assign bit_error_exc2	= rx_ack & tx; //Error im Ack.-Slot wird ignoriert
assign bit_error_exc3	= error_frame & node_error_passive & (error_cnt1 < 3'd7);//Error im passiven Errorfeld wird ignoriert
//Error am Ende des Errorfeldes und des Eof-Feldes werden ignoriert.
assign bit_error_exc4	= (error_frame & (error_cnt1 == 3'd7) & (~enable_error_cnt2)) | (overload_frame & (overload_cnt1 == 3'd7) & (~enable_overload_cnt2));
assign bit_error_exc5	= (error_frame & (error_cnt2 == 3'd7)) | (overload_frame & (overload_cnt2 == 3'd7));
assign bit_error_exc6	= (eof_cnt == 3'd6) & rx_eof & (~transmitter);	
	
//Regel 5 (Fault confinement) --> Bit Error im Active Error Flag oder Overload Flag
assign rule5 = bit_error & ((~node_error_passive) & error_frame & (error_cnt1 < 3'd7)| 
				   overload_frame &            (overload_cnt1 < 3'd7));

	//Regel 3 Ausnahmme 1 - erster Teil (Fault confinement).
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 rule3_exception1_1 <= 1'b0;
	  else if (error_flag_over | rule3_exception1_2)
		 rule3_exception1_1 <=#tp 1'b0;
	  else if (transmitter & node_error_passive & ack_error)
		 rule3_exception1_1 <=#tp 1'b1;
	end


	//Regel 3 Ausnahme 1 - zweiter Teil (Fault confinement).
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 rule3_exception1_2 <= 1'b0;
	  else if (go_error_frame | rule3_exception1_2)
		 rule3_exception1_2 <=#tp 1'b0;
	  else if (rule3_exception1_1 & (error_cnt1 < 3'd7) & sample_strobe_q & (~sample_bit))
		 rule3_exception1_2 <=#tp 1'b1;
	end
	
	//Einleitung zum Senden des Errorframes (vgl. Overload) (alte Rx FSM)
	/* Rx Zustand ERROR */
/*	always @ (posedge can_clk or posedge reset)
	begin
	if (reset)
		error_frame <= 1'b0;
	else if (config_mode || error_frame_lim || go_overload_frame)
		error_frame <=#tp 1'b0;
	else if (go_error_frame)
		error_frame <=#tp 1'b1;
	end */


	//Zaehlt die Bits im Errorframe
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 error_cnt1 <= 3'd0;
	  else if (error_frame_lim | go_error_frame | go_overload_frame)
		 error_cnt1 <=#tp 3'd0;
	  else if (error_frame & tx_strobe & (error_cnt1 < 3'd7))
		 error_cnt1 <=#tp error_cnt1 + 1'b1;
	end


//Signalisiert das Ende des ersten Errorframes (im 7. Bit)
assign error_flag_over = (sample_strobe_q & (error_cnt1 == 3'd7)) & (~enable_error_cnt2);



	//Ermoeglicht zur Steigenden Bitflanke des sample_bit den error_cnt2
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 enable_error_cnt2 <= 1'b0;
	  else if (error_frame_lim | go_error_frame | go_overload_frame)
		 enable_error_cnt2 <=#tp 1'b0;
	  else if (error_frame & (error_flag_over & sample_bit))
		/* Rx Zustand ERROR_LIM */
		 enable_error_cnt2 <=#tp 1'b1;
	end

	//zaehlt die Errorbits nach dem Errorframe (max. 6)
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 error_cnt2 <= 3'd0;
	  else if (error_frame_lim | go_error_frame | go_overload_frame)
		 error_cnt2 <=#tp 3'd0;
	  else if (enable_error_cnt2 & tx_strobe)
		 error_cnt2 <=#tp error_cnt2 + 1'b1;
	end
	
	// verzögertes Zählen dominanter Bits im Overload oder Errorframe
	always @ (posedge can_clk or posedge reset)
   begin
     if (reset)
       delayed_dominant_cnt <= 3'h0;
     else if (enable_error_cnt2 | go_error_frame | enable_overload_cnt2 | go_overload_frame)
       delayed_dominant_cnt <=#tp 3'h0;
     else if (sample_strobe_q & (~sample_bit) & ((error_cnt1 == 3'd7) | (overload_cnt1 == 3'd7)))
       delayed_dominant_cnt <=#tp delayed_dominant_cnt + 1'b1;
   end
	
assign error_capture_code_segment[0] = rx_idle | rx_ide | (rx_id2 & (bit_cnt<6'd13)) | rx_r1 | rx_r0 | rx_dlc | rx_ack | rx_ack_lim | error_frame & node_error_active;
assign error_capture_code_segment[1] = rx_idle | rx_id1 | rx_id2 | rx_dlc | rx_data | rx_data&(data_length>0) | rx_ack_lim | rx_eof | rx_inter | error_frame & node_error_passive;
assign error_capture_code_segment[2] = (rx_id1 & (bit_cnt>6'd7)) | rx_rtr1 | rx_ide | rx_id2 | rx_rtr2 | rx_data&(data_length>0) | rx_r1 | error_frame & node_error_passive | overload_frame | overload_frame_limz;
assign error_capture_code_segment[3] = (rx_id2 & (bit_cnt>6'd4)) | rx_rtr2 | rx_r1 | rx_r0 | rx_dlc | rx_data | rx_data&(data_length>0) | rx_crc | rx_crc_lim | rx_ack | rx_ack_lim | rx_eof | overload_frame;
assign error_capture_code_segment[4] =  rx_data&(data_length>0) | rx_crc_lim | rx_ack | rx_ack_lim | rx_eof | rx_inter | error_frame | error_frame_limz | overload_frame | overload_frame_limz;
assign error_direction		     = ~transmitting;

//CRC-Check-------------------------------------------------------------------------------------------------------------
	
//Anschalten des CRC-Checks (global)
assign go_crc_enable  = hard_sync | go_tx;
assign rst_crc_enable = go_rx_crc1 | go_rx_crc2;
	
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 crc_enable <= 1'b0;
	  else if (rst_crc_enable)
		 crc_enable <=#tp 1'b0;
	  else if (go_crc_enable)
		 crc_enable <=#tp 1'b1;
	end
	
	// Instantiierung des RX CRC module (TX wird ebenfalls in RX-Richtung erstellt)
	/*can_crc i_can_crc_rx 
	(
	.can_clk		(can_clk			),
	.reset			(reset				),
	.data			(sample_bit			),
	.enable			(crc_enable & sample_strobe_q	),
	.initialize		(go_crc_enable			),
	.crc			(calculated_crc			),
	.crc_error		(crc_error			), 
	.error_frame_lim	(error_frame_lim		),
	.config_mode		(config_mode			),
	.crc_strobe		(crc_strobe			),
	.crc_reg		(crc				)
	);*/
	
//assign no_byte0 = rtr1 | (data_length<4'h1);
//assign no_byte1 = rtr1 | (data_length<4'h2);
	
//zeigt ob ein Frame daten enthaelt fuer den ACF
assign no_byte0	  = rtr_frame_rx | (data_length<4'h1);
	
	// ID ok signal generation
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
	  begin
		 id_ok <= 1'b0;
		 match <= 0;
	  end
	  else if (go_rx_crc_lim)              // sample_strobe_q is already included in go_rx_crc_lim
		 begin
			if (extended_mode)
			  begin
				if (ide)				// extended frame message
					begin
						id_ok <=#tp (~(|acf_en)) | (|(match_ext & acf_en));
						match <= match_ext;
					end
				else					// standard frame message
					begin
						id_ok <=#tp (~(|acf_en)) | (|(match_std & acf_en));
						match <= match_std;
					end
			  end
			else
			begin
			  id_ok <=#tp (~(|acf_en)) | (|(match_basic & acf_en));
			  match <= match_basic;
			end
		 end
	  else if (config_mode | go_rx_inter | go_error_frame)        // sample_strobe_q is already included in go_rx_inter
		 id_ok <=#tp 1'b0;
	end

/**
 * neue Rx FSM zur Realisierung der Zustaende auf Seite 54 der Diplomarbeit
 */
always @(posedge can_clk or posedge reset) 
	if(reset | config_mode)
		state <= RX_IDLE;
	else if (clk_fsm)
		state <= next_state;

always @(*)
begin
	next_state		= state;

	case(state)
	RX_IDLE:
		begin
			if(go_rx_id1)
				next_state = RX_ID1;
		end

	RX_ID1:
		begin
			if(go_rx_rtr1)
				next_state = RX_RTR1;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_RTR1:
		begin
			if(go_rx_ide)
				next_state = RX_IDE;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_IDE:
		begin
			if(go_rx_id2)			// extended ID
				next_state = RX_ID2;
			if(go_rx_r0)			// normaler ID
				next_state = RX_R0;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_ID2:
		begin
			if(go_rx_rtr2)
				next_state = RX_RTR2;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_RTR2:
		begin
			if(go_rx_r1)
				next_state = RX_R1;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_R1:
		begin
			if(go_rx_r0)
				next_state = RX_R0;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_R0:
		begin
			if(go_rx_dlc)
				next_state = RX_DLC;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_DLC:
		begin
			if(go_rx_data)			// Daten einlesen
				next_state = RX_DATA;
			if(go_rx_crc1)			// Daten ueberspringen (Remoteframe)
				next_state = RX_CRC;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_DATA:
		begin
			if(go_rx_crc2)			// Bricht bei Laenge = 0 schon nach dem 1. Bit ab
				next_state = RX_CRC;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_CRC:
		begin
			if(go_rx_crc_lim)
				next_state = RX_CRC_LIM;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_CRC_LIM:
		begin
			if(go_rx_ack)
				next_state = RX_ACK;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_ACK:
		begin
			if(go_rx_ack_lim)
				next_state = RX_ACK_LIM;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_ACK_LIM:
		begin
			if(go_rx_eof)
				next_state = RX_EOF;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_EOF:
		begin
			if(go_rx_inter)
				next_state = RX_INTERM;
			if(go_overload_frame)
				next_state = RX_OVERL;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_INTERM:
		begin
			if(go_rx_idle)
				next_state = RX_IDLE;
			if(go_overload_frame)
				next_state = RX_OVERL;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	RX_ERROR:
		begin
			if(error_frame_lim)
				next_state = RX_ERROR_LIM;
		end

	RX_ERROR_LIM:
		begin
			if(~error_frame_limv)
				next_state = RX_IDLE;
		end

	RX_OVERL:
		begin
			if(overload_cnt1 == 3'd7)
				next_state = RX_OVERL_LIM;
		end

	RX_OVERL_LIM:
		begin
			if(overload_cnt2 == 3'd7)
				next_state = RX_IDLE;
			if(go_error_frame)
				next_state = RX_ERROR;
		end

	default:	next_state = RX_IDLE;
	endcase
end
// Signale zur Kennzeichnung der Zustaende
// auf diese Art bleibt es nach der Syntese bei der OneHot Kodierung
assign	rx_idle			= state[0];
assign	rx_id1			= state[1];
assign	rx_rtr1			= state[2];
assign	rx_ide			= state[3];
assign	rx_id2			= state[4];
assign	rx_rtr2			= state[5];
assign	rx_r1			= state[6];
assign	rx_r0			= state[7];
assign	rx_dlc			= state[8];
assign	rx_data			= state[9];
assign	rx_crc			= state[10];
assign	rx_crc_lim		= state[11];
assign	rx_ack			= state[12];
assign	rx_ack_lim		= state[13];
assign	rx_eof			= state[14];
assign	rx_inter		= state[15];
assign	error_frame		= state[16];
assign	error_frame_limz	= state[17];
assign	overload_frame		= state[18];
assign	overload_frame_limz	= state[19];

endmodule
