/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  CMD Interface                                              ////
////  Diese Modul schreibt und liest die Peripherie-Register     ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none

module can_cmd_interface(
	clk_peri,
	can_clk,
	sample_strobe_q,
	reset,

	//Von can_bsp und can_emu
	tx_successful,
	//rx_successful,
	//error_strobe,
	transmitting,
	receiving,
	bus_free,
	node_error_passive,
	node_bus_off,
	error_warning,

	tx_start_adr,
	rx_start_adr,

	error_capture_code,
	error_irq,

	can_mode, 
	abort_tx,
	//en_exception_ir,
	//en_tx_successful_ir,
	//en_rx_successful_ir,
	overload_request,

	extended_mode,

	//Ausgabe an can_emu
	we_error_cnt,
	clear_we_error_cnt,
	data_rx_error_cnt,
	data_tx_error_cnt,
	rx_error_cnt,
	tx_error_cnt,
	error_warning_limit,
	rst_error_code_reg,

	//Ausgabe an can_acf
	we_acf,
	clear_we_acf,
	acf_en,
	acf_adr, 
	ac_e_id,
	am_e_id,
	ac_b_id,
	am_b_id,
	ac_rtr1,
	am_rtr1,
	ac_rtr2,
	am_rtr2,
	ac_data,
	am_data,

	//Ausgabe an can_btl
	baudrate_presc,
	sync_jump_width,
	time_segment1,
	time_segment2,
	sample_mode,

	wr_peri,	//Write Enable vom SpartanMC
	activ_peri,	//aktiviere Peripherieregister

	data_in,	// Datenbus
	data_out,	// Datenbus
	adr,		// Adressbus

	//Ausgabe an can_tx_buffer can_rx_buffer
	//retry_tx,
	tx_request_buffer,
	rx_used_buffer,
	rx_buffer_full,
	send_failed,
	rx_data_written,

	set_exception_ir,
	exception_ir,
	tx_successful_ir,
	rx_successful_ir
);

parameter TX_MAILBOX	= 18; //1..18
parameter RX_MAILBOX	= 18; //1..18
parameter ACF_CHAINS	= 18; //1..18
parameter BASE_ADR	= 10'h3f0;	// Adresse muss durch 16 teilbar sein!

parameter ALL_PORTS	= 15;
parameter DECODSIM_DEB	= "NO";

input				clk_peri;
input				can_clk;
input				sample_strobe_q;
input				reset;

//Von can_bsp und can_emu
input				tx_successful;
//input				rx_successful;
//input				error_strobe;
//input	[4:0]			tx_buffer_adr;
//input	[4:0]			rx_buffer_adr;
input wire			transmitting;
input wire			receiving;
input wire			bus_free;
input wire			node_error_passive;
input wire			node_bus_off;
input wire			error_warning;
input wire	[8:0]	error_capture_code;
input wire				error_irq;

//Signal von can_tx_buffer can_rx_buffer
input wire	[9:0]	rx_start_adr;
input wire	[9:0]	tx_start_adr;
input wire			rx_buffer_full;
input wire			send_failed;
input wire			rx_data_written;

output				set_exception_ir;
output				exception_ir;
output				tx_successful_ir;
output				rx_successful_ir;

//Ausgabe an can_bsp
output	[1:0]			can_mode; //alter reset_mode
output				abort_tx;
//output			en_exception_ir;
//output			en_tx_successful_ir;
//output			en_rx_successful_ir;
output				overload_request;

output				extended_mode;

//Eingaben und Ausgabe mit can_emu
output				we_error_cnt;
input				clear_we_error_cnt;
//output			we_rx_error_cnt;
output	[7:0]			data_rx_error_cnt; 
output	[7:0]			data_tx_error_cnt;
input wire	[8:0]	tx_error_cnt;
input wire	[8:0]	rx_error_cnt;
output	[7:0]			error_warning_limit;
output				rst_error_code_reg;

//Ausgabe an can_acf
output				we_acf;
input				clear_we_acf;
output	[(ACF_CHAINS-1):0]	acf_en;		// Freigabe der Filter zur Bildung von id_ok
output	[4:0]			acf_adr; 
output	[17:0]			ac_e_id;
output	[17:0]			am_e_id;
output	[10:0]			ac_b_id;
output	[10:0]			am_b_id;
output				ac_rtr1;
output				am_rtr1;
output				ac_rtr2;
output				am_rtr2;
output	[3:0]			ac_data;
output	[3:0]			am_data;

//Ausgabe an can_btl
output	[6:0]			baudrate_presc;
output	[1:0]			sync_jump_width;
output	[4:0]			time_segment1;
output	[2:0]			time_segment2;
output				sample_mode;

//Ausgabe an can_tx_buffer can_rx_buffer
//output	[5:0]		retry_tx;
output  [TX_MAILBOX-1:0]	tx_request_buffer;
output  [RX_MAILBOX-1:0]	rx_used_buffer;

/**
 * Register zur Synchronisation mit clk_peri fuer Input Signale die mit CAN_CLK entstehen
 */
// von can_bsp und can_emu
reg				tx_successful_s;
reg				transmitting_s;
reg				receiving_s;
reg				bus_free_s;
reg				node_error_passive_s;
reg				node_bus_off_s;
reg				error_warning_s;
reg	[8:0]			error_capture_code_s;
reg				error_irq_s;

// von can_tx_buffer can_rx_buffer
//reg	[9:0]			rx_start_adr_s;
//reg	[9:0]			tx_start_adr_s;
reg				rx_buffer_full_s;
reg				send_failed_s;
reg				rx_data_written_s;
// von can_emu
reg				clear_we_error_cnt_s;
// von can_acf
reg				clear_we_acf_s;
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

wire				set_exception_ir;
reg				set_exception_ir_s;
reg				set_exception_ir_q;
reg				exception_ir;
reg				tx_successful_ir;
reg				rx_successful_ir;

//Ausgabe an can_bsp
reg	[1:0]			can_mode; //alter reset_mode
reg	[1:0]			can_mode_c;
reg	[1:0]			can_mode_cc;
reg				abort_tx;
reg				en_exception_ir;
reg				en_tx_successful_ir;
reg				en_rx_successful_ir;
reg				overload_request;

reg				extended_mode;

//Ausgabe an can_emu
//reg				we_tx_error_cnt;
//reg				we_rx_error_cnt;
reg	[7:0]			error_warning_limit;
reg	[8:0]			data_rx_error_cnt; 
reg	[8:0]			data_tx_error_cnt;

//Ausgabe an can_acf (Zwischenbuffer :( )
reg	[(ACF_CHAINS-1):0]	acf_en;		// Freigabe der Filter zur Bildung von id_ok
reg	[4:0]			acf_adr; //we_acf
reg	[17:0]			ac_e_id;
reg	[17:0]			am_e_id;
reg	[10:0]			ac_b_id;
reg	[10:0]			am_b_id;
reg				ac_rtr1;
reg				am_rtr1;
reg				ac_rtr2;
reg				am_rtr2;
reg	[3:0]			ac_data;
reg	[3:0]			am_data;

//Ausgabe an can_btl
reg	[6:0]			baudrate_presc;
reg	[1:0]			sync_jump_width;
reg	[4:0]			time_segment1;
reg	[2:0]			time_segment2;
reg				sample_mode;

//Ausgabe an can_tx_buffer can_rx_buffer
//reg	[5:0]			retry_tx;
reg	[TX_MAILBOX-1:0]	tx_request_buffer;
reg	[RX_MAILBOX-1:0]	rx_used_buffer;

//error_code
reg				rx_buffer_full_q;
reg				send_failed_q;
reg				node_bus_off_q;
reg				node_error_passive_q;
reg				error_warning_q;

reg				we_acf;
wire				clear_we_acf;
reg				we_error_cnt;
wire				clear_we_error_cnt;
wire				rst_error_code_reg;
wire				write_error_code_reg;
reg				reset_error_code_reg;

wire				tx_successful;
//wire				rx_successful;

//Lesen/Schreiben der Peripherieregister--------------------------------------------------------------
input wire			wr_peri;	//Write Enable vom SpartanMC
input wire			activ_peri;	//aktiviere Peripherieregister

input wire	[17:0]		data_in;	// Datenbus
output wire	[18-1:0]	data_out;	// Datenbus
input wire	[9:0]			adr;		// Adressbus

wire				clk_peri;
wire				can_clk;
wire				sample_strobe_q;
wire				reset;

//Anlegen des MUX-Moduls zum ansprechen der Register
//Dekodierung der CAN Register Basis Adresse.

parameter	WORK_REG		=0;	//0
parameter	CONFIG_REG		=1;	//2
parameter	BUSTIMING_REG		=2;	//4
parameter	ERROR_CNT_REG		=3;	//6
parameter	ERROR_WARNING_REG	=4;	//8
parameter	ACF_SELECT_ADR		=5;	//10
parameter	ACF_DATA1		=6;	//12
parameter	ACF_MASK1		=7;	//14
parameter	ACF_DATA2		=8;	//16
parameter	ACF_MASK2		=9;	//18
parameter	ERROR_CODE_REG		=10;	//20
parameter	TX_ADR_REG		=11;	//22
parameter	RX_ADR_REG		=12;	//24
parameter	TX_BUFFER_REG		=13;	//26
parameter	RX_BUFFER_REG		=14;	//28

// Dekodierung der Adressen ********************** START *************************************
// chipselect fuer die Statusregister
wire					work;			// Lesen
wire					choose_config;		// Lesen
wire					bustiming;		// Lesen
wire					acf_select;		// Lesen
wire					acf_ac1;		// Lesen
wire					acf_am1;		// Lesen
wire					acf_ac2;		// Lesen
wire					acf_am2;		// Lesen
wire					error_cnt;		// Lesen
wire					warn_state;		// Lesen
wire					error_code;		// Lesen
wire					tx_succ_state;		// Lesen
wire					rx_succ_state;		// Lesen
wire					tx_buffer;		// Lesen
wire					rx_buffer;		// Lesen

wire					work_wr;		// Schreiben
wire					choose_config_wr;	// Schreiben
wire					bustiming_wr;		// Schreiben
wire					acf_select_wr;		// Schreiben
wire					acf_ac1_wr;		// Schreiben
wire					acf_am1_wr;		// Schreiben
wire					acf_ac2_wr;		// Schreiben
wire					acf_am2_wr;		// Schreiben
wire					error_cnt_wr;		// Schreiben
wire					warn_state_wr;		// Schreiben
wire					read_rx_successful_reg;	// Schreiben
wire					read_tx_successful_reg;	// Schreiben
wire					tx_buffer_wr;		// Schreiben
wire					rx_buffer_wr;		// Schreiben

wire					select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	adr		),
		.access_peri(	activ_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);

// chipselect fuer die FIFO-,Status- und Steuerregister

//assign adr_low 			= adr[3:0];

assign work 			= reg_read[WORK_REG];		// Lesen
assign choose_config		= reg_read[CONFIG_REG];		// Lesen
assign bustiming 		= reg_read[BUSTIMING_REG];	// Lesen
assign acf_select 		= reg_read[ACF_SELECT_ADR];	// Lesen
assign acf_ac1 			= reg_read[ACF_DATA1];		// Lesen
assign acf_am1			= reg_read[ACF_MASK1];		// Lesen
assign acf_ac2			= reg_read[ACF_DATA2];		// Lesen
assign acf_am2			= reg_read[ACF_MASK2];		// Lesen
assign error_cnt		= reg_read[ERROR_CNT_REG];	// Lesen
assign warn_state		= reg_read[ERROR_WARNING_REG];	// Lesen
assign error_code		= reg_read[ERROR_CODE_REG];	// Lesen
assign tx_succ_state		= reg_read[TX_ADR_REG];		// Lesen
assign rx_succ_state		= reg_read[RX_ADR_REG];		// Lesen
assign tx_buffer		= reg_read[TX_BUFFER_REG];	// Lesen
assign rx_buffer		= reg_read[RX_BUFFER_REG];	// Lesen

assign work_wr			= reg_write[WORK_REG];		// Schreiben
assign choose_config_wr		= reg_write[CONFIG_REG];	// Schreiben
assign bustiming_wr		= reg_write[BUSTIMING_REG];	// Schreiben
assign acf_select_wr		= reg_write[ACF_SELECT_ADR];	// Schreiben
assign acf_ac1_wr		= reg_write[ACF_DATA1];		// Schreiben
assign acf_am1_wr		= reg_write[ACF_MASK1];		// Schreiben
assign acf_ac2_wr		= reg_write[ACF_DATA2];		// Schreiben
assign acf_am2_wr		= reg_write[ACF_MASK2];		// Schreiben
assign error_cnt_wr		= reg_write[ERROR_CNT_REG];	// Schreiben
assign warn_state_wr		= reg_write[ERROR_WARNING_REG];	// Schreiben
assign write_error_code_reg	= reg_write[ERROR_CODE_REG];	// Schreiben
assign read_rx_successful_reg	= reg_write[RX_ADR_REG];	// Schreiben
assign read_tx_successful_reg	= reg_write[TX_ADR_REG];	// Schreiben
assign tx_buffer_wr		= reg_write[TX_BUFFER_REG];	// Schreiben
assign rx_buffer_wr		= reg_write[RX_BUFFER_REG];	// Schreiben

// Dekodierung der Adressen ********************** END **************************************


// Daten an den Bus legen (Lesen der Can-Controller Register)------------------------------------------------------------
assign data_out[18-1:0]		= work ?		{can_mode_cc, abort_tx, en_exception_ir, en_tx_successful_ir, en_rx_successful_ir, overload_request,
								transmitting_s, receiving_s, bus_free_s} :
				  choose_config ?	{extended_mode} :
				  bustiming ?		{baudrate_presc, sync_jump_width, time_segment1, time_segment2, 
								sample_mode} :
				  acf_select ?		{acf_en} :
				  acf_ac1 ?		{ac_rtr1, ac_rtr2, ac_data, ((acf_select_wr & (can_mode==2'b11)) | we_acf | clear_we_acf_s), ac_b_id} :
				  acf_am1 ?		{am_rtr1, am_rtr2, am_data, 1'b0, am_b_id} :
				  acf_ac2 ?		{ac_e_id} :
				  acf_am2 ?		{am_e_id} :

				  error_cnt ?		{data_rx_error_cnt[8:0], data_tx_error_cnt[8:0]} :
				  warn_state ?		{error_warning_limit} :
				  error_code ?		{error_warning_s, error_capture_code_s[5], 1'b0, error_capture_code_s[8:6], 3'b0,
							error_capture_code_s[4:0], node_error_passive_s, node_bus_off_s, rx_buffer_full_q,
							send_failed_q} : //read only
				  tx_succ_state ?	{tx_start_adr} : //read only
				  rx_succ_state ?	{rx_start_adr} : //read only todo muss noch gelatched werden
				  tx_buffer ?		{tx_request_buffer} :
				  rx_buffer ?		{rx_used_buffer} : 18'h0000;

/**
 * Synchronisation mit clk_peri fuer Input Signale die mit CAN_CLK entstehen
 */
always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		tx_successful_s		<= 1'b0;
		transmitting_s		<= 1'b0;
		receiving_s		<= 1'b0;
		bus_free_s		<= 1'b0;
		node_error_passive_s	<= 1'b0;
		node_bus_off_s		<= 1'b0;
		error_warning_s		<= 1'b0;
		error_capture_code_s	<= 9'b0;
		error_irq_s		<= 1'b0;

//		rx_start_adr_s		<= 10'b0;	// ist schon synchron mit clk_peri
//		tx_start_adr_s		<= 10'b0;	// ist schon synchron mit clk_peri
		rx_buffer_full_s	<= 1'b0;
		send_failed_s		<= 1'b0;
		rx_data_written_s	<= 1'b0;
		clear_we_error_cnt_s	<= 1'b0;
		clear_we_acf_s		<= 1'b0;
	end
	else 
	begin
		tx_successful_s		<= tx_successful;
		transmitting_s		<= transmitting;
		receiving_s		<= receiving;
		bus_free_s		<= bus_free;
		node_error_passive_s	<= node_error_passive;
		node_bus_off_s		<= node_bus_off;
		error_warning_s		<= error_warning;
		error_capture_code_s	<= error_capture_code;
		error_irq_s		<= error_irq;

//		rx_start_adr_s		<= rx_start_adr;	// ist schon synchron mit clk_peri
//		tx_start_adr_s		<= tx_start_adr;	// ist schon synchron mit clk_peri
		rx_buffer_full_s	<= rx_buffer_full;
		send_failed_s		<= send_failed;
		rx_data_written_s	<= rx_data_written;
		clear_we_error_cnt_s	<= clear_we_error_cnt;
		clear_we_acf_s		<= clear_we_acf;
	end
end
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		send_failed_q		<= 1'b0;
	else if (send_failed_s /* | (send_failed_s & rst_error_code_reg) */)		// ????
		send_failed_q		<= 1'b1;
	else if (rst_error_code_reg)
		send_failed_q		<= 1'b0;
end

always @ (posedge clk_peri or posedge reset) //verzoegert nur Signal Interruptursache muss aber beseitigt werden
begin
	if (reset)
		rx_buffer_full_q	<= 1'b0;
	else if (rx_buffer_full_s & sample_strobe_q /* | (rx_buffer_full_s & rst_error_code_reg) */)	// ????
		rx_buffer_full_q	<= 1'b1;
	else if (rst_error_code_reg)
		rx_buffer_full_q	<= 1'b0;
end

assign	set_exception_ir	= ~rst_error_code_reg & (error_irq_s | (error_warning_s & ~error_warning_q)
			| (node_error_passive_s & ~node_error_passive_q) | (node_bus_off_s & ~node_bus_off_q)
			| rx_buffer_full_q | send_failed_q);

// nur synchron zum can_clk koennen Fehler entstehen
always @ (posedge can_clk or posedge reset)
begin
	if (reset)
		begin
		set_exception_ir_s	<= 1'b0;
		end
	else if (en_exception_ir & set_exception_ir)
		set_exception_ir_s	<= 1'b1;
	else if (rst_error_code_reg & ~set_exception_ir)
		set_exception_ir_s	<= 1'b0;
end

always @ (posedge clk_peri or posedge reset) //verzoegert Signale fuer exc_ir zur Flankenabtastung
begin
	if (reset)
	begin
		node_bus_off_q		<= 1'b0;
		node_error_passive_q	<= 1'b0;
		error_warning_q		<= 1'b0;
		set_exception_ir_q	<= 1'b0;
	end
	else 
	begin
		node_bus_off_q		<= node_bus_off_s;
		node_error_passive_q	<= node_error_passive_s;
		error_warning_q		<= error_warning_s;
		set_exception_ir_q	<= set_exception_ir_s;
	end
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		begin
		exception_ir		<= 1'b0;
		reset_error_code_reg	<= 1'b0;
		end
	else if (write_error_code_reg)
		begin
		exception_ir		<= 1'b0;
		reset_error_code_reg	<= 1'b1;
		end
	else if (en_exception_ir & set_exception_ir_s & ~set_exception_ir_q)
		exception_ir		<= 1'b1;
	else
		reset_error_code_reg	<= 1'b0;
end

assign	rst_error_code_reg = write_error_code_reg | reset_error_code_reg;	// 2 Takte Reset

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		tx_successful_ir	<= 1'b0;
	else if (en_tx_successful_ir & ~read_tx_successful_reg & tx_successful_s)
		tx_successful_ir	<= 1'b1;
	else if (read_tx_successful_reg)
		tx_successful_ir	<= 1'b0;
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		rx_successful_ir	<= 1'b0;
	else if (en_rx_successful_ir & ~read_rx_successful_reg & rx_data_written_s)
		rx_successful_ir	<= 1'b1;
	else if (read_rx_successful_reg)
		rx_successful_ir	<= 1'b0;
end

//Schreiben der Can-Controller Register (von SpartanMC aus)---------------------------------------------------------------------------------
always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		can_mode 		<= 2'b11;	// CONFIG Mode einstellen
		abort_tx 		<= 1'b0;
		en_exception_ir		<= 1'b0;	// bei RESET alle Interrups sperren
		en_tx_successful_ir	<= 1'b0;	// bei RESET alle Interrups sperren
		en_rx_successful_ir	<= 1'b0;	// bei RESET alle Interrups sperren
		overload_request	<= 1'b0;
	end
	else if (work_wr)
	begin
		can_mode 		<= data_in[9:8];
		abort_tx 		<= data_in[7];
		en_exception_ir		<= data_in[6];
		en_tx_successful_ir	<= data_in[5];
		en_rx_successful_ir	<= data_in[4];
		overload_request	<= data_in[3];
	end
end

// mit can_clk synchronisieren um auf die Umschaltung warten zu können
always @ (posedge can_clk or posedge reset)
begin
	if (reset)
	begin
		can_mode_c 		<= 2'b11;	// CONFIG Mode einstellen
		can_mode_cc 		<= 2'b11;	// CONFIG Mode einstellen
	end
	else
	begin
		can_mode_c 		<= can_mode;
		can_mode_cc 		<= can_mode_c;
	end
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		extended_mode		<= 1'b1;
	end
	else if (choose_config_wr & (can_mode==2'b11))
	begin
		extended_mode		<= data_in[0];
	end
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		baudrate_presc		<= 6'h6;
		sync_jump_width		<= 2'h1;
		time_segment1		<= 5'hb;
		time_segment2		<= 3'h2;
		sample_mode		<= 1'h0;
	end
	else if (bustiming_wr & (can_mode==2'b11))
	begin
		baudrate_presc		<= data_in[17:11];
		sync_jump_width		<= data_in[10:9];
		time_segment1		<= data_in[8:4];
		time_segment2		<= data_in[3:1];
		sample_mode		<= data_in[0];
	end
end

//assign we_acf = acf_select_wr & (can_mode==2'b11);

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		acf_adr			<= 5'h0;
		we_acf			<= 1'b0;
	end
	else if (acf_select_wr & (can_mode==2'b11))
	begin
		acf_adr			<= data_in[4:0];
		we_acf			<= 1'b1;
	end
	else if (clear_we_acf_s)
		we_acf			<= 1'b0;
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		acf_en			<= 0;	// kein Filter fuer id_ok freigegeben
	end
	else if (acf_select_wr & (can_mode!=2'b11))
	begin
		acf_en			<= data_in[(ACF_CHAINS-1):0];
	end
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		ac_b_id			<= 11'h0;
		ac_rtr1			<= 1'h0;
		ac_rtr2			<= 1'h0;
		ac_data			<= 4'h0;
	end
	else if (acf_ac1_wr & (can_mode==2'b11))
	begin
		ac_b_id			<= data_in[10:0];
		ac_rtr1			<= data_in[17];
		ac_rtr2			<= data_in[16];
		ac_data			<= data_in[15:12];
	end
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		am_b_id			<= 11'h1;
		am_rtr1			<= 1'h1;
		am_rtr2			<= 1'h1;
		am_data			<= 4'h1;
	end
	else if (acf_am1_wr & (can_mode==2'b11))
	begin
		am_b_id			<= data_in[10:0];
		am_rtr1			<= data_in[17];
		am_rtr2			<= data_in[16];
		am_data			<= data_in[15:12];
	end
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		ac_e_id			<= 18'h0;
	else if (acf_ac2_wr & (can_mode==2'b11))
		ac_e_id			<= data_in[17:0];
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		am_e_id			<= 18'h1;
	else if (acf_am2_wr & (can_mode==2'b11))
		am_e_id			<= data_in[17:0];
end

//assign we_error_cnt = error_cnt_wr;

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		data_rx_error_cnt	<= 9'h0;
		data_tx_error_cnt	<= 9'h0;
		we_error_cnt		<= 1'b0;
	end
	else if (error_cnt_wr)
	begin
		data_rx_error_cnt[8:0]	<= {1'b0,data_in[16:9]};
		data_tx_error_cnt[8:0]	<= {1'b0,data_in[7:0]};
		we_error_cnt		<= 1'b1;
	end
	else if (clear_we_error_cnt_s)
	begin
		we_error_cnt		<= 1'b0;
	end
	else
	begin	// zuruecklesen der aktuellen Zaehler im can_emu und mit clk_peri synchronisieren
		data_rx_error_cnt	<= rx_error_cnt;
		data_tx_error_cnt	<= tx_error_cnt;
	end
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		error_warning_limit	<= 8'd96;	//empfohlen von BOSCH
	else if (warn_state_wr)
		error_warning_limit	<= data_in[7:0];
end

//reg [9:0] k;

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		tx_request_buffer	<= 0;
		//j <= 10'h0;
	end
	else if (tx_buffer_wr)
		tx_request_buffer	<= data_in[(TX_MAILBOX-1):0];
	else if (tx_successful_s)
	begin
		//j <= (tx_start_adr>>3);
		tx_request_buffer[(TX_MAILBOX-1)-(tx_start_adr>>3)]		<= 1'b0;
	end 
end

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
		rx_used_buffer		<= 0;
		//k <= 10'h0;
	end
	else if (rx_buffer_wr)
		rx_used_buffer		<= data_in[(RX_MAILBOX-1):0];
	else if (rx_data_written_s)
	begin
		//k <= ((rx_start_adr-(TX_MAILBOX-1))>>3);
		// Die Rx Puffer beginnen jetzt immer hinter dem 18. Tx Puffer auch wenn es weniger als 18 Tx Puffer gibt
		rx_used_buffer[(RX_MAILBOX-1)-((rx_start_adr>>3)-18)]		<= 1'b1;
	end
end

//Ruecksetzen der TX/RX-Bufferregister wird nicht synthetisiert
/*reg [9:0] k,j;

always @ (posedge clk)
begin
	if (rx_successful)
	begin
		k <= ((rx_start_adr-(TX_MAILBOX-1))>>3);
		rx_used_buffer[k]			<= 1'b1;
	end
end

always @ (posedge clk_peri)
begin
	if (tx_successful)
	begin
		j <= (tx_start_adr>>3);
		tx_request_buffer[j]		<= 1'b0;
	end
end*/

endmodule

