/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
/////////////////////////////////////////////////////////////////////
////                                                             ////
////  CRC                                                        ////
////  Berechnet die CRC-Summe                                    ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

// synopsys translate_off
`include "can/src/timescale.vh"
// synopsys translate_on

module can_crc(
	input wire				can_clk,
	input wire		  		reset,
	input wire				data,
	input wire				enable,
	input wire				initialize,
	input wire				error_frame_lim,
	input wire				config_mode,
	input wire				crc_strobe,
	input wire	[14:0]		crc_reg,
	input wire	[14:0]		dlc0_crc,
	input wire				set_dlc0_crc,
	output reg	[14:0]		crc,
	output reg				crc_error
	);

parameter tp = 1;


wire    		      	crc_next;
wire	[14:0]			crc_tmp;


	assign crc_next = data ^ crc[14];
	assign crc_tmp = {crc[13:0], 1'b0};
	
	always @ (posedge can_clk or posedge reset)
	if (reset)
	begin
	    crc <= 15'h0;   
	end
	else
	begin
	  if (initialize | config_mode)
		 crc <= #tp 15'h0; 
	  else if (enable)
		 begin
			if (crc_next)
			  crc <= #tp crc_tmp ^ 15'h4599;
			else
			  crc <= #tp crc_tmp;

		 end
	  else if (set_dlc0_crc)	// data_length == 0
			  crc <= #tp dlc0_crc;   
	end
	
	// CRC error generation
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 crc_error <= 1'b0;
	  else if (config_mode | error_frame_lim)
		 crc_error <=#tp 1'b0;
	  else if (crc_strobe)
		 crc_error <=#tp dlc0_crc != crc_reg;
	//	 crc_error <=#tp crc != crc_reg;
	//	 crc_error <= 1'b0;
	  else
	    crc_error <= 1'b0;
	end
	
endmodule
