/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  EMU                                                        ////
////  Fuehrt die Fehlerbehandlung lt. CAN-Protokoll aus.         ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none
// synopsys translate_off
`include "can/src/timescale.vh"
// synopsys translate_on

module can_emu(
	can_clk,
	reset,
	//Errorarten von:
	//BSP
	bit_error,
	form_error,
	ack_error,
	error_capture_code_segment,
	error_direction,
	sample_strobe,
	sample_strobe_q,
	sample_bit,
	go_overload_frame,
	error_frame_lim,
	error_flag_over,
	transmitter,
	//transmitting,
	tx_request,
	arbitration_lost,
	go_rx_ack_lim,
	go_error_frame,
	error_frame,
	rule5,
	rule3_exception1_2,
	//rx_idle,
	error_cnt1,
	arbitration_field,
	tx_successful,
	tx,
	delayed_dominant_cnt,
	
	//CRC
	crc_error,
	//BSL
	stuff_error,
	//Kommandos
	config_mode,
	listen_only_mode,
	extended_mode,
	error_warning_limit,	//hhe des Errorzaehler bei einer Errorwarnung
	data_rx_error_cnt,
	data_tx_error_cnt,
	rx_error_cnt,
	tx_error_cnt,
	we_error_cnt,		// ist Synchron zu clk_peri
	clear_we_error_cnt,
	read_error_code_capture_reg,
	exception_ir,
	
	//Ausgabe an BSP
	error_strobe,
	node_error_passive,
	node_bus_off,
	bus_free,
	node_error_active,
	
	//Ausgabe an Spartan IRQ-Controller
	error_irq,
	error_warning,
	error_capture_code,
	bit_error_latched,
	form_error_latched,
	stuff_error_latched,
	ack_error_latched
	);

	parameter tp = 1;
	
	input wire         				can_clk;
	input wire         				reset;
	//Kommandos
	input wire					extended_mode;
	input wire					config_mode;
	input wire	[7:0]				data_rx_error_cnt;
	input wire	[7:0]				data_tx_error_cnt;
	input wire					listen_only_mode;
	input wire	[7:0]				error_warning_limit;
	input wire					we_error_cnt;		// ist Synchron zu clk_peri
	output					clear_we_error_cnt;
	input wire					read_error_code_capture_reg;
	//von BSP
	input wire					bit_error;
	input wire					tx_request;
	input wire					form_error;
	input wire					ack_error;
	input wire	[4:0]				error_capture_code_segment;
	input wire					error_direction;
	input wire					sample_strobe;
	input wire					sample_strobe_q;
	input wire					sample_bit;
	input wire					go_overload_frame;
	input wire					error_frame_lim;
	input wire					error_flag_over;
	input wire					transmitter;
	input wire					arbitration_lost;
	input wire					go_rx_ack_lim;
	input wire					go_error_frame;
	input wire					error_frame;
	//input wire					rx_idle;
	input wire					rule5;
	input wire					rule3_exception1_2;
	input wire	[2:0]				error_cnt1;
	input wire					tx_successful;
	input wire					arbitration_field;
	//input wire					transmitting;
	input wire					tx;
	input wire	[2:0]				delayed_dominant_cnt;
	//von CRC
	input wire					crc_error;
	//von BSL
	input wire					stuff_error;
	
	input wire					exception_ir;
	
	//an BSP
	output					error_strobe;
	output					node_error_passive;
	output					node_bus_off;
	output					bus_free;
	output      				node_error_active;
	
	//an Spartan IRQ-Controller
	output					error_irq;
	output					error_warning;		//warnung bei zu hohem Busaufkommen (96 Errors)
	output	[8:0]				error_capture_code;
	output	[8:0]				tx_error_cnt;
	output	[8:0]				rx_error_cnt;
	output					bit_error_latched;
	output					form_error_latched;
	output					stuff_error_latched;
	output					ack_error_latched;

	
	//Fehlerzaehler Register
	reg	[8:0]				tx_error_cnt;
	reg	[8:0]				rx_error_cnt;
	//reg					we_tx_error_cnt;
	//reg					we_rx_error_cnt;
	
	//Error Register
	reg					error_warning;		//warnung bei zu hohem Busaufkommen (96 Errors)
	wire					error_status_on;	//warnung bei zu hohem Busaufkommen (96 Errors)
	wire					error_status_off;	//warnung aus, wenn beide Zaehler <=20 Errors
	reg					bit_error_latched;
	reg					form_error_latched;
	reg					stuff_error_latched;
	reg					ack_error_latched;
	reg					error_flag_over_latched;
	wire					rst_error_latch;
	wire					rule2;
	wire					rule3;
	wire					rule5_stb;
	wire					rule6;
	
	//Error Capture Register
	reg	[8:0]				error_capture_code;
	reg					error_capture_code_blocked;
	reg	[2:0]				error_capture_code_type;
	reg					error_irq;
	
	//Hilfsregister
	reg					node_error_passive;
	reg					node_bus_off;
	reg					bus_free;
	reg	[3:0]				bus_free_cnt;
	reg					bus_free_cnt_enable;
	reg					waiting_for_bus_free;
	reg					node_bus_off_delayed;
	reg					config_mode_delayed;
	reg					clear_we_error_cntrx;
	reg					clear_we_error_cnttx;
	wire					clear_we_error_cnt;
	
	wire					set_bus_error_irq;
	wire					error_strobe;
	wire					node_error_active;
	wire					set_config_mode;
	
	assign error_strobe = /* form_error | stuff_error | bit_error | ack_error | */ crc_error |
								 form_error_latched | stuff_error_latched | 
								 bit_error_latched | ack_error_latched;

	assign	rst_error_latch	= (config_mode | config_mode_delayed | error_frame | go_overload_frame | read_error_code_capture_reg);

	//Latchen der Errors
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 ack_error_latched <= 1'b0;
	  else if (rst_error_latch)
		 ack_error_latched <=#tp 1'b0;
	  else if (ack_error & sample_strobe_q)
		 ack_error_latched <=#tp 1'b1;
	end
	
	
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 bit_error_latched <= 1'b0;
	  else if (rst_error_latch)
		 bit_error_latched <=#tp 1'b0;
	  else if (bit_error & sample_strobe_q) //(bit_error & error_frame)
		 bit_error_latched <=#tp 1'b1;
	end
	
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 stuff_error_latched <= 1'b0;
	  else if (rst_error_latch)
		 stuff_error_latched <=#tp 1'b0;
	  else if (stuff_error & sample_strobe)				// stuff_error kann mit jedem strobe auftreten
		 stuff_error_latched <=#tp 1'b1;
	end
	
	always @ (posedge can_clk or posedge reset)
	begin
	  if (reset)
		 form_error_latched <= 1'b0;
	  else if (rst_error_latch)
		 form_error_latched <=#tp 1'b0;
	  else if (form_error & sample_strobe_q)
		 form_error_latched <=#tp 1'b1;
	end

assign	clear_we_error_cnt	= clear_we_error_cntrx & clear_we_error_cnttx;

assign	rule2	  = (error_flag_over & (~error_flag_over_latched) & (~sample_bit) & (error_cnt1 == 3'd7));
assign	rule3	  = (go_error_frame & (~(transmitter & node_error_passive & ack_error)) & (~(transmitter & stuff_error & 
			arbitration_field & tx & (~sample_bit)))) | (error_frame & rule3_exception1_2);

assign	rule5_stb = rule5 & sample_strobe_q;
assign	rule6	  = (~sample_bit) & (delayed_dominant_cnt == 3'h7);

//rx Errorzaehler	
always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
    begin
    rx_error_cnt <= 9'h0;
    clear_we_error_cntrx <= 1'b0;
    end
  else if (we_error_cnt /*& (~node_bus_off)*/)
    begin
    rx_error_cnt <=#tp {1'b0, data_rx_error_cnt};			//schreibe beliebigen wert auf rx_error_cnt Reg
    clear_we_error_cntrx <= 1'b1;
    end
  else
   begin
     clear_we_error_cntrx <= 1'b0;
     if (set_config_mode)
       rx_error_cnt <=#tp 9'h0;
     else
      begin
        if ((~listen_only_mode) & (~transmitter | arbitration_lost))	//wenn Bus gelesen wird
           begin
             if (go_rx_ack_lim & (~go_error_frame) & (~crc_error) & (rx_error_cnt > 9'h0))//bei bereits gefuellten zaehler und erfolgreichen beendigung eines frames 8.2
               begin
                 if (rx_error_cnt > 9'd127)
                   rx_error_cnt <=#tp 9'd127;				//bei ueber 127 fehlern setze auf zaehler 127 zurueck (119 waere auch moegl.) 8.3
                 else
                   rx_error_cnt <=#tp rx_error_cnt - 1'b1;			//dekrementiere zaehler 8.1
               end
             else if (rx_error_cnt < 9'd128)				//bei kleiner 128 erstelle error gemaess Regeln
               begin
                 if (go_error_frame & (~rule5_stb))			// 1  (rule 5 is just the opposite then rule 1 exception
                   rx_error_cnt <=#tp rx_error_cnt + 1'b1;
                 else if ( (sample_strobe_q & rule2)	|		// 2
                           (go_error_frame & rule5_stb)	|		// 5
                           (sample_strobe_q & rule6)			// 6
                         )
                   rx_error_cnt <=#tp rx_error_cnt + 4'h8;
               end
           end
       end
   end
end

//tx Errorzaehler
always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)						//schreiben des Errorregisters
    begin
    tx_error_cnt <= 9'h0;
    clear_we_error_cnttx <= 1'b0;
    end
  else if (we_error_cnt)
    begin
    tx_error_cnt <=#tp {1'b0, data_tx_error_cnt};			//schreibe beliebigen Wert auf tx_error_cnt
    clear_we_error_cnttx <= 1'b1;
    end
  else
    begin
    clear_we_error_cnttx <= 1'b0;
      if (set_config_mode)
        tx_error_cnt <=#tp 9'd128;
      else if ((tx_error_cnt > 9'd0) & (tx_successful | bus_free))	//(bus_free nach 11x rez. in node_bus_off (oder bei Betreten des config_mode)
									// --> TODO: Zaehlt von 255 runter sollte aber von 128 runterzaehlen)
        tx_error_cnt <=#tp tx_error_cnt - 1'h1;										 //7
      else if (transmitter & (~arbitration_lost))
        begin
          if ( (sample_strobe_q & rule6) |				// 6
               (go_error_frame & rule5_stb) |				// 4  (rule 5 is the same as rule 4)
               (sample_strobe_q & rule3)				// 3
             )
            tx_error_cnt <=#tp tx_error_cnt + 4'h8;
        end
    end
end


always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
    node_error_passive <= 1'b0;
  else if ((rx_error_cnt < 128) & (tx_error_cnt < 9'd128))
    node_error_passive <=#tp 1'b0;
  else if (((rx_error_cnt >= 128) | (tx_error_cnt >= 9'd128)) & (error_frame_lim | go_error_frame | (~config_mode) & config_mode_delayed) & (~node_bus_off)) //9
    node_error_passive <=#tp 1'b1;
end


assign node_error_active = ~(node_error_passive | node_bus_off); //11, 12


always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
    node_bus_off <= 1'b0;
  else if ((rx_error_cnt == 9'h0) & (tx_error_cnt == 9'd0) & (~config_mode) | 
			 (we_error_cnt & (tx_error_cnt < 8'd255) & (rx_error_cnt < 8'd255)))//12
    node_bus_off <=#tp 1'b0;
  else if ((tx_error_cnt >= 9'd256) | (we_error_cnt & (data_tx_error_cnt == 8'd255) & (data_rx_error_cnt == 8'd255))) //10 (auch wenn in tx_error_cnt 255 geschrieben wird)
    node_bus_off <=#tp 1'b1;
end



always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
    bus_free_cnt <= 4'h0;
  else if (sample_strobe_q)
    begin
      if (sample_bit & bus_free_cnt_enable & (bus_free_cnt < 4'd10))//evtuell hier 5 eintragen //10
        bus_free_cnt <=#tp bus_free_cnt + 1'b1;
      else
        bus_free_cnt <=#tp 4'h0;
    end
end


always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
    bus_free_cnt_enable <= 1'b0;
  else if (((~config_mode) & config_mode_delayed) | (node_bus_off_delayed & (~config_mode)) | (error_frame & (delayed_dominant_cnt == 3'h0)))
    bus_free_cnt_enable <=#tp 1'b1;
  else if (sample_strobe_q & sample_bit & (bus_free_cnt==4'd10) & (~node_bus_off))
    bus_free_cnt_enable <=#tp 1'b0;
end


always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
    bus_free <= 1'b0;
  else if (sample_strobe_q & sample_bit & (bus_free_cnt==4'd10) & waiting_for_bus_free)
    bus_free <=#tp 1'b1;
  else
    bus_free <=#tp 1'b0;
end


always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
    waiting_for_bus_free <= 1'b1;
  else if (bus_free & (~node_bus_off))
    waiting_for_bus_free <=#tp 1'b0;
  else if ((node_bus_off_delayed & (~config_mode)) | error_frame_lim | (tx_request & ~transmitter))
    waiting_for_bus_free <=#tp 1'b1;
end


//assign bus_off_on = ~node_bus_off; //wird vielleicht als Output gebraucht

assign set_config_mode	= node_bus_off & (~node_bus_off_delayed);			 //fallende Flanke von node_bus_off setzt reset Mode
assign error_status_on	= extended_mode? ((rx_error_cnt >= error_warning_limit) | (tx_error_cnt >= error_warning_limit)):
                                     ((rx_error_cnt >= 9'd96) | (tx_error_cnt >= 9'd96));//gibt eine warnung bei 96 Fehlern (extended Mode, bei erreichen des Error Warning Regs.) aus
// um staendige Impulse zu vermeiden, erst ausschalten, wenn beide Zähler <=20 sind
assign error_status_off	=  ((rx_error_cnt <= 20) & (tx_error_cnt <= 20));

//assign transmit_status = transmitting  || (extended_mode && waiting_for_bus_free);//merwuerdige Ausgabe
//assign receive_status  = extended_mode ? (waiting_for_bus_free || (!rx_idle) && (!transmitting)) : 
                                         //((!waiting_for_bus_free) && (!rx_idle) && (!transmitting));//merwuerdige Ausgabe

// Die Zaehler koennen nur mit dem can_clk das Limit erreichen
always @ (posedge can_clk or posedge reset)
begin
  if (reset)
	error_warning	<= 0;
  else if (error_status_off)
	error_warning	<= 0;
  else if (error_status_on)
	error_warning	<= 1;
end
/* Error code capture register */
always @ (posedge can_clk or posedge reset)
begin
  if (reset)
    error_capture_code <= 9'h0;
  else if (read_error_code_capture_reg | config_mode)
    error_capture_code <=#tp 9'h0;
  else if (set_bus_error_irq)
    error_capture_code <=#tp {error_capture_code_type, error_direction, error_capture_code_segment};
end

always @ (*)
begin
  if (form_error_latched)
    error_capture_code_type = 3'b001;
  else if (stuff_error_latched)
    error_capture_code_type = 3'b010;
  else if (crc_error)
    error_capture_code_type = 3'b100;
  else if (ack_error_latched)
    error_capture_code_type = 3'b011;
  else if (bit_error_latched)
    error_capture_code_type = 3'b000;
  else
    error_capture_code_type = 3'b111; 
end


assign set_bus_error_irq = go_error_frame & (~error_capture_code_blocked);

//Error_ir erzeugen
always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
    error_irq <= 1'b0;
  else if (set_bus_error_irq)
    error_irq <= 1'b1;
  else 
    error_irq <= 1'b0;
end

always @ (posedge can_clk or posedge reset)
begin
  if (reset)
    error_capture_code_blocked <= 1'b0;
  else if (read_error_code_capture_reg | exception_ir | config_mode)
    error_capture_code_blocked <=#tp 1'b0;
  else if (set_bus_error_irq)
    error_capture_code_blocked <=#tp 1'b1;
end

//Speichern des Error_Flag_over s. BSP
always @ (posedge can_clk or posedge reset)
begin
  if (reset)
	 error_flag_over_latched <= 1'b0;
  else if (error_frame_lim | go_error_frame | go_overload_frame | config_mode)
	 error_flag_over_latched <=#tp 1'b0;
  else if (error_flag_over)
	 error_flag_over_latched <=#tp 1'b1;
end
	
// Erzeugen Verzögerter Signale für config_mode und node_bus_off
always @ (posedge can_clk or posedge reset)
begin
  if (reset)
	 begin
		config_mode_delayed <=#tp 1'b0;
	 end
  else
	 begin
		config_mode_delayed <=#tp config_mode;
	 end
end
	
always @ (posedge can_clk or posedge reset)
begin
  if (reset | config_mode)
	 begin
		node_bus_off_delayed <=#tp 1'b0;
	 end
  else
	 begin
		node_bus_off_delayed <=#tp node_bus_off;
	 end
end
	
endmodule
