/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  IBO                                                        ////
////  Rotiert den Inhalt der Sende-Register vor der Versendung.  ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none

module can_ibo
( 
	input wire		[7:0]			tx_data_0,
	input wire		[7:0]			tx_data_1,
	input wire		[7:0]			tx_data_2,
	input wire		[7:0]			tx_data_3,
	input wire		[7:0]			tx_data_4,
	input wire		[7:0]			tx_data_5,
	input wire		[7:0]			tx_data_6,
	input wire		[7:0]			tx_data_7,
	input wire		[3:0]			tx_dlc,
	input wire		[10:0]			tx_basicID,
	input wire		[17:0]			tx_extID,
	input wire		[14:0]			calculated_crc,

	output wire	[7:0]			inv_tx_data_0,
	output wire	[7:0]			inv_tx_data_1,
	output wire	[7:0]			inv_tx_data_2,
	output wire	[7:0]			inv_tx_data_3,
	output wire	[7:0]			inv_tx_data_4,
	output wire	[7:0]			inv_tx_data_5,
	output wire	[7:0]			inv_tx_data_6,
	output wire	[7:0]			inv_tx_data_7,
	output wire	[3:0]			inv_tx_dlc,
	output wire	[10:0]			inv_basicID,
	output wire	[17:0]			inv_extID,
	output wire	[14:0]			inv_calculated_crc
);



assign inv_tx_data_0[0] = tx_data_0[7];
assign inv_tx_data_0[1] = tx_data_0[6];
assign inv_tx_data_0[2] = tx_data_0[5];
assign inv_tx_data_0[3] = tx_data_0[4];
assign inv_tx_data_0[4] = tx_data_0[3];
assign inv_tx_data_0[5] = tx_data_0[2];
assign inv_tx_data_0[6] = tx_data_0[1];
assign inv_tx_data_0[7] = tx_data_0[0];

assign inv_tx_data_1[0] = tx_data_1[7];
assign inv_tx_data_1[1] = tx_data_1[6];
assign inv_tx_data_1[2] = tx_data_1[5];
assign inv_tx_data_1[3] = tx_data_1[4];
assign inv_tx_data_1[4] = tx_data_1[3];
assign inv_tx_data_1[5] = tx_data_1[2];
assign inv_tx_data_1[6] = tx_data_1[1];
assign inv_tx_data_1[7] = tx_data_1[0];

assign inv_tx_data_2[0] = tx_data_2[7];
assign inv_tx_data_2[1] = tx_data_2[6];
assign inv_tx_data_2[2] = tx_data_2[5];
assign inv_tx_data_2[3] = tx_data_2[4];
assign inv_tx_data_2[4] = tx_data_2[3];
assign inv_tx_data_2[5] = tx_data_2[2];
assign inv_tx_data_2[6] = tx_data_2[1];
assign inv_tx_data_2[7] = tx_data_2[0];

assign inv_tx_data_3[0] = tx_data_3[7];
assign inv_tx_data_3[1] = tx_data_3[6];
assign inv_tx_data_3[2] = tx_data_3[5];
assign inv_tx_data_3[3] = tx_data_3[4];
assign inv_tx_data_3[4] = tx_data_3[3];
assign inv_tx_data_3[5] = tx_data_3[2];
assign inv_tx_data_3[6] = tx_data_3[1];
assign inv_tx_data_3[7] = tx_data_3[0];

assign inv_tx_data_4[0] = tx_data_4[7];
assign inv_tx_data_4[1] = tx_data_4[6];
assign inv_tx_data_4[2] = tx_data_4[5];
assign inv_tx_data_4[3] = tx_data_4[4];
assign inv_tx_data_4[4] = tx_data_4[3];
assign inv_tx_data_4[5] = tx_data_4[2];
assign inv_tx_data_4[6] = tx_data_4[1];
assign inv_tx_data_4[7] = tx_data_4[0];

assign inv_tx_data_5[0] = tx_data_5[7];
assign inv_tx_data_5[1] = tx_data_5[6];
assign inv_tx_data_5[2] = tx_data_5[5];
assign inv_tx_data_5[3] = tx_data_5[4];
assign inv_tx_data_5[4] = tx_data_5[3];
assign inv_tx_data_5[5] = tx_data_5[2];
assign inv_tx_data_5[6] = tx_data_5[1];
assign inv_tx_data_5[7] = tx_data_5[0];

assign inv_tx_data_6[0] = tx_data_6[7];
assign inv_tx_data_6[1] = tx_data_6[6];
assign inv_tx_data_6[2] = tx_data_6[5];
assign inv_tx_data_6[3] = tx_data_6[4];
assign inv_tx_data_6[4] = tx_data_6[3];
assign inv_tx_data_6[5] = tx_data_6[2];
assign inv_tx_data_6[6] = tx_data_6[1];
assign inv_tx_data_6[7] = tx_data_6[0];

assign inv_tx_data_7[0] = tx_data_7[7];
assign inv_tx_data_7[1] = tx_data_7[6];
assign inv_tx_data_7[2] = tx_data_7[5];
assign inv_tx_data_7[3] = tx_data_7[4];
assign inv_tx_data_7[4] = tx_data_7[3];
assign inv_tx_data_7[5] = tx_data_7[2];
assign inv_tx_data_7[6] = tx_data_7[1];
assign inv_tx_data_7[7] = tx_data_7[0];

assign inv_tx_dlc[0] = tx_dlc[3];
assign inv_tx_dlc[1] = tx_dlc[2];
assign inv_tx_dlc[2] = tx_dlc[1];
assign inv_tx_dlc[3] = tx_dlc[0];

assign inv_basicID[0] = tx_basicID[10];
assign inv_basicID[1] = tx_basicID[9];
assign inv_basicID[2] = tx_basicID[8];
assign inv_basicID[3] = tx_basicID[7];
assign inv_basicID[4] = tx_basicID[6];
assign inv_basicID[5] = tx_basicID[5];
assign inv_basicID[6] = tx_basicID[4];
assign inv_basicID[7] = tx_basicID[3];
assign inv_basicID[8] = tx_basicID[2];
assign inv_basicID[9] = tx_basicID[1];
assign inv_basicID[10]= tx_basicID[0];

assign inv_extID[0] = tx_extID[17];
assign inv_extID[1] = tx_extID[16];
assign inv_extID[2] = tx_extID[15];
assign inv_extID[3] = tx_extID[14];
assign inv_extID[4] = tx_extID[13];
assign inv_extID[5] = tx_extID[12];
assign inv_extID[6] = tx_extID[11];
assign inv_extID[7] = tx_extID[10];
assign inv_extID[8] = tx_extID[9];
assign inv_extID[9] = tx_extID[8];
assign inv_extID[10]= tx_extID[7];
assign inv_extID[11]= tx_extID[6];
assign inv_extID[12]= tx_extID[5];
assign inv_extID[13]= tx_extID[4];
assign inv_extID[14]= tx_extID[3];
assign inv_extID[15]= tx_extID[2];
assign inv_extID[16]= tx_extID[1];
assign inv_extID[17]= tx_extID[0];

assign inv_calculated_crc[0] = calculated_crc[14];
assign inv_calculated_crc[1] = calculated_crc[13];
assign inv_calculated_crc[2] = calculated_crc[12];
assign inv_calculated_crc[3] = calculated_crc[11];
assign inv_calculated_crc[4] = calculated_crc[10];
assign inv_calculated_crc[5] = calculated_crc[9];
assign inv_calculated_crc[6] = calculated_crc[8];
assign inv_calculated_crc[7] = calculated_crc[7];
assign inv_calculated_crc[8] = calculated_crc[6];
assign inv_calculated_crc[9] = calculated_crc[5];
assign inv_calculated_crc[10]= calculated_crc[4];
assign inv_calculated_crc[11]= calculated_crc[3];
assign inv_calculated_crc[12]= calculated_crc[2];
assign inv_calculated_crc[13]= calculated_crc[1];
assign inv_calculated_crc[14]= calculated_crc[0];

endmodule
