/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  RX Puffer                                                  ////
////  Diese Modul verwaltet den Empfangspuffer und fuehrt        ////
////  Adressrechnungen aus.                                      ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none

module can_rx_buffer(
	clk_peri,
	reset,
	config_mode,

	rx_used_buffer,
	rx_start_adr,
	receiving,
	ready,
	row_ready0,

	rx_successful,
	write_rx_buffer,
	rx_data_written,

	id_ok,

	rx_buffer_full
);

parameter TX_MAILBOX	= 18; //1..18
parameter RX_MAILBOX	= 18; //1..18
parameter ACF_CHAINS	= 18; //1..18
parameter BASE_ADR	= 6'h8;

input wire				clk_peri;
input wire				reset;
input wire				config_mode;

//Signal von can_bsp (muessen synchronisiert werden)
input wire				receiving;
input wire				rx_successful;
input wire				id_ok;

//Signal von blockram_access
input wire				ready;
input wire				row_ready0;

//Input von can_cmd_interface
//input wire	[9:0]			adr_to_rx_buffer;//Adresse beim entgueltigen lesen einer Nachricht (durch loeschen des flags)

output				rx_buffer_full;
output	[9:0]			rx_start_adr;
//Ausgabe an can_blockram_access
output				write_rx_buffer;
output				rx_data_written;

input wire	[RX_MAILBOX-1:0]	rx_used_buffer;	//Belegte Rx Puffer
wire	[RX_MAILBOX-1:0]	rx_used_adr_decode;
wire	[4:0]			tmp_adr;	//Nummer des 1. freien Puffer abgeleitet aus rx_used_buffer
reg	[5:0]			adr_to_write;	//6 Bit, da noch 18 addiert werden muss und 18+18 >32 ist!

reg				write_rx_buffer;
reg	[9:0]			rx_start_adr;
wire	[9:0]			write_adr;	//Startadresse aus Puffernummer abgeleitet.

reg				state;
wire				rx_buffer_full;
wire				rx_data_written;

//Signal von can_bsp (muessen synchronisiert werden)
reg				receiving_s;
reg				rx_successful_s;
reg				id_ok_s;

parameter	IDLE	= 0;
parameter	WRITE	= 1;


assign	rx_buffer_full		= &rx_used_buffer;

//assign	write_adr		= {2'b0,(adr_to_write+TX_MAILBOX),3'b0}; //Telegramm hat 8 Adressen
assign	write_adr		= {1'b0,(adr_to_write+18),3'b0}; //Telegramm hat 8 Adressen (Rx beginnt immer hinter dem 18. Tx-Buffer

assign	rx_used_adr_decode	= rx_used_buffer;

assign	rx_data_written		= ready & write_rx_buffer;

/**
 * Synchronisation der Signale vom can_bsp
 */
always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
	receiving_s	<= 1'b0;
	rx_successful_s	<= 1'b0;
	id_ok_s		<= 1'b0;
	end
	else
	begin
	receiving_s	<= receiving;
	rx_successful_s	<= rx_successful;
	id_ok_s		<= id_ok;
	end
end


always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
	begin
	adr_to_write	<= 6'h0;
	state		<= IDLE;
	end
	else if (~receiving_s)
	begin
	   if (row_ready0)	//fuehre neue Adresberchnung durch bei steigender Flanke 
		adr_to_write	<= {1'b0,tmp_adr};

	state		<= WRITE;
	end
//	else if (~receiving_s)
//		state		<= IDLE;
end

always @ (posedge clk_peri or posedge reset) 
begin
	if (reset)
	begin
	rx_start_adr	<= 10'h0;
	write_rx_buffer	<= 1'b0;
	end
	else if (config_mode)
	begin
	rx_start_adr	<= 10'h0;
	write_rx_buffer	<= 1'b0;
	end
	else 
	begin
	case (state)
		IDLE:;

		WRITE:
		begin
		if (id_ok_s)
			rx_start_adr <= write_adr;
		if (rx_successful_s & id_ok_s | write_rx_buffer)	//nach vollstaendigen Empfang eines Telegramms
		//(nach dem 8. Byte werden automatisch ID und Controlfield geschrieben
			begin
			write_rx_buffer	<= 1'b1;
			if (ready)
				 write_rx_buffer	<= 1'b0;
			  //rx_used_buffer[i] <= 1'b1;
			end
		end

	endcase
	end
end

generate
	case (RX_MAILBOX)
		1:begin:RX_Single_Buffer
			assign tmp_adr = 	5'b0;
		end
		2:begin:RX_Double_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[1:0]==2'b10;
			assign tmp_adr[1] = 	1'b0;
			assign tmp_adr[2] =	1'b0;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		5'd3:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[2:1]==3'b10;
			assign tmp_adr[1] = 	rx_used_adr_decode[2:0]==3'b110;
			assign tmp_adr[2] =	1'b0;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		4:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[3:2]==4'b10		|
						rx_used_adr_decode[3:0]==4'b1110	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[3:1]==4'b110		|
						rx_used_adr_decode[3:0]==4'b1110	 ;
			assign tmp_adr[2] =	1'b0;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		5:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[4:3]==5'b10		|
						rx_used_adr_decode[4:1]==5'b1110	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[4:2]==5'b110		|
						rx_used_adr_decode[4:1]==5'b1110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[4:0]==5'b1111_0	 ;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		6:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[5:4]==6'b10		|
						rx_used_adr_decode[5:2]==6'b1110 	|
						rx_used_adr_decode[5:0]==6'b1111_10	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[5:3]==6'b110		|
						rx_used_adr_decode[5:2]==6'b1110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[5:1]==6'b1111_0 	|
						rx_used_adr_decode[5:0]==6'b1111_10	 ;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		7:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[6:5]==7'b10		|
						rx_used_adr_decode[6:3]==7'b1110	|
						rx_used_adr_decode[6:1]==7'b1111_10	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[6:4]==7'b110		|
						rx_used_adr_decode[6:3]==7'b1110	|
						rx_used_adr_decode[6:0]==7'b1111_110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[6:2]==7'b1111_0	|
						rx_used_adr_decode[6:1]==7'b1111_10	|
						rx_used_adr_decode[6:0]==7'b1111_110	 ;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		8:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[7:6]==8'b10		|
						rx_used_adr_decode[7:4]==8'b1110	|
						rx_used_adr_decode[7:2]==8'b1111_10	|
						rx_used_adr_decode[7:0]==8'b1111_1110	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[7:5]==8'b110		|
						rx_used_adr_decode[7:4]==8'b1110	|
						rx_used_adr_decode[7:1]==8'b1111_110	|
						rx_used_adr_decode[7:0]==8'b1111_1110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[7:3]==8'b1111_0	|
						rx_used_adr_decode[7:2]==8'b1111_10	|
						rx_used_adr_decode[7:1]==8'b1111_110	|
						rx_used_adr_decode[7:0]==8'b1111_1110	 ;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		9:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[8:7]==9'b10		|
						rx_used_adr_decode[8:5]==9'b1110 	|
						rx_used_adr_decode[8:3]==9'b1111_10	|
						rx_used_adr_decode[8:1]==9'b1111_1110	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[8:6]==9'b110		|
						rx_used_adr_decode[8:5]==9'b1110	|
						rx_used_adr_decode[8:2]==9'b1111_110	|
						rx_used_adr_decode[8:1]==9'b1111_1110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[8:4]==9'b1111_0	|
						rx_used_adr_decode[8:3]==9'b1111_10	|
						rx_used_adr_decode[8:2]==9'b1111_110	|
						rx_used_adr_decode[8:1]==9'b1111_1110	 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[8:0]==9'b1111_1111_0  ;
			assign tmp_adr[4] = 	1'b0;
		end
		10:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[9:8]==10'b10			|
						rx_used_adr_decode[9:6]==10'b1110		|
						rx_used_adr_decode[9:4]==10'b1111_10		|
						rx_used_adr_decode[9:2]==10'b1111_1110		|
						rx_used_adr_decode[9:0]==10'b1111_1111_10	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[9:7]==10'b110		|
						rx_used_adr_decode[9:6]==10'b1110		|
						rx_used_adr_decode[9:3]==10'b1111_110		|
						rx_used_adr_decode[9:2]==10'b1111_1110		 ;
			assign tmp_adr[2] =	rx_used_adr_decode[9:5]==10'b1111_0		|
						rx_used_adr_decode[9:4]==10'b1111_10		|
						rx_used_adr_decode[9:3]==10'b1111_110		|
						rx_used_adr_decode[9:2]==10'b1111_1110		 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[9:1]==10'b1111_1111_0	|
						rx_used_adr_decode[9:0]==10'b1111_1111_10	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		11:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[10:9]==11'b10		|
						rx_used_adr_decode[10:7]==11'b1110		|
						rx_used_adr_decode[10:5]==11'b1111_10		|
						rx_used_adr_decode[10:3]==11'b1111_1110		|
						rx_used_adr_decode[10:1]==11'b1111_1111_10	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[10:8]==11'b110 		|
						rx_used_adr_decode[10:7]==11'b1110		|
						rx_used_adr_decode[10:4]==11'b1111_110		|
						rx_used_adr_decode[10:3]==11'b1111_1110		|
						rx_used_adr_decode[10:0]==11'b1111_1111_110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[10:6]==11'b1111_0		|
						rx_used_adr_decode[10:5]==11'b1111_10		|
						rx_used_adr_decode[10:4]==11'b1111_110		|
						rx_used_adr_decode[10:3]==11'b1111_1110		 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[10:2]==11'b1111_1111_0	|
						rx_used_adr_decode[10:1]==11'b1111_1111_10	|
						rx_used_adr_decode[10:0]==11'b1111_1111_110	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		12:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[11:10]==12'b10		|
						rx_used_adr_decode[11:8] ==12'b1110 		|
						rx_used_adr_decode[11:6] ==12'b1111_10		|
						rx_used_adr_decode[11:4] ==12'b1111_1110 	|
						rx_used_adr_decode[11:2] ==12'b1111_1111_10 	|
						rx_used_adr_decode[11:0] ==12'b1111_1111_1110 	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[11:9] ==12'b110 		|
						rx_used_adr_decode[11:8] ==12'b1110 		|
						rx_used_adr_decode[11:5] ==12'b1111_110 	|
						rx_used_adr_decode[11:4] ==12'b1111_1110 	|
						rx_used_adr_decode[11:1] ==12'b1111_1111_110 	|
						rx_used_adr_decode[11:0] ==12'b1111_1111_1110 	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[11:7] ==12'b1111_0 		|
						rx_used_adr_decode[11:6] ==12'b1111_10 		|
						rx_used_adr_decode[11:5] ==12'b1111_110 	|
						rx_used_adr_decode[11:4] ==12'b1111_1110 	 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[11:3] ==12'b1111_1111_0 	|
						rx_used_adr_decode[11:2] ==12'b1111_1111_10 	|
						rx_used_adr_decode[11:1] ==12'b1111_1111_110 	|
						rx_used_adr_decode[11:0] ==12'b1111_1111_1110 	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		13:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[12:11]==13'b10		|
						rx_used_adr_decode[12:9] ==13'b1110		|
						rx_used_adr_decode[12:7] ==13'b1111_10		|
						rx_used_adr_decode[12:5] ==13'b1111_1110	|
						rx_used_adr_decode[12:3] ==13'b1111_1111_10	|
						rx_used_adr_decode[12:1] ==13'b1111_1111_1110	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[12:10]==13'b110		|
						rx_used_adr_decode[12:9] ==13'b1110		|
						rx_used_adr_decode[12:6] ==13'b1111_110		|
						rx_used_adr_decode[12:5] ==13'b1111_1110	|
						rx_used_adr_decode[12:2] ==13'b1111_1111_110	|
						rx_used_adr_decode[12:1] ==13'b1111_1111_1110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[12:8] ==13'b1111_0		|
						rx_used_adr_decode[12:7] ==13'b1111_10		|
						rx_used_adr_decode[12:6] ==13'b1111_110		|
						rx_used_adr_decode[12:5] ==13'b1111_1110	|
						rx_used_adr_decode[12:0] ==13'b1111_1111_1111_0	 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[12:4] ==13'b1111_1111_0	|
						rx_used_adr_decode[12:3] ==13'b1111_1111_10	|
						rx_used_adr_decode[12:2] ==13'b1111_1111_110	|
						rx_used_adr_decode[12:1] ==13'b1111_1111_1110	|
						rx_used_adr_decode[12:0] ==13'b1111_1111_1111_0	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		14:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[13:12]==14'b10			|
						rx_used_adr_decode[13:10]==14'b1110			|
						rx_used_adr_decode[13:8] ==14'b1111_10			|
						rx_used_adr_decode[13:6] ==14'b1111_1110		|
						rx_used_adr_decode[13:4] ==14'b1111_1111_10		|
						rx_used_adr_decode[13:2] ==14'b1111_1111_1110		|
						rx_used_adr_decode[13:0] ==14'b1111_1111_1111_10	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[13:11]==14'b110			|
						rx_used_adr_decode[13:10]==14'b1110			|
						rx_used_adr_decode[13:7] ==14'b1111_110			|
						rx_used_adr_decode[13:6] ==14'b1111_1110		|
						rx_used_adr_decode[13:3] ==14'b1111_1111_110		|
						rx_used_adr_decode[13:2] ==14'b1111_1111_1110		 ;
			assign tmp_adr[2] =	rx_used_adr_decode[13:9] ==14'b1111_0			|
						rx_used_adr_decode[13:8] ==14'b1111_10			|
						rx_used_adr_decode[13:7] ==14'b1111_110			|
						rx_used_adr_decode[13:6] ==14'b1111_1110		|
						rx_used_adr_decode[13:1] ==14'b1111_1111_1111_0		|
						rx_used_adr_decode[13:0] ==14'b1111_1111_1111_10	 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[13:5] ==14'b1111_1111_0		|
						rx_used_adr_decode[13:4] ==14'b1111_1111_10		|
						rx_used_adr_decode[13:3] ==14'b1111_1111_110		|
						rx_used_adr_decode[13:2] ==14'b1111_1111_1110		|
						rx_used_adr_decode[13:1] ==14'b1111_1111_1111_0		|
						rx_used_adr_decode[13:0] ==14'b1111_1111_1111_10	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		15:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[14:13]==15'b10			|
						rx_used_adr_decode[14:11]==15'b1110			|
						rx_used_adr_decode[14:9] ==15'b1111_10			|
						rx_used_adr_decode[14:7] ==15'b1111_1110		|
						rx_used_adr_decode[14:5] ==15'b1111_1111_10		|
						rx_used_adr_decode[14:3] ==15'b1111_1111_1110		|
						rx_used_adr_decode[14:1] ==15'b1111_1111_1111_10	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[14:12]==15'b110			|
						rx_used_adr_decode[14:11]==15'b1110			|
						rx_used_adr_decode[14:8] ==15'b1111_110			|
						rx_used_adr_decode[14:7] ==15'b1111_1110		|
						rx_used_adr_decode[14:4] ==15'b1111_1111_110		|
						rx_used_adr_decode[14:3] ==15'b1111_1111_1110		|
						rx_used_adr_decode[14:0] ==15'b1111_1111_1111_110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[14:10]==15'b1111_0			|
						rx_used_adr_decode[14:9] ==15'b1111_10			|
						rx_used_adr_decode[14:8] ==15'b1111_110			|
						rx_used_adr_decode[14:7] ==15'b1111_1110		|
						rx_used_adr_decode[14:2] ==15'b1111_1111_1111_0		|
						rx_used_adr_decode[14:1] ==15'b1111_1111_1111_10	|
						rx_used_adr_decode[14:0] ==15'b1111_1111_1111_110	 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[14:6] ==15'b1111_1111_0		|
						rx_used_adr_decode[14:5] ==15'b1111_1111_10		|
						rx_used_adr_decode[14:4] ==15'b1111_1111_110		|
						rx_used_adr_decode[14:3] ==15'b1111_1111_1110		|
						rx_used_adr_decode[14:2] ==15'b1111_1111_1111_0		|
						rx_used_adr_decode[14:1] ==15'b1111_1111_1111_10	|
						rx_used_adr_decode[14:0] ==15'b1111_1111_1111_110	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		16:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[15:14]==16'b10			|
						rx_used_adr_decode[15:12]==16'b1110			|
						rx_used_adr_decode[15:10]==16'b1111_10			|
						rx_used_adr_decode[15:8] ==16'b1111_1110		|
						rx_used_adr_decode[15:6] ==16'b1111_1111_10		|
						rx_used_adr_decode[15:4] ==16'b1111_1111_1110		|
						rx_used_adr_decode[15:2] ==16'b1111_1111_1111_10	|
						rx_used_adr_decode[15:0] ==16'b1111_1111_1111_1110	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[15:13]==16'b110			|
						rx_used_adr_decode[15:12]==16'b1110			|
						rx_used_adr_decode[15:9] ==16'b1111_110			|
						rx_used_adr_decode[15:8] ==16'b1111_1110		|
						rx_used_adr_decode[15:5] ==16'b1111_1111_110		|
						rx_used_adr_decode[15:4] ==16'b1111_1111_1110		|
						rx_used_adr_decode[15:1] ==16'b1111_1111_1111_110	|
						rx_used_adr_decode[15:0] ==16'b1111_1111_1111_1110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[15:11]==16'b1111_0			|
						rx_used_adr_decode[15:10]==16'b1111_10			|
						rx_used_adr_decode[15:9] ==16'b1111_110			|
						rx_used_adr_decode[15:8] ==16'b1111_1110		|
						rx_used_adr_decode[15:3] ==16'b1111_1111_1111_0		|
						rx_used_adr_decode[15:2] ==16'b1111_1111_1111_10	|
						rx_used_adr_decode[15:1] ==16'b1111_1111_1111_110	|
						rx_used_adr_decode[15:0] ==16'b1111_1111_1111_1110	 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[15:7] ==16'b1111_1111_0 		|
						rx_used_adr_decode[15:6] ==16'b1111_1111_10 		|
						rx_used_adr_decode[15:5] ==16'b1111_1111_110 		|
						rx_used_adr_decode[15:4] ==16'b1111_1111_1110 		|
						rx_used_adr_decode[15:3] ==16'b1111_1111_1111_0 	|
						rx_used_adr_decode[15:2] ==16'b1111_1111_1111_10 	|
						rx_used_adr_decode[15:1] ==16'b1111_1111_1111_110	|
						rx_used_adr_decode[15:0] ==16'b1111_1111_1111_1110	 ;
			assign tmp_adr[4] = 	1'b0; 
		end
		17:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[16:15]==17'b10			|
						rx_used_adr_decode[16:13]==17'b1110			|
						rx_used_adr_decode[16:11]==17'b1111_10			|
						rx_used_adr_decode[16:9] ==17'b1111_1110		|
						rx_used_adr_decode[16:7] ==17'b1111_1111_10		|
						rx_used_adr_decode[16:5] ==17'b1111_1111_1110		|
						rx_used_adr_decode[16:3] ==17'b1111_1111_1111_10	|
						rx_used_adr_decode[16:1] ==17'b1111_1111_1111_1110	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[16:14]==17'b110			|
						rx_used_adr_decode[16:13]==17'b1110			|
						rx_used_adr_decode[16:10]==17'b1111_110			|
						rx_used_adr_decode[16:9] ==17'b1111_1110		|
						rx_used_adr_decode[16:6] ==17'b1111_1111_110		|
						rx_used_adr_decode[16:5] ==17'b1111_1111_1110		|
						rx_used_adr_decode[16:2] ==17'b1111_1111_1111_110	|
						rx_used_adr_decode[16:1] ==17'b1111_1111_1111_1110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[16:12]==17'b1111_0			|
						rx_used_adr_decode[16:11]==17'b1111_10			|
						rx_used_adr_decode[16:10]==17'b1111_110			|
						rx_used_adr_decode[16:9] ==17'b1111_1110		|
						rx_used_adr_decode[16:4] ==17'b1111_1111_1111_0		|
						rx_used_adr_decode[16:3] ==17'b1111_1111_1111_10	|
						rx_used_adr_decode[16:2] ==17'b1111_1111_1111_110	|
						rx_used_adr_decode[16:1] ==17'b1111_1111_1111_1110	 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[16:8] ==17'b1111_1111_0		|
						rx_used_adr_decode[16:7] ==17'b1111_1111_10		|
						rx_used_adr_decode[16:6] ==17'b1111_1111_110		|
						rx_used_adr_decode[16:5] ==17'b1111_1111_1110		|
						rx_used_adr_decode[16:4] ==17'b1111_1111_1111_0		|
						rx_used_adr_decode[16:3] ==17'b1111_1111_1111_10	|
						rx_used_adr_decode[16:2] ==17'b1111_1111_1111_110	|
						rx_used_adr_decode[16:1] ==17'b1111_1111_1111_1110	 ;
			assign tmp_adr[4] = 	rx_used_adr_decode[16:0] ==17'b1111_1111_1111_1111_0	 ; 
		end
		18:begin:RX_Buffer_Decoder
			assign tmp_adr[0] = 	rx_used_adr_decode[17:16]==18'b10			|
						rx_used_adr_decode[17:14]==18'b1110			|
						rx_used_adr_decode[17:12]==18'b1111_10			|
						rx_used_adr_decode[17:10]==18'b1111_1110		|
						rx_used_adr_decode[17:8] ==18'b1111_1111_10		|
						rx_used_adr_decode[17:6] ==18'b1111_1111_1110		|
						rx_used_adr_decode[17:4] ==18'b1111_1111_1111_10	|
						rx_used_adr_decode[17:2] ==18'b1111_1111_1111_1110	|
						rx_used_adr_decode[17:0] ==18'b1111_1111_1111_1111_10	 ;
			assign tmp_adr[1] = 	rx_used_adr_decode[17:15]==18'b110			|
						rx_used_adr_decode[17:14]==18'b1110			|
						rx_used_adr_decode[17:11]==18'b1111_110			|
						rx_used_adr_decode[17:10]==18'b1111_1110		|
						rx_used_adr_decode[17:7] ==18'b1111_1111_110		|
						rx_used_adr_decode[17:6] ==18'b1111_1111_1110		|
						rx_used_adr_decode[17:3] ==18'b1111_1111_1111_110	|
						rx_used_adr_decode[17:2] ==18'b1111_1111_1111_1110	 ;
			assign tmp_adr[2] =	rx_used_adr_decode[17:13]==18'b1111_0			|
						rx_used_adr_decode[17:12]==18'b1111_10			|
						rx_used_adr_decode[17:11]==18'b1111_110			|
						rx_used_adr_decode[17:10]==18'b1111_1110		|
						rx_used_adr_decode[17:5] ==18'b1111_1111_1111_0		|
						rx_used_adr_decode[17:4] ==18'b1111_1111_1111_10	|
						rx_used_adr_decode[17:3] ==18'b1111_1111_1111_110	|
						rx_used_adr_decode[17:2] ==18'b1111_1111_1111_1110	 ;
			assign tmp_adr[3] = 	rx_used_adr_decode[17:9] ==18'b1111_1111_0		|
						rx_used_adr_decode[17:8] ==18'b1111_1111_10		|
						rx_used_adr_decode[17:7] ==18'b1111_1111_110		|
						rx_used_adr_decode[17:6] ==18'b1111_1111_1110		|
						rx_used_adr_decode[17:5] ==18'b1111_1111_1111_0		|
						rx_used_adr_decode[17:4] ==18'b1111_1111_1111_10	|
						rx_used_adr_decode[17:3] ==18'b1111_1111_1111_110	|
						rx_used_adr_decode[17:2] ==18'b1111_1111_1111_1110	 ;
			assign tmp_adr[4] = 	rx_used_adr_decode[17:1] ==18'b1111_1111_1111_1111_0	| 
						rx_used_adr_decode[17:0] ==18'b1111_1111_1111_1111_10	 ;
		end
	endcase
endgenerate

endmodule
