/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  TOP                                                        ////
////  In diesem Modul werden alle Komponenten das CAN-           ////
////  Controllers instanziiert.                                  ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////


`default_nettype none

module can_top
	(
	//vom SpartanMC
	clk_peri,
	do_peri,	//Datenbus
	di_peri,	//Datenbus
	addr_peri,	//Adressbus
	access_peri,	//aktiviere Peripherieregister
	wr_peri,	//Write Enable vom SpartanMC
	// Blockram Interface
	mem_access,
	store_access_low,
	store_access_high,
	addr_high,

	can_clk,
	reset,

	//an SpartanMC ir-Controller
	exception_ir,
	tx_successful_ir,
	rx_successful_ir,

	//Ausgang zum CAN-Bus
	can_tx,
	can_rx,

// Zustands Signale der FSM und andere zur Fehlersuche bereitstellen
	error_capture_code_segment,
	bit_error_latched,
	form_error_latched,
	stuff_error_latched,
	crc_error,
	ack_error_latched,
	error_direction,
	set_exception_ir,
	error_warning
);

parameter	DMA_BASE_ADR	= 18'h19800;	// war CANBASE_ADR
parameter	BASE_ADR	= 10'h3f0;
parameter	TX_MAILBOX	= 3; //1..18
parameter	RX_MAILBOX	= 5; //1..18
parameter	ACF_CHAINS	= 2; //1..18

input					can_clk;
input wire				reset;

//Ausgang zum CAN-Bus
input wire			can_rx;
output				can_tx;

//an SpartanMC ir-Controller
output				exception_ir;
output				tx_successful_ir;
output				rx_successful_ir;

//vom SpartanMC
input				clk_peri;
input wire			wr_peri;	//Write Enable vom SpartanMC
input wire			access_peri;	//aktiviere Peripherieregister

output	[18-1:0]		di_peri;	//Datenbus (15 Register + DMA)
input	[17:0]			do_peri;	//Datenbus
input wire	[9:0]		addr_peri;	//Adressbus

input	[7:0]			addr_high;
input				mem_access;
input				store_access_low;	/*Dummy for 9 Bit access*/
input				store_access_high;	/*Dummy for 9 Bit access*/

// Zustands Signale der FSM und weitere Signale zur Fehlersuche
output	[4:0]			error_capture_code_segment;
output				bit_error_latched;
output				form_error_latched;
output				stuff_error_latched;
output				crc_error;
output				ack_error_latched;
output				error_direction;
output				set_exception_ir;
output				error_warning;

//output				rx_idle;
wire				rx_id1;
wire				rx_rtr1;
wire				rx_ide;
wire				rx_id2;
wire				rx_rtr2;
wire				rx_r1;
wire				rx_r0;
wire				rx_dlc;
wire				rx_data;
wire				rx_crc;
wire				rx_crc_lim;
wire				rx_ack;
wire				rx_ack_lim;
wire				rx_eof;
//output				rx_inter;
//output				error_frame;
wire				error_frame_limz;
wire				overload_frame;
wire				overload_frame_limz;


wire	[7:0]			addr_high;
wire				mem_access;
wire				store_access_low;	/*Dummy for 9 Bit access*/
wire            		store_access_high;	/*Dummy for 9 Bit access*/
//an/von DMA-Blockram
wire	[9:0]			adr_dma;
wire				we_dma;
wire	[17:0]			di_dma;
wire	[17:0]			do_dma;


//Ausgabe an can_bsp
wire	[(ACF_CHAINS-1):0]	match_basic;
wire	[(ACF_CHAINS-1):0]	match_std;
wire	[(ACF_CHAINS-1):0]	match_ext;

//Signale von can_emu
wire				node_error_passive;
wire				node_error_active;
wire				node_bus_off;
wire				error_strobe;
wire				tx_successful;
wire				error_warning;
wire				bit_error_latched;
wire				form_error_latched;
wire				stuff_error_latched;
wire				ack_error_latched;

//Ausgabe an can_btl
wire				rx_idle;
wire				transmitting;
wire				transmitter;
wire				go_rx_inter;
wire				rx_inter;
wire				go_overload_frame;
wire				go_error_frame;
wire				go_tx;
wire				send_ack;

//Ausgabe an can_acf
wire				no_byte0;

//Ausgabe an can_bsl
//wire				tx_stuffed;
wire				go_rx_id1;
wire				go_rx_crc_lim;
wire 				rx_crc_lim_stuffing_disable;



//Ausgabe an CRC-Modul
wire	[14:0]			crc;
wire				crc_strobe;
wire				crc_enable_stb;
wire				go_crc_enable;

//Ausgabe an Akzeptanzfiltermodul
wire	[28:0]			idz;
//wire	[3:0]			data0;
wire				rtr1;
wire				rtr2;
wire				ide;

//Ausgabe an emu
wire				bit_error;
wire				form_error;
wire				ack_error;
wire	[4:0]			error_capture_code_segment;
wire				error_direction;

//Kommando Register
wire				rst_error_code_reg;
wire	[4:0]			acf_adr;
//wire	[4:0]			acf_code;

//Output von CAN_btl
wire 				sample_bit;	//abgetastetes Bit (Sample)
wire 				sample_strobe;	//Strobe zur Uebername des Samples
wire				clk_fsm;
wire 				tx_strobe;	//Strobe zum Senden eines Bits auf den Bus
wire 				hard_sync;	//Zeigt an ob eine hard_sync moeglich ist
wire 				tx_next;	//naechstes Sendebit
wire 				sample_bit_pre;	//Vorgaengerbit

//Output
wire				sample_strobe_q;
wire				tx_strobe_q;
wire				tx_unstuffed;
wire				stuff_error;

wire	[14:0]			calculated_crc;
wire	[14:0]			dlc0_crc;
wire				set_dlc0_crc;
wire				crc_error;

//von BSP
wire				error_frame_lim;
wire				error_flag_over;
wire				arbitration_lost;
wire				go_rx_ack_lim;
wire				error_frame;
wire				rule5;
wire				rule3_exception1_2;
wire	[2:0]			error_cnt1;
wire				arbitration_field;
wire				rx_successful;
wire				receiving;


wire	[2:0]			delayed_dominant_cnt;

//an BSP
wire				bus_free;

//Ausabe an Can_bsp
wire				id_ok;

wire	[3:0]			tx_dlc;
wire	[10:0]			tx_basicID;
wire	[17:0]			tx_extID;
wire	[7:0]			tx_data0;
wire	[7:0]			tx_data1;
wire	[7:0]			tx_data2;
wire	[7:0]			tx_data3;
wire	[7:0]			tx_data4;
wire	[7:0]			tx_data5;
wire	[7:0]			tx_data6;
wire	[7:0]			tx_data7;
wire	[6:0]			baudrate_presc;
wire	[1:0]			sync_jump_width;
wire	[4:0]			time_segment1;
wire	[2:0]			time_segment2;
wire	[7:0]			error_warning_limit;
wire	[7:0]			data_rx_error_cnt;
wire	[7:0]			data_tx_error_cnt;
wire	[8:0]			rx_error_cnt;
wire	[8:0]			tx_error_cnt;
wire	[TX_MAILBOX-1:0]	tx_request_buffer;
wire	[5:0]			retry_tx;
wire	[9:0]			tx_start_adr;
wire	[RX_MAILBOX-1:0]	rx_used_buffer;
wire	[9:0]			rx_start_adr;
//261 - "can_top.v" line 559 Connection to output port 'rx_error_cnt'
//2261 - "can_top.v" line 560 Connection to output port 'tx_error_cnt'

wire	[(ACF_CHAINS-1):0]	acf_en;		// Freigabe der Filter zur Bildung von id_ok
wire	[17:0]			ac_e_id;
wire	[17:0]			am_e_id;
wire	[10:0]			ac_b_id;
wire	[10:0]			am_b_id;
wire	[3:0]			ac_data;
wire	[3:0]			am_data;
wire				am_rtr1;
wire				ac_rtr1;
wire				am_rtr2;
wire				ac_rtr2;

wire				clk_peri;
wire	[17:0]			do_peri;
wire	[18-1:0]		di_peri;	//Datenbus (15 Register + DMA)
wire	[18-1:0]		di_peri_io;	// io-Register
wire	[18-1:0]		di_peri_dma;	// DMA-BlockRAM
//259 - "can_top.v" line 590  'adr'
//259 - "can_cmd_interface.v" line 440  'addr'
wire	[ACF_CHAINS-1:0]	match;
wire	[7:0]			rx_data0;
wire	[7:0]			rx_data1;
wire	[7:0]			rx_data2;
wire	[7:0]			rx_data3;
wire	[7:0]			rx_data4;
wire	[7:0]			rx_data5;
wire	[7:0]			rx_data6;
wire	[7:0]			rx_data7;

wire				send_failed;
wire				rx_buffer_full;
wire				tx_successful_ir;
wire				rx_successful_ir;
wire				set_exception_ir;
wire				exception_ir;

//wire				en_exception_ir;
//wire				en_tx_successful_ir;
//wire				en_rx_successful_ir;

wire	[8:0]			error_capture_code;
wire				error_irq;
wire				we_error_cnt;
wire				clear_we_error_cnt;

wire				sample_mode;
wire				we_acf;
wire	[ACF_CHAINS-1:0]	clear_we_acf;

wire				self_test_mode;
wire				config_mode;
wire				listen_only_mode;
wire	[1:0]			can_mode;

wire				rx_data_written;
wire				row_ready0;
wire				row_ready8;

wire	[3:0]			data_length;
wire				tx_request;

//reg				can_tx; 	//Outputreg. zum Bus
//reg				self_tx;	//Outputreg. für Selftest

wire				tx;
reg				rx;
reg				can_tx;

assign	config_mode		= (can_mode==2'b11);
assign	listen_only_mode	= (can_mode==2'b10);
assign	self_test_mode		= (can_mode==2'b01);

wire				can_clk;
//wire				clki	= clk_peri;	// war !clk_peri

wire				extended_mode;
wire				overload_request;
wire				abort_tx;
wire				tx_ide;
wire				tx_rtr;
wire				ready;
wire				read_tx_buffer;
wire				write_rx_buffer;

assign	di_peri	=	di_peri_io | di_peri_dma;

always @ (posedge can_clk or posedge reset)
begin
	if (reset)
	begin
		can_tx	<= tx;
		rx	<= can_rx;
	end
	else if (self_test_mode)
	begin
		rx	<= tx;
		can_tx	<= 1'b1;
	end
	else if (listen_only_mode)
	begin
		rx	<= can_rx;
		can_tx	<= 1'b1;
	end
	else
	begin
		can_tx	<= tx;
		rx	<= can_rx;
	end
end

//-------------------------------------------------------------------------------------------------------------------
can_bsp #(
         .BASE_ADR			(BASE_ADR			),
         .TX_MAILBOX			(TX_MAILBOX			),
         .RX_MAILBOX			(RX_MAILBOX			),
         .ACF_CHAINS			(ACF_CHAINS			))

i_can_bsp
	(
	.can_clk			(can_clk			),
	.reset				(reset				),

	//Ausgabe an can_cmd_interface
	.match				(match				),
	.rx_successful			(rx_successful			),
	.receiving			(receiving			),

	.row_ready8			(row_ready8			),

	//Signale von can_btl
	.sample_strobe_q		(sample_strobe_q		),
	.clk_fsm			(clk_fsm			),
	.sample_bit			(sample_bit			),
	.tx_strobe			(tx_strobe_q			),
	.hard_sync			(hard_sync			),

	//Signale von can_emu
	.node_error_passive		(node_error_passive		),
	.node_error_active		(node_error_active		),
	.node_bus_off			(node_bus_off			),
	.error_strobe			(error_strobe			),
	.tx_successful			(tx_successful			),
	.bus_free			(bus_free			),

	//Signale von can_crc
	.calculated_crc			(calculated_crc			),
	.dlc0_crc			(dlc0_crc			),
	.set_dlc0_crc			(set_dlc0_crc			),

	//Ausgabe an can_btl
	.rx_idle			(rx_idle			),
	.transmitting			(transmitting			),
	.transmitter			(transmitter			),
	.go_rx_inter			(go_rx_inter			),
	.rx_inter			(rx_inter			),
	.tx_next			(tx_next			),
	.go_overload_frame		(go_overload_frame		),
	.go_error_frame			(go_error_frame			),
	.go_tx				(go_tx				),
	.send_ack			(send_ack			),

	//Ausgabe an can_acf
	.no_byte0			(no_byte0			),

	//Ausgabe an can_bsl
	.tx				(tx_unstuffed			),
	.go_rx_id1			(go_rx_id1			),
	.go_rx_crc_lim			(go_rx_crc_lim			),
	.rx_crc_lim_stuffing_disable	(rx_crc_lim_stuffing_disable	),


	//Mode Register
	.config_mode			(config_mode			),
	.listen_only_mode		(listen_only_mode		),
	.extended_mode			(extended_mode			),

	.match_basic			(match_basic			),
	.match_std			(match_std			),
	.match_ext			(match_ext			),
	.acf_en				(acf_en				),
	.id_ok				(id_ok				),

	//DMA Zugriff
	//addr,
	//do_peri,
	//di_peri,

	//Ausgabe an CRC-Modul
	.crc				(crc				),
	.crc_strobe			(crc_strobe			),
	.crc_enable_stb			(crc_enable_stb			),
	.go_crc_enable			(go_crc_enable			),

	//Ausgabe an Akzeptanzfiltermodul
	.idy				(idz				),
//	.data0				(data0				),
	.rtr1				(rtr1				),
	.rtr2				(rtr2				),
	.ide				(ide				),

	//Ausgabe an emu
	.bit_error			(bit_error			),
	.form_error			(form_error			),
	.ack_error			(ack_error			),
	.error_capture_code_segment	(error_capture_code_segment	),
	.error_direction		(error_direction		),
	.error_cnt1			(error_cnt1			),
	.rule5				(rule5				),
	.rule3_exception1_2		(rule3_exception1_2		),
	.error_frame			(error_frame			),
	.arbitration_field		(arbitration_field		),
	.go_rx_ack_lim			(go_rx_ack_lim			),
	.arbitration_lost		(arbitration_lost		),
	.error_flag_over		(error_flag_over		),
	.error_frame_lim		(error_frame_lim		),
	.delayed_dominant_cnt		(delayed_dominant_cnt		),

	//Kommando Register
	.overload_request		(overload_request		),
	.tx_request			(tx_request			),
	.abort_tx			(abort_tx			),

	.self_test_mode			(self_test_mode			),

	//Schreiben
	.rx_data0			(rx_data0			),
	.rx_data1			(rx_data1			),
	.rx_data2			(rx_data2			),
	.rx_data3			(rx_data3			),
	.rx_data4			(rx_data4			),
	.rx_data5			(rx_data5			),
	.rx_data6			(rx_data6			),
	.rx_data7			(rx_data7			),
	.data_length			(data_length			),

	//Temp Senden
	.tx_ide				(tx_ide				),
	.tx_rtr				(tx_rtr				),
	.tx_dlc				(tx_dlc				),
	.tx_basicID			(tx_basicID			),
	.tx_extID			(tx_extID			),
	.tx_data0			(tx_data0			),
	.tx_data1			(tx_data1			),
	.tx_data2			(tx_data2			),
	.tx_data3			(tx_data3			),
	.tx_data4			(tx_data4			),
	.tx_data5			(tx_data5			),
	.tx_data6			(tx_data6			),
	.tx_data7			(tx_data7			),

// Zustands Signale der FSM zur Fehlersuche
//	.rx_idle			(rx_idle			),	// ist schon als OUTPUT declariert.
	.rx_id1				(rx_id1				),
	.rx_rtr1			(rx_rtr1			),
	.rx_ide				(rx_ide				),
	.rx_id2				(rx_id2				),
	.rx_rtr2			(rx_rtr2			),
	.rx_r1				(rx_r1				),
	.rx_r0				(rx_r0				),
	.rx_dlc				(rx_dlc				),
	.rx_data			(rx_data			),
	.rx_crc				(rx_crc				),
	.rx_crc_lim			(rx_crc_lim			),
	.rx_ack				(rx_ack				),
	.rx_ack_lim			(rx_ack_lim			),
	.rx_eof				(rx_eof				),
//	.rx_inter			(rx_inter			),	// ist schon als OUTPUT declariert.
//	.error_frame			(error_frame			),	// ist schon als OUTPUT declariert.
	.error_frame_limz		(error_frame_limz		),
	.overload_frame			(overload_frame			),
	.overload_frame_limz		(overload_frame_limz		)
	);

//--------------------------------------------------------------------------------------------------------------------
can_btl i_can_btl
	(
	.can_clk			(can_clk			),
	.reset				(reset				),
	.rx				(rx				),	//rx vom Bus
	.tx				(tx				),	//tx Status

	//Bus-Timing Register
	.baudrate_presc			(baudrate_presc			),
	.sync_jump_width		(sync_jump_width		),

	.time_segment1			(time_segment1			),	//Prob-Seg + Puffer Seg 1
	.time_segment2			(time_segment2			),
	.samplemode			(sample_mode			),

	//Input von CAN_bsp (Busstatus--> Info fuer Hard- und Soft-Sync. )
	.rx_idle			(rx_idle			),
	.rx_inter			(rx_inter			),
	.transmitter			(transmitter			),	//Knoten sendet eigene Nachricht
	.transmitting			(transmitting			),	//Knoten sendet eigene Nachricht oder Error Frame, Overload Frame

	.go_rx_inter			(go_rx_inter			),
	.go_overload_frame		(go_overload_frame		),
	.go_error_frame			(go_error_frame			),
	.go_tx				(go_tx				),
	.send_ack			(send_ack			),
	//Input von can_emu
	.node_error_passive		(node_error_passive		),

	//Output von CAN_btl
	.sample_bit			(sample_bit			),	//abgetastetes Bit (Sample)
	.sample_strobe			(sample_strobe			),	//Strobe zur Uebername des Samples
	.clk_fsm			(clk_fsm			),
	.tx_strobe			(tx_strobe			),	//Strobe zum Senden eines Bits auf den Bus
	.hard_sync			(hard_sync			),	//Zeigt an ob eine hard_sync moeglich ist
	.tx_next			(tx_next			),	//naechstes Sendebit
	.sample_bit_pre			(sample_bit_pre			)	//Vorgaengerbit
	);

//---------------------------------------------------------------------------------------------------------------------
can_bsl i_can_bsl
	(
	.can_clk			(can_clk			),
	.reset				(reset				),
	.config_mode			(config_mode			),

	//Input von BTL
	.sample_bit			(sample_bit			),
	.sample_bit_pre			(sample_bit_pre			),
	.sample_strobe			(sample_strobe			),
	.tx_strobe			(tx_strobe			),

	//Input von BSP
	.tx				(tx_unstuffed			),
	.tx_next(tx_next),
	.go_overload_frame		(go_overload_frame		),
	.go_error_frame			(go_error_frame			),
	.go_crc_lim			(go_rx_crc_lim			),
	.go_id1				(go_rx_id1			),
	.rx_crc_lim_stuffing_disable	(rx_crc_lim_stuffing_disable	),
	.transmitting			(transmitting			),

	//Output
	.sample_strobe_q		(sample_strobe_q		),
	.tx_strobe_q			(tx_strobe_q			),
	.tx_stuffed			(tx				),
	.stuff_error			(stuff_error			)
	);

//---------------------------------------------------------------------------------------------------------------------
can_crc i_can_crc
	(
	.can_clk			(can_clk			),
	.reset				(reset				),
	.data				(sample_bit			),
	.enable				(crc_enable_stb			),
	.initialize			(go_crc_enable			),
	.crc				(calculated_crc			),
	.dlc0_crc			(dlc0_crc			),
	.set_dlc0_crc			(set_dlc0_crc			),
	.crc_error			(crc_error			),
	.error_frame_lim		(error_frame_lim		),
	.config_mode			(config_mode			),
	.crc_strobe			(crc_strobe			),
	.crc_reg			(crc				)
	);

//---------------------------------------------------------------------------------------------------------------------
can_emu i_can_emu
	(
	.can_clk			(can_clk			),
	.reset				(reset				),

	.we_error_cnt			(we_error_cnt			),
	.clear_we_error_cnt		(clear_we_error_cnt		),
	.read_error_code_capture_reg	(rst_error_code_reg		),
	//Errorarten von:
	//BSP
	.bit_error			(bit_error			),
	.form_error			(form_error			),
	.tx_request			(tx_request			),
	.ack_error			(ack_error			),
	.error_capture_code_segment	(error_capture_code_segment	),
	.error_direction		(error_direction		),
	.sample_strobe			(sample_strobe			),
	.sample_strobe_q		(sample_strobe_q		),
	.sample_bit			(sample_bit			),
	.go_overload_frame		(go_overload_frame		),
	.error_frame_lim		(error_frame_lim		),
	.error_flag_over		(error_flag_over		),
	.transmitter			(transmitter			),
	//.transmitting			(transmitting			),
	.arbitration_lost		(arbitration_lost		),
	.go_rx_ack_lim			(go_rx_ack_lim			),
	.go_error_frame			(go_error_frame			),
	.error_frame			(error_frame			),
	.rule5				(rule5				),
	.rule3_exception1_2		(rule3_exception1_2		),
	//.rx_idle			(rx_idle			),
	.error_cnt1			(error_cnt1			),
	.arbitration_field		(arbitration_field		),
	.tx_successful			(tx_successful			),
	.tx				(tx				),
	.delayed_dominant_cnt		(delayed_dominant_cnt		),

	//CRC
	.crc_error			(crc_error			),
	//BSL
	.stuff_error			(stuff_error			),
	//Kommandos
	.config_mode			(config_mode			),
	.listen_only_mode		(listen_only_mode		),
	.extended_mode			(extended_mode			),
	.error_warning_limit		(error_warning_limit		),	//hoehe des Errorzaehler bei einer Errorwarnung
	.data_rx_error_cnt		(data_rx_error_cnt		),
	.data_tx_error_cnt		(data_tx_error_cnt		),
	.rx_error_cnt			(rx_error_cnt			),
	.tx_error_cnt			(tx_error_cnt			),
	.exception_ir			(exception_ir			),

	//Ausgabe an BSP
	.error_strobe			(error_strobe			),
	.node_error_passive		(node_error_passive		),
	.node_error_active		(node_error_active		),
	.node_bus_off			(node_bus_off			),
	.bus_free			(bus_free			),

	//Ausgabe an Spartan
	.error_irq			(error_irq			),
	.error_warning			(error_warning			),
	.error_capture_code		(error_capture_code		),
	.bit_error_latched		(bit_error_latched		),
	.form_error_latched		(form_error_latched		),
	.stuff_error_latched		(stuff_error_latched		),
	.ack_error_latched		(ack_error_latched		)
	);

//--------------------------------------------------------------------------------------------------------------------

	generate
	genvar	i;
	for (i=1; i<=ACF_CHAINS; i=i+1)
	begin:acf
can_acf	#(
	.ACF_CODE			(i				))
i_can_acf
	 (
	 .can_clk			(can_clk			),
	 .reset				(reset				),


	 /* Mode register */
	 .config_mode			(config_mode			),
	 //acceptance_filter_mode,
	 .extended_mode			(extended_mode			),

	 .we_acf			(we_acf				),
	 .clear_we_acf			(clear_we_acf[i-1]		),

	 //Filterregeln
	 .ac_b_id_0			(ac_b_id			),
	 .am_b_id_0			(am_b_id			),
	 .ac_e_id_0			(ac_e_id			),
	 .am_e_id_0			(am_e_id			),
	 .ac_rtr1_0			(ac_rtr1			),
	 .am_rtr1_0			(am_rtr1			),
	 .ac_rtr2_0			(ac_rtr2			),
	 .am_rtr2_0			(am_rtr2			),
	 .ac_data_0			(ac_data			),
	 .am_data_0			(am_data			),

	 .acf_adr			(acf_adr			),
//	 .acf_code			(gen_tmp			),

	 //.go_rx_crc_lim		(go_rx_crc_lim			),
	 //.go_rx_inter			(go_rx_inter			),
	 //.go_error_frame		(go_error_frame			),

	 //Telegrammteile
	 .idx				(idz				),
	 .data0				(rx_data0[7:4]			),	// high rx_data0 fuer ACF an data0 vom ACF anlegen
	 .rtr1				(rtr1				),
	 .rtr2				(rtr2				),
	 .ide				(ide				),
	 .no_byte0			(no_byte0			),

	 .match_basic			(match_basic[i-1]		),
	 .match_std			(match_std[i-1]			),
	 .match_ext			(match_ext[i-1]			)

	 //.id_ok(id_ok)		//Ausgabe bei akzeptierten Telegramm
	);
	end
	endgenerate

//----------------------------------------------------------------------------------------------------------------------

//instanzieren von can_tx_buffer
can_tx_buffer #(
	.BASE_ADR			(BASE_ADR			),
	.TX_MAILBOX			(TX_MAILBOX			),
	.RX_MAILBOX			(RX_MAILBOX			),
	.ACF_CHAINS			(ACF_CHAINS			))
i_can_tx_buffer
	(
	.can_clk			(can_clk			),
	.clk_peri			(clk_peri			),
	.reset				(reset				),

  	.config_mode			(config_mode			),

	.tx_request_buffer		(tx_request_buffer		),
	.ready				(ready				),
	.retry_tx			(retry_tx			),

	.tx_start_adr 			(tx_start_adr			),

	.error_strobe 			(error_strobe			),
	.transmitter 			(transmitter			),

	.tx_successful 			(tx_successful			),
	.tx_request 			(tx_request			),
	.send_failed			(send_failed			),
	.listen_only_mode		(listen_only_mode		),

	//Ausgabe an can_blockram_access
	.read_tx_buffer 		(read_tx_buffer			)
	);

//----------------------------------------------------------------------------------------------------------------------

//instanzieren von can_rx_buffer
can_rx_buffer #(
	.BASE_ADR			(BASE_ADR			),
	.TX_MAILBOX			(TX_MAILBOX			),
	.RX_MAILBOX			(RX_MAILBOX			),
	.ACF_CHAINS			(ACF_CHAINS			))
i_can_rx_buffer
	(
	//.can_clk			(can_clk			),
	.clk_peri			(clk_peri			),
	.reset				(reset				),
	.config_mode			(config_mode			),
	.rx_used_buffer			(rx_used_buffer			),
	.rx_start_adr 			(rx_start_adr			),
	.receiving			(receiving			),
	.ready				(ready				),
	.rx_successful			(rx_successful			),
	.id_ok				(id_ok				),
	.write_rx_buffer		(write_rx_buffer		),
	.rx_buffer_full			(rx_buffer_full			),
	.rx_data_written		(rx_data_written		),
	.row_ready0			(row_ready0			)
	);

//----------------------------------------------------------------------------------------------------------------------

//parameter BASE_ADDR = 6'h0;

//instanzieren von can_cmd_interface
can_cmd_interface #(
	.BASE_ADR			(BASE_ADR			),
	.TX_MAILBOX			(TX_MAILBOX			),
	.RX_MAILBOX			(RX_MAILBOX			),
	.ACF_CHAINS			(ACF_CHAINS			))
i_can_cmd_interface
	(
	.clk_peri			(clk_peri			),
	.can_clk			(can_clk			),
	.sample_strobe_q		(sample_strobe_q		),
	.reset				(reset				),

	//Von can_bsp und can_emu
	.tx_successful			(tx_successful			),
	//.rx_successful 		(rx_successful			),
	//error_strobe,
	.transmitting 			(transmitting			),
	.receiving 			(receiving			),
	.bus_free 			(bus_free			),
	.node_error_passive 		(node_error_passive),
	.node_bus_off 			(node_bus_off			),
	.error_warning 			(error_warning			),

	.tx_start_adr 			(tx_start_adr			),
	.rx_start_adr 			(rx_start_adr			),

	.error_capture_code		(error_capture_code),
	.error_irq			(error_irq			),

	.can_mode 			(can_mode			), //alter config_mode
	.abort_tx 			(abort_tx			),
	//.en_exception_ir 		(en_exception_ir		),
	//.en_tx_successful_ir		(en_tx_successful_ir		),
	//.en_rx_successful_ir		(en_rx_successful_ir),
	.overload_request 		(overload_request		),
	.extended_mode 			(extended_mode			),
	//Ausgabe an can_emu
	.we_error_cnt 			(we_error_cnt			),
	.clear_we_error_cnt 		(clear_we_error_cnt		),
	//.we_rx_error_cnt 		(we_rx_error_cnt		),
	.data_rx_error_cnt 		(data_rx_error_cnt		),
	.data_tx_error_cnt 		(data_tx_error_cnt		),
	.rx_error_cnt 			(rx_error_cnt			),
	.tx_error_cnt 			(tx_error_cnt			),
	.error_warning_limit		(error_warning_limit		),
	.rst_error_code_reg		(rst_error_code_reg		),

	//Ausgabe an can_acf
	.we_acf 			(we_acf 			),
	.clear_we_acf 			(|clear_we_acf 			),
	.acf_en				(acf_en				),
	.acf_adr 			(acf_adr			),
	.ac_e_id 			(ac_e_id			),
	.am_e_id 			(am_e_id			),
	.ac_b_id 			(ac_b_id			),
	.am_b_id 			(am_b_id			),
	.ac_rtr1 			(ac_rtr1			),
	.am_rtr1 			(am_rtr1			),
	.ac_rtr2 			(ac_rtr2			),
	.am_rtr2 			(am_rtr2			),
	.ac_data 			(ac_data			),
	.am_data 			(am_data			),

	//Ausgabe an can_btl
	.baudrate_presc 		(baudrate_presc			),
	.sync_jump_width 		(sync_jump_width		),
	.time_segment1 			(time_segment1			),
	.time_segment2 			(time_segment2			),
	.sample_mode 			(sample_mode			),

	.wr_peri 			(wr_peri			),	//Write Enable vom SpartanMC
	.activ_peri 			(access_peri			),	//aktiviere Peripherieregister

	.data_in 			(do_peri			),	// Datenbus
	.data_out 			(di_peri_io			),	// Datenbus (15 Register)
	.adr 				(addr_peri			),	// Adressbus

	//Ausgabe an can_tx_buffer can_rx_buffer
	//.retry_tx 			(retry_tx			),
	.tx_request_buffer 		(tx_request_buffer		),
	.rx_used_buffer 		(rx_used_buffer			),

	.rx_buffer_full			(rx_buffer_full			),
	.send_failed			(send_failed			),
	.tx_successful_ir 		(tx_successful_ir		),
	.rx_successful_ir 		(rx_successful_ir		),
	.set_exception_ir		(set_exception_ir		),
	.exception_ir			(exception_ir			),
	.rx_data_written		(rx_data_written		)
	);

//----------------------------------------------------------------------------------------------------------------------

//instanzieren von can_blockram_access
can_blockram_access #(
	.BASE_ADR			(BASE_ADR			),
	.TX_MAILBOX			(TX_MAILBOX			),
	.RX_MAILBOX			(RX_MAILBOX			),
	.ACF_CHAINS			(ACF_CHAINS			))
i_can_blockram_access
	(
	.clk_peri			(clk_peri			),
	.reset				(reset				),

	//Von can_rx_buffer und can_tx_buffer
	.write_rx_buffer		(write_rx_buffer		),
	.tx_start_adr			(tx_start_adr			),
	.read_tx_buffer			(read_tx_buffer			),
	.rx_start_adr			(rx_start_adr			),

	//Ausgabe an can_rx_buffer und can_tx_buffer
	.ready				(ready				),
	//Ausgabe an can_tx_buffer
	.retry_tx			(retry_tx			),

	//Von can_acf
	.match				(match				),

	//Ausgabe an das Blockram
	.dma_adr_A			(adr_dma			),
	.dma_we_A			(we_dma				),
	.dma_dataI_A			(di_dma				),
	.dma_dataO_A			(do_dma				), //zum lesen der Mailbox

	.row_ready0			(row_ready0			),
	.row_ready8			(row_ready8			),


	//Signale von can_crc
	.calculated_crc			(calculated_crc			),

	//Input von can_bsp
	.id				(idz				),
	.ide				(ide				),
	.rtr1				(rtr1				),
	.rtr2				(rtr2				),
	.data_length			(data_length			),
	.rx_data0			(rx_data0			),
	.rx_data1			(rx_data1			),
	.rx_data2			(rx_data2			),
	.rx_data3			(rx_data3			),
	.rx_data4			(rx_data4			),
	.rx_data5			(rx_data5			),
	.rx_data6			(rx_data6			),
	.rx_data7			(rx_data7			),

	//Ausgabe an can_bsp
	.tx_ide				(tx_ide				),
	.tx_rtr				(tx_rtr				),
	.tx_dlc				(tx_dlc				),
	.tx_basicID			(tx_basicID			),
	.tx_extID			(tx_extID			),
	.tx_data0			(tx_data0			),
	.tx_data1			(tx_data1			),
	.tx_data2			(tx_data2			),
	.tx_data3			(tx_data3			),
	.tx_data4			(tx_data4			),
	.tx_data5			(tx_data5			),
	.tx_data6			(tx_data6			),
	.tx_data7			(tx_data7			)
	);


// DMA Puffer fuer den CAN Modul
// Dekodierung des DMA-Speicherblocks
wire            activ_dma0;

pselect dmasel0 (
			.addr(		addr_high[7:0]	),
			.activ_peri(	mem_access	),
			.select(	activ_dma0	)
                );

defparam dmasel0.ADDR_WIDTH = 8;
defparam dmasel0.BASE_WIDTH = 8;
defparam dmasel0.BASE_ADDR  = DMA_BASE_ADR >> 10;	// Oberen 8 Bit der Speicherblockadresse

ram_18_prim		#(
			.WRITE_MODE_A(	"READ_FIRST"			),
			.INIT_A(	18'h00000			),
			.SRVAL_A(	18'h00000			),
			.WRITE_MODE_B(	"READ_FIRST"			),
			.INIT_B(	18'h00000			),
			.SRVAL_B(	18'h00000			),
			.SIM_COLLISION_CHECK("NONE"			)
			)
DMARAM00		(
			.DIA		(di_dma[17:0]			),
			.ADDRA		(adr_dma[9:0]			),
 			.ENA		(1'b1				),      // en_dma
			.WEA		(we_dma				),
			.SSRA		(1'b0				),      // ssr_dma
			.CLKA		(clk_peri			),	// = mem_clk =  clk_peri
			.DOA		(do_dma[17:0]			),
			.DIB		(do_peri[17:0]			),
			.ADDRB		(addr_peri[9:0]			),
			.WEB		((store_access_low|store_access_high) & activ_dma0	),
			.ENB		(1'b1				),
			.SSRB		(! activ_dma0			),
			.CLKB		(clk_peri			),
			.DOB		(di_peri_dma[17:0]		)
			);

// ****************************************************************

endmodule
