/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  TX Puffer                                                  ////
////  Diese Modul verwaltet den Sendepuffer und fuehrt           ////
////  Adressrechnungen aus.                                      ////
////                                                             ////
////  Author: Gerald Hempel                                      ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none

module can_tx_buffer(
	can_clk,
	clk_peri,
	reset,

	config_mode,

	tx_request_buffer, 		//SpartanMC-Register zu versendende Telegramme
	ready,				//Signal von blockram_access beim Beenden den Lesevorgangs
	retry_tx,			//SpartanMC-Register Anzahl der Sendeversuche

	tx_start_adr,			//Startadr. zum Lesen aus dem DMA-Blockram

	error_strobe,
	transmitter,

	tx_successful,
	tx_request,

	//Ausgabe an can_blockram_access
	read_tx_buffer,
	listen_only_mode,
	//Ausgabe an can_cmd_interface
	//adr_nr_tx,
	send_failed
	);

parameter TX_MAILBOX	= 18; //1..18
parameter RX_MAILBOX	= 18; //1..18
parameter ACF_CHAINS	= 18; //1..18
parameter BASE_ADR	= 6'h8;

input wire				can_clk;
input wire				clk_peri;
input wire				reset;

input wire	[TX_MAILBOX-1:0]	tx_request_buffer; 	//SpartanMC-Register zu versendende Telegramme
input wire				ready;			//Signal von blockram_access beim Beenden den Lesevorgangs
input wire	[5:0]			retry_tx;		//SpartanMC-Register Anzahl der Sendeversuche
input wire				error_strobe;		//von can_emu
input wire				transmitter;		//von can_bsp
input wire				tx_successful;		//von can_bsp
input wire				listen_only_mode;	//von can_top
input wire				config_mode;

output	[9:0]			tx_start_adr;		//Startadr. zum Lesen aus dem DMA-Blockram

//Ausgabe an can_blockram_access
output				read_tx_buffer;

//Ausgabe an can_cmd_interface
//output			adr_nr_tx;
output				send_failed;

//Ausgabe an cn_bsp
output				tx_request;

reg				read_tx_buffer;
reg	[9:0]			tx_start_adr;
reg				tx_request;
reg				state;
reg				tx_available;

reg	[5:0]			error_cnt;

//wire	[5:0]			adr_nr_tx;
wire				send_failed;

/**
 * Signale mit clk_peri synchronisieren
 */
reg				error_strobe_s;		//von can_emu
reg				transmitter_s;		//von can_bsp
reg				tx_successful_s;	//von can_bsp

parameter	IDLE	= 0;
parameter	READ	= 1;

//integer	i,i_old;

//assign	adr_nr_tx	= {state, i[4:0]};
//assign		send_failed	= (error_cnt == retry_tx);
assign		send_failed	= (error_cnt > retry_tx);

/**
 * Signale mit clk_peri synchronisieren
 */
always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		begin
		error_strobe_s	<= 1'b0;	//von can_emu
		transmitter_s	<= 1'b0;	//von can_bsp
		tx_successful_s	<= 1'b0;	//von can_bsp
		end
	else
		begin
		error_strobe_s	<= error_strobe;	//von can_emu
		transmitter_s	<= transmitter;		//von can_bsp
		tx_successful_s	<= tx_successful;	//von can_bsp
		end
end


//Finde das Telegramm mit der niedrigsten Adresse
/*always @ (posedge can_clk or posedge reset)
begin
	if (reset)
	begin
		 state <= IDLE;
	end
	else
	begin	
		
		i_old = TX_MAILBOX-1;
		
		for (i=0; i<TX_MAILBOX; i=i+1)
		begin
			if (tx_request_buffer[i]==1 && tx_available && i<=i_old)
			begin
				state 		 <= READ;
				tx_start_adr <= i[9:0]<<3;//i*8
			end
			else if (tx_available && i<=i_old)
				state 		 <= IDLE;
		end
	end	
end*/

wire	need_to_tx		= |tx_request_buffer;
wire	[4:0]			tmp_adr;

always @ (posedge clk_peri or posedge reset)
begin
	if (reset)
		begin
			state <= IDLE;
			tx_start_adr <= 10'h0;
		end
	else if (tx_available)
		begin
			state 		 <= READ;
			//tx_start_adr <= tx_start_adr;//i*8
			tx_start_adr <= {2'b00,tmp_adr,3'b000};//i*8
		end
		else if (~tx_request)
			state 		 <= IDLE;
end
	
//Lese das gefundene Telegramm aus dem DMA-Blockram 
always @ (posedge clk_peri or posedge reset)
begin
   if (reset)
   read_tx_buffer <= 1'b0;
   else
   begin
      case (state)
         IDLE:;
         READ:
         begin	
            if (tx_available)	
               read_tx_buffer <= 1'b1;
            else if (ready)
               read_tx_buffer <= 1'b0;
         end
      endcase
   end
end

 

//Telegramm Sendefehler zaehlen
always @ (posedge can_clk or posedge reset)
begin
    if (reset)
       error_cnt <= 4'h0;
    else if (error_strobe_s && transmitter_s)	// ACHTUNG! error_strobe ist gleich go_error_frame
       error_cnt <= error_cnt + 1'b1;
//    else if (error_cnt == retry_tx) //retry_tx is input
    else if (error_cnt > retry_tx) //retry_tx is input
       error_cnt <= 4'h0;
end

//tx_request verwalten
always @ (posedge clk_peri or posedge reset)
begin
    if (reset)
       tx_request <= 1'b0;
    else if (tx_available && (state==READ))
       tx_request <= 1'b1;
    else if (config_mode)
       tx_request <= 1'b0;  
//    else if (error_cnt == (retry_tx + 1'b1)) //retry_tx is input
    else if (error_cnt > (retry_tx)) //retry_tx is input
       tx_request <= 1'b0;
    else if (tx_successful_s)
       tx_request <= 1'b0;     
end

//setzen von tx_available
always @ (posedge clk_peri or posedge reset)
begin
   if (reset)
      tx_available <= 1'b0;
   else if (~tx_request && need_to_tx && ~transmitter_s && ~listen_only_mode)
      tx_available <= 1'b1;
   else
      tx_available <= 1'b0;
end

generate
	case (TX_MAILBOX)
		1:begin:TX_Single_Buffer
			assign tmp_adr = 	5'b0;
		end
		2:begin:TX_Double_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[1:0]==2'b01;
			assign tmp_adr[1] = 	1'b0;
			assign tmp_adr[2] =	1'b0;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		3:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[2:1]==3'b01;
			assign tmp_adr[1] = 	tx_request_buffer[2:0]==3'b001;
			assign tmp_adr[2] =	1'b0;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		4:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[3:2]==4'b01	|
						tx_request_buffer[3:0]==4'b0001  ;
			assign tmp_adr[1] = 	tx_request_buffer[3:1]==4'b001  |
						tx_request_buffer[3:0]==4'b0001  ;
			assign tmp_adr[2] =	1'b0;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		5:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[4:3]==5'b01    |
						tx_request_buffer[4:1]==5'b0001   ;
			assign tmp_adr[1] = 	tx_request_buffer[4:2]==5'b001   |
						tx_request_buffer[4:1]==5'b0001   ;
			assign tmp_adr[2] =	tx_request_buffer[4:0]==5'b0000_1 ;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		6:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[5:4]==6'b01		|
						tx_request_buffer[5:2]==6'b0001 	|
						tx_request_buffer[5:0]==6'b0000_01	 ;
			assign tmp_adr[1] = 	tx_request_buffer[5:3]==6'b001 		|
						tx_request_buffer[5:2]==6'b0001		 ;
			assign tmp_adr[2] =	tx_request_buffer[5:1]==6'b0000_1 	|
						tx_request_buffer[5:0]==6'b0000_01	 ;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		7:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[6:5]==7'b01 		|
						tx_request_buffer[6:3]==7'b0001 	|
						tx_request_buffer[6:1]==7'b0000_01	 ;
			assign tmp_adr[1] = 	tx_request_buffer[6:4]==7'b001 		|
						tx_request_buffer[6:3]==7'b0001 	|
						tx_request_buffer[6:0]==7'b0000_001	 ;
			assign tmp_adr[2] =	tx_request_buffer[6:2]==7'b0000_1 	|
						tx_request_buffer[6:1]==7'b0000_01	|
						tx_request_buffer[6:0]==7'b0000_001	 ;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		8:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[7:6]==8'b01		|
						tx_request_buffer[7:4]==8'b0001		|
						tx_request_buffer[7:2]==8'b0000_01	|
						tx_request_buffer[7:0]==8'b0000_0001	 ;
			assign tmp_adr[1] = 	tx_request_buffer[7:5]==8'b001		|
						tx_request_buffer[7:4]==8'b0001		|
						tx_request_buffer[7:1]==8'b0000_001	|
						tx_request_buffer[7:0]==8'b0000_0001	 ;
			assign tmp_adr[2] =	tx_request_buffer[7:3]==8'b0000_1	|
						tx_request_buffer[7:2]==8'b0000_01	|
						tx_request_buffer[7:1]==8'b0000_001	|
						tx_request_buffer[7:0]==8'b0000_0001	 ;
			assign tmp_adr[3] = 	1'b0;
			assign tmp_adr[4] = 	1'b0;
		end
		9:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[8:7]==9'b01		|
						tx_request_buffer[8:5]==9'b0001 	|
						tx_request_buffer[8:3]==9'b0000_01	|
						tx_request_buffer[8:1]==9'b0000_0001	 ;
			assign tmp_adr[1] = 	tx_request_buffer[8:6]==9'b001		|
						tx_request_buffer[8:5]==9'b0001		|
						tx_request_buffer[8:2]==9'b0000_001	|
						tx_request_buffer[8:1]==9'b0000_0001	 ;
			assign tmp_adr[2] =	tx_request_buffer[8:4]==9'b0000_1	|
						tx_request_buffer[8:3]==9'b0000_01	|
						tx_request_buffer[8:2]==9'b0000_001	|
						tx_request_buffer[8:1]==9'b0000_0001	 ;
			assign tmp_adr[3] = 	tx_request_buffer[8:0]==9'b0000_0000_1	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		10:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[9:8]==10'b01			|
						tx_request_buffer[9:6]==10'b0001		|
						tx_request_buffer[9:4]==10'b0000_01		|
						tx_request_buffer[9:2]==10'b0000_0001		|
						tx_request_buffer[9:0]==10'b0000_0000_01	 ;
			assign tmp_adr[1] = 	tx_request_buffer[9:7]==10'b001			|
						tx_request_buffer[9:6]==10'b0001		|
						tx_request_buffer[9:3]==10'b0000_001		|
						tx_request_buffer[9:2]==10'b0000_0001		 ;
			assign tmp_adr[2] =	tx_request_buffer[9:5]==10'b0000_1		|
						tx_request_buffer[9:4]==10'b0000_01		|
						tx_request_buffer[9:3]==10'b0000_001		|
						tx_request_buffer[9:2]==10'b0000_0001		 ;
			assign tmp_adr[3] = 	tx_request_buffer[9:1]==10'b0000_0000_1		|
						tx_request_buffer[9:0]==10'b0000_0000_01	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		11:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[10:9]==11'b01			|
						tx_request_buffer[10:7]==11'b0001		|
						tx_request_buffer[10:5]==11'b0000_01		|
						tx_request_buffer[10:3]==11'b0000_0001		|
						tx_request_buffer[10:1]==11'b0000_0000_01	 ;
			assign tmp_adr[1] = 	tx_request_buffer[10:8]==11'b001 		|
						tx_request_buffer[10:7]==11'b0001		|
						tx_request_buffer[10:4]==11'b0000_001		|
						tx_request_buffer[10:3]==11'b0000_0001		|
						tx_request_buffer[10:0]==11'b0000_0000_001	 ;
			assign tmp_adr[2] =	tx_request_buffer[10:6]==11'b0000_1		|
						tx_request_buffer[10:5]==11'b0000_01		|
						tx_request_buffer[10:4]==11'b0000_001		|
						tx_request_buffer[10:3]==11'b0000_0001		 ;
			assign tmp_adr[3] = 	tx_request_buffer[10:2]==11'b0000_0000_1	|
						tx_request_buffer[10:1]==11'b0000_0000_01	|
						tx_request_buffer[10:0]==11'b0000_0000_001	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		12:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[11:10]==12'b01		|
						tx_request_buffer[11:8] ==12'b0001 		|
						tx_request_buffer[11:6] ==12'b0000_01		|
						tx_request_buffer[11:4] ==12'b0000_0001 	|
						tx_request_buffer[11:2] ==12'b0000_0000_01 	|
						tx_request_buffer[11:0] ==12'b0000_0000_0001 	 ;
			assign tmp_adr[1] = 	tx_request_buffer[11:9] ==12'b001 		|
						tx_request_buffer[11:8] ==12'b0001 		|
						tx_request_buffer[11:5] ==12'b0000_001 		|
						tx_request_buffer[11:4] ==12'b0000_0001 	|
						tx_request_buffer[11:1] ==12'b0000_0000_001 	|
						tx_request_buffer[11:0] ==12'b0000_0000_0001 	 ;
			assign tmp_adr[2] =	tx_request_buffer[11:7] ==12'b0000_1 		|
						tx_request_buffer[11:6] ==12'b0000_01 		|
						tx_request_buffer[11:5] ==12'b0000_001 		|
						tx_request_buffer[11:4] ==12'b0000_0001 	 ;
			assign tmp_adr[3] = 	tx_request_buffer[11:3] ==12'b0000_0000_1 	|
						tx_request_buffer[11:2] ==12'b0000_0000_01 	|
						tx_request_buffer[11:1] ==12'b0000_0000_001 	|
						tx_request_buffer[11:0] ==12'b0000_0000_0001 	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		13:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[12:11]==13'b01		|
						tx_request_buffer[12:9] ==13'b0001		|
						tx_request_buffer[12:7] ==13'b0000_01		|
						tx_request_buffer[12:5] ==13'b0000_0001		|
						tx_request_buffer[12:3] ==13'b0000_0000_01	|
						tx_request_buffer[12:1] ==13'b0000_0000_0001	 ;
			assign tmp_adr[1] = 	tx_request_buffer[12:10]==13'b001		|
						tx_request_buffer[12:9] ==13'b0001		|
						tx_request_buffer[12:6] ==13'b0000_001		|
						tx_request_buffer[12:5] ==13'b0000_0001		|
						tx_request_buffer[12:2] ==13'b0000_0000_001	|
						tx_request_buffer[12:1] ==13'b0000_0000_0001	 ;
			assign tmp_adr[2] =	tx_request_buffer[12:8] ==13'b0000_1		|
						tx_request_buffer[12:7] ==13'b0000_01		|
						tx_request_buffer[12:6] ==13'b0000_001		|
						tx_request_buffer[12:5] ==13'b0000_0001		|
						tx_request_buffer[12:0] ==13'b0000_0000_0000_1	 ;
			assign tmp_adr[3] = 	tx_request_buffer[12:4] ==13'b0000_0000_1	|
						tx_request_buffer[12:3] ==13'b0000_0000_01	|
						tx_request_buffer[12:2] ==13'b0000_0000_001	|
						tx_request_buffer[12:1] ==13'b0000_0000_0001	|
						tx_request_buffer[12:0] ==13'b0000_0000_0000_1	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		14:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[13:12]==14'b01		|
						tx_request_buffer[13:10]==14'b0001		|
						tx_request_buffer[13:8] ==14'b0000_01		|
						tx_request_buffer[13:6] ==14'b0000_0001		|
						tx_request_buffer[13:4] ==14'b0000_0000_01	|
						tx_request_buffer[13:2] ==14'b0000_0000_0001	|
						tx_request_buffer[13:0] ==14'b0000_0000_0000_01	 ;
			assign tmp_adr[1] = 	tx_request_buffer[13:11]==14'b001		|
						tx_request_buffer[13:10]==14'b0001		|
						tx_request_buffer[13:7] ==14'b0000_001		|
						tx_request_buffer[13:6] ==14'b0000_0001		|
						tx_request_buffer[13:3] ==14'b0000_0000_001	|
						tx_request_buffer[13:2] ==14'b0000_0000_0001	 ;
			assign tmp_adr[2] =	tx_request_buffer[13:9] ==14'b0000_1		|
						tx_request_buffer[13:8] ==14'b0000_01		|
						tx_request_buffer[13:7] ==14'b0000_001		|
						tx_request_buffer[13:6] ==14'b0000_0001		|
						tx_request_buffer[13:1] ==14'b0000_0000_0000_1	|
						tx_request_buffer[13:0] ==14'b0000_0000_0000_01	 ;
			assign tmp_adr[3] = 	tx_request_buffer[13:5] ==14'b0000_0000_1	|
						tx_request_buffer[13:4] ==14'b0000_0000_01	|
						tx_request_buffer[13:3] ==14'b0000_0000_001	|
						tx_request_buffer[13:2] ==14'b0000_0000_0001	|
						tx_request_buffer[13:1] ==14'b0000_0000_0000_1	|
						tx_request_buffer[13:0] ==14'b0000_0000_0000_01	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		15:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[14:13]==15'b01			|
						tx_request_buffer[14:11]==15'b0001			|
						tx_request_buffer[14:9] ==15'b0000_01			|
						tx_request_buffer[14:7] ==15'b0000_0001			|
						tx_request_buffer[14:5] ==15'b0000_0000_01		|
						tx_request_buffer[14:3] ==15'b0000_0000_0001		|
						tx_request_buffer[14:1] ==15'b0000_0000_0000_01		 ;
			assign tmp_adr[1] = 	tx_request_buffer[14:12]==15'b001			|
						tx_request_buffer[14:11]==15'b0001			|
						tx_request_buffer[14:8] ==15'b0000_001			|
						tx_request_buffer[14:7] ==15'b0000_0001			|
						tx_request_buffer[14:4] ==15'b0000_0000_001		|
						tx_request_buffer[14:3] ==15'b0000_0000_0001		|
						tx_request_buffer[14:0] ==15'b0000_0000_0000_001	 ;
			assign tmp_adr[2] =	tx_request_buffer[14:10]==15'b0000_1			|
						tx_request_buffer[14:9] ==15'b0000_01			|
						tx_request_buffer[14:8] ==15'b0000_001			|
						tx_request_buffer[14:7] ==15'b0000_0001			|
						tx_request_buffer[14:2] ==15'b0000_0000_0000_1		|
						tx_request_buffer[14:1] ==15'b0000_0000_0000_01		|
						tx_request_buffer[14:0] ==15'b0000_0000_0000_001	 ;
			assign tmp_adr[3] = 	tx_request_buffer[14:6] ==15'b0000_0000_1		|
						tx_request_buffer[14:5] ==15'b0000_0000_01		|
						tx_request_buffer[14:4] ==15'b0000_0000_001		|
						tx_request_buffer[14:3] ==15'b0000_0000_0001		|
						tx_request_buffer[14:2] ==15'b0000_0000_0000_1		|
						tx_request_buffer[14:1] ==15'b0000_0000_0000_01		|
						tx_request_buffer[14:0] ==15'b0000_0000_0000_001	 ;
			assign tmp_adr[4] = 	1'b0;
		end
		16:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[15:14]==16'b01			|
						tx_request_buffer[15:12]==16'b0001			|
						tx_request_buffer[15:10]==16'b0000_01			|
						tx_request_buffer[15:8] ==16'b0000_0001			|
						tx_request_buffer[15:6] ==16'b0000_0000_01		|
						tx_request_buffer[15:4] ==16'b0000_0000_0001		|
						tx_request_buffer[15:2] ==16'b0000_0000_0000_01		|
						tx_request_buffer[15:0] ==16'b0000_0000_0000_0001	 ;
			assign tmp_adr[1] = 	tx_request_buffer[15:13]==16'b001			|
						tx_request_buffer[15:12]==16'b0001			|
						tx_request_buffer[15:9] ==16'b0000_001			|
						tx_request_buffer[15:8] ==16'b0000_0001			|
						tx_request_buffer[15:5] ==16'b0000_0000_001		|
						tx_request_buffer[15:4] ==16'b0000_0000_0001		|
						tx_request_buffer[15:1] ==16'b0000_0000_0000_001	|
						tx_request_buffer[15:0] ==16'b0000_0000_0000_0001	 ;
			assign tmp_adr[2] =	tx_request_buffer[15:11]==16'b0000_1			|
						tx_request_buffer[15:10]==16'b0000_01			|
						tx_request_buffer[15:9] ==16'b0000_001			|
						tx_request_buffer[15:8] ==16'b0000_0001			|
						tx_request_buffer[15:3] ==16'b0000_0000_0000_1		|
						tx_request_buffer[15:2] ==16'b0000_0000_0000_01		|
						tx_request_buffer[15:1] ==16'b0000_0000_0000_001	|
						tx_request_buffer[15:0] ==16'b0000_0000_0000_0001	 ;
			assign tmp_adr[3] = 	tx_request_buffer[15:7] ==16'b0000_0000_1 		|
						tx_request_buffer[15:6] ==16'b0000_0000_01 		|
						tx_request_buffer[15:5] ==16'b0000_0000_001 		|
						tx_request_buffer[15:4] ==16'b0000_0000_0001 		|
						tx_request_buffer[15:3] ==16'b0000_0000_0000_1 		|
						tx_request_buffer[15:2] ==16'b0000_0000_0000_01		|
						tx_request_buffer[15:1] ==16'b0000_0000_0000_001	|
						tx_request_buffer[15:0] ==16'b0000_0000_0000_0001	 ;
			assign tmp_adr[4] = 	1'b0; 
		end
		17:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[16:15]==17'b01			|
						tx_request_buffer[16:13]==17'b0001			|
						tx_request_buffer[16:11]==17'b0000_01			|
						tx_request_buffer[16:9] ==17'b0000_0001			|
						tx_request_buffer[16:7] ==17'b0000_0000_01		|
						tx_request_buffer[16:5] ==17'b0000_0000_0001		|
						tx_request_buffer[16:3] ==17'b0000_0000_0000_01		|
						tx_request_buffer[16:1] ==17'b0000_0000_0000_0001	 ;
			assign tmp_adr[1] = 	tx_request_buffer[16:14]==17'b001			|
						tx_request_buffer[16:13]==17'b0001			|
						tx_request_buffer[16:10]==17'b0000_001			|
						tx_request_buffer[16:9] ==17'b0000_0001			|
						tx_request_buffer[16:6] ==17'b0000_0000_001		|
						tx_request_buffer[16:5] ==17'b0000_0000_0001		|
						tx_request_buffer[16:2] ==17'b0000_0000_0000_001	|
						tx_request_buffer[16:1] ==17'b0000_0000_0000_0001	 ;
			assign tmp_adr[2] =	tx_request_buffer[16:12]==17'b0000_1			|
						tx_request_buffer[16:11]==17'b0000_01			|
						tx_request_buffer[16:10]==17'b0000_001			|
						tx_request_buffer[16:9] ==17'b0000_0001			|
						tx_request_buffer[16:0] ==17'b0000_0000_0000_1		|
						tx_request_buffer[16:3] ==17'b0000_0000_0000_01		|
						tx_request_buffer[16:2] ==17'b0000_0000_0000_001	|
						tx_request_buffer[16:1] ==17'b0000_0000_0000_0001	 ;
			assign tmp_adr[3] = 	tx_request_buffer[16:8] ==17'b0000_0000_1		|
						tx_request_buffer[16:7] ==17'b0000_0000_01		|
						tx_request_buffer[16:6] ==17'b0000_0000_001		|
						tx_request_buffer[16:5] ==17'b0000_0000_0001		|
						tx_request_buffer[16:4] ==17'b0000_0000_0000_1		|
						tx_request_buffer[16:3] ==17'b0000_0000_0000_01		|
						tx_request_buffer[16:2] ==17'b0000_0000_0000_001	|
						tx_request_buffer[16:1] ==17'b0000_0000_0000_0001	 ;
			assign tmp_adr[4] = 	tx_request_buffer[16:0] ==17'b0000_0000_0000_0000_1	 ;
		end
		18:begin:TX_Buffer_Decoder
			assign tmp_adr[0] = 	tx_request_buffer[17:16]==18'b01			|
						tx_request_buffer[17:14]==18'b0001			|
						tx_request_buffer[17:12]==18'b0000_01			|
						tx_request_buffer[17:10]==18'b0000_0001			|
						tx_request_buffer[17:8] ==18'b0000_0000_01		|
						tx_request_buffer[17:6] ==18'b0000_0000_0001		|
						tx_request_buffer[17:4] ==18'b0000_0000_0000_01		|
						tx_request_buffer[17:2] ==18'b0000_0000_0000_0001	|
						tx_request_buffer[17:0] ==18'b0000_0000_0000_0000_01	 ;
			assign tmp_adr[1] = 	tx_request_buffer[17:15]==18'b001			|
						tx_request_buffer[17:14]==18'b0001			|
						tx_request_buffer[17:11]==18'b0000_001			|
						tx_request_buffer[17:10]==18'b0000_0001			|
						tx_request_buffer[17:7] ==18'b0000_0000_001		|
						tx_request_buffer[17:6] ==18'b0000_0000_0001		|
						tx_request_buffer[17:3] ==18'b0000_0000_0000_001	|
						tx_request_buffer[17:2] ==18'b0000_0000_0000_0001	 ;
			assign tmp_adr[2] =	tx_request_buffer[17:13]==18'b0000_1			|
						tx_request_buffer[17:12]==18'b0000_01			|
						tx_request_buffer[17:11]==18'b0000_001			|
						tx_request_buffer[17:10]==18'b0000_0001			|
						tx_request_buffer[17:5] ==18'b0000_0000_0000_1		|
						tx_request_buffer[17:4] ==18'b0000_0000_0000_01		|
						tx_request_buffer[17:3] ==18'b0000_0000_0000_001	|
						tx_request_buffer[17:2] ==18'b0000_0000_0000_0001	 ;
			assign tmp_adr[3] = 	tx_request_buffer[17:9] ==18'b0000_0000_1		|
						tx_request_buffer[17:8] ==18'b0000_0000_01		|
						tx_request_buffer[17:7] ==18'b0000_0000_001		|
						tx_request_buffer[17:6] ==18'b0000_0000_0001		|
						tx_request_buffer[17:5] ==18'b0000_0000_0000_1		|
						tx_request_buffer[17:4] ==18'b0000_0000_0000_01		|
						tx_request_buffer[17:3] ==18'b0000_0000_0000_001	|
						tx_request_buffer[17:2] ==18'b0000_0000_0000_0001	 ;
			assign tmp_adr[4] = 	tx_request_buffer[17:1] ==18'b0000_0000_0000_0000_1	| 
						tx_request_buffer[17:0] ==18'b0000_0000_0000_0000_01	 ;
		end
	endcase
endgenerate

/*//Ermitteln der tx_startadr aus i
always @ (posedge can_clk)
begin
  int k;
      
  for (k=0; k<=(adr_to_tx_buffer-(RX_MAILBOX[9:0]*6)); k=k+1)
      begin
          if ((k*6) == (adr_to_tx_buffer-(RX_MAILBOX[9:0]*6)-01'd17))
          begin
             tx_request_buffer[k:(k+1)] <= data_to_tx_buffer;
             //adr_fault <= 1'b0;
          end
         //else
         //    adr_fault <= 1'b1;//geht nicht weil andere sachen auch geschrieben werden müssen
      end
   //end
end*/

//immer bei READ adresse-offset
//

/*always @ (posedge can_clk or posedge reset)
begin
    if (reset)
       prio <= 2'b00;
    else
    begin
        prio <= 2'b00;
        int j;
        for (j=0; j<=TX_MAILBOX; j=j+2)
        begin
            if (tx_request_buffer[j] || tx_request_buffer[j+1])
               new_prio <= {tx_request_buffer[j], tx_request_buffer[j+1]};
               if (new_prio > prio)
                  prio <= new_prio;
        end
    end
end*/


/*always @ (posedge can_clk or posedge reset)
begin
    if (reset)
    begin
       last_request_buffer_adr <= 5'b0;
       state <= IDLE;
    end
    else if (tx_available)
    begin
       adr_to_read1 <= last_request_buffer_adr;
       adr_to_read2 <= last_request_buffer_adr;

       int i;
    
       for (i=0; i<=TX_MAILBOX; i=i+2)
       begin
           if ({tx_request_buffer[i],tx_request_buffer[i]} == prio)
           begin
               if ((i < adr_to_read1) && (i < last_request_buffer_adr))
                   adr_to_read1 <= i;
               if ((i > adr_to_read2) && (i > last_request_buffer_adr))
                   adr_to_read2 <= i;
           end
           if ((adr_to_read1==last_request_buffer_adr) 
           && (adr_to_read2!=last_request_buffer_adr))
           begin
              adr_to_read <= adr_to_read2;
              state <= READ;
              need_to_tx <= 1'b1;
           end
           if ((adr_to_read1!=last_request_buffer_adr) 
           && (adr_to_read2==last_request_buffer_adr))
           begin
              adr_to_read <= adr_to_read1;
              state <= READ;
              need_to_tx <= 1'b1;
           end
           if ((adr_to_read1!=last_request_buffer_adr) 
           && (adr_to_read2!=last_request_buffer_adr))
           begin
              adr_to_read <= adr_to_read2;
              state <= READ; 
              need_to_tx <= 1'b1;
           end
           else
           begin
              state <= IDLE;
              need_to_tx <= 1'b0;
          end
       end
   end
end*/


endmodule
