/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module cyclone4_pll (
			input wire inclk0,
			input wire reset_i,
			output wire c0,
			output wire c1,
			output wire c2,
			output wire c3,
			output wire c4,
			output wire locked);
			
  parameter
    RESET_LEVEL = "LOW_ACTIVE",
    INCLK0_INPUT_FREQUENCY = 20000 ,
    CLK0_DIVIDE_BY = 1 ,
    CLK0_DUTY_CYCLE = 50 ,
    CLK0_MULTIPLY_BY = 1 ,
    CLK0_PHASE_SHIFT = "0",
    CLK1_DIVIDE_BY = 1 ,
    CLK1_DUTY_CYCLE = 50 ,
    CLK1_MULTIPLY_BY = 1 ,
    CLK1_PHASE_SHIFT = "0",
    CLK2_DIVIDE_BY = 1 ,
    CLK2_DUTY_CYCLE = 50 ,
    CLK2_MULTIPLY_BY = 1 ,
    CLK2_PHASE_SHIFT = "0",
    CLK3_DIVIDE_BY = 1 ,
    CLK3_DUTY_CYCLE = 50 ,
    CLK3_MULTIPLY_BY = 1 ,
    CLK3_PHASE_SHIFT = "0",
    CLK4_DIVIDE_BY = 1 ,
    CLK4_DUTY_CYCLE = 50 ,
    CLK4_MULTIPLY_BY = 1 ,
    CLK4_PHASE_SHIFT = "0";

  reg reset_clock;
  reg [2:0] reset_counter;
  wire reset_int;
  
  generate
    if (RESET_LEVEL == "LOW_ACTIVE")
      assign reset_int = ~reset_i;
    else
      assign reset_int = reset_i;
  endgenerate


  // Reset signal hold-down counter
  always @(posedge inclk0) begin
    if (reset_int) begin
      reset_counter <= 3'b0;
      reset_clock <= 1'b1;
    end else begin
      if (reset_counter != 3'b111)
	  reset_counter <= (reset_counter + 1'b1) & 3'b111;
      else
	reset_clock <= 1'b0;
    end
  end

   // PLL Instance
   
	wire	  areset;
	wire [4:0] sub_wire0;
	wire  sub_wire6;
	wire [0:0] sub_wire9 = 1'h0;
	wire [4:4] sub_wire5 = sub_wire0[4:4];
	wire [3:3] sub_wire4 = sub_wire0[3:3];
	wire [2:2] sub_wire3 = sub_wire0[2:2];
	wire [1:1] sub_wire2 = sub_wire0[1:1];
	wire [0:0] sub_wire1 = sub_wire0[0:0];
	assign c0 = sub_wire1;
	assign c1 = sub_wire2;
	assign c2 = sub_wire3;
	assign c3 = sub_wire4;
	assign c4 = sub_wire5;
	assign  locked = sub_wire6;
	wire  sub_wire7 = inclk0;
	wire [1:0] sub_wire8 = {sub_wire9, sub_wire7};
	
	assign areset = reset_clock;

	altpll	altpll_component (
				.areset (areset),
				.inclk (sub_wire8),
				.clk (sub_wire0),
				.locked (sub_wire6),
				.activeclock (),
				.clkbad (),
				.clkena ({6{1'b1}}),
				.clkloss (),
				.clkswitch (1'b0),
				.configupdate (1'b0),
				.enable0 (),
				.enable1 (),
				.extclk (),
				.extclkena ({4{1'b1}}),
				.fbin (1'b1),
				.fbmimicbidir (),
				.fbout (),
				.fref (),
				.icdrclk (),
				.pfdena (1'b1),
				.phasecounterselect ({4{1'b1}}),
				.phasedone (),
				.phasestep (1'b1),
				.phaseupdown (1'b1),
				.pllena (1'b1),
				.scanaclr (1'b0),
				.scanclk (1'b0),
				.scanclkena (1'b1),
				.scandata (1'b0),
				.scandataout (),
				.scandone (),
				.scanread (1'b0),
				.scanwrite (1'b0),
				.sclkout0 (),
				.sclkout1 (),
				.vcooverrange (),
				.vcounderrange ());
	defparam
		altpll_component.bandwidth_type = "AUTO",
		altpll_component.clk0_divide_by = CLK0_DIVIDE_BY,
		altpll_component.clk0_duty_cycle = CLK0_DUTY_CYCLE,
		altpll_component.clk0_multiply_by = CLK0_MULTIPLY_BY,
		altpll_component.clk0_phase_shift = CLK0_PHASE_SHIFT,
		altpll_component.clk1_divide_by = CLK1_DIVIDE_BY,
		altpll_component.clk1_duty_cycle = CLK1_DUTY_CYCLE,
		altpll_component.clk1_multiply_by = CLK1_MULTIPLY_BY,
		altpll_component.clk1_phase_shift = CLK1_PHASE_SHIFT,
		altpll_component.clk2_divide_by = CLK2_DIVIDE_BY,
		altpll_component.clk2_duty_cycle = CLK2_DUTY_CYCLE,
		altpll_component.clk2_multiply_by = CLK2_MULTIPLY_BY,
		altpll_component.clk2_phase_shift = CLK2_PHASE_SHIFT,
		altpll_component.clk3_divide_by = CLK3_DIVIDE_BY,
		altpll_component.clk3_duty_cycle = CLK3_DUTY_CYCLE,
		altpll_component.clk3_multiply_by = CLK3_MULTIPLY_BY,
		altpll_component.clk3_phase_shift = CLK3_PHASE_SHIFT,
		altpll_component.clk4_divide_by = CLK4_DIVIDE_BY,
		altpll_component.clk4_duty_cycle = CLK4_DUTY_CYCLE,
		altpll_component.clk4_multiply_by = CLK4_MULTIPLY_BY,
		altpll_component.clk4_phase_shift = CLK4_PHASE_SHIFT,
		altpll_component.compensate_clock = "CLK0",
		altpll_component.inclk0_input_frequency = INCLK0_INPUT_FREQUENCY,
//		altpll_component.intended_device_family = "Cyclone IV E",
//		altpll_component.lpm_hint = "CBX_MODULE_PREFIX=pllfull",
		altpll_component.lpm_type = "altpll",
		altpll_component.operation_mode = "NORMAL",
		altpll_component.pll_type = "AUTO",
		altpll_component.port_activeclock = "PORT_UNUSED",
		altpll_component.port_areset = "PORT_USED",
		altpll_component.port_clkbad0 = "PORT_UNUSED",
		altpll_component.port_clkbad1 = "PORT_UNUSED",
		altpll_component.port_clkloss = "PORT_UNUSED",
		altpll_component.port_clkswitch = "PORT_UNUSED",
		altpll_component.port_configupdate = "PORT_UNUSED",
		altpll_component.port_fbin = "PORT_UNUSED",
		altpll_component.port_inclk0 = "PORT_USED",
		altpll_component.port_inclk1 = "PORT_UNUSED",
		altpll_component.port_locked = "PORT_USED",
		altpll_component.port_pfdena = "PORT_UNUSED",
		altpll_component.port_phasecounterselect = "PORT_UNUSED",
		altpll_component.port_phasedone = "PORT_UNUSED",
		altpll_component.port_phasestep = "PORT_UNUSED",
		altpll_component.port_phaseupdown = "PORT_UNUSED",
		altpll_component.port_pllena = "PORT_UNUSED",
		altpll_component.port_scanaclr = "PORT_UNUSED",
		altpll_component.port_scanclk = "PORT_UNUSED",
		altpll_component.port_scanclkena = "PORT_UNUSED",
		altpll_component.port_scandata = "PORT_UNUSED",
		altpll_component.port_scandataout = "PORT_UNUSED",
		altpll_component.port_scandone = "PORT_UNUSED",
		altpll_component.port_scanread = "PORT_UNUSED",
		altpll_component.port_scanwrite = "PORT_UNUSED",
		altpll_component.port_clk0 = "PORT_USED",
		altpll_component.port_clk1 = "PORT_USED",
		altpll_component.port_clk2 = "PORT_USED",
		altpll_component.port_clk3 = "PORT_USED",
		altpll_component.port_clk4 = "PORT_USED",
		altpll_component.port_clk5 = "PORT_UNUSED",
		altpll_component.port_clkena0 = "PORT_UNUSED",
		altpll_component.port_clkena1 = "PORT_UNUSED",
		altpll_component.port_clkena2 = "PORT_UNUSED",
		altpll_component.port_clkena3 = "PORT_UNUSED",
		altpll_component.port_clkena4 = "PORT_UNUSED",
		altpll_component.port_clkena5 = "PORT_UNUSED",
		altpll_component.port_extclk0 = "PORT_UNUSED",
		altpll_component.port_extclk1 = "PORT_UNUSED",
		altpll_component.port_extclk2 = "PORT_UNUSED",
		altpll_component.port_extclk3 = "PORT_UNUSED",
		altpll_component.self_reset_on_loss_lock = "OFF",
		altpll_component.width_clock = 5;

endmodule
