/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module clock_simple (
		     input wire  reset_i,
		     input wire  clk_in,
		     output wire clk_sdv,
		     output wire locked);
  parameter
    RESET_LEVEL = "LOW_ACTIVE",
    //SHIFT = 0.0 // currently unused
    MULTIPLY = 2,
    DIVIDE = 2,
    CLKIN_PERIOD = 37.037,
    CLKIN_FREQ_STR =	"156.250000", // needed for lattice ECP3
    CLKOUT_FREQ_FLT =	60.000000 , // needed for lattice ECP3
    CPHASE_STEPS =	0, // needed for Lattice ECP5
    FPHASE_STEPS =	0; // needed for Lattice ECP5

  reg reset_clock;
  reg [2:0] reset_counter;
  wire reset_int;
  

  generate
    if (RESET_LEVEL == "LOW_ACTIVE")
      assign reset_int = ~reset_i;
    else
      assign reset_int = reset_i;
  endgenerate



  // Reset signal hold-down counter
  always @(posedge clk_in) begin
    if (reset_int) begin
      reset_counter <= 3'b0;
      reset_clock <= 1'b1;
    end else begin
      if (reset_counter != 3'b111)
	  reset_counter <= (reset_counter + 1'b1) & 3'b111;
      else
	reset_clock <= 1'b0;
    end
  end


  // CLK Primitive Interface for different Vendors (Toolchains)
  clk_prim #(
	 .CLKFX_DIVIDE(		DIVIDE		),
	 .CLKFX_MULTIPLY(	MULTIPLY	),
	 //.SHIFT(		SHIFT		), // currently unused
     .CLKIN_PERIOD(		CLKIN_PERIOD	),
	 .CLKIN_FREQ_STR(	CLKIN_FREQ_STR	),  // only needed for lattice
	 .CLKOUT_FREQ_FLT(	CLKOUT_FREQ_FLT	)   // only needed for lattice
     )
  clk_prim_inst(
        .CLKIN(        clk_in       ),
        .RST(          reset_clock      ),
        .CLKFX(        clk_sdv      ),
        .LOCKED(       locked       )
    );

endmodule
