/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module ecp5_pll (
			input wire CLKI,
			input wire reset_i,
			output wire CLKOP,
			output wire CLKOS,
			output wire CLKOS2,
			output wire CLKOS3,
			output wire LOCK);
			
  parameter
    RESET_LEVEL = "LOW_ACTIVE",
    CLKI_DIV = 1 ,
    CLKFB_DIV = 1 ,
    CLKOP_DIV = 1 ,
    CLKOS_DIV = 1 ,
    CLKOS2_DIV = 1 ,
    CLKOS3_DIV = 1 ,
    CLKOS_CPHASE = CLKOS_DIV - 1 ,
    CLKOS_FPHASE = 0 ,
    CLKOS2_CPHASE = CLKOS2_DIV - 1 ,
    CLKOS2_FPHASE = 0 ,
    CLKOS3_CPHASE = CLKOS3_DIV - 1 ,
    CLKOS3_FPHASE = 0 ;

  reg reset_clock;
  reg [2:0] reset_counter;
  wire reset_int;
  wire RST;
  
  wire CLKOS3_t;
  wire CLKOS2_t;
  wire CLKOS_t;
  wire CLKOP_t;
  wire CLKFB_t;
  wire scuba_vhi;
  wire scuba_vlo;

  generate
    if (RESET_LEVEL == "LOW_ACTIVE")
      assign reset_int = ~reset_i;
    else
      assign reset_int = reset_i;
  endgenerate



  // Reset signal hold-down counter
  always @(posedge CLKI) begin
    if (reset_int) begin
      reset_counter <= 3'b0;
      reset_clock <= 1'b1;
    end else begin
      if (reset_counter != 3'b111)
	  reset_counter <= (reset_counter + 1'b1) & 3'b111;
      else
	reset_clock <= 1'b0;
    end
  end

   // PLL Instance
   
    VHI scuba_vhi_inst (.Z(scuba_vhi));

    VLO scuba_vlo_inst (.Z(scuba_vlo));

    defparam PLLInst_0.PLLRST_ENA = "ENABLED" ;
    defparam PLLInst_0.INTFB_WAKE = "DISABLED" ;
    defparam PLLInst_0.STDBY_ENABLE = "DISABLED" ;
    defparam PLLInst_0.DPHASE_SOURCE = "DISABLED" ;
    defparam PLLInst_0.CLKOS3_FPHASE = CLKOS3_FPHASE ;
    defparam PLLInst_0.CLKOS3_CPHASE = CLKOS3_CPHASE ;
    defparam PLLInst_0.CLKOS2_FPHASE = CLKOS2_FPHASE ;
    defparam PLLInst_0.CLKOS2_CPHASE = CLKOS2_CPHASE ;
    defparam PLLInst_0.CLKOS_FPHASE = CLKOS_FPHASE ;
    defparam PLLInst_0.CLKOS_CPHASE = CLKOS_CPHASE ;
    defparam PLLInst_0.CLKOP_FPHASE = 0 ;
    defparam PLLInst_0.CLKOP_CPHASE = CLKOP_DIV - 1 ;
    defparam PLLInst_0.PLL_LOCK_MODE = 0 ;
    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
    defparam PLLInst_0.CLKOS_TRIM_POL = "FALLING" ;
    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
    defparam PLLInst_0.CLKOP_TRIM_POL = "FALLING" ;
    defparam PLLInst_0.OUTDIVIDER_MUXD = "DIVD" ;
    defparam PLLInst_0.CLKOS3_ENABLE = "ENABLED" ;
    defparam PLLInst_0.OUTDIVIDER_MUXC = "DIVC" ;
    defparam PLLInst_0.CLKOS2_ENABLE = "ENABLED" ;
    defparam PLLInst_0.OUTDIVIDER_MUXB = "DIVB" ;
    defparam PLLInst_0.CLKOS_ENABLE = "ENABLED" ;
    defparam PLLInst_0.OUTDIVIDER_MUXA = "DIVA" ;
    defparam PLLInst_0.CLKOP_ENABLE = "ENABLED" ;
    defparam PLLInst_0.CLKOS3_DIV = CLKOS3_DIV ;
    defparam PLLInst_0.CLKOS2_DIV = CLKOS2_DIV ;
    defparam PLLInst_0.CLKOS_DIV = CLKOS_DIV ;
    defparam PLLInst_0.CLKOP_DIV = CLKOP_DIV ;
    defparam PLLInst_0.CLKFB_DIV = CLKFB_DIV ;
    defparam PLLInst_0.CLKI_DIV = CLKI_DIV ;
    defparam PLLInst_0.FEEDBK_PATH = "INT_OP" ;
    
    EHXPLLL PLLInst_0 ( .CLKI(CLKI), 
						.CLKFB(CLKFB_t), 
						.PHASESEL1(scuba_vlo), 
						.PHASESEL0(scuba_vlo), 
						.PHASEDIR(scuba_vlo), 
						.PHASESTEP(scuba_vlo), 
						.PHASELOADREG(scuba_vlo), 
						.STDBY(scuba_vlo), 
						.PLLWAKESYNC(scuba_vlo), 
						.RST(RST), 
						.ENCLKOP(scuba_vlo), 
						.ENCLKOS(scuba_vlo), 
						.ENCLKOS2(scuba_vlo), 
						.ENCLKOS3(scuba_vlo), 
						.CLKOP(CLKOP_t), 
						.CLKOS(CLKOS_t), 
						.CLKOS2(CLKOS2_t), 
						.CLKOS3(CLKOS3_t), 
						.LOCK(LOCK), 
						.INTLOCK(), 
						.REFCLK(), 
						.CLKINTFB(CLKFB_t))
						
						/* synthesis ICP_CURRENT="9" */
						/* synthesis LPF_RESISTOR="72" */;


    assign CLKOS3 = CLKOS3_t;
    assign CLKOS2 = CLKOS2_t;
    assign CLKOS = CLKOS_t;
    assign CLKOP = CLKOP_t;
    assign RST = reset_int;

endmodule
