/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Clock Mangement                                            ////
////  Hier werden die vier Clocksignale, die benoetig werden um  ////
////  den Prozessor zu betreiben, erzeugt. Dazu wird mit Hilfe   ////
////  eines DCMs das Eingangs clk Signal aufgearbeitet.          ////
////                                                             ////
////  Author: Falk Hassler                                       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none
`include	"core/src/spartanmc_def.vh"

module clock_user
#(
    parameter	CLKIN_DIVIDE_BY_2	=	"FALSE",
    parameter	CLKDV_DIVIDE10X		=	0,
    parameter	CLKFX_MULTIPLY		=	4,
    parameter	CLKFX_DIVIDE		=	1,
    parameter	CLKIN_PERIOD		=	20,
    parameter	CLKOUT_PHASE_SHIFT	=	"NONE",
    parameter	DESKEW_ADJUST		=	"SYSTEM_SYNCHRONOUS",
    parameter	DFS_FREQUENCY_MODE	=	"LOW",
    parameter	DLL_FREQUENCY_MODE	=	"LOW",
    parameter	DUTY_CYCLE_CORRECTION	=	"TRUE",
    parameter	PHASE_SHIFT		=	0,
    parameter	STARTUP_WAIT		=	"FALSE",
    parameter	CLKDV_DIVIDE		=	CLKDV_DIVIDE10X / 10,
    parameter   RESET_LEVEL             =       "LOW_ACTIVE"
)
(
	input wire reset_i,
	input wire clk_in,
	output wire clk_0,
	output wire clk_90,
	output wire clk_180,
	output wire clk_270,
	output wire clkdv,
	output wire clk2x,
	output wire clk2x180,
	output wire clkfx,
	output wire clkfx180,
    output wire locked
		);

reg [2:0]   reset_counter;
reg         reset_dcm;
wire        reset_int;
  
  generate
    if (RESET_LEVEL == "LOW_ACTIVE")
      assign reset_int = ~reset_i;
    else
      assign reset_int = reset_i;
  endgenerate

always @(posedge clk_in)
begin
    if (reset_int)
    begin
        reset_counter <= 3'b0;
        reset_dcm     <= 1'b1;
    end else begin
        if (reset_counter != 3'b111)
        begin
            reset_counter <= (reset_counter + 1) % 4'b1000;
        end else begin
            reset_dcm <= 1'b0;
        end
    end
end

DCM #	(
		.CLK_FEEDBACK(		"1X"	            	),
		.CLKDV_DIVIDE(		CLKDV_DIVIDE		),
		.CLKFX_DIVIDE(		CLKFX_DIVIDE		),
		.CLKFX_MULTIPLY(	CLKFX_MULTIPLY		),
		.CLKIN_DIVIDE_BY_2(	CLKIN_DIVIDE_BY_2	),
		.CLKIN_PERIOD(		CLKIN_PERIOD		),
		.CLKOUT_PHASE_SHIFT(	CLKOUT_PHASE_SHIFT	),
		.DESKEW_ADJUST(		DESKEW_ADJUST		),
		.DFS_FREQUENCY_MODE(	DFS_FREQUENCY_MODE	),
		.DLL_FREQUENCY_MODE(	DLL_FREQUENCY_MODE	),
		.DUTY_CYCLE_CORRECTION(	DUTY_CYCLE_CORRECTION	),
		.PHASE_SHIFT(		PHASE_SHIFT		),
        
		.STARTUP_WAIT(		STARTUP_WAIT		)
		)
dcm_inst_user	(
		.CLKIN(			clk_in			),
		.CLKFB(			clk_0        		),
		.RST(			reset_dcm		),
		.PSEN(			1'b0			),
		.PSINCDEC(		1'b0			),
		.PSCLK(			1'b0			),
		.DSSEN(			1'b0			),
		.CLK0(			clk_0    		),
		.CLK90(			clk_90			),
		.CLK180(		clk_180		),
		.CLK270(		clk_270			),
		.CLKDV(			clkdv			),
		.CLK2X(			clk2x		),
		.CLK2X180(		clk2x180		),
		.CLKFX(			clkfx			),
		.CLKFX180(		clkfx180		),
		.STATUS(					),
        
		.LOCKED(		locked		),
		.PSDONE(					)
		);


endmodule
