/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription

//connect reset
document.getDefaultPin("reset").ifPresent { resetBtn ->
	def resetPort = module.ports['reset']
	connectIfConnectable(resetPort, resetBtn)

	def level = resetBtn.getConnectorDescription().getActiveType()

	if (resetBtn.inverted)
		level = level.opposite

	if (level==TargetDescription.ActiveType.High) {
		module.parameters.RESET_LEVEL.storedValue = "HIGH_ACTIVE"
	} else if (level==TargetDescription.ActiveType.Low) {
		module.parameters.RESET_LEVEL.storedValue = "LOW_ACTIVE"
	} //else we don't know anything and leave the default parameter value
}

//connect clk
document.getDefaultPin("clock").ifPresent { pin ->
	def clkPort = module.ports['clk_gen']
	connectIfConnectable(clkPort, pin)

	//Set the frequency
	document.getTarget().getTargetDescription().getConfigurationHint("targetFrequency").ifPresent { targetFreq ->
		setfxFreq(pin.frequency.get(), targetFreq)
	}
}

//connect to all spartan cores that don't have any connection
def clock_locked = module.ports["locked"]
def clock_clk = module.ports["clkfx"]
document.subsystems.each {
	def spartanmc_locked = it.getPort("locked")
	def spartanmc_clk = it.getPort("clk")
	connectIfConnectable(clock_locked, spartanmc_locked)
	connectIfConnectable(clock_clk, spartanmc_clk)
}

def setfxFreq(freq, targetFreqInMHz){
	def clk = module

	def targetFreq = Double.parseDouble(targetFreqInMHz) * 1000 * 1000

	def clkfxDiv = clk.parameters.CLKFX_DIVIDE
	def clkfxMul = clk.parameters.CLKFX_MULTIPLY

	def result = [clkfxDiv.choices, clkfxMul.choices].combinations().collect {
		[freq * (it[1].toFloat()) / (it[0].toFloat()), it[0], it[1]]
	}.sort { it[0] }.reverse().findResult { it[0] <= targetFreq ? it : null }

	clkfxDiv.setStoredValue(result[1])
	clkfxMul.setStoredValue(result[2])

	println "Chose clk divider ${result[1]}, clk multiplier ${result[2]} with resulting frequency ${String.format("%.1f MHz", result[0])}"

	return true
}