/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Clock Mangement                                            ////
////  Hier werden die vier Clocksignale, die benoetig werden um  ////
////  den Prozessor zu betreiben, erzeugt. Dazu wird mit Hilfe   ////
////  eines DCMs das Eingangs clk Signal aufgearbeitet.          ////
////                                                             ////
////  Author: Falk Hassler                                       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`default_nettype none
`include    "core/src/spartanmc_def.vh"

module clock    (
	reset,
	clk_gen,
	clk1,
	clk2,
	phase,
	clk2x,
	locked,
	clk270,
	clkdv,
	clk2x180,
	clkfx,
	clkfx180
	);

  parameter 
      RESET_LEVEL             = "LOW_ACTIVE",
      CLKIN_DIVIDE_BY_2       = "FALSE",
      CLKFX_MULTIPLY          = 4,
      CLKFX_DIVIDE            = 1,
      //CLKIN_PERIOD            = 20,
      DFS_FREQUENCY_MODE      = "LOW",
      CLKDV_DIVIDE            = "2.0",
      INSTANTIATE_BUFGS       = "YES",
      INSTANTIATE_BUFGS2      = "YES",
      INSTANTIATE_BUFGFX      = "YES",
      FEEDBACK                = "CLK1";

  input     reset;
  input     clk_gen;	// vom Quarz

  output	clk1;
  output	clk2;
  output	phase;
  output	clk2x;
  output	locked;
  output	clk270;
  output	clkdv;
  output	clk2x180;
  output	clkfx;
  output	clkfx180;

  wire      reset;
  wire      clk_gen;	// vom Quarz

  wire      clk;
  wire      clk1;
  wire      clk2;
  wire      phase;
  wire      clk2x;
  wire      locked;
  wire      clk270;
  wire      clkdv;
  wire      clk2x180;
  wire      clkfx;
  wire      clkfx180;

  wire      clk1_dcm;
  wire      clk2_dcm;
  wire      clk2x_dcm;
  wire      clkfb_in;
  wire      clkfx_dcm;

  reg       clk2r;
  wire      ffclk;

  wire reset_int;

  generate
    if (RESET_LEVEL == "LOW_ACTIVE")
      assign reset_int = ~reset;
    else
      assign reset_int = reset;
  endgenerate

  generate 
    if (INSTANTIATE_BUFGS == "YES")
    begin
      BUFG BUFG_clk_inst(
	     	.O (clk),
	     	.I (clk_gen)
		);

      BUFG BUFG_clk1_inst(
	     	.O (clk1),
	     	.I (clk1_dcm)
		);
    end else begin

	assign	clk	= clk_gen;
	assign	clk1	= clk1_dcm;

    end
  endgenerate

  //generate
  //  if (FEEDBACK == "CLK1")
      assign clkfb_in = clk1;
  //  else if (FEEDBACK == "CLK2X")
  //    assign clkfb_in = clk2x;
    //else assign clkfb_in = 0;
  //endgenerate

  generate 
    if (INSTANTIATE_BUFGS2 == "YES")
    begin
      BUFG BUFG_clk2_inst(
	     	.O (clk2),
	     	.I (clk2_dcm)
		);

      BUFG BUFG_clk2x_inst(
		.O (clk2x),
		.I (clk2x_dcm)
		);
    end else begin

	assign	clk2	= clk2_dcm;
	assign	clk2x	= clk2x_dcm;

    end
  endgenerate

  generate 
    if (INSTANTIATE_BUFGFX == "YES")
    begin
      BUFG BUFG_clkfx_inst(
	     	.O (clkfx),
	     	.I (clkfx_dcm)
		);
    end else begin
	assign	clkfx	= clkfx_dcm;
    end
  endgenerate


  reg	[2:0]	reset_counter;
  reg		reset_dcm;

  always @(posedge clk_gen)
    begin
      if (reset_int)
    begin
          reset_counter <= 3'b0;
          reset_dcm     <= 1'b1;
    end else begin
          if (reset_counter != 3'b111)
            begin
		reset_counter <= (reset_counter + 1) % 4'b1000;
            end else begin
		reset_dcm <= 1'b0;
            end
    end
    end
    
  localparam FB = /*FEEDBACK=="CLK1"?*/"1X"/*:FEEDBACK=="CLK2"?"2X":"NONE"*/;

  DCM # (
	.CLK_FEEDBACK(		FB			),
	.CLKDV_DIVIDE(		CLKDV_DIVIDE		),
	.CLKFX_DIVIDE(		CLKFX_DIVIDE		),
	.CLKFX_MULTIPLY(	CLKFX_MULTIPLY		),
	.CLKIN_DIVIDE_BY_2(	CLKIN_DIVIDE_BY_2	),
	//.CLKIN_PERIOD(		CLKIN_PERIOD		),
	.CLKOUT_PHASE_SHIFT(	"NONE"			),
	.DESKEW_ADJUST(		"SYSTEM_SYNCHRONOUS"	),
	.DFS_FREQUENCY_MODE(	DFS_FREQUENCY_MODE	),
	.DLL_FREQUENCY_MODE(	"LOW"			),
	.DUTY_CYCLE_CORRECTION(	"TRUE"			),
	.PHASE_SHIFT(		0			),
	.STARTUP_WAIT(		"FALSE"			)
     )
  dcm_inst  (
    	.CLKIN(			clk		),
    	.CLKFB(			clkfb_in	),
    	.RST(			reset_dcm	),
    	.PSEN(			1'b0		),
    	.PSINCDEC(		1'b0		),
    	.PSCLK(			1'b0		),
    	.DSSEN(			1'b0		),
    	.CLK0(			clk1_dcm	),
    	.CLK90(			phase		),
    	.CLK180(		clk2_dcm	),
    	.CLK270(		clk270		),
    	.CLKDV(			clkdv		),
    	.CLK2X(			clk2x_dcm	),
    	.CLK2X180(		clk2x180	),
    	.CLKFX(			clkfx_dcm	),
    	.CLKFX180(		clkfx180	),
    	.STATUS(				),
    	.LOCKED(		locked		),
    	.PSDONE(				)
	 );

endmodule
