/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`include    "core/src/spartanmc_def.vh"

module clk_xilinx_mmcm    (
	input wire reset,
	input wire clk_gen,
	
	output wire clk0,
	output wire clk0b,
	output wire clk1,
	output wire clk1b,
	output wire clk2,
	output wire clk2b,
	output wire clk3,
	output wire clk3b,
	output wire clk4,
	output wire clk5,
	output wire clk6,
	output wire locked
	);

	parameter RESET_LEVEL             = "LOW_ACTIVE";
	parameter BANDWIDTH = "OPTIMIZED";   // Jitter programming (OPTIMIZED, HIGH, LOW)
	parameter CLKFBOUT_MULT_F = 5.0;     // Multiply value for all CLKOUT (2.000-64.000).
	parameter CLKFBOUT_PHASE = 0.0;      // Phase offset in degrees of CLKFB (-360.000-360.000).
	parameter CLKIN1_PERIOD = 0.0;       // Input clock period in ns to ps resolution (i.e. 33.333 is 30 MHz).
	parameter CLOCK_DESKEW = "FALSE";     // Insert BUFG in feedback path to deskew the clock
	// CLKOUT0_DIVIDE - CLKOUT6_DIVIDE: Divide amount for each CLKOUT (1-128)
	parameter CLKOUT1_DIVIDE = 1;
	parameter CLKOUT2_DIVIDE = 1;
	parameter CLKOUT3_DIVIDE = 1;
	parameter CLKOUT4_DIVIDE = 1;
	parameter CLKOUT5_DIVIDE = 1;
	parameter CLKOUT6_DIVIDE = 1;
	parameter CLKOUT0_DIVIDE_F = 1.0;    // Divide amount for CLKOUT0 (1.000-128.000).
	// CLKOUT0_DUTY_CYCLE - CLKOUT6_DUTY_CYCLE: Duty cycle for each CLKOUT (0.01-0.99).
	parameter CLKOUT0_DUTY_CYCLE = 0.5;
	parameter CLKOUT1_DUTY_CYCLE = 0.5;
	parameter CLKOUT2_DUTY_CYCLE = 0.5;
	parameter CLKOUT3_DUTY_CYCLE = 0.5;
	parameter CLKOUT4_DUTY_CYCLE = 0.5;
	parameter CLKOUT5_DUTY_CYCLE = 0.5;
	parameter CLKOUT6_DUTY_CYCLE = 0.5;
	// CLKOUT0_PHASE - CLKOUT6_PHASE: Phase offset for each CLKOUT (-360.000-360.000).
	parameter CLKOUT0_PHASE = 0.0;
	parameter CLKOUT1_PHASE = 0.0;
	parameter CLKOUT2_PHASE = 0.0;
	parameter CLKOUT3_PHASE = 0.0;
	parameter CLKOUT4_PHASE = 0.0;
	parameter CLKOUT5_PHASE = 0.0;
	parameter CLKOUT6_PHASE = 0.0;
	parameter CLKOUT4_CASCADE = "FALSE"; // Cascade CLKOUT4 counter with CLKOUT6 (FALSE, TRUE)
	parameter DIVCLK_DIVIDE = 1;         // Master division value (1-106)
	parameter REF_JITTER1 = 0.0;         // Reference input jitter in UI (0.000-0.999).
	parameter STARTUP_WAIT = "FALSE";     // Delays DONE until MMCM is locked (FALSE, TRUE)


	parameter CLKOUT0_BUFG = "TRUE";
	parameter CLKOUT1_BUFG = "TRUE";
	parameter CLKOUT2_BUFG = "TRUE";
	parameter CLKOUT3_BUFG = "TRUE";
	parameter CLKOUT4_BUFG = "TRUE";
	parameter CLKOUT5_BUFG = "TRUE";
	parameter CLKOUT6_BUFG = "TRUE";


  wire reset_int;

  generate
    if (RESET_LEVEL == "LOW_ACTIVE")
      assign reset_int = ~reset;
    else
      assign reset_int = reset;
  endgenerate


  reg	[2:0]	reset_counter;
  reg		reset_dcm;

  always @(posedge clk_gen)
    begin
      if (reset_int)
    begin
          reset_counter <= 3'b0;
          reset_dcm     <= 1'b1;
    end else begin
          if (reset_counter != 3'b111)
            begin
		reset_counter <= (reset_counter +  1'b1);
            end else begin
		reset_dcm <= 1'b0;
            end
    end
    end

wire clk0Int, clk0bInt, clk1Int, clk1bInt, clk2Int, clk2bInt, clk3Int, clk3bInt, clk4Int, clk5Int, clk6Int;

generate

if (CLKOUT0_BUFG=="TRUE") begin
	BUFG bufg0 (
		.I(clk0Int),
		.O(clk0)
	);
	BUFG bufg0b (
		.I(clk0bInt),
		.O(clk0b)
	);
end else begin

	assign clk0 = clk0Int;
	assign clk0b = clk0bInt;

end

if (CLKOUT1_BUFG=="TRUE") begin
	BUFG bufg1 (
		.I(clk1Int),
		.O(clk1)
	);
	BUFG bufg1b (
		.I(clk1bInt),
		.O(clk1b)
	);
end else begin

	assign clk1 = clk1Int;
	assign clk1b = clk1bInt;

end


if (CLKOUT2_BUFG=="TRUE") begin
	BUFG bufg2 (
		.I(clk2Int),
		.O(clk2)
	);
	BUFG bufg2b (
		.I(clk2bInt),
		.O(clk2b)
	);
end else begin

	assign clk2 = clk2Int;
	assign clk2b = clk2bInt;

end


if (CLKOUT3_BUFG=="TRUE") begin
	BUFG bufg3 (
		.I(clk3Int),
		.O(clk3)
	);
	BUFG bufg3b (
		.I(clk3bInt),
		.O(clk3b)
	);
end else begin

	assign clk3 = clk3Int;
	assign clk3b = clk3bInt;

end


if (CLKOUT4_BUFG=="TRUE") begin
	BUFG bufg4 (
		.I(clk4Int),
		.O(clk4)
	);
end else begin

	assign clk4 = clk4Int;

end

if (CLKOUT5_BUFG=="TRUE") begin
	BUFG bufg5 (
		.I(clk5Int),
		.O(clk5)
	);
end else begin

	assign clk5 = clk5Int;

end

if (CLKOUT6_BUFG=="TRUE") begin
	BUFG bufg6 (
		.I(clk6Int),
		.O(clk6)
	);
end else begin

	assign clk6 = clk6Int;

end


endgenerate

wire clkfbout;
wire clkfbin;

generate
if(CLOCK_DESKEW=="TRUE") begin
	BUFG bufg7 (
		.I(clkfbout),
		.O(clkfbin)
	);
end else begin
	assign clkfbin = clkfbout;
end
endgenerate

    

   // MMCME2_BASE: Base Mixed Mode Clock Manager
   //              Artix-7
   // Xilinx HDL Language Template, version 14.7

   MMCME2_BASE #(
      .BANDWIDTH(BANDWIDTH),   // Jitter programming (OPTIMIZED, HIGH, LOW)
      .CLKFBOUT_MULT_F(CLKFBOUT_MULT_F),     // Multiply value for all CLKOUT (2.000-64.000).
      .CLKFBOUT_PHASE(CLKFBOUT_PHASE),      // Phase offset in degrees of CLKFB (-360.000-360.000).
      .CLKIN1_PERIOD(CLKIN1_PERIOD),       // Input clock period in ns to ps resolution (i.e. 33.333 is 30 MHz).
      // CLKOUT0_DIVIDE - CLKOUT6_DIVIDE: Divide amount for each CLKOUT (1-128)
      .CLKOUT1_DIVIDE(CLKOUT1_DIVIDE),
      .CLKOUT2_DIVIDE(CLKOUT2_DIVIDE),
      .CLKOUT3_DIVIDE(CLKOUT3_DIVIDE),
      .CLKOUT4_DIVIDE(CLKOUT4_DIVIDE),
      .CLKOUT5_DIVIDE(CLKOUT5_DIVIDE),
      .CLKOUT6_DIVIDE(CLKOUT6_DIVIDE),
      .CLKOUT0_DIVIDE_F(CLKOUT0_DIVIDE_F),    // Divide amount for CLKOUT0 (1.000-128.000).
      // CLKOUT0_DUTY_CYCLE - CLKOUT6_DUTY_CYCLE: Duty cycle for each CLKOUT (0.01-0.99).
      .CLKOUT0_DUTY_CYCLE(CLKOUT0_DUTY_CYCLE),
      .CLKOUT1_DUTY_CYCLE(CLKOUT1_DUTY_CYCLE),
      .CLKOUT2_DUTY_CYCLE(CLKOUT2_DUTY_CYCLE),
      .CLKOUT3_DUTY_CYCLE(CLKOUT3_DUTY_CYCLE),
      .CLKOUT4_DUTY_CYCLE(CLKOUT4_DUTY_CYCLE),
      .CLKOUT5_DUTY_CYCLE(CLKOUT5_DUTY_CYCLE),
      .CLKOUT6_DUTY_CYCLE(CLKOUT6_DUTY_CYCLE),
      // CLKOUT0_PHASE - CLKOUT6_PHASE: Phase offset for each CLKOUT (-360.000-360.000).
      .CLKOUT0_PHASE(CLKOUT0_PHASE),
      .CLKOUT1_PHASE(CLKOUT1_PHASE),
      .CLKOUT2_PHASE(CLKOUT2_PHASE),
      .CLKOUT3_PHASE(CLKOUT3_PHASE),
      .CLKOUT4_PHASE(CLKOUT4_PHASE),
      .CLKOUT5_PHASE(CLKOUT5_PHASE),
      .CLKOUT6_PHASE(CLKOUT6_PHASE),
      .CLKOUT4_CASCADE(CLKOUT4_CASCADE), // Cascade CLKOUT4 counter with CLKOUT6 (FALSE, TRUE)
      .DIVCLK_DIVIDE(DIVCLK_DIVIDE),         // Master division value (1-106)
      .REF_JITTER1(REF_JITTER1),         // Reference input jitter in UI (0.000-0.999).
      .STARTUP_WAIT(STARTUP_WAIT)     // Delays DONE until MMCM is locked (FALSE, TRUE)
   )
   MMCME2_BASE_inst (
      // Clock Outputs: 1-bit (each) output: User configurable clock outputs
      .CLKOUT0(clk0Int),     // 1-bit output: CLKOUT0
      .CLKOUT0B(clk0bInt),   // 1-bit output: Inverted CLKOUT0
      .CLKOUT1(clk1Int),     // 1-bit output: CLKOUT1
      .CLKOUT1B(clk1bInt),   // 1-bit output: Inverted CLKOUT1
      .CLKOUT2(clk2Int),     // 1-bit output: CLKOUT2
      .CLKOUT2B(clk2bInt),   // 1-bit output: Inverted CLKOUT2
      .CLKOUT3(clk3Int),     // 1-bit output: CLKOUT3
      .CLKOUT3B(clk3bInt),   // 1-bit output: Inverted CLKOUT3
      .CLKOUT4(clk4Int),     // 1-bit output: CLKOUT4
      .CLKOUT5(clk5Int),     // 1-bit output: CLKOUT5
      .CLKOUT6(clk6Int),     // 1-bit output: CLKOUT6
      // Feedback Clocks: 1-bit (each) output: Clock feedback ports
      .CLKFBOUT(clkfbout),   // 1-bit output: Feedback clock
      .CLKFBOUTB(), // 1-bit output: Inverted CLKFBOUT
      // Status Ports: 1-bit (each) output: MMCM status ports
      .LOCKED(locked),       // 1-bit output: LOCK
      // Clock Inputs: 1-bit (each) input: Clock input
      .CLKIN1(clk_gen),       // 1-bit input: Clock
      // Control Ports: 1-bit (each) input: MMCM control ports
      .PWRDWN(1'b0),       // 1-bit input: Power-down
      .RST(reset_dcm),             // 1-bit input: Reset
      // Feedback Clocks: 1-bit (each) input: Clock feedback ports
      .CLKFBIN(clkfbin)      // 1-bit input: Feedback clock
   );
endmodule
