/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


//////////////////////////////////////////////////////////////////////////////////
//Modul:			dual-ported_RAM (DMA memory)
//Autor:
//Version:		20.01.2017
//Description:	Code for asymmectric inference BRAM taken from
//					Xilinx XST User Guide (UG687) page 251
//////////////////////////////////////////////////////////////////////////////////

`timescale	1ns / 10ps
`default_nettype none


module dual_ported_RAM
#(
	parameter WIDTH      = 16,
	parameter SIZE       = 128,
	parameter ADDRWIDTH  = 10
)
(
	input wire					clkA,
	input wire					clkB,
	input wire					enA,
	input wire					enB,
	input wire					weA,
	input wire					weB,
	input wire					rstA,
	input wire	[ADDRWIDTH-1:0]	addrA,
	input wire	[ADDRWIDTH-1:0]	addrB,
	input wire	[WIDTH-1:0]		diA,
	input wire	[WIDTH-1:0]		diB,
	output reg	[WIDTH-1:0]		doA,
	output reg  [WIDTH-1:0]		doB
);
	reg     [WIDTH-1:0]  RAM [0:SIZE-1];
	
	//Port A description
	always @(posedge clkA) begin
		if(enA) begin
			if(weA) begin
				RAM[addrA] <= diA;
		   if(!rstA)
				doA <= diA;
			else
				doA <= 0;
			end else if(!rstA)
				doA <= RAM[addrA];
			else
				doA <= 0;
		end
	end
	
	//Port B description	
	always @(posedge clkB) begin
		if(enB) begin
			if(weB) begin
				RAM[addrB] <= diB;
				doB <= diB;
			end else begin
				doB <= RAM[addrB];
			end
		end
	end
	
endmodule
