/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none

module one_hot_to_bin

#(
parameter ONE_HOT_WIDTH = 'd10,
parameter BIN_WIDTH = 'd9
)
(
input wire reset,
input wire clk,
input wire [ONE_HOT_WIDTH-1:0] one_hot,
output reg [BIN_WIDTH-1:0] binary_encoded

);

wire[BIN_WIDTH-1:0] bin;

genvar k,i;
generate
	for (i=0; i<BIN_WIDTH; i=i+1)
	begin : binloop
		wire [ONE_HOT_WIDTH-1:0] mask;
		for (k=0; k<ONE_HOT_WIDTH; k=k+1)
		begin : maskloop
			assign mask[k] = k[i];
		end	
		assign bin[i] = | (one_hot & mask);
	end
endgenerate



always @ (one_hot or reset) begin

	case (reset)
		1'b0 : binary_encoded <= bin;/*case(one_hot)
			20'd1 : binary_encoded <= 9'd0;
			20'd2 : binary_encoded <= 9'd1;
			20'd4 : binary_encoded <= 9'd2;
			20'd8 : binary_encoded <= 9'd3;
			20'd16 : binary_encoded <= 9'd4;
			20'd32 : binary_encoded <= 9'd5;
			20'd64 : binary_encoded <= 9'd6;
			20'd128 : binary_encoded <= 9'd7;
			20'd256 : binary_encoded <= 9'd8;
			20'd512 : binary_encoded <= 9'd9;
			20'd1024 : binary_encoded <= 9'd10;
			20'd2048 : binary_encoded <= 9'd11;
			20'd4096 : binary_encoded <= 9'd12;
			20'd8192 : binary_encoded <= 9'd13;
			20'd16384 : binary_encoded <= 9'd14;
			20'd32768 : binary_encoded <= 9'd15;
			20'd65536 : binary_encoded <= 9'd16;
			20'd131072 : binary_encoded <= 9'd17;
			20'd262144 : binary_encoded <= 9'd18;
			20'd524288 : binary_encoded <= 9'd19;
			default : binary_encoded <= 9'd0;
		endcase*/
		default : binary_encoded <= 9'd0;

		/*if (one_hot == 1) begin
			binary_encoded <= 9'd0;
			
		end else if (one_hot == 2) begin
			binary_encoded <= 9'd1;
			
		end else if (one_hot == 4) begin
			binary_encoded <= 9'd2;
			
		end else if (one_hot == 8) begin
			binary_encoded <= 9'd3;
			
		end else if (one_hot == 16) begin
			binary_encoded <= 9'd4;
			
		end else if (one_hot == 32) begin
			binary_encoded <= 9'd5;
			
		end else if (one_hot == 64) begin
			binary_encoded <= 9'd6;
			
		end else if (one_hot == 128) begin
			binary_encoded <= 9'd7;
			
		end else if (one_hot == 256) begin
			binary_encoded <= 9'd8;
			
		end else if (one_hot == 512) begin
			binary_encoded <= 9'd9;
			
		end else if (one_hot == 1024) begin
			binary_encoded <= 9'd10;
			
		end else if (one_hot == 2048) begin
			binary_encoded <= 9'd11;
			
		end else if (one_hot == 4096) begin
			binary_encoded <= 9'd12;
			
		end else if (one_hot == 8192) begin
			binary_encoded <= 9'd13;
			
		end else if (one_hot == 16384) begin
			binary_encoded <= 9'd14;
			
		end else if (one_hot == 32768) begin
			binary_encoded <= 9'd15;
			
		end else if (one_hot == 65536) begin
			binary_encoded <= 9'd16;
			
		end else if (one_hot == 131072) begin
			binary_encoded <= 9'd17;
			
		end else if (one_hot == 262144) begin
			binary_encoded <= 9'd18;
			
		end else if (one_hot == 524288) begin
			binary_encoded <= 9'd19;
		end*/
	endcase
end


endmodule

