/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Modul zur Adresserkennung von E/A Modulen des SpartanMC    ////
////                                                             ////
////  Hier wird fuer eine vorgegebene Anzahl von Bits ein        ////
////  Vergleich mit einer Konstanten realisiert, der die Basis-  ////
////  adresse eines E/A Moduls sein soll. Dabei werden die fuer  ////
////  den Modul konstanten Adressbits (<= 10 von addr_peri) mit  ////
////  der Konstanten verglichen. Der Ausgang select wird dann 1, ////
////  wenn die Adressbits uebereinstimmen, und auch das Signal   ////
////  access_peri = activ_peri gleich 1 ist.                     ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`timescale	1ns / 10ps
`default_nettype none

`ifdef RAPIDSOC_MODULE
(* keep_hierarchy="yes" *)
`endif
module pselect #(
	parameter					ADDR_WIDTH = 10,
	parameter					BASE_WIDTH = 10,
	parameter [BASE_WIDTH-1:0]	BASE_ADDR  = 0
	)
	(
	input wire	[ADDR_WIDTH-1:0]	addr,
	input wire						activ_peri,
	output wire						select
	);

assign select = activ_peri && addr==BASE_ADDR;

endmodule

