/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


//////////////////////////////////////////////////////////////////////////////////
// Engineer:       Tobias Engelmann
// Create Date:    04/14/2010 
// Design Name:    SpartanMC generic modules
// Module Name:    spmc_fifo_mem 
//
// Description:    Generic dual ported memory, intended to use with the async fifo module
//
// Dependencies:    spmc_reset.v	Global SpartanMC reset behavior
//////////////////////////////////////////////////////////////////////////////////

`default_nettype none

module spmc_fifo_mem
#(
	parameter	RAM_ADDR_BITS		= 'd4,
	parameter	RAM_WIDTH		= 'd18
)(
	input 	wire				reset,
	input 	wire				clkA,
	input	wire	[RAM_ADDR_BITS-1:0]	addrA,
	input	wire				enableA,
	input	wire				write_enableA,
	input 	wire 	[RAM_WIDTH-1:0]		input_dataA,
	
	input wire				clkB,
	input	wire	[RAM_ADDR_BITS-1:0]	addrB,
	input	wire				enableB,
	output	reg	[RAM_WIDTH-1:0]		output_dataB
);

`ifdef toolchain_quartus
(*ramstyle="auto"*)
`else
(*ram_style="auto"*)
`endif
reg [RAM_WIDTH-1:0] ram_mem [(2**RAM_ADDR_BITS)-1:0];

always @(posedge clkA) 
      if (enableA)
         if (write_enableA)
            ram_mem[addrA] <= input_dataA;

always @(posedge clkB)
	if (reset)
		output_dataB <= 'h0;
	else
		if (enableB) 
			output_dataB <= ram_mem[addrB];

endmodule
