/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


//////////////////////////////////////////////////////////////////////////////////
// Company: 	   TU Dresden
// Engineer:       Tobias Engelmann
// Create Date:    18:44:01 04/14/2010 
// Last Update:	   21.07.2011
// Module Name:    usb2_graycounter 
//////////////////////////////////////////////////////////////////////////////////

`default_nettype none

module spmc_graycounter   #(
	parameter   COUNTER_WIDTH 		= 'd4
)
(
	input	wire				clk,
	input	wire				reset,
	
	input 	wire                         	enable,  	//Count enable.
	input	wire				clear,
	output 	reg  	[COUNTER_WIDTH-1:0]	gray_value, 
	output 	reg  	[COUNTER_WIDTH-1:0]	bin_value  
);

// Usage of asynchronous resets may negatively impact FPGA resources
// and timing. In general faster and smaller FPGA designs will
// result from not using asynchronous resets. Please refer to
// the Synthesis and Simulation Design Guide for more information.

always @(posedge clk)
	if (reset)
	begin
		bin_value <= {{(COUNTER_WIDTH-1){1'b0}}, 1'b1};
		gray_value <= {COUNTER_WIDTH{1'b0}};
	end
	else 
	begin
		if (enable)
		begin
			bin_value <= bin_value + 1;
			gray_value <= (bin_value >> 1) ^ bin_value;
		end
		else if (clear)
		begin
			bin_value <= {{COUNTER_WIDTH{1'b0}}, 1'b1};
			gray_value <= {COUNTER_WIDTH{1'b0}};
		end
	end
endmodule
