/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

`timescale 1ns / 1ps

module xilinx_fifo_with_fill_counter
#(
	parameter	ADDR_BITS		    = 'd10,
	parameter	WIDTH			    = 'd18,
    parameter   DEPTH               = 'd64
)(
	input	wire					reset,
    output  reg    [ADDR_BITS:0]	fill_count,
    input   wire                   	clk,

	input	wire					write_enable,
	input	wire	[WIDTH-1:0]	    write_data,
	
	input	wire					read_enable,
	output	wire	[WIDTH-1:0]	    read_data
);
reg fifo_empty_p1;

always @(posedge clk)
begin
    if (reset)
    begin
        fill_count <= 1'b0;
        fifo_empty_p1 <= 1'b0;
	end else
	begin
		if (write_enable && fill_count == 0)
		begin
			if(fifo_empty_p1)
				fill_count <= 2;
			else
				fill_count <= 0;
		end else if(write_enable && ~read_enable && (fill_count<DEPTH))
		begin
			fill_count <= fill_count+1'b1;
		end else if (~write_enable && read_enable && (fill_count!=0))
		begin
			fill_count <= fill_count-1'b1;
		end else begin
			fill_count <= fill_count + fifo_empty_p1;
		end

		if(fill_count == 0 && write_enable && !fifo_empty_p1) begin
        	fifo_empty_p1 <= 1'b1;
        end else begin
            fifo_empty_p1 <= 1'b0;
        end
	end
end

xlx_fifo #
(
 .DSIZE(WIDTH),
 .ASIZE(ADDR_BITS)
)
fifo0(
.wr_clk(clk),
.rst(reset),
.wr_en(write_enable),
.wr_data(write_data),
.rd_en(read_enable),
.rd_clk(clk), 
.rd_data(read_data),
.full(),
.empty(),
.almost_full()
);
endmodule
