/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    10:09:28 12/16/2011 
// Module Name:    fifo_mem 
//////////////////////////////////////////////////////////////////////////////////
`default_nettype none
module yaf_mem 
        #(
	  parameter DSIZE = 36, // Memory data word width
	  parameter ASIZE = 10	// Number of mem address bits
          ) 
    (
     output wire [DSIZE-1:0] rdata,
     input wire [DSIZE-1:0]  wdata,
     input wire [ASIZE-1:0]  waddr, 
     input wire [ASIZE-1:0]  raddr,
     input wire              wclken, 
     input wire              wfull, 
     input wire              wclk
     );

    // RTL Verilog memory model
    localparam DEPTH = 1<<ASIZE;
    `ifdef toolchain_quartus
    (*ramstyle="auto"*)
    `else
    (*ram_style="auto"*)
    `endif
    reg [DSIZE-1:0] mem [0:DEPTH-1];
    
    /* synopsys translate_off */
    integer i;
    initial
        for (i = 0; i < DEPTH; i = i + 1)
            mem[i] <= {DSIZE {1'b0}};
    /* synopsys translate_on */
    
    assign rdata = mem[raddr];
    always @(posedge wclk) if (wclken && !wfull) mem[waddr] <= wdata;

endmodule
