/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    10:30:28 12/16/2011 
// Module Name:    rdptr_empty 
//////////////////////////////////////////////////////////////////////////////////
`default_nettype none
module yaf_rptr_empty 
#(
  parameter ASIZE = 4,
  parameter ENABLE_PREREAD = "NO"
)(
  output wire	rempty_val,
  output reg	[ASIZE :0] rptr,
  output reg	[ASIZE:0] rbin,
  input  wire	[ASIZE :0] rq2_wptr,
  input  wire   rempty,
  input  wire	rinc, 
  input  wire	rclk, 
  input  wire	rrst_n,
  input  wire	rptr_upd,
  input  wire	rptr_rest
);

wire [ASIZE:0] rgraynext;
wire [ASIZE:0] rbinnext;
//wire rempty_val;
//-------------------
// GRAYSTYLE2 pointer
//-------------------
always @(posedge rclk)
begin
  if (!rrst_n) 
  begin
    rbin <= {ASIZE+1{1'b0}};
  end else begin
    rbin <= rbinnext;
  end
end

// Memory read-address pointer (okay to use binary to address memory)
assign rgraynext = (rbinnext>>1) ^ rbinnext;

//---------------------------------------------------------------
// FIFO empty when the next rptr == synchronized wptr or on reset
//---------------------------------------------------------------
assign rempty_val = (rgraynext == rq2_wptr);
/*
always @(posedge rclk)
begin
  if (!rrst_n) 
    rempty <= 1'b1;
  else 
    rempty <= rempty_val;
end
*/
generate 
  if (ENABLE_PREREAD == "YES")
  begin
    reg [ASIZE:0] rbin_done;
    assign rbinnext = rptr_rest ? rbin_done : rbin + (rinc & ~rempty);
    always @(posedge rclk)
    begin
      if (~rrst_n) 
      begin
        rbin_done <= {ASIZE+1{1'b0}};
        rptr <= {ASIZE+1{1'b0}};
      end else if (rptr_upd) 
      begin
        rbin_done <= rbinnext;
        rptr <= rgraynext; 
      end
    end
  end else begin
    assign rbinnext = rbin + (rinc & ~rempty);
    always @(posedge rclk )
    begin
      if (~rrst_n) 
        rptr <= {ASIZE+1{1'b0}};
      else 
        rptr <= rgraynext; 
    end
  end
endgenerate

endmodule
