/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    10:38:13 12/16/2011 
// Module Name:    wptr_full 
//////////////////////////////////////////////////////////////////////////////////
`default_nettype none
module yaf_wptr_full 
#(
  parameter ASIZE = 4,
  parameter ENABLE_PREWRITE = "NO"
)(
  output wire	wfull_val,
  output reg	[ASIZE :0] wptr,
  output reg	[ASIZE:0] wbin,
  input  wire	[ASIZE :0] wq2_rptr,
  input  wire   wfull,
  input  wire	winc, 
  input  wire	wclk, 
  input  wire	wrst_n,
  input  wire	wptr_upd,
  input  wire	wptr_rest
);

wire [ASIZE:0] wgraynext;
wire [ASIZE:0] wbinnext;
//wire wfull_val;
//-------------------
// GRAYSTYLE2 pointer
//-------------------
always @(posedge wclk)
begin
  if (!wrst_n) 
  begin
    wbin <= {ASIZE+1{1'b0}};
  end else begin
    wbin <= wbinnext;
  end
end

// Memory write-address pointer (okay to use binary to address memory)
assign wgraynext = (wbinnext>>1) ^ wbinnext;

//------------------------------------------------------------------
// Simplified version of the three necessary full-tests:
// assign wfull_val=((wgnext[ASIZE] != wq2_rptr[ASIZE] ) &&
// (wgnext[ASIZE-1] !=wq2_rptr[ASIZE-1]) &&
// (wgnext[ASIZE-2:0]==wq2_rptr[ASIZE-2:0]));
//------------------------------------------------------------------
assign wfull_val = (wgraynext=={~wq2_rptr[ASIZE:ASIZE-1], wq2_rptr[ASIZE-2:0]});
/*
always @(posedge wclk)
begin
  if (!wrst_n) 
    wfull <= 1'b0;
  else 
    wfull <= wfull_val;
end
*/
generate
  if (ENABLE_PREWRITE == "YES")
  begin
    reg [ASIZE:0] wbin_done;
    assign wbinnext = wptr_rest ? wbin_done : wbin + (winc & ~wfull);
    always @(posedge wclk)
    begin
      if (~wrst_n) 
      begin
        wbin_done <= {ASIZE+1{1'b0}};
        wptr <= {ASIZE+1{1'b0}};
      end else if (wptr_upd) 
      begin
        wbin_done <= wbinnext;
        wptr <= wgraynext; 
      end
    end
  end else begin
    assign wbinnext = wbin + (winc & ~wfull);
    always @(posedge wclk)
    begin
      if (~wrst_n) 
        wptr <= {ASIZE+1{1'b0}};
      else 
        wptr <= wgraynext; 
    end
  end
endgenerate

endmodule

