/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// TU-Dresden Chair for Embedded Systems
//////////////////////////////////////////////////////////////////////////////////

//Geschrieben von: Markus Noll, mail@markus-noll.de, TU-Darmstadt 2015


module concentrator_master_dma
#(
       // SPMC Interface
        parameter DMA_BASE_ADR			= 18'h0,
        //parameter DMA_ADDRWIDTH			= 'd10,
        parameter DMA_SIZE				= 'd64,
        parameter DMA_WIDTH				= 'd18,
		parameter NUMBER_OF_SLAVES   = 'd3

)
(
        //*** Connections with SpartanMC master ***
        input   wire            clk_peri,
        input   wire            clk_dma,
        input   wire            reset,		//Reset-Signal
        
        //DMA-Signals
		input wire						mem_access,
		input wire						store_access_low,
		input wire						store_access_high,
		input wire	[9:0]				addr_peri,
		input wire	[7:0]				addr_high,
		input wire	[17:0]				do_peri,
		output wire	[17:0]				di_peri, 
	
		// Verbindungen zum slave
		output reg [((NUMBER_OF_SLAVES*2)-1):0]		status,
		input wire [((NUMBER_OF_SLAVES*18)-1):0] 	data,
	  	input wire [((NUMBER_OF_SLAVES*12)-1):0]	msg_size,
	  	input wire [(NUMBER_OF_SLAVES-1):0]	 		data_available,
	 	input wire [(NUMBER_OF_SLAVES-1):0]    		send
	
);
/////////////////////////////////////////////////////////////////////////////////////
//
// Local Functions and Parameters
//
function integer log2;
input integer value;
	for (log2=0; value>0; log2=log2+1)
	value = value>>1;
endfunction

parameter
	DECODSIM_DEB			= "NO",
    DMA_ADDRWIDTH          = log2(DMA_SIZE - 1);

//inter wires & regs
wire [17:0] data_out_a;
wire select;					//pselect zu Bram
wire [17:0] full_addr;
assign full_addr = {addr_high,addr_peri};
//Bram Wires
wire write;
wire [17:0] doutb;
wire [17:0] dinb;
wire store_access;
//wires für arbitrierer
wire [NUMBER_OF_SLAVES-1:0] grant;
wire [8:0] slave_index_from_encoder;
//Daten von ausgewähltem Slave
wire [17:0] selected_data;
wire selected_data_available;
wire selected_send;
assign selected_data = data[(slave_index_from_encoder*18) +: 18];
assign selected_data_available = data_available[slave_index_from_encoder];
assign selected_send = send[slave_index_from_encoder];

//FSM
reg [DMA_ADDRWIDTH-1:0] addrb_reg;
reg [DMA_ADDRWIDTH-1:0] msg_size_addr;
reg [DMA_ADDRWIDTH-1:0] slave_id_addr;
wire [DMA_ADDRWIDTH-1:0]addrb;
reg [3:0] state;
reg [11:0] counter;//oder 17:0?!
reg [5:0] slave_id;
localparam [3:0]
	IDLE				= 0,
	GET_MSG_SIZE_ADDR	= 1,
	GET_DATA_ADDR	 	= 2,
	GET_SLAVE_ID_ADDR	= 3,
	REC_HEADER			= 4,
	WRITE_SLAVE_ID		= 5,
	REC_DATA			= 6,
	FIN					= 7;
// Dekodierung der Adressen ********************** START *************************************
//pselect module to generate select signal for DMA memory
pselect #(
	.ADDR_WIDTH(18-DMA_ADDRWIDTH),
	.BASE_WIDTH(18-DMA_ADDRWIDTH),
	.BASE_ADDR(DMA_BASE_ADR[17:DMA_ADDRWIDTH])
	) pselect_0 (
	.addr(full_addr[17:DMA_ADDRWIDTH]),
	.activ_peri(mem_access),
	.select(select)
);
//Bram--evtl. nochmal in axi nach der 2ten Methode schauen
assign store_access = store_access_low || store_access_high;
dual_ported_RAM #(
	.WIDTH(DMA_WIDTH),
	.SIZE(DMA_SIZE),
	.ADDRWIDTH(DMA_ADDRWIDTH)
	) DMARAM00 (
	.clkA(clk_peri), 
	.clkB(clk_dma), 
	.enA(1'b1), 
	.enB(1'b1), 
.weA((store_access&&select)), 
	.weB(write),						//TODO:Singnals if to write data or not
	.rstA(!select), 
	.addrA(addr_peri), 
	.addrB(addrb), 
	.diA(do_peri), 
	.diB(dinb),							//TODO
	.doA(data_out_a), 
	.doB(doutb)							//TODO
);
//reg select_delay;
//always @(posedge clk_peri)begin
//	select_delay<=select;
//end
//assign di_peri=(select_delay)?data_out_a:18'b0;
assign di_peri=data_out_a;
//Round_Robin_arbiter
concentrator_arbiter #
(
	.NUMBER_OF_SLAVES(NUMBER_OF_SLAVES)
) arbiter0 (
	.clk (clk_dma),
	.reset (reset),
	.last_value_write_sel (~selected_data_available),
	.req (data_available),   
	.grant(grant)  
);
   
//Encoder für arbiter 
//Wandelt one-hot grant in eine index-Zahl um
one_hot_to_bin # 
(
	.ONE_HOT_WIDTH(NUMBER_OF_SLAVES),
	.BIN_WIDTH(9)
) encoder0 (   
	.reset(reset),
	.clk(clk_dma),
	.one_hot(grant),
	.binary_encoded(slave_index_from_encoder)
);
//Status
always @(posedge clk_dma)begin
	status = (state==REC_HEADER && ~selected_send) || (state==REC_DATA) ? 1000'd2<<(slave_index_from_encoder*2) :	//empfangsbereit
				(state==REC_HEADER && selected_send) || (state==WRITE_SLAVE_ID) ? (1000'd1<<(slave_index_from_encoder*2)) :				//buffer full^^
				1000'd0;																		//frei
end

//FSM
always @(posedge clk_dma)begin
	if(reset)begin
		counter <= 0;
		addrb_reg <= 10'd0;
		state<=IDLE;
	end else begin
		case(state)
			IDLE:begin
				state<=(doutb[1]==1'b1) ? GET_MSG_SIZE_ADDR : IDLE;
			end
			GET_MSG_SIZE_ADDR:begin
				msg_size_addr<=doutb[DMA_ADDRWIDTH-1:1];
				state<=GET_DATA_ADDR;
			end
			GET_DATA_ADDR:begin
				addrb_reg<=doutb[DMA_ADDRWIDTH-1:1];
				state<=GET_SLAVE_ID_ADDR;
			end
			GET_SLAVE_ID_ADDR:begin
				slave_id_addr<=doutb[DMA_ADDRWIDTH-1:1];
				state<=(data_available==0) ? GET_SLAVE_ID_ADDR : REC_HEADER;
			end
			REC_HEADER:begin
				slave_id<=(selected_send) ? selected_data[5:0] : 0;
				counter<=(selected_send) ? selected_data[17:6] : 0;
				state<=(send!=0) ? WRITE_SLAVE_ID : REC_HEADER;
			end
			WRITE_SLAVE_ID:begin
				state<=REC_DATA;
			end
			REC_DATA:begin
				counter<=(selected_send) ? counter-1 : counter;
				state<=(counter==1) && (selected_send) ? FIN : REC_DATA;
				addrb_reg<=(selected_send) ? 
							(counter==1) ? 0 : addrb_reg+1 : addrb_reg ;
			end
			FIN:begin
				state<=IDLE;
			end
			default:begin
				counter <= 0;
				addrb_reg <= 10'd0;
				state<=IDLE;
			end
		endcase
	end
end

assign addrb = (state == IDLE && ~doutb[1]==1'b1) ? 0 :
				(state == IDLE && doutb[1]==1'b1) ? 1 :
				(state == GET_MSG_SIZE_ADDR) ? 2 :
				(state == GET_DATA_ADDR) ? 3 :
				(state == GET_SLAVE_ID_ADDR) ? 3 :
				(state == REC_HEADER) ? msg_size_addr :
				(state == WRITE_SLAVE_ID) ? slave_id_addr :
				(state == REC_DATA) ? addrb_reg :
				(state == FIN) ? 0 : 18'bz;

assign write = reset || selected_send || (state == FIN) || (state == WRITE_SLAVE_ID) ? 1 : 0;
assign dinb = (reset || state==FIN) ? 1 :
				(state == REC_HEADER) ?  selected_data[17:6] :
				(state == WRITE_SLAVE_ID) ? slave_id : selected_data;
endmodule

