/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Execute and Memory                                         ////
////  In dieser Stufe werden die Befehle ausgefuehrt. Sie bein-  ////
////  haltet die ALU und die Specialfunctionregister. Des-       ////
////  weitern ist in dieser Stufe auch der Bypass fuer die ALU   ////
////  Operanten A und B zufinden und dessen Steuerung.           ////
////                                                             ////
////  Author: Falk Hassler                                       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`include	"core/src/spartanmc_def.vh"
`default_nettype none

module ex_mem #(
		parameter	RESET_VECTOR	=	0,
		parameter	INTR_VECTOR		=	32,
		parameter	RAMBLOCKS		=	1,
		parameter	PERIPHERIE_REGS	=	10,
		parameter	SINGLE_SHIFT	=	1,
		parameter	HARDWARE_MUL	=	1,
		parameter	DISABLE_STALLS	=	0,
		parameter	SHOW_MEM		=	0,
		parameter	SHOW_PERI		=	1,
		parameter	SHOW_INSTR		=	1,

		parameter HARDWARE_DEBUGGING=	0,
		parameter COUNT_BREAKPOINTS =	4,
		parameter COUNT_WATCHPOINTS =	4,
		parameter PERFORMANCE_COUNTER	= 0,
		parameter COUNT_CYCLECOUNTERS	= 1,
		parameter COUNT_EVENTCOUNTERS	= 2,
		parameter WIDTH_CYCLECOUNTER	= 36,
		parameter WIDTH_EVENTCOUNTER	= 36,
		parameter BRKPT_TRAP		=	1,
		parameter WTCHPT_TRAP		=	2,
		parameter SINGLE_STEP_TRAP	=	3
	) (
		input	wire			clk,
		input	wire			clk_en,
		input	wire			reset,
		output	wire	[6:0]	window,
		input	wire			inc_window,
		input	wire			dec_window,
		output	wire			cc,
		output	wire			intr_enable,
		input	wire			run_intr,
		input	wire	[17:0]	pc_plusone,
		output	wire	[17:0]	pc_intr,
		output	wire	[9:0]	pc_trap,
		input	wire	[9:0]	reg_no_a,
		input	wire	[9:0]	reg_no_b,
		output	wire	[9:0]	reg_no_awb,
		input	wire	[3:0]	r_no_b,
		input	wire	[3:0]	r_no_b_p1,
		input	wire	[17:0]	reg_a,
		input	wire	[17:0]	reg_b,
		input	wire			bypass1a,
		input	wire			bypass1b,
		input	wire	[17:0]	alu_imm,
		output	reg		[17:0]	pc_delay,
		input	wire	[3:0]	alu_func,
		output	reg		[17:0]	alu_result,
		output	reg		[`ADDR_WIDTH-1:0]	data_addr,
		output	wire			mem_addr_0,
		output	wire	[17:0]	write_data_data,
		output	wire			write_data_low,
		output	wire			write_data_high,
		input	wire			ir_return,
		input	wire			is_not,
		input	wire			is_mem,
		input	wire			is_mem_p1,
		input	wire			is_store,
		input	wire			is_imm,
		input	wire			is_branch,
		input	wire			is_nop,
		input	wire			branch_typ,
		input	wire			branch_neg,
		output	wire			do_delay_bj,
		input	wire			do_delay_bj_p2,
		input	wire			is_jump,
		input	wire			reg_to_pc,
		input	wire			link_pc,
		input	wire			short_trans,
		input	wire			delay_bj_p1,
		input	wire			delay_bj,
		input	wire			set_sfr,
		input	wire			get_sfr,
		input	wire			change_bit,
		input	wire			change_bit_p1,
		input	wire			set_bit,
		input	wire			set_bit_p1,
		input	wire			conditional_wb,
		input	wire			do_wb_cc,
		input	wire			do_ov,
		input	wire			multiply,
		input	wire			is_signed,
		input	wire			last_do_wb,
		input	wire	[9:0]	write_reg_no,
		input	wire	[17:0]	write_reg_data,
		input	wire	[17:0]	last_write_reg_data,
		input	wire			evcnt_ext0,
		input	wire			evcnt_ext1,

		input	wire	[`ADDR_WIDTH-1:0]	dbg_pc_next, //Breakpoint support
		output	wire			dbg_trap,
		output	wire	[7:0]	dbg_trap_no

		// synopsys translate_off
		,
		output	reg		[`OPERATION_LEN*8:0]	operation
		// synopsys translate_on
		);


wire		is_equal_zero_r;
wire		condition_r;
wire		is_equal_zero_c;
wire		condition_c;
wire		condition;
wire	[17:0]	alu_res;
wire		cc_result;
wire	[17:0]	mem_addr;
wire	[17:0]	sfr_data_out;
wire	[35:0]	product;

reg	[3:0]	sfr_adr;
wire		mm;
reg		set_cc;
reg		clear_cc;
reg		set_mm;
reg		clear_mm;
reg		set_intr_en_id;
reg		set_intr_en_p1;
wire		set_intr_enable;
reg		clear_intr_enable_id;
reg		clear_intr_enable_p1;
wire		clear_intr_enable;

reg		[17:0]	alu_op_a;
reg		[17:0]	alu_op_b;

wire	[17:0]	store_a;
reg		[17:0]	high_low;
wire    [8:0]	write_data_data_high;

wire			is_rel_jump;

assign	mem_addr_0	= mem_addr[0];

assign	set_intr_enable		= set_intr_en_id | set_intr_en_p1;
assign	clear_intr_enable	= clear_intr_enable_id | clear_intr_enable_p1;

wire	bypass2a;
wire	bypass2b;

assign	bypass2a	= (write_reg_no == reg_no_a) && last_do_wb;
assign	bypass2b	= (write_reg_no == reg_no_b) && last_do_wb;

wire	[9:0]	link_pc_reg;

`include	"core/util/reg_decode.vh"

assign	link_pc_reg = calc_addr(4'd11, window);
assign	reg_no_awb = link_pc ? link_pc_reg : reg_no_a;

// Bypassmux und dessen Steuerung fuer Operand A
// Bypass nicht aktivieren, wenn Operand A kein Register ist,
// der vorherige Befehl nichts schreibt und bei verzoegerten Spruengen ausser BNEZ und BEQZ

reg		[17:0]	reg_a_in;

always	@(*) begin
	casex ({bypass2a, bypass1a})
		default:	reg_a_in = reg_a;
		2'b1?:		reg_a_in = write_reg_data;			// Register Bypass 2 // bypass from WB to EX
		2'b?1:		reg_a_in = last_write_reg_data;		// Register Bypass 1 // bypass Registerfile
	endcase

	casex (is_mem | is_not)
		default:	alu_op_a = reg_a_in;		// use reg otherwise
		1'b1:		alu_op_a = alu_imm;		// for mem isns or NOT, alu_imm carries displ. / -1
	endcase
end

// Sprungbedingung fuer BxxZ Befehle testen
assign	is_equal_zero_r	= (18'h0 == alu_op_a);
assign	condition_r	= branch_neg ? !is_equal_zero_r : is_equal_zero_r;

// Sprungbedingung fuer BxxZC Befehle testen
assign	is_equal_zero_c	= (1'b0 ==  cc);
assign	condition_c	= branch_neg ? !is_equal_zero_c : is_equal_zero_c;

assign	condition	= branch_typ ? condition_c : condition_r;
assign	do_delay_bj	= (delay_bj_p1 & !do_delay_bj_p2) & (is_jump | condition);
// Do not branch or jump delayed, if the previous instruction has already taken a delayed branch or jump.

assign is_rel_jump = (is_jump & !reg_to_pc) | is_branch;

reg [17:0] reg_b_in;

// Bypassmux und dessen Steuerung fuer Operand B
// Bypass nicht aktivieren, wenn Operand B kein Register ist
// und der vorherige Befehl nichts schreibt
always	@(*) begin
	casex ({bypass2b, bypass1b})
		default:	reg_b_in = reg_b;				// Register B
		2'b1?:		reg_b_in = write_reg_data;		// Register Bypass 2 // bypass from WB to EX
		2'b01:		reg_b_in = last_write_reg_data;	// Register Bypass 1 // bypass Registerfile
	endcase

	casex ({link_pc, delay_bj_p1, is_imm})
		default:	alu_op_b = reg_b_in;
		3'b11?:		alu_op_b = pc_plusone;		// link_pc + delay_slot => link pc after delay slot (pc of after delay slot)
		3'b1??,									// link_pc without delay_slot (for trap and interrupt)
		3'b01?,									// Adresse fuer BxxZ  bei Ja
		3'b001:		alu_op_b = alu_imm;			// I-Befehle
	endcase
end

// Bypassmux und dessen Steuerung fuer pc_delay bei Jump Register
always	@(*)
	casex ({is_rel_jump, is_jump})
		default:	pc_delay = reg_b_in;		// all JR (JRS, JALRS, JALR)
		2'b1?:		pc_delay = alu_imm;			// rel jumps / branches, that calc. addr from displ.
	endcase

// MUX fuer sfr_addr
always @(*)
	if (!multiply)
		sfr_adr = r_no_b_p1[3:0];
	else
		sfr_adr = `SFR_MUL;
		
// Breakpoint Support!!
wire [1:0] dbg_mem_re;
assign dbg_mem_re = {2{is_mem & ~is_store}} & ((short_trans)? ((mem_addr_0)? 2'b01 : 2'b10) : 2'b11);

specialfunction		#(
			.RESET_VECTOR(RESET_VECTOR),
			.INTR_VECTOR(INTR_VECTOR),
			.SINGLE_SHIFT(SINGLE_SHIFT),
 			.HARDWARE_MUL(HARDWARE_MUL),
			.HARDWARE_DEBUGGING(HARDWARE_DEBUGGING),
			.COUNT_BREAKPOINTS(COUNT_BREAKPOINTS),
			.COUNT_WATCHPOINTS(COUNT_WATCHPOINTS),
			.PERFORMANCE_COUNTER(PERFORMANCE_COUNTER),
			.COUNT_EVENTCOUNTERS(COUNT_EVENTCOUNTERS),
			.WIDTH_CYCLECOUNTER(WIDTH_CYCLECOUNTER),
			.WIDTH_EVENTCOUNTER(WIDTH_EVENTCOUNTER),
			.BRKPT_TRAP(BRKPT_TRAP),
			.WTCHPT_TRAP(WTCHPT_TRAP),
			.SINGLE_STEP_TRAP(SINGLE_STEP_TRAP))

 sfr		(
		.clk1(			clk			),
		.clk_en(		clk_en			),
		.reset(			reset			),
		.addr(			sfr_adr			),
		.write(			set_sfr			),
		.data_in(		alu_op_a		),
		.data_out(		sfr_data_out		),
		.inc_window(		inc_window		),
		.dec_window(		dec_window		),
		.set_cc(		set_cc			),
		.clear_cc(		clear_cc		),
		.do_wb_cc(		do_wb_cc		),
		.cc_result(		cc_result		),
		.set_mm(		set_mm			),
		.clear_mm(		clear_mm		),
		.set_intr_enable(	set_intr_enable		),	// sbits s2
		.clear_intr_enable(	clear_intr_enable	),	// cbits s2
		.window(		window			),
		.cc(			cc			),
		.mm(			mm			),
		.intr_enable(		intr_enable		),
		.run_intr(		run_intr		),
		.pc_intr(		pc_intr			),
		.pc_trap(		pc_trap			),
		.product_hi(		product[35:18]		),
		.multiply(		multiply		),
		.dbg_pc_next(		dbg_pc_next		),//Breakpoint support
		.dbg_trap(			dbg_trap		),
		.dbg_trap_no(		dbg_trap_no		),
		.dbg_mem_addr(		data_addr		),
		.dbg_mem_we(		{write_data_high, write_data_low}),
		.dbg_mem_re(		dbg_mem_re		),
		.dbg_delayed_ir(	delay_bj		),
		.is_mem(		is_mem),
		.is_store(		is_store),
		.stall(			~clk_en),
		.is_nop(		is_nop),
		.ir_return(		ir_return),
		.evcnt_ext0(	evcnt_ext0),
		.evcnt_ext1(	evcnt_ext1)
		);

generate
if	(HARDWARE_MUL	== 1)
mult18_prim multiplier 	(
			.P(		product			),
			.A(		alu_op_a		),
			.B(		alu_op_b		)
			);
else
assign	product = 36'h0;
endgenerate

alu			#(
			.RESET_VECTOR(RESET_VECTOR),
			.INTR_VECTOR(INTR_VECTOR),
			.SINGLE_SHIFT(SINGLE_SHIFT),
 			.HARDWARE_MUL(HARDWARE_MUL))

alu_inst		(
			.opa(		alu_op_a		),
			.opb(		alu_op_b		),
			.result(	alu_res			),
			.cc_result(	cc_result		),
			.mem_addr(	mem_addr		),
			.is_signed(	is_signed		),
			.do_ov(		do_ov			),
			.func(		alu_func		)
			);

// ALU Result MUX
always @(*)
	casex({get_sfr, multiply, conditional_wb & !cc})
		3'b100:  alu_result = sfr_data_out;		// MOVS2I
		3'b010:  alu_result = product[17:0];	// MUL
		3'b001:  alu_result = alu_op_a;			// IFADDUI oder IFSUBUI wenn CC = 0
		default: alu_result = alu_res;
	endcase

assign store_a = reg_a_in;

// MUXE zur Auswahl der oberen oder unteren 9 Bit beim MEM Zugriff
// oder alle 18 Bits

// ADR fuer alle Datenzugriffe
always @(mm or alu_res or mem_addr)
	data_addr = { mm, mem_addr[`ADDR_WIDTH-1:1] };

// write Impuls fuer die Befehle s9 und s18
wire store_access    = (is_mem & is_store);
wire s19     = (store_access & !short_trans);
wire s9_low  = store_access & short_trans &  mem_addr[0];
wire s9_high = store_access & short_trans & !mem_addr[0];
assign write_data_low  = s9_low  | s19;
assign write_data_high = s9_high | s19;
			 
// MUX zur Auswahl der oberen oder unteren 9 Bit beim MEM Zugriff
// oder alle 18 Bits
assign write_data_data_high = (s9_high)? store_a[8:0] : store_a[17:9];
assign write_data_data = {write_data_data_high, store_a[8:0]};

//Logik fuer das Setzen und Ruecksetzen einzelner Status Bits mit CBITS oder SBITS
always @(*)
begin
	set_cc			= 0;
	set_mm			= 0;
	set_intr_en_p1		= 0;
	if (change_bit_p1 & set_bit_p1 & !clear_intr_enable)
		case (r_no_b_p1[1:0])
			default	: set_cc		= 1;
			2'h1	: set_mm		= 1;
			2'h2	: set_intr_en_p1	= 1;
		endcase
//	if (ir_return) set_intr_enable	= 1;
end

always @(*)
begin
	clear_cc		= 0;
	clear_mm		= 0;
	clear_intr_enable_p1	= 0;
	if (change_bit_p1 & !set_bit_p1 & !is_jump)
		case (r_no_b_p1[1:0])
			default	: clear_cc 		= 1;
			2'h1	: clear_mm     		= 1;
			2'h2	: clear_intr_enable_p1	= 1;
		endcase
end

always @(*)
begin
	clear_intr_enable_id	= 0;
	set_intr_en_id		= 0;
	if (change_bit & !set_bit)
		if (r_no_b[1:0]==2'h2)
			clear_intr_enable_id	= 1;
	if (change_bit & set_bit)
		if (r_no_b[1:0]==2'h2)
			set_intr_en_id	= 1;
end

// Erzeut einen String der die Operation die momentan in
// der Alu ausgefuehrt wird beschreibt.
// synopsys translate_off

always @(*)
	if (get_sfr)
		$sformat(operation, "s%2d", r_no_b_p1[3:0], sfr_data_out);
	else if (set_sfr & !multiply)
		$sformat(operation, "s%2d <= %h", r_no_b_p1, alu_op_a);
	else if (multiply)
		$sformat(operation, "%h * %h", alu_op_a, alu_op_b);
	else if (is_mem & !is_store)
		$sformat(operation, "mem[%h] => REGS[%4d]", {mm,mem_addr}, reg_no_awb);	// alu_op_b ist addr ohne bit 0
	else if (is_mem &  is_store)
		$sformat(operation, "mem[%h] <= REGS[%4d]", {mm,mem_addr}, reg_no_a);	// alu_op_b ist addr ohne bit 0
	else if (is_jump &  !is_branch & !delay_bj_p1)
		$sformat(operation, "pc+1=%h", alu_op_b);			// JALS, JAL und J
	else if (is_jump &  !is_branch & delay_bj_p1)
		$sformat(operation, "r=%h pc+1=%h", pc_delay, alu_op_b);	// JALRS, JALR, JRS, RFE und JR
	else if (!is_jump & is_branch & delay_bj_p1)
		$sformat(operation, "r=%h adr=%h", alu_op_a, alu_imm);		// BNEZ und BEQZ
	else if (is_jump & is_branch & !delay_bj_p1)
		$sformat(operation, "cc= %1h", cc);				// BNEZC und BEQZC
	else case (alu_func)
			`ALU_ADD:	$sformat(operation, "%h + %h", alu_op_a, alu_op_b);
			`ALU_SUB:	$sformat(operation, "%h - %h", alu_op_a, alu_op_b);
			`ALU_SEQ:	$sformat(operation, "%h  = %h cc = %h", alu_op_a, alu_op_b, cc_result);
			`ALU_SNE:	$sformat(operation, "%h != %h cc = %h", alu_op_a, alu_op_b, cc_result);
			`ALU_SLT:	$sformat(operation, "%h  < %h cc = %h", alu_op_a, alu_op_b, cc_result);
			`ALU_SGT:	$sformat(operation, "%h  > %h cc = %h", alu_op_a, alu_op_b, cc_result);
			`ALU_SLE:	$sformat(operation, "%h <= %h cc = %h", alu_op_a, alu_op_b, cc_result);
			`ALU_SGE:	$sformat(operation, "%h >= %h cc = %h", alu_op_a, alu_op_b, cc_result);
			`ALU_AND:	$sformat(operation, "%h & %h", alu_op_a, alu_op_b);
			`ALU_OR:	$sformat(operation, "%h | %h", alu_op_a, alu_op_b);
			`ALU_XOR:	$sformat(operation, "%h ^ %h", alu_op_a, alu_op_b);
			`ALU_SLL:	$sformat(operation, "%h << 1", alu_op_a, alu_op_b);
			`ALU_SLR:	$sformat(operation, "%h >> 1", alu_op_a, alu_op_b);
			`ALU_SRA:	$sformat(operation, "%h -> %h", alu_op_a, alu_op_b);
			`ALU_SIGNEX:	$sformat(operation, "%h SIGNEX %2d", alu_op_a, alu_op_b[4:0]);
			`ALU_PASSB:	$sformat(operation, "PASSB");
		endcase

// synopsys translate_on

endmodule
