/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Instruction fetch and decode                               ////
////  Diese Pipelinestufe holt die Befehle aus dem Blockram und  ////
////  decodiert sie um die Operaten fuer die ExMem Stufe zu      ////
////  erhalten und die Steuersignale fuer den restlichen Pro-    ////
////  zessor bereit zu stellen                                   ////
////                                                             ////
////  Author: Falk Hassler                                       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`include	"core/src/spartanmc_def.vh"
`default_nettype none

module ifetch_idecode	(
			clk1,
			locked,
			reset,
			was_reset,
			clk_en,
			code_busy,
			interrupt,
			ir_return,
			pc_plusone,
			pc_delay,
			pc_intr,
			pc_trap,
			inc_window,
			dec_window,
			reg_cc,
			intr_enable,
			run_intr,
			reg_window,
			reg_no_a,
			reg_no_b,
			r_no_b,
			reg_a_p1,
			reg_b_p1,
			bypass1a,
			bypass1b,
			alu_imm,
			alu_func,
			write_reg_number,
			write_reg_data,
			write_reg,
			ir,
			code_addr,
			code_jmp,
			// Steuersignale
			is_not,
			is_mem,
			is_store,
			is_imm,
			is_branch,
			is_nop,
			branch_neg,
			branch_typ,
			is_jump,
			reg_to_pc,
			link_pc,
			short_trans,
			is_delay,
			delay_bj_p1,
			do_delay_bj,
			set_sfr,
			get_sfr,
			change_bit,
			set_bit,
			conditional_wb,
			do_wb_cc,
			do_ov,
			do_wb,
			multiply,
			is_signed,
			mem_clk,
			dbg_pc_next, //Breakpoint Support
			exc_trap,
			exc_trap_no //Exception Support

			// synopsys translate_off
			,
			instruction
			// synopsys translate_on

			);

// dualer Logarithmus (abgerundet)
function integer floor_ld(input integer value);
	integer i;
	begin
		floor_ld = -1;
		for (i = 1; i <= value; i = i << 1)
			floor_ld = floor_ld + 1;
	end
endfunction

// Die Folgenden Parameter werden durch die system.v bei der Erzeugung des Modules ueberschrieben.
parameter	RESET_VECTOR	=	0;
parameter	INTR_VECTOR	=	32;
parameter	SINGLE_SHIFT	=	1;
parameter	HARDWARE_MUL	=	1;
parameter	HARDWARE_DEBUGGING = 0;
parameter	STOP_SIM_ON_X = 1;


input wire		clk1;
input wire		locked;
input wire		reset;
input wire		was_reset;
input wire		clk_en;
input wire		code_busy;
input wire		interrupt;
output		inc_window;
output		dec_window;
input wire		reg_cc;
input wire		intr_enable;
output		run_intr;
output	[17:0]	pc_plusone;
input wire	[17:0]	pc_delay;
input	[17:0]	pc_intr;
input	[9:0]	pc_trap;
input wire	[6:0]	reg_window;
output	[9:0]	reg_no_a;
output	[9:0]	reg_no_b;
output	[3:0]	r_no_b;
output	[17:0]	reg_a_p1;		// Das ist der Wert von Register a.
output	[17:0]	reg_b_p1;		// (Ist Teil von pipe_1)
output	wire			bypass1a;
output	wire			bypass1b;
output	reg	[17:0]		alu_imm;
output	[3:0]	alu_func;
input wire	[9:0]	write_reg_number;
input wire	[17:0]	write_reg_data;
input wire		write_reg;

input wire	[17:0]	ir;

output			code_jmp;
output	wire [`ADDR_WIDTH-1:0]	code_addr;

output		is_not;
output		is_mem;
output		is_store;
output		is_imm;
output		is_branch;
output		is_nop;
output		branch_neg;
output		branch_typ;
output		is_jump;
output		reg_to_pc;
output		link_pc;
output		short_trans;
output		is_delay;
input wire		delay_bj_p1;
input wire		do_delay_bj;
output		set_sfr;
output		get_sfr;
output		change_bit;
output		set_bit;
output		conditional_wb;
output		do_wb_cc;
output		do_ov;
output		do_wb;
output		multiply;
output		is_signed;
output	wire 	ir_return;
input wire		mem_clk;

// synopsys translate_off
output	reg	[`INSTRUCTION_LEN*8:0]	instruction;
// synopsys translate_on

reg	[`ADDR_WIDTH-1:0]	pc_new;
reg	[`ADDR_WIDTH-1:0]	pc;

wire	[`ADDR_WIDTH-1:0]	pc_plusone;
wire	[`ADDR_WIDTH-1:0]	pc_jump;
wire	[`ADDR_WIDTH-1:0]	pc_intr;
wire	[`ADDR_WIDTH-9:0]	pc_trap;	// in die unteren 8 Bit wird die TRAP Nr. eingeblendet


//Breakpoint Support
output wire [`ADDR_WIDTH-1:0] dbg_pc_next;
assign dbg_pc_next = pc_new;
//Exception Support
input wire	[7:0]	exc_trap_no;
input wire			exc_trap;
wire				exc_trap_exec;

wire	[17:0]	reg_a_p1;
wire	[17:0]	reg_b_p1;
wire	[9:0]	reg_no_a;
wire	[9:0]	reg_no_b;
wire	[3:0]	r_no_a;
wire	[3:0]	r_no_b;
wire	[17:0]	imm;
wire	[17:0]	imm_high;
wire	[`ADDR_WIDTH-1:0]	offset;
wire	[`ADDR_WIDTH-1:0]	disp;

//		Steuersignale
wire		inc_window;
wire		dec_window;
wire		reg_to_pc;
wire		link_pc;
wire		is_jump;
wire		is_branch;
wire		branch_neg;
wire		branch_typ;
wire		is_short_branch;
wire		is_delay;
wire		is_delayed_rel_jump;
wire		is_not;
wire		is_mem;
wire		is_store;
wire		is_imm;
wire		is_nop;
wire		sign_ext;
wire		load_high;
wire		short_trans;
wire		set_sfr;
wire		get_sfr;
wire		change_bit;
wire		set_bit;
wire		do_wb_without_cond;
wire		conditional_wb;
wire		do_wb_cc;
wire		do_ov;
wire		do_wb = do_wb_without_cond;
wire		multiply;
wire		is_signed;
wire	[3:0]	alu_func;

reg		intr;

`ifdef toolchain_quartus

`elsif toolchain_lattice

`else
// synthesis attribute ASYNC_REG of intr is "TRUE";
`endif

wire		run_intr;
wire		trap;
wire		do_bj;
wire		code_jmp;



// Provide pc_new to the memory,
// but hold pc while no valid instruction is loaded and pc_new is invalid.
assign code_addr	= code_busy || was_reset ? pc : pc_new;

// Indicate that the current code_addr isn't the previous plus one:
assign do_bj		= do_delay_bj | trap | (HARDWARE_DEBUGGING && exc_trap_exec) | intr;
assign code_jmp		= do_bj | code_busy;
		// simply keep code_jmp high, while do_bj is not valid (code_busy != 0)


// Hier werden die Steuersignale aus dem Opcode erzeugt

control_logic	ctrl		(
			.opcode(	{ir[17:16], ir[15:13],
					 ir[4:3], ir[2:0]}	),
			.intr(		intr			),
			.run_intr(	run_intr		),
			.inc_window(	inc_window		),
			.dec_window(	dec_window		),
			.reg_to_pc(	reg_to_pc		),
			.link_pc(	link_pc	),
			.is_jump(	is_jump			),
			.is_branch(	is_branch		),
			.is_delay(	is_delay		),
			.branch_typ(	branch_typ		),
			.branch_neg(	branch_neg		),
			.is_mem(	is_mem			),
			.is_store(	is_store		),
			.is_imm(	is_imm			),
			.sign_ext(	sign_ext		),
			.load_high(	load_high		),
			.short_trans(	short_trans		),
			.set_sfr(	set_sfr			),
			.get_sfr(	get_sfr			),
			.change_bit(	change_bit		),
			.set_bit(	set_bit			),
			.do_wb_cc(	do_wb_cc		),
			.do_ov(		do_ov			),
			.do_wb(		do_wb_without_cond	),
			.conditional_wb(conditional_wb		),
			.multiply(	multiply		),
			.is_signed(	is_signed		),
			.alu_func(	alu_func		),
			.ir_return(	ir_return		),
			.trap(		trap			),
			.is_not(	is_not			),
			.exc_trap_exec(	exc_trap_exec	)
			);

registerfile #(.STOP_SIM_ON_X(STOP_SIM_ON_X)) regs	(
			.clk1(		clk1			),
			.clk_en(	clk_en		),
			.read_reg1(	reg_no_a		),
			.read_reg2(	reg_no_b		),
			.write_reg(	write_reg_number	),
			.write_data(	write_reg_data		),
			.read_data1(	reg_a_p1		),
			.read_data2(	reg_b_p1		),
			.write(		write_reg		)
			);

// IR decodieren
`include	"core/util/reg_decode.vh"

assign	r_no_a		= ir[12:9];
assign	r_no_b		= ir[8:5];
assign	offset		= { {5 {ir[12]          }}, ir[12:0] };
assign	imm		= { {9 {ir[8] & sign_ext}}, ir[8:0]  };
assign	imm_high	= {     ir[8:0]           , 9'h000   };
assign	disp		= { {`ADDR_WIDTH-5 {1'b0}}, ir[4:0]  };

// absolute Registeradressen berechnen
assign	reg_no_a	= calc_addr(r_no_a, reg_window);
assign	reg_no_b	= calc_addr(r_no_b, reg_window);

// Reg A Bypass Steuerung fuer write zu read
assign bypass1a	= ((reg_no_a == write_reg_number) && write_reg);

// Reg B Bypass Steuerung fuer write zu read
assign bypass1b	= ((reg_no_b == write_reg_number) && write_reg);

// is_nop for performance counter
assign is_nop = (({ir[17:13], ir[4:0]} == `OR) && (r_no_a == r_no_b));

// PC
always @(posedge clk1)
	if (reset)
		pc <= RESET_VECTOR;
	else if (clk_en)
		pc <= pc_new;

// PC aendern und gegebenenfalls Spruenge verarbeiten
assign	pc_plusone	= pc + ((reset) ? 18'b0 : 18'b1);
assign	is_short_branch	= is_branch & !branch_typ;
assign	pc_jump		= pc + (is_short_branch ? imm : offset);

// Verzoegerten relativen Sprung erkennen
assign is_delayed_rel_jump = is_delay & !reg_to_pc;

// Interruptbehandlung ausloesen (intr darf nur fuer einen Takt 1 sein!)
always @(posedge clk1)
	if (reset)
		intr <= 1'b0;
	else if (clk_en) begin
    	if (run_intr)
     		intr <= 1'b0;
		else if (interrupt & intr_enable)
			intr <= !is_delay;
		else
			intr <= 1'b0;
	end

generate
if (!HARDWARE_DEBUGGING) begin
	assign exc_trap_exec = 0;
	// Der neue Wert fuer den PC wird durch diesen
	// Multiplexer bestimmt. Auf einen delay Sprung darf
	// nie unmittelbar ein weiterer Sprung folgen.
	always @(*)
		casex ({reset, do_delay_bj, intr, trap})
			default	:	pc_new = pc_plusone; // default + in cases of delay slots
			4'b0001:	pc_new = { pc_trap, r_no_a, r_no_b }; // trap
			4'b001?:	pc_new = pc_intr;
			4'b01??:	pc_new = pc_delay;	// jumps with delay slot
		endcase
end else begin // Exception Support
	reg exc_trap_catchup;
	reg exc_trap_buf;
	reg [7:0] exc_trap_no_buf;
	reg delay_bj_last; // do_delay_bj is unstable, influenced by us
	
	assign exc_trap_exec = (exc_trap_buf | exc_trap_catchup) & !is_delay & !delay_bj_last;
	
	always @(posedge clk1) begin
		if (reset) begin
			exc_trap_catchup <= 0;
			exc_trap_no_buf <= 0;
		end else if(clk_en) begin
			if (exc_trap & (is_delay | delay_bj_last))
				exc_trap_catchup <= 1;
			else if (exc_trap_exec)
				exc_trap_catchup <= 0;

			exc_trap_buf <= exc_trap;
			delay_bj_last <= is_delay;

			if (exc_trap)
				exc_trap_no_buf <= exc_trap_no;
		end
	end

	always @(*)
		casex ({reset, do_delay_bj, exc_trap_exec, intr, trap})
			default	:	pc_new = pc_plusone;
			5'b00001:	pc_new = { pc_trap, r_no_a, r_no_b };
			5'b0001?:	pc_new = pc_intr;
			5'b001??:	pc_new = { pc_trap, exc_trap_no_buf};
			5'b01???:	pc_new = pc_delay;	// jumps with delay slots
		endcase
end
endgenerate

wire intr_or_trap;
assign intr_or_trap = (intr | exc_trap_exec);

always @(*)
	casex ({is_delayed_rel_jump, is_mem, is_not, link_pc, intr_or_trap, is_imm, load_high})
		default:		alu_imm = -1; // not insn, if it fails also very recognizable!!
		7'b100?000:		alu_imm = pc_jump; // branch with delay slot (bxxz, bxxzc)
		7'b0001000:		alu_imm = pc_plusone; // trap insn
		7'b0001100:		alu_imm = pc; // current Insn. is being overwritten by exception / interrupt, link the current pc and execute trap / isr. ex_mem handles delayed linking
		7'b0100000:		alu_imm = disp;		// mem insns (s9/18 l9/18)
		7'b0000010:		alu_imm = imm;	// immediates (addi, ori, andi...)
		7'b0000011:		alu_imm = imm_high;	// LHI
	endcase

reg invalid_insn;

// Hier wird zum Debugen ein String erzeugt in dem
// der aktuelle Befehl mit seiner Adresse und seinen
// Operanden steht
// synopsys translate_off
always @(*) begin
	invalid_insn = 0;
	if (trap)
		$sformat(instruction, "0x%05H  Trap      !!!", pc);
	else
	if (intr)
		$sformat(instruction, "0x%05H  Interrupt !!!", pc);
	else
	if (exc_trap_exec)
		$sformat(instruction, "0x%05H  Trap Exception !!!", pc);
	else
	casex ({ir[17:16], ir[15:13], ir[4:3], ir[2:0]})
		`J	: $sformat(instruction, "0x%05H  J       %5h       ", 	pc, pc+offset);
		`JALS	: $sformat(instruction, "0x%05H  JALS    %5h       ", 	pc, pc+offset);
		`JR	: $sformat(instruction, "0x%05H  JR      r%2d         ", 	pc, r_no_b);
		`JALR	: $sformat(instruction, "0x%05H  JALR    r%2d         ", 	pc, r_no_b);
		`JRS	: $sformat(instruction, "0x%05H  JRS     r%2d         ", 	pc, r_no_b);
		`RFE	: $sformat(instruction, "0x%05H  RFE     r%2d         ", 	pc, r_no_b);
		`JALRS	: $sformat(instruction, "0x%05H  JALRS   r%2d         ", 	pc, r_no_b);

		`BEQZ	: $sformat(instruction, "0x%05H  BEQZ    r%2d, %h  ", 	pc, r_no_a, pc+imm);
		`BNEZ	: $sformat(instruction, "0x%05H  BNEZ    r%2d, %h  ", 	pc, r_no_a, pc+imm);
		`BEQZC	: $sformat(instruction, "0x%05H  BEQZC   %5h       ",	pc, pc+offset);
		`BNEZC	: $sformat(instruction, "0x%05H  BNEZC   %5h       ", 	pc, pc+offset);

		`ADD	: $sformat(instruction, "0x%05H  ADD     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`ADDU	: $sformat(instruction, "0x%05H  ADDU    r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SUB	: $sformat(instruction, "0x%05H  SUB     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SUBU	: $sformat(instruction, "0x%05H  SUBU    r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`AND	: $sformat(instruction, "0x%05H  AND     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`OR	: $sformat(instruction, "0x%05H  OR      r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`XOR	: $sformat(instruction, "0x%05H  XOR     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`NOT	: $sformat(instruction, "0x%05H  NOT     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`MUL	: $sformat(instruction, "0x%05H  MUL     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SEQ	: $sformat(instruction, "0x%05H  SEQ     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SNE	: $sformat(instruction, "0x%05H  SNE     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SLT	: $sformat(instruction, "0x%05H  SLT     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SGT	: $sformat(instruction, "0x%05H  SGT     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SLE	: $sformat(instruction, "0x%05H  SLE     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SGE	: $sformat(instruction, "0x%05H  SGE     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`MOVI2S	: $sformat(instruction, "0x%05H  MOVI2S  r%2d, s%2d    ", 	pc, r_no_a, r_no_b);
		`MOVS2I	: $sformat(instruction, "0x%05H  MOVS2I  r%2d, s%2d    ", 	pc, r_no_a, r_no_b);

		`SLL	: $sformat(instruction, "0x%05H  SLL     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SRL	: $sformat(instruction, "0x%05H  SRL     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SRA	: $sformat(instruction, "0x%05H  SRA     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`MOV	: $sformat(instruction, "0x%05H  MOV     r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SEQU	: $sformat(instruction, "0x%05H  SEQU    r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SNEU	: $sformat(instruction, "0x%05H  SNEU    r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SLTU	: $sformat(instruction, "0x%05H  SLTU    r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SGTU	: $sformat(instruction, "0x%05H  SGTU    r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SLEU	: $sformat(instruction, "0x%05H  SLEU    r%2d, r%2d    ", 	pc, r_no_a, r_no_b);
		`SGEU	: $sformat(instruction, "0x%05H  SGEU    r%2d, r%2d    ", 	pc, r_no_a, r_no_b);

		`ADDI	: $sformat(instruction, "0x%05H  ADDI    r%2d, %h  ", 	pc, r_no_a, imm);
		`ANDI	: $sformat(instruction, "0x%05H  ANDI    r%2d, %h  ", 	pc, r_no_a, imm);
		`ORI	: $sformat(instruction, "0x%05H  ORI     r%2d, %h  ", 	pc, r_no_a, imm);
		`XORI	: $sformat(instruction, "0x%05H  XORI    r%2d, %h  ", 	pc, r_no_a, imm);
		`MULI	: $sformat(instruction, "0x%05H  MULI    r%2d, %h  ", 	pc, r_no_a, imm);
		`SLLI	: $sformat(instruction, "0x%05H  SLLI    r%2d, %h  ", 	pc, r_no_a, imm);
		`SRLI	: $sformat(instruction, "0x%05H  SRLI    r%2d, %h  ", 	pc, r_no_a, imm);
		`SRAI	: $sformat(instruction, "0x%05H  SRAI    r%2d, %h  ", 	pc, r_no_a, imm);

		`SEQI	: $sformat(instruction, "0x%05H  SEQI    r%2d, %h  ", 	pc, r_no_a, imm);
		`SNEI	: $sformat(instruction, "0x%05H  SNEI    r%2d, %h  ", 	pc, r_no_a, imm);
		`SLTI	: $sformat(instruction, "0x%05H  SLTI    r%2d, %h  ", 	pc, r_no_a, imm);
		`SGTI	: $sformat(instruction, "0x%05H  SGTI    r%2d, %h  ", 	pc, r_no_a, imm);
		`SLEI	: $sformat(instruction, "0x%05H  SLEI    r%2d, %h  ", 	pc, r_no_a, imm);
		`SGEI	: $sformat(instruction, "0x%05H  SGEI    r%2d, %h  ", 	pc, r_no_a, imm);

		`SIGEX	: $sformat(instruction, "0x%05H  SIGEX   r%2d, %h  ", 	pc, r_no_a, imm);
		`MOVI	: $sformat(instruction, "0x%05H  MOVI    r%2d, %h  ", 	pc, r_no_a, imm);

		`LHI	: $sformat(instruction, "0x%05H  LHI     r%2d, %h  ", 	pc, r_no_a, imm);
		`S18	: $sformat(instruction, "0x%05H  S18     %2d(r%2d), r%2d", 	pc, disp, r_no_b, r_no_a);
		`S9	: $sformat(instruction, "0x%05H  S9      %2d(r%2d), r%2d", 	pc, disp, r_no_b, r_no_a);
		`L18	: $sformat(instruction, "0x%05H  L18     r%2d, %2d(r%2d)", 	pc, r_no_a, disp, r_no_b);
	 	`L9	: $sformat(instruction, "0x%05H  L9      r%2d, %2d(r%2d)", 	pc, r_no_a, disp, r_no_b);
		`SWAP	: $sformat(instruction, "0x%05H  SWAP    r%2d, %2d(r%2d)", 	pc, r_no_a, disp, r_no_b);

		`SBITS	: $sformat(instruction, "0x%05H  SBITS   s%2d         ", 	pc, r_no_b);
		`CBITS	: $sformat(instruction, "0x%05H  CBITS   s%2d         ", 	pc, r_no_b);

		`IFADDUI: $sformat(instruction, "0x%05H  IFADDUI r%2d, %h   ", 	pc, r_no_a, imm);
		`IFSUBUI: $sformat(instruction, "0x%05H  IFSUBUI r%2d, %h   ", 	pc, r_no_a, imm);

		`TRAP	: $sformat(instruction, "0x%05H  TRAP    0x%1h%1h    ", 	pc, r_no_a, r_no_b);
		default : begin
			  $sformat(instruction, "0x%05H  INVALID             ", 	pc, r_no_a, r_no_b);
			  invalid_insn = 1;
		end
	endcase
end

reg stop = 0;
always @(posedge clk1) begin
	if (~reset && ~was_reset && clk_en && invalid_insn) begin
		$display("Error: Invalid instruction at 0x%05H: 0x%05H!",pc,ir);
		stop <= 1;
	end
	if (stop)
		$stop();
end
// synopsys translate_on


endmodule
