/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale	1ns / 10ps
`default_nettype none

module ormuxy	(
			in_bus,
			or_out
		);

parameter	[31:0]	NUM_BITS = 2;
parameter	[31:0]	NUM_BITS_PAD = ((NUM_BITS-1)/4+1)*4;

input wire	[NUM_BITS-1:0]	in_bus;
output wire			or_out;

genvar		i;

generate
if (NUM_BITS < 5)
begin:USE_LUT_OR_GEN
	wire	[NUM_BITS-1:0]	or_tmp;
	for (i=0; i<NUM_BITS; i=i+1)
	begin:BIT_LOOP
		if (i==0)
		begin:FIRST
			assign or_tmp[i] = in_bus[0];
		end
		else
		begin:REST
			assign or_tmp[i] = or_tmp[i-1] |
					   in_bus[i];
		end
	end

	assign or_out = or_tmp[NUM_BITS-1];
end
else
begin:MORETHAN4_GEN
	wire	[NUM_BITS_PAD/4-1:0]	lut_out;
	wire	[NUM_BITS_PAD-1:0]	in_bus_pad;
	wire	[NUM_BITS_PAD/4-1:0]	or_tmp;

	assign in_bus_pad[NUM_BITS-1:0] = in_bus[NUM_BITS-1:0];
	if (NUM_BITS_PAD>NUM_BITS)
	begin:SET_ZERO
		assign in_bus_pad[NUM_BITS_PAD-1:NUM_BITS] = 0;
	end

	for (i=0; i < NUM_BITS_PAD/4; i=i+1)
	begin:OR_GENERATE
		assign lut_out[i] = !(	in_bus_pad[i*4]	  |
					in_bus_pad[i*4+1] |
					in_bus_pad[i*4+2] |
					in_bus_pad[i*4+3] );

		if (i==0)
		begin:FIRST
			mux_prim FIRSTMUX_I (
						.O(	or_tmp[i]	),
						.CI(	1'b0		),
						.DI(	1'b1		),
						.S(	lut_out[i]	)
					);
		end else
		begin:REST
			mux_prim RESTMUX_I	 (
						.O(	or_tmp[i]	),
						.CI(	or_tmp[i-1]	),
						.DI(	1'b1		),
						.S(	lut_out[i]	)
					);
		end
	end

	assign or_out = or_tmp[NUM_BITS_PAD/4-1];
end
endgenerate

endmodule
