/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`ifndef		SPARTANMC_DEF
`define		SPARTANMC_DEF

`timescale	1  ns / 10 ps

`undef		DMABRAM_ENABLE
`define		SHOW_MEM	1
`define		SHOW_PERI	1


`define		INSTRUCTION_LEN	29
`define		OPERATION_LEN	30

`define		ADDR_WIDTH	18
// Alle ALU Befehle
// Arithmetische und logische Befehle immer MSB = 0
`define		ALU_DONTCARE	4'h0
`define		ALU_ADD		4'h0
`define		ALU_SUB		4'h1
`define		ALU_AND		4'h2
`define		ALU_OR		4'h3
`define		ALU_XOR		4'h4
`define		ALU_SLL		4'h5
`define		ALU_SLR		4'h6
`define		ALU_SRA		4'h7
`define		ALU_SIGNEX	4'h8
`define		ALU_PASSB	4'h9
`define		ALU_SEQ		4'ha
`define		ALU_SNE		4'hb
`define		ALU_SLT		4'hc
`define		ALU_SGT		4'hd
`define		ALU_SLE		4'he
`define		ALU_SGE		4'hf

// Die Specialfunctionregister und ihre Addressen
// im Registerspeicher
`define		SFR_STATUS	4'h0
`define		SFR_LEDS	4'h1 // currently free, no longer implemented
`define		SFR_MUL		4'h2
`define		SFR_CC		4'h3
`define		SFR_IV		4'h4
`define		SFR_TR		4'h5
`define		SFR_DBG_IDX	4'h6
`define		SFR_DBG_DAT	4'h7
`define		SFR_DBG_LAST 4'h8
`define		SFR_PCNT_IDX 4'h9
`define		SFR_PCNT_DAT 4'ha

// Alle Opcodes und die dazugehoerigen
// Memonics
`define		J	10'b00_010_??_???
`define		JALS	10'b00_011_??_???
`define		BEQZ	10'b00_100_??_???
`define		BNEZ	10'b00_101_??_???
`define		BEQZC	10'b00_110_??_???
`define		BNEZC	10'b00_111_??_???
`define		ADDI	10'b01_000_??_???
`define		MOVI	10'b01_001_??_???
`define		LHI	10'b01_010_??_???
`define		SIGEX	10'b01_011_??_???
`define		ANDI	10'b01_100_??_???
`define		ORI	10'b01_101_??_???
`define		XORI	10'b01_110_??_???
`define		MULI	10'b01_111_??_???
`define		L9	10'b10_000_??_???
`define		S9	10'b10_001_??_???
`define		L18	10'b10_010_??_???
`define		S18	10'b10_011_??_???
`define		SLLI	10'b10_100_??_???
`define 	SWAP	10'b10_101_??_???
`define		SRLI	10'b10_110_??_???
`define		SRAI	10'b10_111_??_???
`define		SEQI	10'b11_000_??_???
`define		SNEI	10'b11_001_??_???
`define		SLTI	10'b11_010_??_???
`define		SGTI	10'b11_011_??_???
`define		SLEI	10'b11_100_??_???
`define		SGEI	10'b11_101_??_???
`define		IFADDUI	10'b11_110_??_???
`define		IFSUBUI	10'b11_111_??_???

`define		ORCC	10'b00_000_00_000
`define		ANDCC	10'b00_000_00_001
//`define	MOVI2C	10'b00_000_00_010
//`define	MOVC2I	10'b00_000_00_011
`define		SLL	10'b00_000_00_100
`define		MOV	10'b00_000_00_101
`define		SRL	10'b00_000_00_110
`define		SRA	10'b00_000_00_111
`define		SEQU	10'b00_000_01_000
`define		SNEU	10'b00_000_01_001
`define		SLTU	10'b00_000_01_010
`define		SGTU	10'b00_000_01_011
`define		SLEU	10'b00_000_01_100
`define		SGEU	10'b00_000_01_101
`define		CBITS	10'b00_000_10_110
`define		SBITS	10'b00_000_10_111
`define		NOT	10'b00_000_11_111

`define		RFE	10'b00_001_00_000
`define		TRAP	10'b00_001_00_001
`define		JR	10'b00_001_00_010
`define		JALR	10'b00_001_00_011
`define		JRS	10'b00_001_00_100
`define		JALRS	10'b00_001_00_101
`define		ADD	10'b00_001_10_000
`define		ADDU	10'b00_001_10_001
`define		SUB	10'b00_001_10_010
`define		SUBU	10'b00_001_10_011
`define		AND	10'b00_001_10_100
`define		OR	10'b00_001_10_101
`define		XOR	10'b00_001_10_110
`define		MUL	10'b00_001_10_111
`define		SEQ	10'b00_001_11_000
`define		SNE	10'b00_001_11_001
`define		SLT	10'b00_001_11_010
`define		SGT	10'b00_001_11_011
`define		SLE	10'b00_001_11_100
`define		SGE	10'b00_001_11_101
`define		MOVI2S	10'b00_001_11_110
`define		MOVS2I	10'b00_001_11_111

`endif	
