/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Specialfunctionregister                                    ////
////  In diesem Modul befinden sich die Specialfunctionregister. ////
////  Sie werden einzel implementiert und dann durch einen       ////
////  Multiplexer abhaengig von ihrer Adresse auf data_out ge-   ////
////  legt. Sie koennen aber auch durch unabhaengige Set und     ////
////  Clear Signale von ExMem durch SBITS und CBITS geaendert    ////
////  werden.                                                    ////
////                                                             ////
////  Author: Falk Hassler                                       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`include	"core/src/spartanmc_def.vh"
`include	"core/util/pack_unpack.vh"
`default_nettype none

module specialfunction	(
			clk1,
			reset,
			clk_en,
			addr,
			write,
			data_in,
			data_out,
			inc_window,
			dec_window,
			set_cc,
			clear_cc,
			do_wb_cc,
			cc_result,
			set_mm,
			clear_mm,
			set_intr_enable,
			clear_intr_enable,
			window,
			cc,
			mm,
			intr_enable,
			run_intr,
			pc_intr,
			pc_trap,
			product_hi,
			multiply,
			dbg_pc_next,//Breakpoint support
			dbg_trap,
			dbg_trap_no,
			dbg_mem_addr,
			dbg_mem_we,
			dbg_mem_re,
			dbg_delayed_ir,
			is_mem,
			is_store,
			stall,
			is_nop,
			ir_return,
			evcnt_ext0,
			evcnt_ext1
			);

// Die Folgenden Parameter werden durch die system.v bei der Erzeugung des Modules ueberschrieben.
parameter	RESET_VECTOR	=	0;
parameter	INTR_VECTOR	=	32;
parameter	SINGLE_SHIFT	=	1;
parameter	HARDWARE_MUL	=	1;

parameter HARDWARE_DEBUGGING = 0;
parameter COUNT_BREAKPOINTS = 4;
parameter COUNT_WATCHPOINTS = 4;
parameter BRKPT_TRAP = 1;
parameter WTCHPT_TRAP = 2;
parameter SINGLE_STEP_TRAP = 3;

parameter PERFORMANCE_COUNTER = 0;
parameter COUNT_CYCLECOUNTERS = 1;
parameter COUNT_EVENTCOUNTERS = 2;
parameter WIDTH_CYCLECOUNTER = 36;
parameter WIDTH_EVENTCOUNTER = 36;


input wire		clk1;
input wire		reset;
input wire		clk_en;
input wire	[3:0]	addr;
input wire		write;
input wire	[17:0]	data_in;
output reg	[17:0]	data_out;
input wire		inc_window;
input wire		dec_window;
input wire		set_cc;
input wire		clear_cc;
input wire		do_wb_cc;
input wire		cc_result;
input wire		set_mm;
input wire		clear_mm;
input wire		set_intr_enable;	// sbits s2
input wire		clear_intr_enable;	// cbits s2
input wire	[17:0]	product_hi;
input wire		multiply;
output reg	[6:0]	window;
output	wire	cc;
output reg		mm;
output reg		intr_enable;
input wire		run_intr;
output reg	[17:0]	pc_intr;
output reg	[9:0]	pc_trap;

input wire	[`ADDR_WIDTH-1:0]	dbg_pc_next;//Breakpoint support
output wire		dbg_trap;
output wire [7:0]	dbg_trap_no;
input wire	[`ADDR_WIDTH-1:0]	dbg_mem_addr;
input wire [1:0]	dbg_mem_we;
input wire [1:0]	dbg_mem_re;
input wire			dbg_delayed_ir;

// Performance counter event lines.
input wire is_mem;
input wire is_store;
input wire stall;
input wire is_nop;
input wire ir_return;
input wire evcnt_ext0;
input wire evcnt_ext1;

reg		cc_reg;

reg	[17:0]	mul;
reg		write_window;
reg		write_cc;
reg		write_mm;
reg		write_intr_enable;
reg		write_mul;
reg		write_iv;
reg		write_tr;
reg		write_dbg_idx;
reg		write_dbg_data;


// SFR_STATUS - Register Basis Adresse
always @(posedge clk1 )
	if (reset)
		window <= 6'h00;
	else if (clk_en) begin
		if (inc_window)
			window <= window + 1;
		else if (dec_window)
			window <= window - 1;
		else if (write_window)
			window <= data_in[6:0];
	end

// SFR_CC
always @(posedge clk1 )
	if (reset)
		cc_reg <= 1'b0;
	else if(clk_en) begin
		casex	({clear_cc, set_cc, write_cc, do_wb_cc})
			4'b1000:	cc_reg <= 1'b0;
			4'b0100:	cc_reg <= 1'b1;
			4'b0010:	cc_reg <= data_in[0];
			4'b0001:	cc_reg <= cc_result;
			default:	cc_reg <= cc_reg;
	  	endcase
	end


assign cc = cc_reg;

// SFR_STATUS - MM Bit
always @(posedge clk1 )
	if (reset)
		mm <= 1'b0;
	else if(clk_en) begin
		if (clear_mm)
			mm <= 1'b0;
		else if (set_mm)
			mm <= 1'b1;
		else if (write_mm)
			mm <= data_in[7];
	end

// SFR_STATUS - INT Bit (asynchron mit clear_intr_enable sofort ruecksetzen)

always @(posedge clk1)
	if (reset)
		intr_enable <= 1'b0;
	else if(clk_en) begin
		if (run_intr)
			intr_enable <= 1'b0;
		else if (set_intr_enable)
			intr_enable <= 1'b1;
		else if (clear_intr_enable)
			intr_enable <= 1'b0;
		else if (write_intr_enable)
			intr_enable <= data_in[8];
	end


// SFR_MUL - movi2s
always @(posedge clk1 )
	if (reset)
		mul <= 18'h00000;
	else if(clk_en) begin
		if (write_mul && multiply)
			mul <= product_hi;
		else if (write_mul && !multiply)
			mul <= data_in;
	end


// SFR_IV - movi2s
always @(posedge clk1)
	if (reset)
		pc_intr <= INTR_VECTOR;
	else if (write_iv && clk_en)
		pc_intr <= data_in;



// SFR_TR - movi2s
always @(posedge clk1 )
	if (reset)
		pc_trap <= RESET_VECTOR[17:8];
	else if (write_tr && clk_en)
		pc_trap <= data_in[17:8];



// HARDWARE DEBUGGING
wire [`ADDR_WIDTH-1:0] dbg_data_out;
wire [`ADDR_WIDTH-1:0]	dbg_idx_out;

genvar i;
`PACK_GEN

	parameter DBG_REG_COUNT = 2*COUNT_BREAKPOINTS + 2*COUNT_WATCHPOINTS + 2; // addr + config per reg + 1 info/test reg at 0 + trapped_mem_addr at 1
	parameter BRKPTS_BEGIN = 2;
	parameter WTCHPTS_BEGIN = BRKPTS_BEGIN + COUNT_BREAKPOINTS;
	parameter BRKPTS_CFG_BEGIN = WTCHPTS_BEGIN + COUNT_WATCHPOINTS;
	parameter WTCHPTS_CFG_BEGIN = BRKPTS_CFG_BEGIN + COUNT_BREAKPOINTS;

generate
if (HARDWARE_DEBUGGING) begin
	reg [17:0]	dbg_idx;
	reg 		dbg_enable;
	reg			dbg_step_next;
	
	assign dbg_idx_out = dbg_idx;
	
	reg		[`ADDR_WIDTH-1:0]	brkpts	[0:COUNT_BREAKPOINTS-1];
	reg		[`ADDR_WIDTH-1:0]	wtchpts	[0:COUNT_WATCHPOINTS-1];
	reg		[COUNT_BREAKPOINTS-1:0]		brkpts_config;
	reg		[4:0]	wtchpts_config	[0:COUNT_WATCHPOINTS-1];
	
	wire [7:0] dbg_brk_count;
	wire [7:0] dbg_wtch_count;
	wire [17:0] dbg_info;
	wire [17:0] dbg_trapped_at_masked;
	wire [17:0] trapped_at;
	wire [17:0] trapped_mem_addr;
	assign dbg_brk_count = COUNT_BREAKPOINTS;
	assign dbg_wtch_count = COUNT_WATCHPOINTS;
	assign dbg_info = (dbg_idx == 0)? {1'b1, dbg_wtch_count, dbg_enable, dbg_brk_count} : 18'b0;
	assign dbg_trapped_at_masked = (dbg_idx == 1)? trapped_mem_addr : 18'b0;
	
	
	wire		[`ADDR_WIDTH *COUNT_BREAKPOINTS-1:0]	brkpts_packed;
	//wire		[1 *COUNT_BREAKPOINTS-1:0]				brkpts_config_packed;
	wire		[`ADDR_WIDTH *COUNT_WATCHPOINTS-1:0]	wtchpts_packed;
	wire		[5 *COUNT_WATCHPOINTS-1:0]				wtchpts_config_packed;

	`PACK_ARRAY(`ADDR_WIDTH,COUNT_BREAKPOINTS, brkpts, brkpts_packed, PK1) `PACK_ARRAY_END
	`PACK_ARRAY(`ADDR_WIDTH,COUNT_WATCHPOINTS, wtchpts, wtchpts_packed, PK3) `PACK_ARRAY_END
	`PACK_ARRAY(5,COUNT_WATCHPOINTS, wtchpts_config, wtchpts_config_packed, PK4) `PACK_ARRAY_END

	breakpoint_mgr #(
		.COUNT_BREAKPOINTS(COUNT_BREAKPOINTS),
		.COUNT_WATCHPOINTS(COUNT_WATCHPOINTS),
		.BRKPT_TRAP(BRKPT_TRAP),
		.WTCHPT_TRAP(WTCHPT_TRAP),
		.SINGLE_STEP_TRAP(SINGLE_STEP_TRAP)
	) brkpt_sfr (
		.clk(clk1),
		.reset(reset),
		.clk_en(clk_en),
		.pc_next(dbg_pc_next),
		.mem_addr(dbg_mem_addr),
		.mem_we(dbg_mem_we),
		.mem_re(dbg_mem_re),
		.brkpts_packed(brkpts_packed),
		.wtchpts_packed(wtchpts_packed),
		.brkpts_config(brkpts_config),
		.wtchpts_config_packed(wtchpts_config_packed),
		.enable(dbg_enable),
		.spring_trap(dbg_trap),
		.trap_no(dbg_trap_no),
		.trapped_at(trapped_at),
		.step_next(dbg_step_next),
		.reg_window(window),
		.delayed_ir(dbg_delayed_ir),
		.trapped_mem_addr(trapped_mem_addr)
	);
	
	for (i=0; i <COUNT_BREAKPOINTS; i=i+1) begin: breakpoint_write_access
		always @(posedge clk1) begin	
			if (reset) begin
				brkpts[i] <= 0;
				brkpts_config[i] <= 0;
			end else if (clk_en && write_dbg_data) begin
				if (dbg_idx == BRKPTS_BEGIN + i) // writing breakpoint i
					brkpts[i] <= data_in;
				if (dbg_idx == BRKPTS_CFG_BEGIN + i) // writing breakpoint config i
					brkpts_config[i] <= data_in[0];
			end
		end
	end
	
	for (i=0; i <COUNT_WATCHPOINTS; i=i+1) begin: watchpoint_write_access
		always @(posedge clk1) begin
			if (reset) begin
				wtchpts[i] <= 0;
				wtchpts_config[i] <= 0;
			end else if (clk_en && write_dbg_data) begin
				if (dbg_idx == WTCHPTS_BEGIN + i) // writing watchpoint i
					wtchpts[i] <= data_in;
				if (dbg_idx == WTCHPTS_CFG_BEGIN + i) // writing watchpoint config i
					wtchpts_config[i] <= data_in;
			end
		end
	end
	
	wire [DBG_REG_COUNT*18-1:0] dbg_all_re;
	
	assign dbg_all_re[17:0] = dbg_info;
	assign dbg_all_re[35:18]= dbg_trapped_at_masked;
	
	for (i=0; i <COUNT_BREAKPOINTS; i=i+1) begin: breakpoint_read_access
		assign dbg_all_re[(BRKPTS_BEGIN + i+1)*18-1:(BRKPTS_BEGIN +i)*18] = (dbg_idx == BRKPTS_BEGIN + i)? brkpts[i] : 18'h0;
		assign dbg_all_re[(BRKPTS_CFG_BEGIN + i+1)*18-1:(BRKPTS_CFG_BEGIN +i)*18] = (dbg_idx == BRKPTS_CFG_BEGIN + i)? {17'h0, brkpts_config[i]} : 18'h0;
	end
	
	for (i=0; i <COUNT_WATCHPOINTS; i=i+1) begin: watchpoint_read_access
		assign dbg_all_re[(WTCHPTS_BEGIN + i+1)*18-1:(WTCHPTS_BEGIN +i)*18] = (dbg_idx == WTCHPTS_BEGIN + i)? wtchpts[i] : 18'h0;
		assign dbg_all_re[(WTCHPTS_CFG_BEGIN + i+1)*18-1:(WTCHPTS_CFG_BEGIN +i)*18] = (dbg_idx == WTCHPTS_CFG_BEGIN + i)? {13'b0, wtchpts_config[i]} : 18'h0;
	end
	
	always @(posedge clk1) begin
		if (reset) begin
			dbg_enable <= 0;
			dbg_idx <= 0;
			dbg_step_next <= 0;
		end else if(clk_en) begin
			if (dbg_trap) begin
				dbg_step_next <= 0;
			end else if (dbg_idx == 0 && write_dbg_data) begin
				dbg_enable <= data_in[0] | data_in[8];
				dbg_step_next <= data_in[1];
			end
			
			if (write_dbg_idx)
				dbg_idx <= data_in;
				
		end
	end
	
	orgate #(
		.OR_WIDTH(DBG_REG_COUNT),
		.BUS_WIDTH(18)
	) read_or (
		.a(dbg_all_re),
		.y(dbg_data_out)
	);

end else begin :zero
	assign dbg_idx_out = 0;
	assign dbg_data_out = 0;
	assign dbg_trap = 0;
	assign dbg_trap_no = 0;
end
endgenerate

// Performance counter
reg [`ADDR_WIDTH-1:0] pcnt_idx;
wire [`ADDR_WIDTH-1:0] pcnt_data_out;
reg write_pcnt_idx;

generate
if (PERFORMANCE_COUNTER) begin
	wire pcnt_write_dat;
	assign pcnt_write_dat = write && (addr == `SFR_PCNT_DAT);

	always @(posedge clk1)
		if(reset)
			pcnt_idx <= 18'h00000;
		else if(write_pcnt_idx)
			pcnt_idx <= data_in;
		else if(pcnt_idx[17:16])
			pcnt_idx[17:16] <= 2'h0;

	perfcounter #(
		.COUNT_EVENTCOUNTERS(COUNT_EVENTCOUNTERS),
		.WIDTH_CYCLECOUNTER(WIDTH_CYCLECOUNTER),
		.WIDTH_EVENTCOUNTER(WIDTH_EVENTCOUNTER)
	) pcnt (
		.clk(clk1),
		.reset(reset),
		.reg_write(pcnt_write_dat),
		.reg_idx(pcnt_idx),
		.reg_data_out(pcnt_data_out),
		.reg_data_in(data_in),
		.intr_start(run_intr),
		.intr_end(ir_return),
		.data_read(is_mem & !is_store),
		.data_write(is_mem & is_store),
		.instr_read(1'h0),
		.instr_hit(1'h0),
		.instr_miss(1'h0),
		.pipeline_stall(stall),
		.is_nop(is_nop),
		.ext0(evcnt_ext0),
		.ext1(evcnt_ext1)
	);
end else begin
	assign pcnt_data_out = 0;
	
	/*always @(*)
		pcnt_idx <= 0;*/
end
endgenerate

// movs2i vom SFR_xx	
always @(*)
	case (addr)
		`SFR_STATUS	: data_out = {9'h0, intr_enable, mm, window};
		`SFR_MUL	: data_out = mul;
		`SFR_CC		: data_out = {17'h00000, cc_reg};
		`SFR_IV		: data_out = pc_intr;
		`SFR_TR		: data_out = {pc_trap, 8'h00};
		`SFR_DBG_IDX: data_out = dbg_idx_out;
		`SFR_DBG_DAT: data_out = dbg_data_out;
		`SFR_PCNT_IDX: data_out = pcnt_idx;
		`SFR_PCNT_DAT: data_out = pcnt_data_out;
		default	: data_out = 0;
	endcase

// movi2s zum SFR_xx
always @(addr or write)
begin
	write_window	= 1'b0;
	write_cc		= 1'b0;
	write_mm		= 1'b0;
	write_intr_enable	= 1'b0;
	write_mul		= 1'b0;
	write_iv		= 1'b0;
	write_tr		= 1'b0;
	write_dbg_idx	= 1'b0;
	write_dbg_data	= 1'b0;
	write_pcnt_idx	= 1'b0;
	
	case (addr)
		`SFR_STATUS	: begin
					write_window		= write;
					write_mm		= write;
					write_intr_enable	= write;
				  end
		`SFR_MUL	: write_mul	= write;
		`SFR_CC		: write_cc	= write;
		`SFR_IV		: write_iv	= write;
		`SFR_TR		: write_tr	= write;
		`SFR_DBG_IDX: write_dbg_idx = write;
		`SFR_DBG_DAT: write_dbg_data = write;
		`SFR_PCNT_IDX: write_pcnt_idx = write;
	endcase
end

endmodule
