#!/usr/bin/tclsh

#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

proc fillString {s} {
	set fill {}
	set i 0
	for {set i 0} {$i < [expr 64 - [string length $s]]} {incr i} {
		set fill 0$fill
	}
	return $fill$s
}

proc printinit {init_data_list init_parit_list ipath typ bl} {
	if {$typ == "sim"} {
		puts stdout "// synopsys translate_off\n"
		set i 0
		foreach init_data $init_data_list {
			puts stdout [format "defparam\tDMARAM%02X.INIT_%02X = 256'h%s;" $bl $i $init_data]
			incr i
		}
		set i 0
		foreach init_parit $init_parit_list {
			puts stdout [format "defparam\tDMARAM%02X.INITP_%02X = 256'h%s;" $bl $i $init_parit]
			incr i
		}
		puts stdout "// synopsys translate_on\n"
	} else {
		set i 0
		foreach init_data $init_data_list {
			puts stdout [format "INST\t$ipath/DMARAM%02X\tINIT_%02X = %s;" $bl $i $init_data]
			incr i
		}
		set i 0
		foreach init_parit $init_parit_list {
			puts stdout [format "INST\t$ipath/DMARAM%02X\tINITP_%02X = %s;" $bl $i $init_parit]
			incr i
		}
	}
}

		

proc readfile {fileId ipath typ} {
    set bl 0
	while {![eof $fileId]} {
	    set init_data {}
	    set init_parit {}
	    set init_data_list {}
	    set init_parit_list {}
	    set i 0

	    while {![eof $fileId]} {
#		puts stdout [format "%03x\n" $i]
		    if {$i > 1023} break
		    set zeile [gets $fileId]
	            if {[string length $zeile] < 5} {break} else {
#		    puts stdout [format "%03x\n" $i]
		    incr i
		    set opcode 0x0$zeile
		    set opcode_low [expr $opcode & 0xffff]
		    set opcode_low_hex [format "%04x" $opcode_low]
		    set init_data $opcode_low_hex$init_data
		    if {($i % 2) == 0} {
			incr opcode_high [expr ($opcode & 0x30000) >> 14]
			set opcode_high_hex [format "%1x" $opcode_high]
			set init_parit $opcode_high_hex$init_parit
		    } else {
			set opcode_high [expr ($opcode & 0x30000) >> 16]
		    }
		    if {[string length $init_data] >= 64} {
			lappend init_data_list $init_data
			set init_data {}
		    }
		    if {[string length $init_parit] >= 64} {
			lappend init_parit_list $init_parit
			set init_parit {}
		    }
	        }
	    }

	    # Paritaetsbits fuer die letzte Speicherzelle noch erzeugen,
	    # wenn i nicht durch 2 teilbar war.
	    if {($i % 2) != 0} {
		set opcode_high_hex [format "%1x" $opcode_high]
		set init_parit $opcode_high_hex$init_parit
	    }

	    if {[string length $init_data] != 0} {
		lappend init_data_list [fillString $init_data]
	    }
	    if {[string length $init_parit] != 0} {
		lappend init_parit_list [fillString $init_parit]
	    }

	    printinit $init_data_list $init_parit_list $ipath $typ $bl
	    incr bl
	}
}

# Erstellen der DMA Speicherinhalte fuer den SpartanMC aus dem *.sph Format.
#
# Aufruf fuer die Synthese:   initdmaram.tcl usbcore0/u1/u2 testdma.sph >> spartanmc.ucf
# Aufruf fuer die Simulation: initdmaram.tcl -sim usbcore0/u1/u2 testdma.sph > dmaramsim.v
#
if {$argc < 2} {
	puts stdout "$argv0 \[-sim\] <instanzenpfad> <inputfile>\n"
	exit 1
}

if {$argc == 2} {
	set typ ucf
	set ipath [lindex $argv 0]
	set fileName [lindex $argv 1]
} else {
	if {$argc == 3} {
		if {[lindex $argv 0] == "-sim"} {
			set typ sim
			set ipath [lindex $argv 1]
			set fileName [lindex $argv 2]
		} else {
			puts stdout "invalid option: [lindex $argv 0]\n"
			exit 2
		}
	} else {
		puts stdout "too much parameters!\n"
		exit 3
	}
}

if {[catch {set fileId [open $fileName r]} error]} {
	puts stdout $error
} else {
	readfile $fileId $ipath $typ
	close $fileId
}
