#!/usr/bin/tclsh

#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

proc fillString {s} {
	set fill {}
	set i 0
	for {set i 0} {$i < [expr 64 - [string length $s]]} {incr i} {
		set fill 0$fill
	}
	return $fill$s
}

proc printinit {init_data_list_low init_data_list_high
		init_parit_list_low init_parit_list_high typ bl} {
	if {$typ == "sim"} {
		puts stdout "// synopsys translate_off\n"
		set i 0
		foreach init_data_low $init_data_list_low {
			puts stdout [format "defparam\tMEM_BL\[%0X\].DPRAMA.INIT_%02X  = 256'h%s;" $bl $i $init_data_low]
			incr i
		}
		set i 0
		foreach init_parit_low $init_parit_list_low {
			puts stdout [format "defparam\tMEM_BL\[%0X\].DPRAMA.INITP_%02X = 256'h%s;" $bl $i $init_parit_low]
			incr i
		}
		set i 0
		foreach init_data_high $init_data_list_high {
			puts stdout [format "defparam\tMEM_BL\[%0X\].DPRAMB.INIT_%02X  = 256'h%s;" $bl $i $init_data_high]
			incr i
		}
		set i 0
		foreach init_parit_high $init_parit_list_high {
			puts stdout [format "defparam\tMEM_BL\[%0X\].DPRAMB.INITP_%02X = 256'h%s;" $bl $i $init_parit_high]
			incr i
		}
		puts stdout "// synopsys translate_on\n"
	} else {
		set i 0
		foreach init_data_low $init_data_list_low {
			puts stdout [format "INST\tspartanmc0/ifid/memory/MEM_BL\[%0X\].DPRAMA\tINIT_%02X  = %s;" $bl $i $init_data_low]
			incr i
		}
		set i 0
		foreach init_parit_low $init_parit_list_low {
			puts stdout [format "INST\tspartanmc0/ifid/memory/MEM_BL\[%0X\].DPRAMA\tINITP_%02X = %s;" $bl $i $init_parit_low]
			incr i
		}
		set i 0
		foreach init_data_high $init_data_list_high {
			puts stdout [format "INST\tspartanmc0/ifid/memory/MEM_BL\[%0X\].DPRAMB\tINIT_%02X  = %s;" $bl $i $init_data_high]
			incr i
		}
		set i 0
		foreach init_parit_high $init_parit_list_high {
			puts stdout [format "INST\tspartanmc0/ifid/memory/MEM_BL\[%0X\].DPRAMB\tINITP_%02X = %s;" $bl $i $init_parit_high]
			incr i
		}
	}
}

		

proc readfile {fileId typ} {
    set bl 0
	while {![eof $fileId]} {
	    set opcode_high_low 0
	    set opcode_high_high 0
	    set init_data_low {}
	    set init_data_high {}
	    set init_parit_low {}
	    set init_parit_high {}
	    set init_data_list_low {}
	    set init_data_list_high {}
	    set init_parit_list_low {}
	    set init_parit_list_high {}
	    set i 0

	    while {![eof $fileId]} {
#		puts stdout [format "%03x\n" $i]
		    if {$i > 2047} break
		    set zeile [gets $fileId]
	            if {[string length $zeile] == 0} {break} else {
#		    puts stdout [format "%03x\n" $i]
		    incr i
		    set opcode 0x0$zeile
#		    puts stdout "$i: $opcode"
		    set opcode_low_low [expr $opcode & 0x00ff]
		    set opcode_low_hex_low [format "%02x" $opcode_low_low]
		    set opcode_low_high [expr ($opcode & 0x1fe00) >> 9]
		    set opcode_low_hex_high [format "%02x" $opcode_low_high]
		    set init_data_low $opcode_low_hex_low$init_data_low
		    set init_data_high $opcode_low_hex_high$init_data_high
		    # Pruefbits aller 4 Opcodes erzeugen
		    if {($i % 4) == 0} {
			incr opcode_high_low  [expr ($opcode & 0x0100 ) >> 5]
			incr opcode_high_high [expr ($opcode & 0x20000) >> 14]
			set opcode_high_hex_low [format "%1x" $opcode_high_low]
			set opcode_high_hex_high [format "%1x" $opcode_high_high]
			set init_parit_low $opcode_high_hex_low$init_parit_low
			set init_parit_high $opcode_high_hex_high$init_parit_high
#			puts stdout "$opcode_high_hex_high  $opcode_high_hex_low"
			set opcode_high_low 0
			set opcode_high_high 0
		    } else {
			incr opcode_high_low  [expr ($opcode & 0x0100) >> (9-($i % 4))]
			incr opcode_high_high [expr ($opcode & 0x20000) >> (18-($i % 4))]
		    }

		   if {[string length $init_data_low] >= 64} {
			lappend init_data_list_low $init_data_low
			set init_data_low {}
		    }
		    if {[string length $init_data_high] >= 64} {
			lappend init_data_list_high $init_data_high
			set init_data_high {}
		    }
		    if {[string length $init_parit_low] >= 64} {
			lappend init_parit_list_low $init_parit_low
			set init_parit_low {}
		    }
		    if {[string length $init_parit_high] >= 64} {
			lappend init_parit_list_high $init_parit_high
			set init_parit_high {}
		    }
	        }
	    }

	    # Paritaetsbits fuer die letzten Speicherzellen noch erzeugen,
	    # wenn i nicht durch 4 teilbar war.
	    if {($i % 4) != 0} {
#		puts stdout "$opcode_high_high  $opcode_high_low"
		set opcode_high_hex_low [format "%1x" $opcode_high_low]
		set opcode_high_hex_high [format "%1x" $opcode_high_high]
		set init_parit_low $opcode_high_hex_low$init_parit_low
		set init_parit_high $opcode_high_hex_high$init_parit_high
#		puts stdout "$opcode_high_hex_high  $opcode_high_hex_low"
	    }

	    if {[string length $init_data_low] != 0} {
		lappend init_data_list_low [fillString $init_data_low]
	    }
	    if {[string length $init_data_high] != 0} {
		lappend init_data_list_high [fillString $init_data_high]
	    }
	    if {[string length $init_parit_low] != 0} {
		lappend init_parit_list_low [fillString $init_parit_low]
	    }
	    if {[string length $init_parit_high] != 0} {
		lappend init_parit_list_high [fillString $init_parit_high]
	    }

	    printinit $init_data_list_low $init_data_list_high \
		      $init_parit_list_low $init_parit_list_high $typ $bl

	    incr bl
	}
}

if {$argc < 1} {
	puts stdout "$argv0 \[-sim\] <inputfile>\n"
	exit 1
}

if {$argc == 1} {
	set typ ucf
	set fileName [lindex $argv 0]
} else {
	if {$argc == 2} {
		if {[lindex $argv 0] == "-sim"} {
			set typ sim
			set fileName [lindex $argv 1]
		} else {
			puts stdout "invalid option: [lindex $argv 0]\n"
			exit 2
		}
	} else {
		puts stdout "too much parameters!\n"
		exit 3
	}
}

if {[catch {set fileId [open $fileName r]} error]} {
	puts stdout $error
} else {
	readfile $fileId $typ
	close $fileId
}
