/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none

module connector_tb;

	// Inputs
	reg clk_peri;
	reg [17:0] master_do_peri;
	reg [9:0] master_addr_peri;
	reg master_access_peri;
	reg master_wr_peri;
	
	reg [17:0] slave_do_peri;
	reg [9:0]slave_addr_peri;
	reg slave_access_peri;
	reg slave_wr_peri;
	
	reg reset;
	wire read_enable;

	// Outputs
	wire [17:0] master_di_peri;
	wire [17:0] slave_di_peri;
	wire [17:0] data_to_slave;
	wire [8:0] status;

	// Instantiate the Unit Under Test (UUT)
	core_connector uut_master (
		.clk_peri(clk_peri), 
		.do_peri(master_do_peri), 
		.di_peri(master_di_peri), 
		.addr_peri(master_addr_peri), 
		.access_peri(master_access_peri), 
		.wr_peri(master_wr_peri), 
		.reset(reset), 
		.read_enable(read_enable), 
		.data_to_slave(data_to_slave), 
		.status(status)
	);

	core_connector uut_slave (
		.clk_peri(clk_peri), 
		.do_peri(slave_do_peri), 
		.di_peri(slave_di_peri), 
		.addr_peri(slave_addr_peri), 
		.access_peri(slave_access_peri), 
		.wr_peri(slave_wr_peri), 
		.reset(reset), 
		.read_enable(read_enable), 
		.data_from_master(data_to_slave), 
		.status(status)
	);

	integer i;

	initial begin
		// Initialize Inputs
		clk_peri = 0;
		
		master_do_peri = 0;
		master_addr_peri = 0;
		master_access_peri = 0;
		master_wr_peri = 0;
		
		slave_do_peri = 0;
		slave_addr_peri = 0;
		slave_access_peri = 0;
		slave_wr_peri = 0;
		
		reset = 0;
		i = 0;

		// Wait 100 ns for global reset to finish
		#100;
		
		// Add stimulus here

		
		@(negedge clk_peri); reset <= 1;
		@(negedge clk_peri);
		@(negedge clk_peri);
		@(negedge clk_peri);
		@(negedge clk_peri); reset <= 0;
		
		for(i=0;i<127;i=i+1) begin
			@(negedge clk_peri);master_do_peri <= i; 		master_addr_peri <= 1;	master_access_peri <= 1; master_wr_peri = 1;
			//@(negedge clk_peri);master_do_peri <= 18'h0; master_addr_peri <= 0;	master_access_peri <= 0; master_wr_peri = 0;
		end
		@(negedge clk_peri);master_do_peri <= 18'h0; master_addr_peri <= 0;	master_access_peri <= 0; master_wr_peri = 0;

		@(negedge clk_peri);
		@(negedge clk_peri);
		@(negedge clk_peri);
		@(negedge clk_peri); 
		
		for(i=0;i<127;i=i+1) begin
			@(negedge clk_peri);slave_do_peri <= i; 		slave_addr_peri <= 1;	slave_access_peri <= 1; 
			//@(negedge clk_peri);slave_do_peri <= 18'h0;  slave_addr_peri <= 0;	slave_access_peri <= 0; 
		end
	
	end
    
   always #25 clk_peri=~clk_peri;

	
endmodule

