/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// TU-Dresden Chair for Embedded Systems
//////////////////////////////////////////////////////////////////////////////////
module core_connector_duplex
#(
        // SPMC Interface
        parameter BASE_ADR               = 10'h0,
        parameter FIFO_DEPTH             = 'd128,
        parameter FIFO_WIDTH             = 'd18,
        parameter SLAVE_OFFSET           = 'd8
)
(
        //*** Connections with SpartanMC master ***
        input   wire            clk_peri,	//Systemtakt
        input   wire [17:0]     do_peri,	//Datenbus von MC
        output  wire [17:0]     di_peri,	//Datenbus zu MC
        input   wire [9:0]      addr_peri,	//Adressbus von MC
        input   wire            access_peri,	//Signale fuer Peripheriezugriffe
        input   wire            wr_peri,	//Schreibsignal
        input   wire            reset,		//Reset-Signal

		// Connections with extern slave
		input wire                    read_enable_from_slave,
		output wire [FIFO_WIDTH-1:0]  data_to_slave,
		output wire [17:0]            used_entries_to_slave,
		output wire [17:0]            free_entries_to_slave,
		
		// Connections with extern master
		output wire 				read_enable_to_master,
		input wire [FIFO_WIDTH-1:0] data_from_master,
		input wire [17:0]           used_entries_from_master,
		input wire [17:0]           free_entries_from_master
);
/////////////////////////////////////////////////////////////////////////////////////
//
// Local Functions and Parameters
//

parameter
	ALL_PORTS				= 3,
	DECODSIM_DEB			= "NO";

wire[17:0] di_peri_slave;
wire[17:0] di_peri_master;


core_connector_master #(
	.BASE_ADR(		BASE_ADR	),
	.FIFO_DEPTH(	FIFO_DEPTH	),
	.FIFO_WIDTH(	FIFO_WIDTH	)
	)
	master	(
		.clk_peri(		clk_peri				),
		.do_peri(		do_peri					),
		.di_peri(		di_peri_master			),
		.addr_peri(		addr_peri				),
		.access_peri(	access_peri				),
		.wr_peri(		wr_peri					),
		.reset(			reset					),
		.read_enable(	read_enable_from_slave	),
		.data_to_slave(	data_to_slave			),
		.free_entries(	free_entries_to_slave	),
		.used_entries(	used_entries_to_slave	)		
	);


// Addressdekoder fuer alle master Geraeteregister
wire 						master_select;
wire	[ALL_PORTS-1:0] 	master_reg_select;
wire	[ALL_PORTS-1:0] 	master_reg_read;
wire	[ALL_PORTS-1:0] 	master_reg_write;

reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(			reset				),
		.clk_peri(		clk_peri			),
		.addr_peri(		addr_peri			),
		.access_peri(	access_peri			),
		.wr_peri(		wr_peri				),

		.peri_select(	master_select		),
		.reg_select(	master_reg_select	),
		.reg_read(		master_reg_read		),
		.reg_write(		master_reg_write	)
			);

core_connector_slave #(
	.BASE_ADR(		BASE_ADR + SLAVE_OFFSET	),
	.FIFO_DEPTH(	FIFO_DEPTH				),
	.FIFO_WIDTH(	FIFO_WIDTH				)
	) 
	slave (
		.clk_peri(			clk_peri					),
		.do_peri(			do_peri						),
		.di_peri(			di_peri_slave				),
		.addr_peri(			addr_peri					),
		.access_peri(		access_peri					),
		.wr_peri(			wr_peri						),
		.reset(				reset						),
		.read_enable(		read_enable_to_master		),
		.data_from_master(	data_from_master			),
		.free_entries(		free_entries_from_master	),
		.used_entries(		used_entries_from_master	)		
	);


// Addressdekoder fuer alle slave Geraeteregister
wire 						slave_select;
wire	[ALL_PORTS-1:0] 	slave_reg_select;
wire	[ALL_PORTS-1:0] 	slave_reg_read;
wire	[ALL_PORTS-1:0] 	slave_reg_write;

reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS				),
		.BASE_ADR(	BASE_ADR + SLAVE_OFFSET	),
		.SIM_DEBUG(	DECODSIM_DEB			)
			)
decod2			(
		.reset(			reset				),
		.clk_peri(		clk_peri			),
		.addr_peri(		addr_peri			),
		.access_peri(	access_peri			),
		.wr_peri(		wr_peri				),

		.peri_select(	slave_select		),
		.reg_select(	slave_reg_select	),
		.reg_read(		slave_reg_read		),
		.reg_write(		slave_reg_write		)
			);

assign di_peri [17:0] = (slave_reg_read!=0) ? di_peri_slave :
						(master_reg_read!=0) ? di_peri_master : 18'b0;

endmodule
