/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`timescale 1ns / 1ps


module global_mem_master_tb;

	parameter CONNECTIONS 			= 3;
	parameter CONNECTIONS_BARRIER	= 0;
	parameter CACHE					= "FALSE";
	parameter RAMBLOCKS				= 4;
	parameter BASE_ADDR				= 18'h10000;
	
//inputs
	reg											clk;
	reg											reset;

	reg		[10*CONNECTIONS-1:0]				mem_addr_block;
    reg		[8*CONNECTIONS-1:0]					mem_addr_high;
	reg		[18*CONNECTIONS-1:0]				mem_di;
	reg		[CONNECTIONS-1:0]					mem_wr_low_in;

	reg		[CONNECTIONS-1:0]					mem_wr_high_in;
	reg		[CONNECTIONS-1:0]					mem_access;
	

//output signals
	wire		[18*CONNECTIONS-1:0]			mem_do;
	wire		[17:0]							mem_do_array[CONNECTIONS-1:0];
	wire		[CONNECTIONS-1:0]				mem_busy_out;
	wire 										wait_done;


	// Instantiate the Unit Under Test (UUT)
	global_mem_master 
		#(
			.CONNECTIONS(CONNECTIONS),
			.CACHE(CACHE),
			.RAMBLOCKS(RAMBLOCKS),
			.BASE_ADDR(BASE_ADDR)
		)
	uut (
		.clk(clk), 
		.reset(reset), 
		.mem_addr_block(mem_addr_block),
		.mem_addr_high(mem_addr_high), 
		.mem_di(mem_di), 
		.mem_wr_low_in(mem_wr_low_in), 
		.mem_wr_high_in(mem_wr_high_in), 
		.mem_access(mem_access), 
		.mem_do(mem_do),
		.mem_busy_out(mem_busy_out)
	);

	assign mem_do_array[0]=mem_do[17:0];
	assign mem_do_array[1]=mem_do[35:18];
	assign mem_do_array[2]=mem_do[53:36];

	initial begin
		// Initialize Inputs
		clk = 0;
		reset= 1;
		mem_addr_block= 0;
		mem_addr_high= 0;
		mem_di= 0;
		mem_wr_low_in= 0;
		mem_wr_high_in= 0;
		mem_access= 0;

		// Wait 100 ns for global reset to finish
		#120;

		reset = 0;
		
		//write to the cache		
	
		@(posedge clk)
		mem_addr_block = {10'h1,10'h2,10'h3};
		mem_addr_high = {3{8'h40}};
		mem_di = {18'h1,18'h2,18'h3};
		mem_wr_low_in = 3'b111;
		mem_wr_high_in = 3'b111;
		mem_access = 3'b111;
		
		@(posedge clk)
		mem_addr_block= 0;
		mem_addr_high= 0;
		mem_di= 0;
		mem_wr_low_in= 0;
		mem_wr_high_in= 0;
		mem_access= 0;
		@(negedge clk)
		if (!(mem_busy_out[CONNECTIONS-1:0] == 3'b110)) begin
			$error("Error: Mem_busy not correctly set! expecting 110 but got %b", mem_busy_out[CONNECTIONS-1:0]);
		end
		@(negedge clk)
      	if (!(mem_busy_out[CONNECTIONS-1:0] == 3'b100)) begin
			$error("Error: Mem_busy not correctly set! expecting 100 but got %b", mem_busy_out[CONNECTIONS-1:0]);
		end
		@(negedge clk)
		if (!(mem_busy_out[CONNECTIONS-1:0] == 3'b000)) begin
			$error("Error: Mem_busy not correctly set! expecting 000 but got %b", mem_busy_out[CONNECTIONS-1:0]);
		end

		//read the written


		@(posedge clk)
		mem_addr_block = {10'h3,10'h1,10'h2};
		mem_addr_high = {3{8'h40}};
		mem_di = 0;
		mem_wr_low_in = 3'b000;
		mem_wr_high_in = 3'b000;
		mem_access = 3'b111;

		@(posedge clk)
		mem_addr_block = {10'h3,10'h1,10'h2};
		mem_addr_high = {3{8'h40}};
		mem_di = 0;
		mem_wr_low_in = 3'b000;
		mem_wr_high_in = 3'b000;
		mem_access = 3'b001;
	
		@(negedge clk)
		if (!(mem_do[17:0] == 3'h2)) begin
			$error("Error: global mem should return 2 but received %x",mem_do[17:0]);
		end
		if (!(mem_do[53:18] == 0) && !(mem_busy_out[CONNECTIONS-1:0] == 3'b110)) begin
			$error("Error: mem_do[53:18] should be silent while busy but returns %x",mem_do[53:18]);
		end
		if (!(mem_busy_out[CONNECTIONS-1:0] == 3'b110)) begin
			$error("Error: Mem_busy not correctly set! expecting 110 but got %b", mem_busy_out[CONNECTIONS-1:0]);
		end

		@(posedge clk)
		mem_addr_block= 0;
		mem_addr_high= 0;
		mem_di= 0;
		mem_wr_low_in= 0;
		mem_wr_high_in= 0;
		mem_access= 0;
		


		//read again from cache
		@(negedge clk)
		if (!(mem_do[35:18] == 3'h1)) begin
			$error("Error: global mem should return 1 but received %x",mem_do[35:18]);
		end
		if (!(mem_do[17:0] == 0) && !(mem_do[53:36] == 0) && !(mem_busy_out[CONNECTIONS-1:0] == 3'b100)) begin
			$error("Error: mem_do[53:18] should be silent while busy but returns %x",mem_do[53:18]);
		end
      	if (!(mem_busy_out[CONNECTIONS-1:0] == 3'b101)) begin
			$error("Error: Mem_busy not correctly set! expecting 100 but got %b", mem_busy_out[CONNECTIONS-1:0]);
		end
		@(negedge clk)
		if (!(mem_do[53:36] == 3'h3)) begin
			$error("Error: global mem should return 3 but received %x",mem_do[53:36]);
		end
		if (!(mem_do[35:0] == 0) && !(mem_busy_out[CONNECTIONS-1:0] == 3'b000)) begin
			$error("Error: mem_do[35:0] should be silent while busy but returns %x",mem_do[35:0]);
		end
		if (!(mem_busy_out[CONNECTIONS-1:0] == 3'b001)) begin
			$error("Error: Mem_busy not correctly set! expecting 000 but got %b", mem_busy_out[CONNECTIONS-1:0]);
		end

		@(negedge clk)
		if (!(mem_do[17:0] == 3'h2)) begin
			$error("Error: global mem should return 2 but received %x",mem_do[17:0]);
		end
		if (!(mem_do[53:18] == 0) && !(mem_busy_out[CONNECTIONS-1:0] == 3'b110)) begin
			$error("Error: mem_do[53:18] should be silent while busy but returns %x",mem_do[53:18]);
		end
		if (!(mem_busy_out[CONNECTIONS-1:0] == 3'b000)) begin
			$error("Error: Mem_busy not correctly set! expecting 110 but got %b", mem_busy_out[CONNECTIONS-1:0]);
		end
		@(posedge clk)
		$stop();
	end

	always #5 clk=!clk;
      
endmodule

