/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`default_nettype none
`timescale 1ns / 1ps

module one_word_read_cache_tb;
	parameter CONNECTIONS 	= 2;
	parameter ADDR_LEN	= 1;

	reg							clk;
	reg							reset;

    reg		[ADDR_LEN*CONNECTIONS-1:0]			addr;
	reg		[18*CONNECTIONS-1:0]				data_in;
	reg		[CONNECTIONS-1:0]					wr_low;

	reg		[CONNECTIONS-1:0]					wr_high;
	reg		[CONNECTIONS-1:0]					enable_input;
	wire	[18*CONNECTIONS-1:0]				data_out;
	wire	[CONNECTIONS-1:0]					valid;


one_word_read_cache 
		#(
			.CONNECTIONS(CONNECTIONS),
			.ADDR_LEN(ADDR_LEN)
		)
	uut (
		.clk(clk), 
		.reset(reset), 
		.addr(addr),
		.data_in(data_in), 
		.wr_low(wr_low), 
		.wr_high(wr_high), 
		.enable_input(enable_input), 
		.data_out(data_out),
		.valid(valid)
	);

always #5 clk=!clk;

wire [17:0]	data_out_array [1:0];
assign data_out_array[0] = data_out[17:0];
assign data_out_array[1] = data_out[35:18];

initial begin
	reset=1;
	clk=0;
	addr=0;
	data_in=0;
	wr_low=0;
	wr_high=0;
	enable_input=0;

	#20
	//write to cache
	@(posedge clk)
	reset=0;
	addr=2'b10;
	data_in={18'h2,18'h1};
	wr_low=2'b11;
	wr_high=2'b11;
	enable_input=2'b11;
	
	@(negedge clk)
	if (!(valid == 2'b00)) begin
			$error("Error: valid should be 00 but is %b",valid);
	end
	
	//read cache
	@(posedge clk)
	reset=0;
	addr=2'b10;
	data_in={18'h4,18'h3};
	wr_low=2'b00;
	wr_high=2'b00;
	enable_input=2'b11;
	
	@(negedge clk)
	if (!(valid == 2'b00)) begin
			$error("Error: valid should be 00 but is %b",valid);
	end
	
	@(posedge clk)
	enable_input=2'b00;
	addr=2'b01;

	@(negedge clk)
	if (!(valid == 2'b11)) begin
			$error("Error: valid should be 11 but is %b",valid);
	end
	if (!(data_out_array[0] == 18'h3)) begin
			$error("Error: cache value at data_out[0] should be 3 but is %x", data_out_array[0]);
	end
	if (!(data_out_array[1] == 18'h4)) begin
			$error("Error: cache value at data_out[1] should be 4 but is %x", data_out_array[1]);
	end

	//check enable to now overwrite && valid
	@(posedge clk)
	data_in={18'h6,18'h5};
	enable_input=2'b00;
	
	@(negedge clk)
	@(negedge clk)
	if (!(data_out_array[0] == 18'h3)) begin
			$error("Error: cache value at data_out[0] should be 3 but is %x", data_out_array[0]);
	end
	if (!(data_out_array[1] == 18'h4)) begin
			$error("Error: cache value at data_out[1] should be 4 but is %x", data_out_array[1]);
	end

	//check valid
	@(posedge clk)
	addr=2'b01;
	@(negedge clk)
	@(negedge clk)
	if (!(valid == 2'b00)) begin
			$error("Error: valid should be 00 but is %b",valid);
	end
	
	//overwrite own cache
	@(posedge clk)
	addr=2'b10;
	data_in={18'h8,18'h7};
	wr_low=2'b11;
	wr_high=2'b11;
	enable_input=2'b11;

	@(negedge clk)
	if (!(valid == 2'b00)) begin
			$error("Error: valid should be 00 but is %b",valid);
	end
	//dont write again
	@(posedge clk)
	wr_low=2'b00;
	wr_high=2'b00;
	enable_input=2'b00;

	@(negedge clk)
	if (!(valid == 2'b11)) begin
			$error("Error: valid should be 11 but is %b",valid);
	end
	if (!(data_out_array[0] == 18'h7)) begin
			$error("Error: cache value at data_out[0] should be 7 but is %x", data_out_array[0]);
	end
	if (!(data_out_array[1] == 18'h8)) begin
			$error("Error: cache value at data_out[1] should be 8 but is %x", data_out_array[1]);
	end
	
	//overwrite other cache

	@(posedge clk)
	addr=2'b00;
	data_in={18'h10,18'h0};
	wr_low=2'b10;
	wr_high=2'b10;
	enable_input=2'b10;

	@(negedge clk)
	@(negedge clk)
	if (!(valid == 2'b00)) begin
		$error("Error: valid should be 00 but is %b",valid);
	end
	//stays invalid after write
	@(negedge clk)
	if (!(valid == 2'b00)) begin
		$error("Error: valid should be 00 but is %b",valid);
	end

	//both overwrite a cache
	
		//first fill read cache
	@(posedge clk)
	addr=2'b10;
	data_in={18'h12,18'h11};
	wr_low=2'b00;
	wr_high=2'b00;
	enable_input=2'b11;

		//then overwrite both caches	
	@(posedge clk)
	addr=2'b00;
	data_in={18'h14,18'h13};
	wr_low=2'b11;
	wr_high=2'b11;
	enable_input=2'b11;

		//still valid from previous write
	@(negedge clk)
	if (!(valid == 2'b11)) begin
			$error("Error: valid should be 11 but is %b",valid);
	end
		//invalid after write
	@(negedge clk)
	if (!(valid == 2'b00)) begin
			$error("Error: valid should be 00 but is %b",valid);
	end
		//stays invalid after write
	@(negedge clk)
	if (!(valid == 2'b00)) begin
			$error("Error: valid should be 00 but is %b",valid);
	end
	@(negedge clk)
	$stop();
end
endmodule
