/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Block RAM container                                        ////
////  Contains a Spartan-3 Block RAM primitive of the selected   ////
////  data width (9 bit, 18 bit or 36 bit).                      ////
////                                                             ////
////  Autor: Gregor Koch                                         ////
////                                                             ////
/////////////////////////////////////////////////////////////////////
`default_nettype none

module bram_container
#(
	parameter DATA_WIDTH = 9,
	parameter ADDR_WIDTH = (DATA_WIDTH == 9) ? 11 : (DATA_WIDTH == 18) ? 10 : (DATA_WIDTH == 36) ? 9 : -1
)
(
	clka,
	rsta,
	ena,
	wea,
	addra,
	dia,
	doa,

	clkb,
	rstb,
	enb,
	web,
	addrb,
	dib,
	dob
);


input	wire		clka;
input	wire		rsta;
input	wire		ena;
input	wire		wea;
input	wire	[ADDR_WIDTH-1:0]	addra;
input	wire	[DATA_WIDTH-1:0]	dia;
output	wire	[DATA_WIDTH-1:0]	doa;

input	wire		clkb;
input	wire		rstb;
input	wire		enb;
input	wire		web;
input	wire	[ADDR_WIDTH-1:0]	addrb;
input	wire	[DATA_WIDTH-1:0]	dib;
output	wire	[DATA_WIDTH-1:0]	dob;


generate
if (DATA_WIDTH == 9) begin : GEN
	ram_9_prim BRAM (
		.DIA(	dia[8:0]	),
		.ADDRA(	addra		),
		.ENA(	ena		),
		.WEA(	wea		),
		.SSRA(	rsta		),
		.CLKA(	clka		),
		.DOA(	doa[8:0]	),
		.DIB(	dib[8:0]	),
		.ADDRB(	addrb		),
		.ENB(	enb		),
		.WEB(	web		),
		.SSRB(	rstb		),
		.CLKB(	clkb		),
		.DOB(	dob[8:0]	)
	);
end
else if (DATA_WIDTH == 18) begin : GEN
	ram_18_prim BRAM (	
		.DIA(	dia[17:0]	),
		.ADDRA(	addra		),
		.ENA(	ena		),
		.WEA(	wea		),
		.SSRA(	rsta		),
		.CLKA(	clka		),
		.DOA(	doa[17:0]	),
		.DIB(	dib[17:0]	),
		.ADDRB(	addrb		),
		.ENB(	enb		),
		.WEB(	web		),
		.SSRB(	rstb		),
		.CLKB(	clkb		),
		.DOB(	dob[17:0]	)
	);
end
else if (DATA_WIDTH == 36) begin : GEN
	ram_36_prim BRAM (
		.DIA(	dia[35:0]	),
		.ADDRA(	addra		),
		.ENA(	ena		),
		.WEA(	wea		),
		.SSRA(	rsta		),
		.CLKA(	clka		),
		.DOA(	doa[35:0]	),
		.DIB(	dib[35:0]	),
		.ADDRB(	addrb		),
		.ENB(	enb		),
		.WEB(	web		),
		.SSRB(	rstb		),
		.CLKB(	clkb		),
		.DOB(	dob[35:0]	)
	);
end
endgenerate

endmodule
