/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module sdp_ram
#(
	parameter ADDRESS_WIDTH = 10,
	parameter DATA_WIDTH = 32,
	parameter WE_DATA_WIDTH = DATA_WIDTH,
	parameter ASYNCHRONOUS_READ = 0,
	parameter MEMORY_SIZE = (2**ADDRESS_WIDTH)
 ) (
	input wire clk,
	input wire [DATA_WIDTH/WE_DATA_WIDTH-1:0] we,
	input wire [ADDRESS_WIDTH-1:0] wr_address,
	input wire [DATA_WIDTH-1:0] data_in,
	input wire rd_enable,
	input wire [ADDRESS_WIDTH-1:0] rd_address,
	output wire [DATA_WIDTH-1:0] data_out
);

	reg [DATA_WIDTH-1:0] mem [0:MEMORY_SIZE-1];

	integer i;
	initial begin
		// Init Mem
		for (i = 0; i < MEMORY_SIZE; i = i + 1) begin
			mem[i] <= {(DATA_WIDTH){1'b0}};
		end
	end

	reg [DATA_WIDTH-1:0] read_data_reg;

	always @(posedge clk) begin
		if (rd_enable)
			read_data_reg <= mem[rd_address];
	end

	assign data_out		= ASYNCHRONOUS_READ ? mem[rd_address] : read_data_reg;

genvar x;
generate
for (x = 0; (x+1)*WE_DATA_WIDTH <= DATA_WIDTH; x = x+1) begin:memwrite
	always @(posedge clk)
		if (we[x])
			mem[wr_address][(x+1)*WE_DATA_WIDTH-1:x*WE_DATA_WIDTH] <= data_in[(x+1)*WE_DATA_WIDTH-1:x*WE_DATA_WIDTH];
end
endgenerate

endmodule
