/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none


////////////////////////////////////////////////////////////////////////////////
// Create Date:   08:59:16 07/05/2011
// Design Name:   interface_state_machine
// Module Name:   s6_mcb_interface/interface_tb.v
// Project Name:  s6_mcb_interface
////////////////////////////////////////////////////////////////////////////////

module interface_tb;

	// Inputs
	reg clk_peri;
	reg reset;
	reg select;
	reg wr_peri;
	reg [17:0] do_peri;
	reg [2:0] reg_select;
	reg selfrefresh_mode;

	// Outputs
	wire [31:0] ddr_addr;
	wire [31:0] ddr_data_in;
	wire [17:0] command;
	wire cmd_en;
	wire [2:0] cmd_inst;
	wire wr_en;
	wire rd_en;
	wire selfrefresh_enter;

	// Instantiate the Unit Under Test (UUT)
	interface_state_machine uut (
		.clk_peri(clk_peri), 
		.reset(reset), 
		.select(select), 
		.wr_peri(wr_peri), 
		.do_peri(do_peri), 
		.reg_select(reg_select), 
		.ddr_addr(ddr_addr), 
		.ddr_data_in(ddr_data_in), 
		.command(command), 
		.cmd_en(cmd_en), 
		.cmd_inst(cmd_inst), 
		.wr_en(wr_en), 
		.rd_en(rd_en), 
		.selfrefresh_mode(selfrefresh_mode), 
		.selfrefresh_enter(selfrefresh_enter)
	);

	initial begin
		// Initialize Inputs
		clk_peri = 0;
		reset = 0;
		select = 0;
		wr_peri = 0;
		do_peri = 0;
		reg_select = 0;
		selfrefresh_mode = 0;

		// Wait 100 ns for global reset to finish
		#100;
        
		// Add stimulus here
		reset = 1;
		#100;
		reset = 0;
		#100

		
		@(posedge clk_peri);
			select = 1;
			wr_peri = 1;
			do_peri = 18'h3AAAA;
		@(posedge clk_peri);
			select = 0;
			wr_peri = 0;
			do_peri = 18'h0;
		repeat (5) @(posedge clk_peri);
			select = 1;
			wr_peri = 1;
			reg_select = 0;
			do_peri = 18'h78;
		@(posedge clk_peri);
			select = 0;
			reg_select = 0;
			wr_peri = 0;
			do_peri = 18'h0;
		
		repeat (5)@(posedge clk_peri);
			select = 1;
			wr_peri = 1;
			reg_select = 2;
			do_peri = 18'h3AAAA;
		@(posedge clk_peri);
			select = 0;
			reg_select = 0;
			wr_peri = 0;
			do_peri = 18'h0;
		repeat (5)@(posedge clk_peri);
			select = 1;
			wr_peri = 1;
			reg_select = 3;
			do_peri = 18'h3AAAA;
		@(posedge clk_peri);
			select = 0;
			reg_select = 0;
			wr_peri = 0;
			do_peri = 18'h0;	
		repeat (5)@(posedge clk_peri);
			select = 1;
			wr_peri = 1;
			reg_select = 4;
			do_peri = 18'h3AAAA;
		@(posedge clk_peri);
			select = 0;
			reg_select = 0;
			wr_peri = 0;
			do_peri = 18'h0;
		repeat (5)@(posedge clk_peri);
			select = 1;
			wr_peri = 1;
			reg_select = 0;
			do_peri = 18'h3;
		@(posedge clk_peri);
			select = 0;
			reg_select = 0;
			wr_peri = 0;
			do_peri = 18'h0;
	end
   
	 always  #5  clk_peri =  ! clk_peri; 
	
      
endmodule

