/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
//*****************************************************************************
// (c) Copyright 2009 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//
//*****************************************************************************
//   ____  ____
//  /   /\/   /
// /___/  \  /    Vendor             : Xilinx
// \   \   \/     Version            : 3.8
//  \   \         Application        : MIG
//  /   /         Filename           : example_top #.v
// /___/   /\     Date Last Modified : $Date: 2011/05/27 15:50:38 $
// \   \  /  \    Date Created       : Tue Feb 23 2010
//  \___\/\___\
//
//Device           : Spartan-6
//Design Name      : DDR/DDR2/DDR3/LPDDR 
//Purpose          : This is a template file for the design top module. This module contains 
//                   all the four memory controllers and the two infrastructures. However,
//                   only the enabled modules will be active and others inactive.
//Reference        :
//Revision History :
//*****************************************************************************
`timescale 1ns/1ps
`default_nettype none
(* X_CORE_INFO = "mig_v3_8_ddr3_s6, Coregen 13.2" , CORE_GENERATION_INFO = "ddr3_s6,mig_v3_8,{component_name=mig_38, MEM_INTERFACE_TYPE=DDR3_SDRAM, CLK_PERIOD=3000, MEMORY_PART=mt41j64m16xx-187e, MEMORY_DEVICE_WIDTH=16, OUTPUT_DRV=DIV6, RTT_NOM=DIV4, AUTO_SR=ENABLED, HIGH_TEMP_SR=NORMAL, PORT_CONFIG=Two 32-bit bi-directional and four 32-bit unidirectional ports, MEM_ADDR_ORDER=ROW_BANK_COLUMN, PORT_ENABLE=Port0_Port1_Port2_Port3_Port4_Port5, INPUT_PIN_TERMINATION=CALIB_TERM, DATA_TERMINATION=25 Ohms, CLKFBOUT_MULT_F=2, CLKOUT_DIVIDE=1, DEBUG_PORT=1, INPUT_CLK_TYPE=Differential, LANGUAGE=Verilog, SYNTHESIS_TOOL=Foundation_ISE, NO_OF_CONTROLLERS=1}" *)
module ddr_mcb_sp6 #
(
   parameter MEMCLK_PERIOD_PS        = 3000,       
                                       // Memory data transfer clock period
   parameter CALIB_SOFT_IP        = "TRUE",       
                                       // # = TRUE, Enables the soft calibration logic,
                                       // # = FALSE, Disables the soft calibration logic.
   parameter SIMULATION           = "FALSE",       
                                       // # = TRUE, Simulating the design. Useful to reduce the simulation time,
                                       // # = FALSE, Implementing the design.
   parameter RST_ACT_LOW          = 1,       
                                       // # = 1 for active low reset,
                                       // # = 0 for active high reset.
   parameter INPUT_CLK_TYPE       = "DIFFERENTIAL",       
                                       // input clock type DIFFERENTIAL or SINGLE_ENDED
   parameter MEM_ADDR_ORDER       = "ROW_BANK_COLUMN",       
                                       // The order in which user address is provided to the memory controller,
                                       // ROW_BANK_COLUMN or BANK_ROW_COLUMN
   parameter NUM_DQ_PINS          = 16,       
                                       // External memory data width
   parameter MEM_ADDR_WIDTH       = 13,       
                                       // External memory address width
   parameter MEM_BANKADDR_WIDTH   = 3,        
                                       // External memory bank address width
   parameter PORT_0_MODE           = "(disabled)",
   parameter PORT_1_MODE           = "(disabled)",
   parameter PORT_2_MODE           = "(disabled)",
   parameter PORT_3_MODE           = "(disabled)",
   parameter PORT_4_MODE           = "(disabled)",
   parameter PORT_5_MODE           = "(disabled)",
   parameter CLKOUT_MCB0_DIVIDE       = 1,       
   parameter CLKOUT_MCB1_DIVIDE       = 1,       
   parameter CLKOUT_CALIB_DIVIDE       = 16,       
   parameter CLKOUT_USER_DIVIDE   = 8,       
   parameter CLKIN_MULTIPLY        = 2,       
   parameter CLKIN_DIVIDE        = 1,
   parameter MEM_TRAS             = 37500,       
   parameter MEM_TRCD             = 13130,       
   parameter MEM_TREFI            = 7800000,       
   parameter MEM_TRFC             = 110000,       
   parameter MEM_TRP              = 13130,       
   parameter MEM_TWR              = 15000,       
   parameter MEM_TRTP             = 7500,       
   parameter MEM_TWTR             = 7500,       
   parameter MEM_TYPE             = "DDR3",       
   parameter MEM_DENSITY          = "1Gb",       
   parameter MEM_BURST_LEN        = 8,       
   parameter MEM_CAS_LATENCY      = 6,       
   parameter MEM_NUM_COL_BITS     = 10,       
   parameter MEM_DDR1_2_ODS       = "FULL",       
   parameter MEM_DDR2_RTT         = "150OHMS",       
   parameter MEM_DDR2_DIFF_DQS_EN  = "YES",       
   parameter MEM_DDR2_3_PA_SR     = "FULL",       
   parameter MEM_DDR2_3_HIGH_TEMP_SR  = "NORMAL",       
   parameter MEM_DDR3_CAS_LATENCY  = 6,       
   parameter MEM_DDR3_ODS         = "DIV6",       
   parameter MEM_DDR3_RTT         = "DIV4",       
   parameter MEM_DDR3_CAS_WR_LATENCY  = 5,       
   parameter MEM_DDR3_AUTO_SR     = "ENABLED",       
   parameter MEM_MOBILE_PA_SR     = "FULL",       
   parameter MEM_MDDR_ODS         = "FULL",

   parameter DDR_MEMORY_PART = "" //Dummy parameter for jconfig...
)   

(
   output  wire                                          error,
   inout   wire[NUM_DQ_PINS-1:0]                      dram_dq,
   output  wire[MEM_ADDR_WIDTH-1:0]                   dram_a,
   output  wire[MEM_BANKADDR_WIDTH-1:0]               dram_ba,
   output  wire                                          dram_ras_n,
   output  wire                                          dram_cas_n,
   output  wire                                          dram_we_n,
   output  wire                                          dram_odt,
   output  wire                                          dram_reset_n,
   output  wire                                          dram_cke,
   output  wire                                          dram_dm,
   inout   wire                                          dram_udqs,
   inout   wire                                          dram_udqs_n,
   inout   wire                                          mcb3_rzq,
   inout   wire                                          mcb3_zio,
   output  wire                                          dram_udm,
   input   wire                                          clk_p,
   input   wire                                          clk_n,
   input   wire                                          rst_i,
   inout   wire                                          dram_dqs,
   inout   wire                                          dram_dqs_n,
   output  wire                                          dram_ck,
   output  wire                                          dram_ck_n,
   output  wire                                          user_clk_o,

   input wire [78:0] data_to_mcb_0,
   input wire [78:0] data_to_mcb_1,
   input wire [78:0] data_to_mcb_2,
   input wire [78:0] data_to_mcb_3,
   input wire [78:0] data_to_mcb_4,
   input wire [78:0] data_to_mcb_5,
   output wire [56:0]              data_from_mcb_0,
   output wire [56:0]              data_from_mcb_1,
   output wire [56:0]              data_from_mcb_2,
   output wire [56:0]              data_from_mcb_3,
   output wire [56:0]              data_from_mcb_4,
   output wire [56:0]              data_from_mcb_5

);

localparam [5:0]
  PORT_ENABLE = {
    (PORT_2_MODE != "BIDIRECTIONAL") && (PORT_3_MODE != "BIDIRECTIONAL") && (PORT_5_MODE != "(disabled)"),
    (PORT_2_MODE != "BIDIRECTIONAL") && (PORT_3_MODE != "BIDIRECTIONAL") && (PORT_4_MODE != "(disabled)"),
    (PORT_3_MODE != "(disabled)"),
    (PORT_2_MODE != "(disabled)"),
    (PORT_1_MODE != "(disabled)"),
    (PORT_0_MODE != "(disabled)")
  };

localparam PORT_CONFIG = 
  (PORT_2_MODE == "BIDIRECTIONAL") || (PORT_3_MODE == "BIDIRECTIONAL") ? "B32_B32_B32_B32" :
  {
    "B32_B32",
    (PORT_2_MODE == "WRITE") ? "_W32" : "_R32",
    (PORT_3_MODE == "WRITE") ? "_W32" : "_R32",
    (PORT_4_MODE == "WRITE") ? "_W32" : "_R32",
    (PORT_5_MODE == "WRITE") ? "_W32" : "_R32"
  };


wire                              c3_error;
wire                              c3_calib_done;
wire                              c3_rst0;
wire                              c3_async_rst;
wire                              c3_sysclk_2x;
wire                              c3_sysclk_2x_180;
wire                              c3_pll_ce_0;
wire                              c3_pll_ce_90;
wire                              c3_pll_lock;
wire                              c3_mcb_drp_clk;

// port 0

wire                            c3_p0_clk;
wire                            c3_p0_cmd_en;
wire[2:0]                       c3_p0_cmd_instr;
wire[5:0]                       c3_p0_cmd_bl;
wire[29:0]                      c3_p0_cmd_byte_addr;
wire                            c3_p0_wr_en;
wire                            c3_p0_rd_en;
wire[3:0]       c3_p0_wr_mask;
wire[31:0]  c3_p0_wr_data;

assign c3_p0_clk            = data_to_mcb_0[0];
assign c3_p0_cmd_en         = data_to_mcb_0[1];
assign c3_p0_cmd_instr      = data_to_mcb_0[4:2];
assign c3_p0_cmd_bl         = data_to_mcb_0[11:5];
assign c3_p0_cmd_byte_addr  = data_to_mcb_0[40:12];
assign c3_p0_wr_en          = data_to_mcb_0[41];
assign c3_p0_rd_en          = data_to_mcb_0[42];
assign c3_p0_wr_mask        = data_to_mcb_0[46:43];
assign c3_p0_wr_data        = data_to_mcb_0[78:47];

wire                            c3_p0_cmd_empty;
wire                            c3_p0_cmd_full;
wire[6:0]                       c3_p0_wr_count;
wire                            c3_p0_wr_full;
wire                            c3_p0_wr_empty;
wire                            c3_p0_wr_underrun;
wire                            c3_p0_wr_error;
wire                            c3_p0_rd_full;
wire                            c3_p0_rd_empty;
wire[6:0]                       c3_p0_rd_count;
wire                            c3_p0_rd_overflow;
wire                            c3_p0_rd_error;
wire[31:0]  c3_p0_rd_data;

assign data_from_mcb_0[0]                             = c3_p0_cmd_empty;
assign data_from_mcb_0[1]                             = c3_p0_cmd_full;
assign data_from_mcb_0[8:2]                           = c3_p0_wr_count;
assign data_from_mcb_0[9]                             = c3_p0_wr_full;
assign data_from_mcb_0[10]                            = c3_p0_wr_empty;
assign data_from_mcb_0[11]                            = c3_p0_wr_underrun;
assign data_from_mcb_0[12]                            = c3_p0_wr_error;
assign data_from_mcb_0[13]                            = c3_p0_rd_full;
assign data_from_mcb_0[14]                            = c3_p0_rd_empty;
assign data_from_mcb_0[21:15]                         = c3_p0_rd_count;
assign data_from_mcb_0[22]                            = c3_p0_rd_overflow;
assign data_from_mcb_0[23]                            = c3_p0_rd_error;
assign data_from_mcb_0[24]                            = c3_calib_done;
assign data_from_mcb_0[56:25]  = c3_p0_rd_data;

// port 1

wire                            c3_p1_clk;
wire                            c3_p1_cmd_en;
wire[2:0]                       c3_p1_cmd_instr;
wire[5:0]                       c3_p1_cmd_bl;
wire[29:0]                      c3_p1_cmd_byte_addr;
wire                            c3_p1_wr_en;
wire                            c3_p1_rd_en;
wire[3:0]       c3_p1_wr_mask;
wire[31:0]  c3_p1_wr_data;

assign c3_p1_clk            = data_to_mcb_1[0];    
assign c3_p1_cmd_en         = data_to_mcb_1[1];    
assign c3_p1_cmd_instr      = data_to_mcb_1[4:2];  
assign c3_p1_cmd_bl         = data_to_mcb_1[11:5]; 
assign c3_p1_cmd_byte_addr  = data_to_mcb_1[40:12];
assign c3_p1_wr_en          = data_to_mcb_1[41];   
assign c3_p1_rd_en          = data_to_mcb_1[42];   
assign c3_p1_wr_mask        = data_to_mcb_1[46:43];
assign c3_p1_wr_data        = data_to_mcb_1[78:47];

wire                            c3_p1_cmd_empty;
wire                            c3_p1_cmd_full;
wire[6:0]                       c3_p1_wr_count;
wire                            c3_p1_wr_full;
wire                            c3_p1_wr_empty;
wire                            c3_p1_wr_underrun;
wire                            c3_p1_wr_error;
wire                            c3_p1_rd_full;
wire                            c3_p1_rd_empty;
wire[6:0]                       c3_p1_rd_count;
wire                            c3_p1_rd_overflow;
wire                            c3_p1_rd_error;
wire[31:0]  c3_p1_rd_data;

assign data_from_mcb_1[0]                             = c3_p1_cmd_empty;
assign data_from_mcb_1[1]                             = c3_p1_cmd_full;
assign data_from_mcb_1[8:2]                           = c3_p1_wr_count;
assign data_from_mcb_1[9]                             = c3_p1_wr_full;
assign data_from_mcb_1[10]                            = c3_p1_wr_empty;
assign data_from_mcb_1[11]                            = c3_p1_wr_underrun;
assign data_from_mcb_1[12]                            = c3_p1_wr_error;
assign data_from_mcb_1[13]                            = c3_p1_rd_full;
assign data_from_mcb_1[14]                            = c3_p1_rd_empty;
assign data_from_mcb_1[21:15]                         = c3_p1_rd_count;
assign data_from_mcb_1[22]                            = c3_p1_rd_overflow;
assign data_from_mcb_1[23]                            = c3_p1_rd_error;
assign data_from_mcb_1[24]                            = c3_calib_done;
assign data_from_mcb_1[56:25]  = c3_p1_rd_data;

// port 2

wire                            c3_p2_clk;
wire                            c3_p2_cmd_en;
wire[2:0]                       c3_p2_cmd_instr;
wire[5:0]                       c3_p2_cmd_bl;
wire[29:0]                      c3_p2_cmd_byte_addr;
wire                            c3_p2_wr_en;
wire                            c3_p2_rd_en;
wire[3:0]       c3_p2_wr_mask;
wire[31:0]  c3_p2_wr_data;

assign c3_p2_clk            = data_to_mcb_2[0];    
assign c3_p2_cmd_en         = data_to_mcb_2[1];    
assign c3_p2_cmd_instr      = data_to_mcb_2[4:2];  
assign c3_p2_cmd_bl         = data_to_mcb_2[11:5]; 
assign c3_p2_cmd_byte_addr  = data_to_mcb_2[40:12];
assign c3_p2_wr_en          = data_to_mcb_2[41];   
assign c3_p2_rd_en          = data_to_mcb_2[42];   
assign c3_p2_wr_mask        = data_to_mcb_2[46:43];
assign c3_p2_wr_data        = data_to_mcb_2[78:47];

wire                            c3_p2_cmd_empty;
wire                            c3_p2_cmd_full;
wire[6:0]                       c3_p2_wr_count;
wire                            c3_p2_wr_full;
wire                            c3_p2_wr_empty;
wire                            c3_p2_wr_underrun;
wire                            c3_p2_wr_error;
wire                            c3_p2_rd_full;
wire                            c3_p2_rd_empty;
wire[6:0]                       c3_p2_rd_count;
wire                            c3_p2_rd_overflow;
wire                            c3_p2_rd_error;
wire[31:0]  c3_p2_rd_data;

assign data_from_mcb_2[0]                             = c3_p2_cmd_empty;
assign data_from_mcb_2[1]                             = c3_p2_cmd_full;
assign data_from_mcb_2[8:2]                           = c3_p2_wr_count;
assign data_from_mcb_2[9]                             = c3_p2_wr_full;
assign data_from_mcb_2[10]                            = c3_p2_wr_empty;
assign data_from_mcb_2[11]                            = c3_p2_wr_underrun;
assign data_from_mcb_2[12]                            = c3_p2_wr_error;
assign data_from_mcb_2[13]                            = c3_p2_rd_full;
assign data_from_mcb_2[14]                            = c3_p2_rd_empty;
assign data_from_mcb_2[21:15]                         = c3_p2_rd_count;
assign data_from_mcb_2[22]                            = c3_p2_rd_overflow;
assign data_from_mcb_2[23]                            = c3_p2_rd_error;
assign data_from_mcb_2[24]                            = c3_calib_done;
assign data_from_mcb_2[56:25]  = c3_p2_rd_data;

// port 3

wire                            c3_p3_clk;
wire                            c3_p3_cmd_en;
wire[2:0]                       c3_p3_cmd_instr;
wire[5:0]                       c3_p3_cmd_bl;
wire[29:0]                      c3_p3_cmd_byte_addr;
wire                            c3_p3_wr_en;
wire                            c3_p3_rd_en;
wire[3:0]       c3_p3_wr_mask;
wire[31:0]  c3_p3_wr_data;

assign c3_p3_clk            = data_to_mcb_3[0];    
assign c3_p3_cmd_en         = data_to_mcb_3[1];    
assign c3_p3_cmd_instr      = data_to_mcb_3[4:2];  
assign c3_p3_cmd_bl         = data_to_mcb_3[11:5]; 
assign c3_p3_cmd_byte_addr  = data_to_mcb_3[40:12];
assign c3_p3_wr_en          = data_to_mcb_3[41];   
assign c3_p3_rd_en          = data_to_mcb_3[42];   
assign c3_p3_wr_mask        = data_to_mcb_3[46:43];
assign c3_p3_wr_data        = data_to_mcb_3[78:47];

wire                            c3_p3_cmd_empty;
wire                            c3_p3_cmd_full;
wire[6:0]                       c3_p3_wr_count;
wire                            c3_p3_wr_full;
wire                            c3_p3_wr_empty;
wire                            c3_p3_wr_underrun;
wire                            c3_p3_wr_error;
wire                            c3_p3_rd_full;
wire                            c3_p3_rd_empty;
wire[6:0]                       c3_p3_rd_count;
wire                            c3_p3_rd_overflow;
wire                            c3_p3_rd_error;
wire[31:0]  c3_p3_rd_data;

assign data_from_mcb_3[0]                             = c3_p3_cmd_empty;
assign data_from_mcb_3[1]                             = c3_p3_cmd_full;
assign data_from_mcb_3[8:2]                           = c3_p3_wr_count;
assign data_from_mcb_3[9]                             = c3_p3_wr_full;
assign data_from_mcb_3[10]                            = c3_p3_wr_empty;
assign data_from_mcb_3[11]                            = c3_p3_wr_underrun;
assign data_from_mcb_3[12]                            = c3_p3_wr_error;
assign data_from_mcb_3[13]                            = c3_p3_rd_full;
assign data_from_mcb_3[14]                            = c3_p3_rd_empty;
assign data_from_mcb_3[21:15]                         = c3_p3_rd_count;
assign data_from_mcb_3[22]                            = c3_p3_rd_overflow;
assign data_from_mcb_3[23]                            = c3_p3_rd_error;
assign data_from_mcb_3[24]                            = c3_calib_done;
assign data_from_mcb_3[56:25]  = c3_p3_rd_data;

// port 4

wire                            c3_p4_clk;
wire                            c3_p4_cmd_en;
wire[2:0]                       c3_p4_cmd_instr;
wire[5:0]                       c3_p4_cmd_bl;
wire[29:0]                      c3_p4_cmd_byte_addr;
wire                            c3_p4_wr_en;
wire                            c3_p4_rd_en;
wire[3:0]       c3_p4_wr_mask;
wire[31:0]  c3_p4_wr_data;

assign c3_p4_clk            = data_to_mcb_4[0];    
assign c3_p4_cmd_en         = data_to_mcb_4[1];    
assign c3_p4_cmd_instr      = data_to_mcb_4[4:2];  
assign c3_p4_cmd_bl         = data_to_mcb_4[11:5]; 
assign c3_p4_cmd_byte_addr  = data_to_mcb_4[40:12];
assign c3_p4_wr_en          = data_to_mcb_4[41];   
assign c3_p4_rd_en          = data_to_mcb_4[42];   
assign c3_p4_wr_mask        = data_to_mcb_4[46:43];
assign c3_p4_wr_data        = data_to_mcb_4[78:47];

wire                            c3_p4_cmd_empty;
wire                            c3_p4_cmd_full;
wire[6:0]                       c3_p4_wr_count;
wire                            c3_p4_wr_full;
wire                            c3_p4_wr_empty;
wire                            c3_p4_wr_underrun;
wire                            c3_p4_wr_error;
wire                            c3_p4_rd_full;
wire                            c3_p4_rd_empty;
wire[6:0]                       c3_p4_rd_count;
wire                            c3_p4_rd_overflow;
wire                            c3_p4_rd_error;
wire[31:0]  c3_p4_rd_data;

assign data_from_mcb_4[0]                             = c3_p4_cmd_empty;
assign data_from_mcb_4[1]                             = c3_p4_cmd_full;
assign data_from_mcb_4[8:2]                           = c3_p4_wr_count;
assign data_from_mcb_4[9]                             = c3_p4_wr_full;
assign data_from_mcb_4[10]                            = c3_p4_wr_empty;
assign data_from_mcb_4[11]                            = c3_p4_wr_underrun;
assign data_from_mcb_4[12]                            = c3_p4_wr_error;
assign data_from_mcb_4[13]                            = c3_p4_rd_full;
assign data_from_mcb_4[14]                            = c3_p4_rd_empty;
assign data_from_mcb_4[21:15]                         = c3_p4_rd_count;
assign data_from_mcb_4[22]                            = c3_p4_rd_overflow;
assign data_from_mcb_4[23]                            = c3_p4_rd_error;
assign data_from_mcb_4[24]                            = c3_calib_done;
assign data_from_mcb_4[56:25]  = c3_p4_rd_data;

// port 5

wire                            c3_p5_clk;
wire                            c3_p5_cmd_en;
wire[2:0]                       c3_p5_cmd_instr;
wire[5:0]                       c3_p5_cmd_bl;
wire[29:0]                      c3_p5_cmd_byte_addr;
wire                            c3_p5_wr_en;
wire                            c3_p5_rd_en;
wire[3:0]       c3_p5_wr_mask;
wire[31:0]  c3_p5_wr_data;

assign c3_p5_clk            = data_to_mcb_5[0];    
assign c3_p5_cmd_en         = data_to_mcb_5[1];    
assign c3_p5_cmd_instr      = data_to_mcb_5[4:2];  
assign c3_p5_cmd_bl         = data_to_mcb_5[11:5]; 
assign c3_p5_cmd_byte_addr  = data_to_mcb_5[40:12];
assign c3_p5_wr_en          = data_to_mcb_5[41];   
assign c3_p5_rd_en          = data_to_mcb_5[42];   
assign c3_p5_wr_mask        = data_to_mcb_5[46:43];
assign c3_p5_wr_data        = data_to_mcb_5[78:47];

wire                            c3_p5_cmd_empty;
wire                            c3_p5_cmd_full;
wire[6:0]                       c3_p5_wr_count;
wire                            c3_p5_wr_full;
wire                            c3_p5_wr_empty;
wire                            c3_p5_wr_underrun;
wire                            c3_p5_wr_error;
wire                            c3_p5_rd_full;
wire                            c3_p5_rd_empty;
wire[6:0]                       c3_p5_rd_count;
wire                            c3_p5_rd_overflow;
wire                            c3_p5_rd_error;
wire[31:0]  c3_p5_rd_data;

assign data_from_mcb_5[0]                             = c3_p5_cmd_empty;
assign data_from_mcb_5[1]                             = c3_p5_cmd_full;
assign data_from_mcb_5[8:2]                           = c3_p5_wr_count;
assign data_from_mcb_5[9]                             = c3_p5_wr_full;
assign data_from_mcb_5[10]                            = c3_p5_wr_empty;
assign data_from_mcb_5[11]                            = c3_p5_wr_underrun;
assign data_from_mcb_5[12]                            = c3_p5_wr_error;
assign data_from_mcb_5[13]                            = c3_p5_rd_full;
assign data_from_mcb_5[14]                            = c3_p5_rd_empty;
assign data_from_mcb_5[21:15]                         = c3_p5_rd_count;
assign data_from_mcb_5[22]                            = c3_p5_rd_overflow;
assign data_from_mcb_5[23]                            = c3_p5_rd_error;
assign data_from_mcb_5[24]                            = c3_calib_done;
assign data_from_mcb_5[56:25]  = c3_p5_rd_data;

// The parameter CX_PORT_ENABLE shows all the active user ports in the design.
// For example, the value 6'b111100 tells that only port-2, port-3, port-4
// and port-5 are enabled. The other two ports are inactive. An inactive port
// can be a disabled port or an invisible logical port. Few examples to the 
// invisible logical port are port-4 and port-5 in the user port configuration,
// Config-2: Four 32-bit bi-directional ports and the ports port-2 through
// port-5 in Config-4: Two 64-bit bi-directional ports. Please look into the 
// Chapter-2 of ug388.pdf in the /docs directory for further details.
   
   localparam ARB_ALGORITHM        = 0;       
   localparam ARB_NUM_TIME_SLOTS   = 12;       
   localparam ARB_TIME_SLOT_0      = 18'o012345;       
   localparam ARB_TIME_SLOT_1      = 18'o123450;       
   localparam ARB_TIME_SLOT_2      = 18'o234501;       
   localparam ARB_TIME_SLOT_3      = 18'o345012;       
   localparam ARB_TIME_SLOT_4      = 18'o450123;       
   localparam ARB_TIME_SLOT_5      = 18'o501234;       
   localparam ARB_TIME_SLOT_6      = 18'o012345;       
   localparam ARB_TIME_SLOT_7      = 18'o123450;       
   localparam ARB_TIME_SLOT_8      = 18'o234501;       
   localparam ARB_TIME_SLOT_9      = 18'o345012;       
   localparam ARB_TIME_SLOT_10     = 18'o450123;       
   localparam ARB_TIME_SLOT_11     = 18'o501234;       
   localparam MC_CALIB_BYPASS      = "NO";       
   localparam MC_CALIBRATION_MODE  = "CALIBRATION";       
   localparam MC_CALIBRATION_DELAY  = "HALF";       
   localparam SKIP_IN_TERM_CAL     = 0;       
   localparam SKIP_DYNAMIC_CAL     = 0;       
   localparam LDQSP_TAP_DELAY_VAL  = 0;       
   localparam LDQSN_TAP_DELAY_VAL  = 0;       
   localparam UDQSP_TAP_DELAY_VAL  = 0;       
   localparam UDQSN_TAP_DELAY_VAL  = 0;       
   localparam DQ0_TAP_DELAY_VAL    = 0;       
   localparam DQ1_TAP_DELAY_VAL    = 0;       
   localparam DQ2_TAP_DELAY_VAL    = 0;       
   localparam DQ3_TAP_DELAY_VAL    = 0;       
   localparam DQ4_TAP_DELAY_VAL    = 0;       
   localparam DQ5_TAP_DELAY_VAL    = 0;       
   localparam DQ6_TAP_DELAY_VAL    = 0;       
   localparam DQ7_TAP_DELAY_VAL    = 0;       
   localparam DQ8_TAP_DELAY_VAL    = 0;       
   localparam DQ9_TAP_DELAY_VAL    = 0;       
   localparam DQ10_TAP_DELAY_VAL   = 0;       
   localparam DQ11_TAP_DELAY_VAL   = 0;       
   localparam DQ12_TAP_DELAY_VAL   = 0;       
   localparam DQ13_TAP_DELAY_VAL   = 0;       
   localparam DQ14_TAP_DELAY_VAL   = 0;       
   localparam DQ15_TAP_DELAY_VAL   = 0;       
   localparam INCLK_PERIOD         = ((MEMCLK_PERIOD_PS * CLKIN_MULTIPLY) / (CLKIN_DIVIDE * CLKOUT_MCB0_DIVIDE * 2));       
   localparam ARB_TIME0_SLOT  = {ARB_TIME_SLOT_0[17:15], ARB_TIME_SLOT_0[14:12], ARB_TIME_SLOT_0[11:9], ARB_TIME_SLOT_0[8:6], ARB_TIME_SLOT_0[5:3], ARB_TIME_SLOT_0[2:0]};
   localparam ARB_TIME1_SLOT  = {ARB_TIME_SLOT_1[17:15], ARB_TIME_SLOT_1[14:12], ARB_TIME_SLOT_1[11:9], ARB_TIME_SLOT_1[8:6], ARB_TIME_SLOT_1[5:3], ARB_TIME_SLOT_1[2:0]};
   localparam ARB_TIME2_SLOT  = {ARB_TIME_SLOT_2[17:15], ARB_TIME_SLOT_2[14:12], ARB_TIME_SLOT_2[11:9], ARB_TIME_SLOT_2[8:6], ARB_TIME_SLOT_2[5:3], ARB_TIME_SLOT_2[2:0]};
   localparam ARB_TIME3_SLOT  = {ARB_TIME_SLOT_3[17:15], ARB_TIME_SLOT_3[14:12], ARB_TIME_SLOT_3[11:9], ARB_TIME_SLOT_3[8:6], ARB_TIME_SLOT_3[5:3], ARB_TIME_SLOT_3[2:0]};
   localparam ARB_TIME4_SLOT  = {ARB_TIME_SLOT_4[17:15], ARB_TIME_SLOT_4[14:12], ARB_TIME_SLOT_4[11:9], ARB_TIME_SLOT_4[8:6], ARB_TIME_SLOT_4[5:3], ARB_TIME_SLOT_4[2:0]};
   localparam ARB_TIME5_SLOT  = {ARB_TIME_SLOT_5[17:15], ARB_TIME_SLOT_5[14:12], ARB_TIME_SLOT_5[11:9], ARB_TIME_SLOT_5[8:6], ARB_TIME_SLOT_5[5:3], ARB_TIME_SLOT_5[2:0]};
   localparam ARB_TIME6_SLOT  = {ARB_TIME_SLOT_6[17:15], ARB_TIME_SLOT_6[14:12], ARB_TIME_SLOT_6[11:9], ARB_TIME_SLOT_6[8:6], ARB_TIME_SLOT_6[5:3], ARB_TIME_SLOT_6[2:0]};
   localparam ARB_TIME7_SLOT  = {ARB_TIME_SLOT_7[17:15], ARB_TIME_SLOT_7[14:12], ARB_TIME_SLOT_7[11:9], ARB_TIME_SLOT_7[8:6], ARB_TIME_SLOT_7[5:3], ARB_TIME_SLOT_7[2:0]};
   localparam ARB_TIME8_SLOT  = {ARB_TIME_SLOT_8[17:15], ARB_TIME_SLOT_8[14:12], ARB_TIME_SLOT_8[11:9], ARB_TIME_SLOT_8[8:6], ARB_TIME_SLOT_8[5:3], ARB_TIME_SLOT_8[2:0]};
   localparam ARB_TIME9_SLOT  = {ARB_TIME_SLOT_9[17:15], ARB_TIME_SLOT_9[14:12], ARB_TIME_SLOT_9[11:9], ARB_TIME_SLOT_9[8:6], ARB_TIME_SLOT_9[5:3], ARB_TIME_SLOT_9[2:0]};
   localparam ARB_TIME10_SLOT  = {ARB_TIME_SLOT_10[17:15], ARB_TIME_SLOT_10[14:12], ARB_TIME_SLOT_10[11:9], ARB_TIME_SLOT_10[8:6], ARB_TIME_SLOT_10[5:3], ARB_TIME_SLOT_10[2:0]};
   localparam ARB_TIME11_SLOT  = {ARB_TIME_SLOT_11[17:15], ARB_TIME_SLOT_11[14:12], ARB_TIME_SLOT_11[11:9], ARB_TIME_SLOT_11[8:6], ARB_TIME_SLOT_11[5:3], ARB_TIME_SLOT_11[2:0]};

// debug signals declarations
   wire [255:0] c3_dbg_data;
   wire [1:0]  c3_dbg_trig;
   wire [35:0] c3_control0;
   wire [35:0] c3_control1;
   wire [6:0] c3_vio_out;



   reg   c1_aresetn;
   reg   c3_aresetn;
   reg   c4_aresetn;
   reg   c5_aresetn;

assign error = c3_error;




// Infrastructure-3 instantiation
      infrastructure #
      (
         .C_INCLK_PERIOD                 (INCLK_PERIOD),
         .C_RST_ACT_LOW                  (RST_ACT_LOW),
         .C_INPUT_CLK_TYPE               (INPUT_CLK_TYPE),
         .C_CLKOUT0_DIVIDE               (CLKOUT_MCB0_DIVIDE),
         .C_CLKOUT1_DIVIDE               (CLKOUT_MCB1_DIVIDE),
         .C_CLKOUT2_DIVIDE               (CLKOUT_CALIB_DIVIDE),
         .C_CLKOUT3_DIVIDE               (CLKOUT_USER_DIVIDE),
         .C_CLKFBOUT_MULT                (CLKIN_MULTIPLY),
         .C_DIVCLK_DIVIDE                (CLKIN_DIVIDE)
      )
      memc3_infrastructure_inst
      (

         .sys_clk_p                      (clk_p),  // [input] differential p type clock from board
         .sys_clk_n                      (clk_n),  // [input] differential n type clock from board
         .sys_clk                        (clk_p),  // [input] single ended input clock from board
         .sys_rst_i                      (rst_i),  
         .clk0                           (user_clk_o),       // [output] user clock which determines the operating frequency of user interface ports
         .rst0                           (c3_rst0),
         .async_rst                      (c3_async_rst),
         .sysclk_2x                      (c3_sysclk_2x),
         .sysclk_2x_180                  (c3_sysclk_2x_180),
         .pll_ce_0                       (c3_pll_ce_0),
         .pll_ce_90                      (c3_pll_ce_90),
         .pll_lock                       (c3_pll_lock),
         .mcb_drp_clk                    (c3_mcb_drp_clk)
      );
   


// Controller-3 instantiation
      memc_wrapper #
      (
         .C_MEMCLK_PERIOD                (MEMCLK_PERIOD_PS),   
         .C_CALIB_SOFT_IP                (CALIB_SOFT_IP),
         //synthesis translate_off
         .C_SIMULATION                   (SIMULATION),
         //synthesis translate_on
         .C_ARB_NUM_TIME_SLOTS           (ARB_NUM_TIME_SLOTS),
         .C_ARB_TIME_SLOT_0              (ARB_TIME0_SLOT),
         .C_ARB_TIME_SLOT_1              (ARB_TIME1_SLOT),
         .C_ARB_TIME_SLOT_2              (ARB_TIME2_SLOT),
         .C_ARB_TIME_SLOT_3              (ARB_TIME3_SLOT),
         .C_ARB_TIME_SLOT_4              (ARB_TIME4_SLOT),
         .C_ARB_TIME_SLOT_5              (ARB_TIME5_SLOT),
         .C_ARB_TIME_SLOT_6              (ARB_TIME6_SLOT),
         .C_ARB_TIME_SLOT_7              (ARB_TIME7_SLOT),
         .C_ARB_TIME_SLOT_8              (ARB_TIME8_SLOT),
         .C_ARB_TIME_SLOT_9              (ARB_TIME9_SLOT),
         .C_ARB_TIME_SLOT_10             (ARB_TIME10_SLOT),
         .C_ARB_TIME_SLOT_11             (ARB_TIME11_SLOT),
         .C_ARB_ALGORITHM                (ARB_ALGORITHM),
         .C_PORT_ENABLE                  (PORT_ENABLE),
         .C_PORT_CONFIG                  (PORT_CONFIG),
         .C_MEM_TRAS                     (MEM_TRAS),
         .C_MEM_TRCD                     (MEM_TRCD),
         .C_MEM_TREFI                    (MEM_TREFI),
         .C_MEM_TRFC                     (MEM_TRFC),
         .C_MEM_TRP                      (MEM_TRP),
         .C_MEM_TWR                      (MEM_TWR),
         .C_MEM_TRTP                     (MEM_TRTP),
         .C_MEM_TWTR                     (MEM_TWTR),
         .C_MEM_ADDR_ORDER               (MEM_ADDR_ORDER),
         .C_NUM_DQ_PINS                  (NUM_DQ_PINS),
         .C_MEM_TYPE                     (MEM_TYPE),
         .C_MEM_DENSITY                  (MEM_DENSITY),
         .C_MEM_BURST_LEN                (MEM_BURST_LEN),
         .C_MEM_CAS_LATENCY              (MEM_CAS_LATENCY),
         .C_MEM_ADDR_WIDTH               (MEM_ADDR_WIDTH),
         .C_MEM_BANKADDR_WIDTH           (MEM_BANKADDR_WIDTH),
         .C_MEM_NUM_COL_BITS             (MEM_NUM_COL_BITS),
         .C_MEM_DDR1_2_ODS               (MEM_DDR1_2_ODS),
         .C_MEM_DDR2_RTT                 (MEM_DDR2_RTT),
         .C_MEM_DDR2_DIFF_DQS_EN         (MEM_DDR2_DIFF_DQS_EN),
         .C_MEM_DDR2_3_PA_SR             (MEM_DDR2_3_PA_SR),
         .C_MEM_DDR2_3_HIGH_TEMP_SR      (MEM_DDR2_3_HIGH_TEMP_SR),
         .C_MEM_DDR3_CAS_LATENCY         (MEM_DDR3_CAS_LATENCY),
         .C_MEM_DDR3_ODS                 (MEM_DDR3_ODS),
         .C_MEM_DDR3_RTT                 (MEM_DDR3_RTT),
         .C_MEM_DDR3_CAS_WR_LATENCY      (MEM_DDR3_CAS_WR_LATENCY),
         .C_MEM_DDR3_AUTO_SR             (MEM_DDR3_AUTO_SR),
         .C_MEM_MOBILE_PA_SR             (MEM_MOBILE_PA_SR),
         .C_MEM_MDDR_ODS                 (MEM_MDDR_ODS),
         .C_MC_CALIB_BYPASS              (MC_CALIB_BYPASS),
         .C_MC_CALIBRATION_MODE          (MC_CALIBRATION_MODE),
         .C_MC_CALIBRATION_DELAY         (MC_CALIBRATION_DELAY),
         .C_SKIP_IN_TERM_CAL             (SKIP_IN_TERM_CAL),
         .C_SKIP_DYNAMIC_CAL             (SKIP_DYNAMIC_CAL),
         .LDQSP_TAP_DELAY_VAL            (LDQSP_TAP_DELAY_VAL),
         .UDQSP_TAP_DELAY_VAL            (UDQSP_TAP_DELAY_VAL),
         .LDQSN_TAP_DELAY_VAL            (LDQSN_TAP_DELAY_VAL),
         .UDQSN_TAP_DELAY_VAL            (UDQSN_TAP_DELAY_VAL),
         .DQ0_TAP_DELAY_VAL              (DQ0_TAP_DELAY_VAL),
         .DQ1_TAP_DELAY_VAL              (DQ1_TAP_DELAY_VAL),
         .DQ2_TAP_DELAY_VAL              (DQ2_TAP_DELAY_VAL),
         .DQ3_TAP_DELAY_VAL              (DQ3_TAP_DELAY_VAL),
         .DQ4_TAP_DELAY_VAL              (DQ4_TAP_DELAY_VAL),
         .DQ5_TAP_DELAY_VAL              (DQ5_TAP_DELAY_VAL),
         .DQ6_TAP_DELAY_VAL              (DQ6_TAP_DELAY_VAL),
         .DQ7_TAP_DELAY_VAL              (DQ7_TAP_DELAY_VAL),
         .DQ8_TAP_DELAY_VAL              (DQ8_TAP_DELAY_VAL),
         .DQ9_TAP_DELAY_VAL              (DQ9_TAP_DELAY_VAL),
         .DQ10_TAP_DELAY_VAL             (DQ10_TAP_DELAY_VAL),
         .DQ11_TAP_DELAY_VAL             (DQ11_TAP_DELAY_VAL),
         .DQ12_TAP_DELAY_VAL             (DQ12_TAP_DELAY_VAL),
         .DQ13_TAP_DELAY_VAL             (DQ13_TAP_DELAY_VAL),
         .DQ14_TAP_DELAY_VAL             (DQ14_TAP_DELAY_VAL),
         .DQ15_TAP_DELAY_VAL             (DQ15_TAP_DELAY_VAL),
         .C_P0_MASK_SIZE                 (4),
         .C_P0_DATA_PORT_SIZE            (32),
         .C_P1_MASK_SIZE                 (4),
         .C_P1_DATA_PORT_SIZE            (32)
    )
      
      memc3_wrapper_inst
      (
         .mcbx_dram_addr                 (dram_a), 
         .mcbx_dram_ba                   (dram_ba),
         .mcbx_dram_ras_n                (dram_ras_n), 
         .mcbx_dram_cas_n                (dram_cas_n), 
         .mcbx_dram_we_n                 (dram_we_n), 
         .mcbx_dram_cke                  (dram_cke), 
         .mcbx_dram_clk                  (dram_ck), 
         .mcbx_dram_clk_n                (dram_ck_n), 
         .mcbx_dram_dq                   (dram_dq),
         .mcbx_dram_dqs                  (dram_dqs), 
         .mcbx_dram_dqs_n                (dram_dqs_n), 
         .mcbx_dram_udqs                 (dram_udqs), 
         .mcbx_dram_udqs_n               (dram_udqs_n), 
         .mcbx_dram_udm                  (dram_udm), 
         .mcbx_dram_ldm                  (dram_dm), 
         .mcbx_dram_odt                  (dram_odt), 
         .mcbx_dram_ddr3_rst             (dram_reset_n), 
         .mcbx_rzq                       (mcb3_rzq),
         .mcbx_zio                       (mcb3_zio),
         .calib_done                     (c3_calib_done),
         .async_rst                      (c3_async_rst),
         .sysclk_2x                      (c3_sysclk_2x), 
         .sysclk_2x_180                  (c3_sysclk_2x_180), 
         .pll_ce_0                       (c3_pll_ce_0),
         .pll_ce_90                      (c3_pll_ce_90), 
         .pll_lock                       (c3_pll_lock),
         .mcb_drp_clk                    (c3_mcb_drp_clk), 
     
         // The following port map shows all the six logical user ports. However, all
     // of them may not be active in this design. A port should be enabled to 
     // validate its port map. If it is not,the complete port is going to float 
     // by getting disconnected from the lower level MCB modules. The port enable
     // information of a controller can be obtained from the corresponding local
     // parameter CX_PORT_ENABLE. In such a case, we can simply ignore its port map.
     // The following comments will explain when a port is going to be active.
     // Config-1: Two 32-bit bi-directional and four 32-bit unidirectional ports
     // Config-2: Four 32-bit bi-directional ports
     // Config-3: One 64-bit bi-directional and two 32-bit bi-directional ports
     // Config-4: Two 64-bit bi-directional ports
     // Config-5: One 128-bit bi-directional port

         // User Port-0 command interface will be active only when the port is enabled in 
         // the port configurations Config-1, Config-2, Config-3, Config-4 and Config-5
         .p0_cmd_clk                     (c3_p0_clk), 
         .p0_cmd_en                      (c3_p0_cmd_en), 
         .p0_cmd_instr                   (c3_p0_cmd_instr),
         .p0_cmd_bl                      (c3_p0_cmd_bl), 
         .p0_cmd_byte_addr               (c3_p0_cmd_byte_addr), 
         .p0_cmd_full                    (c3_p0_cmd_full),
         .p0_cmd_empty                   (c3_p0_cmd_empty),
         // User Port-0 data write interface will be active only when the port is enabled in
         // the port configurations Config-1, Config-2, Config-3, Config-4 and Config-5
         .p0_wr_clk                      (c3_p0_clk), 
         .p0_wr_en                       (c3_p0_wr_en),
         .p0_wr_mask                     (c3_p0_wr_mask),
         .p0_wr_data                     (c3_p0_wr_data),
         .p0_wr_full                     (c3_p0_wr_full),
         .p0_wr_count                    (c3_p0_wr_count),
         .p0_wr_empty                    (c3_p0_wr_empty),
         .p0_wr_underrun                 (c3_p0_wr_underrun),
         .p0_wr_error                    (c3_p0_wr_error),
         // User Port-0 data read interface will be active only when the port is enabled in
         // the port configurations Config-1, Config-2, Config-3, Config-4 and Config-5
         .p0_rd_clk                      (c3_p0_clk), 
         .p0_rd_en                       (c3_p0_rd_en),
         .p0_rd_data                     (c3_p0_rd_data),
         .p0_rd_empty                    (c3_p0_rd_empty),
         .p0_rd_count                    (c3_p0_rd_count),
         .p0_rd_full                     (c3_p0_rd_full),
         .p0_rd_overflow                 (c3_p0_rd_overflow),
         .p0_rd_error                    (c3_p0_rd_error),
 
         // User Port-1 command interface will be active only when the port is enabled in 
         // the port configurations Config-1, Config-2, Config-3 and Config-4
         .p1_cmd_clk                     (c3_p1_clk), 
         .p1_cmd_en                      (c3_p1_cmd_en), 
         .p1_cmd_instr                   (c3_p1_cmd_instr),
         .p1_cmd_bl                      (c3_p1_cmd_bl), 
         .p1_cmd_byte_addr               (c3_p1_cmd_byte_addr), 
         .p1_cmd_full                    (c3_p1_cmd_full),
         .p1_cmd_empty                   (c3_p1_cmd_empty),
         // User Port-1 data write interface will be active only when the port is enabled in 
         // the port configurations Config-1, Config-2, Config-3 and Config-4
         .p1_wr_clk                      (c3_p1_clk), 
         .p1_wr_en                       (c3_p1_wr_en),
         .p1_wr_mask                     (c3_p1_wr_mask),
         .p1_wr_data                     (c3_p1_wr_data),
         .p1_wr_full                     (c3_p1_wr_full),
         .p1_wr_count                    (c3_p1_wr_count),
         .p1_wr_empty                    (c3_p1_wr_empty),
         .p1_wr_underrun                 (c3_p1_wr_underrun),
         .p1_wr_error                    (c3_p1_wr_error),
         // User Port-1 data read interface will be active only when the port is enabled in 
         // the port configurations Config-1, Config-2, Config-3 and Config-4
         .p1_rd_clk                      (c3_p1_clk), 
         .p1_rd_en                       (c3_p1_rd_en),
         .p1_rd_data                     (c3_p1_rd_data),
         .p1_rd_empty                    (c3_p1_rd_empty),
         .p1_rd_count                    (c3_p1_rd_count),
         .p1_rd_full                     (c3_p1_rd_full),
         .p1_rd_overflow                 (c3_p1_rd_overflow),
         .p1_rd_error                    (c3_p1_rd_error),
      
         // User Port-2 command interface will be active only when the port is enabled in 
         // the port configurations Config-1, Config-2 and Config-3
         .p2_cmd_clk                     (c3_p2_clk), 
         .p2_cmd_en                      (c3_p2_cmd_en), 
         .p2_cmd_instr                   (c3_p2_cmd_instr),
         .p2_cmd_bl                      (c3_p2_cmd_bl), 
         .p2_cmd_byte_addr               (c3_p2_cmd_byte_addr), 
         .p2_cmd_full                    (c3_p2_cmd_full),
         .p2_cmd_empty                   (c3_p2_cmd_empty),
         // User Port-2 data write interface will be active only when the port is enabled in 
         // the port configurations Config-1 write direction, Config-2 and Config-3
         .p2_wr_clk                      (c3_p2_clk), 
         .p2_wr_en                       (c3_p2_wr_en),
         .p2_wr_mask                     (c3_p2_wr_mask),
         .p2_wr_data                     (c3_p2_wr_data),
         .p2_wr_full                     (c3_p2_wr_full),
         .p2_wr_count                    (c3_p2_wr_count),
         .p2_wr_empty                    (c3_p2_wr_empty),
         .p2_wr_underrun                 (c3_p2_wr_underrun),
         .p2_wr_error                    (c3_p2_wr_error),
         // User Port-2 data read interface will be active only when the port is enabled in 
         // the port configurations Config-1 read direction, Config-2 and Config-3
         .p2_rd_clk                      (c3_p2_clk), 
         .p2_rd_en                       (c3_p2_rd_en),
         .p2_rd_data                     (c3_p2_rd_data),
         .p2_rd_empty                    (c3_p2_rd_empty),
         .p2_rd_count                    (c3_p2_rd_count),
         .p2_rd_full                     (c3_p2_rd_full),
         .p2_rd_overflow                 (c3_p2_rd_overflow),
         .p2_rd_error                    (c3_p2_rd_error),
      
         // User Port-3 command interface will be active only when the port is enabled in 
         // the port configurations Config-1 and Config-2
         .p3_cmd_clk                     (c3_p3_clk), 
         .p3_cmd_en                      (c3_p3_cmd_en), 
         .p3_cmd_instr                   (c3_p3_cmd_instr),
         .p3_cmd_bl                      (c3_p3_cmd_bl), 
         .p3_cmd_byte_addr               (c3_p3_cmd_byte_addr), 
         .p3_cmd_full                    (c3_p3_cmd_full),
         .p3_cmd_empty                   (c3_p3_cmd_empty),
         // User Port-3 data write interface will be active only when the port is enabled in 
         // the port configurations Config-1 write direction and Config-2
         .p3_wr_clk                      (c3_p3_clk), 
         .p3_wr_en                       (c3_p3_wr_en),
         .p3_wr_mask                     (c3_p3_wr_mask),
         .p3_wr_data                     (c3_p3_wr_data),
         .p3_wr_full                     (c3_p3_wr_full),
         .p3_wr_count                    (c3_p3_wr_count),
         .p3_wr_empty                    (c3_p3_wr_empty),
         .p3_wr_underrun                 (c3_p3_wr_underrun),
         .p3_wr_error                    (c3_p3_wr_error),
         // User Port-3 data read interface will be active only when the port is enabled in 
         // the port configurations Config-1 read direction and Config-2
         .p3_rd_clk                      (c3_p3_clk), 
         .p3_rd_en                       (c3_p3_rd_en),
         .p3_rd_data                     (c3_p3_rd_data),
         .p3_rd_empty                    (c3_p3_rd_empty),
         .p3_rd_count                    (c3_p3_rd_count),
         .p3_rd_full                     (c3_p3_rd_full),
         .p3_rd_overflow                 (c3_p3_rd_overflow),
         .p3_rd_error                    (c3_p3_rd_error),
      
         // User Port-4 command interface will be active only when the port is enabled in 
         // the port configuration Config-1
         .p4_cmd_clk                     (c3_p4_clk), 
         .p4_cmd_en                      (c3_p4_cmd_en), 
         .p4_cmd_instr                   (c3_p4_cmd_instr),
         .p4_cmd_bl                      (c3_p4_cmd_bl), 
         .p4_cmd_byte_addr               (c3_p4_cmd_byte_addr), 
         .p4_cmd_full                    (c3_p4_cmd_full),
         .p4_cmd_empty                   (c3_p4_cmd_empty),
         // User Port-4 data write interface will be active only when the port is enabled in 
         // the port configuration Config-1 write direction
         .p4_wr_clk                      (c3_p4_clk), 
         .p4_wr_en                       (c3_p4_wr_en),
         .p4_wr_mask                     (c3_p4_wr_mask),
         .p4_wr_data                     (c3_p4_wr_data),
         .p4_wr_full                     (c3_p4_wr_full),
         .p4_wr_count                    (c3_p4_wr_count),
         .p4_wr_empty                    (c3_p4_wr_empty),
         .p4_wr_underrun                 (c3_p4_wr_underrun),
         .p4_wr_error                    (c3_p4_wr_error),
         // User Port-4 data read interface will be active only when the port is enabled in 
         // the port configuration Config-1 read direction
         .p4_rd_clk                      (c3_p4_clk), 
         .p4_rd_en                       (c3_p4_rd_en),
         .p4_rd_data                     (c3_p4_rd_data),
         .p4_rd_empty                    (c3_p4_rd_empty),
         .p4_rd_count                    (c3_p4_rd_count),
         .p4_rd_full                     (c3_p4_rd_full),
         .p4_rd_overflow                 (c3_p4_rd_overflow),
         .p4_rd_error                    (c3_p4_rd_error),
      
         // User Port-5 command interface will be active only when the port is enabled in 
         // the port configuration Config-1
         .p5_cmd_clk                     (c3_p5_clk), 
         .p5_cmd_en                      (c3_p5_cmd_en), 
         .p5_cmd_instr                   (c3_p5_cmd_instr),
         .p5_cmd_bl                      (c3_p5_cmd_bl), 
         .p5_cmd_byte_addr               (c3_p5_cmd_byte_addr), 
         .p5_cmd_full                    (c3_p5_cmd_full),
         .p5_cmd_empty                   (c3_p5_cmd_empty),
         // User Port-5 data write interface will be active only when the port is enabled in 
         // the port configuration Config-1 write direction
         .p5_wr_clk                      (c3_p5_clk), 
         .p5_wr_en                       (c3_p5_wr_en),
         .p5_wr_mask                     (c3_p5_wr_mask),
         .p5_wr_data                     (c3_p5_wr_data),
         .p5_wr_full                     (c3_p5_wr_full),
         .p5_wr_count                    (c3_p5_wr_count),
         .p5_wr_empty                    (c3_p5_wr_empty),
         .p5_wr_underrun                 (c3_p5_wr_underrun),
         .p5_wr_error                    (c3_p5_wr_error),
         // User Port-5 data read interface will be active only when the port is enabled in 
         // the port configuration Config-1 read direction
         .p5_rd_clk                      (c3_p5_clk), 
         .p5_rd_en                       (c3_p5_rd_en),
         .p5_rd_data                     (c3_p5_rd_data),
         .p5_rd_empty                    (c3_p5_rd_empty),
         .p5_rd_count                    (c3_p5_rd_count),
         .p5_rd_full                     (c3_p5_rd_full),
         .p5_rd_overflow                 (c3_p5_rd_overflow),
         .p5_rd_error                    (c3_p5_rd_error),

         .selfrefresh_enter              (1'b0), 
         .selfrefresh_mode               ()
      );
   
  endmodule   

 


