/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    13:31:05 09/19/2011 
// Module Name:    mcp_peripheral_interface 
//////////////////////////////////////////////////////////////////////////////////
module ddr_mem_port_32
(
	input   wire clk_peri,
	input 	wire [17:0] do_peri,
	output 	wire [17:0] di_peri,
	input 	wire [9:0] addr_peri,
	input 	wire access_peri,
	input 	wire wr_peri,
	input 	wire reset,


    input   wire [56:0] data_from_mcb,
    output  wire [78:0] data_to_mcb,
    output wire [212:0] debug
);




// Die Parameter fuer die Adressen und die Portbreite werden 
// in der system.v durch jConfig eingestellt.
parameter BASE_ADR	        = 10'h0;


parameter CONTROL_ADR      = 0; 
parameter STATUS_ADR       = 1;
parameter ADDRESS_LOW_ADR  = 2;
parameter ADDRESS_HIGH_ADR = 3;
parameter RD_COUNT_ADR     = 4;
parameter DATA_LOW_ADR     = 5;
parameter DATA_HIGH_ADR    = 6;

parameter ALL_PORTS	= 7;
parameter DECODSIM_DEB	= "NO";


// Dekodierung der Adressen ********************** START *************************************
// chipselect fuer die Statusregister
wire cs_control_r;
wire cs_control_w;
wire cs_status_r;
wire cs_status_w;
wire cs_address_low_r;
wire cs_address_low_w;
wire cs_address_high_r;
wire cs_address_high_w;
wire cs_rd_count_r;
wire cs_rd_count_w;
wire cs_data_low_r;
wire cs_data_low_w;
wire cs_data_high_r;
wire cs_data_high_w;

wire select;
wire [ALL_PORTS-1:0] reg_select;
wire [ALL_PORTS-1:0] reg_read;
wire [ALL_PORTS-1:0] reg_write;

// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);

// chipselect
assign cs_control_r      = reg_read  [CONTROL_ADR];
assign cs_control_w      = reg_write [CONTROL_ADR];
assign cs_status_r       = reg_read  [STATUS_ADR];
assign cs_status_w       = reg_write [STATUS_ADR];
assign cs_address_low_r  = reg_read  [ADDRESS_LOW_ADR];
assign cs_address_low_w  = reg_write [ADDRESS_LOW_ADR];
assign cs_address_high_r = reg_read  [ADDRESS_HIGH_ADR];
assign cs_address_high_w = reg_write [ADDRESS_HIGH_ADR];
assign cs_rd_count_r     = reg_read  [RD_COUNT_ADR];
assign cs_rd_count_w     = reg_write [RD_COUNT_ADR];
assign cs_data_low_r     = reg_read  [DATA_LOW_ADR];
assign cs_data_low_w     = reg_write [DATA_LOW_ADR];
assign cs_data_high_r    = reg_read  [DATA_HIGH_ADR];
assign cs_data_high_w    = reg_write [DATA_HIGH_ADR];

// Dekodierung der Adressen ********************** END **************************************


wire            cmd_empty;
wire            cmd_full;
wire            calib_done;

wire            wr_full;
wire            wr_empty;
wire    [6:0]   wr_count;
wire            wr_underrun;
wire            wr_error;


wire    [31:0]  rd_data;
wire            rd_full;
wire            rd_empty;
wire    [6:0]   rd_count;
wire            rd_overflow;
wire            rd_error;

assign cmd_empty   = data_from_mcb[0];
assign cmd_full    = data_from_mcb[1];
assign wr_count    = data_from_mcb[8:2];
assign wr_full     = data_from_mcb[9];
assign wr_empty    = data_from_mcb[10];
assign wr_underrun = data_from_mcb[11];
assign wr_error    = data_from_mcb[12];
assign rd_full     = data_from_mcb[13];
assign rd_empty    = data_from_mcb[14];
assign rd_count    = data_from_mcb[21:15];
assign rd_overflow = data_from_mcb[22];
assign rd_error    = data_from_mcb[23];
assign calib_done  = data_from_mcb[24];
assign rd_data     = data_from_mcb[56:25];

wire        mem_clk;
reg        cmd_en;
reg [2:0]   cmd_instr;
reg [5:0]   cmd_bl;
reg    [29:0]  cmd_byte_addr;
reg        wr_en;
reg                rd_en;
reg    [3:0]     wr_mask;
reg    [31:0]    wr_data;

assign data_to_mcb[0]                   = mem_clk;
assign data_to_mcb[1]                   = cmd_en;
assign data_to_mcb[4:2]                 = cmd_instr;
assign data_to_mcb[11:5]                = cmd_bl;
assign data_to_mcb[40:12]               = cmd_byte_addr;
assign data_to_mcb[41]                  = wr_en;
assign data_to_mcb[42]                  = rd_en;
assign data_to_mcb[46:43]               = wr_mask;
assign data_to_mcb[78:47]               = wr_data;


assign mem_clk = clk_peri;
reg[6:0] rd_count_reserved;
reg[17:0] rd_count_not_bursted;
reg data_layout;
reg low_only;
reg auto_precharge;

// Register writes
always @ (posedge clk_peri) begin
	if (reset) begin
		data_layout <= 0;
		low_only <= 0;
		wr_mask <= 0;
		cmd_byte_addr <= 0;
		auto_precharge <= 0;
	end else begin
		if (cs_control_w) begin
			wr_mask <= do_peri[3:0];
			data_layout <= do_peri[4];
			low_only <= do_peri[5];
			auto_precharge <= do_peri[6];
		end
		if (cs_address_low_w) begin
			cmd_byte_addr[19:2] <= do_peri;
		end else		
		if (cs_address_high_w) begin
			cmd_byte_addr[29:20] <= do_peri[9:0];
		end else if (cmd_en)
			cmd_byte_addr[29:2] <= cmd_byte_addr[29:2]+cmd_bl+1;
		if (cs_data_low_w) begin
			if (data_layout)
				wr_data[15:0] <= do_peri[15:0];
			else
				wr_data[17:0] <= do_peri;
		end
		if (cs_data_high_w) begin
			if (data_layout)
				wr_data[31:16] <= do_peri[15:0];
			else
				wr_data[31:18] <= do_peri[13:0];
		end
	end
end

// Register reads
assign di_peri = cs_control_r      ? {11'h0, auto_precharge, low_only, data_layout, wr_mask} :
                 cs_status_r       ? {3'h0, rd_error, rd_overflow, rd_empty, wr_full, wr_error, wr_underrun, wr_empty, calib_done, rd_count_reserved} :
                 cs_address_low_r  ? cmd_byte_addr[19:2] :
                 cs_address_high_r ? {8'h0, cmd_byte_addr[29:20]} :
                 cs_rd_count_r     ? rd_count_not_bursted :
                 cs_data_low_r     ? (data_layout? {2'b00, rd_data[15:0] } : rd_data[17:0]) :
                 cs_data_high_r    ? (data_layout? {2'b00, rd_data[31:16]} : {4'b0, rd_data[31:18]}) :
                 18'h0;

// WR cmd generation
reg [3:0] wr_cmd_counter;
always @(posedge clk_peri) begin
	if (reset) begin
		wr_cmd_counter <= 0;
	end else begin
		wr_cmd_counter <= wr_cmd_counter + 1;
	end
end

reg[4:0] wr_not_bursted;


reg wr_burstcommmit;

//wr_en generation
reg [1:0] wr_accessed_regs;
always  @(posedge clk_peri) begin
	if (reset) begin
		wr_accessed_regs <= 0;
		wr_en <= 0;
		wr_not_bursted <= 0;
	end else begin
	
		if ((low_only && cs_data_low_w) || 
			(~low_only && ((wr_accessed_regs[1] && cs_data_low_w) || (wr_accessed_regs[0] && cs_data_high_w)))) begin
			
			wr_accessed_regs <= 0;
			wr_en<=1;
			
			if (wr_burstcommmit)
				wr_not_bursted<=1;
			else
				wr_not_bursted <= wr_not_bursted+1;
			
		end else begin
			wr_en<=0;
			if (wr_burstcommmit)
				if (wr_en)
					wr_not_bursted<=1;
				else
					wr_not_bursted<=0;
				
			if (cs_data_high_w)
				wr_accessed_regs[1] <= 1;
			if (cs_data_low_w)
				wr_accessed_regs[0] <= 1;
			if (cs_control_w)
				wr_accessed_regs <= 0;
		end
			
	end
end

//rd_en generation
reg [1:0] rd_accessed_regs;
always  @(posedge clk_peri) begin
	if (reset) begin
		rd_accessed_regs <= 0;
		rd_en <= 1;
	end else begin
	
		if (((low_only && cs_data_low_r) || 
			(~low_only && ((rd_accessed_regs[1] && cs_data_low_r) || (rd_accessed_regs[0] && cs_data_high_r))))
			&& |rd_count_reserved) begin
			
			rd_en<=1;
			
		end else begin
			rd_en<=0;
			
			if (cs_data_high_r)
				rd_accessed_regs[1] <= 1;
			if (cs_data_low_r)
				rd_accessed_regs[0] <= 1;
			if (cs_control_w)
				rd_accessed_regs <= 0;
		end
			
	end
end


//cmd generation
always @(posedge clk_peri) begin
	if (reset || !calib_done) begin
		wr_burstcommmit <= 0;
		cmd_en <= 0;
		cmd_bl <= 0;
		cmd_instr <= 0;
		rd_count_reserved <= 0;
		rd_count_not_bursted <= 0;
	end else begin: cmd
	
		reg [6:0] rd_count_reserved_temp;
	
		if (cs_rd_count_w) begin
			rd_count_not_bursted <= do_peri;
		end
	
		if (rd_en)
			rd_count_reserved_temp = rd_count_reserved-1;
		else
			rd_count_reserved_temp = rd_count_reserved;
		
	
		if (wr_cmd_counter==7 && |wr_not_bursted) begin
			cmd_bl <= wr_not_bursted - 1;
			cmd_instr <= {1'b0, auto_precharge, 1'b0};
			cmd_en <= 1;
			wr_burstcommmit<=1;
		end else if (|rd_count_not_bursted && ~cs_rd_count_w) begin
			//Read 8 word bursts if there is space
			if (rd_count_reserved<(64-8)) begin
			
				//Empty FIFO? Issue one big read request
				//If we don't do this, we issue too many small read requests in the beginning
				//and the cmd fifo will overflow
				if (~|rd_count_reserved) begin
					if (|rd_count_not_bursted[17:6]) begin //>=64 words left?
						cmd_bl <= 63;
						rd_count_reserved_temp = 64;
						rd_count_not_bursted <= rd_count_not_bursted-64;
					end else begin
						cmd_bl <= rd_count_not_bursted-1;
						rd_count_reserved_temp = rd_count_not_bursted;
						rd_count_not_bursted <= 0;
					end
				
				//>=8 words left to read?
				end else if (|rd_count_not_bursted[17:3]) begin
					cmd_bl <= 7; //read 8 words
					rd_count_reserved_temp = rd_count_reserved_temp+8;
					rd_count_not_bursted <= rd_count_not_bursted-8;
				end else begin
					//read the rest
					cmd_bl <= rd_count_not_bursted-1;
					rd_count_reserved_temp = rd_count_reserved_temp+rd_count_not_bursted;
					rd_count_not_bursted <= 0;
				end
				cmd_instr <= {1'b0, auto_precharge, 1'b1};
				cmd_en <= 1;
			end else
				cmd_en<=0;
			wr_burstcommmit<=0;
		end else begin
			cmd_en <= 0;
			wr_burstcommmit<=0;
		end
		
		rd_count_reserved <= rd_count_reserved_temp;
	end

end


assign debug[0]=cs_control_r;
assign debug[1]=cs_control_w;
assign debug[2]=cs_status_r;
assign debug[3]=cs_status_w;
assign debug[4]=cs_address_low_r;
assign debug[5]=cs_address_low_w;
assign debug[6]=cs_address_high_r;
assign debug[7]=cs_address_high_w;
assign debug[8]=cs_rd_count_r;
assign debug[9]=cs_rd_count_w;
assign debug[10]=cs_data_low_r;
assign debug[11]=cs_data_low_w;
assign debug[12]=cs_data_high_r;
assign debug[13]=cs_data_high_w;
assign debug[31:14] = di_peri;
assign debug[49:32] = do_peri;
assign debug[50] = cmd_empty;
assign debug[51] = cmd_full;
assign debug[58:52] = wr_count;
assign debug[59] = wr_full;
assign debug[60] = wr_empty;
assign debug[61] = wr_underrun;
assign debug[62] = wr_error;
assign debug[63] = rd_full;
assign debug[64] = rd_empty;
assign debug[71:65] = rd_count;
assign debug[72] = rd_overflow;
assign debug[73] = rd_error;
assign debug[74] = calib_done;
assign debug[106:75] = rd_data;
assign debug[107]                   = cmd_en;
assign debug[110:108]                 = cmd_instr;
assign debug[117:111]                = cmd_bl;
assign debug[146:118]               = cmd_byte_addr;
assign debug[147]                  = wr_en;
assign debug[148]                  = rd_en;
assign debug[152:149]               = wr_mask;
assign debug[184:153]               = wr_data;
assign debug[185] = auto_precharge;
assign debug[186] = low_only;
assign debug[187] = data_layout;
assign debug[194:188] = rd_count_reserved;
assign debug[212:195] = rd_count_not_bursted;
endmodule
