# store script location as module directory
set mi_7series_vivado_module_dir [file dirname [file normalize [info script]]]

proc build_mig_v4_0 { design_name params } {
	variable mi_7series_vivado_module_dir
	variable projectDir

	# set default parameter values
	array set optional [list \
		-MIG_PRJ_FILENAME "nexys-video_axi.mig.prj" \
		-SystemClock "No Buffer" \
		-ReferenceClock "No Buffer" \
		-TimePeriod 2500 \
		-InputClkFreq 80000000 \
		-MMCM_VCO 800000000 \
		-PHYRatio "4:1" \
		-SysResetPolarity "ACTIVE LOW" \
		-Ordering "Normal" \
		-UserMemoryAddressMap "BANK_ROW_COLUMN" \
		-mrBurstType "Sequential" \
		-C0_C_RD_WR_ARB_ALGORITHM "RD_PRI_REG" \
		-C0_S_AXI_ADDR_WIDTH 32 \
		-C0_S_AXI_DATA_WIDTH 32 \
		-C0_S_AXI_ID_WIDTH 4 \
		-C0_S_AXI_SUPPORTS_NARROW_BURST 0 \
		{*}$params]

	# map parameters
	set MIG_PRJ_FILENAME $optional(-MIG_PRJ_FILENAME)

	# open mig template and new file
	set MIG_PRJ_FILENAME_OUTPUT $projectDir/${design_name}_mig.prj
	file mkdir [file dirname $MIG_PRJ_FILENAME_OUTPUT]
	set fd_mig_prj_template [open $mi_7series_vivado_module_dir/$MIG_PRJ_FILENAME "r"]
	set fd_mig_prj_output [open $MIG_PRJ_FILENAME_OUTPUT w]

	# define names and values to replace in mig.prj (xml syntax)
	array set replace {}
	set replace(SystemClock)    $optional(-SystemClock)
	set replace(ReferenceClock) $optional(-ReferenceClock)
	set replace(TimePeriod)     $optional(-TimePeriod)
	set replace(InputClkFreq)   [format "%g" [expr ($optional(-InputClkFreq)/(10**3))/(10.0**3)]] ;# take max 3 decimals (UNLIKE core generator user interface) and round down
	set replace(MMCM_VCO)       [expr $optional(-MMCM_VCO)/(10**6)] ;# don't take decimals
	set replace(PHYRatio) $optional(-PHYRatio)
	set replace(SysResetPolarity) $optional(-SysResetPolarity)
	set replace(Ordering)       $optional(-Ordering)
	set replace(UserMemoryAddressMap) $optional(-UserMemoryAddressMap)
	set replace(mrBurstType)    $optional(-mrBurstType)
	set replace(C0_C_RD_WR_ARB_ALGORITHM)  $optional(-C0_C_RD_WR_ARB_ALGORITHM)
	set replace(C0_S_AXI_ADDR_WIDTH)       $optional(-C0_S_AXI_ADDR_WIDTH)
	set replace(C0_S_AXI_DATA_WIDTH)       $optional(-C0_S_AXI_DATA_WIDTH)
	set replace(C0_S_AXI_ID_WIDTH)         $optional(-C0_S_AXI_ID_WIDTH)
	set replace(C0_S_AXI_SUPPORTS_NARROW_BURST) $optional(-C0_S_AXI_SUPPORTS_NARROW_BURST)

	# do some xml magic
	while {[gets $fd_mig_prj_template line] >= 0} {
		foreach tag_name [array names replace] {
			if {[string match *$tag_name* $line]} {
				regsub -- "()>.*<()" $line ">$replace($tag_name)<" line_replaced
				set line "$line_replaced<!-- modified by tcl script -->"
			}
		}
		if {[string match *IMPORTANT*MIG\ GUI* $line]} {
			# get rid of MIG warning
			set line "<!-- Generated mig.prj file for SpartanMC based on $mi_7series_vivado_module_dir/$MIG_PRJ_FILENAME -->"
		}
		puts $fd_mig_prj_output $line
	}
	close $fd_mig_prj_template
	close $fd_mig_prj_output

	# create ip core
	create_ip -name mig_7series -vendor xilinx.com -module_name $design_name

	# mig generator needs a current dir ([pwd] ?)
	cd $projectDir

	set_property -dict [list \
		CONFIG.XML_INPUT_FILE $MIG_PRJ_FILENAME_OUTPUT \
		CONFIG.RESET_BOARD_INTERFACE {Custom} \
		CONFIG.MIG_DONT_TOUCH_PARAM {Custom} \
		CONFIG.BOARD_MIG_PARAM {Custom}] \
		[get_ips $design_name]

	# generate verilog files (optional, is done automatically for synthesis)
	generate_target all [get_ips $design_name] -force -quiet
}
