/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

withType("mi_7series", { module ->
	// there should probably be only one such module in the design (TODO: check that)

	// select a mig.prj file depending on the target and
	// preset the parameter values like in that MIG project (MIG = Memory Interface Generator)
	// and define a format string for the pin names of each port:

	def String migPrjFileName
	def Map<String,String> pinFormats
	
	switch(document.target.targetName.toLowerCase()) {
	case "nexys-video":
		migPrjFileName = "nexys-video_axi.mig.prj"

		def params = module.parameters

		params["SYS_CLK_AS_REF_CLK"] = false //allow the use of different clocks, can be the same clock anyway
		params["DIFFERENTIAL_SYS_CLK"] = false
		params["DIFFERENTIAL_REF_CLK"] = false
		params["SINGLE_ENDED_REF_CLK"] = true
		params["INTERNAL_TEMP_MON"] = true

		params["USE_CS_PORT"] = false

		params["CK_WIDTH"] = 1
		params["CKE_WIDTH"] = 1
		params["BANK_WIDTH"] = 3
		params["ROW_WIDTH"] = 15
		params["DQ_WIDTH"] = 16
		params["DQS_WIDTH"] = 2
		params["DM_WIDTH"] = 2
		params["ODT_WIDTH"] = 1

		pinFormats = [
			ddr3_ck_p : "DDR3_CLK%d_P",
			ddr3_ck_n : "DDR3_CLK%d_N",
			ddr3_cke : "DDR3_CKE%d",
			ddr3_reset_n : "DDR3_RESET",
			ddr3_ba : "DDR3_BA%d",
			ddr3_addr : "DDR3_ADDR%d",
			ddr3_ras_n : "DDR3_RAS",
			ddr3_cas_n : "DDR3_CAS",
			ddr3_dq : "DDR3_DQ%d",
			ddr3_dqs_n : "DDR3_DQS%d_N",
			ddr3_dqs_p : "DDR3_DQS%d_P",
			ddr3_we_n : "DDR3_WE",
			ddr3_dm : "DDR3_DM%d",
			ddr3_odt : "DDR3_ODT"
			// clock inputs could be here, too,
			// but can't be auto-connected on this target because of missing 200MHz reference clock pin(s)
		]
	break
	default:
		print "Target is not configured for the $module.moduleDescription.name module yet."
		// TODO: error?
	return
	}

	// set the parameter selecting the mig.prj file path
	module.parameters["MIG_PRJ_FILENAME"] = migPrjFileName

	// connect module ports with pins like described in pinFormats:
	pinFormats.each {
		port = module.ports[it.key]
		pinFormat = it.value
		if (port.connectorWidth > 1)
			connectMultiple(port, 0, document, { String.format(pinFormat, it) })
		else
			connect(port, document.pins[String.format(pinFormat, 0)])
	}

	// add proc_sys_reset module
	def reset_module = document.findByType("proc_sys_reset")
	if (!reset_module.empty && reset_module.size() == 1) {
		println "Using existing reset module"
		reset_module = reset_module[0]
	} else if (!reset_module.empty) {
		println "There is more than one reset module. $module.moduleDescription.name module was not autoconnected."
		return
	} else {
		reset_module = module.getParent() + "proc_sys_reset"
	}

	// find spartanmc core
	def spartan = document.findByType("spartanmc")
	if (spartan.empty || spartan.size() != 1) {
		println "Autoconnect of $module.moduleDescription.name module works only with one SpartanMC core."
		return
	}

	// change connections
	def spartan_locked = spartan[0].getPort("locked")
	def spartan_reset = spartan_locked.getSingleConnectedPort()
	def spartan_clock = spartan[0].getPort("clk").getSingleConnectedPort()
	if (spartan_reset == null || spartan_clock == null) {
		println "SpartanMC is not connected to a clock. $module.moduleDescription.name module was not autoconnected."
		return
	} else if (!spartan_locked.isConnectedTo(reset_module.getPort("Peripheral_aresetn"))) {
		println "Confused by reset connection. $module.moduleDescription.name module was not autoconnected."
		return
	}
	spartan_locked.disconnectAll()
	connectIfConnectable(spartan_locked, module.getPort("init_calib_complete"))
	connectIfConnectable(reset_module.getPort("Peripheral_aresetn"), module.getPort("aresetn"))
	connectIfConnectable(reset_module.getPort("Ext_Reset_In").getSingleConnectedPort(), module.getPort("sys_rst"))
	connectIfConnectable(module.getPort("mmcm_locked"), reset_module.getPort("Dcm_locked"))
	connectIfConnectable(spartan_clock, module.getPort("sys_clk_i"))

})
