/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

withType("mi_7series_vivado", { module ->

	def mem_clk = module.parameters["MEM_CLK"] as Long

	if (mem_clk >= 399_999_990) {
		print "Set CLKOUT0_PHASE = 337.5"
		module.parameters["CLKOUT0_PHASE"] = 337.5d
	}
	else {
		// if ((target uses HP banks) and (memory not low voltage))
		// then CLKOUT0_PHASE = 315
		// else CLKOUT0_PHASE = 0
		switch (document.target.targetName) {
			case "Nexys-Video":
				print "Set CLKOUT0_PHASE = 0.0"
				module.parameters["CLKOUT0_PHASE"] = 0.0d
				break
			default:
				print "onParameterChanged script is not configured for this target yet."
		}
	}

	if (module.parameters["nCK_PER_CLK"].getValue().get() == "2") {
		module.parameters["nCK_PER_CLK_string"] = "2:1"
	} else {
		module.parameters["nCK_PER_CLK_string"] = "4:1"
	}

	// change port configuration depending on *CLK_TYPE
	// for jConfig, there is no difference between NO_BUFFER and SINGLE_ENDED
	switch (module.parameters["SYSCLK_TYPE"].getValue().get()) {
		case "DIFFERENTIAL":
			module.parameters["DIFFERENTIAL_SYS_CLK"] = true
			module.ports["sys_clk_p"].portDescription.required = true
			module.ports["sys_clk_n"].portDescription.required = true
			break
		default:
			module.parameters["DIFFERENTIAL_SYS_CLK"] = false
			module.ports["sys_clk_i"].portDescription.required = true
	}

	switch (module.parameters["REFCLK_TYPE"].getValue().get()) {
		case "DIFFERENTIAL":
			module.parameters["DIFFERENTIAL_REF_CLK"] = true
			module.parameters["SINGLE_ENDED_REF_CLK"] = false
			module.parameters["SYS_CLK_AS_REF_CLK"] = false
			module.ports["clk_ref_p"].portDescription.required = true
			module.ports["clk_ref_n"].portDescription.required = true
			break
		case "USE_SYSTEM_CLOCK":
			module.parameters["DIFFERENTIAL_REF_CLK"] = false
			module.parameters["SINGLE_ENDED_REF_CLK"] = false
			module.parameters["SYS_CLK_AS_REF_CLK"] = true
			break;
		default:
			module.parameters["DIFFERENTIAL_REF_CLK"] = false
			module.parameters["SINGLE_ENDED_REF_CLK"] = true
			module.parameters["SYS_CLK_AS_REF_CLK"] = false
			module.ports["clk_ref_i"].portDescription.required = true
	}

})
