/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

withType("mig", { module ->
	// there should probably be only one such module in the design (TODO: check that)

	// select a mig.prj file depending on the target and
	// preset the parameter values like in that MIG project (MIG = Memory Interface Generator)
	// and define a format string for the pin names of each port:

	def String migPrjFileName
	def Map<String,String> pinFormats
	
	switch(document.target.targetName.toLowerCase()) {
	case "sp605":
		module.parameters["C_S_AXI_DATA_WIDTH"] = 32
		migPrjFileName = "sp605_axi_32bit.mig.prj"

		pinFormats = [
			c3_sys_clk_p: "CLOCK_SYSCLK_P",
			c3_sys_clk_n: "CLOCK_SYSCLK_N",
			mcb3_rzq: "FPGA_ONCHIP_TERM2_RZQ",
			mcb3_zio: "FPGA_ONCHIP_TERM1_ZIO",

			mcb3_dram_ck : "MEM1_CLK_P",
			mcb3_dram_ck_n : "MEM1_CLK_N",
			mcb3_dram_cke : "MEM1_CKE",
			mcb3_dram_reset_n : "MEM1_RESET_B",
			mcb3_dram_ba : "MEM1_BA%d",
			mcb3_dram_a : "MEM1_A%d",
			mcb3_dram_ras_n : "MEM1_RAS_B",
			mcb3_dram_cas_n : "MEM1_CAS_B",
			mcb3_dram_dq : "MEM1_DQ%d",
			mcb3_dram_dqs_n : "MEM1_LDQS_N",
			mcb3_dram_dqs : "MEM1_LDQS_P",
			mcb3_dram_udqs_n : "MEM1_UDQS_N",
			mcb3_dram_udqs : "MEM1_UDQS_P",
			mcb3_dram_we_n : "MEM1_WE_B",
			mcb3_dram_dm : "MEM1_LDM",
			mcb3_dram_udm : "MEM1_UDM",
			mcb3_dram_odt : "MEM1_ODT"
		]
		break
	default:
		print "Target is not configured for the $module.moduleDescription.name module yet."
		// TODO: error?
		return
	}

	// set the parameter selecting the mig.prj file path
	module.parameters["MIG_PRJ_FILENAME"] = migPrjFileName

	// connect module ports with pins like described in pinFormats:
	pinFormats.each {
		port = module.ports[it.key]
		pinFormat = it.value
		if (port.connectorWidth > 1)
			connectMultiple(port, 0, document, { String.format(pinFormat, it) })
		else
			connect(port, document.pins[String.format(pinFormat, 0)])
	}
})
