/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// TU-Dresden Chair for Embedded Systems
//////////////////////////////////////////////////////////////////////////////////

//Geschrieben von: Markus Noll, mail@markus-noll.de, TU-Darmstadt 2015
module dispatcher_slave
#(
	parameter BASE_ADR		= 10'h0,	
  	parameter SLAVE_ID		= 'd0
)
(
	//spmc interface
	input   wire         	 clk_peri,
	input   wire [17:0]  	 do_peri,
	output  wire [17:0]		 di_peri,
	input   wire [9:0]   	 addr_peri,
	input   wire         	 access_peri,
	input   wire         	 wr_peri,
	input   wire          	 reset,
	
	//interface to dispatcher master
	input  wire   [17:0]	data_in,
	input  wire   [5:0]		receiver_id_in,
	input  wire   [11:0]	msg_size_in,
	input  wire				status_in,

	output wire		[11:0]	msg_size_out,
	output wire  			read,
	output wire  			received
);


/////////////////////////////////////////////////////////////////////////////////////
//
// local functions and parameters
//

localparam
   
    P_STATUS			= 0,
    P_MSG_SIZE			= 1,
    P_DATA_OUT     		= 2,
		P_MSG_SIZE_IN	= 3,
    P_RECEIVED       	= 4,
    
    ALL_PORTS			= 5,
    DECODSIM_DEB		= "NO",
    FIFO_ADDR_BITS   	= 1;


//event wires
wire							select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

wire    status_read_sel 		= reg_read[P_STATUS];
wire    msg_size_read_sel 		= reg_read[P_MSG_SIZE];
wire    data_out_read_sel 		= reg_read[P_DATA_OUT];
wire    msg_size_in_read_sel 	= reg_read[P_MSG_SIZE_IN];
wire 	received_read_sel 		= reg_read[P_RECEIVED];

wire    status_write_sel		= reg_write[P_STATUS];
wire    msg_size_write_sel 		= reg_write[P_MSG_SIZE];
wire    data_out_write_sel 		= reg_write[P_DATA_OUT];
wire    msg_size_in_write_sel 	= reg_write[P_MSG_SIZE_IN];
wire    received_write_sel 		= reg_write[P_RECEIVED];

reg_access_decoder	
#(
	.REG_COUNT(ALL_PORTS),
	.BASE_ADR(BASE_ADR),
	.SIM_DEBUG(DECODSIM_DEB)
)
decod1			
(
	.reset(reset),
	.clk_peri(clk_peri),
	.addr_peri(addr_peri),
	.access_peri(access_peri),
	.wr_peri(wr_peri),
	.peri_select(select),
	.reg_select(reg_select),
	.reg_read(reg_read),
	.reg_write(reg_write)
);

//read and received control signals to master
assign read 	= data_out_read_sel;
assign received = received_write_sel;

//spmc peri bus output
assign di_peri[17:0] = 	(status_read_sel && status_in && receiver_id_in == SLAVE_ID) ? 18'd1 :
						(msg_size_in_read_sel) ? {6'b0, msg_size_in} :
						(data_out_read_sel) ? data_in : 18'd0;

//msg_size register logic
reg [11:0] msg_size;
assign msg_size_out = msg_size;

always @ (posedge clk_peri) begin
	if (reset) begin
		msg_size <= 12'b0;
	end else begin
		if (msg_size_write_sel) begin
			msg_size <= do_peri[11:0];
		end
	end
end

endmodule
