/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// TU-Dresden Chair for Embedded Systems
//////////////////////////////////////////////////////////////////////////////////

//Geschrieben von: Markus Noll, mail@markus-noll.de, TU-Darmstadt 2015


//Aufbau & Funktion siehe Thesis
module load_balancing_arbiter#(

	parameter NUMBER_OF_SLAVES = 2,
	parameter FIFO_ADDR_BITS = 5
)
(
	input wire [NUMBER_OF_SLAVES*FIFO_ADDR_BITS-1:0] data_in,
	output wire [5:0] slave_id

);
	 
	 wire [FIFO_ADDR_BITS-1:0] out_cable;
	 
	 genvar i;
	 generate
	 
	 //wenn nur ein Slaves, immer diesen Slave auswählen
	 if (NUMBER_OF_SLAVES == 1) begin 
		assign slave_id = 6'd0;
	
	
	 //wenn nur zwei Slaves, dann nur ein Vergleicher	
	 end else if (NUMBER_OF_SLAVES == 2) begin
	 
		vergleicher #(
		FIFO_ADDR_BITS
		)vgl(
		.a(data_in[FIFO_ADDR_BITS-1:0]),
		.b(data_in[FIFO_ADDR_BITS +: FIFO_ADDR_BITS]),
		.a_id(6'd0),
		.b_id(6'd1),
		.out(out_cable),
		.out_id(slave_id)
		);
		
		
	 //bei mehr als 2 Slaves, baue Vergleichernetzwerk auf
	 end else begin
		
		//Der erste Vergleicher
		wire [NUMBER_OF_SLAVES*FIFO_ADDR_BITS:0] values;
		wire [NUMBER_OF_SLAVES*6:0] ids;
		
		vergleicher #(
		FIFO_ADDR_BITS
		)vgl(
		.a(data_in[FIFO_ADDR_BITS-1:0]),
		.b(data_in[FIFO_ADDR_BITS +: FIFO_ADDR_BITS]),
		.a_id(6'd0),
		.b_id(6'd1),
		.out(values[0 +: FIFO_ADDR_BITS]),
		.out_id(ids [0 +: 6])
		);
		
		//Füge weitere Vergleicher hinzu
		for (i=2;i<NUMBER_OF_SLAVES;i=i+1) begin:vergleicher
		
		vergleicher #(
		FIFO_ADDR_BITS
		)vgl(
		.a(values[(i-2)*FIFO_ADDR_BITS +: FIFO_ADDR_BITS]),
		.b(data_in[i*FIFO_ADDR_BITS +: FIFO_ADDR_BITS]),
		.a_id(ids[(i-2)*6 +: 6]),
		.b_id(i[5:0]),
		.out(values[(i-1)*FIFO_ADDR_BITS +: FIFO_ADDR_BITS]),
		.out_id(ids [(i-1)*6 +: 6])
		);
		
		end
		assign slave_id = ids[(NUMBER_OF_SLAVES-2)*6 +: 6];
		
		end
	 
	 endgenerate


endmodule
