/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// TU-Dresden Chair for Embedded Systems
//////////////////////////////////////////////////////////////////////////////////

//Geschrieben von: Markus Noll, mail@markus-noll.de, TU-Darmstadt 2015
module rr_arbiter
#(
	parameter NUMBER_OF_SLAVES = 'd6
)
(
	input wire reset,
	input wire clk,	
	input wire last_value_write_sel,
	output wire [5:0] slave_id
	
);





wire [NUMBER_OF_SLAVES-1:0] req;
wire [(NUMBER_OF_SLAVES-1):0]grant;

encoder # (
	.NUMBER_OF_SLAVES(NUMBER_OF_SLAVES)
)
enc
(
	.reset(reset),
	.clk(clk),
	.one_hot(grant),
	.binary_encoded(slave_id)

);

//Alle Slaves auf 1 
assign req = 1000'd65535;


//Funktionale Beschreibung siehe Thesis

reg [NUMBER_OF_SLAVES-1:0] mask_logic_out_reg;

// AND-Gate
wire [NUMBER_OF_SLAVES-1:0] AND_OUT;
assign AND_OUT = req & mask_logic_out_reg;

//Mask-Logic => grant as input & mask_logic_output as output
wire [NUMBER_OF_SLAVES-1:0] mask_logic_middle;
wire [NUMBER_OF_SLAVES-1:0] mask_logic_output;
assign mask_logic_middle =  (grant[NUMBER_OF_SLAVES-1:0] | { mask_logic_middle[NUMBER_OF_SLAVES-2:0],1'b0 });
assign mask_logic_output = (grant[NUMBER_OF_SLAVES-1:0] == 10'd0) ? (10'd4095) : (mask_logic_middle^grant);


//unmasked priority arbiter => req as input & UPA_GRANT as output
wire [NUMBER_OF_SLAVES-1:0] UPA_GRANT;
wire [NUMBER_OF_SLAVES-1:0] c_UPA ;
assign c_UPA = {(~req[NUMBER_OF_SLAVES-2:0] & c_UPA[NUMBER_OF_SLAVES-2:0]),1'b1} ;
assign UPA_GRANT = req & c_UPA ; 

//masked priority arbiter => AND_OUT as input & MPA_GRANT as output
wire [NUMBER_OF_SLAVES-1:0] MPA_GRANT;
wire [NUMBER_OF_SLAVES-1:0] c_MPA ;
assign c_MPA = {(~AND_OUT[NUMBER_OF_SLAVES-2:0] & c_MPA[NUMBER_OF_SLAVES-2:0]),1'b1} ;
assign MPA_GRANT = AND_OUT & c_MPA ;

//Multiplexor => grant as output 
assign grant = (AND_OUT == 10'd0) ? UPA_GRANT : MPA_GRANT;


//Control mask logic
always @(posedge clk) begin

	if (reset) begin
		mask_logic_out_reg <= 1000'd0;		
	end else if (last_value_write_sel) begin
		mask_logic_out_reg <= mask_logic_output;
	end 
	
end




 
endmodule

