/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// TU-Dresden Chair for Embedded Systems
//////////////////////////////////////////////////////////////////////////////////

//Geschrieben von: Markus Noll, mail@markus-noll.de, TU-Darmstadt 2015
module dispatcher_master_dma
#(
        // SPMC Interface
        parameter NUMBER_OF_SLAVES = 'd3,      
        parameter DMA_BASE_ADR			= 18'h0,
        parameter DMA_SIZE				= 'd64,
        parameter DMA_WIDTH				= 'd18		
)
(
//*** Connections with SpartanMC master ***
        input   wire            clk_peri,	//Systemtakt
        input   wire            clk_dma,	//DMAtakt
        input   wire            reset,		//Reset-Signal
        
         //DMA-Signals
		input wire						mem_access,
		input wire						store_access_low,			//TODO
		input wire						store_access_high,			//TODO
		input wire	[9:0]				addr_peri,
		input wire	[7:0]				addr_high,
		input wire	[17:0]				do_peri,
		output wire	[17:0]				di_peri, 
	
	//interface to dispatcher slaves
	output wire[17:0]	data_out,
	output reg [5:0]	receiver_id_out,
	output reg [11:0]	msg_size_out,
	output wire			status_out,

	input wire [((NUMBER_OF_SLAVES*12)-1):0]	msg_size_in,
	input wire [NUMBER_OF_SLAVES-1:0]		read,
	input wire [NUMBER_OF_SLAVES-1:0] 		received
);
/////////////////////////////////////////////////////////////////////////////////////
//
// Local Functions and Parameters
//
function integer log2;
input integer value;
	for (log2=0; value>0; log2=log2+1)
	value = value>>1;
endfunction

parameter
	DECODSIM_DEB			= "NO",
    DMA_ADDRWIDTH          = log2(DMA_SIZE - 1);

//inter wires & regs
wire [17:0] data_out_a;
wire select;					//pselect zu Bram
wire [17:0] full_addr;
assign full_addr = {addr_high,addr_peri};
//Bram Wires
wire write;
wire [17:0] doutb;
wire [17:0] dinb;
reg [DMA_ADDRWIDTH-1:0] addrb_reg;
reg [DMA_ADDRWIDTH-1:0] addrb_reg_delay;
wire [DMA_ADDRWIDTH-1:0]addrb;
wire store_access;
assign store_access = store_access_low || store_access_high;
//FSM
reg [3:0] state;
reg [11:0] counter;
localparam [3:0]
	IDLE				= 0,
	GET_COUNT			= 1,
	GET_DATA_ADDR	 	= 2,
	GET_SLAVE_ID		= 3,
	SEND_HEADER			= 4,
	SEND				= 5,
	FIN					= 6;
// Dekodierung der Adressen ********************** START *************************************
//pselect module to generate select signal for DMA memory
pselect #(
	.ADDR_WIDTH(18-DMA_ADDRWIDTH),
	.BASE_WIDTH(18-DMA_ADDRWIDTH),
	.BASE_ADDR(DMA_BASE_ADR[17:DMA_ADDRWIDTH])
	) pselect_0 (
	.addr(full_addr[17:DMA_ADDRWIDTH]),
	.activ_peri(mem_access),
	.select(select)
);

//Bram--evtl. nochmal in axi nach der 2ten Methode schauen
dual_ported_RAM #(
	.WIDTH(DMA_WIDTH),
	.SIZE(DMA_SIZE),
	.ADDRWIDTH(DMA_ADDRWIDTH)
	) DMARAM00 (
	.clkA(clk_peri), 
	.clkB(clk_dma), 
	.enA(1'b1), 
	.enB(1'b1), 
.weA((store_access&&select)), 
	.weB(write),
	.rstA(!select), 
	.addrA(full_addr[DMA_ADDRWIDTH-1:0]), 
	.addrB(addrb), 
	.diA(do_peri), 
	.diB(18'd1),
	.doA(data_out_a), 
	.doB(doutb)
);
//reg select_delay;
//always @(posedge clk_peri)begin
//	select_delay<=select;
//end
//assign di_peri=(select_delay)?data_out_a:18'b0;
assign di_peri=data_out_a;
//FSM		
always @(posedge clk_dma)begin
	if(reset)begin
		counter <= 0;
		addrb_reg <= 10'd0;
		state<=IDLE;
	end else begin
		case(state)
			IDLE:begin
				state<=(doutb[1]==1'b1) ? GET_COUNT : IDLE;
			end
			GET_COUNT:begin
				msg_size_out<=doutb[11:0];
				counter<=doutb[11:0];
				state<=GET_DATA_ADDR;
			end
			GET_DATA_ADDR:begin
				addrb_reg_delay<=doutb[DMA_ADDRWIDTH-1:1];
				addrb_reg<=doutb[DMA_ADDRWIDTH-1:1]+1;
				state<=GET_SLAVE_ID;
			end
			GET_SLAVE_ID:begin
				receiver_id_out<=doutb[5:0];
				state<=SEND;
			end
			SEND_HEADER:begin
				state<=(read) ? SEND : SEND_HEADER;
				msg_size_out<=(read) ? msg_size_out : counter;
				
			end
			SEND:begin
				counter<=(read) ? counter-1 : counter;
				state<= (read)&&(counter==1) ? FIN : SEND;
				addrb_reg_delay<=(read) ? addrb_reg : addrb_reg_delay;
				addrb_reg<=(read) ?
							(counter==1) ? 0 : addrb+1 : addrb_reg;

			end
			FIN:begin
				receiver_id_out<=9'bz;
				msg_size_out<=0;
				state<=IDLE;
			end
			default:begin
				counter <= 0;
				addrb_reg <= 10'd0;
				state<=IDLE;
			end
		endcase
	end
end

assign addrb = (state == IDLE && ~doutb[1]==1'b1) ? 0 :
				(state == IDLE && doutb[1]==1'b1) ? 1 :
				(state == GET_COUNT) ? 2 :
				(state == GET_DATA_ADDR) ? 3 :
				(state == GET_SLAVE_ID) ? doutb[DMA_ADDRWIDTH-1:1] :
				(state == SEND_HEADER) ? addrb_reg_delay :
				(state == SEND && read) ? addrb_reg:  
				(state == SEND && ~read) ? addrb_reg_delay: 
				(state == FIN) ? 0 : 18'bz;

wire send;
assign send = read && (state == SEND || state == SEND_HEADER);
assign data_out = (send) ? doutb : 18'bz;
assign write = (state == FIN || reset) ? 1 : 0;
assign status_out = (state == SEND || state == SEND_HEADER);

//do_peri[8:0] <  254) ? {do_peri} :						//direct id assignment
//do_peri[8:0] == 254) ? {do_peri[17:9],slave_id_rr} :	//round robin arbitration
//do_peri[8:0] == 255) ? {do_peri[17:9],slave_id_lb} :	//load balancing arbitration

endmodule
