/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// TU-Dresden Chair for Embedded Systems
//////////////////////////////////////////////////////////////////////////////////

//Geschrieben von: Markus Noll, mail@markus-noll.de, TU-Darmstadt 2015
module dispatcher_slave_dma
#(
        // SPMC Interface
        parameter SLAVE_ID				= 'd0,   
        parameter DMA_BASE_ADR			= 18'h0,
        parameter DMA_SIZE				= 'd64,
        parameter DMA_WIDTH				= 'd18		
)
(
//*** Connections with SpartanMC master ***
        input   wire            clk_peri,	//Systemtakt
        input   wire            clk_dma,	//DMAtakt
        input   wire            reset,		//Reset-Signal
        
         //DMA-Signals
		input wire						mem_access,
		input wire						store_access_low,			//TODO
		input wire						store_access_high,			//TODO
		input wire	[9:0]				addr_peri,
		input wire	[7:0]				addr_high,
		input wire	[17:0]				do_peri,
		output wire	[17:0]				di_peri, 
	
		//interface to dispatcher master
		input  wire   [17:0]	data_in,
		input  wire   [5:0]		receiver_id_in,
		input  wire   [11:0]	msg_size_in,
		input	 wire			status_in,

		output reg		[11:0]	msg_size_out,
		output wire  			read,
		output wire  			received
);


/////////////////////////////////////////////////////////////////////////////////////
//
// Local Functions and Parameters
//
function integer log2;
input integer value;
	for (log2=0; value>0; log2=log2+1)
	value = value>>1;
endfunction

parameter
	DECODSIM_DEB			= "NO",
    DMA_ADDRWIDTH          = log2(DMA_SIZE - 1);

//inter wires & regs
wire [17:0] data_out_a;
wire select;					//pselect zu Bram
wire [17:0] full_addr;
assign full_addr = {addr_high,addr_peri};
//Bram Wires
wire write;
wire [17:0] doutb;
wire [17:0] dinb;
reg [DMA_ADDRWIDTH-1:0] addrb_reg;
reg [DMA_ADDRWIDTH-1:0] addrb_reg_delay;
wire [DMA_ADDRWIDTH-1:0]addrb;
wire store_access;
assign store_access = store_access_low || store_access_high;
//FSM
reg [3:0] state;
reg [11:0] counter;
reg [17:0] msg_size_addr;
wire rec;
localparam [3:0]
	IDLE				= 0,
	GET_MSG_SIZE_ADDR	= 1,
	GET_DATA_ADDR	 	= 2,
	REC_HEADER			= 3,
	REC_DATA			= 4,
	FIN					= 5;
// Dekodierung der Adressen ********************** START *************************************
//pselect module to generate select signal for DMA memory
pselect #(
	.ADDR_WIDTH(18-DMA_ADDRWIDTH),
	.BASE_WIDTH(18-DMA_ADDRWIDTH),
	.BASE_ADDR(DMA_BASE_ADR[17:DMA_ADDRWIDTH])
	) pselect_0 (
	.addr(full_addr[17:DMA_ADDRWIDTH]),
	.activ_peri(mem_access),
	.select(select)
);

//Bram--evtl. nochmal in axi nach der 2ten Methode schauen
dual_ported_RAM #(
	.WIDTH(DMA_WIDTH),
	.SIZE(DMA_SIZE),
	.ADDRWIDTH(DMA_ADDRWIDTH)
	) DMARAM00 (
	.clkA(clk_peri), 
	.clkB(clk_dma), 
	.enA(1'b1), 
	.enB(1'b1), 
.weA((store_access&&select)), 
	.weB(write),
	.rstA(!select), 
	.addrA(full_addr[DMA_ADDRWIDTH-1:0]), 
	.addrB(addrb), 
	.diA(do_peri), 
	.diB(dinb),
	.doA(data_out_a), 
	.doB(doutb)
);
//reg select_delay;
//always @(posedge clk_peri)begin
//	select_delay<=select;
//end
//assign di_peri=(select_delay)?data_out_a:18'b0;
assign di_peri=data_out_a;
//FSM		
always @(posedge clk_dma)begin
	if(reset)begin
		counter <= 0;
		addrb_reg <= 10'd0;
		state<=IDLE;
	end else begin
		case(state)
			IDLE:begin
				state<=(doutb[1]==1'b1) ? GET_MSG_SIZE_ADDR : IDLE;
			end
			GET_MSG_SIZE_ADDR:begin
				msg_size_addr<=doutb[DMA_ADDRWIDTH-1:1];
				state<=GET_DATA_ADDR;
			end
			GET_DATA_ADDR:begin
				addrb_reg<=doutb[DMA_ADDRWIDTH-1:1];
				msg_size_out<=1;
				state<=REC_HEADER;
			end
			REC_HEADER:begin
				counter<=(rec) ? msg_size_in : 0;
				state<=(rec) ? REC_DATA : REC_HEADER;
			end
			REC_DATA:begin
				counter<=(rec) ? counter-1 : counter;
				state<=(counter==1) && (rec) ? FIN : REC_DATA;
				addrb_reg<=(rec) ? 
							(counter==1) ? 0 : addrb_reg+1 : addrb_reg ;
			end
			FIN:begin
				msg_size_out<=0;
				state<=IDLE;
			end
			default:begin
				counter <= 0;
				addrb_reg <= 10'd0;
				state<=IDLE;
			end
		endcase
	end
end
assign rec = (status_in==1)&&(receiver_id_in==SLAVE_ID);
assign addrb = (state == IDLE && ~doutb[1]==1'b1) ? 0 :
				(state == IDLE && doutb[1]==1'b1) ? 1 :
				(state == GET_MSG_SIZE_ADDR) ? 2 :
				(state == GET_DATA_ADDR) ? msg_size_addr :
				(state == REC_HEADER) ? msg_size_addr :
				(state == REC_DATA) ? addrb_reg:  
				(state == FIN) ? 0 : 18'bz;
assign write = (state == FIN || reset || rec&&(state == REC_HEADER || state == REC_DATA)) ? 1 : 0;
assign dinb = (reset || state==FIN) ? 1 :
				(state == REC_HEADER) ? msg_size_in : data_in;
assign read = rec && (state==REC_DATA);
assign received = state==FIN;

endmodule
