/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Engineer:       Andre Seidowski
// 
// Create Date:    11:22:08 10/10/2008 
// Module Name:    display_graph_function 
// Project Name:   Display-Controller fuer SpartanMC
//
// Description:    Top-Modul fuer alle Grafikfunktionen, das lediglich
//			die eingehenden Signale verteilt und die ausgehenden
//			entsprechend "muxt".
//
//////////////////////////////////////////////////////////////////////////////////
`include "display/src/display_graph_function_common.vh"

module display_graph_function #(
		parameter SCREEN_WIDTH = 320,
		parameter SCREEN_HEIGHT = 240,
		parameter BIT_PER_PIXEL = 3,
		parameter BIT_PER_MEMORY_PIXEL = 2,
		parameter PIXEL_PER_MEMORY_WORD = 4,
		parameter MEM_DATA_WIDTH = 9,
		
		parameter USE_SETPIXEL	= 1,
		parameter USE_LINE	= 1,
		parameter USE_COPYRECT	= 1,
		parameter USE_GETPIXEL	= 1,
		parameter USE_FILLRECT	= 1
	)(
		input wire	clk,
		input wire	reset,

		output wire	[ld(SCREEN_WIDTH*SCREEN_HEIGHT * BIT_PER_MEMORY_PIXEL / (PIXEL_PER_MEMORY_WORD*BIT_PER_MEMORY_PIXEL))-1:0] mem_addr,
		input wire	[MEM_DATA_WIDTH-1:0] mem_readData,
		output wire	[MEM_DATA_WIDTH-1:0] mem_writeData,
		output wire	mem_writeEnable,

		input wire [ld(SCREEN_WIDTH-1)-1:0]	coord_x1,
		input wire [ld(SCREEN_WIDTH-1)-1:0]	coord_x2,
		input wire [ld(SCREEN_WIDTH-1)-1:0]	coord_x3,
		input wire [ld(SCREEN_HEIGHT-1)-1:0]	coord_y1,
		input wire [ld(SCREEN_HEIGHT-1)-1:0]	coord_y2,
		input wire [ld(SCREEN_HEIGHT-1)-1:0]	coord_y3,

		input wire [BIT_PER_MEMORY_PIXEL-1:0] color,
		output wire [BIT_PER_MEMORY_PIXEL-1:0] colorOut,
		input wire optionalFlag1,

		input wire draw,
		output wire ready,

		input wire [ld(`NUMBER_OF_FUNCTIONS)-1:0] functionSelect
	);
	
	`include "display/src/ld.vh"
	
	localparam MEM_ADDR_WIDTH = ld(SCREEN_WIDTH*SCREEN_HEIGHT * BIT_PER_MEMORY_PIXEL / (PIXEL_PER_MEMORY_WORD*BIT_PER_MEMORY_PIXEL));
	
	// SetPixel
	wire pixel_ready;
	wire [BIT_PER_MEMORY_PIXEL-1:0] pixel_color;
	wire pixel_draw;
	wire [ld(SCREEN_WIDTH-1)-1:0] pixel_coord_x;
	wire [ld(SCREEN_HEIGHT-1)-1:0] pixel_coord_y;
	wire [MEM_ADDR_WIDTH-1:0] pixel_mem_addr;

	// Line
	wire line_ready;
	wire line_draw;
	wire [ld(SCREEN_WIDTH-1)-1:0] line_pixelx;
	wire [ld(SCREEN_HEIGHT-1)-1:0] line_pixely;
	wire line_pixelDraw;

	// CopyRect
	wire rect_ready;
	wire rect_draw;
	wire [MEM_ADDR_WIDTH-1:0] rect_mem_addr;
	wire [ld(SCREEN_WIDTH-1)-1:0] rect_pixelx;
	wire [ld(SCREEN_HEIGHT-1)-1:0] rect_pixely;
	wire [BIT_PER_MEMORY_PIXEL-1:0] rect_pixelColor;
	wire rect_pixelDraw;

	// GetPixel
	wire [MEM_ADDR_WIDTH-1:0] getpixel_mem_addr;

	// FillRect
	wire fill_ready;
	wire fill_draw;
	wire [ld(SCREEN_WIDTH-1)-1:0] fill_pixelx;
	wire [ld(SCREEN_HEIGHT-1)-1:0] fill_pixely;
	wire fill_pixelDraw;
	wire line_ready_out;
	
	assign line_ready_out = line_ready;
	
/* Modulinstanziierungen */

	generate
	begin: MODULE_INSTANCES
		if ((USE_SETPIXEL == 1) || (USE_LINE == 1) || (USE_COPYRECT == 1) || (USE_FILLRECT == 1))
		begin: INSTANCE_SETPIXEL
			display_graph_function_setpixel #(
					.SCREEN_WIDTH(SCREEN_WIDTH),
					.SCREEN_HEIGHT(SCREEN_HEIGHT),
					.BIT_PER_PIXEL(BIT_PER_PIXEL),
					.BIT_PER_MEMORY_PIXEL(BIT_PER_MEMORY_PIXEL),
					.PIXEL_PER_MEMORY_WORD(PIXEL_PER_MEMORY_WORD),
					.MEM_DATA_WIDTH(MEM_DATA_WIDTH)
				)
				SETPIXEL (
					.clk(clk),
					.reset(reset),
					.mem_addr(pixel_mem_addr),
					.mem_readData(mem_readData),
					.mem_writeData(mem_writeData),
					.mem_writeEnable(mem_writeEnable),
					.coord_x(pixel_coord_x),
					.coord_y(pixel_coord_y),
					.color(pixel_color),
					.draw(pixel_draw),
					.ready(pixel_ready)
				);
		end

		if (USE_LINE == 1)
		begin: INSTANCE_LINE
			display_graph_function_line #(
					.SCREEN_WIDTH(SCREEN_WIDTH),
					.SCREEN_HEIGHT(SCREEN_HEIGHT)
				)
				LINE(
					.clk(clk),
					.reset(reset),
					.xstart(coord_x1),
					.xend(coord_x2),
					.ystart(coord_y1),
					.yend(coord_y2),
					.draw(line_draw),
					.ready(line_ready),
					
					.pixelx(line_pixelx),
					.pixely(line_pixely),
					.pixelDraw(line_pixelDraw),
					.pixelReady(pixel_ready)
				);
		end

		if (USE_COPYRECT == 1)
		begin: INSTANCE_COPYRECT
			display_graph_function_copyrect #(
					.SCREEN_WIDTH(SCREEN_WIDTH),
					.SCREEN_HEIGHT(SCREEN_HEIGHT),
					.BIT_PER_MEMORY_PIXEL(BIT_PER_MEMORY_PIXEL),
					.PIXEL_PER_MEMORY_WORD(PIXEL_PER_MEMORY_WORD),
					.MEM_DATA_WIDTH(MEM_DATA_WIDTH)
				)
				COPYRECT (
					.clk(clk),
					.reset(reset),
		
					.mem_addr(rect_mem_addr),
					.mem_readData(mem_readData),
		
					.coord_src_x1(coord_x1), .coord_src_x2(coord_x2),
					.coord_src_y1(coord_y1), .coord_src_y2(coord_y2),
					.coord_dest_x(coord_x3), .coord_dest_y(coord_y3),
		
					.transparentColor(color),
					.copyTransparent(optionalFlag1),
		
					.draw(rect_draw),
					.ready(rect_ready),
		
					.pixelx(rect_pixelx),
					.pixely(rect_pixely),
					.pixelColor(rect_pixelColor),
					.pixelDraw(rect_pixelDraw),
					.pixelReady(pixel_ready)
				);
		end

		if (USE_GETPIXEL == 1)
		begin: INSTANCE_GETPIXEL
			display_graph_function_getpixel #(
					.SCREEN_WIDTH(SCREEN_WIDTH),
					.SCREEN_HEIGHT(SCREEN_HEIGHT),
					.BIT_PER_MEMORY_PIXEL(BIT_PER_MEMORY_PIXEL),
					.PIXEL_PER_MEMORY_WORD(PIXEL_PER_MEMORY_WORD),
					.MEM_DATA_WIDTH(MEM_DATA_WIDTH)
				)
				GETPIXEL (
					.mem_addr(getpixel_mem_addr),
					.mem_readData(mem_readData),
					.coord_x(coord_x1),
					.coord_y(coord_y1),
					.color(colorOut)
				);
		end

		if (USE_FILLRECT == 1)
		begin: INSTANCE_FILLRECT
			display_graph_function_fill #(
					.SCREEN_WIDTH(SCREEN_WIDTH),
					.SCREEN_HEIGHT(SCREEN_HEIGHT),
					.BIT_PER_MEMORY_PIXEL(BIT_PER_MEMORY_PIXEL)
				)
				FILLRECT (
					.clk(clk),
					.reset(reset),
					.pixelTopLeft_x(coord_x1),
					.pixelTopLeft_y(coord_y1),
					.pixelBottomRight_x(coord_x2),
					.pixelBottomRight_y(coord_y2),
					.pixelx(fill_pixelx),
					.pixely(fill_pixely),
					.pixelDraw(fill_pixelDraw),
					.pixelReady(pixel_ready),
					.draw(fill_draw),
					.ready(fill_ready)
				);
		end
	end
	endgenerate


/* Signalverteilung */

	// Globale Wires
	assign ready = 		functionSelect == `FUNCTION_PIXEL ? pixel_ready :
				functionSelect == `FUNCTION_LINE ? line_ready :
				functionSelect == `FUNCTION_COPYRECT ? rect_ready :
				functionSelect == `FUNCTION_FILLRECT ? fill_ready :
				1;

	// Adresse von CopyRect/GetPixel nur durchleiten, wenn Funktion ausgewaehlt und das Pixel-Modul im Leerlauf ist
	assign mem_addr =
				(functionSelect == `FUNCTION_COPYRECT) && pixel_ready ? rect_mem_addr :
				(functionSelect == `FUNCTION_GETPIXEL) && pixel_ready ? getpixel_mem_addr :
				pixel_mem_addr;

	generate
	begin: SIGNAL_ASSIGNMENTS
		if ((USE_SETPIXEL == 1) || (USE_LINE == 1) || (USE_COPYRECT == 1) || (USE_FILLRECT == 1))
		begin: ASSIGN_SETPIXEL
			// Wires fuer Modul SETPIXEL
			assign pixel_draw	= functionSelect == `FUNCTION_PIXEL ? draw :
							functionSelect == `FUNCTION_LINE ? line_pixelDraw :
							functionSelect == `FUNCTION_COPYRECT ? rect_pixelDraw :
							functionSelect == `FUNCTION_FILLRECT ? fill_pixelDraw :
							0;
		
			assign pixel_coord_x	= functionSelect == `FUNCTION_PIXEL ? coord_x1 :
							functionSelect == `FUNCTION_LINE ? line_pixelx :
							functionSelect == `FUNCTION_COPYRECT ? rect_pixelx :
							functionSelect == `FUNCTION_FILLRECT ? fill_pixelx :
							0;
		
			assign pixel_coord_y 	= functionSelect == `FUNCTION_PIXEL ? coord_y1 :
							functionSelect == `FUNCTION_LINE ? line_pixely :
							functionSelect == `FUNCTION_COPYRECT ? rect_pixely :
							functionSelect == `FUNCTION_FILLRECT ? fill_pixely :
							0;
		
			assign pixel_color	= functionSelect == `FUNCTION_COPYRECT ? rect_pixelColor :
							color;
		end

		if (USE_LINE == 1)
		begin: ASSIGN_LINE
			// Wires fuer Modul LINETO
			assign line_draw	= functionSelect == `FUNCTION_LINE ? draw :
							0;
		end

		if (USE_COPYRECT == 1)
		begin: ASSIGN_COPYRECT
			// Wires fuer Modul COPYRECT
			assign rect_draw	= functionSelect == `FUNCTION_COPYRECT ? draw :
							0;
		end

		if (USE_FILLRECT == 1)
		begin: ASSIGN_FILLRECT
			// Wire fuer Modul FILLRECT
			assign fill_draw	= functionSelect == `FUNCTION_FILLRECT ? draw :
							0;
		end
	end
	endgenerate
endmodule
