/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Engineer:       Andre Seidowski
// 
// Create Date:    14:53:38 11/17/2008 
// Module Name:    display_graph_function_fill 
// Project Name:   Display-Controller fuer SpartanMC
//
// Description:    Fuellt ein angegebenen Rechteckbereich mit einer gegebenen Farbe
//
//////////////////////////////////////////////////////////////////////////////////
module display_graph_function_fill #(
		parameter SCREEN_WIDTH = 320,
		parameter SCREEN_HEIGHT = 240,
		parameter BIT_PER_MEMORY_PIXEL = 2
	) (
		input wire	clk,
		input wire	reset,
		
		input wire [ld(SCREEN_WIDTH-1)-1:0] pixelTopLeft_x,
		input wire [ld(SCREEN_HEIGHT-1)-1:0]pixelTopLeft_y,
		input wire [ld(SCREEN_WIDTH-1)-1:0] pixelBottomRight_x,
		input wire [ld(SCREEN_HEIGHT-1)-1:0]pixelBottomRight_y,
		
		output reg [ld(SCREEN_WIDTH-1)-1:0] pixelx,
		output reg [ld(SCREEN_HEIGHT-1)-1:0]pixely,
		
		output reg pixelDraw,
		input wire pixelReady,
		
		input wire draw,
		output reg ready
	);
	
	`include "display/src/ld.vh"
	
	reg [2:0] fillState;


	always @(posedge clk or posedge reset)
	begin
		if (reset)
		begin
			fillState <= 0;
			pixelx <= 0;
			pixely <= 0;
			ready <= 1;
			pixelDraw <= 0;
		end
		else begin
			case (fillState)
				0:	if (draw)
					begin
						ready <= 0;
						pixelx <= pixelTopLeft_x;
						pixely <= pixelTopLeft_y;
						pixelDraw <= 1;
						fillState <= 1;
					end
					else begin
						ready <= 1;
						fillState <= 0;
					end
				1:	fillState <= 2;		// Warten auf Pixel-Modul
				2:	if (pixelReady)
					begin
						fillState <= 1;
						pixelDraw <= 1;
						if (pixelx == pixelBottomRight_x)
						begin
							pixelx <= pixelTopLeft_x;
							if (pixely == pixelBottomRight_y)
							begin
								pixelDraw <= 0;
								ready <= 1;
								fillState <= 0;
							end
							else
								pixely <= pixely + 1;
						end
						else
							pixelx <= pixelx + 1;
					end
					else begin
						fillState <= 1;
						pixelDraw <= 0;
					end
				default: fillState <= 0;
			endcase
		end
	end
endmodule
