/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Engineer:       Andre Seidowski
// Create Date:    10:27:02 11/05/2008 
// Module Name:    display_graph_function_getpixel 
// Project Name:   Display-Controller fuer SpartanMC
//
// Description:    Gibt die Farbbelegung des angegebenen Pixels zurueck
//
//////////////////////////////////////////////////////////////////////////////////
module display_graph_function_getpixel #(
		parameter SCREEN_WIDTH = 320,
		parameter SCREEN_HEIGHT = 240,
		parameter BIT_PER_MEMORY_PIXEL = 2,
		parameter PIXEL_PER_MEMORY_WORD = 4,
		parameter MEM_DATA_WIDTH = 9
	)(
		output wire [ld(SCREEN_WIDTH*SCREEN_HEIGHT * BIT_PER_MEMORY_PIXEL / (PIXEL_PER_MEMORY_WORD*BIT_PER_MEMORY_PIXEL))-1:0] mem_addr,
		input wire [MEM_DATA_WIDTH-1:0] mem_readData,
		input wire [ld(SCREEN_WIDTH)-1:0] coord_x,
		input wire [ld(SCREEN_HEIGHT)-1:0] coord_y,
		output wire [BIT_PER_MEMORY_PIXEL-1:0] color
	);

	`include "display/src/ld.vh"

	assign mem_addr = (coord_x/PIXEL_PER_MEMORY_WORD) + (coord_y*(SCREEN_WIDTH/PIXEL_PER_MEMORY_WORD));
	assign color = mem_readData[(coord_x % PIXEL_PER_MEMORY_WORD) * BIT_PER_MEMORY_PIXEL +: BIT_PER_MEMORY_PIXEL];
endmodule
