/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Engineer:       Andre Seidowski
// Create Date:    15:14:27 08/13/2008 
// Module Name:    display_siggen
// Project Name:   Display Controller fuer SpartanMC
// Description:    Signalerzeugung fuer ein Pixelbasiertes Display
//
//	TODO: SCREEN_FINISH_DELAY einbauen... im Moment fuehrt eine Angabe != 0
//		zu fehlerhaftem Timing!!
//	-> erledigt
//
//////////////////////////////////////////////////////////////////////////////////
module display_siggen # (
		parameter SCREEN_WIDTH = 0,		// Anzahl an Spalten
		parameter SCREEN_HEIGHT = 0,		// Anzahl an Zeilen
		parameter BIT_PER_PIXEL = 0,		// Ansteuerung eines Bildpixels
		parameter DATA_WORD_WIDTH = 0,		// Busbreite ans Display
		parameter SCREEN_FINISH_DELAY = 0,	// Takte, die am Ende des Bildschirms gewartet werden, bevor von vorne begonnen wird

		parameter INIT_CLOCKS_TO_VCON_ON = 0,	// Takte nach einem Reset, bis vcon aktiviert wird (Kontrastpannung)
		parameter INIT_CLOCKS_TO_DISPOFF = 0,	// Takte nach einem Reset, bis dispoff aktiviert wird (Signal fuer Logik im Display)

		parameter CL1_CLOCKS_AFTER_RESET = 0,	// Takte, bevor der Zeilentakt nach einem Reset auf High geht
		parameter CL1_CLOCKS_HIGH = 0,		// Takte, die der Zeilentakt auf High bleibt
		parameter CL1_CLOCKS_LOW = 0,		// Takte, die der Zeilentakt auf Low bleibt
		parameter CL1_BEGIN_HIGH_LOW = 1,	// Gibt an, ob eine neu Zeile bei der High-Low-Flanke des Zeilentaktes (1)
							//	oder bei der Low-High-Flanke (0) beginnt
		parameter CL2_CLOCKS_AFTER_RESET = 0,	// Takte, bevor der Pixeltakt nach einem Reset auf High geht
		parameter CL2_CLOCKS_HIGH = 0,		// Takte, die der Pixeltakt auf High bleibt
		parameter CL2_CLOCKS_LOW = 0,		// Takte, die der Pixeltakt auf Low bleibt
		parameter ROW_END_DELAY = 0,		// Takte, die am Zeilenende gewartet werden, bevor mit der naechsten Zeile forgefahren wird
		parameter ROW_BEGIN_DELAY = 0,		// Takte, die am Zeilenanfang gewartet werden, bevor neue Daten uebertragen werden

		parameter FRAMESTART_CLOCKS_AFTER_RESET	= 0, // Takte, bevor der Framestart nach einem Reset auf High geht
		parameter FRAMESTART_CLOCKS_HIGH = 0,	// Takte, die der Framestart auf High bleibt
		parameter FRAMESTART_CLOCKS_LOW = 0,	// Takte, die der Framestart auf Low bleibt

		parameter INVERT_CLOCKS_AFTER_RESET = 0,// Takte nach einem Reset, nach denen das Invertierungssignal erstmalig umgeschaltet wird
		parameter INVERT_CLOCKS_TOGGLE = 0	// Takte, bis das Invertierungssignal umgeschaltet wird
	) (
		input wire		clk,
		input wire		reset,		// !! High-aktiv !!
		
		input wire		displayOn,	// Ist das Display aktiviert?
		
		output wire 		framestart,	// Signalisierung zur Uebertragung der ersten Zeile ans Display
		output wire 		cl1,		// Zeilentakt fuer das Display
		output wire 		cl2,		// Datentakt fuer das Display
		output wire		invert,		// Signal zur Invertierung der Logik
		output reg	vcon,		// Signal zur Aktivierung der Kontrastspannung (Vcon bzw. Vlcd)
		output reg	dispoff,	// Dispoff-Signal fuer Logik im Display (Low-aktiv)
		output wire		vdd		// Versorgungsspannung der Logik im Display de-/aktivieren
	);
	
	`include "display/src/ld.vh"
	function integer max(input integer a, input integer b);
		max = (a > b) ? a : b;
	endfunction

	localparam DATA_WORDS_PER_ROW = (SCREEN_WIDTH*BIT_PER_PIXEL)/DATA_WORD_WIDTH;// Datenwoerter fuer eine Displayzeile

	// Taktzaehler
	reg [ld(FRAMESTART_CLOCKS_HIGH+FRAMESTART_CLOCKS_LOW)-1:0] count_framestart_clock;
	reg [ld(CL1_CLOCKS_HIGH+CL1_CLOCKS_LOW)-1:0]		count_cl1;
	reg [ld(CL2_CLOCKS_HIGH+CL2_CLOCKS_LOW)-1:0]		count_cl2;
	reg [ld(max(ROW_BEGIN_DELAY, ROW_END_DELAY))-1:0]	count_row_begin_end;
	reg [ld(SCREEN_FINISH_DELAY)-1:0]			count_screen_delay;
	reg [ld(INIT_CLOCKS_TO_VCON_ON):0]			count_init_vcon;
	reg [ld(INIT_CLOCKS_TO_DISPOFF):0]			count_init_dispoff;
	reg [ld(INVERT_CLOCKS_TOGGLE)-1:0]			count_invert;	

	// Register zur Erkennung des "Reset-Status", d.h. ob noch die Zeit nach einem Reset gezaehlt wird
	reg reset_framestart_clock;
	reg reset_row_clock;
	reg reset_pixel_clock;
	reg reset_invert_clock;

	// Spalten- und Zeilenzaehler
	reg [ld((SCREEN_WIDTH*BIT_PER_PIXEL)/DATA_WORD_WIDTH)-1:0] data_word_count;
	reg [ld(SCREEN_HEIGHT)-1:0] row_count;
	
	// Signal, welches den Zaehlprozess fuer alle Signale dazu veranlasst, zu pausieren
	// (um das SCREEN_FINISH_DELAY korrekt verarbeiten zu konnen)
	reg globalActive;

	// Ausgabesignale
	reg disp_framestart, disp_cl1, disp_cl2, disp_invert;
	
	
	assign framestart	= disp_framestart;
	assign cl1 		= disp_cl1;
	assign cl2		= disp_cl2;
	assign invert		= disp_invert;
	assign vdd		= !reset;


	// Verarbeitet einen Wechsel der Zeile (inkl. eventueller Pausen)
	// Wird entweder nach der High-Low- oder Low-High-Flanke des Zeilen-
	// taktes durchlaufen - daher lieber ein Task als zwei mal fast der gleiche Code ;)
	task row_end;
	begin
		globalActive <= 1;
		if (row_count == SCREEN_HEIGHT-1)
		begin
			disp_cl1 <= !CL1_BEGIN_HIGH_LOW;
			// das Ende des Bildschirms ist erreicht
			if (SCREEN_FINISH_DELAY == count_screen_delay)
			begin
				// die Pause vor einer neuen Zeile ist vorbei
				row_count <= 0;
				count_screen_delay <= 0;
				
				// Zaehler zuruecksetzen, wenn der Zeilenwechsel bei der High-Low-Flanke kommt,
				// sonst muss erst noch die Low-Phase abgewartet werden, wofuer der Zaehler
				// gebraucht wird.
				count_cl1 <= CL1_BEGIN_HIGH_LOW ? count_cl1 + 1 : 0;
			end
			else begin
				// es muss noch bis zum naechsten Frame gewartet werden
				count_screen_delay <= count_screen_delay + 1;

				globalActive <= 0;
			end
		end
		else begin
			row_count <= row_count + 1;
			disp_cl1 <= !CL1_BEGIN_HIGH_LOW;
			count_cl1 <= count_cl1 + 1;
		end
	end
	endtask
	
	// Initialisierung (DISPOFF) nach Reset
	always @(posedge clk or posedge reset)
	begin
		if (reset)
		begin
			dispoff <= 0;
			count_init_dispoff <= 0;
		end
		else begin
			if (displayOn)
			begin
				// Das oberste Bit im Zaehler-Register signalisiert, dass DISPOFF nun eingeschaltet sein muss
				// damit wird eine Verzoegerung um einen Takt realisiert
				if (count_init_dispoff[ld(INIT_CLOCKS_TO_DISPOFF)])
					dispoff <= 1;
				else begin
					// ...sonst bist zum Ziel zaehlen
					dispoff <= 0;
					if (INIT_CLOCKS_TO_DISPOFF-1 == count_init_dispoff)
						count_init_dispoff[ld(INIT_CLOCKS_TO_DISPOFF)] <= 1;
					else
						count_init_dispoff <= count_init_dispoff + 1;
				end
			end
			else begin
				dispoff <= 0;
				count_init_dispoff <= 0;
			end
		end
	end
	
	// Initialisierung (vcon) nach Reset
	always @(posedge clk or posedge reset)
	begin
		if (reset)
		begin
			vcon <= 0;
			count_init_vcon <= 0;
		end
		else begin
			if (displayOn)
			begin
				// Das oberste Bit im Register signalisiert, dass Vcon/Vlcd nun eingeschaltet sein muss
				// damit wird eine Verzoegerung um einen Takt realisiert
				if (count_init_vcon[ld(INIT_CLOCKS_TO_VCON_ON)])
					vcon <= 1;
				else begin
					vcon <= 0;
					if (INIT_CLOCKS_TO_VCON_ON-1 == count_init_vcon)
						count_init_vcon[ld(INIT_CLOCKS_TO_VCON_ON)] <= 1;
					else
						count_init_vcon <= count_init_vcon + 1;
				end
			end
			else begin
				vcon <= 0;
				count_init_vcon <= 0;
			end
		end
	end
	
	
	// Generiert das Invertierungssignal nach jedem Bildzyklus
	always @(posedge clk or posedge reset)
	begin
		if (reset)
		begin
			disp_invert <= 0;
			reset_invert_clock <= 1;
			count_invert <= 0;
		end
		else begin
			if (globalActive)
			begin
				// noch beim "Einschwingen"?
				if (reset_invert_clock)
				begin
					// kein "-1" damit der Fall "keine Pause (also 0 Takte)" mit abgefangen wird
					if (INVERT_CLOCKS_AFTER_RESET == count_invert)
					begin
						count_invert <= 1;
						reset_invert_clock <= 0;
						disp_invert <= 1;
					end
					else
						count_invert <= count_invert + 1;
				end
				else begin
					// sonst regulaerer Ablauf
					if (INVERT_CLOCKS_TOGGLE == count_invert)
					begin
						count_invert <= 1;
						disp_invert <= !disp_invert;
					end
					else
						count_invert <= count_invert + 1;
				end
			end
		end
	end
	
	
	// Erzeugung Framestart-Signal
	always @(posedge clk or posedge reset)
	begin
		if (reset)
		begin
			count_framestart_clock	<= 0;
			reset_framestart_clock	<= 1;
			disp_framestart		<= 0;
		end
		else begin
			if (globalActive)
			begin
				if (reset_framestart_clock)
				begin
					// nach einem Reset FRAMESTART_CLOCKS_AFTER_RESET Takte warten
					// kein "-1" damit der Fall "keine Pause (also 0 Takte)" ebenfalls beruecksichtigt wird
					if (FRAMESTART_CLOCKS_AFTER_RESET == count_framestart_clock)
					begin
						// bevor der Framestart signalisiert wird
						count_framestart_clock <= 1;
						reset_framestart_clock <= 0;
						disp_framestart <= 1;
					end
					else
						count_framestart_clock <= count_framestart_clock + 1;
				end
				else begin
					// sonst zuerst FRAMESTART_CLOCKS_HIGH Takte den Framestart auf High belassen
					if (count_framestart_clock == FRAMESTART_CLOCKS_HIGH)
					begin
						// ...danach auf Low setzen
						disp_framestart <= 0;
						count_framestart_clock <= count_framestart_clock + 1;
					end
					else
						// ...und nochmal FRAMESTART_CLOCKS_LOW Takte auf Low belassen
						if (count_framestart_clock == FRAMESTART_CLOCKS_HIGH + FRAMESTART_CLOCKS_LOW)
						begin
							// danach wieder von vorne
							disp_framestart <= 1;
							count_framestart_clock <= 1;
						end
						else
							count_framestart_clock <= count_framestart_clock + 1;
				end
			end
		end
	end
	
	// Erzeugung Zeilentakt CL1
	always @(posedge clk or posedge reset)
	begin
		if (reset)
		begin
			count_cl1		<= 0;
			reset_row_clock		<= 1;
			count_screen_delay	<= 0;
			row_count		<= 0;
			disp_cl1		<= 0;
			globalActive		<= 1;
		end
		else begin
			if (reset_row_clock)
			begin
				// Nach einem Reset erst CL1_CLOCKS_AFTER_RESET Takte warten
				if (CL1_CLOCKS_AFTER_RESET == count_cl1)
				begin
					count_cl1 <= 1;
					reset_row_clock <= 0;
					disp_cl1	<= 1;
				end
				else
					count_cl1 <= count_cl1 + 1;
			end
			else begin
				// ...sonst mit dem regulaeren Takt fortfahren
				if (CL1_CLOCKS_HIGH == count_cl1)
				begin
					// High-Phase ist vorbei
					if (CL1_BEGIN_HIGH_LOW)
						row_end;
					else begin
						disp_cl1 <= 0;
						count_cl1 <= count_cl1 + 1;
					end
				end
				else if (CL1_CLOCKS_HIGH + CL1_CLOCKS_LOW == count_cl1)
				begin
					if (!CL1_BEGIN_HIGH_LOW)
						row_end;
					else begin
						disp_cl1 <= 1;
						count_cl1 <= 1;
					end
				end
				else
					count_cl1 <= count_cl1 + 1;
			end
		end
	end
	
	// Erzeugung Datentakt CL2
	always @(posedge clk or posedge reset)
	begin
		if (reset)
		begin
			count_cl2		<= 0;
			reset_pixel_clock	<= 1;
			count_row_begin_end	<= 0;
			
			disp_cl2		<= 0;
			
			data_word_count		<= 0;
		end
		else begin
			if (globalActive)
			begin
				if (reset_pixel_clock)
				begin
					// kein "-1", da sonst der Fall "keine Pause (also 0 Takte)" nicht abgefangen wird
					if (CL2_CLOCKS_AFTER_RESET == count_cl2)
					begin
						count_cl2 <= 0;
						reset_pixel_clock <= 0;
						disp_cl2 <= (ROW_BEGIN_DELAY == 0) ? 1 : 0;
					end
					else
						count_cl2 <= count_cl2 + 1;
				end
				else begin

					// ROW_BEGIN_DELAY abwarten
					if ((ROW_BEGIN_DELAY != 0) && (0 == disp_cl2) && (0 == count_cl2) && (0 == data_word_count))
					begin
						if (ROW_BEGIN_DELAY-1 == count_row_begin_end)
						begin
							count_row_begin_end <= 0;
							count_cl2 <= 0;
							disp_cl2 <= 1;
						end
						else
							count_row_begin_end <= count_row_begin_end + 1;
					end

					// Erzeugung des Datentakts einstellen
					else if (CL2_CLOCKS_HIGH-1 == count_cl2)
					begin
						count_cl2 <= count_cl2 + 1;
						disp_cl2 <= 0;
					end


					// ROW_END_DELAY abwarten
					else if (CL2_CLOCKS_HIGH+CL2_CLOCKS_LOW-1 == count_cl2)
					begin
						// Datentakt wuerde nun erneut beginnen, jedoch u.U. eine Pause einbauen,
						//	wenn das Ende einer Zeile erreicht ist
						if (DATA_WORDS_PER_ROW-1 == data_word_count)
						begin
							// Pause schon zuende?
							if (ROW_END_DELAY == count_row_begin_end)
							begin
								data_word_count <= 0;
								count_row_begin_end <= 0;
								count_cl2 <= 0;
								disp_cl2 <= (ROW_BEGIN_DELAY == 0) ? 1 : 0;
							end
							else
								//sonst weiter warten
								count_row_begin_end <= count_row_begin_end + 1;
						end
						else begin
							data_word_count <= data_word_count + 1;
							count_cl2 <= 0;
							disp_cl2 <= 1;
						end
					end

					// sonst Datentakt erzeugen
					else
						count_cl2 <= count_cl2 + 1;
				end
			end
		end
	end
endmodule
