unit CharGen_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ImgList;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    CBFontSelect: TComboBox;
    Label1: TLabel;
    GroupBox2: TGroupBox;
    EditCharWidth: TEdit;
    Label2: TLabel;
    UpDownCharWidth: TUpDown;
    EditCharHeight: TEdit;
    UpDownCharHeight: TUpDown;
    Label3: TLabel;
    BtnBold: TSpeedButton;
    BtnItalic: TSpeedButton;
    BtnPrevChar: TButton;
    BtnNextChar: TButton;
    LblCurChar: TLabel;
    BtnGenerate: TButton;
    BtnCpyClpbd: TButton;
    BtnSave: TButton;
    BtnOpen: TButton;
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    Panel1: TPanel;
    ImageChar: TImage;
    CBReverse: TCheckBox;
    procedure EditCharChange(Sender: TObject);
    procedure BtnCharChangeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CBFontSelectChange(Sender: TObject);
    procedure BtnFontStyleClick(Sender: TObject);
    procedure BtnGenerateClick(Sender: TObject);
    procedure BtnCpyClpbdClick(Sender: TObject);
    procedure BtnOpenClick(Sender: TObject);
    procedure BtnSaveClick(Sender: TObject);
  private
    { Private-Deklarationen }
    CharIndex: Byte;
    CharBitmapArr: array[0..255] of TBitmap;
    Font: TFont;
    procedure DrawChar(Character: Byte; Width: Integer; Height: Integer; Target: TBitmap);
    procedure UpdateImage;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

uses Clipbrd, IniFiles;

{$R *.dfm}

procedure TForm1.EditCharChange(Sender: TObject);
begin
  ImageChar.Width := UpDownCharWidth.Position * 8;
  ImageChar.Height := UpDownCharHeight.Position * 8;
  Font.Height := UpDownCharHeight.Position - 2;
  UpdateImage;
end;

procedure TForm1.BtnCharChangeClick(Sender: TObject);
begin
  CharIndex := CharIndex + (Sender as TButton).Tag;
  UpdateImage;
end;

procedure TForm1.DrawChar(Character: Byte; Width, Height: Integer;
  Target: TBitmap);

begin
  //SelectClipRgn(Target.Canvas.Handle, CreateRectRgn(0, 0, Width, Height));

  Target.Width := Width;
  Target.Height := Height;
  Target.Canvas.Font := Font;

  // Bild erstmal lschen
  Target.Canvas.Brush.Color := clWhite;
  Target.Canvas.Brush.Style := bsSolid;
  Target.Canvas.FillRect(Target.Canvas.ClipRect);

  // Text schrieben vorbereiten
  Target.Canvas.MoveTo(0, 0);
  Target.Canvas.Pen.Color := clBlack;
  Target.Canvas.Font := Font;

  Target.Canvas.TextOut(0, 0, Chr(Character));
end;

procedure TForm1.UpdateImage;
begin
  if Assigned(CharBitmapArr[CharIndex]) then
  begin
    ImageChar.Canvas.StretchDraw(ImageChar.ClientRect, CharBitmapArr[CharIndex]);
  end
  else begin
    ImageChar.Canvas.Brush.Color := clBtnFace;
    ImageChar.Canvas.FillRect(ImageChar.ClientRect);
  end;

  LblCurChar.Caption := IntToStr(CharIndex) + ' = ' + Chr(CharIndex);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Font := TFont.Create;
  Font.Pitch := fpFixed;
  Font.Name := 'Fixedsys';

  CBFontSelect.Items.Assign(Screen.Fonts);
  CBFontSelect.Sorted := True;
  CBFontSelect.ItemIndex := CBFontSelect.Items.IndexOf(Font.Name);

  CharIndex := 65;  // msste 'A' sein
end;

procedure TForm1.CBFontSelectChange(Sender: TObject);
begin
  Font.Name := CBFontSelect.Text;
//  UpdateImage;
end;

procedure TForm1.BtnFontStyleClick(Sender: TObject);
begin
  if BtnBold.Down and BtnItalic.Down then
    Font.Style := [fsBold, fsItalic]
  else if BtnBold.Down and not BtnItalic.Down then
    Font.Style := [fsBold]
  else if not BtnBold.Down and BtnItalic.Down then
    Font.Style := [fsItalic]
  else if not BtnBold.Down and not BtnItalic.Down then
    Font.Style := [];

//  UpdateImage;
end;

procedure TForm1.BtnGenerateClick(Sender: TObject);
var
  B: Byte;

begin
  for B := 0 to 255 do
  begin
    if not Assigned(CharBitmapArr[B]) then
    begin
      CharBitmapArr[B] := TBitmap.Create;
      CharBitmapArr[B].Monochrome := True;
    end;

    DrawChar(B, UpDownCharWidth.Position, UpDownCharHeight.Position, CharBitmapArr[B]);
  end;
  UpdateImage;
end;

procedure TForm1.BtnCpyClpbdClick(Sender: TObject);
var
  Bpm: TBitmap;
begin
  if Clipboard.HasFormat(CF_BITMAP) then
  begin
    Bpm := TBitmap.Create;
    Bpm.Handle := Clipboard.GetAsHandle(CF_BITMAP);
    if (Bpm.Width = UpDownCharWidth.Position) and (Bpm.Height = UpDownCharHeight.Position) then
    begin
      CharBitmapArr[CharIndex].Free;
      CharBitmapArr[CharIndex] := Bpm;
      CharBitmapArr[CharIndex].Monochrome := True;
      UpdateImage;
    end
    else begin
      ShowMessage('Grafik in der Zwischenablage hat andere Abmessungen als die gegebenen' + #10#13 +
          'Ist: ' + IntToStr(Bpm.Width) + ' x ' + IntToStr(Bpm.Height) + #10#13 +
          'Soll: ' + IntToStr(UpDownCharWidth.Position) + ' x ' + IntToStr(UpDownCharHeight.Position));
    end;
  end;
end;

procedure TForm1.BtnOpenClick(Sender: TObject);
var
  Ini: TIniFile;
  IniFileName, Ext: string;
  Width, Height, I: Integer;
  FS: TFileStream;

  BitLine: Int64;
  LineByteLength: Byte;
  ColumnCount, LineCount: Integer;

begin
  if OpenDlg.Execute then
  begin
    IniFileName := OpenDlg.FileName;
    Ext := ExtractFileExt(IniFileName);
    Delete(IniFileName, Length(IniFileName) - Length(Ext) + 1, Length(ext));
    IniFileName := IniFileName + '.ini';
    Ini := TIniFile.Create(IniFileName);
    Width := Ini.ReadInteger('Main', 'Width', 8);
    Height := Ini.ReadInteger('Main', 'Height', 16);
    Ini.Free;

    FS := TFileStream.Create(OpenDlg.FileName, fmOpenRead);

    LineByteLength := Width div 8;
    if Width mod 8 <> 0 then
      Inc(LineByteLength);


    // Bild-Array fllen
    for I := 0 to 255 do
    begin
      if not Assigned(CharBitmapArr[I]) then
        CharBitmapArr[I] := TBitmap.Create;

      CharBitmapArr[I].Monochrome := True;
      CharBitmapArr[I].Width := Width;
      CharBitmapArr[I].Height := Height;
      CharBitmapArr[I].Canvas.Pen.Color := clBlack;


      for LineCount := 0 to UpDownCharHeight.Position-1 do
        if FS.Read(BitLine, LineByteLength) = LineByteLength then
          for ColumnCount := 0 to UpDownCharWidth.Position-1 do
          begin
            CharBitmapArr[I].Canvas.MoveTo(ColumnCount, LineCount);
            if BitLine and (1 shl ColumnCount) = 0 then
              // Kein Pixel
              CharBitmapArr[I].Canvas.Pixels[ColumnCount, LineCount] := clWhite
            else
              // Ein Pixel
              CharBitmapArr[I].Canvas.Pixels[ColumnCount, LineCount] := clBlack;
          end;

    end;
    FS.Free;
    UpDownCharWidth.Position := Width;
    UpDownCharHeight.Position := Height;
    UpdateImage;
  end;
end;

procedure TForm1.BtnSaveClick(Sender: TObject);
var
  Ini: TIniFile;
  IniFileName, Ext: string;
  I: Integer;
  FS: TFileStream;
  BitStringFile: TextFile;
  HexStringFile: TextFile;
  VHDLFormatFile: TextFile;

  BitLine: Int64;
  LineByteLength: Byte;
  ColumnCount, LineCount: Integer;

  BitString: string;
  HexString: string;
  VHDLString: string;

begin
  if SaveDlg.Execute then
  try
    IniFileName := SaveDlg.FileName;
    Ext := ExtractFileExt(IniFileName);
    Delete(IniFileName, Length(IniFileName) - Length(Ext) + 1, Length(ext));
    AssignFile(BitStringFile, IniFileName + '_BitString.txt');
    AssignFile(HexStringFile, IniFileName + '_HexString.txt');
    AssignFile(VHDLFormatFile, IniFileName + '_VHDLFormat.txt');
    IniFileName := IniFileName + '.ini';
    Ini := TIniFile.Create(IniFileName);

    Ini.WriteInteger('Main', 'Width', UpDownCharWidth.Position);
    Ini.WriteInteger('Main', 'Height', UpDownCharHeight.Position);

    Ini.Free;

    FS := TFileStream.Create(SaveDlg.FileName, fmCreate or fmShareDenyWrite);
    ReWrite(BitStringFile);
    ReWrite(HexStringFile);
    ReWrite(VHDLFormatFile);

    LineByteLength := UpDownCharWidth.Position div 8;
    if UpDownCharWidth.Position mod 8 <> 0 then
      Inc(LineByteLength);

    for I := 0 to 255 do
    begin
      if Assigned(CharBitmapArr[I]) then
      begin
        VHDLString := '';
        for LineCount := 0 to UpDownCharHeight.Position-1 do
        begin
          // BitLine erstellen = Eine Zeile in der Grafik
          BitLine := 0;
          BitString := '';
          for ColumnCount := 0 to UpDownCharWidth.Position-1 do
          begin
            if CharBitmapArr[I].Canvas.Pixels[ColumnCount, LineCount] = clBlack then
            begin
              BitLine := BitLine or (1 shl ColumnCount);
              if CBReverse.Checked then
                BitString := '1' + BitString
              else
                BitString := BitString + '1';
            end
            else
              if CBReverse.Checked then
                BitString := '0' + BitString
              else
                BitString := BitString + '0';
          end;
          HexString := IntToHex(BitLine,LineByteLength * 2);
          VHDLString := VHDLString + '"' + BitString + '", ';

          // BitLine schreiben
          FS.WriteBuffer(BitLine, LineByteLength);
          WriteLn(BitStringFile, BitString);
          WriteLn(HexStringFile, HexString);
        end;
        WriteLn(VHDLFormatFile, VHDLString);
      end
      else begin
        BitLine := 0;
        FS.WriteBuffer(BitLine, LineByteLength);
      end;
    end;
  finally
    FS.Free;
    CloseFile(BitStringFile);
    CloseFile(HexStringFile);
  end;
end;

end.
