/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    16:41:49 09/27/2010 
// Design Name: 
// Module Name:    BlockRam 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
//`default_nettype wire
module BlockRam18to9(
	input wire Reset,
	//port A
	input wire A_Clk,
	input wire [9:0] A_Adr,	// address
	input wire [15:0] A_DataIn,	//data bits
	input wire [1:0] A_ParIn,	// parity bit
	output wire [15:0] A_DataOut,	// data bits
	output wire [1:0] A_ParOut,	// parity bit
	input wire A_Enable,	//	0 disable the port
	input wire A_WriteEnable,	// 1 write operation, 0 read operation
	//port B
	input wire B_Clk,
	input wire [10:0] B_Adr,
	input wire [7:0] B_DataIn,
	input wire B_ParIn,
	output wire [7:0] B_DataOut,
	output wire B_ParOut,
	input wire B_Enable,
	input wire B_WriteEnable
);

RAMB16_S9_S18 #(
	  .INIT_A(9'h000),     // Value of output RAM registers on Port A at startup
      .INIT_B(18'h00000),  // Value of output RAM registers on Port B at startup
      .SRVAL_A(9'h000),    // Port A output value upon SSR assertion
      .SRVAL_B(18'h00000), // Port B output value upon SSR assertion
      .WRITE_MODE_A("WRITE_FIRST"), // WRITE_FIRST, READ_FIRST or NO_CHANGE
      .WRITE_MODE_B("WRITE_FIRST"), // WRITE_FIRST, READ_FIRST or NO_CHANGE
      .SIM_COLLISION_CHECK("GENERATE_X_ONLY")  // "NONE", "WARNING_ONLY", "GENERATE_X_ONLY", "ALL" 
		
   ) blockram18to9 (
      .DOA(B_DataOut),      // Port A 8-bit Data Output
      .DOB(A_DataOut),      // Port B 16-bit Data Output
      .DOPA(B_ParOut),    // Port A 1-bit Parity Output
      .DOPB(A_ParOut),    // Port B 2-bit Parity Output
      .ADDRA(B_Adr),  // Port A 11-bit Address Input
      .ADDRB(A_Adr),  // Port B 10-bit Address Input
      .CLKA(B_Clk),    // Port A Clock
      .CLKB(A_Clk),    // Port B Clock
      .DIA(B_DataIn),      // Port A 8-bit Data Input
      .DIB(A_DataIn),      // Port B 16-bit Data Input
      .DIPA(B_ParIn),    // Port A 1-bit parity Input
      .DIPB(A_ParIn),    // Port-B 2-bit parity Input
      .ENA(B_Enable),      // Port A RAM Enable Input
      .ENB(A_Enable),      // Port B RAM Enable Input
      .SSRA(Reset),    // Port A Synchronous Set/Reset Input
      .SSRB(Reset),    // Port B Synchronous Set/Reset Input
      .WEA(B_WriteEnable),      // Port A Write Enable Input
      .WEB(A_WriteEnable)       // Port B Write Enable Input
   );

endmodule
