/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    17:43:41 07/05/2010 
// Design Name: 
// Module Name:    crc32 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module Crc32_8(
	input wire Clk,
    input wire Strobe,
	 input wire Reset,
    input wire [7:0] Byte,
    output wire [31:0]Crcsum
);
	 
reg [31:0] schift;

always@(posedge Clk or posedge Reset)
if(Reset)
begin
	schift <= 'hFFFFFFFF;	// equal to complement the first 32 bit
end
else
if(Strobe)
begin
	//G(x) = x32 + x26 + x23 + x22 + x16 + x12 + x11 + x10 + x8 + x7 + x5 + x4 + x2 + x + 1

	schift[0] <= schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[1] <= schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[2] <= schift[26] ^ Byte[5] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[3] <= schift[27] ^ Byte[4] ^ schift[26] ^ Byte[5] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6];
	schift[4] <= schift[28] ^ Byte[3] ^ schift[27] ^ Byte[4] ^ schift[26] ^ Byte[5] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[5] <= schift[29] ^ Byte[2] ^ schift[28] ^ Byte[3] ^ schift[27] ^ Byte[4] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[6] <= schift[30] ^ Byte[1] ^ schift[29] ^ Byte[2] ^ schift[28] ^ Byte[3] ^ schift[26] ^ Byte[5] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6];
	schift[7] <= schift[31] ^ Byte[0] ^ schift[30] ^ Byte[1] ^ schift[29] ^ Byte[2] ^ schift[27] ^ Byte[4] ^ schift[26] ^ Byte[5] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[8] <= schift[0] ^ schift[31] ^ Byte[0] ^ schift[30] ^ Byte[1] ^ schift[28] ^ Byte[3] ^ schift[27] ^ Byte[4] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[9] <= schift[1] ^ schift[31] ^ Byte[0] ^ schift[29] ^ Byte[2] ^ schift[28] ^ Byte[3] ^ schift[26] ^ Byte[5] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6];
	schift[10] <= schift[2] ^ schift[30] ^ Byte[1] ^ schift[29] ^ Byte[2] ^ schift[27] ^ Byte[4] ^ schift[26] ^ Byte[5] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[11] <= schift[3] ^ schift[31] ^ Byte[0] ^ schift[30] ^ Byte[1] ^ schift[28] ^ Byte[3] ^ schift[27] ^ Byte[4] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[12] <= schift[4] ^ schift[31] ^ Byte[0] ^ schift[29] ^ Byte[2] ^ schift[28] ^ Byte[3] ^ schift[26] ^ Byte[5] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[13] <= schift[5] ^ schift[30] ^ Byte[1] ^ schift[29] ^ Byte[2] ^ schift[27] ^ Byte[4] ^ schift[26] ^ Byte[5] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6];
	schift[14] <= schift[6] ^ schift[31] ^ Byte[0] ^ schift[30] ^ Byte[1] ^ schift[28] ^ Byte[3] ^ schift[27] ^ Byte[4] ^ schift[26] ^ Byte[5];
	schift[15] <= schift[7] ^ schift[31] ^ Byte[0] ^ schift[29] ^ Byte[2] ^ schift[28] ^ Byte[3] ^ schift[27] ^ Byte[4];
	schift[16] <= schift[8] ^ schift[30] ^ Byte[1] ^ schift[29] ^ Byte[2] ^ schift[28] ^ Byte[3] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[17] <= schift[9] ^ schift[31] ^ Byte[0] ^ schift[30] ^ Byte[1] ^ schift[29] ^ Byte[2] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6];
	schift[18] <= schift[10] ^ schift[31] ^ Byte[0] ^ schift[30] ^ Byte[1] ^ schift[26] ^ Byte[5];
	schift[19] <= schift[11] ^ schift[31] ^ Byte[0] ^ schift[27] ^ Byte[4];
	schift[20] <= schift[12] ^ schift[28] ^ Byte[3];
	schift[21] <= schift[13] ^ schift[29] ^ Byte[2];
	schift[22] <= schift[14] ^ schift[30] ^ Byte[1] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[23] <= schift[15] ^ schift[31] ^ Byte[0] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[24] <= schift[16] ^ schift[26] ^ Byte[5] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6];
	schift[25] <= schift[17] ^ schift[27] ^ Byte[4] ^ schift[26] ^ Byte[5];
	schift[26] <= schift[18] ^ schift[28] ^ Byte[3] ^ schift[27] ^ Byte[4] ^ schift[24] ^ schift[30] ^ Byte[1] ^ Byte[7];
	schift[27] <= schift[19] ^ schift[29] ^ Byte[2] ^ schift[28] ^ Byte[3] ^ schift[25] ^ schift[31] ^ Byte[0] ^ Byte[6];
	schift[28] <= schift[20] ^ schift[30] ^ Byte[1] ^ schift[29] ^ Byte[2] ^ schift[26] ^ Byte[5];
	schift[29] <= schift[21] ^ schift[31] ^ Byte[0] ^ schift[30] ^ Byte[1] ^ schift[27] ^ Byte[4];
	schift[30] <= schift[22] ^ schift[31] ^ Byte[0] ^ schift[28] ^ Byte[3];
	schift[31] <= schift[23] ^ schift[29] ^ Byte[2];

end	//else (Reset = 0)

assign Crcsum = schift;

endmodule
