/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:05:20 11/15/2010 
// Design Name: 
// Module Name:    Ethernet 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//important note
/*
The data is written bytewise in the dma buffer but readed in 18 bit words.
There ByteCnt describe how much bytes are valid. If an uneven byte number is
written, then it is possible that the low byte contains data from another frame.
*/
`default_nettype none
module Ethernet(
//*** Verbindungen mit dem MC die nicht ge�ndert werden d�rfen ***

	input wire                   clk_peri,       //Systemtakt
	input wire           [17:0]  do_peri,        //Datenbus  vom MC
	output wire         [17:0]  di_peri,        //Datenbus  zum MC
	input wire           [9:0]   addr_peri,      //Addressbus vom MC
	input wire                   access_peri,    //Signale fuer Peripheriezugriffe
	input wire                   wr_peri,        //Schreibsignal

	// Blockram Interface
	input wire                   mem_clk,        //BRAM clk
	input wire                   mem_access,
	input wire                   store_access,
	input wire					store_access_low,
	input wire					store_access_high,
	input wire            [7:0]  addr_high,


	//*** Verbindungen mit dem MC die ver�ndert werden d�rfen ***

	input wire                   clk50,				//50 MHz clock
	input wire                   reset,          //Reset-Signal

	//interrupts
	output wire             rx_ir,  //Interrupt for rx event
	output wire				tx_ir,	//Interrupt for tx event


	//MAC signals
	//Port 0 Signals
	//Signals for Transmitting
	input wire TxClk,
	output wire [3:0] Txd,
	output wire TxEn,
	//Signals for Receiving
	input wire RxClk,
	input wire [3:0] Rxd,
	input wire RxErr,
	input wire RxDV,
	input wire Col,
	input wire Crs,


	//Managment Pins
	inout wire Mdio,
	output wire Mdc,
	//additional signals
	output wire PhyReset,
	output wire PwrDownA,
	output wire PwrDownB,
	
	output wire[128:0] debug
);



// Die Parameter fuer die Adressen werden in der system.v eingestellt.
parameter BASE_ADR = 10'h100;        //Offset zur Basisadresse der I/O Ports
parameter DMA_BASE_ADR = 18'h19000;     //4096 Adressen unterhalb der I/O Ports
// Mac address is splitted to avoid number overflow in jconfig
// in version 2 string parameter should be availible, so concat 
// both parameters if it is availible
parameter [23:0] MAC_ADDRESS_HIGH = 24'h0010A4;
parameter [23:0] MAC_ADDRESS_LOW = 24'h7BEA80;

//parameter for the globalToggle-Loop
parameter [4:0] T_HIGH = 0;		// High byte of the Length field
parameter [4:0] T_LOW = 1;		// Low byte of the Length field

//****SpartanMC registers*****

// register 0, mode:
// {IgnoreCrc,IgnoreBroadcast,...,3'bcounterIndex
//  ,IRQEN_Tx1, IRQEN_Tx, IRQEN_Rx1, IRQEN_Rx,PwrDown1, PwrDown,PhyReset, PREMISCIOUS MODE}
//   PREMISCIOUS MODE: 0 == Disabled, 1 == Enabled
//   PhyReset: 1 == mac reset <= 1
//   PwrDownX: Control the PWRDown signal to the PHY, 0 Port is active, 1 Port is inactive
//   IRQEN_XX: interrupt enable flags for the irq
//   IRQ_XXX: indicate which source triggered the interrupt
//   counterIndex: index number of the counter which is displayed at register adress 8
//         0	CrcCounters {2'b0,8'brx1Crc,8'brxCrc}
//         1	rxRecCnt
//         2    rx1RecCnt
//         3    txTransCnt
//         4    tx1TransCnt
reg [3:0] mode;
reg [2:0] counterIndex;
reg irqenRx;
reg irqenTx;
reg irqRx;
reg irqTx;
reg ignoreBroadcast;
reg ignoreCrc;

//store the start Address of the FIRST valid frame (in tx section)
//register 1: txAddrCnt, Address of current frame, which should be sent
reg [10:0] txAddrCnt;


//register 3: rxReadCnt, Address of first valid frame
reg [9:0] rxReadCnt;

// register 5: mdioReg1, Register to access the mdio bus
// {1'bWriteEnable,1'b0,16'bData}

wire mdioStart;
wire mdioWriteEnable;
wire [15:0] mdioData;
wire [15:0] mdioResult;
// register 6: mdioReg2,
// {1'bReady,7'b0,1'bPhyAddress,5'bRegAddress}
wire mdioReady;
reg [4:0] mdioPhyAddr;
reg [4:0] mdioRegAddr;

wire mdioOut;

//****local signals****
//single register delay the reset signal not enough to set byte
//at address 0 on both tx ports to 0
reg [3:0] restart;	//indicate if a reset sig was high

wire selectMem;	//select signal for dma memory access
wire selectReg;	//select signal for register access
wire [17:0] moduleAddr;
wire [17:0] dmaDataOut;
//**MAC side signals**
//mem signals

//rx
wire [7:0] macDataInrx;


wire macWriteEnablerx;


//mac signals
wire [7:0] rxData;
wire [7:0] txData;

wire rxNext;
wire rxLastByte;
wire rxAbort;
wire txNext;
wire txAvail;
wire rxRec;
wire txTrans;
wire promiscuous;
wire rxCrcWrong;

// frame avail indicator for low and high byte of ByteCnt
reg txLowAvail;
reg txHighAvail;


// Counter & Statussignals
//delayed status signals
reg oldRxRec;
reg oldTxTrans;

//indicate if data between mac and cpu is transfered (rx only)
reg rxTransfering;

//start stop indicators
wire rxStart;
reg rxStop;

wire txStop;	//signal that mac should end transmission (no more data)

// ByteCnt have to be readed in 2 steps
wire txPreStart;
reg txStart;
reg txSampleStart;

wire txEndTrans;	//signal that mac had finished transmission

//addresses which trigger overflow
wire [9:0] rxOverAddr0;
wire [9:0] rxOverAddr1;

//store the start Address of the FIRST valid frame (in rx section)
reg [10:0] rxAddrCnt;
//store the offset from the start Address of a frame
reg rxOffset;
reg [10:0] txOffset;
// Address signals
wire [10:0] rxAddr;
wire [10:0] txAddr;
//toggle signal for special handling(address calc) in idle state
reg [4:0] globalToggle;
//priority signals
reg txVLAN;
//intermediary result
wire [10:0] rxPreAddr;
wire [10:0] txPreAddr;

//stop of transmission need to be split in 2 steps
//1: write high byte of ByteCnt, 2: write low byte
reg rxPreStop;

//count the length of the frame
reg [10:0] rxByteCnt;
reg [10:0] txByteCnt;

//indicate when memory overflow occure
wire rxOverflow;

//Mac counters
wire [17:0] activeCounter;
wire [17:0] crcErrorCounter;
wire [17:0] rxRecCnt;
wire [17:0] txTransCnt;


//********************************
// MODULES
//*******************************

pselect #(
		.ADDR_WIDTH     (7),
		.BASE_WIDTH     (7),
		.BASE_ADDR      (DMA_BASE_ADR >> 11) 
	)
 pseldma			 
	(moduleAddr[17:11],mem_access,selectMem
);

parameter  ALL_PORTS		= 16,

	DECODSIM_DEB		= "NO";
wire					select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	selectReg		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);
			


DmaMemory18to8 dmamem18to8(
	.Reset(reset),
	//Port A - SpartanMC
	.A_Clk(mem_clk),
	.A_Addr(moduleAddr[11:0]),
	.A_DataIn(do_peri),
	.A_DataOut(dmaDataOut),
	.A_Enable(1'b1),	//always enable	
	.A_WriteEnable(store_access & selectMem),

	//Port B - Ethernet module
	.B_Clk(clk50),
	.B_Enable(1'b1),	//	0 disable the port
	//Port 0
	//rx
	.B_Addrrx(rxAddr),
	.B_DataInrx(macDataInrx),
	.B_DataOutrx(),
	.B_WriteEnablerx(macWriteEnablerx),	// 1 write operation, 0 read operation
	//tx
	.B_Addrtx(txAddr),
	.B_DataIntx(8'b0),
	.B_DataOuttx(txData),
	.B_WriteEnabletx(restart[0])	// 1 write operation, 0 read operation
);

Mac mac_1(
	.Clk(clk50),	//50 MHz
	.Reset(reset|mode[1]|restart[0]),
	//MII
	//Port 0 Signals
	//Signals for Transmitting
	.TxClk(TxClk),
	.Txd(Txd),
	.TxEn(TxEn),
	//Signals for Receiving
	.RxClk(RxClk),
	.Rxd(Rxd),
	.RxErr(RxErr),
	.RxDV(RxDV),
	.Col(Col),
	.Crs(Crs),
	
	
	
	
	//Cpu Signals
	.TxDataIn(txData),
	.RxDataOut(rxData),
	
	.RxNext(rxNext),	//high when Rx*DataOut is valid
	.RxLastByte(rxLastByte),
	.RxAbort(rxAbort),
	.TxNext(txNext),	//high when next byte should be send
	.TxAvail(txAvail),	//indicate if frame for portx is availible
	.TxLastByte(txStop),
	.RxRec(rxRec), 	//Status signals
	.TxTrans(txTrans),
	.Promiscuous(promiscuous),	//when high every package is send to controler
	.IgnoreCrc(ignoreCrc),
	.IgnoreBroadcast(ignoreBroadcast),
	.RxCrcWrong(rxCrcWrong),
	.MacAddr({MAC_ADDRESS_HIGH,MAC_ADDRESS_LOW}),
	.RxCrcErrorCnt(crcErrorCounter),
	.RxRecCnt(rxRecCnt),
	.TxTransCnt(txTransCnt)
);
wire [4:0] mdioCnt;
MdioModule mdio1(
	.Clk(clk50),
	.Reset(reset),
	.Mdc(Mdc),
	.Mdio(Mdio),
	//control signals
	.Start(mdioStart),
	.WriteEnable(mdioWriteEnable),
	.PhyAddr(mdioPhyAddr),
	.RegAddr(mdioRegAddr),
	.Data(mdioData),
	.Result(mdioResult),
	.Ready(mdioReady),
	.mdioOut(mdioOut),
	.cnt(mdioCnt)
);
//******************
// Implementation
//*****************

assign promiscuous = mode[0];

//interrupt signals
//high when interupt enabled and a interrupt event occured
assign rx_ir = irqenRx && irqRx;
assign tx_ir = irqenTx && irqTx;

//write signals for dma mem(PortB)
assign macWriteEnablerx = rxNext || rxPreStop || rxStop || !rxTransfering;

//address of this module on SpartanMC
assign moduleAddr = {addr_high,addr_peri};

assign txAvail = (txLowAvail || txHighAvail);
//start, stop signals
assign rxStart = oldRxRec == 0 && rxRec == 1;


assign txPreStart = oldTxTrans == 0 && txTrans == 1;

assign txStop = (txByteCnt == 0);

assign txEndTrans = (oldTxTrans == 1 && txTrans == 0);

//address calculation

//special handling for idle case as first word is 2 byte long
//order first rxStop then rxPreStop is important!
assign rxPreAddr = (rxStop) ? rxAddrCnt+1 : (rxPreStop) ? rxAddrCnt :
	(rxOffset) ? rxAddrCnt + rxOffset + rxByteCnt + 1 : rxAddrCnt + globalToggle[0];

assign txPreAddr = (txStart) ? txAddrCnt + 1 : (txPreStart) ? txAddrCnt : txAddrCnt + txOffset;
// 8 bit input: B0 B1 become 18 bit word 2'b0,B1,B0
assign rxAddr = {rxPreAddr[10:1],!rxPreAddr[0]};
assign txAddr = {txPreAddr[10:1],!txPreAddr[0]};

//order first rxStop then rxPreStop is important!
assign macDataInrx = (rxStop) ? rxByteCnt[7:0] : 
			(rxPreStop) ? {5'b0,rxByteCnt[10:8]} : (rxTransfering) ? rxData : 8'b0;

// if frame ends exactly before ReadCnt then the start of the next reception
// will set the Offset to 1 and AddrCnt + Offset are equal ReadCnt

//also possible that override protection is active 4 bytes before ReadCnt
//but than Offset have to be included in the sum
assign rxOverAddr0 = rxAddrCnt[10:1] + rxByteCnt[10:1] + 3;
assign rxOverAddr1 = rxAddrCnt[10:1] + rxByteCnt[10:1] + 1;

assign rxOverflow = rxOverAddr0 == rxReadCnt || rxOverAddr1 == rxReadCnt;

//mdio signals
assign mdioStart = reg_write[5];
assign mdioWriteEnable = do_peri[17];
assign mdioData = do_peri[15:0];

// reset signal for Phy is low active
assign PhyReset = !(reset | mode[1]);

//PwrDown is a low active signal
assign PwrDownA = !mode[2];	//A == port0
assign PwrDownB = !mode[3];	//B == port1

assign activeCounter = (counterIndex == 0) ? crcErrorCounter :
						(counterIndex == 1) ? rxRecCnt :
						(counterIndex == 3) ? txTransCnt : 18'b0;
						
reg selectMem2;
always @(posedge clk_peri) selectMem2 <= selectMem;

assign di_peri = (selectMem2 == 1) ? dmaDataOut :
						(reg_read[0]) ?
							{ignoreCrc,ignoreBroadcast,1'b0,txTrans,1'b0,txAvail,1'b0,
							counterIndex,1'b0,irqenTx,1'b0,irqenRx,mode} :
						(reg_read[1]) ? {8'b0,txAddrCnt} :
						
						(reg_read[3]) ? {8'b0,rxReadCnt} : 
						
						(reg_read[5]) ? {2'b0,mdioResult} : 
						(reg_read[6]) ? 
							{mdioReady,7'b0,mdioPhyAddr,mdioRegAddr} :
						(reg_read[7]) ?
							{14'b0,1'b0,irqTx,1'b0,irqRx} : 
						(reg_read[8]) ? activeCounter :
						(reg_read[9]) ? {2'b0,MAC_ADDRESS_LOW[15:0]} :
						(reg_read[10]) ? {2'b0,MAC_ADDRESS_HIGH[7:0],MAC_ADDRESS_LOW[23:16]} :
						(reg_read[11]) ? {2'b0,MAC_ADDRESS_HIGH[23:8]} :
							//{RxErr1,TxEn1,RxDV1,debTx1State,debRx1State} : 0;
							//{RxErr1,RxDV1,3'b0,debRx1State,Rxd1} : 0;
							//{RxDV1,debRx1State,debRx1Byte} : 
						(reg_read[12]) ?
							{1'b0,1'b0,1'b0,1'b0,1'b0,3'b0,1'b0} :
						//(reg_read[13]) ? {7'b0,rx1AddrCnt} :
						/*(reg_read[14]) ? {debLastDetect,debStop,debAbort,4'b0,rx1ByteCnt} :
						(reg_read[15]) ? {3'b0,debTx1State,1'b0,debTxState} :*/ 0;

//interactions with mac
always@(posedge clk50)
begin
	if(reset)
	begin
		restart <= 4'b1111;

		txAddrCnt <= 0;
		rxAddrCnt <= 0;
		
		rxByteCnt <= 0;
		txByteCnt <= 0;
		
		rxOffset <= 0;
		txOffset <= 0;
		
		rxTransfering <= 0;
		
		rxStop <= 0;
		
		rxPreStop <= 0;
		txStart <= 0;
		txSampleStart <= 0;
		txHighAvail <= 0;
		txLowAvail <= 0;
		globalToggle <= T_HIGH;
		
		txVLAN <= 0;
		
		irqRx <= 0;
		irqTx <= 0;
		
		counterIndex <= 0;

		ignoreCrc <= 0;
		ignoreBroadcast <= 0;
		
		oldRxRec <= 0;
		oldTxTrans <= 0;
	end
	else
	begin
		restart <= {1'b0,restart[3:1]};

		//set delayed signals
		oldRxRec <= rxRec;
		oldTxTrans <= txTrans;
		
		//toogleLoop: LOW->HIGH->LOW->....
		if(globalToggle == T_HIGH)globalToggle <= T_LOW;
		if(globalToggle == T_LOW) globalToggle <= T_HIGH;
		
		txStart <= txPreStart;
		txSampleStart <= txStart;
		
		//determine Avail and prior indicator
		//remember value is 1 clock later availible
		if(txTrans == 0)
		begin
			txOffset <= {8'b0,globalToggle};
			
			if(txOffset[4:0] == T_LOW) txLowAvail <= txData != 0;
			if(txOffset[4:0] == T_HIGH) txHighAvail <= txData != 0;
		end
		else
		begin
			txHighAvail <= 0;
			txLowAvail <= 0;
		end



		//*************************************
		//	RECEIVE
		//*************************************
		
		//counter calculation during data transfer
		//rx
		if(rxStart)
		begin
			rxOffset <= 1;
			rxTransfering <= 1;
			rxByteCnt <= 0;
		end

		if(rxTransfering && rxNext  && !rxOverflow) rxByteCnt <= rxByteCnt + 1;
		
		//set Addr to Addr after current frame, if transfer happened
		rxPreStop <= rxNext && rxLastByte && !rxAbort;
		rxStop <= rxPreStop;
		
		if(rxAbort)
		begin
			rxByteCnt <= 0;
			rxTransfering <= 0;
		end
		
		if(rxStop)
		begin
			//data was transfered and no overflow
			//last word + 1 word
			if((rxByteCnt != 0) && !rxOverflow)
			begin
				rxAddrCnt <= rxAddrCnt + {rxByteCnt[10:1],1'b0} + 2;
				//trigger interupt if it is enabled
				if(irqenRx == 1) irqRx <= 1;
			end

			rxOffset <= 0;
			rxTransfering <= 0;
		end
		
		//***********************************+
		//	TRANSMIT
		//***********************************
		
		//tx
		//txData is valid 1 tick after txStart signals
		if(txStart)
		begin
			txByteCnt[10:8] <= txData[2:0];
		end
		if(txSampleStart)
		begin
			txOffset <= 2;
			txByteCnt <= {txByteCnt[10:8],txData} - 1;
		end
		
		if(txTrans && txNext) txByteCnt <= txByteCnt - 1;

		
		//increase Offset if not lastByte
		if(txTrans && txNext && !txStop) txOffset <= txOffset + 1;

		//change txAddrCnt after transmission
		if(txEndTrans && txOffset != 2)
		begin
			txAddrCnt <= txAddrCnt + {txOffset[10:1],1'b0} + 2;
			//trigger interupt if it is enabled
			if(irqenTx == 1) irqTx <= 1;
		end
		
		//always set txOffset to 0
		if(txEndTrans)
		begin
			txOffset <= 0;
		end
		
		//set rgister value if address match
		if(reg_write[0])
		begin
			counterIndex <= do_peri[10:8];
			ignoreBroadcast <= do_peri[16];
			ignoreCrc <= do_peri[17];
		end
		if(reg_write[1])txAddrCnt <= do_peri[10:0];
		if(reg_write[7])
		begin
			//irq are reseted if writing a 1 to the register location
			//IRQ_Rx
			if(do_peri[0] == 1)	irqRx <= 0;
			//IRQ_Tx
			if(do_peri[2] == 1) irqTx <= 0;
		end
		
	end //end if reset
end


//spartanmc peri handling
always@(posedge clk_peri)
begin
	if(reset)
	begin
		mode <= 0;
		
		irqenRx <= 0;
		irqenTx <= 0;
		
		rxReadCnt <= 0;
		
		mdioPhyAddr <= 0;
		mdioRegAddr <= 0;
	end
	else
	begin
		//set register value if address match
		if(selectReg)
		begin
			if(reg_write[0])
			begin
				mode[3:0] <= do_peri[3:0];
				//set interrupt enable signals
				irqenRx <= do_peri[4];
				irqenTx <= do_peri[6];
			end
			//if(reg_write[1])txAddrCnt <= do_peri[9:0];
			//if(reg_write[2])tx1AddrCnt <= do_peri[9:0];
			if(reg_write[3])rxReadCnt <= do_peri[10:0];
			//if(reg_write[5]);	//done with logic
			if(reg_write[6])
			begin
				mdioPhyAddr <= do_peri[9:5];
				mdioRegAddr <= do_peri[4:0];
			end
		end//end if selectReg
		
	end//end if reset
end

//Stuff for MDIO
/*
assign debug[15:0] = mdioData;
assign debug[31:16] = mdioResult;
assign debug[32]=mdioStart;
assign debug[33]=mdioWriteEnable;
assign debug[34]=mdioReady;
assign debug[35]=Mdc;
assign debug[40:36]=mdioRegAddr;
assign debug[41]=selectReg;
assign debug[57:42]=reg_select;
assign debug[73:58]=reg_read;
assign debug[89:74]=reg_write;
assign debug[90]=wr_peri;
assign debug[95:91]=mdioCnt;
assign debug[100:97]=0;
assign debug[101]=access_peri;
assign debug[119:102]=di_peri;
assign debug[124:120]=mdioPhyAddr;
assign debug[125] = mdioOut;*/



//Stuff for dma
/*assign debug[17:0] = do_peri;
assign debug[35:18] = di_peri;
assign debug[45:36] = addr_peri;
assign debug[46]=access_peri;
assign debug[47]=wr_peri;
assign debug[48]=mem_access;
assign debug[49]=store_access;
assign debug[50]=store_access_low;
assign debug[51]=store_access_high;
assign debug[59:52]=addr_high;
assign debug[60]=selectMem;*/

	
assign debug[10:0] = txAddr;
assign debug[21:11] = txAddrCnt;
assign debug[29:22] = txData;
assign debug[33:30] = Txd;
assign debug[34] = TxEn;

endmodule
