/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:05:20 11/15/2010 
// Design Name: 
// Module Name:    Ethernet 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//important note
/*
The data is written bytewise in the dma buffer but readed in 18 bit words.
There ByteCnt describe how much bytes are valid. If an uneven byte number is
written, then it is possible that the low byte contains data from another frame.
*/
`default_nettype none
module EthernetRx(
//*** Verbindungen mit dem MC die nicht ge�ndert werden d�rfen ***

	input wire                   clk_peri,       //Systemtakt
	input wire           [17:0]  do_peri,        //Datenbus  vom MC
	output wire         [17:0]  di_peri,        //Datenbus  zum MC
	input wire           [9:0]   addr_peri,      //Addressbus vom MC
	input wire                   access_peri,    //Signale fuer Peripheriezugriffe
	input wire                   wr_peri,        //Schreibsignal

	// Blockram Interface
	input wire                   mem_access,
	input wire					store_access_low,
	input wire					store_access_high,
	input wire            [7:0]  addr_high,


	//*** Verbindungen mit dem MC die ver�ndert werden d�rfen ***

	input wire                   reset,          //Reset-Signal

	//interrupts
	output wire             rx_ir,  //Interrupt for rx event


	//MAC signals
	
	//Signals for Receiving
	input wire RxClk,
	input wire [3:0] Rxd,
	input wire RxErr,
	input wire RxDV,
	input wire Col,
	input wire Crs
);



// Die Parameter fuer die Adressen werden in der system.v eingestellt.
parameter BASE_ADR = 10'h100;        //Offset zur Basisadresse der I/O Ports
parameter DMA_BASE_ADR = 18'h19000;     //4096 Adressen unterhalb der I/O Ports
// Mac address is splitted to avoid number overflow in jconfig
// in version 2 string parameter should be availible, so concat 
// both parameters if it is availible
parameter [23:0] MAC_ADDRESS_HIGH = 24'h0010A4;
parameter [23:0] MAC_ADDRESS_LOW = 24'h7BEA80;


//****SpartanMC registers*****


reg promiscuous;
reg irqenRx;
reg irqRx;
reg ignoreBroadcast;
reg ignoreCrc;
reg overflow;



//rxReadCnt, Address of first valid frame
reg [9:0] rxReadCnt_peri, rxReadCnt_buf, rxReadCnt_mac;

//****local signals****
//single register delay the reset signal not enough to set byte
//at address 0 to 0
reg [3:0] restart;	//indicate if a reset sig was high

wire selectMem;	//select signal for dma memory access
wire selectReg;	//select signal for register access
wire [17:0] moduleAddr;
wire [17:0] dmaDataOut;
//**MAC side signals**
//mem signals

//rx
wire [7:0] macDataInrx;


wire macWriteEnablerx;


//mac signals
wire [7:0] rxData;

wire rxNext;
wire rxLastByte;
wire rxAbort;
wire rxRec;
wire rxCrcWrong;



// Counter & Statussignals
//delayed status signals
reg oldRxRec;

//indicate if data between mac and cpu is transfered (rx only)
reg rxTransfering;

//start stop indicators
wire rxStart;
reg rxStop;

//addresses which trigger overflow
wire [9:0] rxOverAddr0;
wire [9:0] rxOverAddr1;

//store the start Address of the FIRST valid frame (in rx section)
reg [10:0] rxAddrCnt;
//store the offset from the start Address of a frame
reg rxOffset;
// Address signals
wire [10:0] rxAddr;
//intermediary result
wire [10:0] rxPreAddr;

//stop of transmission need to be split in 2 steps
//1: write high byte of ByteCnt, 2: write low byte
reg rxPreStop;

//count the length of the frame
reg [10:0] rxByteCnt;

//indicate when memory overflow occure
wire rxOverflow;

//Mac counters
wire [17:0] crcErrorCounter;
wire [17:0] rxRecCnt;

reg globalToggle;

//********************************
// MODULES
//*******************************

pselect #(
		.ADDR_WIDTH     (8),
		.BASE_WIDTH     (8),
		.BASE_ADDR      (DMA_BASE_ADR >> 10) 
	)
 pseldma			 
	(moduleAddr[17:10],mem_access,selectMem
);

parameter  ALL_PORTS		= 8,

	DECODSIM_DEB		= "NO";
wire					select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	selectReg		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);
			


DmaMemory18to8 dmamem18to8(
	.Reset(reset),
	//Port A - SpartanMC
	.A_Clk(clk_peri),
	.A_Addr(moduleAddr[11:0]),
	.A_DataIn(do_peri),
	.A_DataOut(dmaDataOut),
	.A_Enable(1'b1),	//always enable	
	.A_WriteEnable((store_access_high&&store_access_low) & selectMem),

	//Port B - Ethernet module
	.B_Clk(RxClk),
	.B_Enable(1'b1),	//	0 disable the port
	//Port 0
	//rx
	.B_Addr(rxAddr),
	.B_DataIn(macDataInrx),
	.B_DataOut(),
	.B_WriteEnable(macWriteEnablerx)	// 1 write operation, 0 read operation
);

MacRx mac_1(
	.Reset(reset|restart[0]),
	//MII
	//Signals for Receiving
	.RxClk(RxClk),
	.Rxd(Rxd),
	.RxErr(RxErr),
	.RxDV(RxDV),
	.Col(Col),
	.Crs(Crs),
	
	
	
	
	//Cpu Signals
	.RxDataOut(rxData),
	
	.RxNext(rxNext),	//high when Rx*DataOut is valid
	.RxLastByte(rxLastByte),
	.RxAbort(rxAbort),
	.RxRec(rxRec), 	//Status signals
	.Promiscuous(promiscuous),	//when high every package is send to controler
	.IgnoreCrc(ignoreCrc),
	.IgnoreBroadcast(ignoreBroadcast),
	.RxCrcWrong(rxCrcWrong),
	.MacAddr({MAC_ADDRESS_HIGH,MAC_ADDRESS_LOW}),
	.RxCrcErrorCnt(crcErrorCounter),
	.RxRecCnt(rxRecCnt)
);

//******************
// Implementation
//*****************


//interrupt signals
//high when interupt enabled and a interrupt event occured
assign rx_ir = irqenRx && irqRx;

//write signals for dma mem(PortB)
assign macWriteEnablerx = rxNext || rxPreStop || rxStop || !rxTransfering;

//address of this module on SpartanMC
assign moduleAddr = {addr_high,addr_peri};

//start, stop signals
assign rxStart = oldRxRec == 0 && rxRec == 1;



//address calculation

//special handling for idle case as first word is 2 byte long
//order first rxStop then rxPreStop is important!
assign rxPreAddr = (rxStop) ? rxAddrCnt+1 : (rxPreStop) ? rxAddrCnt :
	(rxOffset) ? rxAddrCnt + rxOffset + rxByteCnt + 1 : rxAddrCnt + globalToggle;

// Invert lowest bit
assign rxAddr = {rxPreAddr[10:1],!rxPreAddr[0]};

//order first rxStop then rxPreStop is important!
assign macDataInrx = (rxStop) ? rxByteCnt[7:0] : 
			(rxPreStop) ? {5'b0,rxByteCnt[10:8]} : (rxTransfering) ? rxData : 8'b0;

// if frame ends exactly before ReadCnt then the start of the next reception
// will set the Offset to 1 and AddrCnt + Offset are equal ReadCnt

//also possible that override protection is active 4 bytes before ReadCnt
//but than Offset have to be included in the sum
assign rxOverAddr0 = rxAddrCnt[10:1] + rxByteCnt[10:1] + 3;
assign rxOverAddr1 = rxAddrCnt[10:1] + rxByteCnt[10:1] + 1;

assign rxOverflow = rxOverAddr0 == rxReadCnt_mac || rxOverAddr1 == rxReadCnt_mac;

reg irqReset_buf, irqReset_peri, irqReset_mac, irqReset_mac2;
	
//interactions with mac
always@(posedge RxClk)
begin
	if(reset)
	begin
		restart <= 4'b1111;

		rxAddrCnt <= 0;
		
		rxByteCnt <= 0;
		
		rxOffset <= 0;
		
		rxTransfering <= 0;
		
		rxStop <= 0;
		
		rxPreStop <= 0;
		
		
		irqRx <= 0;		
		
		oldRxRec <= 0;
		
		globalToggle <= 0;
	end
	else
	begin
		restart <= {1'b0,restart[3:1]};

		//set delayed signals
		oldRxRec <= rxRec;
		
		globalToggle <= ~globalToggle;
		
		//*************************************
		//	RECEIVE
		//*************************************
		
		//counter calculation during data transfer
		//rx
		if(rxStart)
		begin
			rxOffset <= 1;
			rxTransfering <= 1;
			rxByteCnt <= 0;
		end

		if(rxTransfering && rxNext  && !rxOverflow) rxByteCnt <= rxByteCnt + 1;
		
		//set Addr to Addr after current frame, if transfer happened
		rxPreStop <= rxLastByte && !rxAbort;
		rxStop <= rxPreStop;
		
		if(rxAbort)
		begin
			rxByteCnt <= 0;
			rxTransfering <= 0;
		end
		
		if(rxStop)
		begin
			//data was transfered and no overflow
			//last word + 1 word
			if((rxByteCnt != 0) && !rxOverflow)
			begin
				rxAddrCnt <= rxAddrCnt + rxByteCnt + rxByteCnt[0] + 2;
				//trigger interupt if it is enabled
				if(irqenRx == 1) irqRx <= 1;
			end

			rxOffset <= 0;
			rxTransfering <= 0;
		end
		
		
		if (irqReset_mac && !irqReset_mac2)
			irqRx<= 0;
		
		
	end //end if reset
end

//Buffer all signals crossing the clock domain twice to remove metastability
always @(posedge RxClk) begin
	irqReset_buf <= irqReset_peri;
	irqReset_mac <= irqReset_buf;
	irqReset_mac2 <= irqReset_mac;
	
	rxReadCnt_buf <= rxReadCnt_peri;
	rxReadCnt_mac <= rxReadCnt_buf;
end

reg selectMem2;
always @(posedge clk_peri) selectMem2 <= selectMem;

assign di_peri = (selectMem2 == 1) ? dmaDataOut :
						(reg_read[0]) ? {13'h0, overflow,ignoreCrc,ignoreBroadcast,irqenRx,promiscuous} :
						(reg_read[1]) ? {8'b0,rxReadCnt_peri} : 
						(reg_read[2]) ? {17'b0,irqRx} : 
						(reg_read[3]) ? rxRecCnt :
						(reg_read[4]) ? crcErrorCounter :
						(reg_read[5]) ? {2'b0,MAC_ADDRESS_LOW[15:0]} :
						(reg_read[6]) ? {2'b0,MAC_ADDRESS_HIGH[7:0],MAC_ADDRESS_LOW[23:16]} :
						(reg_read[7]) ? {2'b0,MAC_ADDRESS_HIGH[23:8]} :
							0;

//spartanmc peri handling
always@(posedge clk_peri)
begin
	if(reset)
	begin
		irqenRx <= 0;
		rxReadCnt_peri <= 0;

		ignoreCrc <= 0;
		ignoreBroadcast <= 0;
		
		overflow <= 0;
	end
	else
	begin
		if(reg_write[0])
		begin
			promiscuous <= do_peri[0];
			irqenRx <= do_peri[1];
			ignoreBroadcast <= do_peri[2];
			ignoreCrc <= do_peri[3];
			overflow <= do_peri[4];
		end
		else begin
			if(rxOverflow)
				overflow <= 1;
		end
		
		if(reg_write[1]) begin
			rxReadCnt_peri <= do_peri[10:0];
		end
		
		if(reg_write[2])
		begin
			irqReset_peri <= 1;
		end
		
		if (!irqRx)
			irqReset_peri <= 0;
		
	end//end if reset
end



endmodule
