/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:05:20 11/15/2010 
// Design Name: 
// Module Name:    Ethernet 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//important note
/*
The data is written bytewise in the dma buffer but readed in 18 bit words.
There ByteCnt describe how much bytes are valid. If an uneven byte number is
written, then it is possible that the low byte contains data from another frame.
*/
`default_nettype none
module EthernetTx(
//*** Verbindungen mit dem MC die nicht ge�ndert werden d�rfen ***

	input wire                   clk_peri,       //Systemtakt
	input wire           [17:0]  do_peri,        //Datenbus  vom MC
	output wire         [17:0]  di_peri,        //Datenbus  zum MC
	input wire           [9:0]   addr_peri,      //Addressbus vom MC
	input wire                   access_peri,    //Signale fuer Peripheriezugriffe
	input wire                   wr_peri,        //Schreibsignal

	// Blockram Interface
	input wire                   mem_access,
	input wire					store_access_low,
	input wire					store_access_high,
	input wire            [7:0]  addr_high,


	//*** Verbindungen mit dem MC die ver�ndert werden d�rfen ***

	input wire                   reset,          //Reset-Signal

	//interrupts
	output wire				tx_ir,	//Interrupt for tx event


	//MAC signals
	//Signals for Transmitting
	input wire TxClk,
	output wire [3:0] Txd,
	output wire TxEn,
	
	output wire [65:0] debug
);



// Die Parameter fuer die Adressen werden in der system.v eingestellt.
parameter BASE_ADR = 10'h100;        //Offset zur Basisadresse der I/O Ports
parameter DMA_BASE_ADR = 18'h19000;     //4096 Adressen unterhalb der I/O Ports

//parameter for the globalToggle-Loop
parameter [4:0] T_HIGH = 0;		// High byte of the Length field
parameter [4:0] T_LOW = 1;		// Low byte of the Length field

//****SpartanMC registers*****

reg irqenTx;
reg irqTx;

//store the start Address of the FIRST valid frame (in tx section)
//register 1: txAddrCnt, Address of current frame, which should be sent
reg [10:0] txAddrCnt;



//****local signals****
//single register delay the reset signal not enough to set byte
//at address 0 on both tx ports to 0
reg [3:0] restart;	//indicate if a reset sig was high

wire selectMem;	//select signal for dma memory access
wire selectReg;	//select signal for register access
wire [17:0] moduleAddr;
wire [17:0] dmaDataOut;
//**MAC side signals**
//mem signals


//mac signals
wire [7:0] txData;


wire txNext;
wire txAvail;

wire txTrans;

// frame avail indicator for low and high byte of ByteCnt
reg txLowAvail;
reg txHighAvail;


// Counter & Statussignals
//delayed status signals
reg oldTxTrans;


reg txLastByte;
reg txStop;	//signal that mac should end transmission (no more data)

// ByteCnt have to be readed in 2 steps
wire txPreStart;
reg txStart;
reg txSampleStart;

wire txEndTrans;	//signal that mac had finished transmission


//store the offset from the start Address of a frame
reg [10:0] txOffset;
// Address signals
wire [10:0] txAddr;
//toggle signal for special handling(address calc) in idle state
reg [4:0] globalToggle;

//intermediary result
wire [10:0] txPreAddr;


//count the length of the frame
reg [10:0] txByteCnt;


//Mac counters
wire [17:0] txTransCnt;


//********************************
// MODULES
//*******************************

pselect #(
		.ADDR_WIDTH     (8),
		.BASE_WIDTH     (8),
		.BASE_ADDR      (DMA_BASE_ADR >> 10) 
	)
 pseldma			 
	(moduleAddr[17:10],mem_access,selectMem
);

parameter  ALL_PORTS		= 4,

	DECODSIM_DEB		= "NO";
wire					select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	selectReg		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);
			


DmaMemory18to8 dmamem18to8(
	.Reset(reset),
	//Port A - SpartanMC
	.A_Clk(clk_peri),
	.A_Addr(moduleAddr[11:0]),
	.A_DataIn(do_peri),
	.A_DataOut(dmaDataOut),
	.A_Enable(1'b1),	//always enable	
	.A_WriteEnable((store_access_high&&store_access_low) & selectMem),

	//Port B - Ethernet module
	.B_Clk(TxClk),
	.B_Enable(1'b1),	//	0 disable the port
	//tx
	.B_Addr(txAddr),
	.B_DataIn(8'b0),
	.B_DataOut(txData),
	.B_WriteEnable(restart[0])	// 1 write operation, 0 read operation
);

MacTx mac_1(
	.Reset(reset|restart[0]),
	//MII
	//Port 0 Signals
	//Signals for Transmitting
	.TxClk(TxClk),
	.Txd(Txd),
	.TxEn(TxEn),
	
	
	//Cpu Signals
	.TxDataIn(txData),
	
	.TxNext(txNext),	//high when next byte should be send
	.TxAvail(txAvail),	//indicate if frame for portx is availible
	.TxLastByte(txStop),
	.TxTrans(txTrans),
	.TxTransCnt(txTransCnt)
);

//******************
// Implementation
//*****************


//interrupt signals
//high when interupt enabled and a interrupt event occured
assign tx_ir = irqenTx && irqTx;


//address of this module on SpartanMC
assign moduleAddr = {addr_high,addr_peri};

assign txAvail = (txLowAvail || txHighAvail);
//start, stop signals



assign txPreStart = oldTxTrans == 0 && txTrans == 1;

assign txEndTrans = (oldTxTrans == 1 && txTrans == 0);

//address calculation


assign txPreAddr = (txStart) ? txAddrCnt + 1 : (txPreStart) ? txAddrCnt : txAddrCnt + txOffset;
// 8 bit input: B0 B1 become 18 bit word 2'b0,B1,B0
assign txAddr = {txPreAddr[10:1],!txPreAddr[0]};


//=== Cross clock domain paths

//mac -> peri
reg txAddrCntAccept_mac;
reg txAddrCntAccept_buf;
reg txAddrCntAccept_peri;

//peri -> mac


reg [10:0] txAddrCnt_peri;
reg txAddrCntReq_peri;
reg irqReset_peri;

reg [10:0] txAddrCnt_buf;
reg txAddrCntReq_buf;
reg irqReset_buf;



reg [10:0] txAddrCnt_mac;
reg txAddrCntReq_mac;
reg irqReset_mac;
reg irqReset_mac2;

//interactions with mac
always@(posedge TxClk)
begin
	if(reset)
	begin
		restart <= 4'b1111;

		txAddrCnt <= 0;
		
		txByteCnt <= 0;
		
		txOffset <= 0;
		
		txStart <= 0;
		txSampleStart <= 0;
		txHighAvail <= 0;
		txLowAvail <= 0;
		globalToggle <= T_HIGH;
		
		irqTx <= 0;
	
		
		oldTxTrans <= 0;
		
		txStop <= 0;
	end
	else
	begin
		restart <= {1'b0,restart[3:1]};

		//set delayed signals
		oldTxTrans <= txTrans;
		
		//toogleLoop: LOW->HIGH->LOW->....
		if(globalToggle == T_HIGH)globalToggle <= T_LOW;
		if(globalToggle == T_LOW) globalToggle <= T_HIGH;
		
		txStart <= txPreStart;
		txSampleStart <= txStart;
		
		//determine Avail and prior indicator
		//remember value is 1 clock later availible
		if(txTrans == 0)
		begin
			txOffset <= {8'b0,globalToggle};
			
			if(txOffset[4:0] == T_LOW) txLowAvail <= txData != 0;
			if(txOffset[4:0] == T_HIGH) txHighAvail <= txData != 0;
		end
		else
		begin
			txHighAvail <= 0;
			txLowAvail <= 0;
		end
		
		//***********************************+
		//	TRANSMIT
		//***********************************
		
		//tx
		//txData is valid 1 tick after txStart signals
		if(txStart)
		begin
			txByteCnt[10:8] <= txData[2:0];
		end
		if(txSampleStart)
		begin
			txOffset <= 2;
			txByteCnt <= {txByteCnt[10:8],txData} - 1;
		end
		
		if(txTrans && txNext) txByteCnt <= txByteCnt - 1;
		
		txLastByte <= txByteCnt == 0;
		txStop <= txLastByte;

		
		//increase Offset if not lastByte
		if(txTrans && txNext && !txStop) txOffset <= txOffset + 1;

		//change txAddrCnt after transmission
		if(txEndTrans && txOffset != 2)
		begin
			txAddrCnt <= txAddrCnt + txOffset + txOffset[0];
			//trigger interupt if it is enabled
			if(irqenTx == 1) irqTx <= 1;
		end
		
		//always set txOffset to 0
		if(txEndTrans)
		begin
			txOffset <= 0;
		end
		
		if (irqReset_mac && !irqReset_mac2)
			irqTx<= 0;
			
			
		if (txAddrCntReq_mac && !txAddrCntAccept_mac) begin
			txAddrCnt <= txAddrCnt_mac;
			txAddrCntAccept_mac <= 1;
		end
		
		if (!txAddrCntReq_mac)
			txAddrCntAccept_mac <= 0;

		
	end //end if reset
end




//Buffer all signals crossing the clock domain twice to remove metastability
always @(posedge TxClk) begin

	txAddrCnt_buf <= txAddrCnt_peri;
	txAddrCntReq_buf <= txAddrCntReq_peri;
	irqReset_buf <= irqReset_peri;
	

	txAddrCnt_mac <= txAddrCnt_buf;
	txAddrCntReq_mac <= txAddrCntReq_buf;
	irqReset_mac <= irqReset_buf;
	
	irqReset_mac2 <= irqReset_mac;
	
end
always @(posedge clk_peri) begin

	txAddrCntAccept_buf <= txAddrCntAccept_mac;
	txAddrCntAccept_peri <= txAddrCntAccept_buf;

end


reg selectMem2;
always @(posedge clk_peri) selectMem2 <= selectMem;

assign di_peri = (selectMem2 == 1) ? dmaDataOut :
						(reg_read[0]) ? {15'b0, txTrans,txAvail,irqenTx} :
						(reg_read[1]) ? {8'b0,txAddrCnt} :
						(reg_read[2]) ? {17'b0,irqTx} : 
						(reg_read[3]) ? txTransCnt : 0;

//spartanmc peri handling
always@(posedge clk_peri)
begin
	if(reset)
	begin
		
	
		irqenTx <= 0;
		
		txAddrCntReq_peri <= 0;
		txAddrCnt_peri <= 0;
		irqReset_peri <= 0;
		
	end
	else
	begin
		if(reg_write[0])
		begin
			//set interrupt enable signals
			irqenTx <= do_peri[0];
		end
		
		if(reg_write[1] && !txAddrCntReq_peri) begin
			txAddrCnt_peri <= do_peri[10:0];
			txAddrCntReq_peri <= 1;
		end
	
		if(reg_write[2] && irqTx)
		begin
			//reset irq
			irqReset_peri <= 1;
		end
		
		
		if (!irqTx)
			irqReset_peri <= 0;
			
		if (txAddrCntAccept_peri)
			txAddrCntReq_peri <= 0;
		
	end//end if reset
end

assign debug[3:0] = restart;
assign debug[14:4] = txAddrCnt;
assign debug[25:15] = txByteCnt;
assign debug[33:26] = txData;
assign debug[34] = txLowAvail;
assign debug[35] = txHighAvail;
assign debug[36] = txAvail;
assign debug[37] = txStart;
assign debug[38] = txSampleStart;
assign debug[49:39] = txAddr;
assign debug[54:50] = globalToggle;
assign debug[65:55] = txOffset;


endmodule
