/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: Matthias Hofmann
// 
// Create Date:    16:51:31 07/05/2010 
// Design Name: 
// Module Name:    mac 
// Project Name: Gro�er Beleg
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
`default_nettype none
module MacRx( 
	input wire Reset,
	
	//MII
	//Port 0 Signals
	//Signals for Receiving
	input wire RxClk,
	input wire [3:0] Rxd,
	input wire RxErr,
	input wire RxDV,
	input wire Col,
	input wire Crs,
	
	//Cpu Signals
	output wire [7:0] RxDataOut,
	output wire RxNext,	//high when Rx*DataOut is valid
	output wire RxLastByte, //high if last Byte of current frame is sent
	output wire RxAbort,	//abort signal for rx ports, indicate when crc error or transfer error occured
	output wire RxRec, 	//Status signals
	input wire Promiscuous,	//when high every package is send to controler
	input wire IgnoreCrc,	//when high all crc errors are ignored
	input wire IgnoreBroadcast,	//when enabled traffic with broadcast adress is ignored
	output wire RxCrcWrong,	//indicator for crc errors, informative signals
	input wire [47:0] MacAddr,
	output reg [17:0] RxCrcErrorCnt, //Number of received packets with crc error
	output wire [17:0] RxRecCnt //Number of received packets
);

parameter[31:0] MAGIC_NUMBER = 32'hC704DD7B;

//includes

`include "ethernet/src/RxStateType.vh"

//local Signals
//counters
//complete received counter
reg [17:0] rxRecCnt;


//generate variable used to change bit order of crcsum
genvar i;

//RX
wire [31:0]rxCrcsum;
wire rxCrcError;
wire rxCrcWrong;	//indicate that crc was not correct, only informative signal
wire rxResetCrc;
wire rxCrcStrobe;
wire [7:0] rxByte;
wire rxStrobe;
wire rxOddPreamble;
wire [8:0] rxState;
wire rxDVIntern;

reg [7:0] rxByteBuf;


//control signals


wire rxEnable;

wire identicalMacRx;
wire broadcastRx;
reg [47:0] macAddrRx;

wire rxMacMatch;

//******************************
//	Modules
//********************************
//Port 0
Crc32_8 crc_rx(RxClk,rxCrcStrobe,rxResetCrc,rxByte,rxCrcsum);
RxByteBuffer rxByteBuffer_inst(RxClk,Reset,Rxd,rxOddPreamble,RxDV,rxByte,rxStrobe,rxDVIntern);
RxState rxState_inst(RxClk,Reset,rxByte,RxErr,rxDVIntern,rxStrobe, rxMacMatch,rxState,rxOddPreamble);


//********************************
// Implementation
//*******************************	 


//NORMAL
assign RxRecCnt = rxRecCnt;
assign rxCrcStrobe = rxStrobe && (rxState == REC_DSTMAC || 
										rxState == REC_SRCMAC ||
										rxState == REC_DATA);




										
assign rxCrcWrong = rxState == CHECK_CRC && rxCrcsum != MAGIC_NUMBER;


assign RxCrcWrong = rxCrcWrong;


assign rxCrcError = rxCrcWrong & !IgnoreCrc;


assign RxAbort = rxCrcError || rxState == ERROR;


assign rxResetCrc = rxState == REC_IDLE;


//accept Mac Address range
assign identicalMacRx = (MacAddr[47:0] == macAddrRx[47:0]);// && (MacAddr[0] == 1'b0);

assign broadcastRx = !IgnoreBroadcast && (macAddrRx == ~48'b0);


assign rxMacMatch = identicalMacRx | broadcastRx | Promiscuous;







assign RxNext = rxCrcStrobe;
//Rx last Byte indicators for cpu
assign RxLastByte = rxState == CHECK_CRC;

assign RxRec = (rxState != REC_IDLE);

//data signals to cpu
assign RxDataOut = rxByte;

always@(posedge RxClk or posedge Reset)
begin
	if(Reset)
	begin
		
		macAddrRx <= 0;
		
		
		RxCrcErrorCnt <= 0;
		rxRecCnt <= 0;
	end
	else
	begin
		//increase crc counter if an crc error signal is asserted
		if(rxCrcError) RxCrcErrorCnt <= RxCrcErrorCnt + 1;
		//increase rec counter if the transmission ends
		//on the end always the crcsum is checked
		if(rxState == CHECK_CRC) rxRecCnt <= rxRecCnt + 1;
		
		//reset MacAddr signal on next package and priority register
		if(rxState == REC_PRE)
		begin
			macAddrRx <= 0;
		end
		
		//store DstMacAddr
		if((rxState == REC_DSTMAC) && rxStrobe) macAddrRx <= {macAddrRx[39:0],rxByte};
		
	end
end

endmodule
