/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: Matthias Hofmann
// 
// Create Date:    16:51:31 07/05/2010 
// Design Name: 
// Module Name:    mac 
// Project Name: Gro�er Beleg
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
`default_nettype none
module MacTx( 
	input wire Reset,
	
	//MII
	//Port 0 Signals
	//Signals for Transmitting
	input wire TxClk,
	output reg [3:0] Txd,
	output reg TxEn,
	
	//Cpu Signals
	input wire [7:0] TxDataIn,
	output wire TxNext,	//high when next byte should be send, else 0!
	input wire TxAvail,	//indicate if frame for portx is availible
	input wire TxLastByte,
	output wire TxTrans,
	
	output wire [17:0] TxTransCnt //Number of transmitted packets
);

parameter[31:0] MAGIC_NUMBER = 32'hC704DD7B;

//includes

`include "ethernet/src/TxStateType.vh"

//local Signals
//counters
//complete transfered counter
reg [17:0] txTransCnt;


//generate variable used to change bit order of txCrcsum
genvar i;

//TX
reg internTxEn;
wire [31:0]txCrcsum;
wire txResetCrc;
wire txCrcStrobe;
wire [1:0] txCnt;
wire [7:0] txCrcByte0;
wire [7:0] txCrcByte1;
wire [7:0] txCrcByte2;
wire [7:0] txCrcByte3;
wire [7:0] txFromMem;	//bytes from InterPortBuffer
wire [7:0] txByte;
wire txStrobe;
wire txStart;
wire txFrameDel;
wire [6:0] txState;


wire [3:0] TxdX;
wire TxEnX;
	
//control signals

reg txDetectEnd;

wire txReadEnable;


//******************************
//	Modules
//********************************
Crc32_8 crc_tx(TxClk,txCrcStrobe,txResetCrc,TxDataIn,txCrcsum);
TxByteBuffer txByteBuffer_inst(TxClk,Reset,txByte,internTxEn,TxdX,TxEnX,txStrobe);

TxState txState_inst(TxClk,Reset,txStart,TxLastByte,txStrobe,txState,txCnt);


//********************************
// Implementation
//*******************************	 

always @(negedge TxClk) begin
	Txd <= TxdX;
	TxEn <= TxEnX;
end


//NORMAL
assign TxTransCnt = txTransCnt;



assign txCrcStrobe = txStrobe && (txState == TRANS_FRAME || txState == TRANS_SFD) && !TxLastByte;


assign txResetCrc = txState == TRANS_IDLE;


//start transmission if TRANS_IDLE and data availible from "active" source
assign txStart = txState == TRANS_IDLE && TxAvail ;

assign txReadEnable = (TxEnX && (txState == TRANS_FRAME || txState == TRANS_SFD));
	

//CrcBytes, the crcsum with changed bit order and inverted
generate
for(i = 0;i<8;i = i +1)
begin:G1
	assign txCrcByte0[i] = !txCrcsum[31-i];
end
for(i = 0;i<8;i = i +1)
begin:G2
	assign txCrcByte1[i] = !txCrcsum[23-i];
end
for(i = 0;i<8;i = i +1)
begin:G3
	assign txCrcByte2[i] = !txCrcsum[15-i];
end
for(i = 0;i<8;i = i +1)
begin:G4
	assign txCrcByte3[i] = !txCrcsum[7-i];
end

endgenerate

//select source of byte for tx1ByteBuffer
assign txByte = (txState == TRANS_PRE) ? 8'h55 :
	(txState == TRANS_SFD) ? 8'hD5 : 
	(txState == TRANS_FRAME) ? TxDataIn : 
	(txState == TRANS_CRC && txCnt == 0) ? txCrcByte0 :
	(txState == TRANS_CRC && txCnt == 1) ? txCrcByte1 :
	(txState == TRANS_CRC && txCnt == 2) ? txCrcByte2 :
	(txState == TRANS_CRC && txCnt == 3) ? txCrcByte3 : 8'b0;

//signals to CPU
// use strobe signal as nextByte signal, if data is transmitted
assign TxNext = txReadEnable && txStrobe;


//display idle state of each port
//compare to DONE is important, as without it unused low byte in the ethernet dmamemory
//can cause an wrong availible signal
assign TxTrans = (txState != TRANS_IDLE) && (txState != DONE);



always@(posedge TxClk or posedge Reset)
begin
	if(Reset)
	begin
		
		
		internTxEn <= 0;
		
		txTransCnt <= 0;
	end
	else
	begin
	
		//delayed signal to detect end of transmission
		if(txState == DONE) txDetectEnd <= 1;
		else txDetectEnd <= 0;
		
		if(txState == DONE)
		begin
			if(txDetectEnd == 0)
			begin
				//increase counter
				txTransCnt <= txTransCnt + 1;
				
			end
		end
	
		if(txStrobe)
		begin
			if((txState != TRANS_IDLE) && (txState != DONE) && (txState != PRIOR_ABORT) && !(txState == TRANS_CRC && txCnt == 3)) internTxEn <= 1;
			else internTxEn <= 0;
		end
		
	end
end

endmodule
