/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:05:20 11/15/2010 
// Design Name: 
// Module Name:    Ethernet 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

`default_nettype none
module MdioTop #(
	parameter WIDTH=1,

	// Die Parameter fuer die Adressen werden in der system.v eingestellt.
	parameter BASE_ADR = 10'h100        //Offset zur Basisadresse der I/O Ports
) (


	input wire                   clk_peri,       //Systemtakt
	input wire           [17:0]  do_peri,        //Datenbus  vom MC
	output wire         [17:0]  di_peri,        //Datenbus  zum MC
	input wire           [9:0]   addr_peri,      //Addressbus vom MC
	input wire                   access_peri,    //Signale fuer Peripheriezugriffe
	input wire                   wr_peri,        //Schreibsignal



	input wire                   clk50,				//50 MHz clock
	input wire                   reset,          //Reset-Signal

	//Managment Pins
	inout wire [WIDTH-1:0] Mdio,
	output wire Mdc,
	output wire Mdio_actual
);


//****SpartanMC registers*****

wire mdioStart;
wire mdioWriteEnable;
wire [15:0] mdioData;
wire [15:0] mdioResult;
// register 6: mdioReg2,
// {1'bReady,7'b0,1'bPhyAddress,5'bRegAddress}
wire mdioReady;
reg [4:0] mdioPhyAddr;
reg [4:0] mdioRegAddr;

wire mdioOut;

//********************************
// MODULES
//*******************************


parameter  ALL_PORTS		= 2,

	DECODSIM_DEB		= "NO";
wire					select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(			),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);

wire [4:0] mdioCnt;
MdioModule #(.WIDTH(WIDTH)) mdio1(
	.Clk(clk50),
	.Reset(reset),
	.Mdc(Mdc),
	.Mdio(Mdio),
	//control signals
	.Start(mdioStart),
	.WriteEnable(mdioWriteEnable),
	.PhyAddr(mdioPhyAddr),
	.RegAddr(mdioRegAddr),
	.Data(mdioData),
	.Result(mdioResult),
	.Ready(mdioReady),
	.mdioOut(mdioOut),
	.cnt(mdioCnt),
	.Mdio_actual(Mdio_actual)
);
//******************
// Implementation
//*****************


//mdio signals
reg startbuf;
reg webuf;
reg [15:0] databuf;
assign mdioStart = startbuf;
assign mdioWriteEnable = webuf;
assign mdioData = databuf;


assign di_peri = (reg_read[0]) ? {2'b0,mdioResult} : 
				 (reg_read[1]) ? {mdioReady,7'b0,mdioPhyAddr,mdioRegAddr} :
				 0;
				 
always@(posedge clk_peri)
begin
        if(reset)
        begin
			mdioPhyAddr <= 0;
			mdioRegAddr <= 0;
			startbuf <= 0;
			webuf <= 0;
			databuf <= 0;
        end

        else
        begin
            // Buffer start signal and data on start of transmission, decouples clock domains (clk_peri and clk50).
			if(reg_write[0])
		    begin
				startbuf <= 1'b1;
				webuf <= do_peri[17];
				databuf <= do_peri[15:0];
			end
			//set register value if address match
			else if(reg_write[1])
			begin
					mdioPhyAddr <= do_peri[9:5];
					mdioRegAddr <= do_peri[4:0];
					$display("wrote phy %X, reg %X, %X", do_peri[9:5], do_peri[4:0], do_peri);
			end
			// Clear startbuf once transmission has started.
			else if(startbuf && !mdioReady)
				startbuf <= 1'b0;
        end
end

endmodule
