/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    17:51:12 09/15/2010 
// Design Name: 
// Module Name:    RxState 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////


module RxState(
	input wire Clk,	//Systemclock
	input wire Reset,
	input wire [7:0] Rxd,	//Rxd from RxByteBuffer (Rxd(0) == LSB)
	input wire RxErr,//RxErr from MII
	input wire RxDV,	//RxDV from MII
	input wire RxStrobe,	//high when Rxd is valid
	input wire MacMatch,
	output wire [8:0] RState,
	output wire OddPreamble	//indicate if preample have an odd length
);

`include "ethernet/src/RxStateType.vh"


reg [8:0]state;
reg [3:0]cnt;
reg vlanTag;
reg odd;

//Implementation
always@(posedge Clk or posedge Reset)
begin
	if(Reset)
	begin
		state <= REC_IDLE;
		cnt <= 4'b0;
		vlanTag <= 1'b0;
		odd <= 0;
	end	//end if(Reset)
	else
	begin
		
		if(RxErr == 1'b1) state <= ERROR;
		else 
		begin
			case(state)
			REC_IDLE:
			begin
				cnt <= 4'b0;
				vlanTag <= 1'b0;
				odd <= 0;
				
				if(RxDV) state <= REC_PRE;
			end
			REC_PRE:
			begin
				//Preamble == 0x55
				//SFD == 0xD5
				if(RxStrobe == 1'b1)
				begin
					if(Rxd == 8'hD5) state <= REC_DSTMAC;
					else if(Rxd[3:0] == 4'hD)
					begin
						state <= REC_DSTMAC;
						odd <= 1;
					end
					else if(Rxd == 8'h55) state <= state;
					else state <= ERROR;
				end
			end
			//MAC is 8 byte long
			REC_DSTMAC:
			begin
				if(RxStrobe == 1'b1)
				begin
					cnt <= cnt + 1;
					if(cnt == 5)
					begin
						cnt <= 4'b0;
						state <= REC_SRCMAC;
					end
				end
			end
			REC_SRCMAC:
			begin
				if(RxStrobe == 1'b1)
				begin
					cnt <= cnt + 1;
					if(cnt == 5)
					begin
						cnt <= 4'b0;
						if (~MacMatch)
							state <= ERROR;
						else
							state <= REC_DATA;
					end
				end
			end
			REC_DATA:
			begin
				if(RxDV == 1'b0) state <=CHECK_CRC;
			end
			CHECK_CRC:
			begin
				state <= REC_IDLE;
			end
			ERROR:
			begin
				state <= REC_IDLE;
			end
			endcase
		end
		
		if((state != REC_IDLE && state != REC_DATA && state != CHECK_CRC
			&& state != ERROR) && RxDV == 0) state <= ERROR; //TODO use default in case...
	end	//end else(Reset == 0)
end	//end always(Clk)

assign RState = state;

assign OddPreamble = odd;

endmodule
