/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`default_nettype none
`timescale 1ns/1ps
module TbEthernetRx #(
	parameter FILENAME="rx.snoop"
)(
	input wire 		rxClk,
	output reg [3:0]	rxData,
	output wire 		rxDv,
	output wire		rxError
);


assign rxError = 0;


integer rxFile;
function [7:0] readByte;
input integer f;
begin
  $fread(readByte,f);
end
endfunction

function [31:0] readInt;
input integer f;
begin
  $fread(readInt,f);
end
endfunction

`define assert(condition, message) if(!(condition)) begin $display("assertion error: ",message); $stop(); end
`define assertFileByte(f,expected) `assert(readByte(f)==expected,"unexpected byte in file")
`define assertFileInt(f,expected) `assert(readInt(f)==expected,"unexpected byte in file")

reg haveData=1;
integer pktStartTime;
integer pktLen;
integer pktSkip = 0;
initial begin
    rxFile = $fopen(FILENAME,"rb");
    //$fscanf(rxFile,"%c",x);
    
    //Assert file header
    `assertFileByte(rxFile,115) //s
    `assertFileByte(rxFile,110) //n
    `assertFileByte(rxFile,111) //o
    `assertFileByte(rxFile,111) //o
    `assertFileByte(rxFile,112) //p
    `assertFileByte(rxFile,0)
    `assertFileByte(rxFile,0)
    `assertFileByte(rxFile,0)
    `assertFileInt(rxFile,2)
    `assertFileInt(rxFile,4)   
    
    haveData <= 0;
    
end

integer preamble = 8;


reg rxNibble = 0;
reg[7:0] rxByte;
reg rxdvint =0;
reg rxdvbefore =0;

always @(haveData) begin



	if (!haveData) begin
	
		while (pktSkip>0) begin:meep
			integer blah;
			blah = readByte(rxFile);
			$display("reading skip byte: %d",blah);
			pktSkip = pktSkip-1;
		end
		
		if ( !$feof(rxFile)) begin
		
    
			//Original packet length
			$display("pkt len: %d",readInt(rxFile));
			//included length
			pktLen = readInt(rxFile);
			$display("inc len: %d",pktLen);
			//Total recorded length is read, calculate amount of padding
			pktSkip = readInt(rxFile)-pktLen-24;
			$display("skiplen: %d",pktSkip);
			$display("drops:   %d",readInt(rxFile));
			pktStartTime = readInt(rxFile);
			$display("time s:  %d",pktStartTime);
			$display("time us: %d",readInt(rxFile));
			preamble = 8;
		end
		haveData <= 1;
	end
end

assign rxDv = rxdvint | rxdvbefore;
always @(posedge rxClk) begin



	if (~rxdvint && ~rxdvbefore) begin
		if (($time()>=pktStartTime) && (pktLen>0)) begin
			rxdvint <= 1;
			$display("starting transmission at %d",$time());
		end
	end else begin
		if (~rxNibble) begin
			if (preamble!=0) begin
				if (preamble==1)
					rxByte=8'hD5;
				else rxByte=8'h55;
				preamble = preamble-1;
			end else
				rxByte = readByte(rxFile);
			
		end else begin
			rxByte = rxByte >> 4;
			if (pktLen==0) begin
				rxdvint<=0;
				haveData <= 0;
			end
			if (preamble==0)
				pktLen = pktLen-1;
		end
		rxNibble = ~ rxNibble;
	end

	rxData <= rxByte[3:0];
	rxdvbefore <= rxdvint;

end

endmodule

