/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`default_nettype none
`timescale 1ns/1ps
module TbEthernetTx #(
	parameter FILENAME = "tx.snoop"
)(

	input wire 		txClk,
	input wire 		txEn,
	input wire [3:0]	txData
);

reg nibble = 0;
reg [3:0] last;

wire [7:0] txDataFull;
assign txDataFull = {txData, last};
reg lastTxEn;

integer txFile;

reg[7:0] packet[999:0];
reg[31:0] dataSize;
reg[31:0] packetStartTime;


initial begin
			txFile = $fopen(FILENAME);
			//File header
			$fwrite(txFile,"snoop");
			$fwrite(txFile,"%c%c%c",0,0,0);
			//Version
			$fwrite(txFile,"%c%c%c%c",0,0,0,2);
			//Ethernet
			$fwrite(txFile,"%c%c%c%c",0,0,0,4);
end

integer i;
always @(posedge txClk) begin
	if (txEn) begin
		if (!lastTxEn)
			packetStartTime <= $time();
		if (nibble) begin
			$display("TX: %X",txDataFull);
			packet[dataSize] <= txDataFull;
			dataSize<=dataSize+1;
		end
		last <= txData;
		nibble <= ~nibble;
	end else begin
		nibble = 0;
		if (lastTxEn) begin: writeFile
			reg[31:0] totalSize;
			reg[31:0] packetSize;
			packetSize = dataSize-8;
			totalSize = dataSize+24-8;



			$fwrite(txFile,"%c%c%c%c",packetSize[31:24],packetSize[23:16],packetSize[15:8],packetSize[7:0]);
			$fwrite(txFile,"%c%c%c%c",packetSize[31:24],packetSize[23:16],packetSize[15:8],packetSize[7:0]);
			
			$fwrite(txFile,"%c%c%c%c",totalSize[31:24],totalSize[23:16],totalSize[15:8],totalSize[7:0]);

			//Drops
			$fwrite(txFile,"%c%c%c%c",0,0,0,0);
			
			//Time
			$fwrite(txFile,"%c%c%c%c",packetStartTime[31:24],packetStartTime[23:16],packetStartTime[15:8],packetStartTime[7:0]);
			$fwrite(txFile,"%c%c%c%c",0,0,0,0);

			

			for (i = 8; i < dataSize; i = i +1) begin
				$fwrite(txFile,"%c",packet[i]);
			end
			
		
		end
		dataSize <= 0;
	end
	lastTxEn <= txEn;

end

endmodule
