/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:30:59 09/24/2010 
// Design Name: 
// Module Name:    TxByteBuffer 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

/*
	Requirements:
		- Nibble have to be stable at least for 10 ns before posedge of TxClk
*/

`default_nettype none
module TxByteBuffer(
	input wire TxClk,	// 25 MHz or 2,5 MHz
	input wire Reset,
	input wire [7:0] Byte,
	input wire TxEnableIn,
	output wire [3:0] Nibble,
	output wire TxEnableOut,
	output wire TxStrobe	//high when new byte should be loaded
    );

reg [7:0] buffer;
reg [7:0] buffer2;
reg cnt;
reg enable;
reg internStrobe;

//reg blockStrobe;

always@(posedge TxClk or posedge Reset)
begin
	if(Reset)
	begin
		buffer <= 0;
		internStrobe <= 0;
	end
	else
	begin
		internStrobe <= 0;
		
		if(cnt==1)
		begin
			internStrobe <= 1;
			buffer <= Byte;
		end
		
	end
end

always@(posedge TxClk or posedge Reset)
begin
	if(Reset)
	begin
		buffer2 <= 0;
		cnt <= 0;
		enable <= 0;
	end
	else
	begin
		buffer2 <= {4'b0,buffer2[7:4]};
		if(cnt == 0)
		begin
			buffer2 <= buffer;
			enable <= TxEnableIn;
		end
		cnt <= cnt + 1;
	end
end

assign TxEnableOut = enable;
assign TxStrobe = internStrobe ;
assign Nibble = buffer2[3:0];

endmodule
