/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ns
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:17:01 09/24/2010 
// Design Name: 
// Module Name:    TxState 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

`default_nettype none
module TxState(
	input wire Clk,
	input wire Reset,
	input wire DataAvailible,	//high if data is in the buffer
	input wire LastByte,	//high if byte from blockram have a framedelimiter==1, all the time
	//not required for full duplex
	//input Col,	//COL from MII
	//input Crs,	//CRS from MII
	input wire TxStrobe,
	output wire [6:0] TState,
	output wire [1:0] CntOut
    );

`include "ethernet/src/TxStateType.vh"

				
reg [3:0] cnt;
reg [6:0] state;
reg start;

always@(posedge Clk or posedge Reset)
begin
if(Reset)
begin
	state <= TRANS_IDLE;
	cnt <= 0;
	start <= 0;
end
else
begin
	if(DataAvailible) start <= 1;
	
	if(TxStrobe)
	begin
		case(state)
		TRANS_IDLE:
		begin
			cnt <= 0;
			if(start)
			begin
				state <= TRANS_PRE;
				start <= 0;
			end
		end
		TRANS_PRE:
		begin
			cnt <= cnt + 1;
			if(cnt == 7)
			begin
				state <= TRANS_SFD;
				cnt <= 0;
			end
		end
		TRANS_SFD:
		begin
			state <= TRANS_FRAME;
		end
		TRANS_FRAME:
		begin
			if(LastByte)
			begin
				state <= TRANS_CRC;
				cnt <= 0;
			end
		end
		TRANS_CRC:
		begin
			cnt <= cnt + 1;
			if(cnt == 3)
			begin
				cnt <= 0;
				state <= DONE;
			end
		end
		DONE:
		begin
			if(TxStrobe)
			begin
				cnt <= cnt + 1;
				//after interframe gap go to idle
				if(cnt == 11)	// 96 bit == 12 byte
				begin
					state <= TRANS_IDLE;
					cnt <= 0;
				end
			end
		end
		endcase
	end// end if(TxStrobe)
end
	
end	//end always

assign TState = state;
assign CntOut = cnt[1:0];

endmodule
