/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale      1ns / 10ps
`default_nettype none

module fancontrol (
	input	wire clk_i,
	input	wire reset_i,
	output 	wire pwm_o
);


parameter [17:0]
	LIMIT = 		18'h101D0,
	DUTY_CYCLE = 	18'h56CE,
	MAX_TEMP = 		60;
	
reg	[17:0]	counter;
wire [2:0] alarm;

SYSMON #(
	.INIT_40(16'h1000), // config reg 0
	.INIT_41(16'h200d), // config reg 1
	.INIT_42(16'h0a00), // config reg 2
	.INIT_48(16'h0101), // Sequencer channel selection
	.INIT_49(16'h0000), // Sequencer channel selection
	.INIT_4A(16'h0100), // Sequencer Average selection
	.INIT_4B(16'h0000), // Sequencer Average selection
	.INIT_4C(16'h0000), // Sequencer Bipolar selection
	.INIT_4D(16'h0000), // Sequencer Bipolar selection
	.INIT_4E(16'h0000), // Sequencer Acq time selection
	.INIT_4F(16'h0000), // Sequencer Acq time selection	
	.INIT_50({((MAX_TEMP+273)*2) & 10'hFFC, 6'b111010}), // Alarm limit register 0
	.INIT_51(16'h5999), // Vccint upper alarm limit
	.INIT_52(16'he000), // Vccaux upper alarm limit
	.INIT_53(16'hca33),  // Temp alarm OT upper
	.INIT_54({((MAX_TEMP+268)*2) & 10'hFFC, 6'b110100}), // Alarm limit register 4
	.INIT_55(16'h5111), // Vccint lower alarm limit
	.INIT_56(16'hcaaa), // Vccaux lower alarm limit
	.INIT_57(16'hae4e)  // Temp alarm OT reset
) my_sysmon (
	.ALM(alarm),
	.DCLK(clk_i),
	.RESET(reset_i)
);


assign pwm_o = (alarm[0]) ? 1'b1 : counter < DUTY_CYCLE;

always @ (posedge clk_i or posedge reset_i)
begin
	if (reset_i)
	begin
		counter <= 18'b0;
	end
	else 
	begin
		if (counter == LIMIT)
		begin
			counter <= 0;
		end
		else 
		begin
			counter <= (counter + 1'b1) & 18'h3FFFF;
		end
	end
end

endmodule
