/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module fifo_in #(

    parameter WIDTH     = 18,
    parameter CNT_WIDTH = 7,
    parameter BASE_ADR  = 18'h0,
    parameter SIM_DEBUG = "NO"

)(

    /* SpartanMC peripheral interface */
    input wire                  clk_peri,
    input wire                  reset,

    input wire                  access_peri,
    input wire                  wr_peri,
    input wire      [9:0]       addr_peri,
    input wire      [17:0]      do_peri,
    output wire     [17:0]      di_peri,
    
    /* FIFO interface */
    input wire      [WIDTH-1:0] fifo_data_i,
    input wire 	    [CNT_WIDTH-1:0] fifo_data_read_cnt_i,
    input wire                  fifo_data_valid_i,
    output wire                 fifo_rd_en_o

);


localparam
    REG_COUNT           = 3,
    ADDR_DATA           = 0,
    ADDR_STATUS         = 1,
    ADDR_CNT		= 2;

wire    [REG_COUNT-1:0]   reg_read;

wire                      cs;

reg_access_decoder reg_access_decoder0 (
    .clk_peri(clk_peri),
    .reset(reset),
    .addr_peri(addr_peri),
    .access_peri(access_peri),
    .wr_peri(wr_peri),
    .peri_select(cs),
    .reg_select(),
    .reg_read(reg_read),
    .reg_write()
);

defparam
    reg_access_decoder0.REG_COUNT   = REG_COUNT,
    reg_access_decoder0.BASE_ADR    = BASE_ADR,
    reg_access_decoder0.SIM_DEBUG   = SIM_DEBUG;

assign fifo_rd_en_o = reg_read[ADDR_DATA];

assign di_peri = 
    reg_read[ADDR_DATA]     ? {{18-WIDTH{1'b0}},fifo_data_i[WIDTH-1:0]} :
    reg_read[ADDR_STATUS]   ? {17'b0,fifo_data_valid_i} :
    reg_read[ADDR_CNT]   ? {{18-CNT_WIDTH{1'b0}},fifo_data_read_cnt_i} :    
    18'b0;

endmodule
