/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ps
`default_nettype none

////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:   13:03:04 12/13/2013
// Design Name:   vhdl_miner
// Module Name:   test.v
// Project Name:  ise_miner
// Target Device:  
// Tool versions:  
// Description: 
//
// Verilog Test Fixture created by ISE for module: vhdl_miner
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////

module test;

	// Inputs
	reg reset;
	reg hash_clk;
	wire  comm_clk = clk_peri;
	wire  start = strobe; 
	wire [255:0] state_i = pin[511:256];
	wire [95:0] data_i = pin[95:0];
	reg [0:0] pre_nonce;
	reg [15:0] fifo_rd_en_i;

	// Outputs
	wire [127:0] fifo_data_o;
	wire [111:0] fifo_data_read_cnt_o;

	wire [15:0] fifo_data_valid_o;


		// Inputs
	reg clk_peri;
	reg [17:0] do_peri;
	reg [9:0] addr_peri;
	reg access_peri;
	reg wr_peri;

	// Outputs
	wire [17:0] di_peri;
	wire [511:0] pin;
	wire strobe;


	reg[7:0] data_buf[63:0];
	



	// Instantiate the Unit Under Test (UUT)
	vhdl_miner #(
		.DEPTH 		 (0),
		.NUMBER_LOCAL (16),
		.NUMBER_BOARDS (1)
	)uut (
		.reset(reset), 
		.hash_clk(hash_clk), 
		.comm_clk(comm_clk), 
		.start(start), 
		.state_i(state_i), 
		.data_i(data_i), 
		.pre_nonce(pre_nonce), 
		.fifo_data_o(fifo_data_o), 
		.fifo_data_read_cnt_o(fifo_data_read_cnt_o),
		.fifo_data_valid_o(fifo_data_valid_o), 
		.fifo_rd_en_i(fifo_rd_en_i)
	);




	// Instantiate the Unit Under Test (UUT)
	port_out_expander uut2 (
		.clk_peri(clk_peri), 
		.do_peri(do_peri), 
		.di_peri(di_peri), 
		.addr_peri(addr_peri), 
		.access_peri(access_peri), 
		.wr_peri(wr_peri), 
		.reset(reset), 
		.pin(pin), 
		.strobe(strobe)
	);



   parameter PERIOD1 = 10;

   initial begin
      hash_clk = 1'b0;
      #(PERIOD1/2);
      forever
         #(PERIOD1/2) hash_clk = ~hash_clk;
   end
   parameter PERIOD2 = 20;

   initial begin
      clk_peri = 1'b0;
      #(PERIOD2/2);
      forever
         #(PERIOD2/2) clk_peri = ~clk_peri;
   end

integer i;
	initial begin
		// Initialize Inputs
		reset = 1;
		hash_clk = 0;
		pre_nonce = 0;
		fifo_rd_en_i = 0;
		clk_peri = 0;
		do_peri = 0;
		addr_peri = 0;
		access_peri = 0;
		wr_peri = 0;
		data_buf[0] =  8'h46;
		data_buf[1] =  8'h79;
      data_buf[2] =  8'hba;
      data_buf[3] =  8'h4e;
      data_buf[4] =  8'hc9;
      data_buf[5] =  8'h98;
      data_buf[6] =  8'h76;
      data_buf[7] =  8'hbf;
      data_buf[8] =  8'h4b;
      data_buf[9] =  8'hfe;
      data_buf[10] =  8'h08;
      data_buf[11] =  8'h60;
      data_buf[12] =  8'h82;
      data_buf[13] =  8'hb4;
      data_buf[14] =  8'h00;
      data_buf[15] =  8'h25;
      data_buf[16] =  8'h4d;
      data_buf[17] =  8'hf6;
      data_buf[18] =  8'hc3;
      data_buf[19] =  8'h56;
      data_buf[20] =  8'h45;
      data_buf[21] =  8'h14;
      data_buf[22] =  8'h71;
      data_buf[23] =  8'h13;
      data_buf[24] =  8'h9a;
      data_buf[25] =  8'h3a;
      data_buf[26] =  8'hfa;
      data_buf[27] =  8'h71;
      data_buf[28] =  8'he4;
      data_buf[29] =  8'h8f;
      data_buf[30] =  8'h54;
      data_buf[31] =  8'h4a;
      data_buf[32] =  8'h00;
      data_buf[33] =  8'h00;
      data_buf[34] =  8'h00;
      data_buf[35] =  8'h00;
      data_buf[36] =  8'h00;
      data_buf[37] =  8'h00;
      data_buf[38] =  8'h00;
      data_buf[39] =  8'h00;
      data_buf[40] =  8'h00;
      data_buf[41] =  8'h00;
      data_buf[42] =  8'h00;
      data_buf[43] =  8'h00;
      data_buf[44] =  8'h00;
      data_buf[45] =  8'h00;
      data_buf[46] =  8'h00;
      data_buf[47] =  8'h00;
      data_buf[48] =  8'h00;
      data_buf[49] =  8'h00;
      data_buf[50] =  8'h00;
      data_buf[51] =  8'h00;
      data_buf[52] =  8'h87;
      data_buf[53] =  8'h32;
      data_buf[54] =  8'h0b;
      data_buf[55] =  8'h1a;
      data_buf[56] =  8'h14;
      data_buf[57] =  8'h26;
      data_buf[58] =  8'h67;
      data_buf[59] =  8'h4f;
      data_buf[60] =  8'h2f;
      data_buf[61] =  8'ha7;
      data_buf[62] =  8'h22;
      data_buf[63] =  8'hce;
		
		
		
		
		i = 0;
		// Wait 100 ns for global reset to finish
		#100;
      reset = 0;
		// Add stimulus here

		#100;
		// Add stimulus here
		
		
		@(posedge clk_peri);
		addr_peri <= 1;
		do_peri <= 64;
		access_peri = 1;
		wr_peri = 1;
		@(posedge clk_peri);
		addr_peri <= 0;
		do_peri <= 0;
		access_peri = 0;
		wr_peri = 0;		
		#100
		for(i=0;i<64;i=i+1) begin
		@(posedge clk_peri);
		addr_peri <= 0;
		do_peri <= data_buf[i];
		access_peri = 1;
		wr_peri = 1;
		@(posedge clk_peri);
		addr_peri <= 0;
		do_peri <= 0;
		access_peri = 0;
		wr_peri = 0;	
		end


		while(1) begin
		@(posedge clk_peri);
		if(fifo_data_valid_o[0] ==1) fifo_rd_en_i[0] = 1;
		else fifo_rd_en_i[0] = 0;
		end

	end
      
endmodule

