/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
*
* Copyright (c) 2011 fpgaminer@bitcoin-mining.com
*
*
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* 
*/


`timescale 1ns/1ps
`default_nettype none


module mc_miner 
#(
	parameter LOOP_LOG2 = 0,
	parameter NUMBER = 2
)
(
	input wire hash_clk,
	input wire [255:0] midstate_i, 
	input wire [95:0] data2_i,
	
	/* FIFO interface */
   input wire 	comm_clk,
	output wire      [(NUMBER*8)-1:0] fifo_data_o,
   output wire      [NUMBER-1:0]     fifo_data_valid_o,
   input wire       [NUMBER-1:0]     fifo_rd_en_i
	
);


	localparam NONCE_WIDTH = 33-NUMBER;
	
	
	
	//// Mining Cores
	reg [255:0] midstate_buf = 0, data_buf = 0;

	wire [NUMBER-1:0] is_golden_ticket;
	wire [(NUMBER*32)-1:0] golden_nonce;

	genvar i;
	generate
		for (i = 0; i < NUMBER; i = i + 1) begin : MCORES

	mining_core # (.LOOP_LOG2(LOOP_LOG2), .NONCE_WIDTH(NONCE_WIDTH), .NONCE_PREFIX(i)) mining_core (
		.hash_clk(hash_clk),
		.rx_midstate(midstate_buf),
		.rx_data(data_buf[95:0]),
		.tx_is_golden_ticket(is_golden_ticket[i]),
		.tx_golden_nonce(golden_nonce[i*32+:32])
	);
	
	goldennonce_fifo nonce_fifo (
			.wr_clk		(hash_clk), // input wr_clk
			.rd_clk		(comm_clk), // input rd_clk
			.din			(golden_nonce[i*32+:32]), // input [31 : 0] din
			.wr_en		(is_golden_ticket[i]), // input wr_en
			.rd_en		(fifo_rd_en_i[i]), // input rd_en
			.dout			(fifo_data_o[i*8+:8]), // output [7 : 0] dout
			.full			(), // output full
			.empty		(), // output empty
			.valid		(fifo_data_valid_o[i]) // output valid

	
	
	);
	end
	endgenerate
	
	//// Virtual Wire Output


	//// Control Unit
	
	always @ (posedge hash_clk)
	begin
			midstate_buf <= midstate_i;
			data_buf <= data2_i;
	end

endmodule

