/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    17:55:46 11/28/2013 
// Design Name: 
// Module Name:    uart_controller 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
`default_nettype none
module uart_controller(
	input wire clk,
	input wire reset,
	
	// miner interface
	output wire [255:0] midstate, 
	output wire [255:0] data2, 

	input wire [31:0] golden_nonce,
   input wire 		   golden_nonce_send,
   output  wire 	   send_busy,

	/* Generic SHI interface */
  input wire [7:0] tx_data_i,
  input wire       tx_valid_i,
  
  output wire [7:0] rx_data_o,
  output wire       rx_en_o,
  input wire       rx_rdy_i


    );

	// Rec Dir
	
	reg [511:0] input_buffer;
   reg [511:0] input_copy;

   assign midstate = input_copy[511:256];
   assign data2 = input_copy[255:0];

   reg [6:0]   demux_state = 7'b0000000;

	always @(posedge clk)if(reset)begin
		input_buffer <= 0;
		input_copy	<= 0;
		demux_state	<= 0;
	end else case (demux_state)
		7'b1000000: begin
			input_copy <= input_buffer;
			demux_state <= 0;
		end
      default:	if(tx_valid_i) begin
	      input_buffer <= input_buffer << 8;
	      input_buffer[7:0] <= tx_data_i;
	      demux_state <= demux_state + 1;
	   end
    endcase // case (demux_state)
	 
	 wire fifo_empty;
	 // Send Dir
	fifo_32to8  fifo_32to8(
		.rst		(reset), // input rst
		.wr_clk	(clk), // input wr_clk
		.rd_clk	(clk), // input rd_clk
		.din		(golden_nonce), // input [31 : 0] din
		.wr_en	(golden_nonce_send), // input wr_en
		.rd_en	(rx_en_o), // input rd_en
		.dout		(rx_data_o), // output [7 : 0] dout
		.full		(send_busy), // output full
		.empty	(fifo_empty) // output empty
	);
	
	assign rx_en_o = (fifo_empty & rx_rdy_i & ~reset); 		
	 
endmodule
