/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    15:36:38 12/12/2013 
// Design Name: 
// Module Name:    vhdl_miner 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module vhdl_miner
#(
	parameter DEPTH 		 = 0,
	parameter NUMBER_LOCAL = 1,
	parameter NUMBER_BOARDS = 1,
	parameter TMP1 = log2(NUMBER_LOCAL)-1,
	parameter TMP2 = log2(NUMBER_BOARDS)-1,
	parameter NONCE_WIDTH = 32 -  TMP1 -TMP2
)
(
	input wire 			 reset,
	input wire 			 hash_clk,
   input wire 			 comm_clk,

	
	input wire 			 start,
	input wire [255:0] state_i,
	input wire [255:0]  data_i,
	
	input wire [TMP2:0] 	 pre_nonce,
	
		/* FIFO interface */
	output wire      [(NUMBER_LOCAL*8)-1:0] fifo_data_o,
	output wire 	  [(NUMBER_LOCAL*6)-1:0] fifo_data_read_cnt_o,
        output wire      [NUMBER_LOCAL-1:0]     fifo_data_valid_o,
        input wire       [NUMBER_LOCAL-1:0]     fifo_rd_en_i


    );

	function integer log2;
	input integer value;
	
	for (log2=0; value>0; log2=log2+1)
		value = value>>1;	
	endfunction

//reg state_fifo_rd_en = 0;
wire state_fifo_empty;
wire [255:0] state_fifo_data;

fifo_256 state_fifo (
  .rst          (reset), // input rst
  .wr_clk       (comm_clk), // input wr_clk
  .rd_clk       (hash_clk), // input rd_clk
  .din          (state_i), // input [255 : 0] din
  .wr_en        (start), // input wr_en
  .rd_en        (~state_fifo_empty), // input rd_en
  .dout         (state_fifo_data), // output [255 : 0] dout
  .full         (), // output full
  .empty        (state_fifo_empty) // output empty
);

//reg data_fifo_rd_en = 0;
wire data_fifo_empty;
wire [255:0] data_fifo_data;
fifo_256 data_fifo (
  .rst          (reset), // input rst
  .wr_clk       (comm_clk), // input wr_clk
  .rd_clk       (hash_clk), // input rd_clk
  .din          (data_i), // input [255 : 0] din
  .wr_en        (start), // input wr_en
  .rd_en        (~data_fifo_empty), // input rd_en
  .dout         (data_fifo_data), // output [255 : 0] dout
  .full         (), // output full
  .empty        (data_fifo_empty) // output empty
);


reg  [NONCE_WIDTH-1:0] nonce;
wire [NONCE_WIDTH-1:0] currnonce = nonce - 2 * 2 ** DEPTH;
reg [95:0]  data = 0;
reg [255:0] state = 0;
wire [NUMBER_LOCAL-1:0] hit ;
wire [NUMBER_LOCAL-1:0] fifo_empty ;
reg [5:0] step = 0;

reg run = 0;


wire hash_clk_buf=hash_clk;
//BUFGCE BUFGCE_inst (
//      .O(hash_clk_buf),   // 1-bit output: Clock buffer output
//      .CE(run), // 1-bit input: Clock enable input for I0 input
//      .I(hash_clk)    // 1-bit input: Primary clock input
//   );

genvar i;
wire [((TMP1)*NUMBER_LOCAL)-1:0] tmp ;
wire [(32*NUMBER_LOCAL)-1:0] akt_nonce   ;
wire [(32*NUMBER_LOCAL)-1:0] golden_nonce;


/*
wire [35:0] control;
icon icon (
    .CONTROL0(control) // INOUT BUS [35:0]
);
chipscope_ila ila (
    .CONTROL(control), // INOUT BUS [35:0]
    .CLK(hash_clk), // IN
    .TRIG0(start), // IN BUS [0:0]
    .TRIG1(run), // IN BUS [0:0]
    .TRIG2(akt_nonce[0]), // IN_BUS [31:0]
    .TRIG3(hit[0]), // IN BUS [0:0]
    .TRIG4(fifo_data_read_cnt_o[6:0]), // IN BUS [6:0]
    .TRIG5(fifo_data_valid_o[0]), // IN BUS [0:0]
    .TRIG6(fifo_rd_en_i[0]) // IN BUS [0:0]
);
*/





generate
	for (i = 0; i < NUMBER_LOCAL; i = i + 1) begin : MCORES
	
	if(NUMBER_LOCAL == 1)begin
	        assign tmp[i] = i;
		assign akt_nonce   [i*32+:32] = {pre_nonce,nonce};
		assign golden_nonce[i*32+:32] = {pre_nonce,currnonce};
	end else begin
	        assign tmp[i*(TMP1)+:(TMP1)] = i;
		assign akt_nonce   [i*32+:32] = {pre_nonce,tmp[i],nonce};
		assign golden_nonce[i*32+:32] = {pre_nonce,tmp[i],currnonce};
	end
	
	miner
	#(
		.DEPTH(DEPTH)
	)
	miner_core (
		.clk(hash_clk_buf), 			// clk
		.step(step), 			// step
		.data(data), 			// data 96bit
		.state(state), 			// 256 bit
		.nonce(akt_nonce[i*32+:32]), 			// nonce
		.hit(hit[i])				// hit signal
	);
	
	goldennonce_fifo nonce_fifo (
			.rst			(reset),
			.wr_clk		(hash_clk), // input wr_clk
			.rd_clk		(comm_clk), // input rd_clk
			.din			(golden_nonce[i]), // input [31 : 0] din
			.wr_en		(hit[i]), // input wr_en
			.rd_en		(fifo_rd_en_i[i]), // input rd_en
			.dout			(fifo_data_o[i*8+:8]), // output [7 : 0] dout
			.full			(), // output full
			.empty		(fifo_empty[i]), // output empty
			//.valid		(fifo_data_valid_o[i]), // output valid
			.rd_data_count(fifo_data_read_cnt_o[i*6+:6]) // output [6 : 0] rd_data_count

	);
	assign fifo_data_valid_o[i] = fifo_empty[i];
	end
endgenerate


localparam UPPER_BOUNCE = 2**(6 - DEPTH)-1;

    

always @(posedge hash_clk) begin
	if(run) begin
		if(~state_fifo_empty) begin
			//nonce <= 28'h00187a2-256;
			nonce <= 0;
         data  <= data_fifo_data[95:0];
			state <= state_fifo_data;
			step  <= 0;
		end else	if(step == UPPER_BOUNCE) begin
			step <= 0;
			nonce <= nonce + 1;
			if(nonce == {NONCE_WIDTH{1'b1}}) run <= 0;
		end else begin
			step <= step + 1;
		end
	end else begin
		if(~state_fifo_empty) begin
			//nonce <= 28'h00187a2-256;
			nonce <= 0;
			data  <= data_fifo_data[95:0];
			state <= state_fifo_data;
			step  <= 0;
			run 	<= 1;
		end else begin
			nonce <= 0;
			data  <= 0;
			state <= 0;
			step  <= 0;
			run <= 0;
		end
	end
end
endmodule
