/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`default_nettype none
`timescale	1ns / 10ps

module hardware_barrier #(
	parameter BASE_ADR		= 10'h40
)(

	input	wire							clk_peri,
	input 	wire 							reset,

	//peri
	input	wire	[9:0]					addr_peri,
	input	wire	[17:0]					do_peri,
	output	wire	[17:0]					di_peri,
	input	wire	access_peri,		//Signale fuer Peripheriezugriffe
    input	wire	wr_peri,		//Schreibsignal

    output wire 						wait_for,
    input wire							wait_done
	);

//peri interace
parameter GLOBAL_MEM	= 0;
parameter ALL_PORTS		= 1;
parameter DECODSIM_DEB		= "NO";

wire select;
wire read_peri;
wire write_peri;

reg done;


always@(posedge clk_peri or posedge reset) begin
	if(reset)
		done <= 1'b0;
	else if(write_peri)
		done <= |do_peri;
end

assign wait_for = done;
assign di_peri = read_peri ? {18 {wait_done}} : 18'b0;


wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
	.REG_COUNT(	ALL_PORTS	),
	.BASE_ADR(	BASE_ADR	),
	.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
	.reset(		reset		),
	.clk_peri(	clk_peri	),
	.addr_peri(	addr_peri	),
	.access_peri(	access_peri	),
	.wr_peri(	wr_peri		),
	.peri_select(	/* NC */),
	.reg_select(	/* NC */),
	.reg_read(	reg_read	),
	.reg_write(	reg_write	)
);

assign	read_peri	= reg_read[GLOBAL_MEM];	// Lesen
assign	write_peri	= reg_write[GLOBAL_MEM];	// Schreiben

endmodule