/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`ifndef __led_encoder
`define __led_encoder
module led_encoder #(
    parameter CLOCK_FREQ_HZ = 25e6,
    parameter BIT_COUNT = 4
)(
    input wire                 clk_i,
    input wire                 reset_i,
    input wire [BIT_COUNT-1:0] value_i,
    output wire                led_o);

`include "common/functions/bitwidth.vh"

localparam
    P_WIDTH             = bitwidth(CLOCK_FREQ_HZ - 1),
    C_WIDTH             = P_WIDTH + 2,
    SC_WIDTH            = bitwidth(BIT_COUNT - 1);


localparam [P_WIDTH-1:0]
    CYCLES_OFF          = CLOCK_FREQ_HZ / 4,
    CYCLES_ON_LONG      = CLOCK_FREQ_HZ / 2,
    CYCLES_ON_SHORT     = CLOCK_FREQ_HZ / 8,
    CYCLES_PAUSE        = CLOCK_FREQ_HZ;

reg     [C_WIDTH-1:0]  counter;
wire    [C_WIDTH-1:0]  counter_max;
wire    [C_WIDTH-1:0]  cycles_on;

reg     [BIT_COUNT-1:0]   shift_val;
reg     [SC_WIDTH-1:0]   shift_cnt;

assign led_o        = counter < cycles_on;
assign cycles_on    = shift_val[BIT_COUNT-1] ? CYCLES_ON_LONG : CYCLES_ON_SHORT;
assign counter_max  = (shift_cnt == (BIT_COUNT - 1)) ? cycles_on + CYCLES_PAUSE + CYCLES_OFF : cycles_on + CYCLES_OFF;

always @ (posedge clk_i)
begin
    if ((counter >= counter_max) || reset_i) 
    begin
        counter <= {C_WIDTH{1'b0}};
    end else begin
        counter <= (counter + 1) % {1'b1, {C_WIDTH {1'b0}}};
    end
end

always @ (posedge clk_i)
begin
    if ((shift_cnt == BIT_COUNT) || reset_i)
    begin
        shift_cnt <= {SC_WIDTH {1'b0}};
        shift_val <= value_i;
    end else begin
        if (counter == counter_max) 
        begin
            shift_cnt <= (shift_cnt + 1) % {1'b1, {SC_WIDTH {1'b0}}};
            shift_val <= {shift_val[BIT_COUNT-2:0], 1'b0};
        end
    end
end

endmodule
`endif
