/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________

This file is based on the work of Richard Herveille.

*/



/////////////////////////////////////////////////////////////////////
////                                                             ////
////  WISHBONE revB.2 compliant I2C Master controller Top-level  ////
////                                                             ////
////                                                             ////
////  Author: Richard Herveille                                  ////
////          richard@asics.ws                                   ////
////          www.asics.ws                                       ////
////                                                             ////
////  Downloaded from: http://www.opencores.org/projects/i2c/    ////
////                                                             ////
/////////////////////////////////////////////////////////////////////
////                                                             ////
//// Copyright (C) 2001 Richard Herveille                        ////
////                    richard@asics.ws                         ////
////                                                             ////
//// This source file may be used and distributed without        ////
//// restriction provided that this copyright statement is not   ////
//// removed from the file and that any derivative work contains ////
//// the original copyright notice and the associated disclaimer.////
////                                                             ////
////     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ////
//// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ////
//// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ////
//// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ////
//// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ////
//// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ////
//// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ////
//// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ////
//// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ////
//// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ////
//// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ////
//// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ////
//// POSSIBILITY OF SUCH DAMAGE.                                 ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

//  CVS Log
//
//  $Id: i2c_master_top.v,v 1.12 2009-01-19 20:29:26 rherveille Exp $
//
//  $Date: 2009-01-19 20:29:26 $
//  $Revision: 1.12 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               Revision 1.11  2005/02/27 09:26:24  rherveille
//               Fixed register overwrite issue.
//               Removed full_case pragma, replaced it by a default statement.
//
//               Revision 1.10  2003/09/01 10:34:38  rherveille
//               Fix a blocking vs. non-blocking error in the wb_dat output mux.
//
//               Revision 1.9  2003/01/09 16:44:45  rherveille
//               Fixed a bug in the Command Register declaration.
//
//               Revision 1.8  2002/12/26 16:05:12  rherveille
//               Small code simplifications
//
//               Revision 1.7  2002/12/26 15:02:32  rherveille
//               Core is now a Multimaster I2C controller
//
//               Revision 1.6  2002/11/30 22:24:40  rherveille
//               Cleaned up code
//
//               Revision 1.5  2001/11/10 10:52:55  rherveille
//               Changed PRER reset value from 0x0000 to 0xffff, conform specs.
//

// synopsys translate_off
`include "i2c_master/src/timescale.vh"
// synopsys translate_on

`include "i2c_master/src/i2c_master_defines.vh"

`default_nettype none


module i2c_master_top
#(
	parameter BASE_ADR	= 10'h0,
//	parameter PRESCALER	= 0,
	parameter CONTROL	= 0,
	parameter TX		= 1,
	parameter RX		= 2,
	parameter COMMAND	= 3,
	parameter STATUS	= 4,

	parameter ALL_PORTS	= 5,		//Anzahl der Ports fuer den Dekoder
	parameter DECODSIM_DEB	= "NO"		//DEBUG mode YES oder NO
)	
(

	// wishbone signals
//*** Verbindungen mit dem MC ***
	input	wire			clk_peri,	//Systemtakt
	input	wire		[17:0]	do_peri,	//Datenbus von MC
	output	wire		[17:0]	di_peri,	//Datenbus zu MC
	input	wire		[9:0]	addr_peri,	//Adressbus von MC
	input	wire			access_peri,	//Signale fuer Peripheriezugriffe
	input	wire			wr_peri,	//Schreibsignal
	input	wire			reset,		//Reset-Signal
	output	reg			intr,		//Interrupt
	
	// I2C signals
	// i2c clock line
        inout	wire			scl,
	// i2c data line
        inout	wire			sda
        );

	wire				scl_pad_i;	// SCL-line input
	wire				scl_pad_o;	// SCL-line output (always 1'b0)
	wire				scl_padoen_o;	// SCL-line output enable (active low)

	wire				sda_pad_i;	// SDA-line input
	wire				sda_pad_o;	// SDA-line output (always 1'b0)
	wire				sda_padoen_o;	// SDA-line output enable (active low)
	reg				tip;		// transfer in progress

	assign	scl		= scl_padoen_o ? 1'bz : scl_pad_o;
	assign	sda		= sda_padoen_o ? 1'bz : sda_pad_o;
	assign	scl_pad_i	= scl;
	assign	sda_pad_i	= sda;

/*
    wire select;
    pselect select_logic	(
        .addr(		addr_peri[9:3]	),
        .activ_peri(	access_peri	),
        .select(	select		)
    );
    defparam select_logic.ADDR_WIDTH	= 7;
    defparam select_logic.BASE_WIDTH	= 7;
    defparam select_logic.BASE_ADDR	= BASE_ADR >> 3;

    //build new wire for better reading
    wire [2:0] reg_select =  addr_peri[2:0];
*/

// Dekodierung der Adressen ********************** START *************************************
// chipselect fuer die Status-, Daten- und Kommandoregister
wire			rd_ctrl;	// Lesen
wire			rd_rx;		// Lesen
wire			rd_tx;		// Lesen
wire			rd_cmd;		// Lesen
wire			rd_status;	// Lesen
wire			wr_ctrl;	// Schreiben
wire			wr_tx;		// Schreiben
wire			wr_cmd;		// Schreiben
wire			cs_rx;		// r/w
wire			cs_tx;		// r/w

wire			select;		// Modul wird angesprochen
wire	[ALL_PORTS-1:0]	reg_select;	// r/w ADR
wire	[ALL_PORTS-1:0]	reg_read;	// read ADR
wire	[ALL_PORTS-1:0]	reg_write;	// write ADR

// Addressdekoder fuer alle Geraeteregister (verwendet pselect.v und
// verzoegert die read Signale um einen Takt fuer die neue Pipeline)
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);

// chipselect fuer die FIFO-,Status- und Steuerregister
assign	rd_ctrl		= reg_read[CONTROL];		// Lesen
assign	rd_rx		= reg_read[RX];			// Lesen
assign	rd_tx		= reg_read[TX];			// Lesen
assign	rd_cmd		= reg_read[COMMAND];		// Lesen
assign	rd_status	= reg_read[STATUS];		// Lesen
assign	wr_ctrl		= reg_write[CONTROL];		// Schreiben
assign	wr_tx		= reg_write[TX];		// Schreiben
assign	wr_cmd		= reg_write[COMMAND];		// Schreiben
assign	cs_rx		= reg_select[RX];		// r/w
assign	cs_tx		= reg_select[TX];		// r/w

// Dekodierung der Adressen ********************** END **************************************


        //
	// variable declarations
	//

	// registers
	reg			[15:0]	prer;		// clock prescale register
	reg			[ 1:0]	ctr;		// control register
	reg			[ 7:0]	txr;		// transmit register
	wire			[ 7:0]	rxr;		// receive register
	reg			[ 7:0]	cr;		// command register
	wire			[ 7:0]	sr;		// status register

	// done signal: command completed, clear command register
	wire				done;

	// core enable signal
	wire				core_en;
	wire				ien;

	// status register signals
	wire				irxack;
	reg				rxack;		// received aknowledge from slave
	reg				irq_flag;	// interrupt pending flag
	wire				i2c_busy;	// bus busy (start signal detected)
	wire				i2c_al;		// i2c bus arbitration lost
	reg				al;		// status register arbitration lost bit

	//
	// module body
	//

	// generate internal reset
	wire				rst_i = reset;

assign	di_peri [17:0]	= (rd_ctrl)	? {ctr, prer}	:
			  (rd_tx)	? {10'b0, txr}	:
			  (rd_rx)	? {10'b0, rxr}	:
			  (rd_cmd)	? {10'b0, cr}	:
			  (rd_status)	? {10'b0, sr}	:
					   18'b0;


	// generate registers
	always @(posedge clk_peri)
	  if (rst_i)
	    begin
	        prer <= #1 16'hffff;
	        ctr  <= #1  2'h0;
	        txr  <= #1  8'h0;
	    end
	  else if (wr_ctrl)
		    begin
			ctr   <= do_peri[17:16] ;
		        prer  <= do_peri[15:0];
		    end
	  else if (wr_tx)
			txr   <= do_peri[7:0];

	// generate command register (special case)
	always @(posedge clk_peri)
	  if (rst_i)
	  begin
	    cr  <= 8'h0;
	    tip <= 1'b0;
	  end
	  else if (core_en & wr_cmd)
		begin
		  cr  <= do_peri[7:0];
		  tip <= do_peri[4] | do_peri[5];  
		end
	  else
	    begin
	        if (done | i2c_al)
		begin		  
		  tip	  <= 1'b0;
	          cr[7:4] <= 4'h0;           // clear command bits when done
	                                        // or when aribitration lost
	        end
	        cr[2:1] <= 2'b0;             // reserved bits
	        cr[0]   <= 1'b0;             // clear IRQ_ACK bit
	    end


	// decode command register
	wire sta  = cr[7];
	wire sto  = cr[6];
	wire rd   = cr[5];
	wire wr   = cr[4];
	wire ack  = cr[3];
	wire iack = cr[0];

	// decode control register
	assign core_en	= ctr[0];
	assign ien	= ctr[1];

	// hookup byte controller block
	i2c_master_byte_ctrl byte_controller (
		.clk      ( clk_peri     ),
		.rst      ( reset        ),
		.ena      ( core_en      ),
		.clk_cnt  ( prer         ),
		.start    ( sta          ),
		.stop     ( sto          ),
		.read     ( rd           ),
		.write    ( wr           ),
		.ack_in   ( ack          ),
		.din      ( txr          ),
		.cmd_ack  ( done         ),
		.ack_out  ( irxack       ),
		.dout     ( rxr          ),
		.i2c_busy ( i2c_busy     ),
		.i2c_al   ( i2c_al       ),
		.scl_i    ( scl_pad_i    ),
		.scl_o    ( scl_pad_o    ),
		.scl_oen  ( scl_padoen_o ),
		.sda_i    ( sda_pad_i    ),
		.sda_o    ( sda_pad_o    ),
		.sda_oen  ( sda_padoen_o )
	);

	// status register block + interrupt request signal
	always @(posedge clk_peri)
	  if (rst_i)
	    begin
	        al       <=  1'b0;
	        rxack    <=  1'b0;
	        irq_flag <=  1'b0;
	    end
	  else
	    begin
	        al       <=  i2c_al | (al & ~sta);
	        rxack    <=  irxack;
	        irq_flag <=  (done | i2c_al | irq_flag) & ~iack; // interrupt request flag is always generated
	    end

	// generate interrupt request signals
	always @(posedge clk_peri)
	  if (rst_i)
	    intr <=  1'b0;
	  else
	    intr <=  irq_flag && ien; // interrupt signal is only generated when IEN (interrupt enable bit is set)

	// assign status register bits
	assign sr[7]   = rxack;
	assign sr[6]   = i2c_busy;
	assign sr[5]   = al;
	assign sr[4:2] = 3'h0; // reserved
	assign sr[1]   = tip;
	assign sr[0]   = irq_flag;

endmodule
