/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________

	based on code from Steve Fielding

*/

`timescale      1ns / 1ps
`default_nettype none

//////////////////////////////////////////////////////////////////////
////                                                              ////
//// i2cSlave.v                                                   ////
////                                                              ////
//// This file is part of the i2cSlave opencores effort.          ////
//// <http://www.opencores.org/cores//>                           ////
////                                                              ////
//// Module Description:                                          ////
//// You will need to modify this file to implement your          ////
//// interface.                                                   ////
////                                                              ////
//// To Do:                                                       ////
////                                                              ////
////                                                              ////
//// Author(s):                                                   ////
//// - Steve Fielding, sfielding@base2designs.com                 ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2008 Steve Fielding and OPENCORES.ORG          ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE. See the GNU Lesser General Public License for more  ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from <http://www.opencores.org/lgpl.shtml>                   ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
`include "i2c_slave/src/i2cSlave_define.vh"
 
 
module i2cSlave #(
	parameter I2C_ADR	= 7'h12,
	parameter DEB_I2C_LEN	= 10,
	parameter SCL_DEL_LEN	= 10,
	parameter SDA_DEL_LEN	= 10,
	parameter ADDR_WIDTH	= 4,
	parameter REG_WIDTH	= 8
)
(
	input	wire				clk,
	input	wire				rst,
	inout	wire				sda,
	inout	wire				scl,

	output	wire				writeEn,
	input	wire	[REG_WIDTH -1 : 0]	dataFromRegIF,
	output	wire	[REG_WIDTH -1 : 0]	dataToRegIF,
 	output	wire	[ADDR_WIDTH -1: 0]	regAddr,
	output	wire				ack_bit,
	output	reg				done,
	output	wire				rd,
	output	wire				set_irqr,
	input	wire				en_sl
);
 
// local wires and regs
reg				sdaDeb;
reg				sclDeb;
reg	[DEB_I2C_LEN-1:0]	sdaPipe;
reg	[DEB_I2C_LEN-1:0]	sclPipe;
 
reg	[SCL_DEL_LEN-1:0]	sclDelayed;
reg	[SDA_DEL_LEN-1:0]	sdaDelayed;
reg	[1:0]			startStopDetState;
wire				clearStartStopDet;
wire				sdaOut;
wire				sdaIn;
wire				sclIn;
reg	[1:0]			rstPipe;
wire				rstSyncToClk;
reg				startEdgeDet;
 
assign	sda	= ((sdaOut | !en_sl) == 1'b0) ? 1'b0 : 1'bz;
assign	sdaIn	= sda;

assign	scl	= 1'bz;
assign	sclIn	= scl;
 
// sync rst rsing edge to clk
always @(posedge clk) begin
  if (rst == 1'b1)
    rstPipe <= 2'b11;
  else
    rstPipe <= {rstPipe[0], 1'b0};
end
 
assign rstSyncToClk = rstPipe[1];
 
// debounce sda and scl
/*
always @(posedge clk) begin
  if (rstSyncToClk == 1'b1) begin
    sdaPipe <= {DEB_I2C_LEN{1'b1}};
    sdaDeb <= 1'b1;
    sclPipe <= {DEB_I2C_LEN{1'b1}};
    sclDeb <= 1'b1;
  end
  else begin
    sdaPipe <= {sdaPipe[DEB_I2C_LEN-2:0], sdaIn};
    sclPipe <= {sclPipe[DEB_I2C_LEN-2:0], sclIn};
    if (&sclPipe[DEB_I2C_LEN-1:1] == 1'b1)
      sclDeb <= 1'b1;
    else if (|sclPipe[DEB_I2C_LEN-1:1] == 1'b0)
      sclDeb <= 1'b0;
    if (&sdaPipe[DEB_I2C_LEN-1:1] == 1'b1)
      sdaDeb <= 1'b1;
    else if (|sdaPipe[DEB_I2C_LEN-1:1] == 1'b0)
      sdaDeb <= 1'b0;
  end
end
 */
// Inputs Synchronisieren
always @(posedge clk) begin
  if (rstSyncToClk == 1'b1) begin
    sdaDeb <= 1'b1;
    sclDeb <= 1'b1;
  end
  else begin
      sdaDeb <= sdaIn;
      sclDeb <= sclIn;
  end
end
 
// delay scl and sda
// sclDelayed is used as a delayed sampling clock
// sdaDelayed is only used for start stop detection
// Because sda hold time from scl falling is 0nS
// sda must be delayed with respect to scl to avoid incorrect
// detection of start/stop at scl falling edge. 
always @(posedge clk) begin
  if (rstSyncToClk == 1'b1) begin
    sclDelayed <= {SCL_DEL_LEN{1'b1}};
    sdaDelayed <= {SDA_DEL_LEN{1'b1}};
  end
  else begin
    sclDelayed <= {sclDelayed[SCL_DEL_LEN-2:0], sclDeb};
    sdaDelayed <= {sdaDelayed[SDA_DEL_LEN-2:0], sdaDeb};
  end
end
 
// start stop detection
always @(posedge clk) begin
  if (rstSyncToClk == 1'b1) begin
    startStopDetState	<= `NULL_DET;
    startEdgeDet	<= 1'b0;
    done		<= 1'b1;
  end
  else begin
    if (sclDeb == 1'b1 && sdaDelayed[SDA_DEL_LEN-2] == 1'b0 && sdaDelayed[SDA_DEL_LEN-1] == 1'b1)
      begin
      startEdgeDet	<= 1'b1;
      done		<= 1'b0;
      end
    else
      startEdgeDet	<= 1'b0;
    if (clearStartStopDet == 1'b1)
      startStopDetState		<= `NULL_DET;
    else if (sclDeb == 1'b1) begin
      if (sdaDelayed[SDA_DEL_LEN-2] == 1'b1 && sdaDelayed[SDA_DEL_LEN-1] == 1'b0)
	begin
        startStopDetState	<= `STOP_DET;
	done			<= 1'b1;
	end
      else if (sdaDelayed[SDA_DEL_LEN-2] == 1'b0 && sdaDelayed[SDA_DEL_LEN-1] == 1'b1)
        startStopDetState	<= `START_DET;
    end
  end
end
 
serialInterface #(
	.I2C_ADR(		I2C_ADR		),
	.ADDR_WIDTH(		ADDR_WIDTH	)
) u_serialInterface (
	.clk(			clk				), 
	.rst(			rstSyncToClk | startEdgeDet	), 
	.dataIn(		dataFromRegIF			), 
	.dataOut(		dataToRegIF			), 
	.writeEn(		writeEn				),
	.regAddr(		regAddr[ADDR_WIDTH -1:0]	), 
	.scl(			sclDelayed[SCL_DEL_LEN-1]	), 
	.sdaIn(			sdaDeb				), 
	.sdaOut(		sdaOut				), 
	.startStopDetState(	startStopDetState		),
	.clearStartStopDet(	clearStartStopDet		),
	.ack_bit(		ack_bit				),
	.set_irqr(		set_irqr			),
	.rd(			rd				)
);
 
 
endmodule
