/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________

	based on code from Steve Fielding

*/

//////////////////////////////////////////////////////////////////////
////                                                              ////
//// i2cSlaveTop.v                                                ////
////                                                              ////
//// This file is part of the i2cSlave opencores effort.          ////
//// <http://www.opencores.org/cores//>                           ////
////                                                              ////
//// Module Description:                                          ////
//// You will need to modify this file to implement your          ////
//// interface.                                                   ////
////                                                              ////
//// To Do:                                                       ////
////                                                              ////
////                                                              ////
//// Author(s):                                                   ////
//// - Steve Fielding, sfielding@base2designs.com                 ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2008 Steve Fielding and OPENCORES.ORG          ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE. See the GNU Lesser General Public License for more  ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from <http://www.opencores.org/lgpl.shtml>                   ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
`include "i2c_slave/src/i2cSlave_define.vh" 
//`include "common/src/pselect.v"
`default_nettype none

module i2cSlaveTop #(
	parameter BASE_ADR	= 10'h0,
//	parameter CPU_CLK	= 7'd33,
	parameter I2C_ADR	= 7'h12,	//Slave Addresse

	parameter DECODSIM_DEB	= "NO",		//DEBUG mode YES oder NO

	parameter DATA_REG	= 2,		//Daten Register
	parameter REG_WIDTH	= 8		//Daten Register Breite (nur n*8 moeglich)
)
(
	inout	wire		sda,
	inout	wire		scl,
	input	wire		clk_peri,	//Systemtakt
	input	wire	[17:0]	do_peri,	//Datenbus von MC
	output	wire	[17:0]	di_peri,	//Datenbus zu MC
	input	wire	[9:0]	addr_peri,	//Adressbus von MC
	input	wire		access_peri,	//Signale fuer Peripheriezugriffe
	input	wire		wr_peri,	//Schreibsignal
	input	wire		reset,		//Reset-Signal
	output	wire		intr		//Interrupt
);

        // ld-Funktion (dualer Logarithmus, aufgerundet)
function integer ld(input integer value);
	integer i;
	begin
		for (i = 0; value > 0; i = i + 1)
				  value = value >> 1;
		ld = i;
	end
endfunction

parameter REGISTERS	= DATA_REG + 1;	// Datenregister + CTRL-Register
parameter REG_MAX	= 16;

// Anzahl der PORTS fuer die Anzeige der Bits berechnen
parameter PORTS		= DATA_REG * REG_WIDTH;

parameter ALL_PORTS	= REGISTERS;		// =Adressbusbreite
parameter ADDR_WIDTH	= ld(DATA_REG-1);	// =Adressbusbreite nur Datenregister

parameter CONTROL	= 0;
parameter TX		= 1;
parameter RX		= 2;


// Dekodierung der Adressen ********************** START *************************************
// chipselect fuer die Status-, Daten- und Kommandoregister
wire			rd_ctrl;	// Lesen
wire			rd_rx;		// Lesen
wire			rd_tx;		// Lesen
wire			wr_ctrl;	// Schreiben
wire			wr_tx;		// Schreiben
wire			cs_rx;		// r/w
wire			cs_tx;		// r/w

wire			select;		// Modul wird angesprochen
wire	[ALL_PORTS-1:0]	reg_select;	// r/w ADR
wire	[ALL_PORTS-1:0]	reg_read;	// read ADR
wire	[ALL_PORTS-1:0]	reg_write;	// write ADR

// Addressdekoder fuer alle Geraeteregister (verwendet pselect.v und
// verzoegert die read Signale um einen Takt fuer die neue Pipeline)
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);

// chipselect fuer die FIFO-,Status- und Steuerregister
assign	rd_ctrl		= reg_read[CONTROL];		// Lesen
assign	rd_rx		= reg_read[RX];			// Lesen
assign	rd_tx		= reg_read[TX];			// Lesen
assign	wr_ctrl		= reg_write[CONTROL];		// Schreiben
assign	wr_tx		= reg_write[TX];		// Schreiben
assign	cs_rx		= reg_select[RX];		// r/w
assign	cs_tx		= reg_select[TX];		// r/w

// Dekodierung der Adressen ********************** END **************************************

wire				writeEnB;
wire	[REG_WIDTH -1 : 0]	diB;
wire	[REG_WIDTH -1 : 0]	doB;
wire	[ADDR_WIDTH -1: 0]	regAddrB;
wire	[REG_WIDTH -1 : 0]	rd_regs;

reg				en;
reg				ien;
reg				a10bit_en;
reg				ir_sigr;
reg				ir_sigw;
wire				ir_sig;
reg	[9:0]			sl_adr;
reg	[7:0]			rx_buf;
reg	[7:0]			tx_buf;
//reg				wr_txreg;

wire				ack_bit;
wire				done_bit;
wire				rd_bit;
wire				set_irqr;

// generate interrupt request signals
always @(posedge clk_peri)
	if (reset)
		ir_sigr	<=  1'b0;
	else if (cs_tx)		// wenn Master liest, muessen die Daten in das Tx Register geschrieben werden
		ir_sigr	<=  1'b0;
	else if (set_irqr)	// lesen vom Master
		ir_sigr	<=  1'b1;

// generate interrupt request signals
always @(posedge clk_peri)
	if (reset)
		ir_sigw	<=  1'b0;
	else if (cs_rx)		// wenn Master sendet sind die Daten im Rx-Register
		ir_sigw	<=  1'b0;
	else if (writeEnB)	// schreiben vom Master
		ir_sigw	<=  1'b1;

assign	ir_sig	= (ir_sigr || ir_sigw);
assign	intr	= ir_sig & ien;

assign	di_peri [17:0]	= (rd_ctrl)	? {ack_bit, done_bit, sl_adr[9:0], rd_bit, ir_sigw, ir_sigr, a10bit_en, ien, en}	:
			  (rd_tx)	? {10'b0, tx_buf[7 : 0]} :
			  (rd_rx)	? {10'b0, rx_buf[7 : 0]} : 18'b0;

// schreiben CTRL-Register
always @(posedge clk_peri)
	if (reset)
	begin
		en		<= 1'b0;
		ien		<= 1'b0;
		a10bit_en	<= 1'b0;
		sl_adr		<= I2C_ADR;
	end
	else if (wr_ctrl)
	begin
		en		<= do_peri[0];
		ien		<= do_peri[1];
		a10bit_en	<= do_peri[2];
		sl_adr		<= do_peri[15:5];
	end

// Tx-Register
always @(posedge clk_peri)
	if (reset)
	begin
		tx_buf		<= 8'b0;
	end
	else if (wr_tx)
	begin
		tx_buf		<= do_peri[7:0];
	end


// Rx-Register
always @(posedge clk_peri)
	if (reset)
	begin
		rx_buf		<= 8'b0;
	end
	else if (writeEnB)
	begin
		rx_buf		<= diB;
	end
/*
registerInterface #(
	.RAM_WIDTH(	REG_WIDTH	),
	.RAM_ADDR_BITS(	ADDR_WIDTH	)
) regFile (
	.clkA(		clk_peri			),
	.clkB(		clk_peri			),
	.diA(		tx_buf				),
	.diB(		diB				),
	.doA(		rd_regs[REG_WIDTH -1 : 0]	),
	.doB(		doB				),
	.enableA(	1'b1				),
	.enableB(	1'b1				),
	.write_enableA(	wr_txreg			),
	.write_enableB(	writeEnB			),
	.addrA(		!(cs_rx	| rd_rx)		),	// iic read aus Tx Register und iic write in RX Register
	.addrB(		rd_bit				)
);
*/
i2cSlave #(
	.I2C_ADR(	I2C_ADR			),
	.DEB_I2C_LEN(	2			),	// Verzoegerungstackte fuer die Flankenerkennung
	.SCL_DEL_LEN(	2			),	// Verzoegerungstackte fuer die Flankenerkennung
	.SDA_DEL_LEN(	2			),	// Flankenerkennung und START / STOP Erkennung
	.REG_WIDTH(	REG_WIDTH		),
	.ADDR_WIDTH(ADDR_WIDTH)
) u_i2cSlave(  
	.clk(		clk_peri		),
	.rst(		reset			),
	.sda(		sda			),
	.scl(		scl			),
	.writeEn(	writeEnB		),
	.dataFromRegIF(	tx_buf			),
	.dataToRegIF(	diB			),
	.regAddr(	regAddrB		),
	.ack_bit(	ack_bit			),
	.done(		done_bit		),
	.rd(		rd_bit			),
	.set_irqr(	set_irqr		),
	.en_sl(		en			)
);
 
endmodule
