/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________

	based on code from Steve Fielding

*/


//////////////////////////////////////////////////////////////////////
////                                                              ////
//// serialInterface.v                                            ////
////                                                              ////
//// This file is part of the i2cSlave opencores effort.          ////
//// <http://www.opencores.org/cores//>                           ////
////                                                              ////
//// Module Description:                                          ////
//// Perform all serial to parallel, and parallel                 ////
//// to serial conversions. Perform device address matching       ////
//// Handle arbitrary length I2C reads terminated by NAK          ////
//// from host, and arbitrary length I2C writes terminated        ////
//// by STOP from host                                            ////
///*********************** NOT IMPLEMENTET ************************///
///* The second byte of a I2C write is always interpreted         *///
///* as a register address, and becomes the base register address *///
///* for all read and write transactions.                         *///
///* I2C WRITE: devAddr, regAddr, data[regAddr], data[regAddr+1], *///
///*                        ..... data[regAddr+N]                 *///
///* I2C READ:  data[regAddr], data[regAddr+1],                   *///
///*                     ..... data[regAddr+N]                    *///
///* Note that when regAddR reaches 255 it will automatically     *///
///****************************************************************///
//// wrap round to 0                                              ////
////                                                              ////
//// To Do:                                                       ////
////                                                              ////
////                                                              ////
//// Author(s):                                                   ////
//// - Steve Fielding, sfielding@base2designs.com                 ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2008 Steve Fielding and OPENCORES.ORG          ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE. See the GNU Lesser General Public License for more  ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from <http://www.opencores.org/lgpl.shtml>                   ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
`include "i2c_slave/src/timescale.vh"
`include "i2c_slave/src/i2cSlave_define.vh"
`default_nettype none

module serialInterface #(
	parameter I2C_ADR	= 7'h12,
	parameter ADDR_WIDTH	= 4
)
(
	clearStartStopDet, 
	clk, 
	dataIn, 
	dataOut, 
	regAddr, 
	rst, 	
	scl, 
	sdaIn, 
	sdaOut, 
	startStopDetState, 
	writeEn,
	ack_bit,
	set_irqr,
	rd
);
input				clk;
input	[7:0]			dataIn;
input				rst;
input				scl;
input				sdaIn;
input	[1:0]			startStopDetState;
output				clearStartStopDet;
output	[7:0]			dataOut;
output	[ADDR_WIDTH -1: 0]	regAddr;
output				sdaOut;
output				writeEn;
output				ack_bit;
output				set_irqr;
output				rd;
 
reg				clearStartStopDet, next_clearStartStopDet;
wire				clk;
wire	[7:0]			dataIn;
reg	[7:0]			dataOut, next_dataOut;
reg 	[ADDR_WIDTH -1: 0]	regAddr, next_regAddr;
wire				rst;
wire				scl;
wire				sdaIn;
reg				sdaOut, next_sdaOut;
wire	[1:0]			startStopDetState;
reg				writeEn, next_writeEn;
reg				ack_bit, next_ack_bit;
wire				set_irqr;
reg				rd, next_rd;
 
// diagram signals declarations
reg	[2:0]			bitCnt, next_bitCnt;
reg	[7:0]			rxData, next_rxData;
reg	[1:0]			streamSt, next_streamSt;
reg	[7:0]			txData, next_txData;
 
// BINARY ENCODED state machine: SISt
// State codes definitions:
`define START			4'b0000
`define CHK_RD_WR		4'b0001
`define READ_RD_LOOP		4'b0010
`define READ_WT_HI		4'b0011
`define READ_CHK_LOOP_FIN	4'b0100
`define READ_WT_LO		4'b0101
`define READ_WT_ACK		4'b0110
`define WRITE_WT_LO		4'b0111
`define WRITE_WT_HI		4'b1000
`define WRITE_CHK_LOOP_FIN	4'b1001
`define WRITE_LOOP_WT_LO	4'b1010
`define WRITE_ST_LOOP		4'b1011
`define WRITE_WT_LO2		4'b1100
`define WRITE_WT_HI2		4'b1101
`define WRITE_CLR_WR		4'b1110
`define WRITE_CLR_ST_STOP	4'b1111
 
reg	[3:0]			OldState_SISt, CurrState_SISt, NextState_SISt;
 
// Diagram actions (continuous assignments allowed only: assign ...)
// diagram ACTION

// Daten wurden vom Master gelesen
assign	set_irqr	= !ack_bit & scl & rd & (streamSt == `STREAM_IDLE) & (CurrState_SISt == `CHK_RD_WR) & (OldState_SISt == `READ_WT_ACK);

// Machine: SISt
 
// NextState logic (combinatorial)
always @ (startStopDetState or streamSt or scl or txData or bitCnt or rxData or sdaIn or regAddr or dataIn or sdaOut or writeEn or dataOut or clearStartStopDet or CurrState_SISt)
begin
  NextState_SISt		<= CurrState_SISt;
  // Set default values for outputs and signals
  next_rd			<= rd;
  next_ack_bit			<= ack_bit;
  next_streamSt			<= streamSt;
  next_txData			<= txData;
  next_rxData			<= rxData;
  next_sdaOut			<= sdaOut;
  next_writeEn			<= writeEn;
  next_dataOut			<= dataOut;
  next_bitCnt			<= bitCnt;
  next_clearStartStopDet	<= clearStartStopDet;
  next_regAddr			<= regAddr;
  case (CurrState_SISt)  // synopsys parallel_case full_case
    `START:										// 0
	begin
	  next_streamSt			<= `STREAM_IDLE;
	  next_txData			<= 8'h00;
	  next_rxData			<= 8'h00;
	  next_sdaOut			<= 1'b1;
	  next_writeEn			<= 1'b0;
	  next_dataOut			<= 8'h00;
	  next_bitCnt			<= 3'b000;
	  next_clearStartStopDet	<= 1'b0;
	  NextState_SISt		<= `CHK_RD_WR;
	end
    `CHK_RD_WR:										// 1
	begin
	  if (streamSt == `STREAM_READ)
	  begin
	    NextState_SISt		<= `WRITE_WT_LO;	//READ_RD_LOOP;
//	    next_txData			<= dataIn;
//	    next_regAddr		<= regAddr + 1'b1;
	    next_bitCnt			<= 3'b000;
	  end
	  else
	  begin
	    NextState_SISt		<= `WRITE_WT_HI;
	    next_rxData			<= 8'hff;
	    next_sdaOut			<= 1'b1;
	  end
	end
    `READ_RD_LOOP:									// 2
	begin
	  if (scl == 1'b0)
	  begin
	    NextState_SISt		<= `READ_WT_HI;
	    next_sdaOut			<= txData [7];
	    next_txData			<= {txData [6:0], 1'b1};	// Mit 1 fuellen damit nach dem 8. Bit sdaOut inaktiv!
	  end
	end
    `READ_WT_HI:									// 3
	begin
	  if (scl == 1'b1)
	  begin
	    NextState_SISt		<= `READ_CHK_LOOP_FIN;
	  end
	end
    `READ_CHK_LOOP_FIN:									// 4
	begin
	  if (bitCnt == 3'b000)
	  begin
	    NextState_SISt		<= `READ_WT_LO;
	  end
	  else
	  begin
	    NextState_SISt		<= `READ_RD_LOOP;
 	    next_bitCnt			<= bitCnt + 1'b1;
	  end
	end
    `READ_WT_LO:									// 5
	begin
	  if (scl == 1'b0)
	  begin
	    NextState_SISt		<= `READ_WT_ACK;
	    next_sdaOut			<= 1'b1;
	  end
	end
    `READ_WT_ACK:									// 6
	begin
	  if (scl == 1'b1)
	  begin
	    NextState_SISt		<= `CHK_RD_WR;
	    next_ack_bit		<= sdaIn;
//	    if (sdaIn == `I2C_NAK)
	    next_streamSt		<= `STREAM_IDLE;
	  end
	end
    `WRITE_WT_LO:									// 7
	begin
	  if ((scl == 1'b1) && (startStopDetState == `STOP_DET))// || 
//	    (streamSt == `STREAM_IDLE && startStopDetState == `NULL_DET)))
	   begin
	     NextState_SISt		<= `WRITE_CLR_ST_STOP;
	    case (startStopDetState)
	    `NULL_DET:
	     	next_bitCnt		<= bitCnt + 1'b1;
	    `START_DET: begin
	    	next_streamSt		<= `STREAM_IDLE;
	    	next_rxData		<= 8'h00;
	    	end
	    default:	;
	    endcase
	    next_streamSt		<= `STREAM_IDLE;
	    next_clearStartStopDet	<= 1'b1;
	   end
	   else if ((scl == 1'b0) && (rd == 1'b0))
	   begin	// WRITE ADR or DATA
	    NextState_SISt		<= `WRITE_ST_LOOP;
	    case (startStopDetState)
	    `NULL_DET:
	    	next_bitCnt		<= bitCnt + 1'b1;
	    `START_DET: begin
	    	next_streamSt		<= `STREAM_IDLE;
	    	next_rxData		<= 8'h00;
	    	end
	    default:	;
	    endcase
	   end
	   else if ((scl == 1'b0) && (rd == 1'b1) && (streamSt == `STREAM_READ))
	   begin	// READ DATA nach ADR
	    NextState_SISt		<= `READ_RD_LOOP;
	    next_bitCnt			<= bitCnt + 1'b1;
	    next_streamSt		<= `STREAM_READ;
	    next_txData			<= dataIn;
	   end
	   else if ((scl == 1'b0) && (rd == 1'b1) && (streamSt == `STREAM_IDLE) && (sdaIn == 1'b1))
	   begin	// READ DATA nach ADR
	    NextState_SISt		<= `READ_RD_LOOP;
	    next_bitCnt			<= bitCnt + 1'b1;
	    next_streamSt		<= `STREAM_READ;
	    next_txData			<= dataIn;
	   end
	end
    `WRITE_WT_HI:									// 8
	begin
	  if (scl == 1'b1)
	  begin
	    next_bitCnt			<= 3'b000;
	    if (rd)
		begin
//		next_streamSt		<= `STREAM_READ;
		next_sdaOut		<= 1'b1;
		end
	    else
		begin
		next_rxData		<= {rxData [6:0], sdaIn};
		end
	    NextState_SISt		<= `WRITE_WT_LO;
	  end
	end
    `WRITE_CHK_LOOP_FIN:								// 9
	 begin
	   if (bitCnt == 3'b111)
	  begin
	    NextState_SISt		<= `WRITE_CLR_WR;
	    if (rd)	next_sdaOut	<= `I2C_NAK;	// kein ACK vom Slave bei Master-READ
	    else	next_sdaOut	<= `I2C_ACK;	// Daten vom Master quittieren
	    case (streamSt)
	     `STREAM_IDLE: begin
		    if (rxData[7:1] == I2C_ADR &&
		     startStopDetState == `START_DET) begin
			    next_sdaOut		<= `I2C_ACK;	// ACK bei Gleichheit immer
			    if (rxData[0] == 1'b1)
			    begin
			    next_streamSt	<= `STREAM_READ;
			    next_rd		<= 1'b1;
			    end
			    else
			    begin
//			    next_streamSt	<= `STREAM_WRITE_ADDR;
			    next_streamSt	<= `STREAM_WRITE_DATA;
			    next_rd		<= 1'b0;
			    end
			   end
		    else
		    next_sdaOut			<= `I2C_NAK;
		    end
/*	    `STREAM_WRITE_ADDR: begin
		    next_streamSt		<= `STREAM_WRITE_DATA;
		    next_regAddr		<= rxData[ADDR_WIDTH -1:0];
		    end	*/
	    `STREAM_WRITE_DATA: begin
		    next_dataOut		<= rxData;
		    next_writeEn		<= 1'b1;
		    end
	    default:
		   next_streamSt		<= streamSt;
	    endcase
	  end
	  else
	  begin
	    NextState_SISt		<= `WRITE_ST_LOOP;
	    next_bitCnt			<= bitCnt + 1'b1;
	  end
	end
    `WRITE_LOOP_WT_LO:									// A
	begin
	  if (scl == 1'b0)
	  begin
	    NextState_SISt		<= `WRITE_CHK_LOOP_FIN;
	  end
	end
    `WRITE_ST_LOOP:									// B
	begin
	  if (scl == 1'b1)
	  begin
	    NextState_SISt		<= `WRITE_LOOP_WT_LO;
	    next_rxData			<= {rxData [6:0], sdaIn};
	  end
	end
    `WRITE_WT_LO2:									// C
	begin
	  if (scl == 1'b0)
	  begin
	    NextState_SISt		<= `CHK_RD_WR;
	    next_sdaOut			<= 1'b1;
	  end
	end
    `WRITE_WT_HI2:									// D
	begin
	  next_clearStartStopDet	<= 1'b0;
	  if (scl == 1'b1)
	  begin
	    NextState_SISt		<= `WRITE_WT_LO2;
	  end
	end
    `WRITE_CLR_WR:									// E
	begin
	  if (writeEn == 1'b1)
//	  next_regAddr			<= regAddr + 1'b1;
	  next_writeEn			<= 1'b0;
	  next_clearStartStopDet	<= 1'b1;
	  NextState_SISt		<= `WRITE_WT_HI2;
	end
    `WRITE_CLR_ST_STOP:									// F
	begin
	  next_clearStartStopDet	<= 1'b0;
	  NextState_SISt		<= `CHK_RD_WR;
	end
  endcase
end
 
// Current State Logic (sequential)
always @ (posedge clk)
begin
  if (rst == 1'b1)
    begin
    CurrState_SISt	<= `START;
    OldState_SISt	<= `START;
    end
  else
    begin
    OldState_SISt	<= CurrState_SISt;
    CurrState_SISt	<= NextState_SISt;
    end
end
 
// Registered outputs logic
always @ (posedge clk)
begin
  if (rst == 1'b1)
  begin
    rd			<= 1'b0;
    sdaOut		<= 1'b1;
    ack_bit		<= 1'b1;	// NAK
    writeEn		<= 1'b0;
    dataOut		<= 8'h00;
    clearStartStopDet	<= 1'b0;
    // regAddr <=     // Initialization in the reset state or default value required!!
    streamSt		<= `STREAM_IDLE;
    txData		<= 8'h00;
    rxData		<= 8'h00;
    bitCnt		<= 3'b000;
  end
  else 
  begin
    rd			<= next_rd;
    ack_bit		<= next_ack_bit;
    sdaOut		<= next_sdaOut;
    writeEn		<= next_writeEn;
    dataOut		<= next_dataOut;
    clearStartStopDet	<= next_clearStartStopDet;
    regAddr		<= next_regAddr;
    streamSt		<= next_streamSt;
    txData		<= next_txData;
    rxData		<= next_rxData;
    bitCnt		<= next_bitCnt;
  end
end
 
endmodule
