
// AUTOR: Hermann Reichert
// THIS CONTENT WAS COPIED FROM spartanmc/hadware/core_memory/src/ mem_local...
//

/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`include	"core/src/spartanmc_def.vh"

module memDualPorted #(
	parameter	BASE_ADDR	=	0,
	parameter	RAMBLOCKS	=	4,
    parameter	SHOW_MEM	=	1
)
(
	input wire							reset,		// TODO possibly combine?!
	input wire							reset2,
	
	// data-bus to spartanmc
	input wire							clk,	
	input wire		[9:0]				mem_addr_block,
	input wire		[7:0]				mem_addr_high,
	input wire		[17:0]				mem_di,
	input wire							mem_wr_low,
	input wire							mem_wr_high,
	input wire							mem_access,
	output wire		[17:0]				mem_do,
	output wire							mem_busy,

	// second-data-bus
	input wire							clk2,
	input wire		[9:0]				mem_addr_block2,
	input wire		[7:0]				mem_addr_high2,
	input wire		[17:0]				mem_di2,
	input wire							mem_wr_low2,
	input wire							mem_wr_high2,
	input wire							mem_access2,
	output wire		[17:0]				mem_do2,
	output wire							mem_busy2
);


assign mem_busy  = 1'b0;
assign mem_busy2 = 1'b0;

// ****************************************************************

wire	[18*RAMBLOCKS-1:0]	mem_do_all;
wire	[18*RAMBLOCKS-1:0]	mem2_do_all;

//assign code_valid = 1;

generate
	if	(RAMBLOCKS	== 1) begin
		assign	mem_do2	=	mem2_do_all[17:0];
		assign	mem_do  =	mem_do_all[17:0];
	end else begin
// ****************************************************************
// OR Speicherports fuer Befehle bei mehr als einem Speicherblock
		orgate	#(
			.OR_WIDTH	(RAMBLOCKS	),
			.BUS_WIDTH	(18		)
			)
		imem_or (
			.a		({mem2_do_all}	),
			.y		(mem_do2		)
			);

		orgate	#(
			.OR_WIDTH	(RAMBLOCKS	),
			.BUS_WIDTH	(18		)
			)
		dmem_or (
			.a		({mem_do_all}	),
			.y		(mem_do		)
			);
	end
endgenerate
// ****************************************************************


// Simulator Outputs

// synthesis translate_off
generate
	if	(SHOW_MEM	== 1)
	always @(posedge clk)
		if (mem_access)
		begin
            $write("[ %9d ]  ", $time);
			if (mem_wr_low | mem_wr_high)
				$display("WR  mem[0x%05H] <= 0x%05H", {mem_addr_high, mem_addr_block}, mem_di);
			else
				$display("RD  mem[0x%05H] => Reg", {mem_addr_high, mem_addr_block});
		end
endgenerate
// synthesis translate_on

// ****************************************************************
// Speicherblock bestehend aus zwei 9 Bit Blockrams und den Dekodern
wire	[RAMBLOCKS-1:0]	access_iram;
reg	[RAMBLOCKS-1:0]	activ_iram;
wire	[RAMBLOCKS-1:0]	access_bram;
reg	[RAMBLOCKS-1:0]	activ_bram;



genvar		i;
generate
for	(i=0; i<RAMBLOCKS; i=i+1)
begin:	MEM_BL
// ****************************************************************
// Dekodierung des Speicherports fuer die Befehle

pselect        #(
			.ADDR_WIDTH	(7		),
			.BASE_WIDTH	(7		),
			.BASE_ADDR	(BASE_ADDR[17:11] + i) // Oberen 7 Bit der Speicherblockadresse
		)
	imemsel	(
			.addr		(mem_addr_high2[7:1] ),
			.activ_peri	(mem_access2 ),
			.select		(access_iram[i]	)
		);

always @(access_iram[i])
activ_iram[i]	=	access_iram[i];

// Dekodierung des Daten Speicherports

pselect        #(
			.ADDR_WIDTH	(7		),
			.BASE_WIDTH	(7		),
			.BASE_ADDR	(BASE_ADDR[17:11] + i) // Oberen 7 Bit der Speicherblockadresse
		)
	memsel	(
			.addr		(mem_addr_high[7:1]	),
			.activ_peri	(mem_access	),
			.select		(access_bram[i]	)
		);

always @(access_bram[i])
activ_bram[i]	=	access_bram[i];
// die SSR Signale werden im Block RAM mit dem CLK Synchronisiert, dadurch Output erst nach CLK im WB
// siehe DS312.pdf Seite 40 SSRA und SSRB Signal

// fuer new_pipe keine zusaetzliche Verzoegerung wie folgt notwendig!
//always @(posedge clka)	// clka = clk1
//bram_read[i]	<=	! activ_bram[i];		// lesen der Daten erst im WB bei new_pipe


// Installierte Blockrams 2*9 Bit = 18 Bit

ram_9_prim  #(
// 		.WRITE_MODE_A(	"WRITE_FIRST"		),
// 		.WRITE_MODE_B(	"WRITE_FIRST"		),
		.WRITE_MODE_A(	"NO_CHANGE"			),
		.WRITE_MODE_B(	"NO_CHANGE"			),
        .INIT_A(	9'h000				),
		.SRVAL_A(	9'h000				),
		.INIT_B(	9'h000				),
		.SRVAL_B(	9'h000				)
		)
DPRAMA		(
		.DIA(		mem_di2[8:0]			),
		.ADDRA(		{mem_addr_high2[0],mem_addr_block2[9:0]}			),
		.ENA(		1'b1			),
		.WEA(		mem_wr_low2	& access_iram[i]				),
		.SSRA(		! activ_iram[i]			),
		.CLKA(		clk2				),
		.DOA(		mem2_do_all[18*i+8:18*i+0]		),
		
		.DIB(		mem_di[8:0]			),
		.ADDRB(		{mem_addr_high[0],mem_addr_block[9:0]}			),
		.ENB(		1'b1				),
		.WEB(		mem_wr_low	& access_bram[i]	),
		.SSRB(		! activ_bram[i]			),
		.CLKB(		clk				),
		.DOB(		mem_do_all[18*i+8:18*i+0]		)
		);

ram_9_prim  #(
// 		.WRITE_MODE_A(	"WRITE_FIRST"		),
// 		.WRITE_MODE_B(	"WRITE_FIRST"		),
		.WRITE_MODE_A(	"NO_CHANGE"			),
		.WRITE_MODE_B(	"NO_CHANGE"			),
        .INIT_A(	9'h000				),
		.SRVAL_A(	9'h000				),
		.INIT_B(	9'h000				),
		.SRVAL_B(	9'h000				)
		)
DPRAMB		(
		.DIA(		mem_di2[17:9]			),
		.ADDRA(		{mem_addr_high2[0],mem_addr_block2[9:0]}			),
		.ENA(		1'b1			),
		.WEA(		mem_wr_high2 & access_iram[i]			),
		.SSRA(		! activ_iram[i]			),
		.CLKA(		clk2				),
		.DOA(		mem2_do_all[18*i+17:18*i+9]		),
		
		.DIB(		mem_di[17:9]			),
		.ADDRB(		{mem_addr_high[0],mem_addr_block[9:0]}			),
		.ENB(		1'b1				),
		.WEB(		mem_wr_high & access_bram[i]	),
		.SSRB(		! activ_bram[i]			),
		.CLKB(		clk				),
		.DOB(		mem_do_all[18*i+17:18*i+9]		)
		);
// ****************************************************************
end
endgenerate

endmodule
