/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none


module multi_mem_switcher
#(
	parameter CONNECTIONS_B							= 'd1,
    parameter RAMBLOCKS_PER_CONNECTION				= 'd1,
	parameter ADR_LEN								= 'd11,
	parameter RAMBLOCK_SELECT_BITS					= 'd1

)
(
        
    input wire									clkA,
	input wire		[17:0]						data_inA,
    output wire		[17:0]						data_outA,
	input wire									wr_lowA,

	input wire									wr_highA,
	input wire		[ADR_LEN-1:0]				mem_addressA,
	
	input wire		[CONNECTIONS_B-1:0]			clkB,
	input wire		[18*CONNECTIONS_B-1:0]		data_inB,
    output wire		[18*CONNECTIONS_B-1:0]		data_outB,
	input wire		[CONNECTIONS_B-1:0]			wr_lowB,

	input wire		[CONNECTIONS_B-1:0]			wr_highB,
	input wire		[CONNECTIONS_B*ADR_LEN-1:0]	mem_addressB,
	
	input wire		[RAMBLOCK_SELECT_BITS-1:0]	ram_selectA,
	input wire		[CONNECTIONS_B*RAMBLOCK_SELECT_BITS-1:0]	ram_selectB
        
);


	
parameter BLOCK_ADR_LEN = ADR_LEN - 11;

wire	[CONNECTIONS_B*RAMBLOCKS_PER_CONNECTION-1:0]	access_bramA;
wire	[CONNECTIONS_B*RAMBLOCKS_PER_CONNECTION-1:0]	access_bramB;

wire	[18*RAMBLOCKS_PER_CONNECTION-1:0]				data_out_allA;
wire	[CONNECTIONS_B*18*RAMBLOCKS_PER_CONNECTION-1:0]	data_out_allB;
wire	[9*RAMBLOCKS_PER_CONNECTION-1:0]	data_out_0B_low;
wire	[9*RAMBLOCKS_PER_CONNECTION-1:0]	data_out_0B_high;
wire	[9*RAMBLOCKS_PER_CONNECTION-1:0]	data_out_0A_low;
wire	[9*RAMBLOCKS_PER_CONNECTION-1:0]	data_out_0A_high;


assign data_out_allB = {data_out_1B_high, data_out_1B_low, data_out_0B_high, data_out_0B_low};
assign data_out_allA = {data_out_1A_high, data_out_1A_low, data_out_0A_high, data_out_0A_low};

		orgate #(
			.OR_WIDTH	(	2*RAMBLOCKS_PER_CONNECTION	),
			.BUS_WIDTH	(	18							)
			)
		dmem_orA (
			.a		( 	data_out_allA	),
			.y		( 	data_outA		)
		);

		orgate #(
        	.OR_WIDTH	(	2*RAMBLOCKS_PER_CONNECTION	),
        	.BUS_WIDTH	(	18							)
        	)
        dmem_orB (
        	.a		( 	data_out_allB	),
        	.y		( 	data_outB		)
        );



genvar i;
genvar j;
generate
for(i = 0; i < RAMBLOCKS_PER_CONNECTION; i = i+1)
begin : MEM_BL

	if (ADR_LEN < 12)
	begin

		assign access_bramA = 1;
		assign access_bramB = 1;

	end else begin

		pselect        #(
			.ADDR_WIDTH	(BLOCK_ADR_LEN		),
			.BASE_WIDTH	(BLOCK_ADR_LEN		),
			.BASE_ADDR	(i					)
			)
		memselA	(
			.addr		(mem_addressA[ADR_LEN-1:11]	),
			.activ_peri	(1'b1						),
			.select		(access_bramA[i]			)
			);

		for(j = 0; j < CONNECTIONS_B; j = j+1)
		begin : SELECT
			pselect        #(
				.ADDR_WIDTH	(BLOCK_ADR_LEN		),
				.BASE_WIDTH	(BLOCK_ADR_LEN		),
				.BASE_ADDR	(i					)
				)
			memselB	(
				.addr		(mem_addressB[(j+1)*ADR_LEN-1:j*ADR_LEN+11]	),
				.activ_peri	(1'b1						),
				.select		(access_bramB[j*RAMBLOCKS_PER_CONNECTION+i]			)
			);
		end

	end

	for(j = 0; j < CONNECTIONS_B+1; j = j+1)
	begin : SLAVE_MEM
//TODO einrücken
ram_9_prim  #(
		.WRITE_MODE_A(	"READ_FIRST"	),
		.INIT_A(	9'h000				),
		.SRVAL_A(	9'h000				),
		.WRITE_MODE_B(	"READ_FIRST"	),
		.INIT_B(	9'h000				),
		.SRVAL_B(	9'h000				)
		)
DPRAMA0		(
		.DIA(		data_inA[8:0]								),
		.ADDRA(		mem_addressA[10:0]							),
		.ENA(		1'b1										),
		.WEA(		!ram_selectA== & wr_lowA	& access_bramA[i]	),
		.SSRA(		ram_select || !access_bramA[i]				),
		.CLKA(		clkA										),
		.DOA(		data_out_0A_low[9*i+8:9*i]				),


		.DIB(		data_inB[8:0]								),
		.ADDRB(		mem_addressB[10:0]							),
		.ENB(		1'b1										),
		.WEB(		ram_select & wr_lowB	& access_bramA[i]	),
		.SSRB(		!ram_select || !access_bramB[i]				),
		.CLKB(		clkB										),
		.DOB(		data_out_0B_low[9*i+8:9*i]				)
		);

ram_9_prim  #(
		.WRITE_MODE_A(	"READ_FIRST"	),
		.INIT_A(	9'h000				),
		.SRVAL_A(	9'h000				),
		.WRITE_MODE_B(	"READ_FIRST"	),
		.INIT_B(	9'h000				),
		.SRVAL_B(	9'h000				)
		)
DPRAMB0		(
		.DIA(		data_inA[17:9]								),
		.ADDRA(		mem_addressA[10:0]							),
		.ENA(		1'b1										),
		.WEA(		!ram_select & wr_highA	& access_bramA[i]	),
		.SSRA(		ram_select || !access_bramA[i]				),
		.CLKA(		clkA										),
		.DOA(		data_out_0A_high[9*i+8:9*i]			),


		.DIB(		data_inB[17:9]								),
		.ADDRB(		mem_addressB[10:0]							),
		.ENB(		1'b1										),
		.WEB(		ram_select & wr_highB	& access_bramA[i]	),
		.SSRB(		!ram_select || !access_bramB[i]				),
		.CLKB(		clkB										),
		.DOB(		data_out_0B_high[9*i+8:9*i]			)
		);

// ****************************************************************
	end
end
endgenerate
endmodule






