/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`timescale 1ns / 1ps
`default_nettype none


module switch
#(
    parameter CONNECTIONS		= 18'h2,
    parameter WIDTH				= 'd1,
	parameter SELECT_BITS		= 'd1
)
(
	input wire		[CONNECTIONS*WIDTH-1:0]			in,
    output wire		[CONNECTIONS*WIDTH-1:0]			out,
	
	input wire		[CONNECTIONS*SELECT_BITS-1:0]	select
        
);

wire		[WIDTH-1:0]								in_array			[CONNECTIONS-1:0];
wire		[SELECT_BITS-1:0]						select_array		[CONNECTIONS-1:0];

wire		[CONNECTIONS*CONNECTIONS*WIDTH-1:0]		in_demuxed;
wire		[CONNECTIONS*CONNECTIONS*WIDTH-1:0]		in_demuxed_defiddle;
 

	
genvar		i;
genvar		j;
generate
	

	for (i=0; i<CONNECTIONS; i=i+1)
   	begin : ARRAY_GEN
		assign in_array[i]			= in		[WIDTH*i+WIDTH-1:WIDTH*i];
		assign select_array[i]		= select	[SELECT_BITS*i+SELECT_BITS-1:SELECT_BITS*i];
	end

	for (i=0; i<CONNECTIONS; i=i+1)
   	begin : DEMUX_N_OR
		demux #(
			.OUTPUTS(		CONNECTIONS		),
			.SELECT_BITS(	SELECT_BITS		),
			.WIDTH(			WIDTH			)
			)demuxed(
			.in(			in_array[i]																),
			.sel(			select_array[i]															),
			.out(			in_demuxed[WIDTH*CONNECTIONS*i+WIDTH*CONNECTIONS-1:WIDTH*CONNECTIONS*i]	) 
		);

		for (j=0; j<CONNECTIONS; j=j+1)
		begin : DEFIDDLE_DEMUXED
			assign in_demuxed_defiddle[i*WIDTH*CONNECTIONS+j*WIDTH+WIDTH-1 : i*WIDTH*CONNECTIONS+j*WIDTH] = in_demuxed[i*WIDTH+j*CONNECTIONS*WIDTH+WIDTH-1 : i*WIDTH+j*CONNECTIONS*WIDTH];
		end

		orgate #(
			.OR_WIDTH	(	CONNECTIONS	),
			.BUS_WIDTH	(	WIDTH		)
		) ored (
			.a 			( in_demuxed_defiddle[WIDTH*CONNECTIONS*i+WIDTH*CONNECTIONS-1:WIDTH*CONNECTIONS*i]	),
			.y 			( out[WIDTH*i+WIDTH-1:WIDTH*i] 														)
);	
	end
endgenerate

endmodule






