/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns/1ps
`default_nettype none
module test_dual_mem_switcher();

	parameter CONNECTIONS							= 18'h2;
    parameter RAMBLOCKS_PER_CONNECTION				= 'd1;
	parameter ADR_LEN								= 'd13;
	parameter RAM_SEL_BITS							= 'd1;


	reg							clk;
	reg		[17:0]				data_inA_array		[CONNECTIONS-1:0];
    wire	[17:0]				data_outA_array		[CONNECTIONS-1:0];
	reg							wr_low_array		[CONNECTIONS-1:0];

	reg							wr_high_array		[CONNECTIONS-1:0];
	reg		[ADR_LEN-1:0]		mem_addressA_array	[CONNECTIONS-1:0];

	reg							ram_select;


	



dual_mem_switcher #(
    .RAMBLOCKS_PER_CONNECTION	(RAMBLOCKS_PER_CONNECTION),
	.ADR_LEN					(ADR_LEN)

)memory_switcher(
    .clkA			( clk						),
	.data_inA		( data_inA_array[0]			),
    .data_outA		( data_outA_array[0]		),
	.wr_lowA		( wr_low_array[0]			),
	
	.wr_highA		( wr_high_array[0]			),
	.mem_addressA	( mem_addressA_array[0]		),

	.clkB			( clk						),
	.data_inB		( data_inA_array[1] 		),
    .data_outB		( data_outA_array[1] 		),
	.wr_lowB		( wr_low_array[1] 		 	),
	
	.wr_highB		( wr_high_array[1]		 	),
	.mem_addressB	( mem_addressA_array[1]	 	),

	.ram_select		( ram_select )      
);




always begin #5 clk = ~clk; end


initial begin
	clk = 0;
	mem_addressA_array[0] = 0;
	mem_addressA_array[1] = 0;
	ram_select=0;
	wr_low_array[0] = 0;
	wr_high_array[0] = 0;
	wr_low_array[1] = 0;
	wr_high_array[1] = 0;
	data_inA_array[0] = 0;
	data_inA_array[1] = 0;
	#105
	mem_addressA_array[0] = 1;
	data_inA_array[0] = 1;
	wr_low_array[0] = 1;
	wr_high_array[0] = 1;

	mem_addressA_array[1] = 17;
	data_inA_array[1] = 17;
	wr_low_array[1] = 1;
	wr_high_array[1] = 1;
	
	@(posedge clk)
	mem_addressA_array[0] = 2;
	data_inA_array[0] = 2;
	
	mem_addressA_array[1] = 18;
	data_inA_array[1] = 18;

	@(posedge clk)
	wr_low_array[0] = 0;
	wr_high_array[1] = 0;

	mem_addressA_array[0] = 3;
	data_inA_array[0] = 18'h2FFFB;
	mem_addressA_array[1] = 19;
	data_inA_array[1] = 18'h2FFFB;

	@(posedge clk)
	wr_low_array[0] = 0;
	wr_high_array[0] = 0;

	wr_low_array[1] = 0;
	wr_high_array[1] = 0;


	mem_addressA_array[0] = 3;
	data_inA_array[0] = 4;
	mem_addressA_array[1] = 19;
	data_inA_array[1] = 20;

	@(posedge clk)
	mem_addressA_array[0] = 1;
	mem_addressA_array[1] = 17;
	@(posedge clk)
	@(negedge clk)
	if (!(data_outA_array[0] == 1)) begin
		$error("Error1: expecting mem read value 1 but got %h", data_outA_array[0]);
	end
	if (!(data_outA_array[1] == 17)) begin
		$error("Error2: expecting mem read value 17 but got %h", data_outA_array[1]);
	end
	
	@(posedge clk)
	//switch mems
	ram_select=!ram_select;
	
	mem_addressA_array[0] = 18;
	mem_addressA_array[1] = 2;
	@(posedge clk)
	@(negedge clk)
	if (!(data_outA_array[0] == 18)) begin
		$error("Error3: expecting mem read value 18 but got %h", data_outA_array[0]);
	end
	if (!(data_outA_array[1] == 2)) begin
		$error("Error4: expecting mem read value 2 but got %h", data_outA_array[1]);
	end

	@(posedge clk)
	mem_addressA_array[0] = 19;
	mem_addressA_array[1] = 3;
	@(posedge clk)
	@(negedge clk)
	if (!(data_outA_array[0] == 18'h1fb)) begin
		$error("Error5: expecting mem read value 18'h1fb but got %h", data_outA_array[0]);
	end
	if (!(data_outA_array[1] == 18'h2fe00)) begin
		$error("Error6: expecting mem read value 18'h2fe00 but got %h", data_outA_array[1]);
	end
	//write to same addresses
	@(posedge clk)
	wr_low_array[0] = 1;
	wr_high_array[0] = 1;
	wr_low_array[1] = 1;
	wr_high_array[1] = 1;
	mem_addressA_array[0] = 4;
	data_inA_array[0] = 4;
	mem_addressA_array[1] = 4;
	data_inA_array[1] = 44;
	
	@(posedge clk)
	wr_low_array[0] = 0;
	wr_high_array[0] = 0;
	wr_low_array[1] = 0;
	wr_high_array[1] = 0;

	//switch mems
	ram_select=!ram_select;
	@(posedge clk)
	@(negedge clk)
	if (!(data_outA_array[0] == 44)) begin
		$error("Error7: expecting mem read value 44 but got %h", data_outA_array[0]);
	end
	if (!(data_outA_array[1] == 4)) begin
		$error("Error8: expecting mem read value 4 but got %h", data_outA_array[1]);
	end
	@(posedge clk)
	$stop();

end

endmodule
