/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
`timescale 1ns / 1ps


module test_mem_swap_multi;

	parameter DMA_SIZE				= 18'h800;
	parameter DMA_ADDRWIDTH			= 'd11;
	parameter SLAVE_CONNECTIONS		= 'd3;
	parameter SOFTWARE_ARBITRATION	= 1'b0;
	parameter BASE_ADR				= 18'h1fc00;
	parameter BASE_ADR_LOW			= 10'h000;
	parameter BASE_ADR_HIGH		= 8'hff;
	parameter DMA_BASE_ADR			= 18'h2000;
	parameter DMA_BASE_ADR_LOW		= 10'h000;
	parameter DMA_BASE_ADR_HIGH	= 8'h8;
	
//inputs
		reg										clk;
		reg										reset;

		reg    [SLAVE_CONNECTIONS:0]     		access_peri;//Signale fuer Peripheriezugriffe
        reg    [SLAVE_CONNECTIONS:0]     		wr_peri;	//Schreibsignal

        //DMA-Signals
		reg		[SLAVE_CONNECTIONS:0]		mem_access;
		reg		[SLAVE_CONNECTIONS:0]		store_access_low;
		reg		[SLAVE_CONNECTIONS:0]		store_access_high;
		reg		[9:0]						addr_peri			[SLAVE_CONNECTIONS:0];
		reg		[7:0]						addr_high			[SLAVE_CONNECTIONS:0];
		reg		[17:0]						do_peri				[SLAVE_CONNECTIONS:0];
		wire	[17:0]						di_peri				[SLAVE_CONNECTIONS:0];  
	



        // Connections with slave-connector
        wire	[SLAVE_CONNECTIONS-1:0]	        		swap_request;
        wire	[SLAVE_CONNECTIONS-1:0]					swap_complete;
        

		wire		[17:0]									data_inA	[SLAVE_CONNECTIONS-1:0];
    	wire		[17:0]									data_outA	[SLAVE_CONNECTIONS-1:0];
		wire		[SLAVE_CONNECTIONS-1:0]					wr_low;
		wire		[SLAVE_CONNECTIONS-1:0]					wr_high;
		wire		[DMA_ADDRWIDTH-1:0]	mem_addressA	[SLAVE_CONNECTIONS-1:0];

mem_swap_multi_master #(
		.BASE_ADR(BASE_ADR),
        .DMA_BASE_ADR(DMA_BASE_ADR),
        .DMA_SIZE(DMA_SIZE),
		.DMA_ADDRWIDTH(DMA_ADDRWIDTH),
		.SLAVE_CONNECTIONS(SLAVE_CONNECTIONS),
		.SOFTWARE_ARBITRATION(SOFTWARE_ARBITRATION)
) master_swapper(
.clk_peri(clk),	//Systemtakt
.reset(reset),		//Reset-Signal
//register wires
.access_peri(access_peri[0]),//Signale fuer Peripheriezugriffe
.wr_peri(wr_peri[0]),	//Schreibsignal

//DMA-Signals
.mem_access(mem_access[0]),
.store_access_low(store_access_low[0]),
.store_access_high(store_access_high[0]),
.addr_peri(addr_peri[0]),
.addr_high(addr_high[0]),
.do_peri(do_peri[0]),
.di_peri(di_peri[0]),  
	

.swap_request(swap_request),
.swap_complete(swap_complete),
.clk({SLAVE_CONNECTIONS{clk}}),
.data_inA({data_inA[2],data_inA[1],data_inA[0]}),
.data_outA({data_outA[2],data_outA[1],data_outA[0]}),
.wr_low(wr_low),
.wr_high(wr_high),
.mem_addressA({mem_addressA[2],mem_addressA[1],mem_addressA[0]})
);

genvar i;
generate
for(i = 1; i < SLAVE_CONNECTIONS+1; i = i+1)
begin : SLAVE_MEM_SWAPPER
	mem_swap_slave #(
			.BASE_ADR(BASE_ADR),
		    .DMA_BASE_ADR(DMA_BASE_ADR),
   		    .DMA_SIZE(DMA_SIZE),
			.DMA_ADDRWIDTH(DMA_ADDRWIDTH)
	) slave_swapper(
		.clk_peri(clk),	//Systemtakt
		.reset(reset),		//Reset-Signal
		//register wires
		.access_peri(access_peri[i]),//Signale fuer Peripheriezugriffe
		.wr_peri(wr_peri[i]),	//Schreibsignal

		//DMA-Signals
		.mem_access(mem_access[i]),
		.store_access_low(store_access_low[i]),
		.store_access_high(store_access_high[i]),
		.addr_peri(addr_peri[i]),
		.addr_high(addr_high[i]),
		.do_peri(do_peri[i]),
		.di_peri(di_peri[i]),  
	

		.swap_request(swap_request[i-1]),
		.swap_complete(swap_complete[i-1]),
		.clk(),
		.data_inA(data_inA[i-1]),
		.data_outA(data_outA[i-1]),
		.wr_low(wr_low[i-1]),
		.wr_high(wr_high[i-1]),
		.mem_addressA(mem_addressA[i-1])
	);
end

endgenerate
	

	initial begin
		// Initialize Inputs
		clk = 0;
		reset= 1;
		access_peri=1'b0;
       	wr_peri=1'b0;
		mem_access=1'b0;
		store_access_low=1'b0;
		store_access_high=1'b0;
		addr_peri[0]=10'b0;
		addr_peri[1]=10'b0;
		addr_peri[2]=10'b0;
		addr_peri[3]=10'b0;
		addr_high[0]=8'b0;
		addr_high[1]=8'b0;
		addr_high[2]=8'b0;
		addr_high[3]=8'b0;
		do_peri[0]=18'b0;
		do_peri[1]=18'b0;
		do_peri[2]=18'b0;
		do_peri[3]=18'b0;


		// Wait 100 ns for global reset to finish
		#125;

		reset = 0;
		
		//write values into mem		
		@(posedge clk)
		addr_high[0]=DMA_BASE_ADR_HIGH;
		addr_high[1]=DMA_BASE_ADR_HIGH;
		addr_high[2]=DMA_BASE_ADR_HIGH;
		addr_high[3]=DMA_BASE_ADR_HIGH;
		addr_peri[0]=DMA_BASE_ADR_LOW;
		addr_peri[1]=DMA_BASE_ADR_LOW+1;
		addr_peri[2]=DMA_BASE_ADR_LOW+2;
		addr_peri[3]=DMA_BASE_ADR_LOW+3;
		do_peri[0]=18'h5;
		do_peri[1]=18'h6;
		do_peri[2]=18'h7;
		do_peri[3]=18'h8;
		mem_access={SLAVE_CONNECTIONS+1{1'b1}};
		store_access_low={SLAVE_CONNECTIONS+1{1'b1}};

		@(posedge clk)
		//finish dma write
		mem_access={SLAVE_CONNECTIONS+1{1'b0}};
		store_access_low={SLAVE_CONNECTIONS+1{1'b0}};



		//-----------------test swap master slave 1------------------


		//swap requests
		addr_high[0]=BASE_ADR_HIGH;
		addr_high[1]=BASE_ADR_HIGH;
		addr_high[2]=BASE_ADR_HIGH;
		addr_high[3]=BASE_ADR_HIGH;
		addr_peri[0]=10'b0;
		addr_peri[1]=10'b0;
		addr_peri[2]=10'b0;
		addr_peri[3]=10'b0;
		access_peri={SLAVE_CONNECTIONS+1{1'b1}};
		wr_peri={SLAVE_CONNECTIONS+1{1'b1}};
		do_peri[0]=18'h1;
		do_peri[1]=18'h1;
		do_peri[2]=18'h1;
		do_peri[3]=18'h1;

		@(posedge clk)
		//reset swap req
		wr_peri={SLAVE_CONNECTIONS+1{1'b0}};
		@(posedge clk)
		@(negedge clk)
		if(!(di_peri[0] == 18'h2)) 
			$error("1swap complete for master not received");
		if(!(di_peri[1] == 18'h2)) 
			$error("1swap complete for slave 1 not received");
		if(di_peri[2] == 18'h2) 
			$error("1swap complete for slave 2 received but not expected");
		if(di_peri[3] == 18'h2) 
			$error("1swap complete for slave 3 received but not expected");
		
		@(posedge clk)
		access_peri={SLAVE_CONNECTIONS+1{1'b0}};
		//read DMA
		addr_high[0]=DMA_BASE_ADR_HIGH;
		addr_high[1]=DMA_BASE_ADR_HIGH;
		addr_high[2]=DMA_BASE_ADR_HIGH;
		addr_high[3]=DMA_BASE_ADR_HIGH;
		addr_peri[0]=DMA_BASE_ADR_LOW+1;
		addr_peri[1]=DMA_BASE_ADR_LOW;
		addr_peri[2]=DMA_BASE_ADR_LOW+2;
		addr_peri[3]=DMA_BASE_ADR_LOW+3;
		mem_access = {SLAVE_CONNECTIONS+1{1'b1}};

		@(posedge clk)
		@(negedge clk)
		if(!(di_peri[0] == 18'h6))
			$error("1master expects 6 but got %d",di_peri[0]);
		if(!(di_peri[1] == 18'h5))
			$error("1slave1 expects 5 but got %d",di_peri[1]);
		if(!(di_peri[2] == 18'h7))
			$error("1slave2 expects 7 but got %d",di_peri[2]);
		if(!(di_peri[3] == 18'h8))
			$error("1slave3 expects 8 but got %d",di_peri[3]);
		
		@(posedge clk)
		//finish dma read
		mem_access={SLAVE_CONNECTIONS+1{1'b0}};

		

		//-----------------test swap master slave 3------------------		



		//swap requests
		addr_high[0]=BASE_ADR_HIGH;
		addr_high[1]=BASE_ADR_HIGH;
		addr_high[2]=BASE_ADR_HIGH;
		addr_high[3]=BASE_ADR_HIGH;
		addr_peri[0]=10'b0;
		addr_peri[1]=10'b0;
		addr_peri[2]=10'b0;
		addr_peri[3]=10'b0;
		access_peri={SLAVE_CONNECTIONS+1{1'b1}};
		wr_peri={SLAVE_CONNECTIONS+1{1'b1}};
		do_peri[0]=18'h1;
		do_peri[1]=18'h1;
		do_peri[2]=18'h1;
		do_peri[3]=18'h1;

		@(posedge clk)
		//reset swap req
		wr_peri={SLAVE_CONNECTIONS+1{1'b0}};
		@(posedge clk)		
		@(negedge clk)
		if(!(di_peri[0] == 18'h2)) 
			$error("2swap complete for master not received");
		if(di_peri[1] == 18'h2) 
			$error("2swap complete for slave 1 received but not expected");
		if(!(di_peri[2] == 18'h2)) 
			$error("2swap complete for slave 2 not received");
		if(di_peri[3] == 18'h2) 
			$error("2swap complete for slave 3 received but not expected");
		
		@(posedge clk)
		access_peri={SLAVE_CONNECTIONS+1{1'b0}};
		//read DMA
		addr_high[0]=DMA_BASE_ADR_HIGH;
		addr_high[1]=DMA_BASE_ADR_HIGH;
		addr_high[2]=DMA_BASE_ADR_HIGH;
		addr_high[3]=DMA_BASE_ADR_HIGH;
		addr_peri[0]=DMA_BASE_ADR_LOW+2;
		addr_peri[1]=DMA_BASE_ADR_LOW;
		addr_peri[2]=DMA_BASE_ADR_LOW+1;
		addr_peri[3]=DMA_BASE_ADR_LOW+3;
		mem_access = {SLAVE_CONNECTIONS+1{1'b1}};
		@(posedge clk)
		@(negedge clk)
		if(!(di_peri[0] == 18'h7))
			$error("2master expects 7 but got %d",di_peri[0]);
		if(!(di_peri[1] == 18'h5))
			$error("2slave1 expects 5 but got %d",di_peri[1]);
		if(!(di_peri[2] == 18'h6))
			$error("2slave2 expects 6 but got %d",di_peri[2]);
		if(!(di_peri[3] == 18'h8))
			$error("2slave3 expects 8 but got %d",di_peri[3]);

		@(posedge clk)
		//finish dma write
		mem_access={SLAVE_CONNECTIONS+1{1'b0}};

		


		//-----------------test swap master slave 4------------------


		//swap requests
		addr_high[0]=BASE_ADR_HIGH;
		addr_high[1]=BASE_ADR_HIGH;
		addr_high[2]=BASE_ADR_HIGH;
		addr_high[3]=BASE_ADR_HIGH;
		addr_peri[0]=10'b0;
		addr_peri[1]=10'b0;
		addr_peri[2]=10'b0;
		addr_peri[3]=10'b0;
		access_peri={SLAVE_CONNECTIONS+1{1'b1}};
		wr_peri={SLAVE_CONNECTIONS+1{1'b1}};
		do_peri[0]=18'h1;
		do_peri[1]=18'h1;
		do_peri[2]=18'h1;
		do_peri[3]=18'h1;

		@(posedge clk)
		//reset swap req
		wr_peri={SLAVE_CONNECTIONS+1{1'b0}};
		@(posedge clk)
		@(negedge clk)
		if(!(di_peri[0] == 18'h2)) 
			$error("3swap complete for master not received");
		if(di_peri[1] == 18'h2) 
			$error("3swap complete for slave 1 received but not expected");
		if(di_peri[2] == 18'h2) 
			$error("3swap complete for slave 2 received but not expected");
		if(!(di_peri[3] == 18'h2)) 
			$error("3swap complete for slave 3 not received");
		
		@(posedge clk)
		access_peri={SLAVE_CONNECTIONS+1{1'b0}};
		//read DMA
		addr_high[0]=DMA_BASE_ADR_HIGH;
		addr_high[1]=DMA_BASE_ADR_HIGH;
		addr_high[2]=DMA_BASE_ADR_HIGH;
		addr_high[3]=DMA_BASE_ADR_HIGH;
		addr_peri[0]=DMA_BASE_ADR_LOW+3;
		addr_peri[1]=DMA_BASE_ADR_LOW+0;
		addr_peri[2]=DMA_BASE_ADR_LOW+1;
		addr_peri[3]=DMA_BASE_ADR_LOW+2;
		mem_access = {SLAVE_CONNECTIONS+1{1'b1}};

		@(posedge clk)
		@(negedge clk)
		if(!(di_peri[0] == 18'h8))
			$error("3master expects 8 but got %d",di_peri[0]);
		if(!(di_peri[1] == 18'h5))
			$error("3slave1 expects 5 but got %d",di_peri[1]);
		if(!(di_peri[2] == 18'h6))
			$error("3slave2 expects 6 but got %d",di_peri[2]);
		if(!(di_peri[3] == 18'h7))
			$error("3slave3 expects 7 but got %d",di_peri[3]);

		@(posedge clk)
		//finish dma write
		mem_access={SLAVE_CONNECTIONS+1{1'b0}};
		
	


		//-----------------test wraparound------------------



		//swap requests
		addr_high[0]=BASE_ADR_HIGH;
		addr_high[1]=BASE_ADR_HIGH;
		addr_high[2]=BASE_ADR_HIGH;
		addr_high[3]=BASE_ADR_HIGH;
		addr_peri[0]=10'b0;
		addr_peri[1]=10'b0;
		addr_peri[2]=10'b0;
		addr_peri[3]=10'b0;
		access_peri={SLAVE_CONNECTIONS+1{1'b1}};
		wr_peri={SLAVE_CONNECTIONS+1{1'b1}};
		do_peri[0]=18'h1;
		do_peri[1]=18'h1;
		do_peri[2]=18'h1;
		do_peri[3]=18'h1;

		@(posedge clk)
		//reset swap req
		wr_peri={SLAVE_CONNECTIONS+1{1'b0}};
		@(posedge clk)
		@(negedge clk)
		if(!(di_peri[0] == 18'h2)) 
			$error("4swap complete for master not received");
		if(!(di_peri[1] == 18'h2)) 
			$error("4swap complete for slave 1 not received");
		if(di_peri[2] == 18'h2) 
			$error("4swap complete for slave 2 received but not expected");
		if(di_peri[3] == 18'h2) 
			$error("4swap complete for slave 3 received but not expected");
		
		@(posedge clk)
		//read DMA
		access_peri={SLAVE_CONNECTIONS+1{1'b0}};
		addr_high[0]=DMA_BASE_ADR_HIGH;
		addr_high[1]=DMA_BASE_ADR_HIGH;
		addr_high[2]=DMA_BASE_ADR_HIGH;
		addr_high[3]=DMA_BASE_ADR_HIGH;
		addr_peri[0]=DMA_BASE_ADR_LOW;
		addr_peri[1]=DMA_BASE_ADR_LOW+3;
		addr_peri[2]=DMA_BASE_ADR_LOW+1;
		addr_peri[3]=DMA_BASE_ADR_LOW+2;
		mem_access = {SLAVE_CONNECTIONS+1{1'b1}};
		
		@(posedge clk)
		@(negedge clk)
		if(!(di_peri[0] == 18'h5))
			$error("4master expects 5 but got %d",di_peri[0]);
		if(!(di_peri[1] == 18'h8))
			$error("4slave1 expects 8 but got %d",di_peri[1]);
		if(!(di_peri[2] == 18'h6))
			$error("4slave2 expects 6 but got %d",di_peri[2]);
		if(!(di_peri[4] == 18'h7))
			$error("4slave3 expects 7 but got %d",di_peri[3]);

		@(posedge clk)
		//finish dma write
		mem_access={SLAVE_CONNECTIONS+1{1'b0}};

		
		
		@(posedge clk)
		$stop();
	end

	always #5 clk=!clk;
      
endmodule

