/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns/1ps
`default_nettype none

module access_watcher (
	input wire clk,
	input wire reset,
	input wire [17:0] code_addr,
	input wire [17:0] mem_addr,
	input wire mem_rd,
	input wire mem_wr,

	input wire global_enable,


	input wire read_offs,
	input wire read_size,
	input wire read_conf,
	input wire write_offs,
	input wire write_size,
	input wire write_conf,

	input wire [17:0] write_data,
	output wire [17:0] read_data,	

	output wire error_code,
	output wire error_wr,
	output wire error_rd
);


reg [17:0] offs;
reg [17:0] size;
reg enabled;
reg allow_code;
reg allow_rd;
reg allow_wr;

always @(posedge clk) begin
	if (reset) begin
		offs <= 0;
		size <= 0;
		enabled <= 0;
		allow_code <= 0;
		allow_rd <= 0;
		allow_wr <= 0;
	end else begin
		if (write_offs)
			offs <= write_data;
		if (write_size)
			size <= write_data;

		if (write_conf) begin
			enabled <= write_data[0];
			allow_code <= write_data[1];
			allow_rd <= write_data[2];
			allow_wr <= write_data[3];
		end
	end
end

assign read_data =	read_offs ? offs :
			read_size ? size :
			read_conf ? { 14'd0, allow_wr, allow_rd, allow_code, enabled } :
			0;

wire code_in_range, data_in_range;

assign code_in_range = code_addr >= offs && code_addr < (offs + size);
assign data_in_range = mem_addr[16:0]  >= offs[17:1] && mem_addr[16:0] < (offs[17:1] + size);

assign error_code = global_enable && enabled && !allow_code && code_in_range;
assign error_wr = global_enable && enabled && !allow_wr && data_in_range && mem_wr;
assign error_rd = global_enable && enabled && !allow_rd && data_in_range && mem_rd;

 
endmodule


