proc build_microblaze_v10 { design_name params } {

	#set default paramater values
	array set optional [list \
-FREQ_HZ "1000000" \
-POLARITY "ACTIVE_LOW" \
-C_DEBUG_ENABLED "1" \
-ILMB_BASE_ADDR "0x00000000" \
-ILMB_ADDR_RANGE "0x00001000" \
-DLMB_BASE_ADDR "0x00000000" \
-DLMB_ADDR_RANGE "0x00001000" \
-C_USE_ICACHE "0" \
-C_USE_DCACHE "0" \
-C_USE_BARREL "0" \
-C_USE_DIV "0" \
-C_USE_HW_MUL "1" \
-C_USE_FPU "0" \
-C_USE_PCMP_INSTR "0" \
-C_USE_REORDER_INSTR "0" \
{*}$params]

	#map parameters
	set FREQ_HZ $optional(-FREQ_HZ)
	set POLARITY $optional(-POLARITY)
	set C_DEBUG_ENABLED $optional(-C_DEBUG_ENABLED)
	set ILMB_BASE_ADDR $optional(-ILMB_BASE_ADDR)
	set ILMB_ADDR_RANGE $optional(-ILMB_ADDR_RANGE)
	set DLMB_BASE_ADDR $optional(-DLMB_BASE_ADDR)
	set DLMB_ADDR_RANGE $optional(-DLMB_ADDR_RANGE)
    set C_USE_ICACHE $optional(-C_USE_ICACHE)
    set C_USE_DCACHE $optional(-C_USE_DCACHE)
	set C_USE_BARREL $optional(-C_USE_BARREL)
	set C_USE_DIV $optional(-C_USE_DIV)
	set C_USE_HW_MUL $optional(-C_USE_HW_MUL)
	set C_USE_FPU $optional(-C_USE_FPU)
	set C_USE_PCMP_INSTR $optional(-C_USE_PCMP_INSTR)
	set C_USE_REORDER_INSTR $optional(-C_USE_REORDER_INSTR)

  # Create interface ports
  set BRAM_PORT_DLMB [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:bram_rtl:1.0 BRAM_PORT_DLMB ]
  set_property -dict [ list \
CONFIG.MASTER_TYPE {BRAM_CTRL} \
 ] $BRAM_PORT_DLMB
  set BRAM_PORT_ILMB [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:bram_rtl:1.0 BRAM_PORT_ILMB ]
  set_property -dict [ list \
CONFIG.MASTER_TYPE {BRAM_CTRL} \
 ] $BRAM_PORT_ILMB
  set INTERRUPT [ create_bd_intf_port -mode Slave -vlnv xilinx.com:interface:mbinterrupt_rtl:1.0 INTERRUPT ]

  # Create ports
  set clk [ create_bd_port -dir I -type clk clk ]
  set_property -dict [ list \
CONFIG.FREQ_HZ $FREQ_HZ \
 ] $clk
  set interconnect_aresetn [ create_bd_port -dir O -from 0 -to 0 -type rst interconnect_aresetn ]
  set locked [ create_bd_port -dir I locked ]
  set peripheral_aresetn [ create_bd_port -dir O -from 0 -to 0 -type rst peripheral_aresetn ]
  set peripheral_reset [ create_bd_port -dir O -from 0 -to 0 -type rst peripheral_reset ]
  set reset [ create_bd_port -dir I -type rst reset ]
  set_property -dict [ list \
CONFIG.POLARITY $POLARITY \
 ] $reset

  # Create instance: microblaze_0, and set properties
  set microblaze_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:microblaze:10.0 $design_name ]
  set_property -dict [ list \
CONFIG.C_D_LMB {1} \
CONFIG.C_I_LMB {1} \
CONFIG.C_D_AXI {1} \
CONFIG.C_DEBUG_ENABLED $C_DEBUG_ENABLED \
CONFIG.C_FREQ $FREQ_HZ \
CONFIG.C_USE_ICACHE $C_USE_ICACHE \
CONFIG.C_USE_DCACHE $C_USE_DCACHE \
CONFIG.C_USE_BARREL $C_USE_BARREL \
CONFIG.C_USE_DIV $C_USE_DIV \
CONFIG.C_USE_HW_MUL $C_USE_HW_MUL \
CONFIG.C_USE_FPU $C_USE_FPU \
CONFIG.C_USE_PCMP_INSTR $C_USE_PCMP_INSTR \
CONFIG.C_USE_REORDER_INSTR $C_USE_REORDER_INSTR \
 ] $microblaze_0

  # Create instance: microblaze_0_local_memory
  create_hier_cell_microblaze_0_local_memory [current_bd_instance .] microblaze_0_local_memory

  # Create instance: rst_Clk_100M, and set properties
  set rst_Clk_100M [ create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset:5.0 rst_Clk_100M ]
  set_property -dict [ list \
CONFIG.RESET_BOARD_INTERFACE {Custom} \
CONFIG.USE_BOARD_FLOW {true} \
 ] $rst_Clk_100M

  # Create interface connections
  connect_bd_intf_net -intf_net INTERRUPT_1 [get_bd_intf_ports INTERRUPT] [get_bd_intf_pins $microblaze_0/INTERRUPT]
  connect_bd_intf_net -intf_net microblaze_0_dlmb_1 [get_bd_intf_pins $microblaze_0/DLMB] [get_bd_intf_pins microblaze_0_local_memory/DLMB]
  connect_bd_intf_net -intf_net microblaze_0_ilmb_1 [get_bd_intf_pins $microblaze_0/ILMB] [get_bd_intf_pins microblaze_0_local_memory/ILMB]
  connect_bd_intf_net -intf_net microblaze_0_local_memory_BRAM_PORT [get_bd_intf_ports BRAM_PORT_DLMB] [get_bd_intf_pins microblaze_0_local_memory/BRAM_PORT_DLMB]
  connect_bd_intf_net -intf_net microblaze_0_local_memory_BRAM_PORT1 [get_bd_intf_ports BRAM_PORT_ILMB] [get_bd_intf_pins microblaze_0_local_memory/BRAM_PORT_ILMB]

  # Create port connections
  connect_bd_net -net dcm_locked_1 [get_bd_ports locked] [get_bd_pins rst_Clk_100M/dcm_locked]
  connect_bd_net -net microblaze_0_Clk [get_bd_ports clk] [get_bd_pins $microblaze_0/Clk] [get_bd_pins microblaze_0_local_memory/LMB_Clk] [get_bd_pins rst_Clk_100M/slowest_sync_clk]
  connect_bd_net -net reset_rtl_1 [get_bd_ports reset] [get_bd_pins rst_Clk_100M/ext_reset_in]
  connect_bd_net -net rst_Clk_100M_bus_struct_reset [get_bd_pins microblaze_0_local_memory/SYS_Rst] [get_bd_pins rst_Clk_100M/bus_struct_reset]
  connect_bd_net -net rst_Clk_100M_interconnect_aresetn [get_bd_ports interconnect_aresetn] [get_bd_pins rst_Clk_100M/interconnect_aresetn]
  connect_bd_net -net rst_Clk_100M_mb_reset [get_bd_pins $microblaze_0/Reset] [get_bd_pins rst_Clk_100M/mb_reset]
  connect_bd_net -net rst_Clk_100M_peripheral_aresetn [get_bd_ports peripheral_aresetn] [get_bd_pins rst_Clk_100M/peripheral_aresetn]
  connect_bd_net -net rst_Clk_100M_peripheral_reset [get_bd_ports peripheral_reset] [get_bd_pins rst_Clk_100M/peripheral_reset]

  # Create dummy peripheral ports
  # UART
  create_bd_port -dir O uart_rtl_txd
  create_bd_port -dir I uart_rtl_rxd
  
  # Create memory address segments
  create_bd_addr_seg -range $DLMB_ADDR_RANGE -offset $DLMB_BASE_ADDR [get_bd_addr_spaces $microblaze_0/Data] [get_bd_addr_segs microblaze_0_local_memory/dlmb_bram_if_cntlr/SLMB/Mem] SEG_dlmb_bram_if_cntlr_Mem
  create_bd_addr_seg -range $ILMB_ADDR_RANGE -offset $ILMB_BASE_ADDR [get_bd_addr_spaces $microblaze_0/Instruction] [get_bd_addr_segs microblaze_0_local_memory/ilmb_bram_if_cntlr/SLMB/Mem] SEG_ilmb_bram_if_cntlr_Mem
}
# End of build_microblaze_v10()


# Hierarchical cell: microblaze_0_local_memory
proc create_hier_cell_microblaze_0_local_memory { parentCell nameHier } {

  variable script_folder

  # Set parent object
  set parentObj [get_bd_cells $parentCell]

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj

  # Create cell and set as current instance
  set hier_obj [create_bd_cell -type hier $nameHier]
  current_bd_instance $hier_obj

  # Create interface pins
  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:bram_rtl:1.0 BRAM_PORT_DLMB
  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:bram_rtl:1.0 BRAM_PORT_ILMB
  create_bd_intf_pin -mode MirroredMaster -vlnv xilinx.com:interface:lmb_rtl:1.0 DLMB
  create_bd_intf_pin -mode MirroredMaster -vlnv xilinx.com:interface:lmb_rtl:1.0 ILMB

  # Create pins
  create_bd_pin -dir I -type clk LMB_Clk
  create_bd_pin -dir I -from 0 -to 0 -type rst SYS_Rst

  # Create instance: dlmb_bram_if_cntlr, and set properties
  set dlmb_bram_if_cntlr [ create_bd_cell -type ip -vlnv xilinx.com:ip:lmb_bram_if_cntlr:4.0 dlmb_bram_if_cntlr ]
  set_property -dict [ list \
CONFIG.C_ECC {0} \
 ] $dlmb_bram_if_cntlr

  # Create instance: dlmb_v10, and set properties
  set dlmb_v10 [ create_bd_cell -type ip -vlnv xilinx.com:ip:lmb_v10:3.0 dlmb_v10 ]

  # Create instance: ilmb_bram_if_cntlr, and set properties
  set ilmb_bram_if_cntlr [ create_bd_cell -type ip -vlnv xilinx.com:ip:lmb_bram_if_cntlr:4.0 ilmb_bram_if_cntlr ]
  set_property -dict [ list \
CONFIG.C_ECC {0} \
 ] $ilmb_bram_if_cntlr

  # Create instance: ilmb_v10, and set properties
  set ilmb_v10 [ create_bd_cell -type ip -vlnv xilinx.com:ip:lmb_v10:3.0 ilmb_v10 ]

  # Create interface connections
  connect_bd_intf_net -intf_net Conn1 [get_bd_intf_pins BRAM_PORT_DLMB] [get_bd_intf_pins dlmb_bram_if_cntlr/BRAM_PORT]
  connect_bd_intf_net -intf_net Conn2 [get_bd_intf_pins BRAM_PORT_ILMB] [get_bd_intf_pins ilmb_bram_if_cntlr/BRAM_PORT]
  connect_bd_intf_net -intf_net microblaze_0_dlmb [get_bd_intf_pins DLMB] [get_bd_intf_pins dlmb_v10/LMB_M]
  connect_bd_intf_net -intf_net microblaze_0_dlmb_bus [get_bd_intf_pins dlmb_bram_if_cntlr/SLMB] [get_bd_intf_pins dlmb_v10/LMB_Sl_0]
  connect_bd_intf_net -intf_net microblaze_0_ilmb [get_bd_intf_pins ILMB] [get_bd_intf_pins ilmb_v10/LMB_M]
  connect_bd_intf_net -intf_net microblaze_0_ilmb_bus [get_bd_intf_pins ilmb_bram_if_cntlr/SLMB] [get_bd_intf_pins ilmb_v10/LMB_Sl_0]

  # Create port connections
  connect_bd_net -net SYS_Rst_1 [get_bd_pins SYS_Rst] [get_bd_pins dlmb_bram_if_cntlr/LMB_Rst] [get_bd_pins dlmb_v10/SYS_Rst] [get_bd_pins ilmb_bram_if_cntlr/LMB_Rst] [get_bd_pins ilmb_v10/SYS_Rst]
  connect_bd_net -net microblaze_0_Clk [get_bd_pins LMB_Clk] [get_bd_pins dlmb_bram_if_cntlr/LMB_Clk] [get_bd_pins dlmb_v10/LMB_Clk] [get_bd_pins ilmb_bram_if_cntlr/LMB_Clk] [get_bd_pins ilmb_v10/LMB_Clk]

  # Restore current instance
  current_bd_instance $oldCurInst
}

#------------------------------------------------ block design utils

#global variable to save the currently open block design
set current_bd_instance 0

proc start_new_bd {design_name} {

	global current_bd_instance

	set list_projs [get_projects -quiet]
	create_bd_design $design_name

	set parentCell [get_bd_cells /]

	# Get object for parentCell
	set parentObj [get_bd_cells $parentCell]
	if { $parentObj == "" } {
		catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
		return
	}

	# Make sure parentObj is hier blk
	set parentType [get_property TYPE $parentObj]
	if { $parentType ne "hier" } {
		catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
		return
	}

	# Set parent object as current
	current_bd_instance $parentObj
}

proc end_bd {} {

	save_bd_design
 	validate_bd_design -force -verbose
}



