/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 10ns / 1ns
`default_nettype none

module stepper_clk_gen(
		clk_in, 
		reset, 
		step_seq_en, 
		rot_dir, 
		speed, 
		step_seq_index, 
		step_cnt_trg
	);

	// ld-Funktion (dualer Logarithmus, aufgerundet)
	function integer ld(input integer value);
		integer i;
		begin
			for (i = 0; value > 0; i = i + 1)
				value = value >> 1;
			ld = i;
		end
	endfunction

	input 	wire		clk_in, reset, step_seq_en, rot_dir;
	input 	wire 	[17:0]	speed;
	output 	wire	[4:0]	step_seq_index;
	output 	wire		step_cnt_trg;

	reg 			trg;

	reg		[18:0]	speed_clk_cnt_reg;
        wire		[17:0]	stepwidth;

        w_lut_rom WLR (
            .clk(clk_in), 
            .we(1'b0), 
            .addr(speed), 
            .din(18'h0), 
            .dout(stepwidth)
        );

	always @ (posedge clk_in) begin
	    if (reset) begin
	   	speed_clk_cnt_reg <= 0;
		trg <= 1'b0;
    	    end else begin 
      		case (step_seq_en)
        		1'b0: begin
          			speed_clk_cnt_reg <= 0;
				trg <= 1'b0;
        		end
        		1'b1: begin
				if (speed_clk_cnt_reg <=  stepwidth) begin
	            			speed_clk_cnt_reg <= speed_clk_cnt_reg + 1;
					trg	<= 1'b0;
				end else begin
					speed_clk_cnt_reg <= 0;
					trg <= 1'b1;
				end
        		end
      		endcase
    	    end
	end
	
	
	reg [4:0] step_seq_index_reg;
	
	always @ (posedge clk_in or posedge reset) begin
		if (reset) begin
			step_seq_index_reg <= 0;
		end else begin
			if (trg) begin
		  		if (rot_dir) begin
  					step_seq_index_reg <= step_seq_index_reg + 1;
  				end else begin
  		  			step_seq_index_reg <= step_seq_index_reg - 1;
  				end
			end
		end
	end
	
	assign step_seq_index = step_seq_index_reg;
	
	edge_detector EDGEDET (clk_in, reset, step_seq_index[0], step_cnt_trg);

endmodule
