/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`ifndef __GENERIC_MEMORY_V
`define __GENERIC_MEMORY_V

`default_nettype none

module generic_memory #(
    parameter       DATA_WIDTH = 8,
    parameter       ADDR_WIDTH = 8,
    parameter       INIT_FILE = "",
    parameter       OUTPUT_REGISTER = "yes"
)(
    input wire                      clk_i,
    input wire                      we_i,
    input wire  [ADDR_WIDTH-1:0]    addr_i,
    input wire  [DATA_WIDTH-1:0]    data_wr_i,
    output reg  [DATA_WIDTH-1:0]    data_rd_o
);

localparam
    DATA_DEPTH = 2**ADDR_WIDTH;

reg     [DATA_WIDTH-1:0]    mem [0:DATA_DEPTH-1];

//generate 
//if (INIT_FILE != "")
//begin
    initial
    begin
        $readmemh(INIT_FILE, mem, 0, DATA_DEPTH-1);
    end
//end
//endgenerate

always @(posedge clk_i)
begin
    if (we_i)
        mem[addr_i] <= data_wr_i;
end

generate
    if (OUTPUT_REGISTER == "yes")
    begin:SYNC_READ
        always @(posedge clk_i)
        begin
            data_rd_o <= mem[addr_i];
        end
    end else
    begin:ASYNC_READ
        always @(*)
        begin
            data_rd_o <= mem[addr_i];
        end
    end
endgenerate

endmodule

`endif
