/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


///////////////////////////////////////////////////////////////////////////////
// Projekt : SpartanMC
// Dateiname : port_hl.v
// Autor : Siegmar Schoene
// Modul : port_hl
//
// Beschreibung : Port_hl ist ein Port mit konfigurierbarer Breite zur
//                Erzeugung von Signalen mit staendig HIGH oder LOW Pegel.
//
///////////////////////////////////////////////////////////////////////////////

`default_nettype none

module port_hl(
	pin
	);

parameter PEGEL		= 0;		// Bitvektor von 0 und 1 in der Breite von PORT_WIDTH
parameter OD_OUTPUT	= 0;
parameter PORT_WIDTH	= 18;

// inputs & outputs
output wire [PORT_WIDTH-1:0]	pin;		// I/O-Pins

// Pegelzuordnung
wire   [PORT_WIDTH-1:0]	out = PEGEL;

// Generierung der PIN Treiber

genvar  j;
generate
for (j=0; j<PORT_WIDTH; j=j+1)
begin:GEN_IOTBUF
	if	(OD_OUTPUT	== 0)
        	assign  pin[j]  = out[j];
	else
        	assign  pin[j]  = (~out[j])?	1'b0: 1'bz;
        end
endgenerate

endmodule

