/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    16:55:13 12/02/2013 
// Design Name: 
// Module Name:    port_out_expander 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module port_out_expander#(
	parameter BLOCKS = 64,
	parameter BLOCKWIDTH = 8,
	parameter BASE_ADR	= 10'h0,

	parameter DAT_ADR	= 0, 
	parameter BLOCK_ADR = 1,
	
	parameter ALL_PORTS	= 2,
	parameter DECODSIM_DEB	= "NO"
    )
	 (
	input	wire		clk_peri,		//Systemtakt
	input	wire	[17:0]	do_peri,		//Datenbus  vom MC
   output	wire	[17:0]	di_peri,		//Datenbus  zum MC
   input	wire	[9:0]	addr_peri,		//Adressbus vom MC
   input	wire		access_peri,		//Signale fuer Peripheriezugriffe
   input	wire		wr_peri,		//Schreibsignal
		  
	input	wire		reset,			//Reset-Signal
   output wire [BLOCKS*BLOCKWIDTH-1:0] pin,
	output reg strobe = 0
);
	 
	 
	// Dekodierung der Adressen ********************** START *************************************
// chipselect fuer die Statusregister (ohne Modemsignale)
wire					cs_out_data_r;	// Lesen
wire					cs_out_blk_r;	// Lesen
wire					cs_out_data_w;	// Schreiben
wire					cs_out_blk_w;	// Schreiben

wire					select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

reg		[17:0]	blk = BLOCKS;		// Output enable
reg		[17:0]	blk_count = 0;		// Output enable
reg		[BLOCKS*BLOCKWIDTH-1:0]	dat;		// Datenregister
assign pin = dat;

// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);

// chipselect fuer die FIFO-,Status- und Steuerregister
assign	cs_out_data_r	= reg_read[DAT_ADR];	// Lesen
assign	cs_out_blk_r	= reg_read[BLOCK_ADR];	// Lesen
assign	cs_out_data_w	= reg_write[DAT_ADR];	// Schreiben
assign	cs_out_blk_w	= reg_write[BLOCK_ADR];	// Schreiben

// In diesem Block werden Daten vom Bus in die Register uebernommen.


always @ (posedge clk_peri)

if(reset)
	begin
	blk <= BLOCKS;
	blk_count <= 0;
	dat <= 0;
	strobe <= 0;
	end
else
// schreiben der Ports
begin	
strobe <= 0;
if(cs_out_blk_w)				
	blk	<= do_peri;
if(cs_out_data_w) begin
	blk_count <= blk_count +1;
	dat <= { dat[((BLOCKS-1)*BLOCKWIDTH-1):0],do_peri[BLOCKWIDTH-1:0]};	
	if(blk_count +1 == blk) begin
		blk_count <= 0;
		strobe <= 1;
		end
		end
end

// lesen der Ports
assign	di_peri [17:0]	= (cs_out_data_r)?	{1'b0,dat[BLOCKWIDTH-1:0]}:
			  (cs_out_blk_r)?	blk_count:	18'b0;

endmodule
