/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale      1ns / 10ps
`default_nettype none

module pseudo_out(clk, fpga, fpga2, pin);


// Parameter
parameter PORT_WIDTH	= 18;				// Portbreite

// Ein- und Ausgänge
input	wire				clk;
input	wire				fpga2;		// FPGA-Pin
output	wire				fpga;		// FPGA-Pin

input	wire	[PORT_WIDTH-1:0]	pin;		// Eingangspins
wire		[PORT_WIDTH-1:0]	temp;

genvar  i;
generate
for (i=0; i<=PORT_WIDTH; i=i+1) begin: loop
	wire in;
	if(i==0) begin
		(* KEEP_HIERARCHY = "yes" *)
		flip_or flipor(clk,fpga2,pin[i],temp[i]);
	end else if(i== PORT_WIDTH-1) begin
		(* KEEP_HIERARCHY = "yes" *)
		flip_or flipor(clk,temp[i-1],pin[i],temp[i]);
	end else if(i== PORT_WIDTH) begin
		(* KEEP_HIERARCHY = "yes" *)
		flip_or flipor(clk,temp[i-1],1'b1,fpga);
	end else begin
		(* KEEP_HIERARCHY = "yes" *)
		flip_or flipor(clk,temp[i-1],pin[i],temp[i]);
	end
	
	
end

endgenerate

endmodule

module flip_or(C, F_IN, OR_IN, OUT);

input C;
input F_IN;
input OR_IN;
output OUT;

parameter PORT_WIDTH	= 18;

wire C;
wire F_IN;
wire OR_IN;
wire OUT;
wire TEMP;

//assign TEMP = F_IN ^ OR_IN;
(* KEEP_HIERARCHY = "yes" *)
LUT5_L #(.INIT(32'h66666666)) xorinst (.I0(F_IN), .I1(OR_IN), .LO(TEMP)); //xor between the two inputs
(* KEEP_HIERARCHY = "yes" *)
FDRE fdre_inst(.D(TEMP), .C(C),.Q(OUT), .R(), .CE());

endmodule


