/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


//////////////////////////////////////////////////////////////////////////////////
// Create Date:    14:54:03 09/27/2011 
// Module Name:    spmc_uart_light_peri 
//////////////////////////////////////////////////////////////////////////////////
`default_nettype none
module dpr_blackbox_t2(
	input	wire		clk_peri,		//Systemtakt
   input	wire	[17:0]	do_peri,		//Datenbus  vom MC
   output	reg	[17:0]	di_peri = 0,		//Datenbus  zum MC
   input	wire	[9:0]	addr_peri,		//Adressbus vom MC
   input	wire		access_peri,		//Signale fuer Peripheriezugriffe
   input	wire		wr_peri,		//Schreibsignal
		  
	input	wire		reset,			//Reset-Signal
   
	input wire start,
	input wire stop,
	input wire clear,
	
	output wire cnt_fast,
	output wire cnt_med,
	output wire cnt_slow
	);

	parameter
	BASE_ADR		= 10'h0,
	ALL_PORTS		= 5,
	DECODSIM_DEB		= "NO";

	

// Dekodierung der Adressen ********************** START *************************************
// chipselect fuer die Statusregister (ohne Modemsignale)


wire					select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);

// Dekodierung der Adressen ********************** END **************************************
reg		[ALL_PORTS-1:0]		reg_select_d = 0;
reg		[ALL_PORTS-1:0]		reg_read_d = 0;
reg		[ALL_PORTS-1:0]		reg_write_d = 0;

wire 		[89:0] cnt_out;

always@(*) begin 
	case (reg_read)
      5'b00001 : begin di_peri <= cnt_out[17:0];  end
      5'b00010 : begin di_peri <= cnt_out[35:18];  end
      5'b00100 : begin di_peri <= cnt_out[53:36]; end
      5'b01000 : begin di_peri <= cnt_out[71:54]; end
      5'b10000 : begin di_peri <= cnt_out[89:72]; end
      default	: begin di_peri <= 18'h00000; end
		endcase
end



(* box_type = "{primitive|black_box|user_black_box}" *)
blackbox_module_t2 bb_t2 (
    .clk				(clk_peri), 
    .reset			(reset), 
    
	 .start			(start),
	 .stop			(stop),
	 .clear			(clear),			
	 
    	 .cnt_fast		(cnt_fast),
	 .cnt_med		(cnt_med),
	 .cnt_slow		(cnt_slow),

	 .cnt_out 		(cnt_out)
	 );

endmodule
