/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    14:54:03 09/27/2011 
// Module Name:    spmc_uart_light_peri 
//////////////////////////////////////////////////////////////////////////////////
module blackbox_module_t3(
	input	wire		clk,		//Systemtakt
   input	wire		reset,			//Reset-Signal
	
	input wire start,
	input wire stop,
	input wire clear,
	
	output wire cnt_fast,
	output wire cnt_med,
	output wire cnt_slow,
	
	output reg [89:0] cnt_out =0
	);

parameter RST = 2'h0;
parameter CNT = 2'h1;
parameter STP = 2'h2;

reg [89:0] counter = 0;
reg [1:0] state = RST;

assign cnt_fast  = counter[0];
assign cnt_med  = counter[25];
assign cnt_slow  = counter[26];
	
always @(posedge clk)
if (reset) begin
	state <= RST;
	counter <= 0; 
	cnt_out <= 0;
end else begin 
	cnt_out <= counter;
	case (state)
	RST: begin 
		counter <= 0;
		state <= STP;
		end
	CNT: begin
		counter <= counter +1;
		if(stop) state <= STP;
		end
	STP: begin
		counter <= counter;
		if(clear) state <= RST;
		else if(start) state <= CNT;
		end
	default: begin
		counter <= counter;	
		end		
	endcase
	
end
	

endmodule
