/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


///////////////////////////////////////////////////////////////////////////////
//
//  Projekt   : SpartanMC
//  Dateiname : top_rot_ta_vhdl.v
//  Autor     : Siegmar Schoene
//  Modul     : top_rot_ta_vhdl
//
//  Beschreibung : Verilog Kopf fuer VHDL-Interfaces
//
//  Dekodierung fuer Rotation rechts und Rotation links.
//
///////////////////////////////////////////////////////////////////////////////

`default_nettype none

module top_rot_ta_vhdl(
 // Signale fuer jedes Interface
	clk_peri,        
	do_peri,
	di_peri,
	addr_peri,
	access_peri,
	wr_peri,
	reset,
	intr,

// Signale des kongreten Moduls
	rot_a,
	rot_b,
	rot_p
	);


// Der Parameter fuer die Adressen des Moduls wird
// in der system.v durch jConfig eingestellt.

parameter BASE_ADR	= 10'h9;

// inputs & output wires des Systembus
input wire				clk_peri;
input wire	[17:0]			do_peri;	// Datenbus
output wire	[17:0]			di_peri;	// Datenbus
input wire	[9:0]			addr_peri;	// Adressbus
input wire				access_peri;	// CS Peripherie
input wire				wr_peri;	// signalisiert Schreibzugriff Peripherie
input wire				reset;
output wire				intr;		// interrupt flag
wire				select;		// Eine Adresse des Moduls wird angesprochen

// Signale des kongreten Moduls
input wire				rot_a;		// Eingangspins vom Rotations Taster
input wire				rot_b;
input wire				rot_p;



// Adressdekoder, der aus der angelegten Adresse ein chipselect erzeugt	
pselect p1	(
			.addr(		addr_peri[9:3]	),
			.activ_peri(	access_peri	),
			.select(	select		)
		);
defparam p1.ADDR_WIDTH	= 7;
defparam p1.BASE_WIDTH	= 7;
defparam p1.BASE_ADDR	= BASE_ADR >> 3;

// Einbindung des Interface als VHDL Modul

rot_ta_vhdl rot_ta_vhdl	(
	.clk_peri	(clk_peri	),
	.do_peri	(do_peri[17:0]	),
	.di_peri	(di_peri[17:0]	),
	.addr_peri	(addr_peri[2:0]	),
	.access_peri	(access_peri	),
	.wr_peri	(wr_peri	),
	.reset		(reset		),
	.intr		(intr		),
   	.select_copy	(select		),

// Signale des kongreten Moduls
	.rot_a		(rot_a		),
	.rot_b		(rot_b		),
	.rot_p		(rot_p		)
	);


endmodule
