import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement
import org.jgrapht.alg.shortestpath.DijkstraShortestPath
import org.jgrapht.graph.DefaultDirectedGraph
import org.jgrapht.graph.DefaultEdge
/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */



def allRouters = document.findByType(moduleName)
def routingTable = calculateRoutingTable()

allRouters.each {
	def currentRouter = it
	def id = it.getParameter(param_routerID).getValue()
	(0..maxRouters-1).each {
		def portToUse = routingTable[id as Integer][it]
		currentRouter.getParameter("${param_destPrefix}${it}").setValue(portToUse)
	}
}
return "done"

/**
 * Method to collect the routing table with [source][destination] = link to use. Link Definition: In the implementation the self connection (loopback) is port 0 if the module has RX connector and the first TX port is started from 1 on. If AMOUNT_OF_INPUTS is 0 then there is no self connection and the first TX port is numbered with 0.
 *
 * @return a routing table
 */
def calculateRoutingTable() {
	def routingTable = new Integer[maxRouters][maxRouters]; // [src][dst] = connection
	def directedGraph = new DefaultDirectedGraph<String, DefaultEdge>(DefaultEdge.class);
	def routers = findByType(moduleName);
	routers.each {
		def src = it.getParameter(param_routerID).getValue();
		def srcInt = src as Integer;
		def hasReceiver = (it.getParameter(param_amountOfInputs).getValue() as Integer) > 0;
		directedGraph.addVertex(src);
		def port = it.getPort(port_dataout)
		port.streamConnections().each {
			def oppositePort = it.getOppositePort(port)
			//print "portb ${oppositePort.getSourceElement()}"
			if (oppositePort.getSourceElement().getType() == IConfigurationElement.ElementType.Peripheral) {
				def connectionNr = it.getMyStart(port) / defaultDataOutWidth;
				// in the implementation the self connection (loopback) is port 0 if the module has RX connector and the first TX port is started from 1 on. If AMOUNT_OF_INPUTS is 0 then there is no self connection and the first TX port is numbered with 0.
				if (hasReceiver) {
					connectionNr++;
				}
				def oppositeRouter = oppositePort.getSourceElement()
				def dst = oppositeRouter.getParameter(param_routerID).getValue();
				Integer dstInt = dst as Integer;
				//fill graph for dijkstra
				directedGraph.addVertex(dst);
				directedGraph.addEdge(src, dst);
				//fill neighbours into the routing table
				routingTable[srcInt][dstInt] = connectionNr;
			}

		}
	}
	//calculate routingtable (DijkstraShortestPath and convert path to outputnumber)
	def path;
	for (src = 0; src < maxRouters; src++) {
		for (dst = 0; dst < maxRouters; dst++) {
			if (src == dst || !directedGraph.containsVertex(src as String) || !directedGraph.containsVertex(dst as String)) {
				routingTable[src][dst] = 0;
			} else {
				path = DijkstraShortestPath.findPathBetween(directedGraph, src as String, dst as String);
				if (path == null || path.size() == 0) {
					routingTable[src][dst] = 0;
				} else {
					def from = path.get(0).getSource() as Integer;
					def via = path.get(0).getTarget() as Integer;
					def to = dst;
					routingTable[from][to] = routingTable[from][via];
				}
			}
		}
	}
	return routingTable
}
