/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/* 

---OOOO---------OOOO               .//
   O   O--------OOOOO------     .//
-x-OOOO    /----OOOO         .//
 | O  O    |              .//
 | O   O   |           .//
 |         |        .//
 |-OOOO    |     .//        
   OOOO----/  .//          //==\\
---OOOO    .//             ||
        .//           010  \\==\\
     .//           010001      ||
  .//           100010100  \\==//
//                         


Created spring 2016 by Felix Sterzelmaier
TU Darmstadt
---Fachgebiet Rechnersysteme---
License: CC0
https://creativecommons.org/publicdomain/zero/1.0

Thanks to Maximilian Flaig and Kris Heid


Decoder
This module receives the 6-bit binary number of the target 
router the message should be transmitted to (from the selector).
It outputs the information to the splitter which port it should use
(one hot encoded)

If the Router has at least one input, it also has a receiver module. So if
TO_DEST_x=0 => tooutputport=1 => is routed to receiver
TO_DEST_x=1 => tooutputport=2 => is routed to first output port
TO_DEST_x=2 => tooutputport=4 => is routed to second output port

If the Router has at no input, it has no receiver module. So if
TO_DEST_x=0 => tooutputport=1 => is routed to first output port
TO_DEST_x=1 => tooutputport=2 => is routed to second output port
TO_DEST_x=2 => tooutputport=4 => is routed to third output port

*/




`timescale 1ns / 1ps
`default_nettype none

module decoder
#(
    parameter SPLITTER_OUT = 'd3,   //how many output ports has the splitter
    parameter TO_DEST_0 = 0,        //if a message should be passed to router with id 0, which output port should it use
    parameter TO_DEST_1 = 0,        //if a message should be passed to router with id 1, which output port should it use
    parameter TO_DEST_2 = 0,        //if a message should be passed to router with id 2, which output port should it use
    parameter TO_DEST_3 = 0,
    parameter TO_DEST_4 = 0,
    parameter TO_DEST_5 = 0,
    parameter TO_DEST_6 = 0,
    parameter TO_DEST_7 = 0,
    parameter TO_DEST_8 = 0,
    parameter TO_DEST_9 = 0,
    parameter TO_DEST_10 = 0,
    parameter TO_DEST_11 = 0,
    parameter TO_DEST_12 = 0,
    parameter TO_DEST_13 = 0,
    parameter TO_DEST_14 = 0,
    parameter TO_DEST_15 = 0,
    parameter TO_DEST_16 = 0,
    parameter TO_DEST_17 = 0,
    parameter TO_DEST_18 = 0,
    parameter TO_DEST_19 = 0,
    parameter TO_DEST_20 = 0,
    parameter TO_DEST_21 = 0,
    parameter TO_DEST_22 = 0,
    parameter TO_DEST_23 = 0,
    parameter TO_DEST_24 = 0,
    parameter TO_DEST_25 = 0,
    parameter TO_DEST_26 = 0,
    parameter TO_DEST_27 = 0,
    parameter TO_DEST_28 = 0,
    parameter TO_DEST_29 = 0,
    parameter TO_DEST_30 = 0,
    parameter TO_DEST_31 = 0
) 
(
    input   wire [5:0]              torouternr,     //message should be directed to this router
    output  wire [SPLITTER_OUT-1:0] tooutputport    //which output should it use(one hot encoding)
);

wire [5:0] binaryoutputport; //output port binary
assign tooutputport = (1'b1 << binaryoutputport); //output port one hot

//read output port from routingtable
assign binaryoutputport =   (torouternr==0) ? TO_DEST_0[5:0] : 
                            (torouternr==1) ? TO_DEST_1[5:0] : 
                            (torouternr==2) ? TO_DEST_2[5:0] : 
                            (torouternr==3) ? TO_DEST_3[5:0] : 
                            (torouternr==4) ? TO_DEST_4[5:0] : 
                            (torouternr==5) ? TO_DEST_5[5:0] : 
                            (torouternr==6) ? TO_DEST_6[5:0] : 
                            (torouternr==7) ? TO_DEST_7[5:0] : 
                            (torouternr==8) ? TO_DEST_8[5:0] : 
                            (torouternr==9) ? TO_DEST_9[5:0] : 
                            (torouternr==10) ? TO_DEST_10[5:0] : 
                            (torouternr==11) ? TO_DEST_11[5:0] : 
                            (torouternr==12) ? TO_DEST_12[5:0] : 
                            (torouternr==13) ? TO_DEST_13[5:0] : 
                            (torouternr==14) ? TO_DEST_14[5:0] : 
                            (torouternr==15) ? TO_DEST_15[5:0] : 
                            (torouternr==16) ? TO_DEST_16[5:0] : 
                            (torouternr==17) ? TO_DEST_17[5:0] : 
                            (torouternr==18) ? TO_DEST_18[5:0] : 
                            (torouternr==19) ? TO_DEST_19[5:0] : 
                            (torouternr==20) ? TO_DEST_20[5:0] : 
                            (torouternr==21) ? TO_DEST_21[5:0] : 
                            (torouternr==22) ? TO_DEST_22[5:0] : 
                            (torouternr==23) ? TO_DEST_23[5:0] : 
                            (torouternr==24) ? TO_DEST_24[5:0] : 
                            (torouternr==25) ? TO_DEST_25[5:0] : 
                            (torouternr==26) ? TO_DEST_26[5:0] : 
                            (torouternr==27) ? TO_DEST_27[5:0] : 
                            (torouternr==28) ? TO_DEST_28[5:0] : 
                            (torouternr==29) ? TO_DEST_29[5:0] : 
                            (torouternr==30) ? TO_DEST_30[5:0] : 
                            (torouternr==31) ? TO_DEST_31[5:0] : 6'b0;
endmodule
