/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/* 

---OOOO---------OOOO               .//
   O   O--------OOOOO------     .//
-x-OOOO    /----OOOO         .//
 | O  O    |              .//
 | O   O   |           .//
 |         |        .//
 |-OOOO    |     .//        
   OOOO----/  .//          //==\\
---OOOO    .//             ||
        .//           010  \\==\\
     .//           010001      ||
  .//           100010100  \\==//
//                         


Created spring 2016 by Felix Sterzelmaier
TU Darmstadt
---Fachgebiet Rechnersysteme---
License: CC0
https://creativecommons.org/publicdomain/zero/1.0

Thanks to Maximilian Flaig and Kris Heid


Router
This is the topmodule of the router module.
It is connected to the spartanmc core and other routermodules.

It uses the modules selector,decoder,splitter, 
sender(if outputs>0) and receiver(if inputs>0).

Also the wires connecting sender,inputs,selector, decoder, 
splitter,receiver and outputs are combinded.

In some cases in comments appears "input ports" and "output ports"
an input port is: 18 bits datain, 1bit requestin, 2bits returnout   (so wires connected in direction of the sending router)
an output port is: 18 bits dataout, 1bit requestout, 2bits returnin (so wires connected in direction of the receiving router)
see $SPARTANMC_ROOT/src/doc/users-manual/src/router/layout.svg IN1, IN2, OUT1, OUT2
*/


`timescale 1ns / 1ps
`default_nettype none
module router
#(
    //Set by JConfig
    parameter DEPTH_OUTPUT_BUFFER = 'd64,   //buffer size (sender)
    parameter DEPTH_INPUT_BUFFER = 'd64,    //buffer size (receiver)
    parameter AMOUNT_OF_INPUTS = 2,         //amount of input ports
    parameter AMOUNT_OF_OUTPUTS = 2,        //amount of output ports
    parameter ROUTER_ID = 0,                //routerid
    parameter BASE_ADR = 10'h0,             //SPMC Interface base address
    parameter TO_DEST_0 = 0,                //if a message should be passed to router with id 0, which output port should it use
    parameter TO_DEST_1 = 0,                //if a message should be passed to router with id 1, which output port should it use
    parameter TO_DEST_2 = 0,                //if a message should be passed to router with id 2, which output port should it use
    parameter TO_DEST_3 = 0,                //...
    parameter TO_DEST_4 = 0,
    parameter TO_DEST_5 = 0,
    parameter TO_DEST_6 = 0,
    parameter TO_DEST_7 = 0,
    parameter TO_DEST_8 = 0,
    parameter TO_DEST_9 = 0,
    parameter TO_DEST_10 = 0,
    parameter TO_DEST_11 = 0,
    parameter TO_DEST_12 = 0,
    parameter TO_DEST_13 = 0,
    parameter TO_DEST_14 = 0,
    parameter TO_DEST_15 = 0,
    parameter TO_DEST_16 = 0,
    parameter TO_DEST_17 = 0,
    parameter TO_DEST_18 = 0,
    parameter TO_DEST_19 = 0,
    parameter TO_DEST_20 = 0,
    parameter TO_DEST_21 = 0,
    parameter TO_DEST_22 = 0,
    parameter TO_DEST_23 = 0,
    parameter TO_DEST_24 = 0,
    parameter TO_DEST_25 = 0,
    parameter TO_DEST_26 = 0,
    parameter TO_DEST_27 = 0,
    parameter TO_DEST_28 = 0,
    parameter TO_DEST_29 = 0,
    parameter TO_DEST_30 = 0,
    parameter TO_DEST_31 = 0
) 
(
  	//Connection to SPMC
 	input   wire            clk_peri,
	input   wire [17:0]     do_peri,
	output  wire [17:0]     di_peri,
	input   wire [9:0]      addr_peri,
	input   wire            access_peri,
	input   wire            wr_peri,
	input   wire            reset,
  
    //Connections to other router modules; wires connected in direction of sender of data
    input   wire [(18*AMOUNT_OF_INPUTS)-1:0] datain,
    input   wire [(AMOUNT_OF_INPUTS)-1:0] requestin,
    output  wire [2*(AMOUNT_OF_INPUTS)-1:0] returnout,

    //Connections to other router modules; wires connected in direction of receiver of data
    output  wire [(18*AMOUNT_OF_OUTPUTS)-1:0] dataout,
    output  wire [(AMOUNT_OF_OUTPUTS)-1:0] requestout,
    input   wire [2*(AMOUNT_OF_OUTPUTS)-1:0] returnin
);


//------------------------------------
//---------local parameters-----------
//------------------------------------

localparam
    DECODSIM_DEB	    = "NO", //debug?
    ALL_PORTS           = 3,    //number of registers for communication with spmc core (always 3)
    HAS_SENDER          = AMOUNT_OF_OUTPUTS > 0 ? 1 : 0,    //1 if sender is synthesized
    HAS_RECEIVER        = AMOUNT_OF_INPUTS > 0 ? 1 : 0,     //1 if receiver is synthesized
    SELECTOR_IN         = HAS_SENDER ? AMOUNT_OF_INPUTS + 1 : AMOUNT_OF_INPUTS,     //amount of inputs in selector
    SPLITTER_OUT        = HAS_RECEIVER ? AMOUNT_OF_OUTPUTS + 1 : AMOUNT_OF_OUTPUTS; //amounts of outputs from splitter

//------------------------------------
//----------internal wires------------
//------------------------------------

//sender outputs
wire [17:0] sender_selector_dataout;
wire sender_selector_requestout;
wire [1:0] sender_selector_returnin;

//selector inputs
wire [(18*SELECTOR_IN)-1:0]     datain_selector;
wire [SELECTOR_IN-1:0]          requestin_selector;
wire [(2*SELECTOR_IN)-1:0]      returnout_selector;

//selector outputs
wire [17:0] selector_splitter_dataout;
wire selector_splitter_requestout;
wire [1:0] selector_splitter_returnin;
wire [5:0] selector_decoder_torouternr; //6 bits because its the maximum from header format

//decoder outputs
wire [SPLITTER_OUT-1:0] decoder_splitter_tooutputport;

//splitter outputs
wire [(18*SPLITTER_OUT)-1:0]     dataout_splitter;
wire [SPLITTER_OUT-1:0]          requestout_splitter;
wire [(2*SPLITTER_OUT)-1:0]      returnin_splitter;

//receiver inputs
wire [17:0] receiver_datain;
wire receiver_requestin;
wire [1:0] receiver_returnout;


//-------------------------------------
//---generate blocks for wire fields---
//-------------------------------------


//SENDER and INPUTS to SELECTOR
generate
    if(HAS_SENDER == 1) begin
        //selector first port to sender
        assign datain_selector[17:0] = sender_selector_dataout[17:0];
        assign requestin_selector[0] = sender_selector_requestout;
        assign sender_selector_returnin[1:0] = returnout_selector[1:0];
        //other ports to input ports
        genvar i;
        for(i = 0; i < AMOUNT_OF_INPUTS; i = i + 1 ) begin: router_selector_infield_withsender
            assign datain_selector[18+((1+i)*18)-1:18+(i*18)] = datain[((1+i)*18)-1:i*18];
            assign requestin_selector[1+i] = requestin[i];
            assign returnout[((1+i)*2)-1:i*2] = returnout_selector[2+((1+i)*2)-1:2+(i*2)];
        end
    end
    if(HAS_SENDER == 0) begin
        //all ports to input ports
        genvar i;
        for(i = 0; i < AMOUNT_OF_INPUTS; i = i + 1 ) begin: router_selector_infield_withoutsender
            assign datain_selector[((1+i)*18)-1:(i*18)] = datain[((1+i)*18)-1:i*18];
            assign requestin_selector[i] = requestin[i];
            assign returnout[((1+i)*2)-1:i*2] = returnout_selector[((1+i)*2)-1:(i*2)]; //error here
        end
    end
endgenerate

//SPLITTER to RECEIVER and OUTPUTS
generate
    if(HAS_RECEIVER == 1) begin
        //first port to receiver
        assign receiver_datain[17:0] = dataout_splitter[17:0];
        assign receiver_requestin = requestout_splitter[0];
        assign returnin_splitter[1:0] = receiver_returnout[1:0];
        //other ports to output ports
        genvar i;
        for(i = 0; i < AMOUNT_OF_OUTPUTS; i = i + 1 ) begin: router_splitter_outfield_withreceiver
            assign dataout[((1+i)*18)-1:(i*18)] = dataout_splitter[((2+i)*18)-1:(i+1)*18];
            assign requestout[i] = requestout_splitter[i+1];
            assign returnin_splitter[((2+i)*2)-1:(i+1)*2] = returnin[((1+i)*2)-1:(i*2)];
        end
    end
    if(HAS_RECEIVER == 0) begin
        //all ports to output ports
        genvar i;
        for(i = 0; i < AMOUNT_OF_OUTPUTS; i = i + 1 ) begin: router_splitter_outfield_withoutreceiver
            assign dataout[((1+i)*18)-1:(i*18)] = dataout_splitter[((1+i)*18)-1:i*18];
            assign requestout[i] = requestout_splitter[i];
            assign returnin_splitter[((1+i)*2)-1:i*2] = returnin[((1+i)*2)-1:(i*2)];
        end
    end
endgenerate

//-------------------------------------
//-------------submodules--------------
//-------------------------------------

generate
    //Sender
	if(HAS_SENDER == 1) begin
        sender # (
            .BUFFER (       DEPTH_INPUT_BUFFER  ),
            .REG_COUNT(	    ALL_PORTS		    ),
            .BASE_ADR(	    BASE_ADR		    ),
            .SIM_DEBUG(	    DECODSIM_DEB	    ),
            .HAS_RECEIVER(  HAS_RECEIVER        ),
            .ROUTER_ID(     ROUTER_ID           )
        )
        sender1 (
            .reset(			reset		),
            .clk_peri(		clk_peri	),
            .addr_peri(		addr_peri	),
            .access_peri(	access_peri	),
            .wr_peri(		wr_peri		),
            .do_peri(       do_peri     ),
            .di_peri(       di_peri     ),
            .dataout(       sender_selector_dataout     ),
            .requestout(    sender_selector_requestout  ),
            .returnin(      sender_selector_returnin    )
        );
    end
    
    //Receiver
	if(HAS_RECEIVER == 1) begin
        receiver # (
            .BUFFER (       DEPTH_OUTPUT_BUFFER ),
            .REG_COUNT(	    ALL_PORTS		    ),
            .BASE_ADR(	    BASE_ADR		    ),
            .SIM_DEBUG(	    DECODSIM_DEB	    ),
            .HAS_SENDER(    HAS_SENDER          )
        )
        receiver1 (
            .reset(			reset		        ),
            .clk_peri(		clk_peri	        ),
            .addr_peri(		addr_peri	        ),
            .access_peri(	access_peri	        ),
            .wr_peri(		wr_peri		        ),
            .do_peri(       do_peri             ),
            .di_peri(       di_peri             ),
            .datain(        receiver_datain     ),
            .requestin(     receiver_requestin  ),
            .returnout(     receiver_returnout  )
        );
    end
endgenerate


//SELECTOR
selector # (
    .SELECTOR_IN(   SELECTOR_IN)
)
selector1 (
    .reset(			reset		),
    .clk_peri(		clk_peri	),
    .dataout(       selector_splitter_dataout),
    .requestout(    selector_splitter_requestout),
    .returnin(      selector_splitter_returnin),
    .routernr(      selector_decoder_torouternr),
    .datain(        datain_selector),
    .requestin(     requestin_selector),
    .returnout(     returnout_selector)
);

//DECODER
decoder # (
    .SPLITTER_OUT   (SPLITTER_OUT),
     .TO_DEST_0     (TO_DEST_0),
     .TO_DEST_1     (TO_DEST_1),
     .TO_DEST_2     (TO_DEST_2),
     .TO_DEST_3     (TO_DEST_3),
     .TO_DEST_4     (TO_DEST_4),
     .TO_DEST_5     (TO_DEST_5),
     .TO_DEST_6     (TO_DEST_6),
     .TO_DEST_7     (TO_DEST_7),
     .TO_DEST_8     (TO_DEST_8),
     .TO_DEST_9     (TO_DEST_9),
     .TO_DEST_10    (TO_DEST_10),
     .TO_DEST_11    (TO_DEST_11),
     .TO_DEST_12    (TO_DEST_12),
     .TO_DEST_13    (TO_DEST_13),
     .TO_DEST_14    (TO_DEST_14),
     .TO_DEST_15    (TO_DEST_15),
     .TO_DEST_16    (TO_DEST_16),
     .TO_DEST_17    (TO_DEST_17),
     .TO_DEST_18    (TO_DEST_18),
     .TO_DEST_19    (TO_DEST_19),
     .TO_DEST_20    (TO_DEST_20),
     .TO_DEST_21    (TO_DEST_21),
     .TO_DEST_22    (TO_DEST_22),
     .TO_DEST_23    (TO_DEST_23),
     .TO_DEST_24    (TO_DEST_24),
     .TO_DEST_25    (TO_DEST_25),
     .TO_DEST_26    (TO_DEST_26),
     .TO_DEST_27    (TO_DEST_27),
     .TO_DEST_28    (TO_DEST_28),
     .TO_DEST_29    (TO_DEST_29),
     .TO_DEST_30    (TO_DEST_30),
     .TO_DEST_31    (TO_DEST_31)
)
decoder1 (
    .torouternr(    selector_decoder_torouternr),
    .tooutputport(  decoder_splitter_tooutputport)
);

//SPLITTER
splitter # (
    .SPLITTER_OUT(  SPLITTER_OUT)
)
splitter1 (
    .datain(        selector_splitter_dataout),
    .requestin(     selector_splitter_requestout),
    .returnout(     selector_splitter_returnin),
    .tooutputport(  decoder_splitter_tooutputport),
    .dataout(       dataout_splitter),
    .requestout(    requestout_splitter),
    .returnin(      returnin_splitter)
);

endmodule
