/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/* 

---OOOO---------OOOO               .//
   O   O--------OOOOO------     .//
-x-OOOO    /----OOOO         .//
 | O  O    |              .//
 | O   O   |           .//
 |         |        .//
 |-OOOO    |     .//        
   OOOO----/  .//          //==\\
---OOOO    .//             ||
        .//           010  \\==\\
     .//           010001      ||
  .//           100010100  \\==//
//                         


Created spring 2016 by Felix Sterzelmaier
TU Darmstadt
---Fachgebiet Rechnersysteme---
License: CC0
https://creativecommons.org/publicdomain/zero/1.0

Thanks to Maximilian Flaig and Kris Heid


Splitter
This module connects the three internal signals data,request,return from
selector to the receiver or one of the outputs. Which of the outputs/receiver is used
is specified by the onehot-encoded signal "tooutputport" from decoder.

Data and request are connected from sender to receiver, while return is connected from receiver to sender!

If the Router has at least one input, it also has a receiver module. So if
tooutputport=1 => is routet to receiver
tooutputport=2 => is routet to first output port
tooutputport=4 => is routet to second output port

If the Router has at no input, it has no receiver module. So if
tooutputport=1 => is routet to first output port
tooutputport=2 => is routet to second output port
tooutputport=4 => is routet to third output port

The first 18bits of dataout, the first bit of requestout and the 
first 2bits of returnin have to be connected to the same router(or the receiver module)

The second 18bits of dataout, the second bit of requestout and the 
second 2bits of returnin have to be connected to the same router

etc...

*/


`timescale 1ns / 1ps
`default_nettype none

module splitter
#(
    parameter SPLITTER_OUT = 'd3 //amount of outputs
) 
(
    //wires connected in direction of sender of data (selector)
    input   wire [17:0]                     datain,
    input   wire                            requestin,
    output  wire [1:0]                      returnout,
    
    input   wire [SPLITTER_OUT-1:0]         tooutputport, //which port should be used? one-hot-encoded. From decoder.
    
    //wires connected in direction of receiver of data
    output  wire [(18*SPLITTER_OUT)-1:0]    dataout,
    output  wire [SPLITTER_OUT-1:0]         requestout,
    input   wire [(2*SPLITTER_OUT)-1:0]     returnin
);


generate
    genvar i;
    for(i = 0; i < SPLITTER_OUT; i = i + 1 ) begin: splitter_outfield
        assign dataout[((1+i)*18)-1:i*18]   = tooutputport[i] ? datain[17:0] : 18'b0; //dataout[i] is datain, or 0
        assign requestout[i]                = tooutputport[i] ? requestin : 1'b0; //requestout[i] is requestin, or 0
        assign returnout                    = tooutputport[i] ? returnin[((1+i)*2)-1:i*2] : 2'bz; //returnout is returnin[i], or z. z is needed, to not have multiple drivers.
    end
endgenerate

endmodule
