/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`default_nettype none

module fifo_throttle
#(
  parameter OFF_CLK_CNT = 100,
  parameter OFF_WORD_CNT = 1
)(
  //general interface
  input  wire clk_i,
  input  wire rst_i,

  //shi interface
  input  wire rx_en_i,
  output wire rx_rdy_o,
  
  //fifo interface
  input  wire fifo_full_i,
  output wire fifo_we_o
);

  `include "common/functions/bitwidth.vh"

    parameter LP_OFF_CLK_CNT_WIDTH = bitwidth(OFF_CLK_CNT);
    parameter LP_OFF_WORD_CNT_WIDTH= bitwidth(OFF_WORD_CNT);
  generate
  if(OFF_CLK_CNT > 0) begin: WITH_FLATRATE_CONTROLL
    reg [LP_OFF_CLK_CNT_WIDTH-1:0]  r_off_clk_cnt;
    reg [LP_OFF_WORD_CNT_WIDTH-1:0] r_off_word_cnt;

    assign fifo_we_o = (r_off_clk_cnt == {(LP_OFF_CLK_CNT_WIDTH){1'b0}});
    assign rx_rdy_o  = (fifo_we_o & (~fifo_full_i));
	 
    always @(posedge clk_i, posedge rst_i) begin
      if(rst_i) begin
        r_off_clk_cnt <= {(LP_OFF_CLK_CNT_WIDTH){1'b0}};
        r_off_word_cnt<= {(LP_OFF_WORD_CNT_WIDTH){1'b0}};
      end
      else if(!fifo_we_o) begin
        r_off_clk_cnt <= r_off_clk_cnt - 1'b1;
      end
      else if(rx_en_i && rx_rdy_o) begin
        if(r_off_word_cnt == (OFF_WORD_CNT - 1)) begin
          r_off_word_cnt<= {(LP_OFF_WORD_CNT_WIDTH){1'b0}};
          r_off_clk_cnt <= OFF_CLK_CNT;
        end
        else begin
          r_off_word_cnt<= r_off_word_cnt + 1'b1;
        end
      end
    end

  end else begin: WITHOUT_FLATRATE_CONTROLL
    assign rx_rdy_o  = ~fifo_full_i;
    assign fifo_we_o = rx_en_i;
  end
  endgenerate

endmodule
