/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module Crc32Generator(
  input  wire Rst_i,
  input  wire Clk_i,
  input  wire En_i,
  input  wire Clr_i,
  input  wire [7:0]  Input_Byte_i,
  output wire [7:0]  Crc_Byte_o
);
  
  `include "common/functions/Crc32.vh"
  reg  [31:0] r_crc_sum;
  wire [7:0]  w_crc_byte0;
  wire [7:0]  w_crc_byte1;
  wire [7:0]  w_crc_byte2;
  wire [7:0]  w_crc_byte3;
  reg  [3:0]  r_byte_select;
  
  genvar i;
  
  generate
  for(i = 0;i<8;i = i +1) begin:G1
    assign w_crc_byte0[i] = !r_crc_sum[31-i];
  end
  
  for(i = 0;i<8;i = i +1) begin:G2
    assign w_crc_byte1[i] = !r_crc_sum[23-i];
  end

  for(i = 0;i<8;i = i +1) begin:G3
    assign w_crc_byte2[i] = !r_crc_sum[15-i];
  end

  for(i = 0;i<8;i = i +1) begin:G4
    assign w_crc_byte3[i] = !r_crc_sum[7-i];
  end
  endgenerate
  
  assign Crc_Byte_o = r_byte_select[0] ? w_crc_byte0 : 
                     (r_byte_select[1] ? w_crc_byte1 : 
		     (r_byte_select[2] ? w_crc_byte2 : 
		     (r_byte_select[3] ? w_crc_byte3 : 8'b0)));

  always @(posedge Clk_i, posedge Rst_i) begin
    if(Rst_i) begin
      r_byte_select <= 4'b0;
    end
    else if(En_i && (r_byte_select == 4'b0)) begin
      r_byte_select <= 4'b0001;
    end
    else if(!En_i && (r_byte_select != 4'b0)) begin
      r_byte_select <= {r_byte_select[2:0], 1'b0};
    end
  end
  
  always @(posedge Clk_i, posedge Rst_i) begin
    if(Rst_i) begin
      r_crc_sum <= 32'hFF_FF_FF_FF;
    end
    else if(En_i) begin
      r_crc_sum <= Crc32(r_crc_sum, Input_Byte_i);
    end
    else if(Clr_i) begin
      r_crc_sum <= 32'hFF_FF_FF_FF;
    end 
  end

endmodule
