/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`default_nettype none

module FrameHeadGenerator
#(
  parameter   p_mac_addr_d    = "00:0a:35:01:fd:dc",//48'h00_0a_35_01_fd_dc,
  parameter   p_mac_addr_s    = "00:0a:35:01:fd:dc",//48'h00_0a_35_01_fd_dc,
  parameter   p_tpid          = 16'h81_00,
  parameter   p_tci           = 16'h45_55,
  parameter   p_type          = 16'h08_00,   
  parameter   p_tx_word_width = 'd8
)(
  input  wire Rst_i,
  input  wire Clk_i,
  input  wire En_i,
  input  wire Clr_i,
  output wire [p_tx_word_width-1:0] Head_Data_o  
);
  
  `include "common/functions/bitwidth.vh"
  `include "shi/gmii/util/mac2int.vh"

  localparam lp_pre         = 8'h55;
  localparam lp_sfd         = 8'hd5; 
  localparam lp_mac_addr_d5 = mac2int((p_mac_addr_d >> 5*24) & 16'hffff);
  localparam lp_mac_addr_d4 = mac2int((p_mac_addr_d >> 4*24) & 16'hffff);
  localparam lp_mac_addr_d3 = mac2int((p_mac_addr_d >> 3*24) & 16'hffff);
  localparam lp_mac_addr_d2 = mac2int((p_mac_addr_d >> 2*24) & 16'hffff);
  localparam lp_mac_addr_d1 = mac2int((p_mac_addr_d >> 1*24) & 16'hffff);
  localparam lp_mac_addr_d0 = mac2int(p_mac_addr_d & 16'hffff);
  localparam lp_mac_addr_s5 = mac2int((p_mac_addr_s >> 5*24) & 16'hffff);
  localparam lp_mac_addr_s4 = mac2int((p_mac_addr_s >> 4*24) & 16'hffff);
  localparam lp_mac_addr_s3 = mac2int((p_mac_addr_s >> 3*24) & 16'hffff);
  localparam lp_mac_addr_s2 = mac2int((p_mac_addr_s >> 2*24) & 16'hffff);
  localparam lp_mac_addr_s1 = mac2int((p_mac_addr_s >> 1*24) & 16'hffff);
  localparam lp_mac_addr_s0 = mac2int(p_mac_addr_s & 16'hffff);
  localparam lp_tpid1       = (p_tpid >>> 8) & 8'hff;
  localparam lp_tpid0       = (p_tpid & 8'hff);
  localparam lp_tci1        = (p_tci  >>> 8) & 8'hff;
  localparam lp_tci0        = (p_tci  & 8'hff);
  localparam lp_type1       = (p_type >>> 8) & 8'hff;
  localparam lp_type0       = (p_type & 8'hff);
  localparam lp_head_size  = 'd26;
  reg [bitwidth(lp_head_size)-1:0] counter;
  reg [p_tx_word_width-1:0] data;
  
  assign Head_Data_o = data;
  
  always @(posedge Clk_i, posedge Rst_i) begin
    if(Rst_i) begin
      data    <= lp_pre;
      counter <= 1;		
    end
    else if(En_i) begin
      counter <= counter + 1'b1;
      
      case(counter)
        //SFD
        'd7: data<=lp_sfd;
        //Zieladresse
        'd8:  data<=lp_mac_addr_d5;'d9:  data<=lp_mac_addr_d4;'d10: data<=lp_mac_addr_d3;
        'd11: data<=lp_mac_addr_d2;'d12: data<=lp_mac_addr_d1;'d13: data<=lp_mac_addr_d0;
	//Quelladresse
        'd14: data<=lp_mac_addr_s5;'d15: data<=lp_mac_addr_s4;'d16: data<=lp_mac_addr_s3;
        'd17: data<=lp_mac_addr_s2;'d18: data<=lp_mac_addr_s1;'d19: data<=lp_mac_addr_s0;
        //TPID
        'd20: data<=lp_tpid1;'d21: data<=lp_tpid0;
        //tci
        'd22: data<=lp_tci1;'d23: data<=lp_tci0;
        //tpye
        'd24: data<=lp_type1;'d25: data<=lp_type0;
      endcase
    end
    else if(Clr_i)begin
      data    <= lp_pre;
      counter <= 1;
    end
  end
  
endmodule
