/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module GmiiControllerCtrl
(
  input  wire Rst_i,
  input  wire Clk_i,

  //interface to Dp
  output reg  Send_Head_o,
  output reg  Send_Data_o,
  output reg  Send_Length_o,
  output reg  Send_Crc_o,
  output reg  Wait_o,
  output reg  Clr_o,  
  input  wire Data_Ready_i, 
  input  wire Head_Done_i,
  input  wire Data_Done_i,
  input  wire Length_Done_i,
  input  wire Crc_Done_i,
  input  wire Wait_Done_i
);

localparam lp_state_cnt      = 'd6,
           lp_idle           = 6'b000001,
           lp_sending_head   = 6'b000010,
           lp_sending_data   = 6'b000100,
           lp_sending_length = 6'b001000,
           lp_sending_crc    = 6'b010000,
           lp_sending_gap    = 6'b100000;

  reg [lp_state_cnt-1:0] r_state_cur, r_state_next;
  
  always @(r_state_cur, Data_Ready_i, Head_Done_i, Data_Done_i, Length_Done_i, Crc_Done_i, Wait_Done_i) begin
    Send_Head_o   = 1'b0;
    Send_Data_o   = 1'b0;
    Send_Length_o = 1'b0;
    Send_Crc_o    = 1'b0;
    Wait_o        = 1'b0;
    Clr_o         = 1'b0;
    r_state_next  = lp_idle;
	 
    case(r_state_cur)
      lp_idle:
        if(Data_Ready_i) begin
          Send_Head_o  = 1'b1;
          r_state_next = lp_sending_head;
        end
        else begin
//          Clr_o         = 1'b1;
          r_state_next  = lp_idle;
        end
      lp_sending_head:
        if(Head_Done_i) begin
          Send_Data_o  = 1'b1;
          r_state_next = lp_sending_data;
        end
        else begin
          Send_Head_o  = 1'b1;
          r_state_next = lp_sending_head;
        end
      lp_sending_data:
        if(Data_Done_i) begin
          Send_Length_o = 1'b1;
          r_state_next  = lp_sending_length;
        end
        else begin
          Send_Data_o   = 1'b1;
          r_state_next  = lp_sending_data;
        end
      lp_sending_length:
        if(Length_Done_i) begin
          Send_Crc_o   = 1'b1;
          r_state_next = lp_sending_crc;
        end
        else begin
          Send_Length_o = 1'b1;
          r_state_next  = lp_sending_length;
        end
      lp_sending_crc:
        if(Crc_Done_i) begin
          Wait_o       = 1'b1;
          r_state_next = lp_sending_gap;
        end
        else begin
          Send_Crc_o    = 1'b1;
          r_state_next  = lp_sending_crc;
        end
      lp_sending_gap:
        if(Wait_Done_i) begin
          Clr_o        = 1'b1;
          r_state_next = lp_idle;
        end
        else begin
          Wait_o        = 1'b1;
          r_state_next  = lp_sending_gap;
        end
      default:
        r_state_next = lp_idle;
    endcase
  end
  
  always @(posedge Clk_i, posedge Rst_i) begin
    if(Rst_i)
      r_state_cur <= lp_idle;
    else 
      r_state_cur <= r_state_next;
  end

endmodule
