/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module GmiiControllerTop
#( 
  parameter   P_MAC_ADDR_D    = "00:0a:35:01:fd:dc",
  parameter   P_MAC_ADDR_S    = "00:0a:35:01:fd:dc",
  parameter   P_TPID          = 16'h81_00,
  parameter   P_TCI           = 16'h45_55,
  parameter   P_TYPE          = 16'h08_00,   
  parameter   P_TX_WORD_WIDTH = 'd8,
  parameter   P_DP_SIZE       = 'd1498 //Die Größe eines Datenpakets. Min:44; Max:1498, da die letzten zwei bytes genutzt werden, als Length-Field. 
)(
  input  wire Rst_i,
  input  wire Clk_i,

  //interface to FIFO
  input  wire [P_TX_WORD_WIDTH-1:0] Rx_Data_i,
  input  wire Rx_En_i,
  output wire Rx_Rdy_o,

  //interface to PHY 
  output wire [P_TX_WORD_WIDTH-1:0] Tx_Data_o,
  output wire Tx_Enable_o,
  output wire Tx_Error_o,
  output wire Tx_Rst_o//ähnlich dem locked-signal 
);
  wire w_send_head, w_send_data, w_send_length, w_send_crc, w_wait, w_clr, w_data_ready, w_head_done, w_data_done, w_length_done, w_crc_done, w_wait_done;

  GmiiControllerCtrl
  gmii_ctrl0(
    .Rst_i(Rst_i),
    .Clk_i(Clk_i),

    //interface to Dp
    .Send_Head_o(w_send_head),
    .Send_Data_o(w_send_data),
    .Send_Length_o(w_send_length),
    .Send_Crc_o(w_send_crc),
    .Wait_o(w_wait),//96 Bits Gap zwischen zwei Paketen notwendig!
    .Clr_o(w_clr),  
    .Data_Ready_i(w_data_ready), 
    .Head_Done_i(w_head_done),
    .Data_Done_i(w_data_done),
    .Length_Done_i(w_length_done),
    .Crc_Done_i(w_crc_done),
    .Wait_Done_i(w_wait_done)
  );
  
  GmiiControllerDp#(
    .p_mac_addr_d(P_MAC_ADDR_D),
    .p_mac_addr_s(P_MAC_ADDR_S),
    .p_tpid(P_TPID),
    .p_tci(P_TCI),
    .p_type(P_TYPE),   
    .p_tx_word_width(P_TX_WORD_WIDTH),
    .p_dp_size(P_DP_SIZE)
  )
  gmii_dp0 (
    .Rst_i(Rst_i),
    .Clk_i(Clk_i),

    //interface to FIFO
    .Rx_Data_i(Rx_Data_i),
    .Rx_En_i(Rx_En_i),
    .Rx_Rdy_o(Rx_Rdy_o),

    //interface to PHY 
    .Tx_Data_o(Tx_Data_o),
    .Tx_Enable_o(Tx_Enable_o),
    .Tx_Error_o(Tx_Error_o),
    .Tx_Rst_o(Tx_Rst_o),

    //interface to Dp
    .Send_Head_i(w_send_head),
    .Send_Data_i(w_send_data),
    .Send_Length_i(w_send_length),
    .Send_Crc_i(w_send_crc),
    .Wait_i(w_wait),
    .Clr_i(w_clr),  
    .Data_Ready_o(w_data_ready), 
    .Head_Done_o(w_head_done),
    .Data_Done_o(w_data_done),
    .Length_Done_o(w_length_done),
    .Crc_Done_o(w_crc_done),
    .Wait_Done_o(w_wait_done)
  );

endmodule
