/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns/1ps
`default_nettype none

`ifndef __shi_mux
`define __shi_mux

module shi_mux #(
    parameter DATA_WIDTH = 18,
    parameter INPUT_PORTS = 2
)(
    input wire [bitwidth(INPUT_PORTS-1)-1:0]    sel_i,

    output wire [DATA_WIDTH-1:0]                tx_data_o,
    output wire                                 tx_valid_o,
    input wire                                  tx_en_i,
    input wire [(INPUT_PORTS * DATA_WIDTH)-1:0] rx_data_i,
    input wire [INPUT_PORTS-1:0]                rx_en_i,
    output wire [INPUT_PORTS-1:0]               rx_rdy_o);

`include "common/functions/bitwidth.vh"

genvar i;

wire [DATA_WIDTH-1:0] input_int [0:INPUT_PORTS-1];

assign
    tx_data_o = input_int[sel_i],
    tx_valid_o = rx_en_i[sel_i];

generate
    for (i = 0; i < INPUT_PORTS; i = i + 1) begin : MUXPORTS
        assign
            rx_rdy_o[i]  = (i == sel_i) ? tx_en_i : 1'b0,
            input_int[i] = rx_data_i[((i + 1) * DATA_WIDTH) -1:(i * DATA_WIDTH)];
    end
endgenerate

endmodule

`endif
