/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    09:17:01 06/11/2012 
// Module Name:    shi_uart_baudgen
//////////////////////////////////////////////////////////////////////////////////
module shi_uart_baudgen
#(
  parameter BAUDRATE_DIVISOR = 2
)(
  input  wire reset_i,
  input  wire clk_i,
  output wire enable_o
);
  `include "shi/uart/util/uart_functions.vh"

  localparam 
  SYNC_BYTE_NUM      = 3,
  BRD_SIZE           = log2(BAUDRATE_DIVISOR - 1),
  SYNC_BYTE_CNT_SIZE = log2(SYNC_BYTE_NUM  * BAUDRATE_DIVISOR * 10);

  wire w_enable;
  reg [SYNC_BYTE_CNT_SIZE-1:0] r_sync_byte_cnt;
  reg [BRD_SIZE-1:0] r_divisor;
  
  assign enable_o = (r_divisor == BAUDRATE_DIVISOR/2);
  assign w_enable = (r_sync_byte_cnt == (SYNC_BYTE_NUM * BAUDRATE_DIVISOR * 10));
  
  always @(posedge clk_i, posedge reset_i) begin
    if(reset_i) begin
      r_divisor       <= {(BRD_SIZE){1'b0}};
      r_sync_byte_cnt <= {(SYNC_BYTE_CNT_SIZE){1'b0}};
    end
    else if(w_enable) begin
      if(r_divisor == (BAUDRATE_DIVISOR - 1'b1))
        r_divisor <= {(BRD_SIZE){1'b0}};
      else
        r_divisor <= r_divisor + 1'b1;
    end
    else begin
      r_sync_byte_cnt <= r_sync_byte_cnt + 1'b1;
    end
  end
  
endmodule
