/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none
/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Internal Setup Engine                                      ////
////                                                             ////
////                                                             ////
////  Author: Rudolf Usselmann                                   ////
////          rudi@asics.ws                                      ////
////                                                             ////
////                                                             ////
////  Based on: http://www.opencores.org/cores/usb1_funct/       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`include "shi/usb/src/usb1_defines.vh"

module usb1_ctrl (	clk, rst,
               		ctrl_setup, ctrl_in, ctrl_out,
               		send_stall, 
			funct_adr, configured, halt,
			hdr0, hdr1, hdr2, hdr3, hdr4, hdr5, hdr6, hdr7,
		 	hdr_done,
		 	rom_start_d,
			ctrl_size,
            addressed
		);

input wire		clk, rst;
input wire		ctrl_setup;
input wire		ctrl_in;
input wire		ctrl_out;

output reg		send_stall;
output reg	[6:0]	funct_adr;
output reg		configured, halt;
output reg      addressed;

input wire	[7:0]	hdr0, hdr1, hdr2, hdr3, hdr4, hdr5, hdr6, hdr7;
input wire		hdr_done;

output reg	[9:0]	rom_start_d;
output reg	[6:0]	ctrl_size;



///////////////////////////////////////////////////////////////////
//
// Local Wires and Registers
//

parameter	IDLE		=	20'b0000_0000_0000_0000_0001,
		GET_HDR		=	20'b0000_0000_0000_0000_0010,
		GET_STATUS_S	=	20'b0000_0000_0000_0000_0100,
		CLEAR_FEATURE_S	=	20'b0000_0000_0000_0000_1000,
		SET_FEATURE_S	=	20'b0000_0000_0000_0001_0000,
		SET_ADDRESS_S	=	20'b0000_0000_0000_0010_0000,
		GET_DESCRIPTOR_S=	20'b0000_0000_0000_0100_0000,
		SET_DESCRIPTOR_S=	20'b0000_0000_0000_1000_0000,
		GET_CONFIG_S	=	20'b0000_0000_0001_0000_0000,
		SET_CONFIG_S	=	20'b0000_0000_0010_0000_0000,
		GET_INTERFACE_S	=	20'b0000_0000_0100_0000_0000,
		SET_INTERFACE_S	=	20'b0000_0000_1000_0000_0000,
		SYNCH_FRAME_S	=	20'b0000_0001_0000_0000_0000,
		WAIT_IN_DATA	=	20'b0000_0010_0000_0000_0000,
		STATUS_IN	=	20'b0000_0100_0000_0000_0000,
		STATUS_OUT	=	20'b0000_1000_0000_0000_0000,
		V_SET_INT_S	=	20'b0001_0000_0000_0000_0000,
		V_GET_STATUS_S	=	20'b0010_0000_0000_0000_0000;

wire	[7:0]	bmReqType, bRequest;
wire	[15:0]	wValue, wLength;
wire	[1:0]	bm_req_type;
wire	[4:0]	bm_req_recp;

reg		get_status, clear_feature, set_feature, set_address;
reg		get_descriptor, set_descriptor, get_config, set_config;
reg		get_interface, set_interface, synch_frame;
reg		hdr_done_r, config_err;
reg		v_set_int, v_set_feature, v_get_status;

reg	[19:0]	state, next_state;
reg	[6:0]	funct_adr_tmp;
reg		set_adr_pending;

reg		in_size_0;
reg		in_size_1;
reg		in_size_2;
wire		high_sel;
reg		write_done;

///////////////////////////////////////////////////////////////////
//
// Current States
//
wire	clr_halt;
wire	set_halt=0;	// FIX_ME

// For this implementation we do not implement HALT for the
// device nor for any of the endpoints. This is useless for
// this device, but can be added here later ...
// FYI, we report device/endpoint errors via interrupts,
// instead of halting the entire or part of the device, much
// nicer for non-critical errors.

assign clr_halt = ctrl_setup;

always @(posedge clk)
	if(!rst)	addressed <= #1 1'b0;
	else
	if(set_address)	addressed <= #1 1'b1;

always @(posedge clk)
	if(!rst)	configured <= #1 1'b0;
	else
	if(set_config)	configured <= #1 1'b1;

always @(posedge clk)
	if(!rst)	halt <= #1 1'b0;
	else
	if(clr_halt)	halt <= #1 1'b0;
	else
	if(set_halt)	halt <= #1 1'b1;

///////////////////////////////////////////////////////////////////
//
// Descriptor ROM
//
reg		rom_sel;
wire		rom_done;
reg	[6:0]	rom_size;
reg	[6:0]	rom_size_dd;
wire	[6:0]	rom_size_d;

always @(wValue)
	case(wValue[11:8])		// synopsys parallel_case
	   4'h1:	rom_start_d = `ROM_START0;
	   4'h2:	rom_start_d = `ROM_START1;
	   4'h3:
		case(wValue[3:0])	// synopsys parallel_case
		   4'h0:	rom_start_d = `ROM_START2A;
		   4'h1:	rom_start_d = `ROM_START2B;
		   4'h2:	rom_start_d = `ROM_START2C;
		   4'h3:	rom_start_d = `ROM_START2D;
		   default:	rom_start_d = `ROM_START2A;
		endcase
	   default:	rom_start_d = 7'h00;
	endcase

always @(wValue)
	case(wValue[11:8])		// synopsys parallel_case
	   4'h1:	rom_size_dd = `ROM_SIZE0;
	   4'h2:	rom_size_dd = `ROM_SIZE1;
	   4'h3:
		case(wValue[3:0])	// synopsys parallel_case
		   4'h0:	rom_size_dd = `ROM_SIZE2A;
		   4'h1:	rom_size_dd = `ROM_SIZE2B;
		   4'h2:	rom_size_dd = `ROM_SIZE2C;
		   4'h3:	rom_size_dd = `ROM_SIZE2D;
		   default:	rom_size_dd = `ROM_SIZE2A;
		endcase
	   default:	rom_size_dd = 7'h01;
	endcase

assign rom_size_d = (rom_size_dd > wLength[6:0]) ? wLength[6:0] : rom_size_dd;

assign rom_done = 1'b1;

///////////////////////////////////////////////////////////////////
//
// Send Data to Host
//

parameter	ZERO_DATA		=	5'b00001,
		ZERO_ONE_DATA		=	5'b00010,
		CONFIG_DATA		=	5'b00100,
		SYNC_FRAME_DATA		=	5'b01000,
		VEND_DATA		=	5'b10000;


always @(posedge clk)
	if(in_size_0)	ctrl_size <= #1 7'h0;
	else
	if(in_size_1)	ctrl_size <= #1 7'h1;
	else
	if(in_size_2)	ctrl_size <= #1 7'h2;
	else
	if(rom_sel)	ctrl_size <= #1 rom_size_d;


///////////////////////////////////////////////////////////////////
//
// Decode Header
//

// Valid bRequest Codes
parameter	GET_STATUS	=	8'h00,
		CLEAR_FEATURE	=	8'h01,
		SET_FEATURE	=	8'h03,
		SET_ADDRESS	=	8'h05,
		GET_DESCRIPTOR	=	8'h06,
		SET_DESCRIPTOR	=	8'h07,
		GET_CONFIG	=	8'h08,
		SET_CONFIG	=	8'h09,
		GET_INTERFACE	=	8'h0a,
		SET_INTERFACE	=	8'h0b,
		SYNCH_FRAME	=	8'h0c;

parameter	V_SET_INT	=	8'h0f;
		

assign bmReqType	= hdr0;
assign bm_req_type	= bmReqType[6:5];	// 0-standard; 1-class; 
						// 2-vendor; 3-RESERVED
assign bm_req_recp = bmReqType[4:0];		// 0-device; 1-interface;
						// 2-endpoint; 3-other
						// 4..31-reserved
assign bRequest =  hdr1;
assign wValue   = {hdr3, hdr2};
assign wLength  = {hdr7, hdr6};

always @(posedge clk)
	hdr_done_r <= #1 hdr_done;

// Standard commands that MUST support
always @(posedge clk)
	get_status <= #1 hdr_done & (bRequest == GET_STATUS) &
			(bm_req_type==2'h0);

always @(posedge clk)
	clear_feature <= #1 hdr_done & (bRequest == CLEAR_FEATURE) &
				(bm_req_type==2'h0);

always @(posedge clk)
	set_feature <= #1 hdr_done & (bRequest == SET_FEATURE) &
			(bm_req_type==2'h0);

always @(posedge clk)
	set_address <= #1 hdr_done & (bRequest == SET_ADDRESS) &
			(bm_req_type==2'h0);

always @(posedge clk)
	get_descriptor <= #1 hdr_done & (bRequest == GET_DESCRIPTOR) &
			(bm_req_type==2'h0);

always @(posedge clk)
	set_descriptor <= #1 hdr_done & (bRequest == SET_DESCRIPTOR) &
			(bm_req_type==2'h0);

always @(posedge clk)
	get_config <= #1 hdr_done & (bRequest == GET_CONFIG) &
			(bm_req_type==2'h0);

always @(posedge clk)
	set_config <= #1 hdr_done & (bRequest == SET_CONFIG) &
			(bm_req_type==2'h0);

always @(posedge clk)
	get_interface <= #1 hdr_done & (bRequest == GET_INTERFACE) &
			(bm_req_type==2'h0);

always @(posedge clk)
	set_interface <= #1 hdr_done & (bRequest == SET_INTERFACE) &
			(bm_req_type==2'h0);

always @(posedge clk)
	synch_frame <= #1 hdr_done & (bRequest == SYNCH_FRAME) &
			(bm_req_type==2'h0);

always @(posedge clk)
	v_set_int <= #1	hdr_done & (bRequest == V_SET_INT) &
			(bm_req_type==2'h2);

always @(posedge clk)
	v_set_feature <= #1 hdr_done & (bRequest == SET_FEATURE) &
			(bm_req_type==2'h2);

always @(posedge clk)
	v_get_status <= #1 hdr_done & (bRequest == GET_STATUS) &
			(bm_req_type==2'h2);

// A config err must cause the device to send a STALL for an ACK
always @(posedge clk)
	config_err <= #1 hdr_done_r & !(get_status | clear_feature |
			set_feature | set_address | get_descriptor |
			set_descriptor | get_config | set_config |
			get_interface | set_interface | synch_frame |
			v_set_int | v_set_feature | v_get_status);

always @(posedge clk)
	send_stall <= #1 config_err;

///////////////////////////////////////////////////////////////////
//
// Set address
//

always @(posedge clk)
	if(!rst)	set_adr_pending <= #1 1'b0;
	else
	if(ctrl_in | ctrl_out | ctrl_setup)
			set_adr_pending <= #1 1'b0;
	else
	if(set_address)	set_adr_pending <= #1 1'b1;

always @(posedge clk)
	if(!rst)	funct_adr_tmp <= #1 7'h0;
	else
	if(set_address)	funct_adr_tmp <= #1 wValue[6:0];

always @(posedge clk)
	if(!rst)	funct_adr <= #1 7'h0;
	else
	if(set_adr_pending & ctrl_in)
			funct_adr <= #1 funct_adr_tmp;

///////////////////////////////////////////////////////////////////
//
// Main FSM
//

always @(posedge clk)
	if(!rst)	state <= #1 IDLE;
	else		state <= next_state;

always @(state or ctrl_setup or ctrl_in or ctrl_out or hdr_done or
	rom_done or wValue or bm_req_recp or
	get_status or clear_feature or set_feature or set_address or
	get_descriptor or set_descriptor or get_config or set_config or
	get_interface or set_interface or synch_frame or v_set_int or
	v_set_feature or v_get_status
	)
begin
	next_state 	= state;
	in_size_0 	= 1'b0;
	in_size_1 	= 1'b0;
	in_size_2 	= 1'b0;
	rom_sel 	= 1'b0;

	case(state)	// synopsys full_case parallel_case

	   IDLE:
		   begin
			if(get_status)		next_state = GET_STATUS_S;
			if(clear_feature)	next_state = CLEAR_FEATURE_S;
			if(set_feature)		next_state = SET_FEATURE_S;
			if(set_address)		next_state = SET_ADDRESS_S;
			if(get_descriptor)	next_state = GET_DESCRIPTOR_S;
			if(set_descriptor)	next_state = SET_DESCRIPTOR_S;
			if(get_config)		next_state = GET_CONFIG_S;
			if(set_config)		next_state = SET_CONFIG_S;
			if(get_interface)	next_state = GET_INTERFACE_S;
			if(set_interface)	next_state = SET_INTERFACE_S;
			if(synch_frame)		next_state = SYNCH_FRAME_S;
			if(v_set_int)		next_state = V_SET_INT_S;
			if(v_set_feature)	next_state = V_SET_INT_S;
			if(v_get_status)	next_state = V_GET_STATUS_S;
		   end

		// Actions for supported commands
	   GET_STATUS_S:
		   begin
			// Returns to host
			// 16'h0001 for device
			// 16'h0000 for interface
			// 16'h0000 for endpoint
			// if(bm_req_recp == 5'h00)	
			//		data_sel = ZERO_ONE_DATA;
			// else		
			//		data_sel = ZERO_DATA;
			// in_size_2 = 1'b1;
			// if(!fifo_full)
			// begin
			//	fifo_we_d = 1'b1;
			//	if(write_done_r)
			//		next_state = WAIT_IN_DATA;
			// end

		   end
	   V_GET_STATUS_S:
		   begin
			// data_sel = VEND_DATA;
			// in_size_2 = 1'b1;
			// if(!fifo_full)
			//   begin
			//	fifo_we_d = 1'b1;
			//	if(write_done_r)
			//		next_state = WAIT_IN_DATA;
			//   end
		   end

	   CLEAR_FEATURE_S:
		   begin
			// just ignore this for now
			next_state = STATUS_IN;
		   end

	   SET_FEATURE_S:
		   begin
			// just ignore this for now
			next_state = STATUS_IN;
		   end

	   SET_ADDRESS_S:
		   begin
			$display("Set Address Request");
			// done elsewhere ....
			next_state = STATUS_IN;
		   end

	   GET_DESCRIPTOR_S:
		   begin
			$display("Get Descriptor Request Value = %h", 
					wValue);
			if(	wValue[15:8] == 8'h01 |
				wValue[15:8] == 8'h02 | 
				wValue[15:8] == 8'h03	)
				rom_sel = 1'b1;
			else
				next_state = IDLE;

			if(rom_done)
				next_state = IDLE;
		   end

	   SET_DESCRIPTOR_S:
		   begin
			// This doesn't do anything since we do not support
			// setting the descriptor
			next_state = IDLE;
		   end

	   GET_CONFIG_S:
		   begin
			// Send one byte back that indicates current status
			// in_size_1 = 1'b1;
			// data_sel = CONFIG_DATA;
			// if(!fifo_full)
			//   begin
			//	fifo_we_d = 1'b1;
			//	next_state = WAIT_IN_DATA;
			//   end
		   end

	   SET_CONFIG_S:
		   begin
			// done elsewhere ....
			next_state = STATUS_IN;
		   end

	   GET_INTERFACE_S:
		   begin
			// Return interface '0'
			// in_size_1 = 1'b1;
			// if(!fifo_full)
			//   begin
			//	fifo_we_d = 1'b1;
			//	next_state = WAIT_IN_DATA;
			//   end
		   end

	   SET_INTERFACE_S:
		   begin
			// just ignore this for now
			next_state = STATUS_IN;
		   end

	   SYNCH_FRAME_S:
		   begin
			// Return Frame current frame number
			// data_sel = SYNC_FRAME_DATA;
			// in_size_2 = 1'b1;
			// if(!fifo_full)
			//   begin
			//	fifo_we_d = 1'b1;
			//	if(write_done_r)	
			//		next_state = WAIT_IN_DATA;
			//   end
		   end

	   V_SET_INT_S:
		   begin
			// done elsewhere ....
			next_state = STATUS_IN;
		   end

	   WAIT_IN_DATA:
		   begin
			if(ctrl_in)	next_state = STATUS_OUT;
		   end

	   STATUS_IN:
		   begin
			in_size_0 = 1'b1;
			if(ctrl_in)	next_state = IDLE;
		   end

	   STATUS_OUT:
		   begin
			if(ctrl_out)	next_state = IDLE;
		   end
	endcase
   end

endmodule
