/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  USB 1.1 function defines file                              ////
////                                                             ////
////                                                             ////
////  Author: Rudolf Usselmann                                   ////
////          rudi@asics.ws                                      ////
////                                                             ////
////                                                             ////
////  Based on: http://www.opencores.org/cores/usb1_funct/       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`timescale 1ns / 10ps

`ifdef  USB_DEFINES
`else
`define USB_DEFINES

`define IR_SOURCES  4

// additions to reflect endpoint status

`define EP_TYPE_NONE     2'h0
`define EP_TYPE_IN       2'h1
`define EP_TYPE_OUT      2'h2
`define EP_TYPE_CTRL     2'h3

// `define USBF_DEBUG
// `define USBF_VERBOSE_DEBUG

// Enable or disable Block Frames
//`define USB1_BF_ENABLE

/////////////////////////////////////////////////////////////////////
//
// Items below this point should NOT be modified by the end user
// UNLESS you know exactly what you are doing !
// Modify at you own risk !!!
//
/////////////////////////////////////////////////////////////////////
// Endpoint Configuration Constants
`define IN          4'b0_001
`define OUT         4'b0_010
`define CTRL        4'b1_100
`define BULK        4'b1_000
`define INT         4'b0_000

`define USB_DEVICE_DESCRIPTOR       8'h01
`define USB_CLASS_VENDOR_SPECIFIC   8'hFF
`define USB_CONFIG_DESCRIPTOR       8'h02
`define USB_SELF_POWERED            8'h40
`define USB_INTERFACE_DESCRIPTOR    8'h04
`define USB_ATTRIBUTE_BULK          8'h02
`define USB_DIRECTION_IN            8'h01
`define USB_DIRECTION_OUT           8'h00
`define USB_STRING_DESCRIPTOR       8'h03
`define USB_ENDPOINT_DESCRIPTOR     8'h05

`define USB_DEVICE_DESCR_SIZE       8'h12
`define USB_CONFIG_DESCR_SIZE       8'h09
`define USB_INTERFACE_DESCR_SIZE    8'h09
`define USB_ENDPOINT_DESCR_SIZE     8'h07

// ROM layout for device configuration data
`define ROM_SIZE0   7'd018  // Device Descriptor Length
`define ROM_SIZE1   7'd062  // Configuration Descriptor Length
`define ROM_SIZE2A  7'd004  // Language ID Descriptor Start Length
`define ROM_SIZE2B  7'd062  // String Descriptor 1 Length
`define ROM_SIZE2C  7'd062  // String Descriptor 2 Length
`define ROM_SIZE2D  7'd062  // String Descriptor 3 Length

// rom start constants are directly used as chip selects for the various ROM
// instances
`define ROM_START0  6'b000001  // Device Descriptor ROM
`define ROM_START1  6'b000010  // Configuration Descriptor ROM
`define ROM_START2A 6'b000100  // Language ID Descriptor ROM
`define ROM_START2B 6'b001000  // String Descriptor 1 Start ROM
`define ROM_START2C 6'b010000  // String Descriptor 2 Start ROM
`define ROM_START2D 6'b100000  // String Descriptor 3 Start ROM

//`define ROM_START0  10'h21  // Device Descriptor Start Address
//`define ROM_START1  10'h2a  // Configuration Descriptor Start Address
//`define ROM_START2A 10'h68  // Language ID Descriptor Start Address
//`define ROM_START2B 10'h6a  // String Descriptor 1 Start Address
//`define ROM_START2C 10'h73  // String Descriptor 2 Start Address
//`define ROM_START2D 10'h7c  // String Descriptor 3 Start Address

// PID Encodings
`define USBF_T_PID_OUT   4'b0001
`define USBF_T_PID_IN    4'b1001
`define USBF_T_PID_SOF   4'b0101
`define USBF_T_PID_SETUP 4'b1101
`define USBF_T_PID_DATA0 4'b0011
`define USBF_T_PID_DATA1 4'b1011
`define USBF_T_PID_DATA2 4'b0111
`define USBF_T_PID_MDATA 4'b1111
`define USBF_T_PID_ACK   4'b0010
`define USBF_T_PID_NACK  4'b1010
`define USBF_T_PID_STALL 4'b1110
`define USBF_T_PID_NYET  4'b0110
`define USBF_T_PID_PRE   4'b1100
`define USBF_T_PID_ERR   4'b1100
`define USBF_T_PID_SPLIT 4'b1000
`define USBF_T_PID_PING  4'b0100
`define USBF_T_PID_RES   4'b0000

// The HMS_DEL is a constant for the "Half Micro Second"
// Clock pulse generator. This constant specifies how many
// Phy clocks there are between two hms_clock pulses. This
// constant plus 2 represents the actual delay.
// Example: For a 60 Mhz (16.667 nS period) Phy Clock, the
// delay must be 30 phy clock: 500ns / 16.667nS = 30 clocks
`define USBF_HMS_DEL    5'h16

// After sending Data in response to an IN token from host, the
// host must reply with an ack. The host has 622nS in Full Speed
// mode and 400nS in High Speed mode to reply. RX_ACK_TO_VAL_FS
// and RX_ACK_TO_VAL_HS are the numbers of UTMI clock cycles
// minus 2 for Full and High Speed modes.
//`define USBF_RX_ACK_TO_VAL_FS 8'd36
`define USBF_RX_ACK_TO_VAL_FS   8'd200

// After sending a OUT token the host must send a data packet.
// The host has 622nS in Full Speed mode and 400nS in High Speed
// mode to send the data packet.
// TX_DATA_TO_VAL_FS and TX_DATA_TO_VAL_HS are is the numbers of
// UTMI clock cycles minus 2.
//`define USBF_TX_DATA_TO_VAL_FS    8'd36
`define USBF_TX_DATA_TO_VAL_FS  8'd200

`endif
