/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`include "shi/usb/src/usb1_defines.vh"
`default_nettype none

module usb1_device_rom(

    input wire [4:0]        addr_i,
    input wire              cs_i,
    output wire [7:0]       data_rd_o

);

parameter [15:0]
    VENDOR_ID    = 16'h6666,
    PRODUCT_ID   = 16'hAFFE;

localparam [7:0]
    DESCR_SIZE          = `USB_DEVICE_DESCR_SIZE,
    DESCR_TYPE          = `USB_DEVICE_DESCRIPTOR,
    CLASS               = `USB_CLASS_VENDOR_SPECIFIC,
    USB_SPEC            = 8'h01,
    SUBCLASS            = 8'h00,
    PROTOCOL            = 8'hFF,
    PACKET_SIZE         = 8'h40,
    VENDOR_DESCR_IDX    = 8'h01,
    PRODUCT_DESCR_IDX   = 8'h02,
    SERIAL_DESCR_IDX    = 8'h03,
    NUM_CONFIGS         = 8'h01;

localparam [15:0]
    RELEASE             = 16'h0010;

reg [7:0] rom_data_rd;

assign data_rd_o = cs_i ? rom_data_rd : 8'h00;

always @(*)
begin
    case(addr_i)
        5'h00: rom_data_rd <= DESCR_SIZE;
        5'h01: rom_data_rd <= DESCR_TYPE;
        5'h02: rom_data_rd <= 8'h00; // reserved, not used
        5'h03: rom_data_rd <= USB_SPEC;
        5'h04: rom_data_rd <= CLASS;
        5'h05: rom_data_rd <= SUBCLASS;
        5'h06: rom_data_rd <= PROTOCOL;
        5'h07: rom_data_rd <= PACKET_SIZE;
        5'h08: rom_data_rd <= VENDOR_ID[7:0];
        5'h09: rom_data_rd <= VENDOR_ID[15:8];
        5'h0A: rom_data_rd <= PRODUCT_ID[7:0];
        5'h0B: rom_data_rd <= PRODUCT_ID[15:8];
        5'h0C: rom_data_rd <= RELEASE[7:0];
        5'h0D: rom_data_rd <= RELEASE[15:8];
        5'h0E: rom_data_rd <= VENDOR_DESCR_IDX;
        5'h0F: rom_data_rd <= PRODUCT_DESCR_IDX;
        5'h10: rom_data_rd <= SERIAL_DESCR_IDX;
        5'h11: rom_data_rd <= NUM_CONFIGS;

        default: rom_data_rd <= 8'h00;
    endcase
end

endmodule
