/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`include "shi/usb/src/usb1_defines.vh"
`default_nettype none

module usb1_strdescr_rom #(

  parameter [1023:0] RAW_DATA   = 0,
  parameter STR_DATA            = "",
  parameter DATA_SIZE           = get_str_length(STR_DATA),
  parameter ROM_SIZE            = (STR_DATA != "") ? (DATA_SIZE * 2) + 2 : DATA_SIZE + 2,
  parameter AWIDTH              = bitwidth(ROM_SIZE)

)(

  input wire [AWIDTH-1:0]   addr_i,
  input wire                cs_i,
  output wire [7:0]         data_rd_o

);

`include "shi/usb/include/usb1_functions.vh"
`include "common/functions/bitwidth.vh"

wire [7:0] rom [ROM_SIZE-1:0];

genvar i;

generate 

  if (STR_DATA != "")
  begin
    for (i=0; i<DATA_SIZE; i=i+1)
    begin: ROM_INIT_STRING
      assign rom[(i*2)+2] = STR_DATA[(DATA_SIZE-i-1)*8+7:(DATA_SIZE-i-1)*8];
      assign rom[(i*2)+3] = 8'h00;
    end
  end else begin 
    for (i=0; i<DATA_SIZE; i=i+1)
    begin : ROM_INIT_RAW
      assign rom[i+2] = RAW_DATA[(DATA_SIZE-i-1)*8+7:(DATA_SIZE-i-1)*8];
    end
  end

endgenerate

assign rom[0] = ROM_SIZE[7:0];
assign rom[1] = `USB_STRING_DESCRIPTOR;

assign data_rd_o = cs_i ? rom[addr_i] : 8'h00;

endmodule
