/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  USB 1.1 PHY                                                ////
////                                                             ////
////                                                             ////
////  Author: Rudolf Usselmann                                   ////
////          rudi@asics.ws                                      ////
////                                                             ////
////                                                             ////
////  Based on: http://www.opencores.org/cores/usb_phy/          ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`timescale 1ns / 10ps
`default_nettype none

module usb_phy(clk, rst, phy_tx_mode, usb_rst,

		// Transciever Interface
		dp, dn,

		// UTMI Interface
		DataOut_i, TxValid_i, TxReady_o, RxValid_o,
		RxActive_o, RxError_o, DataIn_o, LineState_o,

        // Status
        txoe
		);

input wire				clk;
input wire				rst;
input wire				phy_tx_mode;
output reg				usb_rst;
inout wire				dp, dn;
input wire	[7:0]		DataOut_i;
input wire				TxValid_i;
output wire				TxReady_o;
output wire	[7:0]		DataIn_o;
output wire				RxValid_o;
output wire				RxActive_o;
output wire				RxError_o;
output wire	[1:0]		LineState_o;
output wire          	txoe;


///////////////////////////////////////////////////////////////////
//
// Local Wires and Registers
//

reg	[4:0]	rst_cnt;
wire		fs_ce;

wire		txdp, txdn;
wire		rxd, rxdp, rxdn;

///////////////////////////////////////////////////////////////////
//
// Misc Logic
//

assign dp  = txoe ? 1'bz : txdp;
assign dn  = txoe ? 1'bz : txdn;
assign rxd = rxdp & (!rxdn);
assign rxdp = dp;
assign rxdn = dn;

///////////////////////////////////////////////////////////////////
//
// TX Phy
//

//usb_tx_phy i_tx_phy(
//	.clk(		clk		),
//	.rst(		rst		),
//	.fs_ce(		fs_ce		),
//	.phy_mode(	phy_tx_mode	),
//
//	// Transciever Interface
//	.txdp(		txdp		),
//	.txdn(		txdn		),
//	.txoe(		txoe		),
//
//	// UTMI Interface
//	.DataOut_i(	DataOut_i	),
//	.TxValid_i(	TxValid_i	),
//	.TxReady_o(	TxReady_o	)
//	);

usb_tx_phy2 i_tx_phy(
	.clk_i(		clk		),
	.reset_i(		~rst		),
	.bit_shift_en_i(		fs_ce		),

	// Transciever Interface
	.tx_dp_o(		txdp		),
	.tx_dn_o(		txdn		),
	.tx_en_o(		txoe		),

	// UTMI Interface
	.data_i(	DataOut_i	),
	.data_valid_i(	TxValid_i	),
	.data_next_o(	TxReady_o	)
	);

///////////////////////////////////////////////////////////////////
//
// RX Phy and DPLL
//

usb_rx_phy i_rx_phy(
	.clk(		clk		),
	.rst(		rst		),
	.fs_ce(		fs_ce		),

	// Transciever Interface
	.rxd(		rxd		),
	.rxdp(		rxdp		),
	.rxdn(		rxdn		),
	// UTMI Interface
	.DataIn_o(	DataIn_o	),
	.RxValid_o(	RxValid_o	),
	.RxActive_o(	RxActive_o	),
	.RxError_o(	RxError_o	),
	.RxEn_i(	txoe		),
	.LineState(	LineState_o	)
	);

///////////////////////////////////////////////////////////////////
//
// Generate an USB Reset is we see SE0 for at least 2.5uS
//

`ifdef USB_ASYNC_REST
always @(posedge clk or negedge rst)
`else
always @(posedge clk)
`endif
	if(!rst)			rst_cnt <= 5'h0;
	else
	if(LineState_o != 2'h0)		rst_cnt <= 5'h0;
	else
	if(!usb_rst && fs_ce)		rst_cnt <= rst_cnt + 5'h1;

always @(posedge clk)
	usb_rst <= (rst_cnt == 5'h1f);

endmodule
