/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/////////////////////////////////////////////////////////////////////
////                                                             ////
////  USB 1.1 function IP core                                   ////
////                                                             ////
////                                                             ////
////  Author: Rudolf Usselmann                                   ////
////          rudi@asics.ws                                      ////
////                                                             ////
////                                                             ////
////  Based on: http://www.opencores.org/cores/usb1_funct/       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

`timescale 10ps/1ps
`default_nettype none

module usbcore_shi  (

  // clock and reset
  input wire        reset_i,
  input wire        clk_48_mhz_i,
  // phy
  inout wire        usb_data_p_io,
  inout wire        usb_data_n_io,
  output wire       disconnect_o,
  // status
  output wire       tx_active_o,
  output wire       rx_active_o,
  output wire       send_stall_o,
  output wire       configured_o,
  output wire       halt_o,
  output wire       addressed_o,
  output wire       transfer_failed_o,
  output wire       transfer_finished_o,
  // SHI
  input wire        ep1_clk_fifo_i,
  input wire [7:0]  ep1_tx_data_i,
  input wire        ep1_tx_valid_i,
  output wire       ep1_tx_rdy_o,
  output wire [7:0] ep1_rx_data_o,
  output wire       ep1_rx_valid_o,
  input wire        ep1_rx_en_i,
  input wire        ep2_clk_fifo_i,
  input wire [7:0]  ep2_tx_data_i,
  input wire        ep2_tx_valid_i,
  output wire       ep2_tx_rdy_o,
  output wire [7:0] ep2_rx_data_o,
  output wire       ep2_rx_valid_o,
  input wire        ep2_rx_en_i,
  input wire        ep3_clk_fifo_i,
  input wire [7:0]  ep3_tx_data_i,
  input wire        ep3_tx_valid_i,
  output wire       ep3_tx_rdy_o,
  output wire [7:0] ep3_rx_data_o,
  output wire       ep3_rx_valid_o,
  input wire        ep3_rx_en_i,
  input wire        ep4_clk_fifo_i,
  input wire [7:0]  ep4_tx_data_i,
  input wire        ep4_tx_valid_i,
  output wire       ep4_tx_rdy_o,
  output wire [7:0] ep4_rx_data_o,
  output wire       ep4_rx_valid_o,
  input wire        ep4_rx_en_i,
  input wire        ep5_clk_fifo_i,
  input wire [7:0]  ep5_tx_data_i,
  input wire        ep5_tx_valid_i,
  output wire       ep5_tx_rdy_o,
  output wire [7:0] ep5_rx_data_o,
  output wire       ep5_rx_valid_o,
  input wire        ep5_rx_en_i,
  input wire        ep6_clk_fifo_i,
  input wire [7:0]  ep6_tx_data_i,
  input wire        ep6_tx_valid_i,
  output wire       ep6_tx_rdy_o,
  output wire [7:0] ep6_rx_data_o,
  output wire       ep6_rx_valid_o,
  input wire        ep6_rx_en_i
);

parameter
  NUM_EP_IN           = 1,
  NUM_EP_OUT          = 1,
  EP_01_TYPE          = "IN",
  EP_02_TYPE          = "OUT",
  EP_03_TYPE          = "",
  EP_04_TYPE          = "",
  EP_05_TYPE          = "",
  EP_06_TYPE          = "",
  PRODUCT_STRING      = "",
  VENDOR_STRING       = "",
  SERIAL_STRING       = "",
  VENDOR_ID           = 0,
  PRODUCT_ID          = 0,
  LANGUAGE_ID         = 0,
  FIFO_SIZE           = 64;

usbcore_fifo #(
  .NUM_EP_OUT(NUM_EP_OUT),
  .NUM_EP_IN(NUM_EP_IN),
  .EP_01_TYPE(EP_01_TYPE),
  .EP_02_TYPE(EP_02_TYPE),
  .EP_03_TYPE(EP_03_TYPE),
  .EP_04_TYPE(EP_04_TYPE),
  .EP_05_TYPE(EP_05_TYPE),
  .EP_06_TYPE(EP_06_TYPE),
  .PRODUCT_STRING(PRODUCT_STRING),
  .VENDOR_STRING(VENDOR_STRING),
  .SERIAL_STRING(SERIAL_STRING),
  .VENDOR_ID(VENDOR_ID),
  .PRODUCT_ID(PRODUCT_ID),
  .LANGUAGE_ID(LANGUAGE_ID),
  .FIFO_SIZE(FIFO_SIZE)
) usbcore_fifo0  (
  .reset_i(reset_i),
  .clk_fifo_i({
    ep6_clk_fifo_i,
    ep5_clk_fifo_i,
    ep4_clk_fifo_i,
    ep3_clk_fifo_i,
    ep2_clk_fifo_i,
    ep1_clk_fifo_i
  }),
  .clk_48_mhz_i(clk_48_mhz_i),
  .usb_data_p_io(usb_data_p_io),
  .usb_data_n_io(usb_data_n_io),
  .disconnect_o(disconnect_o),
  .tx_active_o(tx_active_o),
  .rx_active_o(rx_active_o),
  .send_stall_o(send_stall_o),
  .configured_o(configured_o),
  .addressed_o(addressed_o),
  .halt_o(halt_o),
  .transfer_finished_o(transfer_finished_o),
  .transfer_failed_o(transfer_failed_o),
  .device_tx_rdy_o({
    ep6_tx_rdy_o,
    ep5_tx_rdy_o,
    ep4_tx_rdy_o,
    ep3_tx_rdy_o,
    ep2_tx_rdy_o,
    ep1_tx_rdy_o
  }),
  .device_tx_valid_i({
    ep6_tx_valid_i,
    ep5_tx_valid_i,
    ep4_tx_valid_i,
    ep3_tx_valid_i,
    ep2_tx_valid_i,
    ep1_tx_valid_i
  }),
  .device_tx_data_i({
    ep6_tx_data_i,
    ep5_tx_data_i,
    ep4_tx_data_i,
    ep3_tx_data_i,
    ep2_tx_data_i,
    ep1_tx_data_i
  }),
  .device_rx_valid_o({
    ep6_rx_valid_o,
    ep5_rx_valid_o,
    ep4_rx_valid_o,
    ep3_rx_valid_o,
    ep2_rx_valid_o,
    ep1_rx_valid_o
  }),
  .device_rx_en_i({
    ep6_rx_en_i,
    ep5_rx_en_i,
    ep4_rx_en_i,
    ep3_rx_en_i,
    ep2_rx_en_i,
    ep1_rx_en_i
  }),
  .device_rx_data_o({
    ep6_rx_data_o,
    ep5_rx_data_o,
    ep4_rx_data_o,
    ep3_rx_data_o,
    ep2_rx_data_o,
    ep1_rx_data_o
  })
);

endmodule
