/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:04:33 12/03/2013 
// Design Name: 
// Module Name:    sysmon 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module sysmon
#(
	parameter ARCHITECTURE   = "VIRTEX6",
	parameter BASE_ADR	= 10'h0,
	
	parameter INIT_40 = 16'h0000,
	parameter INIT_41 = 16'h0000,
	parameter INIT_42 = 16'h0000,
	parameter INIT_43 = 16'h0000,
	parameter INIT_44 = 16'h0000,
	parameter INIT_45 = 16'h0000,
	parameter INIT_46 = 16'h0000,
	parameter INIT_47 = 16'h0000,
	parameter INIT_48 = 16'h0000,
	parameter INIT_49 = 16'h0000,
	parameter INIT_4A = 16'h0000,
	parameter INIT_4B = 16'h0000,
	parameter INIT_4C = 16'h0000,
	parameter INIT_4D = 16'h0000,
	parameter INIT_4E = 16'h0000,
	parameter INIT_4F = 16'h0000,
	parameter INIT_50 = 16'h0000,
	parameter INIT_51 = 16'h0000,
	parameter INIT_52 = 16'h0000,
	parameter INIT_53 = 16'h0000,
	parameter INIT_54 = 16'h0000,
	parameter INIT_55 = 16'h0000,
	parameter INIT_56 = 16'h0000,
	parameter INIT_57 = 16'h0000,
	parameter INIT_58 = 16'h0000,
	parameter INIT_5C = 16'h0000
)
(
    input	wire		clk_peri,
    input	wire [17:0]		do_peri,	// Datenbus
    output	wire [17:0]		di_peri,	// Datenbus
    input	wire [9:0]		addr_peri,	// Adressbus
    input	wire		access_peri,	// chipselect peripherie
    input	wire		wr_peri,	// signalisiert Schreibzugriff Peripherie
    input	wire		reset,
	 
	 input 	wire 		V_P,
	 input	wire		V_N,
	 input 	wire [15:0] VAUX_P, 
	 input 	wire [15:0]	VAUX_N,
	 output wire overtemp
	 
);

// Die Parameter fuer die Adressen werden in system.v eingestellt.


parameter DRP_ADR	= 4'h0;
parameter DRP_DAT	= 4'h1;
parameter STATUS1	= 4'h2;
parameter STATUS2	= 4'h3;
parameter DECODSIM_DEB	= "NO";

wire		[3:0]		reg_select;
wire		[3:0]		reg_read;
wire		[3:0]		reg_write;
wire    peri_select;
// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
		.REG_COUNT(	4	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset		(	reset			),
	        .clk_peri       (       clk_peri        ),
                .addr_peri	(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri		(	wr_peri		),

		.peri_select( peri_select			),
		.reg_select	(       reg_select		),
		.reg_read	(	reg_read	),
		.reg_write	(	reg_write	)
			);

reg  [15:0] spmc_drp_dat;
reg  [6:0] 	spmc_drp_addr;

wire	cs_spmc_drp_dat_r		= reg_read [DRP_DAT];	// Lesen
wire	cs_spmc_drp_addr_r	= reg_read [DRP_ADR];	// Lesen
wire	cs_spmc_drp_dat_w		= reg_write[DRP_DAT];	// Schreiben
wire	cs_spmc_drp_addr_w	= reg_write[DRP_ADR];	// Schreiben
wire 	cs_spmc_status1_r		= reg_read [STATUS1];
wire 	cs_spmc_status2_r		= reg_read [STATUS2];


wire [7:0] alarm;
wire [4:0] channel;
wire [4:0] muxaddr;
wire eoc;
wire eos;
wire busy;


wire [15:0] drp_do;
wire [15:0] drp_di	= spmc_drp_dat;
wire [6:0] 	drp_addr = spmc_drp_addr;
wire 			drp_en	= cs_spmc_drp_addr_w || cs_spmc_drp_dat_w;
wire 			drp_wen	= cs_spmc_drp_dat_w;
wire 			drp_clk 	= clk_peri;
wire			drp_ready;

wire jtag_busy;
wire jtag_locked;
wire jtag_modified;


wire [17:0] status1 = {
	muxaddr, // 17:13
	channel,	//12:8
	alarm	// 7:0
};
wire [17:0] status2 = {
	jtag_modified,		// 7
	jtag_locked,		// 6
	jtag_busy,			// 5
	drp_ready,			//	4
	busy,					// 3
	eos,					// 2
	eoc,					// 1
	overtemp				// 0
};
// Signale müssen vielleicht nach außen gelegt werden
wire 			convst = 0;
wire 			convstclk = 0;

always @ (posedge clk_peri or posedge reset)
if(reset)
	begin
	spmc_drp_dat <= 0;
	spmc_drp_addr<= 0;
	end
else
// schreiben der Ports
begin	
if(cs_spmc_drp_dat_w)				
	spmc_drp_dat	<= do_peri[15:0];
if(cs_spmc_drp_addr_w)
	spmc_drp_addr	<= do_peri[6:0];	
end

// lesen der Ports
assign	di_peri [17:0]	= 
				(cs_spmc_drp_dat_r)	?	{2'h0		,	drp_do			}:
				(cs_spmc_drp_addr_r)	?	{11'h000	,	spmc_drp_addr	}:	
				(cs_spmc_status1_r)	?	{				status1			}:
				(cs_spmc_status2_r)	?	{9'h000	,	status2			}: 18'b0;


generate
	case (ARCHITECTURE)
   "SERIE7": begin: SYSMONSE7
		//artix, Kintex, virtex 7
		XADC #(
      // INIT_40 - INIT_42: XADC configuration registers
      .INIT_40(INIT_40),
      .INIT_41(INIT_41),
      .INIT_42(INIT_42),
      // INIT_48 - INIT_4F: Sequence Registers
      .INIT_48(INIT_48),
      .INIT_49(INIT_49),
      .INIT_4A(INIT_4A),
      .INIT_4B(INIT_4B),
      .INIT_4C(INIT_4C),
      .INIT_4D(INIT_4D),
      .INIT_4E(INIT_4E),
      .INIT_4F(INIT_4F),                // Sequence register 6
      // INIT_50 - INIT_58, INIT5C: Alarm Limit Registers
      .INIT_50(INIT_50),
      .INIT_51(INIT_51),
      .INIT_52(INIT_52),
      .INIT_53(INIT_53),
      .INIT_54(INIT_54),
      .INIT_55(INIT_55),
      .INIT_56(INIT_56),
      .INIT_57(INIT_57),
      .INIT_58(INIT_58),
      .INIT_5C(INIT_5C),
      // Simulation attributes: Set for proper simulation behavior
      .SIM_DEVICE("7SERIES"),            // Select target device (values)
      .SIM_MONITOR_FILE("")  // Analog simulation data file name
   )
   XADC_inst (
      // ALARMS: 8-bit (each) output: ALM, OT
      .ALM				(alarm[7:0]),      // 8-bit output: Output alarm for temp, Vccint, Vccaux and Vccbram
      .OT				(overtemp),                     // 1-bit output: Over-Temperature alarm
      // Dynamic Reconfiguration Port (DRP): 16-bit (each) output: Dynamic Reconfiguration Ports
      .DO				(drp_do),                     // 16-bit output: DRP output data bus
      .DRDY				(drp_ready),                 // 1-bit output: DRP data ready
      // STATUS: 1-bit (each) output: XADC status ports
      .BUSY				(busy),                 // 1-bit output: ADC busy output
      .CHANNEL			(channel),           // 5-bit output: Channel selection outputs
      .EOC				(eoc),                   // 1-bit output: End of Conversion
      .EOS				(eos),                   // 1-bit output: End of Sequence
      .JTAGBUSY		(jtag_busy),         // 1-bit output: JTAG DRP transaction in progress output
      .JTAGLOCKED		(jtag_locked),     	// 1-bit output: JTAG requested DRP port lock
      .JTAGMODIFIED	(jtag_modified), 		// 1-bit output: JTAG Write to the DRP has occurred
      .MUXADDR			(muxaddr),           // 5-bit output: External MUX channel decode
      // Auxiliary Analog-Input Pairs: 16-bit (each) input: VAUXP[15:0], VAUXN[15:0]
      .VAUXN			(VAUX_N),               // 16-bit input: N-side auxiliary analog input
      .VAUXP			(VAUX_P),               // 16-bit input: P-side auxiliary analog input
      // CONTROL and CLOCK: 1-bit (each) input: Reset, conversion start and clock inputs
      .CONVST			(convst),             // 1-bit input: Convert start input
      .CONVSTCLK		(convstclk),       // 1-bit input: Convert start input
      .RESET			(reset),               // 1-bit input: Active-high reset
      // Dedicated Analog Input Pair: 1-bit (each) input: VP/VN
      .VN				(V_N),                     // 1-bit input: N-side analog input
      .VP				(V_P),                     // 1-bit input: P-side analog input
      // Dynamic Reconfiguration Port (DRP): 7-bit (each) input: Dynamic Reconfiguration Ports
      .DADDR			(drp_addr),               // 7-bit input: DRP address bus
      .DCLK				(drp_clk),                 // 1-bit input: DRP clock
      .DEN				(drp_en),                   // 1-bit input: DRP enable signal
      .DI				(drp_di),                     // 16-bit input: DRP input data bus
      .DWE				(drp_wen)                    // 1-bit input: DRP write enable
   );
	end
	"VIRTEX6": begin: SYSMONV6
		// virtex 6
		   SYSMON #(
      // INIT_40 - INIT_42: System Monitor configuration registers
      .INIT_40(INIT_40),
      .INIT_41(INIT_41),
      .INIT_42(INIT_42),
      // INIT_43 - INIT_47: System Monitor Test registers (do not edit)
      .INIT_43(INIT_43),
      .INIT_44(INIT_44),
      .INIT_45(INIT_45),
      .INIT_46(INIT_46),
      .INIT_47(INIT_47),
      // INIT_48 - INIT_4F: Sequence registers for the Channel Sequencer
      .INIT_48(INIT_48),
      .INIT_49(INIT_49),
      .INIT_4A(INIT_4A),
      .INIT_4B(INIT_4B),
      .INIT_4C(INIT_4C),
      .INIT_4D(INIT_4D),
      .INIT_4E(INIT_4E),
      .INIT_4F(INIT_4F),
      // INIT_50 - INIT_57: Alarm threshold registers
      .INIT_50(INIT_50),
      .INIT_51(INIT_51),
      .INIT_52(INIT_52),
      .INIT_53(INIT_53),
      .INIT_54(INIT_54),
      .INIT_55(INIT_55),
      .INIT_56(INIT_56),
      .INIT_57(INIT_57),
      // Simulation attributes: Set for proper simulation behavior
      .SIM_DEVICE("VIRTEX6"),          // Must be set to VIRTEX6
      .SIM_MONITOR_FILE("design.txt")  // Analog simulation data file name
   )
   SYSMON_inst (
      // Alarm Ports: 3-bit (each) output: ALM, OT
      .ALM			(alarm[2:0]),                   // 3-bit output: output alarm for temp, Vccint and Vccaux
      .OT			(overtemp),                     // 1-bit output: Over-Temperature alarm output
      // DRP Ports: 16-bit (each) output: Dynamic Reconfiguration Ports
      .DO			(drp_do),                     // 16-bit output: DRP output data bus
      .DRDY			(drp_ready),                 // 1-bit output: DRP data ready output signal
      // Status Ports: 1-bit (each) output: SYSMON status ports
      .BUSY			(busy),                 // 1-bit output: ADC busy output
      .CHANNEL		(channel),           // 5-bit output: Channel selection outputs
      .EOC			(eoc),                   // 1-bit output: End of Conversion output
      .EOS			(eos),                   // 1-bit output: End of Sequence output
      .JTAGBUSY	(jtag_busy),         // 1-bit output: JTAG DRP transaction in progress output
      .JTAGLOCKED	(jtag_locked),     // 1-bit output: JTAG requested DRP port lock output
      .JTAGMODIFIED(jtag_modified), // 1-bit output: JTAG Write to the DRP has occurred output
      // Auxiliary Analog-Input Pairs: 16-bit (each) input: VAUXP[15:0], VAUXN[15:0]
      .VAUXN		(VAUX_N),               // 16-bit input: N-side auxiliary analog input
      .VAUXP		(VAUX_P),               // 16-bit input: P-side auxiliary analog input
      // Control and Clock Ports: 1-bit (each) input: Reset and Converstion Start
      .CONVST		(convst),             // 1-bit input: Convert start input
      .CONVSTCLK	(convstclk),       // 1-bit input: Convert start input
      .RESET		(reset),               // 1-bit input: Active-high reset input
      // DRP Ports: 7-bit (each) input: Dynamic Reconfiguration Ports
      .DADDR		(drp_addr),               // 7-bit input: DRP input address bus
      .DCLK			(drp_clk),                 // 1-bit input: DRP clock input
      .DEN			(drp_en),                   // 1-bit input: DRP input enable signal
      .DI			(drp_di),                     // 16-bit input: DRP input data bus
      .DWE			(drp_wen),                   // 1-bit input: DRP write enable input
      // Dedicated Analog Input Pair: 1-bit (each) input: VP/VN
      .VN			(V_N),                     // 1-bit input: N-side analog input
      .VP			(V_P)                      // 1-bit input: P-side analog input
   );
	end
	"VIRTEX5": begin: SYSMONV5
		// virtex 5
		SYSMON #(
      .INIT_40(INIT_40), // Configuration register 0
      .INIT_41(INIT_41), // Configuration register 1
      .INIT_42(INIT_42), // Configuration register 2
      .INIT_43(INIT_43), // Test register 0
      .INIT_44(INIT_44), // Test register 1
      .INIT_45(INIT_45), // Test register 2
      .INIT_46(INIT_46), // Test register 3
      .INIT_47(INIT_47), // Test register 4
      .INIT_48(INIT_48), // Sequence register 0
      .INIT_49(INIT_49), // Sequence register 1
      .INIT_4A(INIT_4A), // Sequence register 2
      .INIT_4B(INIT_4B), // Sequence register 3
      .INIT_4C(INIT_4C), // Sequence register 4
      .INIT_4D(INIT_4D), // Sequence register 5
      .INIT_4E(INIT_4E), // Sequence register 6
      .INIT_4F(INIT_4F), // Sequence register 7
      .INIT_50(INIT_50), // Alarm limit register 0
      .INIT_51(INIT_51), // Alarm limit register 1
      .INIT_52(INIT_52), // Alarm limit register 2
      .INIT_53(INIT_53), // Alarm limit register 3
      .INIT_54(INIT_54), // Alarm limit register 4
      .INIT_55(INIT_55), // Alarm limit register 5
      .INIT_56(INIT_56), // Alarm limit register 6
      .INIT_57(INIT_57), // Alarm limit register 7
      .SIM_MONITOR_FILE("") // Simulation analog entry file
   ) SYSMON_inst (
        .ALM			(alarm[2:0]),           // 3-bit output for temp, Vccint and Vccaux
        .OT				(overtemp),             // 1-bit output over temperature alarm
        .BUSY			(busy),         // 1-bit output ADC busy signal
        .CHANNEL		(channel),   // 5-bit output channel selection
        .DO				(drp_do),             // 16-bit output data bus for dynamic reconfig
        .DRDY			(drp_ready),         // 1-bit output data ready for dynamic reconfig
        .EOC			(eoc),           // 1-bit output end of conversion
        .EOS			(eos),           // 1-bit output end of sequence
        .JTAGBUSY		(jtag_busy), // 1-bit output JTAG DRP busy
        .JTAGLOCKED	(jtag_locked), // 1-bit output DRP port lock
        .JTAGMODIFIED(jtag_modified), // 1-bit output JTAG write to DRP
        .CONVST		(convst),     // 1-bit input convert start
        .CONVSTCLK	(convst), // 1-bit input convert start clock
        .DADDR			(drp_addr),       // 7-bit input address bus for dynamic reconfig
        .DCLK			(drp_clk),         // 1-bit input clock for dynamic reconfig
        .DEN			(drp_en),           // 1-bit input enable for dynamic reconfig
        .DI				(drp_di),             // 16-bit input data bus for dynamic reconfig
        .DWE			(drp_wen),           // 1-bit input write enable for dynamic reconfig
        .RESET			(reset),       // 1-bit input active high reset
        .VAUXN			(VAUX_N),       // 16-bit input N-side auxiliary analog input
        .VAUXP			(VAUX_P),       // 16-bit input P-side auxiliary analog input
        .VN				(V_N),             // 1-bit input N-side analog input
        .VP				(V_P)              // 1-bit input P-side analog input
   );
	assign alarm[7:3] = 0;
	end
	default: begin: ICAPDEF
      assign alarm[7:0] = 0;      
		assign overtemp = 0;		
   end	
	endcase
endgenerate

endmodule
