/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale      1ns / 10ps
`default_nettype none

module time_sim_logger #(
    parameter       WIDTH 		= 36,
	parameter		LOG_INPUTS 	= 1,
	parameter 		FILENAME 	= "time"
)(
    input wire                  clk,
	input wire [LOG_INPUTS-1:0]	log,
    output wire                 overflow
);

localparam
    COUNTER_MAX = {WIDTH{1'b1}},
    COUNTER_MIN = {WIDTH{1'b0}},
	LOG_INIT 	= {LOG_INPUTS{1'b0}};

reg 	[WIDTH-1:0]     	counter;
wire 	[WIDTH-1:0]    		counter_next;
reg 	[LOG_INPUTS-1:0] 	prev_log;

assign overflow = (counter == COUNTER_MAX);
assign counter_next = (counter + 1'b1) & {WIDTH{1'b1}}; 

initial begin
   	counter <= COUNTER_MIN;
	prev_log <= LOG_INIT;
end


always @(posedge clk) 
begin
	counter <= counter_next;
	prev_log <= log;
end


genvar i;
generate
	for (i = 0; i < LOG_INPUTS; i = i+1) begin: asdf
		integer timeFile[LOG_INPUTS:0];
		initial begin
			$display("Writing STDOUT to %s",$sformatf("%s_%0d.txt",FILENAME, i));
			timeFile[i] = $fopen($sformatf("%s_%0d.txt",FILENAME, i), "w");
		end

		always @(posedge clk)
		begin
			if (log[i] != prev_log[i])
			begin
				$fwrite(timeFile[i],"%d\n",counter);
				$display("Write %dns to %s_%0d.txt", counter, FILENAME, i);
				$fflush(timeFile[i]);
			end
		end
	end
endgenerate


endmodule
