/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    11:37:26 01/18/2007 
// Module Name:    CycleGen 
//////////////////////////////////////////////////////////////////////////////////
`include "trace_port_analyzer/src/defs.v"


//*** Abkuerzungen ***
`define S_RES		o_state = 0;
//Adressselektion
`define S_A_INI 			//o_addrSel = `ADDR_INIT;
`define S_A_WRR	o_ramCS = {i_pbWrite,!i_pbWrite}; o_ramAddr = i_rowWrite;		//; o_addrSel = `ADDR_WRROW;
`define S_A_WRC	o_ramCS = {i_pbWrite,!i_pbWrite}; o_ramAddr = {i_colWrite[12:10], 1'b1, i_colWrite[9:0]};		//o_addrSel = `ADDR_WRCOL;
`define S_A_RDR	o_ramCS = {i_pbRead,!i_pbRead}; o_ramAddr = i_rowRead;														//o_addrSel = `ADDR_RDROW;
`define S_A_RDC	o_ramCS = {i_pbRead,!i_pbRead}; o_ramAddr = {i_colRead[12:10], 1'b0, i_colRead[9:0]};			//o_addrSel = `ADDR_RDCOL;
`define S_A_RE	 	o_ramCS = 2'b11; o_ramAddr = {i_rowRef[12:10], 1'b0, i_rowRef[9:0]};								//o_addrSel = `ADDR_REFR;
`define S_A_X				//o_addrSel = 3'bxxx;

//NOP
`define S_NOP		o_state = o_state + 1;	o_ramOP = `OP_NOP;		o_ramBA = 2'bxx;

//Active
`define S_ACT0		o_state = o_state + 1;	o_ramOP = `OP_ACTIVE;	o_ramBA = 2'd0;
`define S_ACT1		o_state = o_state + 1;	o_ramOP = `OP_ACTIVE;	o_ramBA = 2'd1;
`define S_ACT2		o_state = o_state + 1;	o_ramOP = `OP_ACTIVE;	o_ramBA = 2'd2;
`define S_ACT3		o_state = o_state + 1;	o_ramOP = `OP_ACTIVE;	o_ramBA = 2'd3;

//Write (mit AUTO PRECHARGE)
`define S_WR0		o_state = o_state + 1;	o_ramOP = `OP_WRITE;		o_ramBA = 2'd0;
`define S_WR1		o_state = o_state + 1;	o_ramOP = `OP_WRITE;		o_ramBA = 2'd1;
`define S_WR2		o_state = o_state + 1;	o_ramOP = `OP_WRITE;		o_ramBA = 2'd2;
`define S_WR3		o_state = o_state + 1;	o_ramOP = `OP_WRITE;		o_ramBA = 2'd3;

//Read (ohne AUTO PRECHARGE)
`define S_RD0		o_state = o_state + 1;	o_ramOP = `OP_READ;		o_ramBA = 2'd0;
`define S_RD1		o_state = o_state + 1;	o_ramOP = `OP_READ;		o_ramBA = 2'd1;
`define S_RD2		o_state = o_state + 1;	o_ramOP = `OP_READ;		o_ramBA = 2'd2;
`define S_RD3		o_state = o_state + 1;	o_ramOP = `OP_READ;		o_ramBA = 2'd3;

//Precharge
`define S_PRE0		o_state = o_state + 1;	o_ramOP = `OP_PRECH;		o_ramBA = 2'd0;
`define S_PRE1		o_state = o_state + 1;	o_ramOP = `OP_PRECH;		o_ramBA = 2'd1;
`define S_PRE2		o_state = o_state + 1;	o_ramOP = `OP_PRECH;		o_ramBA = 2'd2;
`define S_PRE3		o_state = o_state + 1;	o_ramOP = `OP_PRECH;		o_ramBA = 2'd3;

module CycleGen(

	//*** allgemeine Signale ***
	
	input wire 			i_clk,				//Systemtakt
	input wire			i_en,					//Enablesignal zum Abschlaten
	input wire 			i_res,				//Resetsignal
	
	//*** Signale fuer Zustandsuebergange ***
	
	input wire [1:0]		i_ba,					//Bank, auf der gelesen werden soll
	
	//Art der Zugriffssequenz (wird nur beim Sequenzstart abgefragt)
	input wire 			i_write,				//Schreibzugriff
	input wire 			i_read,				//Lesezugriff
	input wire 			i_readAble,			//duales Lesen moeglich
	
	//*** Ausgaenge ***
	
	//Speichersteuerung
	//output reg	[2:0]	o_addrSel,		//Adressauswahl an Adressmultiplexer
	//output reg	[1:0]	o_ba,				//Bankadresse
	//output reg	[2:0]	o_op,				//Operationscode fuer RAM
	//output reg			o_prech,			//Autoprecharge
	
	//Statusausgabe fuer Read- und Write-Controller
	output reg	[4:0]	o_state,			//Zustand der Zustandsmachine (aktueller Takt)
	output reg	[1:0]	o_mode,			//Mode, mit Delay fuer Daten (Zuweisung erst in 2. Takt)
	
	input wire	[2:0]			i_opInit,
	input wire	[1:0]			i_baInit,
	input wire	[13:0]		i_addrInit,
	
	input wire					i_pbWrite,
	input wire	[13:0]		i_rowWrite,
	input wire	[12:0]		i_colWrite,
	input wire					i_pbRead,
	input wire	[13:0]		i_rowRead,
	input wire	[12:0]		i_colRead,
	
	input wire	[13:0]		i_rowRef,
	
	//output reg [2:0]	o_ramOP,
	output wire		[2:0]		op,
	output reg [13:0]	o_ramAddr,
	output reg [1:0]	o_ramCS,
	//output reg [1:0]	o_ramBA
	output wire  [1:0]	bax
	);
	
	reg [1:0]ba;
	reg [1:0]mode;
	reg [1:0]o_ramBA;
	reg [2:0]o_ramOP;
	
	assign bax = i_en ? o_ramBA : i_baInit;
	assign op = i_en ? o_ramOP : i_opInit;

	always @(posedge i_clk)
		if (i_res)
			begin
				ba = 2'bxx;
				mode = `M_REFR;
				//o_op = `OP_NOP;
				//o_ba = 2'bxx;
				//o_prech = 1'bx;
				//o_addrSel = 3'bx00; 
				o_state = 0;
				o_mode = `M_REFR;
			end
		else if (!i_en)
			begin
				ba = 2'bxx;
				mode = `M_REFR;
				//o_op = `OP_NOP;
				//o_ba = 2'bxx;
				//o_prech = 1'bx;
				//o_addrSel = 3'bxxx; 
				o_state = 0;
				o_mode = `M_REFR;
				
				o_ramOP = i_opInit;
				o_ramCS = 2'b11;
				o_ramBA = i_baInit;
				o_ramAddr = i_addrInit;
			end
		else
			begin
				if (5'd0 == o_state)
				begin
					mode = { (i_read&i_readAble) , i_write};	
					ba = i_ba;
				end
				
				//if (5'd2 == o_state)
				if (5'd0 == o_state)
				begin
					o_mode = mode;
					//o_ba = i_ba;
				end
				
				casex ({mode,ba})
					
				//*** Refresh ***
					{`M_REFR,2'bxx}:
						case (o_state)
							5'd0:		begin	`S_ACT0	`S_A_RE		end
							5'd3:		begin	`S_ACT1	`S_A_RE		end
							5'd5:		begin	`S_ACT2	`S_A_RE		end
							5'd7:		begin	`S_ACT3	`S_A_RE		end
							
							5'd8:		begin	`S_PRE0	`S_A_RE		end
							5'd11:	begin	`S_PRE1	`S_A_RE		end
							5'd13:	begin	`S_PRE2	`S_A_RE		end
							5'd15:	begin	`S_PRE3	`S_A_RE		end
							
							5'd16:	begin	`S_NOP	`S_A_RE	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_RE		end
						endcase
					
				//*** Write ***
					{`M_WRITE,2'bxx}:
						case (o_state)
							5'd0:		begin	`S_ACT0	`S_A_WRR		end
							5'd2:		begin	`S_WR0	`S_A_WRC		end
							
							5'd4:		begin	`S_ACT1	`S_A_WRR		end
							5'd6:		begin	`S_WR1	`S_A_WRC		end
							
							5'd8:		begin	`S_ACT2	`S_A_WRR		end
							5'd10:	begin	`S_WR2	`S_A_WRC		end
							
							5'd12:	begin	`S_ACT3	`S_A_WRR		end
							5'd14:	begin	`S_WR3	`S_A_WRC		end	
							
							5'd15:	begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
				//*** Read ***
					{`M_READ,2'd0}:	//Bank 0
						case (o_state)
							5'd0:		begin	`S_ACT0	`S_A_RDR		end
							5'd2:		begin	`S_RD0	`S_A_RDC		end
							5'd3:		begin `S_PRE0	`S_A_X		end
							
							5'd7:		begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
					{`M_READ,2'd1}:	//Bank 1
						case (o_state)
							5'd4:		begin	`S_ACT1	`S_A_RDR		end
							5'd6:		begin	`S_RD1	`S_A_RDC		end
							5'd7:		begin `S_PRE1	`S_A_X		end
							
							5'd8:		begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
					{`M_READ,2'd2}:	//Bank 2
						case (o_state)
							5'd4:		begin	`S_ACT2	`S_A_RDR		end
							5'd6:		begin	`S_RD2	`S_A_RDC		end
							5'd7:		begin `S_PRE2	`S_A_X		end
							
							5'd8:		begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
					{`M_READ,2'd3}:	//Bank 3
						case (o_state)
							5'd7:		begin	`S_ACT3	`S_A_RDR		end
							5'd9:		begin	`S_RD3	`S_A_RDC		end
							5'd10:	begin `S_PRE3	`S_A_X		end
							
							5'd11:	begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
				//*** Dual ***
					{`M_DUAL,2'd0}:	//Bank 0
						case (o_state)
							5'd0:		begin	`S_ACT0	`S_A_WRR		end
							5'd2:		begin	`S_WR0	`S_A_WRC		end
							
							5'd4:		begin	`S_ACT1	`S_A_WRR		end
							5'd6:		begin	`S_WR1	`S_A_WRC		end
							
							5'd8:		begin	`S_ACT2	`S_A_WRR		end
							5'd10:	begin	`S_WR2	`S_A_WRC		end
							
							5'd11:	begin	`S_ACT0	`S_A_RDR		end
							5'd13:	begin	`S_RD0	`S_A_RDC		end
							5'd14:	begin	`S_PRE0	`S_A_X		end
							
							5'd15:	begin	`S_ACT3	`S_A_WRR		end
							5'd17:	begin	`S_WR3	`S_A_WRC		end
							
							5'd18:	begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
					{`M_DUAL,2'd1}:	//Bank 1
						case (o_state)
							5'd0:		begin	`S_ACT0	`S_A_WRR		end
							5'd2:		begin	`S_WR0	`S_A_WRC		end
							
							5'd4:		begin	`S_ACT1	`S_A_WRR		end
							5'd6:		begin	`S_WR1	`S_A_WRC		end
							
							5'd8:		begin	`S_ACT2	`S_A_WRR		end
							5'd10:	begin	`S_WR2	`S_A_WRC		end
							
							5'd12:	begin	`S_ACT3	`S_A_WRR		end
							5'd14:	begin	`S_WR3	`S_A_WRC		end
							
							5'd15:	begin	`S_ACT1	`S_A_RDR		end
							5'd17:	begin	`S_RD1	`S_A_RDC		end
							5'd18:	begin	`S_PRE1	`S_A_X		end
							
							5'd19:	begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
					{`M_DUAL,2'd2}:	//Bank 2
						case (o_state)
							5'd0:		begin	`S_ACT0	`S_A_WRR		end
							5'd2:		begin	`S_WR0	`S_A_WRC		end
							
							5'd3:		begin	`S_ACT2	`S_A_RDR		end
							5'd5:		begin	`S_RD2	`S_A_RDC		end
							5'd6:		begin	`S_PRE2	`S_A_X		end
							
							5'd7:		begin	`S_ACT1	`S_A_WRR		end
							5'd9:		begin	`S_WR1	`S_A_WRC		end
							
							5'd11:	begin	`S_ACT2	`S_A_WRR		end
							5'd13:	begin	`S_WR2	`S_A_WRC		end
							
							5'd15:	begin	`S_ACT3	`S_A_WRR		end
							5'd17:	begin	`S_WR3	`S_A_WRC		end
							
							5'd18:	begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
					{`M_DUAL,2'd3}:	//Bank 3
						case (o_state)
							5'd0:		begin	`S_ACT0	`S_A_WRR		end
							5'd2:		begin	`S_WR0	`S_A_WRC		end
							
							5'd4:		begin	`S_ACT1	`S_A_WRR		end
							5'd6:		begin	`S_WR1	`S_A_WRC		end
							
							5'd7:		begin	`S_ACT3	`S_A_RDR		end
							5'd9:		begin	`S_RD3	`S_A_RDC		end
							5'd10:	begin	`S_PRE3	`S_A_X		end
							
							5'd11:	begin	`S_ACT2	`S_A_WRR		end
							5'd13:	begin	`S_WR2	`S_A_WRC		end
							
							5'd15:	begin	`S_ACT3	`S_A_WRR		end
							5'd17:	begin	`S_WR3	`S_A_WRC		end
							
							5'd18:	begin	`S_NOP	`S_A_X	`S_RES	end
							
							default:	begin	`S_NOP	`S_A_X		end
						endcase
						
					default:		begin	`S_NOP	`S_A_X		end
				endcase
			end	

	/*** DUBUGAUSGABEN ***/
	//integer file;
	
	/*initial
	begin
		file = $fopen("ram.txt");
		$fdisplay(file, "Fileno.: %d", file);
	end*/
	/*
	always @(negedge i_clk)
	begin
	#1;	//verzoegerung fuer dateianordnung
		if (i_en & 0==o_state)
		$fdisplay(32,"----- Start of sequence: %d -----", { (i_read&i_readAble) , i_write} );
	end */
	
endmodule
