/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    17:15:34 01/24/2007 
// Module Name:    FRQCnt 
//////////////////////////////////////////////////////////////////////////////////
module FRQCnt(

	//*** allgemeine Signale ***
	
	i_clk,		//Systemtakt
	i_res,		//Resetsignal
	i_fin,		//Fertig, Aufzeichnungstiefe erreicht

	//*** Signale fuer Frequenzmessung ***
	
	i_tClk,		//Tracetakt
	o_frq			//Anzahl der Tracetake im Rahmen der Messung
	
	);

	parameter
		P_wide = 8,		//Breite des Zaehler mit Systemtakt (best. Messrahmen)
		P_add = 2;		//zusaetzliche Gruesse fuer das Messregister
	
	input wire i_res;
	input wire i_clk;
	input wire i_fin;
	input wire i_tClk;
	output reg [P_wide+P_add-1:0]o_frq;
	
	wire [P_wide-1:0]ref;
	wire [P_wide+P_add-1:0]cnt;
	
	reg resCnt;

	Cnt #(P_wide) iCnt_Ref(ref, i_clk, 1'b1, i_res);
	
	Cnt #(P_wide+P_add) iCnt_Cnt(cnt, i_tClk, 1'b1, (i_res|resCnt) );
	
	always @(ref, i_res)
		if (i_res)
		begin
			resCnt = 0;
			o_frq = 0;
		end
		else
			if ( !i_fin && (0 == ref) )
			begin
				o_frq = cnt;
				resCnt = 1;
			end
			else
				resCnt = 0;


endmodule
