/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    15:23:58 01/15/2007 
// Module Name:    IntBuffer 
//////////////////////////////////////////////////////////////////////////////////
module IntBuffer(

	//*** allgemeine Signale ***
	
	output wire			o_overflow,		//Pufferueberlauf
	
	input wire 			i_clk,			//Systemtakt
	input wire				i_clk90,
	input wire 			i_ddr,			//gibt an, ob DDR auf der Eingangsseite anliegt
	input wire 			i_recTrace,		//gibt an, ob aufgezeichnet werden soll
	input wire 			i_res,			//Resetsignal
	
	//*** Signale fuer RAM-Controller ***

	output wire [63:0]	o_dataH,			//Daten fuer High-Pegel
	output wire [63:0]	o_dataL,			//Daten fuer Low-Pegel
	output wire 			o_newData,		//neue Daten fuer Schreiben stehn bereit
	output wire 			o_readAble,		//gibt an, dass duales Lesen moeglich ist
	
	input wire 			i_fetchBuff,	//Daten vom RAM fertig ausgelesen (1 mal 128 Bit)
	input wire				i_enBuff,
	
	//*** Signale fuer ETM-Zelle am Arm ***

	input wire 			i_tClk,			//Tracetakt

	input wire  [31:0]	i_dataH,			//Daten von Hight-Pegel
	input wire  [31:0]	i_dataL			//Daten von Low-Pegel

	);
	
	/*** DUBUGAUSGABEN ***/
	//integer file;
	//integer file2;
	
	wire writeA0;
	wire writeA1;
	wire writeA2;
	wire writeA3;
	wire res;
	reg resA;
	wire [8:0]addrB;					//Adresse an Port B (9 Bit -> 512 * 2 Doppelworte)
	//wire [8:0]addrB_fin;				//Adresse an Port B (9 Bit -> 512 * 2 Doppelworte)
	wire [8:0]addrA_fin;			//tatsaechliche Adresse, die an allen Blocks anliegt
	wire enPortB;
	
	//assign enPortB = (i_fetchBuff|i_res) & (addrA_fin!=addrB) | ( (1==addrA_fin)&(0==addrB) );
	assign enPortB = i_enBuff & (addrA_fin!=addrB);
/*
	initial
	begin
		file = $fopen("output.txt");
		$fdisplay(file, "Fileno.: %d", file);
		
		file2 = $fopen("MonPortB.txt");
		$fdisplay(file2, "Fileno.: %d", file2);
		$fdisplay(file2, "           Time    | addrB | B0              ##  B1              ##  B2              ## B3");
		$fdisplay(file2, "--------------------------------------------------------------------------------------------------------");
		$fmonitor(file2, "%t %d -> [%h|%d|%d|%d] ## [%h|%d|%d|%d] ## [%h|%d|%d|%d] ## [%h|%d|%d|%d] ** EN=%b", $time,
			addrB,
			o_dataH[63:60], o_dataH[59:52], o_dataH[51:48], o_dataH[47:32],
			o_dataH[31:28], o_dataH[27:20], o_dataH[19:16], o_dataH[15:0],
			o_dataL[63:60], o_dataL[59:52], o_dataL[51:48], o_dataL[47:32],
			o_dataL[31:28], o_dataL[27:20], o_dataL[19:16], o_dataL[15:0],
			enPortB
			);
	end
	
	always @(posedge i_clk)
		if (writeA0 | writeA1 | writeA2 | writeA3)
		$fdisplay(file2, "%t ***** SYSTEM-CLOCK *****", $time);
		
	
	assign res = i_res | resA;

	always @(posedge i_tClk)
	begin
		if (writeA0 | writeA1 | writeA2 | writeA3)
			$fdisplay(file, "%t ....................ENCUE: %d <- [%h|%d|%d|%d] ## [%h|%d|%d|%d]    *** %b", $time,
				addrA_fin,
				i_dataH[31:28], i_dataH[27:20], i_dataH[19:16], i_dataH[15:0],
				i_dataL[31:28], i_dataL[27:20], i_dataL[19:16], i_dataL[15:0],
				{writeA0,writeA1,writeA2,writeA3}
				);
	end
	
	always @(posedge i_clk)
	begin
		if (i_fetchBuff)
		$fdisplay(file2,"%t TAKE: %d (next %d) -> [%h|%d|%d|%d] ## [%h|%d|%d|%d] ## [%h|%d|%d|%d] ## [%h|%d|%d|%d]", $time,
			(addrB-1), addrB,
			o_dataH[63:60], o_dataH[59:52], o_dataH[51:48], o_dataH[47:32],
			o_dataH[31:28], o_dataH[27:20], o_dataH[19:16], o_dataH[15:0],
			o_dataL[63:60], o_dataL[59:52], o_dataL[51:48], o_dataL[47:32],
			o_dataL[31:28], o_dataL[27:20], o_dataL[19:16], o_dataL[15:0],
			);
			
		/*if (5 == addrB)
		begin
			resA = 1;
			#2 resA = 0;
		end
		else
			resA = 0;*/
	//end

	//assign addrB_fin = (addrA_fin==addrB) ? addrA_fin-1 : addrB;

	/**********************************************************************************
	 * Port A - ETM-Seite
	 **********************************************************************************/
//*** Signaldeklaration ***
	wire [10:0]addrA;					//Adresse an Port A (9 Bit -> 512 + 2 Bit fuer Block)
	//wire [8:0] addrA_fin;			//tatsaechliche Adresse, die an allen Blocks anliegt
	
	wire [31:0]diA0;					//Dateneingang an Port A fuer Block 0 und 2
	wire [31:0]diA1;					//Dateneingang an Port A fuer Block 1 und 3
	
/*	wire writeA0;
	wire writeA1;
	wire writeA2;
	wire writeA3;*/

//*** Dateneingaenge ***
	//Demultiplexer fuer wahlweise DDR
	assign diA0 = i_dataH;
	assign diA1 = i_ddr ? i_dataL : i_dataH;

//*** Adresssignale ***
	//Adresszaehler
	Cnt #(11) iCnt_AddrA(addrA, i_tClk, i_recTrace, i_res);
	//Demultiplexer fuer tatsaechliche Adresse der RAMBlocks
	//bei nicht-DDR werden die unteren 2 Bits zur Blockwahl genutzt
	//und bei DDR das unterste, da hier immer die ersten oder die letzten
	//beiden Bloecke ueber das Schreibsignal ausgewaehlt werden
	assign addrA_fin = i_ddr ? addrA[9:1] : addrA[10:2];

//*** Write-Signale **
	//uber Schreibsignal des Blocks erfolgt die Blockauswahl
	assign writeA0 = i_ddr ? ~addrA[0] : (~addrA[1] & ~addrA[0]);		// nDDR: Addr=00
	assign writeA1 = i_ddr ?  addrA[0] : (~addrA[1] &  addrA[0]);		// nDDR: Addr=01
	assign writeA2 = i_ddr ? ~addrA[0] : ( addrA[1] & ~addrA[0]);		// nDDR: Addr=10
	assign writeA3 = i_ddr ?  addrA[0] : ( addrA[1] &  addrA[0]);		// nDDR: Addr=11
	
	/**********************************************************************************
	 * Port B - RAM-Seite
	 **********************************************************************************/
//*** Signaldeklaration ***
	//wire [8:0]addrB;					//Adresse an Port B (9 Bit -> 512 * 2 Doppelworte)
	
	wire [31:0]doB0;					//Datenausgang an Port B fuer Block 0
	wire [31:0]doB1;					//Datenausgang an Port B fuer Block 1
	wire [31:0]doB2;					//Datenausgang an Port B fuer Block 2
	wire [31:0]doB3;					//Datenausgang an Port B fuer Block 3

//*** Datenleitungen ***
	assign o_dataH = {doB0,doB1};
	assign o_dataL = {doB2,doB3};

//*** Adresssignale ***
	//Adresszaehler
	//Cnt #(9) iCnt_AddrB(addrB, i_clk, i_fetchBuff, i_res);
	Cnt #(9) iCnt_AddrB(addrB, i_clk90, i_fetchBuff, i_res);
	
	//*** Freigabe fuer neue Daten ****
	Differ iDiffer(o_newData, o_readAble, o_overflow, addrA_fin, addrB, i_res);
	
	/**********************************************************************************
	 * RAMBlocks
	 **********************************************************************************/
	ram_36_prim Block0	(
			//Port A - ETM-Seite
			.DIA	(	{4'hF,diA0}		),
			.ADDRA(	addrA_fin	),
			.ENA	(	1'b1			),
			.WEA	(	writeA0		),
			.SSRA	(	1'b0			),
			.CLKA	(	i_tClk		),
			//Port B - RAM-Seite
			.DOB	(	doB0			),
			.ADDRB(	addrB			),
			.ENB	(	enPortB),//addrA_fin!=addrB	),
			.WEB	(	1'b0			),
			.SSRB	(	1'b0			),
			.CLKB	(	i_clk90		),
			.DIB	(	{4'hF, 32'h00000000}	),
			.DOA	(					)
			);	

	ram_36_prim Block1	(
			//Port A - ETM-Seite
			.DIA	(	{4'hF,diA1}		),
			.ADDRA(	addrA_fin	),
			.ENA	(	1'b1			),
			.WEA	(	writeA1		),
			.SSRA	(	1'b0			),
			.CLKA	(	i_tClk		),
			//Port B - RAM-Seite
			.DOB	(	doB1			),
			.ADDRB(	addrB			),
			.ENB	(	enPortB),//addrA_fin!=addrB	),
			.WEB	(	1'b0			),
			.SSRB	(	1'b0			),
			.CLKB	(	i_clk90		),
			.DIB	(	{4'hF, 32'h00000000}	),
			.DOA	(					)
			);	

	ram_36_prim Block2	(
			//Port A - ETM-Seite
			.DIA	(	{4'hF,diA0}		),
			.ADDRA(	addrA_fin	),
			.ENA	(	1'b1			),
			.WEA	(	writeA2		),
			.SSRA	(	1'b0			),
			.CLKA	(	i_tClk		),
			//Port B - RAM-Seite
			.DOB	(	doB2			),
			.ADDRB(	addrB			),
			.ENB	(	enPortB),//addrA_fin!=addrB	),
			.WEB	(	1'b0			),
			.SSRB	(	1'b0			),
			.CLKB	(	i_clk90		),
			.DIB	(	{4'hF, 32'h00000000}	),
			.DOA	(					)
			);	

	ram_36_prim Block3	(
			//Port A - ETM-Seite
			.DIA	(	{4'hF,diA1}		),
			.ADDRA(	addrA_fin	),
			.ENA	(	1'b1			),
			.WEA	(	writeA3		),
			.SSRA	(	1'b0			),
			.CLKA	(	i_tClk		),
			//Port B - RAM-Seite
			.DOB	(	doB3			),
			.ADDRB(	addrB			),
			.ENB	(	enPortB),//addrA_fin!=addrB	),
			.WEB	(	1'b0			),
			.SSRB	(	1'b0			),
			.CLKB	(	i_clk90		),
			.DIB	(	{4'hF, 32'h00000000}		),
			.DOA	(					)
			);	

	/*** DUBUGAUSGABEN ***/
/*	integer file;
	
	initial
	begin
		file = $fopen("IntBuff.txt");
		$fdisplay(file, "Fileno.: %d", file);
		$fmonitor(file, "%t *** enPortB=%d", $time,enPortB);
	end
	
/*	always @(posedge i_tClk)
	begin
		//if (i_output)
		$fdisplay(file,"%t IN  PortA: DIA=%h#%h ADDR=%d WEA=%b%b%b%b", $time,
			diA0, diA1,
			addrA_fin,
			writeA0, writeA1, writeA2, writeA3
			);
	end*/

	/*always @(posedge i_clk90)
	begin
		if (enPortB)
		$fdisplay(file,"%t OUT PortB: DOB=%h#%h#%h#%h ADDR=%d", $time,
			doB0, doB1, doB2, doB3,
			addrB
			);
	end*/

endmodule
