/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    17:18:03 01/15/2007 
// Module Name:    LL 
//////////////////////////////////////////////////////////////////////////////////

`include "trace_port_analyzer/src/TPA_glb.v"

module Mux_Bus(o_out, i_sel, i_in);
	parameter P_busw = 8;

	output  reg		[P_busw-1:0]	o_out;
	input wire			 						i_sel;
	input wire				[2*P_busw-1:0]	i_in;

	always @(i_sel, i_in)
		if (0 == i_sel)
			o_out = i_in[P_busw-1 : 0];
		else
			o_out = i_in[2*P_busw-1 : P_busw];
	
endmodule

module Mux(o_out, i_sel, i_in);
	parameter P_wide = 1;

	output  reg							o_out;
	input wire			 [P_wide-1:0]		i_sel;
	input wire			[2**P_wide-1:0]	i_in;

	always @(i_sel, i_in)
		o_out = i_in[i_sel];
	
endmodule

module DeMux(o_out, i_sel, i_in);
	parameter P_wide = 1;

	output reg	[2**P_wide-1:0]	o_out;
	input wire  		[P_wide-1:0]		i_sel;
	input wire 								i_in;

	always @(i_sel, i_in)
	begin
		o_out <= 0;
		o_out[i_sel] <= i_in;
	end

endmodule

module Cnt(o_out, i_clk, i_en, i_res);
	parameter P_wide = 5;
	
	output reg [P_wide-1:0] o_out;
	input wire 						i_clk;
	input wire							i_en;
	input wire 						i_res;
	
	always @(posedge i_clk, posedge i_res)
		if (1 == i_res)
			o_out <= 0;
		else
			if (1 == i_en)
				o_out <= o_out + 1;
			
endmodule

module CntL(o_out, o_full, i_clk, i_en, i_limit, i_res);
	parameter P_wide = 5;
	
	output reg [P_wide-1:0] o_out;
	output reg					o_full;
	input wire 						i_clk;
	input wire							i_en;
	input wire		  [P_wide-1:0] i_limit;
	input wire 						i_res;
	
	always @(posedge i_clk, posedge i_res)
		if (i_res)
			o_out = 0;
		else
			if (i_en)
				if (o_out == i_limit)
					o_out = 0;
				else
					o_out = o_out + 1;

	always @(o_out,i_limit)
		if (o_out == i_limit)
			o_full = 1;
		else
			o_full = 0;

endmodule

module Differ(
	output wire o_data,
	output wire o_readAble,
	output wire o_overflow,
	input wire [8:0]i_addrA,
	input wire [8:0]i_addrB,
	input wire	i_res
	);
	
	reg 	traces;

	wire [8:0]diff;
	
	assign diff 		= (i_addrA - i_addrB);
	assign o_data 		= 15 < diff;
	assign o_readAble	= (diff < 256);
	assign o_overflow	= (0 == diff) & traces;
	
	always @(posedge i_addrA[0], posedge i_res)
		if (i_res)
			traces = 0;
		else
			if (!traces && (i_addrB < i_addrA) )
				traces = 1;

endmodule
