/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    15:12:30 01/15/2007 
// Module Name:    QA 
//////////////////////////////////////////////////////////////////////////////////
`define QUIET		00
`define ENDING		01	
`define TRACE		11

module QA(
	
	//*** allgemeine Signale ***
	
	output wire 			o_recTrace,		//gibt an, ob aufgezeichnet werden soll
	
	input wire				i_trace,			//gibt an, dass Aufzeichnung aktiviert ist
	input wire				i_res,			//Resetsignal
	
	//*** Signale zum TPA-Controller ***
	
	input wire	[23:0]	i_traceDepth,	//Tracetiefe in Schreibzyklen (16 Takte)
	output wire			o_fin,			//Fertig, Aufzeichnungstiefe erreicht
	
	//*** Datenleitungen zum Puffer ***
	
	output wire [31:0]	o_dataH,			//Daten fuer High-Pegel
	output wire [31:0]	o_dataL,			//Daten fuer Low-Pegel
	
	//*** Signale von ETM-Zelle am Arm ***

	input wire				i_ddr,
	input wire				i_clk,
	input wire				i_tClk,			//Tracetakt
	//output			o_internTClk,
	
	//Signale fuer High-Pegel
	input wire	 [7:0]	i_qualiH,		//Qualifier
	input wire	 [3:0]	i_pipeStatH,	//Pipeline Status
	input wire	 [15:0]	i_tracePktH,	//Trace-Pakete
	//Signale fuer Low-Pegel
	input wire	 [7:0]	i_qualiL,		//Qualifier
	input wire	 [3:0]	i_pipeStatL,	//Pipeline Status
	input wire	 [15:0]	i_tracePktL		//Trace-Pakete
	
	);
	
	wire  [3:0]	addCodeH;			//zusaetzliche Signale
	wire  [3:0]	addCodeL;			//zusaetzliche Signale
	wire  [3:0]	addCode;
	reg	[1:0]	state;
	reg	[1:0]	nextState;
	wire	[27:0]count;
	
	//vorerst ohne Analyser der Qualifier immer aufzeichnen
	assign o_recTrace = (`QUIET!=state);
	
	assign addCodeH = addCode;
	assign addCodeL = addCode;	

	assign addCode = {state,state};
	
	assign o_dataH = {addCodeH,i_qualiH,i_pipeStatH,i_tracePktH};
	assign o_dataL = {addCodeL,i_qualiL,i_pipeStatL,i_tracePktL};

	//assign o_internTClk = `ENDING==state ? i_clk : i_tClk;	

	//CntL #(28) iCntL_TraceDepth(count, o_fin, o_internTClk, o_recTrace, {i_traceDepth,4'hF}, i_res);
	CntL #(28) iCntL_TraceDepth(count, o_fin, i_tClk, o_recTrace, {i_traceDepth,4'hF}, i_res);
	
	always @(posedge i_clk, posedge i_res)
	if (i_res)
	begin
		state = `QUIET;
		nextState = `QUIET;
	end
	else
	begin
		case (state)
			`QUIET:
				if (i_trace)
					nextState = `TRACE;
			
			`TRACE:
				//if ( (!i_trace) || count == i_traceDepth )
				if ( o_fin )
					nextState = `ENDING;
			
			`ENDING:
				if ( (i_ddr && (5'h1F == count[4:0])) || (!i_ddr && (6'h1F == count[5:0])) )
					nextState = `QUIET;
		endcase
		
		state = nextState;
	end	
	
	/*** DUBUGAUSGABEN ***/
/*	reg [31:0] file;
	
	initial
	begin
		file = $fopen("packs.txt");
		$fdisplay(file, "Fileno.: %d", file);
	end
		
	always @(posedge i_tClk)
		$fdisplay(file, "%t Now Packs %d ## %d", $time, i_tracePktH, i_tracePktL);
*/
endmodule
