/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    11:58:43 01/25/2007
// Module Name:    RAMCtrl
//////////////////////////////////////////////////////////////////////////////////

`include "trace_port_analyzer/src/defs.v"

module RAMCtrl(

	//*** allgemeine Signale ***

	output wire 	 				o_fetchBuff,	//Zeigerinkrement im Buffer an RAM-Seite
	output wire 	 				o_enBuff,		//Zeigerinkrement im Buffer an RAM-Seite

	input wire						i_clk,
	input wire						i_clk90,
	input wire						i_clk180,
	input wire						i_clk270,
	input wire						i_res,
	input wire						i_readAble,
	input wire						i_loadRes,
	input wire						i_enInitRam,
	input wire						i_fetch,
	input wire						i_resFetch,
	input wire						i_write,

	input wire			[63:0]		i_writeDataH,
	input wire			[63:0]		i_writeDataL,

	//*** interne Parameter

	output wire 					o_initRuns,

	input wire			[12:3]		i_cols,
	input wire			[13:0]		i_rows,
	input wire						i_pb,

	//*** Staussignale ***

	output wire 		[15:0]		o_dataRead,
	output wire						o_validData,

	//*** Signale zum RAM ***

	output wire 		[2:0]		o_ramOP,
	output wire 		[13:0]		o_ramAddr,
	output wire 		[1:0]		o_ramCS,
	output wire 		[1:0]		o_ramBA,
	output wire 		[3:0]		o_ramDM,

	inout wire			[60:0]		io_ramDQ,
	inout wire			[3:0]		io_ramDQS

	);

	parameter	P_SYSPERIOD = 10;						//Taktperiode


	wire			read;

	wire [1:0]	baCyc;
	wire [2:0]	opCyc;
	wire			prech;
	wire [2:0]	addrSel;
	wire [1:0]	mode;
	wire [4:0]	state;

	wire [13:0]	addrInit;
	wire [1:0]	baInit;
	wire [2:0]	opInit;

	wire [12:0]	colRead;
	wire [13:0]	rowRead;
	wire			pbRead;
	wire [1:0]	baRead;

	wire [13:0]	rowRef;

	wire [13:0]	rowWrite;
	wire [12:0]	colWrite;
	wire 			pbWrite;

	wire [63:0]	readDataH;
	wire [63:0]	readDataL;

	wire			ceDQ;
	wire			ceDQS;

	CycleGen iCycleGen (
		.i_clk		(	i_clk				),
		.i_en			(	i_enInitRam & !o_initRuns		),
		.i_res		(	i_res				),
		.i_ba			(	baRead			),
		.i_write		(	i_write			),
		.i_read		(	read				),
		.i_readAble	(	i_readAble		),
		//.o_addrSel	(	addrSel			),
		//.o_ba			(	baCyc				),
		//.o_op			(	opCyc				),
		//.o_prech		(	prech				),
		.o_state		(	state				),
		.o_mode		(	mode				),

		.i_opInit	(	opInit			),
		.i_baInit	(	baInit			),
		.i_addrInit	(	addrInit			),
		.i_pbWrite	(	pbWrite			),
		.i_rowWrite	(	rowWrite			),
		.i_colWrite	(	colWrite			),
		.i_pbRead	(	pbRead			),
		.i_rowRead	(	rowRead			),
		.i_colRead	(	colRead			),
		.i_rowRef	(	rowRef			),

		.o_ramCS		(	o_ramCS			),
		//.o_ramOP		(	o_ramOP			),
		.op			(o_ramOP),
		//.o_ramBA		(	o_ramBA			),
		.bax		(	o_ramBA			),
		.o_ramAddr	(	o_ramAddr		)
		);

	RAMInit #(P_SYSPERIOD) iRAMInit (
		.o_runs		(	o_initRuns		),
		.i_clk		(	i_clk				),
		.i_res		(	i_loadRes		),
		.i_en			(	i_enInitRam		),
		.o_addr		(	addrInit			),
		.o_ba			(	baInit			),
		.o_op			(	opInit			)
		);

	ReadCtrl iReadCtrl (
		.i_clk		(	i_clk				),
		.i_cols		(	i_cols			),
		.i_rows		(	i_rows			),
		.i_pb			(	i_pb				),
		.i_res		(	i_res				),
		.o_read		(	read				),
		.o_ba			(	baRead			),
		.o_col		(	colRead			),
		.o_row		(	rowRead			),
		.o_pb			(	pbRead			),
		.i_mode		(	mode				),
		.i_state		(	state				),
		.i_fetch		(	i_fetch			),
		.i_resFetch	(	i_resFetch		),
		.o_data		(	o_dataRead		),
		.o_validData(	o_validData		),
		.i_data		(	{readDataH,readDataL}		)
		);

	RefGen iRefGen (
		.i_clk		(	i_clk				),
		.i_rows		(	i_rows			),
		.i_res		(	i_res				),
		.o_row		(	rowRef			),
		.i_mode		(	mode				),
		.i_state		(	state				)
		);

	WriteCtrl iWriteCtrl (
		.i_clk		(	i_clk				),
		.i_clk90		(	i_clk90			),
		.i_clk180	(	i_clk180			),
		.i_clk270	(	i_clk270			),
		.i_res		(	i_res				),
		.i_rows		(	i_rows			),
		.i_cols		(	i_cols			),
		.i_pb			(	i_pb				),
		.i_ba			(	baRead			),
		.i_mode		(	mode				),
		.i_state		(	state				),
		.o_row		(	rowWrite			),
		.o_col		(	colWrite			),
		.o_pb			(	pbWrite			),
		.o_fetchBuff(	o_fetchBuff		),
		.o_enBuff	(	o_enBuff			),
		.o_ceDQ		(	ceDQ				),
		.o_ceDQS		(	ceDQS				)
		);

	RAM_DDR_IO iRAM_DDR_IO (
		.i_clk		(	i_clk				),
		.i_clk90		(	i_clk90			),
		.i_clk180	(	i_clk180			),
		.i_clk270	(	i_clk270			),
		.i_res		(	i_res				),
		.i_ceDQ		(	ceDQ				),
		.i_ceDQS		(	ceDQS				),
		.i_dataH		(	i_writeDataH	),
		.i_dataL		(	i_writeDataL	),
		.o_dataH		(	readDataH		),
		.o_dataL		(	readDataL		),
		.io_ramDQ	(	io_ramDQ			),
		.io_ramDQS	(	io_ramDQS		)
	);

	assign o_ramDM = 0;

	//Multiplexer fuer
	/*always @(addrSel, opCyc, baCyc, prech,
				opInit, baInit, addrInit,
				pbRead,  rowRead,  colRead,
				pbWrite, rowWrite, colWrite,
				rowRef)
	begin
		casex (addrSel)
			`ADDR_INIT:
				begin
					o_ramCS   = 2'b11;
					o_ramOP   = opInit;
					o_ramBA	 = baInit;
					o_ramAddr = addrInit;
				end

			`ADDR_WRROW:
				begin
					o_ramCS   = {pbWrite , !pbWrite };
					o_ramOP   = opCyc;
					o_ramBA	 = baCyc;
					o_ramAddr = rowWrite;
				end

			`ADDR_WRCOL:
				begin
					o_ramCS   = {pbWrite , !pbWrite };
					o_ramOP   = opCyc;
					o_ramBA	 = baCyc;
					o_ramAddr[13:11] = colWrite[12:10];
					o_ramAddr[10]    = prech;
					o_ramAddr[9:0]   = colWrite[9:0];
				end

			`ADDR_RDROW:
				begin
					o_ramCS   = {pbRead , !pbRead };
					o_ramOP   = opCyc;
					o_ramBA	 = baCyc;
					o_ramAddr = rowRead;
				end

			`ADDR_RDCOL:
				begin
					o_ramCS   = {pbRead , !pbRead };
					o_ramOP   = opCyc;
					o_ramBA	 = baCyc;
					o_ramAddr[13:11] = colRead[12:10];
					o_ramAddr[10]    = prech;
					o_ramAddr[9:0]   = colRead[9:0];
				end

			`ADDR_REFR:
				begin
					o_ramCS   = {i_pb , 1'b1};
					o_ramOP   = opCyc;
					o_ramBA	 = baCyc;
					o_ramAddr = rowRef;
				end

			default:
				begin
					o_ramCS   = 2'b00;
					o_ramOP   = 3'bxxx;
					o_ramBA	 = 2'bxx;
					o_ramAddr = 14'bxx_xxxx_xxxx_xxxx;
				end

		endcase
	end*/

endmodule
