/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    14:25:57 01/17/2007
// Module Name:    RAMInit
//////////////////////////////////////////////////////////////////////////////////

`include "trace_port_analyzer/src/defs.v"

module RAMInit(

	// *** allgemeine Signale ***

	output wire 					o_runs,	//gibt an, dass die Initialisierung im Gange ist

	input wire 					i_clk,	//Systemtakt
	input wire 					i_res,	//Resetsignal
	input wire						i_en,		//Enablesignal

	//*** Signale zum RAM ***

	output reg	[13:0]	o_addr,	//Adresssignale (werden hier fuer Daten verwendet)
	output reg	[1:0]		o_ba,		//Bankauswahl
	output reg	[2:0]		o_op		//Operationswort

	);

	parameter
		P_OFFS = 0,						//Offset, bis Initialisierungssequenz beginnt
		P_SYSPERIOD = 20,				//Taktperiode
		//P_200us = 200000 / P_SYSPERIOD +1;//Anzahl der Takte fuer 200�s
		P_200us = 2500 +1;//Anzahl der Takte fuer 200�s

	wire [15:0]	state;				//Interner Zustand (Taktzaehler)
	reg			fin;

	//Zaehler fuer Takte
	Cnt #(16) iCnt_State(state, i_clk , (i_en&o_runs), i_res);

	assign o_runs = i_en & !fin;

	always @(state, i_res)
		if (i_res)
		begin
			fin = 0;
			o_addr[10]  <= 0;
			o_addr[5:0] <= 6'd0;
			o_ba[0] <= 0;
			o_op <= `OP_NOP;
		end
		else
		begin
			o_ba[1] <= 0;
			o_addr[13:11] <= 0;
			o_addr[9:6]   <= 0;

			case (state)
				//200�s Delay
				P_OFFS+P_200us:
					begin
						o_addr[10]  <= 0;
						o_addr[5:0] <= 6'd0;
						o_ba[0] <= 0;
						o_op <= `OP_NOP;
					end

				//NOP
				P_OFFS+P_200us+ 1:
					begin
						o_addr[10]  <= 0;
						o_addr[5:0] <= 6'd0;
						o_ba[0] <= 0;
						o_op <= `OP_NOP;
					end

				//PRECHARGE ALL
				P_OFFS+P_200us+ 2:
					begin
						o_addr[10]  <= 1;					//Alle Baenke
						o_addr[5:0] <= 6'd0;
						o_ba[0] <= 0;
						o_op <= `OP_PRECH;
					end

				//Ext. Mode Register Set
				P_OFFS+P_200us+ 3:
					begin
						o_addr[10]  <= 0;
						o_addr[5:0] <= 6'b000_0_0_0;
						o_ba[0] <= 1;						//Extende Mode Register
						o_op <= `OP_MREGSET;
					end

				//normales Mode Register Set
				P_OFFS+P_200us+ 4:
					begin
						o_addr[10]  <= 0;
						o_addr[5:0] <= 6'b10_0_011;	//CL= 2 und Burst = 8
						o_ba[0] <= 0;						//normales Mode Register
						o_op <= `OP_MREGSET;
					end

				//PRECHARGE ALL
				P_OFFS+P_200us+ 205:
					begin
						o_addr[10]  <= 1;					//Alle Baenke
						o_addr[5:0] <= 6'd0;
						o_ba[0] <= 0;
						o_op <= `OP_PRECH;
					end

				//AUTOREFRESH
				P_OFFS+P_200us+ 206:
					begin
						o_addr[10]  <= 0;					//Alle Baenke
						o_addr[5:0] <= 6'd0;
						o_ba[0] <= 0;
						o_op <= `OP_AUREFR;
					end

				//AUTOREFRESH ALL
				P_OFFS+P_200us+ 207:
					begin
						o_addr[10]  <= 0;					//Alle Baenke
						o_addr[5:0] <= 6'd0;
						o_ba[0] <= 0;
						o_op <= `OP_AUREFR;
					end

				//Initialisierung fertig
				P_OFFS+P_200us+ 208:
					begin
						fin = 1;
						o_addr[10]  <= 0;
						o_addr[5:0] <= 6'd0;
						o_ba[0] <= 0;
						o_op <= `OP_NOP;
					end

				default:
					begin
						o_addr[10]  <= 0;
						o_addr[5:0] <= 6'd0;
						o_ba[0] <= 0;
						o_op <= `OP_NOP;
					end
			endcase
		end

endmodule
