/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    10:55:22 02/02/2007 
// Module Name:    DDR_IO 
//////////////////////////////////////////////////////////////////////////////////
module RAM_DDR_IO(

	//*** Taktsignale ***
	
	input wire 				i_clk,
	input wire 				i_clk90,
	input wire					i_clk180,
	input wire 				i_clk270,
	
	input wire					i_res,
	
	//*** Signale fuer intere Verarbeitung ***
	
	input wire					i_ceDQ,
	input wire					i_ceDQS,
	input wire		[63:0]	i_dataH,
	input wire		[63:0]	i_dataL,

	output wire	[63:0]	o_dataH,
	output wire	[63:0]	o_dataL,

	//*** Signale zu den IO_Pins ***
	inout wire		[60:0]	io_ramDQ,
	inout wire		[3:0]	io_ramDQS
	);
	
	wire [60:0]	out;
	wire [60:0]	in;
	wire [3:0]	dqsUB;
	reg 			resDQS;
	
	//wire			c1DQS;
	//reg			ceDQ_buf;
	
	genvar i;
	
	assign o_dataH[63:61] = 2'h0;
	assign o_dataL[63:61] = 2'h0;
	
	generate
		for (i = 0; i < 61; i = i+1)
		begin:padDQ
			//IO-Buffer
			iobuf_prim  buff (
				.IO(	io_ramDQ[i]	),
				.I	(	out[i]		),
				.O	(	in[i]			),
				.T	(	!i_ceDQS		)
			);
		
			//Ausgangsdaten fuer Schreiben
			//FDDRRSE outFF (
			oddr2_prim // #( .DDR_ALIGNMENT("C0") )
			outFF (
				.Q (	out[i]		),
				.D0(	i_dataH[i]	),
				.D1(	i_dataL[i]	),
				//(.C0(	i_clk270		),
				.C0(	!i_clk90		),
				.C1(	i_clk90		),
				.R	(	1'b0			),
				.S	(	1'b0			),
				.CE(	1'b1			)
			);
			
			//Eingangsdaten fuer Lesen
			iddr2_prim inFF (
				.D	(	in[i]			),
				.Q0(	o_dataH[i]	),
				.Q1(	o_dataL[i]	),
				.C0(	i_clk			),
				.C1(	!i_clk		),
				//.C1(	i_clk180		),
				.R	(	1'b0			),
				.S	(	1'b0			),
				.CE(	i_ceDQ		)
			);
			
		end
		
	endgenerate
	
	generate
		for (i = 0; i < 4; i = i+1)
		begin:padDQS
			obuft_prim dqsBuf (
				.I	(	dqsUB[i]			),
				.O	(	io_ramDQS[i]	),
				.T	(	!i_ceDQS			)
			);
			
			oddr2_prim #(.SRTYPE("ASYNC")) dqsFF (
				.Q (		dqsUB[i]		),
				.D0(		1'b0			),
				.D1(		1'b1			),
				//.C0(		i_clk180		),
				.C0(		!i_clk		),
				.C1(		i_clk			),
				.R	(		resDQS|!i_ceDQS	),
				.S	(		1'b0			),
				.CE(		1'b1			)
			);
		end
		
	endgenerate
	
	always @(negedge i_clk)
		resDQS = !i_ceDQS;
	
/*	always @(posedge i_ceDQS, posedge i_clk180)
		if (i_clk180)
			resDQS = 1'b0;		//Ruecksetzen - Ausgeloesst von 180�
		else
			resDQS = 1'b1;		//Impuls bei posedge i_ceDQS*/

	//assign preamble = i_output & !ceDQS;

	//assign ceDQS = i_output;
	/*always @(posedge i_clk90, posedge i_output)
		if (i_output)
			ceDQS = 1;
		else
			ceDQS = i_output;*/

	//always @(posedge i_clk270)
	//	dataL = i_dataL;
	
	//always @(posedge i_clk270)
	//	ceDQ = i_output;
		
	//assign ceDQ = ceDQ_buf;
	
	/*BUFG ceDQ_buf_inst(
		 .I (ceDQ_buf),
		 .O (ceDQ)
		 );*/

	//always @(posedge i_clk270)
	//	ceDQSpost = ceDQS;	
	
	/*** DUBUGAUSGABEN ***/
	/*integer file;
	
	initial
	begin
		file = $fopen("ram.txt");
		$fdisplay(file, "Fileno.: %d", file);
	end
	
	always @(posedge i_clk90, posedge i_clk270)
	begin
		if (i_output)
		$fdisplay(file,"%t [%h|%d|%d|%d] ## [%h|%d|%d|%d]", $time,
			io_ramDQ[61:60], io_ramDQ[59:52], io_ramDQ[51:48], io_ramDQ[47:32],
			io_ramDQ[31:28], io_ramDQ[27:20], io_ramDQ[19:16], io_ramDQ[15:0]
			);
	end*/

endmodule
