/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    15:53:21 01/25/2007
// Module Name:    TPA
//////////////////////////////////////////////////////////////////////////////////
module TPA(

	//*** Verbindungen mit dem MC ***

	input wire			clk_peri,
	input wire 	[17:0]	do_peri,
	output wire	[17:0]	di_peri,
	input wire	[9:0]	addr_peri,
	input wire			access_peri,
	input wire			wr_peri,
	input wire 			i_res,

	output wire			o_int,

	input wire			i_clk,
	input wire			i_clk90,
	input wire			i_clk180,
	input wire			i_clk270,

	//*** Verbindungen nach aussen auf Pins ***

	input wire			i_tClkIn,
	input wire	[7:0]	i_quali,
	input wire	[3:0]	i_pipeStat,
	input wire	[15:0]	i_tracePkt,

	//*** Signale zum RAM ***

	output wire 		o_ramRAS,
	output wire			o_ramCAS,
	output wire			o_ramWE,
	output wire	[13:0]	o_ramAddr,
	output wire [1:0]	o_ramCS,
	output wire	[1:0]	o_ramBA,
	output wire	[3:0]	o_ramDM,

	inout wire	[61:0]	io_ramDQ,
	inout wire	[3:0]	io_ramDQS,

	output wire	[2:0]	o_ck,
	output wire	[2:0]	o_ckn,
	output wire	[1:0]	o_cke

	);

	// Die Parameter fuer die Adressen werden in der system.v eingestellt.
	parameter BASE_ADR	= 80'h0;	//Basisadresse fuer TPA-Geraet
	parameter CONT_ADR	= 1;		//Anzahl der Adressen
	parameter READ_ADR	= 1;		//Anzahl der Eingabe Register

	parameter
		P_SYSPERIOD = 10;		//Taktperiode

	wire			res;
	wire			fin;
	wire	[9:0]	frq;
	wire			ovBuff;
	wire			ddr;
	wire			recTrace;
	wire			write;
	wire			readAble;
	wire			fetchBuff;
	wire			enBuff;
	wire	[31:0]	dataInH;
	wire	[31:0]	dataInL;
	wire			trace;
	wire	[23:0]	traceDepth;
	wire	[13:0]	rows;
	wire	[12:3]	cols;
	wire			pb;
	wire			initRuns;
	wire			validData;
	wire			enInitRam;
	wire	[15:0]	readData;
	wire	[63:0]	wrDataH;
	wire	[63:0]	wrDataL;

	wire			tClk;
	wire 			fetch, resFetch;

	TracePort iTracePort(
		.i_clk			(	i_clk			),
		.i_res			(	res			),
		.i_trace		(	trace			),
		.i_traceDepth		(	traceDepth		),
		.i_tClkIn		(	i_tClkIn		),
		.i_quali		(	i_quali			),
		.i_pipeStat		(	i_pipeStat		),
		.i_tracePkt		(	i_tracePkt		),
		.i_ddr			(	ddr			),
		.o_fin			(	fin			),
		.o_frq			(	frq			),
		.o_recTrace		(	recTrace		),
		.o_dataH		(	dataInH			),
		.o_dataL		(	dataInL			),
		.o_tClk			(	tClk			)

	);

	IntBuffer iIntBuffer (
		.o_overflow		(	ovBuff			),
		.i_clk			(	i_clk			),
		.i_clk90		(	i_clk90			),
		.i_ddr			(	ddr			),
		.i_recTrace		(	recTrace		),
		.i_res			(	res			),
		.o_dataH		(	wrDataH			),
		.o_dataL		(	wrDataL			),
		.o_newData		(	write			),
		.o_readAble		(	readAble		),
		.i_fetchBuff		(	fetchBuff		),
		.i_enBuff		(	enBuff			),
		.i_tClk			(	tClk			),
		.i_dataH		(	dataInH			),
		.i_dataL		(	dataInL			)
		);

	TPACtrl #(BASE_ADR) iTPACtrl (
		.o_res			(	res			),
		.o_trace		(	trace			),
		.i_res			(	i_res			),
		.i_clk			(	i_clk			),
		//.i_clk		(	clk_peri		),
		.o_data			(	di_peri			),
		.i_data			(	do_peri			),
		.i_addr			(	addr_peri		),
		.i_act_peri		(	access_peri		),
		.i_wr_peri		(	wr_peri			),
		.o_int			(	o_int			),
		.o_traceDepth		(	traceDepth		),
		.o_rows			(	rows			),
		.o_cols			(	cols			),
		.o_pb			(	pb			),
		.o_ddr			(	ddr			),
		.o_fetch		(	fetch			),
		.o_resFetch		(	resFetch		),
		.i_overflow		(	ovBuff			),
		.i_readAble		(	readAble		),
		.i_recTrace		(	recTrace		),
		.i_initRuns		(	initRuns		),
		.i_frq			(	frq			),
		.i_fin			(	fin			),
		.i_validData		(	validData		),
		.o_enInitRam		(	enInitRam		),
		.i_readData		(	readData		)
		);

	RAMCtrl #(P_SYSPERIOD) iRAMCtrl (
		.o_fetchBuff		(	fetchBuff		),
		.o_enBuff		(	enBuff			),
		.i_clk			(	i_clk			),
		.i_clk90		(	i_clk90			),
		.i_clk180		(	i_clk180		),
		.i_clk270		(	i_clk270		),
		.i_res			(	res			),
		.i_readAble		(	readAble		),
		.i_loadRes		(	i_res			),
		.i_enInitRam		(	enInitRam		),
		.i_fetch		(	fetch			),
		.i_resFetch		(	resFetch		),
		.i_writeDataH		(	wrDataH			),
		.i_writeDataL		(	wrDataL			),
		.i_write		(	write			),
		.o_initRuns		(	initRuns		),
		.i_cols			(	cols			),
		.i_rows			(	rows			),
		.i_pb			(	pb			),
		.o_dataRead		(	readData		),
		.o_validData		(	validData		),
		.o_ramOP		(	{o_ramRAS , o_ramCAS , o_ramWE} ),
		.o_ramAddr		(	o_ramAddr		),
		.o_ramCS		(	o_ramCS			),
		.o_ramBA		(	o_ramBA			),
		.o_ramDM		(	o_ramDM			),
		.io_ramDQ		(	io_ramDQ		),
		.io_ramDQS		(	io_ramDQS		)
		);

	assign o_ck[0]	= i_clk;
	assign o_ck[1]	= i_clk;
	assign o_ck[2]	= i_clk;
	assign o_ckn[0] = ~i_clk;
	assign o_ckn[1] = ~i_clk;
	assign o_ckn[2] = ~i_clk;

	assign o_cke[0] = 1;
	assign o_cke[1] = 1;

endmodule
