/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    16:37:15 02/07/2007 
// Module Name:    TracePort 
//////////////////////////////////////////////////////////////////////////////////
module TracePort(
	
	input wire					i_clk,
	input wire					i_res,
	input wire					i_trace,
	input wire		[23:0]	i_traceDepth,
	input wire					i_ddr,

	input wire					i_tClkIn,
	input wire		[7:0]		i_quali,
	input wire		[3:0]		i_pipeStat,
	input wire		[15:0]	i_tracePkt,
	
	output wire				o_fin,
	output wire	[9:0]		o_frq,
	output wire				o_recTrace,
	output wire	[31:0]	o_dataH,
	output wire	[31:0]	o_dataL,
	
	output wire				o_tClk
	
	);

	wire	[7:0]		qualiH;
	wire	[3:0]		pipeStatH;
	wire	[15:0]	tracePktH;
	wire	[7:0]		qualiL;
	wire	[3:0]		pipeStatL;
	wire	[15:0]	tracePktL;
	
	FRQCnt iFRQCnt (
		.i_clk			(	i_clk			),
		.i_res			(	i_res			),
		.i_tClk			(	i_tClkIn		),
		.i_fin			(	o_fin			),
		.o_frq			(	o_frq			)
		);

	Trace_DDR_IO iTrace_DDR_IO (
		.i_tClkIn		(	i_tClkIn		),
		.i_ddrQuali		(	i_quali		),
		.i_ddrPipeStat	(	i_pipeStat	),
		.i_ddrTracePkt	(	i_tracePkt	),
		.o_qualiH		(	qualiH		),
		.o_pipeStatH	(	pipeStatH	),
		.o_tracePktH	(	tracePktH	),
		.o_qualiL		(	qualiL		),
		.o_pipeStatL	(	pipeStatL	),
		.o_tracePktL	(	tracePktL	),
		.o_tClk			(	o_tClk		)
	);

	QA iQA (
		.o_recTrace		(	o_recTrace	),
		.i_trace			(	i_trace		),
		.i_res			(	i_res			),
		.i_traceDepth	(	i_traceDepth),
		.o_fin			(	o_fin			),
		.o_dataH			(	o_dataH		),
		.o_dataL			(	o_dataL		),
		.i_ddr			(	i_ddr			),
		.i_clk			(	i_clk			),
		.i_tClk			(	i_tClkIn		),
		//.o_internTClk	(	o_tClk		),
		.i_qualiH		(	qualiH		),
		.i_pipeStatH	(	pipeStatH	),
		.i_tracePktH	(	tracePktH	),
		.i_qualiL		(	qualiL		),
		.i_pipeStatL	(	pipeStatL	),
		.i_tracePktL	(	tracePktL	)
		);



endmodule
