/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 1ps
`default_nettype none
//////////////////////////////////////////////////////////////////////////////////
// Create Date:    16:21:51 02/07/2007 
// Module Name:    Trace_DDR_IO 
//////////////////////////////////////////////////////////////////////////////////
module Trace_DDR_IO(

	input wire					i_tClkIn,
	//input					i_res,
	
	input wire		[7:0]		i_ddrQuali,
	input wire		[3:0]		i_ddrPipeStat,
	input wire		[15:0]	i_ddrTracePkt,

	output wire	[7:0]		o_qualiH,
	output wire	[3:0]		o_pipeStatH,
	output wire	[15:0]	o_tracePktH,
	output wire	[7:0]		o_qualiL,
	output wire	[3:0]		o_pipeStatL,
	output wire	[15:0]	o_tracePktL,
	
	output wire				o_tClk
	
	);
	
	assign o_tClk = i_tClkIn;
	
	genvar i;

	generate
		for (i = 0; i < 8; i = i+1)
		begin:x
			//Qualifier
			iddr2_prim qualiFF (
				.D	(	i_ddrQuali[i]		),
				.Q0(	o_qualiH[i]			),
				.Q1(	o_qualiL[i]			),
				.C0(	i_tClkIn				),	//tClk90				),
				.C1(	!i_tClkIn			),	//!tClk90			),
				.R	(	1'b0					),
				.S	(	1'b0					),
				.CE(	1'b1					)
			);
		end
		
		for (i = 0; i < 4; i = i+1)
		begin:y
			//Pipeline-Staus
			iddr2_prim stateFF (
				.D	(	i_ddrPipeStat[i]	),
				.Q0(	o_pipeStatH[i]		),
				.Q1(	o_pipeStatL[i]		),
				.C0(	i_tClkIn				),	//tClk90				),
				.C1(	!i_tClkIn			),	//!tClk90			),
				.R	(	1'b0					),
				.S	(	1'b0					),
				.CE(	1'b1					)
			);
		end
		
		for (i = 0; i < 16; i = i+1)
		begin:z
			//Trace-Packete
			iddr2_prim pktFF (
				.D	(	i_ddrTracePkt[i]	),
				.Q0(	o_tracePktH[i]		),
				.Q1(	o_tracePktL[i]		),
				.C0(	i_tClkIn				),	//tClk90				),
				.C1(	!i_tClkIn			),	//!tClk90			),
				.R	(	1'b0					),
				.S	(	1'b0					),
				.CE(	1'b1					)
			);	
		end
		
	endgenerate

	/*** DEBUGAUSGABEN ***/
/*	integer file;
	
	initial
	begin
		file = $fopen("first.txt");
		$fdisplay(file, "Fileno.: %d", file);
	end
	
	always @(i_tClkIn)
		$fdisplay(file, "%t CLK=%b IN=%d OUT=%d##%d", $time, i_tClkIn, i_ddrTracePkt, o_tracePktH, o_tracePktL);
*/

endmodule
