/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */



`timescale 1ns / 1ps
`default_nettype none

module uart_testbench ();

reg			clk   = 0;
reg			reset = 0;
wire			uart_clk;
reg	[17:0]		send_data_in , empf_data_in;
wire	[18*2-1:0]	send_data_out, empf_data_out;
reg	[9:0]		send_adr = 10'h0;
reg	[9:0]		empf_adr = 10'h0;
wire			rx;
wire			tx;


initial
begin
	reset = 0; #1000
	reset = 1; #10000
	reset = 0; #1000

	send_adr = 10'h3ff; send_data_in = 18'h00000;
	empf_adr = 10'h3ff; empf_data_in = 18'h00000; #2000;
	send_adr = 10'h003; send_data_in = 18'h002f3; 
	empf_adr = 10'h003; empf_data_in = 18'h002f3; #2000;
	send_adr = 10'h3ff; #100;
	send_adr = 10'h002; send_data_in = 18'h1919;  #100;
	send_adr = 10'h3ff; #100;
end

always
begin
	#20;
	clk = !clk;
end

uart_bclk my_uart_clk (		
			.CLK_IN(	clk		),
			.RST(		reset		),
			.CLK_O(		uart_clk	)
		);

uart_top send_uart (
			.clk(		clk		),
			.uart_clk(	uart_clk	),
			.reset(		reset		),
			.data_in(	send_data_in	),
			.data_out(	send_data_out	),
			.adr(		send_adr	),
			.activ_peri(	1'b1		),
			.rx(		rx		),
			.tx(		tx		),
			.rts(				),
			.dcd(				),
			.dtr(				),
			.ir_tx(				),
			.ir_rx(				)
		);

uart_top empf_uart (
			.clk(		clk		),
			.uart_clk(	uart_clk	),
			.reset(		reset		),
			.data_in(	empf_data_in	),
			.data_out(	empf_data_out	),
			.adr(		empf_adr	),
			.activ_peri(	1'b1		),
			.rx(		tx		),
			.tx(		rx		),
			.rts(				),
			.dcd(				),
			.dtr(				),
			.ir_tx(				),
			.ir_rx(				)
		);

defparam send_uart.BASE_ADR = 7'h0;
defparam empf_uart.BASE_ADR = 7'h0;

endmodule
