/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


/*
Funktionen zur automatischen Berechnung der Teiler zur Erzeugung der Baudrate
fuer die UART. Diese erzeugt somit unabhaengig vom Eingangstakt immer die 
gleichen Baudraten.
*/

function signed abs;

	input integer value;
	 
	begin

	abs = value;

	if (value < 0) begin
		abs = -value;
		end
	
	end

endfunction

function integer log2;

	input integer value;
	
	for (log2=0; value>0; log2=log2+1)
		value = value>>1;
		
endfunction

function integer clkin_freq_ovs;

	input  integer clock_rate;
	input  integer ovs_rate;

	begin
		clkin_freq_ovs = (clock_rate*10)/ovs_rate;
	end

endfunction

function integer baud_teiler;

	input  integer ovs_freq;
	input  integer baud_rate;

	begin
		baud_teiler = (ovs_freq/baud_rate+5)/10;
	end

endfunction

function integer baud_error;

	input integer baud_rate;
	input integer clock_rate;
	input integer ovs_rate;
	input integer current_baud_error;

	integer ovs_freq;
	integer teiler;
	integer err;
	integer err_in;

	begin
//		$display("%d : %d : %d : %d",baud_rate,clock_rate,ovs_rate,current_baud_error);
		err_in	= current_baud_error;
		ovs_freq	= clkin_freq_ovs(clock_rate,ovs_rate);
		teiler	= baud_teiler(ovs_freq,baud_rate);
		err		= abs(((((ovs_freq/10)/teiler)-baud_rate)*100000/baud_rate));
//		$display("%d : %d : %d",ovs_freq,teiler,err);
		if (err > err_in)
			baud_error = err;
		else
			baud_error = err_in;
//		$display("%d",baud_error);
	end

endfunction

function integer max_baud_error;

	input integer clock_rate;
	input integer ovs_rate;

	integer max_baud_err;
	
	begin
	
		max_baud_err	= 0;
		
		max_baud_err	= baud_error(115200,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(57600,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(38400,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(31250,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(19200,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(9600,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(7812,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(4800,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(2400,clock_rate,ovs_rate,max_baud_err);
		max_baud_err	= baud_error(1200,clock_rate,ovs_rate,max_baud_err);
//		max_baud_err	= baud_error(600,clock_rate,ovs_rate,max_baud_err);
//		max_baud_err	= baud_error(300,clock_rate,ovs_rate,max_baud_err);
//		max_baud_err	= baud_error(150,clock_rate,ovs_rate,max_baud_err);
//		max_baud_err	= baud_error(75,clock_rate,ovs_rate,max_baud_err);
//		max_baud_err	= baud_error(50,clock_rate,ovs_rate,max_baud_err);
		max_baud_error	= max_baud_err;

	end
	
endfunction

function integer find_best_ovs_rate;

	input integer clock_rate;

	integer best_max_baud_error;
	integer best_ovs_rate;
	
	integer err;

	begin
		
		best_max_baud_error = max_baud_error(clock_rate,12);
		best_ovs_rate = 12;
		
		err = max_baud_error(clock_rate,14);
		if (err < best_max_baud_error)
		begin
			best_max_baud_error = err;
			best_ovs_rate = 14;
		end
		
		err = max_baud_error(clock_rate,16);
		if (err < best_max_baud_error)
		begin
			best_max_baud_error = err;
			best_ovs_rate = 16;
		end
		
		err = max_baud_error(clock_rate,18);
		if (err < best_max_baud_error)
		begin
			best_max_baud_error = err;
			best_ovs_rate = 18;
		end
		
		err = max_baud_error(clock_rate,20);
		if (err < best_max_baud_error)
		begin
			best_max_baud_error = err;
			best_ovs_rate = 20;
		end
		
		find_best_ovs_rate = best_ovs_rate;
		
	end
	
endfunction
