/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module uart_switcher	(
		locked,
		clk1,
		cpusel,
		select,
		dtr2,
		csel,
                rx,
                tx,
                spmc_rx,
                spmc_tx
		);

// Alle Parameter bis auf FACTORY_JF und LOC mit ihren Standardwerten
// laut Application Note
parameter	UART_COUNT	=	3;

input		wire locked;
input		wire clk1;

input		wire [UART_COUNT-1:0] cpusel;	// Anforderung von den CPUs
input		wire [UART_COUNT-1:0] select;	// Anforderung von den Schaltern
output		reg  [UART_COUNT-1:0] dtr2;	// Selektierung der UARTs
output		reg  [UART_COUNT-1:0] csel;	// Statusabfrage durch die CPUs
input		wire rx;
output		wire tx;

output		wire [UART_COUNT-1:0] spmc_rx;
input		wire [UART_COUNT-1:0] spmc_tx;

wire	[UART_COUNT-1:0] tmp_tx;

wire		reset;

reg	[UART_COUNT-1:0] dtr1;
reg		dtr1c;

assign	reset = !locked;

// entprellen der select Signale
always@(posedge clk1)
	if(reset) begin
		dtr1c	= 0;
		dtr1	= 1;
		dtr2	<= 0;
		csel	<= 0;
		end
	else begin
		// Ring Schieberegister um ein Bit verschieben
		{dtr1c,dtr1} = dtr1 << 1;
		dtr1[0]	= dtr1c;
		// CPU Anmeldung
		if (select == 0)
			csel	<= cpusel;
		else
			csel	<= select;
		// Select Register setzen
		if ((select == dtr1) || ((select == 0) && (csel == dtr1)))
			dtr2	<= dtr1;
	end

genvar i;
   generate
      for (i=0; i < UART_COUNT; i=i+1) 
      begin: RX
         assign spmc_rx[i] = dtr2[i] ? rx : 1'b1; 
	 assign tmp_tx[i]  = dtr2[i] ? spmc_tx[i] : 1'b1;
      end
	endgenerate

assign tx =  & tmp_tx;

endmodule
