/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 10ns / 1ns
`default_nettype none
module ma_filter(	
		clk_peri, 
		reset, 
		in, 
		out
	);

	// ld-Funktion (dualer Logarithmus, aufgerundet)
	function integer ld(input integer value);
		integer i;
		begin
			for (i = 0; value > 0; i = i + 1)
				value = value >> 1;
			ld = i;
		end
	endfunction

	parameter SAMPLE_REG_SIZE = 24;
	parameter SAMPLE_DELAY = 10;
	parameter SYSTEM_FREQUENCY = 25_000;   	// in kHz
	parameter FILTER_TRESHOLD = 18;
	localparam CC_PER_US = SYSTEM_FREQUENCY / 1_000;
	localparam CLK_DIV_WIDTH = ld(CC_PER_US);
	localparam US_CNT_WIDTH = ld(SAMPLE_DELAY);

	input 	wire	clk_peri, reset;
  	input 	wire	in;
  	output 	wire	out;

	reg [CLK_DIV_WIDTH-1:0] clk_div;
	reg smp_trg_reg;

	reg [US_CNT_WIDTH-1:0] us_cnt_reg;
	always @ (posedge clk_peri) begin
    		if (reset) begin
      			clk_div <= 0;
      			us_cnt_reg <= 0;
			smp_trg_reg <= 1'b0;
    		end else begin 
          		if (clk_div < (CC_PER_US-1)) begin
            			clk_div <= clk_div + 1;
          		end else begin 
            			clk_div <= 0;
				if (us_cnt_reg < SAMPLE_DELAY) begin
	            			us_cnt_reg <= us_cnt_reg + 1;
					smp_trg_reg <= 1'b0;
				end else begin
					us_cnt_reg <= 0;
					smp_trg_reg <= 1'b1;
				end
          		end
        	end
    	end

	wire smp_trg_edge;
	wire smp_trg;
	edge_detector E0(	
		.clk_peri(clk_peri), 
		.reset(reset), 
		.det_sig(smp_trg_reg), 
		.edge_ind(smp_trg_edge)
	);

	assign smp_trg = smp_trg_edge & smp_trg_reg;


//	wire [SAMPLE_REG_SIZE-1:0] reg_out;
	wire [SAMPLE_REG_SIZE:0] reg_out;
	shift_reg SR0 (
		.clk_peri(clk_peri), 
		.reset(reset), 
		.clk_en(smp_trg), 
		.in(in), 
		.out(reg_out)
	);
//	defparam SR0.SHIFT_REG_SIZE = SAMPLE_REG_SIZE;
	defparam SR0.SHIFT_REG_SIZE = SAMPLE_REG_SIZE + 1;

	//assign out = reg_out == {SAMPLE_REG_SIZE{1'b1}};

/*	reg [ld(SAMPLE_REG_SIZE)-1:0] sample_sum;
	integer i,sum;
	always @ (posedge clk_peri) begin
		if (reset) begin
			sample_sum <= 0;
		end else begin
			sum = 0;
			for (i=0; i < SAMPLE_REG_SIZE; i = i + 1) begin
				if (reg_out[i]) sum = sum + 1;
			end
			sample_sum <= sum;
		end
	end

	assign out = sum > (SAMPLE_REG_SIZE * 3 / 4);
*/

	wire sum_trg = smp_trg_edge & ~smp_trg_reg;

	reg 	[ld(SAMPLE_REG_SIZE)-1:0]	init_cnt, sum_of_ones;
	reg 								state, next_state;
	parameter							INIT = 1'b0;
	parameter							RUNNING = 1'b1;

	always @ (state or init_cnt) begin
		next_state = 0;
		case (state) 
			INIT: if (init_cnt < (SAMPLE_REG_SIZE-1)) begin
				next_state = INIT;
			end else begin
				next_state = RUNNING;
			end
			RUNNING:
				next_state = RUNNING;
			default: 
				next_state = INIT;
		endcase
	end

	always @ (posedge clk_peri) begin:OUTPUT_LOGIC
   		if(reset) begin
   			init_cnt <= 0;
			sum_of_ones <= 0;
   			state <= INIT;
   		end else begin
   			state <= next_state;
   			case (state)
    			INIT: begin
					if (sum_trg) begin	
		      			init_cnt <= init_cnt + 1;
						sum_of_ones <= sum_of_ones + reg_out[SAMPLE_REG_SIZE];
					end
				end
				RUNNING: begin
					if (sum_trg) sum_of_ones <= sum_of_ones + reg_out[SAMPLE_REG_SIZE] - reg_out[0];
	        	end
    			default: begin
	      			init_cnt <= 0;
					sum_of_ones <= 0;
    			end
  			endcase
		end
  	end

	assign out = sum_of_ones > FILTER_TRESHOLD;
	
endmodule
