/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 10ns / 1ns
`default_nettype none
module pulse_width_obs (
		clk_peri, 
		reset, 
		signal_in, 
		wide_pulse,
		narrow_pulse
	);

	// ld-Funktion (dualer Logarithmus, aufgerundet)
	function integer ld(input integer value);
		integer i;
		begin
			for (i = 0; value > 0; i = i + 1)
				value = value >> 1;
			ld = i;
		end
	endfunction

	parameter 	SYSTEM_FREQUENCY 	= 25_000;   	// in kHz
	parameter 	REQ_PULSE_WIDTH 	= 240;     	// geforderte Mindestpulsbreite (in us)
	localparam 	CC_PER_US 		= SYSTEM_FREQUENCY / 1_000;
	localparam 	CLK_DIV_WIDTH 		= ld(CC_PER_US);
	localparam 	US_CNT_WIDTH		= ld(REQ_PULSE_WIDTH);

	input 	wire 	clk_peri, reset, signal_in;  
	output 	wire 	wide_pulse;
	output	wire	narrow_pulse;
   

/*  inp_sampler IS0 (
    	.clk_peri(clk_peri),
    	.reset(reset),
    	.in(signal_in),
    	.out(wide_pulse)
  );
	defparam IS0.SAMPLE_REG_SIZE = 24;
	defparam IS0.SAMPLE_DELAY = 10;
*/	
	//defparam I0.SAMPLE_REG_SIZE = 25;
	//defparam I0.SAMPLE_DELAY = 20;
	

	reg 		[CLK_DIV_WIDTH-1:0] 	clk_div;
	reg 		[US_CNT_WIDTH:0]	us_cnt;
	reg 					wide_pulse_reg;
	reg						narrow_pulse_reg;

	reg 		[1:0]	state, next_state;
	parameter 	[1:0]	IDLE		=	2'b00;
	parameter	[1:0]	PULSE_PRESENT	=	2'b01;
	parameter	[1:0]	WIDE_ENOUGH	= 	2'b10;
	parameter 	[1:0]	TOO_NARROW	= 	2'b11;


	always @ (state or signal_in or us_cnt) begin
		next_state = 0;
		case(state)
      			IDLE: if(signal_in) begin
				next_state = PULSE_PRESENT;
			end else begin
				next_state = IDLE;
			end
			PULSE_PRESENT: if (signal_in && (us_cnt >= REQ_PULSE_WIDTH)) begin
				next_state = WIDE_ENOUGH;
			end else begin
				if (!signal_in && (us_cnt < REQ_PULSE_WIDTH)) begin
					next_state = TOO_NARROW;
				end else begin
					next_state = PULSE_PRESENT;
				end
			end
      			WIDE_ENOUGH: if (!signal_in) begin
				next_state = IDLE;
			end else begin
				next_state = WIDE_ENOUGH;
			end
				TOO_NARROW:	next_state = IDLE;
      			default: next_state = IDLE;
    		endcase
  	end
  
	always @ (posedge clk_peri) begin:OUTPUT_LOGIC
    		if(reset) begin
      			clk_div <= 0;
				us_cnt <= 0;
      			state <= IDLE;
				wide_pulse_reg <= 1'b0;
				narrow_pulse_reg <= 1'b0;
    		end else begin
      			state <= next_state;
      			case (state)
        			IDLE: begin
		      			clk_div <= 0;
						us_cnt <= 0;
						wide_pulse_reg <= 1'b0;
						narrow_pulse_reg <= 1'b0;
		        	end
					PULSE_PRESENT: begin
          				if (clk_div < (CC_PER_US-1)) begin
			            		clk_div <= clk_div + 1;
          				end else begin 
            					clk_div <= 0;
            					us_cnt <= us_cnt + 1;
          				end
						wide_pulse_reg <= 1'b0;
						narrow_pulse_reg <= 1'b0;
					end	
        			WIDE_ENOUGH: begin
		      			clk_div <= 0;
						us_cnt <= 0;
						wide_pulse_reg <= 1'b1;
						narrow_pulse_reg <= 1'b0;
		        	end
					TOO_NARROW:	begin
		      			clk_div <= 0;
						us_cnt <= 0;
						wide_pulse_reg <= 1'b0;
						narrow_pulse_reg <= 1'b1;
					end
        			default: begin
		      			clk_div <= 0;
						us_cnt <= 0;
						wide_pulse_reg <= 1'b0;
						narrow_pulse_reg <= 1'b0;
        			end
      			endcase
    		end
  	end
	
	assign wide_pulse = wide_pulse_reg;
	assign narrow_pulse = narrow_pulse_reg;
	
endmodule
