/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 10ns / 1ns
`default_nettype none
module us_con_gen(	
		clk_peri, 
		reset, 
		clk_40khz, 
		tune_frequency
	);

	// ld-Funktion (dualer Logarithmus, aufgerundet)
	function integer ld(input integer value);
		integer i;
		begin
			for (i = 0; value > 0; i = i + 1)
				value = value >> 1;
			ld = i;
		end
	endfunction

	parameter SYSTEM_FREQUENCY = 25_000;   	// in kHz
	parameter LOWEST_US_FREQUENCY = 35;		// in kHz
	localparam FREQ_REG_WIDTH = ld(SYSTEM_FREQUENCY/2/LOWEST_US_FREQUENCY);

	input 	wire				clk_peri;
	input 	wire				reset;
	output 	wire				clk_40khz;
	input 	wire	[FREQ_REG_WIDTH-1:0] 	tune_frequency;


	// Modul soll es ermöglichen die Frequenz zwischen 35 und 45 kHz einzustellen 
	//     	--> untere Grenze bestimmt die Breite des Taktteilers (35kHz)
	// 		--> größter Zählerwert des Taktteilers: fsys/35k/2
	reg [FREQ_REG_WIDTH-1:0] clk_div;

	reg clk_40khz_reg;

	// Erzeugen des 40kHz-Signals für den US-Lautsprecher (50% DutyCycle)	
	always @ (posedge clk_peri)
		if (reset) begin
			clk_div <= 0;
			clk_40khz_reg <= 0;
		end else begin 
			if (clk_div < tune_frequency) begin
			    clk_div <= clk_div + 1;
			end else begin 
			    clk_div <= 0;
			    clk_40khz_reg <= !clk_40khz_reg;
			end
		end
		
	assign clk_40khz = clk_40khz_reg;
    
endmodule
