/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 10ps
`default_nettype none

module test_bench;

reg		clk;
reg		rst;
reg		rst_phy;

wire		dp;
wire		dn;
pullup(dp);
pulldown(dn);
wire		locked;

wire		clk_phy;
wire		TxReady;
reg		TxValid;
reg	[7:0]	DataOut;
wire	[7:0]	DataIn;
reg	[7:0]	DataRecived;
wire		RxValid, RxActive,	RxError;
wire	[1:0]	LineState;

reg	[7:0]	packets [1<<6:0];
reg	[6:0]	index;
reg	[7:0]	size;
reg	[7:0]	start;
reg	[7:0]	wait_time;
reg	[7:0]	wait_loop;

reg	[3:0]	funct_sel;
reg			execute;
wire	[6:0]	result;
wire			upper_nibble;

initial
begin
	execute = 1'b1;
	funct_sel = 4'h0; #45000;
	execute = 1'b0; #400;
	execute = 1'b1;
end

initial
begin
	rst = 1'b0;
	TxValid = 1'b0;
	DataOut = 8'h00;
	DataRecived = 8'h00;
	rst_phy = 1'b0; #200;
	rst = 1'b0; #200;
	rst = 1'b1; #2000;
	rst_phy = 1'b1; #2000;

	$readmemh ("transfers.txt", packets);
	index = 0;

	while (packets[index]!=0)
	begin
		size = packets[index]; index = index +1;
		wait_time = packets[index]; index = index + 1;
		start = index;

		for( index = index ;index <= (start + size - 1); index = index + 1)
		begin
			DataOut = packets[index];
			TxValid = 1'b1;
			wait (TxReady);
			wait (!TxReady);
		end
		TxValid = 1'b0;
		for( wait_loop = 0; wait_loop <=wait_time; wait_loop = wait_loop + 1)
			#100;
	end
end

always
begin
	clk = 1'b0; #10;
	clk = 1'b1; #10;
end

always @(posedge RxValid)
	if (RxActive == 1'b1)
		DataRecived <= DataIn;

usb_phy		phy (	.clk(			clk_phy			),
			.rst(			rst_phy			),
			.phy_tx_mode(			1'b1			),
			.usb_rst(						),
			.dp(			dp							),
			.dn(			dn							),
			.DataOut_i(			DataOut			),
			.TxValid_i(			TxValid			),
			.TxReady_o(			TxReady			),
			.DataIn_o(			DataIn			),
			.RxValid_o(			RxValid			),
			.RxActive_o(			RxActive			),
			.RxError_o(			RxError			),
			.LineState_o(			LineState			)	);

usb_tester tester (
					.clk(		clk		),
					.clk_i(		clk_phy	),
					.rst(		rst		),
					.disconnect(			),
					.dp(			dp		),
					.dn(			dn		),
					.funct_sel(	funct_sel	),
					.execute(		execute	),
					.result(		result	),
					.upper_nibble(	upper_nibble )	);


endmodule
