/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale 1ns / 10ps
`default_nettype none

module usb1_dcm	(
      			RST_IN,
      			LOCKED_OUT,
     			CLKIN_IN,
      			CLKFX_OUT,
      			CLKIN_IBUFG_OUT,
      			CLK0_OUT
		);

parameter	CLKIN_DIVIDE_BY_2	=	"FALSE";
parameter	CLKDV_DIVIDE10X		=	20.0;
parameter	CLKFX_MULTIPLY		=	24;
parameter	CLKFX_DIVIDE		=	25;
parameter	CLKIN_PERIOD		=	20;
parameter	CLK_FEEDBACK		=	"1X";
parameter	CLKOUT_PHASE_SHIFT	=	"NONE";
parameter	DESKEW_ADJUST		=	"SYSTEM_SYNCHRONOUS";
parameter	DFS_FREQUENCY_MODE	=	"LOW";
parameter	DLL_FREQUENCY_MODE	=	"LOW";
parameter	DUTY_CYCLE_CORRECTION	=	"TRUE";
parameter	PHASE_SHIFT		=	0;
parameter	STARTUP_WAIT		=	"FALSE";
parameter	CLKDV_DIVIDE		=	CLKDV_DIVIDE10X / 10.0;
input wire		RST_IN;
input wire		CLKIN_IN;

output wire 		LOCKED_OUT;
output wire 		CLKFX_OUT;
output wire 		CLKIN_IBUFG_OUT;
output wire 		CLK0_OUT;

wire 		CLKFB_IN;
wire 		CLK0_BUF;
wire 		CLKFX_BUF;

assign 		CLKIN_IBUFG_OUT = CLKIN_IN;

clk_prim # (
		.CLKFX_DIVIDE(		CLKFX_DIVIDE		),
		.CLKFX_MULTIPLY(	CLKFX_MULTIPLY		),
		.CLKIN_PERIOD(		CLKIN_PERIOD		),
		.PHASE_SHIFT(		PHASE_SHIFT		),
		.INSTANTIATE_BUFGIN( "NO" ),
		.INSTANTIATE_BUFGFX( "YES")
		)
DCM_INST (
		.CLKIN (CLKIN_IN),
		.RST (RST_IN),
		.CLK0 (CLK0_OUT),
		.CLKFX (CLKFX_OUT),
		.LOCKED (LOCKED_OUT)
	);

endmodule
