/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none

module ssr_logic
#(
	parameter WIDTH = 9,
	parameter SRVAL= {WIDTH{1'b0}},
	parameter RAM_MODE="READ_FIRST"
)
(
	CLK,
	DO_in,
    DO_out,
	EN,
	WE,
	SSR
);

    input wire CLK;
    input wire [WIDTH-1:0] DO_in;
    output wire [WIDTH-1:0] DO_out;
    input wire EN;
    input wire WE;
    input wire SSR;

    wire[WIDTH-1:0] ram_out;
    reg [WIDTH-1:0] reg_oldval = {WIDTH{1'b0}};
    reg SSR_sync = 0;
    
    always @(posedge CLK) begin
        SSR_sync <= SSR;
    end
    
    assign ram_out = SSR_sync ? SRVAL : DO_in ;
generate
if (RAM_MODE=="NO_CHANGE") begin

    reg oldval_enable = 1'b0;
    assign DO_out = (oldval_enable) ? reg_oldval : ram_out;

    always @(posedge CLK) begin
        if( oldval_enable ) begin
            if(EN & SSR) begin
                reg_oldval <= SRVAL;
            end
            else begin
                reg_oldval <= reg_oldval;
            end
        end
        else begin
            if(EN & SSR) begin
                reg_oldval <= SRVAL;
            end
            else begin
                reg_oldval <= ram_out;
            end
        end
    end
    
    always @(posedge CLK) begin
        if( (!EN | (EN & WE)) & !oldval_enable) begin
            oldval_enable <= 1'b1;
        end
        else if(EN & !WE & oldval_enable) begin
            oldval_enable <= 1'b0;
        end
        else begin
            oldval_enable <= oldval_enable;
        end
    end

end else begin
    assign DO_out = ram_out;
end
endgenerate
endmodule
