/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none

module clk_prim
#(
	// default parameters for 60 MHz output frequency on ECP3 Evaluation Kit 
	// with 156.25 MHz onboard clk in
	parameter CLKFX_DIVIDE		= 	125,
	parameter CLKFX_MULTIPLY	= 	48,
	parameter SHIFT			= 	0.0, // has to be ASCII for simulation
	parameter CLKIN_PERIOD		=	6.4,
	parameter CLKIN_FREQ_STR	=	"156.250000",
	parameter CLKOUT_FREQ_FLT	=	60.000000,
	parameter CPHASE_STEPS		=	0, // needed for Lattice ECP5
	parameter FPHASE_STEPS          =	0 // needed for Lattice ECP5
)
(
	input wire CLKIN,
	input wire RST,

	output wire CLKFX,
	output wire LOCKED
);

//localparam CLKIN_FREQ_MHZ = (1.0 / (CLKIN_PERIOD+0.0)) * 1000.0 ;
localparam CLKIN_FREQ_MHZ = CLKIN_FREQ_STR;

// choose CLKOP divider to hold VCO frequency (CLKOUT_FREQ_MHZ / CLKOP_DIV) between 500 and 1000 MHz
// optimum VCO frequency is 1000 MHz
//localparam CLKOUT_FREQ_MHZ = CLKIN_FREQ_MHZ * CLKFX_MULTIPLY / CLKFX_DIVIDE;
localparam CLKOUT_FREQ_MHZ = CLKOUT_FREQ_FLT;


wire feedback;
wire CLK0;

wire  scuba_vlo = 1'b0;


// conversion from real to string (PHASEADJ parameter accepts string only)
// needed for simulation
localparam integer value = SHIFT * 10;

localparam digit_4 = value / 1000;
localparam digit_3 = (value / 100) - (digit_4 * 10);
localparam digit_2 = (value / 10) - (digit_4 * 100) - (digit_3 * 10);
localparam digit_1 = value - (digit_4 * 1000) - (digit_3 * 100)- (digit_2 * 10);

localparam digit_4_ascii = digit_4 + 48;
localparam digit_3_ascii = digit_3 + 48;
localparam digit_2_ascii = digit_2 + 48;
localparam digit_1_ascii = digit_1 + 48;

localparam integer digit4_nonzero = | digit_4;
localparam integer digit3_nonzero = | digit_3;

localparam SHIFT_ASCII = 80'd0 + (digit4_nonzero * (digit_4_ascii << (8 * 4))) + (digit3_nonzero * (digit_3_ascii << (8 * 3))) + (digit_2_ascii << (8 * 2)) + (8'd46 << (8 * 1)) + digit_1_ascii;

// supported phase shift values:
// 0.0
// 22.5
// 45.0
// 67.5
// 90.0
// 112.5
// 135.0
// 157.5
// 180.0
// 202.5
// 225.0
// 247.5
// 270.0
// 292.5
// 315.0
// 337.5

wire locked_internal;
assign LOCKED = locked_internal;

generate
	if (CLKOUT_FREQ_MHZ <= 7.8)
	begin
	localparam VCO_DIV = 128;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; // seems to have no impact on simulation
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ; // seems to have no impact on simulation, probably ascii conversion for synthesis needed

	// simulation in modelsim works only if parameter value is given in ascii converted format
	// for synthesis, it has to be checked if a real value (like it is passed by jconfig) or the ascii format is needed

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 8.9)
	begin
	localparam VCO_DIV = 112;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 10.4)
	begin
	localparam VCO_DIV = 96;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 12.5)
	begin
	localparam VCO_DIV = 80;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 15.6) 
	begin
	localparam VCO_DIV = 64;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 20.8)
	begin
	localparam VCO_DIV = 48;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 31.1)
	begin
	localparam VCO_DIV = 32;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 62.5)
	begin
	localparam VCO_DIV = 16;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 125)
	begin
	localparam VCO_DIV = 8;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 250)
	begin
	localparam VCO_DIV = 4;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else if (CLKOUT_FREQ_MHZ <= 500.0)
	begin
	localparam VCO_DIV = 2;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end
	else
	begin
	localparam VCO_DIV = 16;

	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "ENABLED" ; // CLKOK = CLK0 (input clock)
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ; // CLKOS = CLKFX ( can be multiplied, divided, shifted)
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ; // CLKOP not used, but needed as base clock for CLKOS
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ; // unused because CLKOK bypass enabled
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = SHIFT ; // only certain values are possible, others are treated like 0.0
	    defparam PLLInst_0.CLKOK_DIV = 2 ; // unused
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV ; 
	    defparam PLLInst_0.CLKFB_DIV = CLKFX_MULTIPLY ;
	    defparam PLLInst_0.CLKI_DIV = CLKFX_DIVIDE ;
	    defparam PLLInst_0.FIN = CLKIN_FREQ_MHZ ;

	// simulation in modelsim works only if parameter value is given in ascii converted format

	// synopsys translate_off
	  defparam PLLInst_0.PHASEADJ = SHIFT_ASCII; // accepts only certain real values in degrees as a string
	// synopsys translate_on

	    EHXPLLF PLLInst_0 (
				.CLKI(CLKIN), 
				.CLKFB(feedback), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(feedback), 
				.CLKOS(CLKFX), 
				.CLKOK(CLK0), 
				.CLKOK2(), 
				.LOCK(locked_internal), 
				.CLKINTFB());
	end

endgenerate

endmodule

//              /* synthesis FREQUENCY_PIN_CLKOP="25.000000" */
//              /* synthesis FREQUENCY_PIN_CLKOS="25.000000" */
//              /* synthesis FREQUENCY_PIN_CLKI="156.250000" */
//              /* synthesis FREQUENCY_PIN_CLKOK="156.250000" */
//			;


//     // exemplar begin
//     // exemplar attribute PLLInst_0 FREQUENCY_PIN_CLKOP 25.000000
//     // exemplar attribute PLLInst_0 FREQUENCY_PIN_CLKOS 25.000000
//     // exemplar attribute PLLInst_0 FREQUENCY_PIN_CLKI 156.250000
//     // exemplar attribute PLLInst_0 FREQUENCY_PIN_CLKOK 156.250000
//     // exemplar end
