/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none

module clk_sys_prim_lattice
#(
	parameter CLKIN_PERIOD = 6.4,
	parameter CLKIN_FREQ_STR	=	"156.250000",
	parameter CLKOUT_FREQ_FLT	=	156.250000
)(
	input wire CLKIN,
	input wire RST,

	output wire CLK0,
	output wire CLK90,
	output wire CLK180,
	output wire CLK2X,

	output wire LOCKED
);

wire LOCKED_0;
wire LOCKED_1;
wire LOCKED_2;

wire CLK0_FB;

wire  scuba_vlo = 1'b0;

assign LOCKED = LOCKED_0 & LOCKED_1 & LOCKED_2;

// choose CLKOP divider to hold VCO frequency (OUTPUT_FREQ / CLKOP divider) between 500 and 1000 MHz
// localparam OUTPUT_FREQ_1 = (1.0 / (CLKIN_PERIOD+0.0)) * 1000.0 ;

localparam OUTPUT_FREQ_1 = CLKOUT_FREQ_FLT;

generate

	if (OUTPUT_FREQ_1 <= 7.8)
	begin
	localparam VCO_DIV_1 = 128;
	localparam VCO_DIV_2 = 64;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 8.9)
	begin
	localparam VCO_DIV_1 = 112;
	localparam VCO_DIV_2 = 48;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 10.4)
	begin
	localparam VCO_DIV_1 = 96;
	localparam VCO_DIV_2 = 48;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 12.5)
	begin
	localparam VCO_DIV_1 = 80;
	localparam VCO_DIV_2 = 32;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 15.6) 
	begin
	localparam VCO_DIV_1 = 64;
	localparam VCO_DIV_2 = 32;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 20.8)
	begin
	localparam VCO_DIV_1 = 48;
	localparam VCO_DIV_2 = 16;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 31.1)
	begin
	localparam VCO_DIV_1 = 32;
	localparam VCO_DIV_2 = 16;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 62.5)
	begin
	localparam VCO_DIV_1 = 16;
	localparam VCO_DIV_2 = 8;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 125)
	begin
	localparam VCO_DIV_1 = 8;
	localparam VCO_DIV_2 = 4;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 250)
	begin
	localparam VCO_DIV_1 = 4;
	localparam VCO_DIV_2 = 2;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else if (OUTPUT_FREQ_1 <= 500.0)
	begin
	localparam VCO_DIV_1 = 2;
	localparam VCO_DIV_2 = 2; // out of spec, divider 1 would be needed for clk2x output

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end
	else
	begin
	localparam VCO_DIV_1 = 16;
	localparam VCO_DIV_2 = 8;

	//PLL_0 Outputs: CLK0,CLK180
	    defparam PLLInst_0.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_0.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_0.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_0.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_0.DELAY_VAL = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_0.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_0.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_0.DUTY = 8 ;
	    defparam PLLInst_0.PHASEADJ = "180.0" ;
	    defparam PLLInst_0.CLKOK_DIV = 2 ;
	    defparam PLLInst_0.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_0.CLKFB_DIV = 1 ;
	    defparam PLLInst_0.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_0 (
				  .CLKI(CLKIN), 
				  .CLKFB(CLK0), 
				  .RST(RST), 
				  .RSTK(scuba_vlo), 
				  .WRDEL(scuba_vlo), 
				  .DRPAI3(scuba_vlo), 
				  .DRPAI2(scuba_vlo), 
				  .DRPAI1(scuba_vlo), 
				  .DRPAI0(scuba_vlo), 
				  .DFPAI3(scuba_vlo), 
				  .DFPAI2(scuba_vlo), 
				  .DFPAI1(scuba_vlo), 
				  .DFPAI0(scuba_vlo), 
				  .FDA3(scuba_vlo), 
				  .FDA2(scuba_vlo), 
				  .FDA1(scuba_vlo), 
				  .FDA0(scuba_vlo), 
				  .CLKOP(CLK0), 
				  .CLKOS(CLK180), 
				  .CLKOK(), 
				  .CLKOK2(), 
				  .LOCK(LOCKED_0), 
				  .CLKINTFB());


	//PLL_1 Outputs: CLK90
	    defparam PLLInst_1.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_1.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_1.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_1.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_1.DELAY_VAL = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_1.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_1.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_1.DUTY = 8 ;
	    defparam PLLInst_1.PHASEADJ = "90.0" ;
	    defparam PLLInst_1.CLKOK_DIV = 2 ;
	    defparam PLLInst_1.CLKOP_DIV = VCO_DIV_1 ;
	    defparam PLLInst_1.CLKFB_DIV = 1 ;
	    defparam PLLInst_1.CLKI_DIV = 1 ;
	    defparam PLLInst_1.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_1 (
				.CLKI(CLK0), 
				.CLKFB(CLK0_FB), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK0_FB), 
				.CLKOS(CLK90), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_1), 
				.CLKINTFB());


		
	//PLL_2 Outputs: CLK2X
	    defparam PLLInst_2.FEEDBK_PATH = "CLKOP" ;
	    defparam PLLInst_2.CLKOK_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOS_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOP_BYPASS = "DISABLED" ;
	    defparam PLLInst_2.CLKOK_INPUT = "CLKOP" ;
	    defparam PLLInst_2.DELAY_PWD = "DISABLED" ;
	    defparam PLLInst_2.DELAY_VAL = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOS_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.CLKOP_TRIM_DELAY = 0 ;
	    defparam PLLInst_2.CLKOP_TRIM_POL = "RISING" ;
	    defparam PLLInst_2.PHASE_DELAY_CNTL = "STATIC" ;
	    defparam PLLInst_2.DUTY = 8 ;
	    defparam PLLInst_2.PHASEADJ = "0.0" ;
	    defparam PLLInst_2.CLKOK_DIV = 2 ;
	    defparam PLLInst_2.CLKOP_DIV = VCO_DIV_2 ;
	    defparam PLLInst_2.CLKFB_DIV = 2 ;
	    defparam PLLInst_2.CLKI_DIV = 1 ;
	    defparam PLLInst_0.FIN = CLKIN_PERIOD ;

	    EHXPLLF PLLInst_2 (
				.CLKI(CLK0), 
				.CLKFB(CLK2X), 
				.RST(RST), 
				.RSTK(scuba_vlo), 
				.WRDEL(scuba_vlo), 
				.DRPAI3(scuba_vlo), 
				.DRPAI2(scuba_vlo), 
				.DRPAI1(scuba_vlo), 
				.DRPAI0(scuba_vlo), 
				.DFPAI3(scuba_vlo), 
				.DFPAI2(scuba_vlo), 
				.DFPAI1(scuba_vlo), 
				.DFPAI0(scuba_vlo), 
				.FDA3(scuba_vlo), 
				.FDA2(scuba_vlo), 
				.FDA1(scuba_vlo), 
				.FDA0(scuba_vlo), 
				.CLKOP(CLK2X), 
				.CLKOS(), 
				.CLKOK(), 
				.CLKOK2(), 
				.LOCK(LOCKED_2), 
				.CLKINTFB());
	end

endgenerate

endmodule

//              /* synthesis FREQUENCY_PIN_CLKOP="156.250000" */
//              /* synthesis FREQUENCY_PIN_CLKOS="156.250000" */
//              /* synthesis FREQUENCY_PIN_CLKI="156.250000" */
//			  ;


//     // exemplar begin
//     // exemplar attribute PLLInst_0 FREQUENCY_PIN_CLKOP 156.250000
//     // exemplar attribute PLLInst_0 FREQUENCY_PIN_CLKOS 156.250000
//     // exemplar attribute PLLInst_0 FREQUENCY_PIN_CLKI 156.250000
//     // exemplar end


//              /* synthesis FREQUENCY_PIN_CLKOP="156.250000" */
//              /* synthesis FREQUENCY_PIN_CLKOS="156.250000" */
//              /* synthesis FREQUENCY_PIN_CLKI="156.250000" */
//			;


//     // exemplar begin
//     // exemplar attribute PLLInst_1 FREQUENCY_PIN_CLKOP 156.250000
//     // exemplar attribute PLLInst_1 FREQUENCY_PIN_CLKOS 156.250000
//     // exemplar attribute PLLInst_1 FREQUENCY_PIN_CLKI 156.250000
//     // exemplar end

//              /* synthesis FREQUENCY_PIN_CLKOP="312.500000" */
//              /* synthesis FREQUENCY_PIN_CLKI="156.250000" */
//			;

//     // exemplar begin
//     // exemplar attribute PLLInst_2 FREQUENCY_PIN_CLKOP 312.500000
//     // exemplar attribute PLLInst_2 FREQUENCY_PIN_CLKI 156.250000
//     // exemplar end
