/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none
 
module iobuf_prim
#(
parameter DRIVE = 12,
parameter IOSTANDARD = "DEFAULT", 
parameter SLEW = "SLOW"
) (
	O,
	IO,
	I,
	T
);

input wire I;
inout wire IO;
input wire T;

output wire O;

cycloneive_io_ibuf   ibufa_0( 
			.i(IO),
			.o(O)
			`ifndef FORMAL_VERIFICATION
			// synopsys translate_off
			`endif
			,
			.ibar(1'b0)
			`ifndef FORMAL_VERIFICATION
			// synopsys translate_on
			`endif
			);

	defparam
		ibufa_0.bus_hold = "false",
		ibufa_0.differential_mode = "false",
		ibufa_0.lpm_type = "cycloneive_io_ibuf";

cycloneive_io_obuf   obufa_0( 
			.i(I),
			.o(IO),
			.obar(),
			.oe(~T)
			`ifndef FORMAL_VERIFICATION
			// synopsys translate_off
			`endif
			,
			.seriesterminationcontrol({16{1'b0}})
			`ifndef FORMAL_VERIFICATION
			// synopsys translate_on
			`endif
			// synopsys translate_off
			,
			.devoe(1'b1)
			// synopsys translate_on
			);

	defparam
		obufa_0.bus_hold = "false",
		obufa_0.open_drain_output = "false",
		obufa_0.lpm_type = "cycloneive_io_obuf";

endmodule



//      original megafunction output


// module  iobuf01_iobuf_bidir_30p
// 	( 
// 	datain,
// 	dataio,
// 	dataout,
// 	oe) ;
// 	input   [0:0]  datain;
// 	inout   [0:0]  dataio;
// 	output   [0:0]  dataout;
// 	input   [0:0]  oe;
// 
// 	wire  [0:0]   wire_ibufa_o;
// 	wire  [0:0]   wire_obufa_o;
// 
// 	cycloneive_io_ibuf   ibufa_0
// 	( 
// 	.i(dataio),
// 	.o(wire_ibufa_o[0:0])
// 	`ifndef FORMAL_VERIFICATION
// 	// synopsys translate_off
// 	`endif
// 	,
// 	.ibar(1'b0)
// 	`ifndef FORMAL_VERIFICATION
// 	// synopsys translate_on
// 	`endif
// 	);
// 	defparam
// 		ibufa_0.bus_hold = "false",
// 		ibufa_0.differential_mode = "false",
// 		ibufa_0.lpm_type = "cycloneive_io_ibuf";
// 	cycloneive_io_obuf   obufa_0
// 	( 
// 	.i(datain),
// 	.o(wire_obufa_o[0:0]),
// 	.obar(),
// 	.oe(oe)
// 	`ifndef FORMAL_VERIFICATION
// 	// synopsys translate_off
// 	`endif
// 	,
// 	.seriesterminationcontrol({16{1'b0}})
// 	`ifndef FORMAL_VERIFICATION
// 	// synopsys translate_on
// 	`endif
// 	// synopsys translate_off
// 	,
// 	.devoe(1'b1)
// 	// synopsys translate_on
// 	);
// 	defparam
// 		obufa_0.bus_hold = "false",
// 		obufa_0.open_drain_output = "false",
// 		obufa_0.lpm_type = "cycloneive_io_obuf";
// 	assign
// 		dataio = wire_obufa_o,
// 		dataout = wire_ibufa_o;
// endmodule //iobuf01_iobuf_bidir_30p
