/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none
 
module oddr2_prim
#(
parameter DDR_ALIGNMENT = "NONE",
parameter INIT = 0,
parameter SRTYPE = "SYNC"
) (
	D0,
	D1,
	C0, 
	C1, // unused by megafunction, uses also falling edge of C0
	R,
	S,
	CE,
	Q
);


input wire D0;
input wire D1;
input wire C0;
input wire C1;
input wire CE;
input wire R;
input wire S;

output wire Q;


wire reset_internal;
wire set_internal;

generate
if(SRTYPE == "ASYNC")
begin

	assign reset_internal = R;
	assign set_internal = S;

end
else
begin

	
	reg reset_reg_c0;
	reg reset_reg_c1;
	reg set_reg;

	always @(posedge C1) begin
	reset_reg_c1 <= R;
	set_reg = S;
	end

	assign reset_internal = reset_reg_c1;
	assign set_internal = set_reg;

end
endgenerate


altddio_out	ALTDDIO_OUT_component (
			.aclr (reset_internal),
			.datain_h (D0),
			.datain_l (D1),
			.outclock (C0),
			.outclocken (CE),
			.dataout (Q),
			.aset (set_internal),
			.oe (1'b1),
			.oe_out (),
			.sclr (1'b0), // not available on Cylone IV E
			.sset (1'b0));
defparam
	ALTDDIO_OUT_component.extend_oe_disable = "OFF",
	ALTDDIO_OUT_component.intended_device_family = "Cyclone IV E",
	ALTDDIO_OUT_component.invert_output = "OFF",
	ALTDDIO_OUT_component.lpm_hint = "UNUSED",
	ALTDDIO_OUT_component.lpm_type = "altddio_out",
	ALTDDIO_OUT_component.oe_reg = "UNREGISTERED",
	ALTDDIO_OUT_component.power_up_high = "OFF",
	ALTDDIO_OUT_component.width = 1;

endmodule
