/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none

module ram_9_prim
#(
	parameter WRITE_MODE_A = "WRITE_FIRST",
	parameter INIT_A = 9'h000, // only for  xilinx (no function on altera)
	parameter SRVAL_A = 9'h000,
	parameter WRITE_MODE_B = "WRITE_FIRST",
	parameter INIT_B = 9'h000, // only for  xilinx (no function on altera)
	parameter SRVAL_B = 9'h000,
	parameter SIM_COLLISION_CHECK = "ALL" // for testing on xilinx (no function on altera)
)
(
	CLKA,
	CLKB,
	DOA,
	DOB,
	DIA,
	DIB,
	ADDRA,
	ADDRB,
	ENA,
	ENB,
	WEA,
	WEB,
	SSRA,
	SSRB
);

input	wire   CLKA;
input	wire   CLKB;
input	wire [8:0]  DIA;
input	wire [8:0]  DIB;
input	wire [10:0]  ADDRA;
input	wire [10:0]  ADDRB;
input	wire   ENA;
input	wire   ENB;
input	wire   WEA;
input	wire   WEB;
input	wire   SSRA;
input	wire   SSRB;

output	wire [8:0]  DOA;
output	wire [8:0]  DOB;


wire [8:0] DOA_internal;
wire [8:0] DOB_internal;

`include "vendor_prim/quartus/ram_translate_mode.vh"

            ssr_logic #(
            .WIDTH  (       9      ),
            .SRVAL  ( SRVAL_A      ),
	    .RAM_MODE(WRITE_MODE_A)
            ) ssr_logic_inst_a (
            .CLK    ( CLKA         ),
            .DO_in  ( DOA_internal ),
            .DO_out ( DOA          ),
            .EN     ( ENA          ),
            .WE     ( WEA          ),
            .SSR    ( SSRA         )
            );   
            
            ssr_logic #(
            .WIDTH  (       9      ),
            .SRVAL  ( SRVAL_B      ),
	    .RAM_MODE(WRITE_MODE_B)
            ) ssr_logic_inst_b (
            .CLK    ( CLKB         ),
            .DO_in  ( DOB_internal ),
            .DO_out ( DOB          ),
            .EN     ( ENB          ),
            .WE     ( WEB          ),
            .SSR    ( SSRB         )
            );   
            
	altsyncram 	ram_inst (
				.aclr0 (1'b0),
				.aclr1 (1'b0),
				.address_a (ADDRA),
				.address_b (ADDRB),
				.clock0 (CLKA),
				.clock1 (CLKB),
				.clocken0 (ENA),
				.clocken1 (ENB),
				.data_a (DIA),
				.data_b (DIB),
				.wren_a (WEA),
				.wren_b (WEB),
				.q_a (DOA_internal),
				.q_b (DOB_internal),
				.addressstall_a (1'b0),
				.addressstall_b (1'b0),
				.byteena_a (1'b1),
				.byteena_b (1'b1),
				.clocken2 (1'b1),
				.clocken3 (1'b1),
				.eccstatus (),
				.rden_a (1'b1),
				.rden_b (1'b1));

	defparam
		ram_inst.address_reg_b = "CLOCK1",
		ram_inst.clock_enable_input_a = "NORMAL",
		ram_inst.clock_enable_input_b = "NORMAL",
		ram_inst.clock_enable_output_a = "BYPASS",
		ram_inst.clock_enable_output_b = "BYPASS",
		ram_inst.indata_reg_b = "CLOCK1",
		ram_inst.intended_device_family = "Cyclone IV E",
		ram_inst.lpm_type = "altsyncram",
		ram_inst.numwords_a = 2048,
		ram_inst.numwords_b = 2048,
		ram_inst.operation_mode = "BIDIR_DUAL_PORT",
		ram_inst.outdata_aclr_a = "CLEAR0",
		ram_inst.outdata_aclr_b = "CLEAR1",
		ram_inst.outdata_reg_a = "UNREGISTERED",
		ram_inst.outdata_reg_b = "UNREGISTERED",
		ram_inst.power_up_uninitialized = "FALSE",
		ram_inst.ram_block_type = "M9K",
		ram_inst.read_during_write_mode_port_a = WRITE_MODE_A_TRANSLATED,
		ram_inst.read_during_write_mode_port_b = WRITE_MODE_B_TRANSLATED,
		ram_inst.widthad_a = 11,
		ram_inst.widthad_b = 11,
		ram_inst.width_a = 9,
		ram_inst.width_b = 9,
		ram_inst.width_byteena_a = 1,
		ram_inst.width_byteena_b = 1,
		ram_inst.wrcontrol_wraddress_reg_b = "CLOCK1";

endmodule
