/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
`timescale 1ns/1ps
`default_nettype none
module test_ram_modes();


reg clk=0;
reg [17:0] di;
reg we = 0;
wire [17:0] do_write_first;
wire [17:0] do_read_first;
wire [17:0] do_no_change;
reg reset = 1;



ram_18_prim #(

	.WRITE_MODE_A("WRITE_FIRST"),
	.INIT_A(18'h00000), 
	.SRVAL_A(18'h00000), 
	.WRITE_MODE_B("WRITE_FIRST"),
	.INIT_B(18'h00000), 
	.SRVAL_B(18'h00000), 
	.SIM_COLLISION_CHECK("ALL")

) ram_18_write_first(

	.CLKA(clk),
	.DIA(di),
	.ADDRA(10'd0),
	.ENA(1'b1),
	.WEA(we),
	.SSRA(1'b0),
	.DOA(do_write_first),


	//Port B not used
	.CLKB(clk),
	.DIB(18'd0),
	.ADDRB(10'd42),
	.ENB(1'b0),
	.WEB(1'b0),
	.SSRB(1'b0),
	.DOB()

);


ram_18_prim #(

	.WRITE_MODE_A("NO_CHANGE"),
	.INIT_A(18'h00000), 
	.SRVAL_A(18'h00000), 
	.WRITE_MODE_B("NO_CHANGE"),
	.INIT_B(18'h00000), 
	.SRVAL_B(18'h00000), 
	.SIM_COLLISION_CHECK("ALL")
) ram_18_no_change(

	.CLKA(clk),
	.DIA(di),
	.ADDRA(10'd0),
	.ENA(1'b1),
	.WEA(we),
	.SSRA(1'b0),
	.DOA(do_no_change),


	//Port B not used
	.CLKB(clk),
	.DIB(18'd0),
	.ADDRB(10'd42),
	.ENB(1'b0),
	.WEB(1'b0),
	.SSRB(1'b0),
	.DOB()

);


ram_18_prim #(

	.WRITE_MODE_A("READ_FIRST"),
	.INIT_A(18'h00000), 
	.SRVAL_A(18'h00000), 
	.WRITE_MODE_B("READ_FIRST"),
	.INIT_B(18'h00000), 
	.SRVAL_B(18'h00000), 
	.SIM_COLLISION_CHECK("ALL")

) ram_18_read_first(

	.CLKA(clk),
	.DIA(di),
	.ADDRA(10'd0),
	.ENA(1'b1),
	.WEA(we),
	.SSRA(1'b0),
	.DOA(do_read_first),


	//Port B not used
	.CLKB(clk),
	.DIB(18'd0),
	.ADDRB(10'd42),
	.ENB(1'b0),
	.WEB(1'b0),
	.SSRB(1'b0),
	.DOB()

);
always begin #5 clk = ~clk; end


task check;
input [1023:0] name;
input [17:0] expected;
input [17:0] actual;
begin
	if (expected===actual) begin
		$display("%0s: ok",name);
	end else begin
		$error("Error: %0s is %x, but was expected to be %x",name,actual, expected);
	end
end
endtask

initial begin
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	reset <= 0;
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	we<= 1;
	di<=18'h12341;
	@(negedge clk);
	we<=0;
	@(negedge clk);
	we<=1;
	di<=18'h1FFF2;
	@(negedge clk);
	di<=18'h1E0E3;
	@(negedge clk);
	we<= 0;

	check("NO_CHANGE", 18'h12341, do_no_change);
	check("READ_FIRST", 18'h1FFF2, do_read_first);
	check("WRITE_FIRST", 18'h1E0E3, do_write_first);

	@(negedge clk);
	$stop();

end

endmodule
