/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none

module clk_prim
#(
	// default parameters for 60 MHz output frequency on SP601 Evaluation Kit 
	// with 27 MHz onboard oscillator
	parameter CLKFX_DIVIDE		=	9,
	parameter CLKFX_MULTIPLY	=	20,
	parameter SHIFT			=	0.0, // highest valid input value 359.29
	parameter CLKIN_PERIOD		=	37.037,
	parameter CLKIN_FREQ_STR	=	"156.250000", // only needed for lattice ECP3
	parameter CLKOUT_FREQ_FLT	=	60.000000, // only needed for lattice ECP3
	parameter CPHASE_STEPS		=	0, // needed for Lattice ECP5
	parameter FPHASE_STEPS          =	0, // needed for Lattice ECP5
	parameter INSTANTIATE_BUFGFX    = "YES", // currently not selectable in jconfig
	parameter INSTANTIATE_BUFGIN    = "NO", // currently not selectable in jconfig
	parameter INSTANTIATE_BUFG0    = "NO" // currently not selectable in jconfig
)
(
	input wire CLKIN,
	input wire RST,

	output wire CLKFX,
	output wire LOCKED

);
// convert shift from degrees to fraction of 256
localparam integer PHASE_SHIFT = ((SHIFT+0.0) * 256.0) / 360.0 ;

wire CLKIN_dcm;
wire CLKFX_dcm;

wire CLK0;
wire CLK0_dcm;
wire CLKFB_dcm;

generate 
    if (INSTANTIATE_BUFGIN == "YES")
    begin
	BUFG BUFG_clkin_inst(
		.O (CLKIN_dcm),
		.I (CLKIN)
	);
    end
    else
    begin
	assign CLKIN_dcm = CLKIN;
    end
endgenerate

generate 
    if (INSTANTIATE_BUFGFX == "YES")
    begin
	BUFG BUFG_clkfx_inst(
		.O (CLKFX),
		.I (CLKFX_dcm)
	 );
    end
    else
    begin
	assign CLKFX = CLKFX_dcm;
    end
endgenerate

generate 
    if (INSTANTIATE_BUFG0 == "YES")
    begin
	BUFG BUFG_clk0_inst(
		.O (CLK0),
		.I (CLK0_dcm)
	);
    end
    else
    begin
	assign CLK0 = CLK0_dcm;
    end
endgenerate

DCM_SP #(
	.CLKFX_DIVIDE(		CLKFX_DIVIDE	),
	.CLKFX_MULTIPLY(	CLKFX_MULTIPLY	),
	.CLKIN_PERIOD(		CLKIN_PERIOD	),
	.CLKOUT_PHASE_SHIFT(    "FIXED"         ),
    	.PHASE_SHIFT(   	PHASE_SHIFT   	)
)
DCM_inst(
	.CLKIN(		CLKIN_dcm	),
	.CLKFB(		CLK0		),
	.RST(			RST		),
	.PSEN(			1'b0		),
	.PSINCDEC(		1'b0		),
	.PSCLK(		1'b0		),
	.DSSEN(		1'b0		),
	.CLK0(			CLK0_dcm	),
	.CLKFX(		CLKFX_dcm	),
	.LOCKED(		LOCKED		),
	.STATUS(),
	.PSDONE(),
	.CLKFX180(),
	.CLKDV(),
	.CLK90(),
	.CLK2X(),
	.CLK2X180(),
	.CLK270(),
	.CLK180() 
 );



endmodule
