/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none

module clk_sys_prim
#(
	parameter CLKIN_PERIOD		=	37.037,
	parameter CLKIN_FREQ_STR	=	"156.250000", // only needed for lattice
	parameter CLKOUT_FREQ_FLT	=	156.250000 // only needed for lattice
)
(
	input wire CLKIN,
	input wire RST,

	output wire CLK0,
	output wire CLK90,
	output wire CLK180,
	output wire CLK2X,

	output wire LOCKED

);


wire CLK0_dcm;
wire CLKIN_dcm;
wire CLK180_dcm;
wire CLK2X_dcm;



BUFG BUFG_clk_inst(
	.O (CLKIN_dcm),
	.I (CLKIN)
);


BUFG BUFG_clk1_inst(
	.O (CLK0),
	.I (CLK0_dcm)
);

BUFG BUFG_clk2_inst(
	.O (CLK180),
	.I (CLK180_dcm)
);

BUFG BUFG_clk2x_inst(
	.O (CLK2X),
	.I (CLK2X_dcm)
);


DCM #(
	.CLKIN_PERIOD(	CLKIN_PERIOD	)
)
DCM_inst(
	 .CLKIN(	CLKIN_dcm	),
	 .CLKFB(	CLK0		),
	 .RST(		RST		),
	 .PSEN(		1'b0		),
	 .PSINCDEC(	1'b0		),
	 .PSCLK(	1'b0		),
	 .DSSEN(	1'b0		),
	 .CLK0(		CLK0_dcm	),
	 .CLK90(	CLK90		),
	 .CLK180(	CLK180_dcm	),
	 .CLK2X(	CLK2X_dcm	),
	 .LOCKED(	LOCKED		)
 );


endmodule
