/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

 `default_nettype none

module ram_36_prim
#(
	parameter WRITE_MODE_A = "WRITE_FIRST",
	parameter INIT_A = 36'h000000000,  
	parameter SRVAL_A = 36'h000000000,  
	parameter WRITE_MODE_B = "WRITE_FIRST",
    parameter INIT_B = 36'h000000000,  
	parameter SRVAL_B = 36'h000000000,  
	parameter SIM_COLLISION_CHECK = "ALL"
)
(
	CLKA,
	CLKB,
	DOA,
	DOB,
	DIA,
	DIB,
	ADDRA,
	ADDRB,
	ENA,
	ENB,
	WEA,
	WEB,
	SSRA,
	SSRB
);


input wire		CLKA;
input wire		CLKB;
input wire	[35:0]	DIA;
input wire	[35:0]	DIB;
input wire	[8:0]	ADDRA;
input wire	[8:0]	ADDRB;
input wire		ENA;
input wire		ENB;
input wire		WEA;
input wire		WEB;
input wire		SSRA;
input wire		SSRB;

output wire	[35:0]	DOA;
output wire	[35:0]	DOB;


RAMB16_S36_S36  #(
	.WRITE_MODE_A(		WRITE_MODE_A		),
	.INIT_A(			INIT_A				),
	.SRVAL_A(			SRVAL_A				),
	.WRITE_MODE_B(		WRITE_MODE_B		),
	.INIT_B(			INIT_B				),
	.SRVAL_B(			SRVAL_B				),
	.SIM_COLLISION_CHECK(	SIM_COLLISION_CHECK	)
	)
ram_inst(	
	.DIA(		DIA[31:0]		),
	.DIPA(		DIA[35:32]		),
	.ADDRA(		ADDRA			),
	.ENA(		ENA			),
	.WEA(		WEA			),
	.SSRA(		SSRA			),
	.CLKA(		CLKA			),
	.DOA(		DOA[31:0]		),
	.DOPA(		DOA[35:32]		),
	.DIB(		DIB[31:0]		),
	.DIPB(		DIB[35:32]		),
	.ADDRB(		ADDRB			),
	.ENB(		ENB			),
	.WEB(		WEB			),
	.SSRB(		SSRB			),
	.CLKB(		CLKB			),
	.DOB(		DOB[31:0]		),
	.DOPB(		DOB[35:32]		)
);


endmodule
