/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

`default_nettype none

module virtexstartup(fpga_p15,fpga_n15);

output wire fpga_p15;
input wire fpga_n15;

// STARTUP_VIRTEX5: Startup primitive for accessing GSR, GTS, startup sequence
// control, SPI PROM pins, configuration clock and start-up status
// Virtex-5
// Xilinx HDL Libraries Guide, version 9.1i
STARTUP_VIRTEX5 STARTUP_VIRTEX5_inst (
.CFGCLK(),	 		// Config logic clock 1-bit output
.CFGMCLK(),			// Config internal osc clock 1-bit output
.DINSPI(fpga_p15),		// DIN SPI PROM access 1-bit output
.EOS(),				// End Of Startup 1-bit output
.TCKSPI(),			// TCK SPI PROM access 1-bit output
.CLK(),				// Clock input for start-up sequence
.GSR(1'b0),			// Global Set/Reset input (GSR can not be used as a port name)
.GTS(1'b0),			// Global 3-state input (GTS can not be used as a port name)
.USRCCLKO(fpga_n15),		// User CCLK 1-bit input
.USRCCLKTS(1'b0), 		// User CCLK 3-state 1-bit input
.USRDONEO(),		// User Done 1-bit input
.USRDONETS() 		// User Done 3-state 1-bit input
);
// End of STARTUP_VIRTEX5_inst instantiation


endmodule
