/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#ifndef __HARDWARE_BARRIER_H_
#define __HARDWARE_BARRIER_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <peripherals/hardware_barrier.h>

typedef hardware_barrier_regs_t hardware_barrier_t;

void waitForAll(hardware_barrier_t* reg);
void signalForAll(hardware_barrier_t* reg);
extern inline void defineYourGlobalVariablesInitialisationBehaviour();

/*
	The program MUST call one of the following macros outside the main fuction.

	option 1:   DONT_WAIT_FOR_INITIALISATION(hardware_barrier)
			OR
	option 2:   WAIT_FOR_INITIALISATION(hardware_barrier)
			OR
	option 3:   WAIT_FOR_INITIALISATION(bool, hardware_barrier) 	which only waits iff bool==T

*/
#define DONT_WAIT_FOR_INITIALISATION(hardware_barrier) 			inline void defineYourGlobalVariablesInitialisationBehaviour() { signalForAll(&hardware_barrier);  }
#define WAIT_FOR_INITIALISATION_1(hardware_barrier)				inline void defineYourGlobalVariablesInitialisationBehaviour() { signalForAll(&hardware_barrier); waitForAll(&hardware_barrier); }
#define WAIT_FOR_INITIALISATION_2(b, hardware_barrier)			inline void defineYourGlobalVariablesInitialisationBehaviour() { signalForAll(&hardware_barrier); if(b) waitForAll(&hardware_barrier); }




/*
	some crazy macro action to allow "calls" to WAIT_FOR_INITIALISATION(...) with one or two arguments

	#define WAIT_FOR_INITIALISATION(hardware_barrier)  		WAIT_FOR_INITIALISATION(true, hardware_barrier)
    #define WAIT_FOR_INITIALISATION(b, hardware_barrier) 	WAIT_FOR_INITIALISATION(b   , hardware_barrier)
*/
#define WAIT_FOR_INITIALISATION_X(x,A,B,FUNC, ...)  FUNC
#define WAIT_FOR_INITIALISATION(...)     WAIT_FOR_INITIALISATION_X(,##__VA_ARGS__,\
                                          WAIT_FOR_INITIALISATION_2(__VA_ARGS__),\
                                          WAIT_FOR_INITIALISATION_1(__VA_ARGS__),\
                                         )

__attribute__((constructor (301))) void _wait_on_global_variables() { defineYourGlobalVariablesInitialisationBehaviour(); }

#ifdef __cplusplus
}
#endif

#endif
