#ifndef MEMDUMP_H
#define MEMDUMP_H

#ifdef __cplusplus
extern "C" {
#endif

//Set if the upper byte should start at bit 8, not 9
#define MEMDUMP_BM_UPPER_AT_8 1
//Set if both bytes only contain 8 bit of data
#define MEMDUMP_BM_IGNORE_BIT_9 2

//Count bytes instead of words
#define MEMDUMP_BM_COUNT_BYTES 4

/**
 * Dump memory to stdout
 *
 * @param address: The address to print next to the data
 * @param mem: Pointer to first word to dump
 * @param length: Number of words to dump (or bytes if MEMDUMP_BM_COUNT_BYTES is set in bytemode)
 * @param bytemode: Configure converting 18 bit word into bytes. Use a or-combination of MEMDUMP_BM_*
 */
void memdump(const unsigned int address, const unsigned int * const mem, const unsigned int length, unsigned char bytemode);

/**
 * Dump memory from a ring buffer to stdout. Supports wrapping around within the data
 *
 * After dumping the word at (rb_end-1), the next word that is dumped is rb_start
 *
 * @param address: The address to print next to the data
 * @param rb_start: First word inside the ringbuffer
 * @param rb_end: One after last word inside the ringbuffer
 * @param mem: Pointer to first word to dump
 * @param length: Number of words to dump (or bytes if MEMDUMP_BM_COUNT_BYTES is set in bytemode)
 * @param bytemode: Configure converting 18 bit word into bytes. Use a or-combination of MEMDUMP_BM_*
 */
void memdump_ringbuffer(const unsigned int address, const unsigned int* const rb_start, const unsigned int * const rb_end, const unsigned int* const mem, const unsigned int length, unsigned char bytemode);

#ifdef __cplusplus
}
#endif

#endif
