/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#ifndef __MPSOC_H
#define __MPSOC_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <memcopy.h>
#include <peripherals/core_connector_master.h>
#include <peripherals/core_connector_slave.h>
#include <peripherals/core_connector_duplex.h>
#include <peripherals/mem_swap.h>
#include <peripherals/mem_swap_multi_master.h>
#include <peripherals/concentrator_master.h>
#include <peripherals/concentrator_slave.h>
#include <peripherals/dispatcher_master.h>
#include <peripherals/dispatcher_slave.h>
#include <peripherals/core_connector_dma.h>
#include <peripherals/core_connector_dma_base.h>
#include <peripherals/inter_core_communication_rx_dma.h>
#include <peripherals/inter_core_communication_tx_dma.h>
#include <peripherals/mpsoc_status_flags.h>


#define memory_barrier() asm volatile ("")

//---------------------------- general functions ---------------------------

void sender_wait_free(volatile unsigned int *status_reg, volatile unsigned int *msg_size_reg, unsigned int count);
void receiver_wait_data(volatile unsigned int *status_reg, volatile unsigned int *msg_size_reg, unsigned int count);


//------------------------ core connector functions ------------------------

#define CORE_CONN_OK    	0
#define CORE_CONN_FULL  	-1
#define CORE_CONN_EMPTY 	-2

void core_connector_master_send_value(core_connector_master_regs_t *conn, unsigned int value);
int core_connector_master_send_value_nb(core_connector_master_regs_t *conn, unsigned int value);
void core_connector_master_send_data(core_connector_master_regs_t *conn, unsigned int * data, unsigned int count);
int core_connector_master_send_data_nb(core_connector_master_regs_t *conn, unsigned int * data, unsigned int count);

uint18_t core_connector_slave_receive_value(core_connector_slave_regs_t *conn);
int core_connector_slave_receive_value_nb(core_connector_slave_regs_t *conn, unsigned int * value);
void core_connector_slave_receive_data(core_connector_slave_regs_t *conn, unsigned int * data, unsigned int count);
int core_connector_slave_receive_data_nb(core_connector_slave_regs_t *conn, unsigned int * data, unsigned int count);


//------------------------ concentrator functions --------------------------

#define CONCENTRATOR_SEND_OK				 	1
#define CONCENTRATOR_SEND_NO_SEND_PERMISSION	0

#define CONCENTRATOR_RECEIVE_OK								2
#define CONCENTRATOR_RECEIVE_NO_ENTIRE_MESSAGE_AVAILABLE 	1
#define CONCENTRATOR_RECEIVE_NO_DATA_AVAILABLE				0

unsigned int concentrator_master_read_data_nb(concentrator_master_regs_t *master, void * data, unsigned int * count, unsigned int * sender_id);
void concentrator_master_read_data(concentrator_master_regs_t *master, void * data, unsigned int * count, unsigned int * sender_id);
unsigned int concentrator_slave_send_data_nb(concentrator_slave_regs_t *slave, unsigned int * data, unsigned int msgsize);
void concentrator_slave_send_data(concentrator_slave_regs_t *slave, unsigned int * data, unsigned int msgsize);

//-------------------------- dispatcher functions --------------------------

#define DISPATCH_ROUND_ROBIN	62
#define DISPATCH_LOAD_BALANCING	63

#define DISPATCHER_SEND_OK			1
#define DISPATCHER_SEND_BUFFER_FULL 0

#define DISPATCHER_RECEIVE_OK							2
#define DISPATCHER_RECEIVE_NO_ENTIRE_MESSAGE_AVAILABLE 	1
#define DISPATCHER_RECEIVE_NO_DATA_AVAILABLE			0

unsigned int dispatcher_master_send_data_nb(dispatcher_master_regs_t *master, unsigned int * data, unsigned int msgsize, unsigned int slave_id);
void dispatcher_master_send_data(dispatcher_master_regs_t *master, unsigned int * data, unsigned int msgsize, unsigned int slave_id);
unsigned int dispatcher_slave_read_data_nb(dispatcher_slave_regs_t *slave, unsigned int * data, unsigned int * count);
void dispatcher_slave_read_data(dispatcher_slave_regs_t *slave, unsigned int * data, unsigned int * count);

//-------------------------- dma functions --------------------------
void core_connector_master_dma_send_value(core_connector_dma_t *conn, volatile unsigned int * value);
int core_connector_master_dma_send_value_nb(core_connector_dma_t *conn, volatile unsigned int * value);
void core_connector_master_dma_send_data(core_connector_dma_t *conn, volatile unsigned int * data, unsigned int count);
int core_connector_master_dma_send_data_nb(core_connector_dma_t *conn, volatile unsigned int * data, unsigned int count);

void core_connector_slave_dma_receive_value(core_connector_dma_t *conn, volatile unsigned int * value);
int core_connector_slave_dma_receive_value_nb(core_connector_dma_t *conn, volatile unsigned int * value);
void core_connector_slave_dma_receive_data(core_connector_dma_t *conn, volatile unsigned int * data, unsigned int count);
int core_connector_slave_dma_receive_data_nb(core_connector_dma_t *conn, volatile unsigned int * data, unsigned int count);

unsigned int concentrator_slave_dma_send_data_nb(inter_core_communication_tx_dma_t *slave, volatile unsigned int * data, unsigned int msgsize);
void concentrator_slave_dma_send_data(inter_core_communication_tx_dma_t *slave, volatile unsigned int * data, unsigned int msgsize);

unsigned int concentrator_master_dma_read_data_nb(inter_core_communication_rx_dma_t *master, volatile unsigned int *data, volatile unsigned int * count, volatile unsigned int * sender_id);
void concentrator_master_dma_read_data(inter_core_communication_rx_dma_t *master, volatile unsigned int * data, volatile unsigned int * count, volatile unsigned int * sender_id);

unsigned int dispatcher_master_dma_send_data_nb(inter_core_communication_tx_dma_t *master, volatile unsigned int * data, unsigned int msgsize, unsigned int slave_id);
void dispatcher_master_dma_send_data(inter_core_communication_tx_dma_t *master, volatile unsigned int * data, unsigned int msgsize, unsigned int slave_id);

unsigned int dispatcher_slave_dma_read_data_nb(inter_core_communication_rx_dma_t *slave, volatile unsigned int * data, volatile unsigned int * count);
void dispatcher_slave_dma_read_data(inter_core_communication_rx_dma_t *slave, volatile unsigned int * data, volatile unsigned int * count);

void wait_dma_transfer_finished(volatile unsigned int * status);
unsigned int dma_transfer_finished(volatile unsigned int * status);

//-------------------------- memory swapper functions --------------------------
void wait_mem_swapped(volatile unsigned int * status);
unsigned int mem_swap_swap_nb(volatile unsigned int * status);
void mem_swap_swap(volatile unsigned int * status);


#ifdef __cplusplus
}
#endif

#endif
