#ifndef _MUTEX_H
#define _MUTEX_H

#ifdef __cplusplus
extern "C" {
#endif


#include <stdint.h>
#include "globalvariables.h"

#define OPEN 		0xff
#define OPEN_S		"0xff"
#define LOCKED 		0x00
#define LOCKED_S	"0x00"

typedef volatile uint18_t mutex;

//declare a mutex in global memory
#define DECLARE_MUTEX(name) DEFINE_GLOBAL(mutex, name, OPEN)
#define DECLARE_MUTEX_CLOSED(name) DEFINE_GLOBAL(mutex, name, LOCKED)
#define DECLARE_MUTEX_OPEN(name ) DEFINE_GLOBAL(mutex, name, OPEN)

/*
	attempts to lock the mutex
	if the mutex is already locked, it waits until it is opened
		returns once it successfully acquires the lock
*/
void wait(mutex* const m);

/*
	releases the mutex,
	call only on mutex whose lock you currently hold!
*/
void signal(mutex* const m);

/*
	initialise a mutex at runtime
*/
inline static void initialise_mutex_locked(mutex* const m) {
	__asm(
		"movi		r8,		"LOCKED_S"	\n"
		"s18		0(%0),	r8			\n"
		: //output ops
		: "r" (m)
		: "r8", "memory"
	);
}

/*
	initialise a mutex at runtime
*/
inline static void initialise_mutex_open(mutex* const m) {
	__asm(
		"movi		r8,		"OPEN_S"	\n"
		"s18		0(%0),	r8			\n"
		: //output ops
		: "r" (m)
		: "r8", "memory"
	);
}

#ifdef __cplusplus
}
#endif

#endif
