/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________

This file is based on the work of Richard Herveille.

*/



/*
/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Include file for OpenCores I2C Master core                 ////
////                                                             ////
////  File    : oc_i2c_master.h                                  ////
////  Function: c-include file                                   ////
////                                                             ////
////  Authors: Richard Herveille (richard@asics.ws)              ////
////           Filip Miletic                                     ////
////                                                             ////
////           www.opencores.org                                 ////
////                                                             ////
/////////////////////////////////////////////////////////////////////
////                                                             ////
//// Copyright (C) 2001 Richard Herveille                        ////
////                    Filip Miletic                            ////
////                                                             ////
//// This source file may be used and distributed without        ////
//// restriction provided that this copyright statement is not   ////
//// removed from the file and that any derivative work contains ////
//// the original copyright notice and the associated disclaimer.////
////                                                             ////
////     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ////
//// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ////
//// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ////
//// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ////
//// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ////
//// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ////
//// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ////
//// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ////
//// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ////
//// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ////
//// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ////
//// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ////
//// POSSIBILITY OF SUCH DAMAGE.                                 ////
////                                                             ////
/////////////////////////////////////////////////////////////////////
*/
#ifndef __I2C_MASTER_
#define __I2C_MASTER_

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Definitions for the Opencores i2c master core
 */
// Rückgabewerte für non blocking read
#define	I2C_OK		0
#define	I2C_NO_ACK	1

/* --- Definitions for i2c master's registers --- */
	
/* ----- Read-write access                                            */

//#define I2C_PRER	0x00	/* Low byte clock prescaler register  */	
#define	I2C_CTR		0x00	/* Control register                   */	
										
/* ----- Write-only registers                                         */
										
#define	I2C_TXR		0x01	/* Transmit byte register             */	
#define	I2C_CR		0x03	/* Command register                   */	
	
/* ----- Read-only registers                                          */
										
#define	I2C_RXR		0x02	/* Receive byte register              */
#define	I2C_SR		0x04	/* Status register                    */
	
/* ----- Bits definition                                              */
	
/* ----- Control register                                             */
	
#define	I2C_EN		(1<<16)	/* Core enable bit:                   */
				/*      1 - core is enabled           */
				/*      0 - core is disabled          */
#define	I2C_IEN		(1<<17)	/* Interrupt enable bit               */
				/*      1 - Interrupt enabled         */
				/*      0 - Interrupt disabled        */
				/* Other bits in CR are reserved      */

/* ----- Command register bits                                        */
 
#define I2C_STA		(1<<7)	/* Generate (repeated) start condition*/
#define I2C_STO		(1<<6)	/* Generate stop condition            */
#define I2C_RD		(1<<5)	/* Read from slave                    */
#define I2C_WR		(1<<4)	/* Write to slave                     */
#define I2C_NAK		(1<<3)	/* Acknowledge send to slave          */
				/*      0 - ACK                       */
				/*      1 - NACK                      */
#define	I2C_ACK		0
#define I2C_IACK	(1<<0)	/* Interrupt acknowledge              */

/* ----- Status register bits                                         */

#define I2C_RXACK	(1<<7)	/* ACK received from slave            */
				/*      0 - ACK                       */
				/*      1 - NACK                      */
#define I2C_BUSY	(1<<6)	/* Busy bit                           */
#define I2C_AL		(1<<5) 	/* Arbitration lost                   */
#define I2C_TIP		(1<<1)	/* Transfer in progress               */
#define I2C_IF		(1<<0)	/* Interrupt flag                     */

/* bit testing and setting macros                                     */

#define ISSET(reg,bitmask)	((reg)&(bitmask))
#define ISCLEAR(reg,bitmask)	(!(ISSET(reg,bitmask)))
#define BITSET(reg,bitmask)	((reg)|(bitmask))
#define BITCLEAR(reg,bitmask)	((reg)|(~(bitmask)))
#define BITTOGGLE(reg,bitmask)	((reg)^(bitmask))
#define REGMOVE(reg,value)	((reg)=(value))

typedef	volatile struct {
	volatile unsigned int	ctrl;	// (r/w)
	volatile unsigned int	txr;	// (r/w)
	volatile unsigned int	rxr;	// (r)
	volatile unsigned int	cmd;	// (r/w)
	volatile unsigned int	stat;	// (r)
} i2c_master_regs_t;

#ifdef __cplusplus
}
#endif

#endif //define __I2C_MASTER
