/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#ifndef __I2C_SLAVE_H
#define __I2C_SLAVE_H

#ifdef __cplusplus
extern "C" {
#endif
	
/* ----- Read-write access						      */

#define	I2C_SL_CTRL		0x00	/* Control register                   */
#define	I2C_SL_TX		0x01	/* Transmit byte register             */
#define	I2C_SL_RX		0x02	/* Receive byte register              */
	
/* ----- Bits definition						      */

/* ----- Control register						      */

#define	I2C_SL_EN		(1<<0)	/* Core enable bit:                   */
					/*      1 - core is enabled           */
					/*      0 - core is disabled          */
#define	I2C_SL_IEN		(1<<1)	/* Interrupt enable bit               */
					/*      1 - Interrupt enabled         */
					/*      0 - Interrupt disabled        */
#define	I2C_SL_10BIT_EN		(1<<2)	/* 10 Adresse aktiv                   */

#define	I2C_SL_IR_SIGR		(1<<3)	/* Read Interrupt = 1                 */
#define	I2C_SL_IR_SIGW		(1<<4)	/* Write Interrupt = 1                */
#define	I2C_SL_READ		(1<<5)	/* last Transfer 0 = write / 1 = read */
#define	I2C_SL_ADDR(addr)	((addr <<6) )
#define	I2C_SL_RDONE		(1<<16)	/* I2C STOP erkannt und kein START    */
#define	I2C_SL_RACK_F		(1<<17)	/* ACK Bit vom Master                 */

typedef	struct	i2c_slave {
	volatile unsigned int	ctrl;	// (r/w)
	volatile unsigned int	txr;	// (r/w)
	volatile unsigned int	rxr;	// (r)
} i2c_slave_regs_t;

#ifdef __cplusplus
}
#endif

#endif
