/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/
#ifndef __JTAG_H
#define __JTAG_H

#ifdef __cplusplus
extern "C" {
#endif
struct config {
	unsigned busy		:1;
	unsigned gated_tck	:1;
	unsigned tck		:1;
	unsigned tdi		:1;
	unsigned tms		:1;
	unsigned tclk		:1;
	unsigned run		:1;		
	unsigned irqenable	:1;
	unsigned clkdiv		:10;	
};

struct tapaddr {
	unsigned notused	: 8;
	unsigned reverse_receive: 1;
	unsigned reverse_send	: 1;
	unsigned end		: 4;
	unsigned target		: 4;
	
};

struct jtagfields {
	volatile struct config config;
	volatile struct tapaddr tapaddr;
	volatile unsigned int tdidata;
	volatile unsigned int tdilength;
	volatile unsigned int tdodata;
};
#define  JTAG_CTRL 0
#define  JTAG_TAPADDR 1
#define  JTAG_TDIDATA 2
#define  JTAG_TDILENGTH 3
#define  JTAG_TDODATA 4



typedef union jtag {
	struct jtagfields jtagfields;
	volatile unsigned int data[5];
} jtag_regs_t;


#define JTAG_CTRL_BUSY    	0x20000  // 10 0000 0000 0000 0000
#define JTAG_CTRL_GATED_CLK   	0x10000  // 01 0000 0000 0000 0000
#define JTAG_CTRL_TCK    	0x08000  // 00 1000 0000 0000 0000
#define JTAG_CTRL_TDI    	0x04000  // 00 0100 0000 0000 0000
#define JTAG_CTRL_TMS    	0x02000  // 00 0010 0000 0000 0000
#define JTAG_CTRL_TCLK    	0x01000  // 00 0001 0000 0000 0000
#define JTAG_CTRL_RUN   	0x00800  // 00 0000 1000 0000 0000
#define JTAG_CTRL_IRQ_EN    	0x00400  // 00 0000 0100 0000 0000
#define JTAG_CTRL_DIV    	0x003FF  // 00 0000 0011 1111 1111

#define JTAG_TAP_REV_REC    	0x00200  // 00 0000 0010 0000 0000
#define JTAG_TAP_REV_SEND   	0x00100  // 00 0000 0001 0000 0000
#define JTAG_TAP_END    	0x000F0  // 00 0000 0000 1111 0000
#define JTAG_TAP_TARGET    	0x0000F  // 00 0000 0000 0000 1111

#define TEST_LOGIC_RESET  	0x0
#define RUN_TEST_IDLE		0x1
#define SELECT_DR		0x2
#define CAPTURE_DR		0x3
#define SHIFT_DR		0x4
#define EXIT1_DR		0x5
#define PAUSE_DR		0x6
#define EXIT2_DR		0x7
#define UPDATE_DR		0x8
#define SELECT_IR		0x9
#define CAPTURE_IR		0xA
#define SHIFT_IR		0xB
#define EXIT1_IR		0xC
#define PAUSE_IR		0xD
#define EXIT2_IR		0xE
#define UPDATE_IR		0xF

void jtag_set_div(jtag_regs_t *jtag, unsigned int div);
void jtag_enable_irq(jtag_regs_t *jtag);
void jtag_disable_irq(jtag_regs_t *jtag);
void jtag_enable_auto_mode(jtag_regs_t *jtag);
void jtag_disable_auto_mode(jtag_regs_t *jtag);
void jtag_set_tclk(jtag_regs_t *jtag);
void jtag_unset_tclk(jtag_regs_t *jtag);
void jtag_set_tck(jtag_regs_t *jtag);
void jtag_unset_tck(jtag_regs_t *jtag);
void jtag_set_tms(jtag_regs_t *jtag);
void jtag_unset_tms(jtag_regs_t *jtag);
void jtag_set_tdi(jtag_regs_t *jtag);
void jtag_unset_tdi(jtag_regs_t *jtag);
void jtag_set_gated_tck(jtag_regs_t *jtag);
void jtag_unset_gated_tck(jtag_regs_t *jtag);

void jtag_tap_set_target(jtag_regs_t *jtag,unsigned int target);
void jtag_tap_set_end(jtag_regs_t *jtag,unsigned int end);
void jtag_tap_set_target_end(jtag_regs_t *jtag,unsigned int target, unsigned int end);
void jtag_tap_set_rev_send(jtag_regs_t *jtag);
void jtag_tap_unset_rev_send(jtag_regs_t *jtag);
void jtag_tap_set_rev_rec(jtag_regs_t *jtag);
void jtag_tap_unset_rev_rec(jtag_regs_t *jtag);

unsigned int jtag_shift_data(jtag_regs_t *jtag,unsigned int data, unsigned int length);



#ifdef __cplusplus
}
#endif

#endif
