/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/
#ifndef __MICROSTEPPER_H
#define __MICROSTEPPER_H

#ifdef __cplusplus
extern "C" {
#endif

#define STEPPER_RAMP_INT_CNT        5   
#define STEPPER_FSYS                25000
#define STEPPER_FSTART              ((STEPPER_FSYS>>9)+(STEPPER_FSYS>>10)+(STEPPER_FSYS>>11)+(STEPPER_FSYS>>12)+(STEPPER_FSYS>>13)+(STEPPER_FSYS>>15))
#define STEPPER_FMAX                (11*1000*8)
#define STEPPER_DELTAF              (STEPPER_FMAX-STEPPER_FSTART)

#define STEPPER_START_BIT           0
#define STEPPER_ESTOP_BIT           1
#define STEPPER_DIRECTION_BIT       2
#define STEPPER_PWR_DOWN_SEL_BITS   3
#define STEPPER_IDLE_BIT            5

#define STEPPER_NO_IDLE_PWR         0
#define STEPPER_QUARTER_IDLE_PWR    1
#define STEPPER_HALF_IDLE_PWR       2
#define STEPPER_FULL_IDLE_PWR       3

typedef struct stepper {
    volatile unsigned int ramp_conf[STEPPER_RAMP_INT_CNT*2+1];
    volatile unsigned int steps_msb;
    volatile unsigned int steps_lsb;
    volatile unsigned int mode;
    volatile unsigned int speed;
    volatile unsigned int acc;
} microstepper_regs_t;

#ifdef __cplusplus
}
#endif

#endif
