/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#ifndef __ULTRASONIC_H
#define __ULTRASONIC_H

#ifdef __cplusplus
extern "C" {
#endif
#define ULTRASONICBASE ((void *)(SB_ULTRASONIC_0_BASE_ADR))

#define CONTINUOUS_MODE		(1<<2)	//Kontinuierliches Senden (zu Testzwecken)
#define PULSE_MODE		(1<<1)	//Aussenden von Pulsen (nur so sinnvolle Messungen möglich)
#define FSM_ENABLE		(1<<0)

#define ECHO_CNT		SB_ULTRASONIC_0_ECHO_CNT
#define TIMEOUT_VALUE		1023
#define PERIOD_COUNT_INIT	2
#define INITIAL_GUARD_WIDTH	10     
#define INITIAL_TEMPERATURE	20	// Umgebungstemperatur in Grad Celsius
#define INITIAL_CC_PER_CM	1458	// entspricht Anzahl der Taktzyklen pro cm bei FSYS = 25000 kHz und einer Umgebungstemperatur von 20 Grad Celsius 
#define FSYS			25000	// Systemtaktfrequenz in kHz

typedef	struct ultrasonic {
	volatile unsigned int distance[ECHO_CNT];
	volatile unsigned int cc_per_cm;
	volatile unsigned int mode;
	volatile unsigned int frequency;
	volatile unsigned int guard_width;
	volatile unsigned int pulse_form;	
} ultrasonic_regs_t;

void disable_int() {
	unsigned int i;
	struct ultrasonic *us;
	us = (struct ultrasonic *)ULTRASONICBASE;
	i  = us->distance[0];		//Interrupt deaktivieren !
}


void init_pulse_form(unsigned int per_cnt) {
	unsigned int temp = per_cnt;
	unsigned int i;
	struct ultrasonic *us;
	us = (struct ultrasonic *)ULTRASONICBASE;
	for (i=0; i < 8; i++) temp = temp << 1;
	us->pulse_form = per_cnt | temp;   
}

void init_guard_width(unsigned int width) {
	struct ultrasonic *us;
	us = (struct ultrasonic *)ULTRASONICBASE;
	us->guard_width = width;   
}
    

void us_measurement_enable(unsigned int x) {
	struct ultrasonic *us;
	us = (struct ultrasonic *)ULTRASONICBASE;
	us->mode &= ~((unsigned int)FSM_ENABLE);   
	us->mode |= x & 1;   
}

unsigned int get_echo_counter() {   //liefert zurück, wieviele Echos bereits gespeichert wurden
	struct ultrasonic *us;
        unsigned int i,tmp;
	us = (struct ultrasonic *)ULTRASONICBASE;
	tmp = us->mode;   
        for (i=0;i<9;i++) tmp = tmp >> 1;
        return tmp;
}

void calibrate(unsigned int temperature) {
	struct ultrasonic *us;
	us = (struct ultrasonic *)ULTRASONICBASE;
	us->cc_per_cm = INITIAL_CC_PER_CM;  // eigentlich aus Temperatur berechnen!!
        /*  ergibt sich aus 2 * FSYS * 1000 * 100 / (343 * (10000 + 18 * (temperature - 20)))

         dazu hier 36-Bit-Arithmetik implementieren!!!
         */
}

void init_us() {
        init_guard_width(INITIAL_GUARD_WIDTH);
        init_pulse_form(PERIOD_COUNT_INIT);
        calibrate(INITIAL_TEMPERATURE);
}

#ifdef __cplusplus
}
#endif

#endif
