#ifndef _SEMAPHORE_
#define _SEMAPHORE_

#ifdef __cplusplus
extern "C" {
#endif


#include <stdint.h>
#include "mutex.h"

typedef struct semaphore {
	mutex m;
	volatile uint18_t counter;
} semaphore;

//declare a semaphore in global memory
#define DECLARE_SEMAPHORE(name) semaphore name __attribute__((section ( SECTIONNAME ))) = { OPEN, 1 };

/*
	attempts to decrease the semaphore by 1,
	if the semaphore is less or equal to 0, it waits until another core increases the semaphore
		returns once the semaphore is decreased to a number greater or equal to zero
*/
void down(semaphore* s);

/*
	increases the semaphore by 1,
*/
void up(semaphore* s);

/*
	returns the current value of the semaphore
*/
uint18_t value(const semaphore* s);

/*
	initialise a semaphore at runtime
*/
inline static void initialise_semaphore(semaphore* s, const int18_t value) {
	s->counter = value;
	initialise_mutex_open(&s->m);
}

#ifdef __cplusplus
}
#endif

#endif
