/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#ifndef SPMC_SLEEP_H
#define SPMC_SLEEP_H

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Delay execution by 6 + 3 * i clock cycles
 */
void sleep(unsigned int i);

/**
 * Delay execution by 4 + ((3 * inner) + 3) * outer clock cycles
 */
void sleep_sleep(unsigned int inner, unsigned int outer);


#define SLEEP_CYCLES_PER_ITERATION 3

#define CYCLES_PER_MS(frequency) (frequency)/1000
#define CYCLES_PER_US(frequency) CYCLES_PER_MS(frequency)/1000

#define SLEEP_ITERATIONS_PER_MS(frequency) CYCLES_PER_MS(frequency)/SLEEP_CYCLES_PER_ITERATION-1
#define SLEEP_ITERATIONS_PER_US(frequency) CYCLES_PER_US(frequency)/SLEEP_CYCLES_PER_ITERATION-1


/**
 * Declare a new function with the given name, that delays execution by a given amount of microseconds.
 * 
 * Frequency needs to be given in Hertz, as generated by jConfig.
 * 
 * Example:
 * 
 * #include <moduleParameters.h>
 * DEFINE_SLEEP_US(sleep_us, SB_SPARTANMC_0_FREQUENCY)
 * // ...
 * sleep_us(10); //Sleep for 10 us
 */
#define DEFINE_SLEEP_US(name, frequency) static inline void name(unsigned int duration) { sleep_sleep(SLEEP_ITERATIONS_PER_US(frequency),duration);}

/**
 * Declare a new function with the given name, that delays execution by a given amount of milliseconds.
 * 
 * Frequency needs to be given in Hertz, as generated by jConfig.
 *
 * Example:
 * 
 * #include <moduleParameters.h>
 * DEFINE_SLEEP_MS(sleep_ms, SB_SPARTANMC_0_FREQUENCY)
 * // ...
 * sleep_ms(10); //Sleep for 10 ms
 */
#define DEFINE_SLEEP_MS(name, frequency) static inline void name(unsigned int duration) { sleep_sleep(SLEEP_ITERATIONS_PER_MS(frequency),duration);}


#ifdef __cplusplus
}
#endif

#endif
