#ifndef __STDIO_H
#define __STDIO_H

#ifdef __cplusplus
extern "C" {
#endif
#include "stdarg.h"
#define putc(c, stream)     fputc(c, stream)
struct file;
typedef void (*fun_stdio_send_byte) (void * base_adr, char value);
typedef char (*fun_stdio_receive_byte) (void * base_adr);
typedef char (*fun_stdio_receive_byte_nb) (void * base_adr, char *value);


typedef const struct file {
    void * base_addr;
    fun_stdio_send_byte send_byte;
    fun_stdio_receive_byte receive_byte;
    fun_stdio_receive_byte_nb receive_byte_nb;
} FILE;

typedef struct {
    char * current;
    char * end;
} buffer_descr;

#define PRINTF_BUF_SIZE 38

extern FILE * stdin;
extern FILE * stdout;

int puts(const char *s);
int putsn(const char *s);
int putchar(const int c);

int fputc(const int c, FILE *out);
int fputs(const char *s, FILE * out);

char getchar();
int getchar_nb(char *value);
char getc(FILE * stream);
int getc_nb(FILE * stream, char * value);
char * fgets ( char * str, int num, FILE * stream );

char *printf_add_padding(char *digit, int width, char fill);
char *printf_format_bits(unsigned long value, const int shift, int width, char fill, int uppercase, char * printf_buffer);
char *printf_format_bits_buf(unsigned long value, const int shift, int width, char fill, char* buffer, int uppercase);
char *printf_format_uint(unsigned int value, int width, char fill, char * printf_buffer);
char *printf_format_int(int value, int width, char fill, char * printf_buffer);
char *printf_format_longuint(unsigned long int value, int width, char fill, char * printf_buffer);
char *printf_format_longint(long int value, int width, char fill, char * printf_buffer);
const char *printf_format_arg(const char **s, va_list * args, char * printf_buffer);
int vprintf(const char *s, va_list args) __attribute__ ((format (spartanmc_printf, 1, 0)));
int printf(const char *s, ...) __attribute__ ((format (spartanmc_printf, 1, 2)));

int vfprintf(FILE * f, const char *format, va_list args) __attribute__ ((format (spartanmc_printf, 2, 0)));
int fprintf(FILE * f, const char *format, ...) __attribute__ ((format (spartanmc_printf, 2, 3)));


int vsnprintf(char * s, unsigned int n, const char *format, va_list args) __attribute__ ((format (spartanmc_printf, 3, 0)));
int snprintf(char * s, unsigned int n, const char *format, ...) __attribute__ ((format (spartanmc_printf, 3, 4)));



void mem_send_byte (buffer_descr * file, char value);
char mem_receive_byte (buffer_descr * file);
char mem_receive_byte_nb (buffer_descr * file, char *value);

struct file fmemopen(char * buf, unsigned int size, buffer_descr * extra);


// Rueckgabewerte fuer non blocking read
#define RECEIVE_NB_OK           0
#define RECEIVE_NB_NO_DATA      1

#ifdef __cplusplus
}
#endif

#endif
