
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#ifndef UART_H_
#define UART_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <peripherals/uart.h>
#include <peripherals/uart_light.h>
#include <stdint.h>

// section kept for compatibility with older projects, may be removed in the
// future

// Status Signale
#define	RX_EMPTY	UART_RX_EMPTY   
#define	RX_FULL		UART_RX_FULL    
#define	TX_EMPTY	UART_TX_EMPTY   
#define	TX_FULL		UART_TX_FULL    
#define	TX_IRQ_PRE	UART_TX_IRQ_PRE 
#define	TX_IRQ_FLAG	UART_TX_IRQ_FLAG
#define	RX_P_ERR	UART_RX_P_ERR   
#define	RX_F_ERR	UART_RX_F_ERR   
#define	RX_D_ERR	UART_RX_D_ERR   
#define	M_DCD		UART_M_DCD      
#define	M_CTS		UART_M_CTS      
#define	M_DSR		UART_M_DSR      
#define	RX_CLK		UART_RX_CLK     
#define	RX_STOP		UART_RX_STOP    
#define	TX_CLK		UART_TX_CLK     
#define	RST_UART	UART_RST_UART	// UART noch im RESET, wenn = 1
#define	TX_STOP		UART_TX_STOP    
                                    
                                    
// Steuersignale    
#define	RX_EN		UART_RX_EN      
#define	TX_EN		UART_TX_EN      
#define	PARI_EN		UART_PARI_EN    
#define	PARI_EVEN	UART_PARI_EVEN	// 0 = ungerade / 1 = gerade
#define	TWO_STOP	UART_TWO_STOP	// 0 = ein / 1 = zwei Stopbits
                                    
#define	DATA_LEN_5	UART_DATA_LEN_5 
#define	DATA_LEN_6	UART_DATA_LEN_6 
#define	DATA_LEN_7	UART_DATA_LEN_7 
#define	DATA_LEN_8	UART_DATA_LEN_8 
                                    
#define	BPS_115200	UART_BPS_115200 
#define	BPS_57600	UART_BPS_57600  
#define	BPS_38400	UART_BPS_38400  
#define	BPS_31250	UART_BPS_31250	// MIDI Datenrate
#define	BPS_19200	UART_BPS_19200  
#define	BPS_9600	UART_BPS_9600   
#define	BPS_4800	UART_BPS_4800   
#define	BPS_2400	UART_BPS_2400   
#define	BPS_1200	UART_BPS_1200   
#define	BPS_600		UART_BPS_600    
#define	BPS_300		UART_BPS_300    
#define	BPS_150		UART_BPS_150    
#define	BPS_75		UART_BPS_75     
#define	BPS_50		UART_BPS_50     
#define	BPS_7812	UART_BPS_7812	// Boot 68hc11 mit 7812,5 Baud
                                    
#define	RX_IE		UART_RX_IE      
#define	TX_IE		UART_TX_IE      
#define	TX_BREAK	UART_TX_BREAK   

// Interruptfreigabe fuer UART light
#define	RXUL_IE		(1<<9)
#define TXUL_IE		(1<<10)

// Modem Outputs und Richtung
#define	DTR_DSR		UART_DTR_DSR
#define	RTS_CTS		UART_RTS_CTS
#define	DCD		    UART_DCD    
#define	DCD_OUT		UART_DCD_OUT
#define	RTS_OUT		UART_RTS_OUT
#define	DTR_OUT		UART_DTR_OUT

typedef uart_regs_t uart_t;

void uart_write (uart_t *uart, const uint18_t value);
unsigned char uart_read (uart_t *uart);
int uart_read_nb (uart_t *uart, unsigned char *value);
void __attribute__((error("stdio_uart_open is no longer supported. Declare a global variable FILE * stdout = &UART_*_FILE instead"))) stdio_uart_open2(uart_t *uart);

// end compatibility section

void uart_wait_idle(uart_regs_t *uart);
void uart_send(uart_regs_t *uart, const unsigned char value);
unsigned char uart_receive(uart_regs_t *uart);
int uart_receive_nb (uart_regs_t *uart, unsigned char *value);

#define declare_UART_FILE(uart) { \
	.base_addr = (void*) uart, \
	.send_byte = (fun_stdio_send_byte) uart_send, \
	.receive_byte = (fun_stdio_receive_byte) uart_receive, \
	.receive_byte_nb = (fun_stdio_receive_byte_nb) uart_receive_nb \
}

void __attribute__((error("stdio_uart_open is no longer supported. Declare a global variable FILE * stdout = &UART_*_FILE instead"))) stdio_uart_open(uart_regs_t *uart);

#ifdef __cplusplus
}
#endif

#endif /*UART_H_*/
