/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#ifndef __USB_H
#define __USB_H

#ifdef __cplusplus
extern "C" {
#endif

#include <peripherals/usb11.h>

// section kept for compatibility with older projects, may be removed in the
// future

#define USB_ATTRIBUTE_BULK             USB11_ATTRIBUTE_BULK      
#define USB_DIRECTION_IN               USB11_DIRECTION_IN        
#define USB_DIRECTION_OUT              USB11_DIRECTION_OUT       
#define USB_DEVICE_DESCRIPTOR          USB11_DEVICE_DESCRIPTOR   
#define USB_CONFIG_DESCRIPTOR          USB11_CONFIG_DESCRIPTOR   
#define USB_STRING_DESCRIPTOR          USB11_STRING_DESCRIPTOR   
#define USB_INTERFACE_DESCRIPTOR       USB11_INTERFACE_DESCRIPTOR
#define USB_ENDPOINT_DESCRIPTOR        USB11_ENDPOINT_DESCRIPTOR 
#define USB_SELF_POWERED               USB11_SELF_POWERED        

#define USB_CLASS_VENDOR_SPECIFIC      USB11_CLASS_VENDOR_SPECIFIC

/**
 * Konstanten fuer epXXc-Register
 */
#define USB_CTRL_EN                    USB11_CTRL_EN    
#define USB_CTRL_INTR                  USB11_CTRL_INTR  
#define USB_CTRL_MODE                  USB11_CTRL_MODE  
#define USB_CTRL_CTRL                  USB11_CTRL_CTRL  
#define USB_CTRL_OUT                   USB11_CTRL_OUT   
#define USB_CTRL_IN                    USB11_CTRL_IN    
#define USB_CTRL_BUFSEL                USB11_CTRL_BUFSEL
#define USB_CTRL_SIZE                  USB11_CTRL_SIZE  

/**
 * Konstanten fuer das globale Konfigurationsregister
 */
#define USB_EN                         USB11_EN		
#define USB_DI                         USB11_DI		
#define USB_IEN_EP0                    USB11_IEN_EP0	
#define USB_IEP15                      USB11_IEP15		
#define USB_IEP14                      USB11_IEP14		
#define USB_IEP13                      USB11_IEP13		
#define USB_IEP12                      USB11_IEP12		
#define USB_IEP11                      USB11_IEP11		
#define USB_IEP10                      USB11_IEP10		
#define USB_IEP09                      USB11_IEP09		
#define USB_IEP08                      USB11_IEP08		
#define USB_IEP07                      USB11_IEP07		
#define USB_IEP06                      USB11_IEP06		
#define USB_IEP05                      USB11_IEP05		
#define USB_IEP04                      USB11_IEP04		
#define USB_IEP03                      USB11_IEP03		
#define USB_IEP02                      USB11_IEP02		
#define USB_IEP01                      USB11_IEP01		
#define USB_IEP00                      USB11_IEP00		

// end compatibility section

#define USB_ENDPOINT(usb_base, ep_num) { \
    .ctrl = (volatile unsigned int*)(((volatile unsigned char*)usb_base) + ep_num * 4), \
    .data = (volatile unsigned int*)(((volatile unsigned char*)usb_base) + 0x100 + ep_num * 64), \
    .usb = usb_base, \
    .index = (ep_num) \
}

#define USB_ENDPOINT_DB(usb_base, ep_num) { \
    .ctrl = (volatile unsigned int*)(((volatile unsigned char*)usb_base) + ep_num * 4), \
    .data = (volatile unsigned int*)(((volatile unsigned char*)usb_base) + 0x100 + ep_num * 128), \
    .usb = usb_base, \
    .index = (ep_num) \
}

#define USB_BUFFER_CURRENT  0
#define USB_BUFFER_OTHER    1

struct usb_ep {
    volatile unsigned int *ctrl;
    volatile unsigned int *data;
    struct usb* usb;
    int index;
};

void usb_init(usb11_dma_t *usb, unsigned int delay);
int usb_ep_poll_txready(struct usb_ep *ep);
void usb_ep_wait_txready(struct usb_ep *ep);
int usb_ep_poll_rxdata(struct usb_ep *ep);
void usb_ep_wait_rxdata(struct usb_ep *ep);
void usb_ep_packet_send(struct usb_ep *ep, unsigned int size);
void usb_ep_packet_receive(struct usb_ep *ep);
void usb_ep_bufsel(struct usb_ep *ep, int buf_no);
void usb_ep_intr_en(struct usb_ep *ep); 
void usb_ep_intr_dis(struct usb_ep *ep); 
void usb_ep_intr_clear(struct usb_ep *ep); 
volatile unsigned int *usb_ep_get_buffer(struct usb_ep *ep, unsigned int buf);
void usb_ep_switch_buffer(struct usb_ep *ep);

#ifdef __cplusplus
}
#endif

#endif /*USB_H_*/
