/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#ifndef VIDEOI420___
#define VIDEOI420___

#ifdef __cplusplus
extern "C" {
#endif

#include <peripherals/videoI420.h>
// user funktion

void videoI420_init(	VideoI420_regs_t* v,
						unsigned int width,
						unsigned int height,
						
						unsigned int yJump,
						unsigned char* pY1,
						unsigned char* pY2,
						
						unsigned int uvJump,
						unsigned char* pU1,
						unsigned char* pU2,
	
						unsigned char* pV1,
						unsigned char* pV2,
						
						unsigned char scale
					);


// start the videoI420/enable it, buffer ID = 1 or 2
void videoI420_start(VideoI420_regs_t* v, unsigned int buffer);

// stops the videoI420 module, the display will be display the default values
// is also called inside the videoI420_init method.
void videoI420_stop(VideoI420_regs_t* v);


void videoI420_next(VideoI420_regs_t* v, unsigned char* pY, unsigned char* pU, unsigned char* pV);
void videoI420_next_block(VideoI420_regs_t* v, unsigned char* pY, unsigned char* pU, unsigned char* pV);

// sets the id of the next buffer for display
void videoI420_bufferReady(VideoI420_regs_t* v, unsigned int buffer);

// 0 no buffer is free (current is displayed, next is ready for display but not displayed yet)
// 1 first buffer is free
// 2 second buffer is free
// 3 both are free, video is not enables
int  videoI420_getFreeBuffer(VideoI420_regs_t* v);

	
	


#ifdef __cplusplus
}
#endif

#endif  //define VIDEOI420___
