/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#ifndef MALLOC_INTERNAL_H
#define MALLOC_INTERNAL_H

#define RTOS_DONT_USE_TRAP

#define FALSE	0
#define TRUE	!FALSE

#include <stdint.h>
#include <stddef.h>
//-----------------------------------------------------------------------------
// other macros
//-----------------------------------------------------------------------------

#define INLINE_ONLY		static inline __attribute__((always_inline))

#define ROUNDED_DOWN_EVEN(a)	(((a) >> 1) << 1)
#define ROUNDED_UP_EVEN(a)	ROUNDED_DOWN_EVEN((a)+1)

#ifndef MAX
	#define MAX(a,b) (((a)<(b)) ? (b) : (a))
#endif

#ifndef MIN
	#define MIN(a,b) (((a)>(b)) ? (b) : (a))
#endif


//-----------------------------------------------------------------------------
// lists
//-----------------------------------------------------------------------------

// ---------------------------------------------------------------------------
// types
// ---------------------------------------------------------------------------

typedef struct rtos_list_item {
	struct rtos_list_item *previous;
	struct rtos_list_item *next;
} rtos_list_item_t;

// Idee das list_item als Liste zu verwenden
// dem System RT-Thread entnommen:
typedef rtos_list_item_t rtos_list_t;

#define DEFINE_INITIALIZED_EMPTY_LIST(name) rtos_list_t name = { &name, &name }

#define rtos_list_first( list ) ((list)->next)

#define rtos_list_last( list ) ((list)->previous)

#define rtos_list_next( list_item ) ((list_item)->next)

#define rtos_list_previous( list_item ) ((list_item)->previous)

void rtos_list_initialize_( rtos_list_t *list );

//__attribute__((section(".traps"))) void rtos_list_insert_between_( rtos_list_item_t *new_item, rtos_list_item_t *previous, rtos_list_item_t *next )
void rtos_list_insert_between_( rtos_list_item_t *new_item, rtos_list_item_t *previous, rtos_list_item_t *next );

//__attribute__((section(".traps"))) void rtos_list_remove_( rtos_list_item_t *item )
void rtos_list_remove_( rtos_list_item_t *item );

//__attribute__((section(".traps"))) rtos_list_item_t *rtos_list_remove_first_( rtos_list_t *list )
rtos_list_item_t *rtos_list_remove_first_( rtos_list_t *list );

// Idee dieses Makros dem System RT-Thread entnommen:
#define rtos_list_item_owner(list_item, owner_type, member_name)\
	((owner_type *) ((char *)(list_item) - offsetof(owner_type, member_name)))

INLINE_ONLY uint18_t rtos_list_is_empty( rtos_list_t *list )
{
	return list->next == list;
}

INLINE_ONLY void rtos_list_insert_after( rtos_list_item_t *new_item, rtos_list_item_t *previous )
{
	rtos_list_insert_between_(new_item, previous, previous->next);
}

INLINE_ONLY void rtos_list_insert_before( rtos_list_item_t *new_item, rtos_list_item_t *next )
{
	rtos_list_insert_between_(new_item, next->previous, next);
}

#define rtos_list_add_first( list, new_item ) rtos_list_insert_after(new_item, list)

#define rtos_list_add_last( list, new_item ) rtos_list_insert_before(new_item, list)


#endif
