/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#include <stdio.h>
#include <memdump.h>
void memdump(const unsigned int address, const unsigned int * const mem, const unsigned int length, unsigned char bytemode) {
    memdump_ringbuffer(address, 0, (const unsigned int *const) 0x3FFFF, mem, length, bytemode);
}

#define DECODED(x) ((char)(((x)>=32 && (x)<=126)?(x):'.'))

void memdump_ringbuffer(unsigned int address, const unsigned int* const rb_start, const unsigned int * const rb_end, const unsigned int* mem, unsigned int length, unsigned char bytemode) {

    char decoded[18]; //8 content, 1 space, 8 content, 1 null
    decoded[8] = ' ';
    decoded[16] = 0;
    decoded[17] = 0; //Add null byte

    unsigned int count = 0;
    unsigned int lineIdx = 0;
    while (count < length) {
        if (lineIdx == 0) {
            printf("%08x  ", address);
        }

        unsigned int low,high;
        if (bytemode&MEMDUMP_BM_UPPER_AT_8) {
            low = (*mem)&0xFF;
            high = (*mem)>>8;
        } else {
            low = (*mem)&0x1FF;
            high = (*mem)>>9;
        }

        if (bytemode&MEMDUMP_BM_IGNORE_BIT_9)
            printf("%02x %02x ",high&0xFF,low&0xFF);
        else
            printf("%03x %03x ",high,low);

        int decodedIdx = /*lineIdx>3 ? 2*lineIdx+1 :*/ 2*lineIdx;
        decoded[decodedIdx] = DECODED(high);
        decoded[decodedIdx+1] = DECODED(low);


        if (bytemode&MEMDUMP_BM_COUNT_BYTES) {
            count += 2;
            address += 2;
        } else {
            count ++;
            address++;
        }

        mem++;
        lineIdx++;

        if (mem==rb_end)
            mem=rb_start;


        if (lineIdx==4)
            printf(" ");

        if (lineIdx==8) {
            printf(" |");
            putsn(decoded);
            printf("|\n");

            lineIdx = 0;
        }

    }
    if (lineIdx!=0) {
        //End decoded str
        int decodedIdx = /*lineIdx>3 ? 2*lineIdx+1 :*/ 2*lineIdx;
        decoded[decodedIdx]=0;

        if (lineIdx<4)
            printf(" ");

        while ((lineIdx++)<8) {
            if (bytemode&MEMDUMP_BM_IGNORE_BIT_9)
                printf("      ");
            else
                printf("        ");
        }
        printf(" |");
	putsn(decoded);
        printf("|\n");
        printf("%08x\n", address);

    }
}
