/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

#include <string.h>
#pragma GCC diagnostic ignored "-Wimplicit-fallthrough"

__attribute__((optimize("unroll-loops")))
void *memset(void *str, int v, size_t count) {
	if(count == 0) {
		return str;
	}

	if((((unsigned int) str) + count) % 2 != 0) {
    // str + count is odd -> copy one extra byte at end
    	count--;
        *(((unsigned char*) str) + count)  = (unsigned char) v;
        if(count == 0) {
           	return str;
        }
    }
	if((unsigned int) str % 2 != 0) {
    // str is odd -> copy one extra byte at start
       	*((unsigned char*) str) = (unsigned char) v;
        if(--count == 0) {
           	return str;
        }
	int * intp = str;
	str = (void*) (intp+1);
    }

	//only copy count/2 words
    int n = ((count >> 1) + 15) >> 4;
    int switch_var = (count >> 1) & 0xf;
    unsigned int * str_ptr;

	unsigned int data = (unsigned char) v;
	data |= (data << 9);
    str_ptr = (unsigned int *) ((unsigned int) str - 2*(16-switch_var));

    switch (switch_var) {
        case 0	:do{str_ptr=str_ptr+16;
                *(str_ptr) = data;
                case 15	:	*(str_ptr+1) = data;
                case 14	:	*(str_ptr+2) = data;
                case 13	:	*(str_ptr+3) = data;
                case 12	:	*(str_ptr+4) = data;
                case 11	:	*(str_ptr+5) = data;
                case 10	:	*(str_ptr+6) = data;
                case 9	:	*(str_ptr+7) = data;
                case 8	:	*(str_ptr+8) = data;
                case 7	:	*(str_ptr+9) = data;
                case 6	:	*(str_ptr+10) = data;
                case 5	:	*(str_ptr+11) = data;
                case 4	:	*(str_ptr+12) = data;
                case 3	:	*(str_ptr+13) = data;
                case 2	:	*(str_ptr+14) = data;
                case 1	:	*(str_ptr+15) = data;
            }while(--n > 0);
    }
	return str;
}
