/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#include <stdio.h>

static const char * const percent = "%";
static const char * const invalid_format = "%?";

unsigned int printf_scan_int(const char **s) {
    unsigned int result;
    if ((**s >= '0') && (**s <= '9')) {
        result = (unsigned int)**s - '0';
	(*s)++;
        if ((**s >= '0') && (**s <= '9')) {
            result = result * 10 + (unsigned int)**s - '0';
	    (*s)++;
        }
        return result;
    }
    return 1;
}

const char *printf_format_arg(const char **s, va_list * args, char * printf_buffer) {
    //Skip percent
    (*s)++;

    //Flags
    char fill;
    if (**s=='0') {
	fill = '0';
	(*s)++;
    } else fill = ' ';

    //Width
    int width;
    if (**s=='*') {
	width = va_arg(*args, int);
	(*s)++;
    } else {
        width = printf_scan_int(s);
    }

    //Length
    int length = 0;
    if (**s == 'l') {
    	length = 1;
    	(*s)++;
    }

    //Specifier
    char type = **s;
    (*s)++;
    int uppercase = 0;
    switch (type) {
        case 'd':
        case 'i':
        	if (length) {
        		return printf_format_longint(va_arg(*args, long int), width, fill, printf_buffer);
        	} else {
        		return printf_format_int(va_arg(*args, int), width, fill, printf_buffer);
        	}
        case 'u':
        	if (length) {
        		return printf_format_longuint(va_arg(*args, unsigned long int), width, fill, printf_buffer);
        	} else {
        		return printf_format_uint(va_arg(*args, unsigned int), width, fill, printf_buffer);
        	}
        case 'b': {
		unsigned long data;
		if (length) {
			data = va_arg(*args, unsigned long);
		} else {
			data = va_arg(*args, unsigned int);
		}
		return printf_format_bits(data, 1, width, fill,0, printf_buffer);
	}
	case 'c': {
	    char c = (char)va_arg(*args, int);
	    printf_buffer[0] = c;
	    printf_buffer[1] = 0;
	    return printf_buffer;
    	}
	case 'P':
        case 'X':
	    	uppercase = 1;
	    	__attribute__((fallthrough));
	case 'p':
        case 'x': {
		unsigned long data;
		if (length) {
			data = va_arg(*args, unsigned long);
		} else {
			data = va_arg(*args, unsigned int);
		}
		return printf_format_bits(data, 4, width, fill, uppercase, printf_buffer);
	}
	case 'O':
	    	uppercase = 1;
		__attribute__((fallthrough));
        case 'o': {
		unsigned long data;
		if (length) {
			data = va_arg(*args, unsigned long);
		} else {
			data = va_arg(*args, unsigned int);
		}
		return printf_format_bits(data, 3, width, fill, uppercase, printf_buffer);
	}
        case 's':
		return va_arg(*args, char *);
        case '%':
		return percent;
        case 0:
		(*s)--;
		__attribute__ ((fallthrough));
        default:
		return invalid_format;
    }
}
