
; Author: Ramon Wirsch
; Assembler Code needed for Debugging Library
; The Entry-points to the Debugger are all in pure assembler, so that they have full control over all registers (needed to capture any register window)
; As the 2nd-Stage the C-Function handle_exception is called, with all registers already captured in global 'regs[]' and internal reason code in r12 / r4 (Parameter 1)
; In the global char 'update_regs' the c-function instructs what needs to be written back to registers including GPRs / hdw. debug. mode etc. (see debugging_stac.c for more documentation)
; Entry-Points are: tsr_single_Step / tsr_breakpoint_hdw / tsr_breakpoint / tsr_watchpoint. all have their own label, because there currently is no other way to ascertain the actually called trap
; dbg_remote_stop is a special Entry-Point that pops the current register Window to allow Side-Calling it from ISRs and still investigating only the code that was interrupted and not the ISR-Handler
; Internals:
; SFR_CC is saved and restored, interrupts and hdw. debugger are disabled on entry and reenabled on exit (Single step is a special mode of hdw. debugger)
; in case of Software breakpoints, the PC is decremented because the code originally at the current location has not been executed yet, only the trap insn. in its place


; All delay slots are naively filled with NOPs, because the instruction set / delay slots have changed, filling them would break compatibility with the old tune.
; Same is applicable to delays between compares and branches. To not break compatibility with the old instruction set, all delays should remain present

.data

;regs: all 16 GPRs + SFR_STATUS + PC_NEXT (return address in R11)
.global regs
regs:
    .skip 72 ;18*2 (for some reason these are not 9-bit blocks??)
.global pushwin
pushwin:
	.word18 0
	.word18 0
	.word18 0
	.word18 0
	.word18 0
	.word18 0
.global fixup_brk_addr
fixup_brk_addr:
	.word18 0
.global update_regs
update_regs:
    .byte9 0
backup_cc:
	.byte9 0
    
.section .traps, "ax", @progbits

; Entry remote stop
.global dbg_remote_stop ; must be called from interrupt handler!!
dbg_remote_stop:
movs2i r12, SFR_STATUS; hijack reg window of interrupt handler!!
addi r12, -1
movi2s SFR_STATUS, r12
lhi r13, %dhi(__INTR_TAB__)
ori r13, %dlo(__INTR_TAB__)
l18 r13, 0(r13)
movi2s SFR_CC, r13 ; as we will _intr_exit we need handle backup of CC ourselves; NOTE: we won't use RFE!!

movi r9, 4
j capture_regs
nop;TODO

; Entry single step
.global tsr_single_step
tsr_single_step:
movi r9, 3
j capture_regs
nop;TODO

; Entry for hardware breakpoints
.global tsr_breakpoint_hdw
tsr_breakpoint_hdw:
movi r9, 1
j capture_regs
nop;TODO

; Entry for hardware watchpoints
.global tsr_watchpoint
tsr_watchpoint:
movi r9, 2
j capture_regs
nop;TODO

; Entry for software breakpoints
.global tsr_breakpoint
tsr_breakpoint:
movi r9, 0

.global capture_regs
capture_regs:
	cbits INT ;TODO: save INT, restore

	movs2i r15, SFR_CC ;save CC
	lhi r14, %dhi(backup_cc)
	ori r14, %dlo(backup_cc)
	s9 0(r14), r15

	movi r15, 0
	movi2s SFR_DBG_IDX, r15
	movi2s SFR_DBG_DAT, r15 ;deactivate all traps/ breakpoints /steps

	lhi r15, %hi(debuggerAfterStop)
	ori r15, %lo(debuggerAfterStop)
	jalrs r15
	nop
	
    mov r12, r9

    lhi r15, %dhi(regs)
	ori r15, %dlo(regs)
	; saving global registers
	s18 0(r15), r0
	s18 2(r15), r1
    s18 4(r15), r2
    s18 6(r15), r3
    ; save my input regs (callers output regs) ; evolutionary reason why this is not done after moving the reg window (i could read them before i knew how to do that)
    s18 24(r15), r4
    s18 26(r15), r5
    s18 28(r15), r6
    s18 30(r15), r7
    ; backup destination to global r0
    mov r0, r15
    ; move register windows 1 back
    movs2i r1, sfr_status
    mov r3, r1 ;save original sfr_status to r3
    movi r2, 1
    sub r1, r2				;only first 7 bits are reg_window, but if this is done inside a trap, reg_window __HAS__ to be at least 1!!
    movi2s sfr_status, r1 ; move reg_window
    nop ; wait for move to take effect
    ; save callers input regs
    s18 8(r0), r4
    s18 10(r0), r5
    s18 12(r0), r6
    s18 14(r0), r7
    ; save callers local regs
    s18 16(r0), r8
    s18 18(r0), r9
    s18 20(r0), r10
    s18 22(r0), r11
    ; restore original sfr_status
    movi2s sfr_status, r3
    nop ; wait for move to take effect
    ; restore original global regs
    l18 r0, 0(r15)
    mov r13, r1 ; save sfr_status of this frame
    l18 r1, 2(r15)
    l18 r2, 4(r15)
    l18 r3, 6(r15)
    
    ;offset of 31 is limit for s18 x(r15), so add manually
    addi r15, 32
    
    ;save sfr_mul
    ;movs2i r8, sfr_mul
    ;s18 0(r15), r8

    ;save sfr_status
    s18 0(r15), r13
    
    ;save breakpoint pc
    seqi r12, 0
    mov r8, r11 ;convert from word address to byte address
    beqzc skip_sub_pc
    NOP

    movi r14, 1
    sub r8, r14
    
skip_sub_pc:
	lhi r14, %dhi(fixup_brk_addr)
	ori r14, %dlo(fixup_brk_addr)
	s18 0(r14), r8
	slli r8, 2
	s18 2(r15), r8

    movi r13, 0
    lhi r8, %dhi(update_regs)
    ori r8, %dlo(update_regs)
    s9 0(r8), r13
    mov r10, r15 ; keep pointer used to access regs[PC]


    jals handle_exception
	nop;TODO

    ;lhi r8, %dhi(update_regs) ;should still be there
    ;ori r8, %dlo(update_regs)
    l9 r8, 0(r8)
check_reset_regwindow:
    andi r8, 31
    sgei r8, 16
    beqzc check_write
    NOP
    
reset_regwindow:
    movi r8, 0
    movi2s sfr_status, r8
    movi r0, 0
    jr r0
    nop

check_write:
	andi r8, 15
    sgei r8, 8
    beqzc check_single_step
    NOP

write_back:
	lhi r0, %dhi(regs)
	ori r0, %dlo(regs)
    movs2i r1, sfr_status ;read reg_window
    mov r3, r1 ; backup reg_window in r3
    movi r2, 1
    sub r1, r2 ; reg_window -1
    movi2s sfr_status, r1 ; move window
    nop ; wait for move take effect

    l18 r4, 8(r0)
    l18 r5, 10(r0)
    l18 r6, 12(r0)
    l18 r7, 14(r0)

    l18 r8, 16(r0)
    l18 r9, 18(r0)
    l18 r10, 20(r0)
    l18 r11, 22(r0)

    l18 r12, 24(r0)
    l18 r13, 26(r0)
    l18 r14, 28(r0)
    l18 r15, 30(r0)

    movi2s sfr_status, r3 ; move window back
    nop ; wait for move to take effect
    
    ;write SFR back
    mov r3, r0
    addi r3, 32
    l18 r2, 0(r3)
    movi2s SFR_MUL, r2

    l18 r3, 6(r0); caution these registers are backed up and restored as part of each c function, this only works outside of a c function
    l18 r2, 4(r0)
    l18 r1, 2(r0)
    l18 r0, 0(r0) ; while r3-r1 will simply be overwritten by gcc, r0 is the stack pointer and must not be set to anything else

check_single_step:
	andi r8, 7
    sgei r8, 4
    movi r9, 0
    beqzc check_fixup_brk
    NOP
    
single_step_next:
	movi r9, 2
	
check_fixup_brk:
	andi r8, 3
	seqi r8, 2 ; only fixup_brk, no continue at
	beqzc check_continue
    	NOP

fixup_brk:
	lhi r10, %dhi(fixup_brk_addr)
	ori r10, %dlo(fixup_brk_addr)
	l18 r11, 0(r10)
	j breakpoint_end
	nop;TODO

check_continue:
	andi r8, 1
    sgei r8, 1
    beqzc breakpoint_end
    NOP

continue_at:
    l18 r11, 2(r10)
    srli r11,2 ; convert from byte address to word

breakpoint_end:

	lhi r14, %hi(debuggerBeforeContinue)
	ori r14, %lo(debuggerBeforeContinue)
	jalrs r14
	nop

	ori r9, 1
	movi r8, 0
	movi2s SFR_DBG_IDX, r8
	movi2s SFR_DBG_DAT, r9
	
	seqi r9, 3 ; if single step next?
	bnezc leave_interrupts_off ;; leave interrupts off, if only single stepping
	NOP

activate_interrupts:
	lhi r14, %dhi(backup_cc)
	ori r14, %dlo(backup_cc)
	l9 r15, 0(r14)
	movi2s SFR_CC, r15 ; restore CC

	jrs r11
	sbits INT

leave_interrupts_off:
	lhi r14, %dhi(backup_cc)
	ori r14, %dlo(backup_cc)
	l9 r15, 0(r14)
	movi2s SFR_CC, r15 ; restore CC

	jrs r11
	cbits INT


; param1: dest array addr
; param2: src reg window
.global capture_window
capture_window:
    ; save global regs to THIS local
	mov r9, r1
	mov r10, r2

	; move params necessary for saving window to globals
	mov r1, r4 ; target addr in r1
	movs2i r2, SFR_STATUS; org window in r2
	lhi r13, 0x1FF
	ori r13, 0x180
	and r13, r2
	or r13, r5
	movi2s SFR_STATUS, r13
	nop

    ; save input regs
    s18  0(r1), r4
    s18  2(r1), r5
    s18  4(r1), r6
    s18  6(r1), r7
    ; save local regs
	s18  8(r1), r8
    s18 10(r1), r9
    s18 12(r1), r10
    s18 14(r1), r11
    ; save output regs
    s18 16(r1), r12
    s18 18(r1), r13
    s18 20(r1), r14
    s18 22(r1), r15

    ; restore original sfr_status
    movi2s SFR_STATUS, r2
    nop ; wait for move to take effect

    ; restore original global regs
    mov r1, r9
	mov r2, r10

    jrs r11
	nop

; param1: src array addr
; param2: dest reg window
.global write_window
write_window:
    ; save global regs to THIS local
	mov r9, r1
	mov r10, r2

	; move params necessary for saving window to globals
	mov r1, r4 ; target addr in r1
	movs2i r2, SFR_STATUS; org window in r2
	lhi r13, 0x1FF
	ori r13, 0x180
	and r13, r2
	or r13, r5
	movi2s SFR_STATUS, r13
	nop

    ; write input regs
    l18 r4 , 0(r1)
    l18 r5 , 2(r1)
    l18 r6 , 4(r1)
    l18 r7 , 6(r1)
    ; write local regs
	l18 r8 ,  8(r1)
    l18 r9 , 10(r1)
    l18 r10, 12(r1)
    l18 r11, 14(r1)

    ; restore original sfr_status
    movi2s SFR_STATUS, r2
    nop ; wait for move to take effect

    ; restore original global regs
    mov r1, r9
	mov r2, r10

    jrs r11
	nop
	
