/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#include <mpsoc.h>

unsigned int concentrator_master_read_data_nb(concentrator_master_regs_t *master, void *data, unsigned int * count, unsigned int * sender_id) {

    //check if any data in fifo
	master->msg_size = 1;
    if (master->status == BUFFER_EMPTY) return CONCENTRATOR_RECEIVE_NO_DATA_AVAILABLE;

	//decode header only peek to preserve for next call if necessary
    unsigned int header = master->peek_data;
    unsigned int id = header & 0x003F;
    unsigned int msgsize = (header >> 6);
    *sender_id = id;
    *count = msgsize;

	//check if entire msg is in buffer
	master->msg_size = msgsize;
	if (master->status == BUFFER_EMPTY) return CONCENTRATOR_RECEIVE_NO_ENTIRE_MESSAGE_AVAILABLE;

	//no more need to preserve header...delete it from fifo
	master->data_out;

	copy_reg_to_mem(&master->data_out, data, msgsize);

    return CONCENTRATOR_RECEIVE_OK;
}

void concentrator_master_read_data(concentrator_master_regs_t *master, void * data, unsigned int * count, unsigned int * sender_id) {

	//wait for any data in fifo
	master->msg_size = 1;
    while (master->status == BUFFER_EMPTY);

	//decode header
    unsigned int header = master->data_out;
    unsigned int id = header & 0x003F;
    unsigned int msgsize = (header >> 6);
    *sender_id = id;
    *count = msgsize;

	//check if entire msg is in buffer
	master->msg_size = msgsize;
	if (master->status == DATA_AVAILABLE) {
		copy_reg_to_mem(&master->data_out, data, msgsize);
	} else {
		copy_reg_to_mem_with_small_buffer(&master->data_out, data, msgsize, &master->msg_size, &master->status, DATA_AVAILABLE);
	} 
}
