/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#include <mpsoc.h>

unsigned int concentrator_slave_send_data_nb(concentrator_slave_regs_t *slave, unsigned int * data, unsigned int msgsize) {
	
	//set data available request for message and header
    slave->msg_size = msgsize + 1;
    slave->data_available = 1;
    //return if no send permission or not enough buffer space available
    if (slave->status != BUFFER_AVAILABLE){
        slave->data_available = 0;
        return CONCENTRATOR_SEND_NO_SEND_PERMISSION;
    }

    //write header
    slave->data_in = (msgsize<<1)|1;

	//copy whole message
    copy_mem_to_reg(data, &slave->data_in, msgsize);

	slave->data_available = 0;

    return CONCENTRATOR_SEND_OK;
}

void concentrator_slave_send_data(concentrator_slave_regs_t *slave, unsigned int * data, unsigned int msgsize) {
    
	//set data available request for message and header
    slave->msg_size = 1;
	//write the msg size for the header and set data available flag
    slave->data_available = (msgsize<<1)|1;

    //wait for send permission
	while(slave->status == NO_SEND_PERMISSION);

    //write header if the buffer is free
	while(slave->status ^ BUFFER_AVAILABLE);
    slave->data_in = 1;
	
	//reset msg size to jump into apropriate function
	slave->msg_size = msgsize;
    //check if enough space in buffer for entire message
    if(slave->status == BUFFER_FULL){
		//copy in chunks
        copy_mem_to_reg_with_small_buffer(data, &slave->data_in, msgsize, &slave->msg_size, &slave->status, BUFFER_AVAILABLE);
    }
    else{
        //copy whole message
        copy_mem_to_reg(data, &slave->data_in, msgsize);
    }
    slave->data_available = 0;
}
