/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


#include <mpsoc.h>

int core_connector_master_send_value_nb(core_connector_master_regs_t *conn, unsigned int value) {

    conn->msg_size = 1;
	if (conn->status == BUFFER_FULL) {
		return CORE_CONN_FULL;
	}
    conn->data_in = value;
	return CORE_CONN_OK;
}

void core_connector_master_send_value(core_connector_master_regs_t *conn, unsigned int value) {

    conn->msg_size = 1;
	while(conn->status == BUFFER_FULL);
    conn->data_in = value;
}

int core_connector_master_send_data_nb(core_connector_master_regs_t *conn, unsigned int *data, unsigned int count) 
{
    conn->msg_size = count;
    		
    if (conn->status == BUFFER_FULL) {
    	return CORE_CONN_EMPTY;
   	}

	copy_mem_to_reg(data, &conn->data_in, count);

	return CORE_CONN_OK;
}

void core_connector_master_send_data(core_connector_master_regs_t *conn, unsigned int *data, unsigned int count) {

	conn->msg_size=count;
	// see if the buffer is big enough for the whole message
	if(conn->status == BUFFER_FULL){
		// no: send in chunks of 16
		copy_mem_to_reg_with_small_buffer(data, &conn->data_in, count, &conn->msg_size, &conn->status, BUFFER_AVAILABLE);
	}else{
		// yes: send the whole message without checking for free space each 16 blocks 
		copy_mem_to_reg(data, &conn->data_in, count);
	}
}
