/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


#include <mpsoc.h>

int core_connector_slave_receive_value_nb(core_connector_slave_regs_t *conn, unsigned int * value) {

    conn->msg_size = 1;
    if (conn->status == BUFFER_EMPTY) {
        return CORE_CONN_EMPTY;
    }
    *value = conn->data_out;
    return CORE_CONN_OK;
}

unsigned int core_connector_slave_receive_value(core_connector_slave_regs_t *conn) {

	conn->msg_size = 1;
	while(conn->status == BUFFER_EMPTY);
    return conn->data_out;
}

int core_connector_slave_receive_data_nb(core_connector_slave_regs_t *conn, unsigned int * data, unsigned int count) {
	
	conn->msg_size = count;

    if (conn->status == BUFFER_EMPTY) {
       	return CORE_CONN_EMPTY;
    }

    copy_reg_to_mem(&conn->data_out, data, count);

    return CORE_CONN_OK;
}

void core_connector_slave_receive_data(core_connector_slave_regs_t *conn, unsigned int * data, unsigned int count) {

	conn->msg_size=count;
	// see if the buffer already contains the whole message
	if(conn->status == BUFFER_EMPTY){
		// no: receive in chunks of 16
		copy_reg_to_mem_with_small_buffer(&conn->data_out, data, count, &conn->msg_size, &conn->status, DATA_AVAILABLE);
	}else{
		// yes: receive the whole message without checking for data each 16 blocks 
		copy_reg_to_mem(&conn->data_out, data, count);
	}
}
