/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

#include <mpsoc.h>

unsigned int dispatcher_slave_read_data_nb(dispatcher_slave_regs_t *slave, unsigned int * data, unsigned int * count) {

	//check if any data for slave available
	slave->msg_size = 1;
	if (slave->status != DATA_AVAILABLE) return DISPATCHER_RECEIVE_NO_DATA_AVAILABLE;

	//get size of incoming message
    unsigned int msgsize = slave->msg_size_in;
    *count = msgsize;

	//check if entire message is in buffer
	slave->msg_size = msgsize;
	if (slave->status != DATA_AVAILABLE) return DISPATCHER_RECEIVE_NO_ENTIRE_MESSAGE_AVAILABLE;

	//copy data
	copy_reg_to_mem(&slave->data_out, data, msgsize);

	//confirm receive
    slave->received = 1;
    return DISPATCHER_RECEIVE_OK;
}


void dispatcher_slave_read_data(dispatcher_slave_regs_t *slave, unsigned int * data, unsigned int * count) {

    //wait for incoming message
	slave->msg_size = 0;
	while (slave->status != DATA_AVAILABLE);

	//get size of message
    unsigned int msgsize = slave->msg_size_in;
    *count = msgsize;

	//copy data
	slave->msg_size = msgsize;

	if(slave->status != DATA_AVAILABLE) {
		//not entire message in buffer
		copy_reg_to_mem_with_small_buffer(&slave->data_out, data, msgsize, &slave->msg_size, &slave->status, DATA_AVAILABLE);
	} else {
		//entire message already in buffer
		copy_reg_to_mem(&slave->data_out, data, msgsize);
	}

	//confirm receive
    slave->received = 1;
} 
