/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#include "arp.h"

int addArpEntry(struct arpData* data, struct arpEntry* newEntry){

	int temp;
/*
	monitor_print("add arpEntry MAC\r\n");
	monitor_putint(newEntry->mac[0]);
	monitor_print("\r\n");
	monitor_putint(newEntry->mac[1]);
	monitor_print("\r\n");
	monitor_putint(newEntry->mac[2]);
	monitor_print("\r\nIP:\r\n");
	monitor_putint(newEntry->ip[0]);
	monitor_print("\r\n");
	monitor_putint(newEntry->ip[1]);
	monitor_print("\r\n");
*/
	temp = searchArpEntry(data, newEntry->mac);

	if(temp < 0){	//no entry with matching mac
		if(data->nextOffset < data->maxOffset){
			//table has empty slots
			data->nextOffset++;
		}else{	//table is full
			//remove oldest entry
			for(temp=0;temp<data->maxOffset-1;temp++){
				replaceArpEntry(&data->table[temp],
					&data->table[temp+1]);
			}
		}
	}else{	// an entry with matching mac found
		//temp=temp surpress the warning "instruction without effect"
		for(temp=temp;temp<data->nextOffset-1;temp++){
			//move all entries above the matching one
			//one step down
			replaceArpEntry(&data->table[temp],
				&data->table[temp+1]);
		}
	}

	//add new entry
	replaceArpEntry(&data->table[data->nextOffset-1],newEntry);		
/*
	monitor_print("nextOffset ");
	monitor_putint(data->nextOffset);
	monitor_print("\r\n");
*/
	return data->nextOffset;
}

int searchArpEntry(struct arpData* data, const unsigned int* mac){

	int i=0;
	//int x=0;
	int equal=1;

	for(i=0;i<=data->nextOffset-1;i++){
		equal=1;

		/*for(x=0;x<3;x++){
			if(mac[x] != data->table[i].mac[x]){
				equal=0;
				break;
			}
		}*/

		//have to be done this way or lcc can not compile it
		if(mac[0] != data->table[i].mac[0]) equal=0;
		if(mac[1] != data->table[i].mac[1]) equal=0;
		if(mac[2] != data->table[i].mac[2]) equal=0;

		if(equal){
			return i;
		}
	}

	return -1;
}

void replaceArpEntry(struct arpEntry* dest, struct arpEntry* src){
	//copy mac
	dest->mac[0] = src->mac[0];
	dest->mac[1] = src->mac[1];
	dest->mac[2] = src->mac[2];

	//copy ip
	dest->ip[0] = src->ip[0];
	dest->ip[1] = src->ip[1];
}

int handleArpFrame(const volatile unsigned int* frame, struct arpData* data,
	volatile unsigned int* sendFrame, struct arpEntry* own){

/*
	monitor_print("frame[7]: ");
	monitor_putint(frame[7]);
	monitor_print("\r\n");
	monitor_print("frame[8]: ");
	monitor_putint(frame[8]);
	monitor_print("\r\n");
	monitor_print("frame[9]: ");
	monitor_putint(frame[9]);
	monitor_print("\r\n");
	monitor_print("frame[10]: ");
	monitor_putint(frame[10]);
	monitor_print("\r\n");
*/
	//check if hardware address space is ethernet
	if(frame[7] != 1) return -1;

	//check if protocoll address space is IP
	if(frame[8] != 0x0800) return -1;

	//check address lengths of both address spaces
	//MAC is 6 byte long and IP 4 byte
	if(frame[9] != 0x604) return -1;

	//add source fields to arp table
	addArpEntry(data, (struct arpEntry*)&frame[11]);

	//determine opcode
	if(frame[10] == 1){	//ARP request
		//are broadcasts
		//target mac field is invalid
/*
		monitor_print("own IP:\r\n");
		monitor_putint(own->ip[0]);
		monitor_print("\r\n");
		monitor_putint(own->ip[1]);
		monitor_print("\r\n");
		monitor_print("desired ip\r\n");
		monitor_putint(frame[19]);
		monitor_print("\r\n");
		monitor_putint(frame[20]);
		monitor_print("\r\n");
*/
		
		if((frame[19] == own->ip[0]) && (frame[20] == own->ip[1])){
			//own ip address
			writeArpReply(sendFrame,own,
				(struct arpEntry*)&frame[11]);

			return 1;
		}

	}else if(frame[10] != 2){	//no ARP reply or ARP request
		return -1;
	}

	return 0;
}

void writeArpReply(volatile unsigned int* sendFrame, struct arpEntry* own,
	struct arpEntry* target){

	//dest MAC
	sendFrame[1]=target->mac[0];
	sendFrame[2]=target->mac[1];
	sendFrame[3]=target->mac[2];
	//src MAC
	sendFrame[4]=own->mac[0];
	sendFrame[5]=own->mac[1];
	sendFrame[6]=own->mac[2];
	//ether type
	sendFrame[7]=0x806;
	//hardware address
	sendFrame[8]=1;
	//protocoll address
	sendFrame[9]=0x800;
	//length of both addresses
	sendFrame[10]=0x604;
	//opcode
	sendFrame[11]=2;
	//source mac
	sendFrame[12]=own->mac[0];
	sendFrame[13]=own->mac[1];
	sendFrame[14]=own->mac[2];
	//source ip
	sendFrame[15]=own->ip[0];
	sendFrame[16]=own->ip[1];
	//target mac
	sendFrame[17]=target->mac[0];
	sendFrame[18]=target->mac[1];
	sendFrame[19]=target->mac[2];
	//target ip
	sendFrame[20]=target->ip[0];
	sendFrame[21]=target->ip[1];
	
	//terminate
	sendFrame[31]=0;
	//write length
	sendFrame[0]=60;
}

void writeArpRequest(volatile unsigned int* sendFrame, struct arpEntry* own,
	struct arpEntry* target){

	//dest MAC
	sendFrame[1]=0xffff;
	sendFrame[2]=0xffff;
	sendFrame[3]=0xffff;
	//src MAC
	sendFrame[4]=own->mac[0];
	sendFrame[5]=own->mac[1];
	sendFrame[6]=own->mac[2];
	//ether type
	sendFrame[7]=0x806;
	//hardware address
	sendFrame[8]=1;
	//protocoll address
	sendFrame[9]=0x800;
	//length of both addresses
	sendFrame[10]=0x604;
	//opcode
	sendFrame[11]=1;
	//source mac
	sendFrame[12]=own->mac[0];
	sendFrame[13]=own->mac[1];
	sendFrame[14]=own->mac[2];
	//source ip
	sendFrame[15]=own->ip[0];
	sendFrame[16]=own->ip[1];
	//target mac
	sendFrame[17]=0;
	sendFrame[18]=0;
	sendFrame[19]=0;
	//target ip
	sendFrame[20]=target->ip[0];
	sendFrame[21]=target->ip[1];
	
	//terminate
	sendFrame[31]=0;
	//write length
	sendFrame[0]=60;
}

