/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#include <axi.h>

//Initiates an incremental AXI write transaction with a certain start address (parameter addr),
//and burst length (parameter blen) on the buffer selected by the parameter bsel.
//Continues when transaction is finished.

void axi_write_sync(axi_dma_t *axi_p, uint9_t bsel, uint36_t addr, uint9_t blen) {
	volatile unsigned int *w_ctrl;
	volatile unsigned int *w_addr_0, *w_addr_1;
	
	//select control and address registers according to buffer select
	w_ctrl    = (bsel == 0) ? &axi_p->w_ctrl_0    : &axi_p->w_ctrl_1;
	w_addr_0  = (bsel == 0) ? &axi_p->w_addr_0_0  : &axi_p->w_addr_1_0;
	w_addr_1  = (bsel == 0) ? &axi_p->w_addr_0_1  : &axi_p->w_addr_1_1;

	//write address registers
	*w_addr_0 = 0xFFFF & addr;
	*w_addr_1 = 0xFFFF & (addr >> 16);
	//write burst mode, valid bit and burst length to contol register
	*w_ctrl = AXI_CTRL_TYPE_INCR | AXI_CTRL_VALID | (blen & AXI_CTRL_BLEN);
	//poll done bit
	while ((*w_ctrl & AXI_CTRL_DONE) == 0);
}
