/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

#include <mpsoc.h>

#pragma GCC diagnostic ignored "-Wimplicit-fallthrough"

__attribute__((optimize("unroll-loops")))
void copy_reg_to_mem(const volatile unsigned int * const source_reg, unsigned int *  target_mem_start, unsigned int count) {
	if(count == 0) {
		return;
	}
    int n = (count + 15) / 16;
    int switch_var = count%16;
    unsigned int * dat = (void *) ((unsigned int)target_mem_start - 2*(16-switch_var));
    switch (switch_var) {
        case 0	:do{dat=dat+16;
                *(dat)   = *source_reg;
                case 15	:	*(dat+1) = *source_reg;
                case 14	:	*(dat+2) = *source_reg;
                case 13	:	*(dat+3) = *source_reg;
                case 12	:	*(dat+4) = *source_reg;
                case 11	:	*(dat+5) = *source_reg;
                case 10	:	*(dat+6) = *source_reg;
                case 9	:	*(dat+7) = *source_reg;
                case 8	:	*(dat+8) = *source_reg;
                case 7	:	*(dat+9) = *source_reg;
                case 6	:	*(dat+10) = *source_reg;
                case 5	:	*(dat+11) = *source_reg;
                case 4	:	*(dat+12) = *source_reg;
                case 3	:	*(dat+13) = *source_reg;
                case 2	:	*(dat+14) = *source_reg;
                case 1	:	*(dat+15) = *source_reg;
            }while(--n > 0);
    }
}


__attribute__((optimize("unroll-loops")))
void copy_mem_to_reg(const unsigned int * const source_mem_start, volatile unsigned int * const target_reg, unsigned int count) {
	if(count == 0) {
		return;
	}
    int n = (count + 15) / 16;
    int switch_var = count%16;
    unsigned int * dat = (void *) ((unsigned int)source_mem_start - 2*(16-switch_var));

    switch (switch_var) {
        case 0	:do{dat=dat+16;
                *target_reg = *(dat);
                case 15	:	*target_reg = *(dat+1);
                case 14	:	*target_reg = *(dat+2);
                case 13	:	*target_reg = *(dat+3);
                case 12	:	*target_reg = *(dat+4);
                case 11	:	*target_reg = *(dat+5);
                case 10	:	*target_reg = *(dat+6);
                case 9	:	*target_reg = *(dat+7);
                case 8	:	*target_reg = *(dat+8);
                case 7	:	*target_reg = *(dat+9);
                case 6	:	*target_reg = *(dat+10);
                case 5	:	*target_reg = *(dat+11);
                case 4	:	*target_reg = *(dat+12);
                case 3	:	*target_reg = *(dat+13);
                case 2	:	*target_reg = *(dat+14);
                case 1	:	*target_reg = *(dat+15);
            }while(--n > 0);
    }

}

__attribute__((optimize("unroll-loops")))
void copy_reg_to_mem_with_small_buffer(const volatile unsigned int *source_reg, const unsigned int * target_mem_start, unsigned int count, volatile unsigned int * msg_size_reg, volatile unsigned int *status_reg, int send_free) {
	if(count == 0) {
		return;
	}
    int n = (count + 15) / 16;
    int switch_var = count%16;
    unsigned int * dat = (void *) ((unsigned int)target_mem_start - 2*(16-switch_var));


    *msg_size_reg = switch_var;

    while(*status_reg ^ send_free);
    *msg_size_reg=16;

    switch (switch_var) {
        case 0	:do{dat=dat+16;
                while(*status_reg ^ send_free);
                *(dat)   = *source_reg;
                case 15	:	*(dat+1) = *source_reg;
                case 14	:	*(dat+2) = *source_reg;
                case 13	:	*(dat+3) = *source_reg;
                case 12	:	*(dat+4) = *source_reg;
                case 11	:	*(dat+5) = *source_reg;
                case 10	:	*(dat+6) = *source_reg;
                case 9	:	*(dat+7) = *source_reg;
                case 8	:	*(dat+8) = *source_reg;
                case 7	:	*(dat+9) = *source_reg;
                case 6	:	*(dat+10) = *source_reg;
                case 5	:	*(dat+11) = *source_reg;
                case 4	:	*(dat+12) = *source_reg;
                case 3	:	*(dat+13) = *source_reg;
                case 2	:	*(dat+14) = *source_reg;
                case 1	:	*(dat+15) = *source_reg;
            }while(--n > 0);
    }
}
__attribute__((optimize("unroll-loops")))
void copy_mem_to_reg_with_small_buffer(const unsigned int *source_mem_start, volatile unsigned int * const target_reg, unsigned int count, volatile unsigned int *msg_size_reg, volatile unsigned int * status_reg, int send_free) {
    if(count == 0) {
    	return;
    }
	int n = (count + 15) / 16;
    int switch_var = count%16;
    unsigned int * dat = (void *) ((unsigned int)source_mem_start - 2*(16-switch_var));


    *msg_size_reg = switch_var;
    while(*status_reg ^ send_free);
    *msg_size_reg=16;

    switch (switch_var) {

        case 0	:do{dat=dat+16;
                while(*status_reg ^ send_free);
                *target_reg = *(dat);
                case 15	:	*target_reg = *(dat+1);
                case 14	:	*target_reg = *(dat+2);
                case 13	:	*target_reg = *(dat+3);
                case 12	:	*target_reg = *(dat+4);
                case 11	:	*target_reg = *(dat+5);
                case 10	:	*target_reg = *(dat+6);
                case 9	:	*target_reg = *(dat+7);
                case 8	:	*target_reg = *(dat+8);
                case 7	:	*target_reg = *(dat+9);
                case 6	:	*target_reg = *(dat+10);
                case 5	:	*target_reg = *(dat+11);
                case 4	:	*target_reg = *(dat+12);
                case 3	:	*target_reg = *(dat+13);
                case 2	:	*target_reg = *(dat+14);
                case 1	:	*target_reg = *(dat+15);
            }while(--n > 0);
    }

}
