;
; LCD auf dem 3esk in C
;
;
;
; Konstanten fuer die LCD Ports
; =============================
;
; 1. Timer Port fuer die Initialisierungszeiten der LCD
;
;timer		=	(io_base + 0x50)* 2
;
ti_c		=	0 * 2			; Steuerregister
ti_d		=	1 * 2			; maximaler Timerwert
ti_v		=	2 * 2			; aktueller Timerwert
;
; Konstanten fuer ti_c Register
;
ti_en		=	0x01			; Freigabe des Timer
ti_vt		=	0x02			; Freigabe des Vorteiler
ti_v1		=	0x00			; Vorteiler 2^1
ti_v2		=	0x04			;     "     2^2
ti_v3		=	0x08			;     "     2^3
ti_v4		=	0x0c			;     "     2^4
ti_v5		=	0x10			;     "     2^5
ti_v6		=	0x14			;     "     2^6
ti_v7		=	0x18			;     "     2^7
ti_v8		=	0x1c			;     "     2^8
;
ti_ini2		=	ti_en + ti_vt + ti_v8	; bei 25MHz 0,04us*2^8=10,24us
ti_init		=	ti_en + ti_vt + ti_v1	; bei 25MHz 0,04us*2^1= 0,08us
;
ti_end		=	262143			; 262143*0,08us = 20,97144ms
ti_max		=	262142			; 262142*0,08us = 20,97136ms
; die folgenden Zeiten wurden fuer die LCD auf dem ml507 Board vergroessert.
ti41		=	52329			;  51328*0,08us =  4,10624ms
ti16		=	21481			;  20480*0,08us =  1,63840ms
ti01		=	1381			;   1280*0,08us =  0,10240ms
ti004		=	510			;    500*0,08us =  0,04000ms
ti0004		=	50			;     50*0,08us =  0,00400ms
ti0001		=	13			;     13*0,08us =  0,00104ms
ti320n		=	3			;      4*0,08us =  0,00032ms 3 wegen Overhead von wait
ti160n		=	1			;      2*0,08us =  0,00016ms 1 wegen Overhead von wait
;
;
; 2. BI-direktionales Port zur Ansteuerung der LCD
;
;lcd		=	(io_base + 0x78)* 2
;
; Konstanten fuer LCD Port
;
; LCD im 4 Bit Mode mit diesem Port verbunden
;
; Bit 6   Bit 5  Bit 4  Bit 3  Bit 2  Bit 1  Bit 0
; R/W     RS     E      DB 7   DB 6   DB 5   DB 4
;
lcd_da		=	0 * 2			; Daten mit Bits siehe oben
lcd_di		=	3 * 2			; i/o Umschaltung fuer jedes Bit (1=Input)
lcd_dw		=	0x00			; alle Bits Output
lcd_dr		=	0x0f			; DB[7:4] sind Input
lcd_oe		=	2 * 2			; Freigabe der Outputs (1=Enable)
lcd_ow		=	0x7f			; Output Enable fuer alle Bit
lcd_or		=	0x70			; unteren 4 Bit sperren
;
		.text
;
; Programme fuer die LCD auf dem 3esk
;                    ================
;
; ***** lcd_init  --  lcd in C initialisieren *****
; input:   r4  =  adr. LCD Port
;          r5  =  adr. TIMER Port
;          r6  =  Vorteiler vom TIMER (1...8)
;
.global	lcd_init
lcd_init:
;
; Timer initialisieren
;
;		load	(r7,	ports)
		lhi	r7,	%hi(ports)
		ori	r7,	%lo(ports)
		s18	0(r7),	r4		; LCD_BASE
		s18	2(r7),	r5		; TIMER_BASE
		addi	r6,	-1		; 2^8 = 7 bis 2^1 = 0
		andi	r6,	7		; nur 3 Bit zulaessig
		muli	r6,	4		; um 2 Bit nach links verschieben
		ori	r6,	ti_init & 0x1ff	; Timer und Vorteiler Freigabe dazu
		s18	ti_c(r5), r6		; Timer initialisieren
;		load	(r6,	ti_end)		; 20,97144ms
		lhi	r6,	ti_end >> 9
		ori	r6,	ti_end & 0x1ff
		s18	ti_d(r5), r6		; Timer immer bis ti_end laufen lassen.
;
; LCD Port initialisieren
;
;		load	(r6,	lcd_dw)
		movi	r6,	lcd_ow
		s18	lcd_oe(r4), r6		; Alle Outputs freigeben
		movi	r6,	lcd_dw
		s18	lcd_di(r4), r6		; Alle Bits auf Output setzen
;
; Initialisierung der LCD beginnen
;
; 1. Ausgabe -- Funktion Set (wird noch als 8 Bit Kommando betrachtet)
;
		movi	r12,	0x003		; R/W=0 RS=0 E=0 DB[7:4]=3
		jals	e_clk
		nop
;		load	(r13,	ti41)
		lhi	r13,	ti41 >> 9
		ori	r13,	ti41 & 0x1ff
		jals	wait			; 4,1ms Warten
		nop
;
; 2. Ausgabe -- Wiederholen
;
		jals	e_clk
		nop
;		load	(r13,	ti01)
		lhi	r13,	ti01 >> 9
		ori	r13,	ti01 & 0x1ff
		jals	wait			; 0,102ms Warten
		nop
;
; 3. Ausgabe -- Wiederholen
;
		jals	e_clk
		nop
;		load	(r13,	ti004)
		movi	r13,	ti004
		jals	wait			; 0,040ms Warten
		nop
;
; 4. Ausgabe -- auf 4 Bit umstellen (wird noch als 8 Bit Kommando betrachtet)
;
		movi	r12,	0x002		; R/W=0 RS=0 E=0 DB[7:4]=2
		jals	e_clk
		nop
;		load	(r13,	ti004)
		movi	r13,	ti004
		jals	wait			; 0,040ms Warten
		nop
;
; LCD sollte nun im 4 Bit Mode arbeiten. Ab jetzt immer 2*4Bit senden
; und dann warten auf das Busy Flag der LCD. 
;
; Byte Kommandokette zur Initialisierung der LCD senden
;
;		load	(r12,	msg0)		; Initialisierungs Byte Folge
		lhi	r12,	%hi(msg0)
		ori	r12,	%lo(msg0)
		xor	r13,	r13		; CMD senden
		jals	strg
		nop
;		load	(r12,	msg1)		; ASCII Zeichen
;		lhi	r12,	%hi(msg1)
;		ori	r12,	%lo(msg1)
;		jals	lcd_s
;
		jrs	r11
		or	r0,	r0
;
; LCD initialisiert.
;
;
; LCD Unterprogramme
; ==================
;
; CU ist in Zeile 2.
; Zeile 2 in Zeile 1 umladen und Zeile 2 danach loeschen.
;
zei2:		movi	r8,	16		; 16 Zeichen umladen
		movi	r9,	0x80		; Anfangs Addr. von Zeile 1 0x80
		movi	r10,	0xc0		; Anfangs Addr. von Zeile 2 0xc0
umla:		addi	r8,	-1
		mov	r12,	r10
		xor	r13,	r13		; Kommando an LCD senden
		jals	out_lcd			; CU auf Zeile 2 setzen.
		nop
		jals	busy			; Warten bis Kommando ausgefuehrt ist.
		nop
		jals	lcd_rd			; Zeichen von Position in r12 laden.
		nop
		mov	r4,	r12		; Zeichen puffern.
		mov	r12,	r9
		xor	r13,	r13		; Kommando an LCD senden
		jals	out_lcd			; CU auf Zeile 1 setzen
		nop
		movi	r13,	0x20		; Daten an LCD senden
		mov	r12,	r4		; Daten aus Zeile 2
		jals	out_lcd
		nop
		addi	r9,	1		; increment CU Zeile 1
		bnez	r8,	umla		; schon alle Zeichen umgeladen?
		addi	r10,	1		; increment CU Zeile 2 (im delayslot)
;
; Position 0 auf Zeile 2 setzen (Bei TM162 nicht durch CRLF moeglich)
;
		movi	r12,	0xc0		; Anfangs Addr. von Zeile 2 0xc0
		xor	r13,	r13		; Kommando an LCD senden
		jals	out_lcd			; CU auf Zeile 1 setzen
		nop
;
; Zeile 2 nun noch loeschen
;
;		load	(r4,	clr_str)
		lhi	r4,	%hi(clr_str)
		ori	r4,	%lo(clr_str)
		j	lcd_s
		nop
;
;
; LF Zeile nach oben rollen, wenn CU in der unteren Zeile
;
roll:		movi	r13,	ti0004
		jals	wait			; 0,004ms Warten
		nop
		jals	busy			; Status noch einmal lesen, da ADDR erst jetzt inkrementiert ist.
		nop
		mov	r8,	r14		; Status high
		andi	r8,	4
		bnez	r8,	zei2		; CU ist in Zeile 2
		or	r0,	r0
;
; CU ist noch in der oberen Zeile
;
		mov	r12,	r14
		xor	r13,	r13
		movi	r14,	4
		jals	sll36			; sll36
		nop
		or	r12,	r15		; unteren 4 Bit einblenden
		ori	r12,	0xc0		; CMD write Addr. fuer Zeile 2 gebildet
		jals	o_lcd			; LF = an Position in Zeile 2 wechseln.
		nop
		jrs	r11
		or	r0,	r0
;
; CLS auf LCD
;
lcd_cls:	movi	r12,	1		; CLR Display
		xor	r13,	r13		; CMD Senden
		jals	o_lcd
		nop
		j	busy
		nop
;
; CR --> Zum Zeilenanfang
;
zanf:		mov	r12,	r14
		andi	r12,	4
		xor	r13,	r13
		movi	r14,	4
		jals	sll36			; sll36
		nop
		ori	r12,	0x80		; CMD write Addr. gebildet
		jals	o_lcd			; An den Anfang der Zeile setzen.
		nop
		jrs	r11
		or	r0,	r0
;
;
; UP zur Ausgabe von ASCII Zeichen mit Steuerzeichen CR, LF und FF
; ================================
;
; Input r12 (= intern r4) = ASCII Zeichen (auch Steuerzeichen)
;
; ***** lcd_putchar  --  Zeichen in C ausgeben *****
;
.global	lcd_putchar
lcd_putchar:
lcd_dat:	jals	busy			; Warten bis LCD wieder bereit ist.
		seqi	r4,	0x0d		; CR zum Zeilenanfang
		or	r0,	r0
		bnezc	zanf
;
		seqi	r4,	0x0a		; LF Zeile nach oben rollen, wenn CU in der unteren Zeile
		or	r0,	r0
		bnezc	roll
;
		seqi	r4,	0x0c		; FF = CLS
		or	r0,	r0
		bnezc	lcd_cls
;
; Vor der Ausgabe des Zeichens auf das Zeilenende Testen
;
;		load	(r13,	ti0004)
		movi	r13,	ti0004
		jals	wait			; 0,004ms Warten
		nop
		jals	busy			; Status noch einmal lesen, da ADDR erst jetzt inkrementiert ist.
		nop
		mov	r8,	r14		; Status high
		andi	r8,	1		; 2^4 der CU Adresse = Position 16 in der Zeile --> andere Zeile.
		beqz	r8,	no_zend		; noch nicht am Zeilen Ende
		mov	r12,	r14		; wird im delay slot abgearbeitet
;
		andi	r12,	4
		xori	r12,	4		; Bit invertieren
		xor	r13,	r13
		movi	r14,	4
		jals	sll36			; sll36
		nop
		ori	r12,	0x80		; CMD write Addr. gebildet
		jals	o_lcd			; An den Zeilen Anfang der anderen Zeile setzen.
		nop
;
; Zeichen noch ausgeben
;
no_zend:	movi	r5,	0x20		; ASCII senden
		j	out_lcd
		nop
;
; UP READ ASCII Puffer von der LCD nach r12
;
lcd_rd:		movi	r14,	0x20
		jals	rbusy			; read LCD
		nop
		mov	r12,	r14
		xor	r13,	r13
		movi	r14,	4
		jals	sll36			; sll36
		nop
		or	r12,	r15		; low Teil der Daten einfuegen
		mov	r4,	r12
		jrs	r11
		or	r0,	r0
;
;
; sll r12-r13 um r14 Stellen schieben
;
;		im Aufruf	intern
; Wert 36 Bit:	R12 | R13	R4 | R5
; Schiebeschritte:    R14	     R6
;
sll36:		addi	r6,	-1
		slli	r5,	1
		movs2i	r9,	SFR_CC
		slli	r4,	1
		bnez	r6,	sll36
		or	r4,	r9		; CC einfuegen im delay slot
		jrs	r11
		or	r0,	r0
;
;
; UP Busy bit testen lesen
;
busy:		xor	r6,	r6
;
; Warten auf Busy Signal des LCD Modul
;
; Input  r14 (= intern r6) = Maske fuer RS Bit = 0x00 fuer CMD (Status) lesen
;                                              = 0x20 fuer ASCII Zeichen lesen
; Output r14 (= intern r6) = Busy ## Addr[6:4] oder ASCII[7:4]
; Output r15 (= intern r7) =         Addr[3:0] oder ASCII[3:0]
;
rbusy:		movi	r10,	0x40		; Auf read Status stellen
		or	r10,	r6		; Status oder ASCII lesen.
		movi	r9,	lcd_or		; Unteren 4 Bit auf Input setzen
;		load	(r8,	ports)
		lhi	r8,	%hi(ports)
		ori	r8,	%lo(ports)
		l18	r8,	0(r8)
		s18	lcd_oe(r8), r9
		movi	r9,	lcd_dr
		s18	lcd_di(r8), r9		; DB[7:4] sind Input
		s18	lcd_da(r8), r10		; LCD auf Read setzen
		mov	r14,	r6		; Status / ASCII lesen merken.
;
busy0:		movi	r15,	2		; 2 * 4 Bit lesen.
;
busy1:		addi	r15,	-1
		ori	r10,	0x010		; E ein
		s18	lcd_da(r8), r10
		jals	w_eclk			; 160ns warten
		nop
		l18	r7,	lcd_da(r8)	; Status / ASCII lesen (nur 4 Bit)
		andi	r7,	0xf		; Steuersignale ausblenden
		jals	w_eclk			; 160ns warten
		nop
		andi	r10,	0x1ef		; E aus
		s18	lcd_da(r8), r10
;		load	(r13,	ti0001)
		movi	r13,	ti0001
		jals	wait			; 0,001ms Warten
		nop
;
		beqz	r15,	r_low
		or	r0,	r0

		mov	r12,	r7		; Status / ASCII high
		andi	r12,	8		; Busy Flag maskieren
		mov	r6,	r7		; Status / ASCII high nach r6
		j	busy1			; low noch lesen
		nop
;
r_low:		bnez	r14,	r_data		; Ist es ein Daten lesen?
		or	r0,	r0
		bnez	r12,	busy0		; LCD noch Busy?
		or	r0,	r0
;
r_data:		xor	r10,	r10
		s18	lcd_da(r8), r10		; LCD auf Write setzen
		movi	r9,	lcd_ow		; alle Bit auf Output setzen
		s18	lcd_oe(r8), r9		; DB[7:4] sind Output
		movi	r9,	lcd_dw
		s18	lcd_di(r8), r9
str_e:		jrs	r11
		or	r0,	r0
;
;
; UP ASCII Zeichenkette mit Steuerzeichen auf LCD anzeigen.
; =====================
;
; Input r12 (= intern r4) = Addr. der Zeichenkette
;
; ***** lcd_print  --  Zeichenketten in C ausgeben *****
;
.global	lcd_print
lcd_print:
lcd_s:		l9	r12,	0(r4)
		beqz	r12,	str_e
		addi	r4,	1		; wird im delay slot abgearbeitet
		jals	lcd_dat
		nop
		j	lcd_s
		nop
;
;
; ***** lcd_cmds  --  Kommandofolge an LCD in C senden *****
;
.global	lcd_cmds
lcd_cmds:	xor	r13,	r13
		j	strgz
		nop
;
; Up Zeichenkette von LCD-CMD oder ASCII ohne Steuerzeichen auf LCD anzeigen.
;    ============     =======      =====
;
; Input r12 (= intern r4) = Addr. der Zeichenkette
; Input r13 (= intern r5) = Maske fuer RS Bit = 0x00 fuer CMD
;                                             = 0x20 fuer ASCII Zeichen
;
strg:		mov	r13,	r5
strgz:		l9	r12,	0(r4)
		beqz	r12,	str_e
		addi	r4,	1		; wird im delay slot abgearbeitet
		jals	out_lcd
		nop
		j	strgz
		nop
;
;
; ***** lcd_cmd  --  Kommando an LCD in C senden *****
;
.global	lcd_cmd
lcd_cmd:	xor	r5,	r5
;
; Unterprogramm zum senden eines Byte an die LCD mit warten auf Busy
; ===================================
;
; Input r12 (= intern r4) = zu sendendes Byte
; Input r13 (= intern r5) = Maske fuer RS Bit = 0x00 fuer CMD
;                                             = 0x20 fuer ASCII Zeichen
;
out_lcd:	jals	busy			; Warten bis LCD wieder bereit ist.
o_lcd:		xor	r12,	r12
		mov	r13,	r4		; zu sendendes Byte
		movi	r14,	14		; um 14 Bit nach links in r12 schieben
		jals	sll36			; sll36 -- oberen 4 Bit in r12[3:0]
		nop
		mov	r8,	r13		; unteren 4 Bit retten in r13[17:14]
;
; Ausgabe 4 Bit high aus r4[7:4]
;
		or	r12,	r5		; R/W=0 RS=r5 E=0 DB[7:4]=r4[7:4]
		jals	e_clk
		nop
;		load	(r13,	ti0001)		; Verkuerzen um die Zeit des Daten Shift
		movi	r13,	ti0001
		jals	wait			; 0,001ms Warten **********************
		nop
;
		xor	r12,	r12
		mov	r13,	r8		; zu sendendes Byte
		movi	r14,	4		; um 4 Bit nach links in r12 schieben
		jals	sll36			; sll36 -- unteren 4 Bit in r12[3:0]
		nop
;
; Ausgabe 4 Bit low aus r4[3:0]
;
		or	r12,	r5		; R/W=0 RS=r5 E=0 DB[7:4]=r4[3:0]
		jals	e_clk
		nop
		jals	w_eclk			; warten das busy aktiv wird
		nop
		jrs	r11
		or	r0,	r0
;
; Unterprogramm Daten in r12 an LCD senden und E Clk Impuls erzeugen
;
e_clk:
;		load	(r8,	ports)
		lhi	r8,	%hi(ports)
		ori	r8,	%lo(ports)
		l18	r8,	0(r8)
		s18	lcd_da(r8), r4		; Daten an LCD senden
		ori	r4,	0x010		; E ein
		s18	lcd_da(r8), r4
		movi	r13,	ti320n		; E muss >230ns auf 1 stehen
		jals	wait			; 320ns warten
		nop
		andi	r4,	0x1ef		; E aus
		s18	lcd_da(r8), r4
		jrs	r11
		or	r0,	r0
;
; Unterprogramm fuer E Clk Zeit
;
w_eclk:		movi	r5,	ti160n		; E muss > 2*160ns auf 1 stehen
;
; Unterprogramm Wait mit input r13 (= intern r5) = Timer Zyklen
;               ====                               ============
;
wait:
;		load	(r8,	ports)
		lhi	r8,	%hi(ports)
		ori	r8,	%lo(ports)
		l18	r8,	2(r8)
		xor	r9,	r9
		s18	ti_v(r8), r9		; Start der Zeit
;		or	r0,	r0
wait0:		l18	r9, ti_v(r8)		
		sltu	r9,	r5
		or	r0,	r0
		bnezc	wait0
		jrs	r11
		or	r0,	r0
;
endlcdt	=	.
;
		.data
;
		.align9
;
; Portadressen werden in C beim Aufruf uebergeben
;
ports:	
		.word18	0			; LCD_BASE
		.word18	0			; TIMER_BASE
;
; Zeichenketten und Bytefolgen
; =============
;
; 1. Initialisierungs Kommandos fuer die LCD
;
; 7     6     5     4     3     2     1     0
; -------------------------------------------
;
; 0     0     1     DL    N     F     *     *      * = beliebig
; DL=0 - 4 Bit Interface, N=1 - 2 Zeilen Display, F=0 Zeichensatz mit 5*8 Punkten.
;
; 0     0     0     0     0     1     I/D   S
; I/D=1 - Increment CU Position, S=0 - Kein Schieben.
;
; 0     0     0     0     1     D     C     B
; D=1 - Display ein, C=0 - CU aus, B=0 - Blinken aus.
;
; 1     Add   Add   Add   Add   Add   Add   Add
; Setzen der CU Position
;
msg0:	
		.byte9	0x28			; Funktion Set mit DL=0, N=1 und F=0
		.byte9	0x06			; Entry mode set mit I/D=1 und S=0
		.byte9	0x0c			; Display on/off mit D=1, C=0 und B=0
		.byte9	0x01			; Clear Display
;
		.byte9	0			; Ende der Bytekette
;
;
; 3. Fuer die LCD
;
;msg1:		
		.ascii9	"* SpartanMC 18 *\r\n"
;		.asciz9	"TU-Dresden InfMR"
;
; 4. zum loeschen der unteren Zeile bei LF in der unteren Zeile
;
clr_str:
		.asciz9	"                \r"
;
endlcdd	=	.
;
