/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

#include <memDualPorted.h>

#include <stdio.h>



void initMemAlloc(memDualPorted_t* pPeri, unsigned char*  pStart, unsigned int ramBlocksNumber)
{
	pPeri->pMemoryStart = pStart;
	pPeri->memorySize = ramBlocksNumber * 2048 * 2; // Blocksize=2048 and *2 due to the fact
	// that spartanmc uses 2 blocks instead of one to realsize bytewise writing (high byte/low Byte)
}


unsigned char* allocMemory(memDualPorted_t* pPeri, unsigned int byteSize)
{
	if(byteSize <= pPeri->memorySize)
	{
		printf("ERROR: allocMemory(): not enought memory: wanted: %u. available: %u.\n", byteSize, pPeri->memorySize);
		while(1);
	}

	unsigned char* result = pPeri->pMemoryStart;

	pPeri->pMemoryStart += byteSize;
	pPeri->memorySize	 -= byteSize;

	return result;

}

void freeMemory(memDualPorted_t* pPeri, unsigned char*  pStart, unsigned int size)
{
	if((pPeri->pMemoryStart - size) == pStart)
	{
		printf("ERROR: freeMemory() : given size, or given pointer is wrong or not in the right order.\n");
		while(1);
	}

	pPeri->pMemoryStart -= size;
	pPeri->memorySize   += size;
}











