/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#include <videoI420.h>

#include <stdio.h>

void videoI420_init(	
						VideoI420_regs_t* v,
						unsigned int width,
						unsigned int height,
						
						unsigned int yJump,
						unsigned char* pY1,
						unsigned char* pY2,
						
						unsigned int uvJump,
						unsigned char* pU1,
						unsigned char* pU2,
	
						unsigned char* pV1,
						unsigned char* pV2,
						unsigned char scale
					)
{
	videoI420_stop(v);
	
	v->yJump	= yJump;
	v->pY1		= pY1;
	v->pY2		= pY2;
	v->uvJump	= uvJump;
	v->pU1		= pU1;
	v->pU2		= pU2;
	v->pV1		= pV1;
	v->pV2		= pV2;
	v->scale	= scale;
	
	width  = (scale+1) * width;
	height = (scale+1) * height;
	
	
	if(width > 640 || height > 480)
	{
		printf("ERROR: video output: scale*width/scale*height is too big.\n");
		while(1);
	}
	
	
	
	

	// the start and end values are used mainly to generate the 
	// boarder signal in the hardware
	// to get the boarder signal one  clock lead (before)
	// you have to substract from the counter
	// e.g. if you pixel starts at 10, you would have a startX of 9
	// or if its at 0, you would use 799. -> (0:640 img) to (0:799 full area)
	// 										 (0:480 img) to (0:524)
	
	int DELAY_FOR_HARDWARE_SIGNALS = 2;	// delay of 1 is minimum!
	// this value delays the signals of the HDMIsimple.v hardware
	// like newFrame, newPixel, and so on
	// you need also delayed the Y values in the HDMIsimply.v :(

	int tmp = ((640-width)/2)-DELAY_FOR_HARDWARE_SIGNALS;
	if(tmp>=0)
	{
		v->startX = tmp;
		v->endX   = tmp+width;
	}
	else
	{
		v->startX = 799+(1+tmp);
		v->endX   = tmp+width;
	}
	
	/*if(width<636)
	{
		int tmp = ((640-width)/2)-DELAY_FOR_HARDWARE_SIGNALS;
		v->startX = tmp;
		v->endX   = tmp+width;
	}
	else
	{
		int tmp = width - 636;
		v->startX = 799;
		v->endX   = 639;
	}*/
	
	// -1 due to the delay, as you want the signals in hardware before
	// the given row is reached
	tmp = ((480-height)/2)-1;
	
	if(tmp>=0)
	{
		v->startY = tmp;
		v->endY   = tmp+height;
	}
	else
	{
		v->startY = 524+(1+tmp);
		v->endY   = tmp+height;
	}
	
	/*if(height<478)
	{
		int tmp = ((480-height)/2)-1;
		v->startY = tmp;
		v->endY   = tmp+height;
	}
	else
	{
		//TODO
		v->startY = 524;
		v->endY	  = 479;
	}*/
	
}


void videoI420_stop(VideoI420_regs_t* v)
{
	v->enable = 0;	
}


// buffer is 1 or 2
void videoI420_start(VideoI420_regs_t* v, unsigned int buffer)
{
	buffer = buffer-1;
	buffer &= 1;	// use only one bit of the bufferID
	
	videoI420_stop(v);
	
	v->buffer = buffer;
	v->enable = 1;
}


void videoI420_next(VideoI420_regs_t* v, unsigned char* pY, unsigned char* pU, unsigned char* pV)
{
	int freeBuffer;
	
	do
	{
		freeBuffer = videoI420_getFreeBuffer(v);
	}
	while(freeBuffer==0);	// wait till at least on buffer is free
	
	if(freeBuffer==1 || freeBuffer==3)
	{
		v->pY1 = pY;
		v->pU1 = pU;
		v->pV1 = pV;
	}
	else
	{
		v->pY2 = pY;
		v->pU2 = pU;
		v->pV2 = pV;
	}
	
	if(freeBuffer==3)	// means the videoI420 is off
	{
		videoI420_start(v,1);
	}
	else
	{
		if(freeBuffer==1)
			videoI420_bufferReady(v,1);
		else
			videoI420_bufferReady(v,2);
	}
	
	
}

void videoI420_next_block(VideoI420_regs_t* v, unsigned char* pY, unsigned char* pU, unsigned char* pV)
{
	//printf("addr: %x\n",(unsigned int) pY);
	videoI420_next(v, pY, pU, pV);
	while(videoI420_getFreeBuffer(v)==0);	// wait till buffer is swaped
	
}


// buffer is 1 or 2
void videoI420_bufferReady(VideoI420_regs_t* v, unsigned int buffer)
{
	buffer = buffer-1;
	buffer &= 1U;	// use only one bit of the bufferID
	
	v->buffer = buffer;
	
}

// 0 -> no is free
// 1 -> first is free
// 2 -> second is free
// 3 -> both are free
int  videoI420_getFreeBuffer(VideoI420_regs_t* v)
{
	int tmp = v->enable;
	
	if((tmp & 1U) == 0)	// enable bit is off
		return 3;
	else
	{
		if(( (tmp>>1) &1) != ((tmp>>2) &1) )
		{
			return 0;
		}
		else
		{
			if((tmp>>1) & 1)
				return 1;
			else
				return 2;
		}
				
			
	}

	
	
}
























