/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
#include <rtos_internal.h>

extern uint18_t main_task_max_call_level;
extern size_t main_task_stack_size;
extern uint18_t main_task_priority;

extern int18_t isr_max_call_level;
extern uint18_t isr_max_stack_usage;

uint18_t rtos_starting = TRUE;

#define MIN_MAIN_TASK_MAX_CALL_LEVEL	6
// main must be able to create the idle task by calling create_task from the subroutine init_rtos

#define IDLE_TASK_MAX_STACK_USAGE	8	// may need more with another optimization
#define IDLE_TASK_MAX_CALL_LEVEL_INT_EN	3
#define IDLE_TASK_MAX_CALL_LEVEL	4
// The idle task has to be able to free deleted tasks.


__attribute__((constructor (400))) void init_rtos()
{
	task_t main_task;
	uint18_t main_stack_size, main_max_call_level;
	uint18_t idle_stack_size, idle_max_call_level;
	
	extern char end, heap_end;
	size_t heap_size = ROUNDED_DOWN_EVEN(&heap_end - &end + 1);
	initialize_heap(&end, heap_size);
	
	main_stack_size = ROUNDED_UP_EVEN(main_task_stack_size);
	main_max_call_level = main_task_max_call_level;
	if (main_max_call_level < MIN_MAIN_TASK_MAX_CALL_LEVEL)
	{
		// TODO: note ?
		main_max_call_level = MIN_MAIN_TASK_MAX_CALL_LEVEL;
	}
	// there is no minimum for the main tasks stack size because another initial stack is currently in use
	main_task = create_task(NULL, NULL, main_task_priority, main_max_call_level, main_stack_size);
	
	if (main_task == NULL)
	{
		// TODO: error
		while(TRUE);
	}
	
	main_task->regs.lower_limit = -1;
	main_task->regs.upper_limit = main_max_call_level + 1;
	init_regfile_allocator(&main_task->regs);
	
	initialize_scheduler(main_task);
	
	rtos_starting = FALSE;
	
	idle_max_call_level = MAX(IDLE_TASK_MAX_CALL_LEVEL, 
		IDLE_TASK_MAX_CALL_LEVEL_INT_EN + 2 + isr_max_call_level);
		// +2 for the interrupt handling
	idle_stack_size = IDLE_TASK_MAX_STACK_USAGE + isr_max_stack_usage;
	if (create_task(idle_task_code, NULL, 0, idle_max_call_level, idle_stack_size) == NULL)
	{
		// TODO: error
		while(TRUE);
	}
	
	__asm__(
		"mov	r0, %0	\n"
		: //output ops
		: "r" (((int*)main_task->stack) + main_stack_size)
		: "r0"
	);
}
