/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
// Die Semaphoroperationen arbeiten nach dem selben Prinzip wie in eCos

#include <rtos_internal.h>


__attribute__((section(".traps"))) void initialize_semaphore_(semaphore_t *sem, uint18_t value)
{
	rtos_list_initialize(&sem->waiting_tasks);
	sem->value = value;
}

__attribute__((section(".traps"))) void semaphore_down_(semaphore_t *sem)
{
	uint18_t status;
	task_t current_task;
	
	current_task = get_current_task();
	
	STATUS_DISABLE_INT(status);
	
	while (sem->value == 0)
	{
		current_task->ready = FALSE;
		scheduler_remove_task(current_task);
		rtos_list_add_last(&sem->waiting_tasks, &current_task->list_item);
		// TODO: use priority order instead?
		schedule_tasks();
	}
	sem->value--;
	
	STATUS_RESTORE(status);
}

__attribute__((section(".traps"))) void semaphore_up_(semaphore_t *sem)
{
	uint18_t status;
	uint18_t old_value;
	task_t woken_up;
	rtos_list_item_t *list_item;
	
	STATUS_DISABLE_INT(status);
	
	old_value = sem->value;
	sem->value = old_value + 1;
	
	if( old_value == 0 )
	{
		list_item = rtos_list_remove_first(&sem->waiting_tasks);
		if( list_item != &sem->waiting_tasks )
		{
			woken_up = rtos_list_item_owner(list_item, struct tcb, list_item);
			woken_up->ready = TRUE;
			scheduler_add_task(woken_up);
			schedule_tasks();
		}
	}
	
	STATUS_RESTORE(status);
}
