/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#include <loader.h>
#include <loader_cmd.h>

//#pragma GCC push_options
//#pragma GCC optimize ("Os")

__attribute__((section(".loader.text"))) static void loader_print(const char *s) {
    while (*s) {
        loader_send_byte(*s++);
    }
}

__attribute__((section(".loader.text"))) int host_request() {
    unsigned int timeout = 0x3FFFF;
    unsigned int timeout2 = 0x10;
    unsigned int i;
    char c;

    /* tell host that there is a bootloader */
    static const char version[] __attribute__((section(".loader.data"))) = "\nSpMC loader v20140908\n";
    loader_print(version);

    /* let's see if someone talks to us */
    while (!loader_has_byte()) {
        if (timeout-- == 0) {
            if (timeout2-- == 0) {
                return -1;
            }
        }
    }

    /* skip 4 bytes to allow uart synchronisation, then expect 12 times '*' */
    for (i=LOADER_SYNQSEQ_LEN; i; i--) {
        c = loader_receive_byte();
        if (c == LOADER_STOP_SYNC) {
            return -1;
        } else if (c != LOADER_SYNC) {
            loader_send_byte(LOADER_ERR);
        } else {
            loader_send_byte(LOADER_OK);
        }
    }

    return 0;
}

__attribute__((section(".loader.text"))) static void receive18(unsigned int *data, unsigned int *cmd) {
    unsigned int b;

    /* receive 24 bit containing 6 bit command an 18 bit data */

    /* lower 18 bit are data */
    b = loader_receive_byte();
    *data = b;
    b = loader_receive_byte();
    *data |= (b << 8);
    b = loader_receive_byte();
    *data |= (b << 16);
    
    /* upper 6 bit are command */
    *cmd = (b >> 2) & 0x3F;
}

__attribute__((section(".loader.text"))) static void send18(unsigned int data, unsigned int cmd) {
    /* send 24 bit containing 6 bit command an 18 bit data */
    loader_send_byte(data);
    loader_send_byte(data >> 8);
    loader_send_byte((data >> 16) | (cmd << 2));
}

__attribute__((section(".loader.text"))) static void transfer_data() {
    volatile unsigned int *mem_addr = (unsigned int*)0x3FFFF;
    unsigned int data;
    unsigned int cmd;
    unsigned int timeout = 0x3FFFF;
    unsigned int timeout2 = 0x20;
    while (1) {
        receive18(&data, &cmd);
        switch (cmd) {
            case LOADER_CMD_NOP:
                continue;
            case LOADER_CMD_SET_ADDR:
                mem_addr = (unsigned int*)data;
                break;
            case LOADER_CMD_WRITE_DATA:
                *mem_addr++ = data;
                break;
            case LOADER_CMD_READ_DATA:
                data = *mem_addr++;
                break;
            default:
                break;
        }
        send18(data, cmd);
        if (cmd == LOADER_CMD_RESET) {
            while (1) {
                if (timeout-- == 0) {
                    if (timeout2-- == 0) {
                        return;
                    }
                }
                asm("NOP");
            }
        }
    }
}

__attribute__((section(".loader.text"))) void __attribute__((constructor(200))) __loader() {
    loader_init ();
    if (host_request() != 0) {
        return;
    }
    send18((int)&SBI_CORE_ID, 0x0);
    transfer_data();
}

//#pragma GCC pop_options
