/**
 * Copyright (c) 2009, Andreas Wiese
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the <ORGANIZATION> nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "err.h"
#include "progname.h"

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	void __attribute__((__format__(printf,1,0)))
vwarn(const char *fmt, va_list args)
{
	int savederr = errno;

	fprintf(stderr, "%s: ", getprogname());

	if (fmt) {
		vfprintf(stderr, fmt, args);
		fprintf(stderr, ": ");
	}

	fprintf(stderr, "%s\n", strerror(savederr));

	return;
}

	void __attribute__((__format__(printf,1,0)))
vwarnx(const char *fmt, va_list args)
{
	fprintf(stderr, "%s: ", getprogname());

	if (fmt)
		vfprintf(stderr, fmt, args);

	fprintf(stderr, "\n");

	return;
}

	void __attribute__((__noreturn__,__format__(printf,2,0)))
verr(int exval, const char *fmt , va_list args)
{
	vwarn(fmt, args);
	exit(exval);
}

	void __attribute__((__noreturn__,__format__(printf,2,0)))
verrx(int exval, const char *fmt, va_list args)
{
	vwarnx(fmt, args);
	exit(exval);
}

	void __attribute__((__format__(printf,1,2)))
warn(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vwarn(fmt, ap);
	va_end(ap);

	return;
}

	void __attribute__((__format__(printf,1,2)))
warnx(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vwarnx(fmt, ap);
	va_end(ap);

	return;
}

	void __attribute__((__noreturn__,__format__(printf,2,3)))
err(int exval, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	verr(exval, fmt, ap);
	va_end(ap);
}

	void __attribute__((__noreturn__,__format__(printf,2,3)))
errx(int exval, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	verrx(exval, fmt, ap);
	va_end(ap);
}
