package de.tu_darmstadt.rs;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.IntStream;
import java.util.stream.Stream;

public class DomHelpers {
    public static Stream<Node> streamNodeList(NodeList l) {
        return IntStream.range(0,l.getLength()).mapToObj(l::item);
    }

    static class SplitElement {
        public final boolean isSplitRegex;
        public final String text;

        SplitElement(boolean isSplitRegex, String text) {
            this.isSplitRegex = isSplitRegex;
            this.text = text;
        }
    }

    public static List<SplitElement> split(String regex, String input) {
        int index = 0;
        ArrayList<SplitElement> result = new ArrayList<>();
        Matcher m = Pattern.compile(regex).matcher(input);

        // Add segments before each match found
        while(m.find()) {
            if (index == 0 && index == m.start() && m.start() == m.end()) {
                // no empty leading substring included for zero-width match
                // at the beginning of the input char sequence.
                continue;
            }
            String match = input.subSequence(index, m.start()).toString();
            result.add(new SplitElement(false, match));
            result.add(new SplitElement(true, input.subSequence(m.start(), m.end()).toString()));
            index = m.end();


        }

        // If no match was found, return this
        if (index == 0)
            return Collections.singletonList(new SplitElement(false, input));

        // Add remaining segment
        if (index<input.length())
            result.add(new SplitElement(false,input.subSequence(index, input.length()).toString()));

        return result;
    }
}
