<?xml version="1.0" encoding="ISO-8859-1"?>
<!--

    _______________________________________________________________________________

     Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
     Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
     (http://www.rs.tu-darmstadt.de) All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
     are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. All advertising materials mentioning features or use of this software
        must display the following acknowledgement: "This product includes
        software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
        its contributors."

     4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
     BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
     FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
     LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    _______________________________________________________________________________

-->
<xsl:stylesheet version="1.0" 
            xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
            xmlns:fo="http://www.w3.org/1999/XSL/Format" 
            xmlns:svg="http://www.w3.org/2000/svg"
            xmlns:man="http://www.rs.tu-darmstadt.de/ns/spartanmc/manual/">

  <xsl:template match="man:instruction">

    <fo:block width="100%" break-before="page" id="{generate-id(.)}">
      
      <xsl:apply-templates />

    </fo:block>

  </xsl:template>   

  <xsl:template match="man:mnemonic">
    <fo:block>
      <fo:table table-layout="fixed" width="100%" font-family="Helvetica" font-weight="bold" font-size="28pt">
        <fo:table-body>
          <fo:table-row>
              <fo:table-cell text-align="left">
                <fo:block><xsl:value-of select="." /></fo:block>
              </fo:table-cell>
              <fo:table-cell text-align="right">
                <fo:block><xsl:value-of select="." /></fo:block>
              </fo:table-cell>
          </fo:table-row>
        </fo:table-body>
      </fo:table>
    </fo:block>
  </xsl:template>

  <xsl:template match="man:unabbreviated">
    <fo:block font-family="Helvetica" font-weight="normal">
      <xsl:value-of select="." />
    </fo:block>
  </xsl:template>

  <xsl:template match="man:parameters">
      <fo:block margin-top="1cm">
        <fo:block font-weight="bold" margin-bottom="5mm">Mnemonic</fo:block>
        <fo:block start-indent="1.2cm" font-family="Courier">
          <fo:table table-layout="fixed" width="100%">
            <fo:table-body>
              <fo:table-row>
                  <fo:table-cell width="3cm" margin="0" padding="0" font-weight="bold">
                  <fo:block>
                    <xsl:value-of select="../man:mnemonic" />
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell margin="0" padding="0">
                  <fo:block>
                    <xsl:for-each select="man:param">
                        <xsl:if test="position() > 1"><fo:inline font-weight="normal"><xsl:text>, </xsl:text></fo:inline></xsl:if>
                        <fo:inline font-weight="normal"><xsl:value-of select="." /></fo:inline>
                    </xsl:for-each>
                  </fo:block>
                </fo:table-cell>
              </fo:table-row>
            </fo:table-body>
          </fo:table>
        </fo:block>
      </fo:block>
  </xsl:template>

  <xsl:template match="man:graphic">
    <fo:block margin-top="1cm">
      <fo:block text-align="center" space-before.optimum="8pt" space-after.optimum="8pt">
        <xsl:element name="fo:external-graphic">
          <xsl:attribute name="src"><xsl:value-of select="@src"/></xsl:attribute>
        </xsl:element>
      </fo:block>
    </fo:block>
  </xsl:template>

  <xsl:template match="man:pseudocode">
    <fo:block font-weight="bold" margin-bottom="5mm" margin-top="1cm">Pseudocode</fo:block>
    <fo:block start-indent="1.2cm">
      <xsl:apply-templates />
    </fo:block>
  </xsl:template>

  <xsl:template match="man:line">
    <fo:block font-family="Courier" font-style="normal" font-weight="normal" text-align="justify" language="en" hyphenate="true" hyphenation-push-character-count="2" hyphenation-remain-character-count="2">
      <xsl:apply-templates />
    </fo:block>
  </xsl:template>

	<xsl:template match="man:delaySlots">
		<fo:block font-weight="bold" margin-bottom="5mm" margin-top="1cm">Delay Slots</fo:block>
		<xsl:apply-templates />
	</xsl:template>

  <xsl:template match="man:description">
    <fo:block font-weight="bold" margin-bottom="5mm" margin-top="1cm">Description</fo:block>
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match="man:comments">
    <fo:block font-weight="bold" margin-bottom="5mm" margin-top="1cm">Comments</fo:block>
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match="man:index">
    <xsl:if test="@down">
      <fo:inline baseline-shift="sub" font-size="6pt" font-family="Courier"><xsl:value-of select="@down" /></fo:inline>
    </xsl:if>
    <xsl:if test="@up">
      <fo:inline baseline-shift="super" font-size="6pt" font-family="Courier"><xsl:value-of select="@up" /></fo:inline>
    </xsl:if>
  </xsl:template>

  <xsl:template match="man:concat">
    <fo:inline font-family="Symbol"><xsl:text> </xsl:text>&#x23;&#x23;<xsl:text> </xsl:text></fo:inline>
  </xsl:template>

  <xsl:template match="man:not">
    <fo:inline font-family="Symbol"><xsl:text> </xsl:text>&#xAC;<xsl:text> </xsl:text></fo:inline>
  </xsl:template>

  <xsl:template match="man:impl">
    <fo:inline font-family="Symbol"><xsl:text> </xsl:text>&#x2190;<xsl:text> </xsl:text></fo:inline>
  </xsl:template>

  <xsl:template match="man:impr">
    <fo:inline font-family="Symbol"><xsl:text> </xsl:text>&#x2192;<xsl:text> </xsl:text></fo:inline>
  </xsl:template>

  <xsl:template match="man:equiv">
    <fo:inline font-family="Symbol"><xsl:text> </xsl:text>&#x2194;<xsl:text> </xsl:text></fo:inline>
  </xsl:template>

  <xsl:template match="man:shl">
    <fo:inline font-family="Courier"><xsl:text> </xsl:text>&#xAB;<xsl:text> </xsl:text></fo:inline>
  </xsl:template>

  <xsl:template match="man:shr">
    <fo:inline font-family="Courier"><xsl:text> </xsl:text>&#xBB;<xsl:text> </xsl:text></fo:inline>
  </xsl:template>
  
  <xsl:template match="man:chl">
    <fo:inline font-family="Courier"><xsl:text> </xsl:text>&#x2039;<xsl:text></xsl:text></fo:inline>
  </xsl:template>
  
  <xsl:template match="man:chr">
    <fo:inline font-family="Courier"><xsl:text></xsl:text>&#x203A;<xsl:text> </xsl:text></fo:inline>
  </xsl:template>

</xsl:stylesheet>
