<?xml version="1.0" encoding="ISO-8859-1"?>
<!--

    _______________________________________________________________________________

     Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
     Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
     (http://www.rs.tu-darmstadt.de) All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
     are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. All advertising materials mentioning features or use of this software
        must display the following acknowledgement: "This product includes
        software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
        its contributors."

     4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
     BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
     FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
     LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    _______________________________________________________________________________

-->
<xsl:stylesheet version="1.0" 
            xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
            xmlns:fo="http://www.w3.org/1999/XSL/Format" 
            xmlns:svg="http://www.w3.org/2000/svg"
            xmlns:man="http://www.rs.tu-darmstadt.de/ns/spartanmc/manual/">

    <xsl:strip-space elements="*" />      

    <xsl:template match="/">

      <xsl:processing-instruction name="cocoon-format">type="text/xslfo"</xsl:processing-instruction>
      
        <fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">
          <fo:layout-master-set>
              <fo:simple-page-master master-name="a4" 
                                     page-height="29.7cm" 
                                     page-width="21.0cm" 
                                     margin-top="10mm" 
                                     margin-bottom="10mm" 
                                     margin-left="30mm" 
                                     margin-right="20mm">
                  <fo:region-body margin-top="20mm" margin-bottom="15mm"/>
                  <fo:region-before extent="20mm"/>
                  <fo:region-after extent="10mm" />
              </fo:simple-page-master>
      
          </fo:layout-master-set>

          <xsl:apply-templates select="man:manual" mode="bookmark" />
      
          <xsl:apply-templates select="man:manual" />
      
      </fo:root>

    </xsl:template>
    
    <xsl:include href="common.xsl" />
    <xsl:include href="bookmark.xsl" />
    <xsl:include href="instruction.xsl" />
    <xsl:include href="lof.xsl" />
    <xsl:include href="lot.xsl" />
    <xsl:include href="toc.xsl" />

    <xsl:template match="man:chapter">

      <fo:page-sequence master-reference="a4" initial-page-number="1" id="{generate-id(.)}">

        <fo:static-content flow-name="xsl-region-before">
          <xsl:call-template name="page_header" />
        </fo:static-content>
      
        <fo:static-content flow-name="xsl-region-after">
          <xsl:call-template name="page_footer">
	    <xsl:with-param name="prefix"><xsl:number count="man:chapter" />-</xsl:with-param>
	  </xsl:call-template>
        </fo:static-content>
      
        <fo:flow flow-name="xsl-region-body">
     
          <fo:block font-family="Helvetica" font-size="22pt" font-weight="bold" space-after="1cm" keep-with-next="always">
	    <fo:marker marker-class-name="title">
              <xsl:value-of select="man:title"/>
	    </fo:marker>
	    <xsl:call-template name="printChapterHeader" />
          </fo:block>
          <xsl:apply-templates />
      
        </fo:flow>
      
      </fo:page-sequence>
    
    </xsl:template>
    
    <xsl:template match="man:manpage">
      <fo:page-sequence master-reference="a4" initial-page-number="1" id="{generate-id(.)}">
        
        <fo:static-content flow-name="xsl-region-before">
          <xsl:call-template name="page_header" />
        </fo:static-content>

        <fo:static-content flow-name="xsl-region-after">
          <xsl:call-template name="page_footer">
	    <xsl:with-param name="prefix">M<xsl:number count="man:manpage" />-</xsl:with-param>
	  </xsl:call-template>
        </fo:static-content>

        <fo:flow flow-name="xsl-region-body">
          <fo:block font-family="Helvetica" font-size="22pt" font-weight="bold" space-after="1cm" keep-with-next="always">
            <fo:marker marker-class-name="title">
              <xsl:value-of select="man:title" />
            </fo:marker>
            <xsl:call-template name="printManpageHeader" />
          </fo:block>
          <xsl:apply-templates />
        </fo:flow>
      </fo:page-sequence>
    </xsl:template>

    <xsl:template match="man:manual">

      <xsl:call-template name="printTitlePage" />   
      <xsl:call-template name="toc" />
      <xsl:call-template name="lof" />
      <xsl:call-template name="lot" />

      <xsl:apply-templates />

    </xsl:template>

    <xsl:template name="printManpageHeader">
      <xsl:text>MANPAGE &#x2013; </xsl:text><xsl:value-of select="normalize-space(man:name)" /><xsl:text>(</xsl:text><xsl:value-of select="normalize-space(man:man-section)" /><xsl:text>)</xsl:text>
    </xsl:template>

    <xsl:template name="printChapterHeader">
        <xsl:number count="man:chapter" /><xsl:text>. </xsl:text><xsl:value-of select="normalize-space(man:title)" />
    </xsl:template>

    <xsl:template name="printSectionHeader">
      <xsl:choose>
        <xsl:when test="parent::man:manpage">
          <xsl:value-of select="normalize-space(man:title)" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:number count="man:chapter" /><xsl:text>.</xsl:text><xsl:number count="man:section" /><xsl:text>. </xsl:text><xsl:value-of select="normalize-space(man:title)" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template name="printSubsectionHeader">
      <xsl:choose>
        <xsl:when test="ancestor::man:manpage">
          <xsl:value-of select="normalize-space(man:title)" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:number count="man:chapter" /><xsl:text>.</xsl:text><xsl:number count="man:section" /><xsl:text>.</xsl:text><xsl:number count="man:subsection" /><xsl:text>. </xsl:text><xsl:value-of select="normalize-space(man:title)" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template name="printLOFEntry">
      <xsl:number count="man:chapter" /><xsl:text>-</xsl:text><xsl:number count="man:figure" level="any" /><xsl:text> </xsl:text><xsl:value-of select="normalize-space(@caption-text)" /><xsl:text>   </xsl:text>
    </xsl:template>

    <xsl:template name="printLOTEntry">
      <xsl:number count="man:chapter" /><xsl:text>-</xsl:text><xsl:number count="man:table" level="any" /><xsl:text> </xsl:text><xsl:value-of select="normalize-space(@caption-text)" /><xsl:text>   </xsl:text>
    </xsl:template>

    <xsl:template name="printChapterTOCEntry">
      <xsl:number count="man:chapter" /><xsl:text>. </xsl:text><xsl:value-of select="normalize-space(man:title)" /><xsl:text>   </xsl:text>
    </xsl:template>
    
    <xsl:template name="printManpageTOCEntry">
      <xsl:text>MANPAGE &#x2013; </xsl:text>
      <xsl:value-of select="normalize-space(man:name)" />
      <xsl:text>(</xsl:text>
      <xsl:value-of select="normalize-space(man:man-section)" />
      <xsl:text>)   </xsl:text>
    </xsl:template>

    <xsl:template name="printSectionTOCEntry">
      <xsl:number count="man:chapter" /><xsl:text>.</xsl:text><xsl:number count="man:section" /><xsl:text>. </xsl:text><xsl:value-of select="normalize-space(man:title)" /><xsl:text>   </xsl:text>
    </xsl:template>

    <xsl:template name="printSubsectionTOCEntry">
      <xsl:number count="man:chapter" /><xsl:text>.</xsl:text><xsl:number count="man:section" /><xsl:text>.</xsl:text><xsl:number count="man:subsection" /><xsl:text>. </xsl:text><xsl:value-of select="normalize-space(man:title)" /><xsl:text>   </xsl:text>
    </xsl:template>

    <xsl:template name="printPageNumberCitation">
      <xsl:choose>
        <xsl:when test="ancestor-or-self::man:manpage">
          M<xsl:number count="man:manpage" />-<fo:page-number-citation ref-id="{generate-id(.)}" />
        </xsl:when>
      <xsl:otherwise>
        <xsl:number count="man:chapter" />-<fo:page-number-citation ref-id="{generate-id(.)}" />
      </xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template name="printFigureNumber">
      Figure <xsl:number count="man:chapter" /><xsl:text>-</xsl:text><xsl:number count="man:figure" level="any" /><xsl:text>: </xsl:text><xsl:value-of select="normalize-space(@caption-text)" />
    </xsl:template>

    <xsl:template name="printTableNumber">
      Table <xsl:number count="man:chapter" /><xsl:text>-</xsl:text><xsl:number count="man:table" level="any" /><xsl:text>: </xsl:text><xsl:value-of select="normalize-space(@caption-text)" />
    </xsl:template>

</xsl:stylesheet>
