SpartanMC Java Tools
====================

This Readme aims to provide an overview of our java utilities and their libraries.

libDevXML
---------
This library manages all descriptions (xml files) of buses, targets, devices, modules.
The entry point for this library is the class `Library` and its `Library.Builder`, the Builder can be configured however it is needed, from looking up '*SPARTANMC_ROOT*' in `Env` or by adding custom source trees for every supported type of description.

On `build()` the builder crawls all source trees, unmarshalls every description using JAXB, and optionally analyzes or validates them further (references to other descriptions, lookup maps etc.)

All descriptions are readonly with exception of lookup-maps to speed up finding specific entries. All classes representing descriptions end in `Description`.

The `Builder` implements a *JCommander* (Command Line Argument Interpreter) Delegate, that allows reusing the same commandline arguments in every application that makes use of it. Arguments are directly mapped to calls on the *Builder* Instance.
> Functionality might be commented out for the moment, because of a reported bug in gradle's software model, that prevents compiling the needed Annotations

libJConfig
----------
This library contains all logic required to open, create, manipulate, store, validate, build a configuration. It depends on *libDevXML*. All classes that represent instances of descriptions end in `Configuration` and references its description.

The entry point for *libJConfig* is `LibJConfigProject`. From here a `IDocumentHandler` can be obtained, that bundles all operations on a specific configuration. Because the modelling of configurations is extensive and strict, basically every class implements an API interface of similar name (`SubsystemConfiguration` vs. `ISubsystemConfiguration`). Only the interfaces are part of the API, other public methods are needed for internal stuff, and are, if possible, not exposed to users of libJConfig.

Utility functions to simplify accesses that require a bunch of calls are collected in `DocumentUtil` (Ex: Collecting all modules in the current document). Java 8 Streams are used extensively here.

One important paradigm used across *libJConfig* is that the configuration posses very few setters. Most changeable configuration is only exposed via handlers (classes end in `Handler`), that enforce validity and event handling.

Examples for this are:
* `INameHandler`, to ensure global uniqueness of instance names
* `IConnectorHandler`, for every pin or port that can be connected
* `IParameterHandler` and subclasses, that allow checking every parameter against its description and transparently resolving references.

libutil
-------
Contains very versatile code, that is used in multiple projects.

liblogging
----------
Legacy library, that is currently not in use. Contains a *log4j 2.x* plugin that allows capturing logging and using it, e.g. in the UI.

JConfig
-------
*JConfig* contains the `main()` method and UI for *libJConfig*.

CJConfig (*Deprecated*)
--------
Fork of *JConfig*, including incompatible fork of *libJConfig*, for command line access to configurations. Targeted to be replaced in its entirety be a LUA base script interface for *libJConfig* and integration in *JConfig*.

libsim + sim (*Deprecated*)
------------
Software simulator for JConfig configurations. Not migrated to current incompatible *libJConfig*. Targeted to be replaced with a much lighter version that offers GDB integration instead of its one UI for the processor state and code. New Modelling of *libJConfig* + LUA script support, could allow much easier prototyping of peripherals and extensions.

scriptInterpreter
-----------------
LUA wrapper for *libJConfig* API. Hooks into `DocumentHandler.setScriptEngine()`. Will eventually be bundled into *JConfig* with REPL terminal available from inside, and as pure commandline tool.