/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.autoPerf;

import de.tu_darmstadt.rs.autoPerf.config.AutoPerfConfiguration;
import de.tu_darmstadt.rs.autoPerf.transformationPasses.ArraySizeProfileInjector;
import de.tu_darmstadt.rs.autoPerf.transformationPasses.DefaultIncludeWriter;
import de.tu_darmstadt.rs.autoPerf.transformationPasses.FirmwareDuplicatorAutoStreams;
import de.tu_darmstadt.rs.autoPerf.transformationPasses.HardwareGenerator;
import de.tu_darmstadt.rs.autoPerf.transformationPasses.PerfCounterCallInjector;
import de.tu_darmstadt.rs.autoPerf.transformationPasses.UartOpener;
import de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.peripheralDetection.PeripheralDetector;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.ArgumentParser;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.EmptyConfigurationGenerator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.output.SoftwareWriteback;
import de.tu_darmstadt.rs.microStreams.MicroStreams;
import de.tu_darmstadt.rs.microStreams.transformationPasses.TransformationPass;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.CetusParser;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.InputCollector;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.LibraryCollector;
import de.tu_darmstadt.rs.microStreams.transformationPasses.output.HardwareBuilder;
import de.tu_darmstadt.rs.microStreams.transformationPasses.output.LicenseHeaderRemover;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * Base class for autoPerf.
 */
public class AutoPerf extends MicroStreams {


	private List<TransformationPass> workers = new LinkedList<>(
			Arrays.asList(
					new HardwareBuilder(),
					new HardwareGenerator(),
					new FirmwareDuplicatorAutoStreams(),
					new InputCollector(),
					new LibraryCollector(),
					new ArgumentParser(),
					new CetusParser(),
					new PerfCounterCallInjector(),
					new SoftwareWriteback(),
					new LicenseHeaderRemover(),
					new ArraySizeProfileInjector(),
					new UartOpener(),
					new EmptyConfigurationGenerator(new AutoPerfConfiguration()),
					new PeripheralDetector(),
					new DefaultIncludeWriter()
			)
	);


	/**
	 * The constructor.
	 * @param args the cmd line arguments
	 */
	public AutoPerf(String[] args) {
		super(args);
	}

	@Override
	public List<TransformationPass> getWorkers() {
		return workers;
	}
}
