/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.autoPerf.transformationPasses;

import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.ModelComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.core.Core;
import de.tu_darmstadt.rs.microStreams.model.software.include.Include;
import de.tu_darmstadt.rs.microStreams.model.software.task.Task;
import de.tu_darmstadt.rs.microStreams.tools.CetusUtils;
import de.tu_darmstadt.rs.microStreams.transformationPasses.AbstractTransformationPass;

import cetus.hir.AnnotationDeclaration;
import cetus.hir.CodeAnnotation;
import cetus.hir.Procedure;
import cetus.hir.TranslationUnit;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

/**
 * adds al needed includes to software
 */
public class DefaultIncludeWriter extends AbstractTransformationPass {

	public DefaultIncludeWriter() {

	}

	@Override
	public List<Class<? extends ModelComponent>> requiredModelComponents() {
		return Arrays.asList(
				Task.class,
				Core.class
		);
	}

	@Override
	public List<Class<? extends ModelComponent>> providedModelComponents() {
		return Arrays.asList(
				Include.class,
				Task.class
		);
	}

	@Override
	public boolean process(Model model) {

		for (Core core : model.getModelComponents(Core.class)) {
			for (Task task : core.getTasks()) {
				Procedure mainProcedure = CetusUtils.getMainProcedure(CetusUtils.getProgram(model));
				TranslationUnit translationUnit = CetusUtils.findParentOfType(mainProcedure, TranslationUnit.class);
				List<Include> existingIncludes = CetusUtils.getAllIncludes(translationUnit, model);
				List<Include> defaultHeaders = new ArrayList<>(Constants.FilesAndFolders.DEFAULT_HEADERS);
				defaultHeaders.removeAll(existingIncludes);
				defaultHeaders.sort(Comparator.comparing(i -> i.getName()));
				for (Include include : defaultHeaders) {
					translationUnit.addDeclarationFirst(new AnnotationDeclaration(new CodeAnnotation(Constants.C.buildSystemInclude(include.getName()))));
				}

			}
		}
		return true;
	}
}
