/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


import org.gradle.internal.impldep.com.google.common.base.Joiner;
import org.gradle.internal.jacoco.JacocoAgentJar;
import org.gradle.testing.jacoco.plugins.JacocoTaskExtension;

import java.io.File;
import java.lang.reflect.Field;
import java.util.Collection;

/**
 * Workaround for JacocoTaskExtension.getAsJvmArg using relative paths
 *
 * Reimplemented method with absolute paths
 */
public class JacocoJvmArgFormatter {


	static class ArgumentAppender {

		private final StringBuilder builder;
		private boolean anyArgs;

		public ArgumentAppender(StringBuilder builder) {
			this.builder = builder;
		}

		public void append(String name, Object value) {
			if (value != null
					&& !((value instanceof Collection) && ((Collection) value).isEmpty())
					&& !((value instanceof String) && ((String) value).isEmpty())
					&& !((value instanceof Integer) && (value == Integer.valueOf(0)))) {
				if (anyArgs) {
					builder.append(',');
				}
				builder.append(name).append('=');
				if (value instanceof Collection) {
					builder.append(Joiner.on(':').join((Collection) value));
				} else if (value instanceof File) {
					builder.append(((File)value).getAbsoluteFile());
				} else {
					builder.append(value);
				}
				anyArgs = true;
			}
		}
	}

	public static String asJvmArg(JacocoTaskExtension jacoco) {
		StringBuilder builder = new StringBuilder();
		ArgumentAppender argument = new ArgumentAppender(builder);
		JacocoAgentJar agent = getAgent(jacoco);
		builder.append("-javaagent:");
		builder.append(agent.getJar().getAbsoluteFile());
		builder.append('=');
		argument.append("destfile", jacoco.getDestinationFile());
		argument.append("append", jacoco.isAppend());
		argument.append("includes", jacoco.getIncludes());
		argument.append("excludes", jacoco.getExcludes());
		argument.append("exclclassloader", jacoco.getExcludeClassLoaders());
		if (agent.supportsInclNoLocationClasses()) {
			argument.append("inclnolocationclasses", jacoco.isIncludeNoLocationClasses());
		}
		argument.append("sessionid", jacoco.getSessionId());
		argument.append("dumponexit", jacoco.isDumpOnExit());
		argument.append("output", jacoco.getOutput().getAsArg());
		argument.append("address", jacoco.getAddress());
		argument.append("port", jacoco.getPort());
		argument.append("classdumpdir", jacoco.getClassDumpDir());

		if (agent.supportsJmx()) {
			argument.append("jmx", jacoco.isJmx());
		}
		return builder.toString();
	}

	private static JacocoAgentJar getAgent(JacocoTaskExtension jacoco) {
		try {
			Field f = JacocoTaskExtension.class.getDeclaredField("agent");
			f.setAccessible(true);
			return (JacocoAgentJar) f.get(jacoco);
		} catch (NoSuchFieldException | IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

}
