/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package cetus.hir;

public abstract class ExpressionVisitorAdapter<T> implements ExpressionVisitor<T> {
	protected abstract T visitDefault(Expression e);

	@Override
	public T visitNewExpression(NewExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitDeleteExpression(DeleteExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitInfExpression(InfExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitBinaryExpression(BinaryExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitThrowExpression(ThrowExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitIntegerLiteral(IntegerLiteral e) {
		return visitDefault(e);
	}

	@Override
	public T visitBooleanLiteral(BooleanLiteral e) {
		return visitDefault(e);
	}

	@Override
	public T visitEscapeLiteral(EscapeLiteral e) {
		return visitDefault(e);
	}

	@Override
	public T visitStringLiteral(StringLiteral e) {
		return visitDefault(e);
	}

	@Override
	public T visitFloatLiteral(FloatLiteral e) {
		return visitDefault(e);
	}

	@Override
	public T visitCharLiteral(CharLiteral e) {
		return visitDefault(e);
	}

	@Override
	public T visitSizeofExpression(SizeofExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitVaArgExpression(VaArgExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitMinMaxExpression(MinMaxExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitUnaryExpression(UnaryExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitRangeExpression(RangeExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitTypecast(Typecast e) {
		return visitDefault(e);
	}

	@Override
	public T visitConditionalExpression(ConditionalExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitOffsetofExpression(OffsetofExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitSomeExpression(SomeExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitCompoundLiteral(CompoundLiteral e) {
		return visitDefault(e);
	}

	@Override
	public T visitStatementExpression(StatementExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitArrayAccess(ArrayAccess e) {
		return visitDefault(e);
	}

	@Override
	public T visitAlignofExpression(AlignofExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitIDExpression(IDExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitCommaExpression(CommaExpression e) {
		return visitDefault(e);
	}

	@Override
	public T visitFunctionCall(FunctionCall e) {
		return visitDefault(e);
	}
}
