package cetus.hir;

import java.io.PrintWriter;
import java.util.List;
import java.util.ArrayList;

/** Represents a C or C++ pointer. */
public class PointerSpecifier extends Specifier {

    /** * */
    public static final PointerSpecifier UNQUALIFIED = new PointerSpecifier();

    /** * const */
    public static final PointerSpecifier CONST =
            new PointerSpecifier(Specifier.CONST);

    /** * volatile */
    public static final PointerSpecifier VOLATILE =
            new PointerSpecifier(Specifier.VOLATILE);

    /** * restrict */
    public static final PointerSpecifier RESTRICT =
            new PointerSpecifier(Specifier.RESTRICT);
    
    /** * atomic */
    public static final PointerSpecifier ATOMIC =
    		new PointerSpecifier(Specifier.ATOMIC);

    /** * const volatile */
    public static final PointerSpecifier CONST_VOLATILE =
            new PointerSpecifier(Specifier.CONST, Specifier.VOLATILE);
    
    /** * const atomic */
    public static final PointerSpecifier CONST_ATOMIC =
            new PointerSpecifier(Specifier.CONST, Specifier.ATOMIC);

    /** * const restrict */
    public static final PointerSpecifier CONST_RESTRICT =
            new PointerSpecifier(Specifier.CONST, Specifier.RESTRICT);

    /** * restrict volatile */
    public static final PointerSpecifier RESTRICT_VOLATILE =
            new PointerSpecifier(Specifier.RESTRICT, Specifier.VOLATILE);
    
    /** * restrict atomic */
    public static final PointerSpecifier RESTRICT_ATOMIC =
            new PointerSpecifier(Specifier.RESTRICT, Specifier.ATOMIC);
    
    /** * volatile atomic */
    public static final PointerSpecifier VOLATILE_ATOMIC =
            new PointerSpecifier(Specifier.VOLATILE, Specifier.ATOMIC);

    /** * const restrict volatile */
    public static final PointerSpecifier CONST_RESTRICT_VOLATILE =
            new PointerSpecifier(Specifier.CONST, Specifier.RESTRICT,
                                 Specifier.VOLATILE);
    /** * const restrict volatile */
    public static final PointerSpecifier CONST_RESTRICT_ATOMIC =
            new PointerSpecifier(Specifier.CONST, Specifier.RESTRICT,
                                 Specifier.ATOMIC);
    
    /** * const restrict volatile */
    public static final PointerSpecifier CONST_ATOMIC_VOLATILE =
            new PointerSpecifier(Specifier.CONST, Specifier.ATOMIC,
                                 Specifier.VOLATILE);
    
    /** * const restrict volatile */
    public static final PointerSpecifier RESTRICT_VOLATILE_ATOMIC =
            new PointerSpecifier(Specifier.RESTRICT, Specifier.VOLATILE,
            					 Specifier.ATOMIC);
    
    /** * const restrict volatile atomic */
    public static final PointerSpecifier CONST_RESTRICT_VOLATILE_ATOMIC =
            new PointerSpecifier(Specifier.CONST, Specifier.RESTRICT,
                                 Specifier.VOLATILE, Specifier.ATOMIC);

    private List<Specifier> qualifiers;

    private PointerSpecifier(Specifier ... specifiers) {
        if (specifiers.length > 0) {
            qualifiers = new ArrayList<Specifier>(specifiers.length);
            for (Specifier specifier : specifiers) {
                qualifiers.add(specifier);
            }
        } else {
            qualifiers = null;
        }
    }

    public void print(PrintWriter o) {
        o.print("*");
        if (qualifiers != null && !qualifiers.isEmpty()) {
            o.print(" ");
            PrintTools.printListWithSpace(qualifiers, o);
        }
    }

}
