package cetus.hir;

import java.io.PrintWriter;
import java.util.HashMap;

/**
* Operators that act on a single expression.
*/
public enum UnaryOperator implements Printable {

	/**
	 * &amp;
	 */
	ADDRESS_OF(0),

	/**
	 * ~
	 */
	BITWISE_COMPLEMENT(1),

	/**
	 * *
	 */
	DEREFERENCE(2),

	/**
	 * !
	 */
	LOGICAL_NEGATION(3),

	/**
	 * -
	 */
	MINUS(4),

	/**
	 * +
	 */
	PLUS(5),

	/**
	 * --
	 */
	POST_DECREMENT(6),

	/**
	 * ++
	 */
	POST_INCREMENT(7),

	/**
	 * --
	 */
	PRE_DECREMENT(8),

	/**
	 * ++
	 */
	PRE_INCREMENT(9);

    private static HashMap<String, UnaryOperator> op_map =
            new HashMap<String, UnaryOperator>(16);

    private static String[] names = {
            "&", "~", "*", "!", "-", "+", "--", "++", "--", "++" };

	static {
		for (UnaryOperator op : values()) {
			op_map.put(names[op.value], op);
		}
	}

    protected int value;

    /**
    * Used internally -- you may not create arbitrary unary operators
    * and may only use the ones provided as static members.
    *
    * @param value The numeric code of the operator.
    */
    private UnaryOperator(int value) {
        this.value = value;
    }

    /** Creates a unary operator from the specified string. */
    public static UnaryOperator fromString(String s) {
        return op_map.get(s);
    }

    /** Checks if the specified operator implies a side effect. */
    public static boolean hasSideEffects(UnaryOperator op) {
        return (op == UnaryOperator.POST_DECREMENT ||
                op == UnaryOperator.POST_INCREMENT ||
                op == UnaryOperator.PRE_DECREMENT ||
                op == UnaryOperator.PRE_INCREMENT);
    }

    /* It is not necessary to override equals or provide cloning, because
       all possible operators are provided as static objects. */

    /** Prints the operator on the specifier print writer. */
    public void print(PrintWriter o) {
        o.print(names[value]);
    }

    /** Returns a string representation of the operator. */
    @Override
    public String toString() {
        return names[value];
    }

    /**
    * Verifies this operator is valid.
    *
    * @throws IllegalStateException if the operator is invalid.
    */
    public void verify() throws IllegalStateException {
        if (value < 0 || value > 9) {
            throw new IllegalStateException();
        }
    }

}
