/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc;

import de.tu_darmstadt.rs.jcommander.JCommanderRunner;
import de.tu_darmstadt.rs.jcommander.validators.DirectoryExistsValidator;
import de.tu_darmstadt.rs.util.DiffDirectory;

import com.beust.jcommander.Parameter;
import difflib.DiffException;

import java.io.IOException;
import java.nio.file.Path;

/**
 * Entry point for DiffDirectory
 */
@SuppressWarnings("checkstyle:SystemOut")
public class DiffDirectoriesApplication implements Runnable {

	public static final int DEFAULT_CONTEXT_SIZE = 3;
	@Parameter(names = {"--reference"}, description = "Reference Directory", validateValueWith = DirectoryExistsValidator.class, required = true)
	private Path reference;
	@Parameter(names = {"--generated"}, description = "Generated Directory", validateValueWith = DirectoryExistsValidator.class, required = true)
	private Path generated;
	@Parameter(names = "--contextSize", description = "Context size for diffs")
	private int contextSize = DEFAULT_CONTEXT_SIZE;

	public DiffDirectoriesApplication() {
	}

	/**
	 * Entry point
	 * @param args arguments
	 * @throws IOException on broken IO
	 */
	public static void main(String[] args) throws IOException {
		JCommanderRunner.create(new DiffDirectoriesApplication()).run(args);
	}

	public void run() {
		try {
			DiffDirectory.verifyFileContents(reference, generated, p -> true, contextSize, true);
		} catch (DiffException e) {
			System.err.println(e.getMessage());
			System.exit(1);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

}
