/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.evaluation.maxFreq;

/**
 * Result of Maximum Frequency evaluation
 */
public class MaxFreqResult {
	public final TimingInfo timing;
	public final double buildTime;
	public final float multiplier;
	public final float divider;

	public MaxFreqResult(TimingInfo timing, double buildTime, float multiplier, float divider) {
		this.timing = timing;
		this.buildTime = buildTime;
		this.multiplier = multiplier;
		this.divider = divider;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		MaxFreqResult that = (MaxFreqResult) o;

		if (Double.compare(that.buildTime, buildTime) != 0) {
			return false;
		}
		if (Float.compare(that.multiplier, multiplier) != 0) {
			return false;
		}
		if (Float.compare(that.divider, divider) != 0) {
			return false;
		}
		return timing != null? timing.equals(that.timing) : that.timing == null;
	}

	@Override
	public int hashCode() {
		int result;
		long temp;
		result = timing != null? timing.hashCode() : 0;
		temp = Double.doubleToLongBits(buildTime);
		result = 31 * result + (int) (temp ^ (temp >>> 32));
		result = 31 * result + (multiplier != +0.0f? Float.floatToIntBits(multiplier) : 0);
		result = 31 * result + (divider != +0.0f? Float.floatToIntBits(divider) : 0);
		return result;
	}

	@Override
	public String toString() {
		return "MaxFreqResult{" +
				"timing=" + timing +
				", buildTime=" + buildTime +
				", multiplier=" + multiplier +
				", divider=" + divider +
				'}';
	}

}
