/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.evaluation.maxFreq;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

/**
 * Created by wenzel on 07.04.17.
 */
public class VivadoTimingResultParser implements TimingResultParser {

	private static final Logger logger = LoggerFactory.getLogger(VivadoTimingResultParser.class);
	private Path reportPath;

	@Override
	public Collection<TimingInfo> loadTimingInfo(Path buildDir, String outputFilePrefix) throws IOException {
		reportPath = buildDir.resolve("timing.txt");
		int summaryLineAgo = -1;


		Float freq = null;
		Integer numErrors = null;

		for (String line : ((Iterable<String>) Files.lines(reportPath)::iterator)) {

			if (line.equals("| Design Timing Summary")) {
				if (summaryLineAgo != -1) {
					throw new RuntimeException("summary line appears twice");
				}
				summaryLineAgo = 0;
			} else if (summaryLineAgo >= 0) {
				summaryLineAgo++;
				//6 lines after Desing Timing Summary
				if (summaryLineAgo == 6) {

					final String[] split = line.split(" +");
					numErrors = Integer.valueOf(split[3]);
					logger.info("Summary line: {}", Arrays.toString(split));
				}
			}

			if (line.startsWith("  clk0Int    {")) {
				final String[] split = line.split(" +");
				freq = (float) (Float.valueOf(split[5]) * FindMaxFreq.MEGA);
				logger.info("freq line: {}", Arrays.toString(split));
			}
		}
		if (summaryLineAgo == -1) {
			throw new RuntimeException("summary line did not appear");
		}
		return Collections.singleton(new TimingInfo("unknown", freq, null, numErrors));
	}

	@Override
	public Path getTimingReportPath() {
		return reportPath;
	}
}
