/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.evaluation.synthesis;

import com.beust.jcommander.IStringConverter;
import com.beust.jcommander.ParameterException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Comparator;
import java.util.ServiceLoader;
import java.util.stream.StreamSupport;

public class HardwareSourceConverter implements IStringConverter<IHardwareUsageSource> {

	private static final Logger logger = LoggerFactory.getLogger(HardwareSourceConverter.class);
	private static ServiceLoader<IHardwareUsageSource> sourceLoader
			= ServiceLoader.load(IHardwareUsageSource.class);

	@Override
	public IHardwareUsageSource convert(String value) {
		checkIfEmpty();


		for (IHardwareUsageSource source : sourceLoader) {
			if (source.getName().equalsIgnoreCase(value)) {
				return source;
			}
		}

		throw new ParameterException("Invalid source name: " + value);
	}

	private static RuntimeException noSourceExceptionSupplier() {
		return new RuntimeException("No HardwareUsageSources available");
	}

	private static void checkIfEmpty() {
		if (!sourceLoader.iterator().hasNext()) {
			throw noSourceExceptionSupplier();
		}
	}

	public static String describeAvailable() {
		checkIfEmpty();
		StringBuilder result = new StringBuilder();
		result.append("Possible Sources:\n");
		for (IHardwareUsageSource source : sourceLoader) {
			result.append(source.getName()).append(": ").append(source.describe()).append("\n");
		}

		return result.toString();

	}

	public static IHardwareUsageSource getDefault() {
		return StreamSupport.stream(sourceLoader.spliterator(), false)
				.sorted(Comparator.comparing(IHardwareUsageSource::getPriority).reversed())
				.findFirst()
				.orElseThrow(HardwareSourceConverter::noSourceExceptionSupplier);
	}
}
