/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.argumentCollection;

import de.tu_darmstadt.rs.jcommander.converter.StringToLowerCaseConverter;

import com.beust.jcommander.Parameter;

import java.util.List;

/**
 * Represents the map args
 */
public class MapArguments implements Arguments {


	@Parameter(names = {"-activityfile"})
	private String activityFile;
	@Parameter(names = {"-bp"})
	private boolean mapSliceLogic;
	@Parameter(names = {"-c"}, converter = StringToLowerCaseConverter.class)
	private String packSlices = "100";
	@Parameter(names = {"-cm"}, converter = StringToLowerCaseConverter.class)
	private String coverMode = "area";
	@Parameter(names = {"-detail"})
	private boolean generateDetailedMAPReport;
	@Parameter(names = {"-equivalent_register_removal"})
	private boolean removeRedundantRegisters = true;
	@Parameter(names = {"-f"}, converter = StringToLowerCaseConverter.class)
	private String executeCommandFile;
	@Parameter(names = {"-global_opt"}, converter = StringToLowerCaseConverter.class)
	private String globalOptimization = "off";
	@Parameter(names = {"-ignore_keep_hierarchy"})
	private boolean ignoreKeepHierrarchyProperties = false;
	@Parameter(names = {"-intstyle"}, converter = StringToLowerCaseConverter.class)
	private String integrationStyle;
	@Parameter(names = {"-ir"}, converter = StringToLowerCaseConverter.class)
	private String doNotUseRLOCsToGenerateRPMs;
	@Parameter(names = {"-filter"}, converter = StringToLowerCaseConverter.class)
	private String filterFile;
	@Parameter(names = {"-lc"}, converter = StringToLowerCaseConverter.class)
	private String lutCombining = "off";
	@Parameter(names = {"-logic_opt"}, converter = StringToLowerCaseConverter.class)
	private String logicOptimization = "off";
	@Parameter(names = {"-mt"}, converter = StringToLowerCaseConverter.class)
	private String multiThreading = "off";
	@Parameter(names = {"-ntd"})
	private boolean nonTimingDriven;
	@Parameter(names = {"-o"})
	private String outputFileName;
	@Parameter(names = {"-ol"}, converter = StringToLowerCaseConverter.class)
	private String overallEffort = "high";
	@Parameter(names = {"-p"}, converter = StringToLowerCaseConverter.class)
	private String partNumber;
	@Parameter(names = {"-power"}, converter = StringToLowerCaseConverter.class)
	private String powerOptimizations = "on";
	@Parameter(names = {"-pr"}, converter = StringToLowerCaseConverter.class)
	private String packRegistersInIO;
	@Parameter(names = {"-register_duplication"})
	private boolean duplicateRegisters;
	@Parameter(names = {"-r"}, converter = StringToLowerCaseConverter.class)
	private String registerOrdering = "4";
	@Parameter(names = {"-smartguide"}, converter = StringToLowerCaseConverter.class)
	private String smartGuide;
	@Parameter(names = {"-t"})
	private String placerCostTable;
	@Parameter(names = {"-timing"})
	private boolean timingDrivenPackingAndPlacement;
	@Parameter(names = {"-u"})
	private boolean doNotRemoveUnusedLogic;
	@Parameter(names = {"-w"})
	private boolean overwriteExistingFiles;
	@Parameter(names = {"-x"})
	private boolean performanceEvaluationMode;
	@Parameter(names = {"-xe"}, converter = StringToLowerCaseConverter.class)
	private String extraEffortLevel;
	@Parameter(names = {"-xt"})
	private boolean extraPlacerCostTable;
	@Parameter()
	private List<String> inputFiles;

	public MapArguments() {

	}

	@Override
	public String getProgramName() {
		return "map";
	}

	public String toStringForHWEval() {
		return getProgramName() + " -> Overall effort: " + overallEffort + " Extra effort level: " + extraEffortLevel + " Cover mode: " + coverMode;
	}

	public String getActivityFile() {
		return activityFile;
	}

	public void setActivityFile(String activityFile) {
		this.activityFile = activityFile;
	}

	public boolean isMapSliceLogic() {
		return mapSliceLogic;
	}

	public void setMapSliceLogic(boolean mapSliceLogic) {
		this.mapSliceLogic = mapSliceLogic;
	}

	public String getPackSlices() {
		return packSlices;
	}

	public void setPackSlices(String packSlices) {
		this.packSlices = packSlices;
	}

	public String getCoverMode() {
		return coverMode;
	}

	public void setCoverMode(String coverMode) {
		this.coverMode = coverMode;
	}

	public boolean isGenerateDetailedMAPReport() {
		return generateDetailedMAPReport;
	}

	public void setGenerateDetailedMAPReport(boolean generateDetailedMAPReport) {
		this.generateDetailedMAPReport = generateDetailedMAPReport;
	}

	public boolean isRemoveRedundantRegisters() {
		return removeRedundantRegisters;
	}

	public void setRemoveRedundantRegisters(boolean removeRedundantRegisters) {
		this.removeRedundantRegisters = removeRedundantRegisters;
	}

	public String getExecuteCommandFile() {
		return executeCommandFile;
	}

	public void setExecuteCommandFile(String executeCommandFile) {
		this.executeCommandFile = executeCommandFile;
	}

	public String getGlobalOptimization() {
		return globalOptimization;
	}

	public void setGlobalOptimization(String globalOptimization) {
		this.globalOptimization = globalOptimization;
	}

	public boolean isIgnoreKeepHierrarchyProperties() {
		return ignoreKeepHierrarchyProperties;
	}

	public void setIgnoreKeepHierrarchyProperties(boolean ignoreKeepHierrarchyProperties) {
		this.ignoreKeepHierrarchyProperties = ignoreKeepHierrarchyProperties;
	}

	public String getIntegrationStyle() {
		return integrationStyle;
	}

	public void setIntegrationStyle(String integrationStyle) {
		this.integrationStyle = integrationStyle;
	}

	public String getDoNotUseRLOCsToGenerateRPMs() {
		return doNotUseRLOCsToGenerateRPMs;
	}

	public void setDoNotUseRLOCsToGenerateRPMs(String doNotUseRLOCsToGenerateRPMs) {
		this.doNotUseRLOCsToGenerateRPMs = doNotUseRLOCsToGenerateRPMs;
	}

	public String getFilterFile() {
		return filterFile;
	}

	public void setFilterFile(String filterFile) {
		this.filterFile = filterFile;
	}

	public String getLutCombining() {
		return lutCombining;
	}

	public void setLutCombining(String lutCombining) {
		this.lutCombining = lutCombining;
	}

	public String getLogicOptimization() {
		return logicOptimization;
	}

	public void setLogicOptimization(String logicOptimization) {
		this.logicOptimization = logicOptimization;
	}

	public String getMultiThreading() {
		return multiThreading;
	}

	public void setMultiThreading(String multiThreading) {
		this.multiThreading = multiThreading;
	}

	public boolean isNonTimingDriven() {
		return nonTimingDriven;
	}

	public void setNonTimingDriven(boolean nonTimingDriven) {
		this.nonTimingDriven = nonTimingDriven;
	}

	public String getOutputFileName() {
		return outputFileName;
	}

	public void setOutputFileName(String outputFileName) {
		this.outputFileName = outputFileName;
	}

	public String getOverallEffort() {
		return overallEffort;
	}

	public void setOverallEffort(String overallEffort) {
		this.overallEffort = overallEffort;
	}

	public String getPartNumber() {
		return partNumber;
	}

	public void setPartNumber(String partNumber) {
		this.partNumber = partNumber;
	}

	public String getPowerOptimizations() {
		return powerOptimizations;
	}

	public void setPowerOptimizations(String powerOptimizations) {
		this.powerOptimizations = powerOptimizations;
	}

	public String getPackRegistersInIO() {
		return packRegistersInIO;
	}

	public void setPackRegistersInIO(String packRegistersInIO) {
		this.packRegistersInIO = packRegistersInIO;
	}

	public boolean isDuplicateRegisters() {
		return duplicateRegisters;
	}

	public void setDuplicateRegisters(boolean duplicateRegisters) {
		this.duplicateRegisters = duplicateRegisters;
	}

	public String getRegisterOrdering() {
		return registerOrdering;
	}

	public void setRegisterOrdering(String registerOrdering) {
		this.registerOrdering = registerOrdering;
	}

	public String getSmartGuide() {
		return smartGuide;
	}

	public void setSmartGuide(String smartGuide) {
		this.smartGuide = smartGuide;
	}

	public String getPlacerCostTable() {
		return placerCostTable;
	}

	public void setPlacerCostTable(String placerCostTable) {
		this.placerCostTable = placerCostTable;
	}

	public boolean isTimingDrivenPackingAndPlacement() {
		return timingDrivenPackingAndPlacement;
	}

	public void setTimingDrivenPackingAndPlacement(boolean timingDrivenPackingAndPlacement) {
		this.timingDrivenPackingAndPlacement = timingDrivenPackingAndPlacement;
	}

	public boolean isDoNotRemoveUnusedLogic() {
		return doNotRemoveUnusedLogic;
	}

	public void setDoNotRemoveUnusedLogic(boolean doNotRemoveUnusedLogic) {
		this.doNotRemoveUnusedLogic = doNotRemoveUnusedLogic;
	}

	public boolean isOverwriteExistingFiles() {
		return overwriteExistingFiles;
	}

	public void setOverwriteExistingFiles(boolean overwriteExistingFiles) {
		this.overwriteExistingFiles = overwriteExistingFiles;
	}

	public boolean isPerformanceEvaluationMode() {
		return performanceEvaluationMode;
	}

	public void setPerformanceEvaluationMode(boolean performanceEvaluationMode) {
		this.performanceEvaluationMode = performanceEvaluationMode;
	}

	public String getExtraEffortLevel() {
		return extraEffortLevel;
	}

	public void setExtraEffortLevel(String extraEffortLevel) {
		this.extraEffortLevel = extraEffortLevel;
	}

	public boolean isExtraPlacerCostTable() {
		return extraPlacerCostTable;
	}

	public void setExtraPlacerCostTable(boolean extraPlacerCostTable) {
		this.extraPlacerCostTable = extraPlacerCostTable;
	}

	public List<String> getInputFiles() {
		return inputFiles;
	}

	public void setInputFiles(List<String> inputFiles) {
		this.inputFiles = inputFiles;
	}

	@Override
	public int hashCode() {
		int result = activityFile != null? activityFile.hashCode() : 0;
		result = 31 * result + (mapSliceLogic? 1 : 0);
		result = 31 * result + (packSlices != null? packSlices.hashCode() : 0);
		result = 31 * result + (coverMode != null? coverMode.hashCode() : 0);
		result = 31 * result + (generateDetailedMAPReport? 1 : 0);
		result = 31 * result + (removeRedundantRegisters? 1 : 0);
		result = 31 * result + (executeCommandFile != null? executeCommandFile.hashCode() : 0);
		result = 31 * result + (globalOptimization != null? globalOptimization.hashCode() : 0);
		result = 31 * result + (ignoreKeepHierrarchyProperties? 1 : 0);
		result = 31 * result + (integrationStyle != null? integrationStyle.hashCode() : 0);
		result = 31 * result + (doNotUseRLOCsToGenerateRPMs != null? doNotUseRLOCsToGenerateRPMs.hashCode() : 0);
		result = 31 * result + (filterFile != null? filterFile.hashCode() : 0);
		result = 31 * result + (lutCombining != null? lutCombining.hashCode() : 0);
		result = 31 * result + (logicOptimization != null? logicOptimization.hashCode() : 0);
		result = 31 * result + (multiThreading != null? multiThreading.hashCode() : 0);
		result = 31 * result + (nonTimingDriven? 1 : 0);
		result = 31 * result + (outputFileName != null? outputFileName.hashCode() : 0);
		result = 31 * result + (overallEffort != null? overallEffort.hashCode() : 0);
		result = 31 * result + (partNumber != null? partNumber.hashCode() : 0);
		result = 31 * result + (powerOptimizations != null? powerOptimizations.hashCode() : 0);
		result = 31 * result + (packRegistersInIO != null? packRegistersInIO.hashCode() : 0);
		result = 31 * result + (duplicateRegisters? 1 : 0);
		result = 31 * result + (registerOrdering != null? registerOrdering.hashCode() : 0);
		result = 31 * result + (smartGuide != null? smartGuide.hashCode() : 0);
		result = 31 * result + (placerCostTable != null? placerCostTable.hashCode() : 0);
		result = 31 * result + (timingDrivenPackingAndPlacement? 1 : 0);
		result = 31 * result + (doNotRemoveUnusedLogic? 1 : 0);
		result = 31 * result + (overwriteExistingFiles? 1 : 0);
		result = 31 * result + (performanceEvaluationMode? 1 : 0);
		result = 31 * result + (extraEffortLevel != null? extraEffortLevel.hashCode() : 0);
		result = 31 * result + (extraPlacerCostTable? 1 : 0);
		result = 31 * result + (inputFiles != null? inputFiles.hashCode() : 0);
		return result;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		MapArguments that = (MapArguments) o;

		if (mapSliceLogic != that.mapSliceLogic) {
			return false;
		}
		if (generateDetailedMAPReport != that.generateDetailedMAPReport) {
			return false;
		}
		if (removeRedundantRegisters != that.removeRedundantRegisters) {
			return false;
		}
		if (ignoreKeepHierrarchyProperties != that.ignoreKeepHierrarchyProperties) {
			return false;
		}
		if (nonTimingDriven != that.nonTimingDriven) {
			return false;
		}
		if (duplicateRegisters != that.duplicateRegisters) {
			return false;
		}
		if (timingDrivenPackingAndPlacement != that.timingDrivenPackingAndPlacement) {
			return false;
		}
		if (doNotRemoveUnusedLogic != that.doNotRemoveUnusedLogic) {
			return false;
		}
		if (overwriteExistingFiles != that.overwriteExistingFiles) {
			return false;
		}
		if (performanceEvaluationMode != that.performanceEvaluationMode) {
			return false;
		}
		if (extraPlacerCostTable != that.extraPlacerCostTable) {
			return false;
		}
		if (activityFile != null? !activityFile.equals(that.activityFile) : that.activityFile != null) {
			return false;
		}
		if (packSlices != null? !packSlices.equals(that.packSlices) : that.packSlices != null) {
			return false;
		}
		if (coverMode != null? !coverMode.equals(that.coverMode) : that.coverMode != null) {
			return false;
		}
		if (executeCommandFile != null? !executeCommandFile.equals(that.executeCommandFile) : that.executeCommandFile != null) {
			return false;
		}
		if (globalOptimization != null? !globalOptimization.equals(that.globalOptimization) : that.globalOptimization != null) {
			return false;
		}
		if (integrationStyle != null? !integrationStyle.equals(that.integrationStyle) : that.integrationStyle != null) {
			return false;
		}
		if (doNotUseRLOCsToGenerateRPMs != null? !doNotUseRLOCsToGenerateRPMs.equals(that.doNotUseRLOCsToGenerateRPMs) : that.doNotUseRLOCsToGenerateRPMs != null) {
			return false;
		}
		if (filterFile != null? !filterFile.equals(that.filterFile) : that.filterFile != null) {
			return false;
		}
		if (lutCombining != null? !lutCombining.equals(that.lutCombining) : that.lutCombining != null) {
			return false;
		}
		if (logicOptimization != null? !logicOptimization.equals(that.logicOptimization) : that.logicOptimization != null) {
			return false;
		}
		if (multiThreading != null? !multiThreading.equals(that.multiThreading) : that.multiThreading != null) {
			return false;
		}
		if (outputFileName != null? !outputFileName.equals(that.outputFileName) : that.outputFileName != null) {
			return false;
		}
		if (overallEffort != null? !overallEffort.equals(that.overallEffort) : that.overallEffort != null) {
			return false;
		}
		if (partNumber != null? !partNumber.equals(that.partNumber) : that.partNumber != null) {
			return false;
		}
		if (powerOptimizations != null? !powerOptimizations.equals(that.powerOptimizations) : that.powerOptimizations != null) {
			return false;
		}
		if (packRegistersInIO != null? !packRegistersInIO.equals(that.packRegistersInIO) : that.packRegistersInIO != null) {
			return false;
		}
		if (registerOrdering != null? !registerOrdering.equals(that.registerOrdering) : that.registerOrdering != null) {
			return false;
		}
		if (smartGuide != null? !smartGuide.equals(that.smartGuide) : that.smartGuide != null) {
			return false;
		}
		if (placerCostTable != null? !placerCostTable.equals(that.placerCostTable) : that.placerCostTable != null) {
			return false;
		}
		if (extraEffortLevel != null? !extraEffortLevel.equals(that.extraEffortLevel) : that.extraEffortLevel != null) {
			return false;
		}
		return inputFiles != null? inputFiles.equals(that.inputFiles) : that.inputFiles == null;
	}

	@Override
	public String toString() {
		return "MapArguments{" +
				"activityFile='" + activityFile + '\'' +
				", mapSliceLogic=" + mapSliceLogic +
				", packSlices='" + packSlices + '\'' +
				", coverMode='" + coverMode + '\'' +
				", generateDetailedMAPReport=" + generateDetailedMAPReport +
				", removeRedundantRegisters=" + removeRedundantRegisters +
				", executeCommandFile='" + executeCommandFile + '\'' +
				", globalOptimization='" + globalOptimization + '\'' +
				", ignoreKeepHierrarchyProperties=" + ignoreKeepHierrarchyProperties +
				", integrationStyle='" + integrationStyle + '\'' +
				", doNotUseRLOCsToGenerateRPMs='" + doNotUseRLOCsToGenerateRPMs + '\'' +
				", filterFile='" + filterFile + '\'' +
				", lutCombining='" + lutCombining + '\'' +
				", logicOptimization='" + logicOptimization + '\'' +
				", multiThreading='" + multiThreading + '\'' +
				", nonTimingDriven=" + nonTimingDriven +
				", outputFileName='" + outputFileName + '\'' +
				", overallEffort='" + overallEffort + '\'' +
				", partNumber='" + partNumber + '\'' +
				", powerOptimizations='" + powerOptimizations + '\'' +
				", packRegistersInIO='" + packRegistersInIO + '\'' +
				", duplicateRegisters=" + duplicateRegisters +
				", registerOrdering='" + registerOrdering + '\'' +
				", smartGuide='" + smartGuide + '\'' +
				", placerCostTable='" + placerCostTable + '\'' +
				", timingDrivenPackingAndPlacement=" + timingDrivenPackingAndPlacement +
				", doNotRemoveUnusedLogic=" + doNotRemoveUnusedLogic +
				", overwriteExistingFiles=" + overwriteExistingFiles +
				", performanceEvaluationMode=" + performanceEvaluationMode +
				", extraEffortLevel='" + extraEffortLevel + '\'' +
				", extraPlacerCostTable=" + extraPlacerCostTable +
				", inputFiles=" + inputFiles +
				'}';
	}

	@Override
	public int compareTo(Object o) {
		return toString().compareTo(o.toString());
	}
}
