/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.argumentCollection;


import de.tu_darmstadt.rs.jcommander.converter.StringToLowerCaseConverter;

import com.beust.jcommander.Parameter;

import java.util.List;


/**
 * Represents the Arguments of xilinx PAR
 */
public class ParArguments implements Arguments {

	@Parameter(names = {"-activityfile"})
	private String activityFile;
	@Parameter(names = {"-clock_regions"})
	private boolean clockRegions;
	@Parameter(names = {"-f"})
	private String executeCommandFile;
	@Parameter(names = {"-intstyle"}, converter = StringToLowerCaseConverter.class)
	private String integrationStyle;
	@Parameter(names = {"-filter"})
	private String filterFile;
	@Parameter(names = {"-k"}, converter = StringToLowerCaseConverter.class)
	private String reEntrantRouting;
	@Parameter(names = {"-mt"}, converter = StringToLowerCaseConverter.class)
	private String multiThreading = "off";
	@Parameter(names = {"-nopad"})
	private boolean noPad = false;
	@Parameter(names = {"-ntd"})
	private boolean nonTimingDriven;
	@Parameter(names = {"-ol"}, converter = StringToLowerCaseConverter.class)
	private String overallEffort = "std";
	@Parameter(names = {"-p"})
	private boolean noPlacement;
	@Parameter(names = {"-pl"}, converter = StringToLowerCaseConverter.class)
	private String placerEffort;
	@Parameter(names = {"-power"}, converter = StringToLowerCaseConverter.class)
	private String powerAwarePAR;
	@Parameter(names = {"-r"})
	private boolean noRouting;
	@Parameter(names = {"-rl"}, converter = StringToLowerCaseConverter.class)
	private String routerEffortLevel;
	@Parameter(names = {"-smartguide"}, converter = StringToLowerCaseConverter.class)
	private String smartGuide;
	@Parameter(names = {"-t"})
	private String placerCostTable;
	@Parameter(names = {"-w"})
	private boolean overwriteExistingFiles;
	@Parameter(names = {"-x"})
	private boolean performanceEvaluationMode;
	@Parameter(names = {"-xe"}, converter = StringToLowerCaseConverter.class)
	private String extraEffortLevel;
	@Parameter()
	private List<String> inputFiles;

	public ParArguments() {

	}

	public String toStringForHWEval() {
		return getProgramName() + " -> Overall effort: " + overallEffort + " Extra effort level: " +extraEffortLevel;
	}

	public String getActivityFile() {
		return activityFile;
	}

	public void setActivityFile(String activityFile) {
		this.activityFile = activityFile;
	}

	public boolean isClockRegions() {
		return clockRegions;
	}

	public void setClockRegions(boolean clockRegions) {
		this.clockRegions = clockRegions;
	}

	public String getExecuteCommandFile() {
		return executeCommandFile;
	}

	public void setExecuteCommandFile(String executeCommandFile) {
		this.executeCommandFile = executeCommandFile;
	}

	public String getIntegrationStyle() {
		return integrationStyle;
	}

	public void setIntegrationStyle(String integrationStyle) {
		this.integrationStyle = integrationStyle;
	}

	public String getFilterFile() {
		return filterFile;
	}

	public void setFilterFile(String filterFile) {
		this.filterFile = filterFile;
	}

	public String getReEntrantRouting() {
		return reEntrantRouting;
	}

	public void setReEntrantRouting(String reEntrantRouting) {
		this.reEntrantRouting = reEntrantRouting;
	}

	public String getMultiThreading() {
		return multiThreading;
	}

	public void setMultiThreading(String multiThreading) {
		this.multiThreading = multiThreading;
	}

	public boolean isNoPad() {
		return noPad;
	}

	public void setNoPad(boolean noPad) {
		this.noPad = noPad;
	}

	public boolean isNonTimingDriven() {
		return nonTimingDriven;
	}

	public void setNonTimingDriven(boolean nonTimingDriven) {
		this.nonTimingDriven = nonTimingDriven;
	}

	public String getOverallEffort() {
		return overallEffort;
	}

	public void setOverallEffort(String overallEffort) {
		this.overallEffort = overallEffort;
	}

	public boolean isNoPlacement() {
		return noPlacement;
	}

	public void setNoPlacement(boolean noPlacement) {
		this.noPlacement = noPlacement;
	}

	public String getPlacerEffort() {
		return placerEffort;
	}

	public void setPlacerEffort(String placerEffort) {
		this.placerEffort = placerEffort;
	}

	public String getPowerAwarePAR() {
		return powerAwarePAR;
	}

	public void setPowerAwarePAR(String powerAwarePAR) {
		this.powerAwarePAR = powerAwarePAR;
	}

	public boolean isNoRouting() {
		return noRouting;
	}

	public void setNoRouting(boolean noRouting) {
		this.noRouting = noRouting;
	}

	public String getRouterEffortLevel() {
		return routerEffortLevel;
	}

	public void setRouterEffortLevel(String routerEffortLevel) {
		this.routerEffortLevel = routerEffortLevel;
	}

	public String isSmartGuide() {
		return smartGuide;
	}

	public String getPlacerCostTable() {
		return placerCostTable;
	}

	public void setPlacerCostTable(String placerCostTable) {
		this.placerCostTable = placerCostTable;
	}

	public boolean getOverwriteExistingFiles() {
		return overwriteExistingFiles;
	}

	public void setOverwriteExistingFiles(boolean overwriteExistingFiles) {
		this.overwriteExistingFiles = overwriteExistingFiles;
	}

	public boolean getPerformanceEvaluationMode() {
		return performanceEvaluationMode;
	}

	public void setPerformanceEvaluationMode(boolean performanceEvaluationMode) {
		this.performanceEvaluationMode = performanceEvaluationMode;
	}

	public String getExtraEffortLevel() {
		return extraEffortLevel;
	}

	public void setExtraEffortLevel(String extraEffortLevel) {
		this.extraEffortLevel = extraEffortLevel;
	}

	public String getProgramName() {
		return "par";
	}

	public void setSmartGuide(String smartGuide) {
		this.smartGuide = smartGuide;
	}

	@Override
	public int hashCode() {
		int result = activityFile != null? activityFile.hashCode() : 0;
		result = 31 * result + (clockRegions? 1 : 0);
		result = 31 * result + (executeCommandFile != null? executeCommandFile.hashCode() : 0);
		result = 31 * result + (integrationStyle != null? integrationStyle.hashCode() : 0);
		result = 31 * result + (filterFile != null? filterFile.hashCode() : 0);
		result = 31 * result + (reEntrantRouting != null? reEntrantRouting.hashCode() : 0);
		result = 31 * result + (multiThreading != null? multiThreading.hashCode() : 0);
		result = 31 * result + (noPad? 1 : 0);
		result = 31 * result + (nonTimingDriven? 1 : 0);
		result = 31 * result + (overallEffort != null? overallEffort.hashCode() : 0);
		result = 31 * result + (noPlacement? 1 : 0);
		result = 31 * result + (placerEffort != null? placerEffort.hashCode() : 0);
		result = 31 * result + (powerAwarePAR != null? powerAwarePAR.hashCode() : 0);
		result = 31 * result + (noRouting? 1 : 0);
		result = 31 * result + (routerEffortLevel != null? routerEffortLevel.hashCode() : 0);
		result = 31 * result + (smartGuide != null? smartGuide.hashCode() : 0);
		result = 31 * result + (placerCostTable != null? placerCostTable.hashCode() : 0);
		result = 31 * result + (overwriteExistingFiles? 1 : 0);
		result = 31 * result + (performanceEvaluationMode? 1 : 0);
		result = 31 * result + (extraEffortLevel != null? extraEffortLevel.hashCode() : 0);
		return result;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		ParArguments that = (ParArguments) o;

		if (clockRegions != that.clockRegions) {
			return false;
		}
		if (noPad != that.noPad) {
			return false;
		}
		if (nonTimingDriven != that.nonTimingDriven) {
			return false;
		}
		if (noPlacement != that.noPlacement) {
			return false;
		}
		if (noRouting != that.noRouting) {
			return false;
		}
		if (overwriteExistingFiles != that.overwriteExistingFiles) {
			return false;
		}
		if (performanceEvaluationMode != that.performanceEvaluationMode) {
			return false;
		}
		if (activityFile != null? !activityFile.equals(that.activityFile) : that.activityFile != null) {
			return false;
		}
		if (executeCommandFile != null? !executeCommandFile.equals(that.executeCommandFile) : that.executeCommandFile != null) {
			return false;
		}
		if (integrationStyle != null? !integrationStyle.equals(that.integrationStyle) : that.integrationStyle != null) {
			return false;
		}
		if (filterFile != null? !filterFile.equals(that.filterFile) : that.filterFile != null) {
			return false;
		}
		if (reEntrantRouting != null? !reEntrantRouting.equals(that.reEntrantRouting) : that.reEntrantRouting != null) {
			return false;
		}
		if (multiThreading != null? !multiThreading.equals(that.multiThreading) : that.multiThreading != null) {
			return false;
		}
		if (overallEffort != null? !overallEffort.equals(that.overallEffort) : that.overallEffort != null) {
			return false;
		}
		if (placerEffort != null? !placerEffort.equals(that.placerEffort) : that.placerEffort != null) {
			return false;
		}
		if (powerAwarePAR != null? !powerAwarePAR.equals(that.powerAwarePAR) : that.powerAwarePAR != null) {
			return false;
		}
		if (routerEffortLevel != null? !routerEffortLevel.equals(that.routerEffortLevel) : that.routerEffortLevel != null) {
			return false;
		}
		if (smartGuide != null? !smartGuide.equals(that.smartGuide) : that.smartGuide != null) {
			return false;
		}
		if (placerCostTable != null? !placerCostTable.equals(that.placerCostTable) : that.placerCostTable != null) {
			return false;
		}
		return extraEffortLevel != null? extraEffortLevel.equals(that.extraEffortLevel) : that.extraEffortLevel == null;
	}

	@Override
	public String toString() {
		return "ParArguments{" +
				"activityFile='" + activityFile + '\'' +
				", clockRegions=" + clockRegions +
				", executeCommandFile='" + executeCommandFile + '\'' +
				", integrationStyle='" + integrationStyle + '\'' +
				", filterFile='" + filterFile + '\'' +
				", reEntrantRouting='" + reEntrantRouting + '\'' +
				", multiThreading='" + multiThreading + '\'' +
				", noPad=" + noPad +
				", nonTimingDriven=" + nonTimingDriven +
				", overallEffort='" + overallEffort + '\'' +
				", noPlacement=" + noPlacement +
				", placerEffort='" + placerEffort + '\'' +
				", powerAwarePAR='" + powerAwarePAR + '\'' +
				", noRouting=" + noRouting +
				", routerEffortLevel='" + routerEffortLevel + '\'' +
				", smartGuide=" + smartGuide +
				", placerCostTable='" + placerCostTable + '\'' +
				", overwriteExistingFiles='" + overwriteExistingFiles + '\'' +
				", performanceEvaluationMode='" + performanceEvaluationMode + '\'' +
				", extraEffortLevel='" + extraEffortLevel + '\'' +
				'}';
	}

	@Override
	public int compareTo(Object o) {
		return toString().compareTo(o.toString());
	}
}
