/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.argumentCollection;

import de.tu_darmstadt.rs.jcommander.converter.StringToLowerCaseConverter;

import com.beust.jcommander.Parameter;

/**
 * The xst args
 */
public class SynthesisArguments implements Arguments {

	@Parameter(names = {"-async_to_sync"}, converter = StringToLowerCaseConverter.class)
	private boolean asynchronousToSynchronous = false;
	@Parameter(names = {"-iobuf"}, converter = StringToLowerCaseConverter.class)
	private String addIOBuffers = "yes";
	@Parameter(names = {"-auto_bram_packing"}, converter = StringToLowerCaseConverter.class)
	private String automaticBramPacking = "no";
	@Parameter(names = {"-bram_utilization_ratio"}, converter = StringToLowerCaseConverter.class)
	private String bramUtilizationRatio = "100";
	@Parameter(names = {"-bus_delimiter"})
	private String busDelimiter = "<>";
	@Parameter(names = {"-vlgcase"})
	private String caseImplementationStyle = "none";
	@Parameter(names = {"-case"}, converter = StringToLowerCaseConverter.class)
	private String casE = "maintain";
	@Parameter(names = {"-sd"})
	private String coresSearchDirectories;
	@Parameter(names = {"-cross_clock_analysis"}, converter = StringToLowerCaseConverter.class)
	private String crossClockAnalysis = "no";
	@Parameter(names = {"-decoder_extract"}, converter = StringToLowerCaseConverter.class)
	private String decoderExtraction = "yes";
	@Parameter(names = {"-dsp_utilization_ratio"}, converter = StringToLowerCaseConverter.class)
	private String dspUtilizationRatio = "100";
	@Parameter(names = {"-equivalent_register_removal"}, converter = StringToLowerCaseConverter.class)
	private String equivalentRegisterRemoval = "yes";
	@Parameter(names = {"-fsm_encoding"}, converter = StringToLowerCaseConverter.class)
	private String fsmEncodingAlgorithm = "auto";
	@Parameter(names = {"-fsm_style"}, converter = StringToLowerCaseConverter.class)
	private String fsmStyle = "LUT";
	@Parameter(names = {"-rtlview"}, converter = StringToLowerCaseConverter.class)
	private String generateRtlSchematic = "yes";
	@Parameter(names = {"-generics"}, converter = StringToLowerCaseConverter.class)
	private String genericsParameters = "";
	@Parameter(names = {"-glob_opt"}, converter = StringToLowerCaseConverter.class)
	private String globalOptimizationGoal = "AllClockNets";
	@Parameter(names = {"-xsthdpini"})
	private String hdlIniFile;
	@Parameter(names = {"-hierarchy_separator"}, converter = StringToLowerCaseConverter.class)
	private String hierarchySeparator = "/";
	@Parameter(names = {"-keep_hierarchy"}, converter = StringToLowerCaseConverter.class)
	private String keepHierarchy = "no";
	@Parameter(names = {"-lso"})
	private String lsoFiles;
	@Parameter(names = {"-shift_extract"}, converter = StringToLowerCaseConverter.class)
	private String shiftExtract = "yes";
	@Parameter(names = {"-lc"}, converter = StringToLowerCaseConverter.class)
	private String lutCombining;
	@Parameter(names = {"-slice_utilization_ratio"}, converter = StringToLowerCaseConverter.class)
	private String sliceUtilizationRatio = "100";
	@Parameter(names = {"-max_fanout"})
	private String maxFanout;
	@Parameter(names = {"-move_first_stage"}, converter = StringToLowerCaseConverter.class)
	private String  moveFirstFlipFlopStage;
	@Parameter(names = {"-move_last_stage"}, converter = StringToLowerCaseConverter.class)
	private String  moveLastFlipFlopStage;
	@Parameter(names = {"-mult_style"}, converter = StringToLowerCaseConverter.class)
	private String multiplierStyle = "auto";
	@Parameter(names = {"-mux_extract"}, converter = StringToLowerCaseConverter.class)
	private String muxExtraction = "yes";
	@Parameter(names = {"-mux-style"}, converter = StringToLowerCaseConverter.class)
	private String  muxStyle = "auto";
	@Parameter(names = {"-netlist_hierarchy"}, converter = StringToLowerCaseConverter.class)
	private String netlistHierarchy = "as optimized";
	//CHECKSTYLE.OFF: MagicNumber
	@Parameter(names = {"-bufg"})
	private int numberOfClockBuffers = 32;
	@Parameter(names = {"-bufr"})
	private int  numberOfRegionalClockBuffers = 16;
	//CHECKSTYLE.ON: MagicNumber
	@Parameter(names = {"-opt_level"}, converter = StringToLowerCaseConverter.class)
	private String  optimizationEffort = "normal";
	@Parameter(names = {"-opt_mode"}, converter = StringToLowerCaseConverter.class)
	private String optimizationMode = "speed";
	@Parameter(names = {"-optimize_primitives"}, converter = StringToLowerCaseConverter.class)
	private String optimimizePrimitives = "no";
	@Parameter(names = {"-iob"}, converter = StringToLowerCaseConverter.class)
	private String packIORegistersIntoIobs = "auto";
	@Parameter(names = {"-power"}, converter = StringToLowerCaseConverter.class)
	private String powerReduction = "no";
	@Parameter(names = {"-priority_extract"}, converter = StringToLowerCaseConverter.class)
	private String priorityEncoderExtraction = "yes";
	@Parameter(names = {"-ram_extract"}, converter = StringToLowerCaseConverter.class)
	private String ramExtraction = "yes";
	@Parameter(names = {"-ram_style"}, converter = StringToLowerCaseConverter.class)
	private String ramStyle = "auto";
	@Parameter(names = {"-read_cores"}, converter = StringToLowerCaseConverter.class)
	private String readCores = "yes";
	@Parameter(names = {"-reduce_control_sets"}, converter = StringToLowerCaseConverter.class)
	private String reduceControlSets;
	@Parameter(names = {"-register_balancing"}, converter = StringToLowerCaseConverter.class)
	private String registerBalancing = "no";
	@Parameter(names = {"-register_duplication"}, converter = StringToLowerCaseConverter.class)
	private String registerDuplication = "yes";
	@Parameter(names = {"-resource_sharing"}, converter = StringToLowerCaseConverter.class)
	private String resourceSharing = "yes";
	@Parameter(names = {"-rom_extract"}, converter = StringToLowerCaseConverter.class)
	private String romExtraction = "yes";
	@Parameter(names = {"-rom_style"}, converter = StringToLowerCaseConverter.class)
	private String romStyle = "auto";
	@Parameter(names = {"-safe_implementation"}, converter = StringToLowerCaseConverter.class)
	private String safeImplementation = "no";
	@Parameter(names = {"-shreg_extract"}, converter = StringToLowerCaseConverter.class)
	private String shiftRegisterExtraction = "yes";
	@Parameter(names = {"-slice_packing"}, converter = StringToLowerCaseConverter.class)
	private String slicePacking = "yes";
	@Parameter(names = {"-uc"})
	private String synthesisConstraintFile;
	@Parameter(names = {"-use_clock_enable"}, converter = StringToLowerCaseConverter.class)
	private String useClockEnable;
	@Parameter(names = {"-use_dsp48"}, converter = StringToLowerCaseConverter.class)
	private String useDspBlock = "auto";
	@Parameter(names = {"-use_sync_reset"}, converter = StringToLowerCaseConverter.class)
	private String useSynchronousReset;
	@Parameter(names = {"-use_sync_set"}, converter = StringToLowerCaseConverter.class)
	private String useSynchronousSet;
	@Parameter(names = {"-iuc"}, converter = StringToLowerCaseConverter.class)
	private String useSynthesisConstraintFile = "yes";
	@Parameter(names = {"-verilog2001"}, converter = StringToLowerCaseConverter.class)
	private boolean  verilog2001 = true;
	@Parameter(names = {"-vlgincdir"})
	private String verilogIncludeDir;
	@Parameter(names = {"-xsthdpdir"})
	private String workingDirectory = "./xst";
	@Parameter(names = {"-write_timing_constraints"}, converter = StringToLowerCaseConverter.class)
	private String writeTimingConstraints = "no";
	@Parameter(names = {"-xor_collapse"}, converter = StringToLowerCaseConverter.class)
	private boolean xorCollapsing = true;


	public SynthesisArguments() {

	}

	@Override
	public String getProgramName() {
		return "xst";
	}

	public String toStringForHWEval() {
		return getProgramName() + " -> Optimization mode: " + optimizationMode + " Keep hierarchy: " + keepHierarchy;
	}

	@Override
	public int compareTo(Object o) {
		return toString().compareTo(o.toString());
	}


	@Override
	public String toString() {
		return "SynthesisArguments{" +
				", asynchronousToSynchronous=" + asynchronousToSynchronous +
				", addIOBuffers='" + addIOBuffers + '\'' +
				", automaticBramPacking='" + automaticBramPacking + '\'' +
				", bramUtilizationRatio='" + bramUtilizationRatio + '\'' +
				", busDelimiter='" + busDelimiter + '\'' +
				", caseImplementationStyle='" + caseImplementationStyle + '\'' +
				", casE='" + casE + '\'' +
				", coresSearchDirectories='" + coresSearchDirectories + '\'' +
				", crossClockAnalysis='" + crossClockAnalysis + '\'' +
				", decoderExtraction='" + decoderExtraction + '\'' +
				", dspUtilizationRatio='" + dspUtilizationRatio + '\'' +
				", equivalentRegisterRemoval='" + equivalentRegisterRemoval + '\'' +
				", fsmEncodingAlgorithm='" + fsmEncodingAlgorithm + '\'' +
				", fsmStyle='" + fsmStyle + '\'' +
				", generateRtlSchematic='" + generateRtlSchematic + '\'' +
				", genericsParameters='" + genericsParameters + '\'' +
				", globalOptimizationGoal='" + globalOptimizationGoal + '\'' +
				", hdlIniFile='" + hdlIniFile + '\'' +
				", hierarchySeparator='" + hierarchySeparator + '\'' +
				", keepHierarchy='" + keepHierarchy + '\'' +
				", lsoFiles='" + lsoFiles + '\'' +
				", shiftExtract='" + shiftExtract + '\'' +
				", lutCombining='" + lutCombining + '\'' +
				", sliceUtilizationRatio='" + sliceUtilizationRatio + '\'' +
				", maxFanout='" + maxFanout + '\'' +
				", moveFirstFlipFlopStage='" + moveFirstFlipFlopStage + '\'' +
				", moveLastFlipFlopStage='" + moveLastFlipFlopStage + '\'' +
				", multiplierStyle='" + multiplierStyle + '\'' +
				", muxExtraction='" + muxExtraction + '\'' +
				", muxStyle='" + muxStyle + '\'' +
				", netlistHierarchy='" + netlistHierarchy + '\'' +
				", numberOfClockBuffers=" + numberOfClockBuffers +
				", numberOfRegionalClockBuffers=" + numberOfRegionalClockBuffers +
				", optimizationEffort='" + optimizationEffort + '\'' +
				", optimizationMode='" + optimizationMode + '\'' +
				", optimimizePrimitives='" + optimimizePrimitives + '\'' +
				", packIORegistersIntoIobs='" + packIORegistersIntoIobs + '\'' +
				", powerReduction='" + powerReduction + '\'' +
				", priorityEncoderExtraction='" + priorityEncoderExtraction + '\'' +
				", ramExtraction='" + ramExtraction + '\'' +
				", ramStyle='" + ramStyle + '\'' +
				", readCores='" + readCores + '\'' +
				", reduceControlSets='" + reduceControlSets + '\'' +
				", registerBalancing='" + registerBalancing + '\'' +
				", registerDuplication='" + registerDuplication + '\'' +
				", resourceSharing='" + resourceSharing + '\'' +
				", romExtraction='" + romExtraction + '\'' +
				", romStyle='" + romStyle + '\'' +
				", safeImplementation='" + safeImplementation + '\'' +
				", shiftRegisterExtraction='" + shiftRegisterExtraction + '\'' +
				", slicePacking='" + slicePacking + '\'' +
				", synthesisConstraintFile='" + synthesisConstraintFile + '\'' +
				", useClockEnable='" + useClockEnable + '\'' +
				", useDspBlock='" + useDspBlock + '\'' +
				", useSynchronousReset='" + useSynchronousReset + '\'' +
				", useSynchronousSet='" + useSynchronousSet + '\'' +
				", useSynthesisConstraintFile='" + useSynthesisConstraintFile + '\'' +
				", verilog2001=" + verilog2001 +
				", verilogIncludeDir='" + verilogIncludeDir + '\'' +
				", workingDirectory='" + workingDirectory + '\'' +
				", writeTimingConstraints='" + writeTimingConstraints + '\'' +
				", xorCollapsing=" + xorCollapsing +
				'}';
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		SynthesisArguments that = (SynthesisArguments) o;

		if (asynchronousToSynchronous != that.asynchronousToSynchronous) {
			return false;
		}
		if (numberOfClockBuffers != that.numberOfClockBuffers) {
			return false;
		}
		if (numberOfRegionalClockBuffers != that.numberOfRegionalClockBuffers) {
			return false;
		}
		if (verilog2001 != that.verilog2001) {
			return false;
		}
		if (xorCollapsing != that.xorCollapsing) {
			return false;
		}
		if (addIOBuffers != null? !addIOBuffers.equals(that.addIOBuffers) : that.addIOBuffers != null) {
			return false;
		}
		if (automaticBramPacking != null? !automaticBramPacking.equals(that.automaticBramPacking) : that.automaticBramPacking != null) {
			return false;
		}
		if (bramUtilizationRatio != null? !bramUtilizationRatio.equals(that.bramUtilizationRatio) : that.bramUtilizationRatio != null) {
			return false;
		}
		if (busDelimiter != null? !busDelimiter.equals(that.busDelimiter) : that.busDelimiter != null) {
			return false;
		}
		if (caseImplementationStyle != null? !caseImplementationStyle.equals(that.caseImplementationStyle) : that.caseImplementationStyle != null) {
			return false;
		}
		if (casE != null? !casE.equals(that.casE) : that.casE != null) {
			return false;
		}
		if (coresSearchDirectories != null? !coresSearchDirectories.equals(that.coresSearchDirectories) : that.coresSearchDirectories != null) {
			return false;
		}
		if (crossClockAnalysis != null? !crossClockAnalysis.equals(that.crossClockAnalysis) : that.crossClockAnalysis != null) {
			return false;
		}
		if (decoderExtraction != null? !decoderExtraction.equals(that.decoderExtraction) : that.decoderExtraction != null) {
			return false;
		}
		if (dspUtilizationRatio != null? !dspUtilizationRatio.equals(that.dspUtilizationRatio) : that.dspUtilizationRatio != null) {
			return false;
		}
		if (equivalentRegisterRemoval != null? !equivalentRegisterRemoval.equals(that.equivalentRegisterRemoval) : that.equivalentRegisterRemoval != null) {
			return false;
		}
		if (fsmEncodingAlgorithm != null? !fsmEncodingAlgorithm.equals(that.fsmEncodingAlgorithm) : that.fsmEncodingAlgorithm != null) {
			return false;
		}
		if (fsmStyle != null? !fsmStyle.equals(that.fsmStyle) : that.fsmStyle != null) {
			return false;
		}
		if (generateRtlSchematic != null? !generateRtlSchematic.equals(that.generateRtlSchematic) : that.generateRtlSchematic != null) {
			return false;
		}
		if (genericsParameters != null? !genericsParameters.equals(that.genericsParameters) : that.genericsParameters != null) {
			return false;
		}
		if (globalOptimizationGoal != null? !globalOptimizationGoal.equals(that.globalOptimizationGoal) : that.globalOptimizationGoal != null) {
			return false;
		}
		if (hdlIniFile != null? !hdlIniFile.equals(that.hdlIniFile) : that.hdlIniFile != null) {
			return false;
		}
		if (hierarchySeparator != null? !hierarchySeparator.equals(that.hierarchySeparator) : that.hierarchySeparator != null) {
			return false;
		}
		if (keepHierarchy != null? !keepHierarchy.equals(that.keepHierarchy) : that.keepHierarchy != null) {
			return false;
		}
		if (lsoFiles != null? !lsoFiles.equals(that.lsoFiles) : that.lsoFiles != null) {
			return false;
		}
		if (shiftExtract != null? !shiftExtract.equals(that.shiftExtract) : that.shiftExtract != null) {
			return false;
		}
		if (lutCombining != null? !lutCombining.equals(that.lutCombining) : that.lutCombining != null) {
			return false;
		}
		if (sliceUtilizationRatio != null? !sliceUtilizationRatio.equals(that.sliceUtilizationRatio) : that.sliceUtilizationRatio != null) {
			return false;
		}
		if (maxFanout != null? !maxFanout.equals(that.maxFanout) : that.maxFanout != null) {
			return false;
		}
		if (moveFirstFlipFlopStage != null? !moveFirstFlipFlopStage.equals(that.moveFirstFlipFlopStage) : that.moveFirstFlipFlopStage != null) {
			return false;
		}
		if (moveLastFlipFlopStage != null? !moveLastFlipFlopStage.equals(that.moveLastFlipFlopStage) : that.moveLastFlipFlopStage != null) {
			return false;
		}
		if (multiplierStyle != null? !multiplierStyle.equals(that.multiplierStyle) : that.multiplierStyle != null) {
			return false;
		}
		if (muxExtraction != null? !muxExtraction.equals(that.muxExtraction) : that.muxExtraction != null) {
			return false;
		}
		if (muxStyle != null? !muxStyle.equals(that.muxStyle) : that.muxStyle != null) {
			return false;
		}
		if (netlistHierarchy != null? !netlistHierarchy.equals(that.netlistHierarchy) : that.netlistHierarchy != null) {
			return false;
		}
		if (optimizationEffort != null? !optimizationEffort.equals(that.optimizationEffort) : that.optimizationEffort != null) {
			return false;
		}
		if (optimizationMode != null? !optimizationMode.equals(that.optimizationMode) : that.optimizationMode != null) {
			return false;
		}
		if (optimimizePrimitives != null? !optimimizePrimitives.equals(that.optimimizePrimitives) : that.optimimizePrimitives != null) {
			return false;
		}
		if (packIORegistersIntoIobs != null? !packIORegistersIntoIobs.equals(that.packIORegistersIntoIobs) : that.packIORegistersIntoIobs != null) {
			return false;
		}
		if (powerReduction != null? !powerReduction.equals(that.powerReduction) : that.powerReduction != null) {
			return false;
		}
		if (priorityEncoderExtraction != null? !priorityEncoderExtraction.equals(that.priorityEncoderExtraction) : that.priorityEncoderExtraction != null) {
			return false;
		}
		if (ramExtraction != null? !ramExtraction.equals(that.ramExtraction) : that.ramExtraction != null) {
			return false;
		}
		if (ramStyle != null? !ramStyle.equals(that.ramStyle) : that.ramStyle != null) {
			return false;
		}
		if (readCores != null? !readCores.equals(that.readCores) : that.readCores != null) {
			return false;
		}
		if (reduceControlSets != null? !reduceControlSets.equals(that.reduceControlSets) : that.reduceControlSets != null) {
			return false;
		}
		if (registerBalancing != null? !registerBalancing.equals(that.registerBalancing) : that.registerBalancing != null) {
			return false;
		}
		if (registerDuplication != null? !registerDuplication.equals(that.registerDuplication) : that.registerDuplication != null) {
			return false;
		}
		if (resourceSharing != null? !resourceSharing.equals(that.resourceSharing) : that.resourceSharing != null) {
			return false;
		}
		if (romExtraction != null? !romExtraction.equals(that.romExtraction) : that.romExtraction != null) {
			return false;
		}
		if (romStyle != null? !romStyle.equals(that.romStyle) : that.romStyle != null) {
			return false;
		}
		if (safeImplementation != null? !safeImplementation.equals(that.safeImplementation) : that.safeImplementation != null) {
			return false;
		}
		if (shiftRegisterExtraction != null? !shiftRegisterExtraction.equals(that.shiftRegisterExtraction) : that.shiftRegisterExtraction != null) {
			return false;
		}
		if (slicePacking != null? !slicePacking.equals(that.slicePacking) : that.slicePacking != null) {
			return false;
		}
		if (synthesisConstraintFile != null? !synthesisConstraintFile.equals(that.synthesisConstraintFile) : that.synthesisConstraintFile != null) {
			return false;
		}
		if (useClockEnable != null? !useClockEnable.equals(that.useClockEnable) : that.useClockEnable != null) {
			return false;
		}
		if (useDspBlock != null? !useDspBlock.equals(that.useDspBlock) : that.useDspBlock != null) {
			return false;
		}
		if (useSynchronousReset != null? !useSynchronousReset.equals(that.useSynchronousReset) : that.useSynchronousReset != null) {
			return false;
		}
		if (useSynchronousSet != null? !useSynchronousSet.equals(that.useSynchronousSet) : that.useSynchronousSet != null) {
			return false;
		}
		if (useSynthesisConstraintFile != null? !useSynthesisConstraintFile.equals(that.useSynthesisConstraintFile) : that.useSynthesisConstraintFile != null) {
			return false;
		}
		if (verilogIncludeDir != null? !verilogIncludeDir.equals(that.verilogIncludeDir) : that.verilogIncludeDir != null) {
			return false;
		}
		if (workingDirectory != null? !workingDirectory.equals(that.workingDirectory) : that.workingDirectory != null) {
			return false;
		}
		return writeTimingConstraints != null? writeTimingConstraints.equals(that.writeTimingConstraints) : that.writeTimingConstraints == null;
	}

	@Override
	public int hashCode() {
		int result = (asynchronousToSynchronous? 1 : 0);
		result = 31 * result + (addIOBuffers != null? addIOBuffers.hashCode() : 0);
		result = 31 * result + (automaticBramPacking != null? automaticBramPacking.hashCode() : 0);
		result = 31 * result + (bramUtilizationRatio != null? bramUtilizationRatio.hashCode() : 0);
		result = 31 * result + (busDelimiter != null? busDelimiter.hashCode() : 0);
		result = 31 * result + (caseImplementationStyle != null? caseImplementationStyle.hashCode() : 0);
		result = 31 * result + (casE != null? casE.hashCode() : 0);
		result = 31 * result + (coresSearchDirectories != null? coresSearchDirectories.hashCode() : 0);
		result = 31 * result + (crossClockAnalysis != null? crossClockAnalysis.hashCode() : 0);
		result = 31 * result + (decoderExtraction != null? decoderExtraction.hashCode() : 0);
		result = 31 * result + (dspUtilizationRatio != null? dspUtilizationRatio.hashCode() : 0);
		result = 31 * result + (equivalentRegisterRemoval != null? equivalentRegisterRemoval.hashCode() : 0);
		result = 31 * result + (fsmEncodingAlgorithm != null? fsmEncodingAlgorithm.hashCode() : 0);
		result = 31 * result + (fsmStyle != null? fsmStyle.hashCode() : 0);
		result = 31 * result + (generateRtlSchematic != null? generateRtlSchematic.hashCode() : 0);
		result = 31 * result + (genericsParameters != null? genericsParameters.hashCode() : 0);
		result = 31 * result + (globalOptimizationGoal != null? globalOptimizationGoal.hashCode() : 0);
		result = 31 * result + (hdlIniFile != null? hdlIniFile.hashCode() : 0);
		result = 31 * result + (hierarchySeparator != null? hierarchySeparator.hashCode() : 0);
		result = 31 * result + (keepHierarchy != null? keepHierarchy.hashCode() : 0);
		result = 31 * result + (lsoFiles != null? lsoFiles.hashCode() : 0);
		result = 31 * result + (shiftExtract != null? shiftExtract.hashCode() : 0);
		result = 31 * result + (lutCombining != null? lutCombining.hashCode() : 0);
		result = 31 * result + (sliceUtilizationRatio != null? sliceUtilizationRatio.hashCode() : 0);
		result = 31 * result + (maxFanout != null? maxFanout.hashCode() : 0);
		result = 31 * result + (moveFirstFlipFlopStage != null? moveFirstFlipFlopStage.hashCode() : 0);
		result = 31 * result + (moveLastFlipFlopStage != null? moveLastFlipFlopStage.hashCode() : 0);
		result = 31 * result + (multiplierStyle != null? multiplierStyle.hashCode() : 0);
		result = 31 * result + (muxExtraction != null? muxExtraction.hashCode() : 0);
		result = 31 * result + (muxStyle != null? muxStyle.hashCode() : 0);
		result = 31 * result + (netlistHierarchy != null? netlistHierarchy.hashCode() : 0);
		result = 31 * result + numberOfClockBuffers;
		result = 31 * result + numberOfRegionalClockBuffers;
		result = 31 * result + (optimizationEffort != null? optimizationEffort.hashCode() : 0);
		result = 31 * result + (optimizationMode != null? optimizationMode.hashCode() : 0);
		result = 31 * result + (optimimizePrimitives != null? optimimizePrimitives.hashCode() : 0);
		result = 31 * result + (packIORegistersIntoIobs != null? packIORegistersIntoIobs.hashCode() : 0);
		result = 31 * result + (powerReduction != null? powerReduction.hashCode() : 0);
		result = 31 * result + (priorityEncoderExtraction != null? priorityEncoderExtraction.hashCode() : 0);
		result = 31 * result + (ramExtraction != null? ramExtraction.hashCode() : 0);
		result = 31 * result + (ramStyle != null? ramStyle.hashCode() : 0);
		result = 31 * result + (readCores != null? readCores.hashCode() : 0);
		result = 31 * result + (reduceControlSets != null? reduceControlSets.hashCode() : 0);
		result = 31 * result + (registerBalancing != null? registerBalancing.hashCode() : 0);
		result = 31 * result + (registerDuplication != null? registerDuplication.hashCode() : 0);
		result = 31 * result + (resourceSharing != null? resourceSharing.hashCode() : 0);
		result = 31 * result + (romExtraction != null? romExtraction.hashCode() : 0);
		result = 31 * result + (romStyle != null? romStyle.hashCode() : 0);
		result = 31 * result + (safeImplementation != null? safeImplementation.hashCode() : 0);
		result = 31 * result + (shiftRegisterExtraction != null? shiftRegisterExtraction.hashCode() : 0);
		result = 31 * result + (slicePacking != null? slicePacking.hashCode() : 0);
		result = 31 * result + (synthesisConstraintFile != null? synthesisConstraintFile.hashCode() : 0);
		result = 31 * result + (useClockEnable != null? useClockEnable.hashCode() : 0);
		result = 31 * result + (useDspBlock != null? useDspBlock.hashCode() : 0);
		result = 31 * result + (useSynchronousReset != null? useSynchronousReset.hashCode() : 0);
		result = 31 * result + (useSynchronousSet != null? useSynchronousSet.hashCode() : 0);
		result = 31 * result + (useSynthesisConstraintFile != null? useSynthesisConstraintFile.hashCode() : 0);
		result = 31 * result + (verilog2001? 1 : 0);
		result = 31 * result + (verilogIncludeDir != null? verilogIncludeDir.hashCode() : 0);
		result = 31 * result + (workingDirectory != null? workingDirectory.hashCode() : 0);
		result = 31 * result + (writeTimingConstraints != null? writeTimingConstraints.hashCode() : 0);
		result = 31 * result + (xorCollapsing? 1 : 0);
		return result;
	}

	public boolean isAsynchronousToSynchronous() {
		return asynchronousToSynchronous;
	}

	public void setAsynchronousToSynchronous(boolean asynchronousToSynchronous) {
		this.asynchronousToSynchronous = asynchronousToSynchronous;
	}

	public String getAddIOBuffers() {
		return addIOBuffers;
	}

	public void setAddIOBuffers(String addIOBuffers) {
		this.addIOBuffers = addIOBuffers;
	}

	public String getAutomaticBramPacking() {
		return automaticBramPacking;
	}

	public void setAutomaticBramPacking(String automaticBramPacking) {
		this.automaticBramPacking = automaticBramPacking;
	}

	public String getBramUtilizationRatio() {
		return bramUtilizationRatio;
	}

	public void setBramUtilizationRatio(String bramUtilizationRatio) {
		this.bramUtilizationRatio = bramUtilizationRatio;
	}

	public String getBusDelimiter() {
		return busDelimiter;
	}

	public void setBusDelimiter(String busDelimiter) {
		this.busDelimiter = busDelimiter;
	}

	public String getCaseImplementationStyle() {
		return caseImplementationStyle;
	}

	public void setCaseImplementationStyle(String caseImplementationStyle) {
		this.caseImplementationStyle = caseImplementationStyle;
	}

	public String getCasE() {
		return casE;
	}

	public void setCasE(String casE) {
		this.casE = casE;
	}

	public String getCoresSearchDirectories() {
		return coresSearchDirectories;
	}

	public void setCoresSearchDirectories(String coresSearchDirectories) {
		this.coresSearchDirectories = coresSearchDirectories;
	}

	public String getCrossClockAnalysis() {
		return crossClockAnalysis;
	}

	public void setCrossClockAnalysis(String crossClockAnalysis) {
		this.crossClockAnalysis = crossClockAnalysis;
	}

	public String getDecoderExtraction() {
		return decoderExtraction;
	}

	public void setDecoderExtraction(String decoderExtraction) {
		this.decoderExtraction = decoderExtraction;
	}

	public String getDspUtilizationRatio() {
		return dspUtilizationRatio;
	}

	public void setDspUtilizationRatio(String dspUtilizationRatio) {
		this.dspUtilizationRatio = dspUtilizationRatio;
	}

	public String getEquivalentRegisterRemoval() {
		return equivalentRegisterRemoval;
	}

	public void setEquivalentRegisterRemoval(String equivalentRegisterRemoval) {
		this.equivalentRegisterRemoval = equivalentRegisterRemoval;
	}

	public String getFsmEncodingAlgorithm() {
		return fsmEncodingAlgorithm;
	}

	public void setFsmEncodingAlgorithm(String fsmEncodingAlgorithm) {
		this.fsmEncodingAlgorithm = fsmEncodingAlgorithm;
	}

	public String getFsmStyle() {
		return fsmStyle;
	}

	public void setFsmStyle(String fsmStyle) {
		this.fsmStyle = fsmStyle;
	}

	public String getGenerateRtlSchematic() {
		return generateRtlSchematic;
	}

	public void setGenerateRtlSchematic(String generateRtlSchematic) {
		this.generateRtlSchematic = generateRtlSchematic;
	}

	public String getGenericsParameters() {
		return genericsParameters;
	}

	public void setGenericsParameters(String genericsParameters) {
		this.genericsParameters = genericsParameters;
	}

	public String getGlobalOptimizationGoal() {
		return globalOptimizationGoal;
	}

	public void setGlobalOptimizationGoal(String globalOptimizationGoal) {
		this.globalOptimizationGoal = globalOptimizationGoal;
	}

	public String getHdlIniFile() {
		return hdlIniFile;
	}

	public void setHdlIniFile(String hdlIniFile) {
		this.hdlIniFile = hdlIniFile;
	}

	public String getHierarchySeparator() {
		return hierarchySeparator;
	}

	public void setHierarchySeparator(String hierarchySeparator) {
		this.hierarchySeparator = hierarchySeparator;
	}

	public String getKeepHierarchy() {
		return keepHierarchy;
	}

	public void setKeepHierarchy(String keepHierarchy) {
		this.keepHierarchy = keepHierarchy;
	}

	public String getLsoFiles() {
		return lsoFiles;
	}

	public void setLsoFiles(String lsoFiles) {
		this.lsoFiles = lsoFiles;
	}

	public String getShiftExtract() {
		return shiftExtract;
	}

	public void setShiftExtract(String shiftExtract) {
		this.shiftExtract = shiftExtract;
	}

	public String getLutCombining() {
		return lutCombining;
	}

	public void setLutCombining(String lutCombining) {
		this.lutCombining = lutCombining;
	}

	public String getSliceUtilizationRatio() {
		return sliceUtilizationRatio;
	}

	public void setSliceUtilizationRatio(String sliceUtilizationRatio) {
		this.sliceUtilizationRatio = sliceUtilizationRatio;
	}

	public String getMaxFanout() {
		return maxFanout;
	}

	public void setMaxFanout(String maxFanout) {
		this.maxFanout = maxFanout;
	}

	public String getMoveFirstFlipFlopStage() {
		return moveFirstFlipFlopStage;
	}

	public void setMoveFirstFlipFlopStage(String moveFirstFlipFlopStage) {
		this.moveFirstFlipFlopStage = moveFirstFlipFlopStage;
	}

	public String getMoveLastFlipFlopStage() {
		return moveLastFlipFlopStage;
	}

	public void setMoveLastFlipFlopStage(String moveLastFlipFlopStage) {
		this.moveLastFlipFlopStage = moveLastFlipFlopStage;
	}

	public String getMultiplierStyle() {
		return multiplierStyle;
	}

	public void setMultiplierStyle(String multiplierStyle) {
		this.multiplierStyle = multiplierStyle;
	}

	public String getMuxExtraction() {
		return muxExtraction;
	}

	public void setMuxExtraction(String muxExtraction) {
		this.muxExtraction = muxExtraction;
	}

	public String getMuxStyle() {
		return muxStyle;
	}

	public void setMuxStyle(String muxStyle) {
		this.muxStyle = muxStyle;
	}

	public String getNetlistHierarchy() {
		return netlistHierarchy;
	}

	public void setNetlistHierarchy(String netlistHierarchy) {
		this.netlistHierarchy = netlistHierarchy;
	}

	public int getNumberOfClockBuffers() {
		return numberOfClockBuffers;
	}

	public void setNumberOfClockBuffers(int numberOfClockBuffers) {
		this.numberOfClockBuffers = numberOfClockBuffers;
	}

	public int getNumberOfRegionalClockBuffers() {
		return numberOfRegionalClockBuffers;
	}

	public void setNumberOfRegionalClockBuffers(int numberOfRegionalClockBuffers) {
		this.numberOfRegionalClockBuffers = numberOfRegionalClockBuffers;
	}

	public String getOptimizationEffort() {
		return optimizationEffort;
	}

	public void setOptimizationEffort(String optimizationEffort) {
		this.optimizationEffort = optimizationEffort;
	}

	public String getOptimizationMode() {
		return optimizationMode;
	}

	public void setOptimizationMode(String optimizationMode) {
		this.optimizationMode = optimizationMode;
	}

	public String getOptimimizePrimitives() {
		return optimimizePrimitives;
	}

	public void setOptimimizePrimitives(String optimimizePrimitives) {
		this.optimimizePrimitives = optimimizePrimitives;
	}

	public String getPackIORegistersIntoIobs() {
		return packIORegistersIntoIobs;
	}

	public void setPackIORegistersIntoIobs(String packIORegistersIntoIobs) {
		this.packIORegistersIntoIobs = packIORegistersIntoIobs;
	}

	public String getPowerReduction() {
		return powerReduction;
	}

	public void setPowerReduction(String powerReduction) {
		this.powerReduction = powerReduction;
	}

	public String getPriorityEncoderExtraction() {
		return priorityEncoderExtraction;
	}

	public void setPriorityEncoderExtraction(String priorityEncoderExtraction) {
		this.priorityEncoderExtraction = priorityEncoderExtraction;
	}

	public String getRamExtraction() {
		return ramExtraction;
	}

	public void setRamExtraction(String ramExtraction) {
		this.ramExtraction = ramExtraction;
	}

	public String getRamStyle() {
		return ramStyle;
	}

	public void setRamStyle(String ramStyle) {
		this.ramStyle = ramStyle;
	}

	public String getReadCores() {
		return readCores;
	}

	public void setReadCores(String readCores) {
		this.readCores = readCores;
	}

	public String getReduceControlSets() {
		return reduceControlSets;
	}

	public void setReduceControlSets(String reduceControlSets) {
		this.reduceControlSets = reduceControlSets;
	}

	public String getRegisterBalancing() {
		return registerBalancing;
	}

	public void setRegisterBalancing(String registerBalancing) {
		this.registerBalancing = registerBalancing;
	}

	public String getRegisterDuplication() {
		return registerDuplication;
	}

	public void setRegisterDuplication(String registerDuplication) {
		this.registerDuplication = registerDuplication;
	}

	public String getResourceSharing() {
		return resourceSharing;
	}

	public void setResourceSharing(String resourceSharing) {
		this.resourceSharing = resourceSharing;
	}

	public String getRomExtraction() {
		return romExtraction;
	}

	public void setRomExtraction(String romExtraction) {
		this.romExtraction = romExtraction;
	}

	public String getRomStyle() {
		return romStyle;
	}

	public void setRomStyle(String romStyle) {
		this.romStyle = romStyle;
	}

	public String getSafeImplementation() {
		return safeImplementation;
	}

	public void setSafeImplementation(String safeImplementation) {
		this.safeImplementation = safeImplementation;
	}

	public String getShiftRegisterExtraction() {
		return shiftRegisterExtraction;
	}

	public void setShiftRegisterExtraction(String shiftRegisterExtraction) {
		this.shiftRegisterExtraction = shiftRegisterExtraction;
	}

	public String getSlicePacking() {
		return slicePacking;
	}

	public void setSlicePacking(String slicePacking) {
		this.slicePacking = slicePacking;
	}

	public String getSynthesisConstraintFile() {
		return synthesisConstraintFile;
	}

	public void setSynthesisConstraintFile(String synthesisConstraintFile) {
		this.synthesisConstraintFile = synthesisConstraintFile;
	}

	public String getUseClockEnable() {
		return useClockEnable;
	}

	public void setUseClockEnable(String useClockEnable) {
		this.useClockEnable = useClockEnable;
	}

	public String getUseDspBlock() {
		return useDspBlock;
	}

	public void setUseDspBlock(String useDspBlock) {
		this.useDspBlock = useDspBlock;
	}

	public String getUseSynchronousReset() {
		return useSynchronousReset;
	}

	public void setUseSynchronousReset(String useSynchronousReset) {
		this.useSynchronousReset = useSynchronousReset;
	}

	public String getUseSynchronousSet() {
		return useSynchronousSet;
	}

	public void setUseSynchronousSet(String useSynchronousSet) {
		this.useSynchronousSet = useSynchronousSet;
	}

	public String getUseSynthesisConstraintFile() {
		return useSynthesisConstraintFile;
	}

	public void setUseSynthesisConstraintFile(String useSynthesisConstraintFile) {
		this.useSynthesisConstraintFile = useSynthesisConstraintFile;
	}

	public boolean isVerilog2001() {
		return verilog2001;
	}

	public void setVerilog2001(boolean verilog2001) {
		this.verilog2001 = verilog2001;
	}

	public String getVerilogIncludeDir() {
		return verilogIncludeDir;
	}

	public void setVerilogIncludeDir(String verilogIncludeDir) {
		this.verilogIncludeDir = verilogIncludeDir;
	}

	public String getWorkingDirectory() {
		return workingDirectory;
	}

	public void setWorkingDirectory(String workingDirectory) {
		this.workingDirectory = workingDirectory;
	}

	public String getWriteTimingConstraints() {
		return writeTimingConstraints;
	}

	public void setWriteTimingConstraints(String writeTimingConstraints) {
		this.writeTimingConstraints = writeTimingConstraints;
	}

	public boolean isXorCollapsing() {
		return xorCollapsing;
	}

	public void setXorCollapsing(boolean xorCollapsing) {
		this.xorCollapsing = xorCollapsing;
	}
}
