/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.reportParsing;

import de.tu_darmstadt.rs.fpga.resources.FpgaFamily;
import de.tu_darmstadt.rs.fpga.resources.FpgaResource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * Parses the xilinx reports for resources
 */
public class HardwareReportParser {

	private static final Logger log = LoggerFactory.getLogger(HardwareReportParser.class);
	private FpgaFamily device;
	private InputStreamReader inputStreamReader;
	private List<FpgaResource> resources;


	/**
	 * the Constructor
	 *
	 * @param inputStreamReader an input stream for a report
	 * @param device the device to be used
	 */
	public HardwareReportParser(InputStreamReader inputStreamReader, FpgaFamily device) {
		this.inputStreamReader = inputStreamReader;
		resources = new ArrayList<>();
		this.device = device;
	}

	/**
	 * starts parsing
	 */
	public void parse() {
		BufferedReader bis = new BufferedReader(inputStreamReader);
		try {
			for (String line = bis.readLine(); line != null; line = bis.readLine()) {

				Optional<FpgaResource> possibleResource = FpgaResourceFactory.produce(line, device);
				if (possibleResource.isPresent()) {
					resources.add(possibleResource.get());
				}

			}
		} catch (IOException e) {
			log.error("Can not read the input stream");
		}
	}

	public String toString() {
		return "" + resources;
	}

	public List<FpgaResource> getResources() {
		return resources;
	}
}
