/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.evaluation.hardwareUsageTests;

import de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.argumentCollection.ArgumentParser;
import de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.argumentCollection.Arguments;
import de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.argumentCollection.MapArguments;
import de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.argumentCollection.ParArguments;
import de.tu_darmstadt.rs.spartanmc.evaluation.synthesis.argumentCollection.SynthesisArguments;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * Tests if the argument parsers work
 */
@RunWith(Parameterized.class)
public class ArgumentTester {

	private static final Logger log = LoggerFactory.getLogger(HardwareReportTester.class);

	private static final Path jConfigFileName = Paths.get("xc7a.jc3");

	@Rule
	public TemporaryFolder project = new TemporaryFolder();

	@Parameterized.Parameter(0)
	public String testName;
	@Parameterized.Parameter(1)
	public List<Arguments> referenceArguments;


	public ArgumentTester() {

	}

	/**
	 * creates the test data
	 * @return the test data
	 * @throws IOException if any file IO goes wrong
	 */
	@Parameterized.Parameters(name = "{index}: {0}")
	public static Collection<Object[]> data() throws IOException {

		List tests = new ArrayList();
		List<Arguments> testReference = new ArrayList<>();

		//PAR tests
		ParArguments parArguments = new ParArguments();
		parArguments.setMultiThreading("4");
		parArguments.setOverallEffort("high");
		parArguments.setExtraEffortLevel("n");

		testReference.add(parArguments);

		//MAP tests
		MapArguments mapArguments = new MapArguments();
		mapArguments.setTimingDrivenPackingAndPlacement(true);
		mapArguments.setPartNumber("xc7a200t-sbg484-1");
		mapArguments.setLogicOptimization("on");
		mapArguments.setPackRegistersInIO("b");
		mapArguments.setDuplicateRegisters(true);
		mapArguments.setExtraEffortLevel("n");
		mapArguments.setOverallEffort("high");
		mapArguments.setMultiThreading("2");

		testReference.add(mapArguments);

		//xst tests
		SynthesisArguments synthesisArguments = new SynthesisArguments();
		synthesisArguments.setOptimizationMode("speed");
		synthesisArguments.setOptimizationEffort("1");
		synthesisArguments.setUseSynthesisConstraintFile("no");
		synthesisArguments.setKeepHierarchy("no");
		synthesisArguments.setGlobalOptimizationGoal("allclocknets");
		synthesisArguments.setGenerateRtlSchematic("yes");
		synthesisArguments.setReadCores("yes");
		synthesisArguments.setWriteTimingConstraints("no");
		synthesisArguments.setCrossClockAnalysis("no");
		synthesisArguments.setHierarchySeparator("/");
		synthesisArguments.setBusDelimiter("[]");
		synthesisArguments.setCasE("maintain");
		synthesisArguments.setSliceUtilizationRatio("100");
		synthesisArguments.setFsmEncodingAlgorithm("auto");
		synthesisArguments.setFsmStyle("lut");
		synthesisArguments.setRamExtraction("yes");
		synthesisArguments.setRamStyle("auto");
		synthesisArguments.setRomExtraction("yes");
		synthesisArguments.setRomStyle("auto");
		synthesisArguments.setShiftRegisterExtraction("yes");
		synthesisArguments.setResourceSharing("yes");
		synthesisArguments.setMultiplierStyle("auto");
		synthesisArguments.setPackIORegistersIntoIobs("auto");
		synthesisArguments.setMaxFanout("500");
		synthesisArguments.setNumberOfClockBuffers(8);


		testReference.add(synthesisArguments);

		tests.add(new Object[] {  "Synthesis argument parser test", testReference });

		return tests;
	}


	@Test
	public void test() throws IOException {
		try {
			final InputStream inputStream = HardwareReportTester.class.getResourceAsStream(Paths.get("/configuration").resolve(jConfigFileName).toString());
			final Path jconfigDocument = project.getRoot().toPath().resolve(jConfigFileName);

			ArgumentParser argumentParser = new ArgumentParser(LibJConfigProjectGenerator.getLibJconfigProjectFrom(inputStream, jconfigDocument));

			List<Arguments> testArguments = argumentParser.getArguments();

			Collections.sort(testArguments);
			Collections.sort(referenceArguments);

			Assert.assertEquals(referenceArguments, testArguments);
		} catch (IOException e) {
			log.error("Can not create the jconfig document file");
			throw e;
		}


	}






}
