/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram.bramLayout;

import de.tu_darmstadt.rs.spartanmc.jconfig.interchange.memory.IMemoryBlock;

import org.jetbrains.annotations.NotNull;

public class LatticeBramBuilderFactory extends BaseBramBuilderFactory<LatticeBramBuilderFactory.LBramConfig> {
	public enum LBramConfig implements IBramConfig {
		LRam18x1024(18, 1024), LRam9x2048(9, 2048);

		private final int width;
		private final int depth;

		private LBramConfig(int width, int depth) {
			this.width = width;
			this.depth = depth;
		}

		@Override
		public int getDataWidth() {
			return width;
		}

		@Override
		public int getDepth() {
			return depth;
		}

		@Override
		public boolean isPadToFullMemory() {
			return false;
		}
	}

	@Override
	public LBramConfig[] getAvailableConfigs() {
		return LBramConfig.values();
	}

	@NotNull
	@Override
	protected IBramBuilder getBramBuilder(IMemoryBlock block, Integer bramColumnIndex, LBramConfig bramConfig) {
		int maxBlockCount = Math.toIntExact(block.getLength() / bramConfig.getDepth());
		return new GenericBramBuilder(block.getBramPattern(), bramColumnIndex, bramConfig, this::getInitDataName, maxBlockCount);
	}

	private String getInitDataName(int idx) {
		return String.format("INITVAL_%02X", idx);
	}
}
