/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram.bramLayout;

import java.math.BigInteger;
import java.util.List;
import java.util.stream.Stream;

/**
 * Always has 256 bit per init line
 * has init & initp lines.
 *
 */
public class XilinxBram extends GenericBram<XilinxBram.XilinxBramLine> {

	private final List<XilinxBramLine> initps;

	public XilinxBram(String instanceName, XilinxBramBuilderFactory.XBramConfig config, List<XilinxBramLine> inits, List<XilinxBramLine> initps) {
		super(instanceName, config, inits);
		this.initps = initps;
	}

	public List<XilinxBramLine> getInits() {
		return super.getLines();
	}

	public List<XilinxBramLine> getInitps() {
		return initps;
	}

	@Override
	public Stream<XilinxBramLine> streamLines() {
		return Stream.concat(initps.stream(), super.streamLines());
	}

	@Override
	public List<XilinxBramLine> getLines() {
		throw new UnsupportedOperationException("use Xilinx dedicated accessors inits / initps or stream");
	}

	public static class XilinxBramLine extends GenericBram.GenericBramLine {

		private final boolean parity;

		public XilinxBramLine(boolean parity, int idx, BigInteger contents) {
			super(idx, (i) -> resolveName(idx, parity), contents);
			this.parity = parity;
		}

		private static String resolveName(int idx, boolean parity) {
			String fmt = parity? "INITP_%02X" : "INIT_%02X";
			return String.format(fmt, idx);
		}

	}
}
