/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram.memory;

import de.tu_darmstadt.rs.spartanmc.initram.memory.elf.ElfReader;
import de.tu_darmstadt.rs.spartanmc.jconfig.interchange.memory.MemFirmware;
import de.tu_darmstadt.rs.spartanmc.jconfig.interchange.memory.MemoryBlock;

import java.nio.file.Path;
import java.util.HashMap;
import java.util.Map;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ILibJConfigProject.BUILD_DIR;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ILibJConfigProject.FIRMWARE_BUILD_DIR;

public class FirmwareCache {

	private final Path projectDir;
	private final Map<MemFirmware, SectionedMemory> cache = new HashMap<>();

	public FirmwareCache(Path projectDir) {
		this.projectDir = projectDir;
	}

	public Path resolveElfFile(MemFirmware firmware) {
		if (firmware == null) {
			throw new IllegalArgumentException("Block has no firmware!");
		}
		String name = firmware.getName();
		if (name == null) {
			throw new IllegalArgumentException("Firmware "+ firmware +" has no valid name!");
		}
		return projectDir.resolve(BUILD_DIR).resolve(FIRMWARE_BUILD_DIR).resolve(firmware.getName()).resolve("main.elf");
	}

	public SectionedMemory get(MemoryBlock block) {
		return cache.computeIfAbsent(block.getFirmware(), this::readFirmware);
	}

	private SectionedMemory readFirmware(MemFirmware firmware) {
		return ElfReader.read(resolveElfFile(firmware));
	}
}
