/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram.memory;

import de.tu_darmstadt.rs.spartanmc.initram.memory.sections.ISection;

import org.jetbrains.annotations.Nullable;

import java.util.Collection;

/**
 * Representation of binaries such as ELF, keeping section information, but usable as continuous memory
 * Addresses are in bytes just as in binary itself
 */
public interface ISectionedMemory {

	long getStart();

	long getLength();

	byte get(long addr);

	/**
	 * More elaborate form of {@link #read(byte[], long)}. Allows partially filling a buffer
	 *
	 * @param buffer arbitrarily sized buffer
	 * @param addr first byte-address of read
	 * @param bufferOffset first byte-address of write in buffer
	 * @param size amount of bytes to read into buffer
	 */
	void read(byte[] buffer, long addr, int bufferOffset, int size);

	/**
	 * Any arbitrary sized read, can cross sections
	 *
	 * @param buffer any size that is still available from the binary
	 * @param addr first byte-address of read
	 */
	void read(byte[] buffer, long addr);

	/**
	 * Only buffers of the size of the smallest possible memory access of the target are allowed.
	 * Reads must not cross section boarders
	 *
	 * @param buffer buffer of word size (e.g. 4 bytes for SpartanMC)
	 * @param addr first byte-address of read
	 */
	void alignedRead(byte[] buffer, long addr);

	Collection<ISection> getSections();

	@Nullable
	ISection getSectionOf(long addr);

	interface ISectionedMemoryBuilder {

		void addSection(ISection section);

		SectionedMemory build();

		void fillUntil(long addr);
	}
}
