/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram.memory.elf;

import de.tu_darmstadt.rs.spartanmc.initram.memory.sections.ISection;

import com.oracle.max.elf.ELFSectionHeaderTable;

public abstract class BaseElfSection implements ISection {

	protected final ELFSectionHeaderTable.Entry sectionHeader;

	public BaseElfSection(ELFSectionHeaderTable.Entry sectionHeader) {
		this.sectionHeader = sectionHeader;
	}

	@Override
	public String getName() {
		return sectionHeader.getName();
	}

	@Override
	public long getStartAddress() {
		return Math.toIntExact(sectionHeader.getAddr());
	}

	@Override
	public int getSize() {
		return Math.toIntExact(sectionHeader.getSize());
	}

	@Override
	public boolean isCode() {
		return isExecutableSection(sectionHeader);
	}

	@Override
	public boolean isWritable() {
		return isWritableSection(sectionHeader);
	}

	@Override
	public boolean exists() {
		return true;
	}

	public static boolean isWritableSection(ELFSectionHeaderTable.Entry entry) {
		return (entry.getFlags() & ELFSectionHeaderTable.SHF_WRITE) != 0;
	}

	public static boolean isExecutableSection(ELFSectionHeaderTable.Entry entry) {
		return (entry.getFlags() & ELFSectionHeaderTable.SHF_EXECINSTR) != 0;
	}

	public static boolean isNonExecutableSection(ELFSectionHeaderTable.Entry entry) {
		return (entry.getFlags() & ELFSectionHeaderTable.SHF_EXECINSTR) == 0;
	}

	public static boolean isAllocatableSection(ELFSectionHeaderTable.Entry entry) {
		return (entry.getFlags() & ELFSectionHeaderTable.SHF_ALLOC) != 0;
	}

	public static boolean isNoBitsSection(ELFSectionHeaderTable.Entry entry) {
		return (entry.getType() == ELFSectionHeaderTable.SHT_NOBITS);
	}
}
