/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.initram.output;

import de.tu_darmstadt.rs.spartanmc.initram.MemorySource;
import de.tu_darmstadt.rs.spartanmc.initram.bramLayout.IBramLine;
import de.tu_darmstadt.rs.spartanmc.initram.memory.IMappedBlock;
import de.tu_darmstadt.rs.spartanmc.initram.memory.extractors.SpartanMcSectionDependentBlockLineExtractor;
import de.tu_darmstadt.rs.spartanmc.initram.memory.memoryViews.SpartanMcMemoryWordStream;

import java.io.IOException;
import java.io.PrintStream;
import java.nio.file.Files;
import java.nio.file.Path;

/**
 * Apparently this generates simple Hex-Dumps for sections that cannot be flashed. The contents of these dumps are most likely uploaded to the target after the target has been flashed
 * Created by carstenh on 13.07.17.
 */
public class HexOutput extends BramSharedTextOutput {

	public HexOutput(Path outputFile) {
		super(outputFile);
	}

	@Override
	protected String getHeader() {
		return "";
	}

	@Override
	protected String getFooter() {
		return "";
	}

	@Override
	protected String formatBramLine(String bramInstanceName, IBramLine line) {
		return line.getName();
	}

	@Override
	public void output(MemorySource memorySource) throws IOException {

		try (PrintStream out = new PrintStream(Files.newOutputStream(super.outputFile))) {

			SpartanMcSectionDependentBlockLineExtractor extractor = new SpartanMcSectionDependentBlockLineExtractor();

			memorySource.streamMappedBlocks()
				.filter(block -> !block.isBitfileInitializable())
				.map(IMappedBlock::getContents)
				.flatMapToInt(memory -> SpartanMcMemoryWordStream.of(memory, extractor))
				.mapToObj(word -> String.format("%08x", word))
				.forEach(out::println);
		}

		// remove trailing zeros //TODO WTF IS THIS, find out or kill !!
//		for (int i = result.size() - 1; i > 0; i--) {
//			// leave the last zero there (could be termination of a string)
//			if (result.get(i).equals(result.get(i - 1)) && result.get(i).equals(String.format("%08x", 0))) {
//				result.remove(i);
//			} else {
//				break;
//			}
//		}
	}
}
