/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram.output.xilinx.bitfile;

import java.util.function.LongSupplier;

/**
 * Created by wenzel on 22.11.16.
 */
public abstract class BitfilePacketDecoder {

	public abstract int getAddressXorMask();

	public abstract int getOffsetDivider();

	public abstract boolean isCrcReset(PacketData packet, long data);

	public abstract BitfileChecksum createCrc();

	public enum FrameType {
		TYPE1,
		TYPE2;

		/**
		 * Get a FrameType from an int
		 * @param frameType the int
		 * @return a matching frametype
		 */
		public static FrameType fromInt(int frameType) {
			if (frameType==1) {
				return TYPE1;
			} else if (frameType==2) {
				return TYPE2;
			} else {
				throw new RuntimeException("Invalid Frame Type: "+frameType);
			}
		}
	}

	public enum OpCode {
		NOP,
		Read,
		Write;

		/**
		 * Get an opcode from a long
		 * @param encoded the long
		 * @return an opcode
		 */
		public static OpCode decode(long encoded) {
			if (encoded==0) {
				return OpCode.NOP;
			} else if (encoded==1) {
				return OpCode.Read;
			} else if (encoded==2) {
				return OpCode.Write;
			} else {
				throw new RuntimeException("Invalid encoding for Operation: "+encoded);
			}
		}
	}


	/**
	 * Get the number of Bytes in a Header Word
	 * @return the number of Bytes in a Header Word
	 */
	protected abstract int getWordSize();

	/**
	 * Analyze a header word. Does not ask the supplier for the packet contents, only the header
	 * @return Analyze a header word. Does not ask the supplier for the packet contents, only the header
	 */
	protected abstract PacketData analyzePacket(LongSupplier wordSupplier);
}
