/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram.output.xilinx.bitfile;

import de.tu_darmstadt.rs.spartanmc.initram.output.xilinx.xdl.XDLParser;

import java.io.IOException;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by wenzel on 14.03.17.
 */
public class ISEBitfileReplace extends BitfileReplace {
	private final LocFromXdl locFromXdl;

	/**
	 * Replace ram in a bitfile
	 *
	 * @param xdlIn input XDL
	 * @param bitIn input Bitfile
	 * @param bitOut output bitfile
	 * @param location input location file
	 */
	public ISEBitfileReplace(Path xdlIn, Path bitIn, Path bitOut, Path location) {
		super(bitIn, bitOut, location);
		locFromXdl = new LocFromXdl(xdlIn);
	}

	@Override
	protected Map<String, String> loadLocationMap() throws IOException {
		Map<String, String> result = new HashMap<>();
		locFromXdl.parse(result, null);
		return result;
	}

	private static class LocFromXdl extends XDLParser<Map<String, String>, Object, Object> {


		public LocFromXdl(Path inFile) {
			super(inFile);
		}

		@Override
		protected String replaceAttribute(String attribute, Object replacements, String bramName) {
			//Required by XDLParser, we don't need it
			return null;
		}

		@Override
		protected void output(Object outputData, String out) {
			//Required by XDLParser, we don't need it
		}

		@Override
		protected Object getUserData(Map<String, String> locationMap, String instName, String tile, String primitiveSite) {
			locationMap.put(instName, primitiveSite);
			return null;
		}

		@Override
		public void parse(Map<String, String> userData, Object outputData) throws IOException {
			super.parse(userData, outputData);
		}
	}
}
