/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram.output.xilinx.bitfile;

/**
 * Spartan 6 CRC sum
 */

//So much bit shifting
@SuppressWarnings("checkstyle:MagicNumber")
public class Spartan6Crc implements BitfileChecksum {
	//private static final int eLen = 22;
	//private int mBits[] = new int[eLen];
	private int mValue;

	public Spartan6Crc() {
	}

	@Override
	public void updateCrc(long content, int register) {

		int inputWord = (int) (content & 0xFFFF) | (register << 16);

		int temp = (mValue<<1) ^ inputWord;

		if ((mValue &0x200000)!=0) {
			temp ^= 0x9081;
		}

		mValue = temp & 0x3FFFFF;



		/*int[] input = new int[eLen];
		int[] next = new int[eLen];
		// initialize the input with the current register address and data word
		for(int i = 21, mask = 0x20; i >= 16; i--, mask >>= 1)
			input[i] = (register & mask)!=0 ? 1 : 0;
		for(int i = 15, mask = 0x8000; i >= 0; i--, mask >>= 1)
			input[i] = (content & mask)!=0 ? 1 : 0;
		// update the CRC
		for(int i = 21; i >= 16; i--) next[i] = mBits[i - 1] ^ input[i];
		next[15] = mBits[14] ^ input[15] ^ mBits[21];
		for(int i = 14; i >= 13; i--) next[i] = mBits[i - 1] ^ input[i];
		next[12] = mBits[11] ^ input[12] ^ mBits[21];
		for(int i = 11; i >= 8; i--) next[i] = mBits[i - 1] ^ input[i];
		next[7] = mBits[6] ^ input[7] ^ mBits[21];
		for(int i = 6; i >= 1; i--) next[i] = mBits[i - 1] ^ input[i];
		next[0] = input[0] ^ mBits[21];
		// copy the updated bits into place

		mValue = 0;
		for(int i = 0, mask = 1; i < eLen; i++, mask <<= 1) {
			mBits[i] = next[i];
			mValue |= mBits[i]!=0 ? mask : 0;
		}
		*/

	}

	@Override
	public void reset() {
		//mBits = new int[eLen];
		mValue = 0;
	}

	@Override
	public int getValue() {
		return mValue;
	}

	@Override
	public int computeCRCRegValue() {
		return mValue;
	}
}
