/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.initram

import de.tu_darmstadt.rs.spartanmc.initram.memory.extractors.SpartancMcExtractor
import de.tu_darmstadt.rs.spartanmc.initram.memory.sections.ISection
import spock.lang.Specification


class SpartanmcBlockLineExtractorSpec extends Specification {

	private final ISection dataSection = Mock() {
		toString() >> "MockDataSection"
		getName() >> "DataSection"
		isCode() >> false
		isWritable() >> true
	}

	private final ISection codeSection = Mock() {
		toString() >> "MockCodeSection"
		getName() >> "CodeSection"
		isCode() >> true
		isWritable() >> false
	}

	def "Extracts 18bit from Code-Line"() {
		setup:
		def extractor = new SpartancMcExtractor()

		expect:
		extractor.extract18BlockLineFromCode(input as byte[], codeSection) == (result as byte[])

		where:
		input						| result
		[0x00, 0x00, 0x00, 0x00]	| [0x00, 0x00, 0x00, 0x00]
		[0x00, 0x03, 0xFF, 0xFF]	| [0x00, 0x03, 0xFF, 0xFF]
		[0x00, 0x01, 0x55, 0xAA]	| [0x00, 0x01, 0x55, 0xAA]
		[0x0F, 0x06, 0xFF, 0xFF]	| [0x00, 0x02, 0xFF, 0xFF]
		[0xFF, 0xFF, 0xFF, 0xFF]	| [0x00, 0x03, 0xFF, 0xFF]
	}

	def "Extracts 2x9bit from Code-Line"() {
		setup:
		def extractor = new SpartancMcExtractor()

		expect:
		extractor.split18BlockLine(input as byte[], codeSection) == (result as byte[][])

		where:
		input						| result
		[0x00, 0x00, 0x00, 0x00]	| [[0x00, 0x00], [0x00, 0x00]]
		[0x00, 0x02, 0x01, 0x00]	| [[0x01, 0x00], [0x01, 0x00]]
		[0x00, 0x00, 0x02, 0x01]	| [[0x00, 0x01], [0x00, 0x01]]
		[0x00, 0x03, 0xFF, 0xFF]	| [[0x01, 0xFF], [0x01, 0xFF]]
		[0x00, 0x01, 0x55, 0xAA]	| [[0x00, 0xAA], [0x01, 0xAA]]
		[0x0F, 0x06, 0xFF, 0xFF]	| [[0x01, 0x7F], [0x01, 0xFF]]
		[0xFF, 0xFF, 0xFF, 0xFF]	| [[0x01, 0xFF], [0x01, 0xFF]]
	}

	def "Extracts 18bit from Data-Line"() {
		setup:
		def extractor = new SpartancMcExtractor()

		expect:
		extractor.extract9x2BlockLineFromData(input as byte[], dataSection) == (result as byte[])

		where:
		input						| result
		[0x00, 0x00, 0x00, 0x00]	| [0x00, 0x00, 0x00]
		[0x01, 0xFF, 0x01, 0xFF]	| [0x03, 0xFF, 0xFF]
		[0x01, 0x5A, 0x01, 0xA5]	| [0x02, 0xB5, 0xA5]
		[0x0B, 0xC3, 0xF0, 0xD4]	| [0x03, 0x86, 0xD4]
		[0xFF, 0xFF, 0xFF, 0xFF]	| [0x03, 0xFF, 0xFF]
	}

	def "Extracts 2x9bit from Data-Line"() {
		setup:
		def extractor = new SpartancMcExtractor()

		expect:
		extractor.split9x2BlockLines(input as byte[], dataSection) == (result as byte[][])

		where:
		input						| result
		[0x00, 0x00, 0x00, 0x00]	| [[0x00, 0x00], [0x00, 0x00]]
		[0x01, 0xFF, 0x01, 0xFF]	| [[0x01, 0xFF], [0x01, 0xFF]]
		[0x01, 0x5A, 0x00, 0xA5]	| [[0x01, 0x5A], [0x00, 0xA5]]
		[0x0B, 0xC3, 0xF0, 0xD4]	| [[0x01, 0xC3], [0x00, 0xD4]]
		[0xFF, 0xFF, 0xFF, 0xFF]	| [[0x01, 0xFF], [0x01, 0xFF]]
	}


}