/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


package de.tu_darmstadt.rs.spartanmc.jconfig.ui;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import javax.swing.Icon;
import javax.swing.UIManager;

public final class LookAndFeel {

	private static final Logger logger = LoggerFactory.getLogger(LookAndFeel.class);
	private static String lookAndFeelClassName;
	private static boolean isGtk;
	static {
		lookAndFeelClassName = UIManager.getSystemLookAndFeelClassName();

		//If the system LAF is the cross platform LAF, don't change it, but use the LAF that is already set.
		if (UIManager.getCrossPlatformLookAndFeelClassName().equals(lookAndFeelClassName))
			lookAndFeelClassName = UIManager.getLookAndFeel().getClass().getName();

		isGtk = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel".equals(lookAndFeelClassName);
	}


	private static Map<String,Icon> fetchIcons(String... names) {
		HashMap<String,Icon> res = new HashMap<>();
		for (String name : names) {
			res.put(name, UIManager.getIcon(name));
		}
		return res;
	}



	/**
	 * Sets System LAF.
	 * Can apply GTK Hack, that tries to use the regular file chooser instead of the crazy old one
	 * @param regularFileChooserHack whether to apply gtk file chooser hack
	 */
	public static void setupLookAndFeel(boolean regularFileChooserHack) {

		try {
			boolean applyHack = regularFileChooserHack && isGtk;

			Map<String, Icon> fileChooserIcons = null;
			if (applyHack) {
				fileChooserIcons = fetchIcons(
					"FileChooser.newFolderIcon",
					"FileChooser.upFolderIcon",
					"FileChooser.homeFolderIcon",
					"FileChooser.detailsViewIcon",
					"FileChooser.listViewIcon",
					"FileChooser.viewMenuIcon"
				);
			}

			UIManager.setLookAndFeel(lookAndFeelClassName);

			if (applyHack) {
				UIManager.put("FileChooserUI", "javax.swing.plaf.metal.MetalFileChooserUI");
				UIManager.getDefaults().putAll(fileChooserIcons);
			}

		} catch (Exception e) {
			logger.error("Could not acquire System LookAndFeel", e);
		}
	}
}
