/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ResolvedConnector;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.handlers.IAvailablePortsHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.connections.PartialConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.JConfigAction;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.ComponentUtil;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.model.AvailablePortsModel;

import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.util.Set;
import java.util.stream.Collectors;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addGroupTitle;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addWarning;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.clearComponentsFrom;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.endLayout;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridLine;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridRelativeX;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setupGridBag;

/**
 * Created by ramon on 02.03.2016.
 */
public class PartialConnectionDialog extends JDialog {

	public static final Color RED_COLOR = Color.RED;
	public static final Color STD_COLOR = (Color) UIManager.get("TextField.foreground");// Not ideal, since LaF could change this
	public static final Dimension MINIMUM_SIZE = new Dimension(900, 160);
	private static final Logger logger = LoggerFactory.getLogger(PartialConnectionDialog.class);
	private static final int[] COL_SPAN = {70, 70, 50, 150, 70, 50, 50};
	private static final double[] COL_DYNAMIC = {0, 0, 0, 1, 0, 0};
	private final GridBagConstraints constraints;
	private final int firstOptionalComponent;
	private final int firstOptionalLine;
	private final IPortHandler<?> port;
	private final JButton cancelButton;
	private final JButton applyButton;
	private final IModuleConfiguration module;
	private final IAvailablePortsHandler availablePortsHandler;
	private final JButton addButton;
	private final JLabel startHeaderLabel;
	private final JLabel widthHeaderLabel;
	private final JLabel endHeaderLabel;
	private final JLabel opConnHeaderLabel;
	private final JLabel opStartHeaderLabel;
	private final JLabel opEndHeaderLabel;
	private java.util.List<PartialUi> partialUis;
	private final JPanel mainPanel;
	private final JScrollPane scrollPane;
	private int gridLine = 0;
	/**
	 * Create new Partial Connection Dialog
	 * @param owner parent window
	 * @param port left side of partial connection (currently the only partial side)
	 * @param module parent of port
	 * @param availablePortsHandler ports available for connection
	 */
	@SuppressWarnings("MagicNumber")
	public PartialConnectionDialog(Window owner, IPortHandler port, IModuleConfiguration module, IAvailablePortsHandler availablePortsHandler, boolean showWarning, String warningMessage) {
		super(owner);
		this.module = module;
		this.availablePortsHandler = availablePortsHandler;
		this.setTitle("Partial Connection Dialog");

		this.mainPanel = new JPanel();
		constraints = setupGridBag(mainPanel, COL_SPAN, COL_DYNAMIC);
		constraints.insets = new Insets(0, 10, 10, 10);

		setModalityType(ModalityType.DOCUMENT_MODAL);

		if (showWarning) {
			setGridLine(constraints, gridLine);
			gridLine++;
			addWarning(mainPanel, constraints, warningMessage, 0);
		}
		setGridLine(constraints, gridLine);
		gridLine++;
		addGroupTitle(mainPanel, constraints, "Connections with " + port.getAbsoluteConnectorName(), 0);

		this.applyButton = new JButton(new ApplyAction());
		this.cancelButton = new JButton(new CancelAction());
		this.addButton = new JButton(new AddPartialUiAction());

		this.startHeaderLabel = new JLabel("Start");
		this.widthHeaderLabel = new JLabel("Width");
		this.endHeaderLabel = new JLabel("End");
		this.opConnHeaderLabel = new JLabel("Opponent Connector");
		this.opStartHeaderLabel = new JLabel("Start");
		this.opEndHeaderLabel = new JLabel("End");

		setGridLine(constraints, gridLine);
		gridLine++;
		setGridRelativeX(constraints);
		mainPanel.add(startHeaderLabel, constraints);
		mainPanel.add(widthHeaderLabel, constraints);
		mainPanel.add(endHeaderLabel, constraints);
		mainPanel.add(opConnHeaderLabel, constraints);
		mainPanel.add(opStartHeaderLabel, constraints);
		mainPanel.add(opEndHeaderLabel, constraints);


		firstOptionalComponent = 7;
		firstOptionalLine = gridLine;

		this.port = port;
		setMinimumSize(MINIMUM_SIZE);
		collectModel();

		if (partialUis.isEmpty()) {
			partialUis.add(new PartialUi());
		}

		scrollPane = new JScrollPane(mainPanel);
		add(scrollPane);

		readdUi();
		checkValid();
		updateUi();
	}

	private static SpinnerNumberModel getModel(JSpinner spinner) {
		return (SpinnerNumberModel) spinner.getModel();
	}

	public void autoSet(IPortHandler<?> autoSet) {
		findFirstEmptyUi().setOpConnector(autoSet);
	}

	private PartialUi findFirstEmptyUi() {
		for (PartialUi ui : partialUis) {
			if (!ui.hasConnector()) {
				return ui;
			}
		}

		PartialUi ui = new PartialUi();
		partialUis.add(ui);
		return ui;
	}

	private void collectModel() {
		partialUis = port.getPartialConnectorHandler().get().streamPartialConnections().map(PartialUi::new).collect(Collectors.toList());
	}

	private void updateUi() {
		partialUis.forEach(PartialUi::update);
	}

	private void readdUi() {
		clearComponentsFrom(mainPanel, firstOptionalComponent);

		int line = firstOptionalLine;

		for (PartialUi pUi : partialUis) {
			setGridLine(constraints, line++);
			setGridRelativeX(constraints);

			mainPanel.add(pUi.startSpinner, constraints);
			mainPanel.add(pUi.widthSpinner, constraints);
			mainPanel.add(pUi.endLabel, constraints);
			mainPanel.add(pUi.opPortBox, constraints);
			mainPanel.add(pUi.opStartSpinner, constraints);
			mainPanel.add(pUi.opEndLabel, constraints);
			mainPanel.add(pUi.removeButton, constraints);
		}

		setGridLine(constraints, line++);
		mainPanel.add(addButton, constraints);
		setGridLine(constraints, line);
		mainPanel.add(applyButton, constraints);
		setGridRelativeX(constraints);
		mainPanel.add(cancelButton, constraints);

		endLayout(mainPanel, constraints);

		pack();
		revalidate();
		repaint();
	}

	private boolean checkValid() {
		boolean isOverlapAllowed = port.getPartialConnectorHandler().get().isOverlappingAllowed();
		OverlapValidator validator = partialUis.stream().sorted().filter(PartialUi::hasConnector).collect(() -> new OverlapValidator(isOverlapAllowed), OverlapValidator::add, (a, b) -> {
			throw new UnsupportedOperationException();
		});

		boolean valid = validator.isValid(port.getConnectorWidth());
		applyButton.setEnabled(valid);

		boolean isConnectorSpaceLeft = validator.isConnectorSpaceLeft(port.getConnectorWidth());
		addButton.setEnabled(partialUis.size() == 0 || isOverlapAllowed || isConnectorSpaceLeft);

		return valid;
	}

	private ResolvedConnector getResolvedConnector(IPortHandler<?> portHandler) {
		ResolvedConnector resolvedConnector = new ResolvedConnector(portHandler, portHandler.getSourceElement());
		resolvedConnector.setConnectionHint(portHandler);
		return resolvedConnector;
	}

	private void saveConnections() {
		removeNonPartialConnections();

		Set<? extends IConnection<IPortHandler<?>>> existing = port.getPartialConnectorHandler().get().streamPartialConnections().collect(Collectors.toSet());
		//TODO

		Set<IConnection> planned = partialUis.stream().sorted().filter(PartialUi::hasConnector).map(partial -> module.getDocument().createPartialConnection(port, partial.getStart(), partial.opConnector, partial.getOpStart(), partial.getWidth())).collect(Collectors.toSet());

		existing.removeIf(planned::remove);

		existing.forEach(IConnection::disconnect);
		planned.forEach(IConnection::connect);

		logger.info("Made partial connections {}", planned);
	}

	private void removeNonPartialConnections() {
		port.streamConnections().filter(c -> !c.isPartial()).forEach(c -> c.disconnect());
	}


	private static class OverlapValidator {

		private int index = 0;
		private int maxIndex = 0;
		private int accumulatedWidth = 0;
		private boolean valid = true;
		private boolean isOverlapAllowed = false;

		public OverlapValidator(boolean isOverlapAllowed) {
			this.isOverlapAllowed = isOverlapAllowed;
		}

		public OverlapValidator add(PartialUi partial) {
			if ((partial.start < index && !isOverlapAllowed) || partial.width + partial.getOpStart() > partial.opConnector.getConnectorWidth()) {
				valid = false;
			}
			partial.setValid(valid);
			index = partial.start + partial.width;
			maxIndex = Math.max(index, maxIndex);
			accumulatedWidth += partial.width;
			return this;
		}

		public boolean isValid(long totalWidth) {
			return valid && totalWidth >= maxIndex;
		}

		public boolean isConnectorSpaceLeft(long totalWidth) {
			return accumulatedWidth < totalWidth;
		}
	}


	private class ApplyAction extends JConfigAction {

		public ApplyAction() {
			super("Apply", "Apply current configuration");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (!checkValid()) {
				logger.error("Cannot apply invalid partial connections, ignoring");
			} else {
				saveConnections();
				dispose();
			}
		}
	}


	private class CancelAction extends JConfigAction {

		public CancelAction() {
			super("Cancel", "Do not apply changes");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			dispose();
		}
	}


	private class AddPartialUiAction extends JConfigAction {

		public AddPartialUiAction() {
			super("Add", "Add Connection", "12/add.png");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			partialUis.add(new PartialUi());
			readdUi();
			updateUi();

			JScrollBar vertical = scrollPane.getVerticalScrollBar();
			vertical.setValue(vertical.getMaximum());
		}
	}

	private class RemovePartialUiAction extends JConfigAction {

		private final PartialUi ui;

		public RemovePartialUiAction(PartialUi ui) {
			super("Remove", "Remove Connection", "12/remove.png");
			this.ui = ui;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			partialUis.remove(ui);
			readdUi();
			updateUi();
			checkValid();
		}
	}


	private class PartialUi implements ComponentUtil.IPortSelectionChangedListener, ChangeListener, Comparable<PartialUi> {

		protected int start;
		protected int width;
		private int opStart;
		private JLabel startLabel;
		private JSpinner startSpinner;
		private JLabel widthLabel;
		private JSpinner widthSpinner;
		private JLabel endLabel;
		private JComboBox<ResolvedConnector> opPortBox;
		private JLabel opStartLabel;
		private JSpinner opStartSpinner;
		private JLabel opEndLabel;
		private JButton removeButton;
		private IPortHandler<?> opConnector;

		public PartialUi() {
			create();

			setStart(getInitialStart());
			setWidth(0, false);
		}

		public PartialUi(IConnection<IPortHandler<?>> partialConnection) {
			create();

			IPortHandler<?> opConnector = partialConnection.getOppositeConnector(port);

			setOpConnector(opConnector, false);
			setStart(partialConnection.getMyStart(port), false);
			setOpStart(partialConnection.getMyStart(opConnector), false);

			if (partialConnection instanceof PartialConnection) {
				setWidth(((PartialConnection) partialConnection).getWidth(), false);
			}

			initSpinners();
			update();
		}

		private void create() {
			startLabel = new JLabel("Start:", SwingConstants.RIGHT);
			startSpinner = new JSpinner(new SpinnerNumberModel(0, 0, 0, 1));
			startSpinner.addChangeListener(this);
			widthLabel = new JLabel("Width:", SwingConstants.RIGHT);
			widthSpinner = new JSpinner(new SpinnerNumberModel(0, 0, 0, 1));
			widthSpinner.addChangeListener(this);
			opStartLabel = new JLabel("Start:", SwingConstants.RIGHT);
			opStartSpinner = new JSpinner(new SpinnerNumberModel(0, 0, 0, 1));
			opStartSpinner.addChangeListener(this);
			endLabel = new JLabel();
			opEndLabel = new JLabel();
			opPortBox = ComponentUtil.createConnectorComboBox(AvailablePortsModel.getPortModel(availablePortsHandler, module, port), this);
			removeButton = new JButton(new RemovePartialUiAction(this));
		}

		private void setOpConnector(IPortHandler<?> opConnector) {
			setOpConnector(opConnector, true);
		}

		private void setOpConnector(IPortHandler<?> opConnector, boolean check) {
			if (this.opConnector == opConnector) {
				return;
			}

			this.opConnector = opConnector;

			opStart = 0;

			if (opConnector != null) {
				setWidth(getInitialWidth(opConnector), check);
			} else {
				setWidth(0, check);
			}

			initSpinners();
			update();
		}

		public int getWidth() {
			return widthSpinner.isEnabled()? width : IConnection.FULL_CONNECTOR_WIDTH;
		}

		public int getInitialWidth(IPortHandler<?> opConnector) {
			boolean isOverlapAllowed = port.getPartialConnectorHandler().get().isOverlappingAllowed();
			return Math.min((port.getConnectorWidth() - (isOverlapAllowed? 0 : this.start)), opConnector.getConnectorWidth());
		}

		private void setWidth(int width) {
			setWidth(width, true);
		}

		private void setWidth(int width, boolean check) {
			if (width == this.width) {
				return;
			}

			this.width = width;

			update();
			if (check) {
				checkValid();
			}
		}

		private void initSpinners() {
			if (port != null && opConnector != null) {
				SpinnerNumberModel model = getModel(startSpinner);
				model.setMaximum(Math.toIntExact(port.getConnectorWidth()) - 1);
				model.setMinimum(0);

				model = getModel(widthSpinner);
				model.setMaximum(opConnector.getConnectorWidth());
				model.setMinimum(1);

				model = getModel(opStartSpinner);
				model.setMaximum(opConnector.getConnectorWidth() - 1);
				model.setMinimum(0);
			}
		}

		public void update() {
			boolean hasCon = opConnector != null;

			endLabel.setText((hasCon? Integer.toString(start + width - 1) : "-"));
			opEndLabel.setText((hasCon? Integer.toString(opStart + width - 1) : "-"));

			startSpinner.setEnabled(hasCon);
			startSpinner.setValue(start);

			widthSpinner.setEnabled(hasCon);
			widthSpinner.setValue(width);

			opStartSpinner.setEnabled(hasCon);
			opStartSpinner.setValue(opStart);

			Object selectedItem = opPortBox.getSelectedItem();
			if (opConnector == null) {
				opPortBox.setSelectedItem(null);
				setValid(true);
			} else if (selectedItem == null || !((ResolvedConnector) selectedItem).isConnectionHintMatch(opConnector)) {
				opPortBox.setSelectedItem(getResolvedConnector(opConnector));
			}
		}

		@Override
		public void onPortSelectionChanged() {
			Object selectedItem = opPortBox.getSelectedItem();
			if (selectedItem != null) {
				setOpConnector((IPortHandler<?>) ((ResolvedConnector) selectedItem).getConnectorHandler());
			} else {
				setOpConnector(null);
			}
		}

		@Override
		public void stateChanged(ChangeEvent e) {
			if (e.getSource() == startSpinner) {
				setStart(((int) getModel(startSpinner).getValue()));
			} else if (e.getSource() == widthSpinner) {
				setWidth(((int) getModel(widthSpinner).getValue()));
			} else if (e.getSource() == opStartSpinner) {
				setOpStart(((int) getModel(opStartSpinner).getValue()));
			} else {
				logger.warn("Ignoring change from sth. other than startSpinner, widthSpinner or opStartSpinner");
			}
		}

		public int getStart() {
			return (startSpinner.isEnabled())? start : IConnection.FULL_CONNECTOR_WIDTH;
		}

		private void setStart(int value) {
			setStart(value, true);
		}

		private void setStart(int value, boolean check) {
			if (this.start == value) {
				return;
			}

			this.start = value;
			update();
			if (check) {
				checkValid();
			}
		}

		private void setOpStart(int value) {
			setOpStart(value, true);
		}

		private void setOpStart(int value, boolean check) {
			if (this.opStart == value) {
				return;
			}

			this.opStart = value;
			update();
			if (check) {
				checkValid();
			}
		}

		private int getInitialStart() {
			int predecessorIndex = partialUis.size() - 1;

			//init start of first partialUI with 0
			if (predecessorIndex < 0) {
				return 0;
			}

			PartialUi predecessor = partialUis.get(predecessorIndex);

			//check if user has set opConnector on predecessor UI
			if (predecessor.opConnector == null) {
				return 0;
			}

			return Math.min(predecessor.getStart() + predecessor.width, (port.getConnectorWidth() - 1));
		}

		@Override
		public int compareTo(@NotNull PartialUi o) {
			return start - o.start;
		}

		public void setValid(boolean valid) {
			if (hasConnector() && !valid) {
				startSpinner.setForeground(RED_COLOR);
			} else {
				startSpinner.setForeground(STD_COLOR);
			}
		}

		public boolean hasConnector() {
			return opConnector != null;
		}

		@Override
		public String toString() {
			return "PartialUi{" +
				"opConnector=" + opConnector +
				", start=" + start +
				", opStart=" + opStart +
				", width=" + width +
				'}';
		}

		public int getOpStart() {
			return (opStartSpinner.isEnabled())? opStart : 0;
		}
	}

}
