/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.Lua;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPluginEngine;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IScriptManager;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.DocumentAction;

import org.jetbrains.annotations.NotNull;

import java.awt.event.ActionEvent;
import java.util.Optional;

/**
 * Created by melanieb on 04.04.16.
 */
public class AutoScriptAction extends DocumentAction implements IDocumentHandler.IDocumentHandlerListener {

	private boolean selected = false;

	public AutoScriptAction(IDocumentHandler documentHandler) {
		super("Auto-Script", "Enables automatic script execution",
			null, documentHandler);

		documentHandler.addDocumentHandlerListener(this);

		showState();
	}

	/**
	 * Wenn noch cjconfig dazu kommt dafür einen passenden befehl dann rein machen der dies macht
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		Optional<IScriptManager> scriptManager = readState();
		selected = (boolean) getValue(SELECTED_KEY);
		scriptManager.ifPresent(sm -> sm.setAutoScriptEnabled(selected));

		showState();
	}

	@NotNull
	private Optional<IScriptManager> readState() {
		IPluginEngine pluginEngine = documentHandler.getPluginEngine();

		if (pluginEngine != null) {
			Optional<IScriptManager> scriptManager = pluginEngine.getScriptManager();
			if (scriptManager.isPresent()) {
				setEnabled(true);
				selected = scriptManager.get().isAutoScriptEnabled();
				return scriptManager;
			}
		}
		setEnabled(false);
		selected = false;
		return Optional.empty();
	}

	private void showState() {
		putValue(SELECTED_KEY, selected);
	}

	@Override
	public void onDocumentSwitched(boolean valid) {
		if (valid) {
			readState();
			showState();
		}
	}
}



