/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.handlers.DocumentHandler;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.event.ActionEvent;

/**
 * An Action that requires the {@link DocumentHandler} to contain a valid document
 * Actions of this type will disable themselves as long as the handle does not have a valid document
 */
public abstract class ProcessDocumentAction extends DocumentAction implements IDocumentHandler.IDocumentHandlerListener {

	private static final Logger logger = LoggerFactory.getLogger(ProcessDocumentAction.class);

	/**
	 * Create a action that requires a loaded document
	 * @param name name / label of action
	 * @param tooltip tooltip
	 * @param icon icon
	 * @param documentHandler documentHandler to observe for loaded documents
	 */
	public ProcessDocumentAction(String name, String tooltip, String icon, IDocumentHandler documentHandler) {
		super(name, tooltip, icon, documentHandler);

		setEnabled(documentHandler.hasLoadedDocument());
		documentHandler.addDocumentHandlerListener(this);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (!documentHandler.hasLoadedDocument()) {
			logger.warn("actionPerformed on supposedly disabled action. No document available!");

			return;
		}

		processDocument();
	}

	/**
	 * This call guarantees that a valid document is available for processing
	 */
	public abstract void processDocument();

	public void dispose() {
		documentHandler.removeDocumentHandlerListener(this);
	}

	@Override
	public void onDocumentSwitched(boolean valid) {
		setEnabled(valid);
	}
}
