/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.document;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.exceptions.JConfigBuildException;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.exceptions.JConfigBuildValidationException;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.ProcessDocumentAction;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Action to start building current document
 */
public class BuildAction extends ProcessDocumentAction {

	private static final Logger logger = LoggerFactory.getLogger(BuildAction.class);

	private final boolean incremental;

	/**
	 * Create action that starts building the current document of documentHandler
	 * @param documentHandler handler for current document
	 * @param incremental whether to request incremental build of documentHandler
	 */
	public BuildAction(IDocumentHandler documentHandler, boolean incremental) {
		super(incremental? "Build Incrementally" : "Build All",
			"Create project files required for make process",
			"16/make.png",
			documentHandler);
		this.incremental = incremental;
	}

	@Override
	public void processDocument() {
		String buildType = incremental? "Incremental" : "Full";
		try {
			documentHandler.build(incremental);
		} catch (JConfigBuildValidationException e)  {
			logger.error("{} Build was aborted due to failed validation!", buildType);
		} catch (JConfigBuildException e) {
			logger.error("{} Build failed internally", buildType, e);
		}
	}
}
