/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.document;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IApplicationConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.DocumentAction;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.fileHandling.DocumentFileUtil;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.util.UserCanceledException;

import java.awt.Component;
import java.awt.event.ActionEvent;

/**
 * Created by ramonw on 27.11.15.
 */
public class NewDocumentAction extends DocumentAction {

	private final Component parent;
	private final IApplicationConfiguration application;

	public NewDocumentAction(Component parent, IDocumentHandler documentHandler, IApplicationConfiguration application) {
		super("New",
			"Start over with an empty configuration",
			"16/new.png",
			documentHandler);
		this.parent = parent;
		this.application = application;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		try {
			DocumentFileUtil.ensureSavedDocument(parent, documentHandler, false);

			Library library = application.getLibrary();
			TargetDescription target = DocumentFileUtil.askForTarget(parent, library);

			documentHandler.newDocument(target, library.getDeviceDescription(target.getDevices().get(0).getName()));//TODO show selector for picking device
		} catch (UserCanceledException e1) {

		}
	}
}
