/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.ui.components;

import com.jidesoft.hints.ListDataIntelliHints;

import java.util.List;
import javax.swing.text.JTextComponent;

/**
 * Implementation of jidesoft's IntelliHints to display auto completion candidates.
 *
 * Created by tobiass on 08.12.16.
 */
public class AutoCompletionIntelliHints extends ListDataIntelliHints {

	private int identifierStart = 0;

	public AutoCompletionIntelliHints(JTextComponent comp, List completionList, int identifierStart) {
		super(comp, completionList);
		this.identifierStart = identifierStart;

		setFollowCaret(true);
		setHintsEnabled(false);
	}

	@Override
	protected Object getContext() {
		String input = getTextComponent().getText();

		if (input.length() >= identifierStart) {
			return input.substring(identifierStart);
		} else {
			setHintsEnabled(false);
			return input;
		}
	}

	@Override
	public void acceptHint(Object selected) {
		JTextComponent input = getTextComponent();
		input.setText(input.getText().substring(0, identifierStart) + selected);

		//mark parameters
		int parameterStart = selected.toString().lastIndexOf('(') + 1;
		int parameterEnd = selected.toString().lastIndexOf(')');

		if (parameterStart > 0 && parameterEnd > 0 && parameterStart < parameterEnd) {
			String parameters = selected.toString().substring(parameterStart, parameterEnd);
			if (parameters.length() > 0) {
				input.setSelectionStart(input.getText().indexOf(parameters));
				input.setSelectionEnd(input.getText().indexOf(parameters) + parameters.length());
			}
		}
	}

	public void setIdentifierStart(int identifierStart) {
		this.identifierStart = identifierStart;
	}
}
