/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.components;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.ResolvedConnector;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.chrome.ResolvedConnectorComboBoxRenderer;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.model.AvailablePortsModel;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.model.FilteringPortsModel;
import de.tu_darmstadt.rs.util.StringSearchMatcher;

import com.jidesoft.swing.AutoCompletion;
import com.jidesoft.swing.ComboBoxSearchable;
import com.jidesoft.swing.event.SearchableEvent;

import java.awt.Font;
import java.awt.Insets;
import javax.swing.JComboBox;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

/**
 * Created by ramonw on 01.12.15.
 */
public class ComponentUtil {

	public static JTextArea createMessageTextArea(String message, boolean lineWrap, boolean monospaced) {
		JTextArea textArea = new JTextArea(message);
		textArea.setLineWrap(lineWrap);
		textArea.setWrapStyleWord(lineWrap);
		textArea.setEditable(false);
		if (monospaced) {
			textArea.setFont(new Font(Font.MONOSPACED, Font.PLAIN, textArea.getFont().getSize()));
		}
		textArea.setMargin(new Insets(5, 5, 5, 5));
		return textArea;
	}

	public static JComboBox<ResolvedConnector> createConnectorComboBox(AvailablePortsModel<?> model, IPortSelectionChangedListener listener) {
		final FilteringPortsModel filteredModel = new FilteringPortsModel(model);
		JComboBox<ResolvedConnector> comboBox = new JComboBox<>(filteredModel);


		comboBox.setRenderer(new ResolvedConnectorComboBoxRenderer());
		comboBox.setEditable(false);

		comboBox.addPopupMenuListener(new PopupMenuListener() {
			ResolvedConnector oldItem;
			boolean cancel = false;

			@Override
			public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
				oldItem = (ResolvedConnector) comboBox.getSelectedItem();
				cancel=false;
			}

			@Override
			public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
				if (!cancel) {
					SwingUtilities.invokeLater(listener::onPortSelectionChanged);
				}
			}

			@Override
			public void popupMenuCanceled(PopupMenuEvent e) {
				comboBox.setSelectedItem(oldItem);
				cancel=true;
			}
		});


		//TODO check if the combobox width is appropriate // -> long enough
		comboBox.setPrototypeDisplayValue(ResolvedConnector.getDummyPort("/Subsystem_0/slave_core_conn_0.access_peri"));

		final CustomSearchable searchable = new CustomSearchable(comboBox);
		AutoCompletion autoCompletion = new AutoCompletion(comboBox, searchable);
		autoCompletion.setStrict(true);
		autoCompletion.setStrictCompletion(true);
		searchable.setCaseSensitive(true);
		searchable.setWildcardEnabled(true);
		searchable.setFromStart(false);
		filteredModel.setSearchable(searchable);
		comboBox.addPopupMenuListener(filteredModel);

		//Open Popup if user starts search (otherwise we can't detect if the user ends by pressing enter or escape)
		autoCompletion.getSearchable().addSearchableListener(e -> {
			if (e.getID()== SearchableEvent.SEARCHABLE_START) {
				comboBox.showPopup();
			}
		});

		return comboBox;
	}

	public interface IPortSelectionChangedListener {

		void onPortSelectionChanged();
	}


	/**
	 * Searchable that allows access to compare method
	 */
	public static class CustomSearchable extends ComboBoxSearchable {

		public CustomSearchable(JComboBox comboBox) {
			super(comboBox);
		}

		@Override
		public boolean compare(Object element, String searchingText) {
			return super.compare(element, searchingText);
		}

		@Override
		protected boolean compare(String text, String searchingText) {
			return StringSearchMatcher.match(text, searchingText);
		}
	}
}
