/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.components;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameter;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBooleanParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IFloatParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IIntegerParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.AttributeCheckBox;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.AttributeChoiceTextField;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.AttributeComboBox;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.ConvertedParameterTextField;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.IAttributeWatcher;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.UnitConverter;

import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.JComponent;

/**
 * Created by ramon on 05.02.2016.
 */
public class WatchableComponentHolder<A extends IAttributeHandler, T extends JComponent & IAttributeWatcher<A>> implements IAttributeWatcher<A> {

	private static final Logger logger = LoggerFactory.getLogger(WatchableComponentHolder.class);

	private final Class<A> limit;
	private final IParameter parameter;
	private final Library library;
	private T component;

	private WatchableComponentHolder(Class<A> limit, T component, IParameter parameter, Library library) {
		this.limit = limit;
		this.component = component;
		this.parameter = parameter;
		this.library = library;
	}

	public static WatchableComponentHolder<? extends IParameterHandler, ?> getTypedParameterField(IParameter parameter, Library library) {
		switch (parameter.getType()) {
			case FloatType:
				return createTypedParameterField(IFloatParameterHandler.class, parameter, library);
			case HexType:
			case IntType:
			case BinType:
				return createTypedParameterField(IIntegerParameterHandler.class, parameter, library);
			case BoolType:
				return new WatchableComponentHolder<>(IBooleanParameterHandler.class, new AttributeCheckBox(), parameter, library);
			case StringType:
				return createTypedParameterField(IParameterHandler.class, parameter, library);
			default:
				throw new IllegalArgumentException("Invalid parameterType " + parameter.getType() + " on " + parameter.getName());
		}
	}

	@NotNull
	private static <A extends IParameterHandler> WatchableComponentHolder<A, ?> createTypedParameterField(Class<A> limit, IParameter parameter, Library library) {
		if (parameter.isChoicesOnly()) {
			return new WatchableComponentHolder<>(limit, new AttributeComboBox<>(), parameter, library);
		} else if (parameter.hasUiUnit()) {
			try {
				return new WatchableComponentHolder<>(limit, new ConvertedParameterTextField<>(new UnitConverter(parameter.getUiUnit())), parameter, library);
			} catch (Exception e) {
				logger.error("Failed to parse unit config {}", parameter.getUiUnit(), e);
				return new WatchableComponentHolder<>(limit, new AttributeChoiceTextField<>(), parameter, library);
			}
		} else {
			return new WatchableComponentHolder<>(limit, new AttributeChoiceTextField<>(), parameter, library);
		}
	}

	public JComponent getComponent() {
		return component;
	}

	public void attach(IModuleConfiguration<?> element) {
		if (element != null) {
			component.setHandler(element.getParameterHandler(limit, parameter));
		} else {
			component.setHandler(null);
		}
	}

	@Override
	public void setHandler(A handler) {
		throw new UnsupportedOperationException("Use attach");
	}

	@Override
	public void update() {
		component.update();
	}
}
