/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.chrome;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.ResolvedConnector;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.model.FilteringPortsModel;
import de.tu_darmstadt.rs.util.StringSearchMatcher;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.plaf.basic.BasicComboBoxRenderer;
import javax.swing.plaf.basic.BasicLabelUI;
import javax.swing.plaf.synth.SynthContext;
import javax.swing.plaf.synth.SynthLabelUI;

/**
 * Created by Ramon on 26.04.2016.
 */
public class ResolvedConnectorComboBoxRenderer extends BasicComboBoxRenderer {

	private static final Logger logger = LoggerFactory.getLogger(ResolvedConnectorComboBoxRenderer.class);
	private FilteringPortsModel model;

	public ResolvedConnectorComboBoxRenderer() {
		if (getUI() instanceof SynthLabelUI) {
			setUI(new SynthUI());
		} else {
			setUI(new BasicUI());
		}
	}

	@Override
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		ResolvedConnectorComboBoxRenderer comp = (ResolvedConnectorComboBoxRenderer) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

		if (value == null) {
			return comp;
		}

		this.model = (FilteringPortsModel) list.getModel();
		final ResolvedConnector conn = (ResolvedConnector) value;


		comp.setFont(comp.getFont().deriveFont(Font.PLAIN));

		if (!model.isRecommendedConnection(conn)) {
			comp.setFont(comp.getFont().deriveFont(Font.ITALIC));
		} else if (model.isLevenshteinClose(conn)) {
			comp.setFont(comp.getFont().deriveFont(Font.BOLD));
		}


		return comp;
	}

	private void drawBox(String text, int charOffs, int length, FontMetrics metrics, Graphics2D g2d, int xOffs) {
		if (text.length() < charOffs) {
			return;
		}

		if (text.length() < length + charOffs) {
			length = text.length() - charOffs;
		}
		String before = text.substring(0, charOffs);
		int lenBefore = metrics.stringWidth(before);

		String inside = text.substring(charOffs, charOffs + length);
		int lenInside = metrics.stringWidth(inside);

		g2d.fillRoundRect(lenBefore + xOffs, 2, lenInside + 1, ResolvedConnectorComboBoxRenderer.this.getHeight() - 4, 2, 2);
	}


	private void paintHighlights(Graphics g, int textX, int textY, String text) {
		Graphics2D g2d = (Graphics2D) g;
		final Paint oldPaint = g2d.getPaint();
		Color color1 = new Color(212, 213, 182);
		Color color2 = new Color(216, 201, 124);


		FontMetrics metrics = g.getFontMetrics(getFont());

		int h = ResolvedConnectorComboBoxRenderer.this.getHeight();

		GradientPaint gp = new GradientPaint(
			0, 0, color1, 15, h, color2);
		g2d.setPaint(gp);


		if (model.getSearchingText() != null) {

			StringSearchMatcher.matchPositions(text, model.getSearchingText())
				.ifPresent(l -> l.forEach(match -> drawBox(text, match.startIndex, match.length, metrics, g2d, 1)));
		}


		//g2d.setPaint(oldPaint);
	}

	private class BasicUI extends BasicLabelUI {
		@Override
		protected void paintEnabledText(JLabel l, Graphics g, String s, int textX, int textY) {
			paintHighlights(g, textX, textY, s);
			super.paintEnabledText(l, g, s, textX, textY);
		}

		@Override
		protected void paintDisabledText(JLabel l, Graphics g, String s, int textX, int textY) {
			paintHighlights(g, textX, textY, s);
			super.paintDisabledText(l, g, s, textX, textY);
		}
	}

	private class SynthUI extends SynthLabelUI {
		@Override
		protected void paint(SynthContext context, Graphics g) {
			final String text = ((JLabel) context.getComponent()).getText();
			paintHighlights(g, 0, 0, text);
			super.paint(context, g);
		}
	}
}
