/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IApplicationConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.ConfigurationIssueList;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.chrome.SingleLineBorder;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.menus.ModuleMenu;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.resources.IconUtil;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.LinkedList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellEditor;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * Panel that contains the hardware tree and actual contents
 * Bound to a single Document
 */
public class ConfigurationDocumentPane extends JSplitPane implements TreeSelectionListener, IDocumentHandler.IDocumentHandlerListener, IApplicationConfiguration.ILibraryReadyListener {

	private static final Logger logger = LoggerFactory.getLogger(ConfigurationDocumentPane.class);
	private final IApplicationConfiguration application;
	private final IDocumentHandler documentHandler;
	private final ConfigTreeModel treeModel;
	private Component editorPanePlaceholder;

	/**
	 * Incoming message from our Log4j2 hook 'JConfigUiAppender'
	 *
	 * @param message the formatted message, according to the layout of the appender
	 * @param logEvent the base event, from which the level, message, origin etc. can be obtained
	 */
	//	@Override
	//	public void onNewMessage(String message, LogEvent logEvent) {
	//		logEventHandler.onNewLogMessage(message, logEvent);
	//	}


	private ConfigurationIssueList configurationIssues;
	private JSplitPane topSplitter;
	private ModuleEditorPane editorPane;
	private JTree documentTree;
	private ModuleMenu moduleMenu;
	private final ScriptReplPane scriptReplPane;

	/**
	 *
	 */
	public ConfigurationDocumentPane(IApplicationConfiguration application, IDocumentHandler documentHandler) {
		super(JSplitPane.VERTICAL_SPLIT);
		this.application = application;
		this.documentHandler = documentHandler;
		this.treeModel = new ConfigTreeModel();

		// Components

		configurationIssues = new ConfigurationIssueList(this);
		//		fLogArea = ComponentUtil.createMessageTextArea("", false, true);
		//		fLogArea.setPreferredSize(new Dimension(0, 80));
		//		fLogArea.setEditable(false);
		//		JLabel logAreaTitleLabel = new JLabel("Messages");
		//		logEventHandler = new LogEventHandler(logAreaTitleLabel, fLogArea);

		documentTree = new JTree(treeModel);
		documentTree.addTreeSelectionListener(this);
		ConfigTreeCellRenderer configTreeCellRenderer = new ConfigTreeCellRenderer();
		documentTree.setCellRenderer(configTreeCellRenderer);
		documentTree.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_DELETE && !documentTree.isSelectionEmpty()) {
					IConfigurationElement<?, ?, ?> element = (IConfigurationElement<?, ?, ?>) documentTree.getSelectionPath().getLastPathComponent();
					if (element.getType() != IConfigurationElement.ElementType.Configuration) {
						documentHandler.removeModule(element);
					}
				}
			}
		});
		documentTree.setCellEditor(new ConfigTreeCellOuterEditor(documentTree, configTreeCellRenderer));
		documentTree.setEditable(true);
		documentTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		topSplitter = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		topSplitter.setBorder(BorderFactory.createEmptyBorder());
		topSplitter.setLeftComponent(createTitledScroller(new JLabel("System components"), documentTree));

		editorPanePlaceholder = new JPanel();


		setTopComponent(topSplitter);

		setResizeWeight(1.0);

		scriptReplPane = new ScriptReplPane(documentHandler);

		JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		JScrollPane scrollPane = new JScrollPane(configurationIssues);
		scrollPane.setPreferredSize(new Dimension(0, 90));
		tabbedPane.add("Errors", scrollPane);
		tabbedPane.add("Terminal", scriptReplPane);
		setBottomComponent(tabbedPane);

		setDividerLocation(getHeight() - 100);//??? -> not constant!
		topSplitter.setDividerLocation(200);
		topSplitter.setResizeWeight(0);

		if (application.isLibraryReady()) {
			onLibraryReady(application.getLibrary());
		} else {
			documentTree.setEnabled(false);
			application.addLibraryReadyListener(this);
		}

		//TODO maybe force selection on tree, to force update of editor
	}

	@Override
	public void onLibraryReady(Library library) {
		addMenu();
		documentTree.setEnabled(true);
		documentHandler.addDocumentHandlerListener(this);

		if (documentHandler.hasLoadedDocument()) {
			swapDocument();
		}
	}

	private void addMenu() {
		moduleMenu = new ModuleMenu(application,documentTree, application.getLibrary(), documentHandler);
		documentTree.setComponentPopupMenu(moduleMenu);
	}

	private void swapDocument() {
		IConfigurationDocument document = documentHandler.getDocument();

		if (editorPane != null) {
			editorPane = null;
			topSplitter.setRightComponent(null);
		}

		treeModel.setDocument(document);
		configurationIssues.setDocument(document);

		if (document != null) {
			editorPane = new ModuleEditorPane(documentHandler, application);
			topSplitter.setRightComponent(editorPane);
			topSplitter.setDividerLocation(200);
		} else {
			topSplitter.setRightComponent(editorPanePlaceholder);
		}
	}

	@Override
	public void addNotify() {
		super.addNotify();
		logger.trace("addNotify");
	}

	@Override
	public void removeNotify() {
		super.removeNotify();
		logger.trace("removeNotify");
	}


	@Deprecated
	private JComponent createTitledScroller(JLabel label, JComponent component) {
		JScrollPane scroller = new JScrollPane(component);
		scroller.setBorder(BorderFactory.createEmptyBorder());
		label.setFont(UIManager.getFont("TextField.font"));
		label.setBorder(new SingleLineBorder(SwingConstants.BOTTOM));
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(label, BorderLayout.NORTH);
		panel.add(scroller, BorderLayout.CENTER);
		return panel;
	}

	/*
	 * tree selection listener
	 */
	@Override
	public void valueChanged(final TreeSelectionEvent e) {
		if (e.isAddedPath()) {
			IConfigurationElement selectedElement = (IConfigurationElement) e.getPath().getLastPathComponent();
			selectElement(selectedElement, false);
		}
	}

	public void selectElement(IConfigurationElement selectedElement, boolean updateTree) {
		editorPane.setSelectedElement(selectedElement);
		if (updateTree) {
			documentTree.setSelectionPath(new TreePath(treeModel.getPath(selectedElement).toArray()));
		}
	}

	@Override
	public void onDocumentSwitched(boolean valid) {
		swapDocument();
	}

	/**
	 * Required because Java devs are stupid!
	 * {@link DefaultTreeCellEditor.DefaultTextField} is non-static inner class!!
	 */
	private static class ConfigTreeCellOuterEditor extends DefaultTreeCellEditor {

		public ConfigTreeCellOuterEditor(JTree tree, DefaultTreeCellRenderer renderer) {
			super(tree, renderer);
		}

		@Override
		protected TreeCellEditor createTreeCellEditor() {
			Border aBorder = UIManager.getBorder("Tree.editorBorder");
			DefaultCellEditor editor = new ConfigTreeCellInnerEditor(new DefaultTextField(aBorder));

			// One click to edit.
			editor.setClickCountToStart(1);
			return editor;
		}
	}


	/**
	 * Inner CellEditor, to be used in concert with {@link DefaultTreeCellEditor}.
	 * Pick only element nme as content.
	 * Integrate config that is normally done in anonymous class inside {@link DefaultTreeCellEditor#createTreeCellEditor()}
	 */
	private static class ConfigTreeCellInnerEditor extends DefaultCellEditor {

		private ConfigTreeCellInnerEditor(JTextField textField) {
			super(textField);
		}

		@Override
		public boolean shouldSelectCell(EventObject event) {
			boolean retValue = super.shouldSelectCell(event);
			return retValue;
		}

		@Override
		public Component getTreeCellEditorComponent(JTree tree, Object value, boolean isSelected, boolean expanded, boolean leaf, int row) {
			return super.getTreeCellEditorComponent(tree, ((IConfigurationElement<?, ?, ?>) value).getName(), isSelected, expanded, leaf, row);
		}
	}


	private static class ConfigTreeCellRenderer extends DefaultTreeCellRenderer {

		public ConfigTreeCellRenderer() {
			super();

			ImageIcon fallbackIcon = IconUtil.getIconForElement(IConfigurationElement.ElementType.CommonModule, 20);
			ImageIcon folderFallbackIcon = IconUtil.getIconForElement(IConfigurationElement.ElementType.Subsystem, 20);
			setClosedIcon(folderFallbackIcon);
			setOpenIcon(folderFallbackIcon);
			setLeafIcon(fallbackIcon);
		}

		@Override
		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
			super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
			setBorder(BorderFactory.createEmptyBorder(2, 0, 0, 0));

			IConfigurationElement element = (IConfigurationElement) value;
			setIcon(IconUtil.getIconForElement(element.getType(), 20));
			setText(element.getName());
			// TODO: disabled red text for invalid items due to missing implementation of error logic
			//        if (!element.isInRange()) {
			//        	setForeground(Color.RED);
			//        }

			return this;
		}
	}


	private class ConfigTreeModel implements TreeModel, IConfigurationDocument.IConfigurationHierarchyListener {

		private List<TreeModelListener> modelListeners = new ArrayList<>();
		private IConfigurationDocument document;

		public ConfigTreeModel() {

		}

		private LinkedList<IConfigurationElement<?, ?, ?>> getPath(IConfigurationElement<?, ?, ?> element) {
			LinkedList<IConfigurationElement<?, ?, ?>> path = new LinkedList<>();
			IConfigurationElement<?, ?, ?> parent = element;

			do {
				path.addFirst(parent);
			} while ((parent = parent.getParent()) != null);

			return path;
		}

		@Override
		public IConfigurationDocument getRoot() {
			return document;
		}

		@Override
		public Object getChild(Object parent, int index) {
			if (parent instanceof IConfigurationDocument) {
				IConfigurationDocument doc = (IConfigurationDocument) parent;
				int subCount = doc.getSubsystems().size();
				if (index < subCount) {
					return doc.getSubsystems().get(index);
				} else {
					return doc.getModules().get(index - subCount);
				}
			} else if (parent instanceof ISubsystemConfiguration) {
				ISubsystemConfiguration proc = (ISubsystemConfiguration) parent;
				return proc.getModules().get(index);
			} else {
				logger.warn("Node {} has no children", parent);
				return null;
			}
		}

		@Override
		public int getChildCount(Object parent) {
			if (parent instanceof IConfigurationDocument) {
				return ((IConfigurationDocument) parent).getSubsystems().size() + ((IConfigurationDocument) parent).getModules().size();
			} else if (parent instanceof ISubsystemConfiguration) {
				return ((ISubsystemConfiguration) parent).getModules().size();
			} else {
				return 0;
			}
		}

		@Override
		public boolean isLeaf(Object node) {
			return !(node instanceof IConfigurationDocument && (((IConfigurationDocument) node).getSubsystems().size() != 0 || ((IConfigurationDocument) node).getModules().size() != 0))
				&& !(node instanceof ISubsystemConfiguration && ((ISubsystemConfiguration) node).getModules().size() != 0);
		}

		@Override
		public void valueForPathChanged(TreePath path, Object newValue) {
			((IConfigurationElement<?, ?, ?>) path.getLastPathComponent()).getNameHandler().setStoredValue((String) newValue);
		}

		@Override
		public int getIndexOfChild(Object parent, Object child) {
			if (parent == null || child == null) {
				return -1;
			}
			if (parent instanceof IConfigurationDocument && child instanceof IModuleConfiguration) {
				int index = ((IConfigurationDocument) parent).getSubsystems().indexOf(child);
				return (index >= 0)? index : ((IConfigurationDocument) parent).getModules().indexOf(child) + ((IConfigurationDocument) parent).getSubsystems().size();
			} else if (parent instanceof ISubsystemConfiguration && child instanceof IModuleConfiguration) {
				return ((ISubsystemConfiguration) parent).getModules().indexOf(child);
			} else {
				return -1;
			}
		}

		@Override
		public void addTreeModelListener(TreeModelListener l) {
			modelListeners.add(l);
		}

		@Override
		public void removeTreeModelListener(TreeModelListener l) {
			modelListeners.remove(l);
		}

		public void setDocument(IConfigurationDocument document) {
			if (this.document != null) {
				this.document.removeHierarchyListener(this);
			}

			this.document = document;

			if (document != null) {
				document.addHierarchyListener(this);
			}

			fireRootChangedEvent();
		}

		private void fireRootChangedEvent() {
			TreeModelEvent event = new TreeModelEvent(this, new Object[]{getRoot()});
			for (TreeModelListener l : modelListeners) {
				l.treeStructureChanged(event);
			}
		}

		@Override
		public void onModuleAdded(IModuleConfiguration<?> module) {
			TreeModelEvent e = new TreeModelEvent(this, getPath(module.getParent()).toArray(), new int[]{getIndexOfChild(module.getParent(), module)}, new Object[]{module});
			for (TreeModelListener l : modelListeners) {
				l.treeNodesInserted(e);
			}
			documentTree.scrollPathToVisible(new TreePath(getPath(module).toArray()));
		}

		@Override
		public void onModuleRemoved(IConfigurationElement<?, ?, ?> parent, IModuleConfiguration module) {
			TreeModelEvent e = new TreeModelEvent(this, getPath(parent).toArray());
			for (TreeModelListener l : modelListeners) {
				l.treeStructureChanged(e);
			}
			if (documentTree.getSelectionPath() == null) {
				documentTree.setSelectionPath(new TreePath(documentHandler.getDocument()));
			}
		}

		@Override
		public void onElementNameChanged(IConfigurationElement<?, ?, ?> element) {
			editorPane.onElementNameChange(element);
			TreeModelEvent e = new TreeModelEvent(this, getPath(element).toArray());
			for (TreeModelListener l : modelListeners) {
				l.treeNodesChanged(e);
			}
		}
	}
}
