/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressBlock;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayoutMapping;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressSpaceHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IAddressRangeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.JConfigAction;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.IAttributeWatcher;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.BaseVariableElementEditorTab;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addGroupTitle;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.clearComponentsFrom;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.endLayout;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridCoords;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridLine;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridRelativeX;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setupGridBag;

/**
 * Created by ramon on 23.03.2016.
 */
public class AddressSpacePane extends BaseVariableElementEditorTab<ISubsystemConfiguration, AddressSpacePane.SpaceUi, AddressSpaceHandler> implements IConfigurationDocument.IConfigurationHierarchyListener {

	private final GridBagConstraints constraints;
	private final int firstOptionalComponent;
	private final int firstOptionalY;
	private final JCheckBox autoLayoutCheck;
	private final JButton save;
	private AddressLayout addressLayout;

	public AddressSpacePane() {
		constraints = setupGridBag(this, new int[]{80, 80, 30, 80, 100, 40, 80, 80, 0}, new double[]{0, 0, 0, 0, 0, 0, 0, 0, 1});
		setGridLine(constraints, 0);

		autoLayoutCheck = new JCheckBox(new AutoManualAction());
		constraints.gridwidth = 2;
		add(autoLayoutCheck, constraints);
		setGridRelativeX(constraints);
		save = new JButton(new SaveLayoutAction());
		add(save, constraints);
		constraints.gridwidth = 1;

		setGridLine(constraints, 1);
		add(Box.createVerticalStrut(20), constraints);

		setGridLine(constraints, 2);
		addGroupTitle(this, constraints, "Layout", 7);

		setGridLine(constraints, 3);

		add(new JLabel("Address Space"), constraints);
		setGridRelativeX(constraints);
		add(new JLabel("Address Range"), constraints);
		add(new JLabel("Spec"), constraints);
		add(new JLabel("Range Start"), constraints);
		add(new JLabel("Source Module"), constraints);
		add(new JLabel("Type"), constraints);
		add(new JLabel("Start"), constraints);
		add(new JLabel("Length"), constraints);

		firstOptionalComponent = getComponentCount();
		firstOptionalY = 4;
	}

	public static String toHex(long value) {
		return "0x" + Long.toHexString(value);
	}

	private boolean isEditable() {
		return !autoLayoutCheck.isSelected();
	}

	@Override
	protected void detach(ISubsystemConfiguration oldElement) {
		if (oldElement.getDocument() != null) {
			oldElement.getDocument().removeHierarchyListener(this);
		}
	}

	@Override
	protected void attach(ISubsystemConfiguration newElement) {
		newElement.getDocument().addHierarchyListener(this);
	}

	@Override
	protected List<AddressSpaceHandler> updateHandlers(List<AddressSpaceHandler> oldHandlers, ISubsystemConfiguration newElement) {
		addressLayout = AddressLayout.getCurrent(newElement);
		boolean hasMappings = newElement.hasStaticAddressSpaceMappings();
		autoLayoutCheck.setSelected(!hasMappings);
		if (hasMappings) {
			addressLayout.applyMappings(newElement.getStaticAddressLayoutMapping());
		}
		return addressLayout.getSpaceHandlers();
	}

	@Override
	protected List<SpaceUi> updateStructure(ISubsystemConfiguration oldElement, List<AddressSpaceHandler> oldHandlers, ISubsystemConfiguration newElement, List<AddressSpaceHandler> newHandlers) {
		int line = firstOptionalY;
		clearComponentsFrom(this, firstOptionalComponent);

		List<SpaceUi> spaces = new ArrayList<>();

		for (AddressSpaceHandler handler : newHandlers) {
			SpaceUi space = new SpaceUi(handler);
			spaces.add(space);
			setGridLine(constraints, line);
			add(space.nameLabel, constraints);
			setGridRelativeX(constraints);

			for (RangeUi range : space.rangeUis) {
				setGridCoords(constraints, line, 1);
				add(range.nameLabel, constraints);
				setGridRelativeX(constraints);
				add(range.specLabel, constraints);
				add(range.startField, constraints);

				for (BlockUi block : range.blockUis) {
					setGridCoords(constraints, line, 4);
					add(block.sourceLabel, constraints);
					setGridRelativeX(constraints);
					add(block.typeLabel, constraints);
					add(block.startField, constraints);
					add(block.lengthLabel, constraints);
					line++;
				}
				//				add(range.lengthLabel, constraints);
				line++;
			}
		}

		endLayout(this, constraints);

		return spaces;
	}

	@Override
	protected boolean requiresStructuralChange(ISubsystemConfiguration oldElement, List<AddressSpaceHandler> oldHandlers, ISubsystemConfiguration newElement, List<AddressSpaceHandler> newHandlers) {
		return true;
	}

	@Override
	public void onModuleAdded(IModuleConfiguration module) {
		forceUpdateHandlers();
		//		updateAllFields();
	}

	@Override
	public void onModuleRemoved(IConfigurationElement<?, ?, ?> parent, IModuleConfiguration module) {
		forceUpdateHandlers();
		//		updateAllFields();
	}

	@Override
	public void onElementNameChanged(IConfigurationElement<?, ?, ?> element) {

	}


	private class AutoManualAction extends JConfigAction {

		public AutoManualAction() {
			super("Auto Layout", "Uncheck to have control over the address layout", null);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			boolean editable = isEditable();
			save.setEnabled(editable);
			if (editable) {
				updateAllFields();
			} else {
				element.setStaticAddressLayoutMapping(null);
				forceUpdateHandlers();
			}
		}
	}


	private class SaveLayoutAction extends JConfigAction {

		public SaveLayoutAction() {
			super("Save", "Save the current static layout", "16/save.png");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			AddressLayoutMapping mappings = addressLayout.extractStaticMappings();
			element.setStaticAddressLayoutMapping(mappings);
		}
	}


	public class SpaceUi implements IAttributeWatcher<AddressSpaceHandler> {

		private final JLabel nameLabel = new JLabel();
		private final List<RangeUi> rangeUis;
		private AddressSpaceHandler handler;
		private boolean valid;

		public SpaceUi(AddressSpaceHandler handler) {
			rangeUis = handler.getAddressBlocks().stream().map(RangeUi::new).collect(Collectors.toList());
		}

		@Override
		public void setHandler(AddressSpaceHandler handler) {
			this.handler = handler;
			for (int i = 0; i < handler.getAddressBlocks().size(); i++) {
				rangeUis.get(i).setHandler(handler.getAddressBlocks().get(i));
			}
			updateSelf();
		}

		private void updateSelf() {
			if (handler != null) {
				nameLabel.setText(handler.getName());
				showValid();
			}
		}

		@Override
		public void update() {
			updateSelf();
			rangeUis.forEach(RangeUi::update);
		}

		private void showValid() {
			valid = handler.isValid();
			if (isEditable() && !valid) {
				nameLabel.setForeground(Color.RED);
			} else {
				nameLabel.setForeground((Color) UIManager.get("TextField.foreground"));
			}
		}


	}


	private class RangeUi extends FieldListener implements IAttributeWatcher<IAddressRangeHandler> {

		private final List<BlockUi> blockUis;
		private JLabel nameLabel = new JLabel();
		private JTextField startField = new JTextField();
		private JLabel specLabel = new JLabel();
		private IAddressRangeHandler handler;
		private boolean valid;

		public RangeUi(IAddressRangeHandler handler) {
			blockUis = handler.getAddressBlocks().stream().map(h -> new BlockUi()).collect(Collectors.toList());
			//			startField.getDocument().addDocumentListener(this);
			startField.addFocusListener(this);
			startField.addActionListener(this);
		}

		private void updateSelf() {
			if (handler != null) {
				nameLabel.setText(handler.getName());
				startField.setText(toHex(handler.getActualStart()));
				specLabel.setText(handler.getSpecFlags());
				boolean editable = isEditable();
				startField.setEditable(editable);
				startField.setEnabled(editable);
				showValid();
			}
		}

		@Override
		public void setHandler(IAddressRangeHandler handler) {
			this.handler = handler;
			for (int i = 0; i < handler.getAddressBlocks().size(); i++) {
				blockUis.get(i).setHandler(handler.getAddressBlocks().get(i));
			}
			updateSelf();
		}

		private void showValid() {
			valid = handler.isValid();
			Color fg = (isEditable() && !valid)? Color.RED : (Color) UIManager.get("TextField.foreground");

			nameLabel.setForeground(fg);
			startField.setForeground(fg);
		}

		@Override
		public void update() {
			updateSelf();
			blockUis.forEach(BlockUi::update);
		}

		@Override
		public String getValue() {
			return startField.getText();
		}

		@Override
		protected void setValue(long value) {
			if (handler.getActualStart() != value) {
				handler.setActualStart(value);
				updateAllFields();
			}
		}



		@Override
		protected boolean isFieldEditable() {
			return startField.isEditable();
		}


	}


	private class BlockUi extends FieldListener implements IAttributeWatcher<AddressBlock> {

		private JLabel sourceLabel = new JLabel();
		private JTextField startField = new JTextField();
		private JLabel lengthLabel = new JLabel();
		private JLabel typeLabel = new JLabel();
		private AddressBlock handler;
		private boolean valid;

		public BlockUi() {
			startField.addFocusListener(this);
			startField.addActionListener(this);
		}

		private void showValid() {
			valid = handler.isValid();
			Color fg = (isEditable() && !valid)? Color.RED : (Color) UIManager.get("TextField.foreground");
			sourceLabel.setForeground(fg);
			typeLabel.setForeground(fg);
		}

		@Override
		public void setHandler(AddressBlock handler) {
			this.handler = handler;
			update();
		}

		@Override
		public void update() {
			if (handler != null) {
				sourceLabel.setText(handler.getSourceModule().getName());
				lengthLabel.setText(toHex(handler.getLength()));
				typeLabel.setText(handler.getBlockType().shortName());
				startField.setText(toHex(handler.getActualStart()));
				boolean editable = isEditable() && !handler.hasFixedStart();
				startField.setEditable(editable);
				startField.setEnabled(editable);
				showValid();
			}
		}



		@Override
		public String getValue() {
			return startField.getText();
		}

		@Override
		protected boolean isFieldEditable() {
			return startField.isEditable();
		}

		@Override
		protected void setValue(long value) {
			if (handler.getActualStart() != value) {
				handler.setActualStart(value);
				updateAllFields();
			}
		}
	}
}


abstract class FieldListener implements FocusListener, ActionListener, DocumentListener {

	@Override
	public void focusGained(FocusEvent e) {

	}	@Override
	public void actionPerformed(ActionEvent e) {
		setValue();
	}

	@Override
	public void focusLost(FocusEvent e) {
		setValue();
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		setValue();
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		setValue();
	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		setValue();
	}

	public abstract String getValue();

	protected abstract void setValue(long value);

	protected void setValue() {
		if (!isFieldEditable()) {
			return;
		}
		try {
			long val = Long.decode(getValue());
			setValue(val);
		} catch (Exception e) {

		}
	}

	protected abstract boolean isFieldEditable();


}
