/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorGroupDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ResolvedConnector;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IConnectionContainer;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBusHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPartialConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.connections.AddConnectionAction;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.connections.DeleteConnectionAction;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.ComponentUtil;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.chrome.DirectionLabel;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.model.AvailablePortsModel;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.BaseElementEditorTab;

import com.jidesoft.swing.AutoCompletion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.Marker;
import org.slf4j.MarkerFactory;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Objects;
import java.util.stream.Stream;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addAttributeName;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addGroupTitle;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.endLayout;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridLine;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridRelativeX;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setupGridBag;

public abstract class ConnectionPane<T extends IConfigurationElement<PD, PG, IPH> & IConnectionContainer<PG, PD, IPH>, PG extends IConnectorGroupDescription<PD>, PD extends IConnectorDescription, PH extends IPortHandler<?>, IPH extends IConnectorHandler<PH, PD>> extends BaseElementEditorTab<T> implements IConfigurationElement.IElementChangeListener {

	private static final Marker connectionDetails = MarkerFactory.getMarker("ConnectionUiDetails");
	private static final Logger logger = LoggerFactory.getLogger(ConnectionPane.class);
	protected final IDocumentHandler documentHandler;
	private final LinkedList<GroupUi<PH, PD>> connectorGroups = new LinkedList<>();
	private GridBagConstraints constraints;
	private GroupUi<IBusHandler, Bus> busGroup;
	private boolean refreshInProgress = false;


	protected ConnectionPane(IDocumentHandler documentHandler) {
		this.documentHandler = documentHandler;
		constraints = setupGridBag(this, new int[]{150, 50, 20, 200, 0, 0, 0, 0}, new double[]{0, 0, 0, 0.7, 0, 0, 0, 0.3});
	}

	@Override
	protected void detach(T oldElement) {
		oldElement.removeElementListener(this);
	}

	@Override
	protected void attach(T newElement) {
		newElement.addElementListener(this);
	}

	@Override
	protected void setHandlers(T element) {
		createPortUiFromScratch(element);
		readdPortUi();
	}

	@Override
	protected void clearHandlers() {
		// TODO
	}

	private void createPortUiFromScratch(T element) {
		logger.debug("Creating new PortConnection UI");
		connectorGroups.clear();

		GroupUi<IBusHandler, Bus> busGroupUi = new GroupUi<>("Buses");
		for (Bus bus : element.getBusDescriptions()) {
			IBusHandler busHandler = element.getBusHandler(bus);
			BusUi connectorUi = new BusUi(busHandler);
			connectorUi.ensureConnections();

			busGroupUi.addPort(connectorUi);
		}

		busGroup = busGroupUi.ports.size() > 0? busGroupUi : null;

		for (PG portGroup : element.getPortGroupDescriptions()) {
			//			logger.trace(connectionDetails, "Adding PortGroup {}", portGroup);

			GroupUi<PH, PD> groupUi = createGroupUi(portGroup);

			for (PD port : portGroup.getPorts()) {
				//				logger.trace(connectionDetails, "Adding Port {}", port);

				IPH portHandler = element.getPortHandler(port);

				ConnectorUi<IPH, PH, PD> portUi = createConnectorUi(portHandler);//TODO add handler -> ui to a Map.

				portUi.ensureConnections();

				groupUi.addPort(portUi);
			}

			if (!groupUi.ports.isEmpty()) {
				connectorGroups.add(groupUi);
			}
		}
	}

	protected abstract GroupUi<PH, PD> createGroupUi(PG portGroup);

	protected abstract ConnectorUi<IPH, PH, PD> createConnectorUi(IPH connectorHandler);

	void readdPortUi() {
		logger.debug("Re-Adding PortConnection Components to Container");
		this.removeAll();

		int line = 0;

		if (busGroup != null) {
			line = busGroup.addUi(line);
		}

		for (GroupUi<PH, PD> groupUi : connectorGroups) {
			line = groupUi.addUi(line);
		}
		endLayout(this, constraints);
		revalidate();
	}

	@Override
	public void onConnectorChanged(IConfigurationElement<?, ?, ?> element, IConnectorHandler<?, ?> connector, ConnectorChangeType type, IConnection<?> connection) {
		if (type == ConnectorChangeType.Properties) {
			streamConnectorUis().forEach(ConnectorUi::update);
		} else if (type == ConnectorChangeType.Added) {
			ConnectorUi<?, ? extends IConnectorHandler<?, ?>, ? extends IConnectorDescription> ui = streamConnectorUis().filter(cUi -> cUi.isHandling(connector)).findAny().get();
			ui.onConnectionAdded(connection);
		} else if (type == ConnectorChangeType.Removed) {
			streamConnectorUis().filter(cUi -> cUi.isHandling(connector)).findAny().get().onConnectionRemoved(connection);
		}
		//TODO handle remove / add, find appropriate UI wrapper => update that. Our events use the proper chain, and contain much more details than the previous onConnectionChange()
	}

	@Override
	public void onElementNameChanged(IConfigurationElement<?, ?, ?> element) {

	}

	@Override
	public void onAttributeChanged(IConfigurationElement<?, ?, ?> element, String attribute) {

	}

	@Override
	public void onElementDeclarationChanged(IConfigurationElement<?, ?, ?> element) {
		setHandlers(this.element);
	}

	private Stream<? extends ConnectorUi<?, ? extends IConnectorHandler<?, ?>, ? extends IConnectorDescription>> streamConnectorUis() {
		//We have explicit null checks here to be able to tell what was null in case of crashes
		Objects.requireNonNull(connectorGroups);
		Stream<GroupUi<IBusHandler, Bus>> busStream = (busGroup != null ? Stream.of(busGroup) : Stream.empty());
		return Stream.concat(connectorGroups.stream(), busStream).flatMap(g -> g.ports.stream());
	}

	@Deprecated
	private synchronized void requestUiRecreate() { //TODO needed?, as long as we stay on UI thread and are not recursive (!), should not be needed
		if (!refreshInProgress) {
			refreshInProgress = true;
			SwingUtilities.invokeLater(() -> {
				synchronized (ConnectionPane.this) {
					refreshInProgress = false;
				}
				readdPortUi();
			});
		}
	}


	protected class GroupUi<ICH extends IConnectorHandler<?, ?>, P extends IConnectorDescription> {

		public final String groupTitle;
		public final LinkedList<ConnectorUi<?, ICH, P>> ports = new LinkedList<>();

		public GroupUi(String groupTitle) {
			this.groupTitle = groupTitle;
		}

		private void addPort(ConnectorUi<?, ICH, P> connectorUi) {
			ports.add(connectorUi);
		}

		protected int addUi(int line) {
			setGridLine(constraints, line++);

			Container panel = ConnectionPane.this;

			addGroupTitle(panel, constraints, groupTitle, 7);

			for (ConnectorUi<?, ICH, P> connectorUi : ports) {
				line = connectorUi.addUi(line);
				connectorUi.update();
			}

			return line;
		}
	}


	public class PortUiConnection implements ComponentUtil.IPortSelectionChangedListener {

		public JComboBox<ResolvedConnector> connectionComboBox;
		public JButton deleteButton;
		private final ConnectorUi connectorUi;
		private IConnection connection;

		public PortUiConnection(ConnectorUi connectorUi) {
			this.connectorUi = connectorUi;
		}

		@Override
		public String toString() {
			return "PortUiConnection{" +
				"connection=" + connection +
				'}';
		}

		@Override
		public void onPortSelectionChanged() {
			connectorUi.onPortSelectionChanged(this);
		}
	}


	protected class BusUi extends ConnectorUi<IBusHandler, IBusHandler, Bus> {

		public BusUi(IBusHandler portHandler) {
			super(portHandler, portHandler.getConnectorDescription());
		}

		@Override
		protected String getPortDirectionLabel() {
			return DirectionLabel.getBusDirectionLabel(portHandler);
		}

		public String getPortTitle() {
			return port.getName() + " (" + portHandler.getBusType() + ")";
		}

		@Override
		protected String getSizeText() {
			return "";
		}

		@Override
		protected AvailablePortsModel<?> getAvailablePortsModel() {
			return AvailablePortsModel.getBusModel(
				documentHandler.getAvailablePortsHandler(), element, portHandler);
		}

		@Override
		protected IConnection<? extends IBusHandler> createFullConnection(IBusHandler portHandler, IConnectorHandler<?, ?> otherHandler) {
			return documentHandler.getDocument().createBusConnection(portHandler, ((IBusHandler) otherHandler));
		}

		@Override
		protected void doPartialConnection(IConnectorHandler<?, ?> otherHandler) {
			throw new UnsupportedOperationException("No partial bus connections");
		}

		@Override
		protected void addCustomConnectorUi(GridBagConstraints constraints) {

		}
	}


	protected abstract class ConnectorUi<IMPL extends IConnectorHandler<ICH, P>, ICH extends IConnectorHandler<?, ?>, P extends IConnectorDescription> implements DeleteConnectionAction.IDeleteActionListener, AddConnectionAction.AddActionListener {

		public LinkedList<PortUiConnection> connections = new LinkedList<>();
		public JLabel sizeLabel;
		public JLabel directionLabel;
		public JButton addButton;
		protected final IMPL portHandler;
		protected final P port;
		private int keepManual;
		private boolean expectingChange;
		private boolean updating;

		ConnectorUi(IMPL portHandler, P port) {
			this.portHandler = portHandler;
			this.port = port;
			sizeLabel = new JLabel(getSizeText());
			directionLabel = new JLabel(getPortDirectionLabel());
			addButton = new JButton(new AddConnectionAction(this));
			keepManual = portHandler.isConnected()? 0 : 1; // always have 1 line to begin with. either connected or empty manual
		}

		protected abstract String getPortDirectionLabel();

		public abstract String getPortTitle();

		protected abstract String getSizeText();

		private void increaseConnectionUi() {
			PortUiConnection current = new PortUiConnection(this);
			connections.add(current);


			AvailablePortsModel<?> model = getAvailablePortsModel();

			current.connectionComboBox = ComponentUtil.createConnectorComboBox(model, current);

			current.deleteButton = new JButton(new DeleteConnectionAction(this, current));
		}

		protected abstract AvailablePortsModel<?> getAvailablePortsModel();

		@Override
		public void onDelete(ConnectionPane<?, ?, ?, ?, ?>.PortUiConnection connection) {
			if (connection != null && connection.connection != null) {
				connection.connection.disconnect();
			} else {
				keepManual = Math.max(0, keepManual - 1);
				connections.remove(connection);
				readdPortUi();
			}
		}

		private void showConnectionIncompatible() {
			SwingUtilities.invokeLater(() -> JOptionPane.showMessageDialog(ConnectionPane.this, "Ports cannot be connected.", "Connection impossible", JOptionPane.ERROR_MESSAGE));
		}

		private void showConnectionFull() {
			SwingUtilities.invokeLater(() -> JOptionPane.showMessageDialog(ConnectionPane.this, "Port '" + portHandler.getAbsoluteConnectorName() + "' is already full. Remove connections manually to make room and retry!", "Connection impossible", JOptionPane.WARNING_MESSAGE));
		}

		private void confirmReplacingConnectionAndReplace(IConnectorHandler<?, ?> otherHandler) {
			SwingUtilities.invokeLater(() -> {
				IConnectorHandler third = otherHandler.streamConnections().findAny().get().getOppositeConnector(otherHandler);
				int userOpt = JOptionPane.showConfirmDialog(ConnectionPane.this, "Port '" + otherHandler.getAbsoluteConnectorName() + "' is already connected. Replace existing connection to '" + third.getAbsoluteConnectorName() + "'?", "Replace Connection?", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				if (userOpt == JOptionPane.YES_OPTION) {
					otherHandler.disconnectAll();
					createFullConnection(portHandler, otherHandler).connect();
					onConnectionsChanged();//TODO handled already?
				}
			});
		}

		void onPortSelectionChanged(PortUiConnection connection) {
			if (updating) { // ignore changes that we caused internally. (Updating UI)
				return;
			}

			ResolvedConnector selectedItem = (ResolvedConnector) connection.connectionComboBox.getSelectedItem();

			if (connection.connection == null && selectedItem == null) {
				return;
			}

			if (selectedItem.isConnectionHintMatch(connection.connection)) {//JComboBox always fires events on setSelectedElement, even if it knows there was no actual change
				return;
			}

			logger.debug("Port {}@{}: selection changed to {}", port.getName(), connections.indexOf(connection), selectedItem);

			try {
				expectingChange = true; //prevent reacting to each onConnectionChange that we might cause, we will update all connections when we know we are done

				if (connection.connection != null) {
					keepManual++;
					connection.connection.disconnect();
				}
				if (selectedItem != null) {
					IConnectorHandler<?, ?> otherHandler = selectedItem.getConnectorHandler();

					if (!portHandler.isConnectibleTo(otherHandler)) {
						if (!IPartialConnectorHandler.isPartiallyConnectible(portHandler, otherHandler)) {
							connection.connectionComboBox.hidePopup();
							showConnectionIncompatible();
							return;
						}
						connection.connectionComboBox.hidePopup();
						doPartialConnection(otherHandler); // either an exception was thrown or it was already handled
						return; //rely on finally{} updating ui
					}
					if (!otherHandler.isAdditionalConnectionAllowed()) {
						if (otherHandler.getMaxAllowedConnections() == 1) {
							connection.connectionComboBox.hidePopup();
							confirmReplacingConnectionAndReplace(otherHandler);
							return;
						} else {
							connection.connectionComboBox.hidePopup();
							showConnectionFull();
							return;
						}
					}
					keepManual--;
					createFullConnection(portHandler, otherHandler).connect();
				}
			} finally {
				expectingChange = false;// updated here once. either restore old state or update
				onConnectionsChanged();
			}
		}

		protected abstract IConnection<? extends ICH> createFullConnection(IMPL portHandler, IConnectorHandler<?, ?> otherHandler);

		protected abstract void doPartialConnection(IConnectorHandler<?, ?> otherHandler);

		public void onConnectionAdded(IConnection<?> connection) {
			onConnectionsChanged();
		}

		public void onConnectionRemoved(IConnection<?> connection) {
			onConnectionsChanged();
		}

		private void onConnectionsChanged() {
			if (expectingChange) {
				return;//we asked model to change, so ignore model events, so we can do it all at once
			}

			if (ensureConnections()) {
				readdPortUi();
			} else {
				update();
			}
		}

		@Override
		public void onAdd() {
			keepManual++;
			increaseConnectionUi();

			logger.debug("Added PortConnection UI to {}", this);
			readdPortUi();
		}

		boolean ensureConnections() {
			boolean uiCountChange = false;

			int targetCount = Math.max(portHandler.getConnectionCount() + keepManual, 1);

			while (connections.size() > targetCount) {
				connections.removeLast();
				uiCountChange = true;
			}

			while (connections.size() < targetCount) {
				increaseConnectionUi();
				uiCountChange = true;
			}

			return uiCountChange;
		}

		public void update() {
			updating = true;

			Iterator<PortUiConnection> iterator = connections.iterator();
			portHandler.streamConnections().forEach(connection -> {
				PortUiConnection conUi = iterator.next();
				boolean partial = connection.isPartial();
				conUi.connectionComboBox.setEnabled(!partial);
				conUi.connectionComboBox.setEditable(partial);

				if (partial) {
					AutoCompletion autoCompletion = AutoCompletion.getAutoCompletion((JComponent) conUi.connectionComboBox.getEditor().getEditorComponent());
					if (autoCompletion!=null) {
						autoCompletion.uninstallListeners();
					}
				}

				conUi.connection = connection;
				conUi.connectionComboBox.setSelectedItem(getResolvedPort(portHandler, connection));
				conUi.deleteButton.setEnabled(true);
			});

			boolean partial = portHandler.getPartialConnectorHandler().map(IPartialConnectorHandler::isPartiallyConnected).orElse(false);

			boolean hadAny = portHandler.getConnectionCount() > 0;
			boolean deletable = hadAny || connections.size() > 1;
			iterator.forEachRemaining(conUi -> {
				conUi.connection = null;
				conUi.connectionComboBox.setSelectedItem(null);
				conUi.connectionComboBox.setEnabled(!partial);
				conUi.deleteButton.setEnabled(deletable);
			});

			addButton.setEnabled(portHandler.isMultipleConnectionsAllowed() && !partial);

			repaint();
			updating = false;
		}

		private ResolvedConnector getResolvedPort(IConnectorHandler<ICH, P> portHandler, IConnection<? extends ICH> connection) {
			ICH oppositePort = connection.getOppositeConnector(portHandler);
			ResolvedConnector resolvedConnector;
			if (connection.isPartial()) {
				resolvedConnector = new ResolvedConnector(oppositePort, portHandler.getSourceElement(), connection);
			} else {
				resolvedConnector = new ResolvedConnector(oppositePort, portHandler.getSourceElement());
			}
			resolvedConnector.setConnectionHint(connection);
			return resolvedConnector;
		}

		protected int addUi(int line) {
			Container panel = ConnectionPane.this;
			int i = 0;

			for (PortUiConnection conUi : connections) {
				setGridLine(constraints, line++);// \n

				if (i == 0) { // only output name and size on first connection
					addAttributeName(panel, constraints, getPortTitle());//col 0
					setGridRelativeX(constraints);

					add(sizeLabel, constraints);//col 1

					add(directionLabel, constraints);// col2
				} else {
					constraints.gridx = 3;
				}

				add(conUi.connectionComboBox, constraints);//col3

				setGridRelativeX(constraints);

				add(conUi.deleteButton, constraints);//col4

				if (i == 0) {
					add(addButton, constraints);//col5
					addCustomConnectorUi(constraints);
				}

				++i;
			}

			return line;
		}

		protected abstract void addCustomConnectorUi(GridBagConstraints constraints);

		public boolean isHandling(IConnectorHandler<?, ?> connector) {
			return connector == portHandler;
		}
	}

}
