/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IMemoryModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IFirmwareAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IMemoryRegionHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.JConfigAction;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.IAttributeWatcher;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.BaseElementEditorTab;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addAttributeName;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addGroupTitle;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.endLayout;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridLine;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridRelativeX;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setupGridBag;

public class FirmwarePane extends BaseElementEditorTab<IMemoryModuleConfiguration> implements ActionListener, IMemoryModuleConfiguration.IMemoryChangeListener {

	private static final Logger logger = LoggerFactory.getLogger(FirmwarePane.class);
	private final GridBagConstraints constraints;
	//	private final int firstOptionalComponent;
	//	private final int firstOptionlY;

	private final JComboBox<String> firmwareLocationField;
	private IFirmwareAttributeHandler firmwareLocationHandler;


	public FirmwarePane() {
		constraints = setupGridBag(this, new int[]{150, 300, 0, 0, 0}, new double[]{0, 0, 0, 0, 1});

		setGridLine(constraints, 0);
		addGroupTitle(this, constraints, "Firmware", 4);

		setGridLine(constraints, 1);
		addAttributeName(this, constraints, "Location");
		firmwareLocationField = new JComboBox<>();//For now only sub folders of project dir supported, model allows other locations, and sub firmwares for multicore systems
		firmwareLocationField.setEditable(true);
		firmwareLocationField.addActionListener(this);
		//		AutoCompleteDecorator.decorate(firmwareLocationField);
		setGridRelativeX(constraints);
		add(firmwareLocationField, constraints);

		JButton button = new JButton(new RescanFirmwaresAction());
		add(button, constraints);

		//		setGridLine(constraints, 2);
		//		addGroupTitle(this, constraints, "Memory sections", 3);
		//
		//		setGridLine(constraints, 3);
		//		add(new JLabel("Segment"), constraints);
		//		setGridRelativeX(constraints);
		//		add(new JLabel("Address"), constraints);
		//		add(new JLabel("Size"), constraints);
		//
		//		firstOptionalComponent = getComponentCount();
		//		firstOptionlY = 4;
		setGridLine(constraints, 2);
		endLayout(this, constraints);
	}

	public static MemoryAddress[] getPossibleAddresses(IMemoryRegionHandler handler) {
		int adrCount = 4;//TODO
		int adrStep = 4;
		MemoryAddress[] addresses = new MemoryAddress[adrCount];
		for (int i = 0; i < adrCount; i++) {
			addresses[i] = new MemoryAddress(i * adrStep);
		}
		return addresses;
	}

	//	@Override
	//	protected List<? extends IMemoryRegionHandler> updateHandlers(List<? extends IMemoryRegionHandler> oldHandlers, IMemoryModuleConfiguration newElement) {
	//		List<IMemoryRegionHandler> handlers = new ArrayList<>();
	//
	//		for (IMemoryRegionHandler.MemoryRegionType type : IMemoryRegionHandler.MemoryRegionType.values()) {
	//			handlers.add(newElement.getMemoryRegionHandler(type));
	//		}
	//
	//		return handlers;
	//	}

	//	@Override
	//	protected List<MemoryRegionWatcher> updateStructure(IMemoryModuleConfiguration oldElement, List<? extends IMemoryRegionHandler> oldHandlers, IMemoryModuleConfiguration newElement, List<? extends IMemoryRegionHandler> newHandlers) {
	//		clearComponentsFrom(this, firstOptionalComponent);
	//		List<MemoryRegionWatcher> fields = new ArrayList<>();
	//
	//		int line = firstOptionlY;
	//
	//		for (IMemoryRegionHandler.MemoryRegionType type : IMemoryRegionHandler.MemoryRegionType.values()) {
	//			setGridLine(constraints, line++);
	//
	//			JLabel name = new JLabel();
	//			setGridRelativeX(constraints);
	//
	//			JComboBox<MemoryAddress> startAddress = new JComboBox<>();
	//			add(startAddress, constraints);
	//
	//			JTextField size = new JTextField();
	//			add(size, constraints);
	//
	//			fields.add(new MemoryRegionWatcher(name, startAddress, size));
	//		}
	//
	//		endLayout(this, constraints);
	//		return fields;
	//	}

	//	@Override
	//	protected boolean requiresStructuralChange(IMemoryModuleConfiguration oldElement, List<? extends IMemoryRegionHandler> oldHandlers, IMemoryModuleConfiguration newElement, List<? extends IMemoryRegionHandler> newHandlers) {
	//		return oldHandlers.size() != newHandlers.size();
	//	}

	private void rescanFirmwareSuggestions() {
		firmwareLocationHandler = element.getFirmwareLocationHandler();
		firmwareLocationField.removeAllItems();
		firmwareLocationField.setSelectedItem(firmwareLocationHandler.getFirmwareName());

		firmwareLocationHandler.getSuggestedFirmwares().forEach(firmwareLocationField::addItem);
	}

	@Override
	protected void detach(IMemoryModuleConfiguration oldElement) {
		oldElement.removeMemoryListener(this);
	}

	@Override
	protected void attach(IMemoryModuleConfiguration newElement) {
		newElement.addMemoryListener(this);
	}

	@Override
	protected void setHandlers(IMemoryModuleConfiguration element) {
		rescanFirmwareSuggestions();
	}

	@Override
	protected void clearHandlers() {
		firmwareLocationHandler = null;
	}

	@Override
	public void onFirmwareChanged(IMemoryModuleConfiguration element) {
		if (firmwareLocationHandler != null) {
			firmwareLocationField.setSelectedItem(firmwareLocationHandler.getFirmwareName());
		}
	}


	public static class MemoryRegionWatcher implements IAttributeWatcher<IMemoryRegionHandler>, ActionListener {

		private final JLabel name;
		private final JComboBox<MemoryAddress> startAddress;
		private final JTextField size;
		private IMemoryRegionHandler handler;

		public MemoryRegionWatcher(JLabel name, JComboBox<MemoryAddress> startAddress, JTextField size) {
			this.name = name;
			this.startAddress = startAddress;
			this.size = size;
			size.setEditable(false);
			size.setEnabled(false);
			startAddress.addActionListener(this);
		}

		@Override
		public void setHandler(IMemoryRegionHandler handler) {
			this.handler = handler;
		}

		@Override
		public void update() {
			name.setText(handler.getName());
			startAddress.setModel(new DefaultComboBoxModel<>(getPossibleAddresses(handler)));
			startAddress.setSelectedItem(handler.getCurrentAddress());
			size.setText(Integer.toHexString(handler.getSize()));
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			handler.setCurrentAddress(((MemoryAddress) startAddress.getSelectedItem()).value);
		}
	}	@Override
	public void actionPerformed(ActionEvent e) {
		firmwareLocationHandler.setFirmwareName(((String) firmwareLocationField.getSelectedItem()));
	}


	private static class MemoryAddress {

		private final long value;

		public MemoryAddress(long value) {
			this.value = value;
		}

		@Override
		public String toString() {
			return Long.toHexString(value);
		}
	}


	private class RescanFirmwaresAction extends JConfigAction {

		public RescanFirmwaresAction() {
			super(null, "Rescan firmware folders in current firmware location", "16/make.png");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			rescanFirmwareSuggestions();
		}
	}


}
