/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.AttributeTextField;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.IAttributeWatcher;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.BaseVariableElementEditorTab;

import java.awt.GridBagConstraints;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addAttributeName;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addGroupTitle;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.clearComponentsFrom;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.endLayout;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridLine;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridRelativeX;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setupGridBag;


public class GenericAttributePane extends BaseVariableElementEditorTab<IConfigurationElement<?, ?, ?>, IAttributeWatcher<IAttributeHandler>, IAttributeHandler> implements IConfigurationElement.IElementChangeListener {

	private final int firstOptionalComponent;
	private final int firstOptionlY;
	private final GridBagConstraints constraints;

	public GenericAttributePane() {
		constraints = setupGridBag(this, new int[]{150, 300, 0}, new double[]{0, 0.1, 1});

		setGridLine(constraints, 0);
		addGroupTitle(this, constraints, "Attributes", 2);

		firstOptionalComponent = getComponentCount();
		firstOptionlY = 1;
	}

	@Override
	protected void detach(IConfigurationElement<?, ?, ?> oldElement) {
		oldElement.removeElementListener(this);
	}

	@Override
	protected void attach(IConfigurationElement<?, ?, ?> newElement) {
		newElement.addElementListener(this);
	}

	@Override
	protected List<IAttributeHandler> updateHandlers(List<IAttributeHandler> oldHandlers, IConfigurationElement<?, ?, ?> newElement) {
		return newElement.getKnownAttributes().stream().map(newElement::getAttributeHandler).collect(Collectors.toList());
	}

	@Override
	protected List<IAttributeWatcher<IAttributeHandler>> updateStructure(IConfigurationElement<?, ?, ?> oldElement, List<IAttributeHandler> oldHandlers, IConfigurationElement<?, ?, ?> newElement, List<IAttributeHandler> newHandlers) {
		clearComponentsFrom(this, firstOptionalComponent);
		List<IAttributeWatcher<IAttributeHandler>> fields = new ArrayList<>();

		int line = firstOptionlY;

		for (String attr : newElement.getKnownAttributes()) {
			setGridLine(constraints, line++);
			addAttributeName(this, constraints, attr);

			AttributeTextField field = new AttributeTextField();
			fields.add(field);

			setGridRelativeX(constraints);
			add(field, constraints);
		}

		endLayout(this, constraints);
		return fields;
	}

	@Override
	protected boolean requiresStructuralChange(IConfigurationElement<?, ?, ?> oldElement, List<IAttributeHandler> oldHandlers, IConfigurationElement<?, ?, ?> newElement, List<IAttributeHandler> newHandlers) {
		return !oldElement.getKnownAttributes().equals(newElement.getKnownAttributes());
	}

	@Override
	public void onConnectorChanged(IConfigurationElement<?, ?, ?> element, IConnectorHandler<?, ?> connector, ConnectorChangeType type, IConnection<?> connection) {

	}

	@Override
	public void onElementNameChanged(IConfigurationElement<?, ?, ?> element) {

	}

	@Override
	public void onAttributeChanged(IConfigurationElement<?, ?, ?> element, String attribute) {
		updateAllFields();
	}

	@Override
	public void onElementDeclarationChanged(IConfigurationElement<?, ?, ?> element) {

	}
}
