/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.chrome.SingleLineBorder;

import java.awt.Color;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

/**
 * Created by ramonw on 07.12.15.
 */
public class GridBagHelper {

	private static Insets titleInsets = new Insets(0, 0, 15, 0);
	private static Insets generalInsets = new Insets(0, 0, 10, 10);

	public static void clearComponentsFrom(Container container, int firstOptionalComponent) {
		for (int i = container.getComponentCount() - 1; i >= firstOptionalComponent; i--) {
			container.remove(i);
		}
	}

	public static void setGridLine(GridBagConstraints constraints, int line) {
		setGridCoords(constraints, line, 0);
	}

	public static void setGridCoords(GridBagConstraints constraints, int line, int x) {
		constraints.gridy = line;
		constraints.gridx = x;
	}

	public static void setGridRelativeX(GridBagConstraints constraints) {
		constraints.gridx = GridBagConstraints.RELATIVE;
	}

	public static JLabel addAttributeName(Container container, GridBagConstraints constraints, String name) {
		JLabel label = new JLabel(name);
		container.add(label, constraints);
		return label;
	}

	public static JLabel addAttributeDescription(Container container, GridBagConstraints constraints, String description) {
		JLabel label = new JLabel(description);
		label.setEnabled(false);// grey out
		container.add(label, constraints);
		return label;
	}

	public static JLabel addGroupTitle(Container container, GridBagConstraints constraints, String title, int gridwidth) {
		JLabel titleLabel = new JLabel(title);
		titleLabel.setBorder(new SingleLineBorder(SwingConstants.BOTTOM));
		int oldWidth = constraints.gridwidth;
		Insets oldInsets = constraints.insets;
		constraints.gridwidth = gridwidth;
		constraints.insets = titleInsets;
		container.add(titleLabel, constraints);
		constraints.gridwidth = oldWidth;
		constraints.insets = oldInsets;
		return titleLabel;
	}

	public static JLabel addWarning(Container container, GridBagConstraints constraints, String title, int gridwidth) {
		JLabel titleLabel = new JLabel(title, SwingConstants.CENTER);
		titleLabel.setOpaque(true);
		titleLabel.setBackground(Color.ORANGE);
		int oldWidth = constraints.gridwidth;
		Insets oldInsets = constraints.insets;
		constraints.gridwidth = gridwidth;
		constraints.insets = titleInsets;
		container.add(titleLabel, constraints);
		constraints.gridwidth = oldWidth;
		constraints.insets = oldInsets;
		return titleLabel;
	}

	public static GridBagConstraints setupGridBag(Container container, int[] colMinWidths, double[] colWeights) {
		GridBagLayout layout = new GridBagLayout();

		layout.columnWidths = colMinWidths;
		layout.columnWeights = colWeights;

		container.setLayout(layout);

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.BASELINE_LEADING;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = generalInsets;

		return constraints;
	}

	public static void endLayout(Container container, GridBagConstraints constraints) {
		double oldWeight = constraints.weighty;
		constraints.weighty = 1;
		container.add(Box.createGlue(), constraints);
		constraints.weighty = oldWeight;
	}
}
