/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPinHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.chrome.DirectionLabel;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.model.AvailablePortsModel;

import java.awt.GridBagConstraints;

/**
 * Created by Ramon on 12.02.2016.
 */
public class IoPinConnectionPane extends ConnectionPane<IConfigurationDocument, TargetDescription.IoPinGroup, TargetDescription.IoPin, IPortHandler<?>, IPinHandler> {

	public IoPinConnectionPane(IDocumentHandler documentHandler) {
		super(documentHandler);
	}

	@Override
	protected GroupUi<IPortHandler<?>, TargetDescription.IoPin> createGroupUi(TargetDescription.IoPinGroup portGroup) {
		return new GroupUi<>(portGroup.getName());
	}

	@Override
	protected PinUi createConnectorUi(IPinHandler portHandler) {
		return new PinUi(portHandler);
	}

	public class PinUi extends ConnectorUi<IPinHandler, IPortHandler<?>, TargetDescription.IoPin> {

		public PinUi(IPinHandler pinHandler) {
			super(pinHandler, pinHandler.getConnectorDescription());
		}

		@Override
		protected String getPortDirectionLabel() {
			return DirectionLabel.getPinDirectionLabel(portHandler);
		}

		public String getPortTitle() {
			return port.getName();
		}

		@Override
		protected String getSizeText() {
			return "";
		}

		@Override
		protected AvailablePortsModel<?> getAvailablePortsModel() {
			return AvailablePortsModel.getPortModel(documentHandler.getAvailablePortsHandler(), element, portHandler);
		}

		@Override
		protected IConnection<? extends IPortHandler<?>> createFullConnection(IPinHandler portHandler, IConnectorHandler<?, ?> otherHandler) {
			return documentHandler.getDocument().createPortConnection(portHandler, ((IPortHandler<?>) otherHandler));
		}

		@Override
		protected void doPartialConnection(IConnectorHandler<?, ?> otherHandler) {
			throw new UnsupportedOperationException("Not partial pin connections");
		}

		@Override
		protected void addCustomConnectorUi(GridBagConstraints constraints) {

		}
	}
}
