/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.files.devices.DeviceDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPinHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.actions.JConfigAction;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.chrome.DirectionLabel;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.BaseElementEditorTab;
import de.tu_darmstadt.rs.util.UnitUtil;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addGroupTitle;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.clearComponentsFrom;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.endLayout;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridLine;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridRelativeX;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setupGridBag;

public class IoPinPane extends BaseElementEditorTab<IConfigurationDocument> implements IConfigurationElement.IElementChangeListener {

	private static final int TITLESPAN = 11;
	private static ConnectorDirection[] directions = new ConnectorDirection[]{ConnectorDirection.Input, ConnectorDirection.Output, ConnectorDirection.InOut};
	private final int firstOptionalLine;
	private final int firstOptionalComponent;
	private TargetDescription targetDescription;
	private GridBagConstraints constraints;
	private Map<String, List<IPinHandler>> groupedPins;
	private Map<String, List<PinUi>> ui = new HashMap<>();
	private boolean isTerminated = false;

	public IoPinPane() {
		//			NameDir	Loc	Inv	Dir	Std	Drv	Slw	Trm	UC	FREQ
		constraints = setupGridBag(this, new int[]{100, 30, 100, 30, 70, 70, 30, 40, 70, 50, 50, 50}, new double[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1});
		setGridLine(constraints, 0);
		constraints.gridx = 2;
		add(new JLabel("Connected"), constraints);
		setGridRelativeX(constraints);

		add(new JLabel("Inv"), constraints);
		add(new JLabel("Direction"), constraints);
		add(new JLabel("Standard"), constraints);
		add(new JLabel("Drive"), constraints);
		add(new JLabel("Slew"), constraints);
		add(new JLabel("Termination"), constraints);
		add(new JLabel("User Contraints"), constraints);
		add(new JLabel("Clock"), constraints);

		firstOptionalComponent = getComponentCount();
		firstOptionalLine = 2;
	}

	public static String[] toStringArr(List<String> list) {
		String[] res = new String[list.size() + 1];
		res[0] = null;
		for (int i = 0; i < list.size(); i++) {
			res[i + 1] = list.get(i);
		}
		return res;
	}

	@Override
	public void onConnectorChanged(IConfigurationElement<?, ?, ?> element, IConnectorHandler<?, ?> connector, ConnectorChangeType type, IConnection<?> connection) {
		onPinsChanged();
	}

	@Override
	public void onElementNameChanged(IConfigurationElement<?, ?, ?> element) {

	}

	@Override
	public void onAttributeChanged(IConfigurationElement<?, ?, ?> element, String attribute) {

	}

	@Override
	public void onElementDeclarationChanged(IConfigurationElement<?, ?, ?> element) {
		setHandlers(this.element);// force refresh
	}

	private String getConnectedLabel(IPinHandler pinHandler) {
		return pinHandler.streamConnections().map(c -> c.getOppositeConnector(pinHandler).getAbsoluteConnectorName()).collect(Collectors.joining("<br>", "<html>", "</html>"));
	}

	private void setPinHandlers() {
		for (Map.Entry<String, List<IPinHandler>> entry : groupedPins.entrySet()) {
			List<PinUi> pinUis = ui.get(entry.getKey());
			Iterator<IPinHandler> pinIt = entry.getValue().iterator();
			for (PinUi pinUi : pinUis) {
				pinUi.pinHandler = pinIt.next();
				pinUi.update();
			}
		}
	}

	@Override
	protected void detach(IConfigurationDocument oldElement) {
		oldElement.addElementListener(this);
	}

	@Override
	protected void attach(IConfigurationDocument newElement) {
		newElement.addElementListener(this);
	}

	@Override
	protected void setHandlers(IConfigurationDocument element) {
		if (recreateUiIfNecessary()) {
			setPinHandlers();
		} else {
			updateAll();
		}
	}

	@Override
	protected void clearHandlers() {
		//TODO
	}

	public void onPinsChanged() {
		if (recreateUiIfNecessary()) {
			setPinHandlers();
		} else {
			updateAll();
		}

	}

	private void updateAll() {
		ui.values().stream().flatMap(Collection::stream).forEach(PinUi::update);
	}

	private boolean recreateUiIfNecessary() {
		IConfigurationDocument.ITargetConfiguration target = element.getTarget();
		TargetDescription targetDescription = target.getTargetDescription();
		boolean necessary = !(targetDescription != null && targetDescription.equals(this.targetDescription));

		Map<String, List<IPinHandler>> newPins = new HashMap<>();

		for (TargetDescription.IoPinGroup ioPinGroup : targetDescription.getIoPinGroups()) {

			List<IPinHandler> pins = new ArrayList<>();

			for (TargetDescription.IoPin pin : ioPinGroup.getPorts()) {
				IPinHandler portHandler = element.getPortHandler(pin);
				if (portHandler.isConnected()) {
					pins.add(portHandler);
				}
			}

			if (pins.size() > 0) {
				newPins.put(ioPinGroup.getName(), pins);
			}
		}
		necessary |= !newPins.equals(this.groupedPins);

		if (necessary) {
			groupedPins = newPins;
			this.targetDescription = targetDescription;
			recreateUi();
		} else if (!isTerminated) {
			endLayout(this, constraints);
		}
		return necessary;
	}

	private void recreateUi() {
		clearComponentsFrom(this, firstOptionalComponent);
		ui.clear();
		isTerminated = false;

		String[] allowedIoStds = toStringArr(element.getTarget().getDeviceDescription().getIostandards());
		String[] allowedDrives = toStringArr(element.getTarget().getDeviceDescription().getDrives());
		String[] allowedSlews = toStringArr(element.getTarget().getDeviceDescription().getSlewRates());
		List<DeviceDescription.Termination> terminations = element.getTarget().getDeviceDescription().getTerminations();
		DeviceDescription.Termination[] allowedTerminations = terminations.toArray(new DeviceDescription.Termination[terminations.size()]);

		int line = firstOptionalLine;

		for (TargetDescription.IoPinGroup ioPinGroup : targetDescription.getIoPinGroups()) {
			List<IPinHandler> handlers = groupedPins.get(ioPinGroup.getName());

			if (handlers != null) {
				List<PinUi> group = new ArrayList<>();
				ui.put(ioPinGroup.getName(), group);

				setGridLine(constraints, line++);
				addGroupTitle(this, constraints, ioPinGroup.getName(), TITLESPAN);

				for (IPinHandler pin : handlers) {
					PinUi current = new PinUi();
					group.add(current);

					setGridLine(constraints, line++);
					current.name = new JLabel();
					add(current.name, constraints);
					setGridRelativeX(constraints);

					current.dir = new JLabel();
					add(current.dir, constraints);

					current.loc = new JLabel();
					add(current.loc, constraints);

					current.invCheckBox = new JCheckBox(new InvAction(current));
					add(current.invCheckBox, constraints);

					current.directionComboBox = new JComboBox<>(directions);
					current.directionComboBox.addActionListener(new DirectionListener(current));
					add(current.directionComboBox, constraints);

					current.iostdComboBox = new JComboBox<>(allowedIoStds);
					current.iostdComboBox.addActionListener(new IoStdListener(current));
					add(current.iostdComboBox, constraints);

					current.driveComboBox = new JComboBox<>(allowedDrives);
					current.driveComboBox.addActionListener(new DriveListener(current));
					add(current.driveComboBox, constraints);

					current.slewComboBox = new JComboBox<>(allowedSlews);
					current.slewComboBox.addActionListener(new SlewListener(current));
					add(current.slewComboBox, constraints);

					current.terminationComboBox = new JComboBox<>(allowedTerminations);
					current.terminationComboBox.addActionListener(new TerminationListener(current));
					add(current.terminationComboBox, constraints);

					current.customUserConstraints = new JTextField();
					current.customUserConstraints.addActionListener(new CustomUsrConstrainsListener(current));
					add(current.customUserConstraints, constraints);

					current.clockLabel = new JLabel();
					add(current.clockLabel, constraints);

					add(new JButton(new DeleteAction(current)), constraints);
				}
			}
		}
		endLayout(this, constraints);
		isTerminated = true;
	}


	private class PinUi {

		private IPinHandler pinHandler;
		private JCheckBox invCheckBox;
		private JLabel name;
		private JLabel loc;
		private JComboBox<ConnectorDirection> directionComboBox;
		private JComboBox<String> iostdComboBox;
		private JComboBox<String> driveComboBox;
		private JComboBox<String> slewComboBox;
		private JComboBox<DeviceDescription.Termination> terminationComboBox;
		private JTextField customUserConstraints;
		private JLabel dir;
		private JLabel clockLabel;

		public void update() {
			invCheckBox.setSelected(pinHandler.isInverted());
			name.setText(pinHandler.getPrefixedConnectorName());
			dir.setText(DirectionLabel.getPinDirectionLabel(pinHandler));
			loc.setText(getConnectedLabel(pinHandler));
			directionComboBox.setEnabled(!pinHandler.isDirectionLocked());
			directionComboBox.setSelectedItem(pinHandler.getTargetDirection());
			iostdComboBox.setSelectedItem(pinHandler.getIoStd());
			iostdComboBox.setEnabled(!pinHandler.isIoStdLocked());

			boolean inputOrFam = pinHandler.isInputOrFamiliar();
			driveComboBox.setEnabled(!inputOrFam);
			slewComboBox.setEnabled(!inputOrFam);

			driveComboBox.setSelectedItem(pinHandler.getDrive());
			slewComboBox.setSelectedItem(pinHandler.getSlewRate());
			terminationComboBox.setSelectedItem(pinHandler.getTermination());
			customUserConstraints.setText(pinHandler.getCustomUserConstraints());
			clockLabel.setText(UnitUtil.getFreqLabel(pinHandler.getPinDescription().getFreq()));
		}
	}


	private abstract class PinAction extends JConfigAction {

		protected final PinUi current;

		public PinAction(String name, String tooltip, String icon, PinUi current) {
			super(name, tooltip, icon);
			this.current = current;
		}
	}


	private class InvAction extends PinAction {

		public InvAction(PinUi current) {
			super(" ", null, null, current);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			current.pinHandler.setInverted(current.invCheckBox.isSelected());
		}
	}


	private class DeleteAction extends PinAction {

		public DeleteAction(PinUi current) {
			super("Delete", "Disconnect and Reset configuration", "16/remove.png", current);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			current.pinHandler.disconnectAll();
		}
	}


	private abstract class PinActionListener implements ActionListener {

		protected final PinUi current;

		protected PinActionListener(PinUi current) {
			this.current = current;
		}
	}


	private class DirectionListener extends PinActionListener {

		private DirectionListener(PinUi current) {
			super(current);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			ConnectorDirection dir = ((ConnectorDirection) current.directionComboBox.getSelectedItem());
			if (current.pinHandler.getTargetDirection() != dir) {
				current.pinHandler.setTargetDirection(dir);
			}
		}
	}


	private class SlewListener extends PinActionListener {

		protected SlewListener(PinUi current) {
			super(current);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			current.pinHandler.setSlewRate(((String) current.slewComboBox.getSelectedItem()));
		}
	}


	private class IoStdListener extends PinActionListener {

		protected IoStdListener(PinUi current) {
			super(current);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			String selectedItem = (String) current.iostdComboBox.getSelectedItem();
			String ioStd = current.pinHandler.getIoStd();
			if (!Objects.equals(ioStd, selectedItem)) {
				current.pinHandler.setIoStd(selectedItem);
			}
		}
	}


	private class DriveListener extends PinActionListener {

		protected DriveListener(PinUi current) {
			super(current);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			current.pinHandler.setDrive(((String) current.driveComboBox.getSelectedItem()));
		}
	}


	private class TerminationListener extends PinActionListener {

		protected TerminationListener(PinUi current) {
			super(current);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			current.pinHandler.setTermination((DeviceDescription.Termination) current.terminationComboBox.getSelectedItem());
		}
	}


	private class CustomUsrConstrainsListener extends PinActionListener {

		public CustomUsrConstrainsListener(PinUi current) {
			super(current);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			current.pinHandler.setCustomUserConstraints(current.customUserConstraints.getText());
		}
	}
}
