/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameter;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameterGroup;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IApplicationConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.WatchableComponentHolder;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.BaseElementEditorTab;

import java.awt.GridBagConstraints;
import java.util.ArrayList;
import java.util.List;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addAttributeDescription;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addAttributeName;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addGroupTitle;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.endLayout;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridLine;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridRelativeX;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setupGridBag;

/**
 * Tab that lists all Module Parameters that are allowed per assigned ModuleDescription
 *
 * @author Ramon Wirsch
 */
public class ParameterPane extends BaseElementEditorTab<IModuleConfiguration<?>> implements IModuleConfiguration.IModuleChangeListener, IConfigurationElement.IElementChangeListener {

	private final IApplicationConfiguration application;
	private GridBagConstraints constraints;
	private List<WatchableComponentHolder<? extends IAttributeHandler, ?>> fields;

	public ParameterPane(IApplicationConfiguration application) {
		this.application = application;
		constraints = setupGridBag(this, new int[]{150, 300, 0, 0}, new double[]{0, 0, 0.5, 0.5});

	}

	@Override
	protected void detach(IModuleConfiguration<?> oldElement) {
		oldElement.removeModuleListener(this);
		oldElement.addElementListener(this);
	}

	@Override
	protected void attach(IModuleConfiguration<?> newElement) {
		newElement.addModuleListener(this);
		newElement.addElementListener(this);
	}

	@Override
	protected void setHandlers(IModuleConfiguration<?> element) {
		removeAll();
		fields = new ArrayList<>();

		int line = 0;

		for (IParameterGroup group : element.getParameterGroupDescriptions()) {
			if (group.isHidden()) {
				continue;
			}

			setGridLine(constraints, line++);

			addGroupTitle(this, constraints, group.getName(), 3);

			//TODO add description

			for (IParameter parameter : group.getParameters()) {
				setGridLine(constraints, line++);
				addAttributeName(this, constraints, parameter.getName());
				setGridRelativeX(constraints);

				WatchableComponentHolder<? extends IAttributeHandler, ?> field = WatchableComponentHolder.getTypedParameterField(parameter, application.getLibrary());

				add(field.getComponent(), constraints);
				fields.add(field);

				addAttributeDescription(this, constraints, parameter.getDescription());
			}
		}

		for (WatchableComponentHolder field : fields) {
			field.attach(element);
		}

		endLayout(this, constraints);
	}

	@Override
	protected void clearHandlers() {
		for (WatchableComponentHolder field : fields) {
			field.attach(null);
		}
	}

	@Override
	public void onParameterChanged(IConfigurationElement<?, ?, ?> element, IAttributeHandler handler) {
		for (WatchableComponentHolder field : fields) {
			field.update();
		}
	}

	@Override
	public void onConnectorChanged(IConfigurationElement<?, ?, ?> element, IConnectorHandler<?, ?> connector, ConnectorChangeType type, IConnection<?> connection) {

	}

	@Override
	public void onElementNameChanged(IConfigurationElement<?, ?, ?> element) {

	}

	@Override
	public void onAttributeChanged(IConfigurationElement<?, ?, ?> element, String attribute) {

	}

	@Override
	public void onElementDeclarationChanged(IConfigurationElement<?, ?, ?> element) {
		setHandlers(this.element);
	}
}
