/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


package de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.files.devices.DeviceDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IApplicationConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IToolchainAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.AttributeTextField;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.IAttributeWatcher;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.attributes.SelectionComboBox;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.BaseElementEditorTab;

import com.jidesoft.swing.AutoCompletion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.JCheckBox;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addAttributeName;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.addGroupTitle;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.endLayout;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridLine;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setGridRelativeX;
import static de.tu_darmstadt.rs.spartanmc.jconfig.ui.panes.editorTabs.GridBagHelper.setupGridBag;

public class TargetOptionsPane extends BaseElementEditorTab<IConfigurationDocument> implements IConfigurationDocument.IDocumentChangeListener, IConfigurationElement.IElementChangeListener {

	private static final Logger logger = LoggerFactory.getLogger(TargetOptionsPane.class);

	private final IApplicationConfiguration application;
	private final AttributeTextField ucFile;
	private final AttributeTextField ucData;
	private final JTextField familyField;
	private final JTextField jtagId;
	private final SelectionComboBox<DeviceDescription> deviceComboBox;
	private final SelectionComboBox<TargetDescription> targetComboBox;
	private final SelectionComboBox<DeviceDescription.Toolchain> toolchainComboBox;
	private final int firstOptionalLine;
	private final int firstOptionalComponent;
	private GridBagConstraints constraints;
	private DeviceDescription lastDeviceDescription;
	private DeviceDescription.Toolchain lastToolchain;
	private IConfigurationDocument.ITargetConfiguration target;
	private List<ToolchainAttrWatcher> toolchainParams;
	private TargetDescription lastTargetDescription;

	public TargetOptionsPane(IApplicationConfiguration application) {
		this.application = application;
		constraints = setupGridBag(this, new int[]{150, 300, 0}, new double[]{0, 1, 0});

		addGroupTitle(this, constraints, "Target Options", 2);

		setGridLine(constraints, 1);
		addAttributeName(this, constraints, "Target");
		setGridRelativeX(constraints);

		targetComboBox = new SelectionComboBox<>();
		targetComboBox.setEditable(false);
		new AutoCompletion(targetComboBox);
		add(targetComboBox, constraints);

		setGridLine(constraints, 2);
		addGroupTitle(this, constraints, "User Constraints", 2);

		setGridLine(constraints, 3);
		ucData = addAttributeTextField("UC_DATA", true);
		setGridLine(constraints, 4);
		ucFile = addAttributeTextField("UC_FILE", true);

		setGridLine(constraints, 5);
		addGroupTitle(this, constraints, "Device and Toolchain Options", 2);

		setGridLine(constraints, 6);
		addAttributeName(this, constraints, "Device");
		setGridRelativeX(constraints);

		deviceComboBox = new SelectionComboBox<>();
		deviceComboBox.setEditable(false);
		new AutoCompletion(deviceComboBox);
		add(deviceComboBox, constraints);

		setGridLine(constraints, 7);
		addAttributeName(this, constraints, "Family");
		setGridRelativeX(constraints);

		familyField = new JTextField();
		familyField.setEditable(false);
		familyField.setEnabled(false);
		add(familyField, constraints);

		setGridLine(constraints, 8);
		addAttributeName(this, constraints, "JTAG-ID");
		setGridRelativeX(constraints);

		jtagId = new JTextField();
		jtagId.setEditable(false);
		jtagId.setEnabled(false);
		add(jtagId, constraints);

		setGridLine(constraints, 9);
		addAttributeName(this, constraints, "Toolchain");
		setGridRelativeX(constraints);

		toolchainComboBox = new SelectionComboBox<>();
		toolchainComboBox.setEditable(false);
		new AutoCompletion(toolchainComboBox);
		add(toolchainComboBox, constraints);

		firstOptionalComponent = getComponentCount();
		firstOptionalLine = 10;
	}

	private ToolchainAttrWatcher addToolchainParam(String label) {
		addAttributeName(this, constraints, label);
		setGridRelativeX(constraints);

		ToolchainAttrWatcher watcher = new ToolchainAttrWatcher();
		add(watcher.paramContents, constraints);
		add(watcher.defaultCheckBox, constraints);
		return watcher;
	}

	private void setToolchainParamHandlers() {
		Iterator<DeviceDescription.DevParam> it = target.getToolchain().getParams().values().iterator();
		for (ToolchainAttrWatcher toolchainAttrWatcher : toolchainParams) {
			DeviceDescription.DevParam param = it.next();
			toolchainAttrWatcher.setHandler(target.getToolchainParamHandler(param));
		}
	}

	public void onTabContentsChanged() {
		IConfigurationDocument.ITargetConfiguration target = element.getTarget();
		Library library = application.getLibrary();

		TargetDescription targetDescription = target.getTargetDescription();
		DeviceDescription activeDeviceDescription = target.getDeviceDescription();
		DeviceDescription.Toolchain activeToolchain = target.getToolchain();
		if (target != this.target || this.lastTargetDescription != targetDescription || this.lastDeviceDescription != activeDeviceDescription || this.lastToolchain != activeToolchain) {
			setHandlers(element);
			return;
		}

		targetComboBox.update();
		deviceComboBox.update();

		updateJtagIdAndToolchain(library, activeDeviceDescription);
		toolchainParams.forEach(ToolchainAttrWatcher::update);
	}

	private void updateJtagIdAndToolchain(Library library, DeviceDescription activeDeviceDescription) {
		TargetDescription.Device device = target.getDevice();
		jtagId.setText(device.getJtagid());
		familyField.setText(activeDeviceDescription.getFamily());
        toolchainComboBox.setSelectedItem(target.getToolchain());
	}

	private AttributeTextField addAttributeTextField(String label, boolean editable) {
		addAttributeName(this, constraints, label);
		setGridRelativeX(constraints);

		AttributeTextField field = new AttributeTextField();
		field.setEditable(editable);
		field.setEnabled(editable);
		field.setName("tf-"+label);
		add(field, constraints);
		return field;
	}

	@Override
	public void onConnectorChanged(IConfigurationElement<?, ?, ?> element, IConnectorHandler<?, ?> connector, ConnectorChangeType type, IConnection<?> connection) {

	}

	@Override
	public void onElementNameChanged(IConfigurationElement<?, ?, ?> element) {

	}

	@Override
	public void onAttributeChanged(IConfigurationElement<?, ?, ?> element, String attribute) {
		onTabContentsChanged();
	}

	@Override
	public void onElementDeclarationChanged(IConfigurationElement<?, ?, ?> element) {
		onTabContentsChanged();
	}

	@Override
	protected void detach(IConfigurationDocument oldElement) {
		oldElement.removeDocumentListener(this);
	}

	@Override
	protected void attach(IConfigurationDocument newElement) {
		newElement.addDocumentListener(this);
	}

	@Override
	protected void setHandlers(IConfigurationDocument element) {
		GridBagHelper.clearComponentsFrom(this, firstOptionalComponent);

		target = element.getTarget();
		Library library = application.getLibrary();

		targetComboBox.setHandler(target.getTargetSelectionHandler());

		ucFile.setHandler(target.getAttributeHandler(IConfigurationDocument.ITargetConfiguration.ATTRIB_UC_FILE));
		ucData.setHandler(target.getAttributeHandler(IConfigurationDocument.ITargetConfiguration.ATTRIB_UC_DATA));

		TargetDescription targetDescription = target.getTargetDescription();
		deviceComboBox.setHandler(target.getDeviceSelectionHandler());

		DeviceDescription activeDeviceDescription = target.getDeviceDescription();


		toolchainComboBox.setHandler(target.getToolchainSelectionHandler());

		List<ToolchainAttrWatcher> toolchainParams = new ArrayList<>();

		int line = firstOptionalLine;
		for (DeviceDescription.DevParam param : target.getToolchain().getParams().values()) {
			setGridLine(constraints, line++);
			toolchainParams.add(addToolchainParam(param.getName()));
		}
		this.toolchainParams = toolchainParams;
		this.lastTargetDescription = targetDescription;
		this.lastDeviceDescription = activeDeviceDescription;
		this.lastToolchain = target.getToolchain();
		setToolchainParamHandlers();

		updateJtagIdAndToolchain(library, activeDeviceDescription);

		endLayout(this, constraints);


		// When switching from many parameters to few or none, we still see remains of the old parameters.
		// Explicitly repainting fixes that...
		repaint();
	}

	@Override
	protected void clearHandlers() {
		target = null;
		lastDeviceDescription = null;
		lastTargetDescription = null;
		lastToolchain = null;
	}

	@Override
	public void onTargetChanged() {
		onTabContentsChanged();
	}


	private static class ToolchainAttrWatcher implements IAttributeWatcher<IToolchainAttributeHandler>, FocusListener, ActionListener {

		private final JTextArea paramContents;
		private final JCheckBox defaultCheckBox;
		private IToolchainAttributeHandler handler;

		public ToolchainAttrWatcher() {
			paramContents = new JTextArea();
			paramContents.setLineWrap(true);
			defaultCheckBox = new JCheckBox();
			paramContents.addFocusListener(this);
			defaultCheckBox.addActionListener(this);
		}

		@Override
		public void setHandler(IToolchainAttributeHandler handler) {
			this.handler = handler;
			update();
		}

		@Override
		public void update() {
			paramContents.setEditable(!handler.isDefaultValue());
			paramContents.setText(handler.getValue().orElse(""));
			defaultCheckBox.setSelected(handler.isDefaultValue());
		}

		@Override
		public void focusGained(FocusEvent e) {

		}

		@Override
		public void focusLost(FocusEvent e) {
			if (!handler.isDefaultValue()) {
				handler.setStoredValue(paramContents.getText());
			}
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (defaultCheckBox.isSelected()) {
				handler.resetToDefault();
			} else {
				handler.setStoredValue(handler.getDefaultValue());
			}
		}
	}
}
