/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.ui.resources;

import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.ModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.ui.components.chrome.OverlayIcon;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import javax.swing.ImageIcon;

/**
 * Use this to locate icon resources, so that it noes not matter in which package they live
 */
public class IconUtil {

	private static final Map<IConfigurationElement.ElementType, String> configElementIcons = new HashMap<>();
	private static final Map<ModuleDescription.ModuleType, String> moduleDescrIcons = new HashMap<>();
	private static final String fallbackIcon = "default0.png";
	private static final Map<String, ImageIcon> iconCache = new HashMap<>();

	static {
		configElementIcons.put(IConfigurationElement.ElementType.Configuration, "pcb0.png");
		configElementIcons.put(IConfigurationElement.ElementType.CommonModule, "sot0.png");
		configElementIcons.put(IConfigurationElement.ElementType.Subsystem, "cpu0.png");//or pcb0.png
		configElementIcons.put(IConfigurationElement.ElementType.Peripheral, "smd0.png");
		configElementIcons.put(IConfigurationElement.ElementType.MemoryModule, "smd0.png");

		moduleDescrIcons.put(IModuleDescription.ModuleType.Common, "sot0.png");
		moduleDescrIcons.put(IModuleDescription.ModuleType.Peripheral, "smd0.png");
		moduleDescrIcons.put(IModuleDescription.ModuleType.DynamicStructure, "sot0.png");
		moduleDescrIcons.put(IModuleDescription.ModuleType.Memory, "smd0.png");
		moduleDescrIcons.put(IModuleDescription.ModuleType.Code, "smd0.png");
		moduleDescrIcons.put(IModuleDescription.ModuleType.Subsystem, "cpu0.png");//or pcb0.png
	}

	public static ImageIcon getIcon(String icon) {
		ImageIcon cached = iconCache.get(icon);

		if (cached == null) {
			URL data = getIconResource(icon);
			if (data == null) {
				throw new NullPointerException("Icon Data for " + icon + " not found");
			}
			cached = new ImageIcon(data);
			iconCache.put(icon, cached);
		}

		return cached;
	}

	private static URL getIconResource(String icon) {
		return IconUtil.class.getResource("/icons/" + icon);
	}

	private static OverlayIcon getOverlayIcon(String icon) {
		return new OverlayIcon(getIconResource(icon));
	}

	private static ImageIcon getIconForElement(String iconBase, int iconSize/*, OverlayState state*/) {
		if (iconBase == null) {
			iconBase = fallbackIcon;
		}

		String iconPath = iconSize + "/" + iconBase;//TODO make the ?display-state? and overlay state optional

		ImageIcon icon = getIcon(iconPath);

		//		icon.setOverlayImage(getOverlay(state,iconSize),OverlayIcon.BOTTOM_LEFT);
		// This was already commented out: claim: no node validation -> overlay useless.... bravo!
		//		if ((element != null) && (!element.isInRange())) {
		//			icon.setOverlayImage(getOverlayIcon("_error",size),OverlayIcon.BOTTOM_RIGHT);
		//		} else {
		//		icon.setOverlayImage(null,OverlayIcon.BOTTOM_RIGHT);
		//		}
		return icon;
	}

	public static ImageIcon getIconForElement(ModuleDescription.ModuleType elementType, int iconSize) {
		return getIconForElement(moduleDescrIcons.get(elementType),iconSize);

	}

	public static ImageIcon getIconForElement(IConfigurationElement.ElementType elementType, int iconSize) {
		return getIconForElement(configElementIcons.get(elementType),iconSize);
	}

	public enum OverlayState {
		Fine, Questionable, Erroneous
	}
}
