/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
This class represents the prototype of any general object (=module in jConfig) ever able to display
*/

package de.tu_darmstadt.rs.spartanmc.jconfig.ui.schematic;

import java.util.Vector;

public class SchematicGeneralObject implements Comparable<SchematicGeneralObject> {

	private Vector<SchematicGeneralObject> children;
	private Vector<SchematicWireObject> SWOs;
	private SchematicObjectTypeEnum type;
	private SchematicGeneralObject parent;
	private String name = "";
	private String moduleType = "";
	private int memSize = -1;

	public SchematicGeneralObject(SchematicObjectTypeEnum type) {
		if (type != null) {
			this.type = type;
		} else {
			throw new NullPointerException();
		}
		children = new Vector<SchematicGeneralObject>();
		if (isContainingSWO()) {
			SWOs = new Vector<SchematicWireObject>();
		}
	}

	public boolean isOfType(SchematicObjectTypeEnum type) {
		return (this.type == type);
	}

	public boolean isContainingSWO() {
		return (isOfType(SchematicObjectTypeEnum.SUBSYSTEM) || isOfType(SchematicObjectTypeEnum.TOPLEVEL));
	}

	public boolean isConnectable() {
		return (this.type == SchematicObjectTypeEnum.PROCESSOR || this.type == SchematicObjectTypeEnum.PERIPHERAL || this.type == SchematicObjectTypeEnum.GLUE_LOGIC);
	}

	public Vector<SchematicWireObject> getSWOs() {
		return SWOs;
	}

	public void addChild(SchematicGeneralObject child) {
		child.setParent(this);
		children.add(child);
	}

	public Vector<SchematicGeneralObject> getChildren() {
		return children;
	}

	public SchematicObjectTypeEnum getType() {
		return type;
	}

	public SchematicGeneralObject getParent() {
		return parent;
	}

	private void setParent(SchematicGeneralObject parent) {
		this.parent = parent;
	}

	public int getGeneration() {
		if (parent == null) {
			return 0;
		}
		return (parent.getGeneration() + 1);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getModuleType() {
		return moduleType;
	}

	public void setModuleType(String moduleType) {
		this.moduleType = moduleType;
	}

	public SchematicGeneralObject deliverSubsystemIfAvailable() {
		SchematicGeneralObject parentSGO = this;
		while (true) {
			parentSGO = parentSGO.getParent();
			if (parentSGO == null || parentSGO.isContainingSWO()) {
				return parentSGO;
			}
		}
	}

	public int getMemSize() {
		return memSize;
	}

	public void setMemSize(int memSize) {
		this.memSize = memSize;
	}

	public boolean hasMemoryAsChild() {
		for (SchematicGeneralObject s : getChildren()) {
			if (s.isOfType(SchematicObjectTypeEnum.MEMORY)) {
				return true;
			}
		}
		return false;
	}

	public int compareTo(SchematicGeneralObject otherSGO) {
		int x = 0, y = 0;
		if (getType() == SchematicObjectTypeEnum.SCHEMATIC_ROOT) {
			x = 0;
		}
		;
		if (getType() == SchematicObjectTypeEnum.TOPLEVEL) {
			x = 1;
		}
		;
		if (getType() == SchematicObjectTypeEnum.SUBSYSTEM) {
			x = 2;
		}
		;
		if (getType() == SchematicObjectTypeEnum.PROCESSOR) {
			x = 3;
		}
		;
		if (getType() == SchematicObjectTypeEnum.PERIPHERAL) {
			x = 4;
		}
		;
		if (getType() == SchematicObjectTypeEnum.GLUE_LOGIC) {
			x = 4;
		}
		;
		if (getType() == SchematicObjectTypeEnum.MEMORY) {
			x = 5;
		}
		;
		if (otherSGO.getType() == SchematicObjectTypeEnum.SCHEMATIC_ROOT) {
			y = 0;
		}
		;
		if (otherSGO.getType() == SchematicObjectTypeEnum.TOPLEVEL) {
			y = 1;
		}
		;
		if (otherSGO.getType() == SchematicObjectTypeEnum.SUBSYSTEM) {
			y = 2;
		}
		;
		if (otherSGO.getType() == SchematicObjectTypeEnum.PROCESSOR) {
			y = 3;
		}
		;
		if (otherSGO.getType() == SchematicObjectTypeEnum.PERIPHERAL) {
			y = 4;
		}
		;
		if (otherSGO.getType() == SchematicObjectTypeEnum.GLUE_LOGIC) {
			y = 4;
		}
		;
		if (otherSGO.getType() == SchematicObjectTypeEnum.MEMORY) {
			y = 5;
		}
		;
		if (x != y) {
			return x - y;
		}
		if (this.hasMemoryAsChild() != otherSGO.hasMemoryAsChild()) {
			if (this.hasMemoryAsChild()) {
				return -1;
			} else {
				return 1;
			}
		}
		if (getModuleType().compareTo(otherSGO.getModuleType()) == 0) {
			return getName().compareTo(otherSGO.getName());
		} else {
			return getModuleType().compareTo(otherSGO.getModuleType());
		}
	}
}
