/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
This class represents the information needed to paint and export the schematic, first of all the modules themselves.
*/

package de.tu_darmstadt.rs.spartanmc.jconfig.ui.schematic;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IAddressRangeHandler;

import org.apache.batik.anim.dom.SVGDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.svg.SVGDocument;
import org.xml.sax.helpers.DefaultHandler;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.font.GlyphVector;
import java.io.File;
import java.util.Arrays;
import java.util.Optional;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

public class SchematicInformation {

	static int LAYOUT_LEFTPORT_RIGHTBUS = 0;
	private final IConfigurationDocument document;
	private final Library library;

	private SVGGraphics2D g;

	//***BEGIN LAYOUTINFORMATION
	//alternatively: private Vector<String> busLabels = new Vector<String>(Arrays.asList("di_peri","access_peri","addr_peri","clk_peri","do_peri","reset","wr_peri"));
	private Vector<String> busLabels = new Vector<>(Arrays.asList("D_out", "Addr", "D_in"));
	private boolean drawMemory = true;
	private int number_of_bus_lines = 3;
	private int memory_block_size = 2048;
	private int number_of_memory_blocks = 2;
	private int port_label_y_offset = 10;
	private int port_label_y_space = 15;
	private int port_label_border = 2;
	private int port_label_wire_space = 4;
	private int processor_width = 100;
	private int processor_height = 120;
	private int processor_x_offset = 20;
	private int processor_y_offset = 300;
	private int processor_bus_connection_offset = 15;
	private float processor_border_thickness = 1.5F;
	private Color processor_fill_color = new Color(Integer.decode("#b2f0ff"));
	private Color processor_outline_color = Color.black;
	private Color processor_label_color = Color.black;
	private int module_x_distance = 40;
	private int module_width = 90;
	private int module_height = 130;
	private int module_x_offset = 150;
	private int module_y_offset = 70;
	private int module_label_y_offset = 15;
	private int module_bus_connection_offset = 10;
	private float module_border_thickness = 1F;
	private Color module_fill_color = new Color(Integer.decode("#ffdab2"));
	private Color module_outline_color = Color.black;
	private Color module_label_color = Color.black;
	private int memory_height = 90;
	private int memory_x_offset = 10;
	private int memory_y_offset = 17;
	private int memory_x_distance = 150;
	private int memory_y_distance = 60;
	private int memory_label_y_offset = 12;
	private float memory_border_thickness = 1F;
	private Color memory_fill_color = new Color(Integer.decode("#b8ffba"));
	private Color memory_outline_color = Color.black;
	private Color memory_label_color = Color.black;
	private String memory_label_string = "MEM %d + %d";
	private int dma_height = 90;
	private int dma_y_distance = 35;
	private int dma_label_y_offset = 15;
	private float dma_border_thickness = 1F;
	private Color dma_fill_color = new Color(Integer.decode("#ffff99"));
	private Color dma_outline_color = Color.black;
	private Color dma_label_color = Color.black;
	private String dma_label_string = "DMA";
	private int label_linesep = 15;
	private float bus_width_thick = 3.5F;
	private float bus_width_thin = 2.5F;
	private Color bus_color = Color.black;
	private int bus_connector_radius = 4;
	private int bus_label_x_offset = 7;
	private Font moduleLabelFont = new Font(Font.DIALOG, 0, 12);
	private Font portWireLabelFont = new Font(Font.DIALOG, 0, 12);
	private Font portTagLabelFont = new Font(Font.DIALOG, 0, 12);
	private Font busLabelFont = new Font(Font.MONOSPACED, 0, 12);
	//Precomputated and unchangeable values
	private int stroke_cap = BasicStroke.CAP_BUTT;
	private int stroke_join = BasicStroke.JOIN_BEVEL;
	private int layout = LAYOUT_LEFTPORT_RIGHTBUS;
	private int dma_width = module_width;
	private int memory_width = module_width;
	private int module_bus_connection_distance = (int) ((module_width - 2 * module_bus_connection_offset) / (number_of_bus_lines - 1));
	private int processor_bus_connection_distance = (int) (module_bus_connection_distance * 0.618);
	//***END LAYOUTINFORMATION

	//Constructor, converting a given LibJConfig 1 root Node to a version-less SchematicInformation object
	public SchematicInformation(IConfigurationDocument document, Library library) {
		this.document = document;
		this.library = library;
	}

	public void setKeyValuePair(String k, String v) throws NumberFormatException {
		if (k.equals("busLabels")) {
			if (v.equals("$$$")) {
				busLabels.clear();
			} else {
				busLabels.add(v);
			}
		}
		if (k.equals("drawMemory")) {
			drawMemory = Boolean.parseBoolean(v);
		}
		if (k.equals("number_of_bus_lines")) {
			number_of_bus_lines = Integer.decode(v);
		}
		if (k.equals("memory_block_size")) {
			memory_block_size = Integer.decode(v);
		}
		if (k.equals("number_of_memory_blocks")) {
			number_of_memory_blocks = Integer.decode(v);
		}
		if (k.equals("port_label_y_offset")) {
			port_label_y_offset = Integer.decode(v);
		}
		if (k.equals("port_label_y_space")) {
			port_label_y_space = Integer.decode(v);
		}
		if (k.equals("port_label_border")) {
			port_label_border = Integer.decode(v);
		}
		if (k.equals("port_label_wire_space")) {
			port_label_wire_space = Integer.decode(v);
		}
		if (k.equals("processor_width")) {
			processor_width = Integer.decode(v);
		}
		if (k.equals("processor_height")) {
			processor_height = Integer.decode(v);
		}
		if (k.equals("processor_x_offset")) {
			processor_x_offset = Integer.decode(v);
		}
		if (k.equals("processor_y_offset")) {
			processor_y_offset = Integer.decode(v);
		}
		if (k.equals("processor_bus_connection_offset")) {
			processor_bus_connection_offset = Integer.decode(v);
		}
		if (k.equals("processor_border_thickness")) {
			processor_border_thickness = Float.parseFloat(v);
		}
		if (k.equals("processor_fill_color")) {
			processor_fill_color = new Color(Integer.decode(v));
		}
		if (k.equals("processor_outline_color")) {
			processor_outline_color = new Color(Integer.decode(v));
		}
		if (k.equals("processor_label_color")) {
			processor_label_color = new Color(Integer.decode(v));
		}
		if (k.equals("module_x_distance")) {
			module_x_distance = Integer.decode(v);
		}
		if (k.equals("module_width")) {
			module_width = Integer.decode(v);
		}
		if (k.equals("module_height")) {
			module_height = Integer.decode(v);
		}
		if (k.equals("module_x_offset")) {
			module_x_offset = Integer.decode(v);
		}
		if (k.equals("module_y_offset")) {
			module_y_offset = Integer.decode(v);
		}
		if (k.equals("module_label_y_offset")) {
			module_label_y_offset = Integer.decode(v);
		}
		if (k.equals("module_bus_connection_offset")) {
			module_bus_connection_offset = Integer.decode(v);
		}
		if (k.equals("module_border_thickness")) {
			module_border_thickness = Float.parseFloat(v);
		}
		if (k.equals("module_fill_color")) {
			module_fill_color = new Color(Integer.decode(v));
		}
		if (k.equals("module_outline_color")) {
			module_outline_color = new Color(Integer.decode(v));
		}
		if (k.equals("module_label_color")) {
			module_label_color = new Color(Integer.decode(v));
		}
		if (k.equals("memory_height")) {
			memory_height = Integer.decode(v);
		}
		if (k.equals("memory_x_offset")) {
			memory_x_offset = Integer.decode(v);
		}
		if (k.equals("memory_y_offset")) {
			memory_y_offset = Integer.decode(v);
		}
		if (k.equals("memory_x_distance")) {
			memory_x_distance = Integer.decode(v);
		}
		if (k.equals("memory_y_distance")) {
			memory_y_distance = Integer.decode(v);
		}
		if (k.equals("memory_label_y_offset")) {
			memory_label_y_offset = Integer.decode(v);
		}
		if (k.equals("memory_border_thickness")) {
			memory_border_thickness = Float.parseFloat(v);
		}
		if (k.equals("memory_fill_color")) {
			memory_fill_color = new Color(Integer.decode(v));
		}
		if (k.equals("memory_outline_color")) {
			memory_outline_color = new Color(Integer.decode(v));
		}
		if (k.equals("memory_label_color")) {
			memory_label_color = new Color(Integer.decode(v));
		}
		if (k.equals("memory_label_string")) {
			memory_label_string = v;
		}
		if (k.equals("dma_height")) {
			dma_height = Integer.decode(v);
		}
		if (k.equals("dma_y_distance")) {
			dma_y_distance = Integer.decode(v);
		}
		if (k.equals("dma_label_y_offset")) {
			dma_label_y_offset = Integer.decode(v);
		}
		if (k.equals("dma_border_thickness")) {
			dma_border_thickness = Float.parseFloat(v);
		}
		if (k.equals("dma_fill_color")) {
			dma_fill_color = new Color(Integer.decode(v));
		}
		if (k.equals("dma_outline_color")) {
			dma_outline_color = new Color(Integer.decode(v));
		}
		if (k.equals("dma_label_color")) {
			dma_label_color = new Color(Integer.decode(v));
		}
		if (k.equals("dma_label_string")) {
			dma_label_string = v;
		}
		if (k.equals("label_linesep")) {
			label_linesep = Integer.decode(v);
		}
		if (k.equals("bus_width_thick")) {
			bus_width_thick = Float.parseFloat(v);
		}
		if (k.equals("bus_width_thin")) {
			bus_width_thin = Float.parseFloat(v);
		}
		if (k.equals("bus_color")) {
			bus_color = new Color(Integer.decode(v));
		}
		if (k.equals("bus_connector_radius")) {
			bus_connector_radius = Integer.decode(v);
		}
		if (k.equals("bus_label_x_offset")) {
			bus_label_x_offset = Integer.decode(v);
		}
	}

	public void finalizeStylesheet() {
		memory_width = module_width;
		dma_width = memory_width;
		module_bus_connection_distance = (module_width - 2 * module_bus_connection_offset) / (number_of_bus_lines - 1);
		processor_bus_connection_distance = (int) (module_bus_connection_distance * 0.618);
	}

	public void loadVariables(File pathToXML) throws Exception {
		SAXParserFactory factory = SAXParserFactory.newInstance();
		SAXParser saxParser = factory.newSAXParser();
		DefaultHandler handler = new SchematicXMLParser(this);
		saxParser.parse(pathToXML, handler);
	}

	private GlyphVector convertToTextObject(String text, Font font, int maxSize) {
		String fill = "...";
		String workingText = text;
		GlyphVector result = font.layoutGlyphVector(g.getFontRenderContext(), text.toCharArray(), 0, text.length(), 0);
		if (maxSize >= 0 && result.getOutline().getBounds().getWidth() > maxSize) {
			for (int i = text.length() + fill.length(); i > 0; i--) {
				if (i > fill.length()) {
					workingText = workingText.substring(0, i - fill.length() - 1);
					String resultText = workingText.concat(fill);
					result = font.layoutGlyphVector(g.getFontRenderContext(), resultText.toCharArray(), 0, resultText.length(), 0);
				} else {
					fill = fill.substring(0, i - 1);
					result = font.layoutGlyphVector(g.getFontRenderContext(), fill.toCharArray(), 0, fill.length(), 0);
				}
				if (result.getOutline().getBounds().getWidth() <= maxSize) {
					break;
				}
			}
		}
		return result;
	}

	private void drawSchematicOfSubsystem(ISubsystemConfiguration<?> subsystem) {
		int nextXPositionUp = module_x_offset + module_x_distance + module_width;
		int nextXPositionDown = module_x_offset;
		int nextYPositionDown = processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * (number_of_bus_lines - 1) / 2 + module_y_offset;
		int nextYPositionUp = processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * (number_of_bus_lines - 1) / 2 - module_y_offset;

		//draw the processor
		GlyphVector processorLabel = convertToTextObject(subsystem.getName(), moduleLabelFont, processor_width);
		GlyphVector processorTypeLabel = convertToTextObject(subsystem.getModuleDescriptionName(), moduleLabelFont, processor_width);
		g.setPaint(processor_fill_color);
		g.fillRect(processor_x_offset, processor_y_offset, processor_width, processor_height);
		g.setPaint(processor_outline_color);
		g.setStroke(new BasicStroke(processor_border_thickness, stroke_cap, stroke_join));
		g.drawRect(processor_x_offset, processor_y_offset, processor_width, processor_height);
		g.setPaint(processor_label_color);
		g.drawGlyphVector(processorLabel, processor_x_offset + (int) ((processor_width - processorLabel.getOutline().getBounds().getWidth()) / 2), processor_y_offset + module_label_y_offset);
		g.drawGlyphVector(processorTypeLabel, processor_x_offset + (int) ((processor_width - processorTypeLabel.getOutline().getBounds().getWidth()) / 2), processor_y_offset + module_label_y_offset + label_linesep);
		//Draw the memory
		AddressLayout addressLayout = AddressLayout.getCurrent(subsystem);
		Optional<Long> resolvedMemorySize = addressLayout.findSpaceHandler("data").flatMap(dataSpace -> dataSpace.findRangeHandler("data")).map(IAddressRangeHandler::getLength);
		if (drawMemory && resolvedMemorySize.isPresent() && resolvedMemorySize.get() > 0) {
			for (int i = (int) (resolvedMemorySize.get() / memory_block_size - 1); i >= 0; i--) {
				GlyphVector memoryLabel = convertToTextObject(String.format(memory_label_string, i * number_of_memory_blocks + 1, i * number_of_memory_blocks + 2), moduleLabelFont, memory_width);
				g.setPaint(memory_fill_color);
				g.fillRect(memory_x_distance - i * memory_x_offset, nextYPositionUp - i * memory_y_offset - memory_height, memory_width, memory_height);
				g.setPaint(memory_outline_color);
				g.setStroke(new BasicStroke(memory_border_thickness, stroke_cap, stroke_join));
				g.drawRect(memory_x_distance - i * memory_x_offset, nextYPositionUp - i * memory_y_offset - memory_height, memory_width, memory_height);
				g.drawGlyphVector(memoryLabel, memory_x_distance - i * memory_x_offset + (int) ((memory_width - memoryLabel.getOutline().getBounds().getWidth()) / 2), nextYPositionUp - i * memory_y_offset + memory_label_y_offset - memory_height);
			}
			for (int i = 0; i < number_of_bus_lines; i++) {
				g.setStroke(new BasicStroke(bus_width_thin, stroke_cap, stroke_join));
				g.drawLine(memory_x_distance + module_bus_connection_offset + module_bus_connection_distance * i, nextYPositionUp, memory_x_distance + module_bus_connection_offset + module_bus_connection_distance * i, processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * i);
			}
		}

		for (IModuleConfiguration<?> peripheral : subsystem.getModules()) {
			if (peripheral.getType() != IConfigurationElement.ElementType.Peripheral) {
				continue;
			}

			if (peripheral.getModuleDescription().getBuses().stream().anyMatch(b -> b.getBusDescriptionName().equals("spartanmc-dma-bus"))) { // if (drawUp()) {
				//draw as DMA module
				GlyphVector moduleLabel = convertToTextObject(peripheral.getName(), moduleLabelFont, module_width);
				GlyphVector moduleTypeLabel = convertToTextObject(peripheral.getModuleDescriptionName(), moduleLabelFont, module_width);
				GlyphVector dmaLabel = convertToTextObject(dma_label_string, moduleLabelFont, dma_width);
				g.setPaint(module_fill_color);
				g.fillRect(nextXPositionUp, nextYPositionUp - dma_height - dma_y_distance - module_height, module_width, module_height);
				g.setPaint(module_outline_color);
				g.setStroke(new BasicStroke(module_border_thickness, stroke_cap, stroke_join));
				g.drawRect(nextXPositionUp, nextYPositionUp - dma_height - dma_y_distance - module_height, module_width, module_height);
				g.setPaint(dma_fill_color);
				g.fillRect(nextXPositionUp, nextYPositionUp - dma_height, dma_width, dma_height);
				g.setPaint(dma_outline_color);
				g.setStroke(new BasicStroke(module_border_thickness, stroke_cap, stroke_join));
				g.drawRect(nextXPositionUp, nextYPositionUp - dma_height, dma_width, dma_height);
				g.drawGlyphVector(dmaLabel, nextXPositionUp + (int) ((dma_width - dmaLabel.getOutline().getBounds().getWidth()) / 2), nextYPositionUp + dma_label_y_offset - dma_height);
				g.setPaint(module_label_color);
				g.drawGlyphVector(moduleLabel, nextXPositionUp + (int) ((module_width - moduleLabel.getOutline().getBounds().getWidth()) / 2), nextYPositionUp - dma_height - dma_y_distance + module_label_y_offset - module_height);
				g.drawGlyphVector(moduleTypeLabel, nextXPositionUp + (int) ((module_width - moduleTypeLabel.getOutline().getBounds().getWidth()) / 2), nextYPositionUp - dma_height - dma_y_distance + module_label_y_offset + label_linesep - module_height);
				g.setPaint(bus_color);
				for (int i = 0; i < number_of_bus_lines; i++) {
					g.setStroke(new BasicStroke(bus_width_thin, stroke_cap, stroke_join));
					g.drawLine(nextXPositionUp + module_bus_connection_offset + module_bus_connection_distance * i, nextYPositionUp, nextXPositionUp + module_bus_connection_offset + module_bus_connection_distance * i, processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * i);

				}
				for (int i = 0; i < number_of_bus_lines; i++) {
					g.setStroke(new BasicStroke(bus_width_thin, stroke_cap, stroke_join));
					g.drawLine(nextXPositionUp + module_bus_connection_offset + module_bus_connection_distance * i, nextYPositionUp - dma_height, nextXPositionUp + module_bus_connection_offset + module_bus_connection_distance * i, nextYPositionUp - dma_height - dma_y_distance);
				}
				nextXPositionUp += module_x_distance + module_width;
			} else {
				//draw not as DMA module
				GlyphVector moduleLabel = convertToTextObject(peripheral.getName(), moduleLabelFont, module_width);
				GlyphVector moduleTypeLabel = convertToTextObject(peripheral.getModuleDescriptionName(), moduleLabelFont, module_width);
				g.setPaint(module_fill_color);
				g.fillRect(nextXPositionDown, nextYPositionDown, module_width, module_height);
				g.setPaint(module_outline_color);
				g.setStroke(new BasicStroke(module_border_thickness, stroke_cap, stroke_join));
				g.drawRect(nextXPositionDown, nextYPositionDown, module_width, module_height);
				g.setPaint(module_label_color);
				g.drawGlyphVector(moduleLabel, nextXPositionDown + (int) ((module_width - moduleLabel.getOutline().getBounds().getWidth()) / 2), nextYPositionDown + module_label_y_offset);
				g.drawGlyphVector(moduleTypeLabel, nextXPositionDown + (int) ((module_width - moduleTypeLabel.getOutline().getBounds().getWidth()) / 2), nextYPositionDown + module_label_y_offset + label_linesep);
				g.setPaint(bus_color);
				for (int i = 0; i < number_of_bus_lines; i++) {
					g.setStroke(new BasicStroke(bus_width_thin, stroke_cap, stroke_join));
					g.drawLine(nextXPositionDown + module_bus_connection_offset + module_bus_connection_distance * i, nextYPositionDown, nextXPositionDown + module_bus_connection_offset + module_bus_connection_distance * i, processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * i);
				}
				nextXPositionDown += module_x_distance + module_width;
			}
		}

		nextXPositionDown = (nextXPositionDown > nextXPositionUp)? nextXPositionDown : nextXPositionUp;
		nextXPositionDown -= module_x_distance + module_width;
		g.setPaint(bus_color);
		for (int i = 0; i < number_of_bus_lines; i++) {
			GlyphVector busLabel = convertToTextObject(busLabels.get(i), busLabelFont, -1);
			g.setStroke(new BasicStroke(bus_width_thin, stroke_cap, stroke_join));
			g.drawLine(processor_x_offset + processor_width, processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * i, nextXPositionDown + module_bus_connection_offset + module_bus_connection_distance * i, processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * i);
			g.drawGlyphVector(busLabel, nextXPositionDown + module_bus_connection_offset + module_bus_connection_distance * (number_of_bus_lines - 1) + bus_label_x_offset, processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * i + (int) busLabel.getOutline().getBounds().getHeight() / 2);
		}
		for (int i = 0; i < number_of_bus_lines; i++) {
			for (int j = module_x_offset; j < nextXPositionDown; j += module_x_distance + module_width) {
				g.fillOval(module_bus_connection_offset + j - bus_connector_radius + i * module_bus_connection_distance, processor_y_offset + processor_bus_connection_offset + processor_bus_connection_distance * i - bus_connector_radius, 2 * bus_connector_radius, 2 * bus_connector_radius);
			}
		}


	}

	public Object[] getSVGGraphicsInformation() {
		//Due to special requirements of the SVG package, this methods returns in [0] a SVGDocument and in [1] a SVGGraphics2D object
		Object[] result = new Object[2];
		//read (potential) XML-configuration here
		DOMImplementation impl = SVGDOMImplementation.getDOMImplementation();
		String svgNS = SVGDOMImplementation.SVG_NAMESPACE_URI;
		SVGDocument svgDoc = (SVGDocument) impl.createDocument(svgNS, "svg", null);
		g = new SVGGraphics2D(svgDoc);
		//Check wheter there is only one subsystem
		int numberOfSubsystems = document.getSubsystems().size();
		if (numberOfSubsystems > 1) {
			JOptionPane.showMessageDialog(null, "Multicore systems are not yet supported by this version of the schematic module. No schematic was drawn.", "No support", JOptionPane.INFORMATION_MESSAGE);
			result[0] = null;
			result[1] = null;
		} else {
			drawSchematicOfSubsystem(document.getSubsystems().get(0));
			result[0] = svgDoc;
			result[1] = g;
		}
		return result;
	}

}
