/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml;

import java.util.Arrays;

/**
 * @author Ramon Wirsch
 */
public class AttributeUtil {

	private static final String[] BOOLEAN_CHOICES = new String[]{
			"0", "1",// Automatic default even if guessBooleanChoices is called with null
			"FALSE", "TRUE",
			"NO", "YES",
			"DISABLED", "ENABLED",
			"OFF", "ON"};

	/**
	 * If called with <b>null</b>, will return "0", "1"
	 * @param value default value, from which opposite value is interpolated
	 * @return first entry is "false" alias, 2nd is "true" alias
	 */
	public static String[] guessBooleanChoices(String value) {
		int pos = 0;

		for (int i = 0; i < BOOLEAN_CHOICES.length; i++) {
			if (BOOLEAN_CHOICES[i].equalsIgnoreCase(value)) {
				pos = i;
			}
		}
		pos &= 0xFFFE;

		return Arrays.copyOfRange(BOOLEAN_CHOICES, pos, pos + 2);
	}

	public static boolean parseBooleanChoice(String[] choices, String oldValue) {
		return choices[1].equalsIgnoreCase(oldValue); // compare with true alias. everything else is false
	}

	public static String getRadixPrefix(int radix) {
		String prefix = "";
		if (radix == 16) {
			prefix = "0x";
		} else if (radix == 8) {
			prefix = "0";
		} else if (radix == 2) {
			prefix = "0b";
		}
		return prefix;
	}

	public static void verifyOrThrowIllegal(boolean condition) throws IllegalArgumentException {
		if (!condition) {
			throw new IllegalArgumentException();
		}
	}

	/**
	 * Basically Long.decode + support for 0bxxx
	 * @param value String containing number
	 * @param radix radix of number
	 * @return value of number
	 */
	public static long parseLong(String value, int radix) {
		if (radix == 16 || radix == 10 || radix == 8) {
			return Long.decode(value);
		} else {
			String prefix = getRadixPrefix(radix);
			if (value.startsWith(prefix)) {
				value = value.substring(prefix.length());
			}
			return Long.parseLong(value, radix);
		}
	}

}
