/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.devxml.descriptions.calc;

import de.tu_darmstadt.rs.spartanmc.devxml.DevXmlNamespace;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.analysis.ParameterReference;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlIDREF;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Created by Ramon on 07.08.2016.
 */
@XmlRootElement(name = "paramRef", namespace = DevXmlNamespace.NAMESPACE)
public class ParamRef extends CalculatedValue.TerminalCalculatedValue {

	@XmlAttribute(required = true)
	@XmlIDREF
	private ParameterReference param;

	public ParamRef() {
	}

	public ParamRef(ParameterReference param) {
		this.param = param;
	}

	public ParameterReference getParam() {
		return param;
	}

	@Override
	public CalculatedValueResult<? extends Number> resolve(IValueResolver resolver) {
		return resolver.resolveParam(param);
	}

	@Override
	public CalculatedValueResult<? extends String> resolveString(IValueResolver resolver) {
		return resolver.resolveParamString(param);
	}

	@Override
	public boolean isDynamic() {
		return true;
	}

	@Override
	public String toString() {
		return param != null? "{$" + param.getName() + "}" : "{UndefinedRef}";
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		ParamRef paramRef = (ParamRef) o;

		return param != null? param.getName().equals(paramRef.param.getName()) : paramRef.param == null;
	}

	@Override
	public int hashCode() {
		return param != null? param.getName().hashCode() : 0;
	}
}
