/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic;

import de.tu_darmstadt.rs.spartanmc.devxml.configurations.IToolchainConfiguration;
import de.tu_darmstadt.rs.spartanmc.devxml.configurations.IToolchainConfigurationSelector;

import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlEnumValue;
import javax.xml.bind.annotation.XmlID;

/**
 * Created by Ramon on 16.06.2016.
 */
public class ToolchainConfigurationDescription implements IToolchainConfigurationSelector {

	@XmlAttribute
	@XmlID
	private String name;
	@XmlAttribute
	private Mode mode;
	@XmlElement(name = "toolchain")
	private List<SupportedArtifact> toolchains = new ArrayList<>();
	@XmlElement(name = "family")
	private List<SupportedArtifact> families = new ArrayList<>();
	@XmlElement(name = "device")
	private List<SupportedArtifact> devices = new ArrayList<>();
	@XmlElement(name = "target")
	private List<SupportedArtifact> targets = new ArrayList<>();

	public String getName() {
		return name;
	}

	public Mode getMode() {
		return mode;
	}

	public List<SupportedArtifact> getToolchains() {
		return toolchains;
	}

	public List<SupportedArtifact> getFamilies() {
		return families;
	}

	public List<SupportedArtifact> getDevices() {
		return devices;
	}

	public List<SupportedArtifact> getTargets() {
		return targets;
	}

	/**
	 * Check whether this config applies to the given values describing the actual toolchain in use
	 *
	 * @param configuration all properties of a toolchain configuration
	 * @return true if is applicable to current configuration
	 */
	@Override
	public boolean supports(IToolchainConfiguration configuration) {
		if (configuration == IToolchainConfiguration.ANY) {
			return true;
		}

		if (mode == Mode.Exclude) {
			return !(supportsToolchain(configuration)
					|| supportsFamily(configuration)
					|| supportsDevice(configuration)
					|| supportsTarget(configuration));
		}

		return supportsToolchain(configuration)
				|| supportsFamily(configuration)
				|| supportsDevice(configuration)
				|| supportsTarget(configuration);
	}

	@Override
	public boolean supportsTarget(IToolchainConfiguration configuration) {
		return targets.stream().map(SupportedArtifact::getName).anyMatch(n -> n.equals(configuration.getTargetName()));
	}

	@Override
	public boolean supportsDevice(IToolchainConfiguration configuration) {
		return devices.stream().map(SupportedArtifact::getName).anyMatch(n -> n.equals(configuration.getDeviceName()));
	}

	@Override
	public boolean supportsFamily(IToolchainConfiguration configuration) {
		return families.stream().map(SupportedArtifact::getName).anyMatch(n -> n.equals(configuration.getFamilyName()));
	}

	@Override
	public boolean supportsToolchain(IToolchainConfiguration configuration) {
		return toolchains.stream().map(SupportedArtifact::getName).anyMatch(n -> n.equals(configuration.getToolchainImplementation()));
	}

	@XmlEnum
	public enum Mode {
		@XmlEnumValue("include") Include,
		@XmlEnumValue("exclude") Exclude
	}


	public static class SupportedArtifact {

		@XmlAttribute
		private String name;

		public String getName() {
			return name;
		}
	}
}
