/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.calc.CalculatedValue;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.references.IReference;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.references.IReferencer;

import java.util.List;
import java.util.function.ToDoubleFunction;
import java.util.function.ToIntFunction;
import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlEnumValue;

/**
 * Created by Ramon on 18.02.2016.
 */
public interface IParameter extends IReference {

	static boolean doesConflictWithChoices(String value, IParameter description) {
		return description.getChoices() != null && description.getChoices().stream().noneMatch(choice -> choice.getValue().equalsIgnoreCase(value));
	}

	static boolean doesConflictWithChoices(int value, IParameter description, ToIntFunction<String> converter) {
		return description.getChoices() != null && description.getChoices().stream().noneMatch(choice -> converter.applyAsInt(choice.getValue()) == value);
	}

	static boolean doesConflictWithChoices(double value, IParameter description, ToDoubleFunction<String> converter) {
		return description.getChoices() != null && description.getChoices().stream().noneMatch(choice -> converter.applyAsDouble(choice.getValue()) == value);
	}

	/**
	 * Used in such a way, that a reload of module structe is required on changes
	 *
	 * @return true if dynamic
	 */
	boolean isDynamicallyReferenced(); // TODO change name so it makes more sense

	List<String> getInstanceHints();

	boolean hasInstanceHints();

	CalculatedValue.BooleanWrapper getRelevantIfRef();

	boolean hasRelevantIf();

	CalculatedValueResult<Boolean> resolveRelevantIf(IValueResolver resolver);

	boolean isSimulationOnly();

	String getDescription();

	Parameter.Range getRange();

	String getName();

	List<IReferencer> getReferencers();

	String getDefaultValue();

	boolean hasCalculatedValue();

	CalculatedValue getValueRef();

	CalculatedValueResult<? extends Number> resolveValue(IValueResolver resolver);

	ParameterType getType();

	boolean isChoicesOnly();

	boolean hasChoices();

	boolean isEditable();

	boolean isVirtual();

	List<Parameter.Choice> getChoices();

	String getUiUnit();

	boolean hasUiUnit();

	String getTclName();

	@XmlEnum
	enum ParameterType {
		@XmlEnumValue("hex")HexType,
		@XmlEnumValue("int")IntType,
		@XmlEnumValue("float")FloatType,
		@XmlEnumValue("string")StringType,
		@XmlEnumValue("bool")BoolType,
		@XmlEnumValue("bin")BinType
	}
}
