/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.calc.CalculatedValue;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IRedirectedResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.dynamic.IDynamicParameter;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.references.IReferencer;

import java.util.List;

/**
 * Created by Ramon on 21.02.2016.
 */
public abstract class ParameterDecorator implements IDynamicParameter {

	protected final IDynamicParameter org;

	public ParameterDecorator(IDynamicParameter parameter) {
		this.org = parameter;
	}

	@Override
	public boolean isDynamicallyReferenced() {
		return org.isDynamicallyReferenced();
	}

	@Override
	public List<String> getInstanceHints() {
		return org.getInstanceHints();
	}

	@Override
	public boolean hasInstanceHints() {
		return org.hasInstanceHints();
	}

	@Override
	public CalculatedValue.BooleanWrapper getRelevantIfRef() {
		return org.getRelevantIfRef();
	}

	@Override
	public boolean hasRelevantIf() {
		return org.hasRelevantIf();
	}

	@Override
	public CalculatedValueResult<Boolean> resolveRelevantIf(IValueResolver resolver) {
		return org.resolveRelevantIf(resolver);
	}

	@Override
	public boolean isSimulationOnly() {
		return org.isSimulationOnly();
	}

	@Override
	public String getDescription() {
		return org.getDescription();
	}

	@Override
	public Parameter.Range getRange() {
		return org.getRange();
	}

	@Override
	public String getName() {
		return org.getName();
	}

	@Override
	public List<IReferencer> getReferencers() {
		return org.getReferencers();
	}

	@Override
	public String getDefaultValue() {
		return org.getDefaultValue();
	}

	@Override
	public boolean hasCalculatedValue() {
		return org.hasCalculatedValue();
	}

	@Override
	public CalculatedValue getValueRef() {
		return org.getValueRef();
	}

	@Override
	public CalculatedValueResult<? extends Number> resolveValue(IValueResolver resolver) {
		return org.resolveValue(resolver);
	}

	@Override
	public ParameterType getType() {
		return org.getType();
	}

	@Override
	public boolean isChoicesOnly() {
		return org.isChoicesOnly();
	}

	@Override
	public boolean hasChoices() {
		return org.hasChoices();
	}

	@Override
	public boolean isEditable() {
		return org.isEditable();
	}


	@Override
	public CalculatedValueResult<Boolean> isEditable(IValueResolver resolver) {
		return org.isEditable(resolver);
	}

	@Override
	public boolean isVirtual() {
		return org.isVirtual();
	}

	@Override
	public List<Parameter.Choice> getChoices() {
		return org.getChoices();
	}

	@Override
	public String getUiUnit() {
		return org.getUiUnit();
	}

	@Override
	public boolean hasUiUnit() {
		return org.hasUiUnit();
	}

	@Override
	public boolean isDynamicallyEnabled() {
		return org.isDynamicallyEnabled();
	}

	@Override
	public boolean isDynamicallyEditable() {
		return org.isDynamicallyEditable();
	}

	@Override
	public boolean isRedirected() {
		return org.isRedirected();
	}

	@Override
	public CalculatedValueResult<? extends Number> resolveRedirected(IRedirectedResolver resolver) {
		return org.resolveRedirected(resolver);
	}
}
