/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.dynamic;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.calc.CalculatedValue;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dynamic.IDynamicParameterResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameterGroup;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Created by Ramon on 07.08.2016.
 */
@XmlRootElement(name = "dynamic-parameters")
public class DynamicParamterGroupImpl extends DynamicParameterGroup {

	@XmlElement
	private CalculatedValue.WrapperValue implementationCount;

	@Override
	public List<IParameterGroup> getStaticParameterGroups(IDynamicParameterResolver resolver) {
		ArrayList<IParameterGroup> staticGroups = new ArrayList<>();

		if (implementationCount == null) {
			staticGroups.add(new DynamicParameterGroupDecorator(this, getStaticParameters(resolver)));
		} else {
			long count = resolveImplementationCount(resolver).orElse(0L);

			for (int i = 0; i < count; i++) {
				IParameterGroup group = new MultiDynamicParameterGroupDecorator(this, getStaticParameters(resolver), i);
				staticGroups.add(group);
			}
		}
		return staticGroups;
	}

	@Override
	public CalculatedValueResult<Long> resolveImplementationCount(IValueResolver resolver) {
		return implementationCount.resolve(resolver).flatMap(CalculatedValueResult.castOrError(Long.class));
	}

	@Override
	public boolean isDynamicallyResolvable() {
		return true;
	}

	@Override
	public boolean hasDynamicImplementationCount() {
		return implementationCount != null;
	}

	private List<IDynamicParameter> getStaticParameters(IValueResolver resolver) {
		return parameters.stream().map(p -> p.getStaticParameter(resolver)).filter(p -> p != null).collect(Collectors.toList());
	}
}
