/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.calc.CalculatedValue;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dynamic.IDynamicParameterResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.dynamic.IndexedResolver;

/**
 * Created by Ramon on 18.02.2016.
 */
public class MultiPortDecorator implements IPort {

	protected final IPort org;
	private final int index;


	public MultiPortDecorator(IPort dynamicPort, int index) {
		this.org = dynamicPort;
		this.index = index;
	}

	@Override
	public boolean isDirectionLocked() {
		return org.isDirectionLocked();
	}

	@Override
	public Type getType() {
		return org.getType();
	}

	@Override
	public boolean isRequired() {
		return org.isRequired();
	}

	@Override
	public String getName() {
		return org.getName() + index;
	}

	@Override
	public ConnectorDirection getDirection() {
		return org.getDirection();
	}

	@Override
	public int getConstantWidth() {
		return org.getConstantWidth();
	}

	@Override
	public boolean hasCalculatedWidth() {
		return org.hasCalculatedWidth();
	}

	@Override
	public boolean hasDynamicWidth() {
		return org.hasDynamicWidth();
	}

	@Override
	public CalculatedValue getCalculatedWidth() {
		return org.getCalculatedWidth();
	}

	@Override
	public CalculatedValueResult<Long> resolveWidth(IValueResolver resolver) {
		return org.resolveWidth(new IndexedResolver((IDynamicParameterResolver) resolver, index));
	}

	@Override
	public String getDescription() {
		return org.getDescription();
	}
}
