/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.calc.CalculatedValue;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Created by Ramon on 16.06.2016.
 */
@XmlRootElement(name = "port")
public class Port extends BasePort {

	@XmlAttribute(name = "width")
	private int constWidth = 1;

	@XmlElement(name = "width", type = CalculatedValue.WrapperValue.class)
	private CalculatedValue widthRef;

	public Port() {
	}

	public Port(String name, ConnectorDirection direction, String description, boolean required, int constWidth, CalculatedValue widthRef) {
		super(name, direction, description, required);
		this.constWidth = constWidth;
		this.widthRef = widthRef;
	}

	@Override
	public Type getType() {
		return Type.Port;
	}

	@Override
	public int getConstantWidth() {
		return constWidth;
	}

	@Override
	public boolean hasCalculatedWidth() {
		return widthRef != null;
	}

	@Override
	public boolean hasDynamicWidth() {
		return hasCalculatedWidth() && widthRef.isDynamic();
	}

	@Override
	public CalculatedValue getCalculatedWidth() {
		return widthRef;
	}

	@Override
	public CalculatedValueResult<Long> resolveWidth(IValueResolver resolver) {
		if (widthRef != null) {
			return widthRef.resolve(resolver).flatMap(CalculatedValueResult.castOrError(Long.class));
		} else {
			throw new IllegalStateException("No Value to resolve");
		}
	}

	@Override
	public String toString() {
		String sw = widthRef != null? ", width=" + widthRef : ", constWidth=" + constWidth;
		return "Port{" +
			"name='" + name + '\'' +
			", direction=" + direction +
			sw +
			", type=" + getType() +
			", description='" + description + '\'' +
			'}';
	}
}
