/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.dynamic;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dynamic.IDynamicParameterResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.IDynamicContainer;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.BasePortGroup;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IPortGroup;

import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Created by Ramon on 07.08.2016.
 */
@XmlRootElement(name = "ports")
public class DynamicPortGroup extends BasePortGroup implements IDynamicContainer {

	protected List<IDynamicPort> allPorts;
	protected List<IPortGroup> staticGroups;
	@XmlElementRef(type = DynamicPort.class)
	private List<DynamicPort> regularPorts = new ArrayList<>();
	@XmlElementRef(type = DynamicClkPort.class)
	private List<DynamicClkPort> clkPorts = new ArrayList<>();

	public List<IPortGroup> getStaticPortGroups(IDynamicParameterResolver resolver) {
		if (staticGroups == null) {
			staticGroups = new ArrayList<>();
			staticGroups.add(this);
		}
		return staticGroups;
	}

	@Override
	public List<? extends IDynamicPort> getPorts() {
		if (allPorts == null) {
			allPorts = new ArrayList<>();
			allPorts.addAll(regularPorts);
			allPorts.addAll(clkPorts);
		}
		return allPorts;
	}

	@Override
	public boolean isDynamicallyResolvable() {
		return false;
	}

	@Override
	public boolean hasDynamicImplementationCount() {
		return false;
	}

	@Override
	public CalculatedValueResult<Long> resolveImplementationCount(IValueResolver resolver) {
		return CalculatedValueResult.of(1L);
	}


}
