/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml.files.modules;

import de.tu_darmstadt.rs.spartanmc.devxml.DevXmlNamespace;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.calc.CalculatedValue;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.memory.Memory;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlIDREF;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Created by ramon on 18.03.2016.
 */
@XmlRootElement(name = "memory-module", namespace = DevXmlNamespace.NAMESPACE)
public class MemoryModuleDescription extends CommonModuleDescription {

	@XmlElement(name = "memory")
	private List<ExtMemory> memories = new ArrayList<>();

	@Override
	public ModuleType getType() {
		return ModuleType.Memory;
	}

	public List<ExtMemory> getMemories() {
		return memories;
	}

	/**
	 * @deprecated future versions will assign firmware on a per-subsystem basis instead of per memory. The contents property of each Memory controls whether the memory will be initialized with firmware
	 */
	@Deprecated
	public boolean containsFirmware() {
		return true;
	}

	/**
	 * Searches for it in memories
	 * @param bus bus on which to look for memories
	 * @return optional memory
	 */
	public List<ExtMemory> getMemoryExposedOn(Bus bus) {
		return memories.stream().filter(mem -> mem.getExposedOn().stream().anyMatch(exp -> exp.getBus() == bus)).collect(Collectors.toList());
	}

	public static class ExtMemory extends Memory {

		@XmlAttribute
		private String name = "mem";

		@XmlElement
		private List<ExposedOn> exposedOn = new ArrayList<>();

		@XmlElement(name = "count")
		private CalculatedValue.WrapperValue countRef;

		public List<ExposedOn> getExposedOn() {
			return exposedOn;
		}

		public ExposedOn getExposedOn(Bus exposing) {
			return exposedOn.stream().filter(ex -> ex.getBus() == exposing).findAny().get();
		}

		public boolean isShared() {
			return exposedOn.stream().anyMatch(exp -> !exp.getBus().isConnectionsLimited() || exp.getBus().getMaxConnections() > 1);
		}

		public CalculatedValue getCountRef() {
			return countRef;
		}

		public CalculatedValueResult<Integer> resolveCount(IValueResolver resolver) {
			if (countRef != null) {
				return countRef.resolve(resolver).flatMap(CalculatedValueResult.castOrError(Long.class)).map(Math::toIntExact);
			} else {
				return CalculatedValueResult.of(1);
			}
		}

		public String getName() {
			return name;
		}

		//		public boolean isShared() {
		//			return countRef != null && countRef.isShared();
		//		}
	}


	public static class ExposedOn {

		@XmlElement
		private CalculatedValue.WrapperValue at;

		@XmlIDREF
		@XmlAttribute
		private Bus bus;

		public Bus getBus() {
			return bus;
		}

		public CalculatedValue getAtRef() {
			return at;
		}

		public CalculatedValueResult<Long> resolveAt(IValueResolver resolver) {
			return at.resolve(resolver).flatMap(CalculatedValueResult.castOrError(Long.class));
		}
	}
}
