/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml.files.modules;

import de.tu_darmstadt.rs.spartanmc.devxml.DevXmlNamespace;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.memory.Memory;
import de.tu_darmstadt.rs.spartanmc.devxml.jaxb.LongDecodeAdapter;

import java.util.List;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlEnumValue;
import javax.xml.bind.annotation.XmlIDREF;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

/**
 * Created by ramonw on 19.11.15.
 */
@XmlRootElement(name = "peripheral-module", namespace = DevXmlNamespace.NAMESPACE)
public class PeripheralModuleDescription extends StaticModuleDescription {


	@XmlElement
	private RegisterInfo registers;
	@XmlElement
	private DmaMemory memory;

	@Override
	public ModuleType getType() {
		return ModuleType.Peripheral;
	}

	@Deprecated
	public boolean isDmaPeripheral() {
		return memory != null;
	}

	public RegisterInfo getRegisters() {
		return registers;
	}

	public boolean hasRegisters() {
		return registers != null;
	}

	public DmaMemory getMemory() {
		return memory;
	}

	public boolean hasMemory() {
		return memory != null;
	}


	public abstract static class Bits {

		@XmlElement
		private String description;

		@XmlElement(name = "bitval")
		private List<BitVal> bitVals;

		@XmlAttribute
		private String name;

		public String getDescription() {
			return description;
		}

		public List<BitVal> getBitVals() {
			return bitVals;
		}

		public String getName() {
			return name;
		}
	}


	public static class BitGroup extends Bits {

		@XmlAttribute
		private int lsb;

		@XmlAttribute
		private int width;

		public int getLsb() {
			return lsb;
		}

		public int getWidth() {
			return width;
		}
	}


	public static class BitVal {

		@XmlAttribute
		private static String name;

		@XmlAttribute
		private static String value;

		@XmlAttribute(name = "descr")
		private static String description;

		public static String getName() {
			return name;
		}

		public static String getValue() {
			return value;
		}

		public static String getDescription() {
			return description;
		}
	}


	public static class Register extends Bits {

		@XmlElement(name = "bitgroup")
		private List<BitGroup> bitGroups;
		@XmlAttribute
		private int offset;
		@XmlAttribute
		private AccessType access;
		@XmlAttribute
		@XmlJavaTypeAdapter(type = long.class, value = LongDecodeAdapter.class)
		private long defaultvalue;

		public List<BitGroup> getBitGroups() {
			return bitGroups;
		}

		public int getOffset() {
			return offset;
		}

		public AccessType getAccess() {
			return access;
		}

		public long getDefaultvalue() {
			return defaultvalue;
		}

		@XmlEnum
		public enum AccessType {
			@XmlEnumValue("read")Read,
			@XmlEnumValue("write")Write,
			@XmlEnumValue("readwrite")ReadWrite
		}
	}


	public static class RegisterInfo {

		@XmlAttribute
		@XmlIDREF
		private Bus exposedOn;
		/**
		 * Default to old naming scheme, register base adr will be written to verilog parameter BASE_ADR by default
		 */
		@XmlAttribute
		private String baseAdrParam = "BASE_ADR";

		@XmlElement
		private CSupport cSupport;

		@XmlElement(name = "register")
		private List<Register> registers;

		public List<Register> getRegisters() {
			return registers;
		}

		public CSupport getCSupport() {
			return cSupport;
		}


		public Bus getExposedOn() {
			return exposedOn;
		}

		public String getBaseAdrParam() {
			return baseAdrParam;
		}

		public String getName() {
			return "reg";
		}
	}


	public static class CSupport {

		@XmlAttribute(required = true)
		private String typeName;

		@XmlAttribute(required = true)
		private String fileName;

		@XmlAttribute
		private String declareFile;

		public CSupport() {

		}

		public CSupport(String typeName, String fileName, String declareFile) {
			this.typeName = typeName;
			this.fileName = fileName;
			this.declareFile = declareFile;
		}

		public String getFileName() {
			return fileName;
		}

		public String getTypeName() {
			return typeName;
		}

		public String getDeclareFile() {
			return declareFile;
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) {
				return true;
			}
			if (!(o instanceof CSupport)) {
				return false;
			}

			CSupport cSupport = (CSupport) o;

			if (!typeName.equals(cSupport.typeName)) {
				return false;
			}
			if (!fileName.equals(cSupport.fileName)) {
				return false;
			}
			return declareFile != null? declareFile.equals(cSupport.declareFile) : cSupport.declareFile == null;
		}

		@Override
		public int hashCode() {
			int result = typeName.hashCode();
			result = 31 * result + fileName.hashCode();
			result = 31 * result + (declareFile != null? declareFile.hashCode() : 0);
			return result;
		}

		@Override
		public String toString() {
			return "CSupport{" +
				"typeName='" + typeName + '\'' +
				", fileName='" + fileName + '\'' +
				'}';
		}
	}


	public static class DmaMemory extends Memory {

		@XmlAttribute
		private String baseAdrParam = "DMA_BASE_ADR";

		@XmlElement
		private CSupport cSupport;
		@XmlIDREF
		@XmlAttribute
		private Bus exposedOn;

		public String getBaseAdrParam() {
			return baseAdrParam;
		}

		public CSupport getCSupport() {
			return cSupport;
		}

		public Bus getExposedOn() {
			return exposedOn;
		}

		public String getName() {
			return "dma";
		}
	}
}
