/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml.files.modules;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.BaseParameterGroup;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.Parameter;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.BasePortGroup;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.ClkPort;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IPort;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.Port;

import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementRef;

/**
 * Created by Ramon on 16.02.2016.
 */
public abstract class StaticModuleDescription extends ModuleDescription {

	@XmlElement(name = "parameters")
	private List<StaticParameterGroup> parameterGroups = new ArrayList<>();

	@XmlElement(name = "ports")
	private List<StaticPortGroup> portGroups = new ArrayList<>();

	@Override
	public List<StaticParameterGroup> getParameterGroups() {
		return parameterGroups;
	}

	@Override
	public List<StaticPortGroup> getPortGroups() {
		return portGroups;
	}


	public static class StaticParameterGroup extends BaseParameterGroup {

		@XmlElement(name = "parameter")
		private List<Parameter> parameters;

		@Override
		public List<Parameter> getParameters() {
			return parameters;
		}
	}


	public static class StaticPortGroup extends BasePortGroup {

		@XmlElementRef
		private List<Port> ports = new ArrayList<>();

		@XmlElementRef
		private List<ClkPort> clkPorts = new ArrayList<>();

		private List<IPort> allPorts;

		@Override
		public List<? extends IPort> getPorts() {
			if (allPorts == null) {
				allPorts = new ArrayList<>();
				allPorts.addAll(ports);
				allPorts.addAll(clkPorts);
			}
			return allPorts;
		}

		public List<ClkPort> getClkPorts() {
			return clkPorts;
		}

		public List<Port> getRegularPorts() {
			return ports;
		}
	}
}
