/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml.jaxb;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import java.nio.file.Path;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.transform.sax.SAXSource;
import javax.xml.validation.Schema;

/**
 * Created by Ramon on 24.11.2015.
 */
public class Unmarshaller<T> {

	private static final Logger logger = LoggerFactory.getLogger(Unmarshaller.class);

	private XMLReader reader;
	private javax.xml.bind.Unmarshaller unmarshaller;

	public Unmarshaller(Schema schema, Class<T> baseClass) throws JAXBException {
		this(schema, new Class[]{baseClass});
	}

	public Unmarshaller(Schema schema, Class<? extends T>[] classes) throws JAXBException {
		//		SAXParserFactory factory = SAXParserFactory.newInstance();
		//		factory.setValidating(true);
		//		factory.setXIncludeAware(true);
		//		factory.setNamespaceAware(true);
		//		factory.setSchema(schema);
		//		try {
		//			reader = factory.newSAXParser().getXMLReader();
		//		} catch (SAXException e) {
		//			logger.error("Could not read in Schemas", e);
		//		} catch (ParserConfigurationException e) {
		//			logger.error("Could not configure parser", e);
		//		}

		unmarshaller = JAXBContext.newInstance(classes).createUnmarshaller();
		unmarshaller.setSchema(schema);
	}

	public T unmarshal(Path path, Class<T> singleClass) throws JAXBException {
		return unmarshaller.unmarshal(new SAXSource(new InputSource(path.toUri().toString())), singleClass).getValue();
	}

	public T unmarshal(Path path) throws JAXBException {
		return (T) unmarshaller.unmarshal(new SAXSource(new InputSource(path.toUri().toString())));
	}
}
