/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.devxml.mappedBuses;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.CombineType;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IPort;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;

import java.util.List;
import java.util.stream.Stream;

/**
 * Created by Ramon on 06.04.2016.
 */
public class MergedBusPort extends BaseMappedBusPort {

	protected final List<SingleMappedBusPort> ports;
	private final ConnectorDirection direction;
	private final IPort.Type type;
	private final boolean isPerConnection;

	protected MergedBusPort(String name, CombineType combineType, List<SingleMappedBusPort> mappedPorts, ConnectorDirection direction, IPort.Type type, boolean isPerConnection, Bus bus, IModuleDescription moduleDescription) {
		super(name, combineType, bus, moduleDescription);
		this.direction = direction;
		this.type = type;
		this.ports = mappedPorts;
		//		this.baseWidth = baseWidth;
		this.isPerConnection = isPerConnection;
	}

	public static MergedBusPort merge(List<SingleMappedBusPort> mappedPorts) {
		SingleMappedBusPort base = mappedPorts.get(0);//Pick any. Name is equal on all, due to them coming from a Map. rest is verified
		CombineType combineType = base.getCombineType();
		ConnectorDirection direction = base.getDirection();
		IConnectorDescription.Type type = base.getType();
		//		int baseWidth = base.getBaseWidth();
		IModuleDescription moduleDescription = base.getModuleDescription();

		boolean isPerConnection = combineType == CombineType.Parallel;

		for (SingleMappedBusPort p : mappedPorts) {
			if (p.combineType != combineType) {
				throw new BusMergingException(p, combineType);
			}
			if (!p.isAllowMerging()) {
				throw new BusMergingException(p);
			}
			if (p.getDirection() != direction) {
				throw new BusMergingException(p, direction);
			}
			if (p.getType() != type) {
				throw new BusMergingException(p, type);
			}

			//			if (baseWidth != p.getBaseWidth()) {
			//				throw new BusMergingException(p, "all ports must be of identical baseWidth");
			//			}
		}

		return new MergedBusPort(base.localName, base.combineType, mappedPorts, direction, type, isPerConnection, base.getBus(), moduleDescription);
	}

	@Override
	public IPort.Type getType() {
		return type;
	}

	@Override
	public boolean isAllowMerging() {
		return true;
	}

	@Override
	public ConnectorDirection getDirection() {
		return direction;
	}

	@Override
	public boolean isPerConnectionWidth() {
		return isPerConnection;
	}

	@Override
	public Stream<SingleMappedBusPort> streamSingleMappedBusPorts() {
		return ports.stream();
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		toStringIntro(sb);
		toStringPorts(sb);
		return sb.toString();
	}

	public void toStringPorts(StringBuilder sb) {
		sb.append(", combinedBy: ").append(combineType).append(" [");
		for (SingleMappedBusPort port : ports) {
			sb.append(port.getPort()).append(", ");
		}
		sb.append("]}");
	}

	public void toStringIntro(StringBuilder sb) {
		sb.append("Merged{").append(getCanonicalName());
	}

	@Override
	public <T> T apply(IConverter<T> converter) {
		return converter.convert(this, ports);
	}
}
