/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IApplicationConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;

import java.nio.file.Path;
import java.util.List;
import java.util.Optional;

/**
 * Created by Ramon on 26.06.2016.
 */
public interface ILibJConfigProject {

	String SYSTEM_DIR = "system";
	String BUILD_DIR = "build";
	String FIRMWARE_BUILD_DIR = "firmware";
	String FIRMWARE_SRC_DIR = "src";
	String FIRMWARE_INC_DIR = "include";
	String LEGACY_CONFIG_FILE_EXT = "xml";
	String JC3_CONFIG_FILE_EXT = "jc3";
	String LIBJCONFIG_VERSION = "libJConfig-3.0-alpha0";
	String AUTOSAVE_EXT = "autosave." + JC3_CONFIG_FILE_EXT;

	/**
	 * Compute the name of the autosave file, based on the current filename
	 *
	 * @param currentName file name, including extension
	 * @return autosave.jc3 / [name].autosave.jc3
	 */
	static String getAutosaveFileName(String currentName) {
		if (currentName != null) {
			return currentName.replaceFirst("^(.*\\.)(" + JC3_CONFIG_FILE_EXT + ")$", "$1" + AUTOSAVE_EXT);
		} else {
			return AUTOSAVE_EXT;
		}
	}

	/**
	 * Check whether autosave file has the same name as pick
	 * @param autosave an aussave file
	 * @param pick file to open
	 * @return true if autosave matches pick name
	 */
	static boolean doesAutosaveNameMatch(Path autosave, Path pick) {
		String prefix = "^(.*)\\.";
		String autoName = autosave.getFileName().toString().replaceFirst(prefix + AUTOSAVE_EXT + "$", "$1");
		String pickName = pick.getFileName().toString().replaceFirst(prefix + JC3_CONFIG_FILE_EXT + "$", "$1");
		return autoName.equals(pickName);
	}

	Optional<Path> getLastConfigFile();

	IApplicationConfiguration getApplication();

	IDocumentHandler getDocumentHandler();

	Path getProjectDir();

	Optional<Path> findNewestAutoSave();

	List<String> getKnownFirmwares(Path location);

	List<String> getKnownFirmwareSubNames(Path location, String firmwareName);

	String getLibJConfigVersion();

	Path getWorkingDir();

	Path getBuildDir();

	Path getIntermediaryBuildDir();

	boolean isNoVersionCheck();
}
