/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.api;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.calc.CalculatedValue;

import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlEnumValue;

/**
 * Created by Ramon on 12.05.2016.
 */
@XmlEnum
@SuppressWarnings("MagicNumber")
public enum BinaryOutputFormat {
	@XmlEnumValue("Spartan-ELF8")Spartan_ELF8(4, "SpartanMC 32bit ELF-Format", 4),
	@XmlEnumValue("Standard-ELF8")Standard_ELF8(4, "Standard 32bit ELF-Format", 1);

	private final int bytesPerTargetWord;
	private final String name;
	private final int minimumAlignment;

	BinaryOutputFormat(int bytesPerTargetWord, String name, int minimumAlignment) {
		this.bytesPerTargetWord = bytesPerTargetWord;
		this.name = name;
		this.minimumAlignment = minimumAlignment;
	}

	public int getBytesPerTargetWord() {
		return bytesPerTargetWord;
	}

	public int getBytesPerTargetByte() {
		return bytesPerTargetWord >> 1;
	}

	public String getName() {
		return name;
	}

	public long targetBytesToBytes(long value) {
		return value * getBytesPerTargetByte();
	}

	public long targetWordsToBytes(long value) {
		return value * getBytesPerTargetWord();
	}

	public String getToLowerTargetWordCOp() {
		return ">>" + CalculatedValue.AddressSize.getAddressSize(bytesPerTargetWord); // shift by how much to shift between target and ELF
	}

	public String getToUpperTargetWordCOp() {
		return ">>" + (CalculatedValue.AddressSize.getAddressSize(bytesPerTargetWord) + 9); // shift by 9 (upper byte) + how much to shift between target and ELF
	}

	public long targetWordsToTargetBytes(long value) {
		return value * 2;
	}

	/**
	 * Whether there is a additional alignment requirement for everything
	 * @return
	 */
	public boolean hasMinimumAlignment() {
		return minimumAlignment > 1;
	}

	/**
	 * Minimal required alignment for everything for valid binaries in bytes
	 * @return
	 */
	public int getMinimumAlignment() {
		return minimumAlignment;
	}
}
