/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.api;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.configurations.IToolchainConfiguration;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.INamedValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.devices.DeviceDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.MemoryModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.SubsystemModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.ILibJConfigProject;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IGenericAttributeContainer;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IModuleContainer;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBusHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPinHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.ISelectionHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IToolchainAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.ConfigurationIssue;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.IIssueManager;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.IIssueSink;

import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Stream;

/**
 * Created by ramon on 05.02.2016.
 */
public interface IConfigurationDocument extends IModuleContainer<IConfigurationDocument>, IConfigurationElement<TargetDescription.IoPin, TargetDescription.IoPinGroup, IPinHandler>, INamedValueResolver {

	int AUTO = IConnection.FULL_CONNECTOR_WIDTH;

	IDocumentHandler getDocumentHandler();

	BinaryOutputFormat getBinaryOutputFormat();

	void attachLibrary(Library library);

	void addDocumentListener(IDocumentChangeListener listener);

	void removeDocumentListener(IDocumentChangeListener listener);

	boolean addHierarchyListener(IConfigurationDocument.IConfigurationHierarchyListener listener);

	void removeHierarchyListener(IConfigurationDocument.IConfigurationHierarchyListener listener);

	ISubsystemConfiguration<? extends IConfigurationDocument> addSubsystem(SubsystemModuleDescription moduleDescription);

	ISubsystemConfiguration<? extends IConfigurationDocument> addSubsystem(SubsystemModuleDescription moduleDescription, @Nullable String name);

	IMemoryModuleConfiguration<? extends IConfigurationDocument> addMemoryModule(MemoryModuleDescription moduleDescription);

	ILibJConfigProject getProject();

	String getJconfigVersion();

	Library getLibrary();

	IConfigurationDocument.ITargetConfiguration getTarget();

	List<? extends ISubsystemConfiguration<?>> getSubsystems();

	void checkResolvedIssues();

	void addConfigurationIssueListener(IConfigurationIssueListener listener);

	void removeConfigurationIssueListener(Object listener);

	Set<ConfigurationIssue> getConfigurationIssues();

	IConnection<? extends IPortHandler<?>> createPortConnection(IPortHandler<? extends IConnectorDescription> a, IPortHandler<? extends IConnectorDescription> b);

	IConnection<? extends IBusHandler> createBusConnection(IBusHandler a, IBusHandler b);

	IConnection<? extends IPortHandler<?>> createPartialConnection(IPortHandler<? extends IConnectorDescription> a, int aStart, IPortHandler<? extends IConnectorDescription> b, int bStart, int width);

	Set<? extends IConnection<? extends IPortHandler<?>>> getConnections();

	Set<? extends IConnection<? extends IBusHandler>> getBusConnections();

	IIssueSink getIssueSink();

	IIssueManager getIssueManager();

	Stream<? extends ISubsystemConfiguration<?>> streamSubsystems();

	Stream<? extends IModuleConfiguration<? extends IConfigurationDocument>> streamCommonModules();

	Optional<IPinHandler> getDefaultPin(String function);

	/**
	 * Listener for changes in the document
	 * i.e. changes in document hierarchy. All removing / adding of elements through this class!!
	 */
	interface IConfigurationHierarchyListener {

		/**
		 * A new module has been added to this document.
		 * You can find the parent using {@link IModuleConfiguration#getParent()}
		 *
		 * @param module the newly added document. pluginEngine set if possible, but before connections
		 */
		void onModuleAdded(IModuleConfiguration<?> module);

		/**
		 * A module (and its submodules) have been removed from this document
		 *
		 * @param parent the parent from which was removed
		 * @param module the already removed module
		 */
		void onModuleRemoved(IConfigurationElement<?, ?, ?> parent, IModuleConfiguration module);

		/**
		 * An Element has changed its name.
		 * Although in principal a {@link IElementChangeListener} event, needed to update the document tree
		 *
		 * @param element the element in question
		 */
		void onElementNameChanged(IConfigurationElement<?, ?, ?> element);
	}


	interface IConfigurationIssueListener {

		void onNewConfigurationIssue(ConfigurationIssue warning, ConfigurationIssue.IssueType issueType);

		void onConfigurationIssueDismissed(ConfigurationIssue issue);

		void onConfigurationIssueResolved(ConfigurationIssue issue);
	}


	interface IDocumentChangeListener {

		void onTargetChanged();
	}


	public static class Define {

		final String name;
		final String value;

		public Define(String name, String value) {
			this.name = name;
			this.value = value;
		}

		public String getName() {
			return name;
		}

		public String getValue() {
			return value;
		}

		@Override
		public String toString() {
			return format(false, false, '=');
		}

		public String format(boolean quoteEverything, boolean quoteValue, char nameValueSeparator) {
			StringBuilder result = new StringBuilder();
			if (quoteEverything) {
				result.append('"');
			}
			result.append(name);
			if (value != null) {
				result.append(nameValueSeparator);
				if (quoteValue) {
					result.append('"');
				}
				result.append(value);
				if (quoteValue) {
					result.append('"');
				}
			}
			if (quoteEverything) {
				result.append('"');
			}
			return result.toString();
		}
	}


	interface ITargetConfiguration extends IGenericAttributeContainer {

		String ATTRIB_UC_FILE = "ucFile";
		String ATTRIB_UC_DATA = "ucData";

		ISelectionHandler<TargetDescription> getTargetSelectionHandler();

		ISelectionHandler<DeviceDescription> getDeviceSelectionHandler();

		ISelectionHandler<DeviceDescription.Toolchain> getToolchainSelectionHandler();

		IToolchainAttributeHandler getToolchainParamHandler(DeviceDescription.DevParam paramDescription);

		TargetDescription getTargetDescription();

		DeviceDescription getDeviceDescription();

		TargetDescription.Device getDevice();

		IToolchainConfiguration getCurrentConfiguration();

		DeviceDescription.Toolchain getToolchain();

		/**
		 * Get the defines that identify this target for verilog code
		 * @return list of verilog macro names to be defined
		 */
		Stream<Define> streamVerilogDefines(boolean simulation);
	}

}
