/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.api;

import de.tu_darmstadt.rs.spartanmc.jconfig.ILibJConfigProject;
import de.tu_darmstadt.rs.util.SpartanMCFirmwareUtil;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Optional;

/**
 * Created by Ramon on 13.05.2016.
 */
public interface IFirmwareConfiguration {

	static boolean isSubFirmwareFolder(Path dir) {
		return true;//have to design folder structure first;
	}

	boolean hasExternalLocation();

	Path getExternalLocation();

	String getFirmwareName();

	Optional<String> getSubFirmwareName(ISubsystemConfiguration subsystem);

	Path getSubFirmwareLocation(ISubsystemConfiguration subsystem, ILibJConfigProject project);

	/**
	 * Get a path to firmware folder. Whether it is in workingDir or not
	 *
	 * @param project project to find out workingDir
	 * @return a path to dir
	 */
	default Path getLocation(ILibJConfigProject project) {
		Path base = hasExternalLocation()? getExternalLocation() : project.getWorkingDir();
		String firmwareName = getFirmwareName();
		if (firmwareName == null) {
			return null;
		}
		return base.resolve(firmwareName);
	}

	enum FirmwareState {
		Valid,
		Invalid,
		NotSet
	}

	/**
	 * Check if the current firmware is a valid firmware (all files present)
	 *
	 * @param project current project, used to resolve files
	 * @return true if valid
	 */
	default FirmwareState isValidFirmware(ILibJConfigProject project) {

		Path firmwareLocation = getLocation(project);

		if (firmwareLocation == null) {
			//Systems without firmwares are allowed
			return FirmwareState.NotSet;
		}

		if (!Files.exists(firmwareLocation) || !SpartanMCFirmwareUtil.isValidFirmwareFolderStructure(firmwareLocation)) {
			return FirmwareState.Invalid;
		}

		//		Path subLocation = firmwareLocation.resolve(firmware.getSubName());

		//		return Files.exists(subLocation) && isSubFirmwareFolder(subLocation);
		return FirmwareState.Valid;//TODO
	}

	boolean hasFirmware();

	boolean hasSubFirmwares();

	Optional<String> getGlobalFirmwareName();
}
