/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.api;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.INamedValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.MemoryModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IModuleContainer;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IFirmwareAttributeHandler;

import java.util.List;
import java.util.stream.Stream;

/**
 * A Module that
 * <p>
 * Created by ramon on 17.03.2016.
 */
public interface IMemoryModuleConfiguration<P extends IModuleContainer & IConfigurationElement<?, ?, ?> & INamedValueResolver> extends IModuleConfiguration<P> {

	Stream<ISubsystemConfiguration> getConnectedSubsystems();

	List<MemoryModuleDescription.ExtMemory> getMemoriesExposedOn(Bus exposedOn);

	@Override
	MemoryModuleDescription getModuleDescription();

	/**
	 * Register for events on the firmware
	 *
	 * @param listener event listener
	 */
	void addMemoryListener(IMemoryChangeListener listener);

	/**
	 * Unregister for firmware change events
	 *
	 * @param listener listener
	 */
	void removeMemoryListener(IMemoryChangeListener listener);

	IFirmwareConfiguration getFirmware();

	/**
	 * Entry point for changing firmware options and finding firmwares in the current project
	 *
	 * @return firmware handler
	 */
	IFirmwareAttributeHandler getFirmwareLocationHandler();

	/**
	 * Get the actual size of memory in words. Resolved against parameters on this module.
	 *
	 * @param memory memory to resolve
	 * @return memory size in words
	 */
	long getResolvedMemorySize(MemoryModuleDescription.ExtMemory memory);

	/**
	 * Events on firmware
	 */
	interface IMemoryChangeListener {

		/**
		 * A different firmware was picked
		 *
		 * @param element this element
		 */
		void onFirmwareChanged(IMemoryModuleConfiguration element);
	}

}
