/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.api;

import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.MemoryModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.PeripheralModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.SubsystemModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IModuleContainer;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayoutMapping;

import org.jetbrains.annotations.Nullable;

import java.util.stream.Stream;

/**
 * Created by ramon on 05.02.2016.
 */
public interface ISubsystemConfiguration<P extends IConfigurationDocument> extends IModuleConfiguration<P>, IModuleContainer<ISubsystemConfiguration<?>> {

	P getParent();

	SubsystemModuleDescription getModuleDescription();

	/**
	 * Convenience method, returning more detailed configuration type
	 *
	 * @param moduleDescription description of peripheral to add
	 * @return newly created peripheral module
	 */
	IPeripheralModuleConfiguration addPeripheral(PeripheralModuleDescription moduleDescription);

	/**
	 * Convenience method, returning more detailed configuration type
	 *
	 * @param moduleDescription description of peripheral to add
	 * @param name explicit name
	 * @return newly created peripheral module
	 */
	IPeripheralModuleConfiguration addPeripheral(PeripheralModuleDescription moduleDescription, @Nullable String name);

	/**
	 * Convenience method, returning more detailed configuration type
	 *
	 * @param moduleDescription description of common module to add
	 * @return newly created module
	 */
	IModuleConfiguration<? extends ISubsystemConfiguration> addCommonModule(IModuleDescription moduleDescription);//TODO unify? Container can distinguish descriptions, even by getType()

	/**
	 * Convenience method, returning more detailed configuration type
	 *
	 * @param moduleDescription description of memory to add
	 * @return newly created memory module
	 */
	IMemoryModuleConfiguration<? extends ISubsystemConfiguration> addMemoryModule(MemoryModuleDescription moduleDescription);

	Stream<? extends IPeripheralModuleConfiguration<? extends ISubsystemConfiguration>> streamPeripherals();

	Stream<? extends IMemoryModuleConfiguration<? extends ISubsystemConfiguration>> streamMemoryModules();

	Stream<? extends IModuleConfiguration<? extends ISubsystemConfiguration>> streamCommonModules();

	@Deprecated
	/**
	 * @deprecated because this has been extended to return multiple and their is currently no scheme to select 1, and linker gen will not support it anyway
	 */
	Stream<IMemoryModuleConfiguration<?>> streamExportingMemoryConfiguration();

	boolean hasStaticAddressSpaceMappings();

	AddressLayoutMapping getStaticAddressLayoutMapping();

	void setStaticAddressLayoutMapping(AddressLayoutMapping layoutMapping);

	//	void setMemoryConfiguration(IMemoryModuleConfiguration memoryConfiguration);
}
