/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;

import java.util.Optional;
import java.util.stream.Stream;

/**
 * ConnectorHandler that can manage partial connections
 * Created by Ramon on 28.04.2016.
 */
public interface IPartialConnectorHandler<PH extends IConnectorHandler<?, ?>, CD extends IConnectorDescription> extends IConnectorHandler<PH, CD> {

	/**
	 * Convenience Method to wrap calls to {@link #isPartiallyConnectibleTo(IConnectorHandler)} with checks for the handler in question even supporting partial connections at all
	 *
	 * @param portHandler a ConnectorHandler that may or may not support partial connections
	 * @param otherHandler any connectorHandler
	 * @return true if portHandler can be partially connected to otherHandler
	 */
	static boolean isPartiallyConnectible(IConnectorHandler<?, ?> portHandler, IConnectorHandler<?, ?> otherHandler) {
		Optional<? extends IPartialConnectorHandler<?, ?>> partialConnectorHandler = portHandler.getPartialConnectorHandler();
		return partialConnectorHandler.isPresent() && partialConnectorHandler.get().isPartiallyConnectibleTo(otherHandler);
	}

	/**
	 * Stream Partial Connections only
	 *
	 * @return stream of partial connections
	 */
	Stream<? extends IConnection<PH>> streamPartialConnections();

	/**
	 * Query whether this handler currently holds partial connections
	 *
	 * @return true if there are partial connections. Will throw IllegalAccessException if there are both full &amp; partial connections
	 */
	boolean isPartiallyConnected();

	boolean isOverlappingAllowed();

	boolean isAdditionalPartialConnectionAllowed();

	boolean isAdditionalFullConnectionAllowed();

	/**
	 * Calculates whether a part of this connector can be connected to a different port
	 *
	 * @param toPort target port
	 * @return true if a partial connection is possible in principle (connector may be full)
	 */
	boolean isPartiallyConnectibleTo(IConnectorHandler toPort);
}
