/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.buildManager;

import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.DocumentBuilder;

import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Stream;

/**
 * Created by Ramon on 17.05.2016.
 */
public class BuildState {

	private List<BuildException> exceptions = new ArrayList<>();
	private List<BuildResult.BuiltGroup> built = new ArrayList<>();

	public BuildState() {
		super();
	}

	void addException(BuildException e) {
		exceptions.add(e);
	}

	void addBuilt(DocumentBuilder.IOutputGroup group, List<MessageDigest> digests) {
		built.add(new BuildResult.BuiltGroup(group, digests));
	}

	public List<BuildException> getExceptions() {
		return exceptions;
	}

	public List<BuildResult.BuiltGroup> getBuilt() {
		return built;
	}

	BuildState merge(BuildState toMerge) {
		exceptions.addAll(toMerge.exceptions);
		built.addAll(toMerge.built);
		return this;
	}

	public void verifyUniqueOutputGroups(List<? extends DocumentBuilder.IOutputGroup> upToDate) {
		HashSet<String> names = new HashSet<>();
		Stream.concat(built.stream().map(BuildResult.BuiltGroup::getOutputGroup), upToDate.stream()).map(DocumentBuilder.IOutputGroup::getName).forEach(name -> {
			if (!names.add(name)) {
				throw new IllegalStateException("OutputGroups MUST have unique names! " + name);
			}
		});
	}
}
