/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.languages;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBooleanParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IFloatParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IIntegerParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterConverter;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.HeaderGen;
import de.tu_darmstadt.rs.util.GroupWriter;

/**
 * Created by Ramon on 11.05.2016.
 */
public final class CLang {

	public static final String MULTI_COMMENT_START = "/*";
	public static final String MULTI_COMMENT_END = " */";
	public static final String MULTI_COMMENT_SPACER = " *";
	public static final String SINGLE_COMMENT = "// ";

	private static final HeaderGen cHeaderGen = new HeaderGen(MULTI_COMMENT_START, MULTI_COMMENT_END, MULTI_COMMENT_SPACER);
	private static CConverter cConverter = new CConverter();

	private CLang() {}

	public static GroupWriter.ILineMapper<Void> getPragmaOnce() {
		return GroupWriter.ILineMapper.singleLine(aVoid -> "\n#pragma once\n");
	}

	public static String getCDefineName(String name) {
		return name.toUpperCase().replaceAll("[^_A-Z0-9]", "_");
	}

	public static String getCVarName(String name) {
		String s = name.replaceAll("[^-_A-Za-z0-9]", "_");
		return Character.toLowerCase(s.charAt(0)) + s.substring(1);
	}

	public static String getExposedPeriVarName(IPeripheralModuleConfiguration p) {
		return getCVarName(p.getGlobalName());
	}

	public static String getExposedPeriPtrDefineName(IPeripheralModuleConfiguration p) {
		return getCDefineName(p.getName());
	}


	public static String getExposedDmaVarName(IPeripheralModuleConfiguration p) {
		return getCVarName(p.getGlobalName()+"_dma");
	}

	public static String getExposedDmaPtrDefineName(IPeripheralModuleConfiguration p) {
		return getCDefineName(p.getName()+"_DMA");
	}
	public static HeaderGen getCHeaderGen() {
		return cHeaderGen;
	}

	public static IParameterConverter getParameterConverter() {
		return cConverter;
	}

	/**
	 * Created by Ramon on 04.04.2016.
	 */
	private static final class CConverter implements IParameterConverter {

		private CConverter() {}

		@Override
		public String convert(IIntegerParameterHandler param) {
			return param.getValue().get();
		}

		@Override
		public String convert(IFloatParameterHandler param) {
			return param.getValue().get();
		}

		@Override
		public String convert(IParameterHandler param) {
			return String.format("\"%s\"", param.getValue().get());
		}

		@Override
		public String convert(IBooleanParameterHandler param) {
			return param.isSet()? "1" : "0";
		}
	}
}
