/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.interchange.memory;

import de.tu_darmstadt.rs.jaxbUtil.SchemaLessJaxB;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.HeaderGen;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.file.Path;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

public class MemoryMapJaxB extends SchemaLessJaxB<MemoryMap> {

	private static final Logger logger = LoggerFactory.getLogger(MemoryMapJaxB.class);

	public MemoryMapJaxB() {
		super(MemoryMap.class, new QName("http://www.rs.tu-darmstadt.de/ns/spartanmc/memory/jc3", "memory-map", "jc3"));
	}

	@Override
	protected JAXBContext createContext() throws JAXBException {
		return JAXBContext.newInstance(MemoryMap.class);
	}

	public void marshall(MemoryMap map, XMLStreamWriter streamWriter, Path configFile) throws JAXBException {
		Marshaller marshaller = createMarshaller();

		marshaller.setListener(new MarshallerListener(streamWriter, map, configFile));

		marshaller.marshal(createJaxBElement(map), streamWriter);
	}

	private static class MarshallerListener extends Marshaller.Listener {

		private final XMLStreamWriter streamWriter;
		private final Object root;
		private final Path configFile;

		public MarshallerListener(XMLStreamWriter streamWriter, Object root, Path configFile) {
			this.streamWriter = streamWriter;
			this.root = root;
			this.configFile = configFile;
		}

		@Override
		public void beforeMarshal(Object source) {
			if (source == root) {
				try {
					streamWriter.writeComment(new HeaderGen("").getHeaderString("Mapping of firmware address ranges to blockram instances for InitramJ", configFile));
					streamWriter.writeCharacters("\n");
				} catch (XMLStreamException e) {
					logger.error("StreamWriter comment", e);
				}
			}
		}
	}
}
