/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.issues;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;

import org.jetbrains.annotations.NotNull;

import java.util.Optional;

/**
 * Created by Ramon on 21.04.2016.
 */
public class DescriptionIssue extends ConfigurationIssue {

	@NotNull
	private final IConfigurationElement<?, ?, ?> parent;
	private final String name;
	private final String moduleDescriptionName;

	public DescriptionIssue(@NotNull IConfigurationElement<?, ?, ?> parent, String name, String moduleDescriptionName, String messsage, String situation, IssueType level) {
		super(messsage, situation, level);
		this.parent = parent;
		this.name = name;
		this.moduleDescriptionName = moduleDescriptionName;
	}

	@Override
	public String getHintText() {
		StringBuilder sb = new StringBuilder();

		if (parent != null) {
			sb.append("Parent: ").append(parent.getName());
		}
		sb.append(", Module: ").append(name);
		sb.append(", Description: ").append(moduleDescriptionName);

		return sb.toString();
	}

	@Override
	public Actuality getActuality() {
		return Actuality.Unknown;
	}

	@Override
	public Optional<IConfigurationElement<?, ?, ?>> getReferencedModule() {
		return Optional.of(parent);
	}

	@Override
	public int hashCode() {
		int result = super.hashCode();
		result = 31 * result + (parent != null? parent.hashCode() : 0);
		result = 31 * result + (name != null? name.hashCode() : 0);
		result = 31 * result + (moduleDescriptionName != null? moduleDescriptionName.hashCode() : 0);
		return result;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}
		if (!super.equals(o)) {
			return false;
		}

		DescriptionIssue that = (DescriptionIssue) o;

		if (parent != null? !parent.equals(that.parent) : that.parent != null) {
			return false;
		}
		if (name != null? !name.equals(that.name) : that.name != null) {
			return false;
		}
		return moduleDescriptionName != null? moduleDescriptionName.equals(that.moduleDescriptionName) : that.moduleDescriptionName == null;

	}
}
