/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.issues;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;

import java.util.Optional;
import java.util.function.Supplier;

/**
 * @author ramonw
 */
public class GenericIssue extends ConfigurationIssue {

	private final IConfigurationElement<?, ?, ?> ref;
	private final Supplier<String> hint;
	private final Supplier<Actuality> actuality;

	public GenericIssue(IConfigurationElement<?, ?, ?> ref, String message, String situation, IssueType level, Supplier<String> hint, Supplier<Actuality> actuality) {
		super(message, situation, level);
		this.ref = ref;
		this.hint = hint;
		this.actuality = actuality;
	}

	@Override
	public String getHintText() {
		return hint.get();
	}

	@Override
	public Actuality getActuality() {
		return actuality.get();
	}

	@Override
	public Optional<IConfigurationElement<?, ?, ?>> getReferencedModule() {
		return Optional.of(ref);
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof GenericIssue)) {
			return false;
		}
		if (!super.equals(o)) {
			return false;
		}

		GenericIssue that = (GenericIssue) o;

		if (!ref.equals(that.ref)) {
			return false;
		}
		return hint.get().equals(that.hint.get());
	}

	@Override
	public int hashCode() {
		int result = super.hashCode();
		result = 31 * result + ref.hashCode();
		return result;
	}
}
