/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.issues;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Optional;

/**
 * Issue that describes a missing file
 */
public class MissingFileIssue extends ConfigurationIssue {

	private final Path file;
	private final IAttributeHandler handler;
	private final CalculatedValueResult<String> handlerValue;

	/**
	 * Create the issue
	 * @param file file that is missing
	 * @param handler handler that created the reference
	 * @param message a Message
	 * @param situation Situation when issue was detected
	 * @param level Issue level
	 */
	public MissingFileIssue(Path file, IAttributeHandler handler, String message, String situation, IssueType level) {
		super(message, situation, level);
		this.file = file;
		this.handler = handler;
		this.handlerValue = handler != null? handler.getValue() : null;
	}

	/**
	 * Create the issue
	 * @param file file that is missing
	 * @param message a Message
	 * @param situation Situation when issue was detected
	 * @param level Issue level
	 */
	public MissingFileIssue(Path file, String message, String situation, IssueType level) {
		this(file, null, message, situation, level);
	}

	@Override
	public String getHintText() {
		return file.toString();
	}

	@Override
	public Actuality getActuality() {
		//No need to check if the new referenced file exists, because the build will create a new issue for that
		boolean stillReferenced = handler != null && handlerValue.equals(handler.getValue());
		return Files.exists(file) || !stillReferenced? Actuality.Resolved : Actuality.Current;
	}

	@Override
	public Optional<IConfigurationElement<?, ?, ?>> getReferencedModule() {
		return Optional.empty();
	}
}
