/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.issues;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IMemoryAddressBlock;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

public class SectionLayoutIssue extends ConfigurationIssue {

	private final ILinkerSection linkerSection;
	private final IMemoryAddressBlock picked;
	private final List<? extends IMemoryAddressBlock> alternatives;

	public SectionLayoutIssue(ILinkerSection linkerSection, IMemoryAddressBlock picked, List<? extends IMemoryAddressBlock> alternatives, String message, String situation, IssueType level) {
		super(message, situation, level);
		this.linkerSection = linkerSection;
		this.picked = picked;
		this.alternatives = alternatives;
	}

	@Override
	public String getHintText() {
		return "Placed "+ linkerSection.getName() +" into "+ picked +", " +
				"possible locations: "+ alternatives;
	}

	@Override
	public Actuality getActuality() {
		return Actuality.Unknown;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}
		if (!super.equals(o)) {
			return false;
		}
		SectionLayoutIssue that = (SectionLayoutIssue) o;
		return Objects.equals(linkerSection, that.linkerSection) &&
				Objects.equals(picked, that.picked) &&
				Objects.equals(alternatives, that.alternatives);
	}

	@Override
	public int hashCode() {

		return Objects.hash(super.hashCode(), linkerSection, picked, alternatives);
	}

	@Override
	public Optional<IConfigurationElement<?, ?, ?>> getReferencedModule() {
		return Optional.empty();
	}
}
