/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting;

import de.tu_darmstadt.rs.spartanmc.devxml.jaxb.LongDecodeAdapter;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

/**
 * Created by Ramon on 18.05.2016.
 */
@JsonAutoDetect(fieldVisibility = JsonAutoDetect.Visibility.ANY)
public final class AddressLayoutMapping {

	private @NotNull List<AddressSpaceMapping> spaceMappings = new ArrayList<>();

	public AddressLayoutMapping(@NotNull List<AddressSpaceMapping> spaceMappings) {
		this.spaceMappings = spaceMappings;
	}

	private AddressLayoutMapping() {
	}

	@NotNull
	public List<AddressSpaceMapping> getSpaceMappings() {
		return spaceMappings;
	}

	@Override
	public int hashCode() {
		return spaceMappings.hashCode();
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		AddressLayoutMapping that = (AddressLayoutMapping) o;

		return spaceMappings.equals(that.spaceMappings);

	}


	public static class AddressSpaceMapping {

		@XmlAttribute
		private String space;

		@XmlElement(name = "addressRangeMapping")
		@NotNull
		private List<AddressRangeMapping> rangeMappings = new ArrayList<>();

		private AddressSpaceMapping() {

		}

		public AddressSpaceMapping(@NotNull String space, @NotNull List<AddressRangeMapping> rangeMappings) {
			this.space = space;
			this.rangeMappings = rangeMappings;
		}

		public @NotNull String getSpace() {
			return space;
		}

		public @NotNull List<AddressRangeMapping> getRangeMappings() {
			return rangeMappings;
		}

		@Override
		public String toString() {
			return "AddressSpaceMapping{" +
				"space='" + space + '\'' +
				", rangeMappings=" + rangeMappings +
				'}';
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) {
				return true;
			}
			if (o == null || getClass() != o.getClass()) {
				return false;
			}

			AddressSpaceMapping that = (AddressSpaceMapping) o;

			if (!space.equals(that.space)) {
				return false;
			}
			return rangeMappings.equals(that.rangeMappings);

		}

		@Override
		public int hashCode() {
			int result = space.hashCode();
			result = 31 * result + rangeMappings.hashCode();
			return result;
		}
	}


	public static final class AddressRangeMapping {

		@XmlAttribute
		private String range;

		@XmlJavaTypeAdapter(LongDecodeAdapter.class)
		@XmlAttribute
		@NotNull
		private Long start = 0L;

		@XmlElement(name = "addressBlockMapping")
		private List<AddressBlockMapping> blockMappings;

		private AddressRangeMapping() {

		}

		public AddressRangeMapping(@NotNull String range, long start, @NotNull List<AddressBlockMapping> blockMappings) {
			this.range = range;
			this.start = start;
			this.blockMappings = blockMappings;
		}

		public @NotNull String getRange() {
			return range;
		}

		public long getStart() {
			return start;
		}

		public List<AddressBlockMapping> getBlockMappings() {
			return blockMappings;
		}

		@Override
		public String toString() {
			return "AddressRangeMapping{" +
				"range='" + range + '\'' +
				", start==0x" + Long.toHexString(start) +
				", blockMappings=" + blockMappings +
				'}';
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) {
				return true;
			}
			if (o == null || getClass() != o.getClass()) {
				return false;
			}

			AddressRangeMapping that = (AddressRangeMapping) o;

			if (!range.equals(that.range)) {
				return false;
			}
			if (!start.equals(that.start)) {
				return false;
			}
			return blockMappings.equals(that.blockMappings);

		}

		@Override
		public int hashCode() {
			int result = range.hashCode();
			result = 31 * result + start.hashCode();
			result = 31 * result + blockMappings.hashCode();
			return result;
		}
	}

	@Override
	public String toString() {
		return "AddressLayoutMapping{" +
			"spaceMappings=" + spaceMappings +
			'}';
	}


	public static final class AddressBlockMapping {

		@XmlAttribute
		private String block;

		@XmlJavaTypeAdapter(LongDecodeAdapter.class)
		@XmlAttribute
		@NotNull
		private Long start = 0L;

		private AddressBlockMapping() {

		}

		public AddressBlockMapping(@NotNull String name, long actualStart) {
			this.block = name;
			this.start = actualStart;
		}

		public @NotNull String getBlock() {
			return block;
		}

		public long getStart() {
			return start;
		}

		@Override
		public String toString() {
			return "AddressBlockMapping{" +
				"block='" + block + '\'' +
				", start=0x" + Long.toHexString(start) +
				'}';
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) {
				return true;
			}
			if (o == null || getClass() != o.getClass()) {
				return false;
			}

			AddressBlockMapping that = (AddressBlockMapping) o;

			if (!block.equals(that.block)) {
				return false;
			}
			return start.equals(that.start);

		}

		@Override
		public int hashCode() {
			int result = block.hashCode();
			result = 31 * result + start.hashCode();
			return result;
		}
	}


}
