/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.memory.AddressLayoutSpec;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBusHandler;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Optional;

/**
 * Created by ramon on 23.03.2016.
 */
public class AddressSpaceHandler extends AddressBlockContainer<IAddressRangeHandler> {

	protected static final Logger logger = LoggerFactory.getLogger(AddressSpaceHandler.class);

	protected final ISubsystemConfiguration subsystem;
	private final AddressLayoutSpec.AddressSpace space;
	private final IGlobalAddressLayout global;

	AddressSpaceHandler(ISubsystemConfiguration subsystem, AddressLayoutSpec.AddressSpace space, IGlobalAddressLayout global) {
		this.subsystem = subsystem;
		this.space = space;
		this.global = global;
		setMinStart(space.getStart());
	}

	public boolean isValid() {
		long address = space.getStart();
		for (IAddressRangeHandler range : addressBlocks) {
			if (range.getActualStart() < address || !range.isValid()) {
				return false;
			}
			address = range.getActualEnd();
		}
		return address <= space.getEnd();
	}

	public Optional<IAddressRangeHandler> findRangeHandler(String range) {
		return addressBlocks.stream().filter(rh -> rh.getName().equals(range)).findAny();
	}

	public Optional<IAddressRangeHandler> findRangeHandler(IBusHandler connectedTo) {
		return addressBlocks.stream()
				.filter(rh -> rh instanceof AddressRangeHandler && ((AddressRangeHandler) rh).busHandler.streamConnections().anyMatch(c -> c.doesConnect(connectedTo))).findAny();
	}


	public String getName() {
		return space.getName();
	}

	public void layoutSelf() {
		space.getAddressRanges().stream().map(range -> AddressRangeHandler.create(range, subsystem, global)).forEachOrdered(range -> layoutBlock(range, this));
	}

	@Override
	protected RuntimeException createDoesNotFitException(IAddressRangeHandler range) {
		return new IllegalStateException(range + " does not fit into " + space + " on " + subsystem);
	}

	@Override
	protected long findNextPossibleAddress(IAddressRangeHandler range) {
		return Math.max(range.getMinStart(), nextAddress);
	}

	@Override
	protected long getMaxEnd() {
		return space.getEnd();
	}

	@Override
	public long getMinStart() {
		return space.getStart();
	}

	@Override
	public String toString() {
		return "Space{" +
			"name=" + space.getName() +
			", on=" + subsystem.getName() +
			", " + super.toString();
	}
}

