/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.memory.AddressLayoutSpec;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;

import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * Created by Ramon on 27.03.2016.
 */
class MergedRangeHandler extends AddressBlockContainer<AddressBlock<?>> implements IAddressRangeHandler {

	private final List<AddressLayoutSpec.AddressRange> combining;

	/**
	 *
	 * @param combining all RangeSpecs that are merged by this range
	 * @param blocks non-empty ordered list of blocks in this range
	 */
	public MergedRangeHandler(@NotNull List<AddressLayoutSpec.AddressRange> combining, @NotNull List<AddressBlock<?>> blocks) {
		this.combining = combining;
		this.addressBlocks.addAll(blocks.stream().peek(block -> block.rangeHandler = this).collect(Collectors.toList()));
		this.actualStart = blocks.get(0).getActualStart();
		this.nextAddress = blocks.get(blocks.size()-1).getActualEnd();
	}

	@Override
	public RuntimeException createDoesNotFitException(AddressBlock block) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Optional<AddressBlock<?>> findBlock(String name) {
		return addressBlocks.stream().filter(b -> b.getSourceModule().getName().equals(name)).findAny();
	}

	@Override
	public Optional<AddressBlock<?>> findBlock(IPeripheralModuleConfiguration peripheral) {
		return findBlock(peripheral.getName());
	}

	@Override
	public String getName() {
		return "MergedRange";
	}

	@Override
	public long findNextPossibleAddress(AddressBlock block) {
		throw new UnsupportedOperationException();
	}

	@Override
	public long getMaxEnd() {
		throw new UnsupportedOperationException();
	}

	@Override
	public void layoutSelf(AddressSpaceHandler analyzer) {
		throw new UnsupportedOperationException();
	}

	@Override
	public AddressLayoutSpec.BlockType getBlockType() {
		throw new UnsupportedOperationException();
	}

	@Override
	public String toString() {
		return "MergeRangeHandler{" +
			"combining=" + combining +
			", blocks=" + addressBlocks + " }";
	}

	@Override
	public boolean isValid() {
		return isValid(actualStart, nextAddress);
	}

	@Override
	public Optional<IGlobalAddressLayout> getGlobal() {
		return Optional.empty();
	}

	@Override
	public boolean hasFixedStart() {
		return true;
	}

	@Override
	public boolean isReadable() {
		return combining.stream().anyMatch(AddressLayoutSpec.AddressRange::isReadable);
	}

	@Override
	public boolean isWritable() {
		return combining.stream().anyMatch(AddressLayoutSpec.AddressRange::isWritable);
	}

	@Override
	public boolean isExecutable() {
		return combining.stream().anyMatch(AddressLayoutSpec.AddressRange::isExecutable);
	}

	@Override
	public int getAlignmentExp() {
		return 0;
	}

	@Override
	public void setActualStart(long start) {
		throw new UnsupportedOperationException();
	}

	@Override
	public long getMinStart() {
		throw new UnsupportedOperationException();
	}

	@Override
	public AddressLayoutSpec.AddressRange getDescription() {
		throw new UnsupportedOperationException();
	}
}
