/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.BusPort;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.CombineType;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.mappedBuses.SingleMappedBusPort;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBusHandler;

/**
 * Created by ramonw on 12.04.16.
 */
public abstract class BaseMappedBusPortInstance implements IMappedBusPart {

	protected final IBusHandler handler;
	protected final SingleMappedBusPort org;

	public BaseMappedBusPortInstance(SingleMappedBusPort mappedBusPort, IBusHandler handler) {
		this.org = mappedBusPort;
		this.handler = handler;
	}

	@Override
	public IModuleDescription getModuleDescription() {
		return org.getModuleDescription();
	}

	@Override
	public String getPortName() {
		return org.getPortName();
	}

	@Override
	public IConnectorDescription.Type getType() {
		return org.getType();
	}

	@Override
	public boolean isAllowMerging() {
		return org.isAllowMerging();
	}

	@Override
	public CombineType getCombineType() {
		return org.getCombineType();
	}

	@Override
	public ConnectorDirection getDirection() {
		return org.getDirection();
	}

	@Override
	public boolean isPerConnectionWidth() {
		return org.isPerConnectionWidth();
	}

	@Override
	public IConfigurationElement<?, ?, ?> getSourceElement() {
		return handler.getSourceElement();
	}

	@Override
	public boolean isConnected() {
		return handler.isConnected();
	}

	@Override
	public IBusHandler getHandler() {
		return handler;
	}

	@Override
	public BusPort getPort() {
		return org.getPort();
	}

	public boolean hasCalculatedBaseWidth() {
		return org.hasCalculatedBaseWidth();
	}

	public int getStaticBaseWidth() {
		return org.getStaticBaseWidth();
	}

	public CalculatedValueResult<Integer> resolveBaseWidth(IValueResolver resolver) {
		return org.resolveBaseWidth(resolver);
	}

	public int getBaseWidth() {
		if (hasCalculatedBaseWidth()) {
			return resolveBaseWidth(handler).orElseThrow();
		} else {
			return getStaticBaseWidth();
		}
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("SingleInst{").append(getCanonicalName());
		org.toStringDescProp(sb);
		return sb.toString();
	}
}
