/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Kris Heid on 13.01.17.
 */
class BusPortPlacement {

	private static final Logger logger = LoggerFactory.getLogger(BusPortPlacement.class);
	private final List<MappedBusConnectionStub> connections = new ArrayList<>();
	private final IMappedBusPortInstance placingOn;
	private final boolean increment;
	private final boolean forceSingle;
	private int current = 0;
	private IMappedBusPortInstance firstTarget;

	public BusPortPlacement(IMappedBusPortInstance placingOn, boolean increment, boolean forceSingle) {
		this.placingOn = placingOn;
		this.increment = increment;
		this.forceSingle = forceSingle;
	}

	public void place(IMappedBusPortInstance port) {
		if (firstTarget == null) {
			firstTarget = port;
		}

		boolean ignore = false;

		if (firstTarget != null && forceSingle) {
			if (firstTarget != port) {
				ignore = true;
				logger.warn("Merging BusPort with ForceSingle: conflicting connections detected on {}, dropping connection (might be intended for diamond style connections that we cannot check) :: {} <-> {}", placingOn, firstTarget.getCanonicalName(), port.getCanonicalName());
			}
		}

		int start = current;
		int afterEnd = start + port.getBaseWidth();

		MappedBusConnectionStub placed = new MappedBusConnectionStub(start, afterEnd - 1, port, ignore);
		connections.add(placed);
		if (increment) {
			current = afterEnd;
		}
	}

	public List<MappedBusConnectionStub> getConnections() {
		return connections;
	}
}
