/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.CombineType;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.BiConsumer;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collector;

/**
 * Created by ramonw on 12.04.16.
 */
class BusPortPlacer implements Collector<IMappedBusPortInstance, BusPortPlacement, List<MappedBusConnectionStub>> {

	private final boolean checkSingle;
	private final boolean increment;
	private final IMappedBusPortInstance busPortInstance;

	public BusPortPlacer(IMappedBusPortInstance busPortInstance) {
		this.busPortInstance = busPortInstance;
		CombineType combineType = busPortInstance.getCombineType();
		increment = combineType == CombineType.Parallel;
		checkSingle = combineType == CombineType.ForceSingle && busPortInstance.getDirection() != ConnectorDirection.Output;
	}

	@Override
	public Supplier<BusPortPlacement> supplier() {
		return () -> new BusPortPlacement(busPortInstance, increment, checkSingle);
	}

	@Override
	public BiConsumer<BusPortPlacement, IMappedBusPortInstance> accumulator() {
		return BusPortPlacement::place;
	}

	@Override
	public BinaryOperator<BusPortPlacement> combiner() {

		return (a, b) -> {
			throw new UnsupportedOperationException("No parallel");
		};
	}

	@Override
	public Function<BusPortPlacement, List<MappedBusConnectionStub>> finisher() {
		return BusPortPlacement::getConnections;
	}

	@Override
	public Set<Characteristics> characteristics() {
		return Collections.emptySet();
	}

}


