/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.OrderedConnection;

/**
 * Created by ramonw on 12.04.16.
 */
public class MappedBusConnection extends OrderedConnection<IMappedBusPortInstance> implements IConnection<IMappedBusPortInstance> {

	private static final String MAPPED_BUS_CONNECTIONS_ARE_IMMUTABLE = "MappedBusConnections are immutable";
	private final IMappedBusPortInstance a;
	private final int startA;
	private final int endA;
	private final IMappedBusPortInstance b;
	private final int startB;
	private final int endB;
	private final boolean ignore;
	private final boolean partialA;
	private final boolean partialB;

	public MappedBusConnection(IMappedBusPortInstance a, int startA, int endA, IMappedBusPortInstance b, int startB, int endB, boolean ignore, boolean partialA, boolean partialB) {
		this.ignore = ignore;
		this.a = a;
		this.startA = startA;
		this.endA = endA;
		this.b = b;
		this.startB = startB;
		this.endB = endB;
		this.partialA = partialA;
		this.partialB = partialB;
	}

	public boolean isIgnore() {
		return ignore;
	}

	@Override
	public ConnectionType getType() {
		return IConnection.ConnectionType.MappedBus;
	}

	@Override
	public IMappedBusPortInstance getPortA() {
		return a;
	}

	@Override
	public IMappedBusPortInstance getPortB() {
		return b;
	}

	@Override
	public boolean isConnected() {
		return true;
	}

	@Override
	public void connect() {
		throw new UnsupportedOperationException(MAPPED_BUS_CONNECTIONS_ARE_IMMUTABLE);
	}

	@Override
	public void disconnect() {
		throw new UnsupportedOperationException(MAPPED_BUS_CONNECTIONS_ARE_IMMUTABLE);
	}

	@Override
	public boolean isValid(boolean report) {
		return true; // immutable //TODO check whether document has changed since creation?....
	}

	@Override
	public boolean isPartialA() {
		return partialA;
	}

	@Override
	public boolean isPartialB() {
		return partialB;
	}

	@Override
	public int getStartA() {
		return startA;
	}

	@Override
	public int getEndA() {
		return endA;
	}

	@Override
	public int getStartB() {
		return startB;
	}

	@Override
	public int getEndB() {
		return endB;
	}
}
