/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.CombineType;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.mappedBuses.MergedBusPort;
import de.tu_darmstadt.rs.spartanmc.devxml.mappedBuses.SingleMappedBusPort;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBusHandler;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Created by Ramon on 09.04.2016.
 */
public class MergedMappedBusPortInstance implements IMappedBusPortInstance {

	private final MergedBusPort org;
	private final List<InternalMappedBusPortInstance> ports;
	private final BusLayout layout;
	private final int baseWidth;

	public MergedMappedBusPortInstance(MergedBusPort mergedBusPort, IModuleConfiguration module, List<SingleMappedBusPort> ports, BusLayout layout) {
		this.org = mergedBusPort;
		this.layout = layout;
		this.ports = ports.stream().map(single -> new InternalMappedBusPortInstance(single, module.getBusHandler(single.getBus()))).collect(Collectors.toList());

		baseWidth = this.ports.get(0).getBaseWidth();
		if (this.ports.stream().anyMatch(mp -> mp.getBaseWidth() != baseWidth)) {
			throw new IllegalArgumentException("Unmergeable because of differing baseWidths");
		}
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append("MergedInst{").append(getCanonicalName());
		sb.append(", width=$x*").append(getBaseWidth());
		org.toStringPorts(sb);
		return sb.toString();
	}

	@Override
	public IConfigurationElement<?, ?, ?> getSourceElement() {
		return ports.get(0).getSourceElement();
	}

	public boolean isConnected() {
		return ports.stream().anyMatch(IMappedBusPart::isConnected);
	}

	@Override
	public int getConnectionCount() {
		return layout.getConnectionStubs(this).size();
	}

	@Override
	public IModuleDescription getModuleDescription() {
		return org.getModuleDescription();
	}

	@Override
	public String getPortName() {
		return org.getPortName();
	}

	@Override
	public IConnectorDescription.Type getType() {
		return org.getType();
	}

	@Override
	public boolean isAllowMerging() {
		return true;
	}

	@Override
	public CombineType getCombineType() {
		return org.getCombineType();
	}

	@Override
	public ConnectorDirection getDirection() {
		return org.getDirection();
	}

	@Override
	public boolean isPerConnectionWidth() {
		return org.isPerConnectionWidth();
	}

	@Override
	public Stream<? extends BaseMappedBusPortInstance> streamSingleMappedBusPorts() {
		return ports.stream();
	}

	@Override
	public BusLayout getBusLayout() {
		return layout;
	}

	@Override
	public int getBaseWidth() {
		return baseWidth;
	}

	@Override
	public int getBusConnectionCount(IValueResolver resolver) {
		return org.getBusConnectionCount(resolver);
	}


	private class InternalMappedBusPortInstance extends BaseMappedBusPortInstance {

		public InternalMappedBusPortInstance(SingleMappedBusPort mappedBusPort, IBusHandler handler) {
			super(mappedBusPort, handler);
		}

		@Override
		public IMappedBusPortInstance getRootBusPortInstance() {
			return MergedMappedBusPortInstance.this;
		}

		@Override
		public Stream<? extends BaseMappedBusPortInstance> streamSingleMappedBusPorts() {
			throw new UnsupportedOperationException();
		}

		@Override
		public int getBusConnectionCount(IValueResolver resolver) {
			return org.getBusConnectionCount(resolver);
		}


	}
}
