/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.layouting.interrupts;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.OptionalLong;

/**
 * Created by Ramon on 20.06.2016.
 */
public class IndexedInterruptLayout implements IInterruptLayout {

	private final AddressLayout layout;
	private final IPeripheralModuleConfiguration<?> intrCtrl;
	private List<IInterruptRoutine> isrs;

	public IndexedInterruptLayout(ISubsystemConfiguration<?> subsystem, AddressLayout layout) {
		this.layout = layout;
		Optional<? extends IModuleConfiguration<?>> interruptController = IInterruptLayout.findInterruptController(subsystem);


		intrCtrl = (IPeripheralModuleConfiguration<?>) interruptController.orElse(null);

		int isrCount = interruptController.map(this::getIsrCount).orElse(0);

		isrs = new ArrayList<>(isrCount);
		for (int i = 0; i < isrCount; i++) {
			isrs.add(new IndexedIsr(i));
		}
	}

	private int getIsrCount(IModuleConfiguration<?> intrCtrl) {
		return DocumentUtil.getPortHandler(intrCtrl, "intr_in").getConnectorWidth();
	}

	@Override
	public List<IInterruptRoutine> getInterruptRoutines() {
		return isrs;
	}

	@Override
	public OptionalLong getControllerAddress() {
		if (hasController()) {
			return OptionalLong.of(layout.findRegAddressBlock(intrCtrl).orElseThrow(() -> new IllegalStateException("Interrupt controller not connected")).getActualStart());
		} else {
			return OptionalLong.empty();
		}
	}

	@Override
	public boolean hasController() {
		return intrCtrl != null;
	}

	private static class IndexedIsr implements IInterruptRoutine {

		private final String name;
		private final int index;

		public IndexedIsr(int i) {
			index = i;
			name = String.format("isr_%d", i);
		}

		@Override
		public String getName() {
			return name;
		}

		@Override
		public int getIsrIndex() {
			return index;
		}
	}
}
