/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.util;

import de.tu_darmstadt.rs.spartanmc.jconfig.ILibJConfigProject;
import de.tu_darmstadt.rs.util.GroupWriter;

import java.nio.file.Path;

/**
 * Created by Ramon on 05.04.2016.
 */
public class HeaderGen {

	private final String commentStart;
	private final String commentEnd;
	private final String lineStart;

	public HeaderGen(String commentStart, String commentEnd, String lineStart) {
		this.commentStart = commentStart;
		this.commentEnd = commentEnd;
		this.lineStart = lineStart;
	}

	public HeaderGen(String singleLineComment) {
		this.commentEnd = singleLineComment;
		this.commentStart = singleLineComment;
		this.lineStart = singleLineComment;
	}

	public String getHeaderString(final String description, Path configFile) {
		String fileName = configFile != null? configFile.getFileName().toString() : "#Unsaved";
		String sb = commentStart + '\n' +
			String.format("%s Automatically generated by %s\n", lineStart, ILibJConfigProject.LIBJCONFIG_VERSION) +
			String.format("%s Generated From: %s (%s)\n", lineStart, fileName, configFile) +
			String.format("%s Generated at %tc\n", lineStart, System.currentTimeMillis()) +
			lineStart + '\n' +
			String.format("%s %s\n", lineStart, description) +
			commentEnd + "\n";
		return sb;
	}

	public GroupWriter.ILineMapper<Path> getHeader(final String description) {
		return (writer, configFile) -> {
			writer.println(this.getHeaderString(description, configFile));
		};
	}

}
