/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.microStreamsImport.jaxb;

import de.tu_darmstadt.rs.jaxbUtil.PathAdapter;

import java.nio.file.Path;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

/**
 * Created by ramonw on 6/7/16.
 */
public final class Core {

	public static final String PERFORMANCE_COUNTER = "PERFORMANCE_COUNTER";
	public static final String EVENTCOUNTERS = "COUNT_EVENTCOUNTERS";
	public static final Integer EVENTCOUNTERS_AMOUNT = 0;
	public static final String CYCLECOUNTER = "WIDTH_CYCLECOUNTER";
	public static final Integer CYCLECOUNTER_WIDTH = 36;
	public static final String SHOW_INSTUCTIONS = "SHOW_INSTR";

	@XmlAttribute
	@XmlID
	private String name;

	@XmlAttribute
	private String type;

	@XmlAttribute
	@XmlJavaTypeAdapter(PathAdapter.class)
	private Path firmware;

	@XmlAttribute
	private boolean performanceCounter;

	@XmlAttribute
	private boolean globalCodeMem;

	@XmlAttribute
	private boolean globalDataMem;

	private Core() {
	}

	/**
	 *
	 * @param name the name of the core
	 * @param type the type of the processor core
	 * @param firmware the firmware to be used for this core
	 */
	public Core(String name, String type, Path firmware) {
		this.name = name;
		this.type = type;
		this.firmware = firmware;
	}

	public String getName() {
		return name;
	}

	public String getType() {
		return type;
	}

	public Path getFirmware() {
		return firmware;
	}

	public boolean hasPerformanceCounter() {
		return performanceCounter;
	}

	public boolean hasGlobalDataMem() {
		return globalDataMem;
	}

	public boolean hasGlobalCodeMem() {
		return globalCodeMem;
	}

	@Override
	public String toString() {
		return getName();
	}
}
