/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.scripting;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IApplicationConfiguration;

import jline.console.completer.Completer;

import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import static jline.internal.Preconditions.checkNotNull;


/**
 * A completer for the context of the document, e.g. the available modules' names.
 *
 * Created by tobiass on 19.12.16.
 */
public class DocumentContextCompleter implements Completer {

	private final SortedSet<String> context = new TreeSet<String>();

	public DocumentContextCompleter(IApplicationConfiguration application) {
		//add module names to context
		application.getLibrary().getAvailableModules().forEach(module -> context.add(module.getName()));
	}

	/**
	 * Populates candidates with a list of possible completions for the buffer.
	 * The identifier start signs are '"' and '('.
	 *
	 * @param buffer The buffer
	 * @param cursor The current position of the cursor in the buffer
	 * @param candidates The {@link List} of candidates to populate
	 * @return The index of the buffer for which the completion will be relative
	 */
	public int complete(final String buffer, final int cursor, final List<CharSequence> candidates) {
		// buffer could be null
		checkNotNull(candidates);

		//identify identifier start by finding the last """ or "("
		int lastStartSign = Math.max(buffer.lastIndexOf('"'), buffer.lastIndexOf('('));

		if (lastStartSign == -1) {
			return 0;
		}

		String inputContext = buffer.substring(lastStartSign + 1);

		if (inputContext.isEmpty()) {
			candidates.addAll(context);
		} else {
			for (String match : context.tailSet(inputContext)) {
				if (!match.startsWith(inputContext)) {
					break;
				}

				candidates.add(match);
			}
		}

		if (candidates.size() == 1) {
			candidates.set(0, candidates.get(0) + " ");
		}

		return lastStartSign + 1;
	}

}