/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.IIssueSink;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.GlobalAddressLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses.BusLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.interrupts.IInterruptLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.interrupts.IndexedInterruptLayout;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by Ramon on 18.05.2016.
 */
public class LazyBuildProcess implements IBuildProcess {

	private static final Logger logger = LoggerFactory.getLogger(LazyBuildProcess.class);
	private final IConfigurationDocument document;
	private GlobalAddressLayout globalAddressLayout;
	private BusLayout busLayout;
	private Map<ISubsystemConfiguration<?>, IInterruptLayout> intrLayouts = new HashMap<>();


	public LazyBuildProcess(IConfigurationDocument document) {
		this.document = document;
	}

	@Override
	public IConfigurationDocument getDocument() {
		return document;
	}

	@Override
	public GlobalAddressLayout getGlobalAddressLayout() {
		if (globalAddressLayout == null) {
			logger.debug("Layouting global addresses");
			globalAddressLayout = new GlobalAddressLayout(document);
		}
		return globalAddressLayout;
	}

	@Override
	public BusLayout getBusLayout() {
		if (busLayout == null) {
			logger.debug("Layouting buses");
			busLayout = new BusLayout(document);
		}
		return busLayout;
	}

	@Override
	public IIssueSink getIssueSink() {
		return document.getIssueSink();
	}

	@Override
	public IInterruptLayout getInterruptLayout(ISubsystemConfiguration<?> subsystem) {
		return intrLayouts.computeIfAbsent(subsystem, (subsystem1) -> new IndexedInterruptLayout(subsystem1, getGlobalAddressLayout().getLayout(subsystem1)));
	}

}
