/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider;

import de.tu_darmstadt.rs.spartanmc.devxml.sources.SourceRef;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.BaseOutputBuilderProvider;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.DocumentBuilder;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.common.HdlFilesOutput;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Create Altera project files
 */
public class AlteraQuartusHdlFileListBuilderProvider extends BaseOutputBuilderProvider {
	private static final Logger logger = LoggerFactory.getLogger(AlteraQuartusHdlFileListBuilderProvider.class);

	public static final String SPARTANMC_HDL_QSF = "spartanmc_hdl.qsf";
	public static final String HDL_SOURCES_MK = "hdl_sources.mk";
	public static final String SPARTANMC_SIM_WORKLIB = "spartanmc_worklib.fdo";
	public static final String SPARTANMC_SIM_WORKLIB_TIMESIM = "spartanmc_worklib_timesim.fdo";

	public AlteraQuartusHdlFileListBuilderProvider(DocumentBuilder builder) {
		super(builder);
	}

	@Override
	public List<? extends DocumentBuilder.IOutputGroup> getOutputGroups(IBuildProcess buildProcess) {
		List<DocumentBuilder.IOutputGroup> groups = new ArrayList<>();

		IConfigurationDocument document = buildProcess.getDocument();

		Path systemDir = document.getProject().getIntermediaryBuildDir();
		groups.add(new HdlFilesOutput(document,
				Arrays.asList(
						HdlFilesOutput.getMakeOutput(systemDir.resolve(HDL_SOURCES_MK)),
						HdlFilesOutput.getVsimOutput(systemDir.resolve(SPARTANMC_SIM_WORKLIB), SourceRef.FileType.FUNCTIONAL_SIM, document),
						HdlFilesOutput.getVsimOutput(systemDir.resolve(SPARTANMC_SIM_WORKLIB_TIMESIM), SourceRef.FileType.TIMESIM, document)
				),
				systemDir));

		return groups;
	}

}
