/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider;

import de.tu_darmstadt.rs.spartanmc.initram.MemorySource;
import de.tu_darmstadt.rs.spartanmc.initram.bramLayout.QuartusBramBuilderFactory;
import de.tu_darmstadt.rs.spartanmc.initram.output.quartus.MifFilesOutput;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.BaseOutputBuilderProvider;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.DocumentBuilder;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.languages.MakeLang;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputGroups.SingleFileBaseOutputGroup;
import de.tu_darmstadt.rs.util.DigestUtil;
import de.tu_darmstadt.rs.util.IWriterSupplier;

import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Path;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class AlteraQuartusMemoryFileListBuilderProvider extends BaseOutputBuilderProvider {

	private static final String MEMORIES_FILENAME = "memories.mk";
	public static final String JCONFIG_MEMORY_FILES = "JCONFIG_MEMORY_FILES";

	public AlteraQuartusMemoryFileListBuilderProvider(DocumentBuilder builder) {
		super(builder);
	}

	@Override
	public List<? extends DocumentBuilder.IOutputGroup> getOutputGroups(IBuildProcess buildProcess) {
		IConfigurationDocument document = buildProcess.getDocument();

		Path systemDir = document.getProject().getIntermediaryBuildDir();
		return Collections.singletonList(new MemoryFileListGroup(new DigestUtil.PrintWriterSupplier(),systemDir.resolve(MEMORIES_FILENAME), buildProcess, document.getProject().getProjectDir(), document.getProject().getBuildDir()));
	}

	private class MemoryFileListGroup extends SingleFileBaseOutputGroup<Object, PrintWriter> {

		private final List<String> memoryFiles;

		protected MemoryFileListGroup(IWriterSupplier<PrintWriter> writerSupplier, Path output, IBuildProcess buildProcess, Path projectDir, Path buildDir) {
			super(writerSupplier, output);

			QuartusBramBuilderFactory bramBuilderFactory = new QuartusBramBuilderFactory();
			MemorySource memorySource = new MemorySource(buildProcess.getGlobalAddressLayout().getMemoryMap(), projectDir, bramBuilderFactory);
			MifFilesOutput mifOutput = new MifFilesOutput(buildDir);
			memoryFiles = mifOutput.streamAllBramFilenames(memorySource).map(Path::toString).collect(Collectors.toList());

			properties = Collections.singletonMap("files",memoryFiles);
			inputs = Collections.emptySet();
		}

		@Override
		protected void buildContent(PrintWriter writer) throws IOException {

			MakeLang.startMkEntry(writer, JCONFIG_MEMORY_FILES);
			memoryFiles.forEach(f -> MakeLang.writePartialMkEntry(writer, f));
			writer.println();
			MakeLang.endMkEntry(writer, JCONFIG_MEMORY_FILES);
		}
	}
}
