/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IConnectorInstance;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPinHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.BaseOutputBuilderProvider;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.DocumentBuilder;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.languages.CLang;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.languages.VerilogLang;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputGroups.VerilogOutputGroup;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.toolchains.xilinx.XilinxVivadoDocumentBuilder;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.file.Path;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class XilinxVivadoTimesimWrapperBuilderProvider extends BaseOutputBuilderProvider {

	private static final Logger logger = LoggerFactory.getLogger(XilinxVivadoTimesimWrapperBuilderProvider.class);

	public XilinxVivadoTimesimWrapperBuilderProvider(DocumentBuilder builder) {
		super(builder);
	}

	@Override
	public List<? extends DocumentBuilder.IOutputGroup> getOutputGroups(IBuildProcess buildProcess) {
		IConfigurationDocument document = buildProcess.getDocument();
		return Collections.singletonList(new VivadoTimesimWrapper(document.getProject().getIntermediaryBuildDir(), document));
	}

	private static class VivadoTimesimWrapper extends VerilogOutputGroup {


		private final IConfigurationDocument document;

		public VivadoTimesimWrapper(Path systemDir, IConfigurationDocument document) {
			super(systemDir.resolve(VerilogLang.getVerilogName(document.getName()) + "_timesim_wrapper.v"));
			this.document = document;

			inputs = Collections.emptySet();
			properties = new HashMap<>();


			DocumentUtil.streamConnectorHandlers(document).filter(IConnectorInstance::isConnected).forEach(p -> {
				properties.put(p.getPortName(), XilinxVivadoDocumentBuilder.getDefiningValues(p));
			});


		}

		@Override
		protected String getHeader() {
			return CLang.getCHeaderGen().getHeaderString("Timesim Wrapper module (to be able to specify uselib directives)", document.getProject().getDocumentHandler().getSaveLocation());
		}

		@Override
		protected void buildContent() {
			writeTimescale("1ns", "1ps");

			List<IPinHandler> connectedPins = DocumentUtil.streamConnectorHandlers(document).filter(IConnectorInstance::isConnected).sorted(IConnectorInstance.NAME_COMPARATOR).collect(Collectors.toList());

			writeModuleStart(VerilogLang.getVerilogName(document.getName()), connectedPins.stream(), IPinHandler::getTargetDirection, VerilogLang::getPinName);

			writeUselib("timesim");

			writeModuleInstanceStart("spartanmc_timesim");
			writeModuleInstanceParams(Stream.empty());//Unnecessary
			writeModuleInstanceNameAndPorts("wrapped", connectedPins.stream(), VerilogLang::getPinName, VerilogLang::getPinName, ph -> ph.getDirection().getReverse());

			writeUselib(null);

			writeNewline();
			writeModuleEnd();

		}

	}
}
