/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.common;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.constraints.ConstraintVisitor;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.constraints.KeepHierarchy;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.constraints.MixedParamRefs;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.constraints.TimingIgnore;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.constraints.VendorConstraint;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.INamedValueResolver;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;

import java.nio.file.Paths;
import java.util.function.UnaryOperator;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Make Strings from constraints. Useable in
 */
public final class ConstraintReferenceAnalyzer implements ConstraintVisitor<INamedValueResolver, String> {

	private static ConstraintReferenceAnalyzer inst;

	public static String makePropertyFromConstraints(IModuleConfiguration<?> module, String vendor, Boolean timing) {
		if (inst == null) {
			inst = new ConstraintReferenceAnalyzer();
		}
		return module.getModuleDescription().getHdl().getConstraints().stream().map(c -> c.visitIfEnabled(module, inst, module, vendor, timing)).collect(Collectors.joining("\",\"", "\"", "\""));
	}

	private String analyzeRef(INamedValueResolver resolver, MixedParamRefs ref) {
		return ref.resolve(resolver, UnaryOperator.identity(), Paths.get(".")).orElseThrow();
	}

	@Override
	public String visitVendorConstraint(VendorConstraint constraint, INamedValueResolver userData, String vendor, Boolean timing) {
		if(vendor.equals("lattice")){
			return analyzeRef(userData, constraint.getLatticeContent());
		}
		else if(vendor.equals("quartus")){
			if(timing) {
				return analyzeRef(userData, constraint.getQuartusSdcContent());
			}
			else{
				return analyzeRef(userData, constraint.getQuartusQsfContent());
			}
		}
		else{
			return analyzeRef(userData, constraint.getXilinxContent());
		}
	}

	@Override
	public String visitTimingIgnore(TimingIgnore constraint, INamedValueResolver resolver) {
		Stream<MixedParamRefs> insts = constraint.getInstances().stream();
		Stream<MixedParamRefs> nets = constraint.getNets().stream();
		Stream<MixedParamRefs> domainCrossing = constraint.getIgnoreClockCrossings().stream()
				.flatMap(icc -> Stream.of(icc.getFrom(), icc.getTo()));

		final Stream<MixedParamRefs> all = Stream.concat(Stream.concat(insts, nets), domainCrossing);
		return all.map(ref -> analyzeRef(resolver, ref)).collect(Collectors.joining("', '", "'", "'"));
	}

	@Override
	public String visitKeepHierarchy(KeepHierarchy keepHierarchy, INamedValueResolver resolver) {
		return analyzeRef(resolver, keepHierarchy.getInstance());
	}
}
