/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker;

import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.BaseOutputBuilderProvider;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.DocumentBuilder;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sectionLayouts.ISectionLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IFirmwareBinary;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IGlobalAddressLayout;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.file.Path;
import java.util.List;
import java.util.stream.Collectors;

/**
 * This is the generator of the GNU ld script(s) that will be used to link the respective programs
 * This Class was originally conceived to handle the 2 cases:
 * - per core memory: every core has its own memory, 1 linkerscript & elf binary per core
 * - global memory: code is shared between all cores.
 *
 * Yet in practice, this was not enough for people and everybody hacked in support for their specific requirements
 *
 * To support the use-cases that came up in the meantime a lot of logic needs to be added.
 *
 * To prepare for this step I (Ramon) redesigned the section-filters to be more explicit about their requirements on memory to make them generic enough to be reused with different layouting logics, when things started to fall apart (thanks carsten!).
 *
 * This however is only a stopgap solution and is still not designed or capable of to handle complex situations like: some code is shared between 2 cores, other code is local, some data might be shared as well, plus 1 additional core without any shared memory at all.
 *
 * //TODO redesign with a longer talk about which combinations should be allowed, how the requirements are communicated to the user and what options the user should have
 */
public class GnuLinkerMemoryBuilderProvider extends BaseOutputBuilderProvider {

	private static final Logger logger = LoggerFactory.getLogger(GnuLinkerMemoryBuilderProvider.class);

	public GnuLinkerMemoryBuilderProvider(DocumentBuilder builder) {
		super(builder);
	}

	@Override
	public List<? extends DocumentBuilder.IOutputGroup> getOutputGroups(IBuildProcess buildProcess) {
		Path systemDir = buildProcess.getDocument().getProject().getIntermediaryBuildDir();

		IGlobalAddressLayout globalLayout = buildProcess.getGlobalAddressLayout();

		return globalLayout.getBinaries().values().stream()
				.distinct()
				.map(binary -> createOutputGroupFor(binary, systemDir, buildProcess))
				.collect(Collectors.toList());
	}

	private BaseGnuLinkerScriptOutput createOutputGroupFor(IFirmwareBinary binary, Path systemDir, IBuildProcess buildProcess) {
		IFirmwareBinary.Type type = binary.getType();
		switch (type) {
			case SingleCore:
			case PartOfShared:
				AddressLayout layout = binary.getLayout();
				if (type == IFirmwareBinary.Type.SingleCore) {
					return new SingleCoreGnuLinkerScriptOutput(systemDir, layout, binary, buildProcess, ISectionLayout.findLayoutFor(layout));
				} else {
					return new PartOfSharedGnuLinkerScriptOutput(systemDir, layout, binary, buildProcess, ISectionLayout.findLayoutFor(layout));
				}
			case MultiCore:
			default:
				throw new UnsupportedOperationException();
		}
	}

}


