/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IMemoryModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.PhysAddrCounter;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IFirmwareBinary;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IGlobalAddressLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IMemoryAddressBlock;

import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Path;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

public class MultiCoreGnuLinkerScriptOutput extends BaseGnuLinkerScriptOutput {

	private final IGlobalAddressLayout.SharedAddressLayout sharedAddressLayout;

	protected MultiCoreGnuLinkerScriptOutput(Path systemDir, IGlobalAddressLayout.SharedAddressLayout sharedAddressLayout, IFirmwareBinary buildingBinary, IBuildProcess buildProcess) {
		super(systemDir, buildingBinary, buildProcess, null);//TODO
		this.sharedAddressLayout = sharedAddressLayout;

		sharedAddressLayout.addDependencies(inputs, properties);

		sharedAddressLayout.getContainedLayouts().stream().map(AddressLayout::getSubsystem).forEach(subsystem -> addInterruptLayoutDependencies(subsystem, buildProcess.getInterruptLayout(subsystem)));
	}

	@Override
	protected void buildContent(PrintWriter writer) throws IOException {
		Set<AddressLayout> addressLayouts = sharedAddressLayout.getContainedLayouts();

		List<IMemoryModuleConfiguration<?>> globalMems = addressLayouts.stream().flatMap(AddressLayout::streamFirmwareAttachedInitializableBlocks)
				.map(block -> ((IMemoryModuleConfiguration<?>) block.getSourceModule())).collect(toList());

		List<ISubsystemConfiguration> illegals = globalMems.stream().flatMap(IMemoryModuleConfiguration::getConnectedSubsystems).filter(sub -> buildProcess.getInterruptLayout(sub).hasController()).collect(Collectors.toList());
		if (!illegals.isEmpty()) {
			throw new IllegalStateException("The following subsystems have an interrupt controller within a global firmware system: " + illegals);
		}

		throw new UnsupportedOperationException();//TODO
	}

	@Override
	protected void writeLinkerMemoryEntries(PrintWriter writer) {

	}

	@Override
	protected void writeLinkerSectionEntries(PrintWriter writer, Map<? extends IMemoryAddressBlock, ? extends IFirmwareBinary> firmware, PhysAddrCounter counter, Map<? extends IMemoryAddressBlock, ? extends List<? extends ILinkerSection>> placed) {
		throw new UnsupportedOperationException();
	}


}
