/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sectionLayouts;

import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.SpartanMcSections;

import java.util.Arrays;
import java.util.List;

public class ClassicSpartanMcSectionLayout implements ISectionLayout {

	private static final List<ILinkerSection> SECTIONS = Arrays.asList(
			DefaultSections.STARTUP_SECTION,
			DefaultSections.INIT_ARRAY_SECTION,
			DefaultSections.FINI_ARRAY_SECTION,
			DefaultSections.CTOR_SECTION,
			DefaultSections.DTOR_SECTION,
			SpartanMcSections.LOADER_TEXT_SECTION,
			SpartanMcSections.LOADER_DATA_SECTION,
			DefaultSections.INIT_SECTION,
			DefaultSections.FINI_SECTION,
			DefaultSections.CODE_SECTION,
			DefaultSections.DATA_SECTION,
			DefaultSections.BSS_SECTION,
			DefaultSections.SMALL_DATA_SECTION,//TODO makes no sense, small data must be at 0 on SpartanMC, but this screws with the loader pattern and Linker does not know how to put it at the end without us calculating the exact sizes and offsets
			DefaultSections.SMALL_BSS_SECTION,
			DefaultSections.RODATA_SECTION,
			DefaultSections.SMALL_RODATA_SECTIONS,
			SpartanMcSections.TRAP_ISR_SECTION,
			SpartanMcSections.INTERRUPT_TABLE_SECTION,
			SpartanMcSections.DMA_PLACEHOLDER_SECTION,
			SpartanMcSections.GLOBAL_DATA_SECTION,
			DefaultSections.STACK_SECTION
	);

	@Override
	public ILinkerSection getStartupSection() {
		return DefaultSections.STARTUP_SECTION;
	}

	@Override
	public List<ILinkerSection> getAvailableSections() {
		return SECTIONS;
	}

}
