/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.BinaryOutputFormat;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.PhysAddrCounter;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IMemoryAddressBlock;

import java.io.PrintWriter;

public abstract class BaseSection implements ILinkerSection {

	protected void writeSectionLimit(String subPrefix, PrintWriter writer, boolean isEnd) {
		String suffix;
		if (isEnd) {
			suffix = "END";
		} else {
			suffix = "START";
		}

		final String sectionName = (subPrefix + getName()).toUpperCase().replaceAll("[^A-Za-z0-9]","_");

		writer.format("\t\t_%s_SECTION_%s = .;\n", sectionName, suffix);
	}

	protected void writeSectionStart(String subPrefix, PrintWriter writer, String at, String outputOptions) {
		writer.format("\t%-27s %-12s : %-14s {\n", subPrefix + getName(), outputOptions, at);
		//		writeSectionLimit(subPrefix, writer, false);// Linker writes some implicitly, but if that makes problems, wrap ours in PROVIDE()
	}

	protected void writeSectionEnd(String subPrefix, String regionName, PrintWriter writer) {
//		writeSectionLimit(subPrefix, writer, true);// Linker writes some implicitly, but if that makes problems, wrap ours in PROVIDE()
		writer.format("\t}> %s\n\n", regionName);
	}

	protected void writeSectionContent(String subPrefix, String subInputFilter, PrintWriter writer, ISubsystemConfiguration<?> subsystem, BinaryOutputFormat format, IBuildProcess cache, IMemoryAddressBlock block) {
		writer.format("\t\t%-10s ( %-15s )\n", subInputFilter + getInputFilter(), getSectionFilter(subPrefix));
	}

	@Override
	public void writeSection(String subPrefix, String subInputFilter, String regionName, PrintWriter writer, PhysAddrCounter counter, BinaryOutputFormat format, ISubsystemConfiguration<?> subsystem, IBuildProcess cache, IMemoryAddressBlock block, boolean firstInRegion) {
		String at = generateAt(counter);

		String outputOptions = getOutputOptions(block, firstInRegion, format);

		writeSectionStart(subPrefix, writer, at, outputOptions);
		writeSectionContent(subPrefix, subInputFilter, writer, subsystem, format, cache, block);
		writeSectionEnd(subPrefix, regionName, writer);
	}

	protected abstract String getOutputOptions(IMemoryAddressBlock block, boolean firstInRegion, BinaryOutputFormat format);

	protected String generateAt(PhysAddrCounter counter) {
		return (counter.isNeeded())? String.format("AT(0x%-8x)", counter.getAddr()) : "";
	}


}
