/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections;

import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class DefaultSections {
	public static final ILinkerSection STARTUP_SECTION = new DefaultSection(".startup", "crt0.o", false, true, false, false, true);//crt0 must be first, so that the first address of startup is identical to __start

	public static final ILinkerSection INIT_ARRAY_SECTION = new InitFiniArraySection(InitFiniArraySection.Type.Init, true);
	public static final ILinkerSection FINI_ARRAY_SECTION = new InitFiniArraySection(InitFiniArraySection.Type.Fini, true);

	public static final ILinkerSection CTOR_SECTION = new TorsSection(TorsSection.Type.Ctor, true);
	public static final ILinkerSection DTOR_SECTION = new TorsSection(TorsSection.Type.Dtor, true);

	public static final ILinkerSection INIT_SECTION = new DefaultSection(".init", false, true, false, false, true);
	public static final ILinkerSection FINI_SECTION = new DefaultSection(".fini", false, true, false, false, true);

	public static final ILinkerSection CODE_SECTION = new UserSection(".text", ".text .text.*", false, false, false, false, true);

	public static final ILinkerSection DATA_SECTION = new UserSection(".data", false, false, true, true, false);
	public static final ILinkerSection BSS_SECTION = new UserSection(".bss", false, false, true, true, false);

	public static final ILinkerSection RODATA_SECTION = new UserSection(".rodata", false, false, true, false, false);

	public static final ILinkerSection SMALL_DATA_SECTION = new UserSection(".sdata", false, false, true, true, false);
	public static final ILinkerSection SMALL_BSS_SECTION = new UserSection(".sbss", false, false, true, true, false);
	public static final ILinkerSection SMALL_RODATA_SECTIONS = new UserSection(".srodata", false, false, true, false, false);

	public static final ILinkerSection STACK_SECTION = new StackSection();

	/**
	 * Dependencies between sections.
	 * Meaning: Key Section / Group will reference sections in value.
	 * Referenced sections are required to be either global or unified if the source section is global
	 */
	private static void setupStaticallyKnownSectionDeps(Map<ILinkerSection, Set<ILinkerSection>> deps) {
		addDep(deps, STARTUP_SECTION, INIT_SECTION, FINI_SECTION);
		addDep(deps, INIT_SECTION, INIT_ARRAY_SECTION, CTOR_SECTION);
		addDep(deps, FINI_SECTION, FINI_ARRAY_SECTION, DTOR_SECTION);

		addDep(deps, INIT_ARRAY_SECTION, CODE_SECTION);
		// ctor deps are missing, but its not supposed to be used anyway...

		addDep(deps, CODE_SECTION, DATA_SECTION, BSS_SECTION, RODATA_SECTION, SMALL_DATA_SECTION, SMALL_BSS_SECTION, SMALL_RODATA_SECTIONS);
	}

	public static void addDep(Map<ILinkerSection, Set<ILinkerSection>> deps, ILinkerSection key, ILinkerSection... add) {
		deps.merge(key, new HashSet<>(Arrays.asList(add)), DefaultSections::mergeStaticallyKnownSectionDeps);
	}

	public static Set<ILinkerSection> mergeStaticallyKnownSectionDeps(Set<ILinkerSection> existing, Set<ILinkerSection> adding) {
		if (existing != null) {
			adding.addAll(existing);
		}
		return adding;
	}
}
