/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.memory.MemoryContents;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.BinaryOutputFormat;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.PhysAddrCounter;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.IIssueSink;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IAddressRangeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IMemoryAddressBlock;
import de.tu_darmstadt.rs.util.StreamUtil;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

class ManualLinkerSectionsStub extends DefaultSection {


	public ManualLinkerSectionsStub() {
		super("DMA", true, true, true, true, false);
	}

	@Override
	public void writeSection(String subPrefix, String subInputFilter, String regionName, PrintWriter writer, PhysAddrCounter counter, BinaryOutputFormat format, ISubsystemConfiguration<?> subsystem, IBuildProcess cache, IMemoryAddressBlock block, boolean firstInRegion) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean canOutputTo(IMemoryAddressBlock block) {
		IAddressRangeHandler range = block.getRange();
		return !block.isShared() && range.isReadable() && range.isWritable() && !range.isExecutable() && block.getMemory().getContents().getContentConfiguration() instanceof MemoryContents.ManualLinkerSection;
	}

	@Override
	public Collection<? extends StreamUtil.Pair<? extends ILinkerSection, ? extends IMemoryAddressBlock>> bindMultiple(List<? extends IMemoryAddressBlock> possible, IIssueSink issueSink) {
		return possible.stream()
				.filter(p -> {
					Object conf = p.getMemory().getContents().getContentConfiguration();
					return conf instanceof MemoryContents.ManualLinkerSection;
				})
				.map(p -> {
					MemoryContents.ManualLinkerSection conf = ((MemoryContents.ManualLinkerSection) p.getMemory().getContents().getContentConfiguration());
					return new StreamUtil.Pair<>(new DmaSection(conf), p);
				}).collect(Collectors.toList());
	}

	@Override
	public boolean isBindMultiple() {
		return true;
	}

	private class DmaSection implements ILinkerSection {

		private final MemoryContents.ManualLinkerSection conf;

		public DmaSection(MemoryContents.ManualLinkerSection conf) {
			this.conf = conf;
		}

		@Override
		public String getName() {
			return "";
		}

		@Override
		public String getInputFilter() {
			return "*";
		}

		@Override
		public String getSectionFilter(String subPrefix) {
			return subPrefix + ManualLinkerSectionsStub.this.getName();
		}

		@Override
		public int getVersion() {
			return 0;
		}

		@Override
		public void writeSection(String subPrefix, String subInputFilter, String regionName, PrintWriter writer, PhysAddrCounter counter, BinaryOutputFormat format, ISubsystemConfiguration<?> subsystem, IBuildProcess cache, IMemoryAddressBlock block, boolean firstInRegion) {
			String at = generateAt(counter);

			String name = block.getSourceModule().getName();
			String sectionName = conf.resolveSectionName(name);

			long start = format.targetWordsToBytes(block.getActualStart());
			writer.format("\t%s 0x%x : %-14s {\n", sectionName, start, at);
			writeSectionLimit(subPrefix, writer, false);
			writer.format("\t\t%-10s ( %s )\n", subInputFilter + getInputFilter(), sectionName);
			writeSectionEnd(subPrefix, regionName, writer);
		}


		@Override
		public boolean canOutputTo(IMemoryAddressBlock block) {
			return true;
		}

		@Override
		public Collection<? extends StreamUtil.Pair<? extends ILinkerSection, ? extends IMemoryAddressBlock>> bindMultiple(List<? extends IMemoryAddressBlock> possible, IIssueSink issueSink) {
			throw new UnsupportedOperationException();
		}
	}
}
