/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.BinaryOutputFormat;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.PhysAddrCounter;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IMemoryAddressBlock;

import java.io.PrintWriter;

public class SharedDataSection extends DefaultSection {

	public SharedDataSection() {
		super(".globaldata", false, false, true, true, false);
	}

	@Override
	public boolean canOutputTo(IMemoryAddressBlock block) {
		return super.canOutputTo(block) && block.isShared();
	}


	@Override
	protected String getOutputOptions(IMemoryAddressBlock block, boolean firstInRegion, BinaryOutputFormat format) {
		return String.format("0x%-10x", format.targetWordsToBytes(block.getActualStart()));
	}

	@Override
	public void writeSection(String subPrefix, String subInputFilter, String regionName, PrintWriter writer, PhysAddrCounter counter, BinaryOutputFormat format, ISubsystemConfiguration<?> subsystem, IBuildProcess cache, IMemoryAddressBlock block, boolean firstInRegion) {
		writer.println("\t__GLOBAL_MEM_HEAP_START = .;");
		super.writeSection(subPrefix, subInputFilter, regionName, writer, counter, format, subsystem, cache, block, firstInRegion);
		writer.format("\t__GLOBAL_MEM_HEAP_END = 0x%-10x;\n", +format.targetWordsToBytes(block.getActualEnd()));
	}

	@Override
	protected void writeSectionStart(String subPrefix, PrintWriter writer, String at, String outputOptions) {
		writer.format("\t%-27s %-12s : %-14s {\n", subPrefix + getName(), outputOptions, at);
		//		writeSectionLimit(subPrefix, writer, false);// Linker writes some implicitly, but if that makes problems, wrap ours in PROVIDE()
	}

	@Override
	protected void writeSectionContent(String subPrefix, String subInputFilter, PrintWriter writer, ISubsystemConfiguration<?> subsystem, BinaryOutputFormat format, IBuildProcess cache, IMemoryAddressBlock block) {
		super.writeSectionContent(subPrefix, subInputFilter, writer, subsystem, format, cache, block);
	}
}
