/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.BinaryOutputFormat;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IMemoryAddressBlock;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.interrupts.IInterruptLayout;

import java.io.PrintWriter;

class SpartanMcInterruptTableSection extends DefaultSection {

	public SpartanMcInterruptTableSection() {
		super(".intr_tab", "", true, false, true, true, false);
	}

	@Override
	protected void writeSectionContent(String subPrefix, String subInputFilter, PrintWriter writer, ISubsystemConfiguration<?> subsystem, BinaryOutputFormat format, IBuildProcess buildProcess, IMemoryAddressBlock block) {
		writer.println("\t\t__INTR_TAB__ = .;");
		writer.println("\t\tLONG( 0 ) /* CC backup */");
		for (IInterruptLayout.IInterruptRoutine isr : buildProcess.getInterruptLayout(subsystem).getInterruptRoutines()) {
			writer.format("\t\tASSERT(DEFINED(%1$s),\"Error: Interrupt Service Routine %2$d is not defined. Please define the ISR using ISR(%2$d)() {...}\");\n", isr.getName(), isr.getIsrIndex());
			writer.format("\t\tPROVIDE(%s = 0);  /* Ensure assert gets checked before other errors */\n", isr.getName());
			writer.format("\t\tSHORT( (%s%s) & 0x1FF ) /* Upper part, in target words */\n", isr.getName(), format.getToUpperTargetWordCOp());
			writer.format("\t\tSHORT( (%s%s) & 0x1FF ) /* Lower part, in target words */\n", isr.getName(), format.getToLowerTargetWordCOp());
		}
	}
}
