/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections;

import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;

import java.util.Map;
import java.util.Set;

import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.BSS_SECTION;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.CODE_SECTION;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.DATA_SECTION;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.INIT_ARRAY_SECTION;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.RODATA_SECTION;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.SMALL_BSS_SECTION;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.SMALL_DATA_SECTION;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.SMALL_RODATA_SECTIONS;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections.DefaultSections.addDep;

public class SpartanMcSections {
	public static final ILinkerSection TRAP_ISR_SECTION = new AlignedSection(".traps", ".traps .interrupt", 512, false, false, true, false, true);
	public static final ILinkerSection INTERRUPT_TABLE_SECTION = new SpartanMcInterruptTableSection();

	public static final ILinkerSection DMA_PLACEHOLDER_SECTION = new ManualLinkerSectionsStub();

	public static final ILinkerSection LOADER_TEXT_SECTION = new LoaderSection(".loader.text", false, false, true);
	public static final ILinkerSection LOADER_DATA_SECTION = new LoaderSection(".loader.data", true, true, false);
	public static final ILinkerSection GLOBAL_DATA_SECTION = new SharedDataSection();

	public static void setupStaticallyKnownSectionDeps(Map<ILinkerSection, Set<ILinkerSection>> deps) {
		addDep(deps, TRAP_ISR_SECTION, CODE_SECTION, DATA_SECTION, BSS_SECTION, RODATA_SECTION, SMALL_DATA_SECTION, SMALL_BSS_SECTION, SMALL_RODATA_SECTIONS);
		addDep(deps, INTERRUPT_TABLE_SECTION, TRAP_ISR_SECTION);
		addDep(deps, INIT_ARRAY_SECTION, LOADER_TEXT_SECTION);
		addDep(deps, LOADER_TEXT_SECTION, LOADER_DATA_SECTION);

		addDep(deps, DATA_SECTION, DMA_PLACEHOLDER_SECTION);
		addDep(deps, RODATA_SECTION, DMA_PLACEHOLDER_SECTION);
		addDep(deps, CODE_SECTION, DMA_PLACEHOLDER_SECTION);
	}
}
