/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.memory.MemoryContents;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.BinaryOutputFormat;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.PhysAddrCounter;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.IIssueSink;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IMemoryAddressBlock;
import de.tu_darmstadt.rs.util.StreamUtil;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

public class StackSection implements ILinkerSection {

	@Override
	public String getName() {
		return "STACK";
	}

	@Override
	public String getInputFilter() {
		return null;
	}

	@Override
	public String getSectionFilter(String subPrefix) {
		return null;
	}

	@Override
	public void writeSection(String subPrefix, String subInputFilter, String regionName, PrintWriter writer, PhysAddrCounter counter, BinaryOutputFormat format, ISubsystemConfiguration<?> subsystem, IBuildProcess cache, IMemoryAddressBlock block, boolean firstInRegion) {
		writer.format("\t__HEAP_START = .;\n");
		writer.println("\t/* STACK / HEAP from now on */");
		long endAddr = block.getActualEnd();
		writer.format("\tSTACK_TOP = 0x%x; /* internally at daddr 0x%x, caddr 0x%x */\n", format.targetWordsToBytes(endAddr), format.targetWordsToTargetBytes(endAddr), endAddr);
		writer.println();
	}

	@Override
	public boolean canOutputTo(IMemoryAddressBlock block) {
		return !block.isShared() && block.getRange().isReadable() && block.getRange().isWritable() && block.getMemory().getContents().getContentConfiguration() instanceof MemoryContents.GeneralPurposeMemory;
	}

	@Override
	public Collection<? extends StreamUtil.Pair<? extends ILinkerSection, ? extends IMemoryAddressBlock>> bindMultiple(List<? extends IMemoryAddressBlock> possible, IIssueSink issueSink) {
		Optional<? extends IMemoryAddressBlock> targetBlock = possible.stream().max(Comparator.comparing(IMemoryAddressBlock::getActualEnd));
		return targetBlock.<Collection<? extends StreamUtil.Pair<? extends ILinkerSection, ? extends IMemoryAddressBlock>>>map(iMemoryAddressBlock -> Collections.singletonList(new StreamUtil.Pair<>(this, iMemoryAddressBlock))).orElseGet(Collections::emptyList);
	}

	@Override
	public int getVersion() {
		return Objects.hash(getName());
	}
}
