/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.sections;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.memory.MemoryContents;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.linker.api.ILinkerSection;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.linker.SectionLayoutException;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.ConfigurationIssue;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.IIssueSink;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.IMemoryAddressBlock;
import de.tu_darmstadt.rs.util.StreamUtil;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class UserSection extends DefaultSection {

	private static final Logger logger = LoggerFactory.getLogger(UserSection.class);

	public UserSection(String section, String filter, boolean perCoreState, boolean firstStageAccess, boolean requiresReadAccess, boolean requiresWriteAccess, boolean requiresExecuteAccess) {
		super(section, filter, perCoreState, firstStageAccess, requiresReadAccess, requiresWriteAccess, requiresExecuteAccess);
	}

	public UserSection(String sectionAndFilter, boolean perCoreState, boolean firstStageAccess, boolean requiresReadAccess, boolean requiresWriteAccess, boolean requiresExecuteAccess) {
		super(sectionAndFilter, perCoreState, firstStageAccess, requiresReadAccess, requiresWriteAccess, requiresExecuteAccess);
	}

	@Override
	public boolean isBindMultiple() {
		return true;
	}

	protected boolean isUserData() {
		return requiresReadAccess || requiresWriteAccess;
	}

	protected boolean isUserCode() {
		return requiresExecuteAccess;
	}

	@Override
	public Collection<? extends StreamUtil.Pair<? extends ILinkerSection, ? extends IMemoryAddressBlock>> bindMultiple(List<? extends IMemoryAddressBlock> possible, IIssueSink issueSink) {
		List<? extends IMemoryAddressBlock> allowed = possible.stream().filter(p -> {
			Object conf = p.getMemory().getContents().getContentConfiguration();
			return conf instanceof MemoryContents.GeneralPurposeMemory;
		}).filter(p -> {
			MemoryContents.GeneralPurposeMemory conf = (MemoryContents.GeneralPurposeMemory) p.getMemory().getContents().getContentConfiguration();
			return (conf.getUserCode() == MemoryContents.GeneralPurposeMemory.Allowable.Allow || !isUserCode())
					&& (conf.getUserData() == MemoryContents.GeneralPurposeMemory.Allowable.Allow || !isUserData());
		}).collect(Collectors.toList());

		if (allowed.isEmpty()) {
			throw new SectionLayoutException("No memory of "+ possible +" allows placement of user-section "+ getName());
		} else if (allowed.size() > 1) {
			logger.error("Section {} could be placed into multiple blocks ({}), but does not specify a preference, choosing first possible memory", getName(), possible);
			issueSink.report(ConfigurationIssue.createSectionPlacementGuessed(this, allowed.get(0), allowed, "Writing Linkerscript"));
		}

		return Collections.singletonList(new StreamUtil.Pair<>(this, allowed.get(0)));
	}
}
