/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.verilog;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.sources.SourceRef;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.languages.VerilogLang;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputGroups.VerilogOutputGroup;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses.IBusGlue;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses.IMappedBusPortInstance;

import java.nio.file.Path;
import java.util.Collections;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.languages.VerilogLang.getGlueName;
import static de.tu_darmstadt.rs.spartanmc.jconfig.generation.languages.VerilogLang.getWireName;

/**
 * Created by ramonw on 5/27/16.
 */
public class VerilogBusGlue implements IBusGlue<Object> {

	public static final String ORGATE_SRC = "core/src/orgate.v";
	public static final String ORGATE_MODULE = "orgate";
	public static final String BUS_WIDTH_PARAM = "BUS_WIDTH";
	public static final String OR_WIDTH_PARAM = "OR_WIDTH";
	public static final String INPUT_NAME = "a";
	private static final String OUTPUT_NAME = "y";
	private final VerilogOutputGroup output;

	public VerilogBusGlue(VerilogOutputGroup writer) {
		this.output = writer;
	}

	public static Set<SourceRef> getBusGlueSrcs(Path root) {
		return Collections.singleton(new SourceRef(root.resolve("spartanmc/hardware").resolve(ORGATE_SRC)));
	}

	@Override
	public Object glueOrMergedBus(IMappedBusPortInstance busPort, Function<IMappedBusPortInstance, String> glueInputName, Function<IMappedBusPortInstance, String> glueOutputName) {
		int baseWidth = busPort.getBaseWidth();
		String glueName = getGlueName(busPort);

		VerilogOutputGroup.OutputParameter orWidthParam = new VerilogOutputGroup.OutputParameter(OR_WIDTH_PARAM, Integer.toString(busPort.getConnectionCount()));
		VerilogOutputGroup.OutputParameter busWidthParam = new VerilogOutputGroup.OutputParameter(BUS_WIDTH_PARAM, Integer.toString(baseWidth));
		Stream<VerilogOutputGroup.OutputParameter> params = Stream.of(orWidthParam, busWidthParam);
		Stream<TempConnection> connections = Stream.of(
				new TempConnection(INPUT_NAME, glueInputName.apply(busPort), ConnectorDirection.Input),
				new TempConnection(OUTPUT_NAME, glueOutputName.apply(busPort), ConnectorDirection.Output)
		);

		output.writeModuleInstanceStart(ORGATE_MODULE);
		output.writeModuleInstanceParams(params);
		output.writeModuleInstanceNameAndPorts(glueName, connections, c -> c.portName, c -> c.wireName, c -> c.direction);

		return null;
	}

	private String getVerilogConcat(IMappedBusPortInstance busPortInstance) {
		return busPortInstance.streamConnections().map(con ->
				String.format("%s%s", getWireName(con.getOppositeConnector(busPortInstance)), VerilogLang.getVerilogBus(con.getOppositeStart(busPortInstance), con.getOppositeEnd(busPortInstance), con.isOppositePartial(busPortInstance), con.isOppositePartial(busPortInstance))))
			.collect(Collectors.joining(", ", "{ ", " }"));
	}

	private static class TempConnection {

		final String portName;
		final String wireName;
		final ConnectorDirection direction;

		TempConnection(String portName, String wireName, ConnectorDirection direction) {
			this.portName = portName;
			this.wireName = wireName;
			this.direction = direction;
		}
	}
}
