/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.internal;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.exceptions.MissingModuleDescriptionException;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.ConfigurationIssue;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.ConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.ModuleConfiguration;

import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public final class LibAttacher {

	private static final Logger logger = LoggerFactory.getLogger(LibAttacher.class);

	private final List<ModuleConfiguration<?>> remove = new ArrayList<>();
	private final ConfigurationDocument document;
	private final Library library;

	public LibAttacher(ConfigurationDocument document, Library library) {
		this.document = document;
		this.library = library;
	}

	public List<ModuleConfiguration<?>> attach() {
		document.streamModulesRecursive().forEach(it -> {
			if (!attachTo(it)) {
				remove.add(it);
			}
		});

		for (ModuleConfiguration<?> rem : remove) {
			@NotNull IConfigurationElement<?, ?, ?> parent = rem.getParent();
			document.report(ConfigurationIssue.createMissingModuleDescription(parent, rem.getName(), rem.getModuleDescriptionName(), "Attaching to Library"));
			try {
				document.getDocumentHandler().removeModuleInternal(parent, rem, false, true); // remove internal so pluginEngine etc. ignores it
			} catch (Exception e) {
				throw new IllegalStateException("Failed to remove undefined module "+ rem, e);
			}
			rem.invalidateModule();
		}

		return remove;
	}

	private boolean attachTo(ModuleConfiguration<?> module) {
		try {
			module.attachLibrary(library);
			return true;
		} catch (MissingModuleDescriptionException e) {
			logger.error("Missing moduleDescription for module '{}'", module, e);
			return false;
		} catch (Exception e) {
			logger.error("Failed to attach to Library, removing module", e);
			return false;
		}
	}
}
