/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.internal;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.configurations.IToolchainConfiguration;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.HdlDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.ILibJConfigProject;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collector;

public class SimRootModuleCollector {

	private final ILibJConfigProject project;
	private final Library library;
	private final IToolchainConfiguration currentConfig;

	private Set<IModuleDescription> containedDescrs = new HashSet<>();

	//Order is important, multiple usages only once -> TreeSet
	private TreeSet<String> rootsFunctional = new TreeSet<>();
	private TreeSet<String> rootsPostPar = new TreeSet<>();

	public SimRootModuleCollector(ILibJConfigProject project, Library library, Collection<HdlDescription.SingletonSimRootModule> additionalRoots) {

		this.project = project;
		this.library = library;


		final IConfigurationDocument document = project.getDocumentHandler().getDocument();
		if (document == null) {
			throw new NullPointerException("No document!");
		}
		this.currentConfig = document.getTarget().getCurrentConfiguration();

		if (additionalRoots != null) {
			additionalRoots.forEach(this::add);
		}
	}

	public static Collector<? super IModuleConfiguration<?>, SimRootModuleCollector, SimRoots> create(ILibJConfigProject project, Library library, Collection<HdlDescription.SingletonSimRootModule> initialContent) {
		return Collector.of(
				() -> new SimRootModuleCollector(project, library, initialContent),
				SimRootModuleCollector::add,
				(acc, merge) -> {
					throw new UnsupportedOperationException();
				},
				SimRootModuleCollector::getSingletons);
	}

	public static Collector<? super IModuleDescription, SimRootModuleCollector, SimRoots> createDescription(ILibJConfigProject project, Library library, Collection<HdlDescription.SingletonSimRootModule> initialContent) {
		return Collector.of(
				() -> new SimRootModuleCollector(project, library, initialContent),
				SimRootModuleCollector::add,
				(acc, merge) -> {
					throw new UnsupportedOperationException();
				},
				SimRootModuleCollector::getSingletons);
	}

	private void add(HdlDescription.SingletonSimRootModule singleton) {
		if (singleton.getConfiguration() == null || singleton.getConfiguration().supports(currentConfig)) {
			rootsFunctional.add(singleton.getName());
			if (singleton.isIncludeInPostParSim()) {
				rootsPostPar.add(singleton.getName());
			}
		}
	}

	private void add(IModuleDescription description) {
		if (containedDescrs.add(description)) {
			description.getHdl().getSingletonSimRootModules().forEach(this::add);
			description.getHdl().getSources().getModules().forEach(modname -> {
				IModuleDescription descr = library.getModuleDescription(modname);
				add(descr);
			});
		}
	}

	private void add(IModuleConfiguration<?> module) {
		add(module.getModuleDescription());
	}


	private SimRoots getSingletons() {
		return new SimRoots(rootsFunctional, rootsPostPar);
	}

	public static class SimRoots {

		public final TreeSet<String> functionalRoots;
		public final TreeSet<String> postParRoots;


		public SimRoots(TreeSet<String> functionalRoots, TreeSet<String> postParRoots) {
			this.functionalRoots = functionalRoots;
			this.postParRoots = postParRoots;
		}
	}


}
