/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorGroupDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPluginEngine;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.BaseConnectorHandler;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Ramon on 02.12.2015.
 */
public abstract class BaseConfigurationElement<PD extends IConnectorDescription, PG extends IConnectorGroupDescription, PH extends IConnectorHandler<?, ?>> extends ExtensibleConfiguration implements IConfigurationElement<PD, PG, PH>, BaseConnectorHandler.IConnectorHandlerContainer {

	protected final List<IElementChangeListener> elementListeners = new ArrayList<>();

	protected IPluginEngine pluginEngine;

	public void setPluginEngine(IPluginEngine pluginEngine) {
		this.pluginEngine = pluginEngine;
	}

	@Override
	public void addElementListener(IElementChangeListener listener) {
		elementListeners.add(listener);
	}

	@Override
	public void removeElementListener(IElementChangeListener listener) {
		elementListeners.remove(listener);
	}

	@Override
	protected void notifyAttributeChanged(String name) {
		if (pluginEngine != null) {
			pluginEngine.onAttributeChanged(this, name);
		}
		for (IElementChangeListener listener : elementListeners) {
			listener.onAttributeChanged(this, name);
		}
	}

	protected void notifyElementNameChanged() {
		// No notification to pluginEngine, because it will already receive the ones from IHierarchyListener
		for (IElementChangeListener listener : elementListeners) {
			listener.onElementNameChanged(this);
		}
	}

	protected void notifyConnectorChanged(IConnectorHandler<?, ?> connector, IElementChangeListener.ConnectorChangeType type, IConnection connection, boolean invalid) {
		if (!invalid && pluginEngine != null) {
			pluginEngine.onConnectorChanged(this, connector, type, connection);
		}

		for (IElementChangeListener listener : elementListeners) {
			listener.onConnectorChanged(this, connector, type, connection);
		}
	}

	protected void notifyElementDeclarationChanged() {
		if (pluginEngine != null) {
			pluginEngine.onElementDeclarationChanged(this);
		}
		for (IElementChangeListener listener : elementListeners) {
			listener.onElementDeclarationChanged(this);
		}
	}

	@Override
	public void notifyConnectionRemoved(IConnectorHandler<?, ?> connector, IConnection connection, boolean invalid) {
		notifyConnectorChanged(connector, IElementChangeListener.ConnectorChangeType.Removed, connection, invalid);
	}

	@Override
	public void notifyConnectionAdded(IConnectorHandler<?, ?> connector, IConnection connection) {
		notifyConnectorChanged(connector, IElementChangeListener.ConnectorChangeType.Added, connection, false);
	}

	@Override
	public void notifyConnectionPropertiesChanged(IConnectorHandler<?, ?> handler) {
		notifyConnectorChanged(handler, IElementChangeListener.ConnectorChangeType.Properties, null, false);
	}

}
