/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.INamedValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.BusBreakoutModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IBusBreakoutModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleVisitor;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IConnectorInstance;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IModuleContainer;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses.BusLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses.IMappedBusPortInstance;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil;
import de.tu_darmstadt.rs.util.StreamUtil;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Created by wenzel on 14.06.16.
 */
@XmlRootElement(name = "busbreakout-commonModule", namespace = "http://www.rs.tu-darmstadt.de/ns/spartanmc/configuration/jc3")
public class BusBreakoutConfiguration<P extends IModuleContainer<?> & IConfigurationElement<?, ?, ?> & INamedValueResolver> extends CommonModuleConfiguration<P> implements IBusBreakoutModuleConfiguration<P> {

	private static final Logger logger = LoggerFactory.getLogger(BusBreakoutConfiguration.class);

	@XmlAttribute
	private String busDescriptionName;
	@XmlAttribute
	private Bus.Role busRole;

	/**
	 * Only for JAXB
	 */
	private BusBreakoutConfiguration() {
	}

	BusBreakoutConfiguration(String name, BusBreakoutModuleDescription description, P parent, Library library) {
		super(name, description, parent, library);
		this.busDescriptionName = description.getBusDescription().getName();
		this.busRole = description.getBusRole();
	}

	@Override
	public BusBreakoutModuleDescription getModuleDescription() {
		return library.getBusBreakoutDescription(library.getBusDescription(busDescriptionName), busRole);
	}

	@Override
	public <T> T apply(IModuleVisitor<T> visitor) {
		return visitor.applyToBusBreakoutModule(this);
	}

	@Override
	public IConnectorInstance getInternallyConnectedPort(IConnectorInstance port, BusLayout busLayout) {
		List<IMappedBusPortInstance> busPortInstances = busLayout.getBusPortInstances(this);

		if (port instanceof IPortHandler) {
			String connectedName = port.getPortName() + BusBreakoutModuleDescription.BUS_SUFFIX;
			return busPortInstances.stream().filter(p -> p.getPortName().equals(connectedName)).collect(StreamUtil.exactlyOne((l) -> "Found none or multiple items when searching for: " + connectedName + " " + l + " " + busPortInstances));
		} else {
			String connectedName = port.getPortName().substring(0, port.getPortName().length() - BusBreakoutModuleDescription.BUS_SUFFIX.length());
			return DocumentUtil.getPortHandler(this, connectedName);
		}
	}

	@Override
	public String toString() {
		return "BusBreakoutConf" + toModuleConfString();
	}
}
