/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.INamedValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleVisitor;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IModuleContainer;

import javax.xml.bind.annotation.XmlRootElement;

/**
 * Created by ramon on 04.12.2015.
 */
@XmlRootElement(name = "commonModule", namespace = "http://www.rs.tu-darmstadt.de/ns/spartanmc/configuration/jc3")
public class CommonModuleConfiguration<P extends IModuleContainer<?> & IConfigurationElement<?, ?, ?> & INamedValueResolver> extends ModuleConfiguration<P> {

	/**
	 * Only for JAXB
	 */
	protected CommonModuleConfiguration() {
	}

	public CommonModuleConfiguration(String name, IModuleDescription moduleDescription, P parent, Library library) {
		super(name, moduleDescription.getName(), parent, library);
	}


	@Override
	public boolean isUniqueName(String name) {
		return ModuleConfigurationFactory.isUniqueName(name, this, parent.getModules());
	}

	@Override
	public String toString() {
		return "CommonModuleConf" + toModuleConfString();
	}

	@Override
	public P getParent() {
		return parent;
	}

	@Override
	public ConfigurationDocument getDocument() {
		if (isGlobalModule()) {
			return (ConfigurationDocument) parent;
		} else {
			return ((SubsystemConfiguration) parent).getParent();
		}
	}

	@Override
	public String getAbsolutePath() {
		if (isGlobalModule()) {
			return '/' + getName();
		} else {
			//If a module was just removed, parent is already null
			if (parent == null) {
				return getName();
			}
			return parent.getAbsolutePath() + '/' + getName();
		}
	}

	@Override
	public ElementType getType() {
		return ElementType.CommonModule;
	}

	@Override
	public IModuleDescription getModuleDescription() {
		return library.getCommonDescription(getModuleDescriptionName());
	}

	@Override
	public <T> T apply(IModuleVisitor<T> visitor) {
		return visitor.applyToCommonModule(this);
	}

	private boolean isGlobalModule() {
		return parent instanceof ConfigurationDocument;
	}
}
