/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.configurations.IToolchainConfiguration;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.analysis.ClkReference;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.analysis.ParameterReference;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.INamed;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.devices.DeviceDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.MemoryModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.SubsystemModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.LibJConfigProject;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.BinaryOutputFormat;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.OrderedConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IReNameable;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBusHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPinHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.ISelectionHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IToolchainAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.handlers.AvailablePortsHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.handlers.DocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.handlers.NameHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.internal.LibAttacher;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.ConfigurationIssue;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.IIssueManager;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.IIssueSink;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.connections.BaseConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.connections.BusConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.connections.PartialConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.connections.PortConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.BasePortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.BusHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.PinHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.LibraryUtil;

import org.jetbrains.annotations.Nullable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.Marker;
import org.slf4j.MarkerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlValue;


/**
 * New Implementation where known attributes will be directly named, only pluggable attributes will use the old scheme, but wrapped,
 * so that they can be read in directly as HashMaps
 * Connections are kept in a seperate section, but will be automatically linked to their respective instances
 */
@XmlRootElement(name = "configuration", namespace = "http://www.rs.tu-darmstadt.de/ns/spartanmc/configuration/jc3")
public class ConfigurationDocument extends BaseConfigurationElement<TargetDescription.IoPin, TargetDescription.IoPinGroup, IPinHandler> implements IConfigurationDocument, IReNameable, IIssueSink, IIssueManager {

	/**
	 * Enables checking every issue for its actuality on every change to this document.
	 */
	private static final boolean AUTO_RESOLVE_ISSUES = true;
	private static final Marker attributeMarker = MarkerFactory.getMarker("AttributeHandling");
	private static final Marker hierarchyMarker = MarkerFactory.getMarker("DocumentHierarchy");
	private static final Marker connectionMarker = MarkerFactory.getMarker("Connections");
	private static final Logger logger = LoggerFactory.getLogger(ConfigurationDocument.class);
	private static final BinaryOutputFormat ELF8_OUTPUT_FORMAT = BinaryOutputFormat.Spartan_ELF8;
	public Map<String, PinHandler> pinHandlers;
	private final List<IConfigurationHierarchyListener> hierarchyListeners = new ArrayList<>();
	private final List<IDocumentChangeListener> documentListeners = new ArrayList<>();
	private final List<IConfigurationIssueListener> issueListeners = new ArrayList<>();
	private final Set<ConfigurationIssue> configurationIssues = new HashSet<>();
	private NameHandler nameHandler;
	private LibJConfigProject project;
	private DocumentHandler documentHandler;
	//JAXB
	private Library library;
	private String name;
	private String jconfigVersion;
	private TargetConfiguration target;
	private List<SubsystemConfiguration> subsystems = new ArrayList<>();
	private List<ModuleConfiguration<ConfigurationDocument>> childModules = new ArrayList<>();
	private List<BaseConnection<?, IPortHandler<?>>> tempConnections;
	private List<BusConnection> tempBusConnections;
	private Set<BaseConnection<?, IPortHandler<?>>> connections;
	private Set<BusConnection> busConnections;
	private List<PinHandler> tempPinHandlers;
	private AvailablePortsHandler availablePortsHandler;

	/**
	 * Only for JAXB
	 */
	private ConfigurationDocument() {
		tempPinHandlers = new ArrayList<>();
		tempBusConnections = new ArrayList<>();
		tempConnections = new ArrayList<>();
	}

	public ConfigurationDocument(String name, TargetDescription target, DeviceDescription device, String version, DocumentHandler documentHandler, Library library) {
		this.name = name;
		this.target = new TargetConfiguration(target, device, this);
		this.jconfigVersion = version;
		this.documentHandler = documentHandler;
		pinHandlers = new HashMap<>();
		this.library = library;
		initAttributes();
		busConnections = new HashSet<>();
		connections = new HashSet<>();
	}

	public static void verifyLibrary(Library library) {
		if (library == null) {
			throw new IllegalStateException("No library attached!");
		}
	}

	/**
	 * @param module to remove
	 * @param firstList a list where this module could be listed
	 * @param secondList another list
	 * @param parent parent of module / container
	 * @param document document to which it belongs
	 * @param internal a remove we need not tell external listeners like UI about (children of already removed containers)
	 * @param invalid whether module is already invalid (certain steps will be skipped)  @return true if actually removed (false, was not there in the first place)
	 */
	protected static boolean removeModule(ModuleConfiguration module, List<? extends ModuleConfiguration> firstList, List<? extends ModuleConfiguration> secondList, IConfigurationElement<?, ?, ?> parent, ConfigurationDocument document, boolean internal, boolean invalid) {
		if (firstList.remove(module)) {
			module.prepareRemove();
			module.invalidateModule();
			document.notifyModuleRemoved(parent, module, internal, invalid);
			return true;
		} else if (secondList.remove(module)) {
			module.prepareRemove();
			document.notifyModuleRemoved(parent, module, internal, invalid);
			return true;
		} else {
			return false;
		}
	}

	@Override
	public DocumentHandler getDocumentHandler() {
		return documentHandler;
	}

	@Override
	public BinaryOutputFormat getBinaryOutputFormat() {
		return ELF8_OUTPUT_FORMAT;
	}

	@Override
	public void attachLibrary(Library library) {
		if (this.library != null) {
			throw new IllegalStateException("Already attached!");
		}
		this.library = library;

		importConnections(); // import from JAXB so they can be disconnected by LibAttacher

		List<ModuleConfiguration<?>> removed = new LibAttacher(this, library).attach();

		initializeConnections(removed);
	}

	private void importConnections() {
		// BuiltIn JaxB puts entries in HashSets before resolving their children => out hashCode() only works if children are resolved!!
		if (connections == null || busConnections == null) {
			connections = new HashSet<>(tempConnections);
			tempConnections = null;
			busConnections = new HashSet<>(tempBusConnections);
			tempBusConnections = null;
		}
	}

	@Override
	public void addDocumentListener(IDocumentChangeListener listener) {
		documentListeners.add(listener);
	}

	@Override
	public void removeDocumentListener(IDocumentChangeListener listener) {
		documentListeners.remove(listener);
	}

	@Override
	public synchronized boolean addHierarchyListener(IConfigurationDocument.IConfigurationHierarchyListener listener) {
		return hierarchyListeners.add(listener);
	}

	@Override
	public synchronized void removeHierarchyListener(IConfigurationDocument.IConfigurationHierarchyListener listener) {
		hierarchyListeners.remove(listener);
	}

	@Override
	public SubsystemConfiguration addSubsystem(SubsystemModuleDescription moduleDescription) {
		return addSubsystem(moduleDescription, null);
	}

	@Override
	public SubsystemConfiguration addSubsystem(SubsystemModuleDescription moduleDescription, @Nullable String name) {
		verifyLibrary(library);
		String uniqueName = ModuleConfigurationFactory.findUniqueName(name, moduleDescription, this, subsystems);
		SubsystemConfiguration conf = new SubsystemConfiguration(uniqueName, moduleDescription.getName(), this, library);
		subsystems.add(conf);
		notifyModuleAdded(conf);

		return conf;
	}

	@Override
	public MemoryModuleConfiguration<ConfigurationDocument> addMemoryModule(MemoryModuleDescription moduleDescription) {
		return (MemoryModuleConfiguration<ConfigurationDocument>) addModule(moduleDescription);
	}

	@Override
	public LibJConfigProject getProject() {
		return documentHandler.getProject();
	}

	@Override
	@XmlAttribute(name = "jconfig-version")
	public String getJconfigVersion() {
		return jconfigVersion;
	}

	@Override
	public Library getLibrary() {
		return library;
	}

	@Override
	@XmlElement
	public TargetConfiguration getTarget() {
		return target;
	}

	public void setTarget(TargetConfiguration target) {
		this.target = target;
	}

	@Override
	@XmlElementWrapper(name = "subsystems")
	@XmlElement(name = "subsystem", namespace = "http://www.rs.tu-darmstadt.de/ns/spartanmc/configuration/jc3")
	public List<SubsystemConfiguration> getSubsystems() {
		return subsystems;
	}

	public void setSubsystems(List<SubsystemConfiguration> subsystems) {
		this.subsystems = subsystems;
	}

	@Override
	public synchronized void checkResolvedIssues() {
		configurationIssues.removeIf(this::checkResolved);
	}

	@Override
	public synchronized void addConfigurationIssueListener(IConfigurationIssueListener listener) {
		issueListeners.add(listener);
	}

	@Override
	public synchronized void removeConfigurationIssueListener(Object listener) {
		issueListeners.remove(listener);
	}

	@Override
	public synchronized Set<ConfigurationIssue> getConfigurationIssues() {
		return Collections.unmodifiableSet(configurationIssues);
	}

	@Override
	public PortConnection createPortConnection(IPortHandler<? extends IConnectorDescription> a, IPortHandler<? extends IConnectorDescription> b) {
		return new PortConnection(((BasePortHandler) a), ((BasePortHandler) b), this);
	}

	@Override
	public BusConnection createBusConnection(IBusHandler a, IBusHandler b) {
		return new BusConnection(((BusHandler) a), ((BusHandler) b), this);
	}

	@Override
	public PartialConnection createPartialConnection(IPortHandler<? extends IConnectorDescription> a, int aStart, IPortHandler<? extends IConnectorDescription> b, int bStart, int width) {
		return new PartialConnection(((BasePortHandler) a), aStart, ((BasePortHandler) b), bStart, width, this);
	}

	@Override
	public Set<BaseConnection<?, IPortHandler<?>>> getConnections() {
		return Collections.unmodifiableSet(connections);
	}

	@Override
	public Set<BusConnection> getBusConnections() {
		return Collections.unmodifiableSet(busConnections);
	}

	@Override
	public IIssueSink getIssueSink() {
		return this;
	}

	@Override
	public IIssueManager getIssueManager() {
		return this;
	}

	@Override
	public Stream<SubsystemConfiguration> streamSubsystems() {
		return subsystems.stream();
	}

	@Override
	public Stream<? extends ModuleConfiguration<ConfigurationDocument>> streamCommonModules() {
		return childModules.stream();
	}

	private void setBusConnections(Set<BusConnection> busConnections) {
		this.busConnections = busConnections;
	}

	public void setJconfigVersion(String jconfigVersion) {
		this.jconfigVersion = jconfigVersion;
	}

	private void afterUnmarshal(Unmarshaller u, Object parent) {
		mapAttributes();
		pinHandlers = tempPinHandlers.stream().collect(Collectors.toMap(PinHandler::getPortName, Function.identity()));
		tempPinHandlers = null;
	}

	public void initialize(DocumentHandler documentHandler) {
		this.documentHandler = documentHandler;
	}

	private void reportError(ConfigurationIssue error) {
		boolean isNew = configurationIssues.add(error);
		if (isNew) {
			logger.error("Configuration Problem: {}", error);
			notifyNewConfigurationError(error);
		}
	}

	private void reportWarning(ConfigurationIssue warning) {
		boolean isNew = configurationIssues.add(warning);
		if (isNew) {
			logger.warn("Configuration Warning: {}", warning);
			notifyNewConfigurationWarning(warning);
		}
	}

	/**
	 * Required, because otherwise getOppositeConnector might return an uninitialized Connector that crashes on certain actions
	 * Connectors that are not valid under the current library will be removed, connections removed
	 * @param removed
	 */
	private void initializeConnections(List<ModuleConfiguration<?>> removed) {
		Predicate<? super BaseConnection> pred = c -> {
			boolean moduleDropped = removed != null && (removed.contains(c.getPortAInt().getSourceElement()) || removed.contains(c.getPortBInt().getSourceElement()));
			if (moduleDropped) {
				return true; // silent remove, already warned when owner was removed
			}

			BaseConnection.InitResult initResult = c.ensureInitialized();
			boolean initialized = initResult == BaseConnection.InitResult.Success;
			boolean invalid = false;
			if (initialized) {
				invalid = !c.isValid(false);
			}
			if (!initialized) {
				IConnectorHandler stillPresent;
				switch (initResult) {
					case ProblemA:
						stillPresent = c.getPortB();
						break;
					case ProblemB:
						stillPresent = c.getPortA();
						break;
					default:
						stillPresent = null;
				}
				report(ConfigurationIssue.createGhostConnection(stillPresent, c, "Connection Initialization"));
			}
			if (invalid) {
				report(ConfigurationIssue.createMismatchedConnectionRemoved(c, "Connection Initialization"));
			}
			return !initialized || invalid;
		};
		connections.removeIf(pred);
		busConnections.removeIf(pred);
	}

	@XmlElementWrapper(name = "pins")
	@XmlElement(name = "pin")
	private List<PinHandler> getPinHandlers() {
		if (pinHandlers != null) {
			tempPinHandlers = pinHandlers.values().stream().sorted(IConnectorHandler.NAME_COMPARATOR).collect(Collectors.toList());
		}
		return tempPinHandlers;
	}

	public void setPinHandlers(List<PinHandler> pinHandlers) {
		this.tempPinHandlers = pinHandlers;
		this.pinHandlers = null;
	}

	@Override
	public Optional<IPinHandler> getDefaultPin(String function) {
		return target.getTargetDescription().getDefaultPin(function).map(this::getPortHandler);
	}

	@Override
	public PinHandler getPortHandler(TargetDescription.IoPin pinDescription) {
		String name = pinDescription.getName();
		PinHandler handler = pinHandlers.get(name);

		if (handler == null) {
			handler = new PinHandler(this, pinDescription);
			pinHandlers.put(name, handler);
		} else if (!handler.isInitialized()) {
			handler.initialize(pinDescription);
		}

		return handler;
	}

	@Override
	public List<TargetDescription.IoPinGroup> getPortGroupDescriptions() {
		return target.getTargetDescription().getIoPinGroups();
	}

	@Override
	public Optional<TargetDescription.IoPin> findPortDescription(String portName) {
		return target.getTargetDescription().findPinDescription(portName);
	}

	@Override
	public List<Bus> getBusDescriptions() {
		return new ArrayList<>();
	}

	@Override
	public IBusHandler getBusHandler(Bus busDescription) {
		return null;
	}

	public void requestInitialize(PinHandler pinHandler) {
		pinHandler.initialize(target.getTargetDescription().findPinDescription(pinHandler.getPortName()).orElseThrow(() -> new IllegalStateException("Could not find " + pinHandler.getPortName() + " in target description.")));
	}

	public boolean addConnection(BaseConnection connection) {
		if (connection.getType() == IConnection.ConnectionType.Bus) {
			return addBusConnection((BusConnection) connection);
		} else {
			return addPortConnection(connection);
		}
	}

	public boolean removeConnection(BaseConnection connection) {
		if (connection.getType() == IConnection.ConnectionType.Bus) {
			return removeBusConnection(connection);
		} else {
			return removePortConnection(connection);
		}
	}

	private boolean removePortConnection(BaseConnection connection) {
		boolean remove = connections.remove(connection);
		notifyDocumentChange();
		return remove;
	}

	private boolean addPortConnection(BaseConnection connection) {
		boolean add = connections.add(connection);
		notifyDocumentChange();
		return add;
	}

	private boolean addBusConnection(BusConnection connection) {
		boolean add = busConnections.add(connection);
		notifyDocumentChange();
		return add;
	}

	private boolean removeBusConnection(BaseConnection connection) {
		boolean remove = busConnections.remove(connection);
		notifyDocumentChange();
		return remove;
	}

	protected synchronized void notifyModuleAdded(ModuleConfiguration module) {
		if (availablePortsHandler != null) {
			availablePortsHandler.moduleAdded(module);
		}

		for (IConfigurationDocument.IConfigurationHierarchyListener l : hierarchyListeners) {
			l.onModuleAdded(module);
		}

		if (pluginEngine != null) {
			module.setPluginEngine(pluginEngine);
			pluginEngine.onModuleAdded(module);
		}
		logger.debug(hierarchyMarker, "module added: {} to parent {}", module, module.getParent());
		notifyDocumentChange();
	}

	protected synchronized void notifyModuleRemovedFromAvailablePorts(ModuleConfiguration module) {
		if (availablePortsHandler != null) {
			availablePortsHandler.moduleRemoved(module);
		}
	}

	/**
	 * Forward notice of removed module to whom it may concern.
	 * (The module is already removed, any refresh will reflect new state)
	 *
	 * @param parent parent from which it was removed
	 * @param module removed module
	 * @param internal this is a child of another removed module (we need to handle this, but not tell hierarchy listener, because parent has already delisted)
	 * @param moduleInvalid the removed module was invalid, skip everything that requires it to be
	 */
	public synchronized void notifyModuleRemoved(IConfigurationElement<?, ?, ?> parent, ModuleConfiguration module, boolean internal, boolean moduleInvalid) {
		notifyModuleRemovedFromAvailablePorts(module);

		if (!internal) {
			for (IConfigurationHierarchyListener l : hierarchyListeners) {
				l.onModuleRemoved(parent, module);
			}
		}

		if (!moduleInvalid && pluginEngine != null) {
			module.setPluginEngine(null);
			pluginEngine.onModuleRemoved(parent, module);
		}

		logger.debug(hierarchyMarker, "module removed from {}: {}", parent, module);
		notifyDocumentChange();
	}

	protected synchronized void notifyModuleDeclarationChanged(ModuleConfiguration module) {
		if (availablePortsHandler != null) {
			availablePortsHandler.moduleDeclarationChanged(module);
		}

		if (pluginEngine != null) {
			pluginEngine.onElementDeclarationChanged(this);
		}

		logger.debug("module {} changed declaration", module);
		notifyDocumentChange();
	}

	@Override
	public String toString() {
		return "ConfigurationDocument{" +
				"name='" + name + '\'' +
				", project=" + project +
				", jconfigVersion='" + jconfigVersion + '\'' +
				'}';
	}

	@Override
	public CalculatedValueResult<? extends Number> resolveParam(ParameterReference param) {
		throw new UnsupportedOperationException("No parameters on Document");// will not be called, we have no parameters
	}

	@Override
	public CalculatedValueResult<? extends Number> resolveClkPort(ClkReference clkPort) {//Needed for BusClk on target boundary
		throw new UnsupportedOperationException("No ClkPorts on Document");// will not be called, we have nothing that resolves against ClkPorts
	}

	@Override
	public CalculatedValueResult<? extends String> resolveParamString(ParameterReference param) {
		throw new UnsupportedOperationException("No parameters on Document");// will not be called, we have no parameters
	}

	@Override
	public CalculatedValueResult<Double> resolveParamFloat(ParameterReference param) {
		throw new UnsupportedOperationException("No parameters on Document");// will not be called, we have no parameters
	}

	@XmlAttribute
	@Override
	public String getName() {
		return name;
	}

	@Override
	public void setName(String name) {
		this.name = name;
		notifyElementNameChanged();
		notifyNameChanged(this);
	}

	@Override
	public boolean isUniqueName(String name) {
		return true;//No siblings...
	}

	@Override
	public void reportInvalidNameIssue() {
		report(ConfigurationIssue.createInvalidModuleName(this, "Name Validation"));
	}

	@Override
	public String getGlobalName() {
		return "";
	}

	@Override
	public boolean isValid(boolean report) {
		boolean pluginValid = pluginEngine == null || pluginEngine.isValid(this, report);
		if (!pluginValid) {
			logger.info("PluginEngine declared document invalid");
		}
		boolean pinsValid = DocumentUtil.streamConnectorHandlers(this).reduce(true, (v, pin) -> pin.isValid(report) && v, Boolean::logicalAnd);
		return pinsValid && pluginValid;
	}

	@Override
	public IConfigurationElement<?, ?, ?> getParent() {
		return null;
	}

	@Override
	public String getAbsolutePath() {
		return "";
	}

	@Override
	public ElementType getType() {
		return ElementType.Configuration;
	}

	@Override
	public IAttributeHandler getNameHandler() {
		if (nameHandler == null) {
			nameHandler = new NameHandler(this);
		}
		return nameHandler;
	}

	@Override
	public boolean wasRemoved() {
		return documentHandler == null;
	}

	synchronized void notifyNameChanged(IConfigurationElement<?, ?, ?> element) {
		if (pluginEngine != null) {
			pluginEngine.onElementNameChanged(element);
		}

		for (IConfigurationDocument.IConfigurationHierarchyListener l : hierarchyListeners) {
			l.onElementNameChanged(element);
		}
		logger.debug(hierarchyMarker, "element changed name: {}", element);
		notifyDocumentChange();
	}

	@XmlElementWrapper(name = "childModules")
	@XmlElementRef
	@Override
	public List<ModuleConfiguration<ConfigurationDocument>> getModules() {
		return childModules;
	}

	@Override
	public ModuleConfiguration<ConfigurationDocument> addModule(IModuleDescription moduleDescription) {
		return addModule(moduleDescription, null);
	}

	@Override
	public ModuleConfiguration<ConfigurationDocument> addModule(IModuleDescription moduleDescription, @Nullable String name) {
		verifyLibrary(library);
		String uniqueName = ModuleConfigurationFactory.findUniqueName(name, moduleDescription, this, childModules);
		ModuleConfiguration<ConfigurationDocument> conf = ModuleConfigurationFactory.createConfigurationFor(uniqueName, moduleDescription, this, this, library);
		childModules.add(conf);
		notifyModuleAdded(conf);

		return conf;
	}

	@Override
	public boolean removeModule(IModuleConfiguration module) {
		return removeModule((ModuleConfiguration) module, subsystems, childModules, this, this, false, false);
	}

	public boolean removeModuleInternal(IModuleConfiguration module, boolean internal, boolean invalid) {
		return removeModule((ModuleConfiguration) module, subsystems, childModules, this, this, internal, invalid);
	}

	@Override
	public Stream<? extends ModuleConfiguration<ConfigurationDocument>> streamAllModules() {
		return Stream.concat(streamSubsystems(), streamCommonModules());
	}

	@Override
	public Stream<? extends ModuleConfiguration<?>> streamModulesRecursive() {
		Stream<? extends ModuleConfiguration<? extends SubsystemConfiguration>> b = streamSubsystems().flatMap(SubsystemConfiguration::streamModulesRecursive);
		return Stream.concat(streamAllModules(), b);
	}

	private void setChildModules(List<ModuleConfiguration<ConfigurationDocument>> childModules) {
		this.childModules = childModules;
	}

	@XmlElementWrapper(name = "connections")
	@XmlElementRef(namespace = "http://www.rs.tu-darmstadt.de/ns/spartanmc/configuration/jc3")
	private List<BaseConnection<?, IPortHandler<?>>> getTempConnections() {
		if (tempConnections == null) {
			return connections.stream().sorted(OrderedConnection.COMPARATOR).collect(Collectors.toList());
		} else {
			return tempConnections;
		}
	}

	private void setTempConnections(List<BaseConnection<?, IPortHandler<?>>> connections) {
		this.tempConnections = connections;
	}

	public Set<BaseConnection<?, IPortHandler<?>>> getConnectionsInternal() {
		return connections;
	}

	@XmlElementWrapper(name = "busConnections")
	@XmlElement(name = "busConnection", namespace = "http://www.rs.tu-darmstadt.de/ns/spartanmc/configuration/jc3")
	private List<BusConnection> getTempBusConnections() {
		if (tempBusConnections == null) {
			return busConnections.stream().sorted(OrderedConnection.COMPARATOR).collect(Collectors.toList());
		} else {
			return tempBusConnections;
		}
	}

	private void setTempBusConnections(List<BusConnection> busConnections) {
		this.tempBusConnections = busConnections;
	}

	public Set<BusConnection> getBusConnectionsInternal() {
		return busConnections;
	}

	public synchronized AvailablePortsHandler getAvailablePortsHandler() {
		if (availablePortsHandler == null) {
			availablePortsHandler = new AvailablePortsHandler(this);
		}
		return availablePortsHandler;
	}

	/**
	 * Any change to anything, that makes the current state unsaved.
	 */
	public void notifyDocumentChange() {
		if (documentHandler != null) {
			documentHandler.onDocumentChange();
		}

		if (AUTO_RESOLVE_ISSUES && issueListeners.size() > 0 && configurationIssues.size() > 0) {
			checkResolvedIssues();
		}
	}

	private synchronized void notifyNewConfigurationError(ConfigurationIssue issue) {
		for (IConfigurationIssueListener listener : issueListeners) {
			listener.onNewConfigurationIssue(issue, ConfigurationIssue.IssueType.Warning);
		}
	}

	private synchronized void notifyNewConfigurationWarning(ConfigurationIssue issue) {
		for (IConfigurationIssueListener listener : issueListeners) {
			listener.onNewConfigurationIssue(issue, ConfigurationIssue.IssueType.Warning);
		}
	}

	private synchronized void notifyConfigurationIssueDismissed(ConfigurationIssue issue) {
		for (IConfigurationIssueListener listener : issueListeners) {
			listener.onConfigurationIssueDismissed(issue);
		}
	}

	private synchronized void notifyConfigurationIssueResolved(ConfigurationIssue issue) {
		for (IConfigurationIssueListener listener : issueListeners) {
			listener.onConfigurationIssueResolved(issue);
		}
	}

	@Override
	protected void notifyAttributeChanged(String name) {
		super.notifyAttributeChanged(name);
		notifyDocumentChange();
	}

	/**
	 * The whole target-description has changed
	 */
	private void handleTargetDeclarationChange() {
		notifyDocumentChange();

		report(ConfigurationIssue.createTargetDeclarationChange());

		Stream<TargetDescription.IoPin> pinStream = getPortGroupDescriptions().stream().flatMap(pg -> pg.getPorts().stream());

		Map<String, PinHandler> oldHandlers = pinHandlers;
		pinHandlers = pinStream.map(newPin -> new PinMapping(oldHandlers.get(newPin.getName()), newPin)).filter(PinMapping::hasHandler)
				.filter(PinMapping::isConnected).map(pm -> PinHandler.migrate(pm.pinHandler, pm.pinDesc)).filter(Objects::nonNull).collect(Collectors.toMap(PinHandler::getPortName, Function.identity()));

		pinHandlers.keySet().forEach(oldHandlers::remove);

		oldHandlers.values().stream().filter(PinHandler::isConnected).flatMap(PinHandler::streamConnections).forEach(c -> {
			c.disconnect();
			report(ConfigurationIssue.createMismatchedConnectionRemoved(c, "Target Declaration Change"));
		});

		initializeConnections(null);// force check connections still match // may be redundant?

		synchronized (this) {
			if (availablePortsHandler != null) {
				availablePortsHandler.onTargetChange();
			}
		}

		notifyTargetChanged();
		notifyElementDeclarationChanged();
	}


	/**
	 * The toolchain has changed
	 */
	private void handleToolchainChange() {
		notifyTargetChanged();
		notifyElementDeclarationChanged();
	}

	public void notifyPinPropertiesChanged(IPinHandler pinHandler) {
		notifyConnectorChanged(pinHandler, IElementChangeListener.ConnectorChangeType.Properties, null, false);
		notifyDocumentChange();
	}

	/**
	 * Attributes on target changed
	 */
	public void notifyTargetChanged() {

		if (pluginEngine != null) {
			pluginEngine.onTargetChanged();
		}

		logger.debug(attributeMarker, "Target Changed: {}", target);
		for (IDocumentChangeListener listener : documentListeners) {
			listener.onTargetChanged();
		}

		notifyDocumentChange();
	}

	@Override
	public void report(ConfigurationIssue issue) {
		issue.setDocument(this);
		if (issue.getLevel() == ConfigurationIssue.IssueType.Error) {
			reportError(issue);
		} else {
			reportWarning(issue);
		}
	}

	@Override
	public synchronized boolean checkResolved(ConfigurationIssue issue) {
		boolean resolved = issue.getActuality() == ConfigurationIssue.Actuality.Resolved;

		if (resolved) {
			logger.debug("Resolved: {}", issue);
			notifyConfigurationIssueResolved(issue);
		}

		return resolved;
	}

	@Override
	public synchronized boolean dismissIssue(ConfigurationIssue issue) {
		logger.debug("Dismissing: {}", issue);
		boolean removed = configurationIssues.remove(issue);

		if (removed) {
			notifyConfigurationIssueDismissed(issue);
		}

		return removed;
	}


	public static class TargetConfiguration extends ExtensibleConfiguration implements ITargetConfiguration, IToolchainConfiguration {

		private ConfigurationDocument parent;

		@XmlAttribute
		private String descriptionName;
		private SelectionHandler<TargetDescription> targetSelectionHandler;

		@XmlAttribute
		private String targetDevice;
		private SelectionHandler<DeviceDescription> deviceSelectionHandler;

		@XmlAttribute
		private String toolchain;
		private SelectionHandler<DeviceDescription.Toolchain> toolchainSelectionHandler;

		private Map<String, ToolchainParamHandler> toolchainParams;
		private List<ToolchainParamHandler> toolchainParamList;

		/**
		 * JAXB only
		 */
		@Deprecated
		public TargetConfiguration() {
			toolchainParamList = new ArrayList<>();
		}

		public TargetConfiguration(TargetDescription targetDescription, DeviceDescription deviceDescription, ConfigurationDocument parent) {
			this.parent = parent;
			this.descriptionName = targetDescription.getName();
			this.targetDevice = deviceDescription.getName();

			initAttributes();
			toolchainParams = new HashMap<>();

			initializeToolchainParams(deviceDescription);
		}

		private void afterUnmarshal(Unmarshaller u, Object parent) {
			this.parent = (ConfigurationDocument) parent;

			toolchainParams = toolchainParamList.stream().collect(Collectors.toMap(ToolchainParamHandler::getName, Function.identity()));
			toolchainParamList = null;
			mapAttributes();
		}

		private void initializeToolchainParams(DeviceDescription deviceDescription) {
			getToolchain(deviceDescription).getParams().values().forEach(this::getToolchainParamHandler);
		}

		@Override
		public SelectionHandler<TargetDescription> getTargetSelectionHandler() {
			if (targetSelectionHandler == null) {
				targetSelectionHandler = new TargetSelectionHandler(parent.library, LibraryUtil.getSortedTargetDescriptions(parent.library.getAvailableTargets()));//sorting belongs into UI, but then i need to abstrac comboboxes to use custom sorting. This is more efficient
			}

			return targetSelectionHandler;
		}

		@Override
		public SelectionHandler<DeviceDescription> getDeviceSelectionHandler() {
			if (deviceSelectionHandler == null) {
				deviceSelectionHandler = new DeviceSelectionHandler(parent.library, LibraryUtil.getSortedDeviceDescriptions(parent.library.getSupportedDeviceDescriptions(parent.library.getTargetDescription(descriptionName))));
			}

			return deviceSelectionHandler;
		}

		@Override
		public ISelectionHandler<DeviceDescription.Toolchain> getToolchainSelectionHandler() {

			if (toolchainSelectionHandler == null) {
				//Force initialization of toolchain for legacy documents
				getToolchain();

				toolchainSelectionHandler = new ToolchainSelectionHandler(parent.library, getDeviceDescription().getToolchains());
			}
			return toolchainSelectionHandler;
		}

		@Override
		public ToolchainParamHandler getToolchainParamHandler(DeviceDescription.DevParam paramDescription) {
			ToolchainParamHandler handler = toolchainParams.get(paramDescription.getName());
			if (handler == null) {
				handler = new ToolchainParamHandler(parent, paramDescription);
				toolchainParams.put(paramDescription.getName(), handler);
			} else if (!handler.isReady()) {
				handler.prepare(parent, paramDescription);
			}

			return handler;
		}

		@Override
		public TargetDescription getTargetDescription() {
			return parent.library.getTargetDescription(descriptionName);
		}

		@Override
		public DeviceDescription getDeviceDescription() {
			return parent.library.getDeviceDescription(targetDevice);
		}

		/**
		 * Convenience Method for accessing the Device descriptor for the currently selected device on the currently selected target
		 *
		 * @return Device or Exception if jc3 settings are invalid
		 */
		@Override
		public TargetDescription.Device getDevice() {
			return getTargetDescription().getDevices().stream().filter(d -> (d.getName().equals(targetDevice))).findAny().get();
		}

		@Override
		public IToolchainConfiguration getCurrentConfiguration() {
			return this;
		}

		public DeviceDescription.Toolchain getToolchain(DeviceDescription deviceDescription) {

			if (toolchain == null) {
				toolchain = deviceDescription.getToolchains().iterator().next().getName();
				logger.info("No explicit toolchain set. Setting to {}", toolchain);
			}
			try {
				return deviceDescription.getToolchain(toolchain);
			} catch (IllegalArgumentException e) {
				toolchain = getDeviceDescription().getToolchains().iterator().next().getName();
				logger.warn("Toolchain is no longer valid. Setting to {}", toolchain);
				return getDeviceDescription().getToolchain(toolchain);
			}
		}

		@Override
		public DeviceDescription.Toolchain getToolchain() {

			return getToolchain(getDeviceDescription());
		}

		@Override
		public Stream<Define> streamVerilogDefines(boolean simulation) {
			final Stream<Define> result = Stream.of(
					new Define("jc3_toolchain", getToolchainImplementation()),
					new Define("jc3_device", replaceSpecialChars(getDeviceDescription().getName())),
					new Define("jc3_family", replaceSpecialChars(getDeviceDescription().getFamily())),
					new Define("jc3_target", replaceSpecialChars(getTargetDescription().getName()))
			).flatMap(def -> {
				//From a name -> value define, make additionally a define that has both name and value concatenated with "_" as name
				//Needed because verilog has no `if directive in preprocessor, only ifdef...
				return Stream.of(def, new Define(def.getName()+"_"+def.getValue(),null));
			});
			if (simulation) {
				return Stream.concat(Stream.of(new Define("SIMULATION",null)), result);
			} else {
				return result;
			}
		}

		private String replaceSpecialChars(String name) {
			return name.replaceAll("\\W","_");
		}

		@XmlElementWrapper(name = "toolchain")
		@XmlElement(name = "toolchainParam", namespace = "http://www.rs.tu-darmstadt.de/ns/spartanmc/configuration/jc3")
		private List<ToolchainParamHandler> getToolchainParams() {
			if (toolchainParams != null) {
				toolchainParamList = toolchainParams.values().stream()
						.sorted(Comparator.comparing(ToolchainParamHandler::getName))
						.collect(Collectors.toList());
			}
			return toolchainParamList;
		}

		private void setToolchainParams(List<ToolchainParamHandler> toolchainParamList) {
			this.toolchainParamList = toolchainParamList;
			toolchainParams = null;
		}

		@Override
		protected void notifyAttributeChanged(String name) {
			parent.notifyAttributeChanged(name);
		}

		@Override
		public String getDeviceName() {
			return targetDevice;
		}

		@Override
		public String getFamilyName() {
			return getDeviceDescription().getFamily();
		}

		@Override
		public String getTargetName() {
			return descriptionName;
		}

		@Override
		public String getToolchainImplementation() {
			return getToolchain().getToolchain();
		}

		private class TargetSelectionHandler extends SelectionHandler<TargetDescription> {

			protected TargetSelectionHandler(Library library, List<TargetDescription> validValues) {
				super(library, validValues);
			}

			@Override
			protected void setValueInternal(String value, TargetDescription description) {
				descriptionName = value;
				deviceSelectionHandler = null;
				toolchainSelectionHandler = null;
				toolchain = null;
				getDeviceSelectionHandler();

				if (!deviceSelectionHandler.isValid()) {
					DeviceDescription deviceDescription = library.getSupportedDeviceDescriptions(getTargetDescription()).iterator().next();
					logger.warn("Old DeviceDescription is not available on new target. Defaulting to {}", deviceDescription);
					deviceSelectionHandler.setValue(deviceDescription);
				} else {
					parent.handleTargetDeclarationChange();// DeviceSelection will call it for us.
				}
			}

			@Override
			protected String getValueInternal() {
				return descriptionName;
			}

			@Override
			public String toString() {
				return "TargetSelectionHandler{" +
						"target=" + getValueInternal() +
						'}';
			}
		}


		private class DeviceSelectionHandler extends SelectionHandler<DeviceDescription> {

			protected DeviceSelectionHandler(Library library, List<DeviceDescription> validValues) {
				super(library, validValues);
			}

			@Override
			protected void setValueInternal(String value, DeviceDescription description) {
				targetDevice = value;
				toolchainSelectionHandler = null;

				toolchainParams.clear();
				initializeToolchainParams(description);

				logger.warn("Toolchain Params have been reset to their default values");

				parent.handleTargetDeclarationChange();
			}

			@Override
			protected String getValueInternal() {
				return targetDevice;
			}

			@Override
			public String toString() {
				return "DeviceSelectionHandler{" +
						"device=" + getValueInternal() +
						'}';
			}
		}


		private class ToolchainSelectionHandler extends SelectionHandler<DeviceDescription.Toolchain> {

			protected ToolchainSelectionHandler(Library library, List<DeviceDescription.Toolchain> validValues) {
				super(library, validValues);
			}

			@Override
			protected void setValueInternal(String value, DeviceDescription.Toolchain toolchain) {
				TargetConfiguration.this.toolchain = value;

				toolchainParams.clear();
				initializeToolchainParams(getDeviceDescription());

				logger.warn("Toolchain Params have been reset to their default values");

				parent.handleToolchainChange();
			}

			@Override
			protected String getValueInternal() {
				return toolchain;
			}

			@Override
			public String toString() {
				return "ToolchainSelectionHandler{" +
						"toolchain=" + getValueInternal() +
						'}';
			}
		}


		private abstract class SelectionHandler<T extends INamed> implements ISelectionHandler<T> {

			protected final Library library;
			private final List<T> validValues;

			protected SelectionHandler(Library library, List<T> validValues) {
				this.library = library;
				this.validValues = validValues;
			}

			@Override
			public List<T> getValidValues() {
				return validValues;
			}

			@Override
			public boolean isValidValue(T value) {
				return validValues.contains(value);
			}

			@Override
			public boolean isValid() {
				return validValues.stream().anyMatch(d -> d.getName().equals(getValueInternal()));
			}

			@Override
			public T getValue() {
				return validValues.stream().filter(d -> d.getName().equals(getValueInternal())).findAny().orElse(null);
			}

			@Override
			public void setValue(T value) {
				if (!isValidValue(value)) {
					throw new IllegalArgumentException("Value must be listed in validValues. Tried to set " + value + ", allowed: " + validValues);
				}

				String name = value.getName();

				if (name != null && name.equals(getValueInternal())) {
					return;
				}

				setValueInternal(name, value);
			}

			@Override
			public boolean isEditable() {
				return true;
			}

			protected abstract void setValueInternal(String value, T orgValue);

			protected abstract String getValueInternal();
		}

		@Override
		public String toString() {
			return "TargetConfiguration{" +
					"descriptionName='" + descriptionName + '\'' +
					", targetDevice='" + targetDevice + '\'' +
					", toolchainParams=" + toolchainParams +
					", attributes=" + getAttributes() +
					'}';
		}


	}


	private static class PinMapping {

		private final PinHandler pinHandler;
		private final TargetDescription.IoPin pinDesc;

		public PinMapping(PinHandler pinHandler, TargetDescription.IoPin pinDesc) {
			this.pinHandler = pinHandler;
			this.pinDesc = pinDesc;
		}

		public boolean hasHandler() {
			return pinHandler != null;
		}

		public boolean isConnected() {
			return pinHandler.isConnected();
		}
	}


	private static class ToolchainParamHandler implements IToolchainAttributeHandler {

		private ConfigurationDocument parent;
		@XmlAttribute
		private String name;
		@XmlAttribute
		private boolean isDefault;
		private DeviceDescription.DevParam paramDescription;
		@XmlValue
		private String value;

		/**
		 * JaxB only!
		 */
		@Deprecated
		public ToolchainParamHandler() {
		}

		public ToolchainParamHandler(ConfigurationDocument parent, DeviceDescription.DevParam paramDescription) {
			this.parent = parent;
			this.paramDescription = paramDescription;
			this.name = paramDescription.getName();
			this.isDefault = true;
			this.value = paramDescription.getValue();
		}

		private void afterUnmarshal(Unmarshaller u, Object parent) {
			this.parent = ((TargetConfiguration) parent).parent;
		}

		@Override
		public String getName() {
			return name;
		}

		@Override
		public String getDefaultValue() {
			return paramDescription.getValue();
		}

		@Override
		public boolean isDefaultValue() {
			return isDefault;
		}

		@Override
		public void resetToDefault() {
			boolean changed = !isDefault;
			isDefault = true;

			value = isReady()? getDefaultValue() : null;

			if (changed) {
				parent.notifyTargetChanged();
			}
		}

		private void prepare(ConfigurationDocument parent, DeviceDescription.DevParam paramDescription) {
			this.parent = parent;
			this.paramDescription = paramDescription;

			String defaultValue = getDefaultValue();
			if (isDefault && !defaultValue.equals(value)) {
				logger.warn("Toolchain Parameter {} on Target {}: Default has changed from {} to {}", name, parent.target.descriptionName, value, defaultValue);
				parent.report(ConfigurationIssue.createDescriptionChange("Toolchain parameter default has changed", "Toolchain Param Initialization"));
				value = defaultValue;
				parent.notifyTargetChanged();
			}
		}

		private boolean isReady() {
			return paramDescription != null && parent != null;
		}

		@Override
		public boolean isValid(boolean report) {
			return true;
		}

		@Override
		public boolean isValidValue(String value) {
			return true;
		}

		@Override
		public CalculatedValueResult<String> getValue() {
			return CalculatedValueResult.of(isDefault? getDefaultValue() : value);
		}

		/**
		 * @param value null resets param to default value
		 */
		@Override
		public void setStoredValue(String value) {
			boolean changed;
			if (value != null) {
				changed = !value.equals(this.value);
			} else {
				changed = this.value != null;
			}

			changed |= isDefault;

			if (changed) {
				this.value = value;
			}

			if (changed) {
				isDefault = false;
			}

			if (isReady() && changed) {
				parent.notifyTargetChanged();
			}
		}

		@Override
		public CalculatedValueResult<Boolean> isEditable() {
			return CalculatedValueResult.of(true);
		}

		@Override
		public String toString() {
			return "ToolchainParamHandler{" +
					"name='" + name + '\'' +
					", value='" + value + '\'' +
					", isDefault=" + isDefault +
					'}';
		}
	}


}
