/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.references.IReference;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.BaseParameterHandler;

/**
 * Dummy implementation, that contains its value as String. Only intended for unmarshalling with JAXB.
 * Will be replaced with subclasses compliant with description on first use
 *
 * @author Ramon Wirsch
 */
public class DummyParameterHandler extends BaseParameterHandler {

	private String value;
	private String name;

	public DummyParameterHandler() {
		super(null);
	}

	public DummyParameterHandler(String name, String value) {
		super(null);

		this.name = name;
		this.value = value;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public CalculatedValueResult<String> getValue() {
		return CalculatedValueResult.of(value);
	}

	@Override
	public String getStoredValue() {
		return value;
	}

	@Override
	public void setStoredValue(String value) {
		this.value = value;
	}

	@Override
	public CalculatedValueResult<Boolean> isEditable() {
		return CalculatedValueResult.of(false);
	}

	@Override
	public boolean isVirtual() {
		throw new UnsupportedOperationException("DummyParameterHandler should not have been exposed!!");
	}

	@Override
	public void onReferencedParameterChanged(IReference source) {
		// dummy param here
	}

	@Override
	public boolean isValid(boolean report) {
		return false;
	}

	@Override
	public boolean isValidValue(String value) {
		return false;
	}

	@Override
	public String getCanonicalName() {
		throw new UnsupportedOperationException("DummyParameterHandler should not have been exposed!!");
	}

	@Override
	public String getValueConverted(de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterConverter converter) {
		throw new UnsupportedOperationException("DummyParameterHandler should not have been exposed!!");
	}

	@Override
	public CalculatedValueResult<Boolean> isCurrentlyRelevant() {
		throw new UnsupportedOperationException("DummyParameterHandler should not have been exposed!!");
	}

	@Override
	public ModuleConfiguration<?> getSourceElement() {
		throw new UnsupportedOperationException("DummyParameterHandler should not have been exposed!!");
	}
}
