/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IGenericAttributeContainer;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlValue;

/**
 * Created by Ramon on 30.11.2015.
 */
public abstract class ExtensibleConfiguration implements IGenericAttributeContainer {

	private List<StringAttributeHandler> attributeList = new ArrayList<>();

	private Map<String, StringAttributeHandler> attributes;

	@XmlElement(name = "attribute")
	private List<StringAttributeHandler> getAttributeSet() {
		if (attributes != null) {
			attributeList = attributes.values().stream()
					.sorted(Comparator.comparing(StringAttributeHandler::getName))
					.collect(Collectors.toList());;
		}
		return attributeList;
	}

	private void setAttributeList(List<StringAttributeHandler> attributeList) {
		this.attributeList = attributeList;
		attributes = null;
	}

	protected Map<String, StringAttributeHandler> getAttributes() {
		return attributes;
	}

	@Override
	public StringAttributeHandler getAttributeHandler(String attribute) {
		StringAttributeHandler handler = attributes.get(attribute);
		if (handler == null) {
			handler = new StringAttributeHandler(attribute, this);
			attributes.put(attribute, handler);
		}
		return handler;
	}

	@Override
	public Set<String> getKnownAttributes() {
		return attributes.keySet();
	}

	protected void initAttributes() {
		attributeList = null;
		attributes = new HashMap<>();
	}

	protected void mapAttributes() {
		attributes = attributeList.stream().collect(Collectors.toMap(StringAttributeHandler::getName, Function.identity()));
		attributeList = null;
	}

	protected abstract void notifyAttributeChanged(String name);

	public static class StringAttributeHandler implements IAttributeHandler {

		private String value;
		private String name;
		private ExtensibleConfiguration parent;

		@Deprecated
		public StringAttributeHandler() {
		}

		public StringAttributeHandler(String name, ExtensibleConfiguration parent) {
			this.name = name;
			this.parent = parent;
		}

		@XmlAttribute
		public String getName() {
			return name;
		}

		private void setName(String name) {
			this.name = name;
		}

		@Override
		public boolean isValid(boolean report) {
			return true;
		}

		@Override
		public boolean isValidValue(String value) {
			return true;
		}

		@Override
		public CalculatedValueResult<String> getValue() {
			return CalculatedValueResult.ofNullable(value);
		}

		@XmlValue
		public String getStoredValue() {
			return value;
		}

		@Override
		public void setStoredValue(String value) {
			if ((this.value != null && !this.value.equals(value)) || value != null) {
				this.value = value;

				if (parent != null) {
					parent.notifyAttributeChanged(name);
				}
			}
		}

		@Override
		public CalculatedValueResult<Boolean> isEditable() {
			return CalculatedValueResult.of(true);
		}

		private void afterUnmarshal(Unmarshaller u, Object parent) {
			this.parent = (ExtensibleConfiguration) parent;
		}

		@Override
		public String toString() {
			return "StringAttributeHandler{" +
				"name='" + name + '\'' +
				", value='" + value + '\'' +
				'}';
		}
	}


}
