/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3;

import de.tu_darmstadt.rs.spartanmc.devxml.AttributeUtil;
import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.INamedValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.INotifier;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.analysis.ClkReference;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.analysis.ParameterReference;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameter;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameterGroup;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.Parameter;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IPort;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IPortGroup;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.references.IReference;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.references.IReferencer;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IModuleContainer;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IReNameable;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBooleanParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBusHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IFloatParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IIntegerParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.INumberParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterConverter;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.handlers.NameHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.ConfigurationIssue;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.BaseConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.BaseParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.BusHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers.PortHandler;

import org.jetbrains.annotations.Nullable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;

import static de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil.streamAllParameters;
import static de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil.streamBusHandlers;
import static de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil.streamConnectorHandlers;

/**
 * Created by Ramon on 01.12.2015.
 */
public abstract class ModuleConfiguration<P extends IConfigurationElement & INamedValueResolver & IModuleContainer> extends BaseConfigurationElement<IPort, IPortGroup, IPortHandler<IPort>> implements IReNameable, IModuleConfiguration<P>, INotifier {

	private static final Logger logger = LoggerFactory.getLogger(ModuleConfiguration.class);
	protected final List<IModuleChangeListener> moduleListeners = new ArrayList<>();
	protected P parent;
	protected Library library;
	protected Map<String, BaseParameterHandler> parameterHandlers;
	protected Map<String, PortHandler> portHandlers;
	protected Map<String, BusHandler> busHandlers;
	private NameHandler nameHandler;
	private String name;
	private String moduleDescriptionName;
	private List<? extends BaseParameterHandler> tempParameterHandlers;
	private List<PortHandler> tempPortHandlers;
	private List<BusHandler> tempBusHandlers;

	/**
	 * Only for JAXB
	 */
	protected ModuleConfiguration() {
		tempParameterHandlers = new ArrayList<>();
		tempPortHandlers = new ArrayList<>();
		tempBusHandlers = new ArrayList<>();
	}

	public ModuleConfiguration(String name, String moduleDescriptionName, P parent, Library library) {
		this.name = name;
		this.moduleDescriptionName = moduleDescriptionName;
		this.parent = parent;
		this.library = library;

		parameterHandlers = new HashMap<>();
		portHandlers = new HashMap<>();
		busHandlers = new HashMap<>();
		initAttributes();
	}

	public String toModuleConfString() {
		return "{" +
				"name='" + name + '\'' +
				", DescName='" + moduleDescriptionName + '\'' +
				"}";
	}

	private void afterUnmarshal(Unmarshaller u, Object parent) {
		this.parent = (P) parent;
		mapAttributes();
		parameterHandlers = tempParameterHandlers.stream().collect(Collectors.toMap(BaseParameterHandler::getName, Function.identity()));
		tempParameterHandlers = null;
		portHandlers = tempPortHandlers.stream().collect(Collectors.toMap(PortHandler::getPortName, Function.identity()));
		tempPortHandlers = null;
		busHandlers = tempBusHandlers.stream().collect(Collectors.toMap(BusHandler::getPortName, Function.identity()));
		tempBusHandlers = null;
	}

	protected <T extends IAttributeHandler> T getParameterHandlerByName(Class<T> limit, IParameter parameter, String parameterName) {
		BaseParameterHandler parameterHandler = parameterHandlers.get(parameterName);

		BaseParameterHandler res;

		if (parameterHandler == null || parameterHandler instanceof DummyParameterHandler) {
			res = createNewParameterHandler(parameter, parameterHandler);
			parameterHandlers.put(parameterName, res);
		} else {
			res = parameterHandler;
		}

		if (limit.isInstance(res)) {
			return limit.cast(res);
		} else {
			throw new NoSuchElementException("Parameter '" + parameterName + "' is not of type " + limit + ". It is of type " + res.getClass());
		}
	}

	protected BaseParameterHandler createNewParameterHandler(IParameter parameterDescription, IAttributeHandler oldHandler) {
		switch (parameterDescription.getType()) {
			case BinType:
				return new IntegerParameterHandler(parameterDescription, oldHandler, 2);
			case HexType:
				return new IntegerParameterHandler(parameterDescription, oldHandler, 16);
			case IntType:
				return new IntegerParameterHandler(parameterDescription, oldHandler);
			case StringType:
				return new StringParameterHandler(parameterDescription, oldHandler);
			case BoolType:
				return new BooleanParameterHandler(parameterDescription, oldHandler);
			case FloatType:
				return new FloatParameterHandler(parameterDescription, oldHandler);
			default:
				throw new IllegalArgumentException("Unknown parameterType: " + parameterDescription.getType() + " for parameter " + parameterDescription.getName());
		}
	}

	@Override
	public PortHandler getPortHandler(IPort portDescription) {
		String name = portDescription.getName();
		PortHandler handler = portHandlers.get(name);
		if (handler == null) {
			handler = new PortHandler(this, portDescription, getDocument());
			portHandlers.put(name, handler);
		} else {
			if (!handler.isInitialized()) {
				handler.initialize(portDescription, getDocument());
			}
		}
		return handler;
	}

	@Override
	public List<? extends IPortGroup> getPortGroupDescriptions() {
		return getModuleDescription().getPortGroups();
	}

	@Override
	public Optional<? extends IPort> findPortDescription(String portName) {
		return getPortGroupDescriptions().stream().flatMap((group) -> group.getPorts().stream()).filter((port) -> port.getName().equals(portName)).findAny();//TODO "optimize"
	}

	@Override
	public List<Bus> getBusDescriptions() {
		return getModuleDescription().getBuses();
	}

	@Override
	public BusHandler getBusHandler(Bus busDescription) {
		String name = busDescription.getName();
		BusHandler handler = busHandlers.get(name);
		if (handler == null) {
			handler = new BusHandler(this, busDescription, getDocument(), library.getBusDescription(busDescription.getBusDescriptionName()));
			busHandlers.put(name, handler);
		} else {
			if (!handler.isInitialized()) {
				handler.initialize(busDescription, getDocument(), library.getBusDescription(busDescription.getBusDescriptionName()));
			}
		}
		return handler;
	}

	@XmlElement(name = "parameter", type = BaseParameterHandler.class)
	@XmlElementWrapper(name = "parameters")
	private List<? extends BaseParameterHandler> getParameters() {
		if (parameterHandlers != null) {
			tempParameterHandlers = parameterHandlers.values().stream().sorted(IParameterHandler.COMPARATOR).collect(Collectors.toList());
		}
		return tempParameterHandlers;
	}

	private void setParameters(List<? extends BaseParameterHandler> stringParameters) {
		tempParameterHandlers = stringParameters;
		parameterHandlers = null;
	}

	@XmlElement(name = "port")
	@XmlElementWrapper(name = "ports")
	private List<PortHandler> getPorts() {
		if (portHandlers != null) {
			tempPortHandlers = portHandlers.values().stream().sorted(IConnectorHandler.NAME_COMPARATOR).collect(Collectors.toList());
		}
		return tempPortHandlers;
	}

	private void setPorts(List<PortHandler> ports) {
		tempPortHandlers = ports;
		portHandlers = null;
	}

	@XmlElement(name = "bus")
	@XmlElementWrapper(name = "buses")
	private List<BusHandler> getBuses() {
		if (busHandlers != null) {
			tempBusHandlers = busHandlers.values().stream().sorted(IConnectorHandler.NAME_COMPARATOR).collect(Collectors.toList());
		}
		return tempBusHandlers;
	}

	private void setBuses(List<BusHandler> buses) {
		tempBusHandlers = buses;
		busHandlers = null;
	}

	@XmlAttribute
	@Override
	public String getName() {
		return name;
	}

	@Override
	public void setName(String name) {
		this.name = name;
		notifyElementNameChanged();
		if (parent != null) {
			getDocument().notifyNameChanged(this);
		}
	}

	@Override
	public void reportInvalidNameIssue() {
		getDocument().report(ConfigurationIssue.createInvalidModuleName(this, "Validity Check"));
	}

	@Override
	public String getGlobalName() {
		StringBuilder sb = new StringBuilder();
		if (getParent() != getDocument()) {
			sb.append(getParent().getName()).append('_');
		}
		sb.append(getName());

		return sb.toString();
	}

	@Override
	public boolean isValid(boolean report) {
		boolean pluginValid = pluginEngine == null || pluginEngine.isValid(this, report);

		if (!pluginValid) {
			logger.info("PluginEngine declared {} invalid", this);
		}

		boolean nameValid = getNameHandler().isValid(report);

		boolean busValid = streamBusHandlers(this).reduce(true, (v, bus) -> bus.isValid(report) && v, Boolean::logicalAnd);
		boolean portsValid = streamConnectorHandlers(this).reduce(true, (v, port) -> port.isValid(report) && v, Boolean::logicalAnd);
		boolean paramsValid = streamAllParameters(this).reduce(true, (v, bus) -> bus.isValid(report) && v, Boolean::logicalAnd);

		if (!Stream.concat(busHandlers.values().stream(), portHandlers.values().stream()).anyMatch(IConnectorHandler::isConnected)) {
			report(ConfigurationIssue.createModuleAddedButNotUsed(this, "Validity Check"));
		}

		return pluginValid && nameValid && busValid && portsValid && paramsValid;
	}

	public abstract P getParent();

	@Override
	public IAttributeHandler getNameHandler() {
		if (nameHandler == null) {
			nameHandler = new NameHandler(this);
		}
		return nameHandler;
	}

	@Override
	public boolean wasRemoved() {
		return parent == null;
	}

	/**
	 * call if a parameter changes param count / port count
	 */
	protected void notifyDeclarationChanged() {
		for (IElementChangeListener listener : elementListeners) {
			listener.onElementDeclarationChanged(this);
		}
		getDocument().notifyModuleDeclarationChanged(this);
	}

	@Override
	public CalculatedValueResult<? extends Number> resolveParam(ParameterReference param) {
		if (param.isRedirected()) {
			return param.resolveRedirected(new RedirectedClkResolver(this, getDocument()));
		} else {
			return getParameterHandler(INumberParameterHandler.class, Parameter.getBackingParameter(param)).getNumberValue();
		}
	}

	@Override
	public CalculatedValueResult<? extends String> resolveParamString(ParameterReference param) {
		if (param.isRedirected()) {
			throw new UnsupportedOperationException("No String busParams");
		} else {
			return getParameterHandler(IParameterHandler.class, Parameter.getBackingParameter(param)).getValue();
		}
	}

	@Override
	public CalculatedValueResult<? extends Number> resolveClkPort(ClkReference clkRef) {
		if (clkRef.isRedirected()) {
			return new RedirectedClkResolver(this, getDocument()).resolveClk(clkRef);
		} else {
			return clkRef.resolveFrequency(this);
		}
	}

	@Override
	public CalculatedValueResult<Double> resolveParamFloat(ParameterReference param) {
		if (param.isRedirected()) {
			throw new UnsupportedOperationException("No Float busParams");
		} else {
			return getParameterHandler(IFloatParameterHandler.class, Parameter.getBackingParameter(param)).getFloatValue();
		}
	}

	public CalculatedValueResult<? extends Number> resolveClkPort(ClkReference clkRef, RedirectedClkResolver resolver) {
		return resolver.resolveClk(clkRef);
	}

	public void attachLibrary(Library library) {
		if (this.library != null) {
			throw new IllegalStateException("Library already attached!");
		}

		this.library = library;

		getModuleDescription();// Force check if available
	}

	public void requestInitialize(PortHandler portHandler) {
		if (library == null) {
			throw new IllegalStateException("Not attached to library");
		}
		portHandler.initialize(findPortDescription(portHandler.getPortName()).orElseThrow(() -> new NoSuchElementException("No description for" + portHandler)), getDocument());
	}

	public void requestInitialize(BusHandler busHandler) {
		Bus busDescription = getModuleDescription().findBus(busHandler.getPortName()).orElseThrow(() -> new NoSuchElementException("No description for " + busHandler));
		busHandler.initialize(busDescription, getDocument(), library.getBusDescription(busDescription.getBusDescriptionName()));
	}

	public void invalidateBusHandler(BusHandler busHandler) {
		busHandlers.remove(busHandler.getPortName());
	}

	public void invalidatePortHandler(PortHandler portHandler) {
		portHandlers.remove(portHandler.getPortName());
	}

	public void prepareRemove() {
		busHandlers.values().forEach(BaseConnectorHandler::disconnectAll);
		portHandlers.values().forEach(BaseConnectorHandler::disconnectAll);
	}

	public void invalidateModule() {
		this.parent = null;
	}

	@Override
	protected void notifyAttributeChanged(String name) {
		super.notifyAttributeChanged(name);
		getDocument().notifyDocumentChange();
	}

	@Override
	protected void notifyConnectorChanged(IConnectorHandler<?, ?> connector, IElementChangeListener.ConnectorChangeType type, IConnection connection, boolean invalid) {
		super.notifyConnectorChanged(connector, type, connection, invalid);

		if (connector.getType() == IConnectorDescription.Type.Clk) {
			IReference clk = (IReference) connector.getConnectorDescription();
			notifyReferencers(clk.getReferencers(), clk);
		} else if (connector.getType() == IConnectorDescription.Type.Bus) {
			Bus bus = ((BusHandler) connector).getConnectorDescription();
			if (bus != null) {
				bus.getClkPortMappings().forEach(clkMapping -> notifyReferencers(clkMapping.getReferencers(), clkMapping));
			}
		}
		if (type == IElementChangeListener.ConnectorChangeType.Properties && connector.getType() == IConnectorDescription.Type.Bus) {
			getDocument().getAvailablePortsHandler().busDefiningParametersChanged(this, ((IBusHandler) connector));
		}
	}

	protected void notifyParameterChanged(BaseParameterHandler handler) {
		if (pluginEngine != null) {
			pluginEngine.onParameterChanged(this, handler);
		}

		IParameter description = handler.getParameterDescription();
		if (description.isDynamicallyReferenced()) {
			notifyDeclarationChanged();
		} else {
			notifyReferencers(description.getReferencers(), handler.getParameterDescription());
			for (IModuleChangeListener listener : moduleListeners) {
				listener.onParameterChanged(this, handler);
			}
			getDocument().notifyDocumentChange();
		}
	}

	private void notifyReferencers(List<IReferencer> referencers, IReference source) {
		if (referencers != null) {
			referencers.forEach(ref -> {
				if (ref != null) {
					ref.notifySourceChanged(this, source);
				}
			});
		}
	}

	@Override
	public void notifyBus(Bus target, IReference source) {
		BusHandler handler = busHandlers.get(target.getName());
		if (handler != null) {
			handler.onReferencedParameterChanged(source);
			//			notifyConnectorChanged(handler, IElementChangeListener.ConnectorChangeType.Properties, null); // too lazy...
		}
	}

	@Override
	public void notifyPort(IPort target, IReference source) {
		PortHandler handler = portHandlers.get(target.getName());
		if (handler != null) {
			handler.onReferencedParameterChanged(source);
		}
	}

	@Override
	public void notifyParameter(IParameter target, IReference source) {
		BaseParameterHandler handler = parameterHandlers.get(target.getName());
		if (handler != null) {
			handler.onReferencedParameterChanged(source);
		}
	}

	@Override
	public Library getLibrary() {
		return library;
	}

	@Override
	public void addModuleListener(IModuleChangeListener listener) {
		moduleListeners.add(listener);
	}

	@Override
	public void removeModuleListener(IModuleChangeListener listener) {
		moduleListeners.remove(listener);
	}

	@Override
	public ConfigurationDocument getDocument() {
		if (parent instanceof ConfigurationDocument) {
			return (ConfigurationDocument) parent;
		} else {
			return ((SubsystemConfiguration) parent).getParent();
		}
	}

	@Override
	public List<? extends IParameterGroup> getParameterGroupDescriptions() {
		return getModuleDescription().getParameterGroups();
	}

	@Override
	public Optional<? extends IParameter> findParameterDescription(String parameterName) {
		return getParameterGroupDescriptions().stream().flatMap((group) -> group.getParameters().stream()).filter((param) -> param.getName().equals(parameterName)).findAny();//TODO "optimize"
	}

	@Override
	public <T extends IAttributeHandler> T getParameterHandler(Class<T> limit, IParameter parameter) {
		return getParameterHandlerByName(limit, parameter, parameter.getName());
	}

	@Override
	@XmlAttribute
	public String getModuleDescriptionName() {
		return moduleDescriptionName;
	}

	public void setModuleDescriptionName(String moduleDescription) {
		this.moduleDescriptionName = moduleDescription;
	}

	Bus getOppositeMasterBusDescription(Bus exposedOn) {
		return getBusHandler(exposedOn).streamOppositeConnectors().findAny().get().getConnectorDescription();
	}

	protected void report(ConfigurationIssue issue) {
		getDocument().report(issue);
	}

	private abstract class InternalBaseParameterHandler extends BaseParameterHandler {

		/**
		 * Create new parameter, handle value inheritance / init manually or by calling {@link #initInternal}
		 *
		 * @param parameterDescription the description to which this parameter must conform
		 */
		InternalBaseParameterHandler(IParameter parameterDescription) {
			super(parameterDescription);
		}

		/**
		 * Import an existing parameter or initialize a previously unknown parameter to default
		 *
		 * @param parameterDescription the description to which this parameter must conform
		 * @param oldHandler if null, use default, otherwise try to reuse previous value
		 */
		InternalBaseParameterHandler(IParameter parameterDescription, @Nullable IAttributeHandler oldHandler) {
			this(parameterDescription);

			initInternalIfStatic(oldHandler);
		}

		/**
		 * initialize parameter to default or previous value, only if parameter is not dynamic (will be recalculated on every access)
		 *
		 * @param oldHandler if set, previous value is reused
		 */
		protected final void initInternalIfStatic(@Nullable IAttributeHandler oldHandler) {
			if (!parameterDescription.hasCalculatedValue()) {
				initInternal(oldHandler, null);
			}
		}

		/**
		 * initialize parameter to default or previous value, allow migrating invalid old values
		 *
		 * @param oldHandler if set, previous value is reused
		 * @param migrate if set, will be used to infer new value from invalid old value, if not, default is used
		 */
		protected final void initInternal(@Nullable IAttributeHandler oldHandler, @Nullable Function<String, String> migrate) {
			String defaultValue = parameterDescription.getDefaultValue();

			String newValue;

			if (oldHandler != null) {
				CalculatedValueResult<String> oldValue = oldHandler.getValue();

				if (oldValue.isPresent() && isValidValue(oldValue.get())) {
					newValue = oldValue.get();
				} else {
					boolean couldMigrate = false;
					if (oldValue.isPresent()) {
						String migratedVal = null;
						if (migrate != null) {
							migratedVal = migrate.apply(oldValue.get());
							couldMigrate = isValidValue(migratedVal);
							if (!couldMigrate) {
								logger.warn("Parameter {} failed to migrate previous value '{}' to a valid one", getCanonicalName(), oldValue);
							}
						}
						if (couldMigrate) {
							newValue = migratedVal;
							logger.warn("Parameter {} was migrated to '{}', because it was invalid ('{}')", getCanonicalName(), newValue, oldValue);
						} else {
							newValue = defaultValue;
							logger.warn("Parameter {} was reset to default '{}', because it was invalid ('{}')", getCanonicalName(), newValue, oldValue);
						}
					} else {
						newValue = defaultValue;
					}
					report(ConfigurationIssue.createParameterConflictResolved(this, oldValue.get(), "Parameter deserialization", couldMigrate));
				}
			} else {
				newValue = defaultValue;
				logger.trace("Parameter {} was initialized to default '{}', no previous value", getCanonicalName(), newValue);
			}

			if (newValue != null) {
				parseInternal(newValue);
			} else {
				resetToIntrinsicDefault();
			}
		}

		protected abstract void resetToIntrinsicDefault();

		/**
		 * Parse and set the internal value without notifying anyone.
		 * Used during initialization / deserialization, to parse old / default values
		 *
		 * @param newValue a known valid value for this parameter type
		 */
		protected abstract void parseInternal(String newValue);

		@Override
		public String getCanonicalName() {
			return String.format("%s#%s", ModuleConfiguration.this.getGlobalName(), InternalBaseParameterHandler.this.getName());
		}

		@Override
		public CalculatedValueResult<Boolean> isEditable() {
			return CalculatedValueResult.dualMap(
					super.isEditable(),
					isCurrentlyRelevant(),
					(a, b) -> a && b
			);
		}

		@Override
		public CalculatedValueResult<Boolean> isCurrentlyRelevant() {
			return parameterDescription.resolveRelevantIf(ModuleConfiguration.this);
		}

		@Override
		public void onReferencedParameterChanged(IReference source) {
			ModuleConfiguration.this.notifyParameterChanged(this);
		}

		@Override
		public ModuleConfiguration<?> getSourceElement() {
			return ModuleConfiguration.this;
		}


	}


	/**
	 * @author Ramon Wirsch
	 */
	private class BooleanParameterHandler extends InternalBaseParameterHandler implements IBooleanParameterHandler, IIntegerParameterHandler, IFloatParameterHandler {

		private final String[] choices;
		private boolean value;

		BooleanParameterHandler(IParameter parameterDescription, IAttributeHandler oldHandler) {
			super(parameterDescription);

			choices = AttributeUtil.guessBooleanChoices(parameterDescription.getDefaultValue());

			initInternal(oldHandler, oldVal -> {
				String[] oldChoices = AttributeUtil.guessBooleanChoices(oldVal);
				boolean valid = isValidValue(oldChoices, oldVal);
				if (!valid) {
					return "";
				}
				boolean oldBool = AttributeUtil.parseBooleanChoice(oldChoices, oldVal);
				return choices[oldBool? 1 : 0];
			});
		}

		@Override
		protected void resetToIntrinsicDefault() {
			value = false;
		}

		@Override
		protected void parseInternal(String newValue) {
			this.value = AttributeUtil.parseBooleanChoice(choices, newValue);
		}

		@Override
		public boolean isSet() {
			return value;
		}

		@Override
		public void setValue(boolean value) {
			verifyEditable();
			if (this.value != value) {
				this.value = value;
				ModuleConfiguration.this.notifyParameterChanged(this);
			}
		}

		@Override
		public CalculatedValueResult<Long> getIntegerValue() {
			return CalculatedValueResult.of(value? 1L : 0L);
		}

		@Override
		public boolean isValid(boolean report) {
			return true;//impossible to be invalid
		}

		@Override
		public CalculatedValueResult<String> getValue() {
			return CalculatedValueResult.of(getStoredValue());
		}

		@Override
		public String getStoredValue() {
			return choices[value? 1 : 0];
		}

		@Override
		public void setStoredValue(String value) {
			setValue(AttributeUtil.parseBooleanChoice(choices, value));
		}

		@Override
		public boolean isValidValue(String value) {
			return isValidValue(choices, value);
		}

		private boolean isValidValue(String[] choices, String value) {
			return choices[0].equalsIgnoreCase(value) || choices[1].equalsIgnoreCase(value);
		}

		@Override
		public int getRadix() {
			return 2;
		}

		@Override
		public void setValue(double value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public CalculatedValueResult<? extends Number> getNumberValue() {
			return getIntegerValue();
		}

		@Override
		public CalculatedValueResult<Double> getFloatValue() {
			return getIntegerValue().map(l -> (double) l);
		}

		@Override
		public void setValue(long value) {
			throw new UnsupportedOperationException();
		}


		@Override
		public String getValueConverted(de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterConverter converter) {
			return converter.convert((IBooleanParameterHandler) this);
		}

		@Override
		public boolean isValidValue(long value) {
			throw new UnsupportedOperationException();
		}


		@Override
		public boolean isValidValue(double value) {
			throw new UnsupportedOperationException();
		}


	}


	/**
	 * Handler for all Integer values, radix configurable, internally stored as long
	 *
	 * @author Ramon Wirsch
	 */
	private class IntegerParameterHandler extends InternalBaseParameterHandler implements IIntegerParameterHandler, IFloatParameterHandler {

		private final int radix;
		private long value;

		/**
		 * 10 base Integer
		 */
		IntegerParameterHandler(IParameter parameterDescription, IAttributeHandler oldHandler) {
			this(parameterDescription, oldHandler, 10);
		}

		@Override
		protected void resetToIntrinsicDefault() {
			value = 0;
		}

		IntegerParameterHandler(IParameter parameterDescription, IAttributeHandler oldHandler, int radix) {
			super(parameterDescription);
			this.radix = radix;

			initInternalIfStatic(oldHandler);
		}

		private CalculatedValueResult<? extends Number> getDynamicValue() {
			return parameterDescription.resolveValue(ModuleConfiguration.this);
		}

		@Override
		protected void parseInternal(String newValue) {
			try {
				value = parseValue(newValue);
			} catch (NumberFormatException | NullPointerException e) {
				logger.error("Parameter {}: Could not parse '{}' into INT (radix {})", parameterDescription, newValue, radix, e);
				throw e;
			}
		}

		private long parseValue(String value) {
			return AttributeUtil.parseLong(value, radix);
		}

		private boolean isValidValue(String value, boolean report) {
			try {
				long i = parseValue(value);
				return isValidValue(i, report);
			} catch (NumberFormatException e) {
				if (report) {
					report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Parsing"));
				}
				return false;
			}
		}

		private boolean isValidValue(long value, boolean report) {
			boolean inRange = !hasRange() || getRange().isInRange(value);
			boolean choiceConflict = IParameter.doesConflictWithChoices(value, parameterDescription, this::parseValue);
			if (!inRange && report) {
				report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Out of Range"));
			}
			if (choiceConflict && report) {
				report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Choice Conflict"));
			}
			return inRange && !choiceConflict;
		}

		private String getValue(long value) {
			String rval = Long.toString(value, radix);
			return AttributeUtil.getRadixPrefix(radix).concat(rval);
		}

		@Override
		public int getRadix() {
			return radix;
		}


		@Override
		public List<String> getChoices() {
			return parameterDescription.getChoices().stream().map(Parameter.Choice::getValue).map(s -> getValue(parseValue(s))).collect(Collectors.toList());
		}

		@Override
		public CalculatedValueResult<? extends Number> getNumberValue() {
			return getIntegerValue();
		}

		@Override
		public CalculatedValueResult<Double> getFloatValue() {
			return getIntegerValue().map(x -> (double) x);
		}

		@Override
		public CalculatedValueResult<String> getValue() {
			return getIntegerValue().map(this::getValue);
		}

		@Override
		public String getStoredValue() {
			return getValue(value);
		}

		@Override
		public String getValueConverted(IParameterConverter converter) {
			return converter.convert((IIntegerParameterHandler) this);
		}

		@Override
		public CalculatedValueResult<Long> getIntegerValue() {
			return isShowOnly().flatMap(x -> x? getDynamicValue().map(CalculatedValueResult.longOrDoubleMapper(Function.identity(), Math::round)) : CalculatedValueResult.of(value));
		}

		@Override
		public boolean isValid(boolean report) {
			if (!getIntegerValue().isPresent()) {
				if (report) {
					report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Parameter is absent: " + getIntegerValue().toString()));
				}
				return false;
			}
			return isValidValue(getIntegerValue().get(), report);
		}

		@Override
		public boolean isValidValue(String value) {
			return isValidValue(value, false);
		}

		@Override
		public boolean isValidValue(long value) {
			return isValidValue(value, false);
		}

		@Override
		public void setStoredValue(String value) {
			setValue(parseValue(value));
		} //TODO verify radix?

		@Override
		public void setValue(long value) {
			verifyEditable();
			AttributeUtil.verifyOrThrowIllegal(isValidValue(value));
			if (this.value != value) {
				this.value = value;
				ModuleConfiguration.this.notifyParameterChanged(this);
			}
		}

		@Override
		public boolean isValidValue(double value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void setValue(double value) {
			throw new UnsupportedOperationException();
		}
	}


	/**
	 * @author Ramon Wirsch
	 */
	private class StringParameterHandler extends InternalBaseParameterHandler {

		private String value;

		StringParameterHandler(IParameter parameterDescription, IAttributeHandler oldHandler) {
			super(parameterDescription, oldHandler);
		}

		@Override
		protected void resetToIntrinsicDefault() {
			value = "";
		}

		private boolean isValidValue(String value, boolean report) {
			boolean choiceConflict = IParameter.doesConflictWithChoices(value, parameterDescription);
			if (choiceConflict && report) {
				report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Choice Conflict"));
			}
			return !choiceConflict;
		}

		@Override
		protected void parseInternal(String newValue) {
			value = newValue;
		}


		@Override
		public CalculatedValueResult<String> getValue() {
			return CalculatedValueResult.of(value);
		}

		@Override
		public String getStoredValue() {
			return value;
		}

		@Override
		public String getValueConverted(IParameterConverter converter) {
			return converter.convert(this);
		}

		@Override
		public boolean isValid(boolean report) {
			return isValidValue(value, report);
		}

		@Override
		public boolean isValidValue(String value) {
			return isValidValue(value, false);
		}

		@Override
		public void setStoredValue(String value) {
			verifyEditable();
			AttributeUtil.verifyOrThrowIllegal(isValidValue(value));
			if (value != null && !value.equals(this.value)) {
				this.value = value;
				ModuleConfiguration.this.notifyParameterChanged(this);
			}
		}
	}


	private class FloatParameterHandler extends InternalBaseParameterHandler implements IFloatParameterHandler {

		private double value;

		FloatParameterHandler(IParameter parameterDescription, IAttributeHandler oldHandler) {
			super(parameterDescription, oldHandler);
		}

		@Override
		protected void resetToIntrinsicDefault() {
			value = 0;
		}

		private CalculatedValueResult<Double> getDynamicValue() {
			return parameterDescription.getValueRef()
					.resolve(ModuleConfiguration.this)
					.map(CalculatedValueResult.longOrDoubleMapper(l -> (double) l, Function.identity()));
		}

		@Override
		protected void parseInternal(String newValue) {
			try {
				value = parseValue(newValue);
			} catch (NumberFormatException e) {
				logger.error("Parameter {}: Could not parse '{}' into DOUBLE ", parameterDescription, newValue, e);
				throw e;
			}
		}

		private double parseValue(String value) {
			return Double.parseDouble(value);
		}


		private boolean isValidValue(CalculatedValueResult<?> value, boolean report) {
			if (value.isPresent()) {
				Object v = value.get();
				if (v instanceof String) {
					return isValidValue((String) v, report);
				} else if (v instanceof Double) {
					return isValidValue((Double) v, report);
				} else {
					throw new RuntimeException("Unknown value type: " + v.getClass());
				}
			} else {
				if (report) {
					report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Value could not be resolved"));
				}
				return false;
			}
		}

		private boolean isValidValue(String value, boolean report) {
			try {
				double i = parseValue(value);
				return isValidValue(i);
			} catch (NumberFormatException e) {
				if (report) {
					report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Parsing"));
				}
				return false;
			}
		}

		private boolean isValidValue(double value, boolean report) {
			boolean inRange = parameterDescription.getRange() == null || parameterDescription.getRange().isInRange(value);
			boolean choiceConflict = IParameter.doesConflictWithChoices(value, parameterDescription, this::parseValue);
			if (!inRange && report) {
				report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Out of Range"));
			}
			if (choiceConflict && report) {
				report(ConfigurationIssue.createParameterInvalidValue(this, "Parameter Validity Check: Choice Conflict"));
			}
			return inRange && !choiceConflict;
		}

		private String getValue(double value) {
			return Double.toString(value);
		}


		@Override
		public List<String> getChoices() {
			return parameterDescription.getChoices().stream().map(Parameter.Choice::getValue).map(s -> getValue(parseValue(s))).collect(Collectors.toList());
		}


		@Override
		public CalculatedValueResult<String> getValue() {
			return getFloatValue().map(this::getValue);
		}

		@Override
		public String getStoredValue() {
			return getValue(value);
		}

		@Override
		public String getValueConverted(de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterConverter converter) {
			return converter.convert(this);
		}

		@Override
		public CalculatedValueResult<? extends Number> getNumberValue() {
			return getFloatValue();
		}

		@Override
		public CalculatedValueResult<Double> getFloatValue() {
			return isShowOnly().flatMap(x -> x? getDynamicValue() : CalculatedValueResult.of(value));
		}

		@Override
		public boolean isValid(boolean report) {
			return isValidValue(getFloatValue(), report);
		}

		@Override
		public boolean isValidValue(String value) {
			return isValidValue(value, false);
		}

		@Override
		public boolean isValidValue(double value) {
			return isValidValue(value, false);
		}

		@Override
		public void setStoredValue(String value) {
			setValue(parseValue(value));
		}

		@Override
		public void setValue(double value) {
			verifyEditable();
			AttributeUtil.verifyOrThrowIllegal(isValidValue(value));
			if (this.value != value) {
				this.value = value;
				ModuleConfiguration.this.notifyParameterChanged(this);
			}
		}
	}

	@Override
	public String getToplevel() {
		return getModuleDescription().getHdl().getToplevel();
	}
}
