/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameter;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.Parameter;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.references.IReference;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.DummyParameterHandler;

import java.util.List;
import java.util.stream.Collectors;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlType;
import javax.xml.bind.annotation.XmlValue;

/**
 * @author Ramon Wirsch
 */
//Always create DummyParameterHandlers on deserialization
@XmlType(
		factoryClass = BaseParameterHandler.class,
		factoryMethod = "createDummyParameterHandler"
)
public abstract class BaseParameterHandler implements IParameterHandler, IAttributeHandler {

	protected final IParameter parameterDescription;

	protected BaseParameterHandler(IParameter parameterDescription) {
		this.parameterDescription = parameterDescription;
	}

	@Override
	public IParameter getParameterDescription() {
		return parameterDescription;
	}

	@Override
	@XmlAttribute
	public String getName() {
		return parameterDescription.getName();
	}

	public void setName(String name) {
		throw new UnsupportedOperationException("jaxb requires this, but only DummyParamHandler implements");
	}

	@Override
	public boolean isVirtual() {
		return parameterDescription.isVirtual();
	}

	@Override
	public abstract CalculatedValueResult<String> getValue();

	@XmlValue
	public abstract String getStoredValue();

	@Override
	public abstract void setStoredValue(String value);

	@Override
	public CalculatedValueResult<Boolean> isEditable() {
		return CalculatedValueResult.of(parameterDescription.isEditable());
	}

	@Override
	public boolean hasChoices() {
		return parameterDescription.hasChoices();
	}

	@Override
	public List<String> getChoices() {
		return parameterDescription.getChoices().stream().map(Parameter.Choice::getValue).collect(Collectors.toList());
	}

	@Override
	public boolean isChoicesOnly() {
		return parameterDescription.isChoicesOnly();
	}

	@Override
	public boolean hasRange() {
		return parameterDescription.getRange() != null;
	}

	@Override
	public Parameter.Range getRange() {
		return parameterDescription.getRange();
	}

	protected void verifyEditable() throws UnsupportedOperationException {
		if (!parameterDescription.isEditable()) {
			throw new UnsupportedOperationException();
		}
	}

	public CalculatedValueResult<Boolean> isShowOnly() {
		return isEditable().map(e -> !e && parameterDescription.hasCalculatedValue());
	}

	@Override
	public String toString() {
		return "ParameterHandler{" +
			"parameterDescription=" + parameterDescription +
			'}';
	}

	public abstract void onReferencedParameterChanged(IReference source);


	/**
	 * Create a DummyParameterHandler for JaxB deserialization
	 * @return a DummyParameterHandler
	 */
	public static BaseParameterHandler createDummyParameterHandler() {
		return new DummyParameterHandler();
	}

}
