/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.IValueResolver;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;

import java.util.Optional;

/**
 * Created by Ramon on 09.02.2016.
 */
public abstract class BasePortHandler<E extends BaseConnectorHandler.IConnectorHandlerContainer & de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement & IValueResolver, PD extends IConnectorDescription> extends BaseConnectorHandler<BasePortHandler<?, ?>, IPortHandler<?>, E, PD> implements IPortHandler<PD> {

	public BasePortHandler(E sourceElement, PD portDescription) {
		super(sourceElement, portDescription);
	}

	protected BasePortHandler() {
	}

	@Override
	public int getMaxAllowedConnections() {
		return portDescription.getDirection() == ConnectorDirection.Output? 0 : 1;
	}

	@Override
	public boolean isConnectibleTo(IConnectorHandler toPort) {
		IConnectorDescription toPortDescription = toPort.getConnectorDescription();
		return IConnectorDescription.isTypeCompatible(portDescription, toPortDescription)
			&& toPort.getConnectorWidth() == getConnectorWidth()
			&& IConnectorHandler.isDirectionCompatible(this, toPort);
	}

	@Override
	public abstract Optional<? extends PartialBasePortHandler<E, PD>> getPartialConnectorHandler();

	@Override
	public String toString() {
		return "BasePort{" +
			"desc=" + portDescription +
			"}" + super.toString();
	}

}
