/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.handlers;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.dependencies.CalculatedValueResult;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IPort;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.references.IReference;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPartialConnectorHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.issues.ConfigurationIssue;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.ConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.ModuleConfiguration;

import java.util.Optional;

/**
 * Created by ramon on 11.02.2016.
 */
public class PortHandler extends PartialBasePortHandler<ModuleConfiguration, IPort> implements IPartialConnectorHandler<IPortHandler<?>, IPort> {

	private ConfigurationDocument document;

	public PortHandler(ModuleConfiguration sourceElement, IPort portDescription, ConfigurationDocument document) {
		super(sourceElement, portDescription);
		this.document = document;
	}

	private PortHandler() {
	}

	@Override
	protected void requestInitialize() {
		sourceElement.requestInitialize(this);
	}

	@Override
	protected void invalidateOnSource() {
		sourceElement.invalidatePortHandler(this);
	}

	@Override
	public ConfigurationDocument getDocument() {
		return document;
	}

	@Override
	protected String extractPortName(String tempName) {
		return tempName.substring(tempName.lastIndexOf('.') + 1);
	}

	@Override
	public String getPrefixedConnectorName() {
		return '.' + portDescription.getName();
	}

	public void initialize(IPort portDescription, ConfigurationDocument document) {
		super.initialize(portDescription);
		this.document = document;

		isPartiallyConnected(); // verifies cannot be both!
	}

	@Override
	public ConnectorDirection getDirection() {
		return portDescription.getDirection();
	}

	@Override
	public IConnectorDescription.Type getType() {
		return portDescription != null? portDescription.getType() : null;
	}

	@Override
	public int getConnectorWidth() {
		final IPort connectorDescription = getConnectorDescription();
		if (connectorDescription.hasCalculatedWidth()) {
			final CalculatedValueResult<Long> longCalculatedValueResult = connectorDescription.resolveWidth(sourceElement);

			return Math.toIntExact(longCalculatedValueResult.orElseThrow());
		} else {
			return connectorDescription.getConstantWidth();
		}
	}

	@Override
	public void onReferencedParameterChanged(IReference source) {
		if (getType() == IConnectorDescription.Type.Clk) {
			if (getDirection() == ConnectorDirection.Output) {
				streamOppositeConnectors().forEach(ph -> ((BaseConnectorHandler) ph).onReferencedClkChanged(this));
			}
		}

		super.onReferencedParameterChanged(source);
	}

	@Override
	public boolean isValid(boolean report) {
		if (portDescription.isRequired() && !isConnected()) {
			if (report) {
				getDocument().report(ConfigurationIssue.createUnconnected(this, "Port Validation"));
			}
			return false;
		}

		return super.isValid(report);
	}

	@Override
	public boolean isDirectionLocked() {
		return portDescription.isDirectionLocked();
	}

	@Override
	public Optional<PortHandler> getPartialConnectorHandler() {
		return Optional.of(this);
	}

}
