/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.tests

import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayout
import de.tu_darmstadt.rs.spartanmc.jconfig.testing.BaseLibJConfigSpec
import groovy.util.logging.Slf4j

/**
 * Created by Ramon on 31.05.2016.
 */
@Slf4j
class AddressLayoutSpec extends BaseLibJConfigSpec {

	def "can analyze address space"() {
		setup:
		def core = document.addSubsystem(CORE);

		def uart0 = core.addPeripheral(UART);
		def uart1 = core.addPeripheral(UART);
		def uart2 = core.addPeripheral(UART);
		def can = core.addPeripheral(CAN);
		def spi = core.addPeripheral(SPI);
		def usb = core.addPeripheral(USB);
		def mem = core.addCommonModule(LOCAL_MEM);

		when:
		AddressLayout layout = AddressLayout.getCurrent(core);

		log.info("\nspaceHandlers: {}", layout.getSpaceHandlers());
		log.info("\nextractStaticMappings() = {}", layout.extractStaticMappings());

		def dmaRange = (0x2000L..0x2800L)
		def usbStart = layout.findDmaAddressBlock(usb).get().actualStart
		def canStart = layout.findDmaAddressBlock(can).get().actualStart
		def regRange = (0x1fc00L..0x1ffffL)
		def uart0Start = layout.findRegAddressBlock(uart0).get().actualStart
		def uart1Start = layout.findRegAddressBlock(uart1).get().actualStart
		def uart2Start = layout.findRegAddressBlock(uart2).get().actualStart

		then:
		layout.isValid()
		dmaRange.contains(usbStart)
		usbStart % 2**10 == 0L
		dmaRange.contains(canStart)
		canStart % 2**10 == 0L
		Math.abs(usbStart - canStart) >= 0x400
		regRange.contains(uart0Start)
		uart0Start % 2**3 == 0L
		regRange.contains(uart1Start)
		uart1Start % 2**3 == 0L
		regRange.contains(uart2Start)
		uart2Start % 2**3 == 0L
		[uart0Start, uart1Start, uart2Start].collate(2, 1, false) every { pair -> // check pairwise for minimal distance
			Math.abs(pair[0] - pair[1]) >= 0x8
		}
	}
}
