/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.tests

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IMemoryModuleConfiguration
import de.tu_darmstadt.rs.spartanmc.jconfig.testing.BaseLibJConfigSpec
import org.apache.commons.io.FileUtils
import spock.lang.Narrative

import java.nio.file.Paths
/**
 * Created by ramon on 28.10.16.
 */
@Narrative("Specifies all manor of build behavior")
class BuildSpec extends BaseLibJConfigSpec {

	private configureMiniSystemWOMem() {
		def core = document.addSubsystem(CORE)
		def sysclk = document.addCommonModule(SYSCLK)
		def uart = core.addPeripheral(UART)
		document.connect(core.ports.locked, sysclk.ports.locked)
		document.connect(core.ports.clk, sysclk.ports.clkfx)
		document.connect(sysclk.ports.clk_gen, document.pins.CLOCK_USER)
		document.connect(sysclk.ports.reset, document.pins.GPIO_CPU_RESET)
		document.connect(uart.ports.rx, document.pins.USB_1_TX)
		document.connect(uart.ports.tx, document.pins.USB_1_RX)
	}

	def "build config without memory"() {
		setup:
		configureMiniSystemWOMem()

		when:
		project.documentHandler.build(false)
		then:
		notThrown(Exception.class)
		document.configurationIssues.empty
	}

	def "build config with memory"() {
		setup:
		def fwName = "testFW"
		configureMiniSystemWOMem()
		FileUtils.copyDirectory(Paths.get(fwName).toFile(),project.projectDir.resolve(fwName).toFile())
		def mem = (IMemoryModuleConfiguration) document.subsystems[0].add(LOCAL_MEM)
		mem.firmwareLocationHandler.firmwareName = fwName

		expect:
		project.getKnownFirmwares(null).size()==1
		project.getKnownFirmwares(null).get(0)==fwName

		when:
		project.documentHandler.build(false)

		then:
		notThrown(Exception.class)
		document.configurationIssues.empty
	}

	def "refuses to build config with missing required clock"() {
		setup:
		configureMiniSystemWOMem()
		document.pins.CLOCK_USER.disconnectAll()

		when:
		project.documentHandler.build(false)
		then:
		thrown(Exception.class)
		!document.configurationIssues.empty
	}

	def "refuses to build config with missing required connection"() {
		setup:
		configureMiniSystemWOMem()
		document.subsystems[0].ports.locked.disconnectAll()

		when:
		project.documentHandler.build(false)
		then:
		thrown(Exception.class)
		!document.configurationIssues.empty
	}
}
