/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.tests

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument
import de.tu_darmstadt.rs.spartanmc.jconfig.testing.BaseLibJConfigSpec

import static de.tu_darmstadt.rs.spartanmc.jconfig.scripting.ModuleExtension.AUTO

/**
 * Created by Ramon on 31.05.2016.
 */
class ContainerSpec extends BaseLibJConfigSpec {

	def "removal of module removes all connections"() {
		setup:
		def core = document.addSubsystem(CORE)
		def port_in = core.addPeripheral(PORT_IN)
		def uart = core.addPeripheral(UART)
		port_in.parameters["PORT_WIDTH"] = 4

		when:
		def con0 = document.connect(document.pins["GPIO_SWITCH_0"], AUTO, port_in.ports["pin"], 0, AUTO)
		def con1 = document.connect(document.pins["GPIO_SWITCH_1"], AUTO, port_in.ports["pin"], 1, AUTO)
		def con2 = document.connect(document.pins["GPIO_SWITCH_2"], AUTO, port_in.ports["pin"], 2, AUTO)
		def con3 = document.connect(document.pins["GPIO_SWITCH_3"], AUTO, port_in.ports["pin"], 3, AUTO)
		def conIntr = document.connect(core.ports["intr"], uart.ports["intr"])

		then:
		document.connections.size() == 5
		document.busConnections.size() == 2

		when:
		core.removeModule(port_in)
		core.removeModule(uart)

		then:
		(0..3).every { !document.pins["GPIO_SWITCH_$it"].connected }
		document.busConnections.empty
		document.connections.empty
		document.isValid(false)
	}

	def "removal of tree only notifies once"() {
		setup:
		def core = document.addSubsystem(CORE)
		def port_in = core.addPeripheral(PORT_IN)
		def uart = core.addPeripheral(UART)

		def l = Mock(IConfigurationDocument.IConfigurationHierarchyListener)
		document.addHierarchyListener(l)

		when:
		document.removeModule(core)

		then:
		with(l) {
			1 * onModuleRemoved({ it == document }, { it == core })
			0 * _
		}
	}
}
