/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.tests

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.dynamic.DynamicParameter
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.DynamicModuleConfiguration
import de.tu_darmstadt.rs.spartanmc.jconfig.testing.BaseLibJConfigSpec
import de.tu_darmstadt.rs.util.StreamUtil
/**
 * Created by Gregor Koch on 21.08.16.
 */
class DynamicStructureSpec extends BaseLibJConfigSpec {

	DynamicModuleConfiguration test_dyn

	@Override
	def setup() {
		test_dyn = (DynamicModuleConfiguration) document.add(TEST_DYN)
	}

	def "Implementation count for ports and parameters"() {
		when:
		test_dyn.parameters["COUNT"] = cnt

		then:
		test_dyn.ports.keySet().containsAll( (0..<cnt).collect {"simplePort$it".toString()} )
		test_dyn.ports.count { it.key =~ /^simplePort[0-9]*$/ } == cnt
		test_dyn.parameters.keySet().containsAll( (0..<cnt).collect {"P_SIMPLE$it".toString()} )
		test_dyn.parameters.count { it.key =~ /^P_SIMPLE[0-9]*$/ } == cnt

		where:
		cnt << [1, 3, 0]
	}

	def "Reference between indexed port and parameter"() {
		when:
		test_dyn.parameters["WIDTH0"] = 1
		test_dyn.parameters["WIDTH1"] = 4
		test_dyn.parameters["WIDTH2"] = 3

		then:
		test_dyn.ports["dynPort0"].connectorWidth == 1
		test_dyn.ports["dynPort1"].connectorWidth == 4
		test_dyn.ports["dynPort2"].connectorWidth == 3
	}

	def "Dynamic Group without implementation count has no indexes"() {
		expect:
		test_dyn.parameters.keySet().findAll { it =~ /^WIDTH_EDITABLE[0-9]*$/ } == Collections.singleton('WIDTH_EDITABLE')
		test_dyn.ports.keySet().findAll { it =~ /^singlePort[0-9]*$/ } == Collections.singleton('singlePort')
	}

	def "EnabledIf and EditableIf"() {
		setup:
		test_dyn.parameters["COUNT"] = 2

		when:
		test_dyn.parameters["WIDTH_EDITABLE"] = "false"

		then:
		!test_dyn.parameters["WIDTH0"].isEditable().get()
		!test_dyn.parameters["WIDTH1"].isEditable().get()

		when:
		test_dyn.parameters["ENABLED"] = "false"

		then:
		!test_dyn.parameters.count { it.key =~ /^WIDTH[0-9]*$/ }
		!test_dyn.ports.count { it.key =~ /^dynPort[0-9]*$/ }

		when:
		test_dyn.parameters["ENABLED"] = "true"

		then:
		test_dyn.parameters.count { it.key =~ /^WIDTH[0-9]*$/ } == 2
		test_dyn.ports.count { it.key =~ /^dynPort[0-9]*$/ } == 2
	}

	def "Reference dynamic clkPort"() {
		setup:
		long freq = document.pins["CLOCK_USER"].frequency.get()
		long period = 1e12 / freq

		when:
		document.connect(document.pins["CLOCK_USER"], test_dyn.ports["clk"])

		then:
		test_dyn.parameters["PERIOD"] as Long == period
	}

	def "Reference non existing parameters"() {
		setup:
		test_dyn.parameters["COUNT"] = 1

		//Get the raw dynamic parameter as it would be the case in a DynamicParamRef
		DynamicParameter param = test_dyn.getDynamicModuleDescription()
				.@parameterGroups
				.stream()
				.flatMap { pg->pg.parameters.stream() }
				.filter { p -> p.getName().equals("WIDTH") }
				.collect(StreamUtil.exactlyOne())

		expect:
		test_dyn.resolveDynamicParam(param,0).isPresent()
		!test_dyn.resolveDynamicParam(param,1).isPresent()
		!test_dyn.resolveDynamicParam(param,2).isPresent()

	}

}
