/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.tests

import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IIntegerParameterHandler
import de.tu_darmstadt.rs.spartanmc.jconfig.testing.BaseLibJConfigSpec
import spock.lang.Narrative
import spock.lang.Unroll

import static spock.util.matcher.HamcrestMatchers.closeTo
import static spock.util.matcher.HamcrestSupport.expect

@Narrative("Specifies resolving capabilities of libjconfig (parameters, clocks, portWidth)")
class ResolvingSpec extends BaseLibJConfigSpec {


	def "uart resolves clock through bus from target"() {
		setup:
		long targetFreq = document.pins["CLOCK_USER"].frequency.get()
		def sysclk = document.addCommonModule SYSCLK
		def clkfx_div = sysclk.parameters["CLKFX_DIVIDE"] as Long
		def clkfx_mul = sysclk.parameters["CLKFX_MULTIPLY"] as Long
		def core = document.addSubsystem CORE
		document.connect(core.ports["clk"], sysclk.ports["clkfx"])
		document.connect sysclk.ports["clk_gen"], document.pins["CLOCK_USER"]
		def uart = core.addPeripheral(UART)
		def freqParam = uart.parameters["CLOCK_FREQUENCY"]

		expect:
		expect freqParam.integerValue.get(), closeTo((targetFreq / clkfx_div) * clkfx_mul, 0.5f)
	}

	def "memory resolved clock through inverse bus"() {
		setup:
		long targetFreq = document.pins["CLOCK_USER"].frequency.get()

		def sysclk = document.addCommonModule(SYSCLK);

		long clkfx_divide = sysclk.parameters["CLKFX_DIVIDE"] as Long
		long clkfx_multiply = sysclk.parameters["CLKFX_MULTIPLY"] as Long

		def core = document.addSubsystem(CORE);
		document.connect(core.ports["clk"], sysclk.ports["clkfx"])
		document.connect(sysclk.ports["clk_gen"], document.pins["CLOCK_USER"])

		def mem = core.addCommonModule("test_mem")

		IIntegerParameterHandler clock_frequency = mem.getParameter("CLOCK_FREQUENCY")

		expect:
		expect clock_frequency.integerValue.get(), closeTo((targetFreq / clkfx_divide) * clkfx_multiply, 0.5f)
	}

	@Unroll
	def "correct handling of floats, dividing by #a"(String a) {
		setup:
		long targetFreq = document.pins["CLOCK_USER"].frequency.get()
		def sysclk = document.addCommonModule SYSCLK
		def dv_param = sysclk.parameters["CLKDV_DIVIDE"]

		def dv_freq =  sysclk.parameters["CLKDV_OUTPUT_FREQ"]
		document.connect sysclk.ports["clk_gen"], document.pins["CLOCK_USER"]


		dv_param.setStoredValue(a)
		float dv_value = dv_param.floatValue.get()

		expect:
		expect dv_freq.numberValue.get(), closeTo(((targetFreq as Float) / dv_value), 0.5f)

		where:
		a     | _
		"1.5" | _
		"2.0" | _
		"2.5" | _
		"3.0" | _
		"3.5" | _
		"4.0" | _
		"4.5" | _
		"5.0" | _
		"5.5" | _

	}
}
