/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.jconfig.tests

import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.verilog.Wire
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.verilog.WireBit
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.verilog.WireNames
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.outputProvider.verilog.WireType
import spock.lang.Specification
/**
 * Created by wenzel on 05.07.17.
 */
class WireNamesSpec extends Specification {

	static Wire preferSourceA = new Wire("a", "preferSource", 1, WireType.PreferSource, false)
	static WireBit preferSourceABit = new WireBit(preferSourceA,0)

	static Wire preferSourceB = new Wire("b", "preferSource", 1, WireType.PreferSource, false)
	static WireBit preferSourceBBit = new WireBit(preferSourceB,0)

	static Wire needSourceA = new Wire("a", "needSource", 1, WireType.NeedSource, false)
	static WireBit needSourceABit = new WireBit(needSourceA,0)

	static Wire needSourceB = new Wire("b", "needSource", 1, WireType.NeedSource, false)
	static WireBit needSourceBBit = new WireBit(needSourceB,0)

	static Wire regularA = new Wire("a", "regular", 1, WireType.Regular, false)
	static WireBit regularBitA = new WireBit(regularA,0)

	static Wire regularB = new Wire("b", "regular", 1, WireType.Regular, false)
	static WireBit regularBitB = new WireBit(regularB,0)

	static Wire regularC = new Wire("c", "regular", 1, WireType.Regular, false)
	static WireBit regularBitC = new WireBit(regularC,0)

	WireNames wireNames

	def setup() {
		wireNames = new WireNames()
	}

	def "connected wires get the preferred name"() {
		when:
		wireNames.addWireConnection(preferSourceABit, regularBitA)
		wireNames.addWireConnection(regularBitA, regularBitB)

		wireNames.addWireConnection(regularBitC, preferSourceBBit)

		then:
		wireNames.getWireName(preferSourceABit) == preferSourceABit
		wireNames.getWireName(regularBitA) == preferSourceABit
		wireNames.getWireName(regularBitB) == preferSourceABit

		wireNames.getWireName(regularBitC) == preferSourceBBit
		wireNames.getWireName(preferSourceBBit) == preferSourceBBit
	}

	def "connected wires get the needed name"() {
		when:
		wireNames.addWireConnection(preferSourceABit, regularBitA)
		wireNames.addWireConnection(regularBitB, needSourceABit)
		wireNames.addWireConnection(regularBitA, regularBitB)

		wireNames.addWireConnection(regularBitC, preferSourceBBit)
		wireNames.addWireConnection(regularBitC, needSourceBBit)

		then:
		wireNames.getWireName(preferSourceABit) == needSourceABit
		wireNames.getWireName(regularBitB) == needSourceABit
		wireNames.getWireName(regularBitA) == needSourceABit
		wireNames.getWireName(needSourceABit) == needSourceABit

		wireNames.getWireName(needSourceBBit) == needSourceBBit
		wireNames.getWireName(regularBitC) == needSourceBBit
		wireNames.getWireName(preferSourceBBit) == needSourceBBit

	}

	def "multiple needed names are not possible"() {
		when:
		wireNames.addWireConnection(needSourceBBit, regularBitA)
		wireNames.addWireConnection(regularBitA, regularBitB)
		wireNames.addWireConnection(regularBitA, preferSourceBBit)
		wireNames.addWireConnection(regularBitB, needSourceABit)

		then:
		thrown(RuntimeException)
	}

	def "appending to WireBitRange"() {
		when:
		Wire w = new Wire("a", "b", 3, WireType.Regular, false)
		Wire x = new Wire("a", "c", 3, WireType.Regular, false)
		WireNames.WireBitRange r = new WireNames.WireBitRange(w, 0);

		then:
		r.canAppend(new WireBit(w, 1))
		!r.canAppend(new WireBit(w, 2))
		!r.canAppend(new WireBit(x, 1))
	}

	def "verilog connections are created from wire names"() {
		when:
		Wire a = new Wire("x","a",2, WireType.PreferSource, false);
		Wire b = new Wire("x","b",2, WireType.Regular, false);
		wireNames.addWireConnection(new WireBit(a,0),new WireBit(b,0))
		wireNames.addWireConnection(new WireBit(a,1),new WireBit(b,1))

		then:
		wireNames.getWireConnection(b) == 'x$a'
		wireNames.getWireConnection(a) == 'x$a'

		when:
		Wire c = new Wire("x", "c", 2, WireType.PreferSource, true)
		Wire d = new Wire("x", "c", 3, WireType.Regular, false)
		Wire e = new Wire("", "e", 3, WireType.NeedSource, true)
		wireNames.addWireConnection(new WireBit(c, 0), new WireBit(d,0))
		wireNames.addWireConnection(new WireBit(c, 1), new WireBit(d,1))
		wireNames.addWireConnection(new WireBit(e, 0), new WireBit(d,2))

		then:
		wireNames.getWireConnection(c) == 'x$c'
		wireNames.getWireConnection(d) == '{ ~pin_e[0], ~x$c }'
	}

}
