/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.process;


import java.io.IOException;

/**
 * Created by Kris Heid on 02.09.16.
 */
public final class ProgramEvaluator {

	private static final String WHICH = "which";

	private ProgramEvaluator(){}

	/**
	 * executes which the program and looks if the error code indicates a normal termination
	 * @param program a program
	 */
	private static boolean terminatesNormally(ProcessBuilder program) throws IOException {

		int exitCode = ProcessUtil.runAndWaitFor(program);

		if (exitCode != 0) {
			return false;
		}
		return true;
	}

	private static boolean isWhichInstalled() throws IOException {
		return terminatesNormally(new ProcessBuilder(WHICH, WHICH));
	}


	/**
	 * Checks if the program is installed
	 * @param programName the name of the program
	 * @return true if the program is found
	 * @throws Exception if the program *which* can not be found
	 */
	public static boolean isProgramInstalled(String programName) throws Exception {
		if (isWhichInstalled()) {
			return terminatesNormally(new ProcessBuilder(WHICH, programName));
		} else {
			throw new Exception("The program *which* can not be found on the system...unable to evaluate if program exists.");
		}
	}

	/**
	 * Throws an exception if program is not existing or which is not installed or an IOException happens
	 * @param programName the program to be evaluated
	 * @throws Exception If the requested program can not be found
	 */
	public static void probeProgram(String programName) throws Exception {
		if (!isProgramInstalled(programName)) {
			throw new Exception("The program " + programName + " can not be found.");
		}
	}


}
