/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.util;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public abstract class ConnectedGraph<T,C extends ConnectedGraph.ConnectedComponent<T>> {

	private Map<T, C> currentGroup = new HashMap<>();
	private Set<C> activeGroups = new HashSet<>();

	protected abstract T getNewRepresentative(T a, T b);

	public static class ConnectedComponent<T> {
		public final Collection<T> contained;
		public T representative;

		public ConnectedComponent(T element) {
			representative = element;
			contained = new HashSet<>();
			contained.add(element);
		}
	}


	protected C getGroupOrCreate(T element) {
		return currentGroup.computeIfAbsent(element, element2 -> {
			final C wireGroup = createGroup(element2);
			activeGroups.add(wireGroup);
			return wireGroup;
		});
	}

	protected abstract C createGroup(T element);

	protected void connect(T a, T b) {
		C refA = getGroupOrCreate(a);
		C refB = getGroupOrCreate(b);

		T rep = getNewRepresentative(refA.representative, refB.representative);
		C union = unify(refA, refB);
		union.representative = rep;
	}


	private C unify(C a, C b) {


		C smaller;
		C larger;
		if (a.contained.size() > b.contained.size()) {
			smaller = b;
			larger = a;
		} else {
			smaller = a;
			larger = b;
		}

		for (T element : smaller.contained) {
			currentGroup.put(element, larger);
			larger.contained.add(element);
		}
		activeGroups.remove(smaller);

		postMergeActions(larger, smaller);

		return larger;
	}

	protected void postMergeActions(C a, C b) {
	}

	protected C getGroup(T element) {
		return currentGroup.get(element);
	}

	protected Set<C> getActiveGroups() {
		return activeGroups;
	}


	protected boolean containsElement(T element) {
		return currentGroup.containsKey(element);
	}
}
