/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.util;


import com.beust.jcommander.ParameterException;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Created by Kris Heid on 01.09.16.
 */
public final class EnvironmentVariablesUtil {

	public static final String SPARTANMC_ROOT = "SPARTANMC_ROOT";

	private EnvironmentVariablesUtil() {
	}

	/**
	 * @param environmentVariable the String of the environment variable to evaluate
	 * @return the value of the environment variable
	 * @throws ParameterException if the environment variable can not be evaluated
	 */
	public static Path getEnvironmentVariable(String environmentVariable) throws ParameterException {
		String envRoot = System.getenv(environmentVariable);
		if (envRoot == null) {
			throw new ParameterException("Environment variable \"" + environmentVariable + "\" is not set");
		}
		envRoot = FileUtil.replaceTildeWithUserHome(envRoot);
		Path path = null;
		if (envRoot != null && !envRoot.isEmpty()) {
			path = Paths.get(envRoot);
		}
		if (path == null) {
			throw new ParameterException("Environment variable \"" + environmentVariable + "\" points to an invalid Path");
		}
		return path;
	}

	/**
	 *
	 * @param environmentVariable the environment variable to evaluate
	 * @return the value of the environment variable as a Path if this
	 * @throws ParameterException if the environment variable can not be evaluated
	 */
	public static Path getEnvironmentVariableAsFolder(String environmentVariable) throws ParameterException {
		Path path = getEnvironmentVariable(environmentVariable);
		if (!Files.isDirectory(path)) {
			throw new ParameterException("Path " + path + " does not point to a valid directory");
		}
		return path;
	}



}
