/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.util;

import com.beust.jcommander.ParameterException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.file.Path;
import java.nio.file.Paths;


/**
 * Created by Kris Heid on 26.10.16.
 */
public final class SpartanMCDefaultFolders {

	private static final Logger logger = LoggerFactory.getLogger(EnvironmentVariablesUtil.class);

	private SpartanMCDefaultFolders() {
	}

	/**
	 * The path to the c headers relative to the given folder. If nothing is specified, the method dries to evaluate the environemnt variable
	 *
	 * @param spartanmcRoot a specific SpartanMC Root directory
	 * @return the path
	 */
	public static Path cHeaders(Path... spartanmcRoot) {
		Path p;
		if (spartanmcRoot.length > 0) {
			p = spartanmcRoot[0];
		} else {
			p = getSpartanMCRootFromEnvironmentVariable();
		}
		return p.resolve(Folders.SPARTANMC.get()).resolve(Folders.INCLUDE.get());
	}

	/**
	 * The path to the hardware relative to the given folder. If nothing is specified, the method dries to evaluate the environemnt variable
	 *
	 * @param spartanmcRoot a specific SpartanMC Root directory
	 * @return the path
	 */
	public static Path hardware(Path... spartanmcRoot) {
		Path p;
		if (spartanmcRoot.length > 0) {
			p = spartanmcRoot[0];
		} else {
			p = getSpartanMCRootFromEnvironmentVariable();
		}
		return p.resolve(Folders.SPARTANMC.get()).resolve(Folders.HARDWARE.get());
	}

	/**
	 * The path to the documentation pdfs relative to the given folder. If nothing is specified, the method dries to evaluate the environemnt variable
	 *
	 * @param spartanmcRoot a specific SpartanMC Root directory
	 * @return the path
	 */
	public static Path documentationPdf(Path... spartanmcRoot) {
		return getSpartanMCRoot(spartanmcRoot).resolve(Folders.DOC.get()).resolve(Folders.CHAPTERS.get());
	}


	/**
	 * @return the spartanmc root dir
	 */
	public static Path getSpartanMCRootFromEnvironmentVariable() {
		try {
			return EnvironmentVariablesUtil.getEnvironmentVariableAsFolder(EnvironmentVariablesUtil.SPARTANMC_ROOT);
		} catch (ParameterException parameterException) {
			logger.error("Invalid parameter at SpartanMCRoot Evaluation");
		}
		return null;
	}


	private static Path getSpartanMCRoot(Path... spartanmcRoot) {
		Path p;
		if (spartanmcRoot.length > 0) {
			p = spartanmcRoot[0];
		} else {
			p = getSpartanMCRootFromEnvironmentVariable();
		}
		return p;
	}

	public static Path getTestProjects(Path... spartanmcRoot) {
		return getSpartanMCRoot(spartanmcRoot).resolve(Folders.TESTS.get());
	}

	public static Path getMicroStreamsIntegrationtestProjects(Path... spartanmcRoot) {
		return getTestProjects(spartanmcRoot).resolve(Folders.MICROSTREAMS_INTEGRATIONTEST_PROJECTS.get());
	}


	public static Path getLoopOptimizerIntegrationtestProjects(Path... spartanmcRoot) {
		return getTestProjects(spartanmcRoot).resolve(Folders.LOOPOPTIMIZER_INTEGRATIONTEST_PROJECTS.get());
	}

	public static Path getIntegrationtestsGeneral(Path... spartanmcRoot) {
		return getTestProjects(spartanmcRoot).resolve(Folders.INTEGRATIONTESTS_GENERAL.get());
	}

	public static Path getAutoPerfIntegrationtestProjects(Path... spartanmcRoot) {
		return getTestProjects(spartanmcRoot).resolve(Folders.AUTOPERF_INTEGRATIONTESTS.get());
	}

	public static Path getIntegrationtestProjectsFWInput() {
		return Folders.FIRMWARE_TEST_INPUT.getAsPath();
	}

	public static Path getIntegrationtestProjectsReferenceProjects() {
		return Folders.TEST_REFERENCE_PROJECTS.getAsPath();
	}

	public static Path getIntegrationtestResources() {
		return Folders.RESOURCES.getAsPath();
	}

	private enum Folders {
		DOC("doc"),
		SPARTANMC("spartanmc"),
		BUSES("buses"),
		DEVICES("devices"),
		HARDWARE("hardware"),
		INCLUDE("include"),
		LIB_AS("lib_as"),
		LIB_OBJ("lib_obj"),
		SRC("src"),
		TARGETS("targets"),
		TESTS("tests"),
		CHAPTERS("chapters"),
		TOOLS("tools"),
		MICROSTREAMS_INTEGRATIONTEST_PROJECTS("microStreamsIntegrationtestProjects"),
		LOOPOPTIMIZER_INTEGRATIONTEST_PROJECTS("loopOptimizerIntegrationtestProjects"),
		INTEGRATIONTESTS_GENERAL("integrationtests"),
		AUTOPERF_INTEGRATIONTESTS("autoPerfIntegrationtests"),
		FIRMWARE_TEST_INPUT("testFirmware"),
		TEST_REFERENCE_PROJECTS("verifiedOutput"),
		RESOURCES("resources");

		private String folder;

		Folders(String folder) {
			this.folder = folder;
		}

		String get() {
			return folder;
		}

		Path getAsPath() {
			return Paths.get(folder);
		}
	}
}
