/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.loopOptimizer.model;



import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.other.OtherModelComponent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * Here should reside the profiled loop. Imported from the autoperf CSV
 */
public class LoopRuntimeReport implements OtherModelComponent {

	protected Model model;

	/**
	 * Key element of LoopRuntimeReport.
	 * The innermost List represents multiple profiling steps inside one Iteration.
	 * The middle List represents multiple Iterations of one Loop.
	 * The outer List represents multiple Loops that were profiled
	 */
	private final HashMap<String, ArrayList<ArrayList<Integer>>> report;
	private final Logger log = LoggerFactory.getLogger(LoopRuntimeReport.class);

	public LoopRuntimeReport() {
		report = new HashMap<>();
	}

	/**
	 * Adds element to the HashMap representing one loop with the ID key
	 * @param value value
	 * @param  key key
	 * @return true if successful, false if one element had to be removed because the key was already in the HashMap
	 */
	public boolean addLoop(ArrayList<ArrayList<Integer>> value, String key) {
		if (report.put(key, value)!=null) {
			log.warn("There were duplicate keys in the HashMap of LoopRuntimeReport. The key is: " + key +
					". This means that only data from the last run of the loop will be saved and used");
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Returns number of Loops inside report
	 * @return loop count
	 */
	public int getLoopCount() {
		return report.size();
	}

	/**
	 * Returns the report of the loop with Index loopKey
	 * @param loopKey loop key
	 * @return ArrayList of ArrayList of Integers representing one full loop
	 */
	public ArrayList<ArrayList<Integer>> getLoop(String loopKey) {
		return report.get(loopKey);
	}

	@Override
	public Model getModel() {
		return model;
	}

	@Override
	public void setModel(Model m) {
		model = m;
	}
}
