/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.loopOptimizer.model;

import cetus.hir.Traversable;

/**
 * Represents several statement properties
 */
public class StatementDependencyProperties {

	private Traversable statement = null;

	private Traversable lastDepentStatement = null;

	private Integer position = new Integer(-1);

	private Integer lastDepentStatementPostion = new Integer(-1);


	/**
	 * a constructor
	 * @param statement the statement
	 * @param lastDepentStatement the last statement depending on the statement
	 * @param position the position in the code
	 * @param lastDepentStatementPostion the position of the last statement
	 */
	public StatementDependencyProperties(Traversable statement, Traversable lastDepentStatement, Integer position, Integer lastDepentStatementPostion) {
		this.statement = statement;
		this.lastDepentStatement = lastDepentStatement;
		this.position = position;
		this.lastDepentStatementPostion = lastDepentStatementPostion;
	}

	public Traversable getStatement() {
		return statement;
	}

	public void setStatement(Traversable statement) {
		this.statement = statement;
	}

	public Traversable getLastDepentStatement() {
		return lastDepentStatement;
	}

	public void setLastDepentStatement(Traversable lastDepentStatement) {
		this.lastDepentStatement = lastDepentStatement;
	}

	public Integer getPosition() {
		return position;
	}

	public void setPosition(Integer position) {
		this.position = position;
	}

	public Integer getLastDepentStatementPostion() {
		return lastDepentStatementPostion;
	}

	public void setLastDepentStatementPostion(Integer lastDepentStatementPostion) {
		this.lastDepentStatementPostion = lastDepentStatementPostion;
	}

	public String toString() {
		return "[" + position + "] " + statement;
	}
}
