/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.loopOptimizer.transformationPasses;

import de.tu_darmstadt.rs.microStreams.config.IConfiguration;
import de.tu_darmstadt.rs.microStreams.config.LoopOptimizerConfiguration;
import de.tu_darmstadt.rs.microStreams.loopOptimizer.model.LoopRuntimeReport;
import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.ModelComponent;
import de.tu_darmstadt.rs.microStreams.transformationPasses.AbstractTransformationPass;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This should get the profile CSV and transform it into the internal representation: LoopRuntimeReport
 */
public class LoopProfileImporter extends AbstractTransformationPass {
	private final Logger log = LoggerFactory.getLogger(LoopProfileImporter.class);

	public LoopProfileImporter() {
	}

	@Override
	public boolean init() {
		return true;
	}

	@Override
	public List<Class<? extends ModelComponent>> requiredModelComponents() {
		//TODO: auf LoopOptimizerConfiguration ändern
		return Arrays.asList(IConfiguration.class);
	}

	@Override
	public List<Class<? extends ModelComponent>> providedModelComponents() {
		return Arrays.asList(LoopRuntimeReport.class);
	}

	@Override
	public boolean process(Model model) {
		String file = model.getModelComponent(LoopOptimizerConfiguration.class).getApplicationProfile();
		//prevents exception when no profile should be read
		if (!"".equalsIgnoreCase(file)) {
			LoopRuntimeReport report = read(file);
			model.addModelComponent(report);
		}
		return true;
	}

	/**
	 * Reads a csv file at the specified location and parses it as {@link LoopRuntimeReport}
	 * @param file file location
	 * @return the generated {@link LoopRuntimeReport}
	 */
	private LoopRuntimeReport read(String file) {
		BufferedReader br = null;
		String line;
		LoopRuntimeReport report = new LoopRuntimeReport();
		ArrayList<Integer> currentIter = new ArrayList<>();
		ArrayList<ArrayList<Integer>> loop = new ArrayList<>();
		String key = "";

		try {
			br = new BufferedReader(new FileReader(file));
			while ((line = br.readLine()) != null) {
				String[] split = line.split(";");
				if (split[0].equalsIgnoreCase("Loop end")) {
					report.addLoop(loop, key);
					loop = new ArrayList<>();
					key = "";
				} else if (split[0].equalsIgnoreCase("Iteration end")) {
					loop.add(currentIter);
					currentIter = new ArrayList<>();
				} else {
					currentIter.add(Integer.valueOf(split[1]));
					if ("".equals(key)) {
						key = split[0].split(":")[1] + ":" + split[0].split(":")[2];
					}
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (br!=null) {
				try {
					br.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return report;
	}
}
