/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams;

import de.tu_darmstadt.rs.microStreams.config.GuiInspectorConfiguration;
import de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.ConnectionAccessCreator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.CoreConnectionCreator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.CoreConnectionDMAModifier;
import de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.CoreCreator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.EvaluationPeripheralCreator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.peripheralDetection.PeripheralDetector;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.ArgumentParser;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.CetusParser;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.EmptyConfigurationGenerator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.InputCollector;
import de.tu_darmstadt.rs.microStreams.transformationPasses.input.LibraryCollector;
import de.tu_darmstadt.rs.microStreams.transformationPasses.output.GlobalDefinitionRemover;
import de.tu_darmstadt.rs.microStreams.transformationPasses.output.HardwareBuilder;
import de.tu_darmstadt.rs.microStreams.transformationPasses.output.SoftwareBuilder;
import de.tu_darmstadt.rs.microStreams.transformationPasses.output.graph.MicroStreamsDrawer;
import de.tu_darmstadt.rs.microStreams.transformationPasses.TransformationPass;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.ChildtaskCallgraph;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.DependencyCreator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.FunctionCallgraph;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.InnerFlowCreator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.InnerFlowLinker;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.NecessaryFileSearcher;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.Replicate;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.TaskCreator;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.TaskPipeline;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.TransferredCharAddressAligner;
import de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel.VariableAccessCreator;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * This is the µStreams Source-To-Source Compiler base class.
 *
 * @author Jan Weber
 */
public class MicroStreams extends TransformationPassCollector {

	private final List<TransformationPass> workers = new LinkedList<>(
			Arrays.asList(
					new LibraryCollector(),
					new InputCollector(),
					new CetusParser(),
					new TaskCreator(),
					new Replicate(),
					new FunctionCallgraph(),
					new ChildtaskCallgraph(),
					new CoreCreator(),
					new VariableAccessCreator(),
					new InnerFlowCreator(),
					new InnerFlowLinker(),
					new DependencyCreator(),
					new TaskPipeline(),
					new MicroStreamsDrawer(),
					new PeripheralDetector(),
					new CoreConnectionCreator(),
					new ConnectionAccessCreator(),
					new EvaluationPeripheralCreator(),
					new SoftwareBuilder(),
					new HardwareBuilder(),
					new ArgumentParser(),
					new NecessaryFileSearcher(),
					new GlobalDefinitionRemover(),
					new CoreConnectionDMAModifier(),
					new EmptyConfigurationGenerator(new GuiInspectorConfiguration()),
					new TransferredCharAddressAligner()
			)
	);

	/**
	 * The constructor
	 * @param args the cmd line args
	 */
	public MicroStreams(String[] args) {
		super(args);
	}

	public List<TransformationPass> getWorkers() {
		return workers;
	}

}
