/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams;

import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.util.FileUtil;

import org.slf4j.Logger;

/**
 * This class gives some common methods for the interface
 */
public abstract class TransformationPassCollectorLauncher {

	/**
	 * runs µStreams
	 * @param args all cmd arguments
	 * @return true if µStreams ran through
	 */
	public boolean run(String[] args) {

		LogoHeaderPrinter.pringLogo(getLogger()::info, getProgramName());
		long startTime = System.currentTimeMillis();

		boolean runSuccessful;
		if (initWorkerCollector(args).process()) {
			long endTime   = System.currentTimeMillis();
			long totalTime = endTime - startTime;
			getLogger().info(getProgramName() + " finished successfully after " + totalTime + "ms ...");
			runSuccessful = true;
		} else {
			getLogger().warn(getProgramName() + " was not finished successfull, see log for further details.");
			runSuccessful = false;
		}
		FileUtil.deleteDirectoryRecursively(Constants.FilesAndFolders.TMP_FOLDER_MICROSTREAMS_USER.toFile());
		return runSuccessful;
	}

	protected abstract TransformationPassCollector initWorkerCollector(String[] args);

	protected abstract Logger getLogger();

	protected abstract String getProgramName();

	public abstract TransformationPassCollector getWorkerCollector();
}
