/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.config;

import de.tu_darmstadt.rs.jcommander.converter.ReplaceTildeWithUserHomeConverter;
import de.tu_darmstadt.rs.microStreams.config.Configuration;
import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.microStreams.config.IConfiguration;
import de.tu_darmstadt.rs.microStreams.model.Model;

import com.beust.jcommander.Parameter;

/**
 * Holds the configuration of the loop optimizer
 */
public class LoopOptimizerConfiguration extends Configuration implements IConfiguration {

	private Model model;
	@Parameter(names = {"--profile"}, description = "Loop profile from AutoPerf in LoopOpt Mode", converter = ReplaceTildeWithUserHomeConverter.class)
	private String applicationProfile = "";

	@Parameter(names = {"-t", "--taskPragma"}, description = "Set Task-Pragmas (\"#pragma microStreams task\") for microStreams between splitted or fissioned loops")
	private boolean taskPragma = false;

	public LoopOptimizerConfiguration() {
		super();
	}

	public String getApplicationProfile() {
		return applicationProfile;
	}

	public String getProgramName() {
		return Constants.LOOPOPTIMIZER;
	}

	@Override
	public Model getModel() {
		return model;
	}

	@Override
	public void setModel(Model m) {
		model = m;
	}

	public boolean getTaskPragma() {
		return taskPragma;
	}

}
