/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model;

import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.Peripheral;
import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.SimPrint;
import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.Uart;

import org.slf4j.Logger;

import java.util.Collection;

/**
 * Components used for evaluation/ runtime measurement
 */
public class Evaluation implements ModelComponent {

	public static final String EVALUATION_COMPONENT_FOUND_MULTIPLE_TIMES = "Evaluation Component found multiple times: ";
	Logger log = org.slf4j.LoggerFactory.getLogger(this.getClass());

	private Model project;


	private Uart baseUart = null;

	private SimPrint baseSimPrint = null;

	/**
	 * the constructor
	 * @param peripherals peripherals for the evaluation
	 */
	public Evaluation(Collection<Peripheral> peripherals) {

		for (Peripheral peripheral : peripherals) {
			if (peripheral.isEvaluationComponent()) {
				if (peripheral instanceof Uart) {
					if (null == this.baseUart) {
						this.baseUart = (Uart) peripheral;
					} else {
						log.warn(EVALUATION_COMPONENT_FOUND_MULTIPLE_TIMES
								+ peripheral);
					}
				} else if (peripheral instanceof SimPrint) {
					if (null == this.baseSimPrint) {
						this.baseSimPrint = (SimPrint) peripheral;
					} else {
						log.warn(EVALUATION_COMPONENT_FOUND_MULTIPLE_TIMES
								+ peripheral);
					}
				}

			}
		}
	}

	@Override
	public Model getModel() {
		return project;
	}

	@Override
	public void setModel(Model project) {
		this.project = project;
	}

	public Uart getUart() {
		return baseUart;
	}

	public SimPrint getSimPrint() {
		return baseSimPrint;
	}
}
