/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.connection;

import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareComponentFactory;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareModelComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareModelSubComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.core.Core;

/**
 * This {@link HardwareModelComponent} represents a communication connection
 * between two or multiple {@link IConnectionPeripheral ConnectionPeripherals}.
 *
 * @author Jan Weber
 */
public abstract class Connection implements HardwareModelComponent {

	/**
	 * {@link IConnectionPeripheral ConnectionPeripherals} that are connected
	 * with this connection;
	 */
	//private Collection<ConnectionPeripheral> connectionPeripherals = new HashSet<ConnectionPeripheral>();

	private int id;

	private Model project;

	private boolean isEvaluationComponent;

	private ConnectionType connectionType;

	/**
	 * empty public constructor for {@link HardwareComponentFactory}
	 */
	public Connection() {

	}

	@Override
	public boolean addSubComponent(HardwareModelSubComponent component) {
		throw new UnsupportedOperationException("Has no subcomponents.");
	}

	@Override
	public int getId() {
		return id;
	}

	@Override
	public void setId(int id) {
		this.id = id;

	}

	@Override
	public String getName() {
		return getType() + "_" + getId();
	}

	@Override
	public boolean isEvaluationComponent() {
		return isEvaluationComponent;
	}

	@Override
	public void setEvaluationComponent(boolean isEvaluationComponent) {
		this.isEvaluationComponent = isEvaluationComponent;

	}

	@Override
	public Model getModel() {
		return this.project;
	}

	@Override
	public void setModel(Model project) {
		this.project = project;

	}

	/**
	 * see if a connection is possible
	 * @param sendingCore the sending core
	 * @param receivingCore the receiving core
	 * @return true if a connection is possible
	 */
	public abstract boolean isConnecting(Core sendingCore, Core receivingCore);

	@Override
	public abstract String toString();

	public void setConnectionType(ConnectionType type) {
		this.connectionType = type;
	}

	public ConnectionType getConnectionType() {
		return connectionType;
	}

	/**
	 * Tells if the transferred variables need initialization
	 * @return true if the transferred variable need special initialization e.g. they are transferred via DMA
	 */
	public boolean transferredVariablesNeedInitialization() {
		if (Constants.JConfigTypes.DMA_TYPES.contains(getType())) {
			return true;
		}
		else {
			return false;
		}
	}

	/**
	 * Enum of the possible interconnect types
	 */
	public enum ConnectionType {
		POINT_TO_POINT, DISPATCHER, CONCENTRATOR, NETWORK, UNKNOWN
	}

}
