/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.connection;

import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.microStreams.model.software.variable.Variable;
import de.tu_darmstadt.rs.util.StreamUtil;

import cetus.hir.Identifier;

/**
 * Represents a connection peripheral with a DMA interface instead of registers
 * Created by Kris Heid on 12.12.17.
 */
public abstract class ConnectionPeripheralDMA extends ConnectionPeripheral implements IConnectionPeripheralDMA {

	private static final int BRAM_SIZE = 2048;

	@Override
	public Identifier getIdentifierFor(Variable variable) {
		return new Identifier("&" + getStructInstanceName() + "." + variable.getName());
	}

	@Override
	public IConnectionPeripheralDMA getDMAMasterPeripheral() {
		return getConnections().stream()
				.flatMap(c -> c.getConnectionPeripherals().stream())
				.flatMap(StreamUtil.castOrEmpty(IConnectionPeripheralDMA.class))
				.filter(p -> p instanceof MasterPeripheral)
				.distinct()
				.collect(StreamUtil.exactlyOne());
	}

	@Override
	public String getName() {
		return super.getName();
	}

	public String getNameDMAAlias() {
		return getName()  + Constants.JConfigTypes.ADDITIONAL_DMA_APPENDIX;
	}

	public String getDMAHwAddress() {
		return getNameDMAAlias().toUpperCase();
	}

	public String getStructName() {
		return getCore().getName() + "_" + getName() + "_struct";
	}

	public String getStructInstanceName() {
		return getStructName() + "_inst";
	}

	public int roundUpToBrams(int bytes) {
		int rest = BRAM_SIZE - bytes % BRAM_SIZE;
		return bytes + rest;
	}

	/**
	 * Get the section name of a DMA peripheral.
	 *
	 * Used by the Freemarker template processor.
	 *
	 * @return Section name of the peripheral.
	 */
	public String getSectionName() {
		return ".dma." + getName();
	}
}
