/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.connection;

import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.PeripheralParameter;
import de.tu_darmstadt.rs.microStreams.model.software.variable.Variable;

import cetus.hir.ClassDeclaration;
import cetus.hir.NameID;
import cetus.hir.UserSpecifier;
import cetus.hir.VariableDeclaration;
import cetus.hir.VariableDeclarator;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

/**
 * The slave implementation of a DMA connection peripheral
 * Created by Kris Heid on 12.12.17.
 */
public abstract class ConnectionPeripheralSlaveDMA extends ConnectionPeripheralDMA implements SlavePeripheral {

	@Override
	public boolean addVariable(Variable variable) {
		return getDMAMasterPeripheral().addVariable(variable);
	}

	@Override
	public Set<Variable> getVariables() {
		return getDMAMasterPeripheral().getVariables();
	}

	@Override
	public Integer getDMAOffsetOf(Variable variable) {
		return getDMAMasterPeripheral().getDMAOffsetOf(variable);
	}

	@Override
	public int getFirstFreeMemoryOffset() {
		return getDMAMasterPeripheral().getFirstFreeMemoryOffset();
	}

	public ClassDeclaration getStructDeclaration() {
		return ((ConnectionPeripheralMasterDMA)getDMAMasterPeripheral()).getStructDeclaration();
	}

	/**
	 * Returns the definition of the struct containing the variables received via this peripheral.
	 *
	 * @return The definition statement.
	 */
	public VariableDeclaration getStructDefinition() {
		VariableDeclarator varDecl = new VariableDeclarator(new NameID(getStructInstanceName()));

		String structName = ((ConnectionPeripheralMasterDMA)getDMAMasterPeripheral()).getStructName();
		VariableDeclaration varDecn = new VariableDeclaration(new UserSpecifier(new NameID("struct " + structName)), varDecl);
		varDecn.setSemiColon(true);

		return varDecn;
	}

	@Override
	public List<PeripheralParameter> getParameters() {
		final int usedBytes = getFirstFreeMemoryOffset() + getDMAPeripheralControlRegionSize();
		final PeripheralParameter dma_size = new PeripheralParameter("DMA_SIZE", Integer.toString(roundUpToBrams(usedBytes)));
		return Arrays.asList(dma_size);
	}
}
