/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.connection;

import de.tu_darmstadt.rs.microStreams.model.hardware.core.Core;

import java.util.Collection;
import java.util.LinkedHashSet;

/**
 * Direct Inter Core Connection
 */
public class DirectConnection extends Connection {

	private Collection<Core> cores = new LinkedHashSet<Core>();


	public DirectConnection() {
		super();
	}

	/**
	 * Adds an {@link Core} to this {@link DirectConnection}. Returns
	 * true or false, depending whether the operation was successful or not
	 * (Peripheral already added).
	 *
	 * @param core the {@link Core} that shall be added
	 * @return true if this {@link DirectConnection} changed as a result of the call
	 */
	public boolean addConnection(Core core) {
		return cores.add(core);
	}

	public boolean isConnected(Core core) {
		return cores.contains(core);
	}

	@Override
	public boolean isConnecting(Core sendingCore, Core receivingCore) {
		return cores.contains(sendingCore) && cores.contains(receivingCore);
	}

	@Override
	public String toString() {
		return getName() + " [sendingCores = " + cores
				+ " receivingCores = " + cores + ", id=" + getId() + "]\n";
	}

	/**
	 * Removes an {@link IConnectionPeripheral} from the {@link Connection}.
	 * Returns true or false, depending whether the operation was successful or
	 * not.
	 *
	 * @param core peripheral to be disconnected
	 * @return true if remove is successful
	 */
	public boolean removeConnectionPeripheral(Core core) {
		return cores.remove(core);

	}

	@Override
	public String getType() {
		return "DirectConnection";
	}
}
