/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.connection;

import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareModelComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareModelSubComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.core.Core;
import de.tu_darmstadt.rs.microStreams.model.software.include.Include;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Java Hardware representation of a Dispatcher Master
 */
public class DispatcherMasterDMA extends ConnectionPeripheralMasterDMA implements SendPeripheral {

	private static final int DMAPeripheralControlRegionSize = 8;

	public DispatcherMasterDMA() {}

	@Override
	public void validate() throws ConnectionPeripheralsIncompatibleException {
		Collection<IConnectionPeripheral> peripherals = getConnections()
				.stream()
				.flatMap(c -> c.getConnectionPeripherals().stream())
				.collect(Collectors.toList());

		for (IConnectionPeripheral peripheral : peripherals) {
			if (!(peripheral instanceof DispatcherMasterDMA) && !(peripheral instanceof DispatcherSlaveDMA)) {
				throw new ConnectionPeripheralsIncompatibleException(peripheral.getClass() + " is incompatible to " + this.getClass());
			}
			if ((peripheral instanceof DispatcherMasterDMA)	&& !peripheral.equals(this)) {
				throw new ConnectionPeripheralsIncompatibleException("Not more than one " + this.getClass()	+ " in this Connection allowed.");
			}
		}
	}

	@Override
	public String getName() {
		return getType() + "_" + getId();
	}

	@Override
	public String getType() {
		return Constants.JConfigTypes.DISPATCHER_MASTER_DMA;
	}

	@Override
	public String getPeriTypeName() {
		return Constants.JConfigTypes.DISPATCHER_DMA_PERI_TYPE;
	}

	@Override
	public int getDMAPeripheralControlRegionSize() {
		return DMAPeripheralControlRegionSize;
	}
}
