/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.connection;

import de.tu_darmstadt.rs.microStreams.model.software.variable.Variable;
import de.tu_darmstadt.rs.util.StreamUtil;

import cetus.hir.Identifier;

import java.util.Set;

/**
 * Interface for every DMA connection peripheral
 * Created by Kris Heid on 04.12.17.
 */
public interface IConnectionPeripheralDMA extends IConnectionPeripheral {

	/**
	 * Adds a variable to the list of variables to be send over this connection. Each variable should have a memory offset within this peripheral according to its size
	 * @param variable a variable to be added
	 * @return true if the variable was added to the set
	 */
	boolean addVariable(Variable variable);

	/**
	 * A set of variables send over this connection
	 * @return set of variables send over this connection
	 */
	Set<Variable> getVariables();

	/**
	 * Gives the DMA offset, where this peripheral is stored in the peripherals DMA space
	 * @param variable a variable to find
	 * @return the DMA offset, where this peripheral is stored in the peripherals DMA space
	 */
	Integer getDMAOffsetOf(Variable variable);

	/**
	 * Find Identifier for Variable with DMA offset
	 * @param variable a variable
	 * @return the identifier to init the variable with the dma offset
	 */
	Identifier getIdentifierFor(Variable variable);

	/**
	 * Gives the master of the underlying connection
	 * @return the master of the underlying connection, or null if no connection, or no master is there
	 */
	IConnectionPeripheralDMA getDMAMasterPeripheral();

	/**
	 * Gives the last/highest used memory address
	 * @return the highest used memory address
	 */
	int getFirstFreeMemoryOffset();

	/**
	 * gives the offset for control registers of the peripheral before the data in the dMA region comes
	 * @return control register offset
	 */
	int getDMAPeripheralControlRegionSize();

}
