/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.other;

import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.ModelComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareComponentFactory;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareModelComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareModelSubComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.Peripheral;
import de.tu_darmstadt.rs.microStreams.transformationPasses.TransformationPass;
import de.tu_darmstadt.rs.microStreams.model.software.SoftwareModelComponent;

/**
 * <p>
 * This is the {@link OtherModelComponent} interface all other Components of the
 * µStreams {@link Model} inherit from. In the µStreams Source-To-Source Compiler
 * Model and the Operation are disjoint. While {@link Model} is representing the
 * entire Datamodel the µStreams Source-To-Source Compiler is working on,
 * implementations of the {@link TransformationPass} interface are Operations working on
 * this Datamodel.
 * <p>
 * <p>
 * <p>
 * All Components of the Model have to implement the {@link ModelComponent}
 * interface or one of its sub-interfaces: {@link SoftwareModelComponent},
 * {@link HardwareModelComponent}, {@link HardwareModelSubComponent},
 * {@link Peripheral}, {@link OtherModelComponent}. All
 * {@link HardwareModelComponent HardwareModelComponent} implementations and its
 * sub-interface implementations are instantiated through the
 * {@link HardwareComponentFactory} factory to ensure that every
 * {@link HardwareModelComponent} has an unique id, that is necessary for proper
 * hardware generation.
 * </p>
 * <p>
 * <p>
 * All {@link ModelComponent ModelComponents} that are neither belong to the
 * {@link SoftwareModelComponent} nor to the {@link HardwareModelComponent} are
 * gathered under the {@link OtherModelComponent} interface.
 * </p>
 * <p>
 * <p>
 * As soon as a {@link ModelComponent} is added to a {@link Model} instance, the
 * {@link ModelComponent#setModel(Model)} is called. A {@link ModelComponent}
 * implementation have to store this {@link Model} instance and return it, if
 * the {@link ModelComponent#getModel()} method is called.
 * </p>
 *
 * @author Jan Weber
 */
public interface OtherModelComponent extends ModelComponent {

}
