/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.other;

import de.tu_darmstadt.rs.microStreams.model.Model;
import org.apache.commons.io.FilenameUtils;

import java.io.File;

/**
 * <p>
 * This ModelComponent represents a original C Firmware file handed over
 * at the call of µStreams.
 * </p>
 * <p>
 * <p>
 * It stores the actual file and optional a path. The path is only necessary, if
 * the file was discovered by scanning a folder handed over at the call of
 * µStreams. It is used to generate the search path for the C header files.
 * </p>
 *
 * @author Jan Weber
 */
public class SourceFile implements OtherModelComponent {

	private File file;

	private File folder;

	private String ext;

	private Model model;

	/**
	 * the constructor
	 * @param file the according file
	 * @param folder folder containing this fil
	 * @param ext file extension
	 * @throws NullPointerException if one of the params is null
	 */
	public SourceFile(File file, File folder, String ext) throws NullPointerException {
		if (ext == null || folder == null || file == null) {
			throw new NullPointerException("The input can nor be null!");
		} else {
			this.file = file;
			this.folder = folder;
			this.ext = ext;
		}

	}

	public SourceFile(String fileName) throws NullPointerException {
		this(new File(fileName), new File(FilenameUtils.getFullPath(fileName)), FilenameUtils.getExtension(fileName));
	}

	public SourceFile(File inputFile) throws NullPointerException {
		this(inputFile.toString());
	}


	public String toString() {
		return file.getAbsolutePath();
	}

	/**
	 * Returns the {@link File} this {@link SourceFile} represents.
	 *
	 * @return the {@link File} this {@link SourceFile} represents.
	 */
	public File getFile() {
		return file.getAbsoluteFile();
	}

	/**
	 * Returns the folder this {@link SourceFile} is located in. It is returned
	 * <code>null</code>, if this {@link SourceFile} was not found by scanning a
	 * directory, but handed over directly.
	 *
	 * @return the Folder this {@link SourceFile} represents.
	 */
	public File getFolder() {
		return folder;
	}

	@Override
	public Model getModel() {
		return model;
	}

	@Override
	public void setModel(Model m) {
		this.model = m;

	}

	/**
	 * Returns this {@link SourceFile SourcFile's} extension.
	 *
	 * @return this {@link SourceFile SourcFile's} extension.
	 */
	public String getExt() {
		return ext;
	}
}
