/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.software.freemarker;

import de.tu_darmstadt.rs.microStreams.model.software.drivers.CombinedConnectionPeripheralAccess;
import de.tu_darmstadt.rs.microStreams.model.software.drivers.ConnectionPeripheralAccess;

/**
 * A general Body for Freemarker printing.
 */
public class Body {

	private CodeSegment code;

	private ConnectionPeripheralAccess access;

	private EvaluationHardwareAccess evalAccess;

	private CombinedConnectionPeripheralAccess multiAccess;

	private ConnectionPeripheralWait connWait;

	public Body(EvaluationHardwareAccess evalAccess) {
		this.evalAccess = evalAccess;
	}

	public Body(CodeSegment code) {
		this.code = code;
	}

	public Body(ConnectionPeripheralAccess access) {
		this.access = access;
	}

	public Body(CombinedConnectionPeripheralAccess multiAccess) {
		this.multiAccess = multiAccess;
	}

	public Body(ConnectionPeripheralWait connWait) {
		this.connWait = connWait;
	}

	/**
	 * give the type of the body
	 * @return the section type as a string
	 */
	public BodyType getType() {
		if (null != code) {
			return BodyType.CODE;
		} else if (null != access) {
			return BodyType.CONNECTION_PERIPHERAL_ACCESS;
		} else if (null != evalAccess) {
			return BodyType.EVALUATION_ACCESS;
		} else if (null != multiAccess) {
			return BodyType.MULTI_CONNECTION_PERIPHERAL_ACCESS;
		} else if (null != connWait) {
			return BodyType.CONNECTION_PERIPHERAL_WAIT;
		}
		throw new UnsupportedOperationException();
	}

	public CombinedConnectionPeripheralAccess getMultiAccess() {
		return multiAccess;
	}

	public CodeSegment getCode() {
		return code;
	}

	/**
	 * Removes the enclosing brackets at the start and the end of the code otherwise the scope of the resulting program will be shifted.
	 *
	 * @return the code without starting and closing brackets at the beginning and end
	 */
	public String getCodeWithoutScope() {
		String codeString = getCode().print();
		if (codeString.startsWith("{") && codeString.endsWith("}")) {
			codeString = codeString.substring(1, codeString.length() - 1);
		}
		return codeString;
	}

	public ConnectionPeripheralAccess getConnectionPeripheralAccess() {
		return access;
	}

	public EvaluationHardwareAccess getEvaluationHardwareAccess() {
		return evalAccess;
	}

	public ConnectionPeripheralWait getConnectionPeripheralWait() {
		return connWait;
	}

	@Override
	public String toString() {
		return "Body  [" + getType().toString() + "]";
	}

	/**
	 * The different types a Body can be
	 */
	public enum BodyType {
		CODE("CODE"),
		CONNECTION_PERIPHERAL_ACCESS("CONNECTION_PERIPHERAL_ACCESS"),
		EVALUATION_ACCESS("EVALUATION_ACCESS"),
		MULTI_CONNECTION_PERIPHERAL_ACCESS("MULTI_CONNECTION_PERIPHERAL_ACCESS"),
		SYNC_ACCESS("SYNC_ACCESS"),
		CONNECTION_PERIPHERAL_WAIT("CONNECTION_PERIPHERAL_WAIT");

		private final String type;

		BodyType(String type) {
			this.type = type;
		}

		@Override
		public String toString() {
			return type;
		}
	}

}
