/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.software.freemarker;

import de.tu_darmstadt.rs.microStreams.model.software.drivers.PeripheralUsage;

/**
 * Tells where to wait for incoming data from another core or where to wait for free FIFO space when sending data.
 */
public class ConnectionPeripheralWait {
	private boolean isWaitReceive;
	private boolean isWaitSend;
	private String recvPeri;

	/**
	 * Creates a new instance of ConnectionPeripheralWait.
	 *
	 * @param type What to wait for. Can be PeripheralUsage.RECEIVE_WAIT or PeripheralUsage.SEND_WAIT.
	 * @param recvPeri Name of the peripheral. Used to generate register accesses.
	 */
	public ConnectionPeripheralWait(PeripheralUsage type, String recvPeri) {
		this.recvPeri = recvPeri;

		if (type == PeripheralUsage.RECEIVE_WAIT) {
			this.isWaitSend = false;
			this.isWaitReceive = true;
		} else if (type == PeripheralUsage.SEND_WAIT) {
			this.isWaitSend = true;
			this.isWaitReceive = false;
		}
	}

	/**
	 * give isWaitReceive
	 * @return true if this ConnectionPeripheralWait is a wait for receiving data.
	 */
	public boolean getIsWaitReceive() {
		return isWaitReceive;
	}

	/**
	 * give isWaitSend
	 * @return true if this ConnectionPeripheralWait is a wait for sending data.
	 */
	public boolean getIsWaitSend() {
		return isWaitSend;
	}

	public String getStatusReg() {
		return "&" + recvPeri + ".status";
	}

	public String getMsgSizeReg() {
		return "&" + recvPeri + ".msg_size";
	}
}
