/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.software.function;

import de.tu_darmstadt.rs.microStreams.model.interfaces.NeedsIncludes;
import de.tu_darmstadt.rs.microStreams.model.software.include.Include;

import cetus.hir.FunctionCall;

import java.util.Collection;
import java.util.Collections;

/**
 * A function call from the code. This tells what the function is and where it was declared
 */
public class UFunctionCall implements NeedsIncludes {

	private UFunction function;

	private Include include;

	private FunctionCall cetusFunctionCall;

	private boolean isExtern;

	/**
	 * The constructor
	 * @param function A Function that is called from this function call
	 * @param cetusFunctionCall the according cetus function call
	 */
	public UFunctionCall(UFunction function, FunctionCall cetusFunctionCall) {
		isExtern = false;
		this.function = function;
		this.cetusFunctionCall = cetusFunctionCall;
	}

	/**
	 * the constructor
	 * @param include an include file
	 */
	public UFunctionCall(Include include) {
		isExtern = true;
		this.include = include;
		if (null == this.include) {
			throw new IllegalArgumentException("Value include cannot be null");
		}
	}

	public UFunction getFunction() {
		return function;
	}

	@Override
	public Collection<Include> getIncludeFiles() {
		if (!isExtern) {
			return Collections.emptySet();
		} else {
			if (null == include) {
				return null;
			} else {
				return Collections.singleton(include);
			}

		}
	}

	public FunctionCall getCetusFunctionCall() {
		return cetusFunctionCall;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
			+ ((function == null)? 0 : function.hashCode());
		result = prime * result + ((include == null)? 0 : include.hashCode());
		result = prime * result + (isExtern? 1231 : 1237);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		UFunctionCall other = (UFunctionCall) obj;
		if (function == null) {
			if (other.function != null) {
				return false;
			}
		} else if (!function.equals(other.function)) {
			return false;
		}
		if (include == null) {
			if (other.include != null) {
				return false;
			}
		} else if (!include.equals(other.include)) {
			return false;
		}
		if (isExtern != other.isExtern) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "UFunctionCall [function=" + function + ", include=" + include
			+ ", isExtern=" + isExtern + "]";
	}

	public boolean isExtern() {
		return isExtern;
	}


}
