/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.model.software.task.pragma;

import de.tu_darmstadt.rs.microStreams.model.Model;

import cetus.hir.PragmaAnnotation;
import cetus.hir.Traversable;

import java.util.Collection;

/**
 * A merker where in the code the pragmas scope ends
 */
public class EndPragma implements Pragma {

	private PragmaAnnotation pa;
	private Collection<PragmaTag<String>> ppList;
	private Traversable traversable;
	private Model model;

	private EndPragma() {
	}

	/**
	 * The constructor for an end prama
	 * @param pa the pragma annotation
	 * @param ppList the optional pragma tags
	 * @param traversable the traversable of this pragma
	 */
	public EndPragma(PragmaAnnotation pa, Collection<PragmaTag<String>> ppList, Traversable traversable) {
		this.pa = pa;
		this.ppList = ppList;
		this.traversable = traversable;
	}

	public Traversable getTraversable() {
		return traversable;
	}

	@Override
	public void addPragmaParameter(PragmaTag<String> pp) {
		ppList.add(pp);
	}

	@Override
	public PragmaTag<String> getPragmaTag(String ppName) {
		for (PragmaTag<String> pt : ppList) {
			if (ppName.equals(pt.getName())) {
				return pt;
			}
		}
		return null;
	}

	@Override
	public Model getModel() {
		return model;
	}

	@Override
	public void setModel(Model m) {
		model = m;
	}
}
