/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.tools;

import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.util.FileFinder;

import org.slf4j.Logger;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.stream.Stream;

/**
 * This class provides some utilities for C code.
 *
 * @author Jan Weber
 */
public class CUtils {

	private static Logger log = org.slf4j.LoggerFactory.getLogger(CUtils.class);


	public CUtils() {}


	/**
	 * Reformats the given folder by utilizing GNU indent. The function call
	 * requires that GNU indent is installed on the system this code is
	 * executed.
	 *
	 * @param folder - the folder that should be reformatted
	 */
	public static void reformatCodeFolder(Path folder) {
		if (folder.toFile().isDirectory() && folder.toFile().exists()) {
			try {
				Stream<Path> files = FileFinder.find(folder, Constants.FilesAndFolders.C_FILES_EXT_PATTERN);
				files.forEach(f -> reformatCode(f.toFile()));
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Reformats the given c file by utilizing GNU indent. The function call
	 * requires that GNU indent is installed on the system this code is
	 * executed.
	 *
	 * @param file - the file that should be reformatted
	 */
	public static void reformatCode(File file) {

		Runtime rt = Runtime.getRuntime();
		Process proc = null;
		try {
			proc = rt.exec("indent -h");
			proc.waitFor();
		} catch (IOException e) {
			log.warn("Program indent not found. Output files might not look pretty");
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		boolean indentExists = false;
		if (proc != null) {
			int exitValue = proc.exitValue();
			indentExists = (exitValue == 1);
		}


		if (indentExists) {
			File tempFile = new File(file.getParent(), "indent_out");

			String[] command = Constants.Indent.getIndentCommand(file,tempFile);

			Process p = null;

			try {
				p = Runtime.getRuntime().exec(command);
				p.waitFor();
				file.delete();
				tempFile.renameTo(file);
			} catch (Exception e) {
				log.error("Indent could not be executed!", e);
				return;
			} finally {
				tempFile.delete();
			}
			if (p.exitValue() != 0) {
				log.warn("The execution of indent was not successfull with the command: "
					+ Arrays.toString(command));
				log.warn("Output: " + Utils.readStreamToString(p.getInputStream()));
				log.warn("Error: " + Utils.readStreamToString(p.getErrorStream()));

			}
		}

	}
}
