/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.tools.templates;


import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.util.FileUtil;

/**
 * A Freemarker template File
 */
public class TemplateFile {

	private String name;
	private String location;
	private String encoding;

	/**
	 * The Constructor
	 * @param name the filename
	 * @param location location where the file is located e.g. RESSOURCES if inside jar
	 * @param encoding the used file encoding
	 */
	public TemplateFile(String name, String location, String encoding) {
		this.name = name;
		this.location = location;
		this.encoding = encoding;
	}

	public TemplateFile(String name, String location) {
		this(name, location, FileUtil.DEFAULT_FILE_ENCODING);
	}

	public TemplateFile(String name) {
		this(name, "RESSOURCES", FileUtil.DEFAULT_FILE_ENCODING);
	}

	/**
	 * template location
	 * @return where the template is located e.g. RESSOURCES if inside jar
	 */
	public String getLocation() {
		return location;
	}

	/**
	 * the filename
	 * @return The filename of the tempate
	 */
	public String getName() {
		return name;
	}

	public Boolean isEvaluationComponent() {
		return name.startsWith("evaluation");
	}

	public boolean isGlobalVariablesHeader() {
		return getName().equals(Constants.Freemarker.GLOBAL_VARIABLES_HEADER);
	}

	public String toString() {
		return location + "/" + name;
	}
}
