/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.peripheralDetection;

import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.PeripheralModuleDescription;
import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.hardware.core.Core;

import java.util.Set;

/**
 * Manager for the SoCPeripherals
 */
final class SoCPeripheralsManager implements SoCPeripheralsInterface {

	private boolean preRunMatch;
	private SoCPeripherals preRunUsedPeripherals;
	private SoCPeripherals usedPeripherals;
	private UninferableConstants uninferableConstants;
	private Model model;

	public SoCPeripheralsManager(Model model, UninferableConstants uninferableConstants) {
		this.uninferableConstants = uninferableConstants;
		this.model = model;
		reset();
	}

	public void preRunDone() {
		preRunMatch = false;
	}

	public void preActivate() {
		preRunMatch = true;
	}

	public boolean isInPreRun() {
		return preRunMatch;
	}

	private SoCPeripherals getCurrentlyActivePeripherals() {
		if (isInPreRun()) {
			return preRunUsedPeripherals;
		} else {
			return usedPeripherals;
		}
	}

	@Override
	public void addAllPeriesToCore(Core core) {
		usedPeripherals.addAllPeriesToCore(core);
	}

	@Override
	public boolean containsInterrupt(Set<PeripheralModuleDescription> interruptPeripherals) {
		return false;
	}

	@Override
	public String getNameOf(PeripheralModuleDescription peripheralModuleDescription) {
		return getCurrentlyActivePeripherals().getNameOf(peripheralModuleDescription);
	}

	@Override
	public boolean hasPeripheralAndName(PeripheralModuleDescription pmd, String name) {
		return getCurrentlyActivePeripherals().hasPeripheralAndName(pmd, name);
	}

	@Override
	public PeripheralModuleDescription getPeripheralWithName(String name) {
		return getCurrentlyActivePeripherals().getPeripheralWithName(name);
	}

	@Override
	public void add(PeripheralModuleDescription pmd, String name) {
		getCurrentlyActivePeripherals().add(pmd, name);
	}

	@Override
	public void add(PeripheralModuleDescription pmd) {
		getCurrentlyActivePeripherals().add(pmd);
	}

	@Override
	public Set<PeripheralModuleDescription> getPeripherals() {
		return getCurrentlyActivePeripherals().getPeripherals();
	}

	@Override
	public void reset() {
		if (model == null || uninferableConstants == null) {
			throw new IllegalStateException("The model or the uninferable constants should never be null");
		}
		preRunMatch = true;
		preRunUsedPeripherals = new SoCPeripherals(model, uninferableConstants);
		usedPeripherals = new SoCPeripherals(model, uninferableConstants);
	}

}
