/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.peripheralDetection;

import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.hardware.core.Core;
import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.CommonPeripheral;
import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.Peripheral;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Verfies that there are no ressource conflicts for all SoCPeripherals
 */
final class Verifier {

	private static final Logger log = LoggerFactory.getLogger(Verifier.class);

	private Verifier() {

	}

	private static boolean verifyExclusivePeripheralUsage(List<Core> cores) {
		boolean exclusiveUsage = true;
		Map<CommonPeripheral, Integer> peripherals = new HashMap<>();
		for (Core c : cores) {
			for (Peripheral peripheral : c.getPeripherals()) {
				if (peripheral instanceof CommonPeripheral) {
					CommonPeripheral cp = (CommonPeripheral) peripheral;
					if (peripherals.containsKey(cp)) {
						log.warn("Duplicate Peripheral " + cp.getName() + " in Core " + c.getId() + " and " + peripherals.get(cp) + " detected!");
						exclusiveUsage = false;
					} else {
						peripherals.put(cp, Integer.valueOf(c.getId()));
					}
				}
			}
		}
		return exclusiveUsage;
	}

	/*private boolean verifyEachFunctionCallHasAccordingPeripheral(List<Core> cores) {
		for (Core core : cores) {
			Collection<Task> tasks = new HashSet<Task>(core.getTasks());
			for (Task task : tasks) {
				Traversable traversable = task.getTraversable()
				if (traversable == null) {
					Collection<cetus.hir.FunctionCall> functionCalls = FunctionCallgraph.getNestedFunctionCalls(traversable);
					for (FunctionCall functionCall : functionCalls) {
						findPeriToFunctionCall(functionCall);
						if (functionCall.getProcedure() != null) {
							Traversable tr = functionCall.getProcedure();
							findFunctionCalls(tr);
						}
					}
				}

			}
		}
	}*/

	/**
	 * Checking for errors and inconsistencies during Peripheral detection
	 *
	 * @param project The porject to verify
	 * @return true if all checks pass successfully, false otherwise
	 */
	public static boolean verifyProject(Model project, UninferableConstants uninferableConstants) {
		boolean pass = true;
		List<Core> cores = new ArrayList<>(project.getModelComponents(Core.class));

		Set<String> usedPeripherals = cores.stream().flatMap(core -> core.getPeripherals().stream()).map(Peripheral::getName).collect(Collectors.toSet());
		pass = pass && verifyExclusivePeripheralUsage(cores);
		pass = pass && uninferableConstants.allConstantsUsedAsPeripherals(usedPeripherals);
		//pass&=verifyEachFunctionCallHasAccordingPeripheral(cores);
		return pass;
	}

}
