/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.transformationPasses.input;

import de.tu_darmstadt.rs.jcommander.JCommanderRunner;
import de.tu_darmstadt.rs.microStreams.config.Configuration;
import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.microStreams.config.IConfiguration;
import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.ModelComponent;
import de.tu_darmstadt.rs.microStreams.model.other.ProgramArguments;
import de.tu_darmstadt.rs.microStreams.transformationPasses.AbstractTransformationPass;
import de.tu_darmstadt.rs.util.StreamUtil;

import com.beust.jcommander.ParametersDelegate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.List;

/**
 * Parses the arguments from the command line and creates a config file
 */
public class ArgumentParser extends AbstractTransformationPass implements Runnable {

	private static final Logger log = LoggerFactory.getLogger(ArgumentParser.class);
	protected String programName;
	protected Model model;
	@ParametersDelegate
	IConfiguration config;

	public ArgumentParser() {
		programName = Constants.MICROSTREAMS;
	}

	@Override
	public List<Class<? extends ModelComponent>> requiredModelComponents() {
		return Arrays.asList(
				ProgramArguments.class,
				IConfiguration.class
		);
	}

	@Override
	public List<Class<? extends ModelComponent>> providedModelComponents() {
		return Arrays.asList(
				IConfiguration.class
		);
	}

	@Override
	public boolean process(Model m) {
		model = m;
		ProgramArguments programArguments = m.getModelComponents(ProgramArguments.class).stream().collect(StreamUtil.exactlyOne());
		this.config = m.getModelComponent(IConfiguration.class);

		JCommanderRunner.create(this)
				.setProgramName(config.getProgramName())
				.run(programArguments.getArguments());

		m.removeModelComponent(programArguments);

		return true;
	}

	/**
	 * This should be overwritten for any inheriting class giving more arguments
	 *
	 * @return the object to be parsed by the cmd line arg parser
	 */
	protected Object getParseable() {
		return model.getModelComponents(Configuration.class).stream().collect(StreamUtil.exactlyOne());
	}

	@Override
	public void run() {

	}
}
