/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.transformationPasses.output.graph;

import de.tu_darmstadt.rs.microStreams.model.software.task.Task;
import de.tu_darmstadt.rs.microStreams.model.software.task.TaskDependency;

import edu.uci.ics.jung.algorithms.layout.GraphElementAccessor;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.AbstractPopupGraphMousePlugin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Point2D;
import javax.swing.AbstractAction;
import javax.swing.JPopupMenu;

/**
 * a GraphMousePlugin that offers popup
 * menu support
 */
public class PopupGraphMousePlugin extends AbstractPopupGraphMousePlugin implements MouseListener {

	Logger log = LoggerFactory.getLogger(PopupGraphMousePlugin.class);

	public PopupGraphMousePlugin() {
		this(MouseEvent.BUTTON3_MASK);
	}

	public PopupGraphMousePlugin(int modifiers) {
		super(modifiers);
	}

	/**
	 * If this event is over a station (vertex), pop up a menu to
	 * allow the user to perform a few actions; else, pop up a menu over the layout/canvas
	 *
	 * @param e mouse event
	 */
	public void handlePopup(MouseEvent e) {
		final VisualizationViewer<Task, TaskDependency> vv = (VisualizationViewer<Task, TaskDependency>) e.getSource();
		final Point2D ivp = e.getPoint();

		GraphElementAccessor<Task, TaskDependency> pickSupport = vv.getPickSupport();
		JPopupMenu popup = new JPopupMenu();
		if (pickSupport != null) {

			final Task task = pickSupport.getVertex(vv.getGraphLayout(), ivp.getX(), ivp.getY());
			final TaskDependency taskDependency = pickSupport.getEdge(vv.getGraphLayout(), ivp.getX(), ivp.getY());

			if (task != null) {

				popup.add(new AbstractAction("Show task traversable") {

					public void actionPerformed(ActionEvent e) {
						/*JFrame frame = new JFrame(task.toString() + " traversable");
						frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);


						JTextPane codePane = new JTextPane();
						codePane.setEditable(false);
						codePane.setText(task.getTraversable().toString());
						StyledDocument styledDocument = codePane.getStyledDocument();
						JScrollPane scrollPane = new JScrollPane(codePane, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
						scrollPane.setPreferredSize(new Dimension(450, 300));
						frame.getContentPane().add(scrollPane, BorderLayout.CENTER);
						frame.pack();
						frame.setVisible(true);*/
						MicroStreamsDrawer.drawInnerFlow(task);
					}
				});

				popup.add(new AbstractAction("Show used Variables") {

					public void actionPerformed(ActionEvent e) {

						log.info("used vars");
					}
				});

			} else if (taskDependency != null) {
				popup.add(new AbstractAction("Connection") {

					public void actionPerformed(ActionEvent e) {
						log.info("connection");
					}
				});
			} else {
				return;
			}
			if (popup.getComponentCount() > 0) {
				popup.show(vv, e.getX(), e.getY());
			}
		} else { //to pop-up over the canvas/layout rather than over the station

			popup.add(new AbstractAction("????") {
				public void actionPerformed(ActionEvent e) {
					//do something here
				}
			});


			if (popup.getComponentCount() > 0) {
				popup.show(vv, e.getX(), e.getY());
			}
		}

	}
}
