/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel;

import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.ModelComponent;
import de.tu_darmstadt.rs.microStreams.model.software.task.Task;
import de.tu_darmstadt.rs.microStreams.model.software.task.pragma.PragmaTag;
import de.tu_darmstadt.rs.microStreams.model.software.task.pragma.TaskPragma;
import de.tu_darmstadt.rs.microStreams.transformationPasses.AbstractTransformationPass;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

/**
 * Replicates tasks
 */
public class Replicate extends AbstractTransformationPass {

	/* slf4j logger */
	private static final Logger log = LoggerFactory.getLogger(Replicate.class);

	private Model model;


	public Replicate() {
	}

	@Override
	public List<Class<? extends ModelComponent>> requiredModelComponents() {
		return Arrays.asList(
				Task.class
		);
	}

	@Override
	public List<Class<? extends ModelComponent>> providedModelComponents() {
		return Arrays.asList(
				Task.class
		);
	}

	@Override
	public boolean process(Model m) {

		this.model = m;

		Collection<Task> tasks = m.getModelComponents(Task.class);

		for (Task task : tasks) {

			int replicate = needsReplica(task);

			if (-1 == replicate) {
				return false;
			}

			if (replicate > 0) {
				while (0 < --replicate) {
					Task clone = task.replicate();
					m.addModelComponent(clone);
				}
			}

		}

		return true;
	}

	private int needsReplica(Task task) {
		if (task.isBaseTask()) {
			return 0;
		}

		TaskPragma pragma = task.getTaskPragma();
		PragmaTag<String> tag = pragma.getPragmaTag(Constants.Pragma.REPLICATE);
		if (null != tag) {
			Collection<String> entries = tag.getEntryNames();
			if (null != entries && !entries.isEmpty()) {
				if (entries.size() > 1) {
					model.printIgnoreableError(log,"A replicate tag has more than one entry");
					return -1;
				}

				String entry = entries.iterator().next();

				return Integer.parseInt(entry);


			} else {
				model.printIgnoreableError(log,"A replicate tag has no entries.");
				return -1;
			}
		}
		return 0;
	}
}
