/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.microStreams.transformationPasses.swmodel;

import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.software.variable.Variable;
import de.tu_darmstadt.rs.microStreams.transformationPasses.AbstractTransformationPass;

import cetus.hir.ArraySpecifier;
import cetus.hir.Expression;
import cetus.hir.IntegerLiteral;
import cetus.hir.PointerSpecifier;
import cetus.hir.Specifier;
import cetus.hir.Symbol;
import cetus.hir.UserSpecifier;
import org.slf4j.Logger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * "Deprecated" - using sizeof in C template for the moment instead.
 *
 * @author jan
 */
@Deprecated
public class TaskVariableSize extends AbstractTransformationPass {

	private static final int UNKNOWN_SIZE = -1;
	private static final int CHAR_SIZE = 1;
	private static final int SHORT_SIZE = UNKNOWN_SIZE;
	private static final int INT_SIZE = 2;
	private static final int LONG_SIZE = 4;
	private static final int LONG_LONG_SIZE = 8;
	private static final String UNKOWN_PRIMITIVE_SIZE = "Unkown primitive size.";
	private static Logger log = org.slf4j.LoggerFactory.getLogger(TaskVariableSize.class);
	private Model model;


	public TaskVariableSize() {
	}

	private int determineSymbolSize(Symbol symbol) {
		int size = 0;

		@SuppressWarnings("rawtypes")
		List typeSpecifiers = symbol.getTypeSpecifiers();

		if (!containsUserSpecifier(typeSpecifiers)
				&& !containsPointerSpecifier(typeSpecifiers)) {
			size = primitiveSize(getCTypes(typeSpecifiers));
		} else {
			size = userdefinedSize(typeSpecifiers);
		}

		List arraySpec = symbol.getArraySpecifiers();

		for (Object obj : symbol.getArraySpecifiers()) {
			if (obj instanceof ArraySpecifier) {
				ArraySpecifier as = (ArraySpecifier) obj;
				int dimension = as.getNumDimensions();
				while (--dimension >= 0) {
					Expression ex = as.getDimension(dimension);
					if (null == ex) {
						// could not determine
					} else if (ex instanceof IntegerLiteral) {
						size += ((IntegerLiteral) ex).getValue();
					} else {
						// Could not determine
					}
				}
			}
		}

		return size;

	}

	private int userdefinedSize(List typeSpecifiers) {
		// TODO Auto-generated method stub
		return 0;
	}

	@SuppressWarnings({"checkstyle:MagicNumber"})
	private int primitiveSize(List<Specifier> s) {
		switch (s.size()) {
			case 1:
				if (s.get(0).equals(Specifier.CHAR)) {
					return CHAR_SIZE;
				} else if (s.get(0).equals(Specifier.INT)) {
					return INT_SIZE;
				} else if (s.get(0).equals(Specifier.UNSIGNED)) {
					return INT_SIZE;
				} else if (s.get(0).equals(Specifier.SHORT)) {
					log.warn(UNKOWN_PRIMITIVE_SIZE);
					return SHORT_SIZE;
				} else if (s.get(0).equals(Specifier.LONG)) {
					return LONG_SIZE;
				} else if (s.get(0).equals(Specifier.FLOAT)) {
					log.warn(UNKOWN_PRIMITIVE_SIZE);
					return UNKNOWN_SIZE;
				} else if (s.get(0).equals(Specifier.DOUBLE)) {
					log.warn(UNKOWN_PRIMITIVE_SIZE);
					return UNKNOWN_SIZE;
				}

				model.printIgnoreableError(log,UNKOWN_PRIMITIVE_SIZE);
				return UNKNOWN_SIZE;

			case 2:
				if (s.get(0).equals(Specifier.SIGNED)
						|| s.get(0).equals(Specifier.UNSIGNED)) {
					if (s.get(1).equals(Specifier.CHAR)) {
						return CHAR_SIZE;
					} else if (s.get(1).equals(Specifier.SHORT)) {
						log.warn(UNKOWN_PRIMITIVE_SIZE);
						return SHORT_SIZE;
					} else if (s.get(1).equals(Specifier.INT)) {
						return INT_SIZE;
					} else if (s.get(1).equals(Specifier.LONG)) {
						return LONG_SIZE;
					}

				} else if (s.get(0).equals(Specifier.SHORT)) {
					if (s.get(1).equals(Specifier.INT)) {
						log.warn(UNKOWN_PRIMITIVE_SIZE);
						return SHORT_SIZE;
					} else {
						log.warn(UNKOWN_PRIMITIVE_SIZE);
						return UNKNOWN_SIZE;
					}

				} else if (s.get(0).equals(Specifier.LONG)) {
					if (s.get(1).equals(Specifier.INT)) {
						return LONG_SIZE;
					} else if (s.get(1).equals(Specifier.LONG)) {
						return LONG_LONG_SIZE;
					}
				}

				model.printIgnoreableError(log,UNKOWN_PRIMITIVE_SIZE);
				return UNKNOWN_SIZE;

			case 3:
				if (s.get(0).equals(Specifier.SIGNED)
						|| s.get(0).equals(Specifier.UNSIGNED)) {
					if (s.get(1).equals(Specifier.SHORT)) {
						if (s.get(2).equals(Specifier.INT)) {
							log.warn(UNKOWN_PRIMITIVE_SIZE);
							return SHORT_SIZE;
						}
					} else if (s.get(1).equals(Specifier.LONG)) {
						if (s.get(2).equals(Specifier.INT)) {
							return LONG_SIZE;
						} else if (s.get(2).equals(Specifier.LONG)) {
							return LONG_LONG_SIZE;
						}
					}
				} else if (s.get(0).equals(Specifier.LONG)) {
					if (s.get(1).equals(Specifier.LONG)) {
						if (s.get(2).equals(Specifier.INT)) {
							return LONG_LONG_SIZE;
						}
					}
				}

				model.printIgnoreableError(log,UNKOWN_PRIMITIVE_SIZE);
				return UNKNOWN_SIZE;

			case 4:
				if (s.get(0).equals(Specifier.SIGNED)
						|| s.get(0).equals(Specifier.UNSIGNED)) {
					if (s.get(1).equals(Specifier.LONG)) {
						if (s.get(2).equals(Specifier.LONG)) {
							if (s.get(3).equals(Specifier.INT)) {
								return LONG_LONG_SIZE;
							}
						}
					}
				}
				model.printIgnoreableError(log,UNKOWN_PRIMITIVE_SIZE);
				return UNKNOWN_SIZE;

			default:
				model.printIgnoreableError(log,UNKOWN_PRIMITIVE_SIZE);
				return UNKNOWN_SIZE;
		}
	}

	private List<Specifier> getCTypes(@SuppressWarnings("rawtypes") List s) {
		List<Specifier> cTypes = new ArrayList<>();

		for (Object obj : s) {
			if (obj instanceof Specifier && ((Specifier) obj).isCType()) {
				cTypes.add((Specifier) obj);
			}
		}

		return cTypes;
	}

	private boolean containsUserSpecifier(
			@SuppressWarnings("rawtypes") List specifiers) {
		for (Object specifier : specifiers) {
			if (specifier instanceof UserSpecifier) {
				return true;
			}
		}
		return false;
	}

	private boolean containsPointerSpecifier(
			@SuppressWarnings("rawtypes") List specifiers) {
		for (Object specifier : specifiers) {
			if (specifier instanceof PointerSpecifier) {
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean init() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean processable(Model project) {
		return (null != project.getModelComponents(Variable.class));
	}

	@Override
	public boolean process(Model project) {

		this.model = project;
		Collection<Variable> variables = project.getModelComponents(Variable.class);

		for (Variable variable : variables) {
			int size = determineSymbolSize(variable.getVariableDeclarator());
			variable.setSize(size);
		}

		return false;
	}

}
