<#--

    _______________________________________________________________________________

     Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
     Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
     (http://www.rs.tu-darmstadt.de) All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
     are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. All advertising materials mentioning features or use of this software
        must display the following acknowledgement: "This product includes
        software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
        its contributors."

     4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
     BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
     FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
     LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    _______________________________________________________________________________

-->
<#include "../license.ftl">
/*****************************************************************************/
/** \file timercontrol.c
 ** Contains functions to control the evaluation timer.
 **
 ** History:
 **   - 2015-09-01 Version 1.0 Jan Weber: Base Version.
 **   - 2016-12-20 Version 2.0 Laurenz Kamp: Updated to use performance counter.
 **
 *****************************************************************************/
#ifndef __EVALUATION_H__
#define __EVALUATION_H__

#include <peripherals.h>
#include <globalvariables.h>
#include <stdint.h>

enum en_evalTarget {
	RUNTIME, RECEIVE, SEND, RECEIVE_WAIT, SEND_WAIT, NONE
};

struct evalResults {
	volatile uint18_t valid;
	volatile uint36_t results[5];
};

extern int __GLOBAL_MEM_HEAP_END;

typedef enum en_evalTarget en_evalTarget_t;
<#if !task.isBaseTask()>
/**
 ******************************************************************************
 ** \brief Initializes the evaluation module within Child Tasks.
 **
 ** \param  [in]     pstcUart              pointer to the hardware address of
 **                                        the uart that should be used for
 **                                        collecting evaluation results.
 ** \param  [in/out] pstcPortIn            pointer to the hardware address of
 **                                        the port in that is used to receive
 **                                        the exit signal from the BaseTask.            
 **
 ******************************************************************************/
extern void evaluation_initChild(void *resAddrBase);
<#else>
/**
 ******************************************************************************
 ** \brief Initializes the evaluation module within the Base Task.
 **
 ** \param  [in]     pstcUart              pointer to the hardware address of
 **                                        the uart that should be used for
 **                                        collecting evaluation results.
 ** \param  [in]     pstcBaseUart          pointer to the hardware address of
 **										   the timer that should be used
 **										   for PC communication.
 ** \param  [in]     pstcBasePortOut       pointer to the hardware address of
 **                                        the port out that should be used for
 **                                        stopping the evaluation run.
 **
 ******************************************************************************/
<#if task.model.getConfig().isCreateSimulatedTarget()>
extern void evaluation_initBase(void *resAddrBase);
<#else>
extern void evaluation_initBase(void *resAddrBase);
</#if>
</#if>
/**
 ******************************************************************************
 ** \brief Starts the measurement of the given target.
 **
 ** \param  [in]     enTarget              measurement target.
 **
 ******************************************************************************/
extern void evaluation_startMeasurement(en_evalTarget_t enTarget);

/**
 ******************************************************************************
 ** \brief Stops the running measurement and accumulates for the final result.
 **
 ******************************************************************************/
extern void evaluation_stopMeasurement();

/**
 ******************************************************************************
 ** \brief Sends the results of the evaluation to the core hosting the base task.
 ** 	   The core hosting the base task receives the evaluation results with
 ** 	   this procedure and sends them to a PC terminal.
 **
 ******************************************************************************/
extern void evaluation_sendResults();

#endif				/* __EVALUATION_H__ */