<#--

    _______________________________________________________________________________

     Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
     Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
     (http://www.rs.tu-darmstadt.de) All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
     are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. All advertising materials mentioning features or use of this software
        must display the following acknowledgement: "This product includes
        software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
        its contributors."

     4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
     BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
     FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
     LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    _______________________________________________________________________________

-->
<#--
  -- This µStreams Freemarker Template is designed for structures with one
  -- task per core
  -->
<#if 1 < tasks?size >
	<#stop "To many tasks in this core. This template is designed only for one task per core.  ">
</#if>
<#include "../license.ftl">
/*****************************************************************************/
/** \file timercontrol.c
 ** Contains functions to control the evaluation timer.
 **
 ** History:
 **   - 2015-09-01 Version 1.0 Jan Weber: Base Version.
 **
 *****************************************************************************/
/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/
#include "base_types.h"
#include <subsystems/spartanmc_${core_id}/peripherals.h>
#include <timer.h>

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/

/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
#define CORE_ID ${core_id}
#define TASK_ID ${task.ID}
#define IS_BASETASK ${task.isBaseTask()?c}

/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/

/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/


/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/

/*****************************************************************************/
/* Function implementation - global ('extern') and local ('static')          */
/*****************************************************************************/

/**
 ******************************************************************************
 ** \brief Initializes the evaluation timer.
 **
 ** \param  [in/out] pstcTimer             pointer to the hardware address of
 **	                                       the timer that should be initialized
 **	                                       for evaluation.
 **
 ******************************************************************************/
void timercontrol_init(timer_t* pstcTimer) {
	pstcTimer->control &= ~0x0001F;	//Clear all bits.
	pstcTimer->control |= 0x0000E;	//0b00011110 => Enable prescaler,
									// set prescaler to 111,
									// disable timer

	pstcTimer->limit = 0x1FFFF;     // Set highest possible limit.
}

/**
 ******************************************************************************
 ** \brief Starts the evaluation timer.
 **
 ** \param  [in/out] pstcTimer             pointer to the hardware address of
 **	                                       the timer that should be started
 **	                                       for evaluation.
 **
 ******************************************************************************/
void inline timercontrol_restart(timer_t* pstcTimer) {
	pstcTimer->value =    0x00000;      // set counter to 0.
	pstcTimer->control |= 0x00001;		// enable timer timer.
}

/**
 ******************************************************************************
 ** \brief Stops the evaluation timer and returns the measurement result.
 **
 ** \param  [in/out] pstcTimer             pointer to the hardware address of
 **	                                       the timer that should be started
 **	                                       for evaluation.
 **
 ** \return unsigned int - last value of the timer.
 ******************************************************************************/
unsigned int inline timercontrol_stop(timer_t* pstcTimer) {
	pstcTimer->control &= ~0x00001;		// disable timer timer again.
	return pstcTimer->value;
}


