<#--

    _______________________________________________________________________________

     Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
     Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
     (http://www.rs.tu-darmstadt.de) All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
     are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. All advertising materials mentioning features or use of this software
        must display the following acknowledgement: "This product includes
        software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
        its contributors."

     4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
     BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
     FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
     LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    _______________________________________________________________________________

-->
<#--
  -- This µStreams Freemarker Template is designed for structures with one
  -- task per core
  -->
<#if 1 < tasks?size >
	<#stop "To many tasks in this core. This template is designed only for one task per core.  ">
</#if>
<#include "../license.ftl">
/*****************************************************************************/
/** \file main.c
 ** Main component of the µStream Application.
 **
 ** History:
 **   - 2015-05-04 Version 1.0 Jan Weber: Base Version.
 **
 *****************************************************************************/
/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/

#include "base_types.h"
<#-- include all neccessary files, e.g. for connections.  -->
<#list include_files as include>
	${include.print()}
</#list>
<#if hasGlobalMemory && configAllowGlobalMemory>
	#include "globalvariabledefinitions.h"
</#if>

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/

/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/

#define CORE_ID ${core_id}
#define TASK_ID ${task.ID}
#define IS_BASETASK ${task.isBaseTask()?c}

/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/

<#list structs as struct>
	${struct.toString()}
</#list>
/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/

<#list task.getUsedGlobalVariables() as variable>
${variable.getDeclarationWithAssignment()};
</#list>

/*****************************************************************************/
/* Local variables without initialization definitions ('static')             */
/*****************************************************************************/


/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/

<#list task.localFunctions as function>
	<#if function.isDefinedInMain() >
		${function.declaration};
	</#if>
</#list>

/*****************************************************************************/
/* Function implementation - global ('extern') and local ('static')          */
/*****************************************************************************/

<#if containsISRs>
	${printInterruptServiceRoutines}
</#if>
		<#list task.localFunctions as function>
			<#if function.isDefinedInMain() >
				<#list function.body as body>
					<#if body.type == BodyType.CODE>
					${body.code.print()}
					<#elseif body.type == BodyType.CONNECTION_PERIPHERAL_ACCESS>
						<#if body.connectionPeripheralAccess.type == PeripheralUsage.RECEIVE>
							<#if body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CORE_CONNECTOR_SLAVE>
								<#include "../connection_templates/core_connector_receive_api.ftl">
							<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.MEM_SWAP_SLAVE>
								<#include "../connection_templates/mem_swap_send_receive_api.ftl">
							<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.MEM_SWAP_MULTI_MASTER>
								<#include "../connection_templates/mem_swap_send_receive_api.ftl">
							<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CONCENTRATOR_MASTER>
								<#include "../connection_templates/concentrator_receive_api.ftl">
							<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.DISPATCHER_SLAVE>
								<#include "../connection_templates/dispatcher_receive_api.ftl">
							<#else>
								<#assign error = "No template found for: " + body.connectionPeripheralAccess.peripheral.type>
								<#stop error>
							</#if>
						<#elseif body.connectionPeripheralAccess.type == PeripheralUsage.SEND>
							<#if body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CORE_CONNECTOR_MASTER>
								<#include "../connection_templates/core_connector_send_api.ftl">
							<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.MEM_SWAP_DUAL_MASTER>
								<#include "../connection_templates/mem_swap_send_receive_api.ftl">
							<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.MEM_SWAP_MULTI_MASTER>
								<#include "../connection_templates/mem_swap_send_receive_api.ftl">
							<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CONCENTRATOR_SLAVE>
								<#include "../connection_templates/concentrator_send_api.ftl">
							<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.DISPATCHER_MASTER>
								<#include "../connection_templates/dispatcher_send_api.ftl">
							<#else>
								<#assign error = "No template found for: " + body.connectionPeripheralAccess.peripheral.type>
								<#stop error>
							</#if>
						</#if>
					<#elseif body.type == BodyType.MULTI_CONNECTION_PERIPHERAL_ACCESS>
						<#if body.multiAccess.type == PeripheralUsage.RECEIVE>
							<#if body.multiAccess.peripherals?first.type == JConfigTypes.CONCENTRATOR_MASTER>
								<#include "../connection_templates/concentrator_multireceive_api.ftl">
							<#elseif body.multiAccess.peripherals?first.type == JConfigTypes.CORE_CONNECTOR_SLAVE>
								<#include "../connection_templates/core_connector_multireceive_api.ftl">
							</#if>
						<#elseif body.multiAccess.type == PeripheralUsage.SEND>
							<#if body.multiAccess.peripherals?first.type == JConfigTypes.DISPATCHER_MASTER>
								<#include "../connection_templates/dispatcher_multisend_api.ftl">
							<#elseif body.multiAccess.peripherals?first.type == JConfigTypes.CORE_CONNECTOR_MASTER>
								<#include "../connection_templates/core_connector_multisend_api.ftl">
							<#else>
								<#assign error = "No template found for: " + body.multiAccess.peripherals?first.type>
								<#stop error>
							</#if>
						</#if>
					<#elseif body.type == BodyType.EVALUATION_ACCESS>
						<#if body.evaluationHardwareAccess.type == Measurement.START>
						evaluation_startMeasurement(${body.evaluationHardwareAccess.target});
						<#elseif body.evaluationHardwareAccess.type == Measurement.STOP>
						evaluation_stopMeasurement(${body.evaluationHardwareAccess.target});
						</#if>
					<#elseif body.type == BodyType.CONNECTION_PERIPHERAL_WAIT>
						<#if body.connectionPeripheralWait.isWaitReceive == true>
						receiver_wait_data(${body.connectionPeripheralWait.statusReg}, ${body.connectionPeripheralWait.msgSizeReg}, 1);
						<#elseif body.connectionPeripheralWait.isWaitSend == true>
						sender_wait_free(${body.connectionPeripheralWait.statusReg}, ${body.connectionPeripheralWait.msgSizeReg}, 1);
						</#if>
					</#if>
				</#list>
			</#if>
		</#list>



/*****************************************************************************/
/* Global variables definitions and initialisation                           */
/*****************************************************************************/

<#list task.core.peripherals as peripheral>
	<#if peripheral.hasStdioInclude() >
FILE *stdout = &${peripheral.getHwAddress()}_FILE;
FILE *stdin = &${peripheral.getHwAddress()}_FILE;

	</#if>
</#list>
<#list task.core.masterDMAConnection as peri>
	${peri.getStructDeclaration()}
	__attribute__((section("${peri.getSectionName()}"))) ${peri.getStructDefinition(true)}
</#list>
<#list task.core.slaveDMAConnection as peri>
	${peri.getStructDeclaration()}
	__attribute__((section("${peri.getSectionName()}"))) ${peri.getStructDefinition()}
</#list>

/*****************************************************************************/
/* main                                                                      */
/*****************************************************************************/

void main(){

	<#if isEvaluated>
		<#if task.isBaseTask()>
			evaluation_initBase(&__GLOBAL_MEM_HEAP_END);
		<#else>
			evaluation_initChild(&__GLOBAL_MEM_HEAP_END);
		</#if>
	</#if>

	<#if !task.isBaseTask()>
	while(TRUE){
	</#if>
		//local variables
		<#list task.getUsedLocalVariables() as variable>
			${variable.getDeclarationWithAssignment()};
		</#list>
		//local variables originally having initializers received from interconnect peripherals
		<#list task.getUsedVariablesFromPredecessors() as variable>
			<#if !variable.isGlobal()>
				${variable.getDeclaration() + ";"}
			</#if>
		</#list>
			<#list task.getBody() as body>
				<#if body.type == BodyType.CODE>
					${body.getCodeWithoutScope()}
				<#elseif body.type == BodyType.CONNECTION_PERIPHERAL_ACCESS>
					<#if body.connectionPeripheralAccess.type == PeripheralUsage.RECEIVE>
						<#if body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CORE_CONNECTOR_SLAVE>
							<#include "../connection_templates/core_connector_receive_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.MEM_SWAP_SLAVE>
							<#include "../connection_templates/mem_swap_send_receive_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.MEM_SWAP_MULTI_MASTER>
							<#include "../connection_templates/mem_swap_send_receive_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CONCENTRATOR_MASTER>
							<#include "../connection_templates/concentrator_receive_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.DISPATCHER_SLAVE>
							<#include "../connection_templates/dispatcher_receive_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CORE_CONNECTOR_SLAVE_DMA>
							<#include "../connection_templates/core_connector_receive_DMA_api.ftl">
						<#else>
							<#assign error = "No template found for: " + body.connectionPeripheralAccess.peripheral.type>
							<#stop error>
						</#if>
					<#elseif body.connectionPeripheralAccess.type == PeripheralUsage.SEND>
						<#if body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CORE_CONNECTOR_MASTER>
							<#include "../connection_templates/core_connector_send_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.MEM_SWAP_DUAL_MASTER>
							<#include "../connection_templates/mem_swap_send_receive_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.MEM_SWAP_SLAVE>
							<#include "../connection_templates/mem_swap_send_receive_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CONCENTRATOR_SLAVE>
							<#include "../connection_templates/concentrator_send_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.DISPATCHER_MASTER>
							<#include "../connection_templates/dispatcher_send_api.ftl">
						<#elseif body.connectionPeripheralAccess.peripheral.type == JConfigTypes.CORE_CONNECTOR_MASTER_DMA>
								<#include "../connection_templates/core_connector_send_DMA_api.ftl">
						<#else>
							<#assign error = "No template found for: " + body.connectionPeripheralAccess.peripheral.type>
							<#stop error>
						</#if>
					</#if>
				<#elseif body.type == BodyType.MULTI_CONNECTION_PERIPHERAL_ACCESS>
					<#if body.multiAccess.type == PeripheralUsage.RECEIVE>
						<#if body.multiAccess.peripherals?first.type == JConfigTypes.CONCENTRATOR_MASTER>
							<#include "../connection_templates/concentrator_multireceive_api.ftl">
						<#elseif body.multiAccess.peripherals?first.type == JConfigTypes.MEM_SWAP_MULTI_MASTER>
							<#include "../connection_templates/mem_swap_send_receive_api.ftl">
						<#elseif body.multiAccess.peripherals?first.type == JConfigTypes.CORE_CONNECTOR_SLAVE>
							<#include "../connection_templates/core_connector_multireceive_api.ftl">
						</#if>
					<#elseif body.multiAccess.type == PeripheralUsage.SEND>
						<#if body.multiAccess.peripherals?first.type == JConfigTypes.DISPATCHER_MASTER>
							<#include "../connection_templates/dispatcher_multisend_api.ftl">
						<#elseif body.multiAccess.peripherals?first.type == JConfigTypes.MEM_SWAP_MULTI_MASTER>
							<#include "../connection_templates/mem_swap_send_receive_api.ftl">
						<#elseif body.multiAccess.peripherals?first.type == JConfigTypes.CORE_CONNECTOR_MASTER>
							<#include "../connection_templates/core_connector_multisend_api.ftl">
						<#else>
							<#assign error = "No template found for: " + body.multiAccess.peripherals?first.type>
							<#stop error>
						</#if>
					</#if>
				<#elseif body.type == BodyType.SYNC_ACCESS>
                	<#include "../connection_templates/core_connector_sync_signal_api.ftl">
                	<#include "../connection_templates/core_connector_sync_wait_api.ftl">
                <#elseif body.type == BodyType.EVALUATION_ACCESS>
					<#if body.evaluationHardwareAccess.measurement == Measurement.START>
						evaluation_startMeasurement(${body.evaluationHardwareAccess.peripheralUsage});
					<#elseif body.evaluationHardwareAccess.measurement == Measurement.STOP>
						evaluation_stopMeasurement(${body.evaluationHardwareAccess.peripheralUsage});
					</#if>
				<#elseif body.type == BodyType.CONNECTION_PERIPHERAL_WAIT>
					<#if body.connectionPeripheralWait.isWaitReceive == true>
						receiver_wait_data(${body.connectionPeripheralWait.statusReg}, ${body.connectionPeripheralWait.msgSizeReg}, 1);
					<#elseif body.connectionPeripheralWait.isWaitSend == true>
						sender_wait_free(${body.connectionPeripheralWait.statusReg}, ${body.connectionPeripheralWait.msgSizeReg}, 1);
					</#if>
				</#if>
			</#list>
	<#if !task.isBaseTask()>
		<#if isEvaluated>
    	evaluation_sendResults();
    	</#if>
	}
	</#if>

	<#if isEvaluated && task.isBaseTask()>
		evaluation_sendResults();
	</#if>
}
