/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.microStreams.integration_test;

import de.tu_darmstadt.rs.microStreams.system_test.test_reference.TransformationPassCollectorRunner;
import de.tu_darmstadt.rs.microStreams.system_test.test_reference.MicroStreamsRunner;
import de.tu_darmstadt.rs.util.SpartanMCDefaultFolders;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashSet;
import java.util.Scanner;
import java.util.Set;
import java.util.stream.StreamSupport;

/**
 * Overwrites existing project files
 */
public class OverwriteVerifiedOutputs {

	protected OverwriteVerifiedOutputs() {

	}

	/**
	 * overrides project otuputs
	 *
	 * @param args the usual useless
	 */
	@SuppressWarnings("checkstyle:SystemOut")
	public static void main(String[] args) {
		OverwriteVerifiedOutputs overwriteVerifiedOutputs = new OverwriteVerifiedOutputs();
		overwriteVerifiedOutputs.overwrite();
	}

	/**
	 * overrides project outputs
	 */
	@SuppressWarnings("checkstyle:SystemOut")
	protected void overwrite() {
		final Path microStreamsIntegrationtestProjects = getTestFolder();
		final Path fwFolder = SpartanMCDefaultFolders.getIntegrationtestProjectsFWInput();
		final Path referenceProjects = SpartanMCDefaultFolders.getIntegrationtestProjectsReferenceProjects();


		try {
			System.out.println("Overwrite output for: ");
			int[] counter = {0};
			StreamSupport.stream(Files.newDirectoryStream(microStreamsIntegrationtestProjects).spliterator(), false)
					.filter(dir -> Files.exists(dir.resolve(fwFolder)))
					.filter(dir -> Files.exists(dir.resolve(referenceProjects)))
					.sorted()
					.forEach(path -> {
						System.out.println("[" + counter[0]++ + "] \t" + path.getFileName().toString());

					});
			System.out.printf("\nChoices (Comma separated or type all): ");
			Scanner scanner = new Scanner(System.in);
			Set<Integer> choices = new HashSet();
			while (scanner.hasNext()) {
				String next = scanner.next();
				if (next.equalsIgnoreCase("all")){
					for (int i = 0; i < counter[0]; i++) {
						choices.add(i);
					}
					break;
				}
				choices.add(Integer.valueOf(next.replaceAll("[^0-9]", "")));
				if (!next.contains(",")) {
					break;
				}
			}
			counter[0] = 0;
			StreamSupport.stream(Files.newDirectoryStream(microStreamsIntegrationtestProjects).spliterator(), false)
					.filter(dir -> Files.exists(dir.resolve(fwFolder)))
					.filter(dir -> Files.exists(dir.resolve(referenceProjects)))
					.sorted()
					.filter(path -> {
						if (choices.contains(Integer.valueOf(counter[0]++))) {
							return true;
						}
						return false;
					})
					.forEach(path -> {
						final Path targetRef = path.resolve(referenceProjects);

						boolean runFinished = getOutputGenerator(path, fwFolder, targetRef).run();
						if (runFinished) {
							System.out.println("Successfully overwritten " + targetRef);
						} else {
							System.err.print("Ooops Something went wrong!");
							System.exit(1);
						}
					});
		} catch (IOException e) {
			e.printStackTrace();
		}
		return;
	}

	protected TransformationPassCollectorRunner getOutputGenerator(Path testFolder, Path firmwareFolder, Path projectOutput) {
		return new MicroStreamsRunner(testFolder, testFolder.resolve(firmwareFolder).toString(), projectOutput);
	}

	protected Path getTestFolder() {
		return SpartanMCDefaultFolders.getMicroStreamsIntegrationtestProjects();
	}
}
