/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.microStreams.system_test;

import de.tu_darmstadt.rs.util.SpartanMCDefaultFolders;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Associations between constants and peripherals
 */
public final class InputExpOutputLib {

	public static final String UART = "uart";
	public static final String UART_LIGHT = UART + "_light";
	public static final String I2C_MASTER = "i2c_master";
	public static final String SPI_MASTER = "spi_master";
	public static final String CORE_CONNECTOR_MASTER_DMA = "core_connector_master_dma";
	public static final String CORE_CONNECTOR_SLAVE_DMA = "core_connector_slave_dma";
	private static final Path MAIN_IN_FW = Paths.get("src/main.c");
	private static final Path PERIPHERAL_DETECTION_FOLDER = SpartanMCDefaultFolders.getMicroStreamsIntegrationtestProjects().resolve("peripheral_detection");
	private static Path VARIABLE_ALIAS = PERIPHERAL_DETECTION_FOLDER.resolve("variable_alias").resolve(MAIN_IN_FW);
	private static List<List<String>> VARIABLE_ALIAS_EXPECTED_PERI = Arrays.asList(
			Arrays.asList(UART_LIGHT + "_0", UART_LIGHT),
			Arrays.asList(UART_LIGHT + "_1", UART_LIGHT),
			Arrays.asList(UART_LIGHT + "_2", UART_LIGHT),
			Arrays.asList(UART_LIGHT + "_3", UART_LIGHT)
	);
	private static Path CONSTANTS_DETECTION = PERIPHERAL_DETECTION_FOLDER.resolve("constants_detection").resolve(MAIN_IN_FW);
	private static List<List<String>> CONSTANTS_DETECTION_EXPECTED_PERI = Arrays.asList(
			Arrays.asList(UART_LIGHT + "_0", UART_LIGHT),
			Arrays.asList(UART_LIGHT + "_1", UART_LIGHT),
			Arrays.asList(UART_LIGHT + "_2", UART_LIGHT),
			Arrays.asList(UART + "_3", UART),
			Arrays.asList("myspi_masterdev", SPI_MASTER)
	);
	private static Path FUNCTION_CALL_DETECTION = PERIPHERAL_DETECTION_FOLDER.resolve("function_call_detection").resolve(MAIN_IN_FW);
	private static List<List<String>> FUNCTION_CALL_DETECTION_EXPECTED_PERI = Arrays.asList(
			Arrays.asList("iopointer", UART_LIGHT),
			Arrays.asList("usb11_0_dma", I2C_MASTER),
			Arrays.asList("ser_peri_if_0", SPI_MASTER),
			Arrays.asList("ser_peri_if_1", SPI_MASTER)
	);
	private static Path DIFFERENT_PERIPHERALS_WITH_SAME_INCLUDE = PERIPHERAL_DETECTION_FOLDER.resolve("different_peripherals_with_same_include").resolve(MAIN_IN_FW);
	private static List<List<String>> DIFFERENT_PERIPHERALS_WITH_SAME_INCLUDE_EXPECTED_PERI = Arrays.asList(
			Arrays.asList("ccm_0", CORE_CONNECTOR_MASTER_DMA),
			Arrays.asList("ccs_0", CORE_CONNECTOR_SLAVE_DMA)
	);

	private InputExpOutputLib() {
	}

	/**
	 * The IO mapping of the test files
	 *
	 * @return The IO mapping of the test files
	 */
	public static Map<Path, List<List<String>>> getIOMapping() {
		Map<Path, List<List<String>>> io = new HashMap<>();
		//io.put(CONSTANTS_DETECTION, CONSTANTS_DETECTION_EXPECTED_PERI);
		//io.put(VARIABLE_ALIAS, VARIABLE_ALIAS_EXPECTED_PERI);
		io.put(FUNCTION_CALL_DETECTION, FUNCTION_CALL_DETECTION_EXPECTED_PERI);
		io.put(DIFFERENT_PERIPHERALS_WITH_SAME_INCLUDE, DIFFERENT_PERIPHERALS_WITH_SAME_INCLUDE_EXPECTED_PERI);
		return io;
	}


}
