/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.microStreams.system_test.peripheral_detection;

import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.PeripheralModuleDescription;
import de.tu_darmstadt.rs.microStreams.system_test.InputExpOutputLib;
import de.tu_darmstadt.rs.microStreams.transformationPasses.hwmodel.peripheralDetection.PeripheralLibrary;

import cetus.hir.Tools;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

/**
 * Tests if Constants are associated with peripherals
 */
@RunWith(Parameterized.class)
public class ConstantsAssociationTester {

	@Parameterized.Parameter
	public String expectedPeripheralType;
	@Parameterized.Parameter(value = 1)
	public String testConstant;


	public ConstantsAssociationTester() {
	}

	/**
	 * Data collection method
	 * @return Data collection method
	 */
	@Parameterized.Parameters(name = "{index}: Constant \"{1}\"")
	public static List<Object[]> data() {
		return Arrays.asList(
				new Object[]{"usb11", "USB11"},
				new Object[]{InputExpOutputLib.SPI_MASTER, "SPI_MASTER"},
				new Object[]{null, "spi_master"},
				new Object[]{null, "SPI"},
				new Object[]{"can", "XCANX"},
				new Object[]{"core_connector_master", "A_CORE_CONNECTOR_MASTER_123"}
		);
	}

	@BeforeClass
	public static void beforeClass() {
		Tools.exitThrowsException(true);
	}

	/**
	 * The Test
	 */
	@Test
	public void test() {
		PeripheralLibrary peripheralLibrary = null;
		try {
			peripheralLibrary = new PeripheralLibrary(new LinkedHashSet<>());
		} catch (Exception e) {
			e.printStackTrace();
		}
		assertNotNull(peripheralLibrary);

		PeripheralModuleDescription pmd = peripheralLibrary.findPeriToConstant(testConstant);
		if (expectedPeripheralType == null) {
			assertNull("The constant " + testConstant + " should not be associated to" + pmd, pmd);
		} else {
			assertNotNull("No associated peripheral module found with string " + testConstant, pmd);
			assertEquals("Found Peripheral is not matching. Expecting: " + expectedPeripheralType + "and found: " + pmd.getName(), expectedPeripheralType, pmd.getName());
		}

	}
}
