/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.microStreams.system_test.peripheral_detection;

import de.tu_darmstadt.rs.microStreams.config.Constants;
import de.tu_darmstadt.rs.microStreams.model.Model;
import de.tu_darmstadt.rs.microStreams.model.hardware.HardwareModelComponent;
import de.tu_darmstadt.rs.microStreams.model.hardware.core.Core;
import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.CommonPeripheral;
import de.tu_darmstadt.rs.microStreams.model.hardware.peripherals.Peripheral;
import de.tu_darmstadt.rs.microStreams.system_test.InputExpOutputLib;
import de.tu_darmstadt.rs.microStreams.system_test.test_reference.MicroStreamsRunner;

import cetus.hir.Tools;
import org.junit.BeforeClass;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertTrue;

/**
 * Tests for finding peripherals in source code
 */
@RunWith(Parameterized.class)
public class GenericPeripheralTester {

	@Rule
	public TemporaryFolder project = new TemporaryFolder();
	@Parameterized.Parameter
	public Path testfile;
	@Parameterized.Parameter(value = 1)
	public List<List<String>> expPeripherals;
	@Parameterized.Parameter(value = 2)
	public String name;

	public GenericPeripheralTester() {
	}

	private static Collection<CommonPeripheral> data(List<List<String>> toGenerate, Model model) {
		Core core = model.getModelComponents(Core.class).stream().findFirst().get();
		Collection<CommonPeripheral> cp = new ArrayList<>();
		toGenerate.stream().forEach(list -> {
			String name = list.get(0);
			String type = list.get(1);
			CommonPeripheral peripheral = new CommonPeripheral(type, name, core, 0, model, false);
			cp.add(peripheral);
		});
		return cp;
	}

	/**
	 * The data generator
	 * @return The data generator
	 */
	@Parameterized.Parameters(name = "{index}: {2}")
	public static Collection<Object[]> data() {
		List<Object[]> testdata = new LinkedList<>();

		Map<Path, List<List<String>>> io = InputExpOutputLib.getIOMapping();

		io.forEach((testfile, expPeripherals) -> {
			String filename = testfile.getParent().getParent().getFileName().toString();
			testdata.add(new Object[]{testfile, expPeripherals, filename});
		});


		return testdata;
	}

	private static Collection<Peripheral> collectTestResults(Model modelToInitialize) {
		Collection<Peripheral> reality = modelToInitialize.getModelComponents(Core.class).stream()
				.findFirst().get().getPeripherals().stream()
				//filter since we can take initctrl and initctrlp
				.filter(p -> !p.getType().startsWith("intctrl"))
				.collect(Collectors.toList());
		return reality;
	}

	@BeforeClass
	public static void beforeClass() {
		Tools.exitThrowsException(true);
	}

	@Test
	public void test() {

		MicroStreamsRunner objectHolder = new MicroStreamsRunner(null, testfile.toString(), project.getRoot().toPath());
		boolean runFinished = objectHolder.run();
		assertTrue("Runnning " + Constants.MICROSTREAMS + " failed! ", runFinished);

		if (runFinished) {
			Model model = objectHolder.getModel();
			Collection<Peripheral> testResults = collectTestResults(model).stream().sorted(Comparator.comparing(HardwareModelComponent::getName)).collect(Collectors.toList());
			Collection<CommonPeripheral> expectedResults = data(expPeripherals, model).stream().sorted(Comparator.comparing(HardwareModelComponent::getName)).collect(Collectors.toList());

			assertArrayEquals(expectedResults.toArray(), testResults.toArray());
		}
	}
}
