/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.rapidSoc.design;

import edu.byu.ece.rapidSmith.design.Attribute;
import edu.byu.ece.rapidSmith.design.Design;
import edu.byu.ece.rapidSmith.design.Instance;
import edu.byu.ece.rapidSmith.design.Net;
import edu.byu.ece.rapidSmith.device.PrimitiveType;
import edu.byu.ece.rapidSmith.device.WireEnumerator;

import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collection;

/**
 * Compare two xdl files
 */
@SuppressWarnings({"checkstyle:SystemOut","checkstyle:MultipleStringLiterals"})
public final class Comparison {

	private Comparison() {

	}

	/**
	 * MAIN
	 * @param args Arguments
	 */
	public static void main(String[] args) {
		Design broken = new Design();
		Design working = new Design();

		broken.loadXDLFile(Paths.get(args[0]));
		working.loadXDLFile(Paths.get(args[1]));

		boolean ignoreStaticSourced = false;

		Collection<Net> nets = new ArrayList<>(broken.getNets());

		/*Collection<Net> newNets = new ArrayList<>();

		nets.forEach(net -> {
			if (net.getName().endsWith(".O"))
				net.setName(net.getName().substring(0,net.getName().length()-2));
			if (net.getName().endsWith(".T"))
				return;

			newNets.add(net);

		});

		broken.setNets(newNets);*/

		if (broken.getInstances().size() != working.getInstances().size()) {
			System.out.println("different number of instances");
		}

		broken.getInstances().forEach(inst -> {
			Instance workingInst = working.getInstance(inst.getName());
			if (inst.getType() != PrimitiveType.IOB) {
				return;
			}
			if (workingInst == null) {
				System.out.println("working inst for " + inst.getName() + " not found");
				return;
			}
			diffAttributes(inst, workingInst);
		});

		/*
		broken.getNets().stream().forEach(net -> {

            Net workingNet = working.getNet(net.getName());
            if (workingNet==null &&(!net.getName().contains("GLOBAL_LOGIC") || !ignoreStaticSourced))
                System.out.println("net "+net.getName()+" not contained in working");
        });
        working.getNets().stream().forEach(net -> {

            Net ourNet = broken.getNet(net.getName());
            if (ourNet==null &&(!net.getName().contains("GLOBAL_LOGIC") || !ignoreStaticSourced)) {
                System.out.println("net " + net.getName() + " not contained in broken");
                System.out.println(net.toString(working.getWireEnumerator()));
            }
        });

        broken.getNets().stream().forEach(net -> {
            Net workingNet = working.getNet(net.getName());
            if (workingNet!=null) {

                boolean differs = net.getPIPs().size()!=workingNet.getPIPs().size() || net.getPIPs().stream().anyMatch(pip -> (!workingNet.getPIPs().contains(pip)));

                if (!differs && net.getPIPs().size()==0) {
                    diffPins(net,workingNet);
                }
                if (differs) {
                    //if (net.getName().matches(".*pipe_2\\[2[01234].*")) {
                    /*if (net.getName().contains("clkfx") || net.getName().contains("sysclk_0/subsystem_0_sysclk_0_phase")) {
                        net.getPIPs().clear();
                        net.getPIPs().addAll(workingNet.getPIPs());
                        System.out.println("net " + net.getName() + " differs, changed it");
                        net.getPins().forEach(pin -> System.out.println("   "+pin.getInstanceName()+"."+pin.getName()));
                    } else * /{
                        System.out.println("net " + net.getName() + " differs");

                        diffPips(net,workingNet,broken.getWireEnumerator());
                        System.out.println();
                    }
                }
            }
        });*/
		//working.fixSp6IOBs();

		/*broken.setNets(UnrouteNets.combineStaticNets(broken.getNets()));


        //Route static sources again
        Pathfinder router = new Pathfinder(true, false, 1.125f);
        router.setDesign(broken);
        router.setDev(broken.getDevice());
        router.setWe(broken.getWireEnumerator());

        // route design
        router.routeDesign();*/

		broken.saveXDLFile(Paths.get(args[2]));
		//working.saveXDLFile(args[2]);
	}

	/**
	 * Print difference in attributes
	 * @param inst broken instance
	 * @param workingInst working instance
	 */
	public static void diffAttributes(Instance inst, Instance workingInst) {
		if (!inst.getPrimitiveSiteName().equals(workingInst.getPrimitiveSiteName())) {
			System.out.println("inst " + inst.getName() + " has differing primitive sites");
		}

		if (inst.getAttributes().size() != workingInst.getAttributes().size()) {
			System.out.println("different number of attributes in " + inst.getName());
		}

		inst.getAttributes().forEach(attr -> {
			Attribute workingAttr = workingInst.getAttribute(attr.getPhysicalName());
			if (workingAttr == null) {
				System.out.println("attr " + attr.getPhysicalName() + " not found in working.");
				return;
			}

			if (!attr.getLogicalName().equals(workingAttr.getLogicalName())) {
				System.out.println("different logical name in " + inst.getName() + "." + attr.getPhysicalName() + ": " + attr.getLogicalName() + " vs. " + workingAttr.getLogicalName());
			}


			if (!attr.getValue().equals(workingAttr.getValue())) {
				System.out.println("different value in " + inst.getName() + "." + attr.getPhysicalName() + ": " + attr.getValue() + " vs. " + workingAttr.getValue());
				if (inst.getName().contains("usb")) {
					System.out.println("changing");
					attr.setValue(workingAttr.getValue());
				}
			}
		});
	}

	private static void diffPins(Net net, Net workingNet) {
		net.getPins().forEach(pin -> {
			if (!workingNet.getPins().contains(pin)) {
				System.out.println("working " + workingNet.getName() + " does not have " + pin);
			}
		});
		workingNet.getPins().forEach(pin -> {
			if (!net.getPins().contains(pin)) {
				System.out.println("broken " + workingNet.getName() + " does not have " + pin);
			}
		});
		if (net.getSource() != null) {
			if (!net.getSource().equals(workingNet.getSource())) {
				System.out.println("source differs: " + net);
			}
		}
	}

	private static void diffPipsOneWay(Net a, Net b, WireEnumerator we, String message) {
		a.getPIPs().stream().filter(pip -> !b.getPIPs().contains(pip)).forEach(pip -> System.out.println(message + pip.toString(we)));
	}

	private static void diffPips(Net net, Net workingNet, WireEnumerator we) {

		diffPipsOneWay(net, workingNet, we, "working does not have ");
		diffPipsOneWay(workingNet, net, we, "broken does not have ");

	}

	private static void dumpPips(Net net, Design design) {
		net.getPIPs().stream().map(pip -> pip.toString(design.getWireEnumerator())).sorted().forEach(System.out::println);
	}

}
