/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.rapidSoc.design;

import de.tu_darmstadt.rs.util.StreamUtil;

import edu.byu.ece.rapidSmith.design.Design;
import edu.byu.ece.rapidSmith.design.Instance;
import edu.byu.ece.rapidSmith.design.Pin;
import edu.byu.ece.rapidSmith.device.PrimitiveType;
import edu.byu.ece.rapidSmith.primitiveDefs.PrimitiveDefList;
import edu.byu.ece.rapidSmith.router.RoutingElement;
import edu.byu.ece.rapidSmith.util.DeviceDatabaseProvider;

import java.nio.file.Paths;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * Compare clock stuff
 */
@SuppressWarnings("checkstyle:SystemOut")
public final class ComparisonClock {

	private ComparisonClock() {

	}

	/**
	 * Main
	 * @param args arguments
	 */
	public static void main(String[] args) {
		Design broken = new Design();
		Design working = new Design();

		broken.loadXDLFile(Paths.get(args[0]));
		working.loadXDLFile(Paths.get(args[1]));


		PrimitiveDefList primitives = DeviceDatabaseProvider.getDeviceDatabase().loadPrimitiveDefs(broken.getFamilyType());

		Instance clockInst = broken.getInstances().stream().filter(i -> i.getType() == PrimitiveType.IOB)
				.filter(i -> i.getName().contains("CLOCK"))
				.collect(StreamUtil.exactlyOne());

		Instance workingInst = working.getInstance("pin_" + clockInst.getName());

		System.out.println("Broken");
		System.out.println("======");
		dumpConnected(clockInst, primitives);
		System.out.println();
		System.out.println("Working");
		System.out.println("=======");
		dumpConnected(workingInst, primitives);

		broken.saveXDLFile(Paths.get(args[2]));
		//working.saveXDLFile(args[2]);
	}

	private static void dumpConnected(RoutingElement e, PrimitiveDefList primitives, int level) {

		Set<RoutingElement> next = e.getConnectedForward(primitives);
		next.forEach(n -> {
			System.out.println(spaceIt(level * 2) + n);
			if (isOfInterest(n, level)) {
				dumpConnected(n, primitives, level + 1);
			}
		});

	}

	private static void dumpConnected(Instance clockInst, PrimitiveDefList primitives) {
		Pin clockPin = clockInst.getPins().stream().collect(StreamUtil.exactlyOne());
		dumpConnected(clockPin, primitives, 0);
	}

	private static boolean isOfInterest(RoutingElement n, int level) {
		if (n.getInstance().getName().contains("reset")) {
			return false;
		}
		final int maxLevel = 15;
		return level < maxLevel;
	}

	private static String spaceIt(int space) {
		return IntStream.range(0, space).mapToObj(x -> " ").collect(Collectors.joining());
	}

}
