/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.design;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.OrderedConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IConnectorInstance;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.buses.IMappedBusPortInstance;

import java.util.stream.Stream;

import static de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection.ConnectionType.Port;

/**
 * Fake instance to use for glue module outputs
 */
public class FakeBusGlueOutputConnectorInstance implements IConnectorInstance {

	private final IMappedBusPortInstance bpi;
	private final Library library;
	private final IConfigurationDocument document;
	private final IModuleConfiguration<?> glue;

	/**
	 * Create FakeBusGlueOutputConnectorInstance
	 * @param bpi The busPortInstance
	 * @param library The library
	 * @param document The document
	 */
	public FakeBusGlueOutputConnectorInstance(IMappedBusPortInstance bpi, Library library, IConfigurationDocument document) {
		this.bpi = bpi;
		this.library = library;
		this.document = document;

		this.glue = BusGlueToModuleInstance.getOrgate(bpi, library, document);
	}

	@Override
	public IConfigurationElement<?, ?, ?> getSourceElement() {
		return glue;
	}

	@Override
	public String getPortName() {
		return "y";
	}

	@Override
	public ConnectorDirection getDirection() {
		return ConnectorDirection.Output;
	}

	@Override
	public Stream<? extends IConnection<?>> streamConnections() {
		return Stream.of(new OrderedConnection<IConnectorInstance>() {
			@Override
			public ConnectionType getType() {
				return Port;
			}

			@Override
			public IConnectorInstance getPortA() {
				return FakeBusGlueOutputConnectorInstance.this;
			}

			@Override
			public IConnectorInstance getPortB() {
				return bpi;
			}

			@Override
			public boolean isConnected() {
				return true;
			}

			@Override
			public void connect() {
				throw new UnsupportedOperationException();
			}

			@Override
			public void disconnect() {
				throw new UnsupportedOperationException();

			}

			@Override
			public boolean isValid(boolean report) {
				return true;
			}

			@Override
			public boolean isPartialA() {
				return false;
			}

			@Override
			public boolean isPartialB() {
				return false;
			}

			@Override
			public int getStartA() {
				return 0;
			}

			@Override
			public int getEndA() {
				return IConnection.FULL_CONNECTOR_WIDTH;
			}

			@Override
			public int getStartB() {
				return 0;
			}

			@Override
			public int getEndB() {
				return IConnection.FULL_CONNECTOR_WIDTH;
			}
		});
	}

	@Override
	public boolean isConnected() {
		return true;
	}

	@Override
	public IConnectorDescription.Type getType() {
		return bpi.getType();
	}

	@Override
	public int getConnectionCount() {
		return 1;
	}

	@Override
	public int getConnectorWidth() {
		return bpi.getConnectorWidth();
	}

	@Override
	public String toString() {
		return bpi.toString();
	}
}
