/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.design;

import de.tu_darmstadt.rs.rapidSoc.module.ModuleDescriptionInstance;
import de.tu_darmstadt.rs.rapidSoc.module.ModuleDescriptionPort;
import de.tu_darmstadt.rs.rapidSoc.module.ModuleDescriptionVariantPort;
import de.tu_darmstadt.rs.util.StreamUtil;

import edu.byu.ece.rapidSmith.device.Tile;

import java.util.HashSet;
import java.util.Set;
import java.util.stream.Stream;

/**
 * This class represents a group of ports later connected to a net
 * This is required because a true net cannot support  module descriptions or different variants of a module
 */
public class PortNet {

	private ModuleDescriptionInstance sourceModule;
	private String sourcePort;
	private Set<StreamUtil.Pair<ModuleDescriptionInstance, String>> sinks;

	/**
	 * construct a PortNet starting at the given port
	 * @param sourceModule the source module
	 * @param sourcePort the name of the source port
	 */
	public PortNet(ModuleDescriptionInstance sourceModule, String sourcePort) {
		this.sourceModule = sourceModule;
		this.sourcePort = sourcePort;
		this.sinks = new HashSet<>();
	}

	public void addSink(ModuleDescriptionInstance module, String port) {
		sinks.add(new StreamUtil.Pair<>(module, port));
	}

	public boolean hasSinks() {
		return !sinks.isEmpty();
	}

	public ModuleDescriptionInstance getSourceModule() {
		return sourceModule;
	}
	public ModuleDescriptionPort getSourcePort() {
		return sourceModule.getDescription().getPort(sourcePort);
	}

	public Set<StreamUtil.Pair<ModuleDescriptionInstance, String>> getSinks() {
		return sinks;
	}

	public Stream<ModuleDescriptionVariantPort> streamSinkPorts() {
		return sinks.stream().map(pair -> pair.getA().getPlacement().getPort(pair.getB()));
	}

	/**
	 * stream all tiles this {@link PortNet} reaches out to
	 * @return the stream
	 */
	public Stream<Tile> streamTiles() {
		return Stream.concat(
				sourceModule.getPlacement().streamPortTiles(sourcePort),
				sinks.stream()
						.filter(pair -> pair.getA().isPlaced())
						.flatMap(pair -> pair.getA().getPlacement().streamPortTiles(pair.getB()))
		);
	}

	public String toString() {
		return sourceModule+":"+sourcePort+"-->"+sinks.toString();
	}
}
