/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.directories;

import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.file.Path;
import java.util.Collection;
import java.util.Map;

public class ModulePath extends PathWrapper{
	private static final Logger logger = LoggerFactory.getLogger(ModulePath.class);

	public ModulePath(Path sourcePath) {
		super(sourcePath);
	}



	public static String getHashName(Map<String, String> attrMap) {
		return Integer.toHexString(attrMap.hashCode());
	}

/*
	public Stream<ModulePath> getModuleDirectories(Collection<IModuleDescription> md, Map<String, String> attrMap) throws IOException {
		Path hashDir = getHashDir(md, attrMap);
		return getModuleDirectories(hashDir);
	}*/


	public HashPath resolveHashDir(String hash) {
		return new HashPath(getPath().resolve(hash), this);
	}

	/**
	 * get moduleRootDir/moduleName/attrHash
	 * @param attrMap the attributes
	 * @return directory for module attr map
	 * @throws IOException on broken IO
	 */
	public HashPath resolveHashDir(Map<String, String> attrMap) throws IOException {
		HashPath hashDir = resolveHashDir(getHashName(attrMap));

		hashDir.doHashCheck(attrMap);

		return hashDir;
	}

	/**
	 * get moduleRootDir/moduleName/attrHash
	 * @param mcs the module configuration
	 * @return directory for module attr map
	 * @throws IOException on broken IO
	 */
	public HashPath resolveHashDir(Collection<IModuleConfiguration<?>> mcs) throws IOException {
		return resolveHashDir(ModuleDirectories.getAttributeMap(mcs));
	}

	public String getModuleName() {
		return getPath().getFileName().toString();
	}
}
