/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.directories;

import edu.byu.ece.rapidSmith.device.Device;
import edu.byu.ece.rapidSmith.util.DeviceDatabaseProvider;
import edu.byu.ece.rapidSmith.util.PartNameTools;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.stream.Stream;

public class VariantPath extends PathWrapper {

	public static final String AREA_UCF_FILENAME = "area.ucf";
	private final HashPath hashPath;

	public VariantPath(Path sourcePath, HashPath hashPath) {
		super(sourcePath);
		this.hashPath = hashPath;
	}

	public Path resolveSmilePath() {
		return getPath().resolve("build/module");
	}

	public Path resolveSmileModule() {
		return resolveSmilePath().resolve("module.smile");
	}

	public Path resolveSmileDescription(Device device) {
		return resolveSmilePath().resolve(PartNameTools.removeSpeedGrade(device.getPartName()) + ".smile");
	}

	public Path resolveModuleConfig() {
		return getPath().resolve("config.jc3");
	}

	public Path resolveXdl() {
		return getPath().resolve("build/spartanmc_par.xdl");
	}

	public Path resolveAreaUcf() {
		return getPath().resolve(AREA_UCF_FILENAME);
	}

	public void backupConfigFile() {
		Path configFile = resolveModuleConfig();
		if (Files.exists(configFile)) {
			int i = 0;
			Path backupFile = configFile;
			while (Files.exists(backupFile)) {
				i++;
				backupFile = getPath().resolve("config_" + i + ".jc3");
			}
			try {
				Files.move(configFile, backupFile);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

	}

	public Path resolveMakefile() {
		return getPath().resolve("Makefile");
	}

	public Path resolveMapReport() {
		return getPath().resolve("build/reports/spartanmc_map.mrp");
	}

	public Path resolveLockFile() {
		return getPath().resolve("lock");
	}

	public HashPath getHashPath() {
		return hashPath;
	}

	public String getVariantName() {
		return getPath().getFileName().toString();
	}

	public Stream<Device> streamAvailableDevices() throws IOException {
		final Path smileModule = resolveSmileModule();
		return Files.list(resolveSmilePath())
				.filter(p -> !p.equals(smileModule))
				.map(p -> {
					String devName = p.getFileName().toString().replace(".smile", "");
					return DeviceDatabaseProvider.getDeviceDatabase().loadDevice(devName);
				});
	}
}
