/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.module;

import com.fasterxml.jackson.annotation.JsonIdentityReference;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.stream.Collector;
import java.util.stream.Collectors;

public class ModuleDescriptionPort implements edu.byu.ece.rapidSmith.design.INamed {
	static final String PROPERTY_NAME = "name";
	private static final String PROPERTY_PASS = "ptp";

	@NotNull
	@JsonProperty(PROPERTY_NAME)
	private final String name;
	@Nullable
	@JsonProperty(PROPERTY_PASS)
	@JsonIdentityReference(alwaysAsId = true)
	@JsonInclude(JsonInclude.Include.NON_NULL)
	private ModuleDescriptionVariantPort passthroughPort = null;

	public ModuleDescriptionPort(String name) {
		this.name = name;
	}

	public ModuleDescriptionPort(@NotNull String name, @Nullable ModuleDescriptionVariantPort passthroughPort) {
		this.name = name;
		this.passthroughPort = passthroughPort;
	}

	@NotNull
	public String getName() {
		return name;
	}
	@Nullable
	public ModuleDescriptionVariantPort getPassthroughFrom() {
		return passthroughPort;
	}

	public void setPassthroughFrom(ModuleDescriptionVariantPort passthroughPort) {
		this.passthroughPort = passthroughPort;
	}


	public static Collector<ModuleDescriptionVariantPort, ?, ModuleDescriptionPort> unifyPorts() {
		return Collectors.collectingAndThen(Collectors.reducing(null, (a, b) -> {
			if (a == null) {
				return b;
			}
			if (b == null) {
				return a;
			}
			if (!a.getName().equals(b.getName())) {
				throw new IllegalStateException("Trying to unify ports of different names: " + a + " and " + b);
			}

			if ((a.getPassthroughFrom() == null) != (b.getPassthroughFrom() == null)) {
				throw new IllegalStateException("Trying to unify ports with different passthroughs: " + a + " and " + b);
			}

			if (b.getPassthroughFrom()!=null && a.getPassthroughFrom().getName().equals(b.getPassthroughFrom().getName())) {

				throw new IllegalStateException("Trying to unify ports with different passthroughs: " + a + " and " + b);
			}

			return a;
		}), p -> {
			return new ModuleDescriptionPort(p.getName(), p.getPassthroughFrom());
		});
	}

}
