/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.module;

import de.tu_darmstadt.rs.rapidSoc.BoundingBox;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;
import edu.byu.ece.rapidSmith.design.jackson.SetToMapConverter;
import edu.byu.ece.rapidSmith.device.Tile;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Arrays;
import java.util.Objects;
import java.util.stream.IntStream;
import java.util.stream.Stream;

/**
 */
@JsonAutoDetect(fieldVisibility = JsonAutoDetect.Visibility.NONE, getterVisibility = JsonAutoDetect.Visibility.NONE, isGetterVisibility = JsonAutoDetect.Visibility.NONE, setterVisibility = JsonAutoDetect.Visibility.NONE)
@JsonIdentityInfo(property = "name", generator = ObjectIdGenerators.PropertyGenerator.class, scope = ModuleDescriptionVariantPort.class)
public class ModuleDescriptionVariantPort extends ModuleDescriptionPort {

	private static final String PROPERTY_TILES = "tiles";

	@Nullable
	private ModuleDescriptionVariant moduleDescriptionVariant;

	@NotNull
	@JsonProperty(PROPERTY_TILES)
	private final int[] tiles;


	/*
	 * Construct a new ModuleDescriptionVariantPort
	 * @param moduleDescriptionVariant the description of the module this port is a part of
	 * @param name the name of this port
	 * @param tiles the tiles used by this port
	 * @param passthroughPort if this is an output port relaying an input without any logic in between, this contains the port the input is at
	 */
	/*@JsonCreator
	public ModuleDescriptionVariantPort(
			@Nullable @JsonProperty(PROPERTY_DESCRIPTION) final ModuleDescriptionVariant moduleDescriptionVariant,
			@NotNull @JsonProperty(PROPERTY_NAME) final String name,
			//@NotNull @JsonProperty(PROPERTY_TILES) final int[] tiles,
			@Nullable @JsonProperty(PROPERTY_PASS) final ModuleDescriptionVariantPort passthroughPort
	) {
		this.moduleDescriptionVariant = moduleDescriptionVariant;
		this.name = name;
		this.tiles = null;//tiles;
		this.passthroughPort = passthroughPort;
	}*/

	/**
	 * Construct a new ModuleDescriptionVariantPort
	 * @param name the name of this port
	 * @param tiles the tiles used by this port
	 */
	public ModuleDescriptionVariantPort(
			@JsonProperty(ModuleDescriptionPort.PROPERTY_NAME) @NotNull String name,
			@JsonProperty(PROPERTY_TILES) @NotNull int[] tiles
	) {
		/*this(
				null,
				name,
				tiles,
				null
		);*/
		super(name);
		this.tiles = tiles;
	}

	void setModuleDescriptionVariant(@NotNull ModuleDescriptionVariant moduleDescriptionVariant) {
		this.moduleDescriptionVariant = moduleDescriptionVariant;
	}

	@NotNull
	public IntStream streamTileNumbers() {
		return Arrays.stream(tiles);
	}

	@NotNull
	public Stream<Tile> streamTiles() {
		Objects.requireNonNull(moduleDescriptionVariant,"Error streaming tiles: module description is null");
		return streamTileNumbers().mapToObj(moduleDescriptionVariant.getDevice()::getTile);
	}

	@NotNull
	public BoundingBox getBoundingBox() {
		return streamTiles().collect(BoundingBox.getTileBoundingBox());
	}

	@Override
	@NotNull
	public String toString() {
		return "Port "+getName()+" at tiles "+Arrays.toString(tiles);
	}

	public static class ModuleDescriptionVariantPortSetToMapConverter extends SetToMapConverter<ModuleDescriptionVariantPort> {

	}
}
