/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.module_synthesis;

import de.tu_darmstadt.rs.rapidSoc.directories.VariantPath;
import de.tu_darmstadt.rs.rapidSoc.module.JacksonUtil;
import de.tu_darmstadt.rs.rapidSoc.module.ModuleDescriptionFactory;
import de.tu_darmstadt.rs.rapidSoc.module.ModuleDescriptionVariant;
import de.tu_darmstadt.rs.rapidSoc.visualization.ModuleVisualization;
import de.tu_darmstadt.rs.rapidSoc.visualization.ModuleVisualizationFrame;

import edu.byu.ece.rapidSmith.design.Module;
import edu.byu.ece.rapidSmith.device.Device;
import edu.byu.ece.rapidSmith.util.DeviceDatabaseProvider;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

public class JacksonModuleWriter {

	public static void writeAll(Module module, VariantPath variantPath, Path possiblePlacementsDir) throws IOException {

		Files.createDirectories(variantPath.resolveSmilePath());

		JacksonUtil.writeModule(module, variantPath.resolveSmileModule(), true);

		if (possiblePlacementsDir != null) {
			try {
				Files.createDirectories(possiblePlacementsDir);
			} catch (IOException e) {
				throw new RuntimeException(e);
			}
		}


		for (String partName : DeviceDatabaseProvider.getDeviceDatabase().getAvailableParts()) {
			final Device device = DeviceDatabaseProvider.getDeviceDatabase().loadDevice(partName);
			final ModuleDescriptionVariant moduleDescriptionVariant = ModuleDescriptionFactory.getModuleDescription(module, device);
			if (moduleDescriptionVariant == null) {
				continue;
			}

			JacksonUtil.writeModuleDescription(moduleDescriptionVariant, variantPath.resolveSmileDescription(device), true);


			if (possiblePlacementsDir != null) {
				Path p = possiblePlacementsDir.resolve(partName + ".svg");
				final ModuleVisualization visu = new ModuleVisualization(moduleDescriptionVariant, false, true, true, true);
				visu.saveImage(p);
				new ModuleVisualizationFrame(visu).openBlockingUI();
			}
		}

	}
}
