/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.rapidSoc.module_synthesis;


import de.tu_darmstadt.rs.process.ProcessUtil;
import de.tu_darmstadt.rs.process.StreamGobbler;
import de.tu_darmstadt.rs.rapidSoc.LogMarkers;
import de.tu_darmstadt.rs.rapidSoc.RapidSocProject;
import de.tu_darmstadt.rs.rapidSoc.directories.HashPath;
import de.tu_darmstadt.rs.rapidSoc.directories.ModuleDirectories;
import de.tu_darmstadt.rs.rapidSoc.directories.VariantPath;
import de.tu_darmstadt.rs.rapidSoc.module_synthesis.constraints.ConstraintCreator;
import de.tu_darmstadt.rs.rapidSoc.visualization.ModuleInDesignVisualization;
import de.tu_darmstadt.rs.rapidSoc.visualization.VisualizationFrame;
import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.Bus;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.buses.BusPort;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.generic.ConnectorDirection;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IConnectorDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.ports.IPort;
import de.tu_darmstadt.rs.spartanmc.devxml.files.buses.BusDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.BusBreakoutModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.ModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.mappedBuses.SingleMappedBusPort;
import de.tu_darmstadt.rs.spartanmc.jconfig.LibJConfigProject;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IApplicationConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationDocument;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConfigurationElement;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IConnection;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.exceptions.ConnectionMismatchException;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.features.IConnectorInstance;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBusHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IIntegerParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPinHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IPortHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IToolchainAttributeHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.jaxb.jc3.BusBreakoutConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil;
import de.tu_darmstadt.rs.util.SpartanMCMakeUtil;
import de.tu_darmstadt.rs.util.StreamUtil;
import de.tu_darmstadt.rs.util.tracer.Tracer;

import edu.byu.ece.rapidSmith.design.Design;
import edu.byu.ece.rapidSmith.design.Module;
import edu.byu.ece.rapidSmith.design.Net;
import edu.byu.ece.rapidSmith.util.SetHelpers;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UncheckedIOException;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import javax.xml.bind.JAXBException;

/**
 * Creates module designs
 *
 * Created by jakobw on 30.10.15.
 */
public class ModuleSynthesis {

	private static final Logger logger = LoggerFactory.getLogger(ModuleSynthesis.class);

	private static final String ADD_AREA_CONSTRAINTS_MSG = "#Add your area constraints here";
	private static final Pattern FPGA_PIN_EXPRESSION = Pattern.compile("#([A-Z][A-Z0-9_]+)(?:[ ]*\\(([A-Z][A-Z0-9_]+)\\))?");
	private static final String CLKOUT_MODULE_NAME = "clkout";
	private static final String MODULE_TARGET_NAME = "rapidModule";
	public static final String IN_MODULE_EXCLUDE = "inModule_exclude";
	public static final String OUT_MODULE_EXCLUDE = "outModule_exclude";
	private final String descriptionsName;
	private final String variantName;

	private final Collection<IModuleDescription> moduleDescriptions;
	private final Map<String, String> attributeMap;
	private final VariantPath projectPath;
	private final RapidSocProject rapidSocProject;
	private final LibJConfigProject project;
	private final ConstraintCreator constraintCreator;
	private IConfigurationDocument document;

	//JTAG is connected to those pins
	//TODO throw these pins out of the target description
	private List<String> sp601InvalidPins = Arrays.asList("A17", "B18", "D15", "D16", "V17", "V2", "P13", "R16", "G1", "G3", "L3", "L4", "P1", "P2", "A12", "B12");
	private FileLock lock;
	private Path lockFile;


	/**
	 * Create ModuleSynthesis
	 *
	 * @param moduleDescriptions module descriptions to use
	 * @param attributeMap the attributes
	 * @throws IOException on broken IO
	 */

	public ModuleSynthesis(Collection<IModuleDescription> moduleDescriptions, Map<String, String> attributeMap, String variant, RapidSocProject rapidSocProject) {
		this.moduleDescriptions = moduleDescriptions;

		this.descriptionsName = ModuleDirectories.getModuleDescriptionName(moduleDescriptions);
		this.variantName = variant;
		this.rapidSocProject = rapidSocProject;


		Tracer.enterLoop("Module init");


		if (attributeMap == null) {
			throw new RuntimeException("attribute map is null!");
			//attributeMap = ModuleDirectories.getDefaultAttributeMap(moduleDescription);
		}

		this.attributeMap = attributeMap;

		try {
			HashPath hashDir = rapidSocProject.getModuleRootPath().resolveHashDir(moduleDescriptions, attributeMap);

			this.projectPath = hashDir.resolveVariant(variant);
		} catch (IOException e) {
			throw new UncheckedIOException(e);
		}


		this.constraintCreator = rapidSocProject.getConstraintCreatorFactory().makeConstraintCreator(projectPath, rapidSocProject);

		project = new LibJConfigProject(projectPath.getPath(), rapidSocProject.getApplication(), false);

		Tracer.exit();
	}


	public void createSystemFiles() {
		if (!project.getDocumentHandler().hasLoadedDocument()) {
			Path configFile = projectPath.resolveModuleConfig();
			project.getDocumentHandler().open(configFile);
		}
		project.getDocumentHandler().build(false);
	}

	private boolean isValidPin(String pinName) {
		Matcher m = FPGA_PIN_EXPRESSION.matcher(pinName);
		if (m.matches()) {
			String name = m.group(1);
			String loc = m.group(2);
			if (loc == null) {
				loc = name;
			}

			return !sp601InvalidPins.contains(loc);

		} else {
			throw new RuntimeException("matcher does not match");
		}
	}

	private <T, R> R extractPortInfo(Set<T> ports, Function<T, R> extractor) {
		try {
			return ports.stream().map(extractor).collect(StreamUtil.atLeastOneOrEqual());
		} catch (IllegalStateException e) {
			throw new IllegalStateException("fail for " + ports, e);
		}
	}

	private void connectPins(Collection<IModuleConfiguration<?>> modules) {
		//Get all valid pins, grouped by clock capable
		Map<Boolean, Set<IPinHandler>> pins = DocumentUtil.streamConnectorHandlers(document)
				.filter(e -> {
					String name = e.getPortName();
					return !(name.contains("FMC_DP0") || name.contains("SMA") || name.contains("GBTCLK") || name.contains("PCIE"));
				})
				.filter(e -> !sp601InvalidPins.contains(e.getPinDescription().getFpgaPinName()))
				//TODO do not filter all unconfigurable pins, instead assign them only to matching ports
				.filter(e -> e.getPinDescription().getDirection() == ConnectorDirection.Configurable)
				.collect(
						Collectors.partitioningBy(
								p -> (p.getFrequency().isPresent()) || p.getPortName().contains("_CC_"),
								Collectors.toSet()
						)
				);

		//TODO set IO Standard of all pins on same bank to same value

		logger.info(LogMarkers.rapidSoc, "Found {} clock pins", pins.get(true).size());
		logger.info(LogMarkers.rapidSoc, "Found {} non clock pins", pins.get(false).size());

		Iterator<IPinHandler> clockPins = pins.get(true).iterator();
		Iterator<IPinHandler> nonClockPins = pins.get(false).iterator();

		Map<String, Set<IPortHandler<IPort>>> connectors = getAllConnectors(modules);

		// Ermitteln wie viele Input- und Output-Pins unser Modul hat.
		int moduleInputs = 0;
		int moduleOutputs = 0;
		int inputWidth = 0;
		int outputWidth = 0;
		int tempWidth = 0;
		Set<IPortHandler<IPort>> modulePorts;

		for (String name : connectors.keySet()) {
			modulePorts = connectors.get(name);
			Set<ConnectorDirection> directions = modulePorts.stream().map(IConnectorInstance::getDirection).collect(Collectors.toSet());
			tempWidth = extractPortInfo(modulePorts, IConnectorInstance::getConnectorWidth);
			boolean isClock = extractPortInfo(modulePorts, port -> port.getType() == IConnectorDescription.Type.Clk);

			if (directions.size() == 1 && !isClock) {
				if (directions.contains(ConnectorDirection.Input)) {
					moduleInputs++;
					inputWidth += tempWidth;
				}
				if (directions.contains(ConnectorDirection.Output)) {
					moduleOutputs++;
					outputWidth += tempWidth;
				}
			}
		}

		logger.debug("Inputs: " + moduleInputs + " | Outputs: " + moduleOutputs);
		logger.debug("InputWidth: " + inputWidth + " | OutputWidth: " + outputWidth);

		Library library = project.getApplication().getLibrary();
		IPinHandler pin;
		final long defaultClockFreq = 55000000L;

		//Pseudo Input Module mit Breite inputWidth erstellen
		ModuleDescription inModule = library.getCommonDescription("pseudo_in");
		IModuleConfiguration<? extends IConfigurationDocument> inModuleConfiguration = document.addModule(inModule, IN_MODULE_EXCLUDE);

		IIntegerParameterHandler inParam = (IIntegerParameterHandler) DocumentUtil.getParameterHandler(inModuleConfiguration, "PORT_WIDTH");
		inParam.setValue(inputWidth);

		IPortHandler<? extends IPort> inCLK = DocumentUtil.getPortHandler(inModuleConfiguration, "clk");
		IPortHandler<? extends IPort> inFPGA = DocumentUtil.getPortHandler(inModuleConfiguration, "fpga");
		IPortHandler<? extends IPort> inPINs = DocumentUtil.getPortHandler(inModuleConfiguration, "pin");

		//Clock verbinden
		pin = clockPins.next();
		pin.setFrequencyOverride(defaultClockFreq); //TODO allow user to set freq
		connectSinglePort("inCLK", inCLK, pin, true, ConnectorDirection.Input, 1, 0);

		//FPGA Pin verbinden
		pin = nonClockPins.next();
		connectSinglePort("inFPGA", inFPGA, pin, false, ConnectorDirection.Input, 1, 0);


		//Pseudo Output Module mit Breite outputWidth erstellen
		ModuleDescription outModule = library.getCommonDescription("pseudo_out");
		IModuleConfiguration<? extends IConfigurationDocument> outModuleConfiguration = document.addModule(outModule, OUT_MODULE_EXCLUDE);

		IIntegerParameterHandler outParam = (IIntegerParameterHandler) DocumentUtil.getParameterHandler(outModuleConfiguration, "PORT_WIDTH");
		outParam.setValue(outputWidth);

		IPortHandler<? extends IPort> outCLK = DocumentUtil.getPortHandler(outModuleConfiguration, "clk");
		IPortHandler<? extends IPort> outFPGA = DocumentUtil.getPortHandler(outModuleConfiguration, "fpga");
		IPortHandler<? extends IPort> outFPGA2 = DocumentUtil.getPortHandler(outModuleConfiguration, "fpga2");
		IPortHandler<? extends IPort> outPINs = DocumentUtil.getPortHandler(outModuleConfiguration, "pin");

		//Clock verbinden
		pin = clockPins.next();
		pin.setFrequencyOverride(defaultClockFreq); //TODO allow user to set freq
		connectSinglePort("outCLK", outCLK, pin, true, ConnectorDirection.Input, 1, 0);

		//FPGA Pin verbinden
		pin = nonClockPins.next();
		connectSinglePort("outFPGA", outFPGA, pin, false, ConnectorDirection.Output, 1, 0);
		pin = nonClockPins.next();
		connectSinglePort("outFPGA2", outFPGA2, pin, false, ConnectorDirection.Output, 1, 0);

		connectors.forEach((name, ports) -> {
			boolean isClock = extractPortInfo(ports, port -> port.getType() == IConnectorDescription.Type.Clk);


			Set<ConnectorDirection> directions = ports.stream().map(IConnectorInstance::getDirection).collect(Collectors.toSet());

			if (directions.contains(ConnectorDirection.InOut) && directions.size() > 1) {
				throw new IllegalStateException("Cannot mix inout with other directions: " + directions);
			}

			//ConnectorDirection direction = extractPortInfo(ports, IConnectorInstance::getDirection);
			int connectorWidth = extractPortInfo(ports, IConnectorInstance::getConnectorWidth);

			if (directions.contains(ConnectorDirection.InOut) && ports.size() > 1) {
				throw new IllegalStateException("Cannot connect inouts to multiple ports.");
			}

			logger.trace(LogMarkers.rapidSoc, "Handling " + name + ", " + (isClock? "clock" : "not clock") + ", " + directions);

			Set<IPortHandler<IPort>> toOutsidePorts;
			ConnectorDirection direction;
			if (directions.size() > 1) {
				//Sanity check. Directions should be input and output
				if (directions.size() > 2 || !directions.contains(ConnectorDirection.Input) || !directions.contains(ConnectorDirection.Output)) {
					throw new IllegalStateException("Directions should be input and output, but was: " + directions);
				}

				toOutsidePorts = ports.stream().filter(p -> p.getDirection() == ConnectorDirection.Output).collect(Collectors.toSet());
				direction = ConnectorDirection.Output;

				//connect all inputs to some output
				IPortHandler<IPort> representative = toOutsidePorts.iterator().next();
				ports.stream().filter(p -> p.getDirection() == ConnectorDirection.Input).forEach(p -> {
					logger.info(LogMarkers.rapidSoc, "Internal connection: Connecting " + representative.getSourceElement().getName() + "." + representative.getPortName() + " to " + p.getSourceElement().getName() + "." + p.getPortName());
					if (p.getConnectorWidth() > 1) {
						document.createPartialConnection(representative, 0, p, IConnection.FULL_CONNECTOR_WIDTH, IConnection.FULL_CONNECTOR_WIDTH).connect();
					} else {
						document.createPortConnection(representative, p).connect();
					}
				});
			} else {
				toOutsidePorts = ports;
				direction = directions.iterator().next();
			}

			if (isClock || (direction != ConnectorDirection.Input && direction != ConnectorDirection.Output)) {
				for (int i = 0; i < connectorWidth; i++) {
					connectPortBit(clockPins, nonClockPins, name, toOutsidePorts, isClock, direction, connectorWidth, i);
				}
			}
		});

		int inIndex = 0;
		int outIndex = 0;

		for (String name : connectors.keySet()) {
			modulePorts = connectors.get(name);
			Set<ConnectorDirection> directions = modulePorts.stream().map(IConnectorInstance::getDirection).collect(Collectors.toSet());
			tempWidth = extractPortInfo(modulePorts, IConnectorInstance::getConnectorWidth);
			boolean isClock = extractPortInfo(modulePorts, port -> port.getType() == IConnectorDescription.Type.Clk);

			if (directions.size() == 1 && !isClock) {
				if (directions.contains(ConnectorDirection.Input)) {
					//For inputs, connect all
					IPortHandler<IPort> tempPort = modulePorts.iterator().next();
					document.createPartialConnection(inPINs, inIndex, tempPort, 0, tempWidth).connect();
					inIndex += tempWidth;
				}
				if (directions.contains(ConnectorDirection.Output)) {
					IPortHandler<IPort> tempPort = modulePorts.iterator().next();
					document.createPartialConnection(outPINs, outIndex, tempPort, 0, tempWidth).connect();
					outIndex += tempWidth;


				}
			}
		}

	}

	private void connectPortBit(Iterator<IPinHandler> clockPins, Iterator<IPinHandler> nonClockPins, String name, Set<IPortHandler<IPort>> ports, boolean isClock, ConnectorDirection direction, int connectorWidth, int i) {
		IPinHandler pin;
		if (isClock && direction == ConnectorDirection.Input) {
			if (clockPins.hasNext()) {
				pin = clockPins.next();
			} else {
				throw new RuntimeException("not enough clock pins to generate module");
			}
		} else if (nonClockPins.hasNext()) {
			pin = nonClockPins.next();
		} else if (clockPins.hasNext()) {
			pin = clockPins.next();
		} else {
			throw new RuntimeException("not enough pins to generate module");
		}

		if (isClock && direction == ConnectorDirection.Input) {
			String freq = attributeMap.get("CLOCK_FREQUENCY");
			if (freq != null) {
				pin.setFrequencyOverride(Long.parseLong(freq));
			} else {
				final long defaultClockFreq = 55000000L;
				pin.setFrequencyOverride(defaultClockFreq); //TODO allow user to set freq
			}
		}


		if (direction == ConnectorDirection.Output) {
			//If it's an output, select a single representative
			IPortHandler<IPort> port = ports.iterator().next();
			connectSinglePort(name, port, pin, isClock, direction, connectorWidth, i);
		} else {
			//For inputs, connect all
			ports.forEach(port -> {
				connectSinglePort(name, port, pin, isClock, direction, connectorWidth, i);
			});
		}
	}

	/**
	 * Get all connectors a module has, grouped by their local name. Bus connectors are split by inserting a bus breakout
	 * module
	 *
	 * @param modules the modules to use
	 * @return all connectors
	 */
	private Map<String, Set<IPortHandler<IPort>>> getAllConnectors(Collection<IModuleConfiguration<?>> modules) {

		Set<StreamUtil.Pair<String, IPortHandler<IPort>>> bus = instantiateBusBreakouts(modules);


		Map<BusDescription, IBusHandler> busMasters = modules.stream()
				.flatMap(DocumentUtil::streamBusHandlers)
				.filter(busHandler -> busHandler.getConnectorDescription().getRole() == Bus.Role.Master)
				.collect(Collectors.toMap(IBusHandler::getBusDefinition, Function.identity()));

		modules.stream()
				.flatMap(DocumentUtil::streamBusHandlers)
				.filter(busHandler -> busHandler.getConnectorDescription().getRole() == Bus.Role.Slave)
				.forEach(slaveHandler -> {
					//Check if there is a corresponding master available
					IBusHandler masterHandler = busMasters.get(slaveHandler.getBusDefinition());
					if (masterHandler != null && masterHandler.getSourceElement() != slaveHandler.getSourceElement()) {

						logger.info(LogMarkers.rapidSoc, "found internal bus conn: " + slaveHandler.getAbsoluteConnectorName() + " to " + masterHandler.getAbsoluteConnectorName());

						BusBreakoutConfiguration<?> masterBreakout = (BusBreakoutConfiguration<?>) masterHandler.streamOppositeConnectors().collect(StreamUtil.exactlyOne()).getSourceElement();
						BusBreakoutConfiguration<?> slaveBreakout = (BusBreakoutConfiguration<?>) slaveHandler.streamOppositeConnectors().collect(StreamUtil.exactlyOne()).getSourceElement();

						Stream<SingleMappedBusPort> masterMappedBus = mapBus(masterHandler);
						Map<BusPort, SingleMappedBusPort> mappedSlavePorts = mapBus(slaveHandler).collect(Collectors.toMap(SingleMappedBusPort::getPort, Function.identity()));


						masterMappedBus.forEach(masterPort -> {
							logger.info(LogMarkers.rapidSoc, "looking at " + masterPort);
							BusPort busPort = masterPort.getPort();
							BusPort slavePort = slaveHandler.getBusDefinition().getOppositePort(Bus.Role.Master, busPort);


							boolean needsGlue = mappedSlavePorts.get(slavePort).mayRequireBusGlue() || masterPort.mayRequireBusGlue();
							if (!needsGlue) {
								//Get breakout ports
								IPortHandler<? extends IPort> masterBreakoutPort = DocumentUtil.getPortHandler(masterBreakout, slavePort.getName());
								IPortHandler<? extends IPort> slaveBreakoutPort = DocumentUtil.getPortHandler(slaveBreakout, busPort.getName());

								logger.info(LogMarkers.rapidSoc, "connecting " + masterBreakoutPort.getAbsoluteConnectorName() + " to " + slaveBreakoutPort.getAbsoluteConnectorName());

								//TODO simplify once partialconnections and full connections are merged
								if (masterBreakoutPort.getConnectorWidth() > 1) {
									if (masterBreakoutPort.getDirection() == ConnectorDirection.Output) {
										document.createPartialConnection(masterBreakoutPort, 0, slaveBreakoutPort, IConnection.FULL_CONNECTOR_WIDTH, IConnection.FULL_CONNECTOR_WIDTH).connect();
									} else {
										document.createPartialConnection(masterBreakoutPort, IConnection.FULL_CONNECTOR_WIDTH, slaveBreakoutPort, 0, IConnection.FULL_CONNECTOR_WIDTH).connect();
									}
								} else {
									document.createPortConnection(masterBreakoutPort, slaveBreakoutPort).connect();
								}

							} else {
								logger.info(LogMarkers.rapidSoc, "Not connecting " + busPort.getName() + " internally, because glue is needed");
							}
						});

					}
				});

		Stream<StreamUtil.Pair<String, IPortHandler<IPort>>> regular = modules.stream().flatMap(DocumentUtil::streamConnectorHandlers)
				.map(handler -> new StreamUtil.Pair<>(handler.getPortName(), handler));

		return Stream.concat(bus.stream(), regular).filter(pair -> {
			IPortHandler<IPort> port = pair.getB();
			if (port.getDirection() == ConnectorDirection.Input && port.getConnectionCount() > 0) {
				logger.info(LogMarkers.rapidSoc, "Not conneting " + port.getAbsoluteConnectorName() + " to outside, is already sourced");
				return false;
			} else {
				return true;
			}
		}).collect(Collectors.groupingBy(StreamUtil.Pair::getA, Collectors.mapping(StreamUtil.Pair::getB, Collectors.toSet())));
	}

	private Stream<SingleMappedBusPort> mapBus(IBusHandler masterHandler) {
		return SingleMappedBusPort.mapBus(((IModuleConfiguration<?>) masterHandler.getSourceElement()).getModuleDescription(), masterHandler.getConnectorDescription(), project.getApplication().getLibrary());
	}

	private Set<StreamUtil.Pair<String, IPortHandler<IPort>>> instantiateBusBreakouts(Collection<IModuleConfiguration<?>> modules) {
		return modules.stream().flatMap(DocumentUtil::streamBusHandlers)
				.flatMap(busHandler -> {
					if (busHandler.getConnectionCount() > 0) {
						busHandler.streamConnections().forEach(c -> {
							logger.info(LogMarkers.rapidSoc, "Removing connection " + c);
							c.disconnect();
						});
					}

					logger.trace(LogMarkers.rapidSoc, "Creating bus breakout for port " + busHandler);
					BusBreakoutModuleDescription breakoutDescription = project.getApplication().getLibrary().getBusBreakoutDescription(busHandler.getBusDefinition(), busHandler.getConnectorDescription().getRole().getOpposite());


					BusBreakoutConfiguration<IConfigurationDocument> breakoutInstance = (BusBreakoutConfiguration<IConfigurationDocument>) document.addModule(breakoutDescription);


					IBusHandler breakoutHandler;
					try {
						breakoutHandler = DocumentUtil.getBusHandler(breakoutInstance, busHandler.getBusDefinition().getName());
						document.createBusConnection(busHandler, breakoutHandler).connect();
					} catch (ConnectionMismatchException e) {
						throw new RuntimeException("fail in " + busHandler + " in " + busHandler.getSourceElement().getName(), e);
					}


					return DocumentUtil.streamConnectorHandlers(breakoutInstance)
							.map(handler -> new StreamUtil.Pair<>(ModuleExtractorThorsten.findLocalNameForRemoteBusPort(breakoutHandler, handler.getPortName()), handler));
				}).collect(Collectors.toSet());
	}

	private void connectSinglePort(String name, IPortHandler<? extends IPort> port, IPinHandler pin, boolean isClock, ConnectorDirection direction, int connectorWidth, int i) {
		if (isClock && direction == ConnectorDirection.Output) {
			logger.trace(LogMarkers.rapidSoc, "Connecting " + name + " via clkout. selected " + port);
			connectViaClkout(port, pin);
		} else {
			if (connectorWidth > 1) {
				document.createPartialConnection(pin, IConnection.FULL_CONNECTOR_WIDTH, port, i, IConnection.FULL_CONNECTOR_WIDTH).connect();
			} else {
				document.createPortConnection(pin, port).connect();
			}
		}
	}

	private void connectViaClkout(IPortHandler<? extends IPort> port, IPinHandler pin) {
		IModuleDescription descr = project.getApplication().getLibrary().getModuleDescription(CLKOUT_MODULE_NAME);
		if (descr == null) {
			throw new RuntimeException("could not find clkout module");
		}

		String name = descr.getName() + "_" + port.getAbsoluteConnectorName().toLowerCase().replace("/", "").replace(".", "_");
		IModuleConfiguration<? extends IConfigurationDocument> module = document.addModule(descr, name);

		IPortHandler<? extends IPort> i = DocumentUtil.getPortHandler(module, "i");
		IPortHandler<? extends IPort> o = DocumentUtil.getPortHandler(module, "o");

		document.createPortConnection(port, i).connect();
		document.createPortConnection(o, pin).connect();
	}

	protected void visualize(Module m, Set<Net> partialNets, boolean showPossiblePlacements) {
		Design design = m.getInstances().iterator().next().getDesign();


		ModuleInDesignVisualization visu = new ModuleInDesignVisualization(design, false, true, m, true, showPossiblePlacements);
		try {
			new File("visu").mkdirs();
			visu.svgToStream(new FileOutputStream("visu/" + m.getName() + ".svg"));
			new VisualizationFrame<>(visu).openBlockingUI();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public Module extractModule(boolean isCore) {

		return extractModule(false, new HashSet<>(), isCore);
	}

	/**
	 * Extract the module
	 *
	 * @param noDisconnect disable disconnect
	 * @param partialNets set of nets crossing module boundary
	 * @param isCore is the module the core?
	 * @return extracted module
	 */
	public Module extractModule(boolean noDisconnect, Set<Net> partialNets, boolean isCore) {

		Path xdl = projectPath.resolveXdl();
		return extractModule(xdl, noDisconnect, partialNets, isCore);
	}

	/**
	 * Extract the module
	 *
	 * @param xdl the xdl file
	 * @param noDisconnect disable disconnect
	 * @param partialNets set of nets crossing module boundary
	 * @param isCore is the module the core?
	 * @return extracted module
	 */
	public Module extractModule(Path xdl, boolean noDisconnect, Set<Net> partialNets, boolean isCore) {

		if (!Files.exists(xdl)) {
			return null;
		}


		Tracer.enterLoop("Load JConfig document");
		project.getDocumentHandler().open(projectPath.resolveModuleConfig());
		document = project.getDocumentHandler().getDocument();
		Tracer.exit();


		Set<IModuleConfiguration<?>> mod = getVerilogModules();

		verifyAttributeMap(mod);

		Tracer.enterLoop("Load XDL Design");
		Design design = new Design();
		design.loadXDLFile(xdl);
		Tracer.exit();

		ModuleExtractor extractor = new ModuleExtractorNew(design, mod, document);
//		ModuleExtractor extractor = new ModuleExtractorVerifying(design, mod, document, xdl);
//		ModuleExtractor extractor = new ModuleExtractorThorsten(design, mod, document);
		return extractor.generateModule(descriptionsName + "_" + Integer.toHexString(attributeMap.hashCode()) + "_" + variantName, noDisconnect, partialNets, isCore);


	}

	private void verifyAttributeMap(Set<IModuleConfiguration<?>> mod) {
		Tracer.enterLoop("Check attribute map");
		Map<String, String> realAttrMap = ModuleDirectories.getAttributeMap(mod);
		if (!attributeMap.equals(realAttrMap)) {
			throw new RuntimeException("Module " + descriptionsName + " does not have advertised attributes.\n" + SetHelpers.dumpVs(attrMapToArr(attributeMap), attrMapToArr(realAttrMap)));
		}
		Tracer.exit();
	}

	private String[] attrMapToArr(Map<String, String> attrMap) {
		return HashPath.hashCheckContents(attrMap).toArray(String[]::new);
	}

	private Set<IModuleConfiguration<?>> getVerilogModules() {
		return document.streamModulesRecursive().filter(m -> {
			if (m.getModuleDescription().getName().equals("reg") && DocumentUtil.getParameterHandler(m, "RAPIDSOC_EXCLUDE").getValue().orElseThrow().equals("1")) {
				return false;
			}
			if (m.getModuleDescription().getName().equals(CLKOUT_MODULE_NAME)) {
				return false;
			}
			if (m.getModuleDescription().getType() == IModuleDescription.ModuleType.BusBreakout) {
				return false;
			}
			if (m.getModuleDescription().getName().equals("pseudo_in") || m.getModuleDescription().getName().equals("pseudo_out")) {
				return false;
			}
			return true;
		}).collect(Collectors.toSet());
	}

	/**
	 * Generate a module project
	 *
	 * @return true if constraints could be generated
	 *
	 * @throws IOException on broken IO
	 * @throws JAXBException on invalid files
	 */
	public ModuleState generateProject() throws IOException, JAXBException {
		Files.createDirectories(projectPath.getPath());

		logger.trace(LogMarkers.rapidSoc, "Generating project for " + descriptionsName + " in " + projectPath);

		//Create document
		project.getApplication().getLibrary().getAvailableDevices();
		//TODO multiple devices support
		project.getDocumentHandler().newDocument(rapidSocProject.getTarget(), project.getApplication().getLibrary().getDeviceDescription(rapidSocProject.getTarget().getDevices().get(0).getName()));
		document = project.getDocumentHandler().getDocument();

		//Disable reg merging with iob
		IAttributeHandler ucData = document.getTarget().getAttributeHandler(IConfigurationDocument.ITargetConfiguration.ATTRIB_UC_DATA);
		ucData.setStoredValue("INST \"*\" IOB = FALSE;"); //TODO remove this once we support regs in IOBs

		IAttributeHandler ucFile = document.getTarget().getAttributeHandler(IConfigurationDocument.ITargetConfiguration.ATTRIB_UC_FILE);
		ucFile.setStoredValue(VariantPath.AREA_UCF_FILENAME);

		//Set verilog define
		final IToolchainAttributeHandler definesParamHandler = DocumentUtil.getToolchainParamHandler(document.getTarget(), "DEFINES");
		if (definesParamHandler == null) {
			throw new NullPointerException("Target does not have defines parameter! Please add it to device description");
		}
		definesParamHandler.setStoredValue(definesParamHandler.getDefaultValue() + " RAPIDSOC_MODULE");


		ModuleState newState = checkAndGenerateAreaFile();


		ISubsystemConfiguration<?> subsystem = null;

		List<IModuleConfiguration<?>> modules = new ArrayList<>();

		for (IModuleDescription moduleDescription : moduleDescriptions) {

			IConfigurationElement<?, ?, ?> parent = subsystem != null? subsystem : this.document;
			IModuleConfiguration<?> module = project.getDocumentHandler().addModule(parent, moduleDescription);

			if (module instanceof ISubsystemConfiguration<?>) {
				subsystem = (ISubsystemConfiguration<?>) module;
			}

			modules.add(module);

		}


		modules.forEach(this::enterParameters);

		connectPins(modules);

		Path configFile = projectPath.resolveModuleConfig();
		projectPath.backupConfigFile();


		project.getDocumentHandler().setSaveLocation(configFile);
		project.getDocumentHandler().save();


		try {
			copyMakefile(project.getApplication());
		} catch (IOException e) {
			e.printStackTrace();
		}

		return newState;
	}

	/**
	 * Synthesize the project
	 *
	 * @throws IOException on broken IO
	 */
	@SuppressWarnings("checkstyle:SystemOut")
	public void synthesizeProject() throws IOException {
		Tracer.enterLoop("Synthesize module");
		//SpartanMCMakeUtil.runMakeTarget(getProjectPath(), MODULE_TARGET_NAME);
		SpartanMCMakeUtil.runMakeTarget(getProjectPath(), "placeroute", "xdl");
		Tracer.exit();
	}


	public void extractModule() throws IOException {
		Tracer.enterLoop("Extract module");
		Module module = extractModule(false, new HashSet<>(), moduleDescriptions.stream().anyMatch(x -> x.getType() == IModuleDescription.ModuleType.Subsystem));

		JacksonModuleWriter.writeAll(module, projectPath, null);

		Tracer.exit();
	}

	/**
	 * Synthesize the project
	 *
	 * @throws IOException on broken IO
	 */
	@SuppressWarnings("checkstyle:SystemOut")
	public void mapProject() throws IOException {
		Tracer.enterLoop("Map Without Constraint");
		SpartanMCMakeUtil.runMakeTarget(getProjectPath(), "mapping");
		Tracer.exit();

	}

	private boolean hasCustomizedAreaFile() throws IOException {
		Path areaFile = projectPath.resolveAreaUcf();
		return hasCustomizedAreaFile(areaFile);
	}

	private boolean hasCustomizedAreaFile(Path areaFile) throws IOException {
		if (Files.exists(areaFile)) {
			//Check for default template
			List<String> lines = Files.readAllLines(areaFile);
			return !(lines.size() == 1 && lines.get(0).equals(ADD_AREA_CONSTRAINTS_MSG));
		} else {
			return false;
		}
	}

	/**
	 * Check if a customized area file exists. if not, check if there is a module wide one. If yes, symlink to it.
	 * If not, create a template
	 *
	 * @return true if a customized area file exists
	 *
	 * @throws IOException on broken IO
	 */
	private ModuleState checkAndGenerateAreaFile() throws IOException {
		Path areaFile = projectPath.resolveAreaUcf();
		if (hasCustomizedAreaFile(areaFile)) {
			return ModuleState.CONSTRAINED;
		} else {
			//Check for template file
			Path template = areaFile.getParent().resolve("../../area.ucf");
			if (Files.exists(template)) {
				Path templatePath = areaFile.getParent().relativize(template);
				Files.createSymbolicLink(areaFile, templatePath);
				return ModuleState.CONSTRAINED;
			} else {

				try (PrintWriter p = new PrintWriter(Files.newBufferedWriter(areaFile))) {
					p.println(ADD_AREA_CONSTRAINTS_MSG);
				}
				return ModuleState.CREATED;
			}
		}
	}

	private void enterParameters(IModuleConfiguration module) {
		DocumentUtil.streamAllParameters(module)
				.filter((iParameterHandler) -> iParameterHandler.isEditable().get())
				.forEach(param -> {
					String valueToSet = attributeMap.get(param.getName());
					if (valueToSet != null && param.getParameterDescription().resolveRelevantIf(module).get()) {
						if (!param.isEditable().get()) {
							throw new UnsupportedOperationException("cannot set value of " + param);
						}
						param.setStoredValue(valueToSet);
					}
				});
	}

	private void copyMakefile(IApplicationConfiguration application) throws IOException {

		try (PrintWriter pw = new PrintWriter(Files.newBufferedWriter(projectPath.resolveMakefile()))) {
			Files.lines(project.getApplication().getSpartanRoot().resolve("src/scripts/make/template/Makefile.hardware"))
					//Strip escaping
					.map(s -> s.replace("\\\\", "\\").replace("\\\"", "\"").replace("\\$", "$"))
					.map(s -> {
						if (s.equals("JCFLAGS:=")) {
							return s + application.getLibrary().getSourceTreesAsBuilderParameters(application.getSpartanRoot());
						}
						return s;
					})
					.forEachOrdered(pw::println);
		}
	}


	/**
	 * Get the state of a module
	 *
	 * @param checkUpToDate whether to check if the module is up to date
	 * @return the state
	 *
	 * @throws IOException on broken IO
	 */
	public ModuleState getModuleState(boolean checkUpToDate) throws IOException {
		if (!Files.exists(projectPath.resolveModuleConfig())) {
			return ModuleState.MISSING;
		}
		if (!Files.exists(projectPath.resolveXdl())) {
			if (hasCustomizedAreaFile()) {
				return ModuleState.CONSTRAINED;
			} else {
				if (Files.exists(projectPath.resolveMapReport())) {
					return ModuleState.MAPREPORT;
				} else {
					return ModuleState.CREATED;
				}
			}
		}

		if (checkUpToDate && !isUpToDate()) {
			return ModuleState.OUTDATED;
		}

		if (Files.exists(projectPath.resolveSmilePath()) && Files.list(projectPath.resolveSmilePath()).count()>1)
			return ModuleState.AVAILABLE;

		return ModuleState.SYNTHESIZED;
	}

	public Path getProjectPath() {
		return projectPath.getPath();
	}

	/**
	 * Check if the module is up to date
	 *
	 * @return true if it is
	 */
	public boolean isUpToDate() {
		Tracer.enterLoop("Up to date Check");
		try {
			//Invoke make in "question mode". Exit code will be 0 if everything is up to date
			ProcessBuilder builder = new ProcessBuilder("make", "-q", MODULE_TARGET_NAME);

			builder.directory(projectPath.getPath().toFile());

			int returnCode = ProcessUtil.runAndWaitFor(builder);

			return Tracer.exit(returnCode == 0);

		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Run the make target "jconfig_update"
	 *
	 * @throws IOException on broken IO
	 */
	@SuppressWarnings("checkstyle:SystemOut")
	@Deprecated
	public void jconfigUpdate() throws IOException {
		Tracer.enterLoop("JConfig Update");
		ProcessBuilder builder = new ProcessBuilder("make", "jconfig_update", "+args=-f config.jc3");
		builder.directory(projectPath.getPath().toFile());

		int exitCode = ProcessUtil.runAndWaitFor(builder, StreamGobbler.shortenToOneLineToStdOut());

		StreamGobbler.clearLineStdout();

		if (exitCode != 0) {
			throw new IllegalStateException("jconfig_update failed for module " + descriptionsName + " located at " + projectPath + "! exit code " + exitCode);
		}
		Tracer.exit();
	}

	/**
	 * Acquire an exclusive lock of the project dir so that no other invocation of rapidSoc may change the content
	 */
	public synchronized void lockDirectory() {
		if (lock != null) {
			return;
		}


		try {
			Files.createDirectories(projectPath.getPath());

			lockFile = projectPath.resolveLockFile();

			FileChannel channel = FileChannel.open(lockFile, StandardOpenOption.WRITE, StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING);
			logger.info(LogMarkers.rapidSoc, "Acquiring lock on " + lockFile);
			lock = channel.lock();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Release a lock acquired by lockDirectory
	 */
	public synchronized void unlockDirectory() {
		if (lock == null) {
			return;
		}

		try {
			lock.release();
			lock = null;
			Files.delete(lockFile);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Take the module to the next state by doing some work
	 *
	 * @param state current state
	 * @return new state
	 */
	private ModuleState moduleStateStep(ModuleState state) throws IOException, JAXBException {
		switch (state) {
			case MISSING:
				return generateProject();

			case CREATED:
				if (constraintCreator.needsUnconstrainedUsageReport()) {
					createSystemFiles();
					return ModuleState.NOCONSTR_SYSTEM_FILES_CREATED;
				} else {
					constraintCreator.createConstraints();
					return ModuleState.CONSTRAINED;
				}

			case NOCONSTR_SYSTEM_FILES_CREATED:
				//Synthesize without constraints
				if (!rapidSocProject.isNoSynthesize()) {
					mapProject();
					return ModuleState.MAPREPORT;
				} else {
					return ModuleState.NOCONSTR_SYSTEM_FILES_CREATED;
				}

			case MAPREPORT:
				constraintCreator.createConstraints();
				return ModuleState.CONSTRAINED;

			case CONSTRAINED:
				createSystemFiles();
				return ModuleState.SYSTEM_FILES_CREATED;

			case SYSTEM_FILES_CREATED:
			case OUTDATED:
				if (!rapidSocProject.isNoSynthesize()) {
					logger.info(LogMarkers.rapidSoc, "Synthesizing for " + state);
					synthesizeProject();
					return ModuleState.SYNTHESIZED;
				} else {
					return state;
				}

			case SYNTHESIZED:
				extractModule();
				return ModuleState.AVAILABLE;

				//Nothing to do
			case AVAILABLE:
				return ModuleState.AVAILABLE;
		}
		throw new RuntimeException("Unknown case: " + state);
	}

	/**
	 * Try to move the module's state towards AVAILABLE
	 *
	 * @return resulting state
	 */
	public ModuleState tryMakeAvailable(boolean checkUpToDate) {
		try {
			lockDirectory();
			ModuleState ms = getModuleState(checkUpToDate);

			logger.info(LogMarkers.rapidSoc, "Module state is {}", ms);

			//Update the state until we're happy
			while (ms != ModuleState.AVAILABLE) {
				ModuleState newState = moduleStateStep(ms);
				logger.info(LogMarkers.rapidSoc, "New Module state is {}", newState);

				//Exit if no change
				if (newState == ms) {
					break;
				} else {
					ms = newState;
				}
			}
			return ms;

		} catch (IOException | JAXBException e) {
			throw new RuntimeException(e);
		} finally {
			unlockDirectory();
		}
	}

	public ModuleState tryMakeAvailable() {
		return tryMakeAvailable(!rapidSocProject.isNoCheckUpToDate());
	}
}
