/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.module_synthesis;

import de.tu_darmstadt.rs.rapidSoc.RapidSocProject;
import de.tu_darmstadt.rs.rapidSoc.directories.HashPath;
import de.tu_darmstadt.rs.rapidSoc.directories.ModuleDirectories;
import de.tu_darmstadt.rs.rapidSoc.directories.ModuleRootPath;
import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.devxml.files.targets.TargetDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.LibJConfigProject;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.LibJConfigBuilder;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.ParameterException;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import edu.byu.ece.rapidSmith.design.Module;
import edu.byu.ece.rapidSmith.design.Net;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Entry point to visualize synthesized modules
 * Created by wenzel on 23.06.16.
 */
@Parameters(commandNames = {"msa", "modsyn"},commandDescription = "Extracts and visualizes a module from xdl")
public class ModuleSynthesisCommand implements Runnable {
	private static final Logger logger = LoggerFactory.getLogger(ModuleSynthesisCommand.class);

	@Parameter(names = {"-m", "--module"}, description = "Module to extract", required = true)
	String module;

	@Parameter(names = {"--hash"}, description = "Hash", required = true)
	String hash;

	@Parameter(names = {"-v", "--variant"}, description = "Variant", required = true)
	String variant;

	@Parameter(names = {"-s", "--showpossible"}, description = "Show possible placements")
	boolean showPossiblePlacements;

	@ParametersDelegate
	private LibJConfigBuilder projectBuilder = new LibJConfigBuilder("1", true);

	public ModuleSynthesisCommand() {

	}

	@Override
	public void run() {
		final ModuleRootPath moduleRootPath = ModuleDirectories.getRootDir();
		HashPath hashPath = moduleRootPath.resolveModulePath(module).resolveHashDir(hash);
		Map<String, String> attributeMap = hashPath.loadAttributeMap();


		LibJConfigProject project = projectBuilder.build();
		Library library = project.getApplication().getLibrary();

		logger.info("Loading modules {}",module);

		List<IModuleDescription> moduleDescriptions = Arrays.stream(module.split("\\+")).map((name) -> {
			final IModuleDescription d = library.getModuleDescription(name);
			if (d == null) {
				throw new ParameterException("Module \"" + name + "\" not found.");
			}
			return d;
		}).collect(Collectors.toList());
		TargetDescription target = library.getTargetDescription("SP601");


		RapidSocProject rapidProject = new RapidSocProject(false, false, project, moduleRootPath, target);

		ModuleSynthesis inst = new ModuleSynthesis(moduleDescriptions, attributeMap, variant, rapidProject);
		Set<Net> partialNets = new HashSet<>();
		Module m = inst.extractModule(true, partialNets, moduleDescriptions.stream().anyMatch(x -> x.getType() == IModuleDescription.ModuleType.Subsystem));
		inst.visualize(m, partialNets, showPossiblePlacements);


	}
}
