/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.module_synthesis.constraints;

import de.tu_darmstadt.rs.rapidSoc.directories.VariantPath;
import de.tu_darmstadt.rs.util.tracer.Tracer;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ResourceUsage {

	private final int slices;
	private final int bram;
	private final int dsp;

	ResourceUsage(int slices, int bram, int dsp) {
		this.slices = slices;
		this.bram = bram;
		this.dsp = dsp;
	}

	public int getSlices() {
		return slices;
	}

	public int getBram() {
		return bram;
	}

	public int getDsp() {
		return dsp;
	}

	public static ResourceUsage analyzeReport(VariantPath projectPath) throws IOException {
		Tracer.enterLoop("read Map Report");
		int requiredSLICES = 0;
		int requiredRAM = 0;
		int requiredDSP = 0;

		try (BufferedReader br = Files.newBufferedReader(projectPath.resolveMapReport())) {

			String tempLine = br.readLine();

			while (tempLine != null) {
				Pattern regexSLICES = Pattern.compile("(Number of occupied Slices:\\s+)(\\d+)(\\s)");
				Matcher matcherSLICES = regexSLICES.matcher(tempLine);
				if (matcherSLICES.find()) {
					String str = matcherSLICES.group(2);
					requiredSLICES += Integer.parseInt(str);
				}

				Pattern regexDSP = Pattern.compile("(Number of DSP.+:\\s+)(\\d+)(\\s)");
				Matcher matcherDSP = regexDSP.matcher(tempLine);
				if (matcherDSP.find()) {
					String str = matcherDSP.group(2);
					requiredDSP += Integer.parseInt(str);
				}

				Pattern regexRAM = Pattern.compile("(Number of RAM.+:\\s+)(\\d+)(\\s)");
				Matcher matcherRAM = regexRAM.matcher(tempLine);
				if (matcherRAM.find()) {
					String str = matcherRAM.group(2);
					requiredRAM += Integer.parseInt(str);
				}

				tempLine = br.readLine();
			}
		}

		//logger.info("SLICES: " + requiredSLICES);
		//logger.info("DSP: " + requiredDSP);
		//logger.info("RAM: " + requiredRAM);
		Tracer.exit();
		return new ResourceUsage(requiredSLICES, requiredRAM, requiredDSP);
	}
}
