/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.module_synthesis.constraints;

import de.tu_darmstadt.rs.rapidSoc.BoundingBox;
import de.tu_darmstadt.rs.rapidSoc.LogMarkers;
import de.tu_darmstadt.rs.rapidSoc.RapidSocProject;
import de.tu_darmstadt.rs.rapidSoc.directories.VariantPath;

import edu.byu.ece.rapidSmith.device.Device;
import edu.byu.ece.rapidSmith.device.TileType;
import edu.byu.ece.rapidSmith.device.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ThorstenConstraintCreator extends RectangularConstraintCreator {
	private static final Logger logger = LoggerFactory.getLogger(ThorstenConstraintCreator.class);

	public ThorstenConstraintCreator(VariantPath projectPath, RapidSocProject project) {
		super(projectPath, project);
	}

	@Override
	protected BoundingBox getConstraintRectangle(ResourceUsage usage) {
		Device device = getProject().getDevice();
		/**
		 * START
		 */
		logger.debug(LogMarkers.constrainer, "getRows: " + device.getRows());
		logger.debug(LogMarkers.constrainer, "getColumns: " + device.getColumns());

		//Ausgabe des Device als Text Tabelle für Entwicklungszwecke...
		/*
		int monkeySpalten = 18;
		int monkeyStart = 15;
		int monkeyStartRow = 4;
		Column[] monkeyColumn = new Column[monkeySpalten];

		for(int j=0;j<monkeySpalten;j++) {
			monkeyColumn[j] = new Column("%s", Column.Alignment.LEFT);
		}

		final TableFormatter monkeyTable = new TableFormatter(TableFormat.CHARACTERS, monkeyColumn);

		String[] monkeyString = new String[monkeySpalten];
		monkeyString[0] = "";
		for(int j=1;j<monkeySpalten;j++) {
			monkeyString[j] = "" + (j + monkeyStart - 1);
		}
		monkeyTable.addHeaderRow(monkeyString);

		String[] monkeyString1 = new String[monkeySpalten];
		String[] monkeyString2 = new String[monkeySpalten];
		String[] monkeyString3 = new String[monkeySpalten];
		String[] monkeyString4 = new String[monkeySpalten];

		monkeyString1[0] = "Tile";
		monkeyString2[0] = "PRIM_0";
		monkeyString3[0] = "PRIM_1";

		for(int i=monkeyStartRow;i<monkeyStartRow+monkeySpalten;i++) {
			for(int j=monkeyStart;j<monkeySpalten+monkeyStart-1;j++) {
				if((device.getTile(i,j).getType().toString()).contains("HCLK"))
					monkeyString1[j-monkeyStart+1] = "HCLK";
				else
					monkeyString1[j-monkeyStart+1] = device.getTile(i,j).getType().toString();
				monkeyString2[j-monkeyStart+1] = "";
				monkeyString3[j-monkeyStart+1] = "";
				if(device.getTile(i,j).getType() != TileType.NULL)
					if(device.getTile(i,j).getPrimitiveSites() != null) {
						monkeyString2[j - monkeyStart+1] += device.getTile(i, j).getPrimitiveSites()[0].getType().toString();
						if(device.getTile(i,j).getPrimitiveSites().length>1)
							monkeyString3[j - monkeyStart+1] += device.getTile(i, j).getPrimitiveSites()[1].getType().toString();
					}
			}
			monkeyTable.addContentRow(monkeyString1);
			monkeyTable.addContentRow(monkeyString2);
			monkeyTable.addContentRow(monkeyString3);
			monkeyTable.addContentRow(monkeyString4);
		}

		monkeyTable.output().forEach(System.out::println);
		*/


		// Sollen RAM und DSP "verschwendet" werden? Sind die beiden Variablen false wird ein Modul, welches zB nur SLICES benötigt sich nicht über
		// einen Bereich mit RAM oder DSP erstrecken.
		boolean wasteRAM = false;
		boolean wasteDSP = false;

		// Devicegröße bestimmen
		int deviceRows = device.getRows();
		int deviceColumns = device.getColumns();

		/*
		Anfang und Ende des Platzierungsbereichs bestimmen
		Gesamtes FPGA durchgehen, Zeile bestimmen mit der größten Anzahl an SLICES (Typ CLEX)
		Erstes Tile dieser Spalte wird Startpunkt
		 */
		int startRow  = 0;
		int startColumn = 0;

		int countSLICE = 0;
		int countRAM = 0;
		int countDSP = 0;
		int maxSLICE = 0;
		int tempColumn = 0;
		int lastCLBColumn = 0;

		for(int row=0; row<deviceRows; row++) {
			for(int column=0; column<deviceColumns; column++) {
				// Tile vom Typ CLB gefunden, zählen und falls erstes Vorkommen in der Zeile -> Spaltenindex speichern
				if(Utils.isCLB(device.getTile(row,column).getType())) {
					countSLICE++;
					if(tempColumn == 0) {
						tempColumn = column;
					}
					if(tempColumn>lastCLBColumn) {
						lastCLBColumn = tempColumn;
					}
				}

				// Enthält die Zeile die meisten SLICES
				if(countSLICE > maxSLICE) {
					startRow = row;
					startColumn = tempColumn;
					maxSLICE = countSLICE;
				}

			}

			tempColumn = 0;
			countSLICE = 0;
		}

		//vom Rand weg bleiben
		startRow++;
		startColumn+=2;

		logger.debug(LogMarkers.constrainer, "startRow: " + startRow);
		logger.debug(LogMarkers.constrainer, "startColumn: " + startColumn);

		int sideA = ( (int) Math.sqrt(usage.getSlices()) ) + 1;

		int row = startRow;
		int column = startColumn;
		countSLICE = 0;

		// wird DSP oder RAM benötigt?
		boolean needRAM = false;
		boolean needDSP = false;

		// wird DSP oder RAM benötigt?
		if(usage.getDsp() > 0) {
			needDSP = true;
			wasteRAM = true;
			wasteDSP = true;
		}
		if(usage.getBram() > 0) {
			needRAM = true;
			wasteRAM = true;
			wasteDSP = true;
		}

		boolean center = false;
		//Breite bestimmen
		while ((countSLICE < sideA || needRAM || needDSP) && !center) {
			// Haben wir ein Tile vom Typ CLB? -> zählen - und zwar immer 2, denn ein CLB enthält 2 CLEXe
			if (Utils.isCLB(device.getTile(startRow, column).getType())) {
				countSLICE += 2;
			}

			if(device.getTile(startRow, column).getName().contains("REG_V")) {
				//Mitte erreicht
				center = true;
				column -= 2; //Index auf letzte brauchbare Spalte zurück setzen
			}

			logger.debug(LogMarkers.constrainer, "startRow / column / countSLICE: " + startRow + " / " + column + " / " + countSLICE + " : " + device.getTile(startRow,column).getType().toString());

			int tempRow = 0;
			// Haben wir ein Tile vom Typ DSP oder RAM? Zählen falls benötigt, ansonsten verschwenden oder frei lassen
			//
			for(int i=1; i<=3; i++) {

				if (device.getTile(startRow, column).getType() == TileType.NULL) {
					tempRow = startRow+i;
				} else {
					tempRow = startRow;
				}

				if (Utils.isBRAM(device.getTile(tempRow, column).getType())) {
					//wir haben RAM gefunden, was tun?
					if(needRAM) {
						needRAM = false;
					} else if (!wasteRAM) {
						countSLICE = 0;
						startColumn = column + 2;
						break;
					}

				}

				if (Utils.isDSP(device.getTile(tempRow, column).getType())) {
					//wir haben DSP gefunden, was tun?
					if(needDSP) {
						needDSP = false;
					} else if (!wasteDSP) {
						countSLICE = 0;
						startColumn = column + 2;
						break;
					}
				}
			}

			column++;
		}
		column--;

		int endColumn = column;

		countSLICE = 0;
		countDSP = 0;
		countRAM = 0;

		//Höhe bestimmen
		while (countSLICE < usage.getSlices() || countDSP < usage.getDsp() || countRAM < usage.getBram()) {
			// Haben wir ein Tile vom Typ CLB? -> zählen
			for(column = startColumn; column<=endColumn; column++) {
				if (Utils.isCLB(device.getTile(row, column).getType())) {
					countSLICE += 2;
					logger.debug(LogMarkers.constrainer, "row / column / countSLICE: " + row + " / " + column + " / " + countSLICE + " : " + device.getTile(row,column).getType().toString());
				}

				if (Utils.isDSP(device.getTile(row, column).getType())) {
					countDSP += 1;
					logger.debug(LogMarkers.constrainer, "row / column / countDSP: " + row + " / " + column + " / " + countDSP + " : " + device.getTile(row,column).getType().toString());
				}

				if (Utils.isBRAM(device.getTile(row, column).getType())) {
					countRAM += 1;
					logger.debug(LogMarkers.constrainer, "row / column / countRAM: " + row + " / " + column + " / " + countRAM + " : " + device.getTile(row,column).getType().toString());
				}
			}

			row++;

			if(row>deviceRows) {
				if(endColumn == lastCLBColumn) {
					throw new RuntimeException("Constraint konnte nicht gebildet werden. Nicht genügend Ressourcen gefunden");
				} else {
					//wir haben nicht alle Ressourcen gefunden
					//breite auf gesamtes FPGA ausdehen und nochmal versuchen Höhe zu bestimmen
					endColumn = lastCLBColumn;
					row = startRow;
					countSLICE = 0;
					countDSP = 0;
					countRAM = 0;
				}
			}
		}
		row--;

		int endRow = row;

		logger.debug(LogMarkers.constrainer, "startRow: " + startRow);
		logger.debug(LogMarkers.constrainer, "startColumn: " + startColumn);
		logger.debug(LogMarkers.constrainer, "endRow: " + endRow);
		logger.debug(LogMarkers.constrainer, "endColumn: " + endColumn);

		/*String constraintsRange = "";

		//Prüfen ob die untere Ecke in Tile mit 2 Primitives ist, falls nicht Zeile verschieben
		if(device.getTile(endRow, endColumn).getPrimitiveSites() != null) {
			constraintsRange = "RANGE=" + device.getTile(startRow, startColumn).getPrimitiveSites()[0].getName() + ":" + device.getTile(endRow, endColumn).getPrimitiveSites()[1].getName();
		} else {
			while(!Utils.isCLB(device.getTile(endRow, endColumn).getType())) {
				endColumn++;
			}
			constraintsRange = "RANGE=" + device.getTile(startRow, startColumn).getPrimitiveSites()[0].getName() + ":" + device.getTile(endRow, endColumn).getPrimitiveSites()[1].getName();
		}

*/
		return new BoundingBox(startColumn, startRow, endColumn, endRow);
	}

	@Override
	public boolean needsUnconstrainedUsageReport() {
		return true;
	}
}
