/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.rapidSoc.parameters;

import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameter;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.util.StreamUtil;

import edu.byu.ece.rapidSmith.design.Instance;
import edu.byu.ece.rapidSmith.design.ModuleInstance;

import java.util.HashMap;
import java.util.Map;

/**
 * Change the Address of DMA devices
 */
@SuppressWarnings("checkstyle:MultipleStringLiterals")
public class DmaAddressChange extends EqTester {

	private static Map<String, String> decoderNames = new HashMap<>();

	static {
		//TODO move this to the module description
		decoderNames.put("usb11", "dmasel0");
		decoderNames.put("ethernet_tx", "pseldma");
		decoderNames.put("ethernet_rx", "pseldma");
	}

	public DmaAddressChange() {

	}

	@Override
	public boolean hasChangeableParameter(IModuleConfiguration module) {
		if (module instanceof IPeripheralModuleConfiguration) {
			IPeripheralModuleConfiguration peri = (IPeripheralModuleConfiguration) module;
			return peri.getModuleDescription().hasMemory();
		} else {
			return false;
		}
	}

	@Override
	public boolean canChangeAttribute(IModuleDescription md, IParameter parameter) {
		//dma address is not modeled as attribute, so return false
		//TODO really??
		return false;
	}

	@Override
	protected long getTestValue(IModuleConfiguration jModule, IBuildProcess buildProcessCache) {
		return buildProcessCache.getGlobalAddressLayout()
				.getLayout((ISubsystemConfiguration) jModule.getParent())
				.findDmaAddressBlock((IPeripheralModuleConfiguration) jModule)
				.orElseThrow(() -> new IllegalStateException("Module has no address"))
				.getActualStart();
	}

	@Override
	protected Instance getMatchInstance(IModuleConfiguration jModule, ModuleInstance rModule) {
		try {
			//TODO create jconfig attribute
			String decoderName = decoderNames.getOrDefault(jModule.getModuleDescriptionName(), "dmasel0");

			Instance inst = rModule.getInstances().stream()
					.filter(i -> i.getName().contains(decoderName))
					.collect(StreamUtil.exactlyOne());
			return inst;
		} catch (RuntimeException e) {
			//TODO add details to exactlyOne
			throw new RuntimeException("error getting decoder in " + rModule.getName(), e);
		}
	}

	@Override
	protected int getBusOffset(String name) {
		if ("bus_addr_block".equals(name)) {
			return 0;
		}
		if ("bus_addr_high".equals(name)) {
			final int high_offset = 10;
			return high_offset;
		}
		throw new RuntimeException("unexpected name: " + name);
	}
}
