/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.rapidSoc.parameters;


import de.tu_darmstadt.rs.spartanmc.devxml.descriptions.parameters.IParameter;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.IModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.generation.documentBuilder.IBuildProcess;
import de.tu_darmstadt.rs.util.StreamUtil;

import edu.byu.ece.rapidSmith.design.Instance;
import edu.byu.ece.rapidSmith.design.ModuleInstance;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Changes the Address of a peripheral to the one specified in jConfig
 */
public class PeriAddressChange extends EqTester {
	private static final Logger logger = LoggerFactory.getLogger(PeriAddressChange.class);

	private static Map<String, String> decoderNames = new HashMap<>();

	static {
		decoderNames.put("usb11", "???");
	}

	public PeriAddressChange() {

	}

	@Override
	public boolean hasChangeableParameter(IModuleConfiguration module) {
		if (module instanceof IPeripheralModuleConfiguration) {
			IPeripheralModuleConfiguration peri = (IPeripheralModuleConfiguration) module;
			return peri.getModuleDescription().hasRegisters();
		} else {
			return false;
		}
	}

	@Override
	public boolean canChangeAttribute(IModuleDescription md, IParameter parameter) {
		//peri address is not modeled as attribute, so return false
		//TODO really??
		return false;
	}

	@Override
	protected long getTestValue(IModuleConfiguration jModule, IBuildProcess buildProcessCache) {
		return buildProcessCache.getGlobalAddressLayout()
				.getLayout((ISubsystemConfiguration) jModule.getParent())
				.findRegAddressBlock((IPeripheralModuleConfiguration) jModule)
				.orElseThrow(() -> new IllegalStateException("Module has no address"))
				.getActualStart();
	}

	private void logInfoOnFail(ModuleInstance rModule) {

		logger.info("Available Instances: {}", rModule.getInstances().stream().map(Instance::getName).collect(Collectors.joining(", ")));

		rModule.getModule().getPorts().stream().filter(p->p.getName().startsWith("addr"))
				.forEach(port -> {
					logger.info("Port {}", port.getName());
					port.getPins().stream()
							.forEach(pin -> {
								logger.info("  Connected to "+pin.getInstance().getName());
							});
				});
	}

	@Override
	protected Instance getMatchInstance(IModuleConfiguration jModule, ModuleInstance rModule) {
		try {
			//TODO create jconfig attribute
			String decoderName = decoderNames.getOrDefault(jModule.getModuleDescriptionName(), "pselect0");

			return rModule.getInstances().stream()
					.filter(i -> i.getName().contains(decoderName))
					.collect(StreamUtil.exactlyOne());
		} catch (RuntimeException e) {
			logInfoOnFail(rModule);
			throw new RuntimeException("error getting decoder in " + rModule.getName(), e);
		}
	}

	@Override
	protected int getBusOffset(String name) {
		if ("bus_addr_block".equals(name)) {
			return 0;
		}
		throw new RuntimeException("unexpected name: " + name);
	}
}
