/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.placer;

import de.tu_darmstadt.rs.rapidSoc.design.PortNet;
import de.tu_darmstadt.rs.rapidSoc.module.ModuleDescriptionInstance;
import de.tu_darmstadt.rs.rapidSoc.module.ModulePlacement;

import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;
import java.util.Set;

/**
 * Log events related to a wrapped {@link ProposedMoveListener}
 */
public class ProposedMoveLogger implements ProposedMoveListener {

	@NotNull
	private static final Logger logger = LoggerFactory.getLogger(ProposedMoveLogger.class);
	@NotNull
	private final ProposedMoveListener pml;
	@NotNull
	protected final String name;

	/**
	 * Constructor for {@link ProposedMoveLogger}
	 * @param pml the {@link ProposedMoveLogger} to wrap
	 * @param name the name to use for logging
	 */
	public ProposedMoveLogger(@NotNull ProposedMoveListener pml, @NotNull String name) {
		this.pml = pml;
		this.name = name;
		logger.info("Setting up ProposedMoveLogger for listener \""+name+"\"");
	}

	@Override
	public void tryMove(@NotNull ModuleDescriptionInstance mdiToChange, @NotNull ModulePlacement newPlacement) {
		logger.info("Attempting to move {} to {}. Notifying {}", mdiToChange, newPlacement, name);
		pml.tryMove(mdiToChange, newPlacement);
	}

	@Override
	public void moveAccepted() {
		logger.info("Move was accepted. Notifying {}", name);
		pml.moveAccepted();
	}

	@Override
	public void moveRejected() {
		logger.info("Move was rejected. Notifying {}", name);
		pml.moveRejected();
	}

	@Override
	public void initialize(@NotNull Collection<ModuleDescriptionInstance> moduleDescriptionInstances, @NotNull Set<PortNet> portNets) {
		logger.info("Initialize the design for {}", name);
		pml.initialize(moduleDescriptionInstances, portNets);
	}
}
