/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.rapidSoc.placer.overlap;

import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.tu_darmstadt.rs.rapidSoc.placer.ProposedMoveLogger;

/**
 * Logs the result of a wrapped {@link OverlapDetector}
 */
public class OverlapDetectorLogger extends ProposedMoveLogger implements OverlapDetector {

	@NotNull
	private static final Logger logger = LoggerFactory.getLogger(OverlapDetectorLogger.class);
	@NotNull
	private final OverlapDetector od;

	private int accepted = 0;
	private int rejected = 0;

	/**
	 * Constructor for {@link OverlapDetectorLogger}
	 * @param od the {@link OverlapDetector} to wrap
	 * @param name the name to use for logging
	 */
	public OverlapDetectorLogger(@NotNull OverlapDetector od, @NotNull String name) {
		super(od, name);
		this.od = od;
		logger.info("Setting up OverlapDetectorLogger for detector \""+name+"\"");
	}

	/**
	 * Constructor for {@link OverlapDetectorLogger}. Uses od.toString() to get the name for logging
	 * @param od the {@link OverlapDetector} to wrap
	 */
	public OverlapDetectorLogger(@NotNull OverlapDetector od) {
		this(od, od.toString());
	}

	@Override
	public boolean isMoveOk() {
		if (od.isMoveOk()) {
			logger.info("Move accepted by {} ({} total)", name, ++accepted);
			return true;
		} else {
			logger.info("Move rejected by {} ({} total)", name, ++rejected);
			return false;
		}
	}
}
