/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.rapidSoc.module_synthesis.constraints

import de.tu_darmstadt.rs.rapidSoc.BoundingBox
import de.tu_darmstadt.rs.rapidSoc.RapidSocProject
import de.tu_darmstadt.rs.rapidSoc.module_synthesis.RapidSocTestingProjectProvider
import edu.byu.ece.rapidSmith.device.PrimitiveType
import spock.lang.Specification
import spock.lang.Unroll

class RectangularConstraintCreatorSpec extends Specification {

	//Derive a class from RectangularConstraintCreator to make constrainType public
	class TestingRectangularConstraintCreator extends RectangularConstraintCreator {

		protected TestingRectangularConstraintCreator(RapidSocProject project) {
			super(null, project)
		}

		@Override
		protected BoundingBox getConstraintRectangle(ResourceUsage usage) {
			return null
		}

		//Make this public
		@Override
		public String constrainType(PrimitiveType type, BoundingBox constraintCorners) {
			return super.constrainType(type, constraintCorners)
		}

		@Override
		boolean needsUnconstrainedUsageReport() {
			return true
		}
	}

	@Unroll
	def "Correct constraints for #tilesToInclude #primitiveType"(List<String> tilesToInclude, PrimitiveType primitiveType, String targetName, String output) {
		expect:
		def project = RapidSocTestingProjectProvider.getInstance().getRapidSocProject(null, targetName, null, null)
		RectangularConstraintCreator cc = new TestingRectangularConstraintCreator(project)
		BoundingBox bb = tilesToInclude.stream().map {
			project.getDevice().getTile(it)
		}.collect(BoundingBox.getTileBoundingBox())

		cc.constrainType(primitiveType, bb) == output


		where:
		tilesToInclude                                      | primitiveType            | targetName | output
		//The following testcases' outputs are equal to PlanAhead's output
		["CLEXL_X7Y23", "CLEXL_X11Y14", "CLEXM_X8Y8"]       | PrimitiveType.SLICEX     | "SP601"    | "SLICE_X16Y9:SLICE_X17Y22, SLICE_X8Y8:SLICE_X15Y23"
		["CLEXL_X7Y23", "CLEXL_X11Y14", "CLEXM_X8Y8"]       | PrimitiveType.RAMB8BWER  | "SP601"    | ""
		["CLEXL_X7Y24", "CLEXL_X11Y18", "CLEXM_X10Y8"]      | PrimitiveType.SLICEX     | "SP601"    | "SLICE_X16Y9:SLICE_X17Y22, SLICE_X8Y8:SLICE_X15Y24"
		["CLEXL_X7Y21", "CLEXL_X11Y18"]                     | PrimitiveType.SLICEX     | "SP601"    | "SLICE_X8Y18:SLICE_X17Y21"

		["CLEXL_X1Y23", "CLEXL_X11Y18"]                     | PrimitiveType.SLICEX     | "SP601"    | "SLICE_X0Y23:SLICE_X17Y18"
		["CLEXL_X1Y23", "CLEXL_X11Y18"]                     | PrimitiveType.DSP48A1    | "SP601"    | "DSP48_X0Y5:DSP48_X0Y5"
		["CLEXL_X1Y23", "CLEXL_X11Y18"]                     | PrimitiveType.RAMB16BWER | "SP601"    | "RAMB16_X0Y10:RAMB16_X0Y10"
		["CLEXL_X1Y23", "CLEXL_X11Y18"]                     | PrimitiveType.RAMB8BWER  | "SP601"    | "RAMB8_X0Y10:RAMB8_X0Y11"

		["CLEXL_X11Y27", "CLEXL_X15Y23"]                    | PrimitiveType.SLICEX     | "SP601"    | "SLICE_X16Y27:SLICE_X25Y23"


		["CLEXM_X10Y40", "CLEXL_X18Y94", "CLEXM_X15Y103"]   | PrimitiveType.SLICEX     | "SP605"    | "SLICE_X28Y41:SLICE_X29Y102, SLICE_X14Y40:SLICE_X27Y103"

		//The following two results are not equal to what PlanAhead would generate, but nevertheless correct
		["CLEXM_X15Y106", "CLEXL_X30Y106", "CLEXL_X20Y123"] | PrimitiveType.SLICEX     | "SP605"    | "SLICE_X26Y123:SLICE_X51Y106, SLICE_X22Y106:SLICE_X25Y119"
		["CLEXM_X15Y106", "CLEXL_X30Y106", "CLEXM_X19Y119"] | PrimitiveType.SLICEX     | "SP605"    | "SLICE_X24Y119:SLICE_X51Y106, SLICE_X22Y106:SLICE_X23Y111"
	}
}