/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

import edu.byu.ece.rapidSmith.design.Design;
import edu.byu.ece.rapidSmith.design.Instance;
import edu.byu.ece.rapidSmith.device.PrimitiveSite;
import edu.byu.ece.rapidSmith.device.PrimitiveType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.Point;
import java.awt.Rectangle;
import java.nio.file.Paths;
import java.util.function.Supplier;
import java.util.stream.IntStream;
import java.util.stream.Stream;

/**
 * Testing: Fill a design with keepout stuff
 * Created by jwenzel on 19.10.15.
 */
public class FillKeepout {
	private static final Logger logger = LoggerFactory.getLogger(FillKeepout.class);

	private final Design design;

	/**
	 * Create fillkeepout
	 */
	@SuppressWarnings("checkstyle:MagicNumber")
	public FillKeepout() {
		design = new Design();
		design.loadXDLFile(Paths.get("test/spillout/top_map.xdl"));

		Rectangle r = new Rectangle(10, 32, 4, 16);


		logger.info(rectToBoundsStr(r, "SLICE"));


		getPointsAroundRect(r)
				.map(this::primitiveSiteFromPoint)
				.forEach(p -> {
					Instance i = new Instance("keepout_" + p, PrimitiveType.SLICEX);
					i.setSite(p);
					design.addInstance(i);
				});


		design.saveComparableXDLFile(Paths.get("test/spillout/top_keepout.xdl"));
	}

	public static void main(String[] args) {
		new FillKeepout();
	}

	private Stream<Point> getPointsAroundRect(Rectangle r) {
		//All xs around rect, including corners
		Supplier<IntStream> xs = () -> IntStream.rangeClosed(r.x - 1, r.x + r.width);
		//All ys around rect, excluding corners
		Supplier<IntStream> ys = () -> IntStream.range(r.y, r.y + r.height);

		//concat only supports two elements, not any number. using flatMap, we avoid nested concats
		return Stream.of(
				xs.get().mapToObj(x -> new Point(x, r.y - 1)),
				xs.get().mapToObj(x -> new Point(x, r.y + r.height)),
				ys.get().mapToObj(y -> new Point(r.x - 1, y)),
				ys.get().mapToObj(y -> new Point(r.x + r.width, y))
		).flatMap(s -> s);
	}

	private String rectToBoundsStr(Rectangle r, String primType) {
		return primType + "_X" + r.x + "Y" + r.y + ":" + primType + "_X" + (r.x + r.width - 1) + ":" + (r.y + r.height - 1);
	}

	private PrimitiveSite primitiveSiteFromPoint(Point p) {
		String str = "SLICE_X" + p.x + "Y" + p.y;
		PrimitiveSite s = design.getDevice().getPrimitiveSites().get(str);
		if (s == null) {
			throw new RuntimeException("site not found: " + str);
		}
		return s;
	}
}
