/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

import edu.byu.ece.rapidSmith.design.Attribute;
import edu.byu.ece.rapidSmith.design.Design;
import edu.byu.ece.rapidSmith.design.Instance;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Testing: Fill Simulation Blockrams
 * Created by jakobw on 15.12.15.
 */
public class FillSimBlockram {
	private static final Logger logger = LoggerFactory.getLogger(FillSimBlockram.class);

	public FillSimBlockram() {

	}

	/**
	 * Main
	 * @param args arguments
	 * @throws IOException IO Exception
	 */
	public static void main(String[] args) throws IOException {
		Design design = new Design();
		design.loadXDLFile(Paths.get(args[0]));

		PrintStream out = new PrintStream(new FileOutputStream("output.v"));

		BufferedReader in = new BufferedReader(new FileReader(args[1]));

		List<String> ramDefinition = null;
		boolean inRamDefinition = false;
		String line;
		while ((line = in.readLine()) != null) {
			if (!inRamDefinition) {
				if ("  X_RAMB16BWER #(".equals(line)) {
					ramDefinition = new ArrayList<>();
					ramDefinition.add(line);
					inRamDefinition = true;
				}
				out.println(line);
			} else {
				ramDefinition.add(line);
				if ("  );".equals(line)) {

					insertRamContent(ramDefinition, out, design);

					inRamDefinition = false;
				}
			}
		}

	}

	private static void insertRamContent(List<String> ramDefinition, PrintStream out, Design design) {
		//First we need to find the line with the ram name
		String nameLine = null;
		for (int i = 0; i < ramDefinition.size(); i++) {
			String line = ramDefinition.get(i);
			if (line.endsWith("))")) {
				nameLine = ramDefinition.get(i + 1);
				break;
			}
		}
		if (nameLine == null) {
			throw new RuntimeException("did not find name");
		}

		//Cut out name
		String name = nameLine.substring(0, nameLine.length() - 1).trim().substring(1);

		//Find instance in design
		Instance inst = design.getInstance(name);

		logger.info(name + ": " + inst.getName());

		Pattern p = Pattern.compile("\\s*\\.(INIT\\S*) .*");
		ramDefinition.stream().map(line -> {
			Matcher m = p.matcher(line);
			if (m.matches()) {
				String attrName = m.group(1);

				Attribute attr = inst.getAttribute(attrName);


				final int bitsPerHexChar = 4;
				String result = String.format("    .%s ( %d'h%s ),",
						attrName,
						attr.getValue().length() * bitsPerHexChar,
						attr.getValue());

				if (!line.equals(result)) {
					logger.info("changing\n" + line + "to\n" + result);
				}

				return result;
			} else {
				return line;
			}
		}).forEach(out::println);
	}
}
