package dispatcher;

import com.trolltech.qt.gui.*;
import com.trolltech.qt.*;
import com.trolltech.qt.core.*;

import dispatcher.gui.*;
import dispatcher.model.*;
import dispatcher.log.*;
import dispatcher.exception.*;

import libjconfig.common.Log;
import libjconfig.common.LogTitle;

/**
 * Dispatcher entry class. Instantiates all modules such as the application logic, the GUI components
 * and sets up all necessary application threads.
 */
public final class Dispatcher extends QRunnable implements Runnable {
    private Interaction i;
    private NewServer server;
    public String project_file = null;
    private static final LogTitle lt = Log.get("Dispatcher");

    public Dispatcher() {
        i = new Interaction();
    }

    public void run() {
        i.wnd = new QWidget();
        i.ui = new Dispatcher_widget();
        i.ui.setupUi(i.wnd);
        i.log_appender = new QtLogAppender();
        Log.addAppender(i.log_appender.appender);
        i.wnd.show();
        i.open_dialog = new PrjOpen();
        QtJConfigReader r = new QtJConfigReader();
        i.reader = r;
        i.model = new QtDispatcherModel();
        i.server = new NewServer(i.model);
        i.serverlogic = new ServerLogic(i.model, i.server);
        i.initialize();
        i.server.start();
        i.serverlogic.start();
        try {
            if (project_file != null)
                r.parseConfiguration(project_file);
        } catch (ApplicationError e) {
        }
    }

    public static void main(String... args) {
        Dispatcher d;
        QApplication.initialize(args);
        d = new Dispatcher();
        if (args.length > 0)
            d.project_file = args[0];
        d.run();
        QApplication.execStatic();
    }
}
