package dispatcher;

import java.util.*;

import com.trolltech.qt.gui.*;
import com.trolltech.qt.core.*;

import dispatcher.model.*;
import dispatcher.gui.*;
import dispatcher.log.*;

/**
 * This class defines the interactional model of the dispatcher.
 * It connects the application components to one another like the
 * model with the GUI elements.
 *
 * @author Michael Raitza
 */
public class Interaction {
    public QtDispatcherModel model;
    public PrjOpen open_dialog;
    public QWidget wnd;
    public Dispatcher_widget ui;
    public QtJConfigReader reader;
    public GuiController gui_ctrl;
    public NewServer server;
    public ServerLogic serverlogic;
    public QtLogAppender log_appender;

    public Interaction() {
    }

    public void initialize() {
        gui_ctrl = new GuiController(ui, model);
        QApplication.instance().lastWindowClosed.connect(server, "_stop()");
        QApplication.instance().lastWindowClosed.connect(serverlogic, "_stop()");
        QApplication.instance().lastWindowClosed.connect(model.timer(), "dissemble()");
        QApplication.instance().lastWindowClosed.connect(log_appender.appender, "dissemble()");
        open_dialog.file_opened.connect(reader, "parseConfiguration(String)");
        reader.configuration_parsed.connect(model, "load(Collection,Collection,Collection)");
        model.document_data.connect(gui_ctrl, "doc_load(boolean,Collection,Collection,Collection)");
        model.sim_state_changed.connect(server, "sim_state(SimState)");
        model.sim_state_changed.connect(serverlogic, "sim_state(SimState)");
        model.incoming_message.connect(serverlogic, "_resume()");

        /* ui signal connections, must be queued when crossing thread boundaries */
        log_appender.log_entry.connect(gui_ctrl, "append_log(String)", Qt.ConnectionType.QueuedConnection);
        model.sim_state_changed.connect(gui_ctrl, "sim_state(SimState)", Qt.ConnectionType.QueuedConnection);
        model.sim_state_changed.connect(model.timer(), "sim_state(SimState)");
        model.client_connection.connect(gui_ctrl, "connection(ClientModel,boolean)", Qt.ConnectionType.QueuedConnection);
        ui.load_button.clicked.connect(open_dialog, "show_dialog(boolean)");
        ui.run_button.clicked.connect(gui_ctrl, "start_sim(boolean)");
        ui.stop_button.clicked.connect(gui_ctrl, "stop_sim(boolean)");
        ui.reset_button.clicked.connect(serverlogic, "reset_sim(boolean)");
        ui.step_button.clicked.connect(serverlogic, "step_sim(boolean)");
        ui.main_list.currentItemChanged.connect(gui_ctrl, "main_list_item(QListWidgetItem,QListWidgetItem)");
        ui.mode_combobox.currentIndexChanged.connect(gui_ctrl, "sim_speed(int)");

        /* connections to the timer */
        model.timer().register_task(new Runnable() {
                public void run() {
                    serverlogic.send_step();
                } });


        /* some initialisation */
        model.document_loaded();
        log_appender.appender.setup();
        log_appender.appender.update();
    }
}
