package dispatcher;

import java.util.List;
import java.util.ArrayList;

/**
 * Reentrant fair spin lock. PFE jenkov.com
 */
public class SpinLock {

    private static class QueueObject {
        private boolean notified = false;

        public synchronized void _wait() throws InterruptedException {
            while (!notified)
                wait();
            notified = false;
        }

        public synchronized void _notify() {
            notified = true;
            notify();
        }

        public boolean equals(Object o) {
            return this == o;
        }
    }

    private boolean locked = false;
    private int cnt = 0;
    private List<QueueObject> waiting = new ArrayList<QueueObject>();
    private Thread t = null;

    public void lock() throws InterruptedException {
        QueueObject obj = new QueueObject();
        Thread c = Thread.currentThread();
        boolean locked_here = true;
        synchronized(this) {
            waiting.add(obj);
        }

        while (locked_here) {
            synchronized(this) {
                locked_here = (c != t) && (locked || waiting.get(0) != obj);
                if (!locked_here) {
                    locked = true;
                    waiting.remove(obj);
                    t = Thread.currentThread();
                    cnt++;
                    return;
                }
            }
            try {
                obj._wait();
            } catch (InterruptedException e) {
                synchronized(this) {
                    waiting.remove(obj);
                }
                throw e;
            }
        }
    }

    public synchronized void unlock() {
        if (t != Thread.currentThread())
            throw new IllegalMonitorStateException("Unlocked by foreign thread");

        cnt--;
        if (cnt == 0) {
            locked = false;
            t = null;
            if (waiting.size() > 0)
                waiting.get(0)._notify();
        }
    }
}
