package dispatcher.gui;

import java.util.*;
import com.trolltech.qt.*;
import com.trolltech.qt.gui.*;
import com.trolltech.qt.core.*;

import dispatcher.model.*;

public class GuiICModel extends QAbstractTableModel {

    private List<InterconnectModel> ics;
    private ClientModel c;
    private static final String[] col_names = new String[]{ "Base", "Width", "Depth", "Partner", "Xmit" };
    private String[] row_names;

    public GuiICModel(List<InterconnectModel> ics, ClientModel c) {
        this.ics = ics;
        this.c = c;
        int cnt = 0;
        row_names = new String[ics.size()];
        for (InterconnectModel ic : ics) {
            String s = c.id() == ic.master().id() ? "out" : "in";
            row_names[cnt++] = s;
        }
    }

    public int rowCount(QModelIndex parent) {
        return ics.size();
    }

    public int columnCount(QModelIndex parent) {
        return col_names.length;
    }

    public Object headerData(int section, Qt.Orientation orientation, int role) {
        if (role != Qt.ItemDataRole.DisplayRole)
            return null;

        if (orientation == Qt.Orientation.Horizontal)
            return col_names[section];
        else
            return row_names[section];
    }

    public Object data(QModelIndex index, int role) {
        if (role == Qt.ItemDataRole.DisplayRole) {
            InterconnectModel ic = ics.get(index.row());
            switch(index.column()) {
            case 0:
                return (new Formatter()).format("0x%x", c.equals(ic.master())?
                                                ic.master_base():
                                                ic.slave_base()).toString();
            case 1:
                return (new Integer(ic.width())).toString();
            case 2:
                return (new Integer(ic.depth())).toString();
            case 3:
                return c.equals(ic.master())? ic.slave().name(): ic.master().name();
            case 4:
                return "0";
            default:
                return null;
            }
        }
        if (role == Qt.ItemDataRole.TextAlignmentRole) {
            switch(index.column()) {
            case 0:
            case 3:
                return Qt.AlignmentFlag.AlignLeft;
            case 1:
            case 2:
            case 4:
                return Qt.AlignmentFlag.AlignRight;
            }
        }
        return null;
    }
    public String toString() {
        String blubb = "";
        for (InterconnectModel ic : ics) {
            blubb += ic.master().equals(c)? "master:" + Integer.toHexString(ic.master_base()): "slave:" + Integer.toHexString(ic.slave_base()) + "\n";
        }
        return blubb;
    }
}
