package dispatcher.model;

import java.util.*;
import java.net.InetAddress;

import dispatcher.exception.*;

public class ClientModel {
    private int id;
    private String clientstr;
    private String name;
    private InetAddress address;
    private int port;
    private Map<Integer,InterconnectModel> ics = new HashMap<Integer,InterconnectModel>();
    private Map<Integer,SharedMemoryModel> shms = new HashMap<Integer,SharedMemoryModel>();

    public void changeId(int id) {
        this.id = id;
    }

    public void setName(String name) {
        this.name = name;
        clientstr = name + "(0x" + Integer.toHexString(id) + ")";
    }

    public int id() {
        return id;
    }

    public String name() {
        return name;
    }

    public String toString() {
        return clientstr;
    }

    public void address(InetAddress addr) {
        address = addr;
    }

    public InetAddress address() {
        return address;
    }

    public void port(int pt) {
        port = pt;
    }

    public int port() {
        return port;
    }

    public void addIC(InterconnectModel ic, int base) {
        ics.put(base, ic);
    }

    public void removeIC(int base) {
        ics.remove(base);
    }

    public void addShm(SharedMemoryModel shm, int base) {
        shms.put(base, shm);
    }

    public void removeShm(SharedMemoryModel shm, int base) {
        shms.remove(base);
    }

    public ClientModel get_partner(int base_address) throws ApplicationError {
        return ic(base_address).get_partner(this);
    }

    public InterconnectModel ic(int base_address) throws ApplicationError {
        InterconnectModel ic = ics.get(base_address);
        if (ic == null)
            throw new ApplicationError(String.format("Unknown base address to access FIFO: %d", base_address));
        return ic;
    }

    public List<InterconnectModel> interconnects() {
        List<InterconnectModel> l = new ArrayList<InterconnectModel>(ics.size());
        for (InterconnectModel ic : ics.values())
            l.add(ic);
        return l;
    }

    public Map<Integer,InterconnectModel> ic_links() {
        return ics;
    }

    public Map<Integer,SharedMemoryModel> shm_links() {
        return shms;
    }

    public List<SharedMemoryModel> shms() {
        List<SharedMemoryModel> l = new ArrayList<SharedMemoryModel>();
        for (SharedMemoryModel shm : shms.values())
            l.add(shm);
        return l;
    }

    public SharedMemoryModel get_shm(int base_address) throws ApplicationError {
        SharedMemoryModel shm = shms.get(base_address);
        if (shm == null)
            throw new ApplicationError(String.format("Unknown base address to access SHM: %d", base_address));
        return shm;
    }
}
