package dispatcher.model;

import java.util.Queue;
import java.util.LinkedList;
import java.io.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.Socket;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;
import java.nio.channels.ClosedChannelException;
import java.nio.ByteBuffer;

import libsim.*;

import libjconfig.common.Log;
import libjconfig.common.LogTitle;

import dispatcher.exception.*;
import dispatcher.SpinLock;

public class Communication extends DThread {
    private static int thread_count = 0;
    private Thread t = this;
    private NewServer srv;
    private SelectionKey key = null;
    private LogTitle lt;
    private Transaction transaction = null;

    public Communication() {
        super("Communication(" + thread_count++ + ")");
    }

    protected void _run_init() {
        lt = Log.get(t.getName());
    }

    protected void _run() {
        boolean res;
        SocketChannel sc = (SocketChannel)key.channel();
        try {
            if (transaction == null)
                throw new RuntimeException("Transaction null");

            if (transaction.send) {
                sc.write(transaction.msg_buf);
                res = transaction.msg_buf[0].remaining() == 0;
            } else {
                transaction.msg = MessageType.deserialize(sc, transaction.msg_buf);
                res = transaction.msg != null;
            }
            if (res) {
                /*
                Log.debug(lt, "Handled %s message: %s",
                          transaction.send? "outbound": "inbound",
                          transaction.msg.format_msg());
                */
                /*
                if (!transaction.send)
                    System.out.printf("%s: Received %s%n", this.getName(), transaction.msg.format_msg());
                */
                srv.worker_finished(key, transaction.send);
            }
        } catch (ClosedChannelException e) {
            srv.client_terminated(key, transaction.send);
        } catch (IOException e) {
            srv.worker_net_error(key, transaction.send);
        } catch (UnknownMessage e) {
            srv.worker_net_error(key, transaction.send);
        } finally {
            srv.worker_deregister(key, transaction.send);
            _suspend();
        }
    }

    public synchronized void _suspend() {
        transaction = null;
        super._suspend();
    }

    public void server(NewServer srv) {
        this.srv = srv;
    }

    public void transaction(SelectionKey key, Transaction tn) {
        this.key = key;
        if (tn == null)
            throw new NullPointerException();
        this.transaction = tn;
    }
}
