package dispatcher.model;

import libsim.ImplementationError;

public class InterconnectModel {
    private ClientModel master;
    private ClientModel slave;
    private int width;
    private int depth;
    private int master_base;
    private int slave_base;

    /**
     * \brief Sets the master client for this FIFO.
     *
     * Does all the (de-)registration in the client.
     */
    public void setMaster(ClientModel c, int base_addr) {
        if (master != null)
            master.removeIC(master_base);
        c.addIC(this, base_addr);
        master = c;
        master_base = base_addr;
    }

    /**
     * \brief Sets the slave client for this FIFO.
     *
     * Does all the (de-)registration in the client.
     */
    public void setSlave(ClientModel c, int base_addr) {
        if (slave != null)
            slave.removeIC(slave_base);
        c.addIC(this, base_addr);
        slave = c;
        slave_base = base_addr;
    }

    public void setFifoWidth(int w) {
        width = w;
    }

    public void setFifoDepth(int d) {
        depth = d;
    }

    public ClientModel master() {
        return master;
    }

    public ClientModel slave() {
        return slave;
    }

    public int master_base() {
        return master_base;
    }

    public int slave_base() {
        return slave_base;
    }

    public int depth() {
        return depth;
    }

    public int width() {
        return width;
    }

    public boolean isValid() {
        return master != null && slave != null;
    }

    public ClientModel get_partner(ClientModel c) {
        if (c.equals(master))
            return slave;
        else if (c.equals(slave))
            return master;
        throw new ImplementationError("ClientModel not part of this interconnect: " + c.toString());
    }

    public int base(ClientModel c) {
        if (c.equals(master))
            return master_base;
        else if (c.equals(slave))
            return slave_base;
        throw new ImplementationError("ClientModel not part of this interconnect: " + c.toString());
    }
}
