package dispatcher.model;

import java.util.*;

public class ThreadPool {

    private List<Communication> ready;
    private Set<Communication> pool;
    private int max_entries;
    private int entries = 0;
    private boolean inactive;
    private Timer t;
    private long cycle = 5000;

    public ThreadPool() {
        this(25);
    }

    public ThreadPool(int max_threads) {
        max_entries = max_threads;
        ready = Collections.synchronizedList(new LinkedList<Communication>());
        pool = Collections.synchronizedSet(new HashSet<Communication>());
        t = new Timer("ThreadPool Timer");
        t.scheduleAtFixedRate(new TimerTask() {
                public void run() {
                    Communication t = null;
                    synchronized(ThreadPool.this) {
                        if (!inactive || entries == 0) {
                            inactive = true;
                            return;
                        }
                        while (ready.size() > 0 && (t = ready.remove(0)) != null) {
                            t._stop();
                            entries--;
                            pool.remove(t);
                        }
                        Iterator<Communication> it = pool.iterator();
                        while (it.hasNext()) {
                            t = it.next();
                            if (Thread.State.WAITING.equals(t.getState())) {
                                t._stop();
                                entries--;
                                it.remove();
                            }
                        }
                    }
                } }, cycle, cycle);
    }

    /**
     * Returns a ready thread.
     */
    public Communication get() {
        Communication thread = null;

        synchronized(this) {
            inactive = false;
            for (int i = 0; i < ready.size(); i++) {
                thread = ready.remove(0);
                if (thread != null) {
                    if (thread.getState().equals(Thread.State.WAITING))
                        return thread;
                    else
                        ready.add(thread);
                }
            }
        }

        if (entries >= max_entries)
            return null;

        thread = new Communication();
        pool.add(thread);
        entries++;
        return thread;
    }

    /**
     * Marks a thread as ready.
     *
     * @throws IllegalArgumentException if this thread is not part of this pool.
     */
    public void ready(Communication thread) {
        if (!pool.contains(thread))
            throw new IllegalArgumentException("No such thread in pool: " + thread);

        synchronized(this) {
            ready.add(thread);
        }
    }

    public void remove(Communication thread) {
        synchronized(this) {
            pool.remove(thread);
            ready.remove(thread);
        }
    }

    public void status() {
        System.err.printf("%s%n%s%n", pool, ready);
    }

    public synchronized void dissemble() {
        t.cancel();
        for (Communication cc : pool)
            cc._stop();
        pool.clear();
        ready.clear();
    }
}
