/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

package de.tu_darmstadt.rs.sim.application;

import de.tu_darmstadt.rs.jcommander.JCommanderRunner;
import de.tu_darmstadt.rs.sim.engine.AccessManager;
import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.JConfigDocumentManager;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimControlSelector;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.SimulationThread;
import de.tu_darmstadt.rs.sim.engine.net.MpsocManager;
import de.tu_darmstadt.rs.sim.gdb.SimulationManagerGDBInterface;
import de.tu_darmstadt.rs.sim.gui.GuiDefinitions_I;
import de.tu_darmstadt.rs.sim.gui.GuiMessageHandler;
import de.tu_darmstadt.rs.sim.gui.MainWindow;
import de.tu_darmstadt.rs.sim.gui.PreferencesManager;
import de.tu_darmstadt.rs.sim.gui.RefreshThread;
import de.tu_darmstadt.rs.sim.preferences.ProjectManager;
import de.tu_darmstadt.rs.spartanmc.jconfig.LibJConfigProject;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IDocumentHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.LibJConfigBuilder;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.ParametersDelegate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import javax.swing.UIManager;
import javax.xml.bind.JAXBException;

/**
 * loader of the simulator
 */
public class SimApplication implements GuiDefinitions_I, Runnable {

	private static final Logger logger = LoggerFactory.getLogger(SimApplication.class);
	public static final String PROGRAM_NAME = "sim";

	@Parameter(names = {"--host"}, description = "")
	private String host = "localhost";

	@ParametersDelegate
	private LibJConfigBuilder libJConfigBuilder = new LibJConfigBuilder("SpartanMC Simulator 1.7.1", true);

	public static void main(String[] args) throws IOException, JAXBException {
		JCommanderRunner.create(new SimApplication())
				.setProgramName(PROGRAM_NAME)
				.run(args);
	}

	private IDocumentHandler handler;

	public void run() {

		UIManager.put("FileChooser.readOnly", Boolean.TRUE);
		final ProjectManager projectManager = new ProjectManager();
		final GuiMessageHandler guiMessageHandler = new GuiMessageHandler();
		final RefreshThread refreshThread = new RefreshThread(100);
		final SimulationThread simulationThread = new SimulationThread();
		final SimControlSelector simControlSelector = new SimControlSelector(simulationThread);
		final MpsocManager mpsocManager = new MpsocManager(host, 8886, simControlSelector, simulationThread, guiMessageHandler);
		final AddressSpaceManager addressSpaceManager = new AddressSpaceManager(refreshThread, AccessManager.getTheInstance());
		final SimulationManager simulationManager = new SimulationManagerGDBInterface(mpsocManager, refreshThread, simControlSelector, mpsocManager, guiMessageHandler, projectManager, addressSpaceManager, simulationThread);
		libJConfigBuilder.useLastBuiltConfig();
		libJConfigBuilder.autoOpenConfig();
		libJConfigBuilder.executeWithProject(prj -> {
			handler = prj.getDocumentHandler();
			final JConfigDocumentManager documentManager = new JConfigDocumentManager(simulationManager, handler);
			final MainWindow wnd = new MainWindow(simulationManager, simControlSelector, projectManager, guiMessageHandler,
					documentManager, mpsocManager);
			simulationManager.setupSimulation();

			PreferencesManager prefm = new PreferencesManager(wnd, documentManager);
			projectManager.registerComponent(Settings.getInstance());
			projectManager.registerComponent(prefm);
			projectManager.registerComponent(wnd);

			guiMessageHandler.setParent(wnd);
			wnd.setVisible(true);
			simulationThread.setSimulationManager(simulationManager);
			simulationThread.start();
		});

		LibJConfigProject project = libJConfigBuilder.build();

	}
}
