/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 */
package de.tu_darmstadt.rs.sim.console;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;

import java.io.File;
import java.util.StringTokenizer;

/**
 * The OptionParser parse the command line and store the result.
 */
public class OptionParser {

	private File memoryFile = null;
	private File jconfigFile = null;
	private boolean runInstant = false;
	private int subsystemNumber = 0;

	/**
	 * Parse the given command line and store the result. The different options
	 * can be accessed by the getXXX methods.
	 * <p>
	 * supported options -f, -h, -r
	 *
	 * @param args
	 *            command line string.
	 */
	public void parseCmdLine(final String[] args) {
		boolean fileOpt = false;
		boolean jconfigOpt = false;
		boolean subsystemOpt = false;
		boolean connect = false;

		for (String str : args) {
			// print help
			if (str.equals("-h")) {
				printHelp(); // call System.exit
			}

			// memoryFile options
			if (fileOpt) {
				memoryFile = new File(str);
				fileOpt = false;
			} else if (jconfigOpt) {
				jconfigFile = new File(str);
				jconfigOpt = false;
			} else if (subsystemOpt) {
				try {
					subsystemNumber = Integer.parseInt(str);
				} catch (NumberFormatException e) {
					System.err
							.println("the entered value for choosen subsystem is not a number");
					System.exit(-1);
				}

			} else if (connect) {
				StringTokenizer st = new StringTokenizer(str, ":");
				Settings.setSetting(Settings.SERVER_HOST, st.nextToken());
				try {
					Settings.setSetting(Settings.SERVER_PORT,
							Integer.parseInt(st.nextToken()));
				} catch (NumberFormatException e) {
					System.err
							.println("the entered value is not a valid port number");
					System.exit(-1);
				}
				connect = false;

			} else if (str.equals("-f")) {
				fileOpt = true;
			} else if (str.equals("-j")) {
				jconfigOpt = true;
			} else if (str.equals("-s")) {
				subsystemOpt = true;
			} else if (str.equals("-r")) {
				runInstant = true;
			} else if (str.equals("-c")) {
				connect = true;
			}
		}
	}

	/**
	 * Checks if all required options are set.
	 *
	 * @return true if all requirements are met, false otherwise
	 */
	public boolean requiredOptionsSet() {
		return memoryFile != null;
	}

	/**
	 * Print the help message and exit program.
	 */
	public void printHelp() {
		System.out
				.println("Spartan MC Simulator "
						+ SpartanDefinitions_I.PROGRAM_VERSION_NUMBER
						+ "\n"
						+ "-f <file>	loads the file into the de.tu_darmstadt.rs.sim.main memory of the simulator (required)\n"
						+ "-j <jconfig file> import configuration from JConfig\n"
						+ "-s <subsystem_number> the subsystem number, default: 0\n"
						+ "-c <hostname>:<port>   sets the dispatcher settings\n"
						+ "-h			print this help\n"
						+ "-r			start the simulation immediately");

		System.exit(0);
	}

	/**
	 * Getter method for the de.tu_darmstadt.rs.sim.main memory file
	 *
	 * @return de.tu_darmstadt.rs.sim.main memory file
	 */
	public File getMemoryFile() {
		return memoryFile;
	}

	/**
	 *
	 * @return the configuration file from jconfig
	 */
	public File getJConfigFile() {
		return jconfigFile;
	}

	/**
	 *
	 * @return the subsystem number
	 */
	public int getSubsystemNumber() {
		return subsystemNumber;
	}

	/**
	 * Getter method for the run instant option
	 *
	 * @return instant option flag
	 */
	public boolean runInstant() {
		return runInstant;
	}
}
