/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine;

import de.tu_darmstadt.rs.sim.engine.MessageHandler_I.MsgType;
import de.tu_darmstadt.rs.sim.engine.parser.AssemblyParser;
import de.tu_darmstadt.rs.sim.engine.parser.LstFileParser;
import de.tu_darmstadt.rs.sim.engine.parser.ParsingException;
import de.tu_darmstadt.rs.sim.preferences.ProjectComponent_I;
import de.tu_darmstadt.rs.sim.preferences.ProjectManager;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IBooleanParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IIntegerParameterHandler;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressBlock;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayout;
import de.tu_darmstadt.rs.spartanmc.jconfig.util.DocumentUtil;

import libsim.SimulationSpeed_I;

import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.prefs.Preferences;
import java.util.regex.Pattern;
import java.util.stream.Collectors;


/**
 * All simulation's global settings are stored in the Settings class. No
 * instantiation is needed (and permitted), since all attributes and methods are
 * static. The de.tu_darmstadt.rs.sim.preferences loading / storing mechanism will ask to save or
 * restore the current configuration.
 */
public final class Settings implements ProjectComponent_I {


	/** True, if SLL & Co. shift only by 1, regardless of the second argument. **/
	public static int SINGLE_SHIFT = 0;

	/** The last chosen simulation speed. **/
	public static int SIMULATION_SPEED = 1;

	/** The initial address to start each simulation at. **/
	public static int PC_INITIAL = 2;

	/** The initial address to frame each simulation at. **/
	public static int PC_FRAME = 3;

	/** The address to jump onto if an hardware interrupt occurs. **/
	public static int INTERRUPT_JUMP_ADDRESS = 4;

	/** True, if there is a DMA address range to be used. **/
	public static int USING_DMA = 5;

	/*
	 * All address ranges with their start and end address.
	 */
	/**
	 * de.tu_darmstadt.rs.sim.main general purpose memory's start address
	 */
	public static int MEMORY_ADDRESS_MIN = 6;
	/**
	 * de.tu_darmstadt.rs.sim.main general purpose memory's end address
	 */
	public static int MEMORY_ADDRESS_MAX = 7;
	/**
	 * DMA memory's start address (Note: if USING_DMA = false, the DMA's
	 * addresses are not valid and must be ignored.)
	 */
	public static int DMA_ADDRESS_MIN = 8;
	/**
	 * DMA memory's end address (Note: if USING_DMA = false, the DMA's addresses
	 * are not valid and must be ignored.)
	 */
	public static int DMA_ADDRESS_MAX = 9;
	/**
	 * peripheral I/O memory's start address
	 */
	public static int IO_ADDRESS_MIN = 10;
	/**
	 * peripheral I/O memory's end address
	 */
	public static int IO_ADDRESS_MAX = 11;

	/**
	 * Standard font, e.g. for displaying the GUI tables' contents.
	 */
	public static int REGISTER_FONT_PLAIN = 12;
	/**
	 * bold font
	 */
	public static int REGISTER_FONT_BOLD = 13;

	/**
	 * True, if the current Simulation Speed is NOT FAST_NO_REFRESH
	 */
	public static int REFRESH_ENABLE = 14;

	/**
	 * the default port for waiting for a connection with the central server.
	 */
	// public static int LISTEN_PORT = 8087;

	/**
	 * the port on which the simulation server is listening (dynamic setting, is
	 * only set on runtime)
	 */
	public static int SERVER_PORT = 15;

	/**
	 * the name of the host of the simulation server (dynamic setting, is only
	 * set on runtime)
	 */
	public static int SERVER_HOST = 16;

	/**
	 * indicate that no valid SUBSYSTEM_NUMBER is known.
	 */
	public static int SUBSYSTEM_INVALID = 17;
	/**
	 * the number of the subsystem, which was loaded the last time
	 */
	public static int SUBSYSTEM_NUMBER = 18;

	/**
	 * the path to the jconfig file, which was loaded the last time
	 */
	public static int JCONFIG_FILE = 19;
	/**
	 * the path to the project directory, which was used the last time
	 */
	public static int LAST_PROJECT_DIR = 20;
	/**
	 * the path to the project file, which was loaded the last time
	 */
	public static int LAST_PROJECT_FILE = 21;
	/**
	 * the path to the project directory, which was used the last time
	 */
	public static int LAST_PROGRAM_DIR = 22;
	/**
	 * the path to the program file, which was loaded the last time
	 */
	public static int LAST_PROGRAM_FILE = 23;

	private static AddressSpaceManager addressSpace;
	private static SimulationController_I simCtrl;
	private static MessageHandler_I messageHandler;
	private static ProjectManager projectManager;

	private final Map<Integer, Object> settingValues = new HashMap<Integer, Object>();
	private final Map<Integer, List<SettingListener_I>> settingListeners = new HashMap<Integer, List<SettingListener_I>>();


	private static class InstanceHolder {

		public static Settings instance = new Settings();
	}

	/**
	 * set a new setting configuration for a setting
	 * @param settingType the setting's identification
	 * @param value the new value of the setting
	 */
	public static void setSetting(final int settingType, final Object value) {
		InstanceHolder.instance.setSettingInternal(settingType, value);
	}

	/**
	 * get the value of a setting
	 * @param settingType the setting's identification
	 * @return the setting value
	 */
	public static Object getSetting(final int settingType) {
		return InstanceHolder.instance.getSettingInternal(settingType);
	}

	/**
	 * register a listener to a setting change
	 * @param settingType the setting's identification
	 * @param listener the listener to be notified when the setting changes
	 */
	public static void registerListener(final int settingType, final SettingListener_I listener) {
		InstanceHolder.instance.registerListenerInternal(settingType, listener);
	}

	/**
	 * unregister a previously registered listener to a setting change
	 * @param settingType the setting's identification
	 * @param listener the listener to be notified when the setting changes
	 */
	public static void unregisterListener(final int settingType, final SettingListener_I listener) {
		InstanceHolder.instance.unregisterListenerInternal(settingType, listener);
	}


	/**
	 * initialize the settings and provide all necessary references to other objects
	 * @param addressSpace the {@link AddressSpaceManager}
	 * @param simCtrl the {@link SimulationController_I}
	 * @param messageHandler the {@link MessageHandler_I}
	 * @param projectManager the {@link ProjectManager}
	 */
	public final static void init(final AddressSpaceManager addressSpace, final SimulationController_I simCtrl, final MessageHandler_I messageHandler, final ProjectManager projectManager) {
		Settings.addressSpace = addressSpace;
		Settings.simCtrl = simCtrl;
		Settings.messageHandler = messageHandler;
		Settings.projectManager = projectManager;
		InstanceHolder.instance.registerAllListeners();
	}


	private Settings() {
		setDefaults();
	}

	private void registerAllListeners() {
		/* Listener for SPH files */
		registerListener(Settings.LAST_PROGRAM_FILE, new SettingListener_I() {
			private final static String SPH_ENDING = ".initData";

			@Override
			public void onSettingChange(int settingType, Object newValue) {
				final File file = (File) newValue;
				if (file.getAbsolutePath().endsWith(SPH_ENDING)) {
					final AssemblyParser parser = new AssemblyParser(addressSpace);
					addressSpace.delete();
					try {
						parser.loadAssemblyCode(file);
					} catch (ParsingException e) {
						messageHandler.postMessage("Parsing error: " + e.getMessage(), MsgType.ERR);
					}
					simCtrl.resetSimulation();
				}
			}
		});

		/* Listener for LST files */
		registerListener(Settings.LAST_PROGRAM_FILE, new SettingListener_I() {
			/**
			 * file extension for listing files
			 */
			private final static String LST_ENDING = ".lst";

			@Override
			public void onSettingChange(int settingType, Object newValue) {
				final File file = (File) newValue;
				if (file.getAbsolutePath().endsWith(LST_ENDING)) {
					LstFileParser parser = LstFileParser.getTheInstance();
					addressSpace.delete();
					try {
						parser.loadLstFile(file);
					} catch (ParsingException e) {
						messageHandler.postMessage("Parsing error: " + e.getMessage(), MsgType.ERR);
					}
					simCtrl.resetSimulation();
				}
			}
		});
		/*Listener for ELF files*/
		/*registerListener(Settings.LAST_PROGRAM_FILE, new SettingListener_I() {
			private final static String ELF_ENDING=".elf";
			@Override
			public void onSettingChange(int settingType, Object newValue) {
				final File file=(File)newValue;
				if(file.getAbsolutePath().endsWith(ELF_ENDING)){
					new ELFReader().read(file);
					simCtrl.resetSimulation();
				}

			}
		});

		/* Listener for projects */
		registerListener(Settings.LAST_PROJECT_FILE, new SettingListener_I() {

			@Override
			public void onSettingChange(int settingType, Object newValue) {
				String filepath = (String) newValue;
				try {
					projectManager.loadProject(filepath);
				} catch (IOException e) {
					messageHandler.postMessage("Parsing error: " + e.getMessage(), MsgType.ERR);
				}
			}
		});
		/* Listener for JConfig files */
		/* TODO: change JConfig import so that it can happen without user interaction via GUI */
	}

	private void setDefaults() {
		setSettingInternal(SINGLE_SHIFT, false);
		setSettingInternal(SIMULATION_SPEED, SimulationSpeed_I.Speed.FREQ_100);
		setSettingInternal(PC_INITIAL, 0);
		setSettingInternal(PC_FRAME, (52 - 1) * 1024);
		setSettingInternal(INTERRUPT_JUMP_ADDRESS, 0x20);
		setSettingInternal(USING_DMA, true);
		setSettingInternal(MEMORY_ADDRESS_MIN, 0);
		setSettingInternal(MEMORY_ADDRESS_MAX, 0x400 * 100 - 1);
		setSettingInternal(DMA_ADDRESS_MIN, 0x400 * 100);
		setSettingInternal(DMA_ADDRESS_MAX, 0x400 * 104 - 1);
		setSettingInternal(IO_ADDRESS_MIN, 0x400 * 104);
		setSettingInternal(IO_ADDRESS_MAX, 0x400 * 128 - 1);
		setSettingInternal(REGISTER_FONT_PLAIN, new Font("Courier", Font.PLAIN, 12));
		setSettingInternal(REGISTER_FONT_BOLD, new Font("Courier", Font.BOLD, 12));
		setSettingInternal(REFRESH_ENABLE, true);
		setSettingInternal(SERVER_PORT, 8086);
		setSettingInternal(SERVER_HOST, "localhost");
		setSettingInternal(SUBSYSTEM_INVALID, -1);
		setSettingInternal(SUBSYSTEM_NUMBER, getSettingInternal(SUBSYSTEM_INVALID));
		setSettingInternal(JCONFIG_FILE, "");
		setSettingInternal(LAST_PROJECT_DIR, new File("."));
		setSettingInternal(LAST_PROGRAM_DIR, new File("."));
	}

	/**
	 * Stores all settings attributes in the de.tu_darmstadt.rs.sim.preferences.
	 *
	 * @param prefs
	 *            the de.tu_darmstadt.rs.sim.preferences map to use
	 */
	@Override
	public void storeConfiguration(final Preferences prefs) {
		prefs.putInt("SIMULATION_SPEED", ((SimulationSpeed_I.Speed) getSettingInternal(SIMULATION_SPEED)).ordinal());
		prefs.putInt("PC_INITIAL", (Integer) getSettingInternal(PC_INITIAL));
		prefs.putInt("PC_FRAME", (Integer) getSettingInternal(PC_FRAME));
		prefs.putInt("INTERRUPT_ADDRESS", (Integer) getSettingInternal(INTERRUPT_JUMP_ADDRESS));

		prefs.putBoolean("USING_DMA", (Boolean) getSettingInternal(USING_DMA));

		prefs.putInt("MEMORY_ADDRESS_MIN", (Integer) getSettingInternal(MEMORY_ADDRESS_MIN));
		prefs.putInt("MEMORY_ADDRESS_MAX", (Integer) getSettingInternal(MEMORY_ADDRESS_MAX));
		prefs.putInt("DMA_ADDRESS_MIN", (Integer) getSettingInternal(DMA_ADDRESS_MIN));
		prefs.putInt("DMA_ADDRESS_MAX", (Integer) getSettingInternal(DMA_ADDRESS_MAX));

		prefs.put("FONT", ((Font) getSettingInternal(REGISTER_FONT_PLAIN)).getFamily());

		prefs.putInt("SUBSYSTEM_NUMBER", (Integer) getSettingInternal(SUBSYSTEM_NUMBER));

		prefs.putInt("defaultPort", (Integer) getSettingInternal(SERVER_PORT));
		final File lastProjectDir = (File) getSettingInternal(LAST_PROJECT_DIR);
		String lastProjectDirPath = lastProjectDir.getAbsolutePath();
		if (!lastProjectDirPath.endsWith(File.separator)) {
			lastProjectDirPath += '/';
		}
		prefs.put("LAST_PROJECT_DIR", lastProjectDirPath.replace(File.separatorChar, '/'));
		prefs.put("JCONFIG_FILE", getRelativePath(lastProjectDir, new File((String) getSettingInternal(JCONFIG_FILE))).replace(File.separatorChar, '/'));
		final File lastProgramDir = (File) getSettingInternal(LAST_PROGRAM_DIR);
		if (lastProgramDir != null) {
			prefs.put("LAST_PROGRAM_DIR", getRelativePath(lastProjectDir, lastProgramDir).replace(File.separatorChar, '/'));
			final File lastProgramFile = (File) getSettingInternal(LAST_PROGRAM_FILE);
			if (lastProgramFile != null) {
				prefs.put("LAST_PROGRAM", getRelativePath(lastProgramDir, lastProgramFile).replace(File.separatorChar, '/'));
			}
		}
	}

	private static String getRelativePath(final String from, final String to, final boolean isToADirectory) {
		final String separator = Pattern.quote(File.separator);
		final String[] fromPathParts = from.split(separator);
		final String[] toPathParts = to.split(separator);
		int partIndex = 0;
		for (; partIndex < fromPathParts.length; partIndex++) {
			if (!fromPathParts[partIndex].equals(fromPathParts[partIndex])) {//TODO: probable bug!
				break;
			}
		}
		if (partIndex == fromPathParts.length && partIndex == toPathParts.length && isToADirectory) {
			return "./";
		}
		if (partIndex > 0) {
			String resultString = "";
			for (int i = partIndex; i < fromPathParts.length; i++) {
				resultString += ".." + File.separator;
			}
			for (int i = partIndex; i < toPathParts.length; i++) {
				resultString += toPathParts[i] + File.separator;
			}
			if (!isToADirectory) {
				resultString = resultString.substring(0, resultString.lastIndexOf(File.separator));
			}
			return resultString;
		}
		return to;
	}

	private static String getRelativePath(final File from, final File to) {
		final String fromPath;
		final String toPath;
		try {
			fromPath = from.getCanonicalPath();
			toPath = to.getCanonicalPath();
		} catch (IOException e) {
			e.printStackTrace();
			return to.getAbsolutePath();
		}
		return getRelativePath(fromPath, toPath, to.isDirectory());
	}

	/**
	 * Load all attributes out of a given de.tu_darmstadt.rs.sim.preferences map.
	 *
	 * @param prefs
	 *            the de.tu_darmstadt.rs.sim.preferences map
	 */
	@Override
	public void loadConfiguration(final Preferences prefs) {
		setSettingInternal(SINGLE_SHIFT, prefs.getBoolean("SINGLE_SHIFT", (Boolean) getSettingInternal(SINGLE_SHIFT)));
		setSettingInternal(SIMULATION_SPEED, SimulationSpeed_I.Speed.values()[prefs.getInt("SIMULATION_SPEED", ((SimulationSpeed_I.Speed) getSettingInternal(SIMULATION_SPEED)).ordinal())]);
		setSettingInternal(PC_INITIAL, prefs.getInt("PC_INITIAL", (Integer) getSettingInternal(PC_INITIAL)));
		setSettingInternal(PC_FRAME, prefs.getInt("PC_FRAME", (Integer) getSettingInternal(PC_FRAME)));
		setSettingInternal(INTERRUPT_JUMP_ADDRESS, prefs.getInt("INTERRUPT_JUMP_ADDRESS", (Integer) getSettingInternal(INTERRUPT_JUMP_ADDRESS)));
		setSettingInternal(USING_DMA, prefs.getBoolean("USING_DMA", (Boolean) getSettingInternal(USING_DMA)));
		setSettingInternal(MEMORY_ADDRESS_MIN, prefs.getInt("MEMORY_ADDRESS_MIN", (Integer) getSettingInternal(MEMORY_ADDRESS_MIN)));
		setSettingInternal(MEMORY_ADDRESS_MAX, prefs.getInt("MEMORY_ADDRESS_MAX", (Integer) getSettingInternal(MEMORY_ADDRESS_MAX)));
		setSettingInternal(DMA_ADDRESS_MIN, prefs.getInt("DMA_ADDRESS_MIN", (Integer) getSettingInternal(DMA_ADDRESS_MIN)));
		setSettingInternal(DMA_ADDRESS_MAX, prefs.getInt("DMA_ADDRESS_MAX", (Integer) getSettingInternal(DMA_ADDRESS_MAX)));
		setSettingInternal(IO_ADDRESS_MIN, prefs.getInt("IO_ADDRESS_MIN", (Integer) getSettingInternal(IO_ADDRESS_MIN)));
		setSettingInternal(IO_ADDRESS_MAX, prefs.getInt("IO_ADDRESS_MAX", (Integer) getSettingInternal(IO_ADDRESS_MAX)));

		final String family = prefs.get("FONT", "Courier");
		setSettingInternal(REGISTER_FONT_PLAIN, new Font(family, Font.PLAIN, 12));
		setSettingInternal(REGISTER_FONT_BOLD, new Font(family, Font.BOLD, 12));
		setSettingInternal(SUBSYSTEM_NUMBER, prefs.getInt("SUBSYSTEM_NUMBER", (Integer) getSettingInternal(SUBSYSTEM_NUMBER)));
		String jconfigFilePath = prefs.get("JCONFIG_FILE", (String) getSettingInternal(JCONFIG_FILE));
		setSettingInternal(SERVER_PORT, prefs.getInt("defaultPort", (Integer) getSettingInternal(SERVER_PORT)));
		String lastProjectDirectory = prefs.get("LAST_PROJECT_DIR",
				null);
		if (lastProjectDirectory != null) {
			lastProjectDirectory = lastProjectDirectory.replace('/', File.separatorChar);
			if (jconfigFilePath.startsWith("/")) {
				setSettingInternal(JCONFIG_FILE, jconfigFilePath);
			} else {
				jconfigFilePath = jconfigFilePath.replace('/', File.separatorChar);
				try {
					String s = new URI(lastProjectDirectory).resolve(jconfigFilePath).getPath();
					setSettingInternal(JCONFIG_FILE, s);
				} catch (URISyntaxException use) {
					use.printStackTrace();
				}
			}
			setSettingInternal(LAST_PROJECT_DIR, new File(lastProjectDirectory));
			String lastProgramDirectory = prefs.get("LAST_PROGRAM_DIR",
					null);
			if (lastProgramDirectory != null) {
				lastProgramDirectory = lastProgramDirectory.replace('/', File.separatorChar);
				try {
					lastProgramDirectory = new URI(lastProjectDirectory).resolve(lastProgramDirectory).getPath();
					setSettingInternal(LAST_PROGRAM_DIR, new File(lastProgramDirectory));
					String lastProgramFile = prefs.get("LAST_PROGRAM", null);
					if (lastProgramFile != null) {
						lastProgramFile = lastProgramFile.replace('/', File.separatorChar);
						lastProgramFile = new URI(lastProgramDirectory).resolve(lastProgramFile).getPath();
						setSettingInternal(LAST_PROGRAM_FILE, new File(lastProgramFile));
					}
				} catch (URISyntaxException use) {
					// TODO Auto-generated catch block
					use.printStackTrace();
				}
			}
		} else {
			setSettingInternal(JCONFIG_FILE, jconfigFilePath);
		}
	}

	/**
	 * read the parameters
	 *
	 * @param processorNode
	 *            the document node with the parameters
	 */
	public static void importParameter(ISubsystemConfiguration processorNode) {
		boolean singleShift = DocumentUtil.getParameterHandler(processorNode, "SINGLE_SHIFT", IBooleanParameterHandler.class).isSet();
		long interruptJumpAddr = DocumentUtil.getParameterHandler(processorNode, "RESET_VECTOR", IIntegerParameterHandler.class).getIntegerValue().orElseThrow();
		long resetVector = DocumentUtil.getParameterHandler(processorNode, "INTR_VECTOR", IIntegerParameterHandler.class).getIntegerValue().orElseThrow();
		if (singleShift) {
			setSetting(SINGLE_SHIFT, singleShift);
		} else if (interruptJumpAddr != -1) {
			setSetting(INTERRUPT_JUMP_ADDRESS, interruptJumpAddr);
		} else if (resetVector != -1) {
			setSetting(PC_INITIAL, resetVector);
		}
		AddressLayout addressLayout = AddressLayout.getCurrent(processorNode);
		/**
		 * initializes the DMA address block for each peripheral in the subsystem
		 */
		List<IPeripheralModuleConfiguration> peripherals = (List<IPeripheralModuleConfiguration>) processorNode.streamPeripherals().collect(Collectors.toList());
		for (IPeripheralModuleConfiguration peri : peripherals) {
			Optional<AddressBlock.DmaAddressBlock> block = addressLayout.findDmaAddressBlock(peri);
			if (block.isPresent()) {
				setSetting(IO_ADDRESS_MIN, block.get().getActualStart());
				setSetting(IO_ADDRESS_MAX, block.get().getActualEnd());
			}
		}


		// parameters
		/*Collection<VerilogParameter> parameters = processorNode
				.getChildren(VerilogParameter.class);

		for (VerilogParameter param : parameters) {
			String name = param.getName();

			if (name.equals("SINGLE_SHIFT")) {
				setSetting(SINGLE_SHIFT, param.getBooleanValue());
			} else if (name.equals("INTR_VECTOR")) {
				setSetting(INTERRUPT_JUMP_ADDRESS, param.getNumericValue().intValue());
			} else if (name.equals("RESET_VECTOR")) {
				setSetting(PC_INITIAL, param.getNumericValue().intValue());
			}
		}

		// address blocks
		// IO
		IAddressBlock block = processorNode.getChild(Id.IO_BASE_ADR);
		setSetting(IO_ADDRESS_MIN, block.getActualStart());
		setSetting(IO_ADDRESS_MAX, block.getActualEnd());
*/
		// DMA
		// TODO jconfig does not difference between program memory and dma
		// memory
	}

	/**
	 * set a new setting configuration for a setting
	 * @param settingType the setting's identification
	 * @param value the new value of the setting
	 */
	private void setSettingInternal(final int settingType, final Object value) {
		settingValues.put(settingType, value);
		List<SettingListener_I> settingListenerList = settingListeners.get(settingType);
		if (settingListenerList == null) {
			settingListenerList = new ArrayList<SettingListener_I>();
			settingListeners.put(settingType, settingListenerList);
		}
		for (SettingListener_I listener : settingListenerList) {
			listener.onSettingChange(settingType, value);
		}
	}

	/**
	 * get the value of a setting
	 * @param settingType the setting's identification
	 * @return the setting value
	 */
	private Object getSettingInternal(final int settingType) {
		return settingValues.get(settingType);
	}

	/**
	 * register a listener to a setting change
	 * @param settingType the setting's identification
	 * @param listener the listener to be notified when the setting changes
	 */
	private void registerListenerInternal(final int settingType, final SettingListener_I listener) {
		List<SettingListener_I> listenerList = settingListeners.get(settingType);
		if (listenerList == null) {
			listenerList = new ArrayList<SettingListener_I>();
			settingListeners.put(settingType, listenerList);
		}
		if (!listenerList.contains(listener)) {
			listenerList.add(listener);
		}
	}

	/**
	 * unregister a previously registered listener to a setting change
	 * @param settingType the setting's identification
	 * @param listener the listener to be notified when the setting changes
	 */
	private void unregisterListenerInternal(final int settingType, final SettingListener_I listener) {
		List<SettingListener_I> listenerList = settingListeners.get(settingType);
		if (listenerList != null) {
			listenerList.remove(listener);
		}
	}

	/**
	 * retrieve the settings instance
	 * @return the settings instance
	 */
	public static ProjectComponent_I getInstance() {
		return InstanceHolder.instance;
	}

	//	public static String getSummary() {
	//
	//		String str = "SINGLE_SHIFT: " + SINGLE_SHIFT;
	//		str += "   TIMED_EXECUTION_INTERVAL_MS: " + SIMULATION_SPEED;
	//		str += "   PC_INITIAL: " + PC_INITIAL;
	//		str += "   INTERRUPT_ADDRESS: " + INTERRUPT_JUMP_ADDRESS;
	//
	//		str += "   USING_DMA: " + USING_DMA;
	//
	//		str += "   MEMORY_ADDRESS_MIN: "
	//				+ String.format("%x", MEMORY_ADDRESS_MIN);
	//		str += "   MEMORY_ADDRESS_MAX: "
	//				+ String.format("%x", MEMORY_ADDRESS_MAX);
	//		str += "   DMA_ADDRESS_MIN: " + String.format("%x", DMA_ADDRESS_MIN);
	//		str += "   DMA_ADDRESS_MAX: " + String.format("%x", DMA_ADDRESS_MAX);
	//		str += "   IO_ADDRESS_MIN: " + String.format("%x", IO_ADDRESS_MIN);
	//		str += "   IO_ADDRESS_MAX: " + String.format("%x", IO_ADDRESS_MAX);
	//
	//		str += "   FONT: " + REGISTER_FONT_PLAIN.getFamily();
	//
	//		return str;
	//	}
	/*public class ELFReader{
	public void read(File file){
			try {
				RandomAccessFile raf=new RandomAccessFile(file,"r");
				ELFHeader header= ELFLoader.readELFHeader(raf);
				ELFSectionHeaderTable data= ELFLoader.readSHT(raf,header);
				addressSpace.delete();
				for (ELFSectionHeaderTable.Entry entry:data.entries) {
					int []memoryBlock=new ElfToMemContent(ElfToMemContent.InputFormat.ELF8).Byte2Int32bto18b(entry.getOffset(),entry.getSize(),raf);
					for(int address=Math.toIntExact(entry.getAddr());address<entry.getEntrySize();address++){
						addressSpace.setValue(address,memoryBlock[address-Math.toIntExact(entry.getAddr())]);
					}
				}
				RawMemory initData=new ElfToMemContent(ElfToMemContent.InputFormat.ELF8).readInElf(file);
				//File sphFile=new File(initData,"helper.initData");
				AssemblyParser parser=new AssemblyParser(addressSpace);
				//parser.loadAssemblyCode(file.getCanonicalFile());

			}
			catch(Exception e){
				e.printStackTrace();
			}
	}

 }*/

}
