/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2012
 *
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2012
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2012
 */
package de.tu_darmstadt.rs.sim.engine;

import de.tu_darmstadt.rs.sim.engine.MessageHandler_I.MsgType;
import de.tu_darmstadt.rs.sim.engine.assembly.AssemblyLineManager;
import de.tu_darmstadt.rs.sim.engine.assembly.MemoryAssemblyLineProvider;
import de.tu_darmstadt.rs.sim.engine.dcm.DCMManager;
import de.tu_darmstadt.rs.sim.engine.dcm.DCMSignalInvocationManager;
import de.tu_darmstadt.rs.sim.engine.net.MpsocManager;
import de.tu_darmstadt.rs.sim.engine.parser.LstFileParser;
import de.tu_darmstadt.rs.sim.engine.peripheral.PeripheralManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.interrupts.InterruptController;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.PipelineClock;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerManager;
import de.tu_darmstadt.rs.sim.engine.processor.ProcessorCore_I;
import de.tu_darmstadt.rs.sim.engine.processor.RegisterFile;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisters;
import de.tu_darmstadt.rs.sim.engine.processor.SimpleProcessorCore;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineStage;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineUI;
import de.tu_darmstadt.rs.sim.engine.statistics.Statistics;
import de.tu_darmstadt.rs.sim.engine.statistics.Statistics_I;
import de.tu_darmstadt.rs.sim.preferences.ProjectManager;
import de.tu_darmstadt.rs.sim.signal.Signal1;
import de.tu_darmstadt.rs.sim.signal.SignalModified;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;


/**
 * The SimulationManager class administers and executes the simulation itself.
 * It is the engine's major class.
 */
public class SimulationManager implements SpartanDefinitions_I,
		RefreshTrigger_I {

	protected final AddressSpaceManager addressSpaceManager;
	protected final MpsocManager fMpsocManager;

	protected InterruptController irqController;
	protected RegisterFile regFile;
	protected SfRegisters sfr;
	protected final Statistics_I statistics;
	protected final Statistics instructionStats;
	protected final PeripheralManager peripheralMgr;

	protected PipelineClock pipelineClock;
	protected DCMSignalInvocationManager dcmSignalInvocationManager;
	protected DCMManager dcmManager;

	protected ProcessorCore_I processorCore;

	protected final RefreshHandler_I fRefreshHandler;
	protected final List<RefreshListener_I> fRefreshListeners;
	protected final Collection<Integer> fRefreshTasks;
	protected final MessageHandler_I fMsgHandler;

	protected final SimulationController_I fSimCtrl;
	protected final SimulationStepListener_I fStepListener;

	protected final Collection<SimulationComponent_I> fSimComponents;

	protected final AccessManager accessManager;

	protected long fCycleCounter;
	protected final SignalModified fSigModified;
	protected final Signal1<Long> sigCycles;

	/**
	 * Constructor.
	 *
	 * @param mp
	 *            {@link MpsocManager}
	 * @param refreshHandler
	 *            a {@link RefreshHandler_I} to which refresh requests should be
	 *            sent
	 * @param sc
	 *            controller of the simulation state
	 * @param ssl
	 *            a listener for the correctly executed simulation steps
	 * @param mh
	 *            handler for occurring messages
	 * @param pm
	 *            project's component de.tu_darmstadt.rs.sim.preferences manager
	 */
	public SimulationManager(final MpsocManager mp,
							 final RefreshHandler_I refreshHandler,
							 final SimulationController_I sc,
							 final SimulationStepListener_I ssl, final MessageHandler_I mh,
							 final ProjectManager pm, AddressSpaceManager addressSpace) {
		fMsgHandler = mh;
		fMpsocManager = mp;
		fRefreshHandler = refreshHandler;
		fStepListener = ssl;
		fSimCtrl = sc;

		fRefreshListeners = new LinkedList<RefreshListener_I>();
		fRefreshTasks = new HashSet<Integer>();
		fSimComponents = new HashSet<SimulationComponent_I>();
		TimerManager.init(refreshHandler);
		dcmManager = new DCMManager(this);
		pipelineClock = new PipelineClock();
		dcmSignalInvocationManager = new DCMSignalInvocationManager(this);

		accessManager = AccessManager.getTheInstance();
		sfr = new SfRegisters(refreshHandler, accessManager);
		regFile = new RegisterFile(refreshHandler, sfr, this);

		addressSpaceManager = addressSpace;

		pm.registerComponent(accessManager);

		Settings.init(addressSpaceManager, fSimCtrl, fMsgHandler, pm);
		peripheralMgr = new PeripheralManager(this);

		// Peripheral is created dynamically and unknown to the
		// SimulationManager.
		// Peripheral may register to the IoAddressRange to participate at the
		// simulation.

		instructionStats = new Statistics(refreshHandler);
		statistics = instructionStats;
		AssemblyLineManager.createTheInstance(addressSpaceManager);
		final AssemblyLineManager assemblyLineManager = AssemblyLineManager
				.getTheInstance();
		LstFileParser.createTheInstance(addressSpaceManager);
		assemblyLineManager.addLineObjectProvider(LstFileParser
				.getTheInstance());
		assemblyLineManager
				.addLineObjectProvider(new MemoryAssemblyLineProvider(
						addressSpaceManager));
		this.processorCore = new SimpleProcessorCore(this, accessManager);
		processorCore.setRefreshHandler(fRefreshHandler);
		fSimComponents.addAll(processorCore.getSimulationComponents());

		fSimComponents.add(regFile);
		fSimComponents.add(sfr);
		fSimComponents.add(addressSpaceManager);
		fSimComponents.add(mp);
		fSimComponents.add(statistics);

		sigCycles = new Signal1<Long>();
		fSigModified = new SignalModified();
		fSigModified.registerFlushHandler(this, "flush");

		fRefreshHandler.registerDataProducerChangeSignal(fSigModified);
		accessManager.setSimulationController(sc);
		accessManager.setSimulationManager(this);
		refreshHandler.registerTrigger(this);
	}

	/**
	 * called when the modified de.tu_darmstadt.rs.sim.signal is fired
	 */
	public void flush() {
		sigCycles.emit(fCycleCounter);
	}

	/**
	 * register a handler for the cycle update
	 *
	 * @param o
	 *            the object having the method
	 * @param methodName
	 *            the name of the method to be invoked
	 * @param longArgClass
	 *            type of parameter of the method
	 */
	public void registerCycleHandler(final Object o, final String methodName,
									 final Class<Long> longArgClass) {
		sigCycles.registerMethodCallOnEmit(o, methodName, longArgClass);
	}

	/**
	 * unregister a previously registered handler for the cycle update
	 *
	 * @param o
	 *            the object not to be notified any more
	 */
	public void unregisterCycleHandler(final Object o) {
		sigCycles.unregisterMethodCallOnEmit(o);
	}

	/**
	 * get the handler for the refresh events. This handler posts the event to
	 * all refresh listeners.
	 *
	 * @return the refresh handler
	 */
	public RefreshHandler_I getRefreshHandler() {
		return fRefreshHandler;
	}

	/**
	 * retrieve the {@link MpsocManager}
	 *
	 * @return the {@link MpsocManager}
	 */
	public MpsocManager getMpsocManager() {
		return fMpsocManager;
	}

	/**
	 * retrieve the pipeline clock
	 *
	 * @return pipeline clock object
	 */
	public PipelineClock getPipelineClock() {
		return pipelineClock;
	}

	/**
	 * get the manager for the DCMs
	 *
	 * @return the { DCMManager}
	 */
	public DCMManager getDCMManager() {
		return dcmManager;
	}

	/**
	 * retrieve the manager for the invocation order of DCM signals
	 *
	 * @return the {@link DCMSignalInvocationManager}
	 */
	public DCMSignalInvocationManager getDCMSignalInvocationManager() {
		return dcmSignalInvocationManager;
	}

	/**
	 * Executes a simulation step, i.e. a CPU clock. If there is a
	 * SpartanException the step's execution may not have been completed. If
	 * there is a SimulationBreakException, the step has definitely been
	 * executed completely.
	 */
	public void executeStep() {

		fCycleCounter++;

		dcmSignalInvocationManager.doClockingUntilStartOfNextPipelineClock();

		processorCore.onExecuteStep(false);
		regFile.onClock();

		statistics.addInstruction(processorCore.getPipelineUI()
				.getCurrentInstruction(PipelineStage.IFID_STAGE));
		fStepListener
				.simulationStepComplete(SimulationStepListener_I.Result.SUCCESSFUL);

		fSigModified.emit();

	}

	/**
	 * Resets the simulation to initial state
	 */
	public void resetSimulation() {
		fCycleCounter = 0;
		for (SimulationComponent_I sc : fSimComponents) {
			sc.restart();
		}

		requestCompleteRefresh();
		synchronized (this) {
			fRefreshTasks.add(REFRESH_AND_CLEAR);
		}
	}

	/**
	 * Setups the whole simulation. Forwards the request to all devices.
	 */
	public void setupSimulation() {
		for (SimulationComponent_I sc : fSimComponents) {
			sc.setup();
		}
		resetSimulation();
	}

	/**
	 * Retrieves the de.tu_darmstadt.rs.sim.main memory.
	 *
	 * @return the memory
	 */
	public AddressSpaceManager getAddressSpaceManager() {
		return addressSpaceManager;
	}

	/**
	 * Retrieves the register file (general purpose registers).
	 *
	 * @return the register file
	 */
	public RegisterFile getRegisterFile() {
		return regFile;
	}

	/**
	 * Retrieves the special function registers.
	 *
	 * @return the SFR
	 */
	public SfRegisters getSfRegisters() {
		return sfr;
	}

	/**
	 * Retrieves the pipeline's user interface.
	 *
	 * @return the pipeline UI
	 */
	public PipelineUI getPipelineUI() {
		return processorCore.getPipelineUI();
	}

	/**
	 * Retrieves the statistics.
	 *
	 * @return the statistics
	 */
	public Statistics getInstructionStats() {
		return instructionStats;
	}

	/**
	 * Retrieves the interrupt controller.
	 *
	 * @return the interrupt controller
	 */
	public InterruptController getInterruptController() {
		return irqController;
	}

	/**
	 * set the interrupt controller. This is used when loading a configuration.
	 * (Some configurations don't have interrupt controllers therefore an
	 * interrupt controller is not always present.) If an interrupt controller
	 * is used, the simulation of any program takes some more time.
	 *
	 * @param irqController
	 *            the new interrupt controller
	 */
	public void setInterruptController(final InterruptController irqController) {
		fSimComponents.remove(this.irqController);
		this.irqController = irqController;
		if (irqController != null) {
			fSimComponents.add(irqController);
		}
	}

	/**
	 * set the processor core object (to change the processor core
	 * implementation)
	 *
	 * @param processorCore
	 *            the {@link ProcessorCore_I} to be used in the simulation
	 */
	public void setProcessorCore(final ProcessorCore_I processorCore) {
		fSimComponents.removeAll(this.processorCore.getSimulationComponents());
		this.processorCore = processorCore;
		processorCore.setRefreshHandler(fRefreshHandler);
		fSimComponents.addAll(processorCore.getSimulationComponents());
	}

	/**
	 * Retrieves the PeripheralManager.
	 *
	 * @return the PeripheralManager
	 */
	public PeripheralManager getPeripheralManager() {
		return peripheralMgr;
	}

	/**
	 * check whether the address is in the delay register window. This is the
	 * case when a delayed branch execution started.
	 *
	 * @param address
	 *            the address
	 * @return true if the address is in the delay register window, false
	 *         otherwise
	 */
	public boolean isAddressInDelayWindow(final int address) {
		return processorCore.isAddressInDelayWindow(address);
	}

	/**
	 * request a complete refresh
	 */
	public synchronized void requestCompleteRefresh() {
		fRefreshTasks.add(COMPLETE_REFRESH);

	}

	@Override
	public Collection<RefreshListener_I> getRefreshListeners() {
		return fRefreshListeners;
	}

	@Override
	public synchronized int[] getDirtyAddresses() {
		int[] rt = new int[fRefreshTasks.size()];
		int i = 0;
		for (Integer t : fRefreshTasks) {
			rt[i++] = t;
		}
		fRefreshTasks.clear();
		return rt;
	}

	/**
	 * Adds a listener to the event queue.
	 *
	 * @param listener
	 *            the listener
	 */
	public void addRefreshListener(final RefreshListener_I listener) {
		fRefreshListeners.add(listener);
	}

	/**
	 * Removes a listener from the event queue.
	 *
	 * @param listener
	 *            the listener
	 */
	public void removeRefreshListener(final RefreshListener_I listener) {
		fRefreshListeners.remove(listener);
	}

	/**
	 * push an error message and stop the simulation accordingly
	 *
	 * @param message
	 *            message which should be shown
	 */
	public void pushErrorMessage(final String message) {
		fSimCtrl.stopSimulation();
		fMsgHandler.postMessage(message, MsgType.ERR);
	}

	/*public static void de.tu_darmstadt.rs.sim.main(String [] args){
		final ProjectManager prjm = new ProjectManager();
		final GuiMessageHandler gmh = new GuiMessageHandler();
		final RefreshThread rt = new RefreshThread(100);
		final SimulationThread st = new SimulationThread();
		final SimControlSelector sel = new SimControlSelector(st);
		final MpsocManager mp = new MpsocManager("localhost", 8886, sel, st, gmh);
		AddressSpaceManager addressSpaceManager=new AddressSpaceManager(rt,AccessManager.getTheInstance());
		SfRegisters sfRegisters=new SfRegisters(rt,AccessManager.getTheInstance());

		final SimulationManager sm = new SimulationManager(mp, rt, sel, mp,
				gmh, prjm,addressSpaceManager);
		SimulationManagerGDBInterface smgdb =new SimulationManagerGDBInterface(mp,rt,sel,mp,gmh,prjm,sm.getAddressSpaceManager(),sm.getRegisterFile());
		sm.getRegisterFile().setRegister(sm.getRegisterFile().getRegisterDescriptor(5,0),354);
		//System.out.println(sm.getAddressSpaceManager().getValue(0));
		//System.out.println(sm.getAddressSpaceManager().getValue(1));
		//System.out.println(sm.getAddressSpaceManager().hasBreakpoint(1));
		System.out.println(smgdb.getRegister(5));
		//System.out.println(smgdb.getMemory(1));
	}*/
}
