/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine;


/**
 * All global constants and enumerations are defined here. Any class may implement
 * this interface to get a convenient access.
 *
 */
public interface SpartanDefinitions_I {

	/**
	 * the program's version
	 */
	public final static String PROGRAM_VERSION_NUMBER = "1.7";

	/**
	 * This value indicates that something went wrong or something was not found.
	 */
	public final static int INVALID = -1;

	/**
	 * This address specification is used when a refresh handler should update its complete data set.
	 */
	public final static int COMPLETE_REFRESH = -1;

	/**
	 * This address specification is used when a refresh handler should update its registers.
	 */
	public final static int REFRESH_REGISTER_ONLY = -2;

	/**
	 * This address specification is used when a refresh handler should clear/reset its data.
	 */
	public final static int REFRESH_AND_CLEAR = -3;

	/**
	 * the number of registers of the core
	 */
	public final static int REG_FILE_COUNT = 8 * 127 + 8;


	/**
	 * PipelineUI variables access
	 */
	enum VisiblePipelineRegister {
		IFID_PC, IFID_PC_NEW, IFID_IR,
		IFID_OFFSET, IFID_IMM, IFID_IMM_HIGH, IFID_DISP,
		IFID_REG_NO_A, IFID_REG_NO_B, IFID_REG_A,
		IFID_ALU_OPA, IFID_ALU_OPB,
		EXMEM_REG_NO_WB, EXMEM_REG_STORE_SRC,
		EXMEM_ALU_OP_A, EXMEM_ALU_OP_B, EXMEM_ALU_RESULT,
		EXMEM_DATA_ADDR, EXMEM_WRITE_DATA_DATA, EXMEM_SFR_ADDRESS,
		EXMEM_USE_BYPASS_OPA, EXMEM_USE_BYPASS_OPB, EXMEM_USE_BYPASS_REGA, EXMEM_PC_DELAY,

		SFR_MM, SFR_CC, SFR_MUL, SFR_INTERRUPT, SFR_LEDS, SFR_WINDOW, SFR_IV, SFR_TR,

		WB_WRITE_REG_NUMBER, WB_WRITE_REG_DATA, WB_DO_WB
	}


	/**
	 * mask for the lowest 18 bits (bit 0 to bit 17)
	 */
	public final static int BITMASK_18 = 0x3ffff;
	/**
	 * mask for the lowest 17 bits (bit 0 to bit 16)
	 */
	public final static int BITMASK_17 = 0x1ffff;
	/**
	 * mask for the lowest 13 bits (bit 0 to bit 12)
	 */
	public final static int BITMASK_13 = 0x1fff;
	/**
	 * mask for the lowest 10 bits (bit 0 to bit 9)
	 */
	public final static int BITMASK_10 = 0x3ff;
	/**
	 * mask for the highest 10 bits (bit 8 to bit 17)
	 */
	public final static int BITMASK_10H = 0x3ff00;
	/**
	 * mask for the lowest 9 bits (bit 0 to bit 8)
	 */
	public final static int BITMASK_9 = 0x1ff;
	/**
	 * mask for the lowest 8 bits (bit 0 to bit 7)
	 */
	public final static int BITMASK_8 = 0xff;
	/**
	 * mask for the lowest 7 bits (bit 0 to bit 6)
	 */
	public final static int BITMASK_7 = 0x7f;
	/**
	 * mask for the lowest 6 bits (bit 0 to bit 5)
	 */
	public final static int BITMASK_6 = 0x3f;
	/**
	 * mask for the lowest 5 bits (bit 0 to bit 4)
	 */
	public final static int BITMASK_5 = 0x1f;
	/**
	 * mask for the lowest 4 bits (bit 0 to bit 3)
	 */
	public final static int BITMASK_4 = 0xf;
	/**
	 * mask for the lowest 3 bits (bit 0 to bit 2)
	 */
	public final static int BITMASK_3 = 0x7;
	/**
	 * mask for the lowest 2 bits (bit 0 and bit 1)
	 */
	public final static int BITMASK_2 = 0x3;
	/**
	 * mask for the least significant bit. This mask is equivalent to {@link #BIT_0}.
	 */
	public final static int BITMASK_1 = 0x1;

	/**
	 * mask for bit 18 (index is 0 based). This bit can not occur on any SpartanMC register (which contain bit indices [0,17]).
	 */
	public final static int BIT_18 = 0x40000;
	/**
	 * mask for the most significant bit (bit 17)
	 */
	public final static int BIT_17 = 0x20000;
	/**
	 * mask for bit 8 (index is 0 based)
	 */
	public final static int BIT_8 = 0x100;
	/**
	 * mask for bit 7 (index is 0 based)
	 */
	public final static int BIT_7 = 0x80;
	/**
	 * mask for bit 6 (index is 0 based)
	 */
	public final static int BIT_6 = 0x40;
	/**
	 * mask for the least significant bit (bit 0)
	 */
	public final static int BIT_0 = 0x1;
}
