/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.assembly;


import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineStage;

/**
 * Interface for all the assembly's lines which contain data for visualization
 * of the instruction processing. Provides Methods used to show data and
 * influence the simulation process.
 */
public interface AssemblyLine_I {

	/**
	 * get the address of the assembly line (as a data address) if any
	 *
	 * @return address or {@link SpartanDefinitions_I#INVALID} if no address
	 * available
	 */
	int getDataAddress();

	/**
	 * returns the data (e.g. machine instruction or data) at that line
	 *
	 * @return data to be shown at that line
	 */
	String getData();

	/**
	 * returns whether this is a data line or instruction line
	 *
	 * @return true if this line is a data line
	 */
	boolean isData();

	/**
	 * set the {@link engine.processor.pipeline.PipelineStage} which can be
	 * used for later visualization
	 *
	 * @param stage the pipeline stage which works on the address of this assembly
	 * line
	 */
	void setStage(PipelineStage stage);

	/**
	 * get the {@link engine.processor.pipeline.PipelineStage} for
	 * visualizations
	 *
	 * @return the pipeline stage which works on the address of this assembly
	 * line
	 */
	PipelineStage getStage();

	/**
	 * set a watch or breakpoint (this depends on the value)
	 *
	 * @param value information whether there is a watchpoint or breakpoint or
	 * nothing
	 */
	void setBreakOrWatchValue(int value);

	/**
	 * returns the value which was set with {@link #setBreakOrWatchValue(int)}.
	 * This value should contain the information if there is a watchpoint,
	 * breakpoint or nothing
	 *
	 * @return information whether there is a watchpoint or breakpoint or
	 * nothing
	 */
	int getBreakOrWatchValue();
}
