/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.assembly;

/**
 * representation of an assembly's variable shown for debugging/tracing purpose
 */
public class AssemblyVariable {

	private int dataAddress;
	private String name;
	private int lengthIn9Bytes;

	/**
	 * create a new variable which has the given name and which could be found
	 * at the dataAddress in the assembly
	 *
	 * @param dataAddress the address of the variable
	 * @param name the name of the variable
	 */
	public AssemblyVariable(int dataAddress, String name) {
		this.dataAddress = dataAddress;
		this.name = name;
	}

	/**
	 * get the address (in data addressing mode) in the assembly where this
	 * variable can be found
	 *
	 * @return the dataAddress of the variable
	 */
	public int getDataAddress() {
		return dataAddress;
	}

	/**
	 * set the address (in data addressing mode)
	 *
	 * @param address the new address
	 */
	public void setDataAddress(int address) {
		this.dataAddress = address;
	}

	/**
	 * get the variable name
	 *
	 * @return name of this variable
	 */
	public String getName() {
		return name;
	}

	/**
	 * set a new name
	 *
	 * @param name new name of the variable
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * get the length of the variable (in 9 bit bytes)
	 *
	 * @return length of the variable (in 9 bit units)
	 */
	public int getLengthIn9Bytes() {
		return lengthIn9Bytes;
	}

	/**
	 * set a new length in 9 bit units
	 *
	 * @param lengthIn9Bytes new length
	 */
	public void setLengthIn9Bytes(int lengthIn9Bytes) {
		this.lengthIn9Bytes = lengthIn9Bytes;
	}

}
