/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.dcm;

/**
 * Enum with all possible values for the CLKFX_DIVIDE parameter of a DCM
 */
public enum CLKFX_DIVIDE {
	DIVIDE_1(1),
	DIVIDE_2(2),
	DIVIDE_3(3),
	DIVIDE_4(4),
	DIVIDE_5(5),
	DIVIDE_6(6),
	DIVIDE_7(7),
	DIVIDE_8(8),
	DIVIDE_9(9),
	DIVIDE_10(10),
	DIVIDE_11(11),
	DIVIDE_12(12),
	DIVIDE_13(13),
	DIVIDE_14(14),
	DIVIDE_15(15),
	DIVIDE_16(16),
	DIVIDE_17(17),
	DIVIDE_18(18),
	DIVIDE_19(19),
	DIVIDE_20(20),
	DIVIDE_21(21),
	DIVIDE_22(22),
	DIVIDE_23(23),
	DIVIDE_24(24),
	DIVIDE_25(25),
	DIVIDE_26(26),
	DIVIDE_27(27),
	DIVIDE_28(28),
	DIVIDE_29(29),
	DIVIDE_30(30),
	DIVIDE_31(31),
	DIVIDE_32(32);
	/**
	 * default value
	 */
	public static final CLKFX_DIVIDE DEFAULT_VALUE = DIVIDE_1;
	/**
	 * the denominator of the fraction
	 */
	private long denominator;

	/**
	 * Constructor
	 *
	 * @param denominator the denominator
	 */
	private CLKFX_DIVIDE(long denominator) {
		this.denominator = denominator;
	}

	/**
	 * used by the clocking mechanism
	 *
	 * @return denominator of the {@link CLKFX_DIVIDE} value
	 */
	public long getDenominator() {
		return denominator;
	}

	/**
	 * get a {@link CLKFX_DIVIDE} value from a double value
	 *
	 * @param value a double value
	 * @return the {@link CLKFX_DIVIDE} value
	 */
	public static CLKFX_DIVIDE getByValue(long value) {
		for (CLKFX_DIVIDE div : CLKFX_DIVIDE.values()) {
			if (div.getDenominator() == value) {
				return div;
			}
		}
		return DEFAULT_VALUE;
	}

	@Override
	public String toString() {
		return name().substring(7);
	}
}
