/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.dcm;

/**
 * Enum with all possible values for the CLKFX_MULTIPLY parameter of a DCM
 */
public enum CLKFX_MULTIPLY {
	MULTIPLY_2(2),
	MULTIPLY_3(3),
	MULTIPLY_4(4),
	MULTIPLY_5(5),
	MULTIPLY_6(6),
	MULTIPLY_7(7),
	MULTIPLY_8(8),
	MULTIPLY_9(9),
	MULTIPLY_10(10),
	MULTIPLY_11(11),
	MULTIPLY_12(12),
	MULTIPLY_13(13),
	MULTIPLY_14(14),
	MULTIPLY_15(15),
	MULTIPLY_16(16),
	MULTIPLY_17(17),
	MULTIPLY_18(18),
	MULTIPLY_19(19),
	MULTIPLY_20(20),
	MULTIPLY_21(21),
	MULTIPLY_22(22),
	MULTIPLY_23(23),
	MULTIPLY_24(24),
	MULTIPLY_25(25),
	MULTIPLY_26(26),
	MULTIPLY_27(27),
	MULTIPLY_28(28),
	MULTIPLY_29(29),
	MULTIPLY_30(30),
	MULTIPLY_31(31),
	MULTIPLY_32(32);
	/**
	 * default value
	 */
	public static final CLKFX_MULTIPLY DEFAULT_VALUE = MULTIPLY_4;
	/**
	 * the nominator of the fraction
	 */
	private long nominator;

	/**
	 * Constructor
	 *
	 * @param nominator the nominator
	 */
	private CLKFX_MULTIPLY(long nominator) {
		this.nominator = nominator;
	}

	/**
	 * used by the clocking mechanism
	 *
	 * @return nominator of the {@link CLKFX_MULTIPLY} value
	 */
	public long getNominator() {
		return nominator;
	}

	/**
	 * get a {@link CLKFX_MULTIPLY} value from a double value
	 *
	 * @param value a double value
	 * @return the {@link CLKFX_MULTIPLY} value
	 */
	public static CLKFX_MULTIPLY getByValue(int value) {
		for (CLKFX_MULTIPLY mult : CLKFX_MULTIPLY.values()) {
			if (mult.getNominator() == value) {
				return mult;
			}
		}
		return DEFAULT_VALUE;
	}

	@Override
	public String toString() {
		return name().substring(9);
	}
}
