/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine.memory;


import de.tu_darmstadt.rs.sim.engine.AbstractHardwareComponent;
import de.tu_darmstadt.rs.sim.engine.RefreshHandler_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationComponent_I;
import de.tu_darmstadt.rs.sim.engine.processor.Instruction;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineStage;

/**
 * This class represents the de.tu_darmstadt.rs.sim.main memory with its data content. Memory addresses
 * may be connected with watch- and/or breakpoints
 */
public final class MainMemory extends AbstractHardwareComponent implements
		SimulationComponent_I {

	private final static int MIN_ADDRESS = (Integer) Settings.getSetting(Settings.MEMORY_ADDRESS_MIN);
	private final static int MAX_ADDRESS = (Integer) Settings
			.getSetting(Settings.MEMORY_ADDRESS_MAX);
	private static final int ADDRESS_SPACE_SIZE = (MAX_ADDRESS
			- MIN_ADDRESS + 1)
			+ ((Integer) Settings.getSetting(Settings.DMA_ADDRESS_MAX) - (Integer) Settings.getSetting(Settings.DMA_ADDRESS_MIN) + 1);

	private int[] dataArray;

	private int ifid_InstrAddress;
	private int exmem_InstrAddress;
	private int wb_InstrAddress;


	private Instruction[] fInstrCache = new Instruction[0x40000];

	/**
	 * Constructor.
	 *
	 * @param rh
	 *            the {@link RefreshHandler_I} to which refresh requests should
	 *            be sent
	 */
	public MainMemory(RefreshHandler_I rh) {

		super(rh);
		dataArray = new int[ADDRESS_SPACE_SIZE];
	}

	/**
	 * Retrieves the de.tu_darmstadt.rs.sim.main memory size.
	 *
	 * @return the size
	 */
	public int getSize() {
		return MAX_ADDRESS - MIN_ADDRESS + 1;
	}

	/**
	 * Deletes the whole memory.
	 */
	public void delete() {
		dataArray = new int[0x40000];
		fInstrCache = new Instruction[0x40000];
		fireRefreshEvent(COMPLETE_REFRESH);
	}

	/**
	 * Retrieves a memory value.
	 *
	 * @param address
	 *            the value's address
	 * @return the value
	 */
	public int getValueAt(final int address) {
		return dataArray[address];
	}

	/**
	 * Sets a memory value.
	 *
	 * @param address
	 *            the value's address
	 * @param value
	 *            the new value
	 */
	public void setValueAt(final int address, final int value) {
		dataArray[address] = value;
		fInstrCache[address] = null;
		fireRefreshEvent(address);
	}

	/**
	 * Marks a certain address whose instruction is loaded by the pipelines IFID
	 * stage.
	 *
	 * @param address
	 *            the address
	 */
	public void setIfIdStage(final int address) {

		assert ((address == INVALID) || MainMemory.containsAddress(address));
		// invalid address occurs on interrupt!

		wb_InstrAddress = exmem_InstrAddress;
		exmem_InstrAddress = ifid_InstrAddress;
		ifid_InstrAddress = address;
	}

	/**
	 * Retrieves the pipeline stage which is executing the address' instruction
	 * if any.
	 *
	 * @param address
	 *            the address to check for
	 * @return the pipeline stage, if any
	 */
	public PipelineStage getPipeStage(final int address) {
		if (ifid_InstrAddress == address) {
			return PipelineStage.IFID_STAGE;
		}
		if (exmem_InstrAddress == address) {
			return PipelineStage.EXMEM_STAGE;
		}
		if (wb_InstrAddress == address) {
			return PipelineStage.WB_STAGE;
		}

		return PipelineStage.NOT_IN_PIPE;
	}

	/**
	 * Check if an address is a de.tu_darmstadt.rs.sim.main memory address.
	 *
	 * @param address
	 *            the address
	 * @return true, if it is a de.tu_darmstadt.rs.sim.main memory address
	 */
	public static boolean containsAddress(final int address) {

		if (address < MIN_ADDRESS) {
			return false;
		}
		if (address >= ADDRESS_SPACE_SIZE + MIN_ADDRESS) {
			return false;
		}
		return true;
	}

	/**
	 * Executes setup procedures.
	 */
	@Override
	public void setup() {
		restart();
	}

	/**
	 * Restarts the de.tu_darmstadt.rs.sim.main memory.
	 */
	@Override
	public void restart() {
		ifid_InstrAddress = INVALID;
		exmem_InstrAddress = INVALID;
		wb_InstrAddress = INVALID;

		fireRefreshEvent(COMPLETE_REFRESH);
	}
}
