/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */

package de.tu_darmstadt.rs.sim.engine.parser;

import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.engine.memory.MainMemory;
import de.tu_darmstadt.rs.sim.utilities.Formatter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

/**
 * The AssemblyParser parses a given assembly file and fills the memory's data
 * respectively. It offers also functionality to store the memory into a file.
 */
public class AssemblyParser implements SpartanDefinitions_I {

	private final AddressSpaceManager memory;

	/**
	 * Constructor.
	 *
	 * @param memory
	 *            the current de.tu_darmstadt.rs.sim.main memory
	 */
	public AssemblyParser(final AddressSpaceManager memory) {
		this.memory = memory;
	}

	/**
	 * Loads code into the de.tu_darmstadt.rs.sim.main memory.
	 *
	 * @param file
	 *            the Spartan file
	 * @throws ParsingException
	 *             if any error occures
	 */
	public void loadAssemblyCode(final File file) throws ParsingException {

		try {
			final BufferedReader reader = new BufferedReader(new FileReader(
					file));

			String line;
			for (int address = 0; (line = reader.readLine()) != null; ++address) {

				if (!MainMemory.containsAddress(address)) {
					throw new ParsingException(String.format(
							"Cannot write on invalid address: 0x%05x", address));
				}

				final int data = Formatter.parseHexString(line, BITMASK_18);
				if (data != INVALID) {
					memory.writeTo(address, data);
				} else {
					throw new ParsingException(String.format(
							"Invalid data '%s' at line: %d", line, address + 1));
				}
			}

			reader.close();

		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}

	/**
	 * Stores the de.tu_darmstadt.rs.sim.main memory's content into a file.
	 *
	 * @param file
	 *            the Spartan file
	 * @throws ParsingException
	 *             if any error occures
	 */
	public void storeAssemblyCode(final File file) throws ParsingException {

		try {
			final BufferedWriter writer = new BufferedWriter(new FileWriter(
					file));
			int address;

			// 1) Rewind from the memory's end up to the front as long as we
			// find just zeros
			// >> we do not want these zeros in the final file
			final int minAddress = (Integer) Settings
					.getSetting(Settings.MEMORY_ADDRESS_MIN);
			final int maxAddress = (Integer) Settings
					.getSetting(Settings.MEMORY_ADDRESS_MAX);
			for (address = maxAddress; address >= minAddress; --address) {

				assert (MainMemory.containsAddress(address));
				if (memory.readFrom(address) != 0) {
					break;
				}
			}

			if (address < minAddress) {
				throw new ParsingException("Memory is empty.");
			}

			// 2) Remember this upper address bound and restart from the
			// memory's beginning,
			// write all data in the memory until we reach this upper bound
			final int highestAddress = address;
			for (address = minAddress; address <= highestAddress; ++address) {

				assert (MainMemory.containsAddress(address));
				final String line = String.format("%05x",
						memory.readFrom(address));
				writer.write(line);
				writer.newLine();
			}

			writer.flush();
			writer.close();

		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}

}
