/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.parser;


import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;

/**
 * Definitions for parsing listing files including regular expressions and the
 * indices for replacements.
 */
public interface LstParsingDefinitions_I extends SpartanDefinitions_I {

	/**
	 * pattern which is used to identify a section
	 */
	public final static String DISASM_SECTION = "Disassembly of section ";
	/**
	 * pattern for a section's name
	 */
	public final static String SECTION_IDENTIFIER = "[\\.a-z]*";
	/**
	 * pattern for a single hex character
	 */
	public final static String HEX_IDENTIFIER = "[0-9a-fA-F]";
	/**
	 * pattern for an address identifier
	 */
	public final static String ADDRESS_IDENTIFIER = HEX_IDENTIFIER + "*";
	/**
	 * pattern for a symbol
	 */
	public final static String SYMBOL_IDENTIFIER = "[_a-zA-Z][a-zA-Z0-9_]*";
	/**
	 * pattern for an instruction
	 */
	public final static String INSTR_IDENTIFIER = "[^\\t]*";
	/**
	 * pattern for an operand
	 */
	public final static String OPERANDS_IDENTIFIER = "[^;]*";
	/**
	 * pattern for the type of the data value
	 */
	public final static String DATA_TYPE_IDENTIFIER = "[^\\t]*";
	/**
	 * pattern for the data value
	 */
	public final static String DATA_VALUE_IDENTIFIER = "[^\\t]*";
	/**
	 * pattern for a 2 character long hex group
	 */
	public final static String HEX_2 = HEX_IDENTIFIER + "{2}";
	/**
	 * pattern for matching anything
	 */
	public final static String ANYTHING = ".*";
	/**
	 * pattern for additional information on a program line
	 */
	public final static String PROGRAM_LINE_DESC_IDENTIFIER = ".*";
	/**
	 * pattern for a section line
	 */
	public final static String SECTION_REGEX = DISASM_SECTION + "("
			+ SECTION_IDENTIFIER + "):";
	/**
	 * pattern for a program symbol
	 */
	public final static String SYMBOL_REGEX = "(" + ADDRESS_IDENTIFIER + ") <("
			+ SYMBOL_IDENTIFIER + ")>:";
	/**
	 * pattern for a program line
	 */
	public final static String PROGRAM_LINE_REGEX = "\\s*("
			+ ADDRESS_IDENTIFIER + "):\\s{1,}(" + HEX_2 + ") (" + HEX_2 + ") ("
			+ HEX_2 + ") (" + HEX_2 + ")\\s*(" + INSTR_IDENTIFIER + ")\\t*("
			+ OPERANDS_IDENTIFIER + ")\\t{0,1};{0,1} {0,1}("
			+ PROGRAM_LINE_DESC_IDENTIFIER + ")$";
	/**
	 * pattern for a data line
	 */
	public final static String DATA_LINE_REGEX = "\\s*(" + ADDRESS_IDENTIFIER
			+ "):\\s{1,}(" + HEX_2 + ") (" + HEX_2 + ")\\s*("
			+ DATA_TYPE_IDENTIFIER + ")\\t(" + DATA_VALUE_IDENTIFIER
			+ ")\\t{0,1};{0,1} {0,1}(" + PROGRAM_LINE_DESC_IDENTIFIER + ")$";
	/**
	 * pattern for any line
	 */
	public final static String ANYTHING_REGEX = "^(" + ANYTHING + ")$";

	/**
	 * group index for the anything line
	 */
	public final static int ANYTHING_GROUP = 1;
	/**
	 * group index for the section id
	 */
	public final static int SECTION_ID_GROUP = 1;
	/**
	 * group index for the address
	 */
	public final static int ADR_GROUP = 1;
	/**
	 * group index for the first 2 character long hex group of a program line
	 */
	public final static int INSTR_HEX_1 = 2;
	/**
	 * group index for the second 2 character long hex group of a program line
	 */
	public final static int INSTR_HEX_2 = 3;
	/**
	 * group index for the third 2 character long hex group of a program line
	 */
	public final static int INSTR_HEX_3 = 4;
	/**
	 * group index for the fourth 2 character long hex group of a program line
	 */
	public final static int INSTR_HEX_4 = 5;
	/**
	 * group index for the symbolic instruction of a program line
	 */
	public final static int INSTR_SYMBOLIC = 6;
	/**
	 * group index for the symbolic operands of a program line
	 */
	public final static int INSTR_SYMBOLIC_OPERAND = 7;
	/**
	 * group index for the additional information to an instruction of a program
	 * line
	 */
	public final static int INSTR_DESC_GROUP = 8;
	/**
	 * group index for the first 2 character long hex group of a data line
	 */
	public final static int DATA_HEX_1 = 2;
	/**
	 * group index for the second 2 character long hex group of a data line
	 */
	public final static int DATA_HEX_2 = 3;
	/**
	 * group index for the type of the data value of a data line
	 */
	public final static int DATA_TYPE_GROUP = 4;
	/**
	 * group index for the data value of a data line
	 */
	public final static int DATA_VALUE_GROUP = 5;
	/**
	 * group index for the comment of a data line
	 */
	public final static int DATA_COMMENT_GROUP = 6;
	/**
	 * group index for the symbol of a symbol line
	 */
	public final static int SYMBOL_ID_GROUP = 2;
}
