/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.parser;

import de.tu_darmstadt.rs.sim.engine.assembly.AssemblyVariablesManager;

import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Class for matching listing lines containing symbols.
 */
public class LstSymbolLineMatcher extends AbstractLstLineMatcher implements
		LstParsingDefinitions_I {

	private final static Pattern symbolLinePattern = Pattern
			.compile(SYMBOL_REGEX);
	private Matcher matcher;
	private String symbolName;
	private String addressString;
	private int address;
	private final AssemblyVariablesManager manager;

	/**
	 * create a new {@link LstSymbolLineMatcher}
	 *
	 * @param manager the {@link AssemblyVariablesManager} for creation of variables
	 */
	public LstSymbolLineMatcher(final AssemblyVariablesManager manager) {
		this.manager = manager;
	}

	@Override
	public boolean matches(final String line) {
		super.line = line;
		matcher = symbolLinePattern.matcher(line);
		boolean matches = matcher.matches();
		if (matches) {
			symbolName = matcher.group(SYMBOL_ID_GROUP);
			addressString = matcher.group(ADR_GROUP);
			address = getAddressFromString(addressString);
			manager.beginCreationOfVariable(address, symbolName);
		}
		return matches;
	}

	@Override
	public int getAddress() {
		return address;
	}

	@Override
	public String[] getTransformedLineData() {
		return new String[]{"",
				String.format("%08x <%s>:", address, symbolName), ""};
	}

}
