/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine.peripheral;

import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.interrupts.InterruptController;
import de.tu_darmstadt.rs.sim.engine.peripheral.interrupts.InterruptSource_I;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.PipelineClock;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerObserver_I;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerSource_I;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.PeripheralModuleDescription;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Random;

/**
 * The DmaDemonstrator is a periphery unit which can access the DMA memory. It
 * writes random numbers consecutively into the memory (independently from the
 * CPU) and requests a hardware interrupt once it has reached the DMA's last
 * address. It will return to the DMA's beginning address and loops infinitely.
 */
public class DmaDemonstrator extends JConfigPeripheral implements
		InterruptSource_I, TimerObserver_I {

	private final AddressSpaceManager mainMemory;

	private int dmaPointer;
	private int dmaStartAddress;

	private int clockCounter;

	private final Random rand;

	private TimerSource_I timerSource;

	private InterruptController interruptController;
	private final static PeripheralModuleDescription.Register[] registers = new PeripheralModuleDescription.Register[]{new PeripheralModuleDescription.Register() {

		public PeripheralModuleDescription.Register.AccessType getRegisterAccess() {
			return PeripheralModuleDescription.Register.AccessType.ReadWrite;
		}

		public PeripheralModuleDescription.BitGroup[] getSubBitGroups() {
			return new PeripheralModuleDescription.BitGroup[0];
		}


		public PeripheralModuleDescription.BitGroup putSymbolicValue(int value, String symbolicName) {
			return null;
		}


		public String getSymbolicValue(int value) {
			return "";
		}

		@Override
		public String getName() {
			return "someRegister";
		}


		public int getIndex() {
			return 0;
		}


		public int getWidth() {
			return 18;
		}


		public void addSubBitGroup(PeripheralModuleDescription.BitGroup bitGroup) {
			//never used
		}


		public Collection<String> getSymbolicValueList() {
			return new ArrayList<String>(0);
		}


		public int getIntValue(String symbolicValue) {
			return -1;
		}

		@Override
		public String getDescription() {
			return null;
		}


		public String getDescription(String symbolicValueName) {
			return null;
		}


		public void putDescription(String symbolicValueName, String description) {
			//never used
		}
	}};

	/**
	 * Constructor.
	 *
	 * @param simMgr
	 *            the {@link SimulationManager}

	 * @param mainMemory
	 *            the de.tu_darmstadt.rs.sim.main memory address range
	 */
	public DmaDemonstrator(SimulationManager simMgr,
						   final AddressSpaceManager mainMemory) {
		super("DmaDemonstrator", "DmaDemonstrator", ((Boolean) Settings
						.getSetting(Settings.USING_DMA))? (Integer) Settings
						.getSetting(Settings.DMA_ADDRESS_MIN) : INVALID, mainMemory,
				simMgr.getRefreshHandler(), simMgr, registers);

		this.mainMemory = mainMemory;
		this.id = id;

		rand = new Random();
		setup();
	}

	@Override
	public void setup() {
		mainMemory
				.unregisterPeripheral(AddressSpaceManager.NO_IO_ADDRESS, this);
		mainMemory.registerPeripheral(AddressSpaceManager.NO_IO_ADDRESS, 1,
				this);
		dmaStartAddress = ((Boolean) Settings.getSetting(Settings.USING_DMA))? (Integer) Settings
				.getSetting(Settings.DMA_ADDRESS_MIN) : INVALID;
		TimerManager
				.addTimerConnection(PipelineClock.PIPELINE_CLOCK_NAME, this);
		restart();
	}

	@Override
	public void tearDown() {
		mainMemory
				.unregisterPeripheral(AddressSpaceManager.NO_IO_ADDRESS, this);
		timerSource.unregisterTimerObserver(this);
	}

	@Override
	public void restart() {
		dmaPointer = dmaStartAddress;
		clockCounter = 0;
		if (interruptController != null) {
			interruptController.resetInterruptRequest(this);
		}
		fireRefreshEvent(COMPLETE_REFRESH);
	}

	/**
	 * Retrieves the DMA address pointer.
	 *
	 * @return the pointer's value
	 */
	public int getDmaPointer() {
		return dmaPointer;
	}

	/**
	 * for the GUI
	 * @return true if there is an interrupt request, false otherwise
	 */
	public boolean hasInterruptRequest() {
		return (dmaPointer == (Integer) Settings
				.getSetting(Settings.DMA_ADDRESS_MAX));
	}

	@Override
	public void setInterruptController(InterruptController interruptController) {
		this.interruptController = interruptController;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see engine.processor.InterruptSource_I#getInterruptName()
	 */
	@Override
	public String getInterruptName() {
		return "DMA demonstrator IRQ " + id;
	}

	/**
	 * Retrieves the device number.
	 *
	 * @return the number
	 */
	public String getIndex() {
		return id;
	}

	@Override
	public void onClock(int clockValue) {

		if (dmaPointer == INVALID) {
			return; // no DMA range used
		}
		if (clockCounter++ < 2) {
			return;
		}

		clockCounter = 0;

		assert ((Integer) Settings.getSetting(Settings.DMA_ADDRESS_MIN) <= dmaPointer);
		assert ((Integer) Settings.getSetting(Settings.DMA_ADDRESS_MAX) > dmaPointer);
		final int dataAddress = dmaPointer++;
		final int data = rand.nextInt(BITMASK_18 + 1);
		mainMemory.setValue(dataAddress, data);

		if (interruptController != null) {
			if ((dmaPointer == (Integer) Settings
					.getSetting(Settings.DMA_ADDRESS_MAX))) {
				interruptController.postInterruptRequest(this);
			}
			if (dmaPointer > (Integer) Settings
					.getSetting(Settings.DMA_ADDRESS_MAX)) {
				dmaPointer = dmaStartAddress;
				interruptController.resetInterruptRequest(this);
			}
		}

		fireRefreshEvent(COMPLETE_REFRESH);
	}

	@Override
	public void setTimerSource(TimerSource_I timerSource) {
		this.timerSource = timerSource;
		timerSource.registerTimerObserver(this);
	}

	@Override
	public int getRegisterValue(int offset) {
		return INVALID;
	}

	@Override
	public void setRegisterValue(int offset, int value) {
		//no registers
	}
}
