/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
package de.tu_darmstadt.rs.sim.engine.peripheral;


import de.tu_darmstadt.rs.sim.engine.AccessManager;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.interrupts.InterruptController;
import de.tu_darmstadt.rs.sim.engine.peripheral.mpsoc.MasterConnector;
import de.tu_darmstadt.rs.sim.engine.peripheral.mpsoc.SlaveConnector;
import de.tu_darmstadt.rs.sim.engine.peripheral.ports.PortBi;
import de.tu_darmstadt.rs.sim.engine.peripheral.ports.PortIn;
import de.tu_darmstadt.rs.sim.engine.peripheral.ports.PortOut;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.BasicTimer;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.RTI;
import de.tu_darmstadt.rs.sim.engine.peripheral.uart.Uart;
import de.tu_darmstadt.rs.sim.engine.peripheral.uart.UartLight;
import de.tu_darmstadt.rs.sim.engine.processor.InterruptableProcessorCore;
import de.tu_darmstadt.rs.sim.engine.processor.ProcessorCore_I;
import de.tu_darmstadt.rs.sim.engine.processor.SimpleProcessorCore;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.ISubsystemConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.layouting.addressLayouting.AddressLayout;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Create and configure all peripheral used in the simulation.
 * <p>
 * The PeripheralManager implements methods for handling the document tree from
 * JConfig. In addition the root node from the document tree is owned by the
 * ProjectManager.
 */
public class PeripheralManager {

	private final SimulationManager simMgr;
	private List<JConfigPeripheral> peripheralList;
	private ProcessorCore_I processorCore;
	private final List<OnLoadedListener_I> onLoadedListenerList;

	/**
	 * create a new {@link PeripheralManager}
	 *
	 * @param simMgr
	 *            reference to the {@link SimulationManager}
	 */
	public PeripheralManager(SimulationManager simMgr) {
		this.simMgr = simMgr;

		peripheralList = new LinkedList<JConfigPeripheral>();
		onLoadedListenerList = new ArrayList<OnLoadedListener_I>();
	}

	/**
	 * search through the list after a peripheral with matching id.
	 *
	 * @param list
	 *            the peripheral list
	 * @param id
	 *            the id
	 * @return null if no matching peripheral was found, else the peripheral
	 *         with matching id.
	 */
	private static JConfigPeripheral searchPeripheral(
			List<JConfigPeripheral> list, String id) {
		JConfigPeripheral ret = null;

		for (JConfigPeripheral p : list) {
			if (p.getId().equals(id)) {
				ret = p;
				break;
			}
		}

		return ret;
	}

	/**
	 * create new AbstractPeripheral instance and initialize it.
	 *
	 * @param node
	 *            PeripheralNode describing the peripheral
	 * @return null if peripheral is not supported, else the new peripheral
	 *         instance
	 */
	private JConfigPeripheral createPeripheral(IPeripheralModuleConfiguration<?> node, long baseAdr) {
		JConfigPeripheral instance = null;
		// fetch the module type (equal the module name in the module xml file)
		final String peripheralType = node.getModuleDescriptionName();
		switch (peripheralType) {
			case "ddr_mem_port_32":
				instance = new McbInterfaceSmall(node, simMgr, (int) baseAdr);
				break;
			case "master_core_connector":
				instance = new MasterConnector(node, simMgr, (int) baseAdr);
				break;
			case "slave_core_connector":
				instance = new SlaveConnector(node, simMgr, (int) baseAdr);
				break;
			case "intctrl":
				instance = new InterruptController(node, simMgr, (int) baseAdr);
				break;
			case "intctrlp":
				instance = new InterruptController(node, simMgr, (int) baseAdr);
				break;
			case "uart":
				instance = new Uart(node, simMgr, (int) baseAdr);
				break;
			case "uart_light":
				instance = new UartLight(node, simMgr, (int) baseAdr);
				break;
			case "timer":
				instance = new BasicTimer(node, simMgr, (int) baseAdr);
				break;
			case "rti":
				instance = new RTI(node, simMgr, (int) baseAdr);
				break;
			case "fifo_in":
				instance = new FifoIn(node, simMgr, (int) baseAdr);
				break;
			case "fifo_out":
				instance = new FifoOut(node, simMgr, (int) baseAdr);
				break;
			case "port_in":
				instance = new PortIn(node, simMgr, (int) baseAdr);
				break;
			case "port_out":
				instance = new PortOut(node, simMgr, (int) baseAdr);
				break;
			case "port_bi":
				instance = new PortBi(node, simMgr, (int) baseAdr);
				break;
			default:
				instance = null;
		}
		if (instance != null) {
			processorCore = new InterruptableProcessorCore(simMgr,
					new SimpleProcessorCore(simMgr, AccessManager.getTheInstance()));
			processorCore
					.setInterruptController(new InterruptController(node, simMgr, (int) baseAdr));
			simMgr.setInterruptController(new InterruptController(node, simMgr, (int) baseAdr));
		}
		return instance;
	}

	private JConfigPeripheral createPeripheral(IPeripheralModuleConfiguration<?> peri, AddressLayout addressLayout) {
		int baseAdr = (int) addressLayout.findRegAddressBlock(peri).get().getActualStart();
		return createPeripheral(peri, baseAdr);
	}

	/**
	 * load the peripherals from the document
	 *
	 * @param selectedSubsystem
	 *            the selected subsystem node from the document
	 */
	public void onLoadSubsystem(ISubsystemConfiguration<?> selectedSubsystem) {
		AddressLayout addressLayout = AddressLayout.getCurrent(selectedSubsystem);
		List<JConfigPeripheral> oldPeripheral = peripheralList;
		peripheralList = selectedSubsystem.streamPeripherals().
				map(peri -> createPeripheral(peri, addressLayout)).collect(Collectors.toList());
		// fetch all peripheral nodes

		processorCore = new SimpleProcessorCore(simMgr,
				AccessManager.getTheInstance());

		if (peripheralList.isEmpty()) {
			simMgr.pushErrorMessage("Consistency error while loading a subsystem, this should never happen!");
		}

		// remove peripheral missing in the new configuration
		//for (JConfigPeripheral peri : oldPeripheral) {
		//	peri.tearDown();
		//}
		simMgr.setProcessorCore(processorCore);
		for (OnLoadedListener_I listener : onLoadedListenerList) {
			listener.onLoaded(this);
		}
	}

	/**
	 * fetch the peripheral with matching id. If an invalid id is passed no
	 * search is done.
	 *
	 * @param id
	 *            the id of the desired peripheral
	 * @return null if no matching peripheral was found, else the peripheral
	 *         with matching id.
	 */
	public JConfigPeripheral getPeripheral(String id) {
		if (id.equals(JConfigPeripheral.INVALID_ID)) {
			return null;
		}
		return searchPeripheral(peripheralList, id);
	}

	/**
	 * retrieve a peripheral by its name
	 *
	 * @param name
	 *            the name of the peripheral
	 * @return the peripheral if found or null if the peripheral could not be
	 *         found
	 */
	public JConfigPeripheral getPeripheralByName(final String name) {
		for (JConfigPeripheral peripheral : peripheralList) {
			if (peripheral.getPeriName().equals(name)) {
				return peripheral;
			}
		}
		return null;
	}

	/**
	 * retrieve a sorted list with the names of all peripherals
	 *
	 * @return list with the names of all peripherals
	 */
	public List<String> getSortedPeripheralNameList() {
		List<String> peripheralNameList = new ArrayList<String>(
				peripheralList.size());
		for (JConfigPeripheral peripheral : peripheralList) {
			peripheralNameList.add(peripheral.getPeriName());
		}
		Collections.sort(peripheralNameList);
		return peripheralNameList;
	}

	/**
	 * register a listener which will be invoked when subsystem has been loaded
	 *
	 * @param listener
	 *            the listener which should be invoked
	 */
	public void registerOnLoadedListener(OnLoadedListener_I listener) {
		if (!onLoadedListenerList.contains(listener)) {
			onLoadedListenerList.add(listener);
		}
	}

	/**
	 * tell the {@link PeripheralManager} that a previously registered listener
	 * should not be invoked when subsystem has been loaded
	 *
	 * @param listener
	 *            the listener which should not be invoked any more
	 * @return true if the listener was registered before, false otherwise
	 */
	public boolean unregisterOnLoadedListener(OnLoadedListener_I listener) {
		return onLoadedListenerList.remove(listener);
	}
}
