/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

package de.tu_darmstadt.rs.sim.engine.peripheral.mpsoc;

import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.net.MpsocException;
import de.tu_darmstadt.rs.sim.engine.net.MpsocManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.JConfigPeripheral;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.handlers.IIntegerParameterHandler;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;


public abstract class MpsocConnector extends JConfigPeripheral {

	private static final int REG_STATUS = 0;
	private static final int REG_MSGSIZE = 1;

	private final int FIFO_FULL = 0x2;
	private final int FIFO_EMPTY = 0x1;

	protected final MpsocManager fMpsocMgr;
	private final Queue<Integer> fFifo;
	private final Queue<Integer> fEntriesPending;
	private final int fFifoCapacity;
	private int fMsgSize;
	private int fPushReqSize;
	private boolean fPushWait;
	private boolean fPending;

	/**
	 * Create a new connector based on a peripheral instance supplied by
	 * libjconfig.
	 *
	 * @param peri The peripheral instance
	 * @param simulationManager {@link SimulationManager} for required data (address space, refresh handler, MpsocManager)
	 */
	public MpsocConnector(IPeripheralModuleConfiguration peri, final SimulationManager simulationManager, int baseAddrss) {
		super(peri, simulationManager, baseAddrss);
		fMpsocMgr = simulationManager.getMpsocManager();
		IIntegerParameterHandler parameter = (IIntegerParameterHandler) peri.findParameterDescription("FIFO_DEPTH").get();
		fFifoCapacity = (int)(long)parameter.getIntegerValue().orElseThrow();
		fFifo = new ConcurrentLinkedQueue<Integer>();
		fEntriesPending = new ConcurrentLinkedQueue<Integer>();
	}

	/**
	 * called when the FIFO is empty, hook for sub classes
	 */
	protected abstract void onFifoEmpty();

	/**
	 * called when the FIFO is full, hook for sub classes
	 */
	protected abstract void onFifoFull();

	public abstract int getRequestSize();

	private void pushFifoUpdate() {
		if (fPushReqSize > fEntriesPending.size()) {
			return;
		}
		try {
			fMpsocMgr.sendFifoUpdate(getBaseAddr(), fPushReqSize);
		} catch (MpsocException e) {
			e.printStackTrace();
		}
	}

	public final Collection<Integer> getEntriesPending(final int reqSize) {
		if (fEntriesPending.size() < reqSize) {
			fPushReqSize = reqSize;
			//            System.out.printf("entering tx push mode: %d\n", reqSize);
			return Collections.<Integer>emptyList();
		} else {
			synchronized (fEntriesPending) {
				Collection<Integer> ep = new LinkedList<Integer>(fEntriesPending);
				fEntriesPending.clear();
				fPushReqSize = 0;
				return ep;
			}
			//            if (fPushReqSize != 0) {
			//                System.out.printf("pushing update\n");
			//            }
		}
	}

	protected void setMsgSize(final int s) {
		int newMessageSize = s;
		if (newMessageSize > fFifoCapacity) {
			newMessageSize = fFifoCapacity;
		}
		if (fMsgSize != newMessageSize) {
			fPushWait = false;
			fMsgSize = newMessageSize;
		}
	}

	protected int getMsgSize() {
		return fMsgSize;
	}

	private int getFifoFullFlag(final int reqFree) {
		return (fFifo.size() + reqFree > fFifoCapacity)? FIFO_FULL : 0;
	}

	private int getFifoEmptyFlag(int reqUsed) {
		return (fFifo.size() < reqUsed)? FIFO_EMPTY : 0;
	}

	protected synchronized int getFifoStatus(final int msgSize) {
		if (!fPushWait && (getFifoFullFlag(msgSize) != 0)) {
			onFifoFull();
		}
		if (!fPushWait && (getFifoEmptyFlag(msgSize) != 0)) {
			onFifoEmpty();
		}
		int status = getFifoEmptyFlag(msgSize) | getFifoFullFlag(msgSize);
		return status;
	}

	protected final int getFifoUsed() {
		return fFifo.size();
	}

	protected final int getFifoFree() {
		return fFifoCapacity - fFifo.size();
	}

	protected final void offerFifoIfFree(final int value) {
		getFifoStatus(1);
		if (getFifoFree() > 0) {
			synchronized (fEntriesPending) {
				fEntriesPending.add(value);
				fFifo.offer(value);
			}
			if (fPushReqSize > 0) {
				pushFifoUpdate();
			}
		}
	}

	protected final void offerFifo(Collection<Integer> data) {
		if (fPushWait) {
			return;
		}
		fFifo.addAll(data);
	}

	protected final int pollFifoIfAvailable() {
		int value = 0;
		getFifoStatus(1);
		if (getFifoUsed() > 0) {
			synchronized (fEntriesPending) {
				value = fFifo.poll();
				fEntriesPending.add(value);
			}
			if (fPushReqSize > 0) {
				pushFifoUpdate();
			}
		}
		return value;
	}

	protected final void pollFifo(final int cnt) {
		int timesToPoll = cnt;
		if (fPushWait) {
			return;
		}
		while (timesToPoll-- > 0) {
			fFifo.poll();
		}
	}

	protected final synchronized void requestFifoUpdate() {
		if (!fPending) {
			fPending = true;
			fMpsocMgr.requestFifoUpdate(this);
		}
	}

	public synchronized void onRecvUpdate(Collection<Integer> data) {
		fPending = false;
		fPushWait = (data.size() == 0);
	}

	@Override
	public void tearDown() {
		fMpsocMgr.unregister(this);
	}

	@Override
	public void restart() {
		fMsgSize = 1;
		fPushReqSize = 0;
		fPushWait = false;
		fPending = false;
		fFifo.clear();
		fEntriesPending.clear();
	}

	@Override
	public int getRegisterValue(int offset) {
		switch (offset) {
			case REG_STATUS:
				return getFifoStatus(fMsgSize);
			case REG_MSGSIZE:
				return fMsgSize;
			default:
				return 0;
		}
	}

	@Override
	public void setRegisterValue(int offset, int value) {
		switch (offset) {
			case REG_MSGSIZE:
				fMsgSize = value;
				break;
		}
	}
}
