/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.peripheral.ports;

import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.JConfigPeripheral;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;


/**
 * Class representing the port_out peripheral. It is used to control output
 * pins.
 */
public class PortOut extends JConfigPeripheral {

	/**
	 * offset of the DAT register (to the base address)
	 */
	public static final int DAT_OFFSET = 0;
	/**
	 * offset of the OE register (to the base address)
	 */
	public static final int OE_OFFSET = 1;
	private int datRegisterValue;
	private int oeRegisterValue;

	private int numberOfPins;
	private int numberOfPinsMask;

	/**
	 * create a new {@link PortOut}
	 *
	 * @param peri the hardware description
	 * @param simulationManager reference to the {@link SimulationManager}
	 */
	public PortOut(IPeripheralModuleConfiguration peri, SimulationManager simulationManager, int baseAddress) {
		super(peri, simulationManager, baseAddress);
	}

	@Override
	public void importHardwareParameters(IPeripheralModuleConfiguration<?> node) {
		super.importHardwareParameters(node);
		numberOfPins = node.getPortHandler(node.findPortDescription("pout").get()).getConnectorWidth();
		numberOfPinsMask = (1 << numberOfPins) - 1;
	}

	@Override
	public int getRegisterValue(int offset) {
		switch (offset) {
			case DAT_OFFSET:
				return datRegisterValue;
			case OE_OFFSET:
				return oeRegisterValue;
		}
		return 0;
	}

	@Override
	public void setRegisterValue(int offset, int value) {
		switch (offset) {
			case DAT_OFFSET:
				datRegisterValue = value & numberOfPinsMask;
				break;
			case OE_OFFSET:
				oeRegisterValue = value & numberOfPinsMask;
				break;
		}
		fireRefreshEvent(COMPLETE_REFRESH);
	}

}
