/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.peripheral.timer;

import java.util.BitSet;

/**
 * Helper class for reading the jconfig file data
 */
public class SignalHelper {

	/**
	 * retrieve the timer's name from the de.tu_darmstadt.rs.sim.signal name
	 *
	 * @param signal the de.tu_darmstadt.rs.sim.signal's name
	 * @return the timer's name
	 */
	public static String getTimerNameFromConnectedSignal(String signal) {
		String retVal = null;
		int index;
		if (signal.contains("_clk_peri")) {
			retVal = PipelineClock.PIPELINE_CLOCK_NAME;
		} else if ((!signal.contains("_clk_")) && signal.contains("_out")) {
			index = signal.lastIndexOf("_out");
			if (index != -1) {
				retVal = signal.substring(0, index);
				index = retVal.lastIndexOf("_");
				if (index != -1) {
					retVal = signal.substring(0, index);
				} else {
					retVal = null;
				}
			}
		} else if (signal.contains("_clk")) {
			retVal = signal;
		}
		return retVal;
	}

	/**
	 * retrieve the highest bit's index
	 *
	 * @param bits all possible bits
	 * @return index of the highest bit
	 */
	public static int getIndexOfHighestConnectedBit(BitSet bits) {
		int retVal = 0;
		for (int i = bits.nextSetBit(0); i >= 0; i = bits.nextSetBit(i + 1)) {
			retVal = i;
		}
		return retVal;
	}

}
