/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine.processor;

import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;

import java.util.Map;
import java.util.TreeMap;

/**
 * The Instruction class is a storage structure representing a single
 * instruction (opCode, type, operands & String descriptions).
 */
public class Instruction implements SpartanDefinitions_I {

	private static final Map<Integer, Instruction> sInstrCache = new TreeMap<Integer, Instruction>();


	/*	{ir[17:16], ir[15:13]} */
	private static final int SHIFT_AMOUNT_INSTRUCTION_TYPE = 13;
	private static final int SHIFT_AMOUNT_OPA = 9;
	private static final int SHIFT_AMOUNT_OPB = 5;

	private static final int J_HI = 0x02 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b00_010_??_???_...
	private static final int JALS_HI = 0x03 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b00_011_??_???_...
	private static final int BEQZ_HI = 0x04 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b00_100_??_???_...
	private static final int BNEZ_HI = 0x05 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b00_101_??_???_...
	private static final int BEQZC_HI = 0x06 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b00_110_??_???_...
	private static final int BNEZC_HI = 0x07 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b00_111_??_???_...
	private static final int ADDI_HI = 0x08 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b01_000_??_???_...
	private static final int MOVI_HI = 0x09 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b01_001_??_???_...
	private static final int LHI_HI = 0x0a << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b01_010_??_???_...
	private static final int SIGEX_HI = 0x0b << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b01_011_??_???_...
	private static final int ANDI_HI = 0x0c << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b01_100_??_???_...
	private static final int ORI_HI = 0x0d << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b01_101_??_???_...
	private static final int XORI_HI = 0x0e << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b01_110_??_???_...
	private static final int MULI_HI = 0x0f << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b01_111_??_???_...
	private static final int L9_HI = 0x10 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b10_000_??_???_...
	private static final int S9_HI = 0x11 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b10_001_??_???_...
	private static final int L18_HI = 0x12 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b10_010_??_???_...
	private static final int S18_HI = 0x13 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b10_011_??_???_...
	private static final int SLLI_HI = 0x14 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b10_100_??_???_...
	private static final int JF_HI = 0x15 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b10_101_??_???_...
	private static final int SRLI_HI = 0x16 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b10_110_??_???_...
	private static final int SRAI_HI = 0x17 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b10_111_??_???_...
	private static final int SEQI_HI = 0x18 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b11_000_??_???_...
	private static final int SNEI_HI = 0x19 << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b11_001_??_???_...
	private static final int SLTI_HI = 0x1a << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b11_010_??_???_...
	private static final int SGTI_HI = 0x1b << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b11_011_??_???_...
	private static final int SLEI_HI = 0x1c << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b11_100_??_???_...
	private static final int SGEI_HI = 0x1d << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b11_101_??_???_...
	private static final int IFADDUI_HI = 0x1e << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b11_110_??_???_...
	private static final int IFSUBUI_HI = 0x1f << SHIFT_AMOUNT_INSTRUCTION_TYPE;    // 10'b11_111_??_???_...


	/*	{ir[17:16], ir[15:13], ir[4:3], ir[2:0]} */

	/*
	//TODO: check if the following two are necessary
	private static final int ORCC_HILO	= 0x0000;	// 10'b00_000_????????_00_000
	private static final int ANDCC_HILO	= 0x0001;	// 10'b00_000_????????_00_001
	*/
	private static final int MOVI2C_HILO = 0x0002;    // 10'b00_000_????????_00_010
	private static final int MOVC2I_HILO = 0x0003;    // 10'b00_000_????????_00_011
	private static final int SLL_HILO = 0x0004;    // 10'b00_000_????????_00_100
	private static final int MOV_HILO = 0x0005;    // 10'b00_000_????????_00_101
	private static final int SRL_HILO = 0x0006;    // 10'b00_000_????????_00_110
	private static final int SRA_HILO = 0x0007;    // 10'b00_000_????????_00_111
	private static final int SEQU_HILO = 0x0008;    // 10'b00_000_????????_01_000
	private static final int SNEU_HILO = 0x0009;    // 10'b00_000_????????_01_001
	private static final int SLTU_HILO = 0x000a;    // 10'b00_000_????????_01_010
	private static final int SGTU_HILO = 0x000b;    // 10'b00_000_????????_01_011
	private static final int SLEU_HILO = 0x000c;    // 10'b00_000_????????_01_100
	private static final int SGEU_HILO = 0x000d;    // 10'b00_000_????????_01_101
	private static final int CBITS_HILO = 0x0016;    // 10'b00_000_????????_10_110
	private static final int SBITS_HILO = 0x0017;    // 10'b00_000_????????_10_111
	private static final int NOT_HILO = 0x001f;    // 10'b00_000_????????_11_111

	private static final int RFE_HILO = 0x2000;    // 10'b00_001_????????_00_000
	private static final int TRAP_HILO = 0x2001;    // 10'b00_001_????????_00_001
	private static final int JR_HILO = 0x2002;    // 10'b00_001_????????_00_010
	private static final int JALR_HILO = 0x2003;    // 10'b00_001_????????_00_011
	private static final int JRS_HILO = 0x2004;    // 10'b00_001_????????_00_100
	private static final int JALRS_HILO = 0x2005;    // 10'b00_001_????????_00_101
	private static final int ADD_HILO = 0x2010;    // 10'b00_001_????????_10_000
	private static final int ADDU_HILO = 0x2011;    // 10'b00_001_????????_10_001
	private static final int SUB_HILO = 0x2012;    // 10'b00_001_????????_10_010
	private static final int SUBU_HILO = 0x2013;    // 10'b00_001_????????_10_011
	private static final int AND_HILO = 0x2014;    // 10'b00_001_????????_10_100
	private static final int OR_HILO = 0x2015;    // 10'b00_001_????????_10_101
	private static final int XOR_HILO = 0x2016;    // 10'b00_001_????????_10_110
	private static final int MUL_HILO = 0x2017;    // 10'b00_001_????????_10_111
	private static final int SEQ_HILO = 0x2018;    // 10'b00_001_????????_11_000
	private static final int SNE_HILO = 0x2019;    // 10'b00_001_????????_11_001
	private static final int SLT_HILO = 0x201a;    // 10'b00_001_????????_11_010
	private static final int SGT_HILO = 0x201b;    // 10'b00_001_????????_11_011
	private static final int SLE_HILO = 0x201c;    // 10'b00_001_????????_11_100
	private static final int SGE_HILO = 0x201d;    // 10'b00_001_????????_11_101
	private static final int MOVI2S_HILO = 0x201e;    // 10'b00_001_????????_11_110
	private static final int MOVS2I_HILO = 0x201f;    // 10'b00_001_????????_11_111

	// all attributes are READ-ONLY
	/**
	 * the binary instruction
	 */
	public final int instrWord;
	/**
	 * the instruction type
	 */
	public final InstructionType type;
	/**
	 * a description of the instruction (the instruction as a human readable
	 * string)
	 */
	public final String description;

	/**
	 * variable for the first operand (if any)
	 */
	public final int opA;
	/**
	 * variable for the second operand (if any)
	 */
	public final int opB;
	/**
	 * variable for the third operand (if any)
	 */
	public final int opC;

	/**
	 * Constructor. Only used for debugging / testing purposes. Unused operands
	 * are to be set INVALID.
	 *
	 * @param type
	 *            the instruction's type
	 * @param opA
	 *            1st operand (if any)
	 * @param opB
	 *            2nd operand (if any)
	 * @param opC
	 *            3rd operand (if any)
	 */
	protected Instruction(final InstructionType type, final int opA,
						  final int opB, final int opC) {
		this.type = type;
		this.opA = opA;
		this.opB = opB;
		this.opC = opC;
		this.description = type.toString();

		switch (type) {

			/**
			 * R-Type Instructions: opA = Register (4 Bit), opB = Register (4 Bit),
			 * opC = INVALID
			 **/
			/**
			 * Note: if opA or opB are (occasionally) don't care, they must(!) be
			 * set to INVALID
			 **/

			case ADD: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(ADD_HILO);
				break;

			case ADDU: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(ADDU_HILO);
				break;

			case SUB: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SUB_HILO);
				break;

			case SUBU: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SUBU_HILO);
				break;

			case MUL: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(MUL_HILO);
				break;

			case AND: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(AND_HILO);
				break;

			case OR: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(OR_HILO);
				break;

			case XOR: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(XOR_HILO);
				break;

			case NOT: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(NOT_HILO);
				break;

			case SEQ: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SEQ_HILO);
				break;

			case SNE: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SNE_HILO);
				break;

			case SLT: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SLT_HILO);
				break;

			case SGT: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SGT_HILO);
				break;

			case SLE: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SLE_HILO);
				break;

			case SGE: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SGE_HILO);
				break;

			case SLL: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SLL_HILO);
				break;

			case SRL: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SRL_HILO);
				break;

			case SRA: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SRA_HILO);
				break;

			case MOV: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(MOV_HILO);
				break;

			case SEQU: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SEQU_HILO);
				break;

			case SNEU: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SNEU_HILO);
				break;

			case SLTU: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SLTU_HILO);
				break;

			case SGTU: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SGTU_HILO);
				break;

			case SLEU: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SLEU_HILO);
				break;

			case SGEU: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(SGEU_HILO);
				break;

			case MOVI2S: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(MOVI2S_HILO);
				break;

			case MOVS2I: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(MOVS2I_HILO);
				break;

			case MOVI2C: // assert((opA & ~BITMASK_4) == 0); // assert(opB ==
				// INVALID); // assert(opC == INVALID);
				instrWord = MOVI2C_HILO | opA << SHIFT_AMOUNT_OPA;
				break;

			case MOVC2I: // assert((opA & ~BITMASK_4) == 0); // assert(opB ==
				// INVALID); // assert(opC == INVALID);
				instrWord = MOVC2I_HILO | (opA & BITMASK_4) << SHIFT_AMOUNT_OPA;
				break;

			case CBITS: // assert(opA == INVALID); // assert((opB & ~BITMASK_4) ==
				// 0); // assert(opC == INVALID);
				instrWord = assembleRTypeOneOperantIntruction(CBITS_HILO);
				break;

			case SBITS: // assert(opA == INVALID); // assert((opB & ~BITMASK_4) ==
				// 0); // assert(opC == INVALID);
				instrWord = assembleRTypeOneOperantIntruction(SBITS_HILO);
				break;

			case JR: // assert(opA == INVALID); // assert((opB & ~BITMASK_4) == 0);
				// // assert(opC == INVALID);
				instrWord = assembleRTypeOneOperantIntruction(JR_HILO);
				break;

			case JRS: // assert(opA == INVALID); // assert((opB & ~BITMASK_4) == 0);
				// // assert(opC == INVALID);
				instrWord = assembleRTypeOneOperantIntruction(JRS_HILO);
				break;

			case JALR: // assert(opA == INVALID); // assert((opB & ~BITMASK_4) ==
				// 0); // assert(opC == INVALID);
				instrWord = assembleRTypeOneOperantIntruction(JALR_HILO);
				break;

			case JALRS: // assert(opA == INVALID); // assert((opB & ~BITMASK_4) ==
				// 0); // assert(opC == INVALID);
				instrWord = assembleRTypeOneOperantIntruction(JALRS_HILO);
				break;

			case TRAP: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_4) == 0); // assert(opC == INVALID);
				instrWord = assembleRTypeIntruction(TRAP_HILO);
				break;

			case RFE: // assert(opA == INVALID); // assert((opB & ~BITMASK_4) == 0);
				// // assert(opC == INVALID);
				instrWord = assembleRTypeOneOperantIntruction(RFE_HILO);
				break;

			/**
			 * I-Type Instructions: opA = Register (4 Bit), opB = Immediate (9 Bit),
			 * opC = INVALID
			 **/

			case ADDI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(ADDI_HI);
				break;

			case MULI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(MULI_HI);
				break;

			case ANDI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(ANDI_HI);
				break;

			case ORI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(ORI_HI);
				break;

			case XORI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(XORI_HI);
				break;

			case SLLI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SLLI_HI);
				break;

			case SRLI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SRLI_HI);
				break;

			case SRAI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SRAI_HI);
				break;

			case SEQI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SEQI_HI);
				break;

			case SNEI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SNEI_HI);
				break;

			case SLTI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SLTI_HI);
				break;

			case SGTI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SGTI_HI);
				break;

			case SLEI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SLEI_HI);
				break;

			case SGEI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SGEI_HI);
				break;

			case MOVI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(MOVI_HI);
				break;

			case BEQZ: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(BEQZ_HI);
				break;

			case BNEZ: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(BNEZ_HI);
				break;

			case LHI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(LHI_HI);
				break;

			case SIGEX: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(SIGEX_HI);
				break;

			case IFADDUI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(IFADDUI_HI);
				break;

			case IFSUBUI: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert(opC == INVALID);
				instrWord = assembleITypeIntruction(IFSUBUI_HI);
				break;

			/**
			 * J-Type Instructions: opA = Offset (13 Bit), opB = INVALID, opC =
			 * INVALID
			 **/

			case J: // assert((opA & ~BITMASK_13) == 0); // assert(opB == INVALID);
				// // assert(opC == INVALID);
				instrWord = assembleJTypeIntruction(J_HI);
				break;

			case JF: // assert((opA & ~BITMASK_13) == 0); // assert(opB == INVALID);
				// // assert(opC == INVALID);
				instrWord = assembleJTypeIntruction(J_HI);
				break;

			case BEQZC: // assert((opA & ~BITMASK_13) == 0); // assert(opB ==
				// INVALID); // assert(opC == INVALID);
				instrWord = assembleJTypeIntruction(BEQZC_HI);
				break;

			case BNEZC: // assert((opA & ~BITMASK_13) == 0); // assert(opB ==
				// INVALID); // assert(opC == INVALID);
				instrWord = assembleJTypeIntruction(BNEZC_HI);
				break;

			case JALS: // assert((opA & ~BITMASK_13) == 0); // assert(opB ==
				// INVALID); // assert(opC == INVALID);
				instrWord = assembleJTypeIntruction(JALS_HI);
				break;

			/**
			 * M-Type Instructions: opA = Register (4 Bit), opB = Immediate (9 Bit),
			 * opC = Displacement (5 Bit)
			 **/

			case L9: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert((opC & ~BITMASK_5) == 0);
				instrWord = assembleMTypeIntruction(L9_HI);
				break;

			case S9: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert((opC & ~BITMASK_5) == 0);
				instrWord = assembleMTypeIntruction(S9_HI);
				break;

			case L18: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert((opC & ~BITMASK_5) == 0);
				instrWord = assembleMTypeIntruction(L18_HI);
				break;

			case S18: // assert((opA & ~BITMASK_4) == 0); // assert((opB &
				// ~BITMASK_9) == 0); // assert((opC & ~BITMASK_5) == 0);
				instrWord = assembleMTypeIntruction(S18_HI);
				break;

			default:
				instrWord = 0;
				break; // never end up here
		}

	}

	/**
	 * Constructor.
	 *
	 * @param type
	 *            the instruction's type
	 * @param instrWord
	 *            the instruction word
	 * @param opA
	 *            1st operand (or INVALID)
	 * @param opB
	 *            2nd operand (or INVALID)
	 * @param opC
	 *            3rd operand (or INVALID)
	 * @param description
	 *            a mnemonic string description
	 */
	private Instruction(final InstructionType type, final int instrWord,
						final int opA, final int opB, final int opC,
						final String description) {

		this.instrWord = instrWord;
		this.type = type;
		this.opA = opA;
		this.opB = opB;
		this.opC = opC;
		this.description = description;
	}

	/**
	 * Creates an "instruction" representing a hardware interrupt.
	 *
	 * @return the interrupt instruction
	 */
	public static Instruction createInterruptInstruction() {
		return new Instruction(InstructionType.INTERRUPT, INVALID, INVALID,
				INVALID, INVALID, "INTERRUPT");
	}

	/**
	 * Creates an instruction using the instruction word. If the bits do not
	 * refer to a proper instruction, null will be returned.
	 *
	 * @param instrWord
	 *            the instruction word
	 * @return the instruction or null
	 */
	public static Instruction createInstruction(final int instrWord) {

		Instruction instr = sInstrCache.get(instrWord);

		if (instr != null) {
			return instr;
		}

		final InstructionType type;
		int opA = INVALID;
		int opB = INVALID;
		int opC = INVALID;
		final String description;

		// {ir[17:16], ir[15:13]}
		final int opCode_hi = (instrWord & (BITMASK_5 << 13));
		switch (opCode_hi) {

			case J_HI:
				type = InstructionType.J;
				break;
			case JALS_HI:
				type = InstructionType.JALS;
				break;
			case BEQZ_HI:
				type = InstructionType.BEQZ;
				break;
			case BNEZ_HI:
				type = InstructionType.BNEZ;
				break;
			case BEQZC_HI:
				type = InstructionType.BEQZC;
				break;
			case BNEZC_HI:
				type = InstructionType.BNEZC;
				break;
			case ADDI_HI:
				type = InstructionType.ADDI;
				break;
			case MOVI_HI:
				type = InstructionType.MOVI;
				break;
			case LHI_HI:
				type = InstructionType.LHI;
				break;
			case SIGEX_HI:
				type = InstructionType.SIGEX;
				break;
			case ANDI_HI:
				type = InstructionType.ANDI;
				break;
			case ORI_HI:
				type = InstructionType.ORI;
				break;
			case XORI_HI:
				type = InstructionType.XORI;
				break;
			case MULI_HI:
				type = InstructionType.MULI;
				break;
			case L9_HI:
				type = InstructionType.L9;
				break;
			case S9_HI:
				type = InstructionType.S9;
				break;
			case L18_HI:
				type = InstructionType.L18;
				break;
			case S18_HI:
				type = InstructionType.S18;
				break;
			case SLLI_HI:
				type = InstructionType.SLLI;
				break;
			case JF_HI:
				type = InstructionType.JF;
				break;
			case SRLI_HI:
				type = InstructionType.SRLI;
				break;
			case SRAI_HI:
				type = InstructionType.SRAI;
				break;
			case SEQI_HI:
				type = InstructionType.SEQI;
				break;
			case SNEI_HI:
				type = InstructionType.SNEI;
				break;
			case SLTI_HI:
				type = InstructionType.SLTI;
				break;
			case SGTI_HI:
				type = InstructionType.SGTI;
				break;
			case SLEI_HI:
				type = InstructionType.SLEI;
				break;
			case SGEI_HI:
				type = InstructionType.SGEI;
				break;
			case IFADDUI_HI:
				type = InstructionType.IFADDUI;
				break;
			case IFSUBUI_HI:
				type = InstructionType.IFSUBUI;
				break;

			default:
				// {ir[17:16], ir[15:13], ir[4:3], ir[2:0]}
				final int opCode_hilo = ((instrWord & (BITMASK_5 << 13))/* >> 8 */)
						| (instrWord & BITMASK_5);
				switch (opCode_hilo) {

					case MOVI2C_HILO:
						type = InstructionType.MOVI2C;
						break;
					case MOVC2I_HILO:
						type = InstructionType.MOVC2I;
						break;
					case SLL_HILO:
						type = InstructionType.SLL;
						break;
					case MOV_HILO:
						type = InstructionType.MOV;
						break;
					case SRL_HILO:
						type = InstructionType.SRL;
						break;
					case SRA_HILO:
						type = InstructionType.SRA;
						break;
					case SEQU_HILO:
						type = InstructionType.SEQU;
						break;
					case SNEU_HILO:
						type = InstructionType.SNEU;
						break;
					case SLTU_HILO:
						type = InstructionType.SLTU;
						break;
					case SGTU_HILO:
						type = InstructionType.SGTU;
						break;
					case SLEU_HILO:
						type = InstructionType.SLEU;
						break;
					case SGEU_HILO:
						type = InstructionType.SGEU;
						break;
					case CBITS_HILO:
						type = InstructionType.CBITS;
						break;
					case SBITS_HILO:
						type = InstructionType.SBITS;
						break;
					case RFE_HILO:
						type = InstructionType.RFE;
						break;
					case TRAP_HILO:
						type = InstructionType.TRAP;
						break;
					case JR_HILO:
						type = InstructionType.JR;
						break;
					case JALR_HILO:
						type = InstructionType.JALR;
						break;
					case JRS_HILO:
						type = InstructionType.JRS;
						break;
					case JALRS_HILO:
						type = InstructionType.JALRS;
						break;
					case ADD_HILO:
						type = InstructionType.ADD;
						break;
					case ADDU_HILO:
						type = InstructionType.ADDU;
						break;
					case SUB_HILO:
						type = InstructionType.SUB;
						break;
					case SUBU_HILO:
						type = InstructionType.SUBU;
						break;
					case AND_HILO:
						type = InstructionType.AND;
						break;
					case OR_HILO:
						type = InstructionType.OR;
						break;
					case XOR_HILO:
						type = InstructionType.XOR;
						break;
					case NOT_HILO:
						type = InstructionType.NOT;
						break;
					case MUL_HILO:
						type = InstructionType.MUL;
						break;
					case SEQ_HILO:
						type = InstructionType.SEQ;
						break;
					case SNE_HILO:
						type = InstructionType.SNE;
						break;
					case SLT_HILO:
						type = InstructionType.SLT;
						break;
					case SGT_HILO:
						type = InstructionType.SGT;
						break;
					case SLE_HILO:
						type = InstructionType.SLE;
						break;
					case SGE_HILO:
						type = InstructionType.SGE;
						break;
					case MOVI2S_HILO:
						type = InstructionType.MOVI2S;
						break;
					case MOVS2I_HILO:
						type = InstructionType.MOVS2I;
						break;

					default:
						sInstrCache.put(instrWord, null);
						return null; // invalid instruction word
				}

		}

		String spaces = "";

		switch (type) {

			/**
			 * R-Type Instructions: opA = Register (4 Bit), opB = Register (4 Bit),
			 * opC = INVALID
			 **/

			case ADD:
			case ADDU:
			case SUB:
			case SUBU:
			case MUL:
			case AND:
			case OR:
			case XOR:
			case NOT:
			case SEQ:
			case SNE:
			case SLT:
			case SGT:
			case SLE:
			case SGE:
			case SLL:
			case SRL:
			case SRA:
			case MOV:
			case SEQU:
			case SNEU:
			case SLTU:
			case SGTU:
			case SLEU:
			case SGEU:
				opA = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPA)) >>> SHIFT_AMOUNT_OPA;
				opB = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPB)) >>> SHIFT_AMOUNT_OPB;
				opC = INVALID;
				for (int i = 0; i < (7 - type.toString().length()); ++i)
					spaces += " ";
				description = type.toString() + spaces
						+ String.format(" R%d, R%d", opA, opB);
				break;

			case JR:
			case JRS:
			case JALR:
			case JALRS:
			case RFE:
				opA = INVALID;
				opB = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPB)) >>> SHIFT_AMOUNT_OPB;
				opC = INVALID;
				for (int i = 0; i < (7 - type.toString().length()); ++i)
					spaces += " ";
				description = type.toString() + spaces + String.format(" R%d", opB);
				break;

			case MOVI2S:
			case MOVS2I:
				opA = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPA)) >>> SHIFT_AMOUNT_OPA;
				opB = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPB)) >>> SHIFT_AMOUNT_OPB;
				opC = INVALID;
				description = type.toString() + " "
						+ String.format(" R%d 0x%x", opA, opB);
				break;

			case MOVI2C:
			case MOVC2I:
				opA = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPA)) >>> SHIFT_AMOUNT_OPA;
				opB = INVALID; // opCode & (BITMASK_4 >>> 5);
				opC = INVALID;
				description = type.toString() + " " + String.format(" R%d ", opA);
				break;

			case CBITS:
			case SBITS:
				opA = INVALID; // opCode & (BITMASK_4 >>> 9);
				opB = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPB)) >>> SHIFT_AMOUNT_OPB;
				opC = INVALID;
				if (opB == 1) {
					description = type.toString() + "  " + " MM";
				} else if (opB == 2) {
					description = type.toString() + "  " + " INT";
				} else {
					description = type.toString() + "  " + " CC";
				}
				break;

			/**
			 * I-Type Instructions: opA = Register (4 Bit), opB = Immediate (9 Bit),
			 * opC = INVALID
			 **/

			case ADDI:
			case MULI:
			case ANDI:
			case ORI:
			case XORI:
			case SLLI:
			case SRLI:
			case SRAI:
			case SEQI:
			case SNEI:
			case SLTI:
			case SGTI:
			case SLEI:
			case SGEI:
			case MOVI:
			case BEQZ:
			case BNEZ:
			case LHI:
			case SIGEX:
			case IFADDUI:
			case IFSUBUI:
				opA = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPA)) >>> SHIFT_AMOUNT_OPA;
				opB = instrWord & BITMASK_9;
				opC = INVALID;
				for (int i = 0; i < (7 - type.toString().length()); ++i)
					spaces += " ";
				description = type.toString() + spaces
						+ String.format(" R%d, 0x%x", opA, opB);
				break;

			/**
			 * J-Type Instructions: opA = Offset (13 Bit), opB = INVALID, opC =
			 * INVALID
			 **/

			case J:
			case JF:
			case BEQZC:
			case BNEZC:
			case JALS:
				opA = instrWord & BITMASK_13;
				opB = INVALID;
				opC = INVALID;
				for (int i = 0; i < (7 - type.toString().length()); ++i)
					spaces += " ";
				description = type.toString() + spaces
						+ String.format(" 0x%x", opA);
				break;

			/**
			 * M-Type Instructions: opA = Register (4 Bit), opB = Immediate (9 Bit),
			 * opC = Displacement (5 Bit)
			 **/

			case S18:
			case S9:
				opA = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPA)) >>> SHIFT_AMOUNT_OPA;
				opB = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPB)) >>> SHIFT_AMOUNT_OPB;
				opC = instrWord & BITMASK_5;
				for (int i = 0; i < (7 - type.toString().length()); ++i)
					spaces += " ";
				description = type.toString() + spaces
						+ String.format(" 0x%x(R%d), R%d", opC, opB, opA);
				break;

			case L9:
			case L18:
				opA = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPA)) >>> SHIFT_AMOUNT_OPA;
				opB = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPB)) >>> SHIFT_AMOUNT_OPB;
				opC = instrWord & BITMASK_5;
				for (int i = 0; i < (7 - type.toString().length()); ++i)
					spaces += " ";
				description = type.toString() + spaces
						+ String.format(" R%d, 0x%x(R%d)", opA, opC, opB);
				break;

			case TRAP:
				opA = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPA)) >>> SHIFT_AMOUNT_OPA;
				opB = (instrWord & (BITMASK_4 << SHIFT_AMOUNT_OPB)) >>> SHIFT_AMOUNT_OPB;
				final int addr = (opA << 4 | opB);
				description = type.toString() + "   "
						+ String.format(" 0x%x", addr);
				break;

			default:
				opA = opB = opC = INVALID;
				description = null;

		}

		instr = new Instruction(type, instrWord, opA, opB, opC, description);
		sInstrCache.put(instrWord, instr);

		return instr;

	}

	private int assembleRTypeIntruction(int instruction) {
		return instruction | opA << SHIFT_AMOUNT_OPA | opB << SHIFT_AMOUNT_OPB;
	}

	private int assembleRTypeOneOperantIntruction(int instruction) {
		return instruction | opB << SHIFT_AMOUNT_OPB;
	}

	private int assembleITypeIntruction(int instruction) {
		return instruction | opA << SHIFT_AMOUNT_OPA | opB;
	}

	private int assembleMTypeIntruction(int instruction) {
		return instruction | opA << SHIFT_AMOUNT_OPA | opB << SHIFT_AMOUNT_OPB | opC;
	}

	private int assembleJTypeIntruction(int instruction) {
		return instruction | opA;
	}



}
