/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.engine.processor;

import de.tu_darmstadt.rs.sim.engine.RefreshHandler_I;
import de.tu_darmstadt.rs.sim.engine.SimulationComponent_I;
import de.tu_darmstadt.rs.sim.engine.peripheral.interrupts.InterruptController;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineUI;

import java.util.Collection;

/**
 * Interface for a processor core. Depending on whether an interrupt controller
 * is used there exist two implementing classes: {@link SimpleProcessorCore} and
 * {@link InterruptableProcessorCore}.
 */
public interface ProcessorCore_I {

	/**
	 * execute a step / process one instruction in one stage
	 *
	 * @param doInterrupt true if the normal processing should stop due to an interrupt
	 */
	void onExecuteStep(final boolean doInterrupt);

	/**
	 * reset/stop the simulation
	 */
	void onResetSimulation();

	/**
	 * check whether the address is in delay window
	 *
	 * @param address the address to check
	 * @return true if the address is in the delay window
	 */
	boolean isAddressInDelayWindow(final int address);

	/**
	 * set the interrupt controller (if this is possible for the sub class)
	 *
	 * @param irqController the interrupt controller
	 */
	void setInterruptController(final InterruptController irqController);

	/**
	 * get the pipeline informational object
	 *
	 * @return pipeline information
	 */
	PipelineUI getPipelineUI();

	/**
	 * get all components which can be restarted and setup
	 *
	 * @return all simulation components
	 */
	Collection<SimulationComponent_I> getSimulationComponents();

	/**
	 * set a refresh handler
	 *
	 * @param fRefreshHandler the {@link RefreshHandler_I}
	 */
	void setRefreshHandler(RefreshHandler_I fRefreshHandler);
}
