/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine.processor;


import de.tu_darmstadt.rs.sim.engine.AbstractHardwareComponent;
import de.tu_darmstadt.rs.sim.engine.AccessManager;
import de.tu_darmstadt.rs.sim.engine.RefreshHandler_I;
import de.tu_darmstadt.rs.sim.engine.SimulationComponent_I;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;


/**
 * The class RegisterFile represents the Spartan RAM containing the general
 * purpose registers. Values may be addresses globally or locally, i.e. in
 * context of the register file window. The register addresses may be connected
 * with watchpoints.
 */
public class RegisterFile extends AbstractHardwareComponent implements
		SimulationComponent_I {

	private final int[] regArray;
	private SfRegisters sfr;
	private final AccessManager accessManager;
	private final SimulationManager simMgr;

	/**
	 * base address for the general purpose registers
	 */
	public final static int REG_BASE_ADR = 0x40000;

	/**
	 * Constructor.
	 *
	 * @param refreshHandler
	 *            a {@link RefreshHandler_I} to which refresh requests should be
	 *            sent
	 * @param sfr
	 *            the SFRegisters (needed to access the window base value)
	 * @param simMgr
	 *            the SimulationManager (needed to check if register is in a
	 *            distinct register window)
	 */
	public RegisterFile(RefreshHandler_I refreshHandler, final SfRegisters sfr,
						SimulationManager simMgr) {

		super(refreshHandler);
		this.sfr = sfr;
		regArray = new int[REG_FILE_COUNT];
		accessManager = AccessManager.getTheInstance();
		sfr.setWindow(0);
		this.simMgr = simMgr;
	}

	/**
	 * sets the register value in the context of the register window and checks
	 * whether the simulation should be stopped.
	 *
	 * @param regNo
	 *            the local register number [0, 15]
	 * @param value
	 *            the value to be set
	 * @param currWindow
	 *            the register window
	 */
	@Deprecated
	public void setRegisterLocallyWithSideEffects(final int regNo,
												  final int value, final int currWindow) {
		assert ((value & ~BITMASK_18) == 0);
		assert (regNo >= 0);
		assert (regNo < 16);

		final int address = (regNo <= 3)? regNo : regNo + currWindow * 8;
		accessManager.checkForEvent(REG_BASE_ADR + address);
		setRegisterLocally(regNo, value, currWindow);
	}

	public void setRegisterWithSideEffects(final RegisterDescriptor reg, final int value) {
		assert ((value & ~BITMASK_18) == 0);
		assert (reg.localRegister >= 0);
		assert (reg.localRegister < 16);

		final int address = reg.getAddress();
		regArray[address] = value;

		fireRefreshEvent(address);
	}

	public void setRegister(final RegisterDescriptor reg, final int value) {
		accessManager.checkForEvent(REG_BASE_ADR + reg.getAddress());
		setRegisterWithSideEffects(reg, value);
	}


	/**
	 * sets the register value in the context of the register window
	 *
	 * @param regNo
	 *            the local register number [0, 15]
	 * @param value
	 *            the value to be set
	 * @param currWindow
	 *            the register window
	 */
	@Deprecated
	public void setRegisterLocally(final int regNo, final int value,
								   final int currWindow) {
		assert ((value & ~BITMASK_18) == 0);
		assert (regNo >= 0);
		assert (regNo < 16);

		final int address = (regNo <= 3)? regNo : regNo + currWindow * 8;
		regArray[address] = value;

		fireRefreshEvent(address);
	}

	/**
	 * Retrieves a register value in context of the register window and checks
	 * whether the simulation should be stopped.
	 *
	 * @param regNo
	 *            the local register number [0, 15]
	 * @return the register value
	 */
	@Deprecated
	public int getRegisterLocallyWithSideEffects(final int regNo, final int currWindow) {

		assert (regNo >= 0);
		assert (regNo < 16);

		final int address = (regNo <= 3)? regNo : regNo + currWindow * 8;
		accessManager.checkForEvent(REG_BASE_ADR + address);
		return getRegisterLocally(regNo, currWindow);
	}

	@Deprecated
	public int getRegisterLocallyWithSideEffects(final int regNo) {
		return getRegisterLocallyWithSideEffects(regNo, sfr.getWindow());
	}


	public int getRegister(final RegisterDescriptor reg) {
		assert (reg.localRegister >= 0);
		assert (reg.localRegister < 16);

		final int address = reg.getAddress();
		return regArray[address];
	}

	public int getRegisterWithSideEffects(final RegisterDescriptor reg) {
		accessManager.checkForEvent(REG_BASE_ADR + reg.getAddress());
		return getRegister(reg);
	}

	/**
	 * Retrieves a register value in context of the register window.
	 *
	 * @param regNo
	 *            the local register number [0, 15]
	 * @return the register value
	 */
	@Deprecated
	public int getRegisterLocally(final int regNo, final int currWindow) {

		assert (regNo >= 0);
		assert (regNo < 16);

		final int address = (regNo <= 3)? regNo : regNo + currWindow * 8;
		return regArray[address];
	}

	@Deprecated
	public int getRegisterLocally(final int regNo) {
		return getRegisterLocally(regNo, sfr.getWindow());
	}

	/**
	 * Changes a register value, globally addressed (regardless of the register
	 * base window) and checks whether the simulation should be stopped.
	 *
	 * @param address
	 *            register number
	 * @param value
	 *            the new register value
	 */
	@Deprecated
	public void setRegisterGloballyWithSideEffects(final int address,
												   final int value) {
		accessManager.checkForEvent(REG_BASE_ADR + address);
		setRegisterGlobally(address, value);
	}

	/**
	 * Changes a register value, globally addressed (regardless of the register
	 * base window).
	 *
	 * @param address
	 *            register number
	 * @param value
	 *            the new register value
	 */
	@Deprecated
	public void setRegisterGlobally(final int address, final int value) {
		regArray[address] = value;
		fireRefreshEvent(address);
	}

	/**
	 * Retrieves a register value, globally addressed (regardless of the
	 * register base window) and checks whether the simulation should be
	 * stopped.
	 *
	 * @param address
	 *            register's address
	 * @return the register value
	 */
	@Deprecated
	public int getRegisterGloballyWithSideEffects(final int address) {
		accessManager.checkForEvent(REG_BASE_ADR + address);

		return getRegisterGlobally(address);
	}

	/**
	 * Retrieves a register value, globally addressed (regardless of the
	 * register base window).
	 *
	 * @param address
	 *            register's address
	 * @return the register value
	 */
	@Deprecated
	public int getRegisterGlobally(final int address) {
		return regArray[address];
	}

	/**
	 * Retrieves the context-dependend register number out from a global
	 * register address.
	 *
	 * @param globalReg
	 *            global register number
	 * @return local register number, or INVALID if out of context
	 */
	public int globalToLocal(final int globalReg) {

		if (globalReg <= 3) {
			return globalReg;
		}

		final int localReg = globalReg - sfr.getWindow() * 8;
		if ((localReg >= 4) && (localReg <= 15)) {
			return localReg;
		} else {
			return INVALID;
		}
	}

	/**
	 * Retrieves the register window base value. Call will be just forwarded to
	 * the SFRegisters.
	 *
	 * @return the window base
	 */
	public int getRegisterWindowBase() {
		return sfr.getWindow();
	}

	/**
	 * Check whether a (global) register address in within the register file.
	 * The register window does not matter.
	 *
	 * @param address
	 *            the address
	 * @return true, if it is a valid address
	 */
	public static boolean containsAddress(final int address) {

		if (address < 0) {
			return false;
		}
		if (address >= REG_FILE_COUNT) {
			return false;
		}

		return true;
	}

	/**
	 * Clears the register file, e.g. resets the window base. DELETES the
	 * registers' content.
	 */
	@Override
	public void restart() {

		sfr.setWindow(0);
		for (int i = 0; i < regArray.length; i++) {
			regArray[i] = 0;
		}

		fireRefreshEvent(COMPLETE_REFRESH);
	}

	@Override
	public void setup() {
		//unused
	}

	/**
	 * Checks if a watchpoint is connected to a global address.
	 *
	 * @param address
	 *            the address to be checked
	 * @return true, if there is a watchpoint on that address
	 */
	public boolean hasWatchpoint(final int address) {
		return accessManager.hasWatchpoint(REG_BASE_ADR + address);
	}

	/**
	 * Toggles the watchpoint at a global address.
	 *
	 * @param address
	 *            the address
	 */
	public void toggleWatchpoint(final int address) {
		accessManager.toggleWatchpoint(REG_BASE_ADR + address);
		fireRefreshEvent(address);
	}

	/**
	 * check whether the address is in the global register address range
	 *
	 * @param address
	 *            address of the register
	 * @return true if the register is a global register, false otherwise
	 */
	public boolean isAddressGlobalRegister(int address) {
		return (address >= 0) && (address < 4);
	}

	/**
	 * check whether the address is accessible from the ifid stage
	 *
	 * @param address
	 *            address of the register
	 * @return true if the register is accessible from ifid stage, false
	 *         otherwise
	 */
	public boolean isAddressInIfId(int address) {
		final int localReg = address - sfr.getWindow() * 8;
		return (localReg >= 4) && (localReg <= 15);
	}

	/**
	 * check whether the address is accessible from the wb stage
	 *
	 * @param address
	 *            address of the register
	 * @return true if the register is accessible from wb stage, false otherwise
	 */
	public boolean isAddressInWb(int address) {
		return simMgr.isAddressInDelayWindow(address);
	}

	/**
	 * processed each clock
	 */
	public void onClock() {
		fireRefreshEvent(COMPLETE_REFRESH);
	}


	public static RegisterDescriptor getRegisterDescriptor(int localReg, int window) {
		return new RegisterDescriptor((byte) localReg, (short) window);//TODO: reduce var sizes
	}

	/**
	 * Created by ramon on 03.09.15.
	 */
	public static class RegisterDescriptor {

		private byte localRegister;
		private short window;


		private RegisterDescriptor(byte localReg, short window) {
			this.localRegister = localReg;
			this.window = window;
		}

		public int getAddress() {
			return (localRegister <= 3)? localRegister : localRegister + window * 8;
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) {
				return true;
			}
			if (o == null || getClass() != o.getClass()) {
				return false;
			}

			RegisterDescriptor that = (RegisterDescriptor) o;

			return getAddress() == that.getAddress();
		}

		@Override
		public int hashCode() {
			int result = localRegister;
			result = 31 * result + window;
			return result;
		}
	}
}
