/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden
 * 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden
 * 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden
 * 2006
 */
package de.tu_darmstadt.rs.sim.engine.processor;


import de.tu_darmstadt.rs.sim.engine.AbstractHardwareComponent;
import de.tu_darmstadt.rs.sim.engine.AccessManager;
import de.tu_darmstadt.rs.sim.engine.RefreshHandler_I;
import de.tu_darmstadt.rs.sim.engine.SimulationComponent_I;

/**
 * The SfRegisters class contains and administrates the registers: MM, CC, MUL,
 * LEDS, INTERRUPT ENABLE and the register file window base. The registers may
 * be connected with watchpoints.
 */
public class SfRegisters extends AbstractHardwareComponent implements
		SimulationComponent_I {

	private int sfr_mm;
	private int sfr_cc;
	private int sfr_mul; // [17..0]
	private int sfr_leds; // [6..0]
	private int sfr_interrupt;
	private int sfr_window; // [6..0]
	private int sfr_iv; // [17..0]
	private int sfr_tr; // [9..0]

	private final AccessManager accessManager;

	/**
	 * base address for the sf registers
	 */
	public final static int SFR_BASE_ADR = 0x40400;

	/**
	 * Constructor.
	 *
	 * @param refreshHandler
	 *            a {@link RefreshHandler_I} to which refresh requests should be
	 *            sent
	 * @param accessManager
	 *            reference to the {@link AccessManager}
	 */
	public SfRegisters(RefreshHandler_I refreshHandler,
					   AccessManager accessManager) {
		super(refreshHandler);
		this.accessManager = accessManager;
		restart();
	}

	/**
	 * Deletes all register values and reinits the access management.
	 */
	@Override
	public void restart() {

		sfr_mm = 0;
		sfr_cc = 0;
		sfr_mul = 0;
		sfr_leds = 0;
		sfr_interrupt = 0;
		sfr_window = 0;
		sfr_iv = 0x20; //(Integer) Settings.getSetting(Settings.INTERRUPT_JUMP_ADDRESS);
		sfr_tr = 0;

		fireRefreshEvent(COMPLETE_REFRESH);
	}

	@Override
	public void setup() {
		//unused
	}

	/**
	 * Retrieves the MM register value.
	 *
	 * @return the register value
	 */
	public int getMMWithSideEffects() {
		accessManager.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.MM.ordinal());
		return getMM();
	}

	/**
	 * Retrieves the MM register value.
	 *
	 * @return the register value
	 */
	public int getMM() {
		return sfr_mm;
	}

	/**
	 * Retrieves the CC register value.
	 *
	 * @return the register value
	 */
	public int getCCWithSideEffects() {
		accessManager.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.CC.ordinal());
		return getCC();
	}

	/**
	 * Retrieves the CC register value.
	 *
	 * @return the register value
	 */
	public int getCC() {
		return sfr_cc;
	}

	/**
	 * Retrieves the interrupt enable register value.
	 *
	 * @return the register value
	 */
	public int getInterruptEnableWithSideEffects() {

		accessManager
				.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.INT.ordinal());
		return sfr_interrupt;
	}

	/**
	 * Retrieves the interrupt enable register value.
	 *
	 * @return the register value
	 */
	public int getInterruptEnable() {
		return sfr_interrupt;
	}

	/**
	 * Retrieves the MUL register value.
	 *
	 * @return the register value
	 */
	public int getMULWithSideEffects() {
		accessManager
				.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.MUL.ordinal());
		return getMUL();
	}

	/**
	 * Retrieves the MUL register value.
	 *
	 * @return the register value
	 */
	public int getMUL() {
		return sfr_mul;
	}

	/**
	 * Retrieves the LED register value.
	 *
	 * @return the register value
	 */
	public int getLEDSWithSideEffects() {
		accessManager.checkForEvent(SFR_BASE_ADR
				+ SfRegisterEnum.LEDS.ordinal());
		return getLEDS();
	}

	/**
	 * Retrieves the LED register value.
	 *
	 * @return the register value
	 */
	public int getLEDS() {
		return sfr_leds;
	}

	/**
	 * Retrieves the IV register value.
	 *
	 * @return the register value
	 */
	public int getIVWithSideEffects() {
		accessManager.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.IV.ordinal());
		return sfr_iv;
	}

	/**
	 * Retrieves the IV register value.
	 *
	 * @return the register value
	 */
	public int getIV() {
		return sfr_iv;
	}

	/**
	 * Retrieves the TR register value.
	 *
	 * @return the register value
	 */
	public int getTRWithSideEffects() {
		accessManager.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.TR.ordinal());
		return getTR();
	}

	/**
	 * Retrieves the TR register value.
	 *
	 * @return the register value
	 */
	public int getTR() {
		return sfr_tr;
	}

	/**
	 * Retrieves the register file window base.
	 *
	 * @return the window base
	 */
	public int getWindowWithSideEffects() {
		accessManager
				.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.WND.ordinal());
		return getWindow();
	}

	/**
	 * Retrieves the register file window base.
	 *
	 * @return the window base
	 */
	public int getWindow() {
		return sfr_window;
	}

	/**
	 * Increments the register file window base. (MAX window * 8 + 8 = 1024)
	 */
	public void incrementWindowWithSideEffects() {
		accessManager
				.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.WND.ordinal());
		incrementWindow();
	}

	/**
	 * Increments the register file window base. (MAX window * 8 + 8 = 1024)
	 */
	public void incrementWindow() {
		if (sfr_window == (BITMASK_7 - 1)) {
			accessManager.pushErrorMessage("Register File Window Overflow");
		}

		sfr_window++;

		fireRefreshEvent(SfRegisterEnum.WND.ordinal());
	}

	/**
	 * Decrements the register file window base.
	 */
	public void decrementWindowWithSideEffects() {

		accessManager
				.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.WND.ordinal());
		decrementWindow();
	}

	/**
	 * Decrements the register file window base.
	 */
	public void decrementWindow() {

		if (sfr_window == 0) {
			accessManager
					.pushErrorMessage("Register File Window cannot be negative");
		}

		sfr_window--;

		fireRefreshEvent(SfRegisterEnum.WND.ordinal());
	}

	/**
	 * Changes the MM register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setMMWithSideEffects(final int value) {
		assert ((value & ~BITMASK_1) == 0);

		accessManager.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.MM.ordinal());
		setMM(value);
	}

	/**
	 * Changes the MM register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setMM(final int value) {
		assert ((value & ~BITMASK_1) == 0);
		sfr_mm = value;

		fireRefreshEvent(SfRegisterEnum.MM.ordinal());
	}

	/**
	 * Changes the CC register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setCCWithSideEffects(final int value) {
		assert ((value & ~BITMASK_1) == 0);

		accessManager.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.CC.ordinal());
		setCC(value);
	}

	/**
	 * Changes the CC register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setCC(final int value) {
		assert ((value & ~BITMASK_1) == 0);
		sfr_cc = value;

		fireRefreshEvent(SfRegisterEnum.CC.ordinal());
	}

	/**
	 * Changes the interrupt enable register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setInterruptEnableWithSideEffects(final int value) {
		assert ((value & ~BITMASK_1) == 0);

		accessManager
				.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.INT.ordinal());
		setInterruptEnable(value);
	}

	/**
	 * Changes the interrupt enable register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setInterruptEnable(final int value) {
		assert ((value & ~BITMASK_1) == 0);
		sfr_interrupt = value;

		fireRefreshEvent(SfRegisterEnum.INT.ordinal());
	}

	/**
	 * Changes the MUL register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setMULWithSideEffects(final int value) {
		assert ((value & ~BITMASK_18) == 0);

		accessManager
				.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.MUL.ordinal());
		setMUL(value);
	}

	/**
	 * Changes the MUL register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setMUL(final int value) {
		assert ((value & ~BITMASK_18) == 0);
		sfr_mul = value;

		fireRefreshEvent(SfRegisterEnum.MUL.ordinal());
	}

	/**
	 * Changes the LEDS register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setLEDSWithSideEffects(final int value) {
		assert ((value & ~BITMASK_7) == 0);

		accessManager.checkForEvent(SFR_BASE_ADR
				+ SfRegisterEnum.LEDS.ordinal());
		setLEDS(value);
	}

	/**
	 * Changes the LEDS register value.
	 *
	 * @param value
	 *            the new value
	 */
	public void setLEDS(final int value) {
		assert ((value & ~BITMASK_7) == 0);
		sfr_leds = value;

		fireRefreshEvent(SfRegisterEnum.LEDS.ordinal());
	}

	/**
	 * Changes the IV register value. (Interrupt address)
	 *
	 * @param value
	 *            the new value
	 */
	public void setIVWithSideEffects(final int value) {
		assert ((value & ~BITMASK_18) == 0);

		accessManager.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.IV.ordinal());
		setIV(value);
	}

	/**
	 * Changes the IV register value. (Interrupt address)
	 *
	 * @param value
	 *            the new value
	 */
	public void setIV(final int value) {
		assert ((value & ~BITMASK_18) == 0);
		sfr_iv = value;

		fireRefreshEvent(SfRegisterEnum.IV.ordinal());
	}

	/**
	 * Changes the TR register value. (high 10 Bit of TRAP address)
	 *
	 * @param value
	 *            the new value
	 */
	public void setTRWithSideEffects(final int value) {
		assert ((value & ~BITMASK_10) == 0);

		accessManager.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.TR.ordinal());
		setTR(value);
	}

	/**
	 * Changes the TR register value. (high 10 Bit of TRAP address)
	 *
	 * @param value
	 *            the new value
	 */
	public void setTR(final int value) {
		assert ((value & ~BITMASK_10) == 0);
		sfr_tr = value;

		fireRefreshEvent(SfRegisterEnum.TR.ordinal());
	}

	/**
	 * Changes the register file window base value. (MAX window * 8 + 8 = 1024)
	 *
	 * @param value
	 *            the new window base
	 */
	public void setWindowWithSideEffects(final int value) {
		assert (value < BITMASK_7);

		accessManager
				.checkForEvent(SFR_BASE_ADR + SfRegisterEnum.WND.ordinal());
		setWindow(value);
	}

	/**
	 * Changes the register file window base value. (MAX window * 8 + 8 = 1024)
	 *
	 * @param value
	 *            the new window base
	 */
	public void setWindow(final int value) {
		assert (value < BITMASK_7);
		sfr_window = value;

		fireRefreshEvent(SfRegisterEnum.WND.ordinal());
	}

	/**
	 * Checks if a watchpoint is connected to a register.
	 *
	 * @param address
	 *            the register's index in the SfRegisterEnum
	 * @return true, if there is a watchpoint on that register
	 */
	public boolean hasWatchpoint(final int address) {
		return accessManager.hasWatchpoint(SFR_BASE_ADR + address);
	}

	/**
	 * Toggles the watchpoint at a register.
	 *
	 * @param address
	 *            the register's index in the SfRegisterEnum
	 */
	public void toggleWatchpoint(final int address) {
		accessManager.toggleWatchpoint(SFR_BASE_ADR + address);
		fireRefreshEvent(address);
	}
}
