/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine.processor.pipeline;

import de.tu_darmstadt.rs.sim.engine.AccessManager;
import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.SettingListener_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationComponent_I;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.engine.processor.Instruction;
import de.tu_darmstadt.rs.sim.engine.processor.RegisterFile;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisters;
import de.tu_darmstadt.rs.sim.utilities.Formatter;

/**
 * The ExMemStages executes all calculations done in the CPUs Execute & Memory
 * pipeline stage. All register values which are not needed by the current
 * instruction are set to INVALID.
 */
public class ExMemStage implements SpartanDefinitions_I, SimulationComponent_I, SettingListener_I {
	/* SFR Adresses */

	private final static int SFR_ADDR_STATUS = 0x0;
	private final static int SFR_ADDR_LEDS = 0x1;
	private final static int SFR_ADDR_MUL = 0x2;
	private final static int SFR_ADDR_CC = 0x3;
	private final static int SFR_ADDR_IV = 0x4;
	private final static int SFR_ADDR_TR = 0x5;


	// package scoped:
	int alu_op_a;
	int alu_op_b;
	int alu_result;
	int reg_storeSrc; // for mem stores, alu is needed to calc mem adr. but reg_no_a is src of store

	RegisterFile.RegisterDescriptor reg_no_wb; // only needed by Wb Stage
	//	RegisterFile.RegisterDescriptor reg_no_b;
	int sfr_address;
	int data_addr;
	int write_data_data;

	boolean bypassed_opA;
	boolean bypassed_opB;
	boolean bypassed_storeSrc;

	int shiftmask;

	Instruction currentInstr;

	private final RegisterFile regFile;
	boolean isJump;

	private final IfIdStage ifId;
	private final AddressSpaceManager addressSpace;
	private final SfRegisters sfr;
	private AccessManager accessManager;
	private boolean singleShift;
	private Integer dmaAddressMin;
	private Integer dmaAddressMax;

	/**
	 * Constructor.
	 *
	 * @param ifId
	 *            the IFID stage
	 * @param addressSpace
	 *            the address space
	 * @param sfr
	 *            the SF registers
	 * @param regFile
	 *            the general purpose registers
	 * @param accessManager
	 *            reference to the {@link AccessManager}
	 */
	public ExMemStage(final IfIdStage ifId,
					  final AddressSpaceManager addressSpace, final SfRegisters sfr,
					  final RegisterFile regFile, final AccessManager accessManager) {
		this.regFile = regFile;

		this.ifId = ifId;
		this.addressSpace = addressSpace;
		this.sfr = sfr;
		this.accessManager = accessManager;
		singleShift = (Boolean) Settings.getSetting(Settings.SINGLE_SHIFT);
		dmaAddressMin = (Integer) Settings.getSetting(Settings.DMA_ADDRESS_MIN);
		dmaAddressMax = (Integer) Settings.getSetting(Settings.DMA_ADDRESS_MAX);
		Settings.registerListener(Settings.SINGLE_SHIFT, this);
		Settings.registerListener(Settings.DMA_ADDRESS_MIN, this);
		Settings.registerListener(Settings.DMA_ADDRESS_MAX, this);
	}

	/**
	 * Executes the calculation.
	 *
	 * @param overwritten_reg
	 *            the register number beeing overwritten by WB parallely, if any
	 *            (used to bypass)
	 * @param new_reg_data
	 *            the corresponding register value
	 */
	public void executeStep(final RegisterFile.RegisterDescriptor overwritten_reg, final int new_reg_data) {

		// assert((overwritten_reg == INVALID) || ((overwritten_reg &
		// ~BITMASK_4) == 0));
		// assert((new_reg_data == INVALID) || ((new_reg_data & ~BITMASK_18) ==
		// 0));

		/*************************************************************************************/
		/** ALU/MUL INSTRUCTIONS **/
		/** Calculate all instructions using the ALU or MUL unit **/
		/*************************************************************************************/

		restart();

		currentInstr = ifId.currentInstr;
		if (currentInstr == null) {
			return; // nothing to do yet
		}


		// Checl for Bypasses //TODO: bypasses are redundant as WB executes __BEFORE__ EX, and results are already in registerfile.
		bypassed_opA = (overwritten_reg != null && overwritten_reg.equals(ifId.reg_no_a));
		bypassed_opB = (overwritten_reg != null && overwritten_reg.equals(ifId.reg_no_b));

		alu_op_a = bypassed_opA? new_reg_data : ifId.alu_opa;
		alu_op_b = bypassed_opB? new_reg_data : ifId.alu_opb;

		switch (currentInstr.type) {

			case ADD:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a + alu_op_b) & BITMASK_18;
				// OV Status --> CC
				if (((alu_op_a & BIT_17) == (alu_op_b & BIT_17))
						& ((alu_op_a & BIT_17) != (alu_result & BIT_17))) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case ADDU:
			case ADDI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a + alu_op_b) & BITMASK_18;
				break;

			case IFADDUI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				if (sfr.getCCWithSideEffects() == 1) {
					alu_result = (alu_op_a + alu_op_b) & BITMASK_18;
				} else {
					alu_result = (alu_op_a) & BITMASK_18;
				}
				break;

			case SUB:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a - alu_op_b) & BITMASK_18;
				// OV Status --> CC
				if (((alu_op_a & BIT_17) == (~(alu_op_b & BIT_17) & BIT_17))
						& ((alu_op_a & BIT_17) != (alu_result & BIT_17))) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SUBU:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a - alu_op_b) & BITMASK_18;
				break;

			case IFSUBUI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				if (sfr.getCCWithSideEffects() == 1) {
					alu_result = (alu_op_a - alu_op_b) & BITMASK_18;
				} else {
					alu_result = (alu_op_a) & BITMASK_18;
				}
				break;

			case MUL:
			case MULI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				final long m = (long) Formatter.signExt17(alu_op_a)
						* (long) Formatter.signExt17(alu_op_b);
				alu_result = (int) (m & BITMASK_18);
				sfr.setMULWithSideEffects(BITMASK_18 & (int) (m >>> 18));
				break;

			case AND:
			case ANDI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a & alu_op_b) & BITMASK_18;
				break;

			case OR:
			case ORI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a | alu_op_b) & BITMASK_18;
				break;

			case XOR:
			case XORI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a ^ alu_op_b) & BITMASK_18;
				break;

			case NOT:
				alu_op_a = -1;
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a ^ alu_op_b) & BITMASK_18;
				break;

			case SLL:
			case SLLI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				if (singleShift) {
					alu_result = (alu_op_a << 1);
				} else {
					alu_result = (alu_op_a << (alu_op_b & BITMASK_5));
				}

				// alu_result Bit 18 --> CC
				if ((alu_result & BIT_18) != 0) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}

				alu_result &= BITMASK_18;
				break;

			case SRL:
			case SRLI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				// alu_result Bit 0 --> CC
				shiftmask = BIT_0;
				if ((!singleShift)
						& ((alu_op_b & BITMASK_5) > 1)) {
					shiftmask = shiftmask << ((alu_op_b & BITMASK_5) - 1);
				}
				if ((alu_op_a & shiftmask) != 0) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}

				if (singleShift) {
					alu_result = alu_op_a >>> 1;
				} else {
					alu_result = alu_op_a >>> (alu_op_b & BITMASK_5);
				}
				break;

			case SRA:
			case SRAI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				// alu_result Bit 0 --> CC
				shiftmask = BIT_0;
				if ((!singleShift)
						& ((alu_op_b & BITMASK_5) > 1)) {
					shiftmask = shiftmask << ((alu_op_b & BITMASK_5) - 1);
				}
				if ((alu_op_a & shiftmask) != 0) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}

				// make alu_op_a a signed 32-bit value and use the benefits
				// of Java's arithmetical shifting (>> instead of >>>)
				if (singleShift) {
					alu_result = Formatter.signExt17(alu_op_a) >> 1;
				} else {
					alu_result = Formatter.signExt17(alu_op_a) >> (alu_op_b & BITMASK_5);
				}
				alu_result &= BITMASK_18;
				break;

			case SEQ:
			case SEQU:
			case SEQI:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				if (alu_op_a == alu_op_b) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SNE:
			case SNEU:
			case SNEI:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				if (alu_op_a != alu_op_b) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SLT:
			case SLTI:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				// mind the sign extension!!
				if (Formatter.signExt17(alu_op_a) < Formatter.signExt17(alu_op_b)) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SLTU:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				// no sign extension
				if (alu_op_a < alu_op_b) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SGT:
			case SGTI:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				// mind the sign extension!!
				if (Formatter.signExt17(alu_op_a) > Formatter.signExt17(alu_op_b)) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SGTU:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				// no sign extension
				if (alu_op_a > alu_op_b) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SLE:
			case SLEI:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				// mind the sign extension!!
				if (Formatter.signExt17(alu_op_a) <= Formatter.signExt17(alu_op_b)) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SLEU:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				// no sign extension
				if (alu_op_a <= alu_op_b) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SGE:
			case SGEI:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				// mind the sign extension!!
				if (Formatter.signExt17(alu_op_a) >= Formatter.signExt17(alu_op_b)) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SGEU:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);

				// no sign extension
				if (alu_op_a >= alu_op_b) {
					sfr.setCCWithSideEffects(1);
				} else {
					sfr.setCCWithSideEffects(0);
				}
				break;

			case SIGEX:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				switch (alu_op_b & BITMASK_5) {
					case 9: // {{9 {a[8] }}, a[8:0] }
						alu_result = Formatter.signExt8(alu_op_a) & BITMASK_18;
						break;
					case 16: // {{1 {a[15]}}, a[15:0]}
						alu_result = Formatter.signExt15(alu_op_a) & BITMASK_18;
						break;
					default: // {{10 {a[7] }}, a[7:0] }
						alu_result = Formatter.signExt7(alu_op_a) & BITMASK_18;
						break;
				}
				break;

			case LHI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = alu_op_b;
				break;

			case BEQZC:
			case BNEZC:
			/* nothing */
				break;

			case JALR:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert(reg_no_wb == 11);

				alu_result = ifId.alu_opb; // b is misused for return address, even though reg_no_b is the dest. adr. reg => circumvent reg bypass
				ifId.pc_delay = regFile.getRegister(ifId.reg_no_b); //Read after WB was executed, bypass not necessary //TODO: unify with normal procedure
				ifId.delay_bj = true;
				break;

			case JALRS:
				ifId.pc_delay = regFile.getRegister(ifId.reg_no_b); //Read after WB was executed, bypass not necessary //TODO: unify with normal procedure
				ifId.delay_bj = true;
				//$FALL-THROUGH$
			case INTERRUPT:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert(reg_no_wb == 11);

				alu_result = ifId.alu_opb; // b is misused for return address, even though reg_no_b is the dest. adr. reg => circumvent reg bypass
				sfr.incrementWindowWithSideEffects();
				break;
			case JALS:
				ifId.pc_delay = ifId.pc + ifId.offset;
				ifId.pc_delay &= BITMASK_18;
				ifId.delay_bj = true;

				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert(reg_no_wb == 11);

				alu_result = ifId.alu_opb; // b is misused for return address, even though reg_no_b is the dest. adr. reg => circumvent reg bypass
				sfr.incrementWindowWithSideEffects();
				break;

			case JRS:
			case RFE:
				ifId.pc_delay = regFile.getRegister(ifId.reg_no_b); //Read after WB was executed, bypass not necessary //TODO: unify with normal procedure
				ifId.delay_bj = true;
				sfr.decrementWindowWithSideEffects();
				break;

			case TRAP:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert(reg_no_wb == 11);

				alu_result = ifId.alu_opb; // b is misused for return address, even though reg_no_b is the dest. adr. reg => circumvent reg bypass
				sfr.incrementWindowWithSideEffects();
				break;

			case L9: {
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);
				alu_op_a = ifId.disp; // Ignore Forwarding, reg_no_a is wb reg, but alu_op_a is displacement

				final int result = (alu_op_a + alu_op_b) & BITMASK_18;
				// Mind the weird data address calculation!
				data_addr = (sfr.getMMWithSideEffects() == 1)? BIT_17
						| (result >>> 1) : (result >>> 1);

				final int lmd;
				// if (memory.containsAddress(data_addr)) {
				accessManager.checkForEvent(data_addr);
				lmd = addressSpace.readFrom(data_addr);
				// } else {
				// lmd = INVALID;
				// throw new
				// SpartanException(String.format("Invalid data address: 0x%05x",
				// data_addr));
				// }

				if ((result & BIT_0) != 0) {
					alu_result = lmd & BITMASK_9;
				} else {
					alu_result = lmd >>> 9;
				}

				// assert((alu_result & ~BITMASK_18) == 0);
				break;
			}

			case L18: {
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);
				alu_op_a = ifId.disp; // Ignore Forwarding, reg_no_a is wb reg, but alu_op_a is displacement

				final int result = (alu_op_a + alu_op_b) & BITMASK_18;
				// Mind the weird data address calculation!
				data_addr = (sfr.getMMWithSideEffects() == 1)? BIT_17
						| (result >>> 1) : (result >>> 1);

				// if (memory.containsAddress(data_addr)) {
				accessManager.checkForEvent(data_addr);
				alu_result = addressSpace.readFrom(data_addr);
				// } else {
				// throw new
				// SpartanException(String.format("Invalid data address: 0x%05x",
				// data_addr));
				// }

				// assert((alu_result & ~BITMASK_18) == 0);
				break;
			}

			case S9: {
				reg_storeSrc = alu_op_a;
				alu_op_a = ifId.disp;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_storeSrc & ~BITMASK_18) == 0);

				final int result = (alu_op_a + alu_op_b) & BITMASK_18;
				// Mind the weird data address calculation!
				data_addr = (sfr.getMMWithSideEffects() == 1)? BIT_17
						| (result >>> 1) : (result >>> 1);

				// if (memory.containsAddress(data_addr)) {
				if ((dmaAddressMin <= data_addr)
						&& (dmaAddressMax > data_addr)) {
					System.out.println("DMA Invalid");
					write_data_data = INVALID;
					accessManager.pushErrorMessage(String.format(
							"S9 cannot access DMA address: 0x%05x", data_addr));
				} else {
					write_data_data = addressSpace.readFrom(data_addr);
				}
				// } else {
				// write_data_data = INVALID;
				// throw new
				// SpartanException(String.format("Invalid data address: 0x%05x",
				// data_addr));
				// }

				if ((result & BIT_0) != 0) {
					write_data_data = (write_data_data & (BITMASK_9 << 9))
							| (reg_storeSrc & BITMASK_9);
				} else {
					write_data_data = (reg_storeSrc & BITMASK_9) << 9
							| (write_data_data & BITMASK_9);
				}

				// if (memory.containsAddress(data_addr)) {
				accessManager.checkForEvent(data_addr);
				addressSpace.writeTo(data_addr, write_data_data);
				// }
				break;
			}

			case S18: {
				reg_storeSrc = alu_op_a;
				alu_op_a = ifId.disp;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_storeSrc & ~BITMASK_18) == 0);

				final int result = (alu_op_a + alu_op_b) & BITMASK_18;
				// Mind the weird data address calculation!
				data_addr = (sfr.getMMWithSideEffects() == 1)? BIT_17
						| (result >>> 1) : (result >>> 1);

				write_data_data = reg_storeSrc;

				// if (memory.containsAddress(data_addr)) {
				accessManager.checkForEvent(data_addr);
				addressSpace.writeTo(data_addr, write_data_data);
				// } else {
				// throw new
				// SpartanException(String.format("Invalid data address: 0x%05x",
				// data_addr));
				// }
				break;
			}

			case MOV:
			case MOVI:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_b & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = alu_op_b;
				break;

			case MOVI2C:
				// assert((alu_op_a & ~BITMASK_18) == 0);

				sfr.setCCWithSideEffects(alu_op_a & BIT_0);
				break;

			case MOVC2I: //TODO: remove, non-existant in hardware?
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);

				alu_result = (alu_op_a & ~BIT_0) | sfr.getCCWithSideEffects();
				break;

			case MOVI2S:
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((reg_no_b & ~BITMASK_4) == 0);

				sfr_address = ifId.sfr_adr & BITMASK_3;
				switch (sfr_address) {

					case SFR_ADDR_CC:
						sfr.setCCWithSideEffects(alu_op_a & BIT_0);
						break;
					case SFR_ADDR_MUL:
						sfr.setMULWithSideEffects(alu_op_a);
						break;
					case SFR_ADDR_LEDS:
						sfr.setLEDSWithSideEffects(alu_op_a & BITMASK_7);
						break;
					case SFR_ADDR_IV:
						sfr.setIVWithSideEffects(alu_op_a);
						break;
					case SFR_ADDR_TR:
						sfr.setTRWithSideEffects((alu_op_a & BITMASK_10H) >> 8);
						break;
					case SFR_ADDR_STATUS:
						sfr.setInterruptEnableWithSideEffects((alu_op_a & BIT_8) >> 8);
						sfr.setMMWithSideEffects((alu_op_a & BIT_7) >> 7);
						sfr.setWindowWithSideEffects(alu_op_a & BITMASK_7);
						break;
					default:
						System.err.println("Illegal write to unknown SFR reg: " + sfr_address);
						//TODO: warning: write to illegal SFR!!
						break;
				}
				break;

			case MOVS2I:
				reg_no_wb = ifId.reg_no_a;
				// assert((alu_op_a & ~BITMASK_18) == 0);
				// assert((reg_no_wb & ~BITMASK_18) == 0);
				// assert((reg_no_b & ~BITMASK_4) == 0);

				sfr_address = ifId.sfr_adr & BITMASK_3;
				switch (sfr_address) {

					case SFR_ADDR_CC:
						alu_result = sfr.getCCWithSideEffects();
						break;
					case SFR_ADDR_MUL:
						alu_result = sfr.getMULWithSideEffects();
						break;
					case SFR_ADDR_LEDS:
						alu_result = sfr.getLEDSWithSideEffects();
						break;
					case SFR_ADDR_IV:
						alu_result = sfr.getIVWithSideEffects();
						break;
					case SFR_ADDR_TR:
						alu_result = (sfr.getTRWithSideEffects()) << 8;
						break;
					case SFR_ADDR_STATUS:
						alu_result = sfr.getWindowWithSideEffects()
								| (sfr.getMMWithSideEffects() << 7)
								| (sfr.getInterruptEnableWithSideEffects() << 8);
						break;
					default:
						System.err.println("Illegal read from unknown SFR reg: " + sfr_address);
						alu_result = 0;
						break;
				}
				break;

			case SBITS:
				// assert((reg_no_b & ~BITMASK_4) == 0);

				switch (ifId.bit_select & BITMASK_2) {

					case 1:
						sfr.setMMWithSideEffects(1);
						break;
					case 2:
						sfr.setInterruptEnableWithSideEffects(1);
						break;
					default:
						sfr.setCCWithSideEffects(1);
						break;
				}
				break;

			case CBITS:
				// assert((reg_no_b & ~BITMASK_4) == 0);

				switch (ifId.bit_select & BITMASK_2) {

					case 1:
						sfr.setMMWithSideEffects(0);
						break;
					case 2:
						sfr.setInterruptEnableWithSideEffects(0);
						break;
					default:
						sfr.setCCWithSideEffects(0);
						break;
				}
				break;

			// new for pc_delay
			case BEQZ:
				if (alu_op_a == 0) {
					ifId.pc_delay = ifId.pc + ifId.imm;
				} else {
					ifId.pc_delay = ifId.pc + 2;
				}
				ifId.pc_delay &= BITMASK_18;
				ifId.delay_bj = true;
				break;

			case BNEZ:
				if (alu_op_a != 0) {
					ifId.pc_delay = ifId.pc + ifId.imm;
				} else {
					ifId.pc_delay = ifId.pc + 2;
				}
				ifId.pc_delay &= BITMASK_18;
				ifId.delay_bj = true;
				break;

			case J:
				ifId.pc_delay = ifId.pc + ifId.offset;
				ifId.pc_delay &= BITMASK_18;
				ifId.delay_bj = true;
				break;
			case JR:
				ifId.pc_delay = regFile.getRegister(ifId.reg_no_b); //Read after WB was executed, bypass not necessary //TODO: unify with normal procedure
				ifId.delay_bj = true;
				break;
		}

	} // the method's end... puh

	/**
	 * Clears the pipe stage's registers.
	 */
	@Override
	public void restart() {

		currentInstr = null;

		reg_storeSrc = INVALID;
		reg_no_wb = null;
		//		reg_no_b = INVALID;
		sfr_address = INVALID;
		data_addr = INVALID;
		write_data_data = INVALID;
		alu_op_a = INVALID;
		alu_op_b = INVALID;
		alu_result = INVALID;

		bypassed_opA = false;
		bypassed_opB = false;

		ifId.delay_bj = false;
	}

	@Override
	public void setup() {
		//unused
	}

	@Override
	public String toString() {

		String str = "";
		if (reg_storeSrc != INVALID) {
			str += String.format("reg_storeSrc: 0x%x  ", reg_storeSrc);
		}
		if (reg_no_wb != null) {
			str += String.format("reg_no_wb: 0x%x  ", reg_no_wb);
		}
		if (sfr_address != INVALID) {
			str += String.format("sfr_address: 0x%x  ", sfr_address);
		}
		if (data_addr != INVALID) {
			str += String.format("data_addr: 0x%x  ", data_addr);
		}
		if (write_data_data != INVALID) {
			str += String.format("write_data_data: 0x%x  ", write_data_data);
		}
		if (alu_op_a != INVALID) {
			str += String.format("alu_op_a: 0x%x  ", alu_op_a);
		}
		if (alu_op_b != INVALID) {
			str += String.format("alu_op_b: 0x%x  ", alu_op_b);
		}
		if (alu_result != INVALID) {
			str += String.format("alu_result: 0x%x  ", alu_result);
		}
		if (bypassed_opA) {
			str += "bypassed_opA  ";
		}
		if (bypassed_opB) {
			str += "bypassed_opB  ";
		}

		return str;
	}

	@Override
	public void onSettingChange(int settingType, Object newValue) {
		if (settingType == Settings.SINGLE_SHIFT) {
			singleShift = (Boolean) Settings.getSetting(Settings.SINGLE_SHIFT);
		} else if (settingType == Settings.DMA_ADDRESS_MIN) {
			dmaAddressMin = (Integer) Settings.getSetting(Settings.DMA_ADDRESS_MIN);
		} else if (settingType == Settings.DMA_ADDRESS_MAX) {
			dmaAddressMax = (Integer) Settings.getSetting(Settings.DMA_ADDRESS_MAX);
		}
	}

}
