/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine.processor.pipeline;

import de.tu_darmstadt.rs.sim.engine.AccessManager;
import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.SimulationComponent_I;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.engine.memory.InstructionPort;
import de.tu_darmstadt.rs.sim.engine.memory.MainMemory;
import de.tu_darmstadt.rs.sim.engine.peripheral.interrupts.InterruptController;
import de.tu_darmstadt.rs.sim.engine.processor.Instruction;
import de.tu_darmstadt.rs.sim.engine.processor.InstructionType;
import de.tu_darmstadt.rs.sim.engine.processor.RegisterFile;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisters;
import de.tu_darmstadt.rs.sim.utilities.Formatter;

/**
 * The IfIdStages executes all calculations done in the CPUs Instruction Fetch &
 * Decode pipeline stage. All register values which are not needed by the
 * current instruction are set to INVALID.
 */
public class IfIdStage implements SpartanDefinitions_I, SimulationComponent_I {

	// package scoped:
	int frame = (52 - 1) * 1024; // 52 memory blocks are 104 BlockRAMs
	int pc;
	int pc_new;
	int ir;
	int offset;
	int imm;
	int imm_high;
	int disp;
	RegisterFile.RegisterDescriptor reg_no_b;
	RegisterFile.RegisterDescriptor reg_no_a;
	int reg_a;
	byte bit_select;
	byte sfr_adr;
	int alu_opa;
	int alu_opb;

	Instruction currentInstr;

	private final AddressSpaceManager addressSpace;
	private final RegisterFile regFile;
	private InterruptController irqController;
	private final SfRegisters sfr;
	private final InstructionPort instructionPort;

	int pc_delay;
	boolean delay_bj;
	private AccessManager accessManager;

	/**
	 * Constructor.
	 *
	 * @param addressSpace
	 *            the de.tu_darmstadt.rs.sim.main memory
	 * @param regFile
	 *            the register file
	 * @param irqController
	 *            the interrupt controller
	 * @param sfr
	 *            the SF registers
	 * @param accessManager
	 *            reference to the {@link AccessManager}
	 */
	public IfIdStage(final AddressSpaceManager addressSpace,
					 final RegisterFile regFile,
					 final InterruptController irqController, final SfRegisters sfr,
					 final AccessManager accessManager) {

		this.addressSpace = addressSpace;
		this.regFile = regFile;
		this.irqController = irqController;
		this.sfr = sfr;
		this.accessManager = accessManager;

		instructionPort = addressSpace.getInstructionPort();

		clear();
	}

	/**
	 * Sets the next PC value (i.e. the memory's address to be read next).
	 *
	 * @param _pc
	 *            the new pc value
	 */
	public void initiate(final int _pc) {
		assert (MainMemory.containsAddress(_pc));
		pc_new = _pc;
	}

	/**
	 * Sets the FRAME value (i.e. the memory's address to be read next).
	 *
	 * @param _frame
	 *            the new JF offset
	 */
	public void setFrame(final int _frame) {
		assert (MainMemory.containsAddress(_frame));
		frame = _frame;
	}

	/**
	 * Executes the calculation.
	 *
	 * @param old_cc
	 *            the (old) CC value BEFORE the ExMem stage has been executed
	 *            (and maybe changed it)
	 * @param interrupt
	 *            true, if an interrupt request must be executed now (instead of
	 *            the next instruction)
	 */
	public void executeStep(final int old_cc, final int new_cc, final boolean interrupt, final int ifIdRegWindow) {

		clear();

		/*************************************************************************************/
		/** 1) Fetch the next instruction **/
		/*************************************************************************************/
		pc = pc_new;
		if (!MainMemory.containsAddress(pc)) {
			accessManager.pushErrorMessage("Invalid program counter address: "
					+ pc);
		}

		if (interrupt && !delay_bj) {
			disableInterrupts();
			currentInstr = Instruction.createInterruptInstruction();
			addressSpace.setIfIdStage(INVALID);
			ir = INVALID;
		} else {
			accessManager.checkForEvent(pc);
			currentInstr = instructionPort.getInstructionAt(pc);
			addressSpace.setIfIdStage(pc);
			if (currentInstr == null) {
				accessManager.pushErrorMessage(String.format(
						"Invalid instruction at 0x%05x", pc));
				ir = INVALID;
				return;
			}

			ir = currentInstr.instrWord; // ir == memory.getValueAt(pc)
		}

		/*************************************************************************************/
		/**
		 * 2) Decode the instruction's opcode and calculate all necessary values
		 **/
		/*************************************************************************************/
		decodeInstruction(ifIdRegWindow);

		/*************************************************************************************/
		/** 3) Calculate a brand new PC **/
		/*************************************************************************************/
		calculateNewPC(new_cc);//TODO: eval if correct

		assert ((pc_new & ~BITMASK_18) == 0); // just to be sure

		if (delay_bj) {
			pc_new = pc_delay;
		} else {
			pc_delay = INVALID;
		}
	}

	private final void disableInterrupts() {
		sfr.setInterruptEnableWithSideEffects(0);
	}

	private void decodeInstruction(final int ifIdRegWindow) {//TODO: simply delay the regwindow inside SFR, also applicable for SPMC Core, if all changes to regWindow have a delay slot
		switch (currentInstr.type) {

			case ADD:
			case ADDU:
			case SUB:
			case SUBU:
			case MUL:
			case AND:
			case OR:
			case XOR:
			case SLL:
			case SRL:
			case SRA:
			case SEQ:
			case SEQU:
			case SNE:
			case SNEU:
			case SLT:
			case SLTU:
			case SGT:
			case SGTU:
			case SLE:
			case SLEU:
			case SGE:
			case SGEU:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				alu_opa = regFile.getRegisterWithSideEffects(reg_no_a);
				alu_opb = regFile.getRegisterWithSideEffects(reg_no_b);
				break;

			case NOT:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				alu_opa = -1;
				alu_opb = regFile.getRegisterWithSideEffects(reg_no_b);
				break;

			case ANDI:
			case ORI:
			case XORI:
			case IFADDUI:
			case IFSUBUI:
			case SIGEX:
			case SLLI:
			case SRLI:
			case SRAI:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				imm = currentInstr.opB; // no sign extension
				alu_opa = regFile.getRegisterWithSideEffects(reg_no_a);
				alu_opb = imm;
				break;

			case ADDI:
			case MULI:
			case SEQI:
			case SNEI:
			case SLTI:
			case SGTI:
			case SLEI:
			case SGEI:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				imm = Formatter.signExt8(currentInstr.opB) & BITMASK_18;
				alu_opa = regFile.getRegisterWithSideEffects(reg_no_a);
				alu_opb = imm;
				break;

			case LHI:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				imm_high = currentInstr.opB << 9;
				alu_opa = regFile.getRegisterWithSideEffects(reg_no_a);
				alu_opb = imm_high;
				break;

			case BEQZ:
			case BNEZ:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				imm = Formatter.signExt8(currentInstr.opB) & BITMASK_18;
				reg_a = regFile.getRegisterWithSideEffects(reg_no_a);
				alu_opa = reg_a;
				break;

			case BEQZC:
			case BNEZC:
			case J:
				offset = Formatter.signExt12(currentInstr.opA) & BITMASK_18;
				break;

			case JR:
			case JRS:
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				//			reg_b = regFile.getRegisterWithSideEffects(reg_no_b);
				break;

			case JALR:
				reg_no_a = RegisterFile.getRegisterDescriptor(11, ifIdRegWindow);// ret adr written no this window
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				//			reg_b = regFile.getRegisterWithSideEffects(reg_no_b);
				alu_opb = pc + 2;
				break;
			case JALRS:
				reg_no_a = RegisterFile.getRegisterDescriptor(11, ifIdRegWindow + 1);//return adr written to next window
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				//			reg_b = regFile.getRegisterWithSideEffects(reg_no_b);
				alu_opb = pc + 2;
				break;

			case JALS:
				reg_no_a = RegisterFile.getRegisterDescriptor(11, ifIdRegWindow + 1);//return adr written to next window
				offset = Formatter.signExt12(currentInstr.opA) & BITMASK_18;
				alu_opb = pc + 2;
				break;

			case RFE:
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				//			reg_b = regFile.getRegisterWithSideEffects(reg_no_b);
				if (irqController != null) {
					irqController.RfeInIfId(); // inform the IRQ ctrl
				}
				break;

			case TRAP:
				reg_no_a = RegisterFile.getRegisterDescriptor(11, ifIdRegWindow + 1);//return adr written to next window
				alu_opb = pc + 1;
				break;

			case L9:
			case L18:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				disp = currentInstr.opC; // swap disp and alu_opa in exMem
				alu_opb = regFile.getRegisterWithSideEffects(reg_no_b);
				break;

			case S9:
			case S18:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				disp = currentInstr.opC;
				alu_opa = regFile.getRegisterWithSideEffects(reg_no_a);// will be swapped with disp in ExMem
				alu_opb = regFile.getRegisterWithSideEffects(reg_no_b);

				break;

			case MOV:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				reg_no_b = RegisterFile.getRegisterDescriptor(currentInstr.opB, ifIdRegWindow);
				alu_opb = regFile.getRegisterWithSideEffects(reg_no_b);
				break;

			case MOVI:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				imm = currentInstr.opB; // no sign extension
				alu_opb = imm;
				break;

			case MOVI2C:
			case MOVC2I:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				alu_opa = regFile.getRegisterWithSideEffects(reg_no_a);
				break;

			case MOVI2S:
			case MOVS2I:
				reg_no_a = RegisterFile.getRegisterDescriptor(currentInstr.opA, ifIdRegWindow);
				sfr_adr = (byte) currentInstr.opB;
				alu_opa = regFile.getRegisterWithSideEffects(reg_no_a);
				break;

			case SBITS:
			case CBITS:
				bit_select = (byte) currentInstr.opB;
				if ((bit_select & BITMASK_2) == 2) {
					sfr.setInterruptEnableWithSideEffects(0); //Does this reflect Hardware?
				}
				break;

			case INTERRUPT:
				reg_no_a = RegisterFile.getRegisterDescriptor(11, ifIdRegWindow);
				alu_opb = pc; // NOT pc+1 ! (current pc has not been executed yet!)
				// sfr interrupt enable bit will be reset by the interrupt
				// controller
				break;

		}
	}

	private final void calculateNewPC(final int new_cc) {
		switch (currentInstr.type) {
			case BEQZC:
				if (new_cc == 0) {
					pc_new = pc + offset;
				} else {
					pc_new = pc + 4;
				}
				pc_new &= BITMASK_18;
				break;

			case BNEZC:
				if (new_cc != 0) {
					pc_new = pc + offset;
				} else {
					pc_new = pc + 4;
				}
				pc_new &= BITMASK_18;
				break;

			//~ case J:
			//~ case JALS:
			//~ pc_new = pc + offset;
			//~ pc_new &= BITMASK_18;
			//~ break;

			case TRAP:
				pc_new = ((sfr.getTRWithSideEffects()) << 8)
						| currentInstr.opA << 4 | currentInstr.opB;
				break;

			case INTERRUPT:
				pc_new = sfr.getIVWithSideEffects(); // Settings.INTERRUPT_JUMP_ADDRESS;
				break;

			default:
				pc_new = pc + 4;
				pc_new &= BITMASK_18;
				break;
		}
	}

	/**
	 * check whether this stage has decoded a jump instruction which uses the
	 * delay slot mechanism
	 *
	 * @return true if this stage has decoded a delayed jump, false otherwise
	 */
	public final boolean hasDelayedJump() {
		if (currentInstr == null) {
			return false;
		}
		InstructionType t = currentInstr.type;
		return (t == InstructionType.JR) || (t == InstructionType.JRS)
				|| (t == InstructionType.JALR) || (t == InstructionType.JALRS)
				|| (t == InstructionType.RFE) || (t == InstructionType.JALS) || (t == InstructionType.J);
	}

	/**
	 * Clears the pipe stage's registers.
	 */
	public void clear() {

		currentInstr = null;

		pc = INVALID;
		// do NOT clear pc_new !
		ir = INVALID;
		offset = INVALID;
		imm = INVALID;
		imm_high = INVALID;
		disp = INVALID;
		reg_no_b = null;
		reg_no_a = null;
		reg_a = INVALID;
		sfr_adr = INVALID;
		bit_select = INVALID;
		alu_opa = INVALID;
		alu_opb = INVALID;
	}

	@Override
	public void restart() {
		pc_new = 0;
		pc_delay = INVALID;
		clear();
	}

	@Override
	public void setup() {
		// nothing to do here
	}

	@Override
	public String toString() {

		String str = "";
		if (pc != INVALID) {
			str += String.format("pc: 0x%x  ", pc);
		}
		if (pc_new != INVALID) {
			str += String.format("pc_new: 0x%x  ", pc_new);
		}
		if (ir != INVALID) {
			str += String.format("ir: 0x%x  ", ir);
		}
		if (offset != INVALID) {
			str += String.format("offset: 0x%x  ", offset);
		}
		if (imm != INVALID) {
			str += String.format("imm: 0x%x  ", imm);
		}
		if (imm_high != INVALID) {
			str += String.format("imm_high: 0x%x  ", imm_high);
		}
		if (disp != INVALID) {
			str += String.format("disp: 0x%x  ", disp);
		}
		if (reg_no_a != null) {
			str += String.format("reg_no_wb: 0x%x  ", reg_no_a.getAddress());//TODO: use .toString mehtod of RegisterDescriptor
		}
		if (reg_no_b != null) {
			str += String.format("reg_no_b: 0x%x  ", reg_no_b.getAddress());
		}
		if (reg_a != INVALID) {
			str += String.format("reg_storeSrc: 0x%x  ", reg_a);
		}
		if (sfr_adr != INVALID) {
			str += String.format("sfr_adr: 0x%x  ", sfr_adr);
		}
		if (bit_select != INVALID) {
			str += String.format("bit_select: 0x%x  ", bit_select);
		}
		if (alu_opa != INVALID) {
			str += String.format("alu_opa: 0x%x  ", alu_opa);
		}
		if (alu_opb != INVALID) {
			str += String.format("alu_opb: 0x%x  ", alu_opb);
		}

		return str;
	}

	/**
	 * set the new interrupt controller
	 *
	 * @param irqController
	 *            the new interrupt controller
	 */
	public void setInterruptController(InterruptController irqController) {
		this.irqController = irqController;
	}

}
