/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden
 * 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden
 * 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden
 * 2006
 */
package de.tu_darmstadt.rs.sim.engine.processor.pipeline;

import de.tu_darmstadt.rs.sim.engine.AbstractHardwareComponent;
import de.tu_darmstadt.rs.sim.engine.RefreshHandler_I;
import de.tu_darmstadt.rs.sim.engine.memory.MainMemory;
import de.tu_darmstadt.rs.sim.engine.processor.Instruction;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisters;


/**
 * The Pipeline UI offers read access to the pipeline's internal registers and
 * the SF registers to external components (i.e. esp. the GUI). The pipeline
 * cannot be accessed directly.
 */
public class PipelineUI extends AbstractHardwareComponent {

	private final IfIdStage ifId;
	private final ExMemStage exMem;
	private final WbStage wb;
	private final SfRegisters sfr;

	/**
	 * Constructor.
	 *
	 * @param refreshHandler
	 *            a {@link RefreshHandler_I} to which refresh requests should be
	 *            sent
	 * @param ifId
	 *            the IFID stage
	 * @param exMem
	 *            the EXMEM stage
	 * @param wb
	 *            the WB stage
	 * @param sfr
	 *            the SF registers
	 */
	public PipelineUI(final RefreshHandler_I refreshHandler,
					  final IfIdStage ifId, final ExMemStage exMem, final WbStage wb,
					  final SfRegisters sfr) {

		super(refreshHandler);

		this.ifId = ifId;
		this.exMem = exMem;
		this.wb = wb;
		this.sfr = sfr;

	}

	/**
	 * Retrieves a register value.
	 *
	 * @param register
	 *            the register to be read
	 * @return the register's value -- may be INVALID
	 */
	public int getRegister(final VisiblePipelineRegister register) {

		switch (register) {

			case IFID_PC:
				return ifId.pc;
			case IFID_PC_NEW:
				return ifId.pc_new;
			case IFID_IR:
				return ifId.ir;

			case IFID_OFFSET:
				return ifId.offset;
			case IFID_IMM:
				return ifId.imm;
			case IFID_IMM_HIGH:
				return ifId.imm_high;
			case IFID_DISP:
				return ifId.disp;
			case IFID_REG_NO_A:
				return (ifId.reg_no_a != null)? ifId.reg_no_a.getAddress() : INVALID;
			case IFID_REG_NO_B:
				return (ifId.reg_no_b != null)? ifId.reg_no_b.getAddress() : INVALID;
			case IFID_REG_A:
				return ifId.reg_a;
			case IFID_ALU_OPA:
				return ifId.alu_opa;
			case IFID_ALU_OPB:
				return ifId.alu_opb;

			case EXMEM_REG_NO_WB:
				return (exMem.reg_no_wb != null)? exMem.reg_no_wb.getAddress() : INVALID;
			case EXMEM_REG_STORE_SRC:
				return exMem.reg_storeSrc;
			case EXMEM_ALU_OP_A:
				return exMem.alu_op_a;
			case EXMEM_ALU_OP_B:
				return exMem.alu_op_b;
			case EXMEM_ALU_RESULT:
				return exMem.alu_result;
			case EXMEM_DATA_ADDR:
				return exMem.data_addr;
			case EXMEM_SFR_ADDRESS:
				return exMem.sfr_address;
			case EXMEM_WRITE_DATA_DATA:
				return exMem.write_data_data;
			case EXMEM_USE_BYPASS_OPA:
				return exMem.bypassed_opA? 1 : INVALID;// boolean incompatible
			case EXMEM_USE_BYPASS_OPB:
				return exMem.bypassed_opB? 1 : INVALID; // UI expects 1 or INVALID (-1)
			case EXMEM_PC_DELAY:
				return ifId.pc_delay;

			case SFR_MM:
				return sfr.getMM();
			case SFR_CC:
				return sfr.getCC();
			case SFR_MUL:
				return sfr.getMUL();
			case SFR_INTERRUPT:
				return sfr.getInterruptEnable();
			case SFR_LEDS:
				return sfr.getLEDS();
			case SFR_IV:
				return sfr.getIV();
			case SFR_TR:
				return sfr.getTR();
			case SFR_WINDOW:
				return sfr.getWindow();

			case WB_WRITE_REG_NUMBER:
				return (wb.write_reg_number != null)? wb.write_reg_number.getAddress() : INVALID;
			case WB_WRITE_REG_DATA:
				return wb.write_reg_data;
			case WB_DO_WB:
				return wb.do_wb;
			default:
				return INVALID;

		}

	}

	/**
	 * Sets a new PC.
	 *
	 * @param value
	 *            a valid PC value
	 */
	public void setNewPC(final int value) {

		assert (MainMemory.containsAddress(value));
		ifId.pc_new = value;

		fireRefreshEvent(COMPLETE_REFRESH);
	}

	/**
	 *
	 * @return address that is going to be fetched next
	 */
	public int getCurrentAddress() {
		return ifId.pc_new;
	}

	/**
	 * Retrieves the instruction in the given pipeline stage.
	 *
	 * @param pipeStage
	 *            the stage
	 * @return the instruction
	 */
	public Instruction getCurrentInstruction(final PipelineStage pipeStage) {

		switch (pipeStage) {

			case IFID_STAGE:
				if (ifId.currentInstr == null) {
					return null;
				} else {
					return ifId.currentInstr;
				}

			case EXMEM_STAGE:
				if (exMem.currentInstr == null) {
					return null;
				} else {
					return exMem.currentInstr;
				}

			case WB_STAGE:
				if (wb.currentInstr == null) {
					return null;
				} else {
					return wb.currentInstr;
				}

			case NOT_IN_PIPE:
				return null; // never end up here
		}

		return null; // never end up here
	}

	/**
	 * inform the listeners that a refresh is needed
	 *
	 * @param address
	 *            address where the data changed
	 */
	public void requestRefresh(int address) {
		fireRefreshEvent(address);
	}
}
