/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.engine.processor.pipeline;

import de.tu_darmstadt.rs.sim.engine.SimulationComponent_I;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.engine.processor.Instruction;
import de.tu_darmstadt.rs.sim.engine.processor.InstructionType;
import de.tu_darmstadt.rs.sim.engine.processor.RegisterFile;

/**
 * The IfIdStages executes all calculations done in the CPUs Write Back pipeline
 * stage. All register values which are not needed by the current instruction
 * are set to INVALID.
 */
public class WbStage implements SpartanDefinitions_I, SimulationComponent_I {

	// package scoped:
	RegisterFile.RegisterDescriptor write_reg_number;
	int write_reg_data;
	int do_wb;

	Instruction currentInstr;

	private ExMemStage exMem;
	private RegisterFile regFile;

	/**
	 * Constructor.
	 *
	 * @param exMem
	 *            the EXMEM stage
	 * @param regFile
	 *            the register file
	 */
	public WbStage(final ExMemStage exMem, final RegisterFile regFile) {

		this.exMem = exMem;
		this.regFile = regFile;

	}

	/**
	 * Executes the calculation.
	 *
	 * @param regWindow
	 *            the old window (required when a delayed jump is executed)
	 */
	public void executeStep(int regWindow) {

		clear();

		currentInstr = exMem.currentInstr;
		if (currentInstr == null) {
			return; // nothing to do yet
		}

		// do_wb = conditional_wb ? do_wb_without_cond & reg_cc :
		// do_wb_without_cond;
		switch (currentInstr.type) {

			case JALS:
			case JALR:
			case JALRS:
			case ADD:
			case ADDU:
			case ADDI:
			case IFADDUI:
			case SUB:
			case SUBU:
			case IFSUBUI:
			case AND:
			case ANDI:
			case OR:
			case ORI:
			case XOR:
			case XORI:
			case NOT:
			case MUL:
			case MULI:
			case MOVS2I:
			case MOVC2I:
			case SLL:
			case SLLI:
			case SRL:
			case SRLI:
			case SRA:
			case SRAI:
			case MOV:
			case MOVI:
			case SIGEX:
			case LHI:
			case L18:
			case L9:
			case TRAP:
			case INTERRUPT:
				do_wb = 1;
				break;
		/*
		 * case IFADDUI: case IFSUBUI: if (sfr.getCC(AccessType.WATCHPOINT) ==
		 * 1) do_wb = 1; else do_wb = 0; break;
		 */
			default:
				do_wb = 0;

		}

		if (do_wb == 1) {
			write_reg_number = exMem.reg_no_wb;
			write_reg_data = exMem.alu_result;
			assert ((write_reg_number.getAddress() & ~BITMASK_18) == 0);
			assert ((write_reg_data & ~BITMASK_18) == 0);
			regFile.setRegisterWithSideEffects(write_reg_number, write_reg_data);
		} else if ((currentInstr.type == InstructionType.IFADDUI)
				|| (currentInstr.type == InstructionType.IFSUBUI)) {

			write_reg_number = exMem.reg_no_wb;
			write_reg_data = exMem.alu_result;
			assert ((write_reg_number.getAddress() & ~BITMASK_18) == 0);
			assert ((write_reg_data & ~BITMASK_18) == 0);

			// do not write the SFR, since CC = 0
		}

	}

	/**
	 * Clears the pipe stage's registers.
	 */
	public void clear() {
		currentInstr = null;

		write_reg_number = null;
		write_reg_data = INVALID;
		do_wb = INVALID;
	}

	@Override
	public void restart() {
		clear();
	}

	@Override
	public void setup() {
		//unused
	}

	@Override
	public String toString() {

		String str = "";
		if (do_wb != INVALID) {
			str += String.format("do_wb: 0x%x  ", do_wb);
		}
		if (write_reg_number != null) {
			str += String.format("write_reg_number: 0x%x  ", write_reg_number.getAddress());
		}
		if (write_reg_data != INVALID) {
			str += String.format("write_reg_data: 0x%x  ", write_reg_data);
		}

		return str;
	}

	/**
	 * Retrieves the register number which has been "written back", if any.
	 *
	 * @return the register number or INVALID
	 */
	public RegisterFile.RegisterDescriptor getOverwrittenRegNo() {
		return write_reg_number;
	}

	/**
	 * Retrieves the register value which has been "written back", if any.
	 *
	 * @return the value or INVALID
	 */
	public int getOverwrittenRegValue() {
		return write_reg_data;
	}

	// Note: getOverwrittenRegNo() and getOverwrittenRegValue() are (only)
	// necessary for the SimulationManager
	// to get these values and put them as parameters into the
	// exMem.execute(...) call (being executed
	// after this WB stage). The ExMem stage cannot access these values
	// directly, because its constructor
	// would need a pointer to this class. But this classes constructor already
	// needs a pointer to the
	// ExMem stage... like the egg and the hen. This solution here is a little
	// ugly, but it works :)

}
