/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gdb;

import org.slf4j.LoggerFactory;

/**
 * Created by heid on 08.12.15.
 */
public class Packet {


	private String packet;
	private String body;
	private String hash;
	private boolean verifiedHash = false;

	/**
	 * Constructor
	 *
	 * @param packet the string of the packet to be constructed
	 */
	public Packet(String packet) {
		if (packet == null) {
			LoggerFactory.getLogger(Packet.class).error("Received Packet is null");
		}
		this.packet = packet;
	}

	/**
	 * @param body the body to be verified
	 * @return true if the checksum is correct, false otherwise
	 */
	private boolean verifyHash(String body) {
		verifiedHash = false;
		if (body == null) {
			return verifiedHash;
		}
		String bh = calculateHash(body);
		String h = getHash();
		if (h.equals("==")) {
			verifiedHash = true;
		} else if (bh.equalsIgnoreCase(h)) {
			verifiedHash = true;
		}
		return verifiedHash;
	}

	/**
	 * @param value the value to be hashed
	 * @return the calculated checksum of the string value
	 */
	public static String calculateHash(String value) {
		char[] ca = value.toCharArray();
		int count = 0;
		for (int i = 0; i < ca.length; i++) {
			count = count + ca[i];
		}
		count = count % 256;
		String ret = Integer.toHexString(count);
		if (ret.length() == 1) {
			ret = 0 + ret;
		}
		return ret;
	}

	private String removeACKs(String packet) {
		if (packet == null || packet.isEmpty()) {
			return "";
		}
		packet = packet.replaceAll("\\+", "");
		packet = packet.replaceAll("\\-", "");
		return packet;
	}

	public String getPacket() {
		return packet;
	}

	public boolean isHashVerified(String body) {
		return verifyHash(body);
	}

	/**
	 * isolates the de.tu_darmstadt.rs.sim.main command from the unnecessary characters added only to maintain the gdb format
	 *
	 * @return the packet request as a String
	 */
	public String getBody() {
		if (verifiedHash) {
			return body;
		} else {
			String p = getPacket();
			p = p.substring(p.indexOf(Symbols.startSequence) + 1, p.indexOf(Symbols.stopSequence));
			if (verifyHash(p)) {
				body = p;
				return body;
			} else {
				return null;
			}

		}
	}

	/**
	 * @return only the two hash digits as String
	 */
	public String getHash() {
		if (hash != null) {
			return hash;
		} else {
			String h = removeACKs(getPacket());
			h = h.substring(h.indexOf(Symbols.stopSequence) + 1);
			if (h.length() != 2) {
				LoggerFactory.getLogger(Packet.class).error("Hash is too long");
			} else {
				hash = h;
				return hash;
			}
		}
		return null;
	}

	public String toString() {
		return packet;
	}
}
