/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gdb;

/**
 * Created by heid on 09.12.15.
 */
public final class PacketBuilder {

	private PacketBuilder() {}


	public static Packet OK() {
		return buildHeader("OK");
	}

	public static Packet emptyPacket() {
		return buildHeader("");
	}

	/**
	 * @param sig the signal
	 * @return the de.tu_darmstadt.rs.sim.signal number of the given de.tu_darmstadt.rs.sim.signal
	 */
	public static Packet stopReason(Symbols.GDBSignal sig) {
		return buildHeader("S" + sig.get2DigitHEXNr());
	}

	/**
	 * @return the de.tu_darmstadt.rs.sim.signal that caused the simulator to stop
	 */
	public static Packet stopReason() {
		return stopReason(Symbols.GDBSignal.GDB_SIGNAL_TRAP);
	}

	/**
	 * The qTStatus Reply
	 *
	 * @param running running is a single digit 1 if the trace is presently running, or 0 if not.
	 * @return An according Packet if running is 0 or 1 and null otherwise
	 */
	public static Packet qTStatus(int running) {
		if (running == 0 || running == 1) {
			return buildHeader("T" + running);
		}
		return null;
	}

	/**
	 * The qTStatus Reply
	 *
	 * @param running running is a single digit 1 if the trace is presently running, or 0 if not.
	 * @param tnotrun if true the option field is appended
	 * @return An according Packet if running is 0 or 1 and null otherwise
	 */
	public static Packet qTStatus(int running, boolean tnotrun) {
		Packet p = qTStatus(running);
		if (tnotrun) {
			p = appendField(p, "tnotrun" + Symbols.splitCMD + "0");
		}
		return p;
	}

	/**
	 * @param body the body of the packet to be
	 * @return packet after adding the first'$' and last characters'#' and the checksum
	 * to the forwarded data string
	 */
	public static Packet buildHeader(String body) {
		StringBuffer sb = new StringBuffer();
		sb.append(Symbols.startSequence);
		sb.append(body);
		sb.append(Symbols.stopSequence);
		sb.append(Packet.calculateHash(body));
		Packet p = null;
		try {
			p = new Packet(sb.toString());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return p;
	}

	/**
	 * @param pkt a packet to append to
	 * @param field a field that shall be appended
	 * @return packet after adding a semicolon followed by the String field at the end of the packet
	 */
	public static Packet appendField(Packet pkt, String field) {
		Packet p = pkt;
		try {
			String s = pkt.getBody() + Symbols.splitFields + field;
			p = buildHeader(s);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return p;
	}
}
