/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui;

import java.awt.Color;

/**
 * All global constants and enumerations are defined here. Any class may implement
 * this interface to get a convenient access.
 */
public interface GuiDefinitions_I {


	/* Colors */

	/**
	 * color for the ifid stage
	 */
	public final static Color IFID_COLOR = new Color(0, 128, 192);
	/**
	 * color for the exmem stage
	 */
	public final static Color EXMEM_COLOR = new Color(250, 120, 210);
	/**
	 * color for the wb stage
	 */
	public final static Color WB_COLOR = new Color(0, 200, 100);

	/**
	 * color for the DMA memory range in the settings panel
	 */
	public final static Color DMA_RANGE_COLOR = new Color(180, 180, 255);
	/**
	 * color for the memory range of IO address range in the settings panel
	 */
	public final static Color IO_RANGE_COLOR = new Color(240, 200, 170);
	/**
	 * color for the de.tu_darmstadt.rs.sim.main memory range in the settings panel
	 */
	public final static Color MEMORY_RANGE_COLOR = new Color(200, 230, 230);

	/**
	 * color for the selected line in the assembly window
	 */
	public final static Color LINE_SELECTED_COLOR = new Color(255, 235, 190);
	public final static Color LIGHT_GRAY_COLOR = new Color(220, 220, 220);

	/**
	 * name text for the stop button
	 */
	public final static String STOP_BUTTON_TXT = "Stop";
	/**
	 * tool tip text for the stop button
	 */
	public final static String STOP_BUTTON_HINT = "Stop program execution";
	/**
	 * name text for the run button
	 */
	public final static String START_BUTTON_TXT = "Run";
	/**
	 * tool tip text for the start button
	 */
	public final static String START_BUTTON_HINT = "Start continous program execution";
	/**
	 * name text for the step button
	 */
	public final static String STEP_BUTTON_TXT = "Step";
	/**
	 * tool tip text for the step button
	 */
	public final static String STEP_BUTTON_HINT = "Execute a single MCU cycle";
	/**
	 * name text for the reset button
	 */
	public final static String RESTART_BUTTON_TXT = "Reset";
	/**
	 * tool tip text for the restart button
	 */
	public final static String RESTART_BUTTON_HINT = "Reset the MCU and all simulation components";
	/**
	 * tool tip text for the speed selection combo box
	 */
	public final static String SPEED_COMBOBOX_HINT = "Set speed for continuous program execution";

	/**
	 * a breakpoint's color
	 */
	public final static Color BREAKPOINT_COLOR = Color.RED;
	/**
	 * a watchpoint's color
	 */
	public final static Color WATCHPOINT_COLOR = new Color(255, 115, 50);


	/**
	 * types of data to refresh, specifies what should be refreshed/updated
	 */
	enum RefreshType {
		REGISTERFILE,    // refresh only the register file
		MEMORY,        // only the memory / dma / io range
		SFR,            // only the special function registers
		PIPELINE,        // only the pc_new and all pipeline panels
		EVERYTHING        // everything without any exception
	}
}

