/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */

package de.tu_darmstadt.rs.sim.gui;

import de.tu_darmstadt.rs.sim.engine.JConfigDocumentManager;
import de.tu_darmstadt.rs.sim.engine.MessageHandler_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationController_I;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.engine.net.MpsocManager;
import de.tu_darmstadt.rs.sim.gdb.GDBTerminal;
import de.tu_darmstadt.rs.sim.gdb.GdbServer;
import de.tu_darmstadt.rs.sim.gui.controls.SignalInternalFrame;
import de.tu_darmstadt.rs.sim.gui.controls.SignalMenuItem;
import de.tu_darmstadt.rs.sim.gui.controls.UserAction;
import de.tu_darmstadt.rs.sim.preferences.ProjectComponent_I;
import de.tu_darmstadt.rs.sim.preferences.ProjectManager;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JToggleButton;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * The MainWindow is the GUI major class.
 */
@SuppressWarnings("serial")
public class MainWindow extends JFrame implements ProjectComponent_I, ChangeListener, ActionListener, ItemListener {

	private final String LAST_PROGRAM_DIR_KEY = "lastProgramDirectory";
	private final String LAST_PROJECT_DIR_KEY = "lastProjectDirectory";
	private final String LAST_PROGRAM_FILE_KEY = "lastProgramFile";

	private final List<SubWindow> subWindowList;

	private final MenuBar menuBar;
	private final ToolBar toolBar;

	private final JDesktopPane desktopPane;
	private final JToggleButton gdbServer_button;
	private final JButton gdbClient_button;

	private boolean serverButtonPressed = false;
	private boolean serverButtonActionPerformed = false;

	private GdbServer gdbServer;

	private SimulationManager simulationManager = null;

	private final static String basicName = "Spartan MC Simulator "
			+ SpartanDefinitions_I.PROGRAM_VERSION_NUMBER;

	public final SignalMenuItem mpsocOfflineMenuItem;
	public final SignalMenuItem mpsocAsyncMenuItem;
	public final SignalMenuItem mpsocSyncMenuItem;


	/**
	 * Constructor.
	 *
	 * @param simulationMgr
	 *            the SimulationManager as interface to the backend
	 * @param simulationControllerI
	 *            controller of the simulation state
	 * @param projectManager
	 *            project's component de.tu_darmstadt.rs.sim.preferences manager
	 * @param messageHandlerI
	 *            handler for occurring messages
	 * @param documentManager
	 *            manager of the configuration document
	 * @param mpsocManager the {@link MpsocManager}
	 */
	public MainWindow(final SimulationManager simulationMgr,
					  final SimulationController_I simulationControllerI, final ProjectManager projectManager,
					  final MessageHandler_I messageHandlerI,
					  final JConfigDocumentManager documentManager, final MpsocManager mpsocManager) {
		super(basicName);

		simulationManager = simulationMgr;

		MessageBox.init(this);
		SubWindow.init(this, simulationMgr);

		subWindowList = new ArrayList<SubWindow>(20);

		/* Main Window initialization */

		setSize(new Dimension(1000, 700));
		try {
			UIManager.put("swing.boldMetal", Boolean.FALSE);
		} catch (Exception e) { /* nothing */
		}


		setLocationRelativeTo(null);
		setLayout(new BorderLayout());

		desktopPane = new JDesktopPane();

		mpsocOfflineMenuItem = new SignalMenuItem(UserAction.MPSOC_DISCONNECT);
		mpsocAsyncMenuItem = new SignalMenuItem(UserAction.MPSOC_CONNECT_ASYNC);
		mpsocSyncMenuItem = new SignalMenuItem(UserAction.MPSOC_CONNECT_SYNC);

		/* menu & tool bar */

		menuBar = new MenuBar(this, simulationMgr, simulationControllerI, messageHandlerI, documentManager, projectManager, mpsocManager);

		setJMenuBar(menuBar);

		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent arg0) {
				menuBar.onExit();
			}
		});

		toolBar = new ToolBar(this, simulationMgr, simulationControllerI, desktopPane);

		gdbServer_button = new JToggleButton("GDB Server", false);
		gdbClient_button = new JButton("start GDB Client");

		gdbServer_button.setSize(100, 25);
		gdbClient_button.setSize(100, 25);

		gdbServer_button.addActionListener(this);
		gdbServer_button.addChangeListener(this);
		gdbServer_button.addItemListener(this);
		gdbClient_button.addActionListener(this);
		gdbClient_button.addChangeListener(this);
		gdbClient_button.addItemListener(this);

		toolBar.add(gdbServer_button);
		toolBar.add(gdbClient_button);


		add(toolBar, BorderLayout.NORTH);
		desktopPane.setBackground(Color.LIGHT_GRAY);

		add(desktopPane, BorderLayout.CENTER);

	}

	@Override
	public void setVisible(boolean b) {
		for (final SubWindow wnd : subWindowList) {
			wnd.onMainWndSetVisible();
		}
		super.setVisible(true);
	}


	/**
	 * extend the title of the MainWindow.
	 *
	 * @param extension
	 *            the String, which is appended after the basic simulator name
	 */
	public void setTitleExtention(String extension) {
		setTitle(basicName + " - " + extension);
	}

	/**
	 * Retrieves a list of all current sub windows.
	 *
	 * @return the list
	 */
	public List<SubWindow> getCurrentSubWindows() {
		return subWindowList;
	}

	/**
	 * detaches a sub window from the GUI and removes it from the windows list.
	 * (called via reflection, see source code of
	 * {@link #attachNewSubWindow(SubWindow)})
	 *
	 * @param fr
	 *            the new sub window to add
	 */
	public void detachSubWindow(final SignalInternalFrame fr) {
		SubWindow sw = (SubWindow) fr;
		subWindowList.remove(sw);
		menuBar.updateWindowMenu(subWindowList);
	}

	/**
	 * Attach a new sub window to the GUI and stores it in the windows list.
	 *
	 * @param subWnd
	 *            the new sub window to add
	 */
	public void attachNewSubWindow(final SubWindow subWnd) {
		subWnd.registerCloseHandler(this, "detachSubWindow",
				SignalInternalFrame.class);

		subWindowList.add(subWnd);

		desktopPane.add(subWnd);

		menuBar.updateWindowMenu(subWindowList);

		try {
			subWnd.setSelected(true);
		} catch (PropertyVetoException e) { /* does not matter */
		}

	}

	/**
	 * Loads the MainWindow's de.tu_darmstadt.rs.sim.preferences.
	 *
	 * @param prefs
	 *            the de.tu_darmstadt.rs.sim.preferences map
	 */
	@Override
	public void loadConfiguration(final Preferences prefs) {
		if (prefs == null) {
			return;
		}

		// MainWindow itself
		if (prefs.getBoolean("maximized", true)) {
			setExtendedState(JFrame.MAXIMIZED_BOTH);
		}

		final int x = prefs.getInt("boundsX", getBounds().x);
		final int y = prefs.getInt("boundsY", getBounds().y);
		final int width = prefs.getInt("boundsWidth", getBounds().width);
		final int height = prefs.getInt("boundsHeight", getBounds().height);

		setBounds(x, y, width, height);

		/* for the support of the old format (for the new one see Settings) - BEGIN*/
		String lastProjectDirectory = prefs.get(LAST_PROJECT_DIR_KEY,
				null);
		if (lastProjectDirectory != null) {
			lastProjectDirectory = lastProjectDirectory.replace('/', File.separatorChar);
			Settings.setSetting(Settings.LAST_PROJECT_DIR, new File(lastProjectDirectory));
			String lastProgramDirectory = prefs.get(LAST_PROGRAM_DIR_KEY,
					null);
			if (lastProgramDirectory != null) {
				lastProgramDirectory = lastProgramDirectory.replace('/', File.separatorChar);
				try {
					lastProgramDirectory = new URI(lastProjectDirectory).resolve(lastProgramDirectory).getPath();
					Settings.setSetting(Settings.LAST_PROGRAM_DIR, new File(lastProgramDirectory));
					String lastProgramFile = prefs.get(LAST_PROGRAM_FILE_KEY, null);
					if (lastProgramFile != null) {
						lastProgramFile = lastProgramFile.replace('/', File.separatorChar);
						lastProgramFile = new URI(lastProgramDirectory).resolve(lastProgramFile).getPath();
						Settings.setSetting(Settings.LAST_PROGRAM_FILE, new File(lastProgramFile));
					}
				} catch (URISyntaxException use) {
					// TODO Auto-generated catch block
					use.printStackTrace();
				}
			}
		}
		/* END */

		// SubWindows
		try {
			final List<SubWindow> subWindowList = new ArrayList<SubWindow>(this.subWindowList);
			for (SubWindow window : subWindowList) {
				detachSubWindow(window);
			}
			for (int i = 0; true; ++i) {
				if (!prefs.nodeExists("subwindow" + i)) {
					break;
				}

				SubWindow wnd = SubWindow.createByPreferences(prefs
						.node("subwindow" + i));

				if (wnd == null) {
					// JConfig configuration changed and periphery is not longer
					// available
					// -> skip it
					continue;
				}

				attachNewSubWindow(wnd);
			}

		} catch (BackingStoreException e) {
			// does not matter...
		}
	}

	/**
	 * Stores the MainWindow's de.tu_darmstadt.rs.sim.preferences.
	 *
	 * @param prefs
	 *            the de.tu_darmstadt.rs.sim.preferences map
	 */
	@Override
	public void storeConfiguration(final Preferences prefs) {

		assert (prefs != null);

		// MainWindow itself
		final boolean bool = (getExtendedState() == JFrame.MAXIMIZED_BOTH);
		prefs.putBoolean("maximized", bool);

		final Rectangle rect = getBounds();
		prefs.putInt("boundsX", rect.x);
		prefs.putInt("boundsY", rect.y);
		prefs.putInt("boundsWidth", rect.width);
		prefs.putInt("boundsHeight", rect.height);

		// SubWindows
		SubWindow wnd = null;

		for (int i = 0; i < subWindowList.size(); ++i) {

			wnd = subWindowList.get(i);
			wnd.storeConfiguration(prefs.node("subwindow" + i));
		}
	}

	/**
	 * Retrieves a sub window at a certain index (in the windows list)
	 *
	 * @param index
	 *            the list index
	 * @return the SubWindow instance
	 */
	public SubWindow getSubWindowAt(final int index) {
		return subWindowList.get(index);
	}

	/**
	 * Dispose all existing SubWindows.
	 */
	public void disposeAllSubWindows() {
		final int size = subWindowList.size();

		for (int i = 0; i < size; i++) {
			SubWindow subWnd = subWindowList.remove(0);
			subWnd.dispose();
		}
	}

	/**
	 * selects a sub window (called via signals (java reflections used))
	 *
	 * @param w
	 *            the internal window
	 */
	public void selectSubWindow(JInternalFrame w) {
		try {
			w.setSelected(true);
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == gdbServer_button) {
			serverButtonActionPerformed = true;
		}
		if (e.getSource() == gdbClient_button) {
			new Thread(new GDBTerminal()).start();
		}
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() == gdbServer_button) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				serverButtonPressed = true;
			} else {
				serverButtonPressed = false;
			}
		}
	}


	@Override
	public void stateChanged(ChangeEvent e) {
		if (serverButtonActionPerformed) {
			serverButtonActionPerformed = false;
			if (serverButtonPressed) {
				gdbServer = new GdbServer();
				gdbServer.start();
			} else {
				gdbServer.terminate();
			}
		}
	}

}
