/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
package de.tu_darmstadt.rs.sim.gui.McbSmall;

import de.tu_darmstadt.rs.sim.engine.peripheral.McbInterfaceSmall;
import de.tu_darmstadt.rs.sim.utilities.FifoPanel;

import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.TitledBorder;

/**
 * Panel to show the FIFOs' contents
 */
public class McbFifoPanel extends JPanel implements AdjustmentListener {

	private static final long serialVersionUID = 4445162053838929972L;

	private final FifoPanel writePanel;
	private final FifoPanel readPanel;

	private final JScrollPane writeScroll;
	private final JScrollPane readScroll;

	private final JScrollBar toScroll;

	private final McbInterfaceSmall mcb;

	/**
	 * create a new FIFO panel for the MCB
	 * @param mcb the MCB object which contains the data
	 */
	public McbFifoPanel(McbInterfaceSmall mcb) {
		super();

		this.mcb = mcb;

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		writePanel = new FifoPanel(64, FifoPanel.HORIZONTAL);
		readPanel = new FifoPanel(64, FifoPanel.HORIZONTAL);

		writeScroll = new JScrollPane(writePanel);
		writeScroll.setBorder(new TitledBorder("Write Data Fifo"));
		writeScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		writeScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		add(writeScroll);

		readScroll = new JScrollPane(readPanel);
		readScroll.setBorder(new TitledBorder("Read Data Fifo"));
		readScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		readScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		add(readScroll);

		//scroll read and writeScroll simultaneously
		toScroll = writeScroll.getHorizontalScrollBar();

		JScrollBar scrollbar = readScroll.getHorizontalScrollBar();
		scrollbar.addAdjustmentListener(this);
	}

	/**
	 * refresh the shown data
	 */
	public void refresh() {
		final int size = readPanel.getFifoSize();

		for (int i = 0; i < size; i++) {
			readPanel.setValue(i, mcb.readAtOffset(McbInterfaceSmall.DATA_READ_OFFSET, i));
			writePanel.setValue(i, mcb.readAtOffset(McbInterfaceSmall.DATA_WRITE_OFFSET, i));
		}
	}

	@Override
	public void adjustmentValueChanged(AdjustmentEvent e) {
		int value = e.getAdjustable().getValue();
		toScroll.setValue(value);
	}
}
