/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

package de.tu_darmstadt.rs.sim.gui.McbSmall;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EnumMap;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Window to configure the MCB view settings
 */
@SuppressWarnings("serial")
public class McbGraphicSettingsDialog extends JDialog {

	/**
	 * Names of the settings for the view
	 */
	public enum Option {
		/**
		 * the memory offset
		 */
		OFFSET("Memory Offset (hex)"),
		/**
		 * the width of the image in pixels
		 */
		WIDTH("Width (px)"),
		/**
		 * the height of the image in pixels
		 */
		HEIGHT("Height (px)"),
		/**
		 * the number of pixels contained in one word
		 */
		PIXEL_PER_WORD("Pixel per Word"),
		/**
		 * the number of bits which define one pixel
		 */
		BIT_PER_PIXEL("Bit per Pixel"),
		/**
		 * mask for the red portion of a pixel
		 */
		RED_MASK("Red mask (hex)"),
		/**
		 * mask for the green portion of a pixel
		 */
		GREEN_MASK("Green mask (hex)"),
		/**
		 * mask for the blue portion of a pixel
		 */
		BLUE_MASK("Blue mask (hex)");

		private String fLabel;

		private Option(String label) {
			fLabel = label;
		}

		/**
		 * retrieve the option's label
		 *
		 * @return label of the option
		 */
		public String getLabel() {
			return fLabel;
		}
	}


	private final JTextField fTFOffset;
	private final JTextField fTFWidth;
	private final JTextField fTFHeight;
	private final JTextField fTFBitPerPixel;
	private final JTextField fTFPixelPerWord;
	private final JTextField fTFRedMask;
	private final JTextField fTFGreenMask;
	private final JTextField fTFBlueMask;
	private final JButton fBTOk;
	private final JButton fBTCancel;
	private final EnumMap<Option, JTextField> fOptions;

	/**
	 * create a new {@link McbGraphicSettingsDialog}
	 *
	 * @param l listener to be invoked when the OK button is clicked
	 */
	public McbGraphicSettingsDialog(final ChangeListener l) {
		super((JFrame) null, "Memory Graphic View Settings", true);
		setLayout(new BorderLayout());
		fOptions = new EnumMap<Option, JTextField>(Option.class);
		fTFOffset = new JTextField("0");
		fTFWidth = new JTextField("100");
		fTFHeight = new JTextField("100");
		fTFBitPerPixel = new JTextField("8");
		fTFPixelPerWord = new JTextField("1");
		fTFRedMask = new JTextField("ff");
		fTFGreenMask = new JTextField("ff");
		fTFBlueMask = new JTextField("ff");
		fBTOk = new JButton("OK");
		fBTCancel = new JButton("Cancel");
		JPanel memPanel = createTitledPanel("Memory Region");
		addOption(memPanel, Option.OFFSET, fTFOffset);
		addOption(memPanel, Option.WIDTH, fTFWidth);
		addOption(memPanel, Option.HEIGHT, fTFHeight);
		addOption(memPanel, Option.PIXEL_PER_WORD, fTFPixelPerWord);
		JPanel pixPanel = createTitledPanel("Pixel Format");
		addOption(pixPanel, Option.BIT_PER_PIXEL, fTFBitPerPixel);
		addOption(pixPanel, Option.RED_MASK, fTFRedMask);
		addOption(pixPanel, Option.GREEN_MASK, fTFGreenMask);
		addOption(pixPanel, Option.BLUE_MASK, fTFBlueMask);
		JPanel dialogPanel = new JPanel(new GridLayout(1, 0));
		dialogPanel.add(memPanel);
		dialogPanel.add(pixPanel);
		add(dialogPanel, BorderLayout.CENTER);
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		buttonPanel.add(fBTOk);
		buttonPanel.add(fBTCancel);
		add(buttonPanel, BorderLayout.SOUTH);
		setResizable(false);
		pack();

		final JDialog dialog = this;
		fBTOk.addActionListener(
				new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						l.stateChanged(new ChangeEvent(dialog));
					}
				}
		);
		fBTCancel.addActionListener(
				new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				}
		);
	}

	private void addOption(Container parent, Option o, JTextField c) {
		parent.add(new JLabel(o.getLabel()));
		parent.add(c);
		fOptions.put(o, c);
	}

	private JPanel createTitledPanel(String title) {
		JPanel p = new JPanel(new GridLayout(0, 2, 5, 2));
		p.setBorder(BorderFactory.createTitledBorder(title));
		return p;
	}

	/**
	 * retrieve an option's value
	 *
	 * @param o the option
	 * @return textual representation of the option's value
	 */
	public String getOptionValue(Option o) {
		return fOptions.get(o).getText();
	}

	/**
	 * set an option's value
	 *
	 * @param o the option
	 * @param value the new value
	 */
	public void setOptionValue(Option o, Object value) {
		fOptions.get(o).setText(value.toString());
	}

	/**
	 * show the dialog
	 *
	 * @param parent the parent component
	 */
	public void show(Component parent) {
		setLocationRelativeTo(parent);
		setVisible(true);
	}
}
