/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
package de.tu_darmstadt.rs.sim.gui.McbSmall;

import de.tu_darmstadt.rs.sim.engine.peripheral.McbInterfaceSmall;
import de.tu_darmstadt.rs.sim.gui.AbstractView;
import de.tu_darmstadt.rs.sim.gui.MainWindow;
import de.tu_darmstadt.rs.sim.gui.McbSmall.McbGraphicSettingsDialog.Option;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.ColorModel;
import java.awt.image.DirectColorModel;
import java.util.prefs.Preferences;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

//import javax.swing.JLabel;
//import javax.swing.JTextField;


/**
 * interpret the memory as gray scale image and display it.
 */
@SuppressWarnings("serial")
public class McbGraphicView extends AbstractView implements ChangeListener {

	private final McbInterfaceSmall fMem;

	private JCheckBox fCBAutoUpdate;
	private JButton fBTSettings;
	private JButton fBTRefresh;
	private JComponent fCPImage;
	private JFrame fParent;

	private int fWidth;
	private int fHeight;
	private int fBitPerPix;
	private int fPixPerWord;
	private int fOffset;
	private int fRedMask;
	private int fGreenMask;
	private int fBlueMask;
	private Image fImage;
	private ColorModel fColorModel;

	// applied settings
	// private int offset = 0;
	/**
	 * bitPerPixel may only be set with the setBitPerPixel method!
	 */
	// private int bitPerPixel = 0;

	// image parameters
	// private int pixelDataMask = 0;
	// private float grayStep = 0;
	// private int bitWindow = 0;

	/**
	 * create a new {@link McbGraphicView}
	 *
	 * @param mcb
	 *            the MCB object containing the data
	 * @param parent
	 *            reference to the de.tu_darmstadt.rs.sim.main window
	 */
	public McbGraphicView(McbInterfaceSmall mcb, MainWindow parent) {
		fMem = mcb;
		fParent = parent;
		fOffset = 0;
		fWidth = 100;
		fHeight = 100;
		fBitPerPix = 8;
		fPixPerWord = 1;
		fRedMask = 0xFF;
		fGreenMask = 0xFF;
		fBlueMask = 0xFF;
		fColorModel = new DirectColorModel(fBitPerPix, fRedMask, fGreenMask,
				fBlueMask);
		setLayout(new BorderLayout(5, 5));
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		add(createControlPanel(), BorderLayout.SOUTH);
		fCPImage = new JComponent() {
			@Override
			public void paint(Graphics g) {
				super.paint(g);
				if (fImage == null) {
					return;
				}
				int x = (getWidth() - fImage.getWidth(null)) / 2;
				int y = (getHeight() - fImage.getHeight(null)) / 2;
				g.drawImage(fImage, x, y, null);
			}
		};
		JScrollPane sp = new JScrollPane(fCPImage);
		add(sp, BorderLayout.CENTER);
		applySettings();
		mcb.addRefreshListener(this);
	}

	private JComponent createControlPanel() {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.LINE_AXIS));
		fBTSettings = new JButton("Settings...");
		final ChangeListener l = this;
		fBTSettings.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				McbGraphicSettingsDialog dialog = new McbGraphicSettingsDialog(
						l);
				dialog.setOptionValue(Option.OFFSET,
						Integer.toHexString(fOffset));
				dialog.setOptionValue(Option.WIDTH, fWidth);
				dialog.setOptionValue(Option.HEIGHT, fHeight);
				dialog.setOptionValue(Option.BIT_PER_PIXEL, fBitPerPix);
				dialog.setOptionValue(Option.PIXEL_PER_WORD, fPixPerWord);
				dialog.setOptionValue(Option.RED_MASK,
						Integer.toHexString(fRedMask));
				dialog.setOptionValue(Option.GREEN_MASK,
						Integer.toHexString(fGreenMask));
				dialog.setOptionValue(Option.BLUE_MASK,
						Integer.toHexString(fBlueMask));
				dialog.show(fParent);
			}
		});
		fBTRefresh = new JButton("Refresh");
		fBTRefresh.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				refreshImage();
			}
		});
		fCBAutoUpdate = new JCheckBox("Auto refresh", false);
		p.add(fBTRefresh);
		p.add(Box.createHorizontalStrut(5));
		p.add(fCBAutoUpdate);
		p.add(Box.createHorizontalGlue());
		p.add(fBTSettings);
		return p;
	}

/*    private JTextField addFixedSizeTextField(JPanel p, String text, int cols) {
		JTextField tf = new JTextField(text, cols);
	    tf.setMaximumSize(tf.getPreferredSize());
	    p.add(tf);
	    return tf; 
	}

	private JTextField addLabeledTextField(JPanel p, String label, String text, int cols) {
	    JTextField tf;
	    p.add(new JLabel(label));
	    p.add(Box.createRigidArea(new Dimension(3,0)));
	    tf = addFixedSizeTextField(p, text, cols);
	    p.add(Box.createRigidArea(new Dimension(10,0)));
	    return tf;
	}*/

	private void applySettings() {
		fImage = createImage(new McbImageProducer(fMem, fOffset, fWidth,
				fHeight, fPixPerWord, fColorModel));
		fCPImage.setMinimumSize(new Dimension(fWidth, fHeight));
		fCPImage.repaint();
		setName(String.format("DDR Graphic View (0x%05X, %dx%d)", fOffset,
				fWidth, fHeight));
	}

	private void refreshImage() {
		fImage.flush();
		fCPImage.repaint();
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		McbGraphicSettingsDialog dialog = (McbGraphicSettingsDialog) e
				.getSource();
		int width;
		int height;
		int offs;
		int bitPerPix;
		int pixPerWord;
		int redMask;
		int greenMask;
		int blueMask;
		ColorModel cm;
		try {
			width = Integer.parseInt(dialog.getOptionValue(Option.WIDTH));
			if (width <= 0) {
				throw new IllegalArgumentException("Width must be positive");
			}
			height = Integer.parseInt(dialog.getOptionValue(Option.HEIGHT));
			if (height <= 0) {
				throw new IllegalArgumentException("Height must be posititve");
			}
			offs = Integer.parseInt(dialog.getOptionValue(Option.OFFSET), 16);
			bitPerPix = Integer.parseInt(dialog
					.getOptionValue(Option.BIT_PER_PIXEL));
			if ((bitPerPix <= 0) || (bitPerPix > 18)) {
				throw new IllegalArgumentException(
						"Bits per pixel must be between 1 and 18");
			}
			pixPerWord = Integer.parseInt(dialog
					.getOptionValue(Option.PIXEL_PER_WORD));
			if ((pixPerWord <= 0) || (pixPerWord > 18 / bitPerPix)) {
				throw new IllegalArgumentException(String.format(
						"Pixel per word must be between 1 and %d",
						18 / bitPerPix));
			}
			redMask = Integer.parseInt(dialog.getOptionValue(Option.RED_MASK),
					16);
			greenMask = Integer.parseInt(
					dialog.getOptionValue(Option.GREEN_MASK), 16);
			blueMask = Integer.parseInt(
					dialog.getOptionValue(Option.BLUE_MASK), 16);
			cm = new DirectColorModel(bitPerPix, redMask, greenMask, blueMask);
		} catch (NumberFormatException ex) {
			MessageBox.showError(String.format("Invalid number (%s)",
					ex.getMessage()));
			return;
		} catch (IllegalArgumentException ex) {
			MessageBox.showError(String.format("Value out of range (%s)",
					ex.getMessage()));
			return;
		}
		fPixPerWord = pixPerWord;
		fBitPerPix = bitPerPix;
		fWidth = width;
		fHeight = height;
		fOffset = offs;
		fRedMask = redMask;
		fGreenMask = greenMask;
		fBlueMask = blueMask;
		fColorModel = cm;
		applySettings();
		dialog.dispose();
	}

	@Override
	public void refresh(final int address) {
		if (!fCBAutoUpdate.isSelected()) {
			return;
		}
		refreshImage();
	}

	@Override
	public void loadConfiguration(final Preferences prefs) {
		if (prefs != null) {
			fWidth = prefs.getInt("width", fWidth);
			fHeight = prefs.getInt("height", fHeight);
			fBitPerPix = prefs.getInt("bitPerPix", fBitPerPix);
			fPixPerWord = prefs.getInt("pixPerWord", fPixPerWord);
			fOffset = prefs.getInt("offset", fOffset);
			fRedMask = prefs.getInt("redMask", fRedMask);
			fGreenMask = prefs.getInt("greenMask", fGreenMask);
			fBlueMask = prefs.getInt("blueMask", fBlueMask);
			fCBAutoUpdate.setSelected(prefs.getBoolean("autoUpdate", false));
			fColorModel = new DirectColorModel(fBitPerPix, fRedMask,
					fGreenMask, fBlueMask);
		}
		applySettings();
	}

	@Override
	public void storeConfiguration(final Preferences prefs) {
		prefs.putInt("width", fWidth);
		prefs.putInt("height", fHeight);
		prefs.putInt("bitPerPix", fBitPerPix);
		prefs.putInt("pixPerWord", fPixPerWord);
		prefs.putInt("offset", fOffset);
		prefs.putInt("redMask", fRedMask);
		prefs.putInt("greenMask", fGreenMask);
		prefs.putInt("blueMask", fBlueMask);
		prefs.putBoolean("autoUpdate", fCBAutoUpdate.isSelected());
	}

	@Override
	public void onViewDispose() {
		// nothing to do
	}

}
