/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

package de.tu_darmstadt.rs.sim.gui.McbSmall;

import de.tu_darmstadt.rs.sim.engine.peripheral.McbInterfaceSmall;

import java.awt.image.ColorModel;
import java.awt.image.ImageConsumer;
import java.awt.image.ImageProducer;

/**
 * Helper class to create the image from the memory.
 */
public class McbImageProducer implements ImageProducer {

	private McbInterfaceSmall fMem;
	private ColorModel fColorModel;
	private final int fOffset;
	private final int fWidth;
	private final int fHeight;
	private final int[] fPixels;
	private final int fBitPerPix;
	private final int fPixPerWord;
	private final int fPixCount;

	/**
	 * create a new {@link McbImageProducer}
	 *
	 * @param mem the MCB object containing the data
	 * @param offset offset to the memory's start address
	 * @param width width of the picture
	 * @param height height of the picture
	 * @param pixPerWord the number of pixels per data word
	 * @param colorModel the color de.tu_darmstadt.rs.sim.model for the pixels
	 */
	public McbImageProducer(McbInterfaceSmall mem, int offset, int width,
							int height, int pixPerWord, ColorModel colorModel) {
		fMem = mem;
		fWidth = width;
		fHeight = height;
		fOffset = offset;
		fPixPerWord = pixPerWord;
		fPixCount = fWidth * fHeight;
		/* safety extra space when last pixel is not on a word boundary */
		fPixels = new int[fPixCount + pixPerWord - 1];
		fColorModel = colorModel;
		fBitPerPix = fColorModel.getPixelSize();
	}

	private void updatePixelData() {
		int i;
		int j;
		int addr = fOffset;
		int val;
		for (i = 0; i < fPixCount; i += fPixPerWord) {
			val = fMem.getMemoryAt(addr++);
			for (j = fPixPerWord - 1; j >= 0; j--) {
				fPixels[i + j] = val;
				val >>= fBitPerPix;
			}
		}
	}

	@Override
	public void addConsumer(ImageConsumer ic) {
		startProduction(ic);
	}

	@Override
	public void removeConsumer(ImageConsumer ic) {
		//nothing to do here
	}

	@Override
	public boolean isConsumer(ImageConsumer ic) {
		return false;
	}

	@Override
	public void requestTopDownLeftRightResend(ImageConsumer ic) {
		startProduction(ic);
	}

	@Override
	public void startProduction(ImageConsumer ic) {
		updatePixelData();
		ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT
				| ImageConsumer.COMPLETESCANLINES | ImageConsumer.SINGLEFRAME);
		ic.setColorModel(fColorModel);
		ic.setDimensions(fWidth, fHeight);
		ic.setPixels(0, 0, fWidth, fHeight, fColorModel, fPixels, 0, fWidth);
		ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
	}

}
