/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
package de.tu_darmstadt.rs.sim.gui.McbSmall;

import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.engine.peripheral.McbInterfaceSmall;
import de.tu_darmstadt.rs.sim.gui.memory.Data18MemoryPanel;
import de.tu_darmstadt.rs.sim.gui.memory.MemoryPanel;
import de.tu_darmstadt.rs.sim.utilities.ConvenientButton;
import de.tu_darmstadt.rs.sim.utilities.Formatter;
import de.tu_darmstadt.rs.sim.utilities.MemoryAccess_I;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Panel showing the used memory of the MCB
 */
public class McbMemoryPanel extends JPanel implements ActionListener,
		SpartanDefinitions_I {

	private static final long serialVersionUID = -2111434546232810381L;

	private final ConvenientButton changeScope;
	private final JTextField fromTextField;
	private final JTextField toTextField;

	private final MemoryPanel memoryTable;
	private final McbWrapper wrapper;

	/**
	 * create a new {@link McbMemoryPanel}
	 *
	 * @param mcb
	 *            the MCB containing the data
	 */
	public McbMemoryPanel(final McbInterfaceSmall mcb) {
		setLayout(new BorderLayout());

		fromTextField = new JTextField(6);
		toTextField = new JTextField(6);

		// add controls
		final JPanel controlPanel = new JPanel();
		controlPanel.add(new JLabel("From"));
		controlPanel.add(fromTextField);
		controlPanel.add(new JLabel("To"));
		controlPanel.add(toTextField);
		changeScope = new ConvenientButton("Change scope",
				"change the displayed memory region", this, controlPanel);
		add(controlPanel, BorderLayout.NORTH);

		// add memory table
		wrapper = new McbWrapper(mcb);

		memoryTable = new MemoryPanel(wrapper, new Data18MemoryPanel());

		add(memoryTable, BorderLayout.CENTER);

		// init text fields

		fromTextField.setText(String.format("%05x", wrapper.from << 1));
		toTextField.setText(String.format("%05x", wrapper.to << 1));
	}

	/**
	 * refresh the shown data
	 *
	 * @param address
	 *            the address at which the data has changed
	 */
	public void refresh(final int address) {
		// register only refreshs are ignored
		memoryTable.refresh(address);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == changeScope) {

			final int offset = Formatter.parseHexString(
					fromTextField.getText(), BITMASK_18);
			final int toAddress = Formatter.parseHexString(
					toTextField.getText(), BITMASK_18);

			if ((offset == INVALID) || (toAddress == INVALID)) {
				MessageBox.showError("The entered value is not a number");
			}

			final int range = toAddress - offset;

			if (range <= 0) {
				MessageBox
						.showError("The specified memory range have to positive");
				return;
			}

			wrapper.from = (offset >> 1);
			wrapper.to = (toAddress >> 1);

			memoryTable.refresh(COMPLETE_REFRESH);
		}
	}

	// inner class
	private class McbWrapper implements MemoryAccess_I {

		private final McbInterfaceSmall mcb;
		public int from;
		public int to;

		public McbWrapper(final McbInterfaceSmall mcb) {
			this.mcb = mcb;
			from = 0;
			to = 0x80;
		}

		@Override
		public int getValue(final int address) {
			return mcb.getMemoryAt(address);
		}

		@Override
		public void setValue(final int address, final int value) {
			// setting values not supported
		}

		@Override
		public boolean hasWatchpoint(final int address) {
			// shared memory does not contain instructions
			return false;
		}

		@Override
		public void toggleWatchpoint(final int address) {
			// shared memory does not contain instructions -> do nothing
		}

		@Override
		public int getSize() {
			return to - from;
		}
	}
}
