/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */

package de.tu_darmstadt.rs.sim.gui;

import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.JConfigDocumentManager;
import de.tu_darmstadt.rs.sim.engine.MessageHandler_I;
import de.tu_darmstadt.rs.sim.engine.SettingListener_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationController_I;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.net.MpsocManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.OnLoadedListener_I;
import de.tu_darmstadt.rs.sim.gui.controls.SignalMenuItem;
import de.tu_darmstadt.rs.sim.gui.controls.SignalOptionsMenu;
import de.tu_darmstadt.rs.sim.gui.controls.UserAction;
import de.tu_darmstadt.rs.sim.gui.dialogs.AboutDialog;
import de.tu_darmstadt.rs.sim.gui.settings.SettingsDialog;
import de.tu_darmstadt.rs.sim.preferences.ProjectManager;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import libsim.SimulationSpeed_I;
import libsim.SimulationSpeed_I.Speed;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import javax.swing.Box;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

/**
 * The GUI's menu bar.
 */
@SuppressWarnings("serial")
public class MenuBar extends JMenuBar implements ActionListener,
		OnLoadedListener_I, SettingListener_I {

	private final JMenuItem fileExitItem;
	private final JMenuItem fileStoreItem;
	private final JMenuItem fileLoadItem;
	private final JMenuItem importJConfigItem;
	private final JMenuItem reloadJConfigItem;
	private final JMenuItem saveProjectItem;
	private final JMenuItem loadProjectItem;
	private final JMenuItem loadLstFileItem;

	private final JMenuItem controlSettingsItem;

	private final JMenu viewMenu;
	private final JMenu windowMenu;

	private final JMenuItem helpAboutItem;

	private final MainWindow mainWnd;
	private final SignalOptionsMenu<SimulationSpeed_I.Speed> speedMenu;

	private final SimulationManager simulationManager;
	private final SimulationController_I simCtrl;

	private final FileHelper fileHelper;
	private final JConfigMechanism jconfigMech;
	private final FileHelperMechanism_I projectMech;
	private final FileHelperMechanism_I lstMech;
	private final FileHelperMechanism_I sphMech;

	/**
	 * Constructor.
	 *
	 * @param mainWnd
	 *            the owning MainWindow
	 * @param manager
	 *            the SimulationManager
	 * @param sc
	 *            controller of the simulation state
	 * @param mh
	 *            handler for occurring messages
	 * @param documentManager
	 *            manager of the configuration document
	 * @param pm
	 *            project's component de.tu_darmstadt.rs.sim.preferences manager
	 * @param mp
	 *            the {@link MpsocManager}
	 */
	public MenuBar(final MainWindow mainWnd, final SimulationManager manager,
				   final SimulationController_I sc, final MessageHandler_I mh,
				   final JConfigDocumentManager documentManager,
				   final ProjectManager pm, final MpsocManager mp) {
		super();

		this.mainWnd = mainWnd;

		final JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');
		final JMenu controlMenu = new JMenu("Simulation");
		controlMenu.setMnemonic('C');
		viewMenu = new JMenu("Open View");
		viewMenu.setMnemonic('V');
		final JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic('H');

		importJConfigItem = new JMenuItem("Load JConfig Document");
		importJConfigItem.addActionListener(this);
		fileMenu.add(importJConfigItem);

		reloadJConfigItem = new JMenuItem("Reload JConfig Document");
		reloadJConfigItem.addActionListener(this);
		fileMenu.add(reloadJConfigItem);

		loadLstFileItem = new JMenuItem("Load Firmware Listing");
		loadLstFileItem.addActionListener(this);
		fileMenu.add(loadLstFileItem);

		fileLoadItem = new JMenuItem("Load Firmware Program");
		fileLoadItem.addActionListener(this);
		fileMenu.add(fileLoadItem);

		fileStoreItem = new JMenuItem("Store Firmware Program");
		fileStoreItem.addActionListener(this);
		fileMenu.add(fileStoreItem);

		fileMenu.addSeparator();

		loadProjectItem = new JMenuItem("Load Project Environment");
		loadProjectItem.addActionListener(this);
		fileMenu.add(loadProjectItem);

		saveProjectItem = new JMenuItem("Save Project Environment");
		saveProjectItem.addActionListener(this);
		fileMenu.add(saveProjectItem);

		fileMenu.addSeparator();

		fileExitItem = new JMenuItem("Exit");
		fileExitItem.addActionListener(this);
		fileMenu.add(fileExitItem);

		speedMenu = new SignalOptionsMenu<SimulationSpeed_I.Speed>("Speed",
				SimulationSpeed_I.Speed.values(), SimulationSpeed_I.Speed.class);
		controlMenu.add(speedMenu);

		controlMenu.addSeparator();

		final SignalMenuItem resetMenuItem = new SignalMenuItem(
				UserAction.RESET_SIMULATION);
		final SignalMenuItem runMenuItem = new SignalMenuItem(
				UserAction.RUN_SIMULATION);
		final SignalMenuItem stopMenuItem = new SignalMenuItem(
				UserAction.STOP_SIMULATION);
		final SignalMenuItem stepMenuItem = new SignalMenuItem(
				UserAction.STEP_SIMULATION);

		stopMenuItem.registerClickHandler(sc, "stopSimulation");
		runMenuItem.registerClickHandler(sc, "runSimulation");
		resetMenuItem.registerClickHandler(sc, "resetSimulation");
		stepMenuItem.registerClickHandler(sc, "stepSimulation");

		controlMenu.add(runMenuItem);
		controlMenu.add(stopMenuItem);
		controlMenu.add(stepMenuItem);
		controlMenu.add(resetMenuItem);

		controlMenu.addSeparator();

		final SignalOptionsMenu<MpsocManager.ConnState> connMenu = new SignalOptionsMenu<MpsocManager.ConnState>(
				"Connection", MpsocManager.ConnState.class);
		connMenu.addOption(MpsocManager.ConnState.OFFLINE);
		connMenu.addOption(MpsocManager.ConnState.ONLINE);
		connMenu.addOption(MpsocManager.ConnState.SYNCHRONIZED);

		connMenu.registerSelectHandler(mp, "setConnState",
				MpsocManager.ConnState.class);

		controlMenu.add(connMenu);

		controlMenu.addSeparator();

		controlSettingsItem = new JMenuItem("Settings");
		controlSettingsItem.addActionListener(this);
		controlMenu.add(controlSettingsItem);

		manager.getPeripheralManager().registerOnLoadedListener(this);
		manager.getDCMManager().registerOnLoadedListener(this);
		helpAboutItem = new JMenuItem("About");
		helpAboutItem.addActionListener(this);
		helpMenu.add(helpAboutItem);

		add(fileMenu);
		add(controlMenu);
		add(viewMenu);
		windowMenu = new JMenu("Window");
		windowMenu.setMnemonic('W');

		add(windowMenu);
		add(Box.createHorizontalGlue());
		add(helpMenu);
		Settings.registerListener(Settings.SIMULATION_SPEED, this);
		speedMenu.setSelectedOption((Speed) Settings.getSetting(Settings.SIMULATION_SPEED));
		speedMenu.registerSelectHandler(sc, "setSimulationSpeed", Speed.class);

		this.simulationManager = manager;
		final AddressSpaceManager addressSpace = simulationManager
				.getAddressSpaceManager();
		this.simCtrl = sc;

		fileHelper = new FileHelper(mainWnd, mh, sc);
		jconfigMech = new JConfigMechanism(mainWnd,
				simulationManager.getPeripheralManager(), documentManager);
		projectMech = new ProjectMechanism(mainWnd, pm);
		lstMech = new LstMechanism();
		sphMech = new SphMechanism(addressSpace);

	}

	/**
	 * update the window menu
	 *
	 * @param subWindowList
	 *            the list of the windows to appear in the menu
	 */
	public final void updateWindowMenu(final List<SubWindow> subWindowList) {
		windowMenu.removeAll();
		for (SubWindow sw : subWindowList) {
			SignalMenuItem it = new SignalMenuItem(sw.getTitle(), "");
			it.unregisterAllClickHandlers();
			it.registerClickHandler(sw, "select");
			windowMenu.add(it);
		}
	}

	@Override
	public void actionPerformed(final ActionEvent event) {
		boolean wasErrorShown = false;
		String cmd = event.getActionCommand();

		if (cmd.equals(fileExitItem.getText())) {
			onExit();
		} else if (event.getSource() == importJConfigItem) {
			fileHelper.showLoadDialog(jconfigMech);
		} else if (event.getSource() == reloadJConfigItem) {
			simCtrl.stopSimulation();
			jconfigMech.onReloadJConfig();
		} else if (event.getSource() == loadProjectItem) {
			fileHelper.showLoadDialog(projectMech);
		} else if (event.getSource() == saveProjectItem) {
			fileHelper.showSaveDialog(projectMech);
		} else if (cmd.equals(fileLoadItem.getText())) {
			// choose the wanted file:
			fileHelper.showLoadDialog(sphMech);
			checkJConfigFile();
		} else if (cmd.equals(fileStoreItem.getText())) {
			fileHelper.showSaveDialog(sphMech);
		} else if (cmd.equals(loadLstFileItem.getText())) {
			// choose the wanted file:
			fileHelper.showLoadDialog(lstMech);
			checkJConfigFile();
		} else if (cmd.equals(controlSettingsItem.getText())) {
			simCtrl.stopSimulation();

			final SettingsDialog dialog = new SettingsDialog(mainWnd);

			if (dialog.showModal() == SettingsDialog.OK_PRESSED) {
				simulationManager.setupSimulation();
			}
		} else if (cmd.equals(helpAboutItem.getText())) {
			final AboutDialog dialog = new AboutDialog(mainWnd);
			dialog.setVisible(true);
		} else {
			SubWindow subWindow = SubWindow.createSubWindow(cmd);
			if (subWindow != null) {
				mainWnd.attachNewSubWindow(subWindow);
			} else {
				// missing peripheral is only reason for a failed creation
				if (!wasErrorShown) {
					MessageBox.showError("The peripheral view for \"" + cmd
									+ "\" cannot be loaded ",
							"Creation of SubWindow failed");
				}
			}
		}
	}

	/**
	 * called when the window is closed
	 */
	public void onExit() {
		if (MessageBox.showConfirm("Are you sure that you want to quit?",
				"Quit")) {
			simCtrl.stopSimulation();

			// preferenceMgr.storeAllPreferences();
			System.exit(0);
		}
	}

	/**
	 * check if the jconfig file was loaded before
	 */
	private void checkJConfigFile() {
		final String path = (String) Settings.getSetting(Settings.JCONFIG_FILE);
		if (path == null || "".equals(path)) {
			MessageBox
					.showInfo(
							"No hardware configuration found. Please open a JConfig file.",
							"Jconfig File missing");
		}
	}

	private void addAsItems(List<String> menuItems) {
		for (String menuItem : menuItems) {
			if (menuItem.equals("-")) {
				viewMenu.addSeparator();
				continue;
			}
			final JMenuItem item = new JMenuItem(menuItem);
			item.addActionListener(this);
			viewMenu.add(item);
		}
	}

	@Override
	public void onLoaded(Object loadedObject) {
		viewMenu.removeAll();
		List<String> viewNames = SubWindow.getCreatableSubWindowsNames();
		addAsItems(viewNames);
	}

	@Override
	public void onSettingChange(final int settingType, final Object newValue) {
		speedMenu.setSelectedOption((Speed) newValue);
	}
}
