/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2012
 *
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2012
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2012
 */
package de.tu_darmstadt.rs.sim.gui;

import de.tu_darmstadt.rs.sim.engine.JConfigDocumentManager;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.preferences.ProjectComponent_I;

import java.util.prefs.Preferences;

/**
 * The PreferencesManager imports and exports the user's de.tu_darmstadt.rs.sim.preferences.
 */
public class PreferencesManager implements ProjectComponent_I {

	private final JConfigDocumentManager documentManager;
	private final MainWindow mainWnd;

	/**
	 * Constructor.
	 *
	 * @param mainWnd
	 *            the MainWindow
	 * @param documentManager
	 *            manager for the document data
	 */
	public PreferencesManager(final MainWindow mainWnd,
							  JConfigDocumentManager documentManager) {
		this.mainWnd = mainWnd;
		this.documentManager = documentManager;

		// display a splash screen when loading the jconfig environment
		new JConfigLibSplash();
	}

	@Override
	public void loadConfiguration(Preferences prefs) {
		int subSystemNumber = (Integer) Settings
				.getSetting(Settings.SUBSYSTEM_NUMBER);
		String fileName = (String) Settings.getSetting(Settings.JCONFIG_FILE);
		if ((!fileName.equals(""))
				&& subSystemNumber != (Integer) Settings
				.getSetting(Settings.SUBSYSTEM_INVALID)) {
			JConfigDocumentManager jconfigDocumentManager = documentManager;
			jconfigDocumentManager.setJConfigFile(fileName);

			// update MainWindow title
			final String subsystemName = jconfigDocumentManager
					.getSubsystemNames()[subSystemNumber];
			mainWnd.setTitleExtention(subsystemName);

			// load project
			jconfigDocumentManager.loadSubsystem(subSystemNumber);
		}

	}

	@Override
	public void storeConfiguration(Preferences prefs) {
		// do nothing
	}
}
