/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.preferences.ProjectManager;

import java.io.File;
import java.io.IOException;

/**
 * Mechanism which saves and loads a project file
 */
public class ProjectMechanism implements FileHelperMechanism_I {

	/**
	 * file extension for project files
	 */
	private final static String PROJECT_ENDING = ".spmc";

	private final MainWindow mainWnd;
	private final ProjectManager projectManager;

	/**
	 * create a new {@link ProjectMechanism}
	 *
	 * @param mainWnd window which must be updated when loaded
	 * @param pm reference to the {@link ProjectManager} for holding the
	 * project data
	 */
	public ProjectMechanism(final MainWindow mainWnd, final ProjectManager pm) {
		this.mainWnd = mainWnd;
		this.projectManager = pm;
	}

	@Override
	public String getFileSuffix() {
		return PROJECT_ENDING;
	}

	@Override
	public String getDefaultDirectory() {
		return ((File) Settings.getSetting(Settings.LAST_PROJECT_DIR))
				.getAbsolutePath();
	}

	@Override
	public void onApprovedLoad(final File directory, final File approvedFile)
			throws IOException {
		final String filepath = approvedFile.getAbsolutePath();

		mainWnd.disposeAllSubWindows();
		Settings.setSetting(Settings.LAST_PROJECT_FILE, filepath);
	}

	@Override
	public void onApprovedSave(final File directory, final File approvedFile)
			throws IOException {
		String filepath = approvedFile.getAbsolutePath();

		// add file extension if missing
		if (!filepath.endsWith(PROJECT_ENDING)) {
			filepath += PROJECT_ENDING;
		}

		projectManager.saveProject(filepath);
		Settings.setSetting(Settings.LAST_PROJECT_FILE, filepath);

		// remember project directory
		Settings.setSetting(Settings.LAST_PROJECT_DIR, directory);
	}
}
