/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui;

import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.parser.AssemblyParser;
import de.tu_darmstadt.rs.sim.engine.parser.ParsingException;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.io.File;

/**
 * Mechanism which saves and loads a SPH files
 */
public class SphMechanism implements FileHelperMechanism_I {

	private final static String SPH_ENDING = ".initData";
	private final AddressSpaceManager addressSpace;

	/**
	 * create a new {@link SphMechanism}
	 *
	 * @param addressSpace the address space to load data from when saving
	 */
	public SphMechanism(final AddressSpaceManager addressSpace) {
		this.addressSpace = addressSpace;
	}

	@Override
	public String getFileSuffix() {
		return SPH_ENDING;
	}

	@Override
	public String getDefaultDirectory() {
		return ((File) Settings.getSetting(Settings.LAST_PROGRAM_DIR))
				.getAbsolutePath();
	}

	@Override
	public void onApprovedLoad(final File directory, final File approvedFile) {
		Settings.setSetting(Settings.LAST_PROGRAM_DIR, directory);
		assert ((directory == null) || directory.isDirectory());
		Settings.setSetting(Settings.LAST_PROGRAM_FILE, approvedFile);
	}

	@Override
	public void onApprovedSave(final File directory, final File approvedFile) {
		try {
			final AssemblyParser parser = new AssemblyParser(addressSpace);
			String fileName = approvedFile.getPath();

			Settings.setSetting(Settings.LAST_PROGRAM_DIR, directory);
			assert ((directory == null) || directory.isDirectory());
			Settings.setSetting(Settings.LAST_PROGRAM_FILE, approvedFile);

			if (!fileName.endsWith(SPH_ENDING)) {
				fileName += SPH_ENDING;
			}
			parser.storeAssemblyCode(new File(fileName));

		} catch (ParsingException e) {
			MessageBox.showError(e.getMessage(), "Parsing error.");
		}
	}
}
