/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
  _______________________________________________________________________________

  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
  (http://www.mr.inf.tu-dresden.de) All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

  3. All advertising materials mentioning features or use of this software
  must display the following acknowledgement: "This product includes
  software developed by the TU Dresden Chair for Embedded Systems and
  its contributors."

  4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
  names of its contributors may be used to endorse or promote products
  derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  _______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui;

import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.engine.dcm.AbstractDCM;
import de.tu_darmstadt.rs.sim.engine.peripheral.JConfigPeripheral;
import de.tu_darmstadt.rs.sim.engine.peripheral.PeripheralManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.TearDownListener_I;
import de.tu_darmstadt.rs.sim.gui.McbSmall.McbGraphicView;
import de.tu_darmstadt.rs.sim.gui.McbSmall.McbSmallView;
import de.tu_darmstadt.rs.sim.gui.assembly.AssemblyView;
import de.tu_darmstadt.rs.sim.gui.assembly.variables.VariablesView;
import de.tu_darmstadt.rs.sim.gui.controls.SignalInternalFrame;
import de.tu_darmstadt.rs.sim.gui.dcm.DCMSignalActivationView;
import de.tu_darmstadt.rs.sim.gui.dcm.DCMView;
import de.tu_darmstadt.rs.sim.gui.generic.BaseAddressPane;
import de.tu_darmstadt.rs.sim.gui.generic.GenericPeripheralView;
import de.tu_darmstadt.rs.sim.gui.interrupt.InterruptView;
import de.tu_darmstadt.rs.sim.gui.memory.MemoryView;
import de.tu_darmstadt.rs.sim.gui.pipeline.PipelineView;
import de.tu_darmstadt.rs.sim.gui.registers.RegistersView;
import de.tu_darmstadt.rs.sim.gui.statistics.StatisticsView;
import de.tu_darmstadt.rs.sim.gui.timer.BasicTimerView;
import de.tu_darmstadt.rs.sim.gui.timer.RTIView;
import de.tu_darmstadt.rs.sim.gui.uart.UartView;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.awt.Rectangle;
import java.beans.PropertyVetoException;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.prefs.Preferences;

/**
 * Internal Window to accommodate Views.
 */
@SuppressWarnings("serial")
public class SubWindow extends SignalInternalFrame implements
		SpartanDefinitions_I, TearDownListener_I {

	private static final String PERIPHERALID_KEY = "peripheralId";

	private final AbstractView view;
	private final JConfigPeripheral peripheral;

	private boolean doMinimize;

	private static MainWindow mainWindow;
	private static SimulationManager simulationManager;
	private static final Map<String, List<Class<? extends AbstractView>>> peripheralToViewListMapping = new HashMap<String, List<Class<? extends AbstractView>>>();
	private static final Map<String, Class<? extends AbstractView>> guiViewTypeToViewMapping = new HashMap<String, Class<? extends AbstractView>>();
	private static final Map<String, String> viewTypeToViewMapping = new HashMap<String, String>();
	private static final HashSet<String> displayedPeripherals = new HashSet<String>();

	/**
	 * initialize the {@link SubWindow} helper class
	 *
	 * @param mainWindow
	 *            the parent of all {@link SubWindow} instances
	 * @param simulationManager
	 *            reference to the simulationManager
	 */
	public static void init(final MainWindow mainWindow,
							final SimulationManager simulationManager) {
		SubWindow.mainWindow = mainWindow;
		SubWindow.simulationManager = simulationManager;
	}

	static {
		viewTypeToViewMapping.put("MEMORY_VIEW", "Memory");
		viewTypeToViewMapping.put("ASSEMBLY_VIEW", "Assembly");
		viewTypeToViewMapping.put("REGISTERS_VIEW", "Registers");
		viewTypeToViewMapping.put("PIPELINE_VIEW", "Pipeline");
		viewTypeToViewMapping.put("STATISTICS_VIEW", "Statistics");
		viewTypeToViewMapping.put("DCM_SIGNAL_ACTIVATION_VIEW",
				"DCM Activation");
		viewTypeToViewMapping.put("ASSEMBLY_VARIABLES_VIEW", "Variables");

		List<Class<? extends AbstractView>> viewList = new ArrayList<Class<? extends AbstractView>>(
				1);
		viewList.add(InterruptView.class);
		peripheralToViewListMapping.put("intctrl", viewList);
		peripheralToViewListMapping.put("intctrlp", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(1);
		viewList.add(UartView.class);
		peripheralToViewListMapping.put("uart", viewList);
		peripheralToViewListMapping.put("uart_light", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(1);
		viewList.add(RTIView.class);
		peripheralToViewListMapping.put("rti", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(1);
		viewList.add(BasicTimerView.class);
		peripheralToViewListMapping.put("timer", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(2);
		viewList.add(McbSmallView.class);
		viewList.add(McbGraphicView.class);
		peripheralToViewListMapping.put("ddr_mem_port_32", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(0);
		peripheralToViewListMapping.put("fifo_in", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(0);
		peripheralToViewListMapping.put("fifo_out", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(0);
		peripheralToViewListMapping.put("port_in", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(0);
		peripheralToViewListMapping.put("port_out", viewList);
		viewList = new ArrayList<Class<? extends AbstractView>>(0);
		peripheralToViewListMapping.put("port_bi", viewList);
		// TODO: add DMA_VIEW and CONNECTOR_VIEW

		guiViewTypeToViewMapping.put("Memory", MemoryView.class);
		guiViewTypeToViewMapping.put("Assembly", AssemblyView.class);
		guiViewTypeToViewMapping.put("Pipeline", PipelineView.class);
		guiViewTypeToViewMapping.put("Registers", RegistersView.class);
		guiViewTypeToViewMapping.put("Statistics", StatisticsView.class);
		guiViewTypeToViewMapping.put("Variables", VariablesView.class);
		guiViewTypeToViewMapping.put("DCM Activation",
				DCMSignalActivationView.class);
	}

	/**
	 * return the names of all peripheral independent views
	 *
	 * @return list of names of the views
	 */
	private static List<String> getPeripheralIndependentViews() {
		List<String> returnValue = new ArrayList<String>();
		Set<String> guiViews = guiViewTypeToViewMapping.keySet();
		for (String view : guiViews) {
			returnValue.add(view);
		}
		return returnValue;
	}

	/**
	 * Constructor.
	 *
	 * @param view
	 *            the view itself
	 * @param peripheral
	 *            the peripheral represented by the new SubWindow, can be null
	 *            if SubWindow with no peripheral view is created.
	 */
	private SubWindow(final AbstractView view,
					  final JConfigPeripheral peripheral) {

		super(view.getName(), true, true, false, true);
		this.view = view;
		this.peripheral = peripheral;

		if (peripheral != null) {
			displayedPeripherals.add(peripheral.getPeriName());
			peripheral.registerTearDownListener(this);
		}
		add(view);
		setVisible(true);
		simulationManager.addRefreshListener(view);
		sigFrameClosed.registerMethodCallOnEmit(this, "frameClosed", SignalInternalFrame.class);
	}

	/**
	 * called when the frame is closed
	 * @param fr unused
	 */
	public void frameClosed(final SignalInternalFrame fr) {
		simulationManager.removeRefreshListener(view);
	}

	@Override
	public String getTitle() {
		if (view != null) {
			return view.getName();
		} else {
			return super.getTitle();
		}
	}

	/**
	 * Retrieves the view.
	 *
	 * @return the view
	 */
	public AbstractView getView() {
		return view;
	}

	/**
	 * Refreshes the subwindow's view completely. Will be called by the
	 * MainWindow.
	 */
	public void refreshCompletely() {
		view.refresh(COMPLETE_REFRESH);
	}

	private static AbstractView createView(
			final Class<? extends AbstractView> createrClass,
			final Class<?> parameterClasses[], Object... parameters) {
		AbstractView view = null;
		if (createrClass != null) {
			try {
				Constructor<? extends AbstractView> constructor = null;
				try {
					constructor = createrClass
							.getConstructor(parameterClasses);
				} catch (NoSuchMethodException e) {
					// don't care, try some other constructors!
				}
				if (constructor != null) {
					try {
						view = constructor.newInstance(parameters);
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} else {
					Constructor<?>[] constructorList = createrClass.getConstructors();
					outerLoop:
					for (Constructor<?> constructorIter : constructorList) {
						Class<?>[] parameterTypes = constructorIter.getParameterTypes();
						if (parameterTypes.length == parameterClasses.length) {
							for (int i = 0; i < parameterTypes.length; i++) {
								if (parameterTypes[i].isAssignableFrom(parameterClasses[i])) {
									if (i == parameterClasses.length - 1) {
										try {
											view = (AbstractView) constructorIter.newInstance(parameters);
											break outerLoop;
										} catch (Exception e) {
											// TODO Auto-generated catch block
											e.printStackTrace();
										}
									}
								}
							}
						}
					}
				}
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return view;
	}

	private static AbstractView getViewOf(final String viewType) {
		Class<? extends AbstractView> createrClass = guiViewTypeToViewMapping
				.get(viewType);
		AbstractView view = createView(createrClass, new Class[]{
						SimulationManager.class, MainWindow.class}, simulationManager,
				mainWindow);
		if (view == null) {
			view = createView(createrClass,
					new Class[]{SimulationManager.class}, simulationManager);
			if (view == null) {
				view = createView(createrClass, new Class[]{});
			}
		}
		return view;
	}

	/**
	 * Creates a SubWindow using de.tu_darmstadt.rs.sim.preferences. If the SubWindow displays a
	 * peripheral the creation can fail.
	 *
	 * @param prefs
	 *            the de.tu_darmstadt.rs.sim.preferences
	 * @return the SubWindow instance, or null if SubWindow can not be created.
	 */
	public static SubWindow createByPreferences(final Preferences prefs) {
		final String strType = prefs.get("ViewType", null);

		// if SubWindow contain a peripheral view, fetch the same peripheral, as
		// used before.
		PeripheralManager pMgr = simulationManager.getPeripheralManager();
		String peripheralId = prefs.get(PERIPHERALID_KEY,
				JConfigPeripheral.INVALID_ID);
		JConfigPeripheral restoredPeripheral = pMgr.getPeripheral(peripheralId);
		SubWindow wnd = null;
		// create the SubWindow
		if (restoredPeripheral != null) {
			wnd = createWindow(restoredPeripheral, prefs);
		} else {
			String viewName = viewTypeToViewMapping.get(strType);
			if (viewName == null) {
				MessageBox.showError("The ViewType \"" + strType
						+ "\" cannot be recreated");
			} else {
				wnd = createPeripheralIndependentWindow(viewName, prefs);
			}
		}

		if (wnd == null) {
			return null;
		}

		// initialize
		final int x = prefs.getInt("boundsX", wnd.getBounds().x);
		final int y = prefs.getInt("boundsY", wnd.getBounds().y);
		final int width = prefs.getInt("boundsWidth", wnd.getBounds().width);
		final int height = prefs.getInt("boundsHeight", wnd.getBounds().height);
		final boolean minimized = prefs.getBoolean("minimized", false);

		wnd.setBounds(x, y, width, height);
		wnd.doMinimize = minimized;

		return wnd;
	}

	/**
	 * Saves the SubWindow's configurations as de.tu_darmstadt.rs.sim.preferences.
	 *
	 * @param prefs
	 *            the de.tu_darmstadt.rs.sim.preferences map
	 */
	public void storeConfiguration(final Preferences prefs) {
		if (peripheral == null) {
			Set<String> viewTypes = viewTypeToViewMapping.keySet();
			String result;
			for (String viewType : viewTypes) {
				result = viewTypeToViewMapping.get(viewType);
				if (guiViewTypeToViewMapping.get(result)
						.equals(view.getClass())) {
					prefs.put("ViewType", viewType);
				}
			}
		}

		final Rectangle rect = getBounds();
		prefs.putInt("boundsX", rect.x);
		prefs.putInt("boundsY", rect.y);
		prefs.putInt("boundsWidth", rect.width);
		prefs.putInt("boundsHeight", rect.height);
		prefs.putBoolean("minimized", isIcon);

		if (peripheral != null) {
			prefs.put(PERIPHERALID_KEY, peripheral.getId());
		}

		view.storeConfiguration(prefs.node("view"));
	}

	/**
	 * Will be called if the MainWindow is set visible. Minimizes the SubWindow
	 * if necessary.
	 */
	public void onMainWndSetVisible() {
		try {
			setIcon(doMinimize);
		} catch (PropertyVetoException e) {/* Never mind... */
		}
	}

	@Override
	public void dispose() {

		/*
		 * Do we really need to pause simulation when disposing a view? If so,
		 * this feature must be re-implemented
		 */
		// boolean wasRunning = false;
		// if ( eventHandler.simulationState == SimulationState.RUNNING) {
		// eventHandler.onStop();
		// wasRunning = true;
		// }

		if (peripheral != null) {
			peripheral.removeTearDownListener(this);
		}

		view.onViewDispose();
		super.dispose();

		// if ((wasRunning) && (eventHandler.simulationState ==
		// SimulationState.DEFAULT)) {
		// // Note: there could have been thrown a SpartanException or
		// SimulationBreakException
		// // on the last execution step, if so do NOT re-start
		// eventHandler.onStart();
		// }
	}

	@Override
	public void performTearDownEvent(final JConfigPeripheral peripheral) {
		assert (this.peripheral == peripheral);
		displayedPeripherals.remove(peripheral.getPeriName());
		dispose();
	}

	@Override
	public String toString() {
		return getTitle();
	}

	/**
	 * called when the {@link SubWindow} gets selected
	 */
	public void select() {
		try {
			setSelected(true);
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
	}

	/**
	 * create a view for a peripheral
	 *
	 * @param peripheral
	 *            peripheral implementation
	 * @param prefs
	 *            the view's loaded settings
	 * @return a new {@link SubWindow}
	 */
	public static SubWindow createWindow(final JConfigPeripheral peripheral,
										 final Preferences prefs) {
		GenericPeripheralView view;
		List<AbstractView> subViewList;
		String peripheralType = peripheral.getPeriType();
		boolean wasPeripheralSpecificViewAdded = false;
		if (peripheralType != null) {
			view = new GenericPeripheralView(peripheral, simulationManager.getAddressSpaceManager());
			subViewList = getViewsOf(peripheralType, peripheral);
			for (int index = 0; index < subViewList.size(); index++) {
				view.addPeripheralSpecificView(subViewList.get(index));
				wasPeripheralSpecificViewAdded = true;
			}
			if (!wasPeripheralSpecificViewAdded) {
				view.addPeripheralSpecificView(new BaseAddressPane(peripheral));
			}
			if (prefs != null) {
				view.loadConfiguration(prefs.node("view"));
			}
			final SubWindow subWnd = new SubWindow(view, peripheral);
			subWnd.setSize(view.getPreferredSize());
			return subWnd;
		}
		return null;
	}

	/**
	 * create a view for a hardware component (peripheral, DCM)
	 *
	 * @param hardwareComponentName
	 *            name of the hardware component
	 * @param prefs
	 *            the view's loaded settings
	 * @return a new {@link SubWindow}
	 */
	private static SubWindow createWindow(final String hardwareComponentName,
										  final Preferences prefs) {
		final JConfigPeripheral peripheral = simulationManager.getPeripheralManager().getPeripheralByName(hardwareComponentName);
		if (peripheral != null) {
			return createWindow(peripheral, prefs);
		} else {
			final AbstractDCM dcm = simulationManager.getDCMManager().getDCMByName(hardwareComponentName);
			if (dcm != null) {
				return createWindow(dcm, prefs);
			}
		}
		return null;
	}

	private static SubWindow createWindow(final AbstractDCM dcm, final Preferences prefs) {
		AbstractView view = new DCMView(dcm);
		if (prefs != null) {
			view.loadConfiguration(prefs.node("view"));
		}
		final SubWindow subWindow = new SubWindow(view, null);
		subWindow.setSize(view.getPreferredSize());
		return subWindow;
	}

	/**
	 * create a sub window that can be shown
	 * @param name the name of the sub window
	 * @return the created sub window
	 */
	public static SubWindow createSubWindow(final String name) {
		SubWindow subWindow = null;
		for (final String view : getPeripheralIndependentViews()) {
			if (name.equals(view)) {
				subWindow = createPeripheralIndependentWindow(
						view, null);
			}
		}
		if (subWindow == null) {
			if (displayedPeripherals.contains(name)) {
				if (MessageBox.showConfirm("The peripheral view for \"" + name
								+ "\" already exists. This will open a second view.",
						"Creation of SubWindow")) {
					subWindow = createWindow(name, null);
				}
			} else {
				subWindow = createWindow(name, null);
			}
		}
		return subWindow;
	}

	/**
	 * return the names of all views that can be created for this simulation. This list contains a dash (-) whenever there begins a new sub list.
	 * @return list with all names
	 */
	public static List<String> getCreatableSubWindowsNames() {
		final List<String> viewNames = new ArrayList<String>();
		final List<String> peripheralIndependentViews = getPeripheralIndependentViews();
		Collections.sort(peripheralIndependentViews);
		for (final String view : peripheralIndependentViews) {
			viewNames.add(view);
		}
		viewNames.add("-");
		final List<String> sortedPeripheralNameList = simulationManager.getPeripheralManager().getSortedPeripheralNameList();
		viewNames.addAll(sortedPeripheralNameList);
		final List<String> sortedDCMNameList = simulationManager.getDCMManager().getSortedDCMNameList();
		if (!sortedDCMNameList.isEmpty()) {
			viewNames.add("-");
			viewNames.addAll(sortedDCMNameList);
		}
		return viewNames;
	}

	/**
	 * create a peripheral independent window
	 *
	 * @param name
	 *            name of the window
	 * @param prefs
	 *            the view's loaded settings
	 * @return a new {@link SubWindow}
	 */
	private static SubWindow createPeripheralIndependentWindow(final String name,
															   final Preferences prefs) {
		AbstractView view = getViewOf(name);
		if (prefs != null) {
			view.loadConfiguration(prefs.node("view"));
		}
		final SubWindow subWnd = new SubWindow(view, null);
		subWnd.setSize(view.getPreferredSize());
		return subWnd;
	}

	private static List<AbstractView> getViewsOf(final String peripheralType,
												 final JConfigPeripheral peripheral) {
		List<Class<? extends AbstractView>> createrClassList = peripheralToViewListMapping
				.get(peripheralType);
		List<AbstractView> subViewList = new ArrayList<AbstractView>(
				createrClassList.size());
		AbstractView subView;
		for (int i = 0; i < createrClassList.size(); i++) {
			subView = createView(createrClassList.get(i), new Class[]{
							peripheral.getClass(), MainWindow.class}, peripheral,
					mainWindow);
			if (subView != null) {
				subViewList.add(subView);
			}
			subView = createView(createrClassList.get(i), new Class[]{
					peripheral.getClass()}, peripheral);
			if (subView != null) {
				subViewList.add(subView);
			}
		}
		return subViewList;
	}
}
