/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui;

import de.tu_darmstadt.rs.sim.engine.SettingListener_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.gui.controls.SignalButton;
import de.tu_darmstadt.rs.sim.gui.controls.SignalComboBox;
import de.tu_darmstadt.rs.sim.gui.controls.UserAction;

import libsim.SimulationSpeed_I;

import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JToolBar;

/**
 * The GUI's toolbar.
 */
@SuppressWarnings("serial")
public class ToolBar extends JToolBar implements GuiDefinitions_I, SettingListener_I {

	private final SignalComboBox<SimulationSpeed_I.Speed> speedComboBox;

	/**
	 * Constructor.
	 * @param mainWnd the de.tu_darmstadt.rs.sim.main window which holds this tool bar
	 *
	 * @param simMgr
	 *            unused
	 * @param buttonClickHandler
	 *            object which contains methods to be invoked on button clicks
	 *            via reflection
	 * @param desktopPane pane containing the sub windows
	 */
	public ToolBar(final MainWindow mainWnd, final SimulationManager simMgr,
				   final Object buttonClickHandler, final JDesktopPane desktopPane) {

		super();

		final SignalButton resetButton = new SignalButton(
				UserAction.RESET_SIMULATION);
		final SignalButton runButton = new SignalButton(
				UserAction.RUN_SIMULATION);
		final SignalButton stopButton = new SignalButton(
				UserAction.STOP_SIMULATION);
		final SignalButton stepButton = new SignalButton(
				UserAction.STEP_SIMULATION);

		stopButton.registerClickHandler(buttonClickHandler, "stopSimulation");
		runButton.registerClickHandler(buttonClickHandler, "runSimulation");
		resetButton.registerClickHandler(buttonClickHandler, "resetSimulation");
		stepButton.registerClickHandler(buttonClickHandler, "stepSimulation");

		setFloatable(false);
		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

		add(runButton);
		add(stopButton);
		add(stepButton);
		add(Box.createHorizontalStrut(10));
		add(resetButton);
		addSeparator();

		speedComboBox = new SignalComboBox<SimulationSpeed_I.Speed>(
				SimulationSpeed_I.Speed.values(),
				SimulationSpeed_I.Speed.class, SPEED_COMBOBOX_HINT);
		speedComboBox.setFocusable(false);
		speedComboBox.setMaximumSize(speedComboBox.getPreferredSize());
		Settings.registerListener(Settings.SIMULATION_SPEED, this);

		// add(mainWnd.speedComboBox);
		// addSeparator();

		final CycleCounterView ccv = new CycleCounterView(simMgr);
		add(ccv);

		add(Box.createHorizontalGlue());
		//
		final SignalComboBox<JInternalFrame> windowComboBox = new SignalComboBox<JInternalFrame>(
				new SubWindowComboBoxModel(desktopPane), JInternalFrame.class,
				"Select active subwindow");
		windowComboBox.setFocusable(false);
		windowComboBox.setPreferredSize(new Dimension(300, 25));
		windowComboBox.setMaximumSize(windowComboBox.getPreferredSize());
		windowComboBox.registerSelectHandler(mainWnd, "selectSubWindow",
				JInternalFrame.class);

		// add(mainWnd.windowComboBox);
	}

	@Override
	public void onSettingChange(int settingType, Object newValue) {
		speedComboBox.setSelectedItem(newValue);
	}
}
