/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */

package de.tu_darmstadt.rs.sim.gui.assembly;

import de.tu_darmstadt.rs.sim.engine.SettingListener_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.assembly.AssemblyLine_I;
import de.tu_darmstadt.rs.sim.gui.GuiDefinitions_I;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * Cell renderer for the AssemblyView's table.
 */
@SuppressWarnings("serial")
public class AssemblyCellRenderer extends DefaultTableCellRenderer implements
		AssemblyViewDefinitions_I, GuiDefinitions_I, SettingListener_I {

	private final AssemblyView view;

	private Font normalFont;
	private Font boldFont;

	/**
	 * Constructor.
	 *
	 * @param view
	 *            the AssemblyView
	 */
	public AssemblyCellRenderer(final AssemblyView view) {
		super();
		this.view = view;
		normalFont = (Font) Settings.getSetting(Settings.REGISTER_FONT_PLAIN);
		boldFont = (Font) Settings.getSetting(Settings.REGISTER_FONT_BOLD);
		Settings.registerListener(Settings.REGISTER_FONT_PLAIN, this);
		Settings.registerListener(Settings.REGISTER_FONT_BOLD, this);
	}

	@Override
	public Component getTableCellRendererComponent(final JTable table,
												   final Object value, final boolean isSelected,
												   final boolean hasFocus, final int row, final int col) {

		JComponent label = null;
		JTextArea area = null;

		AssemblyLine_I assemblyLine = (AssemblyLine_I) value;
		switch (col) {

			case LINE_COL:
				final int dataAddress = assemblyLine.getDataAddress();
				String addressStr = "";
				if (dataAddress != INVALID) {
					if (view.isUsing9BitAddresses()) {
						addressStr += String.format("%x", dataAddress);
					} else {
						addressStr += String.format("%x", dataAddress >> 1);
					}
				}
				label = new JLabel(addressStr);
				label.setOpaque(true);
				if (isSelected) {
					label.setBackground(LINE_SELECTED_COLOR);
				} else {
					label.setBackground(LIGHT_GRAY_COLOR);
				}
				label.setForeground(Color.BLACK);
				label.setFont(normalFont);
				((JLabel) label).setHorizontalAlignment(JLabel.CENTER);
				label.setToolTipText("Click here to set a breakpoint");
				break;

			case BREAK_COL:
				String stopMarker = "";
				label = new JLabel();
				final int breakOrWatchValue = assemblyLine.getBreakOrWatchValue();
				if (breakOrWatchValue == BREAK_VAL) {
					stopMarker = "break";
					label.setForeground(BREAKPOINT_COLOR);
				} else if (breakOrWatchValue == WATCH_VAL) {
					stopMarker = "watch";
					label.setForeground(WATCHPOINT_COLOR);
				}
				((JLabel) label).setText(stopMarker);
				label.setOpaque(true);
				if (isSelected) {
					label.setBackground(LINE_SELECTED_COLOR);
				} else {
					label.setBackground(Color.WHITE);
				}
				label.setFont(new Font("Arial", Font.BOLD | Font.ITALIC, 12));
				((JLabel) label).setHorizontalAlignment(JLabel.CENTER);
				label.setToolTipText("Click here to set a breakpoint");
				break;

			case INSTR_COL:
				label = new JLabel(assemblyLine.getData());
				label.setOpaque(true);
				((JLabel) label).setHorizontalAlignment(JLabel.LEFT);
				if (isSelected) {
					label.setBackground(LINE_SELECTED_COLOR);
				} else {
					label.setBackground(Color.WHITE);
				}

				switch (assemblyLine.getStage()) {

					case IFID_STAGE:
						label.setForeground(IFID_COLOR);
						label.setFont(boldFont);
						break;
					case EXMEM_STAGE:
						label.setForeground(EXMEM_COLOR);
						label.setFont(boldFont);
						break;
					case WB_STAGE:
						label.setForeground(WB_COLOR);
						label.setFont(boldFont);
						break;
					default:
						label.setForeground(Color.BLACK);
						label.setFont(normalFont);
						break;
				}
				break;

			case SYMBOL_COL:
				area = new JTextArea(assemblyLine.getData());
				area.setOpaque(true);
				if (isSelected) {
					area.setBackground(LINE_SELECTED_COLOR);
				} else {
					area.setBackground(Color.WHITE);
				}

				area.setForeground(Color.BLACK);
				area.setFont(normalFont);
				int rowHeight = table.getRowHeight();
				table.setRowHeight(row, rowHeight * area.getLineCount());
				area.setAlignmentX(0);
				return area;
		}

		return label;
	}

	@Override
	public void onSettingChange(int settingType, Object newValue) {
		if (settingType == Settings.REGISTER_FONT_PLAIN) {
			normalFont = (Font) newValue;
		} else if (settingType == Settings.REGISTER_FONT_BOLD) {
			boldFont = (Font) newValue;
		}
		view.refresh(COMPLETE_REFRESH);
	}

}
