/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.assembly;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;

/**
 * Pop-up menu for altering the settings of the {@link AssemblyView}
 */
@SuppressWarnings("serial")
public class AssemblyPopupMenu extends JPopupMenu implements ActionListener {

	private AssemblyView view;
	private JCheckBoxMenuItem autoscrollItem;
	private JRadioButtonMenuItem nineBitItem;
	private JRadioButtonMenuItem eightteenBitItem;
	private List<JRadioButtonMenuItem> codeProviderButtons;

	/**
	 * create a new {@link AssemblyPopupMenu}
	 *
	 * @param view reference to the {@link AssemblyView} whose settings should be
	 * contained
	 */
	public AssemblyPopupMenu(AssemblyView view) {
		this.view = view;
		codeProviderButtons = new ArrayList<JRadioButtonMenuItem>();
		autoscrollItem = new JCheckBoxMenuItem("Autoscroll");
		autoscrollItem.addActionListener(this);
		nineBitItem = new JRadioButtonMenuItem("9 bit addresses");
		nineBitItem.addActionListener(this);
		eightteenBitItem = new JRadioButtonMenuItem("18 bit addresses");
		eightteenBitItem.addActionListener(this);
		add(autoscrollItem);
		addSeparator();
		add(nineBitItem);
		add(eightteenBitItem);
		addSeparator();
		updateView();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == autoscrollItem) {
			changeAutoScroll();
		} else if ((e.getSource() == nineBitItem)
				|| (e.getSource() == eightteenBitItem)) {
			changeNineBitAddresses();
		} else if (codeProviderButtons.contains(e.getSource())) {
			int index = codeProviderButtons.indexOf(e.getSource());
			changeCodeProvider(index);
		}
	}

	/**
	 * update the pop-up menu's shown data
	 */
	public void updateView() {
		List<String> codeProviderNames = view.getCodeProviderNames();
		JRadioButtonMenuItem newRadioItem;
		String selectedCodeProviderName;
		autoscrollItem.setState(view.isFollowingStages());
		if (view.isUsing9BitAddresses()) {
			nineBitItem.setSelected(true);
			eightteenBitItem.setSelected(false);
		} else {
			eightteenBitItem.setSelected(true);
			nineBitItem.setSelected(false);
		}
		for (JRadioButtonMenuItem radioItem : codeProviderButtons) {
			radioItem.removeActionListener(this);
			remove(radioItem);
		}
		codeProviderButtons.clear();
		selectedCodeProviderName = view.getSelectedCodeProvider();
		for (String s : codeProviderNames) {
			newRadioItem = new JRadioButtonMenuItem(s);
			newRadioItem.addActionListener(this);
			if (s.equals(selectedCodeProviderName)) {
				newRadioItem.setSelected(true);
			}
			codeProviderButtons.add(newRadioItem);
			add(newRadioItem);
		}
	}

	private void changeAutoScroll() {
		view.changeIsFollowingStages(!view.isFollowingStages());
		updateView();
	}

	private void changeNineBitAddresses() {
		view.changeUse9BitAddresses(!view.isUsing9BitAddresses());
		updateView();
	}

	private void changeCodeProvider(int index) {
		view.changeCodeProvider(index);
		updateView();
	}
}
