/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.assembly;

import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.SettingListener_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.assembly.AssemblyLineManager;
import de.tu_darmstadt.rs.sim.engine.memory.MainMemory;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineUI;
import de.tu_darmstadt.rs.sim.gui.AbstractView;
import de.tu_darmstadt.rs.sim.utilities.ConvenientButton;
import de.tu_darmstadt.rs.sim.utilities.Formatter;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;
import java.util.prefs.Preferences;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * Main class for displaying the assembler code.
 */
@SuppressWarnings("serial")
public class AssemblyView extends AbstractView implements
		AssemblyViewDefinitions_I, ActionListener, MouseListener,
		SettingListener_I {

	int startingAddress;
	int endingAddress;

	private final JTable table;
	private final JScrollPane scrollPane;
	private final DefaultTableColumnModel columnModel;
	private final AssemblyTableModel tableModel;

	final AddressSpaceManager memory;
	final PipelineUI pipeUI;

	private final JTextField startingAddressField;
	private final JTextField endingAddressField;
	private final ConvenientButton scopeButton;
	private final JTextField gotoAddressField;
	private final ConvenientButton gotoButton;
	private final ConvenientButton settingsButton;
	private boolean isFollowingStages = true;
	private boolean uses9BitAddresses;
	private final JPanel panel;
	private final AssemblyPopupMenu popup;

	private Font normalFont;

	private boolean requiresViewportUpdate;

	/**
	 * Constructor.
	 *
	 * @param simulationMgr
	 *            the SimulationManager
	 */
	public AssemblyView(final SimulationManager simulationMgr) {

		super();
		endingAddress = 0x19999;
		normalFont = (Font) Settings.getSetting(Settings.REGISTER_FONT_PLAIN);
		this.memory = simulationMgr.getAddressSpaceManager();
		this.pipeUI = simulationMgr.getPipelineUI();
		simulationMgr.registerCycleHandler(this, "onCycleUpdated", Long.class);

		setName("Assembly");
		setPreferredSize(new Dimension(300, 400));

		tableModel = new AssemblyTableModel(this);
		columnModel = new DefaultTableColumnModel();

		final TableColumn col0 = new TableColumn(LINE_COL);
		col0.setMinWidth(60);
		col0.setMaxWidth(60);
		columnModel.addColumn(col0);

		final TableColumn col1 = new TableColumn(BREAK_COL);
		col1.setMaxWidth(50);
		col1.setMinWidth(50);
		columnModel.addColumn(col1);

		final TableColumn col2 = new TableColumn(INSTR_COL);
		col2.setPreferredWidth(60);
		columnModel.addColumn(col2);

		table = new JTable(tableModel, columnModel);
		table.setDefaultRenderer(Object.class, new AssemblyCellRenderer(this));
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
		table.setShowGrid(false);
		table.setIntercellSpacing(new Dimension(0, 0));
		table.addMouseListener(this);

		scrollPane = new JScrollPane(table);
		scrollPane.getViewport().setBackground(Color.WHITE);

		panel = new JPanel();
		panel.setLayout(new FlowLayout());
		panel.setSize(new Dimension(this.getWidth(), 20));

		settingsButton = new ConvenientButton("Settings",
				"Change various settings", this, panel);

		gotoAddressField = new JTextField(7);
		gotoAddressField.setFont(normalFont);
		panel.add(gotoAddressField);
		gotoButton = new ConvenientButton("Goto address",
				"Change the position", this, panel);

		startingAddressField = new JTextField(7);
		startingAddressField.setFont(normalFont);
		panel.add(startingAddressField);
		startingAddressField.setText(String.format("%x", startingAddress));

		panel.add(new JLabel("to"));

		endingAddressField = new JTextField(7);
		endingAddressField.setFont(normalFont);
		panel.add(endingAddressField);
		endingAddressField.setText(String.format("%x", endingAddress));

		scopeButton = new ConvenientButton("Change scope",
				"Changes the view to the defined limits", this, panel);

		setLayout(new BorderLayout());
		add(panel, BorderLayout.NORTH);
		add(scrollPane, BorderLayout.CENTER);

		memory.addRefreshListener(this);
		pipeUI.addRefreshListener(this);

		uses9BitAddresses = true;
		popup = new AssemblyPopupMenu(this);
		Settings.registerListener(Settings.REGISTER_FONT_PLAIN, this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(final ActionEvent event) {

		if (event.getActionCommand().equals(scopeButton.getText())) {

			final int start = Formatter.parseHexString(
					startingAddressField.getText(), BITMASK_18);
			final int end = Formatter.parseHexString(
					endingAddressField.getText(), BITMASK_18);

			if (!MainMemory.containsAddress(start)) {
				MessageBox.showError("Starting address is not valid.");
				return;
			}

			if (!MainMemory.containsAddress(end)) {
				MessageBox.showError("Ending address is not valid.");
				return;
			}

			if (start >= end) {
				MessageBox
						.showError("A starting address should not be larger than the ending one...");
				return;
			}

			startingAddress = start;
			endingAddress = end;

			startingAddressField
					.setText(String.format("%05x", startingAddress));
			endingAddressField.setText(String.format("%05x", endingAddress));

			// only refresh this de.tu_darmstadt.rs.sim.gui:
			tableModel.fireTableDataChanged();
		} else if (event.getSource() == gotoButton) {
			scrollToAddress(Formatter.parseHexString(
					gotoAddressField.getText(), BITMASK_18), true);
		} else if (event.getSource() == settingsButton) {
			popup.updateView();
			popup.show(panel, settingsButton.getX(), settingsButton.getY()
					+ settingsButton.getHeight());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseClicked(final MouseEvent event) {

		// breakpoint and watchpoint sensitivity only in column 0 and 1
		if (table.columnAtPoint(event.getPoint()) <= 1) {
			final int row = table.rowAtPoint(event.getPoint());
			AssemblyLineManager.getTheInstance().toggle(row);
			refresh(COMPLETE_REFRESH);
		}
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// never used
	}

	@Override
	public void mouseReleased(MouseEvent mE) {
		// never used
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// never used
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// never used
	}

	@Override
	public void storeConfiguration(final Preferences prefs) {

		assert (prefs != null);
		prefs.putInt("startingAddress", startingAddress);
		prefs.putInt("endingAddress", endingAddress);
		prefs.putBoolean("autoscroll", isFollowingStages);
		prefs.putBoolean("uses9BitAddresses", uses9BitAddresses);
		AssemblyLineManager manager = AssemblyLineManager.getTheInstance();
		prefs.put("code provider", manager.getSelectedAssemblyLineProvider());
	}

	@Override
	public void loadConfiguration(final Preferences prefs) {

		if (prefs != null) {
			final int start = prefs.getInt("startingAddress",
					(Integer) Settings.getSetting(Settings.MEMORY_ADDRESS_MIN));
			final int end = prefs.getInt("endingAddress",
					(Integer) Settings.getSetting(Settings.MEMORY_ADDRESS_MAX));

			startingAddress = (MainMemory.containsAddress(start))? start
					: (Integer) Settings
					.getSetting(Settings.MEMORY_ADDRESS_MIN);
			endingAddress = (MainMemory.containsAddress(end))? end
					: (Integer) Settings
					.getSetting(Settings.MEMORY_ADDRESS_MAX);
			isFollowingStages = prefs.getBoolean("autoscroll", false);
			uses9BitAddresses = prefs.getBoolean("uses9BitAddresses", true);
			String providerName = prefs.get("code provider", null);
			if (providerName != null) {
				AssemblyLineManager manager = AssemblyLineManager
						.getTheInstance();
				List<String> names = manager.getAssemblyLineProviderNames();
				for (int i = 0; i < names.size(); i++) {
					if (names.get(i).equals(providerName)) {
						manager.selectAssemblyLineProvider(i);
						break;
					}
				}
			}

		} else {
			// brand new view:
			startingAddress = (Integer) Settings
					.getSetting(Settings.MEMORY_ADDRESS_MIN);
			endingAddress = (Integer) Settings
					.getSetting(Settings.MEMORY_ADDRESS_MAX);
		}

		startingAddressField.setText(String.format("%05x", startingAddress));
		endingAddressField.setText(String.format("%05x", endingAddress));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.RefreshListener_I#refresh(int)
	 */
	@Override
	public void refresh(final int address) {
		tableModel.fireTableDataChanged();
		table.repaint();

		if (requiresViewportUpdate && isFollowingStages) {
			scrollToAddress(
					pipeUI.getRegister(VisiblePipelineRegister.IFID_PC), false);
			requiresViewportUpdate = false;
		}
	}

	@Override
	public void onViewDispose() {
		memory.removeRefreshListener(this);
		pipeUI.removeRefreshListener(this);
		tableModel.onViewDispose();
	}

	/**
	 * scroll the view so the address is shown in the view
	 *
	 * @param address
	 *            the address to be visible
	 * @param toTop
	 *            true if the address should be should as the first line in the
	 *            view, false otherwise
	 */
	public void scrollToAddress(int address, boolean toTop) {
		if (address < 0) {
			return;
		}
		JViewport viewport = scrollPane.getViewport();
		int rowIndex = findRowIndexForScrolling(address);
		if (rowIndex < 0) {
			return;
		}
		Rectangle rectangle = table.getCellRect(rowIndex, LINE_COL, true);
		Rectangle viewportRectangle = viewport.getViewRect();
		int newHeight = rectangle.y - viewportRectangle.y;
		if (toTop && newHeight > 0) {
			newHeight += (viewportRectangle.height - rectangle.height);
		}
		rectangle.y = newHeight;
		viewport.scrollRectToVisible(rectangle);
	}

	/**
	 * find the row index for the address
	 *
	 * @param address
	 *            the address whose row index should be found
	 * @return the row index of the address
	 */
	public int findRowIndexForScrolling(int address) {
		return AssemblyLineManager.getTheInstance().findRowIndexOfAddress(
				address);
	}

	/**
	 * set whether the assembly view should scroll to the address which is
	 * currently in the program counter
	 *
	 * @param newIsFollowingValue
	 *            true if the view should follow the program counter address,
	 *            false otherwise
	 */
	public void changeIsFollowingStages(boolean newIsFollowingValue) {
		isFollowingStages = newIsFollowingValue;
	}

	/**
	 * check whether the view is following the pipeline stages
	 *
	 * @return true if the view follows the stages, false otherwise
	 */
	public boolean isFollowingStages() {
		return isFollowingStages;
	}

	/**
	 * change the view to use 9 bit addresses (data addressing mode)
	 *
	 * @param newValue
	 *            true if the view should use data addresses instead default
	 *            addresses
	 */
	public void changeUse9BitAddresses(boolean newValue) {
		uses9BitAddresses = newValue;
		tableModel.fireTableDataChanged();
	}

	/**
	 * check whether the view is using 9 bit addresses (data addressing)
	 *
	 * @return true if the view uses data addresses, false otherwise
	 */
	public boolean isUsing9BitAddresses() {
		return uses9BitAddresses;
	}

	/**
	 * select a code provider
	 *
	 * @param index
	 *            index of the code provider in the list of all code providers
	 */
	public void changeCodeProvider(int index) {
		AssemblyLineManager manager = AssemblyLineManager.getTheInstance();
		manager.selectAssemblyLineProvider(index);
		tableModel.fireTableDataChanged();
		if (isFollowingStages) {
			scrollToAddress(
					pipeUI.getRegister(VisiblePipelineRegister.IFID_PC), false);
		}
	}

	/**
	 * retrieve the name of the current code provider
	 *
	 * @return name of the code provider
	 */
	public String getSelectedCodeProvider() {
		AssemblyLineManager manager = AssemblyLineManager.getTheInstance();
		return manager.getSelectedAssemblyLineProvider();
	}

	/**
	 * retrieve all available code providers
	 *
	 * @return list with names of all code providers
	 */
	public List<String> getCodeProviderNames() {
		AssemblyLineManager manager = AssemblyLineManager.getTheInstance();
		return manager.getAssemblyLineProviderNames();
	}

	@Override
	public void onSettingChange(int settingType, Object newValue) {
		if (settingType == Settings.REGISTER_FONT_PLAIN) {
			normalFont = (Font) newValue;
		}
		refresh(COMPLETE_REFRESH);
	}

	/**
	 * called whenever a step was executed
	 * callback method
	 * @param cycle number of the currently executed cycle
	 */
	public void onCycleUpdated(final Long cycle) {
		requiresViewportUpdate = true;
	}
}
