/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.assembly.variables;

import de.tu_darmstadt.rs.sim.engine.assembly.AssemblyVariable;
import de.tu_darmstadt.rs.sim.engine.assembly.AssemblyVariablesManager;

import javax.swing.table.AbstractTableModel;

/**
 * Table de.tu_darmstadt.rs.sim.model containing the variables
 */
@SuppressWarnings("serial")
public class VariablesTableModel extends AbstractTableModel implements
		VariableViewDefinitions_I {

	private AssemblyVariablesManager manager;

	/**
	 * create a new {@link VariablesTableModel}
	 *
	 * @param manager reference to the {@link AssemblyVariablesManager} which holds
	 * all variables
	 */
	public VariablesTableModel(AssemblyVariablesManager manager) {
		this.manager = manager;
	}

	@Override
	public int getRowCount() {
		return manager.getNumberOfVariables();
	}

	@Override
	public int getColumnCount() {
		return 5;
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return columnIndex == VALUE_CHOOSE_COLUMN;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		AssemblyVariable variable = manager.getVariableAtIndex(rowIndex);
		switch (columnIndex) {
			case ADRESS_COLUMN:
				return "" + String.format("0x%05x", variable.getDataAddress());
			case NAME_COLUMN:
				return "" + variable.getName();
			case WIDTH_COLUMN:
				return "" + variable.getLengthIn9Bytes();
			case VALUE_COLUMN:
				return "" + manager.getInterpretedValueOf(variable);
			case VALUE_CHOOSE_COLUMN:
				return "" + manager.getInterpretationOfVariable(variable);
		}
		return null;
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		manager.setSelectedInterpretation(rowIndex, (String) aValue);
		fireTableDataChanged();
	}

}
