/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

package de.tu_darmstadt.rs.sim.gui.controls;

import de.tu_darmstadt.rs.sim.signal.Signal0;
import de.tu_darmstadt.rs.sim.signal.Signal1;

import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;

/**
 * Combo box which emits signals when an item is selected/deselected
 *
 * @param <T> the type of the contained elements
 */
@SuppressWarnings("serial")
public class SignalComboBox<T> extends JComboBox<T> implements ItemListener {

	private final Signal1<T> sigItemSelected;
	private final Signal1<T> sigItemDeselected;
	private final Signal0 sigEdited;
	private final Class<T> c;

	/**
	 * create a new {@link SignalComboBox}
	 *
	 * @param c class for the contained data
	 * @param hint tool tip text to be shown
	 */
	public SignalComboBox(Class<T> c, String hint) {
		super();
		this.c = c;
		setToolTipText(hint);
		sigEdited = new Signal0();
		sigItemSelected = new Signal1<T>();
		sigItemDeselected = new Signal1<T>();
		addActionListener(this);
		addItemListener(this);
	}

	/**
	 * create a new {@link SignalComboBox}
	 *
	 * @param model the combo box's data de.tu_darmstadt.rs.sim.model
	 * @param c class for the contained data
	 * @param hint tool tip text to be shown
	 */
	public SignalComboBox(ComboBoxModel<T> model, Class<T> c, String hint) {
		this(c, hint);
		setModel(model);
	}

	/**
	 * create a new {@link SignalComboBox}
	 *
	 * @param items the combo box's data
	 * @param c class for the contained data
	 * @param hint tool tip text to be shown
	 */
	public SignalComboBox(T[] items, Class<T> c, String hint) {
		this(c, hint);
		setModel(new DefaultComboBoxModel<T>(items));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		sigEdited.emit();
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		Object obj = e.getItem();
		T typedItem;
		if (c.isInstance(obj)) {
			typedItem = (c.cast(obj));
			switch (e.getStateChange()) {
				case ItemEvent.SELECTED:
					sigItemSelected.emit(typedItem);
					break;
				case ItemEvent.DESELECTED:
					sigItemDeselected.emit(typedItem);
					break;
			}
		}
	}

	/**
	 * register a handler for the selection
	 *
	 * @param o the object having the method
	 * @param methodName the name of the method to be invoked
	 * @param argClass the type of the parameter of the method
	 */
	public void registerSelectHandler(final Object o, final String methodName,
									  final Class<T> argClass) {
		sigItemSelected.registerMethodCallOnEmit(o, methodName, argClass);
	}

	/**
	 * unregister a previously registered handler for the selection
	 *
	 * @param o the object not to be invoked any more
	 */
	public void unregisterSelectHandler(final Object o) {
		sigItemSelected.unregisterMethodCallOnEmit(o);
	}

	/**
	 * register a handler for the deselection
	 *
	 * @param o the object having the method
	 * @param methodName the name of the method to be invoked
	 * @param selObjectClass the parameter of the method
	 */
	public void registerDeselectHandler(final Object o,
										final String methodName, final Class<T> selObjectClass) {
		sigItemDeselected.registerMethodCallOnEmit(o, methodName,
				selObjectClass);
	}

	/**
	 * unregister a previously registered handler for the deselection
	 *
	 * @param o the object not to be invoked any more
	 */
	public void unregisterDeselectHandler(final Object o) {
		sigItemDeselected.unregisterMethodCallOnEmit(o);
	}
}
