/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

package de.tu_darmstadt.rs.sim.gui.controls;

import de.tu_darmstadt.rs.sim.signal.Signal1;

import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

/**
 * Internal frame which fires a de.tu_darmstadt.rs.sim.signal when closed
 */
@SuppressWarnings("serial")
public class SignalInternalFrame extends JInternalFrame implements
		InternalFrameListener {

	/**
	 * de.tu_darmstadt.rs.sim.signal to be fired when the frame closes
	 */
	protected final Signal1<SignalInternalFrame> sigFrameClosed;

	/**
	 * create a new {@link SignalInternalFrame}
	 *
	 * @param nm title of the frame
	 * @param rs true if the frame should be resizable, false otherwise
	 * @param cl true if the frame should be closable, false otherwise
	 * @param mx true if the frame should be maximizable, false otherwise
	 * @param ic true if the frame should be iconifiable, false otherwise
	 */
	public SignalInternalFrame(String nm, boolean rs, boolean cl, boolean mx,
							   boolean ic) {
		super(nm, rs, cl, mx, ic);
		sigFrameClosed = new Signal1<SignalInternalFrame>();
		addInternalFrameListener(this);
	}

	@Override
	public void internalFrameClosed(final InternalFrameEvent e) {
		sigFrameClosed.emit(this);
	}

	/**
	 * register a handler for the close operation
	 *
	 * @param o the object having the method
	 * @param methodName the name of the method to be invoked
	 * @param frameClass the parameter of the method
	 */
	public void registerCloseHandler(Object o, String methodName,
									 Class<SignalInternalFrame> frameClass) {
		sigFrameClosed.registerMethodCallOnEmit(o, methodName, frameClass);
	}

	/**
	 * unregister a previously registered handler for the close operation
	 *
	 * @param o the method call object
	 */
	public void unregisterCloseHandler(Object o) {
		sigFrameClosed.unregisterMethodCallOnEmit(o);
	}

	@Override
	public void internalFrameActivated(final InternalFrameEvent e) {
		//never used
	}

	@Override
	public void internalFrameClosing(final InternalFrameEvent e) {
		//never used
	}

	@Override
	public void internalFrameDeactivated(final InternalFrameEvent e) {
		//never used
	}

	@Override
	public void internalFrameDeiconified(final InternalFrameEvent e) {
		//never used
	}

	@Override
	public void internalFrameIconified(final InternalFrameEvent e) {
		//never used
	}

	@Override
	public void internalFrameOpened(final InternalFrameEvent e) {
		//never used
	}
}
