/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

package de.tu_darmstadt.rs.sim.gui.controls;

import de.tu_darmstadt.rs.sim.gui.Action_I;
import de.tu_darmstadt.rs.sim.signal.Signal0;
import de.tu_darmstadt.rs.sim.signal.Signal1;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JRadioButtonMenuItem;

/**
 * Menu item which fires a clicked and a select de.tu_darmstadt.rs.sim.signal when clicked
 *
 * @param <T> data type of internal data send when a selection de.tu_darmstadt.rs.sim.signal occurs
 */
@SuppressWarnings("serial")
public class SignalOptionMenuItem<T> extends JRadioButtonMenuItem implements ActionListener {

	private final Signal0 sigClicked;
	private final Signal1<T> sigSelected;
	private final T fOption;

	/**
	 * create a new {@link SignalOptionMenuItem}
	 *
	 * @param text the text of this item
	 * @param hint the tool tip text
	 * @param opt additional data to be send to selection handlers
	 */
	public SignalOptionMenuItem(String text, String hint, T opt) {
		super(text);
		fOption = opt;
		setToolTipText(hint);
		sigClicked = new Signal0();
		sigSelected = new Signal1<T>();
		addActionListener(this);
	}

	/**
	 * create a new {@link SignalOptionMenuItem}
	 *
	 * @param opt additional data to be send to selection handlers, used also for item text
	 */
	public SignalOptionMenuItem(T opt) {
		this(opt.toString(), "", opt);
	}

	/**
	 * create a new {@link SignalOptionMenuItem}
	 *
	 * @param a data provider for the item text and tool tip text
	 * @param opt additional data to be send to selection handlers
	 */
	public SignalOptionMenuItem(Action_I a, T opt) {
		this(a.getName(), a.getHint(), opt);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		sigClicked.emit();
		sigSelected.emit(fOption);
	}

	/**
	 * register a handler for the selection
	 *
	 * @param o the object having the method
	 * @param methodName the name of the method to be invoked
	 * @param argClass the type of the parameter of the method
	 */
	public void registerSelectHandler(final Object o, final String methodName,
									  final Class<T> argClass) {
		sigSelected.registerMethodCallOnEmit(o, methodName, argClass);
	}

	/**
	 * unregister a previously registered handler for the selection
	 *
	 * @param o the object not to be invoked any more
	 */
	public void unregisterSelectHandler(final Object o) {
		sigSelected.unregisterMethodCallOnEmit(o);
	}

	/**
	 * register a handler for the click
	 *
	 * @param o the object having the method
	 * @param methodName the name of the method to be invoked
	 */
	public void registerClickHandler(final Object o, final String methodName) {
		sigClicked.registerMethodCallOnEmit(o, methodName);
	}

	/**
	 * unregister a previously registered handler for the click
	 *
	 * @param o the object not to be invoked any more
	 */
	public void unregisterClickHandler(final Object o) {
		sigClicked.unregisterMethodCallOnEmit(o);
	}
}

