/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

package de.tu_darmstadt.rs.sim.gui.controls;

import de.tu_darmstadt.rs.sim.signal.Signal1;

import java.util.HashMap;
import java.util.Map;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JMenu;

/**
 * Menu which fires a select de.tu_darmstadt.rs.sim.signal when an item is selected
 *
 * @param <T> data type of internal data send when a selection de.tu_darmstadt.rs.sim.signal occurs
 */
@SuppressWarnings("serial")
public class SignalOptionsMenu<T> extends JMenu {

	private final Class<T> clazz;
	private final Signal1<T> sigSelected;
	private final ButtonGroup fBtnGroup;
	private final Map<T, ButtonModel> fItemForOption;

	/**
	 * create a new {@link SignalOptionMenuItem}
	 *
	 * @param label the label of this item
	 * @param clazz the class of the options
	 */
	public SignalOptionsMenu(String label, Class<T> clazz) {
		super(label);
		this.clazz = clazz;
		sigSelected = new Signal1<T>();
		fBtnGroup = new ButtonGroup();
		fItemForOption = new HashMap<T, ButtonModel>();
	}

	/**
	 * create a new {@link SignalOptionMenuItem}
	 *
	 * @param label the label of this item
	 * @param options the elements of this menu
	 * @param clazz the class of the options
	 */
	public SignalOptionsMenu(String label, T[] options, Class<T> clazz) {
		this(label, clazz);
		for (T o : options) {
			addOption(o);
		}
	}

	/**
	 * called whenever a menu item is selected
	 *
	 * @param opt the selected data
	 */
	public void selected(T opt) {
		sigSelected.emit(opt);
	}

	/**
	 * add an item to the menu
	 *
	 * @param opt the data to be added
	 */
	public void addOption(T opt) {
		SignalOptionMenuItem<T> it = new SignalOptionMenuItem<T>(opt);
		fBtnGroup.add(it);
		it.registerSelectHandler(this, "selected", clazz);
		add(it);
		fItemForOption.put(opt, it.getModel());
	}

	/**
	 * set the selected item
	 *
	 * @param opt the selected data
	 */
	public void setSelectedOption(T opt) {
		ButtonModel bm = fItemForOption.get(opt);
		if (bm == null) {
			return;
		}
		bm.setSelected(true);
	}

	/**
	 * register a handler for the selection
	 *
	 * @param o the object having the method
	 * @param methodName the name of the method to be invoked
	 * @param argClass the type of the parameter of the method
	 */
	public void registerSelectHandler(final Object o, final String methodName,
									  final Class<T> argClass) {
		sigSelected.registerMethodCallOnEmit(o, methodName, argClass);
	}

	/**
	 * unregister a previously registered handler for the selection
	 *
	 * @param o the object not to be invoked any more
	 */
	public void unregisterSelectHandler(final Object o) {
		sigSelected.unregisterMethodCallOnEmit(o);
	}
}
