/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.dcm;

import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.dcm.DCMSignalInvocationManager;
import de.tu_darmstadt.rs.sim.gui.AbstractView;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.List;
import java.util.prefs.Preferences;
import javax.swing.JTextArea;

/**
 * GUI representation of the {@link DCMSignalInvocationManager}'s list
 */
@SuppressWarnings("serial")
public class DCMSignalActivationView extends AbstractView {

	private DCMSignalInvocationManager dcmSigInvMgr;
	private JTextArea signalArea;
	private final GridBagLayout layout;

	private static final int MIN_WIDTH = 256;
	private static final int MIN_HEIGHT = 128;

	/**
	 * create a new view
	 *
	 * @param simulationManager reference to the {@link SimulationManager}
	 */
	public DCMSignalActivationView(final SimulationManager simulationManager) {
		this.dcmSigInvMgr = simulationManager.getDCMSignalInvocationManager();
		layout = new GridBagLayout();
		setLayout(layout);
		setName("DCM Signal Activation Order");
		signalArea = new JTextArea(30, 10);
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.fill = GridBagConstraints.BOTH;
		layout.setConstraints(signalArea, constraints);
		add(signalArea);
		setPreferredSize(new Dimension(MIN_WIDTH, MIN_HEIGHT));
		refresh(0);
		dcmSigInvMgr.addRefreshListener(this);
	}

	@Override
	public void loadConfiguration(Preferences prefs) {
		//nothing to do here
	}

	@Override
	public void onViewDispose() {
		//nothing to do here
	}

	@Override
	public void storeConfiguration(Preferences prefs) {
		//nothing to do here
	}

	@Override
	public void refresh(int address) {
		List<String> signalInvocationList = dcmSigInvMgr
				.getOrderOfInvokedSignals();
		String stringOfAllSignalInvocations = "";
		for (String sigInvocationName : signalInvocationList) {
			stringOfAllSignalInvocations += sigInvocationName + "\n";
		}
		signalArea.setText(stringOfAllSignalInvocations);
	}
}
