/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.dcm;

import de.tu_darmstadt.rs.sim.engine.dcm.AbstractDCM;
import de.tu_darmstadt.rs.sim.engine.dcm.AbstractDCMSignal;
import de.tu_darmstadt.rs.sim.gui.AbstractView;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.prefs.Preferences;
import javax.swing.JLabel;

/**
 * GUI representation of the DCM
 */
@SuppressWarnings("serial")
public class DCMView extends AbstractView {

	/**
	 * default name for the input clock de.tu_darmstadt.rs.sim.signal
	 */
	public static final String CLOCK_GENERATOR = "Clock Generator";

	private static final int MIN_WIDTH = 256;
	private static final int MIN_HEIGHT = 128;

	private String clkinDivideBy2Value;
	private String clkdvDivideValue;
	private String clkfxMultiplyValue;
	private String clkfxDivideValue;
	private String input;
	private JLabel inputLabel;
	private JLabel clkinDivideBy2Label;
	private JLabel clkdvDivideLabel;
	private JLabel clkfxMultiplyLabel;
	private JLabel clkfxDivideLabel;

	/**
	 * create a new view
	 *
	 * @param dcm the {@link AbstractDCM} instance
	 */
	public DCMView(final AbstractDCM dcm) {
		setName(dcm.getDCMName());
		clkinDivideBy2Value = "" + dcm.isClkInDivideBy2();
		//clkdvDivideValue = dcm.getClkDvDivide().toString();
		clkfxMultiplyValue = dcm.getClkFxMultiply().toString();
		clkfxDivideValue = dcm.getClkFxDivide().toString();
		AbstractDCMSignal inputSignal = dcm.getInputSignal();
		if (inputSignal == null) {
			input = CLOCK_GENERATOR;
		} else {
			input = dcm.getInputSignal().getUniqueTimerName();
		}
		GridLayout layout = new GridLayout(5, 2);
		setLayout(layout);
		JLabel nameLabel = new JLabel("Input:");
		add(nameLabel);
		inputLabel = new JLabel(input);
		add(inputLabel);
		nameLabel = new JLabel("CLKIN_DIVIDE_BY_2:");
		add(nameLabel);
		clkinDivideBy2Label = new JLabel(clkinDivideBy2Value);
		add(clkinDivideBy2Label);
		nameLabel = new JLabel("CLKDV_DIVIDE:");
		add(nameLabel);
		clkdvDivideLabel = new JLabel(clkdvDivideValue);
		add(clkdvDivideLabel);
		nameLabel = new JLabel("CLKFX_MULTIPLY:");
		add(nameLabel);
		clkfxMultiplyLabel = new JLabel(clkfxMultiplyValue);
		add(clkfxMultiplyLabel);
		nameLabel = new JLabel("CLKFX_DIVIDE:");
		add(nameLabel);
		clkfxDivideLabel = new JLabel(clkfxDivideValue);
		add(clkfxDivideLabel);
		setPreferredSize(new Dimension(MIN_WIDTH, MIN_HEIGHT));
	}

	@Override
	public void loadConfiguration(Preferences prefs) {
		//nothing to do here
	}

	@Override
	public void onViewDispose() {
		//nothing to do here
	}

	@Override
	public void storeConfiguration(Preferences prefs) {
		//nothing to do here
	}

	@Override
	public void refresh(int address) {
		//nothing to do here
	}
}
