/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.dma;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.engine.peripheral.DmaDemonstrator;
import de.tu_darmstadt.rs.sim.gui.AbstractView;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.util.HashSet;
import java.util.prefs.Preferences;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;
import javax.swing.text.html.HTMLEditorKit;

/**
 * GUI class to display the DMA demonstrator device.
 */
@SuppressWarnings("serial")
public class DmaDemonstratorView extends AbstractView implements
		SpartanDefinitions_I {

	private static HashSet<Integer> DMA_DEMONSTRATOR_VIEW_INDEX_SET = new HashSet<Integer>();

	private final static String TEXT = "<html> <body> <font face=\"Arial\"> "
			+ "This DMA demonstrator can be dynamically added to and removed "
			+ "from the simulation. It writes consecutively "
			+ "every 3 CPU clocks a random number into the DMA memory. <br>"
			+ "If the last DMA address has been reached, an interrupt will be "
			+ "requested and the pointer will return to the DMA's beginning address."
			+ "</font> </body> </html> ";

	private final JLabel pointerLabel;
	private final JLabel irqLabel;
	private DmaDemonstrator dmaDemonstrator;
	private final SimulationManager simulationMgr;

	/**
	 * Constructor.
	 *
	 * @param simulationMgr
	 *            the SimulationManager
	 */
	public DmaDemonstratorView(final SimulationManager simulationMgr) {

		super();
		this.simulationMgr = simulationMgr;

		setPreferredSize(new Dimension(400, 200));

		setName("DMA Demonstrator " + dmaDemonstrator.getIndex());

		setLayout(new BorderLayout());

		final JEditorPane pane = new JEditorPane();
		pane.setEditorKit(new HTMLEditorKit());
		pane.setEditable(false);
		pane.setFocusable(false);
		pane.setText(TEXT);

		final JScrollPane scroll = new JScrollPane(pane);

		add(scroll, BorderLayout.CENTER);

		pointerLabel = new JLabel();
		pointerLabel.setBorder(new TitledBorder("Address pointer"));

		irqLabel = new JLabel();
		irqLabel.setBorder(new TitledBorder("Interrupt"));

		final JPanel subPanel = new JPanel();
		subPanel.setLayout(new GridLayout(1, 2));
		subPanel.add(pointerLabel);
		subPanel.add(irqLabel);
		add(subPanel, BorderLayout.SOUTH);

		dmaDemonstrator.addRefreshListener(this);
	}

	/**
	 * Retrieves the smallest unused static DMA device index.
	 *
	 * @return the index
	 */
	private static int findMeAnUnusedIndex() {
		for (int i = 0; true; ++i) {
			if (!DMA_DEMONSTRATOR_VIEW_INDEX_SET.contains(new Integer(i))) {
				return i;
			}
		}
	}

	@Override
	public void storeConfiguration(final Preferences prefs) {
		prefs.put("index", dmaDemonstrator.getIndex());
	}

	@Override
	public void loadConfiguration(final Preferences prefs) {

		final int unitIndex;
		if (prefs != null) {
			unitIndex = prefs.getInt("index", findMeAnUnusedIndex());
		} else {
			unitIndex = findMeAnUnusedIndex();
		}

		dmaDemonstrator = new DmaDemonstrator(simulationMgr,
				simulationMgr.getAddressSpaceManager());

		DMA_DEMONSTRATOR_VIEW_INDEX_SET.add(new Integer(unitIndex));
	}

	@Override
	public void onViewDispose() {

		dmaDemonstrator.removeRefreshListener(this);
		dmaDemonstrator.tearDown();
		DMA_DEMONSTRATOR_VIEW_INDEX_SET.remove(new Integer(dmaDemonstrator
				.getIndex()));
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see engine.RefreshListener_I#refresh(int)
	 */
	@Override
	public void refresh(final int address) {

		pointerLabel.setFont((Font) Settings
				.getSetting(Settings.REGISTER_FONT_PLAIN));

		final int addr = dmaDemonstrator.getDmaPointer();
		if (addr != INVALID) {
			pointerLabel.setText(String.format("%05x", addr));
		} else {
			pointerLabel.setText("invalid");
		}

		if (dmaDemonstrator.hasInterruptRequest()) {
			irqLabel.setText("Interrupt requested");
		} else {
			irqLabel.setText("No interrupt");
		}
	}

}
