/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.generic;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.peripheral.JConfigPeripheral;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

/**
 * A panel which shows the base I/O address.
 */
@SuppressWarnings("serial")
public class BaseAddressPanel extends JPanel {

	private JConfigPeripheral peripheral;
	private JLabel addressLabel;

	/**
	 * Constructor
	 *
	 * @param peripheral the peripheral whose base address is shown
	 */
	public BaseAddressPanel(JConfigPeripheral peripheral) {
		this.peripheral = peripheral;

		setBorder(new TitledBorder("IO base address"));
		setLayout(new GridLayout(1, 0));
		final JPanel subPanel0 = new JPanel();

		addressLabel = new JLabel();
		addressLabel.setBackground(Color.WHITE);
		addressLabel.setOpaque(true);
		addressLabel.setBorder(new EtchedBorder());
		subPanel0.add(addressLabel);
		JLabel registerLabel = new JLabel("[ Number of registers: "
				+ peripheral.getRegisters().length + " ]");
		subPanel0.add(registerLabel);
		add(subPanel0);
		refresh();
	}

	/**
	 * refresh the address panel
	 */
	public void refresh() {
		final int base = peripheral.getBaseAddr();
		final String txt = String.format("    %05x    ", base);
		addressLabel.setText(txt);
		addressLabel.setFont((Font) Settings
				.getSetting(Settings.REGISTER_FONT_PLAIN));

	}

}
