/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

package de.tu_darmstadt.rs.sim.gui.generic;

import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.JConfigPeripheral;
import de.tu_darmstadt.rs.sim.gui.AbstractView;
import de.tu_darmstadt.rs.sim.gui.peripheralRegisters.PeripheralRegisterView;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.ByteArrayOutputStream;
import java.util.Collection;
import java.util.LinkedList;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

/**
 * Generic View for peripherals (contains tabs for: input stream, output stream,
 * peripheral registers and peripheral specific views)
 */
@SuppressWarnings("serial")
public class GenericPeripheralView extends AbstractView {

	private final AddressSpaceManager addressSpaceManager;
	private JTabbedPane fTabs;
	private Collection<StreamOutputPane> fOutputPanes;
	private Collection<AbstractView> fSubViews;
	private FileInputPane fPAFileInput;
	private JConfigPeripheral fPeripheral;
	private ByteArrayOutputStream fOutputStream;

	/**
	 * create a new View for a peripheral
	 *
	 * @param peripheral the peripheral to be displayed
	 * @param addressSpaceManager the addressSpaceManager which toggles watchpoints
	 */
	public GenericPeripheralView(JConfigPeripheral peripheral, AddressSpaceManager addressSpaceManager) {
		super();
		this.addressSpaceManager = addressSpaceManager;
		setName(String.format("%s [%s]", peripheral.getPeriName(),
				peripheral.getPeriType()));
		fPeripheral = peripheral;
		fOutputStream = new ByteArrayOutputStream();
		fSubViews = new LinkedList<AbstractView>();
		fOutputPanes = new LinkedList<StreamOutputPane>();
		fTabs = new JTabbedPane();
		setLayout(new BorderLayout());
		if (peripheral.setOutputStream(null)) {
			createOutputTab(peripheral);
		}
		if (peripheral.setInputStream(null)) {
			createInputTab();
		}
		if ((peripheral.getRegisters() != null)
				&& peripheral.getRegisters().length != 0) {
			createRegisterTab(peripheral);
		}
		add(fTabs, BorderLayout.CENTER);
		peripheral.addRefreshListener(this);
		peripheral.setOutputStream(fOutputStream);
	}

	private void createRegisterTab(JConfigPeripheral peripheral) {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));
		p.setName("Registers");
		PeripheralRegisterView view = new PeripheralRegisterView(peripheral, addressSpaceManager);
		fSubViews.add(view);
		p.add(view);
		fTabs.add(p);
	}

	private void createOutputTab(JConfigPeripheral peripheral) {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));
		p.setName("Output");
		addOutputPane(p, new TextAreaOutputPane(peripheral));
		addOutputPane(p, new StdoutOutputPane());
		addOutputPane(p, new FileOutputPane());
		fTabs.add(p);
	}

	private void addOutputPane(JPanel c, StreamOutputPane p) {
		fOutputPanes.add(p);
		fSubViews.add(p);
		c.add(p);
	}

	private void createInputTab() {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));
		p.setName("Input");
		fPAFileInput = new FileInputPane(fPeripheral);
		fSubViews.add(fPAFileInput);
		p.add(fPAFileInput);
		p.add(Box.createVerticalGlue());
		fTabs.add(p);
	}

	/**
	 * add a view with peripheral specific information
	 *
	 * @param view a peripheral specific view to be added
	 */
	public void addPeripheralSpecificView(AbstractView view) {
		boolean wasSomeThingAdded = false;
		int compCount = view.getComponentCount();
		int tabCount;
		JPanel p = null;
		for (int i = 0; i < compCount; i++) {
			Component comp = view.getComponent(i);
			if (comp instanceof JTabbedPane) {
				JTabbedPane tPane = (JTabbedPane) comp;
				tabCount = tPane.getTabCount();
				for (int j = 0; j < tabCount; j++) {
					p = new JPanel();
					p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));
					p.setName(tPane.getTitleAt(0));
					wasSomeThingAdded = true;
					p.add(tPane.getComponent(0));
					fTabs.add(p);
				}
			}
		}
		if (!wasSomeThingAdded) {
			p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));
			p.setName(view.getName());
			p.add(view);
			fTabs.add(p);
		}
		fSubViews.add(view);
		if (p != null) {
			fTabs.setSelectedComponent(p);
		}
	}

	@Override
	public void onViewDispose() {
		//nothing to do here
	}

	@Override
	public void storeConfiguration(final Preferences prefs) {
		if (prefs == null) {
			return;
		}
		int i = 0;
		for (AbstractView v : fSubViews) {
			v.storeConfiguration(prefs.node("view" + i));
			i++;
		}
	}

	@Override
	public void loadConfiguration(final Preferences prefs) {
		if (prefs == null) {
			return;
		}
		int i = 0;
		for (AbstractView v : fSubViews) {
			try {
				if (prefs.nodeExists("view" + i)) {
					v.loadConfiguration(prefs.node("view" + i));
					i++;
				}
			} catch (BackingStoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	public void refresh(int method) {
		synchronized (fOutputStream) {
			if (fOutputStream.size() > 0) {
				for (StreamOutputPane p : fOutputPanes) {
					p.formatData(fOutputStream.toByteArray());
				}
				fOutputStream.reset();
			}
		}
	}
}
