/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

package de.tu_darmstadt.rs.sim.gui.generic;

import de.tu_darmstadt.rs.sim.gui.AbstractView;

import java.awt.BorderLayout;
import java.util.prefs.Preferences;
import javax.swing.BorderFactory;

/**
 * Base class for all panes which can write output data
 */
@SuppressWarnings("serial")
public abstract class StreamOutputPane extends AbstractView {

	private OutputFormatPane fFormatPane;

	/**
	 * create a new pane
	 *
	 * @param title the title of the new pane
	 */
	public StreamOutputPane(String title) {
		setName(title);
		setBorder(BorderFactory.createTitledBorder(title));
		setLayout(new BorderLayout());
		fFormatPane = new OutputFormatPane();
		add(fFormatPane, BorderLayout.SOUTH);
	}

	/**
	 * format and add the data according to the internal formatting scheme
	 *
	 * @param data the data to be added
	 */
	public void formatData(byte[] data) {
		appendOutput(fFormatPane.getSelectedFormat().formatData(data));
	}

	@Override
	public void loadConfiguration(final Preferences prefs) {
		fFormatPane.setSelectedFormatByName(prefs.get("format", ""));
	}

	@Override
	public void storeConfiguration(final Preferences prefs) {
		prefs.put("format", fFormatPane.getSelectedFormat().toString());
	}

	@Override
	public void onViewDispose() {
		//nothing to do here
	}

	/**
	 * add the data to the pane
	 *
	 * @param data the data to be added
	 */
	protected abstract void appendOutput(byte[] data);

}
