/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.interrupt;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.peripheral.interrupts.InterruptController;
import de.tu_darmstadt.rs.sim.gui.GuiDefinitions_I;
import de.tu_darmstadt.rs.sim.gui.generic.BaseAddressPanel;
import de.tu_darmstadt.rs.sim.utilities.Formatter;
import de.tu_darmstadt.rs.sim.utilities.ToolTipGuru;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 * The InterruptView's upper GUI panel.
 */
@SuppressWarnings("serial")
public class InterruptNorthPanel extends JPanel implements
		InterruptViewDefinitions_I, GuiDefinitions_I {

	private final JTable dataTable;
	private final TableColumnModel dataColumnModel;
	private final AbstractTableModel dataTableModel;

	private final BaseAddressPanel baseAddressPanel;

	private Font font;

	/**
	 * Constructor.
	 *
	 * @param view
	 *            the InterruptView
	 */
	public InterruptNorthPanel(final InterruptView view) {

		super();
		font = (Font) Settings.getSetting(Settings.REGISTER_FONT_PLAIN);
		setLayout(new GridLayout(2, 1));
		baseAddressPanel = new BaseAddressPanel(view.irqController);

		final JPanel subPanel1 = new JPanel();
		subPanel1.setBorder(new TitledBorder("Status"));

		dataColumnModel = new DefaultTableColumnModel();
		final TableColumn c0 = new TableColumn(0);
		c0.setPreferredWidth(150);
		dataColumnModel.addColumn(c0);

		final TableColumn c1 = new TableColumn(1);
		c1.setPreferredWidth(100);
		dataColumnModel.addColumn(c1);

		final TableColumn c2 = new TableColumn(2);
		c2.setPreferredWidth(190);
		dataColumnModel.addColumn(c2);

		dataTableModel = new AbstractTableModel() {
			@Override
			public int getRowCount() {
				return 2;
			}

			@Override
			public int getColumnCount() {
				return 3;
			}

			@Override
			public Object getValueAt(final int row, final int col) {
				switch (col) {
					case 0:
						if (row == InterruptController.ALL_IRQ_ADDRESS_OFFSET) {
							return "All interrupts";
						} else {
							return "Prior interrupt";
						}
					case 1:
						if (row == InterruptController.ALL_IRQ_ADDRESS_OFFSET) {
							return String.format("%05x",
									view.irqController.getInterruptIoValue());
						} else {
							return String.format("%05x",
									view.irqController.getPriorityIoValue());
						}
					default:
						if (row == InterruptController.ALL_IRQ_ADDRESS_OFFSET) {
							return Formatter.intToBinaryString(
									view.irqController.getInterruptIoValue(), true);
						} else {
							return Formatter.intToBinaryString(
									view.irqController.getPriorityIoValue(), true);
						}
				}
			}

			@Override
			public boolean isCellEditable(final int row, final int col) {
				return false;
			}
		};

		dataTable = new JTable(dataTableModel, dataColumnModel);
		dataTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		dataTable.getTableHeader().setReorderingAllowed(false);
		dataTable.setDefaultRenderer(Object.class,
				new DefaultTableCellRenderer() {
					@Override
					public Component getTableCellRendererComponent(
							final JTable t, final Object value,
							final boolean isSelected, final boolean hasFocus,
							final int row, final int col) {

						final Component component = super
								.getTableCellRendererComponent(t, value,
										false /* isSelected */, (col != 0)
												&& hasFocus, row, col);
						final JLabel label = (JLabel) component;
						label.setHorizontalAlignment(JLabel.CENTER);
						label.setVerticalAlignment(JLabel.BOTTOM);
						if (col == 0) {
							label.setBackground(LIGHT_GRAY_COLOR);
						} else {
							label.setBackground(Color.WHITE);
						}

						if (col == 1) {
							label.setToolTipText(ToolTipGuru.create(
									(String) value, true));
						} else {
							label.setToolTipText(null);
						}

						if (col == 0) {
							label.setForeground(Color.BLACK);
						} else if (!label.getText().equals("00000")
								&& !label.getText().equals(
								"00 0000 0000 0000 0000")) {
							label.setForeground(Color.RED);
						} else {
							label.setForeground(Color.BLACK);
						}

						label.setFont(font);
						label.setToolTipText(ToolTipGuru.create((String) value,
								true));
						return label;
					}
				});
		dataTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		subPanel1.add(dataTable);

		add(baseAddressPanel);
		add(subPanel1);

		refresh();
	}

	/**
	 * Refreshes the panel.
	 */
	public void refresh() {
		baseAddressPanel.refresh();
		dataTableModel.fireTableRowsUpdated(0, 1);
	}

	/**
	 * set the Font for the content
	 *
	 * @param font
	 *            the new font
	 */
	public void setContentFont(Font font) {
		this.font = font;
	}
}
