/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.interrupt;

import de.tu_darmstadt.rs.sim.engine.SettingListener_I;
import de.tu_darmstadt.rs.sim.engine.peripheral.interrupts.InterruptController;
import de.tu_darmstadt.rs.sim.gui.AbstractView;
import de.tu_darmstadt.rs.sim.utilities.ConvenientButton;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.prefs.Preferences;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 * The InterruptView displays the interrupt controller.
 */
@SuppressWarnings("serial")
public class InterruptView extends AbstractView implements ActionListener,
		InterruptViewDefinitions_I, SettingListener_I {

	final InterruptController irqController;

	private final InterruptNorthPanel northPanel;
	private final JScrollPane centerPanel;
	private final JPanel southPanel;

	private final JTable table;
	private final TableColumnModel columnModel;
	private final InterruptTableModel tableModel;

	private final ConvenientButton priorityUpButton;
	private final ConvenientButton priorityDownButton;
	private final InterruptCellRenderer cellRenderer;

	private int lastRowCount = -1;

	/**
	 * Constructor.
	 *
	 * @param irqController
	 *            the {@link InterruptController}
	 */
	public InterruptView(final InterruptController irqController) {

		super();

		this.irqController = irqController;

		setName("Interrupt Controller");
		setLayout(new BorderLayout(2, 2));
		setPreferredSize(new Dimension(500, 400));

		northPanel = new InterruptNorthPanel(this);
		columnModel = new DefaultTableColumnModel();

		final TableColumn col0 = new TableColumn(PRIORITY_COL);
		col0.setHeaderValue("Priority");
		col0.setPreferredWidth(40);
		columnModel.addColumn(col0);

		final TableColumn col1 = new TableColumn(NAME_COL);
		col1.setHeaderValue("Interrupt Name");
		col1.setPreferredWidth(150);
		columnModel.addColumn(col1);

		tableModel = new InterruptTableModel(this);

		table = new JTable(tableModel, columnModel);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getTableHeader().setReorderingAllowed(false);
		cellRenderer = new InterruptCellRenderer();
		table.setDefaultRenderer(Object.class, cellRenderer);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);

		centerPanel = new JScrollPane();
		centerPanel.setViewportView(table);

		southPanel = new JPanel();

		priorityUpButton = new ConvenientButton("Priority ++",
				"Increments the interrupts priority", this, southPanel);
		priorityDownButton = new ConvenientButton("Priority --",
				"Decrements the interrupts priority", this, southPanel);

		add(northPanel, BorderLayout.NORTH);
		add(centerPanel, BorderLayout.CENTER);
		add(southPanel, BorderLayout.SOUTH);

		irqController.addRefreshListener(this); // important
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(final ActionEvent event) {

		if (event.getActionCommand().equals(priorityUpButton.getText())) {

			final int row = table.getSelectedRow();
			final int priority = tableModel.translateRowToPriority(row);
			if ((row != -1) && (priority < irqController.getHighestPriority())) {

				irqController.incrementInterruptPriority(priority);
				tableModel.fireTableDataChanged();
				table.setRowSelectionInterval(row - 1, row - 1);
			}

		} else if (event.getActionCommand()
				.equals(priorityDownButton.getText())) {

			final int row = table.getSelectedRow();
			final int priority = tableModel.translateRowToPriority(row);
			if ((row != -1) && (priority > 1)) {

				irqController.decrementInterruptPriority(priority);
				tableModel.fireTableDataChanged();
				table.setRowSelectionInterval(row + 1, row + 1);
			}
		}
	}

	@Override
	public void storeConfiguration(final Preferences prefs) {
		// nothing to do here
	}

	@Override
	public void loadConfiguration(final Preferences prefs) {
		// nothing to do here
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.RefreshListener_I#refresh(int)
	 */
	@Override
	public void refresh(final int address) {

		northPanel.refresh();
		final int rowCount = tableModel.getRowCount();

		if (lastRowCount != rowCount) {
			tableModel.fireTableDataChanged();
			lastRowCount = rowCount;

		} else if (rowCount > 0) {
			tableModel.fireTableRowsUpdated(0, rowCount - 1);
		}
	}

	@Override
	public void onViewDispose() {
		irqController.removeRefreshListener(this);
	}

	@Override
	public void onSettingChange(int settingType, Object newValue) {
		northPanel.setContentFont((Font) newValue);
		cellRenderer.setContentFont((Font) newValue);
	}
}
