/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.memory;

import de.tu_darmstadt.rs.sim.engine.SettingListener_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.gui.AbstractView;
import de.tu_darmstadt.rs.sim.utilities.ConvenientButton;
import de.tu_darmstadt.rs.sim.utilities.Formatter;
import de.tu_darmstadt.rs.sim.utilities.MemoryAccess_I;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.prefs.Preferences;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

/**
 * The MemoryView displays the de.tu_darmstadt.rs.sim.main memory's content and the DMA memory's
 * content (if any).
 */
@SuppressWarnings("serial")
public class MemoryView extends AbstractView implements ActionListener,
		MemoryViewDefinitions_I, SettingListener_I {

	final SimulationManager simulationMgr;

	private final List<MemoryPanel> memoryPanelList;

	private final JTabbedPane tabbed;
	private final ConvenientButton deleteAllButton;
	private final ConvenientButton deleteSelectionButton;
	private final JTextField gotoAddressField;
	private final ConvenientButton gotoButton;

	/**
	 * Constructor.
	 *
	 * @param simulationMgr
	 *            the SimulationManager
	 */
	public MemoryView(final SimulationManager simulationMgr) {

		super();
		this.simulationMgr = simulationMgr;

		setName("Main Memory & DMA Memory");

		memoryPanelList = new ArrayList<MemoryPanel>();
		final MemoryAccess_I memoryAccess = simulationMgr.getAddressSpaceManager();
		memoryPanelList.add(new MemoryPanel(memoryAccess, new Default18MemoryPanel()));
		memoryPanelList.add(new MemoryPanel(memoryAccess, new Default9MemoryPanel()));
		memoryPanelList.add(new MemoryPanel(memoryAccess, new Data18MemoryPanel()));
		memoryPanelList.add(new MemoryPanel(memoryAccess, new Data9MemoryPanel()));
		tabbed = new JTabbedPane(JTabbedPane.BOTTOM);
		for (MemoryPanel memoryPanel : memoryPanelList) {
			tabbed.add(memoryPanel);
		}
		tabbed.setFocusable(false);

		final JPanel subPanel = new JPanel();
		gotoAddressField = new JTextField(7);
		gotoAddressField.setFont((Font) Settings
				.getSetting(Settings.REGISTER_FONT_PLAIN));
		subPanel.add(gotoAddressField);
		gotoButton = new ConvenientButton("Goto address",
				"Change the position", this, subPanel);

		deleteAllButton = new ConvenientButton("Delete all",
				"Deletes the whole memory content.", this, subPanel);

		deleteSelectionButton = new ConvenientButton("Delete Selection",
				"Deletes the memory content of the current cell selection.",
				this, subPanel);

		setLayout(new BorderLayout());
		add(tabbed, BorderLayout.CENTER);
		add(subPanel, BorderLayout.NORTH);

		simulationMgr.getAddressSpaceManager().addRefreshListener(this);
		tabbed.setSelectedIndex(2);
	}

	@Override
	public void storeConfiguration(final Preferences prefs) {
		// nothing
	}

	@Override
	public void loadConfiguration(final Preferences prefs) {
		// nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(final ActionEvent event) {
		final Component selectedComponent = tabbed.getComponentAt(tabbed
				.getSelectedIndex());
		final MemoryPanel memoryPanel = (MemoryPanel) selectedComponent;

		if (event.getActionCommand().equals(deleteSelectionButton.getText())) {

			final boolean sure = MessageBox
					.showConfirm(
							"Are you sure that you want to delete the current selection?",
							"Delete");
			if (!sure) {
				return;
			}

			memoryPanel.deleteSelection();

		} else if (event.getActionCommand().equals(deleteAllButton.getText())) {

			if (MessageBox.showConfirm(
					"Are you sure that you want to delete the de.tu_darmstadt.rs.sim.main memory?",
					"Delete")) {
				simulationMgr.getAddressSpaceManager().delete();
			}
		} else if (event.getSource() == gotoButton) {
			memoryPanel.scrollToAddress(
					Formatter.parseHexString(gotoAddressField.getText(),
							BITMASK_18) & ~0xf, true);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.RefreshListener_I#refresh(int)
	 */
	@Override
	public void refresh(final int address) {

		if (address == COMPLETE_REFRESH) {
			for (MemoryPanel memoryPanel : memoryPanelList) {
				memoryPanel.tableModel.fireTableDataChanged();
			}
		} else {
			for (MemoryPanel memoryPanel : memoryPanelList) {
				memoryPanel.refresh(address);
			}
		}
	}

	@Override
	public void onViewDispose() {
		simulationMgr.getAddressSpaceManager().removeRefreshListener(this);
	}

	@Override
	public void onSettingChange(int settingType, Object newValue) {
		gotoAddressField.setFont((Font) newValue);
		for (MemoryPanel memoryPanel : memoryPanelList) {
			memoryPanel.tableModel.fireTableDataChanged();
		}
	}
}
