/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.peripheralRegisters;

import de.tu_darmstadt.rs.sim.engine.peripheral.register.PeripheralRegisterUtil;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.PeripheralModuleDescription;


/**
 * Line in the {@link PeripheralRegisterView} containing information about a bit
 * group. Objects of this class are shown when the user opens a
 * {@link RegisterNode}
 */
public class BitGroupNode implements RegisterNode_I, RegisterTableDefinitions_I {

	private RegisterNode registerNode;
	private PeripheralModuleDescription.BitGroup bitGroup;
	private int bitWidth;

	/**
	 * create a new {@link BitGroupNode}
	 *
	 * @param registerNode the parent register node which contains this node
	 * @param bitGroup the bit group which provides the data
	 */
	public BitGroupNode(RegisterNode registerNode, PeripheralModuleDescription.BitGroup bitGroup) {
		this.registerNode = registerNode;
		this.bitGroup = bitGroup;
		this.bitWidth = bitGroup.getWidth();
	}

	@Override
	public void setExpanded(boolean expanded) {
		// cannot be expanded
	}

	@Override
	public boolean isExpanded() {
		return true;
	}

	@Override
	public boolean canExpand() {
		return false;
	}

	@Override
	public void addChild(RegisterNode_I childNode) {
		// cannot have any
	}

	@Override
	public Object getValue(int columnIndex) {
		final int registerValue = registerNode.getTheRegisterValue();
		switch (columnIndex) {
			case OFFSET_COLUMN:
				return -1;
			case NAME_COLUMN:
				return bitGroup.getName();
		/*case BIT_INDEX_COLUMN:
			if (bitGroup.getWidth() > 1) {
				return bitGroup.getIndex() + ".."
						+ (bitGroup.getIndex() + bitGroup.getWidth() - 1);
			} else {
				return "" + bitGroup.getIndex();
			}*/
			case BIN_COLUMN:
			case HEX_COLUMN:
				return PeripheralRegisterUtil.getShiftedBitGroupValue(
						registerValue, bitGroup);
		/*case SYMBOL_COLUMN:
			return bitGroup.getSymbolicValue(PeripheralRegisterUtil
					.getShiftedBitGroupValue(registerValue, bitGroup));*/
			default:
				return bitGroup;
		}
	}

	@Override
	public void setValue(Object theValue, int columnIndex) {
		switch (columnIndex) {
			case BIN_COLUMN:
			case HEX_COLUMN:
				final int oldRegValue = registerNode.getTheRegisterValue();
				final int newRegValue = PeripheralRegisterUtil.putBitGroupValue(
						oldRegValue, bitGroup, (Integer) theValue);
				registerNode.setTheRegisterValue(newRegValue);
				break;
		/*case SYMBOL_COLUMN:
			String strValue = (String) theValue;
			if (strValue != null && !strValue.equals("")) {
				final int oldRegValue1 = registerNode.getTheRegisterValue();
				final int newRegValue1 = PeripheralRegisterUtil
						.putBitGroupValue(oldRegValue1, bitGroup,
								bitGroup.getIntValue(strValue));
				registerNode.setTheRegisterValue(newRegValue1);
			}
			break;*/
		}
	}

	@Override
	public int getChildCount() {
		// cannot have any
		return 0;
	}

	@Override
	public RegisterNode_I getChildAt(int index) {
		// cannot have any
		return null;
	}

	@Override
	public int getBitWidth() {
		return bitWidth;
	}

	@Override
	public int getLevel() {
		return 1;
	}

	@Override
	public String getDescription() {
		return bitGroup.getDescription();
	}

}
