/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.peripheralRegisters;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

/**
 * Class representing a component for changing a bit (like a check box)
 */
@SuppressWarnings("serial")
public class BitValueSelector extends JPanel implements ActionListener {

	private boolean bitValue;
	private JLabel label;
	private JCheckBox checkbox;
	private List<ActionListener> actionListenerList;

	/**
	 * create a new {@link BitValueSelector}
	 */
	public BitValueSelector() {
		GridBagLayout layout = new GridBagLayout();
		setLayout(layout);
		label = new JLabel();
		label.setOpaque(true);
		label.setHorizontalTextPosition(SwingConstants.CENTER);
		label.setHorizontalAlignment(SwingConstants.CENTER);
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(label, constraints);
		checkbox = new JCheckBox();
		checkbox.setOpaque(true);
		checkbox.addActionListener(this);
		checkbox.setBorder(null);
		layout.setConstraints(checkbox, constraints);
		add(label);
		add(checkbox);
		updateElements();
		setBorder(null);
		setOpaque(true);
		setBackground(Color.WHITE);
		label.setBackground(Color.WHITE);
		actionListenerList = new ArrayList<ActionListener>();
	}

	private void updateElements() {
		bitValue = checkbox.isSelected();
		if (bitValue) {
			label.setText("1");
		} else {
			label.setText("0");
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == checkbox) {
			updateElements();
			informActionListeners(e);
		}
	}

	private void informActionListeners(ActionEvent e) {
		for (ActionListener actionListener : actionListenerList) {
			actionListener.actionPerformed(e);
		}
	}

	/**
	 * get the bit value
	 *
	 * @return the value of the bit (1 or 0)
	 */
	public int getValue() {
		return bitValue? 1 : 0;
	}

	/**
	 * set the bit value
	 *
	 * @param value the value of the bit (true for 1, false for 0)
	 */
	public void setValue(final boolean value) {
		checkbox.setSelected(value);
		updateElements();
	}

	/**
	 * add a listener for receiving events when the bit changes
	 *
	 * @param actionListener the listener to be added
	 */
	public void addActionListener(ActionListener actionListener) {
		if (!actionListenerList.contains(actionListener)) {
			actionListenerList.add(actionListener);
		}
	}

	/**
	 * remove a listener from receiving events when the bit changes
	 *
	 * @param actionListener the listener to be removed
	 * @return true if the actionListener was added previously
	 */
	public boolean removeActionListener(ActionListener actionListener) {
		return actionListenerList.remove(actionListener);
	}
}
