/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.peripheralRegisters;

import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.JConfigPeripheral;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JTable;
import javax.swing.SwingUtilities;

/**
 * Mouse Listener for the RegisterView which toggles watchpoint when clicked on a register
 */
public class PeripheralRegisterMouseListener implements MouseListener {

	private final AddressSpaceManager addressSpaceManager;
	private final JConfigPeripheral peripheral;
	private final JTable table;
	private final RegisterModel tableModel;

	/**
	 * create a new listener
	 *
	 * @param view the {@link PeripheralRegisterView} which shows the registers
	 * @param peripheral the peripheral whose registers are shown in the view
	 * @param addressSpaceManager the addressSpaceManager which toggles watchpoints
	 */
	public PeripheralRegisterMouseListener(PeripheralRegisterView view, JConfigPeripheral peripheral, AddressSpaceManager addressSpaceManager) {
		this.addressSpaceManager = addressSpaceManager;
		table = view.getTable();
		tableModel = (RegisterModel) table.getModel();
		this.peripheral = peripheral;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		//nothing to do here
	}

	@Override
	public void mousePressed(MouseEvent e) {
		//nothing to do here
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		if (SwingUtilities.isRightMouseButton(e)) {
			int row = table.rowAtPoint(e.getPoint());
			Object o = tableModel.getValueAt(row, 0);
			RegisterNode_I node = (RegisterNode_I) o;
			int offset = (Integer) node.getValue(RegisterTableDefinitions_I.OFFSET_COLUMN);
			while (offset == -1 && row >= 0) {
				o = tableModel.getValueAt(row, 0);
				node = (RegisterNode_I) o;
				offset = (Integer) node.getValue(RegisterTableDefinitions_I.OFFSET_COLUMN);
				row--;
			}
			int defaultAddress = peripheral.getBaseAddr() + offset;
			addressSpaceManager.toggleWatchpoint(defaultAddress);
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		//nothing to do here
	}

	@Override
	public void mouseExited(MouseEvent e) {
		//nothing to do here
	}
}
