/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.peripheralRegisters;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.AbstractCellEditor;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;


/**
 * Cell editor for the peripheral register view
 */
@SuppressWarnings("serial")
public class RegisterCellEditor extends AbstractCellEditor implements
		TableCellEditor, RegisterTableDefinitions_I, ActionListener {

	private HexRegisterCell[] hexCells;
	private BinRegisterCell[] binCells;
	private ExpandButton expandButton;
	private RegisterCell_I lastUsed;
	private boolean lastExpanded;
	private boolean lastExpandedValue;
	private SymbolicCell symbolicCell;

	/**
	 * create a new {@link RegisterCellEditor}
	 */
	public RegisterCellEditor() {
		hexCells = new HexRegisterCell[18];
		binCells = new BinRegisterCell[18];
		for (int i = 0; i < 18; i++) {
			hexCells[i] = new HexRegisterCell(i + 1);
			binCells[i] = new BinRegisterCell(i + 1);
			binCells[i].addActionListener(this);
		}
		expandButton = new ExpandButton();
		expandButton.addActionListener(this);
		symbolicCell = new SymbolicCell(this);
	}

	@Override
	public Component getTableCellEditorComponent(JTable table, Object value,
												 boolean isSelected, int row, int column) {
		RegisterNode_I node = (RegisterNode_I) value;
		HexRegisterCell hexCell;
		BinRegisterCell binCell;
		switch (column) {
			case EXPAND_COLUMN:
				if (node.canExpand()) {
					lastExpandedValue = node.isExpanded();
					expandButton.setExpanded(lastExpandedValue);
					lastExpanded = true;
					return expandButton;
				} else {
					assert (false);
					return null;
				}
			case HEX_COLUMN:
				hexCell = hexCells[node.getBitWidth() - 1];
				lastUsed = hexCell;
				hexCell.setValue((Integer) node.getValue(HEX_COLUMN));
				break;
			case BIN_COLUMN:
				binCell = binCells[node.getBitWidth() - 1];
				lastUsed = binCell;
				binCell.setValue((Integer) node.getValue(BIN_COLUMN));
				break;
			case SYMBOL_COLUMN:
				symbolicCell.setValue(node.getValue(-1).toString(), node.getValue(SYMBOL_COLUMN).toString());
				lastUsed = symbolicCell;
				break;
			default:
				return null;
		}
		return (Component) lastUsed;
	}

	@Override
	public Object getCellEditorValue() {
		if (lastExpanded) {
			lastExpanded = false;
			return !lastExpandedValue;
		}
		return lastUsed.getValue();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		stopCellEditing();
	}
}
