/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.peripheralRegisters;

import java.awt.Component;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;


/**
 * Cell renderer for the peripheral register view
 */
@SuppressWarnings("serial")
public class RegisterCellRenderer extends JLabel implements TableCellRenderer,
		RegisterTableDefinitions_I {

	private BinRegisterCell[] binCells;
	private ExpandButton expandButton;

	/**
	 * create a new {@link RegisterCellRenderer}
	 */
	public RegisterCellRenderer() {
		binCells = new BinRegisterCell[18];
		for (int i = 0; i < 18; i++) {
			binCells[i] = new BinRegisterCell(i + 1);
		}
		expandButton = new ExpandButton();
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
												   boolean isSelected, boolean hasFocus, int row, int column) {
		String stringValue;
		BinRegisterCell binCell;
		RegisterNode_I node = (RegisterNode_I) value;
		String description;
		switch (column) {
			case EXPAND_COLUMN:
				if (node.canExpand()) {
					expandButton.setExpanded(node.isExpanded());
					return expandButton;
				} else {
					stringValue = "";
					break;
				}
			case HEX_COLUMN:
				int iValue = (Integer) node.getValue(HEX_COLUMN);
				stringValue = String.format("0x%0"
						+ getHexWidth(node.getBitWidth()) + "x", iValue);
				break;
			case BIN_COLUMN:
				binCell = binCells[node.getBitWidth() - 1];
				binCell.setValue((Integer) node.getValue(BIN_COLUMN));
				return binCell;
			case OFFSET_COLUMN:
				int intValue = (Integer) node.getValue(OFFSET_COLUMN);
				if (intValue >= 0) {
					stringValue = "" + intValue;
				} else {
					stringValue = "";
				}
				break;
			case NAME_COLUMN:
				stringValue = "";
				for (int i = 0; i < node.getLevel(); i++) {
					stringValue += "  ";
				}
				stringValue += (String) node.getValue(column);
				description = node.getDescription();
				setToolTipText(description);
				break;
			case SYMBOL_COLUMN:
				description = node.getDescription();
				setToolTipText(description);
			default:
				stringValue = (String) node.getValue(column);
				break;
		}
		setText(stringValue);
		return this;
	}

	private int getHexWidth(final int bitWidth) {
		int returnValue = bitWidth / 4;
		if (returnValue * 4 < bitWidth) {
			returnValue++;
		}
		return returnValue;
	}

}
