/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.peripheralRegisters;

import de.tu_darmstadt.rs.sim.engine.peripheral.JConfigPeripheral;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.PeripheralModuleDescription;

import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;


/**
 * Table de.tu_darmstadt.rs.sim.model for the peripheral register's data
 */
@SuppressWarnings("serial")
public class RegisterModel extends AbstractTableModel implements
		RegisterTableDefinitions_I {

	private List<RegisterNode_I> registerNodeList;

	private int currentCount;

	/**
	 * create a new {@link RegisterModel} for the peripheral
	 *
	 * @param peripheral the peripheral
	 * @param registers register meta data
	 */
	public RegisterModel(JConfigPeripheral peripheral, PeripheralModuleDescription.Register[] registers) {
		registerNodeList = new ArrayList<RegisterNode_I>();
		RegisterNode_I node;
		for (int i = 0; i < registers.length; i++) {
			node = new RegisterNode(peripheral, registers[i], i);
			registerNodeList.add(node);
		}
	}

	@Override
	public int getColumnCount() {
		return COLUMN_COUNT;
	}

	@Override
	public String getColumnName(int columnIndex) {
		switch (columnIndex) {
			case OFFSET_COLUMN:
				return "Offset";
			case NAME_COLUMN:
				return "Name";
			case BIT_INDEX_COLUMN:
				return "Bits";
			case HEX_COLUMN:
				return "Value (Hex)";
			case BIN_COLUMN:
				return "Value (Bin)";
			default:
				return "";
		}
	}

	@Override
	public int getRowCount() {
		return count(registerNodeList);
	}

	private int count(List<RegisterNode_I> list) {
		currentCount = 0;
		for (RegisterNode_I node : list) {
			count(node);
		}
		return currentCount;
	}

	private void count(RegisterNode_I node) {
		currentCount++;
		if (node.canExpand() && node.isExpanded()) {
			for (int i = 0; i < node.getChildCount(); i++) {
				count(node.getChildAt(i));
			}
		}
	}

	private RegisterNode_I getNodeOfRowIndex(int rowIndex) {
		currentCount = 0;
		RegisterNode_I returnValue = null;
		for (RegisterNode_I node : registerNodeList) {
			returnValue = getNodeOfRowIndex(node, rowIndex);
			if (returnValue != null) {
				break;
			}
		}
		return returnValue;
	}

	private RegisterNode_I getNodeOfRowIndex(RegisterNode_I registerNode,
											 int rowIndex) {
		RegisterNode_I returnValue = null;
		if (currentCount == rowIndex) {
			returnValue = registerNode;
		} else {
			currentCount++;
			if (registerNode.canExpand() && registerNode.isExpanded()) {
				for (int i = 0; i < registerNode.getChildCount(); i++) {
					returnValue = getNodeOfRowIndex(registerNode.getChildAt(i),
							rowIndex);
					if (returnValue != null) {
						break;
					}
				}
			}
		}
		return returnValue;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		RegisterNode_I node = getNodeOfRowIndex(rowIndex);
		return node;
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		if (columnIndex == EXPAND_COLUMN) {
			RegisterNode_I node = getNodeOfRowIndex(rowIndex);
			if (node.canExpand()) {
				return true;
			}
			return false;
		}
		return ((columnIndex == HEX_COLUMN) || (columnIndex == BIN_COLUMN) || (columnIndex == SYMBOL_COLUMN));
	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		RegisterNode_I node = getNodeOfRowIndex(rowIndex);
		if (columnIndex == EXPAND_COLUMN) {
			node.setExpanded(!node.isExpanded());
		} else {
			node.setValue(value, columnIndex);
		}
		fireTableStructureChanged();
	}

}
