/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.peripheralRegisters;

/**
 * interface for all peripheral register data which can be presented
 */
public interface RegisterNode_I {

	/**
	 * set the expand state
	 *
	 * @param expanded true if the node should be expanded, false if it is collapsed
	 */
	void setExpanded(boolean expanded);

	/**
	 * check the expand state
	 *
	 * @return true if the node is expanded, false if it is collapsed
	 */
	boolean isExpanded();

	/**
	 * check whether the {@link RegisterNode_I} can be expanded
	 *
	 * @return true if the node can be expanded, false otherwise
	 */
	boolean canExpand();

	/**
	 * add a child {@link RegisterNode_I}
	 *
	 * @param childNode the node to be added
	 */
	void addChild(RegisterNode_I childNode);

	/**
	 * get the value for the column with index columnIndex
	 *
	 * @param columnIndex the index of the column
	 * @return the value to be shown in the column
	 */
	Object getValue(int columnIndex);

	/**
	 * set the value in a particular column
	 *
	 * @param theValue the value to be set
	 * @param columnIndex the index of the column
	 */
	void setValue(Object theValue, int columnIndex);

	/**
	 * get the number of child nodes
	 *
	 * @return number of child nodes
	 */
	int getChildCount();

	/**
	 * get a particular direct child node
	 *
	 * @param index the index of the child
	 * @return the child {@link RegisterNode_I}
	 */
	RegisterNode_I getChildAt(int index);

	/**
	 * get the number of bits of the value
	 *
	 * @return width in bits
	 */
	int getBitWidth();

	/**
	 * get the level of the node (number of parents)
	 *
	 * @return the level
	 */
	int getLevel();

	/**
	 * get a description of the node
	 *
	 * @return description
	 */
	String getDescription();

}
