/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.pipeline;

import de.tu_darmstadt.rs.sim.engine.processor.Instruction;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineStage;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineUI;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.border.LineBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * Displays the EXMEM stage's internals.
 */
@SuppressWarnings("serial")
public class ExMemPanel extends JPanel implements PipelineViewDefinitions_I {

	private final PipelineUI pipeUI;

	private final JLabel instr_Label;
	private final JLabel bypass_Label;
	private final JPanel subPanelNorth;

	private final JTable table;
	private final AbstractTableModel tableModel;
	private final String[][] tableData;

	private int rowCount;

	/**
	 * Constructor.
	 *
	 * @param pipeUI
	 *            the PipelineUI
	 */
	public ExMemPanel(final PipelineUI pipeUI) {

		super();
		this.pipeUI = pipeUI;

		rowCount = EXMEM_ROW_COUNT; // expanded View

		setLayout(new BorderLayout(5, 5));
		setBorder(new LineBorder(EXMEM_COLOR, 2));

		subPanelNorth = new JPanel();
		subPanelNorth.setLayout(new GridLayout(2, 1));

		instr_Label = new JLabel();
		instr_Label.setHorizontalAlignment(JLabel.CENTER);
		instr_Label.setFont(new Font("Arial", Font.BOLD, 12));
		subPanelNorth.add(instr_Label);

		bypass_Label = new JLabel();
		bypass_Label.setHorizontalAlignment(JLabel.CENTER);
		bypass_Label.setFont(new Font("Arial", Font.ITALIC, 12));
		subPanelNorth.add(bypass_Label);

		add(subPanelNorth, BorderLayout.NORTH);

		tableData = new String[EXMEM_ROW_COUNT][2];

		tableModel = new AbstractTableModel() {
			@Override
			public int getColumnCount() {
				return 2;
			}

			@Override
			public int getRowCount() {
				return rowCount;
			}

			@Override
			public Object getValueAt(int row, int col) {
				return tableData[row][col];
			}

			@Override
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};

		DefaultTableColumnModel columnModel = new DefaultTableColumnModel();

		columnModel.addColumn(new TableColumn(NAME_COL));
		columnModel.addColumn(new TableColumn(VALUE_COL, 10));

		table = new JTable(tableModel, columnModel);
		table.setDefaultRenderer(Object.class, new PanelCellRenderer());
		table.setShowGrid(false);
		table.setOpaque(false);
		add(table, BorderLayout.CENTER);

	}

	/**
	 * Toggles the stage's details on / off.
	 */
	public void toggleDetails() {

		if (rowCount == 0) {
			rowCount = EXMEM_ROW_COUNT;
			bypass_Label.setVisible(true);
		} else {
			rowCount = 0;
			bypass_Label.setVisible(false);
		}
		refreshPanel();
	}

	/**
	 * Clears the display showing "--" afterwards.
	 */
	public void clear() {

		instr_Label.setText(" -- ");
		bypass_Label.setText(null);

		for (int row = 0; row < EXMEM_ROW_COUNT; row++) {
			for (int col = 0; col < 2; col++) {
				tableData[row][col] = null;
			}
		}
	}

	/**
	 * Refreshes the panel's content.
	 */
	public void refreshPanel() {

		clear();

		final Instruction instr = pipeUI
				.getCurrentInstruction(PipelineStage.EXMEM_STAGE);
		if (instr != null) {
			instr_Label.setText(instr.description);
		} else {
			// Pipeline not in use yet
			instr_Label.setText("--");
			tableModel.fireTableDataChanged();
			return;
		}

		if (rowCount == 0) {
			// minimized --> no need to get all register values
			tableModel.fireTableDataChanged();
			return;
		}

		/****************************** Obligatory registers **********************************/

		final int bypass_opa = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_USE_BYPASS_OPA);
		final int bypass_opb = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_USE_BYPASS_OPB);

		// set the text depending on the bypass combination
		// (occures only if the same register is used more than once in a single
		// instruction)
		// note: do not care if some combination may never occur
		if ((bypass_opa == 1) && (bypass_opb == INVALID)) {
			bypass_Label.setText("Bypass operand A");
		} else if ((bypass_opa == INVALID) && (bypass_opb == 1)) {
			bypass_Label.setText("Bypass operand B");
		} else if ((bypass_opa == INVALID) && (bypass_opb == INVALID)) {
			bypass_Label.setText("Bypass register A");
		} else if ((bypass_opa == 1) && (bypass_opb == 1)) {
			bypass_Label.setText("Bypass operand A & B");
		} else {
			bypass_Label.setText("No bypass");
		}

		/****************************** Facultative registers ********************************/

		int index = 0;

		final int reg_no_a = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_REG_NO_WB);
		if (reg_no_a != INVALID) {
			tableData[index][NAME_COL] = " reg_no_a: ";
			tableData[index++][VALUE_COL] = String.format("%05x", reg_no_a);
		}

		final int reg_a = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_REG_STORE_SRC);
		if (reg_a != INVALID) {
			tableData[index][NAME_COL] = " reg_a: ";
			tableData[index++][VALUE_COL] = String.format("%05x", reg_a);
		}

		final int alu_op_a = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_ALU_OP_A);
		if (alu_op_a != INVALID) {
			tableData[index][NAME_COL] = " alu_op_a: ";
			tableData[index++][VALUE_COL] = String.format("%05x", alu_op_a);
		}

		final int alu_op_b = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_ALU_OP_B);
		if (alu_op_b != INVALID) {
			tableData[index][NAME_COL] = " alu_op_b: ";
			tableData[index++][VALUE_COL] = String.format("%05x", alu_op_b);
		}

		final int sfr_addr = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_SFR_ADDRESS);
		if (sfr_addr != INVALID) {
			tableData[index][NAME_COL] = " sfr_addr: ";
			tableData[index++][VALUE_COL] = String.format("%05x", sfr_addr);
		}

		final int data_addr = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_DATA_ADDR);
		if (data_addr != INVALID) {
			tableData[index][NAME_COL] = " data_addr: ";
			tableData[index++][VALUE_COL] = String.format("%05x", data_addr);
		}

		final int write_data_data = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_WRITE_DATA_DATA);
		if (write_data_data != INVALID) {
			tableData[index][NAME_COL] = " write_data_data: ";
			tableData[index++][VALUE_COL] = String.format("%05x",
					write_data_data);
		}

		final int alu_result = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_ALU_RESULT);
		if (alu_result != INVALID) {
			tableData[index][NAME_COL] = " alu_result: ";
			tableData[index++][VALUE_COL] = String.format("%05x", alu_result);
		}

		final int pc_delay = pipeUI
				.getRegister(VisiblePipelineRegister.EXMEM_PC_DELAY);
		if (pc_delay != INVALID) {
			tableData[index][NAME_COL] = " pc_delay: ";
			tableData[index++][VALUE_COL] = String.format("%05x", pc_delay);
		}

		repaint();
	}

}
