/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.pipeline;

import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineUI;
import de.tu_darmstadt.rs.sim.gui.AbstractView;
import de.tu_darmstadt.rs.sim.gui.MainWindow;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.net.URL;
import java.util.prefs.Preferences;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;
import javax.swing.border.EmptyBorder;

/**
 * The PipelineView displays the pipeline's internal registers, separated into
 * IFID stage, EXMEM stage and WB stage.
 */
@SuppressWarnings("serial")
public class PipelineView extends AbstractView
		implements PipelineViewDefinitions_I, ItemListener {


	private final PipelineUI pipeUI;

	private final JPanel mainPanel;
	private final JPanel topComponent;
	private final JLabel centerComponent;
	private final JPanel bottomComponent;
	private final IfIdPanel ifIdPanel;
	private final ExMemPanel exMemPanel;
	private final WbPanel wbPanel;
	private final ImageIcon pipeImage;
	private final JToggleButton minMaxToggleButton;


	/**
	 * Constructor.
	 * @param simulationMgr the SimulationManager
	 */
	public PipelineView(final SimulationManager simulationMgr) {

		super();
		setName("Pipeline Stages");
		this.pipeUI = simulationMgr.getPipelineUI();

		final URL url = MainWindow.class.getResource("/images/pipeline.jpg");
		pipeImage = new ImageIcon(url);

		mainPanel = new JPanel();
		mainPanel.setBorder(new EmptyBorder(0, 20, 20, 20));
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBackground(Color.WHITE);

		final JScrollPane scroll = new JScrollPane(mainPanel);
		scroll.setPreferredSize(new Dimension(750, 400));
		scroll.getHorizontalScrollBar().setUnitIncrement(40);
		scroll.getVerticalScrollBar().setUnitIncrement(40);
		setLayout(new BorderLayout());
		add(scroll, BorderLayout.CENTER);

		minMaxToggleButton = new JCheckBox("Show details", true);
		minMaxToggleButton.setBackground(Color.WHITE);
		minMaxToggleButton.setVisible(true);
		minMaxToggleButton.setFocusable(false);
		minMaxToggleButton.addItemListener(this);

		topComponent = new JPanel();
		topComponent.setBackground(Color.WHITE);
		topComponent.setLayout(new BorderLayout());
		topComponent.add(minMaxToggleButton, BorderLayout.EAST);
		mainPanel.add(topComponent, BorderLayout.NORTH);

		centerComponent = new JLabel();
		centerComponent.setHorizontalAlignment(JLabel.CENTER);
		centerComponent.setIcon(pipeImage);
		centerComponent.setVisible(true);
		mainPanel.add(centerComponent, BorderLayout.CENTER);

		bottomComponent = new JPanel();
		bottomComponent.setLayout(new GridLayout(1, 3, 10, 10));
		bottomComponent.setBackground(Color.WHITE);
		mainPanel.add(bottomComponent, BorderLayout.SOUTH);


		ifIdPanel = new IfIdPanel(pipeUI);
		bottomComponent.add(ifIdPanel);

		exMemPanel = new ExMemPanel(pipeUI);
		bottomComponent.add(exMemPanel);

		wbPanel = new WbPanel(pipeUI);
		bottomComponent.add(wbPanel);

		pipeUI.addRefreshListener(this);
	}


	/* (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	@Override
	public void itemStateChanged(final ItemEvent event) {

		int change = event.getStateChange();
		if (change == ItemEvent.SELECTED) {
			ifIdPanel.toggleDetails();
			exMemPanel.toggleDetails();
			wbPanel.toggleDetails();

		} else if (change == ItemEvent.DESELECTED) {
			ifIdPanel.toggleDetails();
			exMemPanel.toggleDetails();
			wbPanel.toggleDetails();
		}
	}


	@Override
	public void storeConfiguration(final Preferences prefs) {

		assert (prefs != null);
		prefs.putBoolean("minMax", minMaxToggleButton.isSelected());
	}


	@Override
	public void loadConfiguration(final Preferences prefs) {

		if (prefs != null) {
			minMaxToggleButton.setSelected(prefs.getBoolean("minMax", true));

		} else {
			// brand new view
			minMaxToggleButton.setSelected(true);
		}
	}


	/* (non-Javadoc)
	 * @see engine.RefreshListener_I#refresh(int)
	 */
	@Override
	public void refresh(final int address) {

		ifIdPanel.refreshPanel();
		exMemPanel.refreshPanel();
		wbPanel.refreshPanel();
	}


	@Override
	public void onViewDispose() {
		pipeUI.removeRefreshListener(this);
	}

}
