/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.pipeline;

import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;
import de.tu_darmstadt.rs.sim.gui.GuiDefinitions_I;

/**
 * The PipelineView's definitions.
 */
public interface PipelineViewDefinitions_I extends SpartanDefinitions_I,
		GuiDefinitions_I {

	/**
	 * index of the column showing the name of the internal de.tu_darmstadt.rs.sim.signal
	 */
	public final static int NAME_COL = 0;
	/**
	 * index of the column showing the current value of the de.tu_darmstadt.rs.sim.signal
	 */
	public final static int VALUE_COL = 1;

	/**
	 * index of the row showing the program counter
	 */
	public final static int IFID_PC_ROW = 0;
	/**
	 * index of the row showing the new program counter
	 */
	public final static int IFID_PC_NEW_ROW = 1;
	/**
	 * index of the row showing the instruction data
	 */
	public final static int IFID_IR_ROW = 2;

	/**
	 * index of the ifid's dynamic row 1
	 */
	public final static int IFID_DYN_0_ROW = 3;
	/**
	 * index of the ifid's dynamic row 2
	 */
	public final static int IFID_DYN_1_ROW = 4;
	/**
	 * index of the ifid's dynamic row 3
	 */
	public final static int IFID_DYN_2_ROW = 5;
	/**
	 * index of the ifid's dynamic row 4
	 */
	public final static int IFID_DYN_3_ROW = 6;
	/**
	 * index of the ifid's dynamic row 5
	 */
	public final static int IFID_DYN_4_ROW = 7;
	/**
	 * index of the ifid's dynamic row 6
	 */
	public final static int IFID_DYN_5_ROW = 8;

	/**
	 * how many dynamic rows are possible in the ifid stage?
	 */
	public final static int IFID_DYN_COUNT = 6;
	/**
	 * how many rows are possible in the ifid stage?
	 */
	public final static int IFID_ROW_COUNT = IFID_DYN_COUNT + 3;

	/**
	 * index of the exmem's dynamic row 1
	 */
	public final static int EXMEM_DYN_0_ROW = 0;
	/**
	 * index of the exmem's dynamic row 2
	 */
	public final static int EXMEM_DYN_1_ROW = 1;
	/**
	 * index of the exmem's dynamic row 3
	 */
	public final static int EXMEM_DYN_2_ROW = 2;
	/**
	 * index of the exmem's dynamic row 4
	 */
	public final static int EXMEM_DYN_3_ROW = 3;
	/**
	 * index of the exmem's dynamic row 5
	 */
	public final static int EXMEM_DYN_4_ROW = 4;
	/**
	 * index of the exmem's dynamic row 6
	 */
	public final static int EXMEM_DYN_5_ROW = 5;
	/**
	 * index of the exmem's dynamic row 7
	 */
	public final static int EXMEM_DYN_6_ROW = 6;
	/**
	 * index of the exmem's dynamic row 8
	 */
	public final static int EXMEM_DYN_7_ROW = 7;
	/**
	 * index of the exmem's dynamic row 9
	 */
	public final static int EXMEM_DYN_8_ROW = 8;

	/**
	 * how many dynamic rows are possible in the exmem stage?
	 */
	public final static int EXMEM_DYN_COUNT = 9;
	/**
	 * how many rows are possible in the exmem stage?
	 */
	public final static int EXMEM_ROW_COUNT = EXMEM_DYN_COUNT;

	/**
	 * index of the row showing the do write back de.tu_darmstadt.rs.sim.signal
	 */
	public final static int WB_DO_WB_ROW = 0;
	/**
	 * index of the row showing the register number
	 */
	public final static int WB_WRITE_REG_NUMBER_ROW = 1;
	/**
	 * index of the row showing the data to be used in the wb stage
	 */
	public final static int WB_WRITE_REG_DATA_ROW = 2;

	/**
	 * how many rows are possible in the wb stage?
	 */
	public final static int WB_ROW_COUNT = 9;
}
