/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.pipeline;

import de.tu_darmstadt.rs.sim.engine.processor.Instruction;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineStage;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineUI;

import java.awt.BorderLayout;
import java.awt.Font;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.border.LineBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * Displays the WB stage's internals.
 */
@SuppressWarnings("serial")
public class WbPanel extends JPanel implements PipelineViewDefinitions_I {

	private final PipelineUI pipeUI;

	private final JLabel instr_Label;
	private final JTable table;
	private final AbstractTableModel tableModel;
	private final String[][] tableData;

	private int rowCount;

	/**
	 * Constructor.
	 *
	 * @param pipeUI
	 *            the PipelineUI
	 */
	public WbPanel(PipelineUI pipeUI) {

		super();
		this.pipeUI = pipeUI;

		rowCount = WB_ROW_COUNT;

		setLayout(new BorderLayout(5, 5));
		setBorder(new LineBorder(WB_COLOR, 2));

		instr_Label = new JLabel();
		instr_Label.setHorizontalAlignment(JLabel.CENTER);
		instr_Label.setFont(new Font("Arial", Font.BOLD, 12));
		add(instr_Label, BorderLayout.NORTH);

		tableData = new String[WB_ROW_COUNT][2];

		tableModel = new AbstractTableModel() {
			@Override
			public int getColumnCount() {
				return 2;
			}

			@Override
			public int getRowCount() {
				return rowCount;
			}

			@Override
			public Object getValueAt(int row, int col) {
				return tableData[row][col];
			}

			@Override
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};

		DefaultTableColumnModel columnModel = new DefaultTableColumnModel();

		columnModel.addColumn(new TableColumn(NAME_COL));
		columnModel.addColumn(new TableColumn(VALUE_COL, 10));
		table = new JTable(tableModel, columnModel);
		table.setDefaultRenderer(Object.class, new PanelCellRenderer());
		table.setShowGrid(false);
		table.setOpaque(false);
		add(table, BorderLayout.CENTER);
	}

	/**
	 * Toggles the stage's details on / off.
	 */
	public void toggleDetails() {
		if (rowCount == 0) {
			rowCount = WB_ROW_COUNT;
		} else {
			rowCount = 0;
		}
		refreshPanel();
	}

	/**
	 * Clears the display showing "--" afterwards.
	 */
	public void clear() {

		instr_Label.setText(" -- ");
		for (int row = 0; row < WB_ROW_COUNT; row++) {
			for (int col = 0; col < 2; col++) {
				tableData[row][col] = null;
			}
		}
	}

	/**
	 * Refreshes the panel's content.
	 */
	public void refreshPanel() {

		clear();

		final Instruction instr = pipeUI
				.getCurrentInstruction(PipelineStage.WB_STAGE);
		if (instr != null) {
			instr_Label.setText(instr.description);
		} else {
			// Pipeline not in use yet
			instr_Label.setText("--");
			tableModel.fireTableDataChanged();
			return;
		}

		if (rowCount == 0) {
			// minimized --> no need to get all register values
			tableModel.fireTableDataChanged();
			return;
		}

		/****************************** Obligatory registers **********************************/

		final int do_wb = pipeUI.getRegister(VisiblePipelineRegister.WB_DO_WB);
		assert (do_wb != INVALID);
		tableData[WB_DO_WB_ROW][NAME_COL] = " do_wb: ";
		tableData[WB_DO_WB_ROW][VALUE_COL] = Integer.toString(do_wb);

		/****************************** Facultative registers ********************************/

		final int write_reg_number = pipeUI
				.getRegister(VisiblePipelineRegister.WB_WRITE_REG_NUMBER);
		final int write_reg_data = pipeUI
				.getRegister(VisiblePipelineRegister.WB_WRITE_REG_DATA);

		if (write_reg_number != INVALID) {
			assert (write_reg_data != INVALID);

			tableData[WB_WRITE_REG_NUMBER_ROW][NAME_COL] = " write_reg_number: ";
			tableData[WB_WRITE_REG_NUMBER_ROW][VALUE_COL] = String.format(
					"%05x", write_reg_number);

			tableData[WB_WRITE_REG_DATA_ROW][NAME_COL] = " write_reg_data: ";
			tableData[WB_WRITE_REG_DATA_ROW][VALUE_COL] = String.format("%05x",
					write_reg_data);
		}

		tableModel.fireTableDataChanged();
	}

}
