/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.registers.registerfile;

import de.tu_darmstadt.rs.sim.engine.RefreshListener_I;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.processor.RegisterFile;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisterEnum;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisters;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.DefaultCellEditor;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * The RegisterFileView displays the register file consisting of a table for the
 * 4 global registers and another one for the non-global registers.
 */
@SuppressWarnings("serial")
public class RegisterFileView extends JPanel implements MouseListener,
		RegisterFileViewDefinitions_I, RefreshListener_I {

	private final RegisterFile regFile;
	private final SfRegisters sfr;

	private final JTable bottomTable;
	private final JTable topTable;
	private final RegisterFileTableModel bottomTableModel;
	private final RegisterFileTableModel topTableModel;

	private final DefaultTableColumnModel columnModel;

	/**
	 * Constructor.
	 *
	 * @param mgr
	 *            the SimulationManager
	 */
	public RegisterFileView(final SimulationManager mgr) {

		super();
		setName("Register File");
		this.regFile = mgr.getRegisterFile();
		this.sfr = mgr.getSfRegisters();

		bottomTableModel = new RegisterFileTableModel(regFile) {
			@Override
			public int calculateRegFileAddress(int row, int col) {
				return row * 4 + col - 1 + 4;
			}

			@Override
			public int getRowCount() {
				return (REG_FILE_COUNT - 4) / 4;
			}
		};

		topTableModel = new RegisterFileTableModel(regFile) {
			@Override
			public int calculateRegFileAddress(int row, int col) {
				return col - 1;
			}

			@Override
			public int getRowCount() {
				return 1;
			}
		};

		columnModel = new DefaultTableColumnModel();

		final TableColumn col0 = new TableColumn(NUMB_COL);
		col0.setMinWidth(20);
		columnModel.addColumn(col0);

		for (int i = 0; i < 4; i++) {
			final TableColumn col1 = new TableColumn(i + 1);
			col1.setMaxWidth(50);
			col1.setMinWidth(50);
			columnModel.addColumn(col1);
		}

		bottomTable = new JTable(bottomTableModel, columnModel);
		bottomTable.setDefaultRenderer(Object.class,
				new RegisterFileCellRenderer(regFile) {
					@Override
					public int calculateRegFileAddress(int row, int col) {
						return row * 4 + col - 1 + 4;
					}

				});
		bottomTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		bottomTable.setDragEnabled(false);

		bottomTable.getTableHeader().setReorderingAllowed(false);
		bottomTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		bottomTable.setRowHeight(15);
		bottomTable.addMouseListener(this);

		topTable = new JTable(topTableModel, columnModel);
		topTable.setDefaultRenderer(Object.class, new RegisterFileCellRenderer(regFile) {
			@Override
			public int calculateRegFileAddress(int row, int col) {
				return col - 1;
			}

		});

		topTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		topTable.setDragEnabled(false);
		topTable.setOpaque(false);
		topTable.getTableHeader().setReorderingAllowed(false);
		topTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		topTable.setRowHeight(15);
		topTable.addMouseListener(this);

		final JScrollPane topScroll = new JScrollPane(topTable);
		final JScrollPane bottomScroll = new JScrollPane(bottomTable);

		topScroll.setPreferredSize(new Dimension(
				topTable.getPreferredSize().width,
				topTable.getPreferredSize().height + 3));
		// NOTE: the topScroll will never be used, since the topTable has only
		// one single row, but it is necessary to shift the table a little
		// to the right, according to the bottomTable

		setLayout(new BorderLayout());
		add(topScroll, BorderLayout.NORTH);
		add(bottomScroll, BorderLayout.CENTER);

		setPreferredSize(new Dimension(topTable.getPreferredSize().width + 30,
				200));
		topTable.setCellEditor(new DefaultCellEditor(new JTextField()));

		regFile.addRefreshListener(this);
		sfr.addRefreshListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseClicked(final MouseEvent event) {

		// react only to right mouse click
		if (event.getButton() != MouseEvent.BUTTON3) {
			return;
		}

		if (event.getSource() == bottomTable) {

			final int row = bottomTable.rowAtPoint(event.getPoint());
			final int col = bottomTable.columnAtPoint(event.getPoint());

			if (col != NUMB_COL) {
				int addr = row * 4 + col - 1 + 4;
				regFile.toggleWatchpoint(addr);
			}

		} else if (event.getSource() == topTable) {

			final int col = topTable.columnAtPoint(event.getPoint());

			if (col != NUMB_COL) {
				int addr = col - 1;
				regFile.toggleWatchpoint(addr);
			}
		}
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// never used
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// never used
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// never used
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// never used
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.RefreshListener_I#refresh(int)
	 */
	@Override
	public void refresh(final int address) {

		if (address == SfRegisterEnum.WND.ordinal()) {
			// Note: method can be invoked by the register file or the SFR,
			// if it is the SFR: we have to refresh everything in case of a
			// changed WND
			// if it is the register file: this total refresh does not harm
			topTableModel.fireTableRowsUpdated(0, 0);
			bottomTableModel.fireTableRowsUpdated(0,
					bottomTableModel.getRowCount() - 1);

		} else if (address == COMPLETE_REFRESH) {
			topTableModel.fireTableRowsUpdated(0, 0);
			bottomTableModel.fireTableRowsUpdated(0,
					bottomTableModel.getRowCount() - 1);
			bottomTableModel.fireTableDataChanged();

		} else {
			if (address < 4) {
				topTableModel.fireTableRowsUpdated(0, 0);
			} else {
				bottomTableModel.fireTableRowsUpdated((address - 4) / 4,
						(address - 4) / 4);
			}

		}

	}

	/**
	 * Called when the view is to be disposed.
	 */
	public void onDispose() {
		regFile.removeRefreshListener(this);
		sfr.removeRefreshListener(this);
	}

}
