/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
  _______________________________________________________________________________

  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
  (http://www.mr.inf.tu-dresden.de) All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

  3. All advertising materials mentioning features or use of this software
  must display the following acknowledgement: "This product includes
  software developed by the TU Dresden Chair for Embedded Systems and
  its contributors."

  4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
  names of its contributors may be used to endorse or promote products
  derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  _______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.registers.sfr;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Polygon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;

/**
 * Visualizes a 7-segment-display based on the LED register's actual value.
 * Implementation taken from: Handbuch der Java-Programmierung G. Krüger.
 * HTML-Ausgabe 4.0.2. Addison-Wesley, 2004
 */
@SuppressWarnings("serial")
public class SfRegistersSegmentDisplay extends JPanel {

	private final int[][] polysx = {{1, 2, 8, 9, 8, 2}, // segment 0
			{9, 10, 10, 9, 8, 8}, // segment 1
			{9, 10, 10, 9, 8, 8}, // segment 2
			{1, 2, 8, 9, 8, 2}, // segment 3
			{1, 2, 2, 1, 0, 0}, // segment 4
			{1, 2, 2, 1, 0, 0}, // segment 5
			{1, 2, 8, 9, 8, 2}, // segment 6
	};

	private final int[][] polysy = {{1, 0, 0, 1, 2, 2}, // segment 0
			{1, 2, 8, 9, 8, 2}, // segment 1
			{9, 10, 16, 17, 16, 10}, // segment 2
			{17, 16, 16, 17, 18, 18}, // segment 3
			{9, 10, 16, 17, 16, 10}, // segment 4
			{1, 2, 8, 9, 8, 2}, // segment 5
			{9, 8, 8, 9, 10, 10}, // segment 6
	};

	private final JPanel segmentPanel;

	/**
	 * Constructor.
	 *
	 * @param view
	 *            the SfRegistersView
	 */
	public SfRegistersSegmentDisplay(final SfRegistersView view) {

		super();
		segmentPanel = new JPanel() {

			@Override
			public void paint(final Graphics g) {
				super.paint(g);

				final Color color = Color.BLUE;
				final int OFFSET_X = 10;
				final int OFFSET_Y = 7;
				final int dx = getSize().width / 16;
				final int dy = getSize().height / 20;

				final int leds = view.sfr.getLEDS();

				g.setColor(color);
				for (int i = 0; i < 7; ++i) {

					if ((leds & (0x1 << i)) != 0) {
						final Polygon poly = new Polygon();
						for (int j = 0; j < 6; ++j) {
							poly.addPoint(dx * polysx[i][j] + OFFSET_X, dy
									* polysy[i][j] + OFFSET_Y);
						}
						g.fillPolygon(poly);
					}
				}

				g.setColor(getBackground());

				g.drawLine(OFFSET_X, OFFSET_Y, dx * 10 + OFFSET_X, dy * 10
						+ OFFSET_Y);
				g.drawLine(OFFSET_X, 1 + OFFSET_Y, dx * 10 + OFFSET_X, dy * 10
						+ 1 + OFFSET_Y);

				g.drawLine(OFFSET_X, 8 * dy + OFFSET_Y, 10 * dx + OFFSET_X, 18
						* dy + OFFSET_Y);
				g.drawLine(OFFSET_X, 8 * dy + 1 + OFFSET_Y, 10 * dx + OFFSET_X,
						18 * dy + 1 + OFFSET_Y);

				g.drawLine(OFFSET_X, 10 * dy + OFFSET_Y, 10 * dx + OFFSET_X,
						OFFSET_Y);
				g.drawLine(OFFSET_X, 10 * dy + 1 + OFFSET_Y,
						10 * dx + OFFSET_X, 1 + OFFSET_Y);

				g.drawLine(OFFSET_X, 18 * dy + OFFSET_Y, 10 * dx + OFFSET_X, 8
						* dy + OFFSET_Y);
				g.drawLine(OFFSET_X, 18 * dy + 1 + OFFSET_Y,
						10 * dx + OFFSET_X, 8 * dy + 1 + OFFSET_Y);
			}
		};

		segmentPanel.setPreferredSize(new Dimension(5 * 8, 5 * 10));
		segmentPanel.setBorder(new EtchedBorder(EtchedBorder.RAISED));

		add(new JLabel("LED 7-Segment Display"));
		add(segmentPanel, BorderLayout.CENTER);
	}
}
