/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.registers.sfr;

import de.tu_darmstadt.rs.sim.engine.memory.MainMemory;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisters;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineUI;
import de.tu_darmstadt.rs.sim.utilities.Formatter;

import javax.swing.table.AbstractTableModel;


/**
 * Table de.tu_darmstadt.rs.sim.model for the SFR table.
 */
@SuppressWarnings("serial")
public class SfRegistersTableModel extends AbstractTableModel implements SfRegistersViewDefinitions_I {


	private final PipelineUI pipeUI;
	private final SfRegisters sfr;


	/**
	 * Constructor.
	 * @param view the SfRegistersView
	 */
	public SfRegistersTableModel(final SfRegistersView view) {
		super();
		this.pipeUI = view.pipeUI;
		this.sfr = view.sfr;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	@Override
	public int getRowCount() {
		return ROW_COUNT;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	@Override
	public int getColumnCount() {
		return COLUMN_COUNT;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	@Override
	public Object getValueAt(final int row, final int col) {


		switch (row) {

			case PC_ROW:
				if (col == NAME_COL) {
					return "New PC";
				} else {
					int pc_new = pipeUI.getRegister(VisiblePipelineRegister.IFID_PC_NEW);
					if (pc_new == INVALID) {
						return "--";
					} else {
						return String.format("%05x", pc_new);
					}
				}

			case MUL_ROW:
				if (col == NAME_COL) {
					return "MUL";
				} else {
					return String.format("%05x", pipeUI.getRegister(VisiblePipelineRegister.SFR_MUL));
				}

			case LEDS_ROW:
				if (col == NAME_COL) {
					return "LEDS";
				} else {
					return String.format("%02x", pipeUI.getRegister(VisiblePipelineRegister.SFR_LEDS));
				}

			case IV_ROW:
				if (col == NAME_COL) {
					return "IV";
				} else {
					return String.format("%01x", pipeUI.getRegister(VisiblePipelineRegister.SFR_IV));
				}

			case TR_ROW:
				if (col == NAME_COL) {
					return "TR";
				} else {
					return String.format("%01x", pipeUI.getRegister(VisiblePipelineRegister.SFR_TR));
				}

			case WND_ROW:
				if (col == NAME_COL) {
					return "WND";
				} else {
					return String.format("%02x", pipeUI.getRegister(VisiblePipelineRegister.SFR_WINDOW));
				}

			case CC_ROW:
				if (col == NAME_COL) {
					return "CC";
				} else {
					return String.format("%01x", pipeUI.getRegister(VisiblePipelineRegister.SFR_CC));
				}

			case MM_ROW:
				if (col == NAME_COL) {
					return "MM";
				} else {
					return String.format("%01x", pipeUI.getRegister(VisiblePipelineRegister.SFR_MM));
				}

			default:
				assert (row == INT_ROW);
				if (col == NAME_COL) {
					return "INT";
				} else {
					return String.format("%01x", pipeUI.getRegister(VisiblePipelineRegister.SFR_INTERRUPT));
				}
		}
	}


	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	@Override
	public boolean isCellEditable(final int row, final int col) {
		return (col != NAME_COL);
	}


	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#setValueAt(Object, int, int)
	 */
	@Override
	public void setValueAt(final Object aValue, final int row, final int col) {

		assert (col == VALUE_COL);

		int val = Formatter.parseHexString((String) aValue, BITMASK_18);
		if (val == INVALID) {
			return;
		}


		switch (row) {

			case PC_ROW:
				if (!MainMemory.containsAddress(val)) {
					return;
				} else {
					pipeUI.setNewPC(val);
				}
				break;

			case MUL_ROW:
				if ((val & ~BITMASK_18) != 0) {
					return;
				} else {
					sfr.setMUL(val);
				}
				break;

			case LEDS_ROW:
				if ((val & ~BITMASK_7) != 0) {
					return;
				} else {
					sfr.setLEDS(val);
				}
				break;

			case IV_ROW:
				if ((val & ~BITMASK_18) != 0) {
					return;
				} else {
					sfr.setIV(val);
				}
				break;

			case TR_ROW:
				if ((val & ~BITMASK_10H) != 0) {
					return;
				} else {
					sfr.setTR(val);
				}
				break;

			case WND_ROW:
				if ((val & ~BITMASK_6) != 0) {
					return;
				} else {
					sfr.setWindow(val);
				}
				break;

			case CC_ROW:
				if ((val & ~BITMASK_1) != 0) {
					return;
				} else {
					sfr.setCC(val);
				}
				break;

			case MM_ROW:
				if ((val & ~BITMASK_1) != 0) {
					return;
				} else {
					sfr.setMM(val);
				}
				break;

			case INT_ROW:
				if ((val & ~BITMASK_1) != 0) {
					return;
				} else {
					sfr.setInterruptEnable(val);
				}
				break;

		}
	}


}
