/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.registers.sfr;

import de.tu_darmstadt.rs.sim.engine.RefreshListener_I;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisterEnum;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisters;
import de.tu_darmstadt.rs.sim.engine.processor.pipeline.PipelineUI;

import java.awt.BorderLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * The SFRegistersView displays the SFR, the next PC and a 7-segment LED
 * display.
 */
@SuppressWarnings("serial")
public class SfRegistersView extends JComponent implements
		SfRegistersViewDefinitions_I, RefreshListener_I, MouseListener {

	private final JTable table;
	private final SfRegistersTableModel tableModel;
	private final DefaultTableColumnModel columnModel;

	private final SfRegistersSegmentDisplay segmentDisplay;
	final SfRegisters sfr;
	final PipelineUI pipeUI;

	/**
	 * Constructor.
	 *
	 * @param mgr
	 *            the SimulationManager
	 */
	public SfRegistersView(final SimulationManager mgr) {

		super();
		this.sfr = mgr.getSfRegisters();
		this.pipeUI = mgr.getPipelineUI();
		setName("CPU Registers");

		tableModel = new SfRegistersTableModel(this);
		columnModel = new DefaultTableColumnModel();

		final TableColumn col0 = new TableColumn(NAME_COL);
		col0.setMinWidth(90);
		col0.setMaxWidth(90);
		columnModel.addColumn(col0);

		final TableColumn col1 = new TableColumn(VALUE_COL);

		col1.setMinWidth(90);
		columnModel.addColumn(col1);

		table = new JTable(tableModel, columnModel);
		table.setDefaultRenderer(Object.class,
				new SfRegistersCellRenderer(this));
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setDragEnabled(false);
		table.addMouseListener(this);
		table.getTableHeader().setReorderingAllowed(false);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
		table.setRowHeight(15);
		setBorder(new TitledBorder(getName()));
		setLayout(new BorderLayout());

		final JPanel subPanel = new JPanel();
		subPanel.add(table);
		add(subPanel, BorderLayout.CENTER);

		segmentDisplay = new SfRegistersSegmentDisplay(this);
		add(segmentDisplay, BorderLayout.SOUTH);

		sfr.addRefreshListener(this);
		pipeUI.addRefreshListener(this); // necessary for PC NEW
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseClicked(final MouseEvent event) {

		// react only to right mouse click
		if (event.getButton() != MouseEvent.BUTTON3) {
			return;
		}

		final int row = table.rowAtPoint(event.getPoint());
		final int col = table.columnAtPoint(event.getPoint());

		if (col != NAME_COL) {
			final int addr;
			switch (row) {
				case MUL_ROW:
					addr = SfRegisterEnum.MUL.ordinal();
					break;
				case LEDS_ROW:
					addr = SfRegisterEnum.LEDS.ordinal();
					break;
				case WND_ROW:
					addr = SfRegisterEnum.WND.ordinal();
					break;
				case CC_ROW:
					addr = SfRegisterEnum.CC.ordinal();
					break;
				case MM_ROW:
					addr = SfRegisterEnum.MM.ordinal();
					break;
				case INT_ROW:
					addr = SfRegisterEnum.INT.ordinal();
					break;
				case IV_ROW:
					addr = SfRegisterEnum.IV.ordinal();
					break;
				case TR_ROW:
					addr = SfRegisterEnum.TR.ordinal();
					break;
				default:
					assert (row == PC_ROW);
					addr = INVALID;
			}
			if (addr != INVALID) {
				sfr.toggleWatchpoint(addr);
			}
		}
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// never used
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// never used
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// never used
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// never used
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.RefreshListener_I#refresh(int)
	 */
	@Override
	public void refresh(final int address) {
		tableModel.fireTableRowsUpdated(0, tableModel.getRowCount() - 1);
		segmentDisplay.repaint();
	}

	/**
	 * Called when the view is to be disposed.
	 */
	public void onDispose() {
		sfr.removeRefreshListener(this);
	}

}
