/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 */
package de.tu_darmstadt.rs.sim.gui.settings;

import de.tu_darmstadt.rs.sim.gui.MainWindow;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import javax.swing.JPanel;

/**
 * Container for all general setting panels.
 *
 */
public class GeneralSettingsPanel extends JPanel {

	private static final long serialVersionUID = -6687065819044579329L;
	private final ShiftingPanel shiftingPanel;
	private final PcAddressPanel pcAddressPanel;
	private final InterruptAddressPanel interruptAddressPanel;
	private final FontChooserPanel fontsPanel;
	private final SegmentationPanel segmentationPanel;

	/**
	 * create a new {@link GeneralSettingsPanel}
	 * @param mainWnd the window 
	 */
	public GeneralSettingsPanel(final MainWindow mainWnd) {
		setLayout(new BorderLayout());

		final JPanel leftPanel = new JPanel();
		leftPanel.setLayout(new GridLayout(4, 1));
		add(leftPanel, BorderLayout.CENTER);

		segmentationPanel = new SegmentationPanel();
		add(segmentationPanel, BorderLayout.EAST);


	    /* choice between default shifting or single shifting */

		shiftingPanel = new ShiftingPanel();
		leftPanel.add(shiftingPanel);

	    /* Addresses */

		pcAddressPanel = new PcAddressPanel(mainWnd);
		leftPanel.add(pcAddressPanel);
		interruptAddressPanel = new InterruptAddressPanel(mainWnd);
		leftPanel.add(interruptAddressPanel);


	    /* Fonts */
		fontsPanel = new FontChooserPanel(mainWnd);
		leftPanel.add(fontsPanel);
	}


	/**
	 * Call the commitChanges method from every child panel.
	 */
	public void commitChanges() {
		segmentationPanel.commitChanges();    // the address space settings
		// commit prior to PC & Interrupt addresses!
		pcAddressPanel.commitChanges();
		interruptAddressPanel.commitChanges();
		shiftingPanel.commitChanges();
		fontsPanel.commitChanges();
	}
}
