/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2011
 */
package de.tu_darmstadt.rs.sim.gui.settings;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.awt.FlowLayout;
import java.awt.Font;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

/**
 * Container for all global settings related with multicore simulation.
 *
 */
public class MulticoreSettingsPanel extends JPanel {

	private static final long serialVersionUID = 7006828564157832299L;
	private final JTextField portTextField;
	private final JTextField hostTextField;

	/**
	 * create a new {@link MulticoreSettingsPanel}
	 */
	public MulticoreSettingsPanel() {

		JPanel dispatcherPanel = new JPanel();
		dispatcherPanel.setBorder(new TitledBorder("Dispatcher Settings"));
		dispatcherPanel.setLayout(new FlowLayout());

		JLabel hostLabel = new JLabel("Host Name:");
		dispatcherPanel.add(hostLabel);
		hostTextField = new JTextField(30);
		hostTextField.setFont((Font) Settings
				.getSetting(Settings.REGISTER_FONT_PLAIN));
		hostTextField.setHorizontalAlignment(JTextField.CENTER);
		dispatcherPanel.add(hostTextField);

		JLabel portLabel = new JLabel("Port:");
		dispatcherPanel.add(portLabel);
		portTextField = new JTextField(10);
		portTextField.setFont((Font) Settings
				.getSetting(Settings.REGISTER_FONT_PLAIN));
		portTextField.setHorizontalAlignment(JTextField.CENTER);
		dispatcherPanel.add(portTextField);

		add(dispatcherPanel);

		portTextField.setText(String.valueOf(Settings
				.getSetting(Settings.SERVER_PORT)));
		hostTextField.setText(String.valueOf(Settings
				.getSetting(Settings.SERVER_HOST)));
	}

	/**
	 * commit all changes from every child
	 */
	public void commitChanges() {
		Settings.setSetting(Settings.SERVER_HOST, hostTextField.getText());
		try {
			Settings.setSetting(Settings.SERVER_PORT,
					Integer.parseInt(portTextField.getText()));
		} catch (NumberFormatException e) {
			MessageBox.showError(portTextField.getText()
					+ " is not a valid port number.");
		}
	}
}
