/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.settings;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.gui.GuiDefinitions_I;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

/**
 * Settings panel to define the de.tu_darmstadt.rs.sim.main memory's, the DMA memory's and the IO
 * address range.
 */
@SuppressWarnings("serial")
public class SegmentationPanel extends JPanel implements ItemListener,
		GuiDefinitions_I {

	private final JComboBox<String> comboUpperMemory;
	private final JComboBox<String> comboUpperDma;
	private final JComboBox<String> comboUpperIo;
	// private final JComboBox comboMemoryBlock;

	private final JLabel memoryLabel;
	private final JLabel dmaLabel;
	private final JLabel ioLabel;
	// private final JLabel memblkLabel;

	private final JLabel memoryAddressLabel;
	private final JLabel dmaAddressLabel;
	private final JLabel ioAddressLabel;
	// private final JLabel memblkAddressLabel;

	private final Rectangle mainRectangle;

	private int ioBlockCount;
	private int dmaBlockCount;

	private int memoryBlockCount;
	// private int memoryBlockFrameCount;

	private final static int BLOCK_SIZE = 0x400;
	private final static String NO_DMA_ITEM_STRING = "    --";

	private boolean listenerLock = false;

	/**
	 * Constructor.
	 */
	public SegmentationPanel() {

		setPreferredSize(new Dimension(290, 250));
		setBorder(new TitledBorder("Address space"));
		setLayout(null);

		mainRectangle = new Rectangle(205, 20, 70, 217);

		comboUpperIo = new JComboBox<String>();
		comboUpperIo.setBounds(110, 20, 80, 20);
		comboUpperIo.setFocusable(false);

		comboUpperDma = new JComboBox<String>();
		comboUpperDma.setBounds(110, 80, 80, 20);
		comboUpperDma.setFocusable(false);

		comboUpperMemory = new JComboBox<String>();
		comboUpperMemory.setBounds(110, 140, 80, 20);
		comboUpperMemory.setFocusable(false);
		/*
		 * comboMemoryBlock = new JComboBox(); comboMemoryBlock.setBounds(110,
		 * 200, 80, 20); comboMemoryBlock.setFocusable(false);
		 */
		final Font normalFont = (Font) Settings
				.getSetting(Settings.REGISTER_FONT_PLAIN);

		// initiate the blocks:
		final int minIOAddress = (Integer) Settings
				.getSetting(Settings.IO_ADDRESS_MIN);
		final int maxIOAddress = (Integer) Settings
				.getSetting(Settings.IO_ADDRESS_MAX);
		final int minDMAAddress = (Integer) Settings
				.getSetting(Settings.DMA_ADDRESS_MIN);
		final int maxDMAAddress = (Integer) Settings
				.getSetting(Settings.DMA_ADDRESS_MAX);
		final int maxAddress = (Integer) Settings
				.getSetting(Settings.MEMORY_ADDRESS_MAX);
		ioBlockCount = (maxIOAddress - minIOAddress + 1) / BLOCK_SIZE;

		if ((Boolean) Settings.getSetting(Settings.USING_DMA)) {
			dmaBlockCount = (maxDMAAddress - minDMAAddress + 1) / BLOCK_SIZE;
		} else {
			dmaBlockCount = 0;
		}

		memoryBlockCount = (maxAddress + 1) / BLOCK_SIZE;

		// memoryBlockFrameCount = Settings.PC_FRAME / BLOCK_SIZE; //
		// memoryBlockCount - 1 MaxMem - 1024

		adjustComboBoxes();

		// important: add listener after adjusting the combos
		comboUpperMemory.addItemListener(this);
		comboUpperDma.addItemListener(this);
		comboUpperIo.addItemListener(this);
		// comboMemoryBlock.addItemListener(this);

		add(comboUpperMemory);
		add(comboUpperDma);
		add(comboUpperIo);
		// add(comboMemoryBlock);

		ioLabel = new JLabel("IO size:");
		ioLabel.setBounds(10, 20, 100, 20);
		dmaLabel = new JLabel("DMA size:");
		dmaLabel.setBounds(10, 80, 100, 20);
		memoryLabel = new JLabel("Memory size:");
		memoryLabel.setBounds(10, 140, 100, 20);
		/*
		 * memblkLabel = new JLabel("Memory Blocks:"); memblkLabel.setBounds(10,
		 * 200, 100, 20);
		 */

		add(ioLabel);
		add(dmaLabel);
		add(memoryLabel);
		// add(memblkLabel);

		ioAddressLabel = new JLabel();
		ioAddressLabel.setBounds(10, 40, 150, 20);
		ioAddressLabel.setFont(normalFont);
		ioAddressLabel.setHorizontalAlignment(JLabel.CENTER);
		dmaAddressLabel = new JLabel();
		dmaAddressLabel.setBounds(10, 100, 150, 20);
		dmaAddressLabel.setFont(normalFont);
		dmaAddressLabel.setHorizontalAlignment(JLabel.CENTER);
		memoryAddressLabel = new JLabel();
		memoryAddressLabel.setBounds(10, 160, 150, 20);
		memoryAddressLabel.setFont(normalFont);
		memoryAddressLabel.setHorizontalAlignment(JLabel.CENTER);
		/*
		 * memblkAddressLabel = new JLabel(); memblkAddressLabel.setBounds(10,
		 * 220, 150, 20);
		 * memblkAddressLabel.setFont(Settings.REGISTER_FONT_PLAIN);
		 * memblkAddressLabel.setHorizontalAlignment(JLabel.CENTER);
		 */
		updateLabelTexts();

		add(ioAddressLabel);
		add(dmaAddressLabel);
		add(memoryAddressLabel);
		// add(memblkAddressLabel);
	}

	@Override
	public void paint(final Graphics g) {
		super.paint(g);

		g.setColor(IO_RANGE_COLOR);
		g.drawRect(5, 15, 195, 45);

		g.setColor(DMA_RANGE_COLOR);
		g.drawRect(5, 75, 195, 45);

		g.setColor(MEMORY_RANGE_COLOR);
		g.drawRect(5, 135, 195, 45);

		int totalBlockCount = memoryBlockCount + dmaBlockCount + ioBlockCount;
		int ioHeight = (int) ((float) (ioBlockCount * mainRectangle.height) / totalBlockCount);
		int dmaHeight = (int) ((float) (dmaBlockCount * mainRectangle.height) / totalBlockCount);

		// de.tu_darmstadt.rs.sim.main rectangle:
		g.setColor(Color.BLACK);
		g.drawRect(mainRectangle.x, mainRectangle.y, mainRectangle.width,
				mainRectangle.height);

		// separation lines:
		g.setColor(Color.BLACK);
		g.drawLine(mainRectangle.x, mainRectangle.y + 1 + ioHeight,
				mainRectangle.x + mainRectangle.width, mainRectangle.y + 1
						+ ioHeight);
		g.drawLine(mainRectangle.x, mainRectangle.y + 2 + ioHeight + dmaHeight,
				mainRectangle.x + mainRectangle.width, mainRectangle.y + 2
						+ ioHeight + dmaHeight);

		// IO rectangle:
		g.setColor(IO_RANGE_COLOR);
		g.fillRect(mainRectangle.x + 1, mainRectangle.y + 1,
				mainRectangle.width - 1, ioHeight);

		// DMA rectangle:
		g.setColor(DMA_RANGE_COLOR);
		g.fillRect(mainRectangle.x + 1, mainRectangle.y + 1 + ioHeight + 1,
				mainRectangle.width - 1, dmaHeight);

		// memory rectangle:
		g.setColor(MEMORY_RANGE_COLOR);
		g.fillRect(mainRectangle.x + 1, mainRectangle.y + 1 + ioHeight + 1
				+ dmaHeight + 1, mainRectangle.width - 1, mainRectangle.height
				- 3 - dmaHeight - ioHeight);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	@Override
	public void itemStateChanged(final ItemEvent event) {

		if (event.getStateChange() != ItemEvent.SELECTED) {
			return;
		}

		if (listenerLock) {
			return; // avoid recursive listener invokation
		}
		listenerLock = true;

		ioBlockCount = Integer.parseInt(((String) comboUpperIo
				.getSelectedItem()).replace(" K", ""));
		memoryBlockCount = Integer.parseInt(((String) comboUpperMemory
				.getSelectedItem()).replace(" K", ""));
		// memoryBlockFrameCount =
		// Integer.parseInt(((String)comboMemoryBlock.getSelectedItem()).replace(" K",
		// ""));

		String dmaText = (String) comboUpperDma.getSelectedItem();

		if (!dmaText.equals(NO_DMA_ITEM_STRING)) {
			dmaBlockCount = Integer.parseInt((dmaText).replace(" K", ""));
		} else {
			dmaBlockCount = 0;
		}

		assert ((memoryBlockCount + ioBlockCount + dmaBlockCount) <= 256);
		assert ((memoryBlockCount + ioBlockCount + dmaBlockCount) >= (1 + 0 + 1));

		// update combos, graphic & labels:
		adjustComboBoxes();
		repaint(mainRectangle);
		updateLabelTexts();

		listenerLock = false;
	}

	/**
	 * Updates the labels showing the first and last address of each address
	 * range segment.
	 */
	public void updateLabelTexts() {

		String str;
		/*
		 * str = String.format("%05x", BLOCK_SIZE * memoryBlockFrameCount);
		 * memblkAddressLabel.setText(str);
		 */
		str = String.format("%05x - 00000", BLOCK_SIZE * memoryBlockCount - 1);
		memoryAddressLabel.setText(str);

		if (dmaBlockCount != 0) {
			str = String.format("%05x - %05x", BLOCK_SIZE
					* (dmaBlockCount + memoryBlockCount) - 1, BLOCK_SIZE
					* memoryBlockCount);
			dmaAddressLabel.setText(str);

		} else {
			str = "No DMA.";
			dmaAddressLabel.setText(str);
		}

		str = String.format("%05x - %05x", BLOCK_SIZE
						* (ioBlockCount + dmaBlockCount + memoryBlockCount) - 1,
				BLOCK_SIZE * (dmaBlockCount + memoryBlockCount));
		ioAddressLabel.setText(str);
	}

	/**
	 * Refills the combo boxes' choosable items after the address range
	 * segmentation has been altered.
	 */
	private void adjustComboBoxes() {

		// comboMemoryBlock.removeAllItems();
		comboUpperMemory.removeAllItems();
		comboUpperDma.removeAllItems();
		comboUpperIo.removeAllItems();

		final int yetUnusedCount = 256 - memoryBlockCount - ioBlockCount
				- dmaBlockCount;
		assert (yetUnusedCount >= 0);
		/*
		 * for (int i = memoryBlockCount-1; i > 0; i--) {
		 * comboMemoryBlock.addItem(i + " K"); }
		 */
		for (int i = yetUnusedCount + memoryBlockCount; i > 0; i--) {
			comboUpperMemory.addItem(i + " K");
		}

		for (int i = yetUnusedCount + dmaBlockCount; i > 0; i--) {
			comboUpperDma.addItem(i + " K");
		}
		comboUpperDma.addItem(NO_DMA_ITEM_STRING);

		for (int i = yetUnusedCount + ioBlockCount; i > 0; i--) {
			comboUpperIo.addItem(i + " K");
		}

		comboUpperIo.setSelectedItem(ioBlockCount + " K");
		comboUpperMemory.setSelectedItem(memoryBlockCount + " K");
		// comboMemoryBlock.setSelectedItem(memoryBlockFrameCount + " K");

		if (dmaBlockCount > 0) {
			comboUpperDma.setSelectedItem(dmaBlockCount + " K");
		} else {
			comboUpperDma.setSelectedItem(NO_DMA_ITEM_STRING);
		}

	}

	/**
	 * Commits the changes (if any).
	 */
	public void commitChanges() {
		//nothing to do
	}
}
