/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2011
 *
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2011
 */
/*
  _______________________________________________________________________________

  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
  (http://www.mr.inf.tu-dresden.de) All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

  3. All advertising materials mentioning features or use of this software
  must display the following acknowledgement: "This product includes
  software developed by the TU Dresden Chair for Embedded Systems and
  its contributors."

  4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
  names of its contributors may be used to endorse or promote products
  derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  _______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit, Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006, 2011
 */
package de.tu_darmstadt.rs.sim.gui.settings;

import de.tu_darmstadt.rs.sim.gui.MainWindow;
import de.tu_darmstadt.rs.sim.utilities.ConvenientButton;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * The SettingsDialog is a modal dialog to let the user change the simulation's
 * global parameters.
 */
@SuppressWarnings("serial")
public class SettingsDialog extends JDialog implements ActionListener {

	/**
	 * constant for the return value of {@link #showModal()} indicating that the
	 * button OK was pressed
	 */
	public static final boolean OK_PRESSED = true;

	private final JPanel buttonPanel;
	private final ConvenientButton okButton;
	private final ConvenientButton cancelButton;
	private final JLabel labelHint;

	private final GeneralSettingsPanel generalPanel;
	private final MulticoreSettingsPanel multicorePanel;

	private boolean commitChanges; // true, if OK is pressed

	/**
	 * Constructor.
	 *
	 * @param mainWnd
	 *            the MainWindow
	 */
	public SettingsDialog(final MainWindow mainWnd) {

		super(mainWnd);
		setTitle("Settings");
		setLayout(new BorderLayout());
		setModal(true);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

		commitChanges = false;

		try {
			UIManager.put("swing.boldMetal", Boolean.FALSE);
			UIManager.setLookAndFeel(new MetalLookAndFeel());
		} catch (Exception e) { /* nothing */
		}

		// create tabbed panels
		final JTabbedPane tabbed = new JTabbedPane();
		tabbed.setFocusable(false);

		generalPanel = new GeneralSettingsPanel(mainWnd);
		tabbed.add(generalPanel, "General");

		multicorePanel = new MulticoreSettingsPanel();
		tabbed.add(multicorePanel, "Multicore");

		add(tabbed, BorderLayout.CENTER);

		/* And the always visible buttons */

		buttonPanel = new JPanel();
		okButton = new ConvenientButton("Apply",
				"Applies the new settings and restarts the simulation", this,
				buttonPanel);
		cancelButton = new ConvenientButton("Cancel",
				"Returns with no changes", this, buttonPanel);

		labelHint = new JLabel(
				"Note: Pressing Apply will restart the simulation.");
		labelHint.setHorizontalAlignment(JLabel.CENTER);
		buttonPanel.add(labelHint);
		add(buttonPanel, BorderLayout.SOUTH);

		// resize dialog
		pack();
		setLocationRelativeTo(null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(final ActionEvent event) {

		if (event.getActionCommand().equals(cancelButton.getText())) {
			setVisible(false);
			dispose();

		} else if (event.getActionCommand().equals(okButton.getText())) {
			generalPanel.commitChanges();
			multicorePanel.commitChanges();

			commitChanges = true;

			setVisible(false);
			dispose();
		}
	}

	/**
	 * For debugging only.
	 *
	 * @param args
	 *            unused
	 */
	public static void main(String[] args) {
		SettingsDialog dialog = new SettingsDialog(null);
		dialog.setVisible(true);
	}

	/**
	 * Starts and shows the modal dialog. The Method will not return until the
	 * user presses "Apply" or "Cancel".
	 *
	 * @return true, if "Apply" was pressed
	 */
	public boolean showModal() {

		setVisible(true);
		// thread will "stop" here until the modal window is closed
		return commitChanges;
	}

}
