/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.statistics;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.gui.MainWindow;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.GregorianCalendar;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

/**
 * Exports the (two) statistics into HTML and CSV files.
 */
public class StatisticsExporter implements StatisticsViewDefinitions_I {

	private final FirstStatisticsTableModel firstTableModel;
	private final SecondStatisticsTableModel secondTableModel;
	private final MainWindow mainWnd;


	/**
	 * Constructor.
	 * @param view the StatisticsView
	 */
	public StatisticsExporter(final StatisticsView view) {

		this.firstTableModel = view.firstTableModel;
		this.secondTableModel = view.secondTableModel;
		this.mainWnd = view.mainWnd;
	}

	/**
	 * Exports the statistics.
	 * @throws IOException if any error occures
	 */
	public void exportToFile() throws IOException {

		final JFileChooser fileChooser = new JFileChooser((File) Settings.getSetting(Settings.LAST_PROGRAM_DIR));
		fileChooser.setFileFilter(new FileFilter() {
			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return true;
				} else if (f.getPath().endsWith(HTML_ENDING)) {
					return true;
				} else {
					return false;
				}
			}

			@Override
			public String getDescription() {
				return "*" + HTML_ENDING + ", *" + CSV_ENDING;
			}
		});

		final int returnVal = fileChooser.showSaveDialog(mainWnd);


		if (returnVal == JFileChooser.APPROVE_OPTION) {

			final File file = fileChooser.getSelectedFile();

			String shortFileName = file.getName();
			if (shortFileName.endsWith(HTML_ENDING)) {
				shortFileName = shortFileName.substring(0, shortFileName.length() - HTML_ENDING.length());
			} else if (shortFileName.endsWith(CSV_ENDING)) {
				shortFileName = shortFileName.substring(0, shortFileName.length() - CSV_ENDING.length());
			}


			String longFileName = file.getPath();
			if (longFileName.endsWith(HTML_ENDING)) {
				longFileName = longFileName.substring(0, longFileName.length() - HTML_ENDING.length());
			} else if (longFileName.endsWith(CSV_ENDING)) {
				longFileName = longFileName.substring(0, longFileName.length() - CSV_ENDING.length());
			}

			String msg = "The following files will be created: \n";
			msg += String.format("   %s_1.html \n   %s_2.html \n   %s_1.csv \n   %s_2.csv \n",
					shortFileName, shortFileName, shortFileName, shortFileName);
			msg += "Note: Any existing files will be overwritten.";

			if (MessageBox.showConfirm(msg, "Exporting statistics")) {
				printFirstCSV(new File(longFileName + "_1" + CSV_ENDING));
				printSecondCSV(new File(longFileName + "_2" + CSV_ENDING));
				printFirstHTML(new File(longFileName + "_1" + HTML_ENDING));
				printSecondHTML(new File(longFileName + "_2" + HTML_ENDING));
			}
		}

	}


	/**
	 * Utility function to create the first HTML file.
	 * @param file a given file
	 * @throws IOException if any IO error occures
	 */
	private void printFirstHTML(final File file) throws IOException {

		final BufferedWriter writer = new BufferedWriter(new FileWriter(file));

		writer.write("<html> <body> <font face=\"Courier\">");
		writer.newLine();
		writer.write("<p> Spartan MC Simulator Statistics -- ");
		writer.write(new GregorianCalendar().getTime().toString() + " </p> <br>");
		writer.newLine();

		/********************************* FIRST TABLE ****************************************/

		writer.write(" <table cellspacing=\"0\" border=\"1\">");
		writer.newLine();
		for (int row = 0; row < ROW_COUNT_FIRST; row++) {

			writer.write(" <tr> <td width=\"110\" align=\"left\"> <b> ");
			writer.write(firstTableModel.getValueAt(row, NAME_COL_FIRST).toString());
			writer.write(" </b> </td> <td width=\"110\" align=\"right\"> ");

			final String value = firstTableModel.getValueAt(row, COUNT_COL_FIRST).toString();
			if (value.equals("0")) {
				writer.write(value);
			} else {
				writer.write("<b> " + value + " </b>");
			}

			writer.write(" </td> <td width=\"110\" align=\"right\"> ");

			final String percentage = firstTableModel.getValueAt(row, PERCENTAGE_COL_FIRST).toString();
			if (percentage.equals("0,00 %") || percentage.equals("0.00 %")) {
				writer.write(percentage);
			} else {
				writer.write("<b> " + percentage + " </b>");
			}

			writer.write(" </td> </tr> ");
			writer.newLine();
		}

		writer.write("</table> </font> </body> </html>");
		writer.newLine();

		writer.flush();
		writer.close();
	}


	/**
	 * Utility function to create the second HTML file.
	 * @param file a given file
	 * @throws IOException if any IO error occures
	 */
	private void printSecondHTML(final File file) throws IOException {

		final BufferedWriter writer = new BufferedWriter(new FileWriter(file));

		writer.write("<html> <body> <font face=\"Courier\">");
		writer.newLine();
		writer.write("<p> Spartan MC Simulator Statistics -- ");
		writer.write(new GregorianCalendar().getTime().toString() + " </p> <br>");
		writer.newLine();


		/********************************* SECOND TABLE ****************************************/

		writer.write(" <table width=\"" + 111 * ROW_COUNT_SECOND + "\" cellspacing=\"0\" border=\"1\">");
		writer.newLine();
		for (int row = 0; row < ROW_COUNT_SECOND; row++) {

			writer.write("<tr> ");

			for (int col = 0; col < ROW_COUNT_SECOND; col++) {

				final String value = secondTableModel.getValueAt(row, col).toString();

				writer.write("<td width=\"110\" align=\"center\"> ");
				if ((row == 0) || (col == 0) || (!value.equals("0"))) {
					writer.write("<b> ");
				}
				writer.write(value);
				if ((row == 0) || (col == 0) || (!value.equals("0"))) {
					writer.write(" </b> ");
				}
				writer.write("</td> ");
			}

			writer.write("</tr> ");
			writer.newLine();
		}

		writer.write("</table> </font> </body> </html>");
		writer.newLine();

		writer.flush();
		writer.close();
	}


	/**
	 * Utility function to create the first CSV file.
	 * @param file a given file
	 * @throws IOException if any IO error occures
	 */
	private void printFirstCSV(final File file) throws IOException {

		final BufferedWriter writer = new BufferedWriter(new FileWriter(file));

		/********************************* FIRST TABLE ****************************************/

		// Note: first row and last column are not printed here!
		for (int row = 1; row < ROW_COUNT_FIRST; row++) {
			writer.write(firstTableModel.getValueAt(row, NAME_COL_FIRST).toString());
			writer.write("; ");
			writer.write(firstTableModel.getValueAt(row, COUNT_COL_FIRST).toString());
			writer.newLine();
		}

		writer.flush();
		writer.close();
	}


	/**
	 * Utility function to create the second CSV file.
	 * @param file a given file
	 * @throws IOException if any IO error occures
	 */
	private void printSecondCSV(final File file) throws IOException {

		final BufferedWriter writer = new BufferedWriter(new FileWriter(file));

		/********************************* SECOND TABLE ****************************************/

		for (int row = 0; row < ROW_COUNT_SECOND; row++) {

			for (int col = 0; col < ROW_COUNT_SECOND - 1; col++) {
				writer.write(secondTableModel.getValueAt(row, col).toString());
				writer.write("; ");
			}
			writer.write(secondTableModel.getValueAt(row, ROW_COUNT_SECOND - 1).toString());
			writer.newLine();
		}

		writer.flush();
		writer.close();
	}


}
