/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */

package de.tu_darmstadt.rs.sim.gui.statistics;

import de.tu_darmstadt.rs.sim.engine.SettingListener_I;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.statistics.Statistics;
import de.tu_darmstadt.rs.sim.gui.MainWindow;
import de.tu_darmstadt.rs.sim.gui.SimpleView;
import de.tu_darmstadt.rs.sim.gui.controls.SignalButton;
import de.tu_darmstadt.rs.sim.model.StatisticsModel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.io.IOException;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * Displays the simulation's statistics with the option to export them.
 */
@SuppressWarnings("serial")
public class StatisticsView extends SimpleView implements
		StatisticsViewDefinitions_I, SettingListener_I {

	private final Statistics statistics;
	final MainWindow mainWnd;

	private final JTable firstTable;
	final FirstStatisticsTableModel firstTableModel;

	private final JTable secondTable;
	final SecondStatisticsTableModel secondTableModel;

	private final JTabbedPane tabbedPane;
	private final JPanel buttonPane;

	private final SignalButton buttonClear;
	private final SignalButton buttonExport;

	private final FirstStatisticsCellRenderer firstCellRenderer;
	private final SecondStatisticsCellRenderer secondCellRenderer;

	/**
	 * Constructor.
	 *
	 * @param simulationManager
	 *            the {@link SimulationManager}
	 * @param mainWnd
	 *            the {@link MainWindow}
	 */
	public StatisticsView(final SimulationManager simulationManager,
						  final MainWindow mainWnd) {
		super();
		this.statistics = simulationManager.getInstructionStats();
		this.mainWnd = mainWnd;

		setName("Statistics");

		firstTableModel = new FirstStatisticsTableModel();
		final DefaultTableColumnModel firstColumnModel = new DefaultTableColumnModel();

		firstColumnModel.addColumn(new TableColumn(NAME_COL_FIRST, 80));
		firstColumnModel.addColumn(new TableColumn(COUNT_COL_FIRST, 60));
		firstColumnModel.addColumn(new TableColumn(PERCENTAGE_COL_FIRST, 80));

		firstTable = new JTable(firstTableModel, firstColumnModel);
		firstCellRenderer = new FirstStatisticsCellRenderer();
		firstTable.setDefaultRenderer(Object.class, firstCellRenderer);
		firstTable.setOpaque(false);

		final DefaultTableColumnModel secondColumnModel = new DefaultTableColumnModel();

		for (int c = 0; c < COLUMN_COUNT_SECOND; c++) {
			secondColumnModel.addColumn(new TableColumn(c));
		}

		secondTableModel = new SecondStatisticsTableModel();
		secondTable = new JTable(secondTableModel, secondColumnModel);
		secondCellRenderer = new SecondStatisticsCellRenderer();
		secondTable.setDefaultRenderer(Object.class, secondCellRenderer);
		secondTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		secondTable.setShowGrid(true);
		secondTable.setOpaque(false);

		buttonPane = new JPanel();
		buttonPane.setLayout(new FlowLayout());

		buttonClear = new SignalButton("Clear", "Clears all statistics values");
		buttonClear.registerClickHandler(statistics, "restart");
		buttonExport = new SignalButton("Export",
				"Exports the current statistics into an html file");
		buttonExport.registerClickHandler(this, "export");
		buttonPane.add(buttonClear);
		buttonPane.add(buttonExport);

		setLayout(new BorderLayout());

		tabbedPane = new JTabbedPane();
		tabbedPane.add(new JScrollPane(firstTable));
		tabbedPane.add(new JScrollPane(secondTable));
		tabbedPane.setTitleAt(0, "Instructions count");
		tabbedPane.setTitleAt(1, "Contigous instructions");
		tabbedPane.setFocusable(false);

		add(buttonPane, BorderLayout.NORTH);
		add(tabbedPane, BorderLayout.CENTER);

		setPreferredSize(new Dimension(300, 300));

		statistics.registerDataHandlerMethod(this, "updateStatistics",
				StatisticsModel.class);
		Settings.registerListener(Settings.REGISTER_FONT_PLAIN, this);
		Settings.registerListener(Settings.REGISTER_FONT_BOLD, this);
		statistics.flush();
	}

	/**
	 * called when the export button is clicked
	 */
	public void export() {
		final StatisticsExporter exporter = new StatisticsExporter(this);
		try {
			exporter.exportToFile();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * update the de.tu_darmstadt.rs.sim.model for the statistics
	 *
	 * @param sm
	 *            data de.tu_darmstadt.rs.sim.model
	 */
	public void updateStatistics(final StatisticsModel sm) {
		firstTableModel.setStatisticsModel(sm);
		secondTableModel.setStatisticsModel(sm);
	}

	@Override
	public void onViewDispose() {
		statistics.unregisterDataHandlerMethod(this);
	}

	@Override
	public void onSettingChange(final int settingType, final Object newValue) {
		Font font = (Font) newValue;
		if (settingType == Settings.REGISTER_FONT_PLAIN) {
			firstCellRenderer.setContentPlainFont(font);
			secondCellRenderer.setContentPlainFont(font);
		} else if (settingType == Settings.REGISTER_FONT_BOLD) {
			firstCellRenderer.setContentBoldFont(font);
			secondCellRenderer.setContentBoldFont(font);
		}
	}
}
