/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.timer;

import de.tu_darmstadt.rs.sim.engine.RefreshListener_I;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * A Panel which shows the properties of a BasicTimer
 */
@SuppressWarnings("serial")
public class BasicTimerPropsPanel extends JPanel implements
		SpartanDefinitions_I, RefreshListener_I {

	private JLabel enabledLabel;
	private JLabel preEnabledLabel;
	private JLabel inputDividedByLabel;
	private JLabel preValueLabel;
	private ConnectionPanel connectionPanel;

	private int constraints_x;
	private int constraints_y;

	/**
	 * Constructor
	 *
	 * @param view view which should contain the new {@link BasicTimerPropsPanel}
	 * instance
	 */
	public BasicTimerPropsPanel(BasicTimerView view) {
		GridBagLayout layout = new GridBagLayout();
		setLayout(layout);
		JLabel nameLabel = new JLabel("Timer Enabled:");
		enabledLabel = new JLabel("false");
		GridBagConstraints constraints = getConstraints();
		layout.setConstraints(nameLabel, constraints);
		add(nameLabel);
		constraints = getConstraints();
		layout.setConstraints(enabledLabel, constraints);
		add(enabledLabel);
		nameLabel = new JLabel("Pre Enabled:");
		preEnabledLabel = new JLabel("false");
		constraints = getConstraints();
		layout.setConstraints(nameLabel, constraints);
		add(nameLabel);
		constraints = getConstraints();
		layout.setConstraints(preEnabledLabel, constraints);
		add(preEnabledLabel);
		inputDividedByLabel = new JLabel("Input Divided By:");
		preValueLabel = new JLabel("0");
		constraints = getConstraints();
		layout.setConstraints(inputDividedByLabel, constraints);
		add(inputDividedByLabel);
		constraints = getConstraints();
		layout.setConstraints(preValueLabel, constraints);
		add(preValueLabel);
		connectionPanel = new ConnectionPanel(view);
		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 4;
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.gridwidth = 2;
		constraints.gridheight = 2;
		constraints.fill = GridBagConstraints.BOTH;
		layout.setConstraints(connectionPanel, constraints);
		add(connectionPanel);
		refresh(0);
	}

	/**
	 * easy GridBagConstraints creation
	 *
	 * @return new constraints
	 */
	private GridBagConstraints getConstraints() {
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = constraints_x;
		constraints.gridy = constraints_y;
		constraints_x++;
		if (constraints_x > 1) {
			constraints_x = 0;
			constraints_y++;
		}
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.fill = GridBagConstraints.BOTH;
		return constraints;
	}

	/**
	 * show whether the timer is enabled
	 *
	 * @param timerEnabled the value
	 */
	public void setTimerEnabled(boolean timerEnabled) {
		enabledLabel.setText("" + timerEnabled);
	}

	/**
	 * show whether the timer's precounter is enabled
	 *
	 * @param preEnabled the value
	 */
	public void setPreEnabled(boolean preEnabled) {
		preEnabledLabel.setText("" + preEnabled);
		if (!preEnabled) {
			inputDividedByLabel.setEnabled(false);
			preValueLabel.setEnabled(false);
		} else {
			inputDividedByLabel.setEnabled(true);
			preValueLabel.setEnabled(true);
		}
	}

	/**
	 * put the precounter's divisor on the GUI
	 *
	 * @param preValue the divisor
	 */
	public void setPreValue(int preValue) {
		preValueLabel.setText("" + preValue);
	}

	@Override
	public void refresh(int address) {
		if (address != REFRESH_REGISTER_ONLY) {
			connectionPanel.refresh(address);
		}
	}
}
