/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.timer;

import de.tu_darmstadt.rs.sim.engine.RefreshListener_I;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerManager;

import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.List;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

/**
 * shows the input source of a timer
 */
@SuppressWarnings("serial")
public class ConnectionPanel extends JPanel implements ItemListener, RefreshListener_I {

	private TimerPropsPanel_I timerPropsPanel;
	private JComboBox<String> outputBitSelector;
	private JComboBox<String> inputSelector;

	/**
	 * create a new {@link ConnectionPanel}
	 *
	 * @param panel the owner of the new {@link ConnectionPanel} instance
	 */
	public ConnectionPanel(TimerPropsPanel_I panel) {
		timerPropsPanel = panel;
		setOpaque(true);
		setBorder(new TitledBorder("Input Source"));
		GridLayout layout = new GridLayout(2, 2);
		setLayout(layout);
		JLabel nameLabel;
		nameLabel = new JLabel("Input Source:");
		inputSelector = new JComboBox<String>();
		inputSelector.addItemListener(this);
		inputSelector.setFocusable(false);
		add(nameLabel);
		add(inputSelector);
		nameLabel = new JLabel("Source Bit:");
		outputBitSelector = new JComboBox<String>();
		addOutputBitPossibilities(17);
		outputBitSelector.addItemListener(this);
		outputBitSelector.setFocusable(false);
		add(nameLabel);
		add(outputBitSelector);

	}

	/**
	 * add the possibilities to the JComboBox
	 *
	 * @param numPossibilities the number of Possibilities
	 */
	private void addOutputBitPossibilities(int numPossibilities) {
		outputBitSelector.removeAllItems();
		for (int i = 0; i < numPossibilities; i++) {
			outputBitSelector.addItem("" + i);
		}
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		if (e.getSource().equals(inputSelector)) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				String itemText = (String) inputSelector.getSelectedItem();
				setTimerConnection(itemText);
			}
		} else {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				timerPropsPanel.getTimer().setInputFilterBit(outputBitSelector.getSelectedIndex());
			}
		}
	}

	/**
	 * set the timer connection
	 *
	 * @param sourceName the selected source
	 */
	public void setTimerConnection(String sourceName) {
		TimerManager.addTimerConnection(sourceName, timerPropsPanel.getTimer());
	}

	@Override
	public void refresh(int unusedValue) {
		outputBitSelector.removeItemListener(this);
		inputSelector.removeItemListener(this);
		inputSelector.removeAllItems();
		List<String> timerSources = TimerManager.getAllTimerNames();
		timerSources.remove(timerPropsPanel.getTimer().getUniqueTimerName());
		for (String timerSource : timerSources) {
			inputSelector.addItem(timerSource);
		}
		String timerSource = timerPropsPanel.getTimer().getTimerSourceString();
		setSelectedInput(timerSource);
		//int numberOfBits=TimerManager.getOutputBitsOfTimer(timerSource);
		//addOutputBitPossibilities(numberOfBits);
		setSelectedBit(timerPropsPanel.getTimer().getInputFilterBit());
		inputSelector.addItemListener(this);
		outputBitSelector.addItemListener(this);
	}

	private void setSelectedBit(int bit) {
		int numberOfItems = outputBitSelector.getItemCount();
		int theIndex = (bit >= numberOfItems)? (numberOfItems - 1) : bit;
		outputBitSelector.setSelectedIndex(theIndex);
	}

	/**
	 * show a change of the selected item
	 *
	 * @param item the selected item
	 */
	public void setSelectedInput(String item) {
		inputSelector.setSelectedItem(item);
	}


}
