/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.timer;

import de.tu_darmstadt.rs.sim.engine.RefreshListener_I;
import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * A Panel which shows the properties of a RTI
 */
@SuppressWarnings("serial")
public class RTIPropsPanel extends JPanel implements SpartanDefinitions_I, RefreshListener_I {

	private JLabel enabledLabel;
	private JLabel interruptLabel;
	private JLabel preLabel;
	private JLabel preValueLabel;
	private JLabel hasInterruptLabel;
	private ConnectionPanel connectionPanel;
	private int constraints_x;
	private int constraints_y;
	private Color grayColor;

	/**
	 * Constructor
	 *
	 * @param view view which should contain the new {@link RTIPropsPanel} instance
	 */
	public RTIPropsPanel(RTIView view) {
		GridBagLayout layout = new GridBagLayout();
		setLayout(layout);
		JLabel nameLabel = new JLabel("RTI Enabled:");
		enabledLabel = new JLabel("false");
		GridBagConstraints constraints = getConstraints();
		layout.setConstraints(nameLabel, constraints);
		add(nameLabel);
		constraints = getConstraints();
		layout.setConstraints(enabledLabel, constraints);
		add(enabledLabel);
		nameLabel = new JLabel("Interrupt Enabled:");
		interruptLabel = new JLabel("false");
		interruptLabel.setOpaque(true);
		constraints = getConstraints();
		layout.setConstraints(nameLabel, constraints);
		add(nameLabel);
		constraints = getConstraints();
		layout.setConstraints(interruptLabel, constraints);
		add(interruptLabel);
		nameLabel = new JLabel("Input Divided By:");
		preLabel = new JLabel("0");
		constraints = getConstraints();
		layout.setConstraints(nameLabel, constraints);
		add(nameLabel);
		constraints = getConstraints();
		layout.setConstraints(preLabel, constraints);
		add(preLabel);
		nameLabel = new JLabel("Precounter value:");
		preValueLabel = new JLabel("0");
		constraints = getConstraints();
		layout.setConstraints(nameLabel, constraints);
		add(nameLabel);
		constraints = getConstraints();
		layout.setConstraints(preValueLabel, constraints);
		add(preValueLabel);
		nameLabel = new JLabel("has Interrupt:");
		hasInterruptLabel = new JLabel("false");
		hasInterruptLabel.setOpaque(true);
		constraints = getConstraints();
		layout.setConstraints(nameLabel, constraints);
		add(nameLabel);
		constraints = getConstraints();
		layout.setConstraints(hasInterruptLabel, constraints);
		add(hasInterruptLabel);
		connectionPanel = new ConnectionPanel(view);
		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 6;
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.gridwidth = 2;
		constraints.gridheight = 2;
		constraints.fill = GridBagConstraints.BOTH;
		layout.setConstraints(connectionPanel, constraints);
		add(connectionPanel);
		grayColor = interruptLabel.getBackground();
		refresh(0);
	}

	/**
	 * easy GridBagConstraints creation
	 *
	 * @return new constraints
	 */
	private GridBagConstraints getConstraints() {
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = constraints_x;
		constraints.gridy = constraints_y;
		constraints_x++;
		if (constraints_x > 1) {
			constraints_x = 0;
			constraints_y++;
		}
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.fill = GridBagConstraints.BOTH;
		return constraints;
	}

	/**
	 * show whether the RTI is enabled
	 *
	 * @param rtiEnabled the value
	 */
	public void setRTIEnable(boolean rtiEnabled) {
		enabledLabel.setText("" + rtiEnabled);
	}

	/**
	 * show whether the interrupts are enabled
	 *
	 * @param interruptEnabled the value
	 */
	public void setInterruptEnable(boolean interruptEnabled) {
		interruptLabel.setText("" + interruptEnabled);
		Color color;
		if (interruptEnabled) {
			color = Color.GREEN;
		} else {
			color = grayColor;
		}
		interruptLabel.setBackground(color);
	}

	/**
	 * put the precounter's divisor on the GUI
	 *
	 * @param preValue the divisor
	 */
	public void setPreValue(int preValue) {
		preLabel.setText("" + preValue);
	}

	/**
	 * show whether an interrupt has been requested
	 *
	 * @param hasInterrupt true if an interrupt has been requested, false otherwise
	 */
	public void setHasInterrupt(boolean hasInterrupt) {
		hasInterruptLabel.setText("" + hasInterrupt);
		Color color;
		if (hasInterrupt) {
			color = Color.GREEN;
		} else {
			color = grayColor;
		}
		hasInterruptLabel.setBackground(color);
	}

	/**
	 * put the precounter's current count value on the GUI
	 *
	 * @param precounterValue the current count value
	 */
	public void setPrecounterValue(int precounterValue) {
		preValueLabel.setText("" + precounterValue);
	}

	@Override
	public void refresh(int address) {
		if (address != REFRESH_REGISTER_ONLY) {
			connectionPanel.refresh(address);
		}
	}
}
