/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package de.tu_darmstadt.rs.sim.gui.timer;

import de.tu_darmstadt.rs.sim.engine.peripheral.timer.AbstractTimer;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.RTI;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerManager;
import de.tu_darmstadt.rs.sim.gui.AbstractView;
import de.tu_darmstadt.rs.sim.gui.generic.BaseAddressPanel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.prefs.Preferences;
import javax.swing.border.EmptyBorder;

/**
 * GUI representation of the RTI
 */
@SuppressWarnings("serial")
public class RTIView extends AbstractView implements TimerPropsPanel_I {

	private final BaseAddressPanel baseAddressPanel;
	private final RTIPropsPanel rtiPropsPanel;
	private final GridBagLayout layout;

	private final RTI rti;

	/**
	 * create a new view
	 *
	 * @param rti the {@link RTI} which this view represents
	 */
	public RTIView(final RTI rti) {
		this.rti = rti;
		setName(rti.getUniqueTimerName());
		rti.addRefreshListener(this);
		layout = new GridBagLayout();
		setLayout(layout);
		baseAddressPanel = new BaseAddressPanel(rti);
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.weightx = 1;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		layout.setConstraints(baseAddressPanel, constraints);
		rtiPropsPanel = new RTIPropsPanel(this);
		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.fill = GridBagConstraints.BOTH;
		layout.setConstraints(rtiPropsPanel, constraints);
		rtiPropsPanel.setBorder(new EmptyBorder(5, 10, 5, 10));
		add(baseAddressPanel);
		add(rtiPropsPanel);
		rtiPropsPanel.refresh(0);
		TimerManager.addRefreshListener(rtiPropsPanel);
	}

	@Override
	public void loadConfiguration(Preferences prefs) {
		//nothing to do here
	}

	@Override
	public void onViewDispose() {
		rti.removeRefreshListener(this);
	}

	@Override
	public void storeConfiguration(Preferences prefs) {
		//nothing to do here
	}

	@Override
	public void refresh(int address) {
		baseAddressPanel.refresh();
		rtiPropsPanel.setRTIEnable(rti.getRTIEnable());
		rtiPropsPanel.setInterruptEnable(rti.getInterruptEnabled());
		rtiPropsPanel.setPreValue(rti.getPreDivider());
		rtiPropsPanel.setPrecounterValue(rti.getPrecounterValue());
		rtiPropsPanel.setHasInterrupt(rti.hasInterruptRequest());
		rtiPropsPanel.refresh(address);
	}

	@Override
	public AbstractTimer getTimer() {
		return rti;
	}

}
