/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.uart;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Panel to accomodate some buttons.
 */
@SuppressWarnings("serial")
public class UartButtonPanel extends JPanel implements ItemListener, ActionListener {


	private final UartView view;
	private final JComboBox<String> combo;
	private final JCheckBox suppressUartOutputCheckbox;


	/**
	 * Constructor.
	 * @param view the UartView
	 */
	public UartButtonPanel(final UartView view) {
		super();

		this.view = view;


		setLayout(new GridLayout(1, 3));

		final JPanel subPanel1 = new JPanel();
		suppressUartOutputCheckbox = new JCheckBox("Suppress UART output");
		suppressUartOutputCheckbox.addActionListener(this);
		subPanel1.add(new JLabel("Simulated UART speed:"));

		combo = new JComboBox<String>();
		combo.setToolTipText("Transmit / receive one byte each n simulation steps");
		for (int i = 1; i < 6; ++i) {
			combo.addItem("one byte every " + 1 * (int) Math.pow(10, i) + " clocks");
			combo.addItem("one byte every " + 2 * (int) Math.pow(10, i) + " clocks");
			combo.addItem("one byte every " + 3 * (int) Math.pow(10, i) + " clocks");
			combo.addItem("one byte every " + 4 * (int) Math.pow(10, i) + " clocks");
			combo.addItem("one byte every " + 5 * (int) Math.pow(10, i) + " clocks");
			combo.addItem("one byte every " + 6 * (int) Math.pow(10, i) + " clocks");
			combo.addItem("one byte every " + 7 * (int) Math.pow(10, i) + " clocks");
			combo.addItem("one byte every " + 8 * (int) Math.pow(10, i) + " clocks");
			combo.addItem("one byte every " + 9 * (int) Math.pow(10, i) + " clocks");
		}
		combo.addItemListener(this);
		combo.setFocusable(false);
		subPanel1.add(combo);

		subPanel1.add(suppressUartOutputCheckbox);
		combo.setSelectedItem("one byte every 200 clocks");

		add(subPanel1);
	}


	/* (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	@Override
	public void itemStateChanged(final ItemEvent event) {

		if (event.getStateChange() != ItemEvent.SELECTED) {
			return;
		}

		String selection = ((String) combo.getSelectedItem()).replace("one byte every ", "");
		selection = selection.replace(" clocks", "");

		final int counter = Integer.parseInt(selection);
		assert (counter >= 10);
		view.uart.setCounterMax(counter);
	}


	/**
	 * Refreshes the displayed values.
	 */
	public void refresh() {

		final int counter = view.uart.getCounterMax();
		combo.setSelectedItem("one byte every " + counter + " clocks");
	}

	@Override
	public void actionPerformed(final ActionEvent event) {

		if (event.getSource() == suppressUartOutputCheckbox) {
			view.setSuppressUartOutput(suppressUartOutputCheckbox.isSelected());
		}
	}


}
