/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.uart;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.peripheral.uart.AbstractUart;
import de.tu_darmstadt.rs.sim.engine.peripheral.uart.Uart;
import de.tu_darmstadt.rs.sim.utilities.Formatter;
import de.tu_darmstadt.rs.sim.utilities.ToolTipGuru;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * Panel to visualize the UART control register.
 */
@SuppressWarnings("serial")
public class UartControlSubPanel extends JPanel implements
		UartViewDefinitions_I {

	private final JTable table;
	private final AbstractTableModel tableModel;
	private final DefaultTableColumnModel columnModel;
	private final JLabel flagsLabel0;
	private final JLabel flagsLabel1;
	private final UartView view;

	/**
	 * Constructor.
	 *
	 * @param view
	 *            the UartView
	 */
	public UartControlSubPanel(final UartView view) {

		super();
		setBorder(new TitledBorder("Control register"));

		this.view = view;

		final JPanel subPanel0 = new JPanel();
		subPanel0.setLayout(new FlowLayout());
		final JPanel subPanel1 = new JPanel();
		subPanel1.setLayout(new GridLayout(2, 1));

		setLayout(new GridLayout(2, 1));
		add(subPanel0);
		add(subPanel1);

		// the table:
		columnModel = new DefaultTableColumnModel();

		final TableColumn col1 = new TableColumn(HEX_COL);
		col1.setMinWidth(100);
		col1.setMaxWidth(100);
		columnModel.addColumn(col1);

		final TableColumn col2 = new TableColumn(BIN_COL);
		col2.setMinWidth(190);
		col2.setMaxWidth(190);
		columnModel.addColumn(col2);

		tableModel = new AbstractTableModel() {
			@Override
			public int getRowCount() {
				return 2;
			}

			@Override
			public int getColumnCount() {
				return 2;
			}

			@Override
			public Object getValueAt(final int row, final int col) {

				switch (row) {
					case NAME_ROW:
						if (col == HEX_COL) {
							return "Value hex";
						}
						return "Value bin";
					default:
						if (col == HEX_COL) {
							return String.format("%05x",
									view.uart.getControlIoValue());
						}
						return Formatter.intToBinaryString(
								view.uart.getControlIoValue(), true);
				}
			}

			@Override
			public boolean isCellEditable(final int row, final int col) {
				return (row != NAME_ROW);
			}

			@Override
			public void setValueAt(final Object aValue, final int row,
								   final int col) {
				assert (row != NAME_ROW);

				final int value;
				switch (col) {
					case HEX_COL:
						value = Formatter.parseHexString((String) aValue,
								BITMASK_18);
						if (value != INVALID) {
							view.uart.setControlIoValue(value);
						}
						break;
					default:
						value = Formatter.parseBinString((String) aValue,
								BITMASK_18);
						if (value != INVALID) {
							view.uart.setControlIoValue(value);
						}
						break;
				}
			}
		};

		table = new JTable(tableModel, columnModel);
		table.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() {
			@Override
			public Component getTableCellRendererComponent(final JTable t,
														   final Object value, final boolean isSelected,
														   final boolean hasFocus, final int row, final int col) {

				final Component component = super
						.getTableCellRendererComponent(t, value,
								false /* isSelected */, (row != NAME_ROW)
										& hasFocus, row, col);
				final JLabel label = (JLabel) component;
				label.setHorizontalAlignment(JLabel.CENTER);
				label.setVerticalAlignment(JLabel.BOTTOM);
				label.setForeground(Color.BLACK);
				label.setFont((Font) Settings
						.getSetting(Settings.REGISTER_FONT_PLAIN));
				switch (row) {
					case NAME_ROW:
						label.setBackground(LIGHT_GRAY_COLOR);
						label.setToolTipText(null);
						break;
					default:
						label.setBackground(Color.WHITE);
						if (col == HEX_COL) {
							label.setToolTipText(ToolTipGuru.create((String) value,
									true));
						} else {
							label.setToolTipText(null);
						}
						break;
				}

				return label;
			}
		});

		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setDragEnabled(false);
		table.getTableHeader().setReorderingAllowed(false);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
		subPanel0.add(table);

		flagsLabel0 = new JLabel();
		flagsLabel0.setForeground(Color.BLUE);
		flagsLabel1 = new JLabel();
		flagsLabel1.setForeground(Color.BLUE);

		final JPanel sub0 = new JPanel();
		sub0.setLayout(new FlowLayout());
		sub0.add(flagsLabel0);
		subPanel1.add(sub0);

		final JPanel sub1 = new JPanel();
		sub1.setLayout(new FlowLayout());
		sub1.add(flagsLabel1);
		subPanel1.add(sub1);

		refresh(); // necessary to let the layout be arranged neatly
	}

	/**
	 * Refreshes the displayed values.
	 */
	void refresh() {

		tableModel.fireTableDataChanged();

		String flags = "";
		final int control = view.uart.getControlIoValue();

		if ((control & Uart.CONTROL_RX_ENABLE) != 0) {
			flags += "   Rx enabled   ";
		} else {
			flags += "   Rx disabled  ";
		}

		if ((control & Uart.CONTROL_TX_ENABLE) != 0) {
			flags += "   Tx enabled   ";
		} else {
			flags += "   Tx disabled  ";
		}

		if ((control & Uart.CONTROL_RX_IRQ_ENABLE) != 0) {
			flags += "   Rx IRQ enabled   ";
		} else {
			flags += "   Rx IRQ disabled  ";
		}

		if ((control & Uart.CONTROL_TX_IRQ_ENABLE) != 0) {
			flags += "   Tx IRQ enabled   ";
		} else {
			flags += "   Tx IRQ disabled  ";
		}

		flagsLabel0.setText(flags);

		flags = "";
		if ((control & Uart.CONTROL_PARITY_ENABLE) == 0) {
			flags += "   No parity   ";
		} else if ((control & Uart.CONTROL_PARITY_EVEN) != 0) {
			flags += "   Parity even ";
		} else {
			flags += "   Parity odd  ";
		}

		if ((control & Uart.CONTROL_TWO_STOP_BITS) != 0) {
			flags += "   2 stop bits  ";
		} else {
			flags += "   1 stop bit   ";
		}

		switch (control & Uart.CONTROL_DATA_LEN_MASK) {
			case Uart.CONTROL_DATA_LENGTH_8:
				flags += "   8 data bits  ";
				break;
			case Uart.CONTROL_DATA_LENGTH_7:
				flags += "   7 data bits  ";
				break;
			case Uart.CONTROL_DATA_LENGTH_6:
				flags += "   6 data bits  ";
				break;
			case Uart.CONTROL_DATA_LENGTH_5:
				flags += "   5 data bits  ";
				break;
			default:
				flags += "   8 data bits  ";
		}

		switch (control & AbstractUart.CONTROL_BAUDRATE_MASK) {
			case AbstractUart.CONTROL_BAUDRATE_115200:
				flags += "   115200 baud ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_57600:
				flags += "   57600 baud  ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_38400:
				flags += "   38400 baud  ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_31250:
				flags += "   31250 baud  ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_19200:
				flags += "   19200 baud  ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_9600:
				flags += "   9600 baud   ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_4800:
				flags += "   4800 baud   ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_2400:
				flags += "   2400 baud   ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_1200:
				flags += "   1200 baud   ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_600:
				flags += "   600 baud    ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_300:
				flags += "   300 baud    ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_150:
				flags += "   150 baud    ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_75:
				flags += "   75 baud     ";
				break;
			case AbstractUart.CONTROL_BAUDRATE_50:
				flags += "   50 baud     ";
				break;
			default:
				flags += "   7812 baud   ";
		}

		flagsLabel1.setText(flags);
	}

}
