/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.uart;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.utilities.Formatter;
import de.tu_darmstadt.rs.sim.utilities.ToolTipGuru;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * Panel to visualize the UART Tx FIFO register.
 */
@SuppressWarnings("serial")
public class UartFifoTxSubPanel extends JPanel implements UartViewDefinitions_I {

	private final JTable table;
	private final AbstractTableModel tableModel;
	private final DefaultTableColumnModel columnModel;

	/**
	 * Constructor.
	 *
	 * @param view
	 *            the UartView
	 */
	public UartFifoTxSubPanel(final UartView view) {

		super();
		setLayout(new FlowLayout(FlowLayout.CENTER, FlowLayout.CENTER, 5));
		setBorder(new TitledBorder("FIFO Tx registers"));

		columnModel = new DefaultTableColumnModel();

		for (int i = 0; i < 9; ++i) {
			final TableColumn col = new TableColumn(i);
			col.setMinWidth(40);
			col.setMaxWidth(40);
			columnModel.addColumn(col);
		}

		tableModel = new AbstractTableModel() {
			@Override
			public int getRowCount() {
				return 2;
			}

			@Override
			public int getColumnCount() {
				return 9;
			}

			@Override
			public Object getValueAt(final int row, final int col) {

				if (col != SHIFT_REG_COL) {
					switch (row) {
						case NAME_ROW:
							return 7 - col;
						default:
							return String.format("%02x",
									view.uart.getFifoTxValueAt(7 - col));
					}
				}
				switch (row) {
					case NAME_ROW:
						return ">>";
					default:
						final int val = view.uart.getTxShiftRegister();
						if (val != INVALID) {
							return String.format("%02x", val);
						}
						return "--";
				}
			}

			@Override
			public boolean isCellEditable(final int row, final int col) {
				if (row == NAME_ROW) {
					return false;
				}
				return (col != SHIFT_REG_COL);
			}

			@Override
			public void setValueAt(final Object aValue, final int row,
								   final int col) {
				assert (row != NAME_ROW);
				assert (col != SHIFT_REG_COL);

				final int value = Formatter.parseHexString((String) aValue,
						BITMASK_8);
				if (value != INVALID) {
					view.uart.setFifoTxValueAt(7 - col, value);
				}

			}
		};

		table = new JTable(tableModel, columnModel);
		table.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() {
			@Override
			public Component getTableCellRendererComponent(final JTable t,
														   final Object value, final boolean isSelected,
														   final boolean hasFocus, final int row, final int col) {

				final Component component = super
						.getTableCellRendererComponent(t, value,
								false /* isSelected */, (row != NAME_ROW)
										& hasFocus, row, col);
				final JLabel label = (JLabel) component;
				label.setHorizontalAlignment(JLabel.CENTER);
				label.setVerticalAlignment(JLabel.BOTTOM);
				label.setForeground(Color.BLACK);
				label.setFont((Font) Settings
						.getSetting(Settings.REGISTER_FONT_PLAIN));
				switch (row) {
					case NAME_ROW:
						label.setBackground(LIGHT_GRAY_COLOR);
						label.setToolTipText(null);
						break;
					default:
						label.setBackground(Color.WHITE);
						label.setToolTipText(ToolTipGuru.create((String) value,
								false));

						if ((col == 7 - view.uart.getFifoTxByteIndex())
								&& (col != SHIFT_REG_COL)) {
							label.setFont((Font) Settings
									.getSetting(Settings.REGISTER_FONT_BOLD));

						} else if ((col == SHIFT_REG_COL)
								&& (!label.getText().equals("--"))) {
							label.setFont((Font) Settings
									.getSetting(Settings.REGISTER_FONT_BOLD));
						}
						break;
				}

				return label;
			}
		});

		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setDragEnabled(false);
		table.getTableHeader().setReorderingAllowed(false);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);

		add(table);
	}

	/**
	 * Refreshes the displayed values.
	 */
	public void refresh() {
		tableModel.fireTableDataChanged();
	}
}
