/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.uart;

import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.peripheral.uart.UartStream_I;
import de.tu_darmstadt.rs.sim.utilities.ConvenientButton;
import de.tu_darmstadt.rs.sim.utilities.MessageBox;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

/**
 * Panel to visualize two de.tu_darmstadt.rs.sim.console fields for the UART input/output.
 */
@SuppressWarnings("serial")
public class UartStreamPanel extends JPanel implements ActionListener,
		UartViewDefinitions_I, UartStream_I {

	private class TextUpdater implements Runnable {

		private final StringBuilder fBuffer = new StringBuilder();
		private JTextArea fText;

		public TextUpdater(JTextArea text) {
			fText = text;
		}

		public synchronized void appendChar(char c) {
			fBuffer.append(c);
			if (fBuffer.length() == 1) {
				SwingUtilities.invokeLater(this);
			}
		}

		@Override
		public synchronized void run() {
			fText.append(fBuffer.toString());
			fBuffer.setLength(0);
		}
	}


	final JTextArea textStreamIn;
	int streamInCharIndex;

	final JTextArea textStreamOut;
	private final JScrollPane scrollStreamOut;
	private final TextUpdater textOutUpdater;

	final JComboBox<String> newlineCombo;
	final JCheckBox enableStdoutCheckbox;
	private final ConvenientButton clearButton;
	private final ConvenientButton sendTextButton;
	final ConvenientButton sendBinButton;

	private final UartView view;

	/**
	 * Constructor.
	 *
	 * @param view
	 *            the UartView
	 */
	public UartStreamPanel(final UartView view) {

		super();
		this.view = view;

		setLayout(new BorderLayout());

		final JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new GridLayout(2, 1));
		add(centerPanel, BorderLayout.CENTER);

		final JPanel northPanel = new JPanel();
		northPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		add(northPanel, BorderLayout.NORTH);

		newlineCombo = new JComboBox<String>();
		newlineCombo.addItem("Newline character 0xA");
		newlineCombo.addItem("Newline character 0xD");
		newlineCombo.addItem("Newline character 0xD 0xA");
		newlineCombo.setFocusable(false);

		northPanel.add(newlineCombo);

		clearButton = new ConvenientButton("Clear streams",
				"Clears the content of the UART in/out streams.", this,
				northPanel);

		enableStdoutCheckbox = new JCheckBox("Write to stdout");
		northPanel.add(enableStdoutCheckbox);
		enableStdoutCheckbox.addActionListener(this);

		sendTextButton = new ConvenientButton("Send text file",
				"Transmits a complete file (text mode).", this, northPanel);

		sendBinButton = new ConvenientButton("Send binary file",
				"Transmits a complete file (binary mode).", this, northPanel);

		streamInCharIndex = 0;

		textStreamIn = new JTextArea() {
			@Override
			protected void processKeyEvent(final KeyEvent event) {

				if (getSelectedText() != null) {
					return; // do not allow to select any text
				}
				if (getCaretPosition() != getText().length()) {
					return; // do not allow to change the caret's position
				}

				if (event.getKeyCode() == KeyEvent.VK_DELETE) {
					return;
				}
				if (event.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
					return;
				}
				if (event.getKeyCode() == KeyEvent.VK_UP) {
					return;
				}
				if (event.getKeyCode() == KeyEvent.VK_LEFT) {
					return;
				}
				if (event.getKeyCode() == KeyEvent.VK_PAGE_UP) {
					return;
				}

				super.processKeyEvent(event);

			}
		};

		textStreamIn.setBorder(new EtchedBorder());

		final JPanel subPanel1 = new JPanel();
		subPanel1.setLayout(new FlowLayout());

		final JPanel subPanel0 = new JPanel();
		subPanel0.setLayout(new BorderLayout());
		subPanel0.setBorder(new TitledBorder("Input stream"));
		subPanel0.add(new JScrollPane(textStreamIn), BorderLayout.CENTER);
		centerPanel.add(subPanel0);

		textStreamOut = new JTextArea() {

			@Override
			public void append(final String str) {
				super.append(str);
				setCaretPosition(getText().length()); // == auto scrolling
			}

			@Override
			protected void processKeyEvent(final KeyEvent event) {

				textStreamIn.dispatchEvent(event); // pipe it to the input text
				// field
			}
		};

		textStreamOut.setBorder(new EtchedBorder());
		textStreamOut.setEditable(false);

		scrollStreamOut = new JScrollPane(textStreamOut);

		final JPanel subPanel2 = new JPanel();
		subPanel2.setLayout(new FlowLayout());

		final JPanel subPanel3 = new JPanel();
		subPanel3.setLayout(new BorderLayout());
		subPanel3.setBorder(new TitledBorder("Output stream"));
		subPanel3.add(scrollStreamOut, BorderLayout.CENTER);
		centerPanel.add(subPanel3);

		textOutUpdater = new TextUpdater(textStreamOut);

	}

	/**
	 * Refreshes the displayed values.
	 */
	public void refresh() {
		textStreamIn.setFont((Font) Settings
				.getSetting(Settings.REGISTER_FONT_PLAIN));
		textStreamOut.setFont((Font) Settings
				.getSetting(Settings.REGISTER_FONT_PLAIN));
	}

	private File chooseFile() {

		final JFileChooser fileChooser = new JFileChooser(
				(File) Settings.getSetting(Settings.LAST_PROGRAM_DIR));
		final int returnVal = fileChooser.showOpenDialog(view.mainWnd);
		final File f;

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			f = fileChooser.getSelectedFile();
			Settings.setSetting(Settings.LAST_PROJECT_DIR, f.getParent());
			return f;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(final ActionEvent event) {

		if (event.getActionCommand().equals(clearButton.getText())) {
			textStreamIn.setText("");
			textStreamOut.setText("");
			streamInCharIndex = 0;

		} else if (event.getSource() == sendTextButton) {

			final File file = chooseFile();

			if (file != null) {
				try {
					final BufferedReader reader = new BufferedReader(
							new FileReader(file));

					byte b[] = {0x0a};
					String line;
					String content = "";
					for (; (line = reader.readLine()) != null; ) {
						content += line + new String(b, "US-ASCII");
					}

					textStreamIn.append(content + new String(b, "US-ASCII"));

					reader.close();

				} catch (IOException e) {
					MessageBox.showError(e.getMessage(), "IO Exception");
					return;
				}
			}

		} else if (event.getSource() == sendBinButton) {
			if (!view.stopBinaryTransfer()) {
				final File file = chooseFile();
				view.startBinaryFileTransfer(file);
			}

		} else if (event.getSource() == enableStdoutCheckbox) {

			view.setWriteToStdout(enableStdoutCheckbox.isSelected());

		}
	}

	private boolean nextByteIs0A = false;

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.periphery.UartStream_I#onReceiveNextByte()
	 */
	@Override
	public byte onReceiveNextByte() {

		if (nextByteIs0A) {
			nextByteIs0A = false;
			streamInCharIndex++;
			return 0x0a;
		}

		final String txt = textStreamIn.getText();
		assert (txt.length() > 0);

		char ch = txt.charAt(streamInCharIndex);

		if ((ch == 0xa)
				&& (newlineCombo.getSelectedIndex() == NEWLINE_0D0A_SELECTION)) {
			// this is a NEWLINE, displayed by a "0A",
			// we have to convert the "0A" into "0D 0A"

			nextByteIs0A = true;
			ch = 0xd;
			// Note: do NOT increment the streamInCharIndex

		} else if ((ch == 0xa)
				&& (newlineCombo.getSelectedIndex() == NEWLINE_0D_SELECTION)) {
			// this is a NEWLINE, displayed by a "0A",
			// we have to convert the "0A" into "0D"

			assert (nextByteIs0A == false);
			ch = 0xd;
			streamInCharIndex++;

		} else {
			streamInCharIndex++; // do not forget
		}

		return (byte) ch;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.periphery.UartStream_I#hasNextByte()
	 */
	@Override
	public boolean hasNextByte() {
		return (textStreamIn.getText().length() - 1 >= streamInCharIndex);
	}

	private boolean ignoreNext0A = false;

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.periphery.UartStream_I#onTransmitNextByte(byte)
	 */
	@Override
	public void onTransmitNextByte(final byte nextByte) {
		byte theByte = nextByte;

		if (view.isSuppressUartOutput()) {
			return;
		}

		if (ignoreNext0A) {
			ignoreNext0A = false;
			if (theByte == 0x0a) {
				return;
			}
		}

		if (theByte == 0x0d) {
			// convert to 0A for the JTextArea
			theByte = 0x0a;

			// but remember, in case the next byte is a 0A itself (i.e. avoid
			// double newlines)
			ignoreNext0A = true;
		}

		textOutUpdater.appendChar((char) theByte);

	}
}
