/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.gui.uart;

import de.tu_darmstadt.rs.sim.engine.peripheral.uart.AbstractUart;
import de.tu_darmstadt.rs.sim.engine.peripheral.uart.Uart;
import de.tu_darmstadt.rs.sim.gui.AbstractView;
import de.tu_darmstadt.rs.sim.gui.MainWindow;

import java.awt.BorderLayout;
import java.io.File;
import java.util.prefs.Preferences;
import javax.swing.JTabbedPane;

/**
 * The UartView visualizes the UART device with its registers and its IO-stream
 * as a de.tu_darmstadt.rs.sim.console.
 */
@SuppressWarnings("serial")
public class UartView extends AbstractView implements UartViewDefinitions_I {

	final UartRegisterPanel registerPanel;
	final UartStreamPanel streamPanel;
	final UartButtonPanel buttonPanel;
	private UartBinaryFileTransfer binaryTransfer;
	private boolean suppressUartOutput;

	AbstractUart uart;
	final MainWindow mainWnd;

	/**
	 * Constructor.
	 *
	 * @param uart
	 *            the UART
	 * @param mainWnd
	 *            the MainWindow
	 */
	public UartView(final AbstractUart uart, final MainWindow mainWnd) {

		super();
		this.mainWnd = mainWnd;

		this.uart = uart;
		streamPanel = new UartStreamPanel(this);

		setLayout(new BorderLayout());

		setName(uart.getPeriName());

		final JTabbedPane tabbed = new JTabbedPane();
		tabbed.setFocusable(false);

		registerPanel = new UartRegisterPanel(this);
		tabbed.add(registerPanel, "Registers");

		uart.setInputSource(streamPanel);
		uart.addOutputDestination(streamPanel);

		tabbed.add(streamPanel, "Data streams");

		buttonPanel = new UartButtonPanel(this);

		add(tabbed, BorderLayout.CENTER);
		add(buttonPanel, BorderLayout.SOUTH);

		uart.addRefreshListener(this);

	}

	@Override
	public void storeConfiguration(final Preferences prefs) {

		assert (prefs != null);
		prefs.putInt("counterMax", uart.getCounterMax());
		// prefs.putInt("index", uart.getIndex());
		prefs.putInt("newline", streamPanel.newlineCombo.getSelectedIndex());
		prefs.putBoolean("stdout",
				streamPanel.enableStdoutCheckbox.isSelected());
	}

	@Override
	public void loadConfiguration(Preferences prefs) {
		if (prefs != null) {

			// restore the old de.tu_darmstadt.rs.sim.preferences:
			setWriteToStdout(prefs.getBoolean("stdout", false));

			streamPanel.newlineCombo.setSelectedIndex(prefs
					.getInt("newline", 1));

		} else {
			// brand new view:
			streamPanel.newlineCombo.setSelectedIndex(1);
		}
	}

	@Override
	public void onViewDispose() {
		uart.removeRefreshListener(this);
		uart.removeInputSource();
		uart.removeOutputDestination(streamPanel);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see engine.RefreshListener_I#refresh(int)
	 */
	@Override
	public void refresh(final int address) {
		registerPanel.refresh();
		buttonPanel.refresh();
		streamPanel.refresh();
	}

	/**
	 * Sets whether uart output is written to stdout.
	 *
	 * @param value
	 *            true means write to stdout
	 */
	public void setWriteToStdout(boolean value) {
		uart.setWriteToStdOut(value);
		streamPanel.enableStdoutCheckbox.setSelected(value);
	}

	/**
	 * start reading the selected file and use its data as an input stream for
	 * the {@link Uart}
	 *
	 * @param f
	 *            the file to read data from
	 */
	public void startBinaryFileTransfer(File f) {
		if (binaryTransfer != null) {
			stopBinaryTransfer();
		}
		binaryTransfer = new UartBinaryFileTransfer(this, uart, f);
		uart.setInputSource(binaryTransfer);
		streamPanel.sendBinButton.setText("Abort transfer");
	}

	/**
	 * stop using data from file
	 *
	 * @return true if a binary file transfer has been started previously, false
	 *         otherwise
	 */
	public boolean stopBinaryTransfer() {
		boolean result = (binaryTransfer != null);
		uart.setInputSource(streamPanel);
		binaryTransfer = null;
		streamPanel.sendBinButton.setText("Send binary file");
		return result;
	}

	/**
	 * suppress the output in GUI if set to true
	 *
	 * @param value
	 *            true if the output should be suppressed, false otherwise
	 */
	public void setSuppressUartOutput(boolean value) {
		suppressUartOutput = value;
	}

	/**
	 * check if the output in GUI is suppressed
	 *
	 * @return true if the output is suppressed, false otherwise
	 */
	public boolean isSuppressUartOutput() {
		return suppressUartOutput;
	}
}
