/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
package de.tu_darmstadt.rs.sim.preferences;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import java.util.prefs.Preferences;

/**
 * Reader object for the preference XML data
 */
public class PreferencesXmlHandler implements ContentHandler {

	private final ProjectPreferences root;

	private Preferences currentPref;

	/**
	 * create a new {@link PreferencesXmlHandler}
	 *
	 * @param root
	 *            the top level project preference containing all other
	 *            de.tu_darmstadt.rs.sim.preferences
	 */
	public PreferencesXmlHandler(final ProjectPreferences root) {
		this.root = root;
		currentPref = null;
	}

	/**
	 * retrieve the value of the attribute with the given name. If attribute
	 * does not exist a SAXException is thrown.
	 *
	 * @param attributes
	 * @param name
	 * @return the value of the attribute with the given name.
	 * @throws SAXException
	 */
	private String getAttributeValue(Attributes attributes, String name)
			throws SAXException {
		String value = attributes.getValue(name);
		if (value == null) {
			throw new SAXException("malformed xml file");
		}

		return value;
	}

	/**
	 * check if currentPref is not null
	 *
	 * @throws SAXException
	 *             when currentPref == null
	 */
	private void validCurrentPref() throws SAXException {
		if (currentPref == null) {
			throw new SAXException("malformed xml file");
		}
	}

	@Override
	public void startElement(String uri, String localName, String qName,
							 Attributes attributes) throws SAXException {

		if (localName.equals("root")) {
			String type = getAttributeValue(attributes, "type");

			if (type.equals("user")) {
				currentPref = root;
			}

		} else {
			if (localName.equals("node")) {
				validCurrentPref();

				String name = getAttributeValue(attributes, "name");

				currentPref = currentPref.node(name);

			} else if (localName.equals("entry")) {
				validCurrentPref();

				String key = getAttributeValue(attributes, "key");
				String value = getAttributeValue(attributes, "value");

				currentPref.put(key, value);
			}
		}

	}

	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {

		if (localName.equals("root")) {
			currentPref = null;

		} else {
			if (currentPref != null) {
				if (localName.equals("node")) {
					currentPref = currentPref.parent();
				}
			}
		}

	}

	@Override
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		// unused
	}

	@Override
	public void endDocument() throws SAXException {
		// unused
	}

	@Override
	public void endPrefixMapping(String prefix) throws SAXException {
		// unused
	}

	@Override
	public void ignorableWhitespace(char[] ch, int start, int length)
			throws SAXException {
		// unused
	}

	@Override
	public void processingInstruction(String target, String data)
			throws SAXException {
		// unused
	}

	@Override
	public void setDocumentLocator(Locator locator) {
		// unused
	}

	@Override
	public void skippedEntity(String name) throws SAXException {
		// unused
	}

	@Override
	public void startDocument() throws SAXException {
		// unused
	}

	@Override
	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {
		// unused
	}
}
