/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

package de.tu_darmstadt.rs.sim.signal;

/**
 * Class for signaling a modified event
 */
public class SignalModified extends Signal {

	/**
	 * de.tu_darmstadt.rs.sim.signal for flush
	 */
	private final Signal0 sigFlush;
	private boolean fArmed;

	/**
	 * create a 'modified' de.tu_darmstadt.rs.sim.signal which can be emitted later
	 */
	public SignalModified() {
		sigFlush = new Signal0();
		sigFlush.registerMethodCallOnEmit(this, "reset");
		fArmed = true;
	}

	/**
	 * fires the de.tu_darmstadt.rs.sim.signal (which calls connected methods)
	 */
	public void emit() {
		if (!fArmed) {
			return;
		}
		fArmed = false;
		emitInternal(sigFlush);
	}

	/**
	 * reset the de.tu_darmstadt.rs.sim.signal (must be done when the de.tu_darmstadt.rs.sim.signal was emitted, otherwise the
	 * de.tu_darmstadt.rs.sim.signal cannot be fired a second time) (called internally by de.tu_darmstadt.rs.sim.signal emit
	 * process)
	 */
	public final void reset() {
		fArmed = true;
	}

	/**
	 * register a modification handler (method which has 1 parameter/argument)
	 *
	 * @param o the object having the method (and containing runtime data)
	 * @param methodNameOfFlushHandler the method name of the flush method
	 * @param sigFlushArg the parameter of the flush method
	 */
	public void registerModifiedHandler(final Object o,
										final String methodNameOfFlushHandler,
										final Class<Signal0> sigFlushArg) {
		registerMethodCallOnEmitInternal(o, methodNameOfFlushHandler,
				sigFlushArg);
	}

	/**
	 * unregister a previously registered modification handler
	 *
	 * @param o the object having the method (and containing runtime data)
	 */
	public void unregisterModifiedHandler(final Object o) {
		unregisterMethodCallOnEmit(o);
	}

	/**
	 * register a flush handler (method which has no parameters/arguments)
	 *
	 * @param o the object having the method (and containing runtime data)
	 * @param methodNameOfFlushHandler the method name of the flush method
	 */
	public void registerFlushHandler(final Object o,
									 final String methodNameOfFlushHandler) {
		sigFlush.registerMethodCallOnEmit(o, methodNameOfFlushHandler);
	}

	/**
	 * unregister a previously registered flush handler
	 *
	 * @param o the object not to be invoked any more
	 */
	public void unregisterFlushHandler(final Object o) {
		sigFlush.unregisterMethodCallOnEmit(o);
	}
}
