/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 *
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Matthias Hofmann
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2012
 */
package de.tu_darmstadt.rs.sim.utilities;

import java.awt.Dimension;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.LinkedList;
import java.util.List;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Helper class for views of new periphery. The FifoPanel can display values for
 * 18 Bit fifos. The entries can be horizontal or vertical positioned.
 */
public class FifoPanel extends JPanel {

	private static final long serialVersionUID = 6408207763246947667L;

	/**
	 * horizontal orientation of the panel
	 */
	public final static int HORIZONTAL = BoxLayout.X_AXIS;
	/**
	 * vertical orientation of the panel
	 */
	public final static int VERTICAL = BoxLayout.Y_AXIS;

	private final List<CellPanel> cells;
	private final int orientation;

	/**
	 * creates a new FifoPanel with the given starting size and name. The cells
	 * are vertical placed.
	 *
	 * @param fifoSize
	 *            the starting size of the fifo
	 */
	public FifoPanel(int fifoSize) {
		this(fifoSize, VERTICAL);
	}

	/**
	 * creates a new FifoPanel with given size, name and orientation
	 *
	 * @param fifoSize
	 *            the starting size of the fifo
	 * @param orientation
	 *            the orientation of the fifo cells. Possible values:
	 *            HORIZONTAL, or VERTICAL. Any other value will result in a
	 *            vertical orientation.
	 */
	public FifoPanel(int fifoSize, int orientation) {
		super();

		switch (orientation) {
			case VERTICAL:
			case HORIZONTAL:
				this.orientation = orientation;
				break;

			default:
				this.orientation = VERTICAL;
		}

		setLayout(new BoxLayout(this, this.orientation));

		cells = new LinkedList<CellPanel>();

		resizeFifo(fifoSize);
	}

	/**
	 * Sets a value in the FifoPanel at the given position
	 *
	 * @param position
	 *            the position of the fifo entry. Starting at 0.
	 * @param value
	 *            the value to set. If null is passed, the cell is marked empty
	 *            (shows "-").
	 * @return true, if the value was set. False, if the position is invalid.
	 */
	public boolean setValue(final int position, final Integer value) {
		if (position >= cells.size()) {
			return false;
		}

		CellPanel cp = cells.get(position);
		if (cp == null) {
			return false;
		}

		cp.setValue(value);

		return true;
	}

	/**
	 * enlarge or truncate the cells to the specified size.
	 *
	 * @param size
	 *            the new number of cells
	 */
	public void resizeFifo(int size) {
		int currentSize = cells.size();

		if (currentSize == size) {
			return;
		}

		if (currentSize > size) {
			for (int i = 0; i < currentSize - size; i++) {
				CellPanel cp = cells.remove(currentSize - i - 1);

				assert (cp != null);

				remove(cp);
			}
		} else {
			assert (currentSize < size);

			for (int i = currentSize; i < size; i++) {
				CellPanel cp = new CellPanel(i);
				cells.add(cp);
				add(cp);
			}
		}

		validate();
	}

	/**
	 * @return the number of displayed fifo entries
	 */
	public int getFifoSize() {
		return cells.size();
	}

	/* *** inner class *** */
	private class CellPanel extends JPanel {

		private static final long serialVersionUID = 1662035442842830885L;

		private final JLabel numberLabel;
		private final JTextField valueTextField;

		public CellPanel(int cellNumber) {
			if (orientation == HORIZONTAL) {
				setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			}

			numberLabel = new JLabel(String.format("%2d", cellNumber));
			add(numberLabel);

			valueTextField = new JTextField(4); // enough space for 5 digits
			disableTextField(valueTextField);
			valueTextField.setHorizontalAlignment(JTextField.CENTER);
			setValue(null);
			add(valueTextField);

			Dimension dim = getPreferredSize();
			if (orientation == VERTICAL) {
				dim.height = valueTextField.getPreferredSize().height;
			} else {
				dim.width = valueTextField.getPreferredSize().width;
			}

			setPreferredSize(dim);
		}

		/**
		 * disable the text field without alter its style.
		 *
		 * @param field
		 */
		private void disableTextField(JTextField field) {
			KeyListener[] kl = field.getKeyListeners();
			for (KeyListener l : kl) {
				field.removeKeyListener(l);
			}

			MouseListener[] ml = field.getMouseListeners();
			for (MouseListener l : ml) {
				field.removeMouseListener(l);
			}

			MouseMotionListener[] mml = field.getMouseMotionListeners();
			for (MouseMotionListener l : mml) {
				field.removeMouseMotionListener(l);
			}
		}

		public void setValue(Integer value) {
			if (value == null) {
				valueTextField.setText("-");
			} else {
				valueTextField.setText(String.format("%x", value));
			}
		}
	}
}
