/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.utilities;

import de.tu_darmstadt.rs.sim.engine.SpartanDefinitions_I;

/**
 * The Formatter offers a lot of useful functions to modify strings and integers
 * which represents register (data) values.
 *
 */
public class Formatter implements SpartanDefinitions_I {

	/**
	 * Converts an integer (of 18 bit length) into a binary string description.
	 *
	 * @param arg
	 *            the integer
	 * @param leadingZeros
	 *            should the returned string contain leading zeros
	 * @return a string with 18 digits
	 */
	public static String intToBinaryString(final int arg,
										   final boolean leadingZeros) {

		String bin = "";
		int val_help = arg;
		for (int i = 17; i >= 0; i--) {
			int temp = val_help - (0x1 << i);
			if (temp > 0) {
				val_help = temp;
				bin += "1";
			} else if (temp < 0) {
				bin += "0";
			} else {
				// temp == 0
				bin += "1";
				for (int j = i - 1; j >= 0; j--)
					bin += "0";
				break;
			}
		}

		// delete leading zeros if needed
		if (!leadingZeros) {
			int index = bin.indexOf("1");
			if (index != -1) {
				bin = bin.substring(index);
			} else {
				bin = "0";
			}
		}

		int len = bin.length();
		String str = "";
		for (int i = len - 1; i >= 0; i--) {
			str = bin.charAt(i) + str;
			if ((((len - i) % 4) == 0) && (i != 0)) {
				str = " " + str;
			}
		}

		return str;
	}

	/**
	 * Converts a long (of 64 bit length) into a binary string description.
	 *
	 * @param arg
	 *            the integer
	 * @return a string with 64 digits
	 */
	public static String longToBinaryString(final long arg) {

		String bin = "";
		long val_help = arg;
		for (int i = 63; i >= 0; i--) {
			long temp = val_help - (0x1L << i);
			if (temp > 0) {
				val_help = temp;
				bin += "1";
			} else if (temp < 0) {
				bin += "0";
			} else {
				// temp == 0
				bin += "1";
				for (int j = i - 1; j >= 0; j--)
					bin += "0";
				break;
			}
		}

		int len = bin.length();
		String str = "";
		for (int i = len - 1; i >= 0; i--) {
			str = bin.charAt(i) + str;
			if ((((len - i) % 4) == 0) && (i != 0)) {
				str = " " + str;
			}
		}

		return str;
	}

	/**
	 * Sets all bits at position > 8 to 1, if bit at position 8 is 1.
	 *
	 * @param value
	 *            the value to be extended
	 * @return the new value
	 */
	public static int signExt8(final int value) {

		if ((value & 0x100) != 0) {
			return 0xfffffe00 | (value & 0x1ff);
		} else {
			return (value & 0x1ff);
		}
	}

	/**
	 * Sets all bits at position > 17 to 1, if bit at position 17 is 1.
	 *
	 * @param value
	 *            the value to be extended
	 * @return the new value
	 */
	public static int signExt17(final int value) {

		if ((value & 0x20000) != 0) {
			return 0xfffc0000 | (value & 0x3ffff);
		} else {
			return value & 0x3ffff;
		}
	}

	/**
	 * Sets all bits at position > 12 to 1, if bit at position 12 is 1.
	 *
	 * @param value
	 *            the value to be extended
	 * @return the new value
	 */
	public static int signExt12(final int value) {

		if ((value & 0x1000) != 0) {
			return 0xffffe000 | (value & 0x1fff);
		} else {
			return value & 0x1fff;
		}
	}

	/**
	 * Sets all bits at position > 16 to 1, if bit at position 16 is 1.
	 *
	 * @param value
	 *            the value to be extended
	 * @return the new value
	 */
	public static int signExt16(final int value) {

		if ((value & 0x10000) != 0) {
			return 0xfffe0000 | (value & 0x1ffff);
		} else {
			return value & 0x1ffff;
		}
	}

	/**
	 * Sets all bits at position > 15 to 1, if bit at position 15 is 1.
	 *
	 * @param value
	 *            the value to be extended
	 * @return the new value
	 */
	public static int signExt15(final int value) {

		if ((value & 0x8000) != 0) {
			return 0xffff0000 | (value & 0xffff);
		} else {
			return value & 0xffff;
		}
	}

	/**
	 * Sets all bits at position > 7 to 1, if bit at position 7 is 1.
	 *
	 * @param value
	 *            the value to be extended
	 * @return the new value
	 */
	public static int signExt7(final int value) {

		if ((value & 0x80) != 0) {
			return 0xffffff00 | (value & 0xff);
		} else {
			return (value & 0xff);
		}
	}

	/**
	 * Sets all bits at position > 35 to 1, if bit at position 35 is 1.
	 *
	 * @param value
	 *            the value to be extended
	 * @return the new value
	 */
	public static long signExt35(final long value) {
		if ((value & 0x800000000L) != 0) {
			return 0xfffffff000000000L | (value & 0xfffffffffL);
		} else {
			return (value & 0xfffffffffL);
		}
	}

	/**
	 * Parses a string containing a value in hex notation and checks via a bit
	 * mask, if its value range has not been exceeded.
	 *
	 * @param str
	 *            the hexadecimal string
	 * @param bitmask
	 *            the bit mask
	 * @return the integer or INVALID
	 */
	public static int parseHexString(final String str, final int bitmask) {

		if (str == null) {
			return INVALID;
		}

		String str_help = str.replaceFirst("0x", "");
		str_help = str_help.replaceFirst("0X", "");
		str_help = str_help.replaceFirst("x", "");
		str_help = str_help.replaceFirst("X", "");

		int i = INVALID;
		try {
			i = Integer.parseInt(str_help, 16);
		} catch (NumberFormatException e) {
			return INVALID;
		}

		if ((i & ~bitmask) != 0) {
			return INVALID;
		} else {
			return i;
		}
	}

	/**
	 * Parses a string containing a value in binary notation and checks via a
	 * bit mask, if its value range has not been exceeded.
	 *
	 * @param str
	 *            the binary string
	 * @param bitmask
	 *            the bit mask
	 * @return the integer or INVALID
	 */
	public static int parseBinString(final String str, final int bitmask) {

		if (str == null) {
			return INVALID;
		}
		String str_help = str.replaceFirst("b", "");
		str_help = str_help.replaceFirst("B", "");

		String str2 = "";
		for (int i = 0; i < str_help.length(); ++i) {
			if (str_help.charAt(i) == ' ') {
				continue;
			}
			str2 += str_help.charAt(i);
		}

		int i = INVALID;
		try {
			i = Integer.parseInt(str2, 2);
		} catch (NumberFormatException e) {
			return INVALID;
		}

		if ((i & ~bitmask) != 0) {
			return INVALID;
		} else {
			return i;
		}
	}

}
