/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 *
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
 */

/**
 * Spartan MC Simulator
 * Stefan Spickereit
 * Department for Computer Science, Chair for Embedded Systems
 * TU Dresden, 2006
 */
package de.tu_darmstadt.rs.sim.utilities;

import de.tu_darmstadt.rs.sim.gui.MainWindow;

import javax.swing.JOptionPane;

/**
 * The MessageBox offers convenient static methods to show the most common
 * message box types. JOptionPane is used internally. No extra functionality is
 * implemented.
 */
public class MessageBox {

	private static MainWindow mainWnd = null; // will be set by the MainWindow

	/**
	 * initialize the {@link MessageBox} helper
	 *
	 * @param mainWindow
	 *            the de.tu_darmstadt.rs.sim.main window which should be parent of all
	 *            {@link MessageBox} instances
	 */
	public static void init(MainWindow mainWindow) {
		mainWnd = mainWindow;
	}

	/**
	 * Shows an error message box.
	 *
	 * @param text
	 *            the message string
	 */
	public static void showError(final String text) {
		JOptionPane.showMessageDialog(mainWnd, text, "Error",
				JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Shows an error message box.
	 *
	 * @param text
	 *            the message string
	 * @param title
	 *            the boxes title
	 */
	public static void showError(final String text, final String title) {
		JOptionPane.showMessageDialog(mainWnd, text, title,
				JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Shows an information message box.
	 *
	 * @param text
	 *            the message string
	 * @param title
	 *            the boxes title
	 */
	public static void showInfo(final String text, final String title) {
		JOptionPane.showMessageDialog(mainWnd, text, title,
				JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * Shows a confirmation message box (OK or cancel option).
	 *
	 * @param text
	 *            the message string
	 * @param title
	 *            the boxes title
	 * @return true, if OK has been pressed
	 */
	public static boolean showConfirm(final String text, final String title) {
		return (JOptionPane.showConfirmDialog(mainWnd, text, title,
				JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION);
	}

	/**
	 * Shows a (text) input message box.
	 *
	 * @param text
	 *            the message string
	 * @param title
	 *            the boxes title
	 * @return the inserted string or null
	 */
	public static String showInput(final String text, final String title) {
		return (String) JOptionPane.showInputDialog(mainWnd, text, title,
				JOptionPane.QUESTION_MESSAGE, null, null, null);
	}
}
