/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package tests.engine.dcm;

import static org.junit.Assert.*;

import de.tu_darmstadt.rs.sim.engine.AccessManager;
import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import junit.framework.TestSuite;

import org.junit.BeforeClass;
import org.junit.Test;

import tests.mocks.ProjectManagerMock;
import tests.mocks.RefreshHandlerMock;
import tests.mocks.SimulationControllerMock;

import de.tu_darmstadt.rs.sim.engine.SimulationManager;
import de.tu_darmstadt.rs.sim.engine.dcm.AbstractDCM;

import de.tu_darmstadt.rs.sim.engine.dcm.CLKFX_DIVIDE;
import de.tu_darmstadt.rs.sim.engine.dcm.CLKFX_MULTIPLY;
import de.tu_darmstadt.rs.sim.engine.dcm.DCMManager;
import de.tu_darmstadt.rs.sim.engine.dcm.DCMSignalInvocationManager;
import de.tu_darmstadt.rs.sim.engine.dcm.SystemClockDCM;

import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerObserver_I;
import de.tu_darmstadt.rs.sim.engine.peripheral.timer.TimerSource_I;

/**
 * class for testing the {@link DCMSignalInvocationManager}
 *
 */
public class TestDCM extends TestSuite implements TimerObserver_I {
	private static DCMSignalInvocationManager invocationManager;
	private static RefreshHandlerMock refreshHandler;
	private static SimulationControllerMock simulationController;
	private static ProjectManagerMock projectManager;
	private static int clockIndex;
	private static AddressSpaceManager manager;

	private static class TimerObserver implements TimerObserver_I {

		@Override
		public void onClock(int clockValue) {
			clockIndex++;
		}

		@Override
		public void setTimerSource(TimerSource_I timerSource) {
			timerSource.registerTimerObserver(this);
		}

	}

	/**
	 * called before the first dispatcher.test is started
	 */
	@BeforeClass
	public static void setUpBeforeClass() {
		refreshHandler = new RefreshHandlerMock();
		simulationController = new SimulationControllerMock();
		projectManager = new ProjectManagerMock();
		manager=new AddressSpaceManager(refreshHandler, AccessManager.getTheInstance());
		SimulationManager simManager = new SimulationManager(null,
				refreshHandler, simulationController, null, null,
				projectManager,manager);
		/*AbstractDCM dcm = new UserClockDCM(simManager, "DCM1");
		dcm.setClkDvDivide(CLKDV_DIVIDE.DIVIDE_5);
		dcm.setClkFxMultiply(CLKFX_MULTIPLY.MULTIPLY_2);
		dcm.setClkFxDivide(CLKFX_DIVIDE.DIVIDE_3);
		// connect TestDCM to dcm->fx180
		String signalName = dcm.getDCMName() + "_clkfx180";
		TimerManager.addTimerConnection(signalName, new TimerObserver());*/
		AbstractDCM dcm2 = new SystemClockDCM(simManager, "DCM2");
		// connect dcm2 to dcm->dv
		DCMManager manager = simManager.getDCMManager();
		manager.connectDCMToSignal( "_clkdv", dcm2);
		// connect pipeline to dcm2->dv
		String signalName = dcm2.getDCMName() + "_clkdv";
		TimerManager.addTimerConnection(signalName,
				simManager.getPipelineClock());
		invocationManager = simManager.getDCMSignalInvocationManager();
	}

	/**
	 * Test the behavior
	 */
	//Deprecated ... DCM should not be used
	/*@Test
	public void testInvocations() {
		int[] values = { 14, 26, 40 };
		for (int i = 0; i < values.length; i++) {
			invocationManager.doClockingUntilStartOfNextPipelineClock();
			assertEquals(values[i], clockIndex);
		}
		invocationManager.doClockingUntilStartOfNextPipelineClock();
	}*/

	@Override
	public void onClock(int value) {
		clockIndex++;
	}

	@Override
	public void setTimerSource(TimerSource_I timerSource) {
		timerSource.registerTimerObserver(this);
	}

}
