package tests.gdb; /**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
/**
 * Created by nourhane on 29.08.16.
 */
import de.tu_darmstadt.rs.sim.engine.AccessManager;
import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.Settings;
import de.tu_darmstadt.rs.sim.engine.processor.RegisterFile;
import de.tu_darmstadt.rs.sim.engine.processor.SfRegisters;
import de.tu_darmstadt.rs.sim.gui.RefreshThread;
import junit.framework.TestSuite;


import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import de.tu_darmstadt.rs.sim.gdb.GdbServer;
import de.tu_darmstadt.rs.sim.gdb.SimulationManagerGDBInterface;

import static junit.framework.TestCase.assertEquals;
import static junit.framework.TestCase.assertFalse;
import static junit.framework.TestCase.assertTrue;
import static org.junit.Assert.assertArrayEquals;


public class SimulationManagerGDBInterfaceTest extends TestSuite {
	private static GdbServer gdb;
	private static SimulationManagerGDBInterface simulator;
	private static AddressSpaceManager addressSpaceManager;
	private static RegisterFile regFile;
	private static SfRegisters sfRegisters;

	@BeforeClass
	public static void setUpBeforeClass(){
		RefreshThread refreshThread=new RefreshThread(100);
		addressSpaceManager=new AddressSpaceManager(refreshThread, AccessManager.getTheInstance());
		sfRegisters=new SfRegisters(refreshThread,AccessManager.getTheInstance());
		regFile=new RegisterFile(refreshThread,sfRegisters,simulator);
		gdb=new GdbServer();
		simulator=gdb.getSimulator();
	}
	@AfterClass
	public static void afterClass(){
		simulator.kill();
	}
	@Test
	public void getAndSetMemoryTest(){
		/**
		 * memory insertion always starts at 0
		 */
		int address=0;
		int value=(int)(Math.random()*100);
		simulator.setMemory(address,value);
		assertEquals(value,simulator.getMemory(address));
	}
	@Test
	public void getAndSetRegisterTest(){
		int register=12;
		int data=(int)(Math.random()*100);
		simulator.setRegister(register,data);
		assertEquals(data,simulator.getRegister(register));
	}
	@Test
	public void getRegisterWindowBaseTest(){
		int[]actual=new int[]{1,2,3,4,5,6,7,8,9,10,11,12};
		int[]expected=new int[12];
		int current=0;
		for (int i=4;i<16;i++){
			simulator.setRegister(i,actual[current]);
			current++;
		}
		current=0;
		for(int i=4;i<16;i++){
			expected[current]=simulator.getRegisterWindowBaseValues(i,simulator.getSfRegisters().getWindow());
			current++;
		}
		assertArrayEquals(expected,actual);
	}
	@Test
	public void killAndResetTest(){
		simulator.kill();
		for (int i=0;i<simulator.getAddressSpaceManager().getSize();i++){
			assertFalse(simulator.getAddressSpaceManager().hasBreakpoint(i));
			assertFalse(simulator.getAddressSpaceManager().hasWatchpoint(i));
		}
		assertEquals(Settings.getSetting(Settings.PC_INITIAL),simulator.getPipelineUI().getCurrentAddress());
		assertEquals(0,simulator.getSfRegisters().getWindow());

	}
	//@Test
	public void stepTest(){
		int address=(int)(Math.random()*100);
		int current=simulator.getPipelineUI().getCurrentAddress();
		simulator.stepFromCurrentAddress();
		assertEquals(current+4,simulator.getAddress());
		simulator.stepFromGivenAddress(address);
		assertEquals(address+4,simulator.getAddress());

	}
	//@Test
	public void continueTest(){
		int address=(int)(Math.random()*100);
		int current=simulator.getPipelineUI().getCurrentAddress();
		simulator.continueFromCurrentAddress();
		assertEquals(current,simulator.getAddress());
		simulator.continueAtGivenAddress(address);
		assertEquals(address,simulator.getAddress());
	}
	@Test
	public void setAndClearWatchpointTest(){
		int address=100;
		if(!simulator.getAddressSpaceManager().hasWatchpoint(address))
			simulator.setWatchpoint(address);
		assertTrue(simulator.getAddressSpaceManager().hasWatchpoint(address));
		simulator.clearWatchpoint(address);
		assertFalse(simulator.getAddressSpaceManager().hasWatchpoint(address));
	}
	@Test
	public void setAndClearBreakpointTest(){
		int address=150;
		if(!simulator.getAddressSpaceManager().hasBreakpoint(address))
			simulator.setHardwareBreakpoint(address);
		assertTrue(simulator.getAddressSpaceManager().hasBreakpoint(address));
		simulator.clearHardwareBreakpoint(address);
		assertFalse(simulator.getAddressSpaceManager().hasBreakpoint(address));
		simulator.setMemoryBreakpoint(address);
		assertTrue(simulator.getAddressSpaceManager().hasBreakpoint(address));
		simulator.clearMemoryBreakpoint(address);
		assertFalse(simulator.getAddressSpaceManager().hasBreakpoint(address));
	}

}