/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package tests.mocks;

import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.PeripheralModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.api.IPeripheralModuleConfiguration;

import java.util.ArrayList;
import java.util.Collection;

import de.tu_darmstadt.rs.sim.engine.AddressSpaceManager;
import de.tu_darmstadt.rs.sim.engine.peripheral.JConfigPeripheral;
import de.tu_darmstadt.rs.sim.gui.RefreshThread;

/**
 * Class for checking the correct use/handling of peripherals
 */
public class PeripheralMock extends JConfigPeripheral{
	private final static int NUMBER_OF_REGISTERS=4;
	private static final PeripheralModuleDescription.Register [] registers;
	static{
		registers=new PeripheralModuleDescription.Register [NUMBER_OF_REGISTERS];
		for(int i=0;i<NUMBER_OF_REGISTERS;i++){
			registers[i]=new PeripheralModuleDescription.Register (){

				public AccessType  getRegisterAccess() {
					return AccessType.ReadWrite;
				}

				@Override
				public String getName() {
					return "someRegister";
				}


				public PeripheralModuleDescription.BitGroup [] getSubBitGroups() {
					return new PeripheralModuleDescription.BitGroup [0];
				}


				public PeripheralModuleDescription.BitGroup putSymbolicValue(int value,
						String symbolicName) {
					return null;
				}


				public String getSymbolicValue(int value) {
					return null;
				}


				public int getIndex() {
					return 0;
				}


				public int getWidth() {
					return 18;
				}


				public void addSubBitGroup(PeripheralModuleDescription.BitGroup  bitGroup) {
					//nothing to do here - this is a mock object
				}


				public Collection<String> getSymbolicValueList() {
					return new ArrayList<String>(0);
				}


				public int getIntValue(String symbolicValue) {
					return -1;
				}

				@Override
				public String getDescription() {
					return null;
				}

				public String getDescription(String symbolicValueName) {
					return null;
				}

				public void putDescription(String symbolicValueName,
						String description) {
					//nothing to do here - this is a mock object
				}

			};
		}
	}
	private final AddressSpaceManager addressSpaceManager;
	private final int baseAddress;
	private final int size;
	private String lastCommand="none";

	/**
	 * create a new {@link PeripheralMock}
	 * @param baseAddress base I/O address of the peripheral
	 * @param addressSpaceManager reference to the {@link AddressSpaceManager}
	 * @param refreshThread for initializing (super() call)
	 */
	public PeripheralMock(final int baseAddress, final AddressSpaceManager addressSpaceManager, final RefreshThread refreshThread){
		super("PeripheralTestObject", "PeripheralTestObject", baseAddress,addressSpaceManager,refreshThread,null,registers);
		this.baseAddress=baseAddress;
		this.size=registers.length;
		this.addressSpaceManager=addressSpaceManager;
	}
	@Override
	public void setup() {
		addressSpaceManager.registerPeripheral(baseAddress, size, this);
		lastCommand="setup";
	}

	@Override
	public void tearDown() {
		addressSpaceManager.unregisterPeripheral(baseAddress, this);
	}

	@Override
	public void restart() {
		lastCommand="restarted";
	}

	@Override
    public int readRegisterValue(int offset){
		lastCommand="read at "+(baseAddress+offset);
		return 0;
    }

	@Override
    public void writeRegisterValue(int offset, int value){
		lastCommand="write at "+(baseAddress+offset)+": "+value;
    }


	public void importHardwareParameters(IPeripheralModuleConfiguration node) {
		//nothing to do here - this is a mock object
	}

	@Override
	public String[] getParameters() {
		// This is a mock object:
		return null;
	}

	@Override
	public boolean setParameter(int parId, int value) {
		// This is a mock object:
		return false;
	}

	/**
	 * reset the last command to "none"
	 */
	public void resetLastCommand(){
		lastCommand="none";
	}

	/**
	 * retrieve the last issued command (call to a peripheral method)
	 * @return string representation of the command
	 */
	public String getLastCommand(){
		return lastCommand;
	}
	@Override
	public int getRegisterValue(int offset) {
		lastCommand="get value at "+(baseAddress+offset);
		return 0;
	}
	@Override
	public void setRegisterValue(int offset, int value) {
		lastCommand="set value at "+(baseAddress+offset)+": "+value;
	}
}

