/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc;

import de.tu_darmstadt.rs.process.ProcessUtil;
import de.tu_darmstadt.rs.spartanmc.jconfig.TestingProjectProvider;

import org.junit.Assume;
import org.junit.Test;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

/**
 * Crawls a path and searches files for regex expressions
 */
public class RegexFileCrawler {

	private List<String> ignoredFiles;
	private String regex;
	private Function<String, String>  errorMessageWriter;
	private Path root;

	public RegexFileCrawler(List<String> ignoredFiles, String regex, Function<String, String> errorMessageWriter, Path rootPath) {
		this.ignoredFiles = ignoredFiles;
		this.regex = regex;
		this.errorMessageWriter = errorMessageWriter;
		this.root = rootPath;
	}

	public void test() throws IOException {

		Assume.assumeTrue("Not in a Git repo", isGitDirectory(root));

		ProcessBuilder pb = new ProcessBuilder("git", "grep", "--extended-regexp", regex);
		pb.directory(root.toFile());

		List<String> output = new ArrayList<>();
		ProcessUtil.runAndWaitFor(pb, (Consumer<String>) output::add);

		final String filtered = output.stream()
				.filter(line -> ignoredFiles.stream()
						.map(file -> Paths.get(file))
						.noneMatch(file -> Paths.get(line.substring(0, line.indexOf(":"))).startsWith(file)))
				.collect(Collectors.joining("\n"));

		if (!filtered.isEmpty()) {
			throw new AssertionError(errorMessageWriter.apply(filtered));
		}
	}

	private boolean isGitDirectory(Path possibleGitDir) {
		while (possibleGitDir != null) {
			if (Files.exists(possibleGitDir.resolve(".git"))) {
				return true;
			}
			possibleGitDir = possibleGitDir.getParent();
		}
		return false;
	}
}
