/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.asyncReset;

import de.tu_darmstadt.rs.spartanmc.RegexFileCrawler;
import de.tu_darmstadt.rs.spartanmc.jconfig.TestingProjectProvider;
import de.tu_darmstadt.rs.util.SpartanMCDefaultFolders;

import org.junit.Test;

import java.io.IOException;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.List;

/**
 * Check that no file in the repository has an asynchromous reset
 */
public final class AsyncResetTest {

	private static List<String> ignoredFiles = Arrays.asList(
			"axi_interconnect/src/ict106_axi_clock_converter.v",
			"microstepper/src/ramp_fsm.v",
			"display/src/display.v",
			"ddr/xilinx/mcb_sp6/src/mcb_soft_calibration.v",
			"ddr/xilinx/mcb_sp6/src/v6_data_gen.v",
			"ddr/xilinx/mcb_sp6/src/cmd_gen.v",
			"ddr/xilinx/mi_7series/src/ddr3_model.v",
			"common/src/fifo.v",
			"common/src/spmc_async_fifo.v",
			"common/src/one_hot_to_bin.v",
			"shi/",
			"usb11/src/usb_rx_phy.v",
			"usb11/src/usb_phy.v",
			"usb11/src/blockram.v",
			"can/",
			"display",
			"fancontrol",
			"fpgaminer",
			"hardware_barrier",
			"heartbeat",
			"microstepper",
			"timer/src/timer.v",
			"timer_rti/src/timer_rti.v",
			"timer_watchdog/src/timer_watchdog.v",
			"sysmon"

	);

	private static final String REGEX = "\\s*always\\s*@\\s*\\(.*([^p]reset|[^fi]rst).*";
	private static final String TIPS_URL = "https://www.xilinx.com/support/documentation/white_papers/wp231.pdf";

	public AsyncResetTest() {
	}

	@Test
	public void testAsyncReset() throws IOException {
		final Path root = SpartanMCDefaultFolders.hardware(TestingProjectProvider.getSpartanRoot().normalize());
		RegexFileCrawler crawler = new RegexFileCrawler(ignoredFiles, REGEX, f -> "It seems that there is an asynchronous reset in the following files:\n " + f + "\n\nUsing asynchronous reset has a negative impact on the synthesis. Read: "+ TIPS_URL +" for more info", root);
		crawler.test();
	}


}
