/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.header_check;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import java.io.IOException;
import java.util.Arrays;

@RunWith(Parameterized.class)
public class CCommentRemoverTest {

	@Parameterized.Parameters(name = "{0} {1}")
	public static Iterable<?> data() throws IOException {

		return Arrays.asList(
				new Object[]{
						"x//a\nb\nc",
						CCommentRemover.BackSlashCommentEOL.Error,
						"x\nb\nc",
						null
				},
				new Object[]{
						"//a\nb\n//c",
						CCommentRemover.BackSlashCommentEOL.Error,
						"b",
						null
				},
				new Object[]{
						"//a\\\nb\nc",
						CCommentRemover.BackSlashCommentEOL.ContinueComment,
						"c",
						null
				},
				new Object[]{
						"//a\\\nb\nc",
						CCommentRemover.BackSlashCommentEOL.EndComment,
						"b\nc",
						null
				},
				new Object[]{
						"//a\\\nb\nc",
						CCommentRemover.BackSlashCommentEOL.Error,
						null,
						CCommentRemover.CommentException.class
				},
				new Object[]{
						"/*a*b\n/\nc*/d",
						CCommentRemover.BackSlashCommentEOL.Error,
						"d",
						null
				},
				new Object[]{
						"//a\\\n//b\nc",
						CCommentRemover.BackSlashCommentEOL.ErrorIfSignificant,
						"c",
						null
				},
				new Object[]{
						"//a\\\nx//b\nc",
						CCommentRemover.BackSlashCommentEOL.ErrorIfSignificant,
						null,
						CCommentRemover.CommentException.class
				},
				new Object[]{
						"//a\\\n/x//b\nc",
						CCommentRemover.BackSlashCommentEOL.ErrorIfSignificant,
						null,
						CCommentRemover.CommentException.class
				},
				new Object[]{
						"s//a\\\n/*x//b\nc",
						CCommentRemover.BackSlashCommentEOL.ErrorIfSignificant,
						"s",
						null
				}
		);
	}

	@Parameterized.Parameter
	public String input;

	@Parameterized.Parameter(value = 1)
	public CCommentRemover.BackSlashCommentEOL backSlashCommentEOL;

	@Parameterized.Parameter(value = 2)
	public String output;

	@Parameterized.Parameter(value = 3)
	public Class<Exception> exceptionClass;

	@Test
	public void test() {

		final String[] split = input.split("\n");
		CCommentRemover remover = new CCommentRemover(Arrays.spliterator(split), backSlashCommentEOL);

		try {

			final String[] actual = remover.stream().toArray(String[]::new);

			if (exceptionClass != null) {
				Assert.fail("Expected " + exceptionClass + " to be thrown");
			}

			String[] expected = output.split("\n");

			Assert.assertArrayEquals(expected, actual);

		} catch (Exception e) {
			//Unexpected? --> rethrow
			if (exceptionClass == null) {
				throw e;
			}

			Assert.assertTrue("Expect thrown exception to be of type " + exceptionClass, exceptionClass.isInstance(e));
		}


	}

}
