/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.integration_test;

/**
 * Created by wenzel on 11.01.17.
 */
public class ModelsimOutputParser extends OutputParser {

	@Override
	protected boolean isErrorLine(String line) {
		//Regular modelsim error
		if (line.matches("#\\s*(\\*\\*)?\\s*(Fatal|Error):.*")) {
			return true;
		}

		//Processor core error detection
		if (line.matches("#\\s*\\[\\s*\\d+\\s*\\]\\s*Error:")) {
			return true;
		}

		//Multiline modelsim error
		if (line.startsWith("# ** at ")) {
			return true;
		}

		//Error from gcc which is called by modelsim
		if (line.matches("# [^ ]+: (internal compiler )?error:.*")) {
			return true;
		}

		//Error from ld which is called by modelsim
		if (line.matches("# /[^ ]+: .*")) {
			return true;
		}

		//Output diff at end of integration test
		//We match german and english output...
		if (line.matches("(Dateien|Files) .* [ua]nd .* (sind verschieden\\.|differ)")) {
			return true;
		}

		//Make: JConfig should not update unmodified files
		if (line.startsWith("Error: JConfig should not")) {
			return true;
		}

		//Make: unknown rule
		if (line.matches("make: \\*\\*\\* (Keine Regel|No rule to make).*")) {
			return true;
		}

		//Integration Test: Diff at end
		if (line.matches("--- .*")) {
			isInDiff = true;
			return true;
		}

		if (isInDiff) {
			if (line.matches("\\.\\./testcase\\.mk.*")) {
				System.out.println("disabling in diff at " + line);
				isInDiff = false;
			} else {
				//The line "make[1]: *** [testSim] Fehler 1" line from stderr pops up after or within the diff.
				//filter it out.
				return !line.startsWith("make[1]");
			}
		}

		return false;
	}

	@Override
	protected boolean isWarningLine(String line) {
		if (line.matches("# \\*\\* Warning:.*")) {
			return true;
		}

		return false;
	}
}
