/*
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */

package de.tu_darmstadt.rs.spartanmc.integration_test;

import de.tu_darmstadt.rs.spartanmc.devxml.Library;
import de.tu_darmstadt.rs.spartanmc.devxml.files.modules.PeripheralModuleDescription;
import de.tu_darmstadt.rs.spartanmc.jconfig.TestingProjectProvider;
import de.tu_darmstadt.rs.util.StreamUtil;

import org.junit.Test;

import java.io.IOException;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Verify that no two peripherals have the same type name from different files
 */
public class RegsTypeUniqueness {

	public RegsTypeUniqueness() {
	}

	private PeripheralModuleDescription.CSupport cSupportWithoutDeclareFile(PeripheralModuleDescription.CSupport orig) {
		return new PeripheralModuleDescription.CSupport(orig.getTypeName(), orig.getFileName(), null);
	}

	@Test
	public void verifyModuleNameUniqueness() throws IOException {
		Library library = TestingProjectProvider.getInstance().getLibrary();

		final Map<PeripheralModuleDescription.CSupport, Set<String>> modulesByCSupport = library.getAvailableModules().stream()
				.flatMap(module -> {
					if (!(module instanceof PeripheralModuleDescription)) {
						return Stream.empty();
					}

					PeripheralModuleDescription peripheral = (PeripheralModuleDescription) module;

					final Stream.Builder<StreamUtil.Pair<PeripheralModuleDescription.CSupport, String>> builder = Stream.builder();

					if (peripheral.getRegisters() != null && peripheral.getRegisters().getCSupport() != null) {
						builder.accept(new StreamUtil.Pair<>(cSupportWithoutDeclareFile(peripheral.getRegisters().getCSupport()), peripheral.getName()));
					}

					if (peripheral.getMemory() != null && peripheral.getMemory().getCSupport() != null) {
						builder.accept(new StreamUtil.Pair<>(cSupportWithoutDeclareFile(peripheral.getMemory().getCSupport()), peripheral.getName()));
					}


					return builder.build();
				})
				.collect(Collectors.groupingBy(StreamUtil.Pair::getA, Collectors.mapping(StreamUtil.Pair::getB, Collectors.toSet())));

		final Map<String, Set<PeripheralModuleDescription.CSupport>> cSupportByTypeName =
				modulesByCSupport.keySet().stream().collect(Collectors.groupingBy(PeripheralModuleDescription.CSupport::getTypeName, Collectors.toSet()));

		final String result = cSupportByTypeName.keySet().stream()
				.filter((typeName) -> cSupportByTypeName.get(typeName).size() > 1)
				.map(typeName -> {
					final String modules = cSupportByTypeName.get(typeName).stream()
						.map(cs -> "from " + cs.getFileName() + " in " + modulesByCSupport.get(cs))
						.collect(Collectors.joining(", "));
					return String.format("Multiple differing types called %s are used: %s", typeName, modules);
				})
				.collect(Collectors.joining("\n"));

		if (!result.isEmpty()) {
			throw new AssertionError(result);
		}


	}
}
