#ifndef __LOADER_H
#define __LOADER_H

#include <elf.h>
#include <stdio.h>

// Exit values
#define EXIT_OPTIONS    2
#define EXIT_FILEIO     3
#define EXIT_MEMALLOC   4
#define EXIT_INVALIDELF 5

// Loader Version
#define VERSION_PATTERN     "SpMC loader v%8[0-9]"
#define VERSION_NUMBER      "20140908"

// 18 Bit handling
#define MASK_18BIT  0x3FFFF

// Dump options
#define OPT_DUMP_HEADER     1
#define OPT_DUMP_CONTENT    2

struct elf_file {
    FILE *elf_infile;
    Elf32_Ehdr elf_header;
    uint32_t elf_num_sections;
    Elf32_Shdr *elf_section_hdrs;
    uint32_t **elf_section_data;
    uint32_t *elf_masked_sections;
};

int elf_open_file(struct elf_file *elf, char *filename);
void elf_close_file(struct elf_file *elf);
void elf_select_sections(struct elf_file *elf, char **names, int exclude);
char *elf_get_section_name(struct elf_file *elf, uint32_t sec_idx);
uint32_t *elf_get_section_data(struct elf_file *elf, uint32_t sec_idx);
Elf32_Sym *elf_find_symbol(struct elf_file *elf, char *sym_name);
Elf32_Shdr *elf_find_section(struct elf_file *elf, char *sec_name);

int uart_device_open(char *devname);
int uart_readln(char *buf, int max_size, int timeout_ms);
int uart_read(void *buf, int size, int timeout_ms);
int uart_write(void *buf, int cnt);
void uart_device_close();
void uart_flush();

int loader_upload_file(struct elf_file *elf, int verify_only);
int loader_test_uart(void);

void action_init();
void action_finalize();
void action_print(char *action, ...);
void action_start(char *action, ...);
void action_progress(char *fmt, ...);
void action_errmsg(char *fmt, ...);
int action_end(int status);

#endif /* __LOADER_H */
