# Checks whether a variable is 
# set and allows to specify any 
# custom message to display in 
# case of failure
# $1: variable name to check
# $2: item name
# $3: failure message
# $4: additional information
#
AC_DEFUN(
	AC_CHECK_ITEM_LOG,[
		if test x$$1 = x; then
			if test -n "$4"; then
				msg="$3: $2 ($4)"
			else
				msg="$3: $2"
			fi
			error_messages="$error_messages AS_ESCAPE([$msg],[ ])"
		fi
	]
)

# Checks whether an environment
# variable is set
# $1: variable name to check
# $2: message if not set 
AC_DEFUN(
	AC_CHECK_VAR_LOG,[
		AC_MSG_CHECKING([whether \$$1 is set])
		if test -z "$$1"; then
			AC_MSG_RESULT([no])
		else
			AC_MSG_RESULT([\$$1])
		fi
		AC_CHECK_ITEM_LOG([$1],[\$$1],[variable not set],[$2])
	]
)

# Like AC_CHECK_PROGS but stores
# information for later error
# reporting
# $1: variable to set
# $2: program(s) to check
# $3: error message if not found
AC_DEFUN(
	AC_CHECK_PROGS_LOG,[
		AC_CHECK_PROGS([$1],[$2],[no])
		test x$$1 = xno && $1=
		AC_CHECK_ITEM_LOG([$1],[$2],[program not found],[$3])
	]
)

# Lists all missing programs
# logged by priot calls to
# AC_CHECK_PROGS_LOG
# $1: message to show on top of list 
AC_DEFUN(
	AC_REPORT_ERRORS,[
		if test -n "$error_messages"; then
			$as_echo
			$as_echo
			AC_MSG_NOTICE([*** $1])
			$as_echo
			AS_ECHO([$error_messages])
			$as_echo
			$as_echo
			exit 1
		fi
	]
)

# Shows an error message
# in case of failure
# $1: messag to show
AC_DEFUN(
	AC_FAIL,[
		msg=$1
		AC_MSG_NOTICE([*** missing program or incomplete configuration])
		AC_MSG_NOTICE([reason: $msg])
		AC_MSG_NOTICE([try './configure --help' or])
		AC_MSG_NOTICE(['man $manpage' for more information])
		AC_MSG_NOTICE([*** FAILED])
		exit 1
	]
)

# Checks wether a given buildfile
# runs sucessfully with ant
# requires ${ANT}
# $1: buildfile to use
AC_DEFUN(AC_TRY_ANT,[
BUILDFILE=$1
AC_MSG_CHECKING([if ant is working])
TESTCMD='${ANT} -q -f ${BUILDFILE}'
if ! eval $TESTCMD; then
  AC_FAIL(['ant failed, most likely a required task is missing. See ant-output above for details!'])
fi
])

