#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Defines low level internal functions used by
# every makefile
#
include $(SPARTANMC_ROOT)/src/scripts/make/common/definitions.mk

# Disable implicit rules, we always say what we want to do
.SUFFIXES:

# Bracket characters will confuse make when used in variable assignments.
# For shell printf we can used the corresponding ASCII hex codes
OBRACE:=\x28
CBRACE:=\x29

# Print a fancy logo

# The Unicode characters in the logo confuse ModelSim and no output is displayed
# after the logo. Therefore, we don't display it in that case.

ifneq ($(DEFINES),SIM)
$(info $(shell printf '\033[0m'))
$(info $(SPACE)┌───────────────────────────────┐)
$(info $(SPACE)│── ▄▄▄▄▄ ────┬───┐          ◢ ◤│)
$(info $(SPACE)│┬─ █   █ ────┤   │O──────◢ ◤   │)
$(info $(SPACE)││  █▄▄▄█ ┌───┴───┘    ◢ ◤      │)
$(info $(SPACE)││  █  █▖ │         ◢ ◤         │    SpartanMC SoC Kit)
$(info $(SPACE)││  █   █ │      ◢ ◤            │    Computer Systems Group)
$(info $(SPACE)│└─┬───┐  │   ◢ ◤       ▄▄▄▄▄   │    TU Darmstadt)
$(info $(SPACE)│  │   ├──┘◢ ◤          █       │)
$(info $(SPACE)│──┴───┘◢ ◤       010   █▄▄▄▄   │)
$(info $(SPACE)│    ◢ ◤       010001       █   │)
$(info $(SPACE)│ ◢ ◤       100010100   ▄▄▄▄█   │)
$(info $(SPACE)└───────────────────────────────┘)
$(info )
endif

$(call MAKE_MSG,reading makefiles ...)
$(eval $(MAKEFILE_PROLOG))

# Variable will be empty if configure.mk is not present 
# -> configure run required
CONFIGURE_MK=$(realpath $(SPARTANMC_ROOT)/src/scripts/make/common/configure.mk)
ifneq ($(CONFIGURE_MK),)

  .PHONY: default
  default: what

  include $(CONFIGURE_MK)

  export SH_SCRIPTS:=$(SPARTANMC_SCRIPTS)/sh

  $(AC_CONFIG_SOURCES):
	@true

  include $(MAIN_MK)

else

$(call FATAL_ERROR, Your SpartanMC configuration is not complete.$(NEWLINE)  Please run './autogen.sh && ./configure' from your SpartanMC root directory.$(NEWLINE)  (currently set to '$(SPARTANMC_ROOT)'))


endif
