#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Defines common make functions
#

$(eval $(MAKEFILE_PROLOG))

# Find out the width of our terminal ...
COLUMNS:=$(shell which tput > /dev/null && tput cols 2>/dev/null)

# ... and use fall back to default of 80
ifeq ($(COLUMNS),)
    COLUMNS:=80
endif

# Horizontal lines for pretty formatted output
HLINE:=$(subst $(SPACE),_,$(shell printf "%$(COLUMNS)s"))
HLINE_STRONG:=$(subst $(SPACE),=,$(shell printf "%$(COLUMNS)s"))

# Allow bold text for even more pretty formatted output :)
ifndef +no-text-fmt
    export ANSI_BOLD:=\x1B[1m
    export ANSI_BOLDOFF:=\x1B[0m
else
    export ANSI_BOLD:=
    export ANSI_BOLDOFF:=
endif

# BUILD_DIR variable is required
ifeq ($(BUILD_DIR),)
    $(error BUILD_DIR not set)
endif

# Print a message with target annotation
# 1: Message
# 2: Prefix string inserted after "make:"
TARGET_MESSAGE = \
	 printf "$(MAKE):$(2) [$(call MAKE_RELATIVE,$@)] $(1)\n" | $(SH_SCRIPTS)/pretty-print

# Print a bold message with target annotation
# 1: Message
# 2: Prefix string inserted after "make:"
TARGET_MESSAGE_BOLD = \
	printf "$(ANSI_BOLD)" && \
	$(call TARGET_MESSAGE,$(1),$(2)) && \
	printf "$(ANSI_BOLDOFF)"

# Print a error message with target annotation
TARGET_ERROR = \
	printf "\n" && \
	$(call TARGET_MESSAGE_BOLD,Error\n \n$(1), ***) && \
	printf "\n" && \
	false

# Print done message
ECHO_TARGET_DONE = \
	$(call TARGET_MESSAGE,done\n)

# Prints a title message surrounded by a hotizontal line at top and bottom
# 1: long title
# 2: short title
TITLE=$(ANSI_BOLD)$(HLINE)\n\n  $(1)$(ANSI_BOLDOFF) [$(2)]\n$(ANSI_BOLD)$(HLINE)$(ANSI_BOLDOFF)

# Calls helper script make-build to execute a build command
# and prints a done message when finished
# 1: Command to run
# 2: Title
# 3: Options to make-build script
BUILD_BASE = \
	$(SH_SCRIPTS)/make-build "$(1) $(4)" "$(2)" $(3) && \
	$(ECHO_TARGET_DONE)

# Executes build command template and shows the corresponding title before
# executing the actual command
# 1: Name of command template to run
# 2: Options to make-build script
BUILD = \
	cd $(BUILD_DIR) && \
	$(call BUILD_NOCD,$(1),$(2),$(3))

BUILD_NOCD = \
	$(call BUILD_BASE,$($(1)_CMD),$(call TITLE,$($(1)_PROCESS),$(@F)),$(2),$(3))

# Transforms an absolute path into a relative one
# 1: Path to transform
MAKE_RELATIVE = \
	$(subst $(CURDIR)/$(2),,$(abspath $(1)))

# Performs a simple remove on all specified files
# but filters out all items that are not relative
# 1: List of items to delete
REMOVE = \
	if test -n '$(1)'; then \
		rm -rfv $(filter-out /%,$(1)); \
	fi

# Export a variable and store the variables name in a list keeping track of
# all exported variables
# 1: variable name
# 2: optional part specifying a value 
#    (must be "=<value>")
define EXPORT
  export $(1)$(2)
  EXPORTED_VARIABLES+=$(1)
endef


# Save variable content in a file. Its name is returned.
# It is only touched if its content changes.
# Can be used to depend on a variable value.
# 1: Variable name
# 2: Directory to save to
VAR_VALUE_TO_FILE =  $(shell \
		mkdir -p $(2); \
		echo $(subst ",\",$($(1))) > $(2)/$(1)-temp; \
		$(SH_SCRIPTS)/move-if-changed.sh $(2)/$(1)-temp $(2)/$(1); \
		echo $(2)/$(1) \
	)


