#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Provides all targets to build a firmware
# target from a set of source files using
# gcc and binutils toolchain.
#

# be independent of other SpartanMC cores and reset variables
INT_BASE:=

#$(eval $(call MAKEFILE_PROLOG,$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM): ))

include $(SPARTANMC_ROOT)/src/scripts/make/common/localvars.mk
AVAILABLE_TARGETS+=firmware
HELP_firmware:=Build firmware binary $(OBRACE)*.elf$(CBRACE)

.SUFFIXES:

D_FWHOME:=$(JCONFIG_FW_$(FW_ID)_SOURCE)

SOURCE_DIR:=
OBJECTS:=
OBJECTS_SET_IDS:=
INCLUDE_DIRS:=
LINKER_SCRIPT:=
LIB_AS_FILES:=
LIB_OBJ_FILES:=
LIBRARIES:=
DEFINES:=
CONFIG_BUILD_MK:=$(D_FWHOME)/config-build.mk

include $(CONFIG_BUILD_MK)

MAKEFILE_LIST_FIRMWARE:=$(MAKEFILE_LIST_FIRMWARE) $(CONFIG_BUILD_MK)

# include-message
$(eval $(call MAKEFILE_PROLOG, ($(FW_ID))))

include $(GENERATED_FILES_DIR)/firmwares/$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM)/hardware.mk

D_SRC:=$(abspath $(D_FWHOME)/$(SOURCE_DIR))
D_SPMC:=$(SPARTANMC_ROOT)/spartanmc
D_TOOLS:=$(SPARTANMC_ROOT)/tools
D_LIB_AS:=$(D_SPMC)/lib_as
D_LIB_LD:=$(D_SPMC)/lib_ld
LD_INC:=$(abspath $(addprefix $(D_FWHOME)/,$(INCLUDE_DIRS))) 
LD_SYSTEMINC = $(abspath $(GENERATED_FILES_DIR) $(D_SPMC)/include $(SPARTANMC_ROOT)/spartanmc/lib_obj/include)
LN_LIBS:=$(LIB_OBJ_FILES) $(ADDITIONAL_LIB_OBJ_FILES)
LN_LIBS?=$(LIBRARIES)
LN_LIBAS:=$(LIB_AS_FILES)
L_DEFINES:=$(DEFINES)
L_SYMBOLS:=$(SYMBOLS)

# add system/subsystems/SUB to include path, if current firmware is a local firmware => peripherals.h is on include path!
ifneq ($(JCONFIG_FW_$(FW_ID)_SUBSYSTEM),)
	LD_SYSTEMINC+=$(GENERATED_FILES_DIR)/subsystems/$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM)
endif

# flags
_C_CPP_COMMON_FLAGS:=-Werror=implicit-function-declaration -Wmissing-include-dirs -Wformat -ffreestanding -Wsystem-headers -flto
_C_CPP_COMMON_FLAGS+=$(foreach incdir, $(LD_SYSTEMINC),-isystem $(incdir))
_C_CPP_COMMON_FLAGS+=$(foreach incdir, $(LD_INC),-I$(incdir))
_C_CPP_COMMON_FLAGS+=$(foreach define, $(L_DEFINES),-D$(define))
_C_CPP_COMMON_FLAGS+=$(CFLAGS)

_CFLAGS:=-std=gnu11
_CFLAGS+=$(_C_CPP_COMMON_FLAGS)

_CPPFLAGS:=-std=c++17
_CPPFLAGS+=$(_C_CPP_COMMON_FLAGS)

ifdef ASFLAGS
  _ASFLAGS:=$(ASFLAGS)
else
  _ASFLAGS:=
endif
_ASFLAGS+=-I$(PROJECT_DIR)
_ASFLAGS+=$(foreach define, $(L_DEFINES),-Wa,--defsym=$(define)=1)
_ASFLAGS+=-I$(D_SRC)

ifdef LDFLAGS
  _LDFLAGS:=$(LDFLAGS)
else
  _LDFLAGS:=
endif
_LDFLAGS+=-static
_LDFLAGS+=-L$(SPARTANMC_ROOT)/spartanmc/lib_obj/build

ifneq ($(LINKER_SCRIPT),)
  
  LDSCRIPT:=$(D_FWHOME)/$(LINKER_SCRIPT)
else
  LDSCRIPT:=$(GENERATED_FILES_DIR)/$(JCONFIG_FW_$(FW_ID)_LINKERSCRIPT)
endif
_LDFLAGS+=$(foreach symbol, $(L_SYMBOLS),-Wl,--defsym,$(symbol))

# flags checksum
FW_FLAGS_CSUM:=$(shell printf "%s" "$(_CFLAGS)$(_ASFLAGS)$(_LDFLAGS)" | $(SPARTANMC_SCRIPTS)/sh/csum "%08X")

# files
D_BUILD:=$(BUILD_DIR)/firmware/$(FW_ID)
SDK_DIR:=$(BUILD_DIR)/project/$(JCONFIG_PRJNAME).sdk
LF_C:=$(foreach obj,$(OBJECTS),$(wildcard $(D_SRC)/$(obj).c))
LF_CPP:=$(foreach obj,$(OBJECTS),$(wildcard $(D_SRC)/$(obj).cpp))
LF_S:=$(foreach obj,$(OBJECTS),$(wildcard $(D_SRC)/$(obj).s))
LF_INC:=$(foreach dir,$(LD_INC) $(C_INCLUDE_PATH) $(D_SPMC)/include/peri,$(wildcard $(dir)/*.h))
LF_LIBAS:=$(foreach obj,$(LN_LIBAS),$(D_BUILD)/$(obj).o)
LF_A:=$(foreach obj,$(LN_LIBS),$(SPARTANMC_ROOT)/spartanmc/lib_obj/build/lib$(obj).a)

# add system/subsystems/SUB/peripherals.c to c files, if current firmware is a local firmware
ifneq ($(JCONFIG_FW_$(FW_ID)_SUBSYSTEM),)
	LF_C_GENERATED_DIR:=$(GENERATED_FILES_DIR)/subsystems/$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM)
	LF_C+=$(LF_C_GENERATED_DIR)/peripherals_generated_jconfig.c
endif

LF_OBJ:=$(foreach obj,$(notdir $(basename $(sort $(LF_C) $(LF_CPP) $(LF_S)))),$(D_BUILD)/$(obj).o)

ifeq ($(HWACC_ENABLED),1)
  ifeq ($(HWACC_PLUGIN_ROOT),)
    $(error HWACC_PLUGIN_ROOT is not set. This variable should point to your hardware accelerator plugin directory)
  endif

  _CFLAGS+=-I$(HWACC_PLUGIN_ROOT)/include
  LF_OBJ:=$(LF_OBJ) $(D_BUILD)/hwacc_wrappers.a
  _LDFLAGS:=$(D_BUILD)/hwacc_wrappers.a $(_LDFLAGS) -L$(HWACC_PLUGIN_ROOT)/libacc -lacc
endif

P_MAIN:=$(D_BUILD)/main
F_ELF:=$(P_MAIN).elf
F_LST:=$(P_MAIN).lst

ifneq ($(DEBUG),)
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)INCLUDE_DIRS = $(INCLUDE_DIRS))
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)LD_INC = $(LD_INC))
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)_CFLAGS = $(_CFLAGS))
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)_ASFLAGS = $(_ASFLAGS))
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)_LDFLAGS = $(_LDFLAGS))
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)LF_C = $(LF_C))
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)LF_CPP = $(LF_CPP))
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)LF_S = $(LF_S))
  $(info )
  $(info $(SPACE)$(SPACE)$(SPACE)LF_OBJ = $(LF_OBJ))
  $(info )
endif


LIBS:=$(foreach lib,$(LN_LIBS),-l$(lib))


$(eval $(call EXPORT,SPARTANMC_GCC))

# create makefile for firmware build with exported variable values
MAKEFILE_FWBUILD:=$(D_BUILD)/fw_build_$(FW_FLAGS_CSUM)_$(FW_ID).mk
-include $(MAKEFILE_FWBUILD)
MAKEFILE_LIST_FIRMWARE:=$(MAKEFILE_LIST_FIRMWARE) $(MAKEFILE_FWBUILD)
NEWLINE:=\n


# define set of variables
FW_BUILD_LOCAL_VARIABLES:= \
	SDK_DIR \
	D_BUILD \
	D_SRC \
	_CFLAGS \
	_CPPFLAGS \
	_ASFLAGS \
	LF_LIBAS \
	LF_OBJ \
	LF_C \
	HWACC_EXTRA_CFLAGS_SYNTHESIS \
	HWACC_EXTRA_CFLAGS_ANALYSIS \
	P_MAIN \
	_LDFLAGS \
	LDSCRIPT\
	LIBS \
	F_SYSTEM_S \
	LF_A \
	SBI_CORE_ID \
	F_ELF \
	F_LST\
	LF_C_GENERATED_DIR

$(D_BUILD):
	@mkdir -p $@

$(call local_export,$(FW_ID),$(FW_BUILD_LOCAL_VARIABLES))


LF_OBJ_FILE=$(call VAR_VALUE_TO_FILE,LF_OBJ,${D_BUILD})

$(D_BUILD)/fw_build_$(FW_FLAGS_CSUM)_%.mk: $(SPARTANMC_ROOT)/src/scripts/make/template/gcc-build.mk $(LF_OBJ_FILE) $(filter-out $(D_BUILD)/%.mk,$(MAKEFILE_LIST)) | $(D_BUILD)
	echo LF_OBJ_FILE: $(LF_OBJ_FILE)
	@\
	$(call local_import,$*,$(FW_BUILD_LOCAL_VARIABLES)) \
	$(SPARTANMC_ROOT)/src/scripts/sh/rdtmp $< > $@
	# remove build makefiles from old builds with different CFLAGS
	@find $(@D) -name "fw_build_*_$*.mk" -not -name "$(@F)" | while read f; do rm $$f; done

SUBSYSTEM:=$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM)

# create one makefile for each memory section with exported variable values
FW_SECTION_LOCAL_VARIABLES:= \
	D_BUILD \
	SEC_ID \
	SPH \
	BRAM \
	SUBSYSTEM \
	MODULE

define assign_section_variables
  SPH=$(D_BUILD)/$($(1)_MODULE).sph
  BRAM=$($(1)_BRAM)
  MODULE=$($(1)_MODULE)
endef


# MAKEFILE_LIST is too much for this
$(D_BUILD)/fw_section_%.mk: $(SPARTANMC_ROOT)/src/scripts/make/template/fw-section.mk $(filter-out $(D_BUILD)/%.mk,$(MAKEFILE_LIST)) | $(D_BUILD)
	@\
	$(call local_import,$*,$(FW_SECTION_LOCAL_VARIABLES)) \
	$(SPARTANMC_ROOT)/src/scripts/sh/rdtmp $< > $@

#$(error break)

ASM_FILES:=$(ASM_FILES) $(LF_S)
FW_SOURCE_FILES:=$(FW_SOURCE_FILES) $(LF_S) $(LF_C) $(LF_INC)

.PHONY: firmware

firmware: $(F_ELF) $(F_LST) $(SPH)
