#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Provides all targets to build firmware for behavioural simulation using
# the spartanmc simulator
#

$(eval $(call MAKEFILE_PROLOG,$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM): ))

AVAILABLE_TARGETS+=fwbehave
HELP_fwbehave=Build firmware binaries for SpartanMC behavioural simulation

include $(SPARTANMC_ROOT)/src/scripts/make/common/localvars.mk

D_FWHOME:=$(JCONFIG_FW_$(FW_ID)_SOURCE)

MAIN:=
SOURCE_DIR:=
OBJECTS:=
INCLUDE_DIRS:=
DEFINES_B:=
LDFLAGS_B:=
CFLAGS_B:=
CONFIG_BUILD_MK:=$(D_FWHOME)/config-build.mk

-include $(CONFIG_BUILD_MK)

D_SPMC:=$(SPARTANMC_ROOT)/spartanmc
D_BUILD:=$(BUILD_DIR)/behavioural/$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM)
D_SRC:=$(abspath $(D_FWHOME)/$(SOURCE_DIR))
LD_INC:=$(abspath $(addprefix $(D_FWHOME)/,$(INCLUDE_DIRS)) $(PROJECT_DIR) $(D_SPMC)/lib_sim)
L_DEFINES:=$(DEFINES_B) BEHAVIOURAL
F_SYSTEM_H:=$(GENERATED_FILES_DIR)/$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM).h
C_INCLUDE_PATH:=$(SPARTANMC_ROOT)/spartanmc/lib_obj/include
LF_C:=$(foreach obj,$(OBJECTS),$(wildcard $(D_SRC)/$(obj).c))
LF_INC:=$(foreach dir,$(LD_INC) $(C_INCLUDE_PATH),$(wildcard $(dir)/*.h))
LF_OBJ:=$(foreach obj,$(notdir $(basename $(sort $(LF_C)))),$(D_BUILD)/$(obj).o)
P_MAIN:=$(D_BUILD)/$(basename $(notdir $(MAIN)))
F_BIN:=$(P_MAIN)

fwbehave: $(F_BIN)

$(F_BIN): $(SPARTANMC_LIB_SIM)

# flags
_CFLAGS:=$(CFLAGS_B)
_CFLAGS+=-Werror=implicit-function-declaration -Wmissing-include-dirs
_CFLAGS+=$(foreach incdir, $(LD_INC),-I$(incdir))
_CFLAGS+=$(foreach define, $(L_DEFINES),-D$(define))
_CFLAGS+=-DSBI_CORE_ID=$(JCONFIG_$(JCONFIG_FW_$(FW_ID)_SUBSYSTEM)_ID)

_LDFLAGS:=-pthread
ifdef LDFLAGS
  _LDFLAGS+=$(LDFLAGS_B)
endif
_LDFLAGS+=-pthread -L$(SPARTANMC_ROOT)/spartanmc/lib_sim -lsim -lrt -Wl,--defsym=SBI_CORE_ID=$(SBI_CORE_ID)

-include $(D_BUILD)/fw_build_$(FW_ID)_b.mk
MAKEFILE_LIST_FIRMWARE:=$(MAKEFILE_LIST_FIRMWARE) $(D_BUILD)/fw_build_$(FW_ID)_b.mk

# define set of variables
FW_BUILD_B_LOCAL_VARIABLES:= \
	D_BUILD \
	D_SRC \
	_CFLAGS \
	LF_OBJ \
	P_MAIN \
	LF_INC \
	CONFIG_BUILD_MK \
	F_BIN \
	_LDFLAGS

$(D_BUILD):
	@mkdir -p $@

$(call local_export,$(FW_ID)_b,$(FW_BUILD_B_LOCAL_VARIABLES))

$(D_BUILD)/fw_build_%.mk: $(SPARTANMC_ROOT)/src/scripts/make/template/simb-build.mk $(filter-out $(D_BUILD)/fw_build_%.mk,$(MAKEFILE_LIST)) $(LF_C) | $(D_BUILD)
	@\
	$(call local_import,$*,$(FW_BUILD_B_LOCAL_VARIABLES)) \
	$(SPARTANMC_ROOT)/src/scripts/sh/rdtmp $< > $@
