#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Hardware build process
#

# Project file for XST

$(eval $(MAKEFILE_PROLOG))

include $(SPARTANMC_ROOT)/src/scripts/make/project/checks.mk

# System wide UCF file

IP_DIR:=$(BUILD_DIR)/ipcore/
TEMP_DIR:=$(BUILD_DIR)/temp
XST_LIB_DIR:=$(BUILD_DIR)/xst_lib
REPORT_DIR:=$(BUILD_DIR)/reports

XCO_SOURCE_FILES:=$(foreach DIR,$(MODULE_SOURCE_DIRS),$(wildcard $(DIR)/*.xco)) 
XCO_GEN_SOURCE_FILES:=$(wildcard $(PROJECT_DIR)/system/*.xco)

XCO_VIRTUAL_FILES:=$(addprefix $(IP_DIR),$(addsuffix .ipcore,$(basename $(notdir $(XCO_GEN_SOURCE_FILES)))))

PRJ:=$(GENERATED_FILES_DIR)/spartanmc.prj
XST:=$(GENERATED_FILES_DIR)/spartanmc.xst

# Commands
SYN_PROCESS:=Synthesis
XCO_PROCESS:=IP-CoreGeneration
NGDBUILD_PROCESS:=Translate
MAP_PROCESS:=Mapping
PAR_PROCESS:=Place and route
NETGEN_PROCESS:=Netlist generation
BITGEN_PROCESS:=Bitfile generation
PROGRAM_PROCESS:=Program bitfile into target device

SYN_CMD:=$(XILINX_XST_ENV) $(XILINX_XST) -ifn $(XST) -ofn $(BUILD_ROOT).syr -intstyle xflow
NGDBUILD_CMD:=$(XILINX_NGDBUILD_ENV) $(XILINX_NGDBUILD) $(JCONFIG_OPT_TRANSLATE) -uc $(UCF) -intstyle xflow $(XNGDFLAGS) $(NGC) -sd $(IP_DIR) $(NGD)
COREGEN_CMD:=$(XILINX_XCO_ENV) $(XILINX_XCO) -p $(CGP) -b 
MAP_CMD:=$(XILINX_MAP_ENV) $(XILINX_MAP) -w -detail -intstyle xflow $(JCONFIG_OPT_MAP) -o $(NCD_MAP) $(NGD) $(PCF)
PAR_CMD:=$(XILINX_PAR_ENV) $(XILINX_PAR) -w -intstyle xflow $(JCONFIG_OPT_PAR) $(NCD_MAP) $(NCD_PAR) $(PCF)
NETGEN_CMD:=$(XILINX_NETGEN_ENV) $(XILINX_NETGEN) -w -intstyle xflow $(JCONFIG_OPT_NETGEN) -sdf_anno false -pcf $(PCF) -ofmt verilog -sim $(NCD_PAR) $(TIMESIM_REGULAR)
BITGEN_CMD:=$(XILINX_BITGEN_ENV) $(XILINX_BITGEN) -w -intstyle xflow $(JCONFIG_OPT_BITGEN) -f $(BIT_UT) $(NCD_PAR) $(BIT_REG) $(PCF)
PROGRAM_CMD:=$(XILINX_IMPACT_ENV) $(XILINX_IMPACT) -batch $(IMPACT_CMD) $(JCONFIG_OPT_PROGRAM)

# Targets
.PHONY: all ipcores synthesis translate mapping placeroute constraints program timing_analysis chipscope

AVAILABLE_TARGETS:=all ipcores synthesis translate mapping placeroute program synthesis constraints timing_analysis chipscope $(AVAILABLE_TARGETS)
HELP_all=Build whole system $(OBRACE)create bitfile$(CBRACE)
HELP_ipcores=Generate all IPCores $(OBRACE)coregen$(CBRACE)
HELP_synthesis=Run synthesis $(OBRACE)xst$(CBRACE)
HELP_translate=Run translate $(OBRACE)ngdbuild$(CBRACE)
HELP_placeroute=Run place and route $(OBRACE)par$(CBRACE)
HELP_mapping=Run mapping $(OBRACE)map$(CBRACE)
HELP_constraints=Build user constraints file $(OBRACE)*.ucf$(CBRACE)
HELP_program=Program current bitfile into FPGA device $(OBRACE)impact$(CBRACE)
HELP_timing_analysis=Run Xilinx Timing Analyzer on current design
HELP_chipscope=Launch Xilinx ChipScope Analyzer

all: $(BIT_REG)
	cp $(BIT_REG) $(BIT)
synthesis: $(NGC)
translate: $(NGD)
mapping: $(NCD_MAP) $(PCF)
placeroute: $(NCD_PAR)
constraints: $(UCF)

# Directories
$(TEMP_DIR) $(XST_LIB_DIR) $(REPORT_DIR) $(IP_DIR):
	@mkdir -p $@


# Sythesis (XST)
$(NGC): $(PRJ) $(JCONFIG_HDL_SOURCES) $(PROJECT_MK) | $(TEMP_DIR) $(XST_LIB_DIR) $(REPORT_DIR) $(XILINX_XST) $(IP_DIR)
	@touch $(LSO)
	@$(call BUILD,SYN)
	@mv $(BUILD_ROOT).syr $(REPORT_DIR)

$(eval $(call EXPORT,JCONFIG_PRJNAME))
$(eval $(call EXPORT,JCONFIG_TOP_MODULE))
$(eval $(call EXPORT,JCONFIG_OPT_SYNTHESIS))

$(XST): $(PRJ)
#	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/spartanmc.xst > $@
	@$(ECHO_TARGET_DONE)

$(PRJ): $(XCO) $(XCO_VIRTUAL_FILES) $(PROJECT_MK) $(MAKEFILE_LIST_HARDWARE) $(JCONFIG_HDL_SOURCES) | $(GENERATED_FILES_TIMESTAMP) $(BUILD_DIR) $(IP_DIR)
	@touch $(PRJ)
	@$(ECHO_TARGET_DONE)

# Translate
#ifneq ($(JCONFIG_VERSION),1)
#    $(NGD): $(NGC) $(UCF) | $(XILINX_NGDBUILD)
#else
$(NGD): $(NGC) $(UCF) $(PROJECT_MK)| $(XILINX_NGDBUILD)
#endif
	@echo $(XCO_V)
	@echo $(XCO_NGC)
	@rm -rf xlnx_auto_0_xdb
	@$(call BUILD,NGDBUILD)
	@mv $(BUILD_ROOT).bld $(REPORT_DIR)

$(eval $(call EXPORT,JCONFIG_MODULE_IDS))
$(eval $(call EXPORT,JCONFIG_INSTANCE_MAP))
$(eval $(call EXPORT,JCONFIG_CONSTRAINTS_MAP))
$(eval $(call EXPORT,JCONFIG_PATH_MAP))
$(eval $(call EXPORT,SYSCLK_INPUT_FREQ_MHZ))


$(eval $(call EXPORT,JCONFIG_COREGEN_CONSTRAINTS))
$(eval $(call EXPORT,JCONFIG_COREGEN_CONSTRAINTS_MODIFIERS))

$(UCF): $(UCF_PROGRAMRAM) $(JCONFIG_CONSTRAINTS_FILE) $(JCONFIG_COREGEN_CONSTRAINTS)
	@$(SH_SCRIPTS)/mkucf $(JCONFIG_CONSTRAINTS_FILE) $(UCF_PROGRAMRAM) > $@
	@$(ECHO_TARGET_DONE)

#COREGEN
ipcores: $(XCO) $(XCO_VIRTUAL_FILES) | $(IP_DIR)
	
$(XCO): $(XCO_SOURCE_FILES) $(CGP)
	@$(SH_SCRIPTS)/buildipcores $(COREGEN_CMD) $(XCO_SOURCE_FILES)
	@touch $(XCO)
	@$(ECHO_TARGET_DONE)
	
$(IP_DIR)%.ipcore : $(PROJECT_DIR)/system/%.xco
	$(COREGEN_CMD) $<
	@touch $@
	@$(ECHO_TARGET_DONE)

$(eval $(call EXPORT,TEMPDIR,:=$(call MAKE_RELATIVE,$(IP_DIR),$(notdir $(BUILD_DIR))/)))

$(CGP): | $(IP_DIR)
	@$(SH_SCRIPTS)/mkcgp $(SPARTANMC_TEMPLATES)/coregen.cgp $(JCONFIG_OPT_TRANSLATE) > $@
	@$(ECHO_TARGET_DONE)

# Map
$(PCF) $(NCD_MAP): $(NGD) $(PROJECT_MK)| $(XILINX_MAP)
	@$(call BUILD,MAP)
	@mv $(BUILD_ROOT)_map.mrp $(REPORT_DIR)

# Place and route
$(NCD_PAR): $(NCD_MAP) $(PCF) $(PROJECT_MK)| $(XILINX_PAR) $(ORG_DIR)
	@$(call BUILD,PAR)
	cp $(NCD_PAR) $(NCD_PAR_ORG)
	@mv $(BUILD_ROOT)_par.par $(REPORT_DIR)
	$(TIMING_VIOLATION_CALL) --parsePlaceRouteTiming

$(eval $(call EXPORT,FILE_OPTION,:=$(call MAKE_RELATIVE,$(BIT),$(notdir $(BUILD_DIR))/)))
$(eval $(call EXPORT,JCONFIG_JTAGID))

# org dir
$(ORG_DIR):
	mkdir $(ORG_DIR)

# Bitfile
$(BIT_REG): $(PCF) $(NCD_PAR) $(BIT_UT) $(PROJECT_MK)| $(XILINX_BITGEN)
	@$(call BUILD,BITGEN)
	@mv $(BGN) $(DRC) $(REPORT_DIR)
	#MSK File does not get changed by UpdateRam stuff --> Move once right now
	@mv $(MSK_REG) $(MSK)
	@printf "\n\n"
	@$(call TARGET_MESSAGE,System synthesis complete. You may now run 'make program' to configure your FPGA device with the current design.)
	@printf "\n"


$(BIT_UT): $(TEMPLATE_UT_SRC)
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/spartanmc.ut > $@
	@$(ECHO_TARGET_DONE)

# Program

$(eval $(call EXPORT,JCONFIG_JTAGID))
$(eval $(call EXPORT,BUILD_DIR))
$(eval $(call EXPORT,BIT))
$(eval $(call EXPORT,JCONFIG_OPT_IMPACT_PROGRAM_SWITCH))

$(IMPACT_CMD): $(TEMPLATE_IMPACT_CMD_SRC) $(PROJECT_MK) Makefile
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/impact.cmd > $@
	@$(ECHO_TARGET_DONE)

program:  timingViolation $(IMPACT_CMD) | $(XILINX_IMPACT)
	@if test ! -f $(BIT); then \
		$(call TARGET_ERROR,Please use 'make all' first to generate the necessary bitfile!); \
	fi
	@$(call BUILD,PROGRAM)
	@mv $(BUILD_DIR)/_impactbatch.log $(REPORT_DIR)

timing_analysis: $(PCF) $(NCD_PAR)
	$(XILINX_TIMINGAN_ENV) $(XILINX_TIMINGAN) $(NCD_PAR) $(PCF)

chipscope: $(CPJ)
	@$(XILINX_FIRMWARE_PATH)/analyzer -project $(CPJ)


planAhead: $(PCF) $(NCD_PAR)
	@echo "create_project -force project_1 $(BUILD_DIR)/planAheadProject -part $(JCONFIG_DEVICE)" > $(BUILD_DIR)/planAheadImport.tcl
	@echo "set_property design_mode GateLvl [current_fileset]" >> $(BUILD_DIR)/planAheadImport.tcl
	@echo "add_files -norecurse $(NGC)" >> $(BUILD_DIR)/planAheadImport.tcl
	@echo "import_files -force -norecurse" >> $(BUILD_DIR)/planAheadImport.tcl
	@echo "import_files -fileset constrs_1 -force -norecurse $(UCF)" >> $(BUILD_DIR)/planAheadImport.tcl
	@echo "import_as_run -run impl_1 $(NCD_PAR)" >> $(BUILD_DIR)/planAheadImport.tcl
	@echo "open_run impl_1" >> $(BUILD_DIR)/planAheadImport.tcl
	cd "$(BUILD_DIR)"; $(XILINX_PLANAHEAD_ENV) $(XILINX_PLANAHEAD) -source $(BUILD_DIR)/planAheadImport.tcl

#Ram Updating



#TARGETS
AVAILABLE_TARGETS+=bitgen updateRam
HELP_bitgen:= Build a new Bitfile with new Firmware $(OBRACE)*.bit$(CBRACE) [directory of XDL-Replace-File can specified by PATH]
HELP_updateRam:= Build a new Bitfile with new Firmware $(OBRACE)*.bit$(CBRACE)
ARGS_bitgen:=+path=PATH

#FILES
BIT_FILE:=$(BUILD_ROOT).bit

#Commands
NCD2XDL_PROCESS:=Conversion NCD2XDL
XDL_REPLACE_PROCESS:=REPLACE PARTS in XDL of NCD-FILE
SPH_BUILD_PROCESS:=BUILD SPH-FILE
XDL2NCD_PROCESS:=Conversion XDL2NCD
REPLACERAM_PROCESS:=Replace RAM


#Variables
XDL_PAR_DELTA:=$(XDL_PAR)_DELTA
BIT_DELTA:=$(BIT)_DELTA
NCD_PAR_DELTA:=$(NCD_PAR)_DELTA
BIT_REPLACED:=$(BUILD_ROOT)_replaced.bit

#ifeq ($(origin +path),command line)
#	XDL_REPLACE_CMD:=$(SPARTANMC_INITRAMJ) -o $(XDL_PAR) -i XDL $(abspath $(+path))
#else
#	XDL_REPLACE_CMD:=$(SPARTANMC_INITRAMJ) -o $(XDL_PAR) -i $(ELF_FILES)
#endif
XDL_REPLACE_CMD:=$(SPARTANMC_INITRAMJ) --xdlin $(XDL_PAR) --xdlout $(XDL_PAR_DELTA) -m $(MEM_MAP)

NCD2XDL_CMD:=$(XILINX_XDL_ENV) $(XILINX_XDL) -ncd2xdl -noformat $(NCD_PAR_ORG) $(XDL_PAR)

XDL2NCD_CMD:=$(XILINX_XDL_ENV) $(XILINX_XDL) -xdl2ncd $(XDL_PAR_DELTA) $(NCD_PAR)

REPLACERAM_CMD:=$(SPARTANMC_INITRAMJ) --xdlin $(XDL_PAR) -m $(MEM_MAP) --bitin $(BIT_REG) --bitout $(BIT_REPLACED) --location $(LOCATION)

.PHONY: bitgen

$(XDL_PAR): $(NCD_PAR_ORG)
#XDL-File erstellung aus NCD-File
	@$(call BUILD,NCD2XDL)
	
	
xdl: $(XDL_PAR)

$(XDL_PAR_DELTA): $(XDL_PAR) $(ELF_FILES)
#xdl replace xdl file
	@$(call BUILD,XDL_REPLACE)

$(NCD_PAR_DELTA): $(XDL_PAR_DELTA) $(XILINX_XDL_ENV) $(XILINX_XDL)
#xdl2ncd
	@$(call BUILD,XDL2NCD)
	touch $(NCD_PAR_DELTA)

$(BIT_DELTA): $(NCD_PAR_DELTA)
#bitbuild
	@$(call BUILD,BITGEN)
	touch $(BIT_DELTA)

bitgen:timingViolation $(BIT_DELTA)
	cp $(BIT_REG) $(BIT)

$(BIT_REPLACED): $(XDL_PAR) $(ELF_FILES)
	@$(call BUILD,REPLACERAM)	

updateRam:timingViolation $(BIT_REPLACED)
	cp $(BIT_REPLACED) $(BIT)
	

#Rapid Prototyping


$(RAPID_XDL): $(ELF_FILES) $(PROJECT_MK)  $(JCONFIG_XML_FILE) | $(GENERATED_FILES_TIMESTAMP) $(BUILD_DIR)
	$(SPARTANMC_RAPIDSOC) design --project-dir $(PROJECT_DIR) --config-file $(JCONFIG_XML_FILE) $(+args)

$(RAPID_XDL_OWNROUTER): $(ELF_FILES) $(PROJECT_MK)  $(JCONFIG_XML_FILE) | $(GENERATED_FILES_TIMESTAMP) $(BUILD_DIR)
	$(SPARTANMC_RAPIDSOC) design --project-dir $(PROJECT_DIR) --config-file $(JCONFIG_XML_FILE) --route $(+args)

$(RAPID_MOD) : $(NCD_PAR) $(XDL_PAR)
	$(SPARTANMC_RAPIDSOC) module $(+args)

$(RAPID_NCD): $(RAPID_XDL)
	$(XILINX_XDL_ENV) $(XILINX_XDL) -xdl2ncd $(RAPID_XDL) $(RAPID_NCD)


$(RAPID_NCD_OWNROUTER): $(RAPID_XDL_OWNROUTER)
	$(XILINX_XDL_ENV) $(XILINX_XDL) -xdl2ncd $(RAPID_XDL_OWNROUTER) $(RAPID_NCD_OWNROUTER)

$(RAPID_NCD_ROUTED): $(RAPID_NCD) $(BUILD_DIR)/rapidSoc.pcf
	$(XILINX_PAR_ENV) $(XILINX_PAR) -kw $(RAPID_NCD) $(RAPID_NCD_ROUTED)

$(BUILD_DIR)/comparison.ncd: $(BUILD_DIR)/comparison.xdl
	$(XILINX_XDL_ENV) $(XILINX_XDL) -xdl2ncd $(BUILD_DIR)/comparison.xdl $(BUILD_DIR)/comparison.ncd


$(BUILD_DIR)/comparison_routed.ncd: $(BUILD_DIR)/comparison.ncd $(BUILD_DIR)/comparison.pcf
	$(XILINX_PAR_ENV) time $(XILINX_PAR) -kw $(BUILD_DIR)/comparison.ncd $(BUILD_DIR)/comparison_routed.ncd

#comparison: $(BUILD_DIR)/comparison_routed.ncd
#	$(XILINX_BITGEN_ENV) $(XILINX_BITGEN) -w $(BUILD_DIR)/comparison_routed.ncd $(BIT)

comparison: $(BUILD_DIR)/comparison.ncd
	$(XILINX_BITGEN_ENV) $(XILINX_BITGEN) -w $(BUILD_DIR)/comparison.ncd $(BIT)


$(RAPID_BIT_OWNROUTER): $(RAPID_NCD_OWNROUTER)
	$(XILINX_BITGEN_ENV) $(XILINX_BITGEN) -w $(RAPID_NCD_OWNROUTER) $(RAPID_BIT)

$(RAPID_BIT_ROUTED): $(RAPID_NCD_ROUTED)
	$(XILINX_BITGEN_ENV) $(XILINX_BITGEN) -w $(RAPID_NCD_ROUTED) $(RAPID_BIT_ROUTED)
	$(TIMING_VIOLATION_CALL) --parseRapidSocTiming

.PHONY: rapidSoc rapidSocOwnRouter rapidModule
rapidModule: $(RAPID_MOD)


rapidSocOwnRouter: $(RAPID_BIT_OWNROUTER)
	cp $(RAPID_BIT_OWNROUTER) $(BIT)
rapidSoc: $(RAPID_BIT_ROUTED)
	cp $(RAPID_BIT_ROUTED) $(BIT)

$(TIMESIM_RAPIDSOC) $(TIMESIM_SDF_RAPIDSOC): $(RAPID_NCD_ROUTED)
	$(XILINX_NETGEN_ENV) $(XILINX_NETGEN) -w -intstyle xflow $(JCONFIG_OPT_NETGEN) -sdf_anno false -ofmt verilog -sim $(RAPID_NCD_ROUTED) $(TIMESIM_RAPIDSOC)

rapidSocTimesim: $(TIMESIM_RAPIDSOC) $(TIMESIM_SDF_RAPIDSOC)
	cp $(TIMESIM_RAPIDSOC) $(TIMESIM)
	cp $(TIMESIM_SDF_RAPIDSOC) $(TIMESIM_SDF)
