#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

# Lattice toolchain
PAR_NCD:=$(BUILD_ROOT)_par.ncd
MAP_NCD:=$(BUILD_ROOT)_map.ncd
MAP_PRF:=$(BUILD_ROOT)_map.prf
NGD:=$(BUILD_ROOT).ngd
NGO:=$(BUILD_ROOT).ngo
PRJ:=$(BUILD_ROOT).tcl
SYNPROJ:=$(BUILD_ROOT).synproj
PGR_CMDFILE:=$(BUILD_ROOT).xcf
PGR_VERFILE:=$(BUILD_ROOT)_ver.xcf
bindir:=$(LATTICEDIR)/bin/lin64
USR_LPF:=$(abspath $(JCONFIG_USR_CONSTRAINTS_FILE))

# synthesis tool
#LATTICE_SYNTHESIS_TOOL:=synplify
LATTICE_SYNTHESIS_TOOL:=lse

$(eval $(call EXPORT,HDL))
$(eval $(call EXPORT,LATTICEDIR))
$(eval $(call EXPORT,BITFILE,:=$(BUILD_ROOT).bit))
$(eval $(call EXPORT,EDIF,:=$(BUILD_ROOT).edi))
$(eval $(call EXPORT,LSE_EDIF,:=$(BUILD_ROOT).edf))
$(eval $(call EXPORT,LSE_NGO,:=$(BUILD_ROOT).ngo))
$(eval $(call EXPORT,SYNP_LOG,:=$(BUILD_ROOT).srf))
$(eval $(call EXPORT,LSE_LOG,:=$(BUILD_ROOT).log))
$(eval $(call EXPORT,LSE_RAMINIT,:=$(BUILD_ROOT)_lse_programram.v))
$(eval $(call EXPORT,USR_LPF))

# constraints
LPF:=$(BUILD_ROOT).lpf
#LPFS:=$(LPF)s
#ifeq "$(JCONFIG_PROJECT_CONSTRAINTS)" ""
#USR_LPF:=$(PWD)/project.lpf
#else
USR_LPF:=$(abspath $(JCONFIG_USR_CONSTRAINTS_FILE))
#endif
#CONSTRAINT_FILES=$(shell test -e $(LPFS) && cat $(LPFS))
#CONSTRAINT_FILES=$(shell test -e $(LPF) && cat $(LPF))

# Targets
HELP_all:=Build the whole system $(OBRACE)alias for bitfile$(CBRACE)
HELP_synthesis:=Run synthesis $(OBRACE)lattice synthesis engine$(CBRACE)
HELP_translate:=Run translate $(OBRACE)ngdbuild$(CBRACE)
HELP_placeroute:=Run place and route $(OBRACE)mpartrce$(CBRACE)
HELP_mapping:=Run mapping $(OBRACE)map$(CBRACE)
HELP_constraints:=Create logical prefrences file $(OBRACE)*.lpf$(CBRACE)
HELP_bitfile:=Create bitfile
HELP_program:=Program current bitfile into FPGA device $(OBRACE)pgrcmd$(CBRACE)
.SUFFIXES:
.PHONY: all program synthesis translate mapping placeroute constraints bitfile verify
AVAILABLE_TARGETS+=all program synthesis translate mapping placeroute constraints bitfile

all: bitfile

bitfile: $(BITFILE)

# program: $(PGR_CMDFILE)
# 	@if [ ! -e $(BITFILE) ]; then printf "\nError: No bitfile present! Generate bitfile with 'make bitfile' \n\n"; exit 1; fi
# 	@$(SPARTANMC_DETACH) -d -v403 -p6010
# 	@rm -f $(BUILD_ROOT)_pgr.log
# 	@cd $(BUILD_DIR); \
# 	bindir=$(bindir) . $(bindir)/diamond_env; \
# 	LD_PRELOAD=$(bindir)/libftd2xx.so $(LATTICE_PGRCMD) -logfile $(BUILD_ROOT)_pgr.log -infile $(PGR_CMDFILE) 2>/dev/null; \
# 	export E=$$?; \
# 	$(SPARTANMC_DETACH) -a -v403 -p6010; \
# 	exit $$E

# verify: $(PGR_VERFILE)
# 	@if [ ! -e $(BITFILE) ]; then printf "\nError: No bitfile present! Generate bitfile with 'make bitfile' \n\n"; exit 1; fi
# 	@$(SPARTANMC_DETACH) -d -v403 -p6010
# 	@rm -f $(BUILD_ROOT)_ver.log
# 	@cd $(BUILD_DIR); \
# 	bindir=$(bindir) . $(bindir)/diamond_env; \
# 	LD_PRELOAD=$(bindir)/libftd2xx.so $(LATTICE_PGRCMD) -logfile $(BUILD_ROOT)_ver.log -infile $(PGR_VERFILE) 2>/dev/null; \
# 	export E=$$?; \
# 	$(SPARTANMC_DETACH) -a -v403 -p6010; \
# 	exit $$E

program: $(PGR_CMDFILE)
	@if [ ! -e $(BITFILE) ]; then printf "\nError: No bitfile present! Generate bitfile with 'make bitfile' \n\n"; exit 1; fi
	@rm -f $(BUILD_ROOT)_pgr.log
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_PGRCMD) -logfile $(BUILD_ROOT)_pgr.log -infile $(PGR_CMDFILE) 2>/dev/null; \
	export E=$$?; \
	exit $$E

verify: $(PGR_VERFILE)
	@if [ ! -e $(BITFILE) ]; then printf "\nError: No bitfile present! Generate bitfile with 'make bitfile' \n\n"; exit 1; fi
	@rm -f $(BUILD_ROOT)_ver.log
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_PGRCMD) -logfile $(BUILD_ROOT)_ver.log -infile $(PGR_VERFILE) 2>/dev/null; \
	export E=$$?; \
	exit $$E

ifeq ($(LATTICE_SYNTHESIS_TOOL), lse)
synthesis: $(LSE_NGO)
else
synthesis: $(EDIF)
endif
translate: $(NGD)
mapping: $(MAP_NCD)
placeroute: $(PAR_NCD)
constraints: $(LPF)

$(PGR_CMDFILE): $(JCONFIG_PROJECT_MK_FILE) $(JCONFIG_PROGRAM_MK_FILE) | $(BUILD_DIR)
	$(SH_SCRIPTS)/mkpgr "Fast Program" $@

$(PGR_VERFILE): $(JCONFIG_PROJECT_MK_FILE) $(JCONFIG_PROGRAM_MK_FILE) | $(BUILD_DIR)
	$(SH_SCRIPTS)/mkpgr "Verify Only" $@

$(BITFILE): $(PAR_NCD)
#	echo  -n "hier medit ausfuehren";
#	read ;
#	echo "continuing";
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_BITGEN) -w $< $@ $(MAP_PRF)

$(PAR_NCD): $(MAP_NCD)
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_PAR) -w $(JCONFIG_OPT_PAR) $(MAP_NCD) $@ $(MAP_PRF)

$(MAP_NCD): $(NGD) $(LPF)
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_MAP) $(JCONFIG_OPT_MAP) $< $(LPF) -o $@ -pr $(MAP_PRF)

$(NGD): $(NGO)
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_NGDBUILD) $(JCONFIG_OPT_TRANSLATE) -p $(LATTICEDIR)/ispfpga/$(JCONFIG_OPT_TRANSLATE_LIB)/data $< $@
	
$(LPF): MEMGEN $(JCONFIG_PROJECT_MK_FILE) $(GENERATED_FILES_TIMESTAMP) $(USR_LPF) $(JCONFIG_CONSTRAINTS_FILE) | $(BUILD_DIR)
	@$(SH_SCRIPTS)/mklpf $(JCONFIG_CONSTRAINTS_FILE) $(USR_LPF) > $@
	@$(ECHO_TARGET_DONE)

ifeq ($(LATTICE_SYNTHESIS_TOOL), lse)
$(NGO): $(LSE_NGO)
#$(NGO): $(LSE_EDIF)
#	@cd $(BUILD_DIR); \
#	bindir=$(bindir) . $(bindir)/diamond_env; \
#	$(LATTICE_EDIF) $(JCONFIG_OPT_EDIF) -path $(PWD) $(LSE_EDIF) $@
else
$(NGO): $(EDIF)
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_EDIF) $(JCONFIG_OPT_EDIF) -path $(PWD) $(EDIF) $@
endif

$(EDIF): $(PRJ)
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_SYNP) -nolog -prj $(PRJ);

$(LSE_NGO): $(SYNPROJ)
	@cd $(BUILD_DIR); \
	bindir=$(bindir) . $(bindir)/diamond_env; \
	$(LATTICE_LSE) -f $(SYNPROJ);

$(PRJ): $(JCONFIG_PROJECT_MK_FILE) $(LPF) $(JCONFIG_PROGRAM_MK_FILE) $(GENERATED_FILES_TIMESTAMP) $(HDL) | $(BUILD_DIR)
	$(SH_SCRIPTS)/mktcl $(PRJ)

$(SYNPROJ): $(JCONFIG_PROJECT_MK_FILE) $(LPF) $(JCONFIG_PROGRAM_MK_FILE) $(GENERATED_FILES_TIMESTAMP) $(HDL) | $(BUILD_DIR)
	$(SH_SCRIPTS)/mksynproj $(SYNPROJ)

MEMGEN: mem

mem: $(MEM_MAP) $(ELF_FILES) | $(BUILD_DIR)
	@$(call BUILD,ELf2MEM)
	$(SPARTANMC_INITRAMJ) --lse build/spartanmc_lse_programram.v --memory $<
	@$(ECHO_TARGET_DONE)
