#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Main makefile for whole project build
#

ifeq ($(MAIN_MK),)

MAIN_MK:=$(SPARTANMC_ROOT)/src/scripts/make/project/main.mk
include $(SPARTANMC_ROOT)/src/scripts/make/common/base.mk

else

PROJECT_DIR:=$(realpath .)
GENERATED_FILES_DIR:=$(PROJECT_DIR)/system
PROJECT_MK:=$(wildcard $(GENERATED_FILES_DIR)/project.mk)
HDL_SOURCES_MK:=$(GENERATED_FILES_DIR)/hdl_sources.mk
#GENERATED_FILES_TIMESTAMP:=$(GENERATED_FILES_DIR)/.timestamp

ifneq ($(PROJECT_MK),)
  include $(PROJECT_MK)
  REAL_JCONFIG_SYSTEM_DIR:=$(realpath $(JCONFIG_SYSTEM_DIR))
  ifeq ($(REAL_JCONFIG_SYSTEM_DIR),$(GENERATED_FILES_DIR))
    PROGRAM_MK:=$(abspath $(GENERATED_FILES_DIR)/firmwares.mk)
    include $(PROGRAM_MK)
    include $(HDL_SOURCES_MK)
  else
    $(call MAKE_MSG,Project was moved. Not loading firmware configuration)
  endif
endif

include $(SPARTANMC_ROOT)/src/scripts/make/common/buildsetup.mk

# Directories
BUILD_ROOT:=$(BUILD_DIR)/$(JCONFIG_PRJNAME)

# Files
MODULE_SOURCE_DIRS:=$(JCONFIG_MODULE_DIRS)

CLOSING_BRACKET=)

INFOTEXT += \
	Project:\s$(realpath .) \
	Device:\s$(JCONFIG_TARGET_PLATFORM) \
	Firmware: \
	$(foreach ID,$(JCONFIG_FIRMWARE_IDS),\s\s$(JCONFIG_FW_$(ID)_SOURCE) )
		
ifneq ($(MAKECMDGOALS),)
	ifeq ($(words $(MAKECMDGOALS)),1)
		TARGET:=target
	else
		TARGET:=targets
	endif
	INFOTEXT += Building\s$(TARGET):\s$(subst $(SPACE),\s,$(MAKECMDGOALS))
endif

include $(SPARTANMC_ROOT)/src/scripts/make/common/functions.mk

include $(SPARTANMC_ROOT)/src/scripts/make/project/filedefs.mk
include $(SPARTANMC_ROOT)/src/scripts/make/project/newfirmware.mk
include $(SPARTANMC_ROOT)/src/scripts/make/project/jconfig.mk
include $(SPARTANMC_ROOT)/src/scripts/make/project/spmcsim.mk
include $(SPARTANMC_ROOT)/src/scripts/make/project/microstreams.mk
include $(SPARTANMC_ROOT)/src/scripts/make/project/microstreams-jconfig-import.mk
include $(SPARTANMC_ROOT)/src/scripts/make/project/autoPerf.mk
include $(SPARTANMC_ROOT)/src/scripts/make/project/evaluation.mk
include $(SPARTANMC_ROOT)/src/scripts/make/project/loopOptimizer.mk

MAKEFILE_LIST_HARDWARE=$(filter-out $(MAKEFILE_LIST_FIRMWARE),$(MAKEFILE_LIST))



include $(SPARTANMC_ROOT)/src/scripts/make/project/newsim.mk

# Toolchain selection
ifneq ($(PROJECT_MK),)
  include $(SPARTANMC_ROOT)/src/scripts/make/project/firmware.mk
  ifeq ($(JCONFIG_TOOLCHAIN),ise)
    ifeq ($(ENABLE_ISE),yes)
      include $(SPARTANMC_ROOT)/src/scripts/make/project/ise-toolchain.mk
    else
      WARNINGS+=toolchain
      W_toolchain:='Requested ISE toolchain but toolchain not available.'
      include $(SPARTANMC_ROOT)/src/scripts/make/project/no-toolchain.mk
    endif
  else
  ifeq ($(JCONFIG_TOOLCHAIN),lattice)
    ifeq ($(ENABLE_DIAMOND),yes)
      include $(SPARTANMC_ROOT)/src/scripts/make/project/lattice-toolchain.mk
    else
      WARNINGS+=toolchain
      W_toolchain:='Requested Lattice toolchain but toolchain not available.'
      include $(SPARTANMC_ROOT)/src/scripts/make/project/no-toolchain.mk
    endif
  else
  ifeq ($(JCONFIG_TOOLCHAIN),quartus)
    ifeq ($(ENABLE_QUARTUS),yes)
      include $(SPARTANMC_ROOT)/src/scripts/make/project/quartus-toolchain.mk
    else
      WARNINGS+=toolchain
      W_toolchain:='Requested Quartus toolchain but toolchain not available.'
      include $(SPARTANMC_ROOT)/src/scripts/make/project/no-toolchain.mk
    endif
  else
  ifeq ($(JCONFIG_TOOLCHAIN),vivado)
    ifeq ($(ENABLE_VIVADO),yes)
      include $(SPARTANMC_ROOT)/src/scripts/make/project/vivado-toolchain.mk
    else
      WARNINGS+=toolchain
      W_toolchain:='Requested Vivado toolchain but toolchain not available.'
      include $(SPARTANMC_ROOT)/src/scripts/make/project/no-toolchain.mk
    endif
  else
    include $(SPARTANMC_ROOT)/src/scripts/make/project/no-toolchain.mk
  endif
  endif
  endif
  endif
  ifeq ($(ENABLE_MODELSIM),yes)
    include $(SPARTANMC_ROOT)/src/scripts/make/project/simulation.mk
  endif
endif

CLEAN_LIST+=$(call MAKE_RELATIVE,$(wildcard $(BUILD_DIR)/*))
CLOBBER_LIST+=$(GENERATED_FILES)

include $(SPARTANMC_ROOT)/src/scripts/make/common/clean.mk
include $(SPARTANMC_ROOT)/src/scripts/make/common/what.mk

# Error handling
$(GENERATED_FILES_DIR)/%:
	@$(call TARGET_MESSAGE_BOLD,System configuration file '$(@F)' is missing. Please run system builder ('make jconfig')!)

$(XILINX_XCO) $(XILINX_XST) $(XILINX_NGDBUILD) $(XILINX_MAP) $(XILINX_PAR) $(XILINX_NETGEN) $(XILINX_BITGEN) $(XILINX_IMPACT):
	@which $@ > /dev/null || \
	$(call TARGET_ERROR,Required program '$(@F)' not found. Please check your ISE installation!)

endif
