#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Targets to setup a new firmware directory for use with GCC
#

$(eval $(MAKEFILE_PROLOG))

NEW_FW_HOME:=$(abspath $(+path))
NEW_FW_SUBDIRS:=$(addprefix $(NEW_FW_HOME)/, src include)

FMW_CFG:=$(NEW_FW_HOME)/config-build.mk

FWHOME_CMD=mkdir -p $(NEW_FW_HOME)
FWHOME_PROCESS:=Setting up new firmware directory

.PHONY: newfirmware newfirmware_checks

newfirmware_checks:
ifneq ($(origin +path),command line)
	@$(call TARGET_ERROR,no firmware directory given\n\n    usage: $(MAKE) newfirmware $(ARGS_newfirmware))
endif
ifneq ($(shell $(SH_SCRIPTS)/check-dir-empty $(NEW_FW_HOME)),)
	@$(call TARGET_ERROR,'$(NEW_FW_HOME)' does not denote an empty directory!)
endif
	@$(ECHO_TARGET_DONE)

newfirmware: newfirmware_checks $(FMW_CFG) $(NEW_FW_SUBDIRS)
	@$(ECHO_TARGET_DONE)

$(NEW_FW_HOME):
	@$(call BUILD_NOCD,FWHOME,--noecho)

$(NEW_FW_SUBDIRS): $(NEW_FW_HOME)
	@mkdir -p $@
	@$(ECHO_TARGET_DONE)

$(eval $(call EXPORT,NEW_FW_FILE))

$(FMW_CFG): $(NEW_FW_HOME)
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/config-build-gcc.mk > $@
	@$(ECHO_TARGET_DONE)
