#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Targets to setup a new simulation directory
#

$(eval $(MAKEFILE_PROLOG))

BACKTICK=`
BACKTICK=`

ifneq ($(PROJECT_MK),)
# Targets
AVAILABLE_TARGETS+=newsim
HELP_newsim=Setup new simulation directory in PATH
ARGS_newsim=+path=PATH
endif

# Simulation directory
ifeq ($(MAKECMDGOALS),newsim)
  SIM_HOME:= $(+path)
  SIM_HOME:=$(abspath $(SIM_HOME))
endif

# Files
MODELSIM_INI:=$(SIM_HOME)/modelsim.ini
TESTBENCH_FDO:=$(SIM_HOME)/testbench.fdo
TIMESIM_FDO:=$(SIM_HOME)/post-place-route-sim.fdo
TESTBENCH_V:=$(SIM_HOME)/testbench.v
VLOG_OPT:=$(SIM_HOME)/vlog.opt
VSIM_DIR:=$(SIM_HOME)/vsim

# Commands
SIMHOME_CMD=mkdir -p $@
SIMHOME_PROCESS:=Setting up new simulation directory

ifneq ($(SIM_HOME),)
  RELATIVE_PROJECT_PATH:=$(shell $(SPARTANMC_ROOT)/src/scripts/sh/pathto.sh $(SIM_HOME) $(CURDIR))
endif

ifneq ($(PROJECT_MK),)
.PHONY: newsim newsim_checks

newsim_checks:
ifneq ($(origin +path),command line)
	@$(call TARGET_ERROR,no directory given\n\n    usage: $(MAKE) newsim $(ARGS_newsim))
endif
ifneq ($(shell $(SH_SCRIPTS)/check-dir-empty $(SIM_HOME)),)
	@$(call TARGET_ERROR,'$(SIM_HOME)' does not denote an empty directory!)
endif
	@$(ECHO_TARGET_DONE)

newsim: newsim_checks $(TESTBENCH_FDO) $(TIMESIM_FDO) $(TESTBENCH_V) $(MODELSIM_INI) $(VLOG_OPT) $(VSIM_DIR)
	@$(ECHO_TARGET_DONE)
endif

$(SIM_HOME):
	@$(call BUILD_NOCD,SIMHOME,--noecho)

$(VSIM_DIR): $(SIM_HOME)
	@mkdir -p $@
	@$(ECHO_TARGET_DONE)

$(eval $(call EXPORT,OPTION_FILE,:=$(subst $(SIM_HOME)/,,$(VLOG_OPT))))
$(eval $(call EXPORT,TRANSCRIPT_FILE,:=$(subst $(SIM_HOME)/,,$(VSIM_DIR)/transcript)))
$(eval $(call EXPORT,WORK,:=$(subst $(SIM_HOME)/,,$(VSIM_DIR)/work)))

ifeq ($(NOQUIET),)
$(eval $(call EXPORT,QUIET,:=quiet))
endif

$(MODELSIM_INI): $(SIM_HOME)
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/modelsim.ini > $@
	@$(ECHO_TARGET_DONE)

$(VLOG_OPT): $(SIM_HOME)
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/vlog.opt > $@
	@$(ECHO_TARGET_DONE)

$(eval $(call EXPORT,RELATIVE_PROJECT_PATH))

$(TESTBENCH_FDO): $(SIM_HOME)
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/testbench.fdo > $@
	@$(ECHO_TARGET_DONE)


$(TIMESIM_FDO): $(SIM_HOME)
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/post-place-route-sim.fdo > $@
	@$(ECHO_TARGET_DONE)


$(eval $(call EXPORT,JCONFIG_TOP_MODULE))


$(TESTBENCH_V): $(SIM_HOME)
	@cp $(GENERATED_FILES_DIR)/module_sim_$(JCONFIG_TOP_MODULE).v $(TESTBENCH_V)
	@$(ECHO_TARGET_DONE)


# Error handling
ifneq ($(QUARTUS_SIMLIBDIR),)
$(QUARTUS_SIMLIBDIR)/%:
	@$(call TARGET_ERROR,Simulation library '$(@F)' not found. Currently$(,) your \$$QUARTUS_SIMLIBDIR is set to '$(QUARTUS_SIMLIBDIR)'. Please check your settings.)
endif
ifneq ($(SPMC_LATTICE_SIMLIBDIR),)
$(SPMC_LATTICE_SIMLIBDIR)/%:
	@$(call TARGET_ERROR,Simulation library '$(@F)' not found. Currently$(,) your \$$SPMC_LATTICE_SIMLIBDIR is set to '$(SPMC_LATTICE_SIMLIBDIR)'. Please check your settings.)
endif
ifneq ($(LMC_HOME),)
$(LMC_HOME)/%:
	@$(call TARGET_ERROR,Simulation library '$(@F)' not found. Currently$(,) your \$$LMC_HOME is set to '$(LMC_HOME)'. Please check your settings.)
endif
