#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

# Altera toolchain
SOFFILE:=$(BUILD_ROOT).sof
QPF:=$(BUILD_ROOT).qpf
QSF:=$(BUILD_ROOT).qsf
QSF_JC:=$(GENERATED_FILES_DIR)/spartanmc.qsf
DBDIR:=$(BUILD_DIR)/db
MAP:=$(BUILD_ROOT).map.rpt
FIT:=$(BUILD_ROOT).fit.rpt
ASM:=$(BUILD_ROOT).sof
CREATE_SDC_TCL:=$(BUILD_ROOT)_sta_map.tcl
SDC:=$(BUILD_ROOT)_map.sdc
QUARTUS_STA:=$(QUARTUSBINDIR)/quartus_sta
QUARTUS_CPF:=$(QUARTUSBINDIR)/quartus_cpf
QUARTUS_CDB:=$(QUARTUSBINDIR)/quartus_cdb

MEMORY_FILES_MK:=$(GENERATED_FILES_DIR)/memories.mk
include $(MEMORY_FILES_MK)


$(eval $(call EXPORT,HDL))
$(eval $(call EXPORT,SDC))
$(eval $(call EXPORT,BUILD_DIR))

# Targets
HELP_all:=Build the whole system (alias for bitfile)
HELP_mapping:=Run analysis, synthesis and mapping (map)
HELP_placeroute:=Run place and route (mpartrce)
HELP_bitfile:=Create bitfile
HELP_program:=Program current bitfile into FPGA device (pgrcmd)

define RUN_TOOL
@cd $(BUILD_DIR); \
LD_LIBRARY_PATH=$(QUARTUSBINDIR) \
PATH=$(QUARTUSBINDIR):$(PATH)
endef


.SUFFIXES:
.PHONY: all program synthesis translate mapping placeroute bitfile updateRam mif update_mif
AVAILABLE_TARGETS+=all program mapping placeroute bitfile updateRam mif update_mif

all: bitfile

program:
	@if [ ! -e $(ASM) ]; then printf "\nError: No bitstream file present! Generate file with 'make bitfile'!\n\n"; exit 1; fi
	$(RUN_TOOL) $(QUARTUS_PGM) $(JCONFIG_OPT_PGM)

timing: $(FIT)
	$(RUN_TOOL) $(QUARTUS_STA) $(JCONFIG_PRJNAME)

bitfile: $(ASM)
placeroute: $(FIT)
mapping: $(MAP)

$(ASM): $(FIT) $(QSF) $(QPF) | $(BUILD_DIR)
	$(RUN_TOOL) $(QUARTUS_ASM) --read_settings_files=on --write_settings_files=off $(JCONFIG_PRJNAME)

$(FIT): $(SDC) $(QSF) $(QPF) | $(BUILD_DIR)
	$(RUN_TOOL) $(QUARTUS_FIT) --read_settings_files=on --write_settings_files=off $(JCONFIG_PRJNAME)

$(SDC): $(CREATE_SDC_TCL) $(MAP) | $(BUILD_DIR)
	$(RUN_TOOL) $(QUARTUS_STA) -t $(CREATE_SDC_TCL)
	@touch $(SDC)

$(TIMESIM): $(FIT)
	$(RUN_TOOL) $(QUARTUS_EDA) --simulation --tool=modelsim --format=verilog --read_settings_files=on --write_settings_files=off $(JCONFIG_PRJNAME)
	#remove sdf_annotate statement, since it does not work if we are in a different directory...
	grep -v "sdf_annotate" $(BUILD_DIR)/simulation/modelsim/spartanmc.vo > $(TIMESIM)
	cp $(BUILD_DIR)/simulation/modelsim/spartanmc_v.sdo $(BUILD_DIR)/spartanmc_timesim.sdf

$(CREATE_SDC_TCL): $(JCONFIG_PROJECT_MK_FILE) | $(BUILD_DIR)
	$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/quartus_create_sdc.tcl > $@

$(MAP): $(QSF) $(QPF) $(JCONFIG_HDL_SOURCES) $(JCONFIG_MEMORY_FILES) | $(BUILD_DIR)
	$(RUN_TOOL) $(QUARTUS_MAP) --read_settings_files=on --write_settings_files=off $(JCONFIG_PRJNAME)

$(QPF): | $(BUILD_DIR)
	@printf "PROJECT_REVISION = \"$(JCONFIG_PRJNAME)\"\n" > $@

#QSF file must be in build dir, so copy it...
$(QSF): $(QSF_JC) | $(BUILD_DIR)
	cp $(QSF_JC) $(QSF)
	
update_mif: mif 
	@$(call BUILD,UPDATE_MIF)
	$(RUN_TOOL) $(QUARTUS_CDB) --update_mif $(JCONFIG_PRJNAME)
	
updateRam: update_mif
	@$(call BUILD,UPDATE_SOF)
	$(RUN_TOOL) $(QUARTUS_ASM) --read_settings_files=on --write_settings_files=off $(JCONFIG_PRJNAME)
	

mif: $(JCONFIG_MEMORY_FILES)

$(JCONFIG_MEMORY_FILES): $(MEM_MAP) $(ELF_FILES) | $(BUILD_DIR)
	$(SPARTANMC_INITRAMJ) --mif build --memory $<
