#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC build process ###
#
# Hardware build process
#

# Project file for XST

$(eval $(MAKEFILE_PROLOG))

include $(SPARTANMC_ROOT)/src/scripts/make/project/checks.mk

TCL_SCRIPTS:=$(SPARTANMC_SCRIPTS)/tcl
TCL_GENPROJECT:=$(GENERATED_FILES_DIR)/genproject.tcl
TCL_SYNTHESIS:=$(TCL_SCRIPTS)/vivado_synthesis.tcl
TCL_IMPLEMENTATION:=$(TCL_SCRIPTS)/vivado_implementation.tcl
TCL_BITSTREAM:=$(TCL_SCRIPTS)/vivado_bitstream.tcl
TCL_TIMESIM:=$(TCL_SCRIPTS)/vivado_timesim.tcl
TCL_PRIMLOC:=$(TCL_SCRIPTS)/vivado_primloc.tcl
TCL_PROGRAM:=$(TCL_SCRIPTS)/vivado_program.tcl
TCL_ENSURE_PROJECT_LOADED:=$(TCL_SCRIPTS)/vivado_ensure_project_loaded.tcl
TCL_ENSURE_CHECKPOINT_LOADED:=$(TCL_SCRIPTS)/vivado_ensure_checkpoint_loaded.tcl
TCL_EXPORT_TO_SDK:=$(TCL_SCRIPTS)/vivado_export_to_sdk.tcl

TCL_PATHS:=$(BUILD_DIR)/vivado-paths.tcl

VIVADO_PROJECTDIR:=$(BUILD_DIR)/project
VIVADO_PROJECT:=$(VIVADO_PROJECTDIR)/$(JCONFIG_PRJNAME).xpr
#Vivado touches the project file as it pleases. we need an additional file to keep track of up to date info
VIVADO_PROJECTDEPEND:=$(VIVADO_PROJECTDIR)/my_project
VIVADO_RUNS:=$(VIVADO_PROJECTDIR)/$(JCONFIG_PRJNAME).runs

VIVADO_SYNTH_OUT:=$(VIVADO_RUNS)/synth_1/$(JCONFIG_TOP_MODULE).dcp
VIVADO_IMPL_OUT:=$(VIVADO_RUNS)/impl_1/$(JCONFIG_TOP_MODULE)_routed.dcp
VIVADO_SYSDEF_OUT:=$(VIVADO_RUNS)/impl_1/$(JCONFIG_TOP_MODULE).sysdef
VIVADO_TIMING_OUT:=$(BUILD_DIR)/timing.txt

VIVADO_PRIMLOC:=$(BUILD_DIR)/spartanmc.primloc
VIVADO_TIMESIM_VERILOG:=$(BUILD_DIR)/spartanmc_timesim.v
VIVADO_TIMESIM_SDF:=$(BUILD_DIR)/spartanmc_timesim.sdf
VIVADO_BITFILE_REGULAR:=$(BUILD_DIR)/spartanmc_regular.bit
VIVADO_BITFILE_REPLACED:=$(BUILD_DIR)/spartanmc_rep.bit
VIVADO_BITFILE:=$(BUILD_DIR)/spartanmc.bit
VIVADO_LOCATION:=$(BUILD_DIR)/spartanmc_regular.ll

WRAPPER:=$(SPARTANMC_VIVADO_WRAPPER) --executable $(VIVADO) --project $(PROJECT_DIR)
RUNVIVADO:=$(WRAPPER) --script $(TCL_PATHS) --script

PROJECT_PROCESS:=Generate Project
PROJECT_CMD:=$(RUNVIVADO) $(TCL_GENPROJECT)
HELP_project=Create Vivado project

SYNTHESIS_PROCESS:=Synthesis
SYNTHESIS_CMD:=$(RUNVIVADO) $(TCL_SYNTHESIS)
HELP_synthesis=Synthesize design

IMPLEMENTATION_PROCESS:=Implementation
IMPLEMENTATION_CMD:=$(RUNVIVADO) $(TCL_IMPLEMENTATION)
HELP_implementation=Implement design

ALL_PROCESS:=Create Bitstream
ALL_CMD:=$(RUNVIVADO) $(TCL_BITSTREAM)
HELP_all=Create bitstream


timesim_PROCESS:=Create Timing Simulation
TIMESIM_CMD:=$(RUNVIVADO) $(TCL_TIMESIM)
HELP_timesim=Create Timing Simulation Verilog File

UPDATERAM_PROCESS:=Update Ram
UPDATERAM_CMD:=$(SPARTANMC_INITRAMJ) --primLoc $(VIVADO_PRIMLOC) -m $(MEM_MAP) --bitin $(VIVADO_BITFILE) --bitout $(VIVADO_BITFILE_REPLACED) --location $(VIVADO_LOCATION)
HELP_updateRam=Update Bitfile with current program

PROGRAM_PROCESS:=Program
PROGRAM_CMD:=$(RUNVIVADO) $(TCL_PROGRAM)
HELP_program=Program target FPGA

VIVADO_PROCESS:=Vivado
VIVADO_CMD:=cd $(BUILD_DIR) && $(VIVADO) -source $(TCL_PATHS) $(VIVADO_PROJECT)
HELP_vivado=Open Vivado GUI

STOPSERVER_PROCESS:=Vivado
STOPSERVER_CMD:=$(WRAPPER) --stopForProject
HELP_stopserver=Stop any running vivado instances

EXPORT_TO_SDK_PROCESS:=Export to SDK
EXPORT_TO_SDK_CMD:=$(RUNVIVADO) $(TCL_EXPORT_TO_SDK)
HELP_export_to_sdk=Export system to Xilinx SDK

AVAILABLE_TARGETS:=project synthesis implementation vivado all updateRam stopserver export_to_sdk timesim $(AVAILABLE_TARGETS)


$(eval $(call EXPORT,VIVADO_PROJECTDIR))
$(eval $(call EXPORT,VIVADO_BITFILE_REGULAR))
$(eval $(call EXPORT,TCL_PROGRAMRAM))
$(eval $(call EXPORT,VIVADO_IMPL_OUT))
$(eval $(call EXPORT,VIVADO_TIMING_OUT))
$(eval $(call EXPORT,VIVADO_PRIMLOC))
$(eval $(call EXPORT,VIVADO_PROJECT))
$(eval $(call EXPORT,VIVADO_TIMESIM_VERILOG))
$(eval $(call EXPORT,VIVADO_TIMESIM_SDF))
$(eval $(call EXPORT,TCL_ENSURE_PROJECT_LOADED))
$(eval $(call EXPORT,TCL_ENSURE_CHECKPOINT_LOADED))
$(eval $(call EXPORT,JCONFIG_PRJNAME))
$(eval $(call EXPORT,GENERATED_FILES_DIR))
$(TCL_PATHS): $(SPARTANMC_TEMPLATES)/vivado-paths.tcl | $(BUILD_DIR)
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/vivado-paths.tcl > $@
	@$(ECHO_TARGET_DONE)


$(VIVADO_PROJECTDEPEND): $(JCONFIG_HDL_SOURCES) $(TCL_GENPROJECT) $(TCL_PATHS)
	@$(call BUILD,PROJECT)
	@touch $(VIVADO_PROJECTDEPEND)

$(VIVADO_SYNTH_OUT): $(VIVADO_PROJECTDEPEND) $(TCL_PATHS) $(TCL_SYNTHESIS)
	@$(call BUILD,SYNTHESIS)

$(VIVADO_IMPL_OUT): $(VIVADO_SYNTH_OUT) $(VIVADO_PROJECTDEPEND) $(TCL_PATHS) $(TCL_IMPLEMENTATION)
	@$(call BUILD,IMPLEMENTATION)
	$(TIMING_VIOLATION_CALL) --parsePlaceRouteTiming

$(VIVADO_TIMESIM_VERILOG):  $(VIVADO_IMPL_OUT) $(TCL_PATHS) $(TCL_PROGRAMRAM) $(TCL_TIMESIM)
	@$(call BUILD,TIMESIM)

$(VIVADO_BITFILE_REGULAR): $(VIVADO_IMPL_OUT) $(TCL_PATHS) $(TCL_PROGRAMRAM) $(TCL_BITSTREAM)
	@$(call BUILD,ALL)

$(VIVADO_BITFILE_REPLACED): $(TCL_PATHS) $(TCL_PROGRAMRAM)
	@$(call BUILD,UPDATERAM)

$(VIVADO_PRIMLOC): $(VIVADO_IMPL_OUT) $(TCL_PATHS) $(TCL_PRIMLOC)
	$(RUNVIVADO) $(TCL_PRIMLOC)

vivado: $(VIVADO_PROJECTDEPEND) $(TCL_PATHS)
	@$(call BUILD,VIVADO)

program: $(VIVADO_PROJECTDEPEND) $(TCL_PATHS)
	@if test ! -f $(VIVADO_BITFILE); then \
		$(call TARGET_ERROR,Please use 'make all' first to generate the necessary bitfile!); \
	fi
	@$(call BUILD,PROGRAM)

stopserver:
	@$(call BUILD,STOPSERVER)

export_to_sdk: $(VIVADO_IMPL_OUT) $(TCL_PATHS)
	@$(call BUILD,EXPORT_TO_SDK)

.PHONY: project synthesis implementation vivado all primloc stopserver export_to_sdk timesim

project: $(VIVADO_PROJECTDEPEND)
synthesis: $(VIVADO_SYNTH_OUT)
implementation: $(VIVADO_IMPL_OUT)
all: $(VIVADO_BITFILE_REGULAR)
	cp $(VIVADO_BITFILE_REGULAR) $(VIVADO_BITFILE)
primloc: $(VIVADO_PRIMLOC)
timesim: $(VIVADO_TIMESIM_VERILOG)
updateRam: $(VIVADO_BITFILE_REPLACED) timingViolation
	cp $(VIVADO_BITFILE_REPLACED) $(VIVADO_BITFILE)
