#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC install ###
#
# Provides targets to setup udev-rules and
# firware for usb cable driver
#

$(eval $(MAKEFILE_PROLOG))

ifeq ($(MAKECMDGOALS),cablesetup)
endif



ifeq ($(UDEV_VERSION_GE_150),yes)
    UDEV_TOKEN_BUS:=SUBSYSTEMS
    UDEV_TOKEN_SYSFS:=ATTR
    UDEV_TOKEN_TEMPNODE:=\$$tempnode
else
    UDEV_TOKEN_BUS:=BUS
    UDEV_TOKEN_SYSFS:=SYSFS
    UDEV_TOKEN_TEMPNODE:=\$$TEMPNODE
endif

CABLE_GROUP:=xilinx

ifeq ($(MAKECMDGOALS),cablesetup)
  ifneq ($(+group),)
    CABLE_GROUP:=$(+group)
  endif
endif

XILINX_VENDOR_ID:=03fd
RULES:=$(BUILD_DIR)/99-xilinx-usb-cables.rules

# Creates an udev-rule to set access right
# 1: product id
# 2: mode
# 3: group
UDEV_USB_ACCESS_RULE = \
	printf "$(UDEV_TOKEN_SYSFS){idVendor}==\"$(XILINX_VENDOR_ID)\", $(UDEV_TOKEN_SYSFS){idProduct}==\"$(1)\", MODE=\"$(2)\", GROUP=\"$(3)\", RUN+=\"/bin/sh -c '/bin/chmod $(2) \$$env{DEVICE}; /bin/chgrp $(3) \$$env{DEVICE}'\"\n"

# Creates an udev-rule to load xilinx firmware
# 1: product id
# 2: firmware file
UDEV_XILINX_FIRMWARE_RULE = \
	printf "$(UDEV_TOKEN_BUS)==\"usb\", ACTION==\"add\", $(UDEV_TOKEN_SYSFS){idVendor}==\"$(XILINX_VENDOR_ID)\", $(UDEV_TOKEN_SYSFS){idProduct}==\"$(1)\", RUN+=\"$(FXLOAD) -v -t fx2 -I /usr/share/$(2) -D $(UDEV_TOKEN_TEMPNODE)\"\n"

# Creates all necessary udev-rules
CREATE_RULES = \
	$(call UDEV_USB_ACCESS_RULE,0008,660,$(CABLE_GROUP)) && \
	$(call UDEV_XILINX_FIRMWARE_RULE,0007,xusbdfwu.hex) && \
	$(call UDEV_XILINX_FIRMWARE_RULE,0009,xusb_xup.hex) && \
	$(call UDEV_XILINX_FIRMWARE_RULE,000d,xusb_emb.hex) && \
	$(call UDEV_XILINX_FIRMWARE_RULE,000f,xusb_xlp.hex) && \
	$(call UDEV_XILINX_FIRMWARE_RULE,0013,xusb_xp2.hex) && \
	$(call UDEV_XILINX_FIRMWARE_RULE,0015,xusb_xse.hex)

XILINX_HEX:=$(addprefix /usr/share/,xusbdfwu.hex xusb_xup.hex xusb_emb.hex xusb_xlp.hex xusb_xp2.hex xusb_xse.hex)

AVAILABLE_TARGETS+=cablesetup

HELP_cablesetup=Create udev rules $(OBRACE)*.rules$(CBRACE) for Xilinx usb programming cables with access rights granted to specified group $(OBRACE)default is 'xilinx'$(CBRACE)
ARGS_cablesetup:=[+group=GROUP]

CHECK_CABLESETUP = \
	need_cable_setup= && \
	for f in $(RULES) $(XILINX_HEX); do \
		if test ! -f $$f; then \
			need_cable_setup=yes && \
			break; \
		fi; \
	done; \
	if test x$$need_cable_setup = xyes; then \
		$(call TARGET_MESSAGE_BOLD,run 'make cablesetup' to create udev configuration file for the usb programming cable); \
	else \
		$(call TARGET_MESSAGE,setup for USB programming cables seems to be OK); \
	fi

.PHONY: cablesetup

ifeq ($(words $(MAKECMDGOALS)),1)
  
  cablesetup: $(RULES) $(XILINX_HEX)
	  @$(call TARGET_MESSAGE,After copying all required files unplug and reconnect any \
		  currently connected USB programming cables for the new settings to be applied. \
		  If access to USB devices is restricted$(COMMA) users need to be member of group \
		  '$(CABLE_GROUP)' to use usb programming cables.\n)
	  @$(ECHO_TARGET_DONE)

else

  cablesetup:
	  @$(call TARGET_MESSAGE_BOLD,please run as standalone target)
	  @$(call TARGET_ERROR,usage: make cablesetup [+group=GROUP])

endif

/usr/share/xusb%.hex: $(XILINX_FIRMWARE_PATH)/xusb%.hex
	@$(call TARGET_MESSAGE_BOLD,Please copy xilinx firmware file '$^' to '/usr/share/')

force:

$(RULES): force | $(BUILD_DIR)
	@($(CREATE_RULES)) > $@
	@printf "\n"
	@$(call TARGET_MESSAGE_BOLD,\nPlease copy created udev rules file '$(RULES)' to your udev rules directory\n)
