#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC install ###
#
# Targets for building the SpartanMC toolchain
#

$(eval $(MAKEFILE_PROLOG))

GENERATED_DIRS:=bin man doc spartanmc/lib_obj/build
TOOLSCLOBBER_CMD=rm -rf $(GENERATED_DIRS)
TOOLSCLOBBER_PROCESS:=Removing built tools

SPACE:=
SPACE+=

AVAILABLE_TARGETS+=buildtools

# Template to invoke external build of a SparatnMC-SoC-Kit component
# 1: unique id for external build (required to create unique variables)
# 2: target that will be build
# 3: directory where to invoke the build from
# 4: regex specifying the source files considered for dependencies
# 5: "MAKE" or "ANT" to specify build tool
# 6: target to invoke in external build process (may be empty)

define TOOLS_TEMPLATE
ANT_$(1)_PROCESS:=Building/updating component
ANT_$(1)_CMD:=$(ANT) -f $(3)/build.xml $(6)
MAKE_$(1)_PROCESS:=Building/updating component
MAKE_$(1)_CMD:=$(MAKE) -C $(3) $(6)
$(call MAKE_MSG,$(SPACE)$(SPACE)scanning $(3)/...)
SOURCES:=$$(shell find $(3) -not -path "*svn*" -type f -regex $(4))
#$$(call MAKE_MSG,$(SPACE)$(SPACE)$(SPACE)$(SPACE)$$(SOURCES))
$(2): $$(filter-out $(2),$$(SOURCES))
	@$$(call BUILD_NOCD,$(5)_$(1))
	@touch $(2)
buildtools: $(2)
endef

GRADLE_CMD:=./gradlew install
GRADLE_PROCESS:=Let Gradle setup docs, tools and descriptions

HELP_buildtools=Alias for "./gradlew install". Use that for more granular control

buildtools:
	@$(call BUILD_NOCD,GRADLE)

.PHONY: toolsclobber buildtools

HELP_toolsclobber=Remove hardcoded directories such as 'build' and 'bin'

toolsclobber:
	@$(call BUILD_NOCD,TOOLSCLOBBER)

ifeq ($(WITH_DETACH),yes)
    $(eval $(call TOOLS_TEMPLATE,12,$$(SPARTANMC_DETACH),$(SPARTANMC_ROOT)/src/detach,'.*',MAKE))
endif
ifeq ($(ENABLE_GCC),yes)
#    $(eval $(call TOOLS_TEMPLATE,13,$$(SPARTANMC_LIBS_CHK),$(SPARTANMC_ROOT)/spartanmc/lib_obj,'.*/\(src/.*\|Makefile\)',MAKE))
endif
ifneq ($(HAS_LIBUSB_DRIVER),yes)
  ifeq ($(WITH_USBDRIVER),yes)
    $(eval $(call TOOLS_TEMPLATE,16,$$(SPARTANMC_ROOT)/bin/usb-driver/libusb-driver.so,$(SPARTANMC_ROOT)/src/usb-driver,'.*',MAKE))
  endif
endif
