#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC install ###
#
# Main makefile for SpartanMC environment setup
#

BUILD_DIR:=build
MAN_FILE:=$(SPARTANMC_ROOT)/doc/man/spartanmc.1

AVAILABLE_TARGETS+=setup unconfigure
HELP_setup=Perform all required steps to setup SpartanMC-SoC-Kit
HELP_unconfigure=Remove all files generated by configure and make $(OBRACE)requires re-run of configure$(CBRACE)

include $(SPARTANMC_ROOT)/src/scripts/make/common/buildsetup.mk
include $(SPARTANMC_ROOT)/src/scripts/make/common/functions.mk

include $(SPARTANMC_ROOT)/src/scripts/make/common/man.mk

ifeq ($(ENABLE_CABLESETUP),yes)
  include $(SPARTANMC_ROOT)/src/scripts/make/repo/cablesetup.mk
endif
include $(SPARTANMC_ROOT)/src/scripts/make/repo/components.mk
include $(SPARTANMC_ROOT)/src/scripts/make/repo/newproject.mk
include $(SPARTANMC_ROOT)/src/scripts/make/repo/tests.mk
include $(SPARTANMC_ROOT)/src/scripts/make/common/what.mk

.PHONY: setup unconfigure reconfigure

setup: buildtools
ifeq ($(ENABLE_ISE),yes)	
	@$(CHECK_CABLESETUP)
ifeq ($(WITH_USBDRIVER),yes)
	@$(call TARGET_MESSAGE_BOLD,If you plan to run 'impact' manually from the command line$(,) you have to set '$(IMPACT_LD_PRELOAD)' to get the cable driver working properly)
endif
endif
ifeq ($(ENV_SPARTANMC_ROOT),)
	@$(call TARGET_MESSAGE_BOLD,Please add 'SPARTANMC_ROOT=$(SPARTANMC_ROOT)' to your environment setting to make the SpartanMC-SoC-Kit working properly!)

endif
	@$(ECHO_TARGET_DONE)

unconfigure: toolsclobber
	@$(call REMOVE,$(call MAKE_RELATIVE,$(AC_CONFIG_FILES)) config.log config.status) && \
	printf "\n" && \
	$(call TARGET_MESSAGE_BOLD,*** all generated files removed) && \
	$(call TARGET_MESSAGE_BOLD,*** you need to re-run 'configure') && \
	if test -n '$(CONFIGURE_OPTS)'; then \
		$(call TARGET_MESSAGE,    (used configure options: $(CONFIGURE_OPTS))); \
	fi && \
	printf "\n\n" && \
	$(ECHO_TARGET_DONE)
