#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC install ###
#
# Targets for creating a new project
#

$(eval $(MAKEFILE_PROLOG))

ifeq ($(MAKECMDGOALS),newproject)
  PROJECT_HOME:= $(+path)
endif

PRJ_MK:=$(PROJECT_HOME)/Makefile

PROJECTHOME_CMD=mkdir -p $@
PROJECTHOME_PROCESS:=Setting up new project directory

AVAILABLE_TARGETS+=newproject
HELP_newproject=Create new project in PATH
ARGS_newproject=+path=PATH

.PHONY: newproject newproject_checks

newproject_checks:
ifneq ($(origin +path),command line)
	@$(call TARGET_ERROR,no project directory given\n\n    usage: $(MAKE) newproject $(ARGS_newproject))
endif
ifneq ($(shell $(SH_SCRIPTS)/check-dir-empty $(PROJECT_HOME)),)
	@$(call TARGET_ERROR,'$(PROJECT_HOME)' does not denote an empty directory!)
endif
ifeq ($(ENV_SPARTANMC_ROOT),)
	@$(call TARGET_MESSAGE_BOLD,NOTE: From this revision on$(COMMA) the setting of SPARTANMC_ROOT is no longer maintained by the project makefile. You need to set the variable SPARTANMC_ROOT within your shell environment by e.g. adding 'SPARTANMC_ROOT=$(SPARTANMC_ROOT)' to your .bashrc or similar file.)
endif

newproject: newproject_checks $(PRJ_MK)
	@$(ECHO_TARGET_DONE)

$(PROJECT_HOME):
	@$(call BUILD_NOCD,PROJECTHOME,--noecho)

$(PRJ_MK): $(PROJECT_HOME)
	@$(SH_SCRIPTS)/rdtmp $(SPARTANMC_TEMPLATES)/Makefile.hardware > $@
	@$(ECHO_TARGET_DONE)
