#_______________________________________________________________________________
#
# Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
# (http://www.mr.inf.tu-dresden.de) All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement: "This product includes
#    software developed by the TU Dresden Chair for Embedded Systems and
#    its contributors."
#
# 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#_______________________________________________________________________________
#

### Include makefile for SpartanMC install ###
#
# Targets implementing tests to check build system 
# integrity
#

$(eval $(MAKEFILE_PROLOG))

AVAILABLE_TARGETS+=integrity_check
HELP_integrity_check:=Run a sequence of tests to check integrity of the SpartanMC installation. PLATFORM specifies the target hardware for final system implementation. Can be 'nohw' to stop after bitfile generation.
ARGS_integrity_check:=+target=PLATFORM

INTEGRITY_CHECK_PROCESS:=Running integrity check on build system.
INTEGRITY_CHECK_CMD:=true

SUBMAKE=$(BUILD_DIR)/submake
SUBMAKE_CMD=$(SUBMAKE)
SUBMAKE_PROCESS=Invoking make: $(3)

RECONFIGURE_CMD=$(CONFIGURE_COMMAND)
RECONFIGURE_PROCESS:=Running configure with stored options

NO_HARDWARE_PLATFORM:=nohw
SUPPORTED_HARDWARE_PLATFORMS:=$(patsubst integrity-check-%.xml, \n  %,$(notdir $(wildcard $(SPARTANMC_TEMPLATES)/integrity-check-*.xml)))

TEST_PROJECT_PATH:=$(shell mktemp -d)
ifneq ($(+target),)
  TEST_PROJECT_CONFIG:=$(SPARTANMC_TEMPLATES)/integrity-check-$(+target).xml
else
  TEST_PROJECT_CONFIG:=$(SPARTANMC_TEMPLATES)/integrity-check.xml
endif

.PHONY: integrity_check_title check_spartanmc_root check_targets_project check_implementation force

checks_integrity_check:
ifneq ($(origin +target),command line)
	@$(call TARGET_ERROR,usage: $(MAKE) integrity_check $(ARGS_integrity_check)\n$(ANSI_BOLDOFF)\nValid values for PLATFORM are:\n$(SUPPORTED_HARDWARE_PLATFORMS))
endif

integrity_check: integrity_check_title checks_integrity_check $(TEST_PROJECT_CONFIG) check_setup check_targets_repo check_implementation
	@printf "\n"
	@$(call TARGET_MESSAGE_BOLD,Integrity test successfully passed)
	@$(ECHO_TARGET_DONE)

integrity_check_title:
	@$(call BUILD_NOCD,INTEGRITY_CHECK,--noecho)

force:

$(SUBMAKE): $(MAKEFILE_LIST) force | $(BUILD_DIR)
	@printf "set -e -f\n" > $@
	@printf "unset %s\n" "$(EXPORTED_VARIABLES)" >> $@
	@printf "set +f\n" >> $@
	@printf "oldifs=$$IFS\n" >> $@
	@printf "IFS=\n" >> $@
	@printf "IFS=$$oldifs $(MAKE) \$$@\n" >> $@
	@chmod a+x $@
	@$(ECHO_TARGET_DONE)

check_spartanmc_root:
ifneq ($(ENV_SPARTANMC_ROOT),$(PWD))
	@$(call TARGET_ERROR,Environmant variable SPARTANMC_ROOT does not point to the current directory.\n \nExpected value: $(PWD)\nCurrent value:  $(ENV_SPARTANMC_ROOT))
endif
	@$(ECHO_TARGET_DONE)

check_setup: $(SUBMAKE) check_spartanmc_root ./autogen.sh force
	@tmp=$$(mktemp) && cp -p $(SUBMAKE) $$tmp && $(call BUILD_NOCD,SUBMAKE,,unconfigure) && mkdir $(BUILD_DIR) && cp -p $$tmp $(SUBMAKE)
	@./autogen.sh
	@$(call BUILD_NOCD,RECONFIGURE)
	@$(call BUILD_NOCD,SUBMAKE,,setup)

check_targets_repo: $(SUBMAKE)
ifeq ($(ENABLE_CABLESETUP),yes)
	@$(call BUILD_NOCD,SUBMAKE,,cablesetup)
endif

# create a test project

check_targets_project: $(TEST_PROJECT_PATH)/build/spartanmc.bit

$(TEST_PROJECT_PATH)/Makefile: $(SUBMAKE)
	@$(call BUILD_NOCD,SUBMAKE,,newproject +path=$(TEST_PROJECT_PATH))

$(TEST_PROJECT_PATH)/integrity-check.xml: $(TEST_PROJECT_CONFIG) $(TEST_PROJECT_PATH)/Makefile
	@cp $< $@

$(TEST_PROJECT_PATH)/system: $(TEST_PROJECT_PATH)/integrity-check.xml $(SUBMAKE)
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) jconfig +args='-g integrity-check.xml')

$(TEST_PROJECT_PATH)/firmware: $(TEST_PROJECT_PATH)/system $(SUBMAKE)
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) newfirmware +path="firmware")

$(TEST_PROJECT_PATH)/firmware/src/main.c: $(TEST_PROJECT_PATH)/firmware
	@cp $(SPARTANMC_TEMPLATES)/integrity-check.c $@

$(TEST_PROJECT_PATH)/build/spartanmc.bit: $(TEST_PROJECT_PATH)/firmware/src/main.c $(SUBMAKE)
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) firmware clean)
ifeq ($(ENABLE_ISE),yes)
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) constraints clean)
endif
ifeq ($(ENABLE_MODELSIM),yes)
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) sim clobber)
endif
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) jconfig +args='-g integrity-check.xml')
ifeq ($(ENABLE_ISE),yes)
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) all)
endif
ifeq ($(ENABLE_MODELSIM),yes)
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) timesim)
endif

$(SPARTANMC_TEMPLATES)/integrity-check-%.xml:
	@printf "\n"
	@$(call TARGET_MESSAGE_BOLD,\n \nThe hardware target platform '$*' is currently not supported for integrity testing. To support the requested platform you will need a configuration file '$(@F)' in the templates directory.\n$(ANSI_BOLDOFF)\nThe following targets are currently supported for integrity testing:\n$(SUPPORTED_HARDWARE_PLATFORMS) \n \nBy specifying '+target=$(NO_HARDWARE_PLATFORM)' you can run all of the integrity tests without requiring specific hardware. Implementing and running the test design on a hardware platform will not be tested in that case.\n\n)
	@false

check_implementation: $(TEST_PROJECT_CONFIG) check_targets_project $(SUBMAKE)
ifeq ($(+target),$(NO_HARDWARE_PLATFORM))
	@printf "\n"
	@$(call TARGET_MESSAGE,Implementing your system in real hardware will not be tested since you did not specify a hardware platform $(OBRACE)option: '+target=$(NO_HARDWARE_PLATFORM)$(CBRACE))
else
	@$(call BUILD_NOCD,SUBMAKE,,-C $(TEST_PROJECT_PATH) program)
	@printf "\n"
	@$(call TARGET_MESSAGE,Your test system has been successfully implemented on the target platform '$(+target)'. Check activity of the heartbeat led $(OBRACE)blinking at constant interval$(CBRACE) to see if the design is working properly.)
endif

.PHONY: tests
tests:
	./gradlew test 
