ifneq (\\$(BUILD_WITH_SDK),1)

# Compile to assembler

# Include makefile from accelerator plugin if extraction of hardware
# accelerators are enabled.
ifeq (\\$(HWACC_ENABLED),1)

HWACC_OUTPUT_DIR := ${D_BUILD}
HWACC_SOURCE_FILES := ${LF_C}
CFLAGS := ${_CFLAGS}
CC := \\$(SPARTANMC_GCC)
AR := \\$(SPARTANMC_AR)
HWACC_DEPENDENCIES := \\$(LF_INC) \\$(MAKEFILE_LIST) \\$(CONFIG_BUILD_MK) \\$(GENERATED_FILES_DIR)/firmwares/\\$(JCONFIG_FW_\\$(FW_ID)_SUBSYSTEM)/hardware.mk

include \\$(HWACC_PLUGIN_ROOT)/common/accelerators.mk

else # HWACC_ENABLED == 0

# Create empty verilog files if hardware accelerator is disabled but module is still present
${D_BUILD}/%.v:
	@touch \\$@


# Compile peripherals_generated_jconfig.c to assembler (w/o hardware accelerators)
${D_BUILD}/%.s: ${LF_C_GENERATED_DIR}/%.c \\$(SPARTANMC_GCC) \\$(LF_INC) \\$(MAKEFILE_LIST) \\$(CONFIG_BUILD_MK) | ${D_BUILD}
	\\$(SPARTANMC_GCC) -S ${_CFLAGS} \\$< -o \\$@

# Compile to assembler (w/o hardware accelerators)
${D_BUILD}/%.s: ${D_SRC}/%.c \\$(SPARTANMC_GCC) \\$(LF_INC) \\$(MAKEFILE_LIST) \\$(CONFIG_BUILD_MK) | ${D_BUILD}
	\\$(SPARTANMC_GCC) -S ${_CFLAGS} \\$< -o \\$@

# Compile to assembler
${D_BUILD}/%.s: ${D_SRC}/%.cpp \\$(SPARTANMC_GCC) \\$(LF_INC) \\$(MAKEFILE_LIST) \\$(CONFIG_BUILD_MK) | ${D_BUILD}
	\\$(SPARTANMC_G++) -S ${_CPPFLAGS} \\$< -o \\$@

endif # HWACC_ENABLED

${F_ELF}: \\$(SPARTANMC_LIBS_CHK) \\$(SPARTANMC_LD) \\$(SPARTANMC_GCC) ${LF_LIBAS} ${LF_OBJ} ${LF_A} ${LDSCRIPT}
	\\$(SPARTANMC_GCC) ${LF_LIBAS} \\$(sort ${LF_OBJ}) -N ${_LDFLAGS} ${LIBS} -T${LDSCRIPT} -o \\$@ -Wl,-Map,${F_ELF}.map
	@\\$(call ECHO_TARGET_DONE)

# Compile C files without linking
# Keep dependency of assembler file for easier debugging
# For direct compilation, replace ${D_BUILD}/%.s with ${D_SRC}/%.c
${D_BUILD}/%.o: ${D_BUILD}/%.s \\$(SPARTANMC_AS) \\$(SPARTANMC_GCC) \\$(LF_INC) \\$(MAKEFILE_LIST) \\$(CONFIG_BUILD_MK) | ${D_BUILD}
	\\$(SPARTANMC_GCC) -c ${_CFLAGS} ${_ASFLAGS} \\$< -o \\$@
	@\\$(call ECHO_TARGET_DONE)

# Assemble S files without linking
${D_BUILD}/%.o: ${D_SRC}/%.s \\$(SPARTANMC_AS) \\$(SPARTANMC_GCC) \\$(MAKEFILE_LIST) \\$(CONFIG_BUILD_MK) | ${D_BUILD}
	\\$(SPARTANMC_GCC) -c ${_ASFLAGS} \\$< -o \\$@
	@\\$(call ECHO_TARGET_DONE)

# Assemble S files from lib_as
${D_BUILD}/%.o: \\$(D_LIB_AS)/%.s \\$(SPARTANMC_AS) \\$(SPARTANMC_GCC) ${F_SYSTEM_S} \\$(MAKEFILE_LIST) | ${D_BUILD}
	\\$(SPARTANMC_GCC) -c ${_ASFLAGS} \\$< -o \\$@
	@\\$(call ECHO_TARGET_DONE)

${D_BUILD}/%.sphl: ${D_BUILD}/%.sph
	printf \\"%s\\\\n\\" ${SBI_CORE_ID} > \\$@
	cat \\$< >> \\$@
	@\\$(call ECHO_TARGET_DONE)

${F_LST}: ${F_ELF}
	\\$(SPARTANMC_OBJDUMP) -D ${F_ELF} > \\$@

.PRECIOUS: ${D_BUILD}/%.s

ELF_FILES+=${F_ELF}


else # BUILD_WITH_SDK == 1

.SECONDEXPANSION:
${F_ELF}: ${SDK_DIR}/\\$(SDK_PROJECT_NAME)/Release/\\$(SDK_PROJECT_NAME).elf
	cp -f $< \\$@

ELF_FILES+=${F_ELF}

endif # BUILD_WITH_SDK
