proc getLibroot {} {
	global TOOLCHAIN
	global env
	# check whether environment variable pointing to simulation libraries is set
	if { $TOOLCHAIN == "quartus" } { 
		if {![info exists env(QUARTUS_SIMLIBDIR)]} {
			return -code error { Error: QUARTUS_SIMLIBDIR not set! Please specify this variable within your shell environment. It should point to the path where your quartus simulation libraries (altera_mf_ver, lpm_ver, etc.) could be found. }
		}
		return $env(QUARTUS_SIMLIBDIR)

	} elseif { $TOOLCHAIN == "lattice" } { 
		if {![info exists env(SPMC_LATTICE_SIMLIBDIR)]} {
			return -code error { Error: SPMC_LATTICE_SIMLIBDIR not set! Please specify this variable within your shell environment. It should point to the path where your lattice simulation libraries (e. g. ecp5u_vlg) could be found. }
		}
		return $env(SPMC_LATTICE_SIMLIBDIR)
	} elseif { $TOOLCHAIN == "ise" } { 
		if {![info exists env(LMC_HOME)]} {
			return -code error { Error: LMC_HOME not set! Please specify this variable within your shell environment. It should point to the path where your xilinx simulation libraries (xilinxcorelib_ver, unimacro_ver, etc.) could be found. }
		}
		return $env(LMC_HOME)
	} elseif { $TOOLCHAIN == "vivado" } { 
		if {![info exists env(VIVADO_SIMLIBDIR)]} {
			return -code error { Error: VIVADO_SIMLIBDIR not set! Please specify this variable within your shell environment. It should point to the path where your xilinx simulation libraries (xilinxcorelib_ver, unimacro_ver, etc.) could be found. }
		}
		return $env(VIVADO_SIMLIBDIR)
	} else {
		return -code error { ERROR: Unknown toolchain }
	}
}


proc updateLibraries { libroot } {
	global DEVICE_SIMLIBS
	foreach simlib $DEVICE_SIMLIBS {
		vmap $simlib $libroot/$simlib
	}
}

proc updateLibrariesIfNeeded { libroot } {
	global DEVICE_SIMLIBS


	set oldLibFile "vsim/oldLibraries.txt"

	#load old libraries
	if { [file exists $oldLibFile] } {
		set fileId [open $oldLibFile r]
		set oldLibraries [read $fileId]
		close $fileId
	} else {
		set oldLibraries ""
	}

	#need to update vmap stuff?
	if { $oldLibraries != $DEVICE_SIMLIBS } {
		foreach simlib $oldLibraries { vmap -del $simlib }
		updateLibraries $libroot
	}



	# update saved list of libraries
	set fileId [open $oldLibFile "w"]
	puts -nonewline $fileId $DEVICE_SIMLIBS
	close $fileId



}
