/*
_______________________________________________________________________________

 Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 (http://www.mr.inf.tu-dresden.de) All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement: "This product includes
    software developed by the TU Dresden Chair for Embedded Systems and
    its contributors."

 4. Neither the name of the TU Dresden Chair for Embedded Systems nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS AND
 CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
_______________________________________________________________________________
*/

#include <string.h>
#include <stdio.h>
#include "jtagmon.h"

enum tap_states {
	TEST_LOGIC_RESET,
	RUN_TEST_IDLE,
	SELECT_DR,
	CAPTURE_DR,
	SHIFT_DR,
	EXIT1_DR,
	PAUSE_DR,
	EXIT2_DR,
	UPDATE_DR,
	SELECT_IR,
	CAPTURE_IR,
	SHIFT_IR,
	EXIT1_IR,
	PAUSE_IR,
	EXIT2_IR,
	UPDATE_IR
};

void jtagmon(unsigned char tck, unsigned char tms, unsigned char tdi) {
	static unsigned char last_tck = 1;
	static char tdi_written = 0;
	static int state = TEST_LOGIC_RESET;
	static char state_text[32] = "Test Logic Reset";
	char last_state_text[32];
	int last_state = state;

	strcpy(last_state_text, state_text);

	if (!last_tck && tck) {
		switch(state) {
			case TEST_LOGIC_RESET:
				if (tms) {
					state = TEST_LOGIC_RESET;
				} else {
					state = RUN_TEST_IDLE;
				}
				break;
			case RUN_TEST_IDLE:
				if (tms) {
					state = SELECT_DR;
				} else {
					state = RUN_TEST_IDLE;
				}
				break;
			case SELECT_DR:
				if (tms) {
					state = SELECT_IR;
				} else {
					state = CAPTURE_DR;
				}
				break;
			case CAPTURE_DR:
				if (tms) {
					state = EXIT1_DR;
				} else {
					state = SHIFT_DR;
				}
				break;
			case SHIFT_DR:
				if (tms) {
					state = EXIT1_DR;
				} else {
					state = SHIFT_DR;
				}
				break;
			case EXIT1_DR:
				if (tms) {
					state = UPDATE_DR;
				} else {
					state = PAUSE_DR;
				}
				break;
			case PAUSE_DR:
				if (tms) {
					state = EXIT2_DR;
				} else {
					state = PAUSE_DR;
				}
				break;
			case EXIT2_DR:
				if (tms) {
					state = UPDATE_DR;
				} else {
					state = SHIFT_DR;
				}
				break;
			case UPDATE_DR:
				if (tms) {
					state = SELECT_DR;
				} else {
					state = RUN_TEST_IDLE;
				}
				break;
			case SELECT_IR:
				if (tms) {
					state = TEST_LOGIC_RESET;
				} else {
					state = CAPTURE_IR;
				}
				break;
			case CAPTURE_IR:
				if (tms) {
					state = EXIT1_IR;
				} else {
					state = SHIFT_IR;
				}
				break;
			case SHIFT_IR:
				if (tms) {
					state = EXIT1_IR;
				} else {
					state = SHIFT_IR;
				}
				break;
			case EXIT1_IR:
				if (tms) {
					state = UPDATE_IR;
				} else {
					state = PAUSE_IR;
				}
				break;
			case PAUSE_IR:
				if (tms) {
					state = EXIT2_IR;
				} else {
					state = PAUSE_IR;
				}
				break;
			case EXIT2_IR:
				if (tms) {
					state = UPDATE_IR;
				} else {
					state = SHIFT_IR;
				}
				break;
			case UPDATE_IR:
				if (tms) {
					state = SELECT_DR;
				} else {
					state = RUN_TEST_IDLE;
				}
				break;
		}

		switch(state) {
			case TEST_LOGIC_RESET:
				strcpy(state_text, "Test Logic Reset");
				break;
			case RUN_TEST_IDLE:
				strcpy(state_text, "Run-Test / Idle");
				break;
			case SELECT_DR:
				strcpy(state_text, "Select-DR");
				break;
			case CAPTURE_DR:
				strcpy(state_text, "Capture-DR");
				break;
			case SHIFT_DR:
				strcpy(state_text, "Shift-DR");
				break;
			case EXIT1_DR:
				strcpy(state_text, "Exit1-DR");
				break;
			case PAUSE_DR:
				strcpy(state_text, "Pause-DR");
				break;
			case EXIT2_DR:
				strcpy(state_text, "Exit2-DR");
				break;
			case UPDATE_DR:
				strcpy(state_text, "Update-DR");
				break;
			case SELECT_IR:
				strcpy(state_text, "Select-IR");
				break;
			case CAPTURE_IR:
				strcpy(state_text, "Capture-IR");
				break;
			case SHIFT_IR:
				strcpy(state_text, "Shift-IR");
				break;
			case EXIT1_IR:
				strcpy(state_text, "Exit1-IR");
				break;
			case PAUSE_IR:
				strcpy(state_text, "Pause-IR");
				break;
			case EXIT2_IR:
				strcpy(state_text, "Exit2-IR");
				break;
			case UPDATE_IR:
				strcpy(state_text, "Update-IR");
				break;
		}

		if (last_state != state) {
			if (tdi_written)
				fprintf(stderr,"\n");

			fprintf(stderr,"TAP state transition from %s to %s\n", last_state_text, state_text);
			tdi_written = 0;
		} else {
			fprintf(stderr,"%d",(tdi ? 1 : 0));
			tdi_written = 1;
		}
	}

	last_tck = tck;
}
