#include <perf.h>
#include <stdio.h>
#include <peripherals.h>
#include "algorithm.h"
#include "inBytes.h"
/* #define debugCorrelation */
/* #define debugGauss */
/* #define debugBacksubst */
/* #define debugOutAdaptive */
/* #define debugOutput */
/* Bug in compiler pipeline somewhere. for some combinations of output defines, we need to include this printf, or the */
/* code breaks */
/* #define magicalPrintf */
/* #define printOutput */
FILE *stdout = &UART_LIGHT_0_FILE;
signed int signExtend8toInt(signed char x) {
    signed int r;
    /* Sign extend from 8 bits */
  __asm__("sigex %0, 8": "=r"(r):"0"(x));
    return r;
}

signed char signExtend8to9(signed char x) {
    return ((signed char) signExtend8toInt(x));
}

int readAdaptiveInput(int inByteAdaptive, int totalInBytes,
		      int samplesForAdaptivePass, long *correlationSum,
		      int *correlationHistory) {
    while ((samplesForAdaptivePass < 2048) && (inByteAdaptive < totalInBytes)) {
	/* read next sample */
	int sample = 0;
	{
	    int i = 0;
	    for (; i < 2; i++) {
		sample |=
		    ((inBytes[inByteAdaptive++] & 255) <<
		     ((1 ? ((2 - i) - 1) : i) * 8));
	    }
	}
	if (1 && (sample >= (1 << ((2 * 8) - 1)))) {
	    sample -= (1 << (2 * 8));
	}
	/* reset accumulators */
	if (samplesForAdaptivePass == 0) {
	    {
		int i = 0;
		for (; i <= 9; i++) {
		    correlationSum[i] = 0;
		}
	    }
	}
	/* buffer sample for correlation */
	{
	    int i = 9;
	    for (; i > 0; i--) {
		correlationHistory[i] = correlationHistory[i - 1];
	    }
	}
	correlationHistory[0] = sample;
	/* calculate (non normalized) autocorrelation values */
	{
	    int i = 0;
	    for (; (i <= samplesForAdaptivePass) && (i <= 9); i++) {
		/* Use 18bit 18bit -> 36 bit multiplication */
		long delta = ((long) sample) * correlationHistory[i];
		correlationSum[i] += delta;
	    }
	}
	samplesForAdaptivePass++;
    }
    return samplesForAdaptivePass;
}

void solveEquGauss(int r, int k, long abs, long maxAbs, int *correlationValue,
		   long *A) {
    {
	int i = 0;
	for (; i < (9 - 1); i++) {
	    maxAbs = 0;
	    for (r = (i + 1); r < 9; r++) {
		for (k = (i + 1); k < 9; k++) {
		    long x = A[(r * 9) + i];
		    long y = A[(i * 9) + k];
		    long z = A[(i * 9) + i];
		    /* long mul = x y; */
		    long mul = x * y;
		    long delta = mul / z;
		    A[(r * 9) + k] -= delta;
		    abs = A[(r * 9) + k];
		    if (abs < 0) {
			abs = (-abs);
		    }
		    if (abs > maxAbs) {
			maxAbs = abs;
		    }
		}
		correlationValue[r + 1] -=
		    ((A[(r * 9) + i] * correlationValue[i + 1]) /
		     A[(i * 9) + i]);
		abs = correlationValue[r + 1];
		if (abs < 0) {
		    abs = (-abs);
		}
		if (abs > maxAbs) {
		    maxAbs = abs;
		}
		A[(r * 9) + i] = 0;
		/* reduce values to 16bit */
		while (maxAbs >= (1 << 15)) {
		    maxAbs /= 2;
		    correlationValue[r + 1] >>= 1;
		    for (k = (i + 1); k <= 9; k++) {
			A[(r * 9) + k] >>= 1;
		    }
		}
	    }
	}
    }
}

long backSubstitution(long *coefficients, int k, long abs,
		      int *correlationValue, long *A) {
    {
	int i = 9;
	for (; (i--) != 0;) {
	    coefficients[i] = correlationValue[i + 1];
	    for (k = (9 - 1); k > i; k--) {
		long delta = A[(i * 9) + k] * coefficients[k];
		long shiftedDelta = delta >> 12;
		coefficients[i] -= shiftedDelta;
	    }
	    long shifted = coefficients[i] << 12;
	    coefficients[i] = (shifted / A[(i * 9) + i]);
	    /* fallback to simple dpcm if coefficients do not fit into fixedpoint format bits */
	    abs = coefficients[i];
	    if (abs < 0) {
		abs = (-abs);
	    }
	    if (abs >= (1 << ((4 + 12) - 1))) {
		coefficients[0] = (1 << 12);
		for (k = 1; k < 9; k++) {
		    coefficients[k] = 0;
		}
		break;
	    }
	}
    }
    return abs;
}

int writeCoefficients(signed char *outBytesAdaptive, long *coefficients,
		      int k) {
    int byteCounterForAdaptivePass = 0;
    int bufferForAdaptive = 0;
    int bitsForAdaptive = 0;
    for (k = 0; k < 9; k++) {
	int i = 4 + 12;
	while (i != 0) {
	    if ((i + bitsForAdaptive) >= 8) {
		bitsForAdaptive = (8 - bitsForAdaptive);
		i = (i - bitsForAdaptive);
		int shiftedA = bufferForAdaptive << bitsForAdaptive;
		signed char shiftedB = (1 << bitsForAdaptive) - 1;
		long shiftedC = coefficients[k] >> i;
		signed char anded = shiftedB & shiftedC;
		signed int andedSigex = signExtend8toInt(anded);
		outBytesAdaptive[byteCounterForAdaptivePass++] =
		    ((signed char) (shiftedA | andedSigex));
		bitsForAdaptive = 0;
	    }
	    else {
		bitsForAdaptive += i;
		bufferForAdaptive =
		    ((bufferForAdaptive << i) |
		     (((1 << i) - 1) & coefficients[k]));
		i = 0;
	    }
	}
    }
    return k;
}

void calcAutocorrelation(int samplesForAdaptivePass, long *correlationSum,
			 int *correlationValue, long *abs, long *maxAbs) {
    {
	int i = 0;
	for (; i <= 9; i++) {
	    correlationValue[i] =
		((int) (correlationSum[i] / (samplesForAdaptivePass - i)));
	    (*abs) =
		((correlationValue[i] <
		  0) ? (-correlationValue[i]) : correlationValue[i]);
	    if ((*abs) > (*maxAbs)) {
		(*maxAbs) = (*abs);
	    }
	}
    }
}

void extractEqu(int *correlationValue, long *A, int *k, long *maxAbs) {
    /* reduce values to 16bit (to avoid overflows in integer arithmetic) */
    while ((*maxAbs) >= (1 << 15)) {
	(*maxAbs) /= 2;
	{
	    int i = 0;
	    for (; i <= 9; i++) {
		correlationValue[i] >>= 1;
	    }
	}
    }
    /* derive linear equation system Ax=B from autocorrelation values, B[i]=correlationValue[i+1] */
    {
	int i = 0;
	for (; i < 9; i++) {
	    for ((*k) = 0; (*k) < 9; (*k)++) {
		int n = (i * 9) + (*k);
		int c =
		    correlationValue[((i > (*k)) ? (i - (*k)) : ((*k) - i))];
		A[n] = c;
	    }
	}
    }
}

long readSample(signed char *outBytes, int *inByte, int *outByte) {
    long sample = 0;
    {
	int i = 0;
	for (; i < 2; i++) {
	    if ((*inByte) < (9 * 2)) {
		outBytes[(*outByte)++] = inBytes[*inByte];
	    }
	    sample |=
		((inBytes[(*inByte)++] & 255) <<
		 ((1 ? ((2 - i) - 1) : i) * 8));
	}
    }
    long signCompare = 1 << ((2 * 8) - 1);
    if (1 && (sample >= signCompare)) {
	sample -= (1 << (2 * 8));
    }
    return sample;
}

int predict(int *predictionHistory, long *coefficientsNew) {
    int prediction;
    if (9 == 0) {
	prediction = predictionHistory[0];
    }
    else {
	prediction = 0;
	{
	    int i = 0;
	    for (; i < 9; i++) {
		prediction +=
		    ((predictionHistory[i] * coefficientsNew[i]) >> 12);
	    }
	}
    }
    return prediction;
}

void writeOnes(signed char *outBytes, int ones, int *outByte, int *bits,
	       char *buffer) {
    while (ones != 0) {
	if ((ones + (*bits)) >= 8) {
	    (*bits) = (8 - (*bits));
	    ones -= (*bits);
	    outBytes[(*outByte)++] =
		signExtend8to9((signed char) (((*buffer) << (*bits)) |
					      ((1 << (*bits)) - 1)));
	    (*bits) = 0;
	}
	else {
	    (*bits) += ones;
	    char bufferShifted = (*buffer) << ones;
	    char onesShifted = (1 << ones) - 1;
	    (*buffer) = ((bufferShifted | onesShifted) & 255);
	    ones = 0;
	}
    }
}

void writeZeros(signed char *outBytes, int *outByte, int *bits, char *buffer) {
    if ((*bits) == 7) {
	outBytes[(*outByte)++] = signExtend8to9((*buffer) << 1);
	(*bits) = 0;
    }
    else {
	(*buffer) <<= 1;
	(*buffer) &= 255;
	(*bits) += 1;
    }
}

void writeResidualBlock(signed char *outBytes, int error, int *outByte,
			int *bits, char *buffer) {
    int i = 4;
    while (i != 0) {
	if ((i + (*bits)) >= 8) {
	    (*bits) = (8 - (*bits));
	    i -= (*bits);
	    outBytes[(*outByte)++] =
		signExtend8to9(((*buffer) << (*bits)) |
			       (((1 << (*bits)) - 1) & (error >> i)));
	    (*bits) = 0;
	}
	else {
	    (*bits) += i;
	    (*buffer) =
		((char) (((*buffer) << i) | (((1 << i) - 1) & error)));
	    (*buffer) &= 255;
	    i = 0;
	}
    }
}

void prepareRice(long sample, int prediction, int *error, int *ones) {
    (*error) = (sample - prediction);
    /* prediction error */
    (*error) = (((*error) >= 0) ? (2 * (*error)) : (((-2) * (*error)) - 1));
    /* code spreading to positive values */
    (*ones) = ((*error) >> 4);
    /* run length of unary code */
    (*error) = ((*error) & ((1 << 4) - 1));
    /* residual for block code */
}

int copyOldValues(int outByte, signed char *outBytes,
		  const signed char *outBytesAdaptive,
		  const long *coefficients, long *coefficientsNew) {
    {
	int c = 0;
	for (; c < ((9 * (4 + 12)) / 8); c++) {
	    outBytes[outByte++] = outBytesAdaptive[c];
	}
    }
    {
	int i = 0;
	for (; i < 9; i++) {
	    coefficientsNew[i] = coefficients[i];
	}
    }
    return outByte;
}

struct perf_auto_result perf_results[] = {
    {.name = "main.c:main:if:496"},
    {.name =
     "main.c:main:samplesForAdaptivePass=readAdaptiveInput(inByteAdaptive, totalInBytes, samplesForAdaptivePass, correlationSum, correlationHistory):531"},
    {.name =
     "main.c:main:calcAutocorrelation(samplesForAdaptivePass, correlationSum, correlationValue, ( & abs), ( & maxAbs)):541"},
    {.name =
     "main.c:main:extractEqu(correlationValue, A, ( & k), ( & maxAbs)):548"},
    {.name =
     "main.c:main:solveEquGauss(r, k, abs, maxAbs, correlationValue, A):552"},
    {.name =
     "main.c:main:backSubstitution(coefficients, k, abs, correlationValue, A):563"},
    {.name =
     "main.c:main:writeCoefficients(outBytesAdaptive, coefficients, k):572"},
    {.name =
     "main.c:main:outByte=copyOldValues(outByte, outBytes, outBytesAdaptive, coefficients, coefficientsNew):589"},
    {.name = "main.c:main:for:592"},
    {.name = "main.c:main:for:632"},
    {.name = "main.c:main:if:669"},
    {.name = "main.c:main:printf(\"Compressed size: %d\\n\", outByte):678"},
    {.name = "main.c:main:printf(\"EOF\\n\"):681"},
};
void main() {
    perf_auto_init();
    /* debugging_initialize(); */
    /* stdio_uart_light_open(&UART_LIGHT_0); */
    /*      test(); */
    /* printf("algo start \n"); */
    int inByte = 0;
    /* number of read input bytes */
    int inByteAdaptive = 0;
    int outByte = 0;
    /* number of written output bytes */
    /* printf("outBytesAdaptivelength = %d\n", outBytesAdaptivelength); */
    int predictionHistory[9];
    /* last samples used for prediction */
    int totalInBytes = 4096;
    /* printf("totalInBytes = %d\n", totalInBytes); */
    int totalBlockRemainder = totalInBytes % 2048;
    /* printf("totalBlockRemainder = %d\n", totalBlockRemainder); */
    int totalBlocks = totalInBytes / 2048;
    /* printf("totalBlocks = %d\n", totalBlocks); */
    /* printf("totalBlockRemainder = %d\n", totalBlockRemainder); */
    signed char outBytes[1510];
    signed char outBytesAdaptive[((9 * (4 + 12)) / 8)];
    perf_auto_start();
    if ((totalBlockRemainder > 0)) {
	totalBlocks++;
    }
    perf_auto_stop(0, perf_results);
    /* printf("totalBlocks = %d\n", totalBlocks); */
    /* two different histories are required predict first sample of new block from last samples of old block after the */
    /* new block was passed ones for autocorrelation */
    int blocks = 0;
    /*      for (blocks = 0; blocks < totalBlocks; blocks = blocks + 2) { */
    /* printf("blocks = %d\n", blocks); */
    char lastBlock = blocks < totalBlocks;
    /* adapt coefficients to current block */
    int r, k;
    long abs;
    long maxAbs = 0;
    /* used for numerical stability */
    int correlationValue[(9 + 1)];
    /* autocorrelation values */
    long A[(9 * 9)];
    /* quadratic matrix of linear equation system */
    long coefficients[9];
    /* prediction coefficients */
    int samplesForAdaptivePass = 0;
    long correlationSum[(9 + 1)];
    /* accumulators for autocorrelation values */
    int correlationHistory[(9 + 1)];
    /* last samples used for autocorrelation */
    perf_auto_start();
    samplesForAdaptivePass =
	readAdaptiveInput(inByteAdaptive, totalInBytes,
			  samplesForAdaptivePass, correlationSum,
			  correlationHistory);
    perf_auto_stop(1, perf_results);
    /* end of block reached? */
    /* calculate autocorrelation values */
    perf_auto_start();
    calcAutocorrelation(samplesForAdaptivePass, correlationSum,
			correlationValue, (&abs), (&maxAbs));
    perf_auto_stop(2, perf_results);
    perf_auto_start();
    extractEqu(correlationValue, A, (&k), (&maxAbs));
    perf_auto_stop(3, perf_results);
    /* solve linear equation system by gaussian elimination (without pivot search) to calculate coefficients */
    perf_auto_start();
    solveEquGauss(r, k, abs, maxAbs, correlationValue, A);
    perf_auto_stop(4, perf_results);
    /* backsubstitution */
    perf_auto_start();
    backSubstitution(coefficients, k, abs, correlationValue, A);
    perf_auto_stop(5, perf_results);
    /* write coefficients to output stream */
    perf_auto_start();
    writeCoefficients(outBytesAdaptive, coefficients, k);
    perf_auto_stop(6, perf_results);
    /* predict sample and encode prediction error (except for the first unencoded samples) */
    /*  */
    /* REAL PASS */
    /*  */
    int bits = 0;
    /* number of valid bits in buffer */
    char buffer = 0;
    /* bitbuffer for packing bitstream into bytes */
    long coefficientsNew[9] = { 0 };
    /* prediction coefficients */
    perf_auto_start();
    outByte =
	copyOldValues(outByte, outBytes, outBytesAdaptive, coefficients,
		      coefficientsNew);
    perf_auto_stop(7, perf_results);
    perf_auto_start();
    {
	int samples = 0;
	for (; samples < (2048 / 2); samples++) {
	    /* read next sample */
	    long sample = readSample(outBytes, (&inByte), (&outByte));
	    if ((inByte > (9 * 2))) {
		/* staticadaptive prediction */
		int prediction = predict(predictionHistory, coefficientsNew);
		/* prepare output of rice coder */
		int error = 0;
		int ones = 0;
		prepareRice(sample, prediction, (&error), (&ones));
		/* write ones */
		writeOnes(outBytes, ones, (&outByte), (&bits), (&buffer));
		/* write zero (to terminate unary code) */
		writeZeros(outBytes, (&outByte), (&bits), (&buffer));
		/* write residual block code */
		writeResidualBlock(outBytes, error, (&outByte), (&bits),
				   (&buffer));
	    }
	    /* buffer sample for next prediction */
	    {
		int i = 9 - 1;
		for (; i > 0; i--) {
		    predictionHistory[i] = predictionHistory[(i - 1)];
		}
	    }
	    predictionHistory[0] = sample;
	}
    }
    perf_auto_stop(8, perf_results);
    perf_auto_start();
    {
	int samples = 2048 / 2;
	for (; samples < 2048; samples++) {
	    /* read next sample */
	    long sample = readSample(outBytes, (&inByte), (&outByte));
	    if ((inByte > (9 * 2))) {
		/* staticadaptive prediction */
		int prediction = predict(predictionHistory, coefficientsNew);
		/* prepare output of rice coder */
		int error = 0;
		int ones = 0;
		prepareRice(sample, prediction, (&error), (&ones));
		/* write ones */
		writeOnes(outBytes, ones, (&outByte), (&bits), (&buffer));
		/* write zero (to terminate unary code) */
		writeZeros(outBytes, (&outByte), (&bits), (&buffer));
		/* write residual block code */
		writeResidualBlock(outBytes, error, (&outByte), (&bits),
				   (&buffer));
	    }
	    /* buffer sample for next prediction */
	    {
		int i = 9 - 1;
		for (; i > 0; i--) {
		    predictionHistory[i] = predictionHistory[(i - 1)];
		}
	    }
	    predictionHistory[0] = sample;
	}
    }
    perf_auto_stop(9, perf_results);
    /* 1-padding to full byte => avoids misinterpretation of padding as additional sample (unterminated unary code) */
    perf_auto_start();
    if ((lastBlock && (bits > 0))) {
	outBytes[(outByte++)] =
	    ((char) ((buffer << (8 - bits)) | ((1 << (8 - bits)) - 1)));
    }
    perf_auto_stop(10, perf_results);
    /* } */
    perf_auto_start();
    printf("Compressed size: %d\n", outByte);
    perf_auto_stop(11, perf_results);
    /* return outByte; */
    /* } */
    perf_auto_start();
    printf("EOF\n");
    perf_auto_stop(12, perf_results);
    perf_auto_print(13, perf_results);
}
