#include <stdio.h>
#include <peripherals.h>
#include "algorithm.h"
#include "inBytes.h"


//#define debugCorrelation
//#define debugGauss
//#define debugBacksubst
//#define debugOutAdaptive
//#define debugOutput
//Bug in compiler / pipeline somewhere. for some combinations of output defines, we need to include this printf, or the
//code breaks
//#define magicalPrintf
//#define printOutput
#define TASKS 2

FILE * stdout = &SIM_PRINT_0_FILE;

signed int signExtend8toInt(signed char x) {
#ifndef __x86_64
	signed int r;
    //Sign extend from 8 bits
    __asm__ ("sigex %0, 8"
    : "=r" (r)
    : "0" (x)
    );
    return r;
#else
	return x;
#endif

}

signed char signExtend8to9(signed char x) {
	return (signed char) signExtend8toInt(x);
}

int readAdaptiveInput(int inByteAdaptive, int totalInBytes, int samplesForAdaptivePass, long *correlationSum,
                       int *correlationHistory) {
	while ((samplesForAdaptivePass < blockSize) && (inByteAdaptive < totalInBytes)) {


            // read next sample
            int sample = 0;
            for (int i = 0; i < SAMPLE_BYTES; i++) {
                sample |= (inBytes[inByteAdaptive++] & 0xFF) << ((SAMPLE_BIG_ENDIAN ? SAMPLE_BYTES - i - 1 : i) * 8);
            }
            if (SAMPLE_SIGNED && sample >= 1 << (SAMPLE_BYTES * 8 - 1)) {
                sample -= 1 << (SAMPLE_BYTES * 8);
            }

            // reset accumulators
            if (samplesForAdaptivePass == 0) {
                for (int i = 0; i <= order; i++) correlationSum[i] = 0;
            }

            // buffer sample for correlation
            for (int i = order; i > 0; i--) correlationHistory[i] = correlationHistory[i - 1];
            correlationHistory[0] = sample;

            // calculate (non normalized) autocorrelation values
            for (int i = 0; i <= samplesForAdaptivePass && i <= order; i++) {
#ifdef debugCorrelation
                printf("sample = %d\n", sample);
                printLong("correlationHistory",i,correlationHistory[i]);
#endif


                //Use 18bit * 18bit -> 36 bit multiplication
                long delta = ((long)sample) * correlationHistory[i];
                correlationSum[i] += delta;
#ifdef debugCorrelation
                printLong("delta",-1,delta);
                printLong("correlationSum",i,correlationSum[i]);
#endif
            }

            samplesForAdaptivePass++;
        }
	return samplesForAdaptivePass;
}

void solveEquGauss(int k, long abs, long maxAbs, int *correlationValue, long *A) {
	for (int i = 0; i < order - 1; i++) {
            maxAbs = 0;
            for (int r = i + 1; r < order; r++) {
#ifdef debugGauss
                printf("i = %d\n", i);
                printf("r = %d\n", r);
#endif
                for (k = i + 1; k < order; k++) {
                    long x = A[r * order + i];
                    long y = A[i * order + k];
                    long z = A[i * order + i];
                    //long mul = x * y;
                    long mul =x*y;
                    long delta = mul / z;
                    A[r * order + k] -= delta;
#ifdef debugGauss
                    printf("k = %d\n", k);
                    printLong("x",-1,x);
                    printLong("y",-1,y);
                    printLong("z",-1,z);
                    printLong("mul",-1,mul);
                    printLong("delta",-1,delta);
                    printLong("A",r * order + k, A[r * order + k]);
#endif
                    abs = A[r * order + k];
                    if (abs < 0) abs = -abs;
                    if (abs > maxAbs) maxAbs = abs;
                }
#ifdef debugGauss
                for (int i=0;i<order*order;i++) {
                    printLong("A",i,A[i]);
                }
                printLong("maxAbs",-1,maxAbs);
#endif

                correlationValue[r + 1] -= (A[r * order + i] * correlationValue[i + 1]) / A[i * order + i];
                abs = correlationValue[r + 1];
                if (abs < 0) abs = -abs;
                if (abs > maxAbs) maxAbs = abs;
                A[r * order + i] = 0;

                // reduce values to 16bit
                while (maxAbs >= 1 << 15) {
                    maxAbs /= 2;
                    correlationValue[r + 1] >>= 1;
                    for (k = i + 1; k <= order; k++) A[r * order + k] >>= 1;
                }


#ifdef debugGauss
                for (int i=0;i<order*order;i++) {
                    printLong("A",i,A[i]);
                }

                for (int i=0;i<order+1;i++) {
                    printLong("correlationValue",i,correlationValue[i]);
                }
#endif
            }
        }
}

long backSubstitution(long *coefficients, int k, long abs, int *correlationValue, long *A) {
	for (int i = order; i-- != 0; ) {
            coefficients[i] = correlationValue[i + 1];
#ifdef debugBacksubst
            printLong("coefficients",i,coefficients[i]);
#endif
            for (k = order - 1; k > i; k--) {
#ifdef debugBacksubst
                printf("k = %d\n", k);
#endif
                long delta = A[i * order + k] * coefficients[k];
                long shiftedDelta = delta >> FRACTION;
#ifdef debugBacksubst
                printLong("delta",-1,delta);
                printLong("shiftedDelta",-1,shiftedDelta);
                if (shiftedDelta > INTMAX_MAX || shiftedDelta < INTMAX_MIN )
                    printf("too big\n");
#endif

                coefficients[i] -= shiftedDelta;
#ifdef debugBacksubst
                printLong("coefficients",i,coefficients[i]);
#endif
            }

            long shifted = coefficients[i] << FRACTION;
#ifdef debugBacksubst
            printLong("shifted",-1,shifted);
#endif

            coefficients[i] = shifted / A[i * order + i];

#ifdef debugBacksubst
            printLong("coefficients",i,coefficients[i]);
#endif

            // fallback to simple dpcm if coefficients do not fit into fixedpoint format bits
            abs = coefficients[i];
            if (abs < 0) abs = -abs;
            if (abs >= 1 << (INTEGER + FRACTION - 1)) {
#ifdef debugBacksubst
                printf("break");
#endif
                coefficients[0] = 1 << FRACTION;
                for (k = 1; k < order; k++) coefficients[k] = 0;
                break;
            }
        }
	return abs;
}

int writeCoefficients(signed char *outBytesAdaptive, long *coefficients, int k) {
	int byteCounterForAdaptivePass = 0;
	int bufferForAdaptive=0;
	int bitsForAdaptive=0;
	for (k = 0; k < order; k++) {

#ifdef debugOutAdaptive
            printf("k = %d\n", k);
#endif
            int i = INTEGER + FRACTION;
            while (i != 0) {
#ifdef debugOutAdaptive
                printf("i = %d\n", i);
#endif
                if (i + bitsForAdaptive >= 8) {
#ifdef debugOutAdaptive
                    printf("bigger\n");
#endif
                    bitsForAdaptive = 8 - bitsForAdaptive;
                    i = i - bitsForAdaptive;
#ifdef debugOutAdaptive
                    printf("bitsForAdaptive = %d\n", bitsForAdaptive);
                    printf("i = %d\n", i);
#endif
                    int shiftedA = bufferForAdaptive << bitsForAdaptive;
                    signed char shiftedB = ((1 << bitsForAdaptive) - 1);


                    long shiftedC = (coefficients[k] >> i);
                    signed char anded = (shiftedB & shiftedC);


                    signed int andedSigex = signExtend8toInt(anded);

                    outBytesAdaptive[byteCounterForAdaptivePass++] = (signed char) (shiftedA | andedSigex);
#ifdef debugOutAdaptive
                    printf("shiftedA = %d\n", shiftedA);
                    //signed int shiftedBSigex = shiftedB;
                    //printf("shiftedB = %d\n", shiftedBSigex); //TODO check sign extend
                    printLong("shiftedC",-1,shiftedC);

                    printf("anded = %d\n", andedSigex); //TODO check sign extend
                    printf("outBytesAdaptive[%d] = %d\n",
                           byteCounterForAdaptivePass-1,
                           outBytesAdaptive[byteCounterForAdaptivePass - 1]);
#endif
                    bitsForAdaptive = 0;
                } else {
#ifdef debugOutAdaptive
                    printf("smaller\n");
#endif
                    bitsForAdaptive += i;
                    bufferForAdaptive = ((bufferForAdaptive << i) | (((1 << i) - 1) & coefficients[k]));
                    i = 0;
#ifdef debugOutAdaptive
                    printf("bitsForAdaptive = %d\n", bitsForAdaptive);
                    printf("bufferForAdaptive = %d\n", bufferForAdaptive);
#endif
                }
            }
        }
	return k;
}

void calcAutocorrelation(int samplesForAdaptivePass, long *correlationSum, int *correlationValue, long *abs,
                         long *maxAbs) {
	for (int i = 0; i <= order; i++) {
            correlationValue[i] = (int) (correlationSum[i] / (samplesForAdaptivePass - i));
            (*abs) = correlationValue[i] < 0 ? -correlationValue[i] : correlationValue[i];
            if ((*abs) > (*maxAbs)) (*maxAbs) = (*abs);
        }
}

void extractEqu(int *correlationValue, long *A, long *maxAbs) {// reduce values to 16bit (to avoid overflows in integer arithmetic)
	while ((*maxAbs) >= 1 << 15) {
            (*maxAbs) /= 2;
            for (int i = 0; i <= order; i++) correlationValue[i] >>= 1;
        }

	// derive linear equation system Ax=B from autocorrelation values, B[i]=correlationValue[i+1]
	for (int i = 0; i < order; i++) {
            for (int k = 0; k < order; k++) {
                int n = i * order + k;
                int c = correlationValue[i > k ? i - k : k - i];
                A[n] = c;
#ifdef debugGauss
                printLong("A",n,A[n]);
#endif
            }
        }
}

long readSample(signed char *outBytes, int *inByte, int *outByte) {
	long sample=0;
	for (int i = 0; i < SAMPLE_BYTES; i++) {
#ifdef magicalPrintf
                printf("i2 = %d\n", i);
#endif
                if ((*inByte) < predictionHistorylength * SAMPLE_BYTES) {
                    outBytes[(*outByte)++] = inBytes[(*inByte)];
#ifdef printOutput
                    printf("B. outBytes[%d] = %d\n", (*outByte)-1,outBytes[(*outByte) - 1]);
#endif
                }
#ifdef debugOutput
                printLong("sample",-1,sample);
#endif
                sample |= (inBytes[(*inByte)++] & 0xFF) << ((SAMPLE_BIG_ENDIAN ? SAMPLE_BYTES - i - 1 : i) * 8);
#ifdef debugOutput
                printLong("sample",-1,sample);
#endif
            }
	long signCompare = 1L << (SAMPLE_BYTES * 8 - 1);
#ifdef debugOutput
	printLong("signCompare",-1,signCompare);
#endif
	if (SAMPLE_SIGNED && sample >= signCompare) {
		sample -= 1 << (SAMPLE_BYTES * 8);
#ifdef debugOutput
		printLong("signed stuff, sample",-1,sample);
#endif
	}
	return sample;
}

int predict(int *predictionHistory, long *coefficientsNew) {
	int prediction;
	if (order == 0) {
                    prediction = predictionHistory[0];
                } else {
                    prediction = 0;
                    for (int i = 0; i < predictionHistorylength; i++) {
                        prediction += (predictionHistory[i] * coefficientsNew[i]) >> FRACTION;
#ifdef debugOutput
                        printf("i = %d\n", i);
                        printf("prediction = %d\n", prediction);
#endif
                    }
                }
	return prediction;
}

void writeOnes(signed char *outBytes, int ones, int *outByte, int *bits, char *buffer) {
	while (ones != 0) {
#ifdef debugOutput
                    printf("ones neq 0\n");
#endif
                    if (ones + (*bits) >= 8) {
                        (*bits) = 8 - (*bits);
                        ones -= (*bits);
#ifdef debugOutput
                        printf("ones = %d\n", ones);
                        printf("bits = %d\n", bits);
#endif
                        outBytes[(*outByte)++] = signExtend8to9(
                                (signed char) (((*buffer) << (*bits)) | ((1 << (*bits)) - 1)));
#ifdef printOutput
                        printf("C. outBytes[%d] = %d\n", (*outByte)-1,outBytes[(*outByte) - 1]);
#endif
                        (*bits) = 0;
                    } else {
#ifdef debugOutput
                        printf("ones = %d\n", ones);
                        printf("X. buffer = %d\n", signExtend8toInt(buffer));
#endif
                        (*bits) += ones;
                        char bufferShifted = ((*buffer) << ones);
                        char onesShifted = ((1 << ones) - 1);
                        (*buffer) = (bufferShifted | onesShifted) & 0xFF;
                        ones = 0;
#ifdef debugOutput
                        printf("bufferShifted = %d\n", signExtend8toInt(bufferShifted));
                        printf("onesShifted = %d\n", onesShifted);
                        printf("bits = %d\n", bits);
                        printf("A. buffer = %d\n", signExtend8toInt(buffer));
#endif
                    }
                }
}

void writeZeros(signed char *outBytes, int *outByte, int *bits, char *buffer) {
	if ((*bits) == 7) {
                    outBytes[(*outByte)++] = signExtend8to9((*buffer) << 1);
#ifdef printOutput
                    printf("D. outBytes[%d] = %d\n", (*outByte)-1,outBytes[(*outByte) - 1]);
#endif
                    (*bits) = 0;
                } else {
                    (*buffer) <<= 1;
                    (*buffer) &= 0xFF;
                    (*bits) += 1;
#ifdef debugOutput
                    printf("B. buffer = %d\n", signExtend8toInt(buffer));
                    printf("bits = %d\n", bits);
#endif
                }
}

void writeResidualBlock(signed char *outBytes, int error, int *outByte, int *bits, char *buffer) {
	int i = riceCoefficient;
	while (i != 0) {
#ifdef debugOutput
                    printf("i = %d\n", i);
#endif
                    if (i + (*bits) >= 8) {

                        (*bits) = 8 - (*bits);
                        i -= (*bits);
#ifdef debugOutput
                        printf("bits = %d\n", bits);
                        printf("i = %d\n", i);
#endif

                        outBytes[(*outByte)++] = signExtend8to9(((*buffer) << (*bits)) | (((1 << (*bits)) - 1) & (error >> i)));
#ifdef printOutput
                        printf("E. outBytes[%d] = %d\n", (*outByte)-1,outBytes[(*outByte) - 1]);
#endif
                        (*bits) = 0;
                    } else {
                        (*bits) += i;
                        (*buffer) = (char) (((*buffer) << i) | (((1 << i) - 1) & error));
                        (*buffer) &= 0xFF;
                        i = 0;

#ifdef debugOutput
                        printf("bits = %d\n", bits);
                        printf("C. buffer = %d\n", signExtend8toInt(buffer));
#endif
                    }
                }
}

void prepareRice(long sample, int prediction, int *error, int *ones) {
	(*error) = sample - prediction;                 // prediction error
#ifdef debugOutput
	printf("error = %d\n", error);
#endif
	(*error) = (*error) >= 0 ? 2 * (*error) : -2 * (*error) - 1;   // code spreading to positive values
#ifdef debugOutput
	printf("error = %d\n", error);
#endif
	(*ones) = (*error) >> riceCoefficient;            // run length of unary code
#ifdef debugOutput
	printf("ones = %d\n", ones);
#endif
	(*error) = (*error) & ((1 << riceCoefficient) - 1);    // residual for block code
#ifdef debugOutput
	printf("error = %d\n", error);
#endif

}

int copyOldValues(int outByte, signed char *outBytes, const signed char *outBytesAdaptive, const long *coefficients,
                  long *coefficientsNew) {
#ifdef adaptivePassEnabled
	for(int c=0;c<outBytesAdaptivelength;c++){
            outBytes[outByte++]=outBytesAdaptive[c];
#ifdef printOutput
            printf("A. outBytes[%d] = %d\n", outByte-1,outBytes[outByte - 1]);
#endif
        }
#ifdef printOutput
	printf("end of adaptive samples\n");
#endif
#endif

	for (int i=0;i<order;i++){
            coefficientsNew[i]=coefficients[i];
        }
	return outByte;
}

void main() {
	//debugging_initialize();
	//stdio_uart_light_open(&UART_LIGHT_0);
//	test();
	//printf("algo start \n");

	int inByte = 0;                               // number of read input bytes
	int inByteAdaptive = 0;
	int outByte = 0;                               // number of written output bytes



	//printf("outBytesAdaptivelength = %d\n", outBytesAdaptivelength);

	int predictionHistory[predictionHistorylength]; // last samples used for prediction
	int totalInBytes = inByteslength;
	//printf("totalInBytes = %d\n", totalInBytes);
	int totalBlockRemainder = totalInBytes % blockSize;
	//printf("totalBlockRemainder = %d\n", totalBlockRemainder);
	int totalBlocks = totalInBytes / blockSize;
	//printf("totalBlocks = %d\n", totalBlocks);
	//printf("totalBlockRemainder = %d\n", totalBlockRemainder);

	signed char outBytes[1510];
	signed char outBytesAdaptive[outBytesAdaptivelength];

	if (totalBlockRemainder > 0) {
		totalBlocks++;
	}

	//printf("totalBlocks = %d\n", totalBlocks);



	// two different histories are required predict first sample of new block from last samples of old block after the
	// new block was passed ones for autocorrelation
	int blocks = 0;
//	for (blocks = 0; blocks < totalBlocks; blocks = blocks + 2) {
	//printf("blocks = %d\n", blocks);
	char lastBlock = blocks < totalBlocks;
	// adapt coefficients to current block
	int k=order;
	long abs;
    long maxAbs = 0;                      // used for numerical stability
	int correlationValue[order + 1];       // autocorrelation values
	long A[order * order];   // quadratic matrix of linear equation system


	long coefficients[order];// prediction coefficients
#ifdef adaptivePassEnabled

	int samplesForAdaptivePass = 0;
	long correlationSum[order + 1];// accumulators for autocorrelation values
	int correlationHistory[order + 1];                // last samples used for autocorrelation


#ifdef debugCorrelation
    printf("samplesForAdaptivePass = %d\n", samplesForAdaptivePass);
    printf("inByteAdaptive = %d\n", inByteAdaptive);
#endif
	samplesForAdaptivePass = readAdaptiveInput(inByteAdaptive, totalInBytes, samplesForAdaptivePass, correlationSum,
											   correlationHistory);

	// end of block reached?


#if TASKS >= 3
#pragma microstreams task
#endif

	// calculate autocorrelation values
	calcAutocorrelation(samplesForAdaptivePass, correlationSum, correlationValue, &abs, &maxAbs);

#ifdef debugCorrelation
    for (int i=0;i<=order;i++) {
        printf("correlationValue[%d] = %d\n", i, correlationValue[i]);
    }
#endif
	extractEqu(correlationValue, A, &maxAbs);


		// solve linear equation system by gaussian elimination (without pivot search) to calculate coefficients
		solveEquGauss(k, abs, maxAbs, correlationValue, A);
#ifdef debugBacksubst
        for (int i=0;i<order*order;i++) {
            printLong("A",i,A[i]);
        }

        for (int i=0;i<order+1;i++) {
            printLong("correlationValue",i,correlationValue[i]);
        }
#endif
		// backsubstitution
		backSubstitution(coefficients, k, abs, correlationValue, A);

#ifdef debugOutAdaptive
        for (int i=0;i<order;i++) {
            printLong("coefficients",i,coefficients[i]);
        }
#endif

		// write coefficients to output stream
		writeCoefficients(outBytesAdaptive, coefficients, k);
		// predict sample and encode prediction error (except for the first unencoded samples)
#endif


#if TASKS == 2 || TASKS == 4
#pragma microstreams task
#endif
	///////////////////////////////////////////////////////////////////////////
	///////////////////REAL PASS///////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////


	int bits = 0;                               // number of valid bits in buffer
	char buffer = 0;                               // bitbuffer for packing bitstream into bytes
	long coefficientsNew[order]={0};                  // prediction coefficients

	outByte = copyOldValues(outByte, outBytes, outBytesAdaptive, coefficients, coefficientsNew);

#if TASKS >= 3
		for (int samples = 0; samples < blockSize / 2; samples++) {
#else
		#pragma microStreams loopOpt 2 distribution(38,62)
		for (int samples = 0; samples < blockSize; samples++) {
#endif 
			// read next sample
			long sample = readSample(outBytes, &inByte, &outByte);


			if (inByte > predictionHistorylength * SAMPLE_BYTES) {
#ifdef debugOutput
				printf("bigger than predictionHistorylength\n");
#endif

				// static/adaptive prediction
				int prediction = predict(predictionHistory, coefficientsNew);

				// prepare output of rice coder

				int error = 0;
				int ones = 0;
				prepareRice(sample, prediction, &error, &ones);

				// write ones
				writeOnes(outBytes, ones, &outByte, &bits, &buffer);

				// write zero (to terminate unary code)
				writeZeros(outBytes, &outByte, &bits, &buffer);

				// write residual block code
				writeResidualBlock(outBytes, error, &outByte, &bits, &buffer);
			}

			// buffer sample for next prediction
			for (int i = predictionHistorylength - 1; i > 0; i--) predictionHistory[i] = predictionHistory[i - 1];
			predictionHistory[0] = sample;


		}

#if TASKS >= 3
#pragma microstreams task

	for (int samples=blockSize/2;samples<blockSize;samples++){
		// read next sample
		long sample=readSample(outBytes, &inByte, &outByte);



		if (inByte > predictionHistorylength * SAMPLE_BYTES) {
#ifdef debugOutput
			printf("bigger than predictionHistorylength\n");
#endif

			// static/adaptive prediction
			int prediction = predict(predictionHistory, coefficientsNew);

			// prepare output of rice coder

			int error = 0;
			int ones=0;
			prepareRice(sample, prediction, &error, &ones);

			// write ones
			writeOnes(outBytes, ones, &outByte, &bits, &buffer);

			// write zero (to terminate unary code)
			writeZeros(outBytes, &outByte, &bits, &buffer);

			// write residual block code
			writeResidualBlock(outBytes, error, &outByte, &bits, &buffer);
		}

		// buffer sample for next prediction
		for (int i = predictionHistorylength - 1; i > 0; i--) predictionHistory[i] = predictionHistory[i - 1];
		predictionHistory[0] = sample;


	}
#endif
    // 1-padding to full byte => avoids misinterpretation of padding as additional sample (unterminated unary code)
    if (lastBlock && bits > 0){
        outBytes[outByte++] = (char) (((buffer) << (8 - bits)) | ((1 << (8 - bits)) - 1));
#ifdef printOutput
        printf("F. outBytes[%d] = %d\n", outByte-1,outBytes[outByte - 1]);
#endif
    }

    //}
	
    printf("Compressed size: %d\n",outByte);
    //return outByte;
//}
	printf("EOF\n");
}
