#include <stdio.h>
#include <peripherals.h>

void assertIntSigned(char * message, int expected, int actual) {
	if (expected!=actual) {
		printf("expected %d but got %d in ",expected,actual);
		printf("%s\n",message);
	}
}

void assertIntUnsigned(char * message, int expected, int actual) {
	if (expected!=actual) {
		printf("expected %d but got %d in ",expected,actual);
		printf("%s\n",message);
	}
}

void printLong(long value) {
    printf("%X %X",(int)(value>>18)&0x3FFFF, (int)(value&0x3FFFF));
}

void __attribute__((noinline,noclone)) assertLongSigned(char * message, long expected, long actual) {
	if (expected!=actual) {
		printf("expected ");
		printLong(expected);
		printf(" but got ");
		printLong(actual);
		printf(" in %s\n",message);
	}
}

void __attribute__((noinline,noclone)) assertLongUnsigned(char * message, long expected, long actual) {
	if (expected!=actual) {
		printf("expected ");
		printLong(expected);
		printf(" but got ");
		printLong(actual);
		printf(" in %s\n",message);
	}
}

unsigned long __attribute__((noinline,noclone,optimize("Os"))) shrLong(unsigned long a, const int b) {
	return a >> b;
}

unsigned long __attribute__((noinline,noclone,optimize("Os"))) shlLong(unsigned long a, const int b) {
	return a << b;
}


#define assertFunc(func,a,b,res) assertLongUnsigned(TOSTRING(func) "(" TOSTRING(a) ", " TOSTRING(b) ")", res, func(a,b))

#define testShrLong(a,b,res) assertFunc(shrLong, a, b, res)
#define testShrLongQuad(a,b,res) do { \
	testShrLong(a,b,res); \
	testShrLong(a,b+1,res>>1); \
	testShrLong(a,b+2,res>>2); \
	testShrLong(a,b+3,res>>3); \
	} while(0)

#define testShlLong(a,b,res) assertFunc(shlLong, a, b, res)
#define testShlLongQuad(a,b,res) do { \
	testShlLong(a,b,res); \
	testShlLong(a,b+1,res<<1); \
	testShlLong(a,b+2,res<<2); \
	testShlLong(a,b+3,res<<3); \
	} while(0)

#define TOSTRING(a) #a


FILE * stdout = &SIM_PRINT_0_FILE;

void main() {
	printf("start\n");

	
	testShrLongQuad(0xF23456789, 0,0xF23456789);
	testShrLongQuad(0xF23456789, 4,0xF2345678);
	testShrLongQuad(0xF23456789, 8,0xF234567);
	testShrLongQuad(0xF23456789,12,0xF23456);
	testShrLongQuad(0xF23456789,16,0xF2345);
	testShrLongQuad(0xF23456789,20,0xF234);
	testShrLongQuad(0xF23456789,24,0xF23);
	testShrLongQuad(0xF23456789,28,0xF2);
	testShrLongQuad(0xF23456789,32,0xF);

	testShlLongQuad(0xF23456789, 0,0xF23456789);
	testShlLongQuad(0xF23456789, 4,0x234567890);
	testShlLongQuad(0xF23456789, 8,0x345678900);
	testShlLongQuad(0xF23456789,12,0x456789000);
	testShlLongQuad(0xF23456789,16,0x567890000);
	testShlLongQuad(0xF23456789,20,0x678900000);
	testShlLongQuad(0xF23456789,24,0x789000000);
	testShlLongQuad(0xF23456789,28,0x890000000);
	testShlLongQuad(0xF23456789,32,0x900000000);




	printf("end\n\n\n\n");
}
