/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale      1ns / 10ps
`default_nettype none

module invalidaccess(
	clk,
	do_peri_renamed,
	di_peri_renamed,
	addr_peri_renamed,
	access_peri,
	wr_peri,
	reset_renamed,
	addr_high,
	mem_access,
	store_access_low,
	store_access_high
	);


parameter BASE_ADR	= 10'h0;

parameter DMA_BASE_ADR	= 10'h0;

parameter ALL_PORTS	= 2;
parameter DECODSIM_DEB	= "NO";


input	wire				clk;
input	wire	[17:0]			do_peri_renamed;	// Datenbus 
output	wire	[17:0]			di_peri_renamed;	// Datenbus
input	wire	[9:0]			addr_peri_renamed;	// Adressbus
input	wire				access_peri;	// chipselect Peripherie
input	wire				wr_peri;	// Peripherie-Schreibzugriff
input	wire				reset_renamed;
input wire[7:0] addr_high;
input wire mem_access;
input wire store_access_high;
input wire store_access_low;

	
wire					select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

reg definesok;
wire selectMem;
// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset_renamed		),
		.clk_peri(	clk	),
		.addr_peri(	addr_peri_renamed	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	),
		.reg_select()
			);

pselect #(
		.ADDR_WIDTH     (8),
		.BASE_WIDTH     (8),
		.BASE_ADDR      (DMA_BASE_ADR >> 10) 
	)
 pseldma			 
	(addr_high,mem_access,selectMem);

reg behave_invalid;
reg any_select_last;

reg selectMem_last;

//Mem access needs to be delayed one cycle to account for BRAM read
always @(posedge clk)
	selectMem_last <= selectMem;

wire any_select;
assign any_select = selectMem_last || reg_read;

always @(posedge clk)
	any_select_last <= any_select;

always @(posedge clk) begin
	if (reset_renamed)
		behave_invalid <= 0;
	if (reg_write)
		behave_invalid <= do_peri_renamed[0];
end

//One cycle too late --> invalid
assign di_peri_renamed = reset_renamed ? 18'b0 :
	behave_invalid ? any_select_last : any_select;

endmodule

