#include <stdio.h>
#include <peripherals.h>

#pragma GCC diagnostic ignored "-Woverflow"

void assertIntSigned(char * message, int expected, int actual) {
	if (expected!=actual) {
		printf("expected %d but got %d in ",expected,actual);
		printf("%s\n",message);
	}
}

void assertIntUnsigned(char * message, int expected, int actual) {
	if (expected!=actual) {
		printf("expected %d but got %d in ",expected,actual);
		printf("%s\n",message);
	}
}

void __attribute__((noinline,noclone)) assertLongSigned(char * message, long expected, long actual) {
	if (expected!=actual) {
		printf("expected %lx but got %lx in %s\n",expected, actual, message);
	}
}

void __attribute__((noinline,noclone)) assertLongUnsigned(char * message, long expected, long actual) {
	if (expected!=actual) {
		printf("expected %lx but got %lx in %s\n",expected, actual, message);
	}
}

int __attribute__((noinline,noclone)) mulIntSigned(int a, int b) {
	return a*b;
}

long __attribute__((noinline,noclone)) mulWideningSigned(int a, int b) {
	return ((long)a)*b;
}

long __attribute__((noinline,noclone)) mulLongSigned(long a, long b) {
	return a*b;
}

unsigned int __attribute__((noinline,noclone)) mulIntUnsigned(unsigned int a, unsigned int b) {
	return a*b;
}

unsigned long __attribute__((noinline,noclone)) mulWideningUnsigned(unsigned int a, unsigned int b) {
	return ((long)a)*b;
}


unsigned long __attribute__((noinline,noclone)) mulLongUnsigned(unsigned long a, unsigned long  b) {
	return a*b;
}

signed int __attribute__((noinline,noclone)) charToInt(signed char x) {
	return x;
}

long __attribute__((noinline,noclone)) intToLong(long x) {
	return x;
}


#define testIntSigned(a,b,res) assertIntSigned("signed int " TOSTRING(a) " * " TOSTRING(b),(res),mulIntSigned(a,b))
#define testIntUnsigned(a,b,res) assertIntUnsigned("unsigned int " TOSTRING(a) " * " TOSTRING(b),(res),mulIntUnsigned(a,b))
#define testIntSignedPermutations(a,b,res) do { testIntSigned(a,b,res); testIntSigned(-a,b,-res); testIntSigned(a,-b,-res); testIntSigned(-a,-b,res); testIntUnsigned(a,b,res);} while(0)


#define testWideningSigned(a,b,res) assertLongSigned("signed widening " TOSTRING(a) " * " TOSTRING(b),(res),mulWideningSigned(a,b))
#define testWideningUnsigned(a,b,res) assertLongUnsigned("unsigned widening " TOSTRING(a) " * " TOSTRING(b),(res),mulWideningUnsigned(a,b))


#define TOSTRING(a) #a
#define testLongSigned(a,b,res) assertLongSigned("signed long " TOSTRING(a) " * " TOSTRING(b),(res),mulLongSigned(a,b))
#define testLongUnsigned(a,b,res) assertLongUnsigned("unsigned long " TOSTRING(a) " * " TOSTRING(b),(res),mulLongUnsigned(a,b))

#define testWideningSignedPermutations(a,b,res) do { \
	testWideningSigned(a,b,res); \
	testWideningSigned(-a,b,-res); \
	testWideningSigned(a,-b,-res); \
	testWideningSigned(-a,-b,res); \
	testLongSigned(a,b,res); \
	testLongSigned(-a,b,-res); \
	testLongSigned(a,-b,-res); \
	testLongSigned(-a,-b,res); \
	testLongUnsigned(a,b,res); \
	testWideningUnsigned(a,b,res);\
	} while(0)

#define PRIMITIVE_CAT(a, ...) a ## __VA_ARGS__

//the preprocessor does not correctly sign extend, to we need to define our own function....
#define negLong(x) PRIMITIVE_CAT(0x10000, 00000-x)

#define testLongSignedPermutations(a,b,res) do { \
	testLongSigned(a,b,res); \
	testLongSigned(negLong(a),b,negLong(res)); \
	testLongSigned(a,negLong(b),negLong(res)); \
	testLongSigned(negLong(a),negLong(b),res); \
	testLongUnsigned(a,b,res); \
	} while(0)

FILE * stdout = &SIM_PRINT_0_FILE;

void main() {
	putsn("start\n");

	testIntSignedPermutations(1,1,1);
	testIntSignedPermutations(1,0,0);
	testIntSignedPermutations(0,1,0);
	testIntSignedPermutations(10,20,200);

	testIntSignedPermutations(2,0x8000,0x10000);

	testWideningSignedPermutations(1,1,1);
	testWideningSignedPermutations(1,0,0);
	testWideningSignedPermutations(0,1,0);
	testWideningSignedPermutations(10,20,200);


	testWideningSignedPermutations(5000,5000,25000000);

	testWideningSignedPermutations(7000,7000,49000000);

	//Check that no sign extension occurrs
	testWideningUnsigned(0x30000,1,0x30000);
	testWideningUnsigned(0x30000,0x10,0x300000);

	testLongSignedPermutations(0x100000010,2,0x200000020);
	testLongSignedPermutations(4,0x10000000F,0x40000003C);


	testLongSignedPermutations(4,0x10000000F,0x40000003C);

	//test bit between low and high
	testLongSignedPermutations(0x1F0000,5,0x9B0000);

	testLongSignedPermutations(0x3FFFF,1,0x3FFFF);
	//test uppermost bit
	testLongUnsigned(0x7FFFFFFFF,2,0xFFFFFFFFE);
	testLongUnsigned(0xFFFFFFFFF,1,0xFFFFFFFFF);
	testLongUnsigned(0xF00000000,1,0xF00000000);


	assertIntSigned("convert char -1 to int",-1, charToInt(-1));
	assertLongSigned("convert int -1 to long",-1L, intToLong(-1));


	putsn("end\n\n\n\n");
}
