/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


///////////////////////////////////////////////////////////////////////////////
// Projekt : SpartanMC
// Dateiname : port_out.v
// Autor : Steffen Kunze
// Modul : port_out
//
// Beschreibung : Port_out ist ein Ausgangs-Port mit konfigurierbarer Breite.
//		  Es existiert ein output enable fuer die einzelnen Bits.
//
///////////////////////////////////////////////////////////////////////////////

`timescale      1ns / 10ps
`default_nettype none

module read_undefined(
	clk_peri,
	do_peri,
	di_peri,
	addr_peri,
	access_peri,
	wr_peri,
	reset
	);


parameter BASE_ADR	= 10'h0;
parameter PORT_WIDTH	= 18;

parameter DAT_ADR	= 0; 

parameter ALL_PORTS	= 1;
parameter DECODSIM_DEB	= "NO";


// inputs & outputs des Systembus
input	wire				clk_peri;
input	wire	[PORT_WIDTH-1:0]	do_peri;	// Datenbus 
output	wire	[17:0]			di_peri;	// Datenbus
input	wire	[9:0]			addr_peri;	// Adressbus
input	wire				access_peri;	// chipselect Peripherie
input	wire				wr_peri;	// Peripherie-Schreibzugriff
input	wire				reset;


// Dekodierung der Adressen ********************** START *************************************
// chipselect fuer die Statusregister (ohne Modemsignale)
wire					cs_out_data_r;	// Lesen

wire					select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;
wire		[ALL_PORTS-1:0]		reg_select;

// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	),
		.reg_select(     reg_select        ) // port unused, but needed in module instantiation to suppress warnings during integration tests
			);

// chipselect fuer die FIFO-,Status- und Steuerregister
assign	cs_out_data_r	= reg_read[DAT_ADR];	// Lesen
// Dekodierung der Adressen ********************** END **************************************


assign	di_peri [17:0]	= cs_out_data_r?18'HXXXX:
				'b0;


endmodule
