#include <stdio.h>
#include <peripherals.h>
#include <stall_control.h>
#include <perf.h>

FILE * stdout = &SIM_PRINT_0_FILE;

unsigned int  __attribute__ ((aligned (16))) stalldata[4] = {42, 42, 42, 42};



void __attribute((noinline,noclone)) resetCounts( unsigned int peri_data_init, unsigned int mem_init) {
    COUNT_ACCESS_0.data = peri_data_init;
    COUNT_ACCESS_0.read_count = 0;
    COUNT_ACCESS_0.write_count = 0;
    stalldata[0] = mem_init;
}

void __attribute((noinline,noclone)) copyWord0(volatile unsigned int * from, volatile unsigned int * to) {
    asm volatile (
    "L18 o0, 0(%0)\n"
    "S18 0(%1), o0"
    :/* out */
    :/* in */
    	"r"(from),
	"r"(to)
    :/* clobber */
    	"o0"
    );
}

void __attribute((noinline,noclone)) copyWord1(volatile unsigned int * from, volatile unsigned int * to) {
    asm volatile (
    "L18 o0, 0(%0)\n"
            "NOP\n"
            "S18 0(%1), o0"
    :/* out */
    :/* in */
    "r"(from),
    "r"(to)
    :/* clobber */
    "o0"
    );
}
void __attribute((noinline,noclone)) copyWord2(volatile unsigned int * from, volatile unsigned int * to) {
    asm volatile (
    "L18 o0, 0(%0)\n"
            "NOP\n"
            "NOP\n"
            "S18 0(%1), o0"
    :/* out */
    :/* in */
    "r"(from),
    "r"(to)
    :/* clobber */
    "o0"
    );
}
void __attribute((noinline,noclone)) copyWord3(volatile unsigned int * from, volatile unsigned int * to) {
    asm volatile (
    "L18 o0, 0(%0)\n"
            "NOP\n"
            "NOP\n"
            "NOP\n"
            "S18 0(%1), o0"
    :/* out */
    :/* in */
    "r"(from),
    "r"(to)
    :/* clobber */
    "o0"
    );
}

void __attribute((noinline,noclone)) printStats(volatile unsigned int * to) {

    printf("r/w: %d/%d, content: %d\n", COUNT_ACCESS_0.read_count,COUNT_ACCESS_0.write_count,*to);
}

void (*copyFunctions[])(volatile unsigned int *, volatile unsigned int *) = {
        copyWord0,
        copyWord1,
        copyWord2,
        copyWord3
};

const int numCopyFunctions = sizeof(copyFunctions) / sizeof(copyFunctions[0]);


void __attribute((noinline,noclone)) testCopyFunctions(volatile unsigned int * from, volatile unsigned int * to, unsigned int stall_enable, unsigned int peri_data_init) {
    for (unsigned int i=0;i<numCopyFunctions;i++) {

        printf("%d NOPS: ",i);

        resetCounts(peri_data_init+i, 42+i);


	STALL_CONTROL_0.enable = stall_enable;

        copyFunctions[i](from, to);


	STALL_CONTROL_0.enable = 0;

	printStats(to);

    }

}

unsigned int __attribute((noinline,noclone)) getStallEnable() {
    return 0;
}

void copyMultipleMarker(unsigned int * in, unsigned int * out);
void copyMultiple(unsigned int * in, unsigned int * out);

uint36_t cyclesFromStallEnables(int stallEnables) {
    int dataEnables = stallEnables & 0xF;
    int codeEnables = (stallEnables>>4) & 0xF;


    unsigned int durations = STALL_CONTROL_0.stall_durations;
    unsigned int dataCycles = durations & 0x1FF;
    unsigned int codeCycles = durations >> 9;

    uint36_t sum = 0;
    for (int i = 0;i<=4;i++) {
	int currBit = 1<<i;
	unsigned int currDataCycles = dataEnables & currBit ? dataCycles : 0;
	unsigned int currCodeCycles = codeEnables & currBit ? codeCycles : 0;

	sum += currDataCycles > currCodeCycles ? currDataCycles : currCodeCycles;
    }
    return sum;
}

int  __attribute((noinline,noclone)) verifyCorrectness(unsigned int * arr, int stallEnables, uint36_t cycles) {
    	if (cycles != cyclesFromStallEnables(stallEnables)) {
	    return 0;
	}
	for (unsigned int i = 0;i<4;i++) {
	    STALL_CONTROL_0.select_stats = i;

	    if (STALL_CONTROL_0.access_count_code!=1)
		return 0;

	    if (STALL_CONTROL_0.access_count_data!=1)
		return 0;

	    if (arr[i]!=(i+100))
		return 0;
	}
    return 1;
}

void printArr(unsigned int * arr) {
    for (int i=0;i<4;i++) {
	if (i > 0) printf(", ");
	printf("%03d",arr[i]);
    }
}

void printAccessCounts() {
    printf(" c/d:");
    for (unsigned int i=0;i<4;i++) {
	STALL_CONTROL_0.select_stats = i;

	printf(" %d/%d",STALL_CONTROL_0.access_count_code, STALL_CONTROL_0.access_count_data);
    }
}

void  __attribute((noinline,noclone)) fillOutdata(unsigned int * data) {
    for (unsigned int i=0;i<4;i++)
	data[i] = 20+i;
}

uint36_t getNostallCycles() {
    STALL_CONTROL_0.enable = 0;
    unsigned int outdata[] = {99,99,99,99};
    copyMultipleMarker(stalldata, outdata);
    return perf_read_ccnt(0, 0);
}

int copyMultipleAndPrint(unsigned int i, uint36_t nostallCycles) {
    stalldata[0] = 100;
    stalldata[1] = 101;
    stalldata[2] = 102;
    stalldata[3] = 103;

    //Reset all access counts
    perf_reset();
    STALL_CONTROL_0.access_count_code = 0;

    unsigned int outdata[] = {99,99,99,99};
    fillOutdata(outdata);
    STALL_CONTROL_0.enable = i;
    copyMultipleMarker(stalldata, outdata);
    STALL_CONTROL_0.enable = 0;

    uint36_t cycles = perf_read_ccnt(0, 0) - nostallCycles;
    //printf("%ld\n",cycles);

    if (!verifyCorrectness(outdata, i, cycles)) {
	printf("%02x: ",i);
	printArr(outdata);
	printAccessCounts();
	printf(" %ld exp %ld\n",cycles,cyclesFromStallEnables(i));
	return 0;
    }
    return 1;
}

void testCopyMultiple(uint36_t nostallCycles) {
    int ok = 1;
    for (unsigned int i = 0; i < 0x100; i++) {
	//{unsigned int i=0x1;
	if (!copyMultipleAndPrint(i, nostallCycles)) {
	    ok = 0;
	}
    }
    if (ok)
	printf("all ok\n");
}

const unsigned int desired_alignment = 14;

//Own function, not inlineable to work around #1153
unsigned int  __attribute((noinline,noclone)) getCopyMultipleAddr() {
    return (unsigned int) &copyMultiple;
}

void * getCodeAddr() {
    return (void *) (getCopyMultipleAddr() + 16 - desired_alignment);
}

void main() {

    //Check for alignment
    if (((unsigned int )&stalldata[0]) & 15) {
	printf("not aligned data! %p\n", &stalldata[0]);
	stopSimulation(&SIM_STOPPER_0);
	return;
    }

    // We want copy multiple's address % 16 to be desired_alignment. STALL_CONTROL's code_addr is
    // then set to the next multiple of 16

    if ((getCopyMultipleAddr() & 15)!=desired_alignment) {
	printf("not aligned code! %p\n", &copyMultiple);
	stopSimulation(&SIM_STOPPER_0);
	return;
    }

    STALL_CONTROL_0.data_addr = &stalldata[0];
    STALL_CONTROL_0.code_addr = getCodeAddr();

    STALL_CONTROL_0.stall_durations = 1<<9 | 1;

    unsigned int stall_enable = getStallEnable();

    printf("MEM -> PERI\n===========\n");
    testCopyFunctions(&stalldata[0], &COUNT_ACCESS_0.data, stall_enable,0);
    printf("\nPERI -> MEM\n===========\n");
    testCopyFunctions(&COUNT_ACCESS_0.data, &stalldata[0], stall_enable,10);

    uint36_t nostallCycles = getNostallCycles();
    for (unsigned int durationCode = 1;durationCode < 4;durationCode++) {
	for (unsigned int durationData = 1; durationData < 4; durationData++) {
	    printf("\nmulti access lengths %d/%d\n",durationCode, durationData);
	    printf("========================\n");

	    STALL_CONTROL_0.stall_durations = durationCode<<9 | durationData;
	    testCopyMultiple(nostallCycles);
	}
    }

    stopSimulation(&SIM_STOPPER_0);
}
