/**
 * _______________________________________________________________________________
 *
 *  Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 *  Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 *  (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgement: "This product includes
 *     software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 *     its contributors."
 *
 *  4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */


`timescale      1ns / 10ps
`default_nettype none

module count_access(
	clk_peri,
	do_peri,
	di_peri,
	addr_peri,
	access_peri,
	wr_peri,
	reset
	);

parameter BASE_ADR	= 10'h0;
parameter PORT_WIDTH	= 18;

parameter ALL_PORTS	= 12;
parameter DECODSIM_DEB	= "NO";

// inputs & outputs des Systembus
input	wire				clk_peri;
input	wire	[17:0]	do_peri;	// Datenbus 
output	wire	[17:0]			di_peri;	// Datenbus
input	wire	[9:0]			addr_peri;	// Adressbus
input	wire				access_peri;	// chipselect Peripherie
input	wire				wr_peri;	// Peripherie-Schreibzugriff
input	wire				reset;




wire					select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

// Addressdekoder fuer alle Geraeteregister
reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	select		),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	),
		.reg_select()
			);

reg [17:0] count_read;
reg [17:0] count_write;
reg [17:0] data;




always @ (posedge clk_peri)
	if(reset) begin
		count_read <= 0;
		count_write <= 0;
		data <= 0;
	end else begin
		if (reg_read[0]) begin
			data <= 0;
			count_read <= count_read + 1;
		end
		if (reg_write[0]) begin
			data <= do_peri;
			count_write <= count_write + 1;
		end
		if (reg_write[1])
			count_read <= do_peri;
		if (reg_write[2])
			count_write <= do_peri;

end


assign	di_peri		= (reg_read[0])?data:
			  (reg_read[1])?count_read: 
			  (reg_read[2])?count_write:18'b0;


endmodule
