.PHONY: test testSim

# MODELSIM_INI and VLOG_OPT only get defined once jConfig has generated the project.mk file.
# therefore, the simulation is run in a separate make process that is only started after jConfig has finished.


DIFFCALL:=$(SPARTANMC_DIFF_DIRECTORIES) --reference expected --generated .

ifndef VSIMSCRIPT
VSIMSCRIPT:=testbench.fdo
endif

testSim: $(MODELSIM_INI) $(VLOG_OPT) | $(VSIM_DIR)
	@truncate --size=0 --no-create sim/output.txt 
	@rm -rf sim/vsim
	@mkdir sim/vsim
	@cd sim && $(MODELSIM) -do $(VSIMSCRIPT) -c </dev/null
ifeq ($(EXPECTED_ERROR),)
	@! grep -q "Error: " sim/vsim/transcript || (echo "FATAL: transcript contains errors" && exit 1) # Check for errors in transcript
else
	@ grep -q $(EXPECTED_ERROR) sim/vsim/transcript || (echo "FATAL: transcript does not contain expected error '$(EXPECTED_ERROR)'" && exit 1) # Check for errors in transcript
endif
ifeq ($(NO_DIFF),)
	$(ADDITIONALCMD)
	@$(DIFFCALL)
endif


test:
	rm -rf system
	$(BEFOREJC)
ifeq ($(NO_JCONFIG),)
	$(MAKE) clean jconfig +args="-q -u -a -f config.jc3"
	$(AFTERFIRSTJC)
	#Check to see if everything is up to date
	@mkdir -p build
	$(MAKE) jconfig_update +args="--script ../saveFile.groovy" 2>&1 | tee build/jcOutput.txt
	@grep -q "Written: 0 groups" build/jcOutput.txt || (echo "Error: JConfig should not update anything when called again with the same input."; exit 1)
	@! grep --binary-files=without-match -R "CalculatedValueResult{.*}" system || (echo "Error: JConfig should not output CalculatedValueResult{*}."; exit 1) # Look for CalculatedValueResult error
endif
ifeq ($(NO_SIMULATION),)
	$(MAKE) testSim
else
	$(ADDITIONALCMD)
ifeq ($(NO_DIFF),)
	@$(DIFFCALL)
endif
endif
