#include "peripherals.h"	// I/O-Adessen und Registerstrukturen aller installierten Interfaces
#include "moduleParameters.h"	// in der Hardwere eingestellte Parameter aller installierten Interfaces
#include <usb.h>

#define NUM_ENDPOINTS   SB_USB11_0_ENDPOINTS

struct usb USB11_0_DMA __attribute__((section(".dma.usb11_0"))) = {
    .ep01c = (                               USB_CTRL_MODE | USB_CTRL_IN | 8 ), // EP1 tx nicht bereit kein Intr. setzen
    .ep02c = (                               USB_CTRL_MODE | USB_CTRL_OUT    ), // EP2 rx nicht bereit kein Intr. setzen
//  .ep03c = ( USB_CTRL_EN | USB_CTRL_INTR | USB_CTRL_MODE | USB_CTRL_IN | 8 ),
//  .ep04c = ( USB_CTRL_EN | USB_CTRL_INTR | USB_CTRL_MODE | USB_CTRL_OUT    ),
//  .ep05c = ( USB_CTRL_EN | USB_CTRL_INTR | USB_CTRL_MODE | USB_CTRL_IN | 8 ),
//  .ep06c = ( USB_CTRL_EN | USB_CTRL_INTR | USB_CTRL_MODE | USB_CTRL_OUT    ),

    .device = {
        .hdr.size           = 0x12,		//num_configs - hdr.size +1
        .hdr.type           = USB_DEVICE_DESCRIPTOR,
        .usb_spec           = 0x01,
        .class              = USB_CLASS_VENDOR_SPECIFIC,
        .subclass           = 0x00,
        .protocol           = 0xFF,
        .packet_size        = 0x40,
        .vendor_id_high     = 0x66,
        .vendor_id_low      = 0x66,
        .product_id_high    = 0xAF,
        .product_id_low     = 0xFE,
        .release_low        = 0x10,
        .release_high       = 0x00,
        .vendor_descr_idx   = 0x01,
        .product_descr_idx  = 0x02,
        .serial_descr_idx   = 0x03,
        .num_configs        = 0x01
    },
    .config = {
        .hdr.size           = 0x09,		//power_cons - hdr.size +1
        .hdr.type           = USB_CONFIG_DESCRIPTOR,
        .config_size_low    = sizeof(struct usb_config_descr)
                            + sizeof(struct usb_iface_descr)
                            + sizeof(struct usb_endpoint_descr) * NUM_ENDPOINTS,
        .config_size_high   = 0x00,
        .num_ifaces         = 0x01,
        .index              = 0x01,
        .descr_idx          = 0x00,
        .power_mode         = USB_SELF_POWERED,
        .power_cons         = 0x00
    },
    .iface = {
        .hdr.size           = 0x09,		//descr_idx - hdr.size +1
        .hdr.type           = USB_INTERFACE_DESCRIPTOR,
        .index              = 0x00,
        .alt_setting        = 0x00,
        .num_endpoints      = NUM_ENDPOINTS,
        .class              = USB_CLASS_VENDOR_SPECIFIC,
        .subclass           = 0x01,
        .protocol           = 0xFF,
        .descr_idx          = 0x02
    },
    .ep01 = {
        .hdr.size           = 0x07,		//poll_interval - hdr.size +1
        .hdr.type           = USB_ENDPOINT_DESCRIPTOR,
        .adr                = {
            .direction      = USB_DIRECTION_IN,
            .number         = 1
        },
        .attr               = USB_ATTRIBUTE_BULK,
        .packet_size_low    = 0x40,
        .packet_size_high   = 0x00,
        .poll_interval      = 0x01
    },
    .ep02 = {
        .hdr.size           = 0x07,		//poll_interval - hdr.size +1
        .hdr.type           = USB_ENDPOINT_DESCRIPTOR,
        .adr                = {
            .direction      = USB_DIRECTION_OUT,
            .number         = 2
        },
        .attr               = USB_ATTRIBUTE_BULK,
        .packet_size_low    = 0x40,
        .packet_size_high   = 0x00,
        .poll_interval      = 0x01
    },
    .ep03 = {
        .hdr.size           = 0x07,		//poll_interval - hdr.size +1
        .hdr.type           = USB_ENDPOINT_DESCRIPTOR,
        .adr                = {
            .direction      = USB_DIRECTION_IN,
            .number         = 3
        },
        .attr               = USB_ATTRIBUTE_BULK,
        .packet_size_low    = 0x40,
        .packet_size_high   = 0x00,
        .poll_interval      = 0x01
    },
    .ep04 = {
        .hdr.size           = 0x07,		//poll_interval - hdr.size +1
        .hdr.type           = USB_ENDPOINT_DESCRIPTOR,
        .adr                = {
            .direction      = USB_DIRECTION_OUT,
            .number         = 4
        },
        .attr               = USB_ATTRIBUTE_BULK,
        .packet_size_low    = 0x40,
        .packet_size_high   = 0x00,
        .poll_interval      = 0x01
    },
    .ep05 = {
        .hdr.size           = 0x07,		//poll_interval - hdr.size +1
        .hdr.type           = USB_ENDPOINT_DESCRIPTOR,
        .adr                = {
            .direction      = USB_DIRECTION_IN,
            .number         = 5
        },
        .attr               = USB_ATTRIBUTE_BULK,
        .packet_size_low    = 0x40,
        .packet_size_high   = 0x00,
        .poll_interval      = 0x01
    },
    .ep06 = {
        .hdr.size           = 0x07,		//poll_interval - hdr.size +1
        .hdr.type           = USB_ENDPOINT_DESCRIPTOR,
        .adr                = {
            .direction      = USB_DIRECTION_OUT,
            .number         = 6
        },
        .attr               = USB_ATTRIBUTE_BULK,
        .packet_size_low    = 0x40,
        .packet_size_high   = 0x00,
        .poll_interval      = 0x01
    },
    .lang = {
        .hdr.size           = 0x04,		//lang_id_high - hdr.size +1
        .hdr.type           = USB_STRING_DESCRIPTOR,
        .lang_id_low        = 0x09,
        .lang_id_high       = 0x04
    },
    .vendor_str_hdr.size    = 0x8,		//product_str_hdr.size - vendor_str_hdr.size
    .vendor_str_hdr.type    = USB_STRING_DESCRIPTOR,
    .vendor_str             = {'T',0,'U',0,'D',0},
    .product_str_hdr.size   = 0xA,		//serial_str_hdr.size - product_str_hdr.size
    .product_str_hdr.type   = USB_STRING_DESCRIPTOR,
    .product_str            = {'S',0,'p',0,'M',0,'C',0},
    .serial_str_hdr.size    = 0x8,		//data00 - serial_str_hdr.size
    .serial_str_hdr.type    = USB_STRING_DESCRIPTOR,
    .serial_str             = {'0',0,'.',0,'0',0},
// Vorinitialisierung der Puffer mit 8 Byte (es sind maximal 6 EP moeglich)
    .data01                 = {0x06030,0x06030,0x06031,0x00000},	// EP01 bei einfacher Pufferung oder EP00.2
    .data02                 = {0x06030,0x06030,0x06032,0x00000},	// EP02 bei einfacher Pufferung oder EP01.1
    .data03                 = {0x06030,0x06030,0x06033,0x00000},	// EP03 bei einfacher Pufferung oder EP01.2
    .data04                 = {0x06030,0x06030,0x06034,0x00000},	// EP04 bei einfacher Pufferung oder EP02.1
    .data05                 = {0x06030,0x06030,0x06035,0x00000},	// EP05 bei einfacher Pufferung oder EP02.2
    .data06                 = {0x06030,0x06030,0x06036,0x00000},	// EP06 bei einfacher Pufferung oder EP03.1
    .data07                 = {0x06030,0x06030,0x06037,0x00000},	// EP07 bei einfacher Pufferung oder EP03.2
    .data08                 = {0x06030,0x06030,0x06038,0x00000},	// EP08 bei einfacher Pufferung oder EP04.1
    .data09                 = {0x06030,0x06030,0x06039,0x00000},	// EP09 bei einfacher Pufferung oder EP04.2
    .data10                 = {0x06030,0x06030,0x06230,0x00000},	// EP10 bei einfacher Pufferung oder EP05.1
    .data11                 = {0x06030,0x06030,0x06231,0x00000},	// EP11 bei einfacher Pufferung oder EP05.2
    .data12                 = {0x06030,0x06030,0x06232,0x00000},	// EP12 bei einfacher Pufferung oder EP06.1
    .data13                 = {0x06030,0x06030,0x06233,0x00000}		// EP13 bei einfacher Pufferung oder EP06.2
};
