module testdma(
        input	wire		access_peri,
	input   wire	[7:0]   addr_high,
        input	wire	[9:0]	addr_peri,
	input	wire		clk_peri,
        output	wire	[17:0]	di_peri,
        input	wire	[17:0]	do_peri,
	input   wire            mem_access,
	input	wire		reset,
	input   wire            store_access_high,
	input   wire            store_access_low,
        input	wire		wr_peri	
);


parameter BASE_ADR = 10'h0;
parameter DMA_BASE_ADR = 18'h0;


wire selectMem;

pselect #(
		.ADDR_WIDTH     (8),
		.BASE_WIDTH     (8),
		.BASE_ADDR      (DMA_BASE_ADR >> 10) 
	)
 pseldma			 
	(addr_high,mem_access,selectMem);




parameter  ALL_PORTS		= 2,

	DECODSIM_DEB		= "NO";
wire					select;
wire		[ALL_PORTS-1:0]		reg_select;
wire		[ALL_PORTS-1:0]		reg_read;
wire		[ALL_PORTS-1:0]		reg_write;

reg_access_decoder	#(
		.REG_COUNT(	ALL_PORTS	),
		.BASE_ADR(	BASE_ADR	),
		.SIM_DEBUG(	DECODSIM_DEB	)
			)
decod1			(
		.reset(		reset		),
		.clk_peri(	clk_peri	),
		.addr_peri(	addr_peri	),
		.access_peri(	access_peri	),
		.wr_peri(	wr_peri		),

		.peri_select(	selectReg	),
		.reg_select(	reg_select	),
		.reg_read(	reg_read	),
		.reg_write(	reg_write	)
			);
			

reg [17:0] addr;
wire [17:0] datard;

always @(posedge clk_peri) begin
	if (~reset & reg_write[0])
		addr <= do_peri;
end

wire[17:0] dmaDataOut;

assign di_peri = dmaDataOut | datard;


ram_18_prim #(.SIM_COLLISION_CHECK("GENERATE_X_ONLY")) dmaram(
//DMA port
	.CLKA(clk_peri),
	.DOA(dmaDataOut),
	.DIA(do_peri),
	.ADDRA(addr_peri),
	.ENA(1),
	.WEA(wr_peri & selectMem),
	.SSRA(~selectMem),

//Peri port
	.CLKB(clk_peri),
	.DOB(datard),
	.DIB(do_peri),
	.ADDRB(addr),
	.ENB(1),
	.WEB(reg_write[1]),
	.SSRB(~reg_select[1])
);

endmodule
