`timescale 1ns / 1ps 
`default_nettype none
`uselib lib=spartanmc
module uartToFile
#(
	parameter FILENAME = "output.txt"
)
(
	input wire rx
);

reg clock;

// 10.0MHz clock on clock
initial clock=0;
always #(50) clock=~clock;


wire word_rx_invalid;
reg word_rx_valid;
reg word_rx_valid_last;
reg word_rx_valid_last2;
always @(posedge clock) begin
	//Never read in consecutive cycles, fifo empty needs some cycles to update...
	word_rx_valid_last <= word_rx_valid;
	word_rx_valid_last2 <= word_rx_valid_last;
	word_rx_valid <= ~word_rx_invalid && ~word_rx_valid && ~word_rx_valid_last && ~word_rx_valid_last2;
end

wire [7:0] word_rx;

`include "uart_light/util/uart_functions.vh"


reg reset = 1;
reg[3:0] reset_counter = 3;

always @(posedge clock) begin
	if (reset_counter>0) begin
		reset_counter <= reset_counter-1;
		reset <= 1;
	end else
		reset <= 0;
end


localparam 	
	CLOCK_FREQUENCY         = 10000000,
	BAUDRATE                = 115200,
	SAMPLING_COUNT		= find_best_ovs_rate(CLOCK_FREQUENCY, BAUDRATE),
	SC_SIZE			= log2(SAMPLING_COUNT),

	BR_DIVISOR_TX		= CLOCK_FREQUENCY/BAUDRATE,
	BR_DIVISOR_RX		= baud_divisor(ovs_freq_10x(CLOCK_FREQUENCY, SAMPLING_COUNT), BAUDRATE),
	BRD_SIZE_TX		= log2(BR_DIVISOR_TX - 1),
	BRD_SIZE_RX		= log2(BR_DIVISOR_RX - 1);

uart_light

	#(
		.FIFO_ADDR_BITS_RX(3),
		.FIFO_ADDR_BITS_TX(3),

		.BR_DIVISOR_TX(BR_DIVISOR_TX),
		.BR_DIVISOR_RX(BR_DIVISOR_RX),
		.BRD_SIZE_TX(BRD_SIZE_TX),
		.BRD_SIZE_RX(BRD_SIZE_RX),

		.SAMPLING_COUNT(SAMPLING_COUNT),
		.SC_SIZE(SC_SIZE)
	)
 testuart (

		.reset(reset),
		.clk_peri(clock),

		//TX
		.tx(),
		.word_ready(1'd0),
		.word_tx(8'd0),
		.fifo_tx_full(),
		.fifo_tx_empty(),
		.fifo_tx_read(),
		
		//rx
		.rx(rx),
		.read_ready(word_rx_valid && ~reset),
		.word_rx(word_rx),
		.fifo_rx_full(),
		.fifo_rx_empty(word_rx_invalid)
  );


integer txFile;


initial begin
	$display("Writing UART to %s",FILENAME);
	txFile = $fopen(FILENAME);
end

always @(posedge clock) begin
	if (word_rx_valid) begin

		$fwrite(txFile,"%c",word_rx);
		$fflush(txFile);
		if (word_rx>31)
			$display("TB uart tx %X %c",word_rx,word_rx);
		else
			$display("TB uart tx %X",word_rx);
	end
end

endmodule
/*
`include "uart_light/src/uart_light.v"
`include "uart_light/src/uart_light_clk_gen.v"
`include "uart_light/src/uart_light_tx.v"
`include "uart_light/src/uart_light_rx.v"
`include "uart_light/src/uart_light_rx_ctrl.v"
`include "uart_light/src/uart_light_rx_dp.v"
`include "uart_light/src/uart_light_tx_ctrl.v"
`include "uart_light/src/uart_light_tx_dp.v"
`include "common/src/yaf_sync_w2r.v"
`include "common/src/yaf_rptr_empty.v"
`include "common/src/yaf_sync_r2w.v"
`include "common/src/yaf.v"
`include "common/src/yaf_mem.v"
`include "common/src/yaf_wptr_full.v"
*/
`uselib
