/*  */
/*                                                                         */
/*  Copyright (September 1997) Honeywell Inc. Unpublished - All rights     */
/*  reserved. This material may be reproduced by, or for, the U.S.         */
/*  Government pursuant to the copyright license under the clause at       */
/*  DFARS 252.227-7013 (Oct. 1988).                                        */
/*                                                                         */
/*  */
/*

 * compressimage
 * Version 1.0
 *
 * File         : compress.c
 * Company      : Honeywell Technology Center, Minneapolis, MN 55412
 * Date         : November 10 1997
 * Authors      : Luiz Pires and Deepa Pandalai
 * Description  : This file contains all the routines required for a simple
 *              : wavelet based image compression of a 512x512 8-bit pixel
 *              : image in PGM format. The compression process consists of
 *              : four basic steps: Wavelet transform, quantization,
 *              : run length encoding and entropy coding. Each of these 
 *              : steps is implemented as a function and is described below. 
 *              : This program will expect the name of the pgm image to
 *              : be compressed as a required input. It will also accept an
 *              : optional compression-factor parameter specifying how 
 *              : aggresively the image should be compressed: A compression
 *              : factor indicates minimal compression and maximum: image
 *              : quality. Conversely, a compression factor of 255 indicates
 *              : maximum compression with higher degradation to the image
 *              : quality. If the compression factor is not indicated, a  
 *              : default compression rate of 128 will be used. The compressed
 *              : image file format is specific to this program a  will be
 *              : indicated by a ".cmp" extension.
 *              : NOTE: The wavelet transform routine implements a modified
 *              : version of the biorthogonal (2-2) Cohen-Daubechies-Feauveau
 *              : wavelet. The routine employs the lifting scheme to simplify
 *              : the wavelet implementation. As it stands it requires only
 *              : integer adds and shifts. A reference on the transform used
 *              : is "A. Cohen, I. Daubechies, J. Feauveau, ``Biorthogonal
 *              : Bases of Compactly Supported Wavelets,'' Comm. Pure Appl.
 *              : Math., 45 (1992). More effective wavelet filters may be found
 *              : in "J. Villasenor, B. Belzer, J. Liao, ``Wavelet Filter
 *              : Evaluation for Image Compression,'' IEEE Transactions on
 *              : Image Processing, vol. 2, pp. 1053-1060, August 1995. 
 * Modifications:
 *  Yanbing Li
 * Various optimizations (memory usage, procedure inling, loop optimizations, 
 * strength reduction etc.)
 * Modify for running on ACEII

 */
/* #include<systime.h> */
#include "globals.h"
#include <subsystems/spartanmc_0/peripherals.h>
#include <stdio.h>
#include "img128.h"
char in_name[80], out_name[80];
int nbytes_g;
/* Quantization variables */
/*

   The image will be divided into 10 blocks, the first four will be of
   size 64x64 pixels (4096 pixels), then three of size 128x128 (16384 pixels)
   and the remaining three of size 256x256 pixels (65536 pixels).
   These blocks will be arranged as represented below:

   +---+---+-------+---------------+
   | 4K| 4K|       |               |
   +---+---|  16K  |               |
   | 4K| 4K|       |               |
   +---+---+-------+      65K      |
   |       |       |               |
   |  16K  |  16K  |               |
   |       |       |               |
   +-------+-------+---------------+
   |               |               |
   |               |               |
   |               |               |
   |      65K      |      65K      |
   |               |               |
   |               |               |
   |               |               |
   +---------------+---------------+

 */
int blockminval_g[10], blockmaxval_g[10];
int quant_buf_g[16384];
/* struct timeval tv1, tv2; */
/* struct timezone tz1, tz2; */
/* RLE variables */
unsigned char rle_buf_g[16384];
/* reuse int_data */
unsigned int rle_size_g[10];
/* Huffman Encoding variables */
unsigned char codep_g[16384];
/* reuse quant_buf */
/* Function Declarations */
int read_image(char *);
void forward_wavelet();
void quantization(int *int_data, int *quant_buf, int *blockminval,
		  int *blockmaxval);
void RLE_encode(int *quant_buf, unsigned char *rle_buf, int *rle_size);
void entropy_encode(unsigned char *rle_buf, int *rle_size,
		    unsigned char *codep, int *nbytes);
/* int write_compressed_file(char); */
void forward_wavelet1(int, int *);
void forward_wavelet2(int, int *);
void block_quantize(int x, int y, int size, int num, int *int_data,
		    int *quant_buf, int *blockminval, int *blockmaxval);
/*

 * Routine: main(int argc, char **argv)
 * Expects image file name and the scale factor for compression as
 * input parameters. Calls all the required routines for compression.
 * The compressed file is the output of write_compressed_file().
 * This file format is specific to this program and includes all image
 * information required for decompression. 

 */
main(int argc, char * *argv) {
    /* printf("-- End initialization. \n-- Wavelet transform ... ...\n"); */
    /* stdio_uart_light_open(UART_LIGHT_0); */
    /* printf("    Core2 running ...\n"); */
    forward_wavelet1_task(128, &int_data_g128[0]);
#pragma microStreams task
    forward_wavelet2_task(128, &int_data_g128[0]);
#pragma microStreams task
    forward_wavelet1(64, &int_data_g128[0]);
#pragma microStreams task
    forward_wavelet2(64, &int_data_g128[0]);
#pragma microStreams task
    forward_wavelet1(32, &int_data_g128[0]);
    forward_wavelet2(32, &int_data_g128[0]);
#pragma microStreams task
    /* printf("-- End wavelet. \n-- Quantization ... ...\n"); */
    quantization(&int_data_g128[0], &quant_buf_g[0], &blockminval_g[0],
		 &blockmaxval_g[0]);
#pragma microStreams task
    /* printf("-- End quantization. \n-- RLE encoding ... ...\n"); */
    RLE_encode(&quant_buf_g[0], &rle_buf_g[0], &rle_size_g[0]);
#pragma microStreams task
    entropy_encode(&rle_buf_g[0], &rle_size_g[0], &codep_g[0], &nbytes_g);
    /* printf("-- End entropy encoding. \n-- Write compressed file ... ...\n"); */
    /* write_compressed_file(in_name); */
    while (1) {
	;
    }
}

/*

 * Routine: void quantization() input: int_data,     output: quant_buf, blockminval, blockmaxval
 * 

 */
void quantization(int *int_data, int *quant_buf, int *blockminval,
		  int *blockmaxval) {
    int bl, num;
    num = 0;
    block_quantize(0, 0, (128 / 4) >> 1, num++, int_data, quant_buf,
		   blockminval, blockmaxval);
    /* yb */
    /* yb */
    for (bl = ((128 / 4) >> 1); bl < (128 >> 1); bl <<= 1) {
	block_quantize(0, bl, bl, num++, int_data, quant_buf, blockminval,
		       blockmaxval);
	block_quantize(bl, 0, bl, num++, int_data, quant_buf, blockminval,
		       blockmaxval);
	block_quantize(bl, bl, bl, num++, int_data, quant_buf, blockminval,
		       blockmaxval);
    }
}

/*

 * Routine: block_quantize(int x, int y, int size, int num)

 */
void block_quantize(int x, int y, int size, int num, int *int_data,
		    int *quant_buf, int *blockminval, int *blockmaxval) {
    static int thresh[17];
    /* valid: 1-15 */
    int buf_pts[10] = { 0, 256, 512, 768, 1024, 2048, 3072, 4096, 8192 };
    int i, j, val;
    int minval, maxval;
    int qsize;
    int *quant_buf_pt = quant_buf + buf_pts[num];
    int tmp_classify = 0;
    int xsize, ysize;
    int blockthresh[10] =
	{ 0, 39, 27, 104, 79, 50, 191, 99999, 99999, 99999 };
    minval = 10000;
    maxval = (-10000);
    xsize = (x + size);
    ysize = (y + size);
    for (i = x; i < xsize; i++) {
	for (j = y; j < ysize; j = (j + 1)) {
	    /* k-s: yes */
	    val = int_data[i + (j * 128)];
	    /* no need to change, shifts are free in FPGA */
	    minval = ((val < minval) ? val : minval);
	    maxval = ((val > maxval) ? val : maxval);
	}
	/* printf("out of hw kernel %d\n", i); */
    }
    /* yb: 16->15? since 16 not used */
    for (i = 1; i <= 16; i++) {
	thresh[i] = (minval + (((i * (maxval - minval)) + 8) >> 4));
    }
    qsize = 0;
    for (i = x; i < (x + size); i++) {
	for (j = y; j < (y + size); j++) {
	    /* ks: no */
	    /* 115 rows, too big to fit in hardware */
	    val = int_data[i + (128 * j)];
	    if (((val > 0) ? val : (0 - val)) < blockthresh[num]) {
		quant_buf_pt[qsize] = 16;
		qsize++;
	    }
	    else {
		if (val > thresh[8]) {
		    if (val > thresh[12]) {
			if (val > thresh[14]) {
			    if (val > thresh[15]) {
				tmp_classify = 15;
			    }
			    else {
				tmp_classify = 14;
			    }
			}
			else {
			    if (val > thresh[13]) {
				tmp_classify = 13;
			    }
			    else {
				tmp_classify = 12;
			    }
			}
		    }
		    else {
			if (val > thresh[10]) {
			    if (val > thresh[11]) {
				tmp_classify = 11;
			    }
			    else {
				tmp_classify = 10;
			    }
			}
			else {
			    if (val > thresh[9]) {
				tmp_classify = 9;
			    }
			    else {
				tmp_classify = 8;
			    }
			}
		    }
		}
		else {
		    if (val > thresh[4]) {
			if (val > thresh[6]) {
			    if (val > thresh[7]) {
				tmp_classify = 7;
			    }
			    else {
				tmp_classify = 6;
			    }
			}
			else {
			    if (val > thresh[5]) {
				tmp_classify = 5;
			    }
			    else {
				tmp_classify = 4;
			    }
			}
		    }
		    else {
			if (val > thresh[2]) {
			    if (val > thresh[3]) {
				tmp_classify = 3;
			    }
			    else {
				tmp_classify = 2;
			    }
			}
			else {
			    if (val > thresh[1]) {
				tmp_classify = 1;
			    }
			    else {
				tmp_classify = 0;
			    }
			}
		    }
		}
		quant_buf_pt[qsize] = tmp_classify;
		qsize++;
	    }
	}
    }
    blockminval[num] = minval;
    blockmaxval[num] = maxval;
}

/*

 * Routine: void  RLE_encode()

 */
/* input: quand_buf    output: rle_buf, rle_size */
void RLE_encode(int *quant_buf, unsigned char *rle_buf, int *rle_size) {
    /*  */
    /*  */
    /*  */
    /*  */
    int blocksize[10] =
	{ 256, 256, 256, 256, 1024, 1024, 1024, 4096, 4096, 4096 };
    int buf_pts[10] = { 0, 256, 512, 768, 1024, 2048, 3072, 4096, 8192 };
    unsigned int bl;
    int i, j, count;
    int *quant_buf_pt;
    unsigned char *rle_buf_pt;
    /* NBLKS; */
    for (bl = 0; bl < 7; bl++) {
	/* block_RLE_encode(bl); inlined */
	quant_buf_pt = (quant_buf + buf_pts[bl]);
	rle_buf_pt = (rle_buf + buf_pts[bl]);
	i = (j = 0);
	while (i < blocksize[bl]) {
	    if (quant_buf_pt[i] != 16) {
		rle_buf_pt[j] = quant_buf_pt[i];
	    }
	    else {
		count = 0;
		while (quant_buf_pt[i] == 16) {
		    /* k-s: no */
		    i++;
		    count++;
		    /* if((count==256-ZERO_MARK)||(i==IMG_SIZE)) break; */
		    if ((count == (256 - 16)) || (i == blocksize[bl])) {
			break;
		    }
		}
		i--;
		rle_buf_pt[j] = ((count + 16) - 1);
	    }
	    j++;
	    i++;
	}
	rle_size[bl] = j;
    }
}

/*

 * Routine: void entropy_encode()

 */
/* input: rle_buf, rle_size   output:codep, nbytes...done here */
void entropy_encode(unsigned char *rle_buf, int *rle_size,
		    unsigned char *codep, int *nbytes) {
    int buf_pts[10] = { 0, 256, 512, 768, 1024, 2048, 3072, 4096, 8192 };
    /* Parameters for a fixed huffman encoding tree. These parameters are  */
    /* expected to work reasonably well with most images.                  */
    int HufSize[256] =
	{ 9, 7, 6, 5, 4, 4, 3, 4, 3, 3, 4, 5, 6, 6, 7, 8, 4, 5, 5, 6, 6, 7, 7,
7, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 10, 9, 10, 10, 10, 10, 10, 10, 10, 10, 10,
10, 11, 10, 11, 11, 10, 10, 11, 11, 11, 11, 11, 11, 12, 12, 12, 12, 11, 11, 12,
11, 12, 12, 12, 12, 11, 13, 12, 12, 12, 12, 13, 12, 13, 13, 13, 14, 13, 13, 14,
13, 13, 13, 13, 14, 13, 13, 13, 13, 13, 13, 13, 13, 13, 14, 13, 14, 13, 13, 14,
15, 13, 13, 14, 13, 14, 13, 14, 13, 14, 14, 14, 14, 13, 13, 13, 13, 14, 13, 13,
13, 13, 14, 13, 15, 14, 15, 14, 14, 13, 14, 16, 14, 14, 13, 12, 12, 13, 15, 15,
17, 18, 18, 18, 17, 17, 15, 17, 18, 16, 17, 17, 16, 16, 18, 18, 16, 18, 16, 18,
18, 18, 16, 16, 16, 18, 15, 16, 18, 15, 16, 18, 16, 16, 18, 18, 18, 18, 16, 18,
16, 18, 18, 18, 16, 18, 18, 18, 18, 18, 18, 18, 18, 18, 16, 16, 18, 18, 17, 18,
18, 18, 18, 17, 15, 18, 18, 15, 15, 18, 17, 16, 16, 15, 16, 16, 18, 18, 16, 17,
18, 18, 15, 18, 16, 16, 18, 18, 18, 18, 16, 18, 18, 16, 18, 16, 18, 18, 18, 18,
18, 18, 18, 16, 18, 18, 17, 18, 18, 9 };
    int HufVal[256] =
	{ 151, 102, 7, 22, 0, 11, 1, 15, 2, 4, 14, 21, 55, 37, 56, 237, 3, 29,
    8, 13, 24, 103, 5, 6, 215, 45, 198, 248, 423, 407, 365, 295, 453, 422, 326, 855,
    376, 599, 173, 109, 167, 941, 429, 23, 197, 709, 325, 279, 166, 1063, 679, 582, 550,
    685, 535, 39, 1702, 678, 1093, 2599, 2669, 1815, 3415, 837, 1094, 87, 1830, 1703, 38, 3751,
    3607, 294, 621, 3366, 1709, 2839, 2117, 3863, 791, 1559, 5207, 4902, 9510, 2854, 7447, 7959, 1575,
    7255, 5741, 3693, 2391, 7789, 7749, 5414, 1303, 6214, 1861, 5655, 343, 5671, 3623, 7853, 551, 5957, 581,
    6487, 26967, 5399, 4439, 12909, 5701, 9559, 2629, 14679, 4166, 12357, 1111, 8743, 1367, 6725, 70, 3159, 806,
    9303, 3757, 6950, 1605, 4647, 5158, 1645, 11845, 5463, 16151, 11815, 13350, 4677, 3351, 10583, 13655, 11543, 69,
    2135, 3909, 2118, 21101, 1062, 108887, 27686, 158758, 82982, 118853, 40487, 1318, 106023, 17446, 28229, 24103, 89639,
    11302, 17702, 148518, 52262, 60997, 183334, 50470, 93222, 224294, 44070, 32295, 6182, 38950, 175142, 3110, 22566, 43046, 4165,
    55334, 174118, 19494, 52805, 108582, 239654, 109606, 240678, 50214, 117798, 4717, 125990, 257062, 121894, 23590, 252966, 56358,
    187430, 116774, 247846, 18470, 149542, 51238, 182310, 14374, 47142, 91174, 222246, 56871, 25638, 59430, 190502, 84006, 122407, 15911,
    215078, 42022, 30758, 2086, 173094, 65063, 48166, 10278, 3653, 31782, 37485, 20549, 151621, 64550, 130599, 86085, 217157, 32535, 107558, 26662,
    58406, 238630, 105510, 236582, 39974, 9254, 171046, 156710, 7206, 80934, 7719, 212006, 53317, 184389, 15398, 248870, 146470, 124966, 20037, 256038,
    60454, 43351, 191526, 214054, 120 };
    int i, j, nb;
    int rlecount;
    unsigned char *rle_buf_pt;
    unsigned ii, nbits;
    int val;
    int bytn, bitn;
    unsigned char ich;
    rlecount = 0;
    nb = 0;
    for (i = 0; i < 7; i++) {
	rle_buf_pt = (rle_buf + buf_pts[i]);
	for (j = 0; j < rle_size[i]; j++) {
	    /* k-s: no */
	    /* outer loop, not considered for hardware */
	    /* hufenc(rle_buf_pt[j],&nb); inline */
	    ich = rle_buf_pt[j];
	    nbits = HufSize[ich];
	    val = HufVal[ich];
	    for (ii = 0; ii < nbits; ii++) {
		/* k-s: no  */
		bytn = (nb >> 3);
		bitn = (nb & 7);
		if ((val & (1 << ii)) > 0) {
		    codep[bytn] |= (1 << bitn);
		}
		nb++;
	    }
	}
	rlecount += rle_size[i];
    }
    (*nbytes) = (nb >> 3);
    stdio_uart_light_open(&UART_LIGHT_0);
    printf("    Compressed size is %d bytes\n", (*nbytes) + 88);
}

/*

 * Routine: int write_compressed_file()
 ********************************************************s
 */
/*

   int write_compressed_file(in_name)
   charin_name;
   { 
   FILE *fp_out_file;

   strcpy(out_name, "output.cmp"); 

   if ((fp_out_file= fopen (out_name, "wb")) == NULL) {
   printf ("\n Could not open file %s \n\n",out_name);
   return(1);
   }

   fwrite(blockminval_g, sizeof(int) ,BLKS_SAVED     , fp_out_file);
   fwrite(blockmaxval_g, sizeof(int) ,BLKS_SAVED    , fp_out_file);
   fwrite(rle_size_g    , sizeof(int) ,BLKS_SAVED     , fp_out_file);
   fwrite(&nbytes_g     , sizeof(int) ,1     , fp_out_file);
   fwrite(codep_g       , sizeof(char),nbytes_g, fp_out_file);

   fclose(fp_out_file);

   printf("    Compressed size is %d bytes\n",nbytes_g+88);
   }

 */
/*

 * Routine: void forward_wavelet()

 */
void forward_wavelet1(int nt, int *data) {
    /* inout int_data */
    int i;
    int d_tmp0, d_tmp1, s_tmp0, s_tmp1;
    int mid, ii;
    int *x;
    int s[256], d[256];
    for (i = 0; i < (nt * 128); i += 128) {
	mid = ((nt >> 1) - 1);
	x = (&data[i]);
	s[0] = x[0];
	d[0] = x[1];
	s[1] = x[2];
	s[mid] = x[2 * mid];
	d[mid] = x[(2 * mid) + 1];
	d[0] = (((d[0] << 1) - s[0]) - s[1]);
	s[0] = (s[0] + (d[0] >> 2));
	d_tmp0 = d[0];
	s_tmp0 = s[1];
	for (ii = 1; ii < mid; ii++) {
	    /* k-s: yes */
	    s_tmp1 = x[(2 * ii) + 2];
	    d_tmp1 = (((x[(2 * ii) + 1] << 1) - s_tmp0) - s_tmp1);
	    d[ii] = d_tmp1;
	    s[ii] = (s_tmp0 + ((d_tmp0 + d_tmp1) >> 3));
	    d_tmp0 = d_tmp1;
	    s_tmp0 = s_tmp1;
	}
	d[mid] = ((d[mid] - s[mid]) << 1);
	s[mid] = (s[mid] + ((d[mid - 1] + d[mid]) >> 3));
	for (ii = 0; ii <= mid; ii++) {
	    /* k-s: no */
	    /* not working correctly individually, don't select it */
	    x[ii] = s[ii];
	    x[(ii + mid) + 1] = d[ii];
	}
    }
}

void forward_wavelet1_task(int nt, int *data) {
    /* inout int_data */
    int i;
    int d_tmp0, d_tmp1, s_tmp0, s_tmp1;
    int mid, ii;
    int *x;
    int s[256], d[256];
    for (i = 0; i < (0 + ((((nt * 128) - 0) * 1l) / 3l)); i += 128) {
	mid = ((nt >> 1) - 1);
	x = (&data[i]);
	s[0] = x[0];
	d[0] = x[1];
	s[1] = x[2];
	s[mid] = x[2 * mid];
	d[mid] = x[(2 * mid) + 1];
	d[0] = (((d[0] << 1) - s[0]) - s[1]);
	s[0] = (s[0] + (d[0] >> 2));
	d_tmp0 = d[0];
	s_tmp0 = s[1];
	for (ii = 1; ii < mid; ii++) {
	    /* k-s: yes */
	    s_tmp1 = x[(2 * ii) + 2];
	    d_tmp1 = (((x[(2 * ii) + 1] << 1) - s_tmp0) - s_tmp1);
	    d[ii] = d_tmp1;
	    s[ii] = (s_tmp0 + ((d_tmp0 + d_tmp1) >> 3));
	    d_tmp0 = d_tmp1;
	    s_tmp0 = s_tmp1;
	}
	d[mid] = ((d[mid] - s[mid]) << 1);
	s[mid] = (s[mid] + ((d[mid - 1] + d[mid]) >> 3));
	for (ii = 0; ii <= mid; ii++) {
	    /* k-s: no */
	    /* not working correctly individually, don't select it */
	    x[ii] = s[ii];
	    x[(ii + mid) + 1] = d[ii];
	}
    }
#pragma microStreams task
    for (; i < (0 + ((((nt * 128) - 0) * 2l) / 3l)); i += 128) {
	mid = ((nt >> 1) - 1);
	x = (&data[i]);
	s[0] = x[0];
	d[0] = x[1];
	s[1] = x[2];
	s[mid] = x[2 * mid];
	d[mid] = x[(2 * mid) + 1];
	d[0] = (((d[0] << 1) - s[0]) - s[1]);
	s[0] = (s[0] + (d[0] >> 2));
	d_tmp0 = d[0];
	s_tmp0 = s[1];
	for (ii = 1; ii < mid; ii++) {
	    /* k-s: yes */
	    s_tmp1 = x[(2 * ii) + 2];
	    d_tmp1 = (((x[(2 * ii) + 1] << 1) - s_tmp0) - s_tmp1);
	    d[ii] = d_tmp1;
	    s[ii] = (s_tmp0 + ((d_tmp0 + d_tmp1) >> 3));
	    d_tmp0 = d_tmp1;
	    s_tmp0 = s_tmp1;
	}
	d[mid] = ((d[mid] - s[mid]) << 1);
	s[mid] = (s[mid] + ((d[mid - 1] + d[mid]) >> 3));
	for (ii = 0; ii <= mid; ii++) {
	    /* k-s: no */
	    /* not working correctly individually, don't select it */
	    x[ii] = s[ii];
	    x[(ii + mid) + 1] = d[ii];
	}
    }
#pragma microStreams task
    for (; i < (0 + ((((nt * 128) - 0) * 3l) / 3l)); i += 128) {
	mid = ((nt >> 1) - 1);
	x = (&data[i]);
	s[0] = x[0];
	d[0] = x[1];
	s[1] = x[2];
	s[mid] = x[2 * mid];
	d[mid] = x[(2 * mid) + 1];
	d[0] = (((d[0] << 1) - s[0]) - s[1]);
	s[0] = (s[0] + (d[0] >> 2));
	d_tmp0 = d[0];
	s_tmp0 = s[1];
	for (ii = 1; ii < mid; ii++) {
	    /* k-s: yes */
	    s_tmp1 = x[(2 * ii) + 2];
	    d_tmp1 = (((x[(2 * ii) + 1] << 1) - s_tmp0) - s_tmp1);
	    d[ii] = d_tmp1;
	    s[ii] = (s_tmp0 + ((d_tmp0 + d_tmp1) >> 3));
	    d_tmp0 = d_tmp1;
	    s_tmp0 = s_tmp1;
	}
	d[mid] = ((d[mid] - s[mid]) << 1);
	s[mid] = (s[mid] + ((d[mid - 1] + d[mid]) >> 3));
	for (ii = 0; ii <= mid; ii++) {
	    /* k-s: no */
	    /* not working correctly individually, don't select it */
	    x[ii] = s[ii];
	    x[(ii + mid) + 1] = d[ii];
	}
    }
}

void forward_wavelet2(int nt, int *data) {
    /* inout int_data */
    int i;
    int d_tmp0, d_tmp1, s_tmp0, s_tmp1;
    int mid, ii;
    int *x;
    int s[256], d[256];
    for (i = 0; i < nt; i++) {
	/* fcdf22(&int_data[i],nt,COL); inline */
	mid = ((nt >> 1) - 1);
	x = (&data[i]);
	s[0] = x[0];
	d[0] = x[128];
	s[1] = x[128 << 1];
	s[mid] = x[(128 << 1) * mid];
	d[mid] = x[((128 << 1) * mid) + 128];
	d[0] = (((d[0] << 1) - s[0]) - s[1]);
	s[0] = (s[0] + (d[0] >> 2));
	d_tmp0 = d[0];
	s_tmp0 = s[1];
	for (ii = 1; ii < mid; ii++) {
	    /* k-s: yes */
	    s_tmp1 = x[(2 * 128) * (ii + 1)];
	    d_tmp1 = (((x[((2 * 128) * ii) + 128] << 1) - s_tmp0) - s_tmp1);
	    d[ii] = d_tmp1;
	    s[ii] = (s_tmp0 + ((d_tmp0 + d_tmp1) >> 3));
	    d_tmp0 = d_tmp1;
	    s_tmp0 = s_tmp1;
	}
	d[mid] = ((d[mid] << 1) - (s[mid] << 1));
	s[mid] = (s[mid] + ((d[mid - 1] + d[mid]) >> 3));
	for (ii = 0; ii <= mid; ii++) {
	    /* k-s: no */
	    /* not working correctly individually, don't select it */
	    x[ii * 128] = s[ii];
	    x[((ii + mid) + 1) * 128] = d[ii];
	}
    }
}

void forward_wavelet2_task(int nt, int *data) {
    /* inout int_data */
    int i;
    int d_tmp0, d_tmp1, s_tmp0, s_tmp1;
    int mid, ii;
    int *x;
    int s[256], d[256];
    for (i = 0; i < (0 + (((nt - 0) * 1l) / 3l)); i++) {
	/* fcdf22(&int_data[i],nt,COL); inline */
	mid = ((nt >> 1) - 1);
	x = (&data[i]);
	s[0] = x[0];
	d[0] = x[128];
	s[1] = x[128 << 1];
	s[mid] = x[(128 << 1) * mid];
	d[mid] = x[((128 << 1) * mid) + 128];
	d[0] = (((d[0] << 1) - s[0]) - s[1]);
	s[0] = (s[0] + (d[0] >> 2));
	d_tmp0 = d[0];
	s_tmp0 = s[1];
	for (ii = 1; ii < mid; ii++) {
	    /* k-s: yes */
	    s_tmp1 = x[(2 * 128) * (ii + 1)];
	    d_tmp1 = (((x[((2 * 128) * ii) + 128] << 1) - s_tmp0) - s_tmp1);
	    d[ii] = d_tmp1;
	    s[ii] = (s_tmp0 + ((d_tmp0 + d_tmp1) >> 3));
	    d_tmp0 = d_tmp1;
	    s_tmp0 = s_tmp1;
	}
	d[mid] = ((d[mid] << 1) - (s[mid] << 1));
	s[mid] = (s[mid] + ((d[mid - 1] + d[mid]) >> 3));
	for (ii = 0; ii <= mid; ii++) {
	    /* k-s: no */
	    /* not working correctly individually, don't select it */
	    x[ii * 128] = s[ii];
	    x[((ii + mid) + 1) * 128] = d[ii];
	}
    }
#pragma microStreams task
    for (; i < (0 + (((nt - 0) * 2l) / 3l)); i++) {
	/* fcdf22(&int_data[i],nt,COL); inline */
	mid = ((nt >> 1) - 1);
	x = (&data[i]);
	s[0] = x[0];
	d[0] = x[128];
	s[1] = x[128 << 1];
	s[mid] = x[(128 << 1) * mid];
	d[mid] = x[((128 << 1) * mid) + 128];
	d[0] = (((d[0] << 1) - s[0]) - s[1]);
	s[0] = (s[0] + (d[0] >> 2));
	d_tmp0 = d[0];
	s_tmp0 = s[1];
	for (ii = 1; ii < mid; ii++) {
	    /* k-s: yes */
	    s_tmp1 = x[(2 * 128) * (ii + 1)];
	    d_tmp1 = (((x[((2 * 128) * ii) + 128] << 1) - s_tmp0) - s_tmp1);
	    d[ii] = d_tmp1;
	    s[ii] = (s_tmp0 + ((d_tmp0 + d_tmp1) >> 3));
	    d_tmp0 = d_tmp1;
	    s_tmp0 = s_tmp1;
	}
	d[mid] = ((d[mid] << 1) - (s[mid] << 1));
	s[mid] = (s[mid] + ((d[mid - 1] + d[mid]) >> 3));
	for (ii = 0; ii <= mid; ii++) {
	    /* k-s: no */
	    /* not working correctly individually, don't select it */
	    x[ii * 128] = s[ii];
	    x[((ii + mid) + 1) * 128] = d[ii];
	}
    }
#pragma microStreams task
    for (; i < (0 + (((nt - 0) * 3l) / 3l)); i++) {
	/* fcdf22(&int_data[i],nt,COL); inline */
	mid = ((nt >> 1) - 1);
	x = (&data[i]);
	s[0] = x[0];
	d[0] = x[128];
	s[1] = x[128 << 1];
	s[mid] = x[(128 << 1) * mid];
	d[mid] = x[((128 << 1) * mid) + 128];
	d[0] = (((d[0] << 1) - s[0]) - s[1]);
	s[0] = (s[0] + (d[0] >> 2));
	d_tmp0 = d[0];
	s_tmp0 = s[1];
	for (ii = 1; ii < mid; ii++) {
	    /* k-s: yes */
	    s_tmp1 = x[(2 * 128) * (ii + 1)];
	    d_tmp1 = (((x[((2 * 128) * ii) + 128] << 1) - s_tmp0) - s_tmp1);
	    d[ii] = d_tmp1;
	    s[ii] = (s_tmp0 + ((d_tmp0 + d_tmp1) >> 3));
	    d_tmp0 = d_tmp1;
	    s_tmp0 = s_tmp1;
	}
	d[mid] = ((d[mid] << 1) - (s[mid] << 1));
	s[mid] = (s[mid] + ((d[mid - 1] + d[mid]) >> 3));
	for (ii = 0; ii <= mid; ii++) {
	    /* k-s: no */
	    /* not working correctly individually, don't select it */
	    x[ii * 128] = s[ii];
	    x[((ii + mid) + 1) * 128] = d[ii];
	}
    }
}
