/*
   _______________________________________________________________________________

   Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
   Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
   (http://www.rs.tu-darmstadt.de) All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement: "This product includes
   software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
   its contributors."

   4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
   CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   _______________________________________________________________________________
   *//*****************************************************************************/
/** \file main.c
 ** Main component of the µStream Application.
 **
 ** History:
 **   - 2015-05-04 Version 1.0 Jan Weber: Base Version.
 **
 *****************************************************************************/
/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/

#include "base_types.h"
#include "inBytes.h"
#include <stdio.h>
#include <evaluation.h>
#include <peripherals.h>

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/

/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/

#define CORE_ID 0
#define TASK_ID 0
#define IS_BASETASK true

/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/

/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/


/*****************************************************************************/
/* Local variables without initialization definitions ('static')             */
/*****************************************************************************/


/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/

int readAdaptiveInput(int inByteAdaptive, int totalInBytes,
		      int samplesForAdaptivePass, long *correlationSum,
		      int *correlationHistory);

/*****************************************************************************/
/* Function implementation - global ('extern') and local ('static')          */
/*****************************************************************************/

int readAdaptiveInput(int inByteAdaptive, int totalInBytes,
		      int samplesForAdaptivePass, long *correlationSum,
		      int *correlationHistory) {
    while ((samplesForAdaptivePass < 2048) && (inByteAdaptive < totalInBytes)) {
	/* read next sample */
	int sample = 0;
	{
	    int i = 0;
	    for (; i < 2; i++) {
		sample |=
		    ((inBytes[(inByteAdaptive++)] & 255) <<
		     ((1 ? ((2 - i) - 1) : i) * 8));
	    }
	}
	if ((1 && (sample >= (1 << ((2 * 8) - 1))))) {
	    sample -= (1 << (2 * 8));
	}
	/* reset accumulators */
	if ((samplesForAdaptivePass == 0)) {
	    {
		int i = 0;
		for (; i <= 9; i++) {
		    correlationSum[i] = 0;
		}
	    }
	}
	/* buffer sample for correlation */
	{
	    int i = 9;
	    for (; i > 0; i--) {
		correlationHistory[i] = correlationHistory[(i - 1)];
	    }
	}
	correlationHistory[0] = sample;
	/* calculate (non normalized) autocorrelation values */
	{
	    int i = 0;
	    for (; (i <= samplesForAdaptivePass) && (i <= 9); i++) {
		/* Use 18bit 18bit -> 36 bit multiplication */
		long delta = ((long) sample) * correlationHistory[i];
		correlationSum[i] += delta;
	    }
	}
	samplesForAdaptivePass++;
    }
    return samplesForAdaptivePass;
}




/*****************************************************************************/
/* Global variables definitions and initialisation                           */
/*****************************************************************************/

FILE *stdout = &UART_LIGHT_0_FILE;
FILE *stdin = &UART_LIGHT_0_FILE;


/*****************************************************************************/
/* main                                                                      */
/*****************************************************************************/

void main() {

    evaluation_initBase(&__GLOBAL_MEM_HEAP_END);

    //local variables
    int correlationHistory[(9 + 1)];
    long correlationSum[(9 + 1)];
    //local variables originally having initializers received from interconnect peripherals
    evaluation_startMeasurement(RUNTIME);
    {
	/* debugging_initialize(); */
	/* stdio_uart_light_open(&UART_LIGHT_0); */
	/*      test(); */
	/* printf("algo start \n"); */
	int inByte = 0;
	/* number of read input bytes */
	int inByteAdaptive = 0;
	int outByte = 0;
	/* number of written output bytes */
	/* printf("outBytesAdaptivelength = %d\n", outBytesAdaptivelength); */
	/* last samples used for prediction */
	int totalInBytes = 4096;
	/* printf("totalInBytes = %d\n", totalInBytes); */
	int totalBlockRemainder = totalInBytes % 2048;
	/* printf("totalBlockRemainder = %d\n", totalBlockRemainder); */
	int totalBlocks = totalInBytes / 2048;
	/* printf("totalBlocks = %d\n", totalBlocks); */
	/* printf("totalBlockRemainder = %d\n", totalBlockRemainder); */
	if ((totalBlockRemainder > 0)) {
	    totalBlocks++;
	}
	/* printf("totalBlocks = %d\n", totalBlocks); */
	/* two different histories are required predict first sample of new block from last samples of old block after the */
	/* new block was passed ones for autocorrelation */
	int blocks = 0;
	/*      for (blocks = 0; blocks < totalBlocks; blocks = blocks + 2) { */
	/* printf("blocks = %d\n", blocks); */
	char lastBlock __attribute__ ((aligned)) = blocks < totalBlocks;
	/* adapt coefficients to current block */
	long maxAbs = 0;
	/* used for numerical stability */
	/* autocorrelation values */
	/* quadratic matrix of linear equation system */
	/* prediction coefficients */
	int samplesForAdaptivePass = 0;
	/* accumulators for autocorrelation values */
	/* last samples used for autocorrelation */
	samplesForAdaptivePass =
	    readAdaptiveInput(inByteAdaptive, totalInBytes,
			      samplesForAdaptivePass, correlationSum,
			      correlationHistory);
	/* end of block reached? */

	evaluation_stopMeasurement(RUNTIME);
	evaluation_startMeasurement(SEND_WAIT);
	sender_wait_free(&CORE_CONNECTOR_MASTER_0.status,
			 &CORE_CONNECTOR_MASTER_0.msg_size, 1);
	evaluation_stopMeasurement(SEND_WAIT);
	evaluation_startMeasurement(SEND);
	// sends the variable correlationSum to another core.
	core_connector_master_send_data(&CORE_CONNECTOR_MASTER_0,
					&correlationSum,
					(sizeof correlationSum) / 2 +
					(sizeof correlationSum) % 2);
	// sends the variable inByte to another core.
	core_connector_master_send_data(&CORE_CONNECTOR_MASTER_0, &inByte,
					(sizeof inByte) / 2 +
					(sizeof inByte) % 2);
	// sends the variable lastBlock to another core.
	core_connector_master_send_data(&CORE_CONNECTOR_MASTER_0, &lastBlock,
					(sizeof lastBlock) / 2 +
					(sizeof lastBlock) % 2);
	// sends the variable maxAbs to another core.
	core_connector_master_send_data(&CORE_CONNECTOR_MASTER_0, &maxAbs,
					(sizeof maxAbs) / 2 +
					(sizeof maxAbs) % 2);
	// sends the variable outByte to another core.
	core_connector_master_send_data(&CORE_CONNECTOR_MASTER_0, &outByte,
					(sizeof outByte) / 2 +
					(sizeof outByte) % 2);
	// sends the variable samplesForAdaptivePass to another core.
	core_connector_master_send_data(&CORE_CONNECTOR_MASTER_0,
					&samplesForAdaptivePass,
					(sizeof samplesForAdaptivePass) / 2 +
					(sizeof samplesForAdaptivePass) % 2);
	evaluation_stopMeasurement(SEND);
	evaluation_startMeasurement(RUNTIME);

    }
    evaluation_stopMeasurement(RUNTIME);

    evaluation_sendResults();
}
