/*
   _______________________________________________________________________________

   Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
   Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
   (http://www.rs.tu-darmstadt.de) All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement: "This product includes
   software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
   its contributors."

   4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
   CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   _______________________________________________________________________________
   *//*****************************************************************************/
/** \file main.c
 ** Main component of the µStream Application.
 **
 ** History:
 **   - 2015-05-04 Version 1.0 Jan Weber: Base Version.
 **
 *****************************************************************************/
/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/

#include "base_types.h"
#include "inBytes.h"
#include <stdio.h>
#include <evaluation.h>
#include <peripherals.h>

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/

/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/

#define CORE_ID 3
#define TASK_ID 3
#define IS_BASETASK false

/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/

/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/


/*****************************************************************************/
/* Local variables without initialization definitions ('static')             */
/*****************************************************************************/


/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/

long readSample(signed char *outBytes, int *inByte, int *outByte);
int predict(int *predictionHistory, long *coefficientsNew);
void prepareRice(long sample, int prediction, int *error, int *ones);
void writeOnes(signed char *outBytes, int ones, int *outByte, int *bits,
	       char *buffer);
signed char signExtend8to9(signed char x);
signed int signExtend8toInt(signed char x);
void writeZeros(signed char *outBytes, int *outByte, int *bits, char *buffer);
void writeResidualBlock(signed char *outBytes, int error, int *outByte,
			int *bits, char *buffer);

/*****************************************************************************/
/* Function implementation - global ('extern') and local ('static')          */
/*****************************************************************************/

long readSample(signed char *outBytes, int *inByte, int *outByte) {
    long sample = 0;
    {
	int i = 0;
	for (; i < 2; i++) {
	    if (((*inByte) < (9 * 2))) {
		outBytes[((*outByte)++)] = inBytes[(*inByte)];
	    }
	    sample |=
		((inBytes[((*inByte)++)] & 255) <<
		 ((1 ? ((2 - i) - 1) : i) * 8));
	}
    }
    long signCompare = 1 << ((2 * 8) - 1);
    if ((1 && (sample >= signCompare))) {
	sample -= (1 << (2 * 8));
    }
    return sample;
}

int predict(int *predictionHistory, long *coefficientsNew) {
    int prediction;
    if ((9 == 0)) {
	prediction = predictionHistory[0];
    }
    else {
	prediction = 0;
	{
	    int i = 0;
	    for (; i < 9; i++) {
		prediction +=
		    ((predictionHistory[i] * coefficientsNew[i]) >> 12);
	    }
	}
    }
    return prediction;
}

void prepareRice(long sample, int prediction, int *error, int *ones) {
    (*error) = (sample - prediction);
    /* prediction error */
    (*error) = (((*error) >= 0) ? (2 * (*error)) : (((-2) * (*error)) - 1));
    /* code spreading to positive values */
    (*ones) = ((*error) >> 4);
    /* run length of unary code */
    (*error) = ((*error) & ((1 << 4) - 1));
    /* residual for block code */
}

void writeOnes(signed char *outBytes, int ones, int *outByte, int *bits,
	       char *buffer) {
    while (ones != 0) {
	if (((ones + (*bits)) >= 8)) {
	    (*bits) = (8 - (*bits));
	    ones -= (*bits);
	    outBytes[((*outByte)++)] =
		signExtend8to9(((signed char) (((*buffer) << (*bits)) |
					       ((1 << (*bits)) - 1))));
	    (*bits) = 0;
	}
	else {
	    (*bits) += ones;
	    char bufferShifted = (*buffer) << ones;
	    char onesShifted = (1 << ones) - 1;
	    (*buffer) = ((bufferShifted | onesShifted) & 255);
	    ones = 0;
	}
    }
}

signed char signExtend8to9(signed char x) {
    return ((signed char) signExtend8toInt(x));
}

signed int signExtend8toInt(signed char x) {
    signed int r;
    /* Sign extend from 8 bits */
  __asm__("sigex %0, 8": "=r"(r):"0"(x));
    return r;
}

void writeZeros(signed char *outBytes, int *outByte, int *bits, char *buffer) {
    if (((*bits) == 7)) {
	outBytes[((*outByte)++)] = signExtend8to9(((*buffer) << 1));
	(*bits) = 0;
    }
    else {
	(*buffer) <<= 1;
	(*buffer) &= 255;
	(*bits) += 1;
    }
}

void writeResidualBlock(signed char *outBytes, int error, int *outByte,
			int *bits, char *buffer) {
    int i = 4;
    while (i != 0) {
	if (((i + (*bits)) >= 8)) {
	    (*bits) = (8 - (*bits));
	    i -= (*bits);
	    outBytes[((*outByte)++)] =
		signExtend8to9((((*buffer) << (*bits)) |
				(((1 << (*bits)) - 1) & (error >> i))));
	    (*bits) = 0;
	}
	else {
	    (*bits) += i;
	    (*buffer) =
		((char) (((*buffer) << i) | (((1 << i) - 1) & error)));
	    (*buffer) &= 255;
	    i = 0;
	}
    }
}




/*****************************************************************************/
/* Global variables definitions and initialisation                           */
/*****************************************************************************/

FILE *stdout = &UART_LIGHT_0_FILE;
FILE *stdin = &UART_LIGHT_0_FILE;


/*****************************************************************************/
/* main                                                                      */
/*****************************************************************************/

void main() {

    evaluation_initChild(&__GLOBAL_MEM_HEAP_END);

    while (TRUE) {
	//local variables
	//local variables originally having initializers received from interconnect peripherals
	int bits;
	char buffer __attribute__ ((aligned));
	long coefficientsNew[9];
	int inByte;
	char lastBlock __attribute__ ((aligned));
	int outByte;
	signed char outBytes[1510] __attribute__ ((aligned));
	int predictionHistory[9];
	evaluation_startMeasurement(RECEIVE_WAIT);
	receiver_wait_data(&CORE_CONNECTOR_SLAVE_0.status,
			   &CORE_CONNECTOR_SLAVE_0.msg_size, 1);
	evaluation_stopMeasurement(RECEIVE_WAIT);
	evaluation_startMeasurement(RECEIVE);
	// receives the variable bits from another core.
	core_connector_slave_receive_data(&CORE_CONNECTOR_SLAVE_0, &bits,
					  (sizeof bits) / 2 +
					  (sizeof bits) % 2);
	// receives the variable buffer from another core.
	core_connector_slave_receive_data(&CORE_CONNECTOR_SLAVE_0, &buffer,
					  (sizeof buffer) / 2 +
					  (sizeof buffer) % 2);
	// receives the variable coefficientsNew from another core.
	core_connector_slave_receive_data(&CORE_CONNECTOR_SLAVE_0,
					  &coefficientsNew,
					  (sizeof coefficientsNew) / 2 +
					  (sizeof coefficientsNew) % 2);
	// receives the variable inByte from another core.
	core_connector_slave_receive_data(&CORE_CONNECTOR_SLAVE_0, &inByte,
					  (sizeof inByte) / 2 +
					  (sizeof inByte) % 2);
	// receives the variable lastBlock from another core.
	core_connector_slave_receive_data(&CORE_CONNECTOR_SLAVE_0, &lastBlock,
					  (sizeof lastBlock) / 2 +
					  (sizeof lastBlock) % 2);
	// receives the variable outByte from another core.
	core_connector_slave_receive_data(&CORE_CONNECTOR_SLAVE_0, &outByte,
					  (sizeof outByte) / 2 +
					  (sizeof outByte) % 2);
	// receives the variable outBytes from another core.
	core_connector_slave_receive_data(&CORE_CONNECTOR_SLAVE_0, &outBytes,
					  (sizeof outBytes) / 2 +
					  (sizeof outBytes) % 2);
	// receives the variable predictionHistory from another core.
	core_connector_slave_receive_data(&CORE_CONNECTOR_SLAVE_0,
					  &predictionHistory,
					  (sizeof predictionHistory) / 2 +
					  (sizeof predictionHistory) % 2);
	evaluation_stopMeasurement(RECEIVE);
	evaluation_startMeasurement(RUNTIME);

	{
	    int samples = 2048 / 2;
	    for (; samples < 2048; samples++) {
		/* read next sample */
		long sample = readSample(outBytes, (&inByte), (&outByte));
		if ((inByte > (9 * 2))) {
		    /* staticadaptive prediction */
		    int prediction =
			predict(predictionHistory, coefficientsNew);
		    /* prepare output of rice coder */
		    int error = 0;
		    int ones = 0;
		    prepareRice(sample, prediction, (&error), (&ones));
		    /* write ones */
		    writeOnes(outBytes, ones, (&outByte), (&bits), (&buffer));
		    /* write zero (to terminate unary code) */
		    writeZeros(outBytes, (&outByte), (&bits), (&buffer));
		    /* write residual block code */
		    writeResidualBlock(outBytes, error, (&outByte), (&bits),
				       (&buffer));
		}
		/* buffer sample for next prediction */
		{
		    int i = 9 - 1;
		    for (; i > 0; i--) {
			predictionHistory[i] = predictionHistory[(i - 1)];
		    }
		}
		predictionHistory[0] = sample;
	    }
	}
	/* 1-padding to full byte => avoids misinterpretation of padding as additional sample (unterminated unary code) */
	if ((lastBlock && (bits > 0))) {
	    outBytes[(outByte++)] =
		((char) ((buffer << (8 - bits)) | ((1 << (8 - bits)) - 1)));
	}
	/* } */
	printf("Compressed size: %d\n", outByte);
	/* return outByte; */
	/* } */
	printf("EOF\n");

	evaluation_stopMeasurement(RUNTIME);
	evaluation_sendResults();
    }

}
