package de.tu_darmstadt.rs.spartanmc.jconfig.gradle

import de.tu_darmstadt.rs.spartanmc.devxml.Library
import de.tu_darmstadt.rs.spartanmc.jconfig.IApplicationConfiguration
import de.tu_darmstadt.rs.spartanmc.jconfig.model.LibJConfigProject
import de.tu_darmstadt.rs.spartanmc.jconfig.model.api.IDocumentHandler
import de.tu_darmstadt.rs.spartanmc.jconfig.model.generation.DocumentBuilder
import de.tu_darmstadt.rs.spartanmc.jconfig.model.generation.IBuildManager
import de.tu_darmstadt.rs.spartanmc.jconfig.model.jaxb.jc3.ConfigurationDocument
import org.gradle.api.DefaultTask
import org.gradle.api.tasks.InputFile
import org.gradle.api.tasks.InputFiles
import org.gradle.api.tasks.OutputFiles
import org.gradle.api.tasks.TaskAction
import org.gradle.api.tasks.incremental.IncrementalTaskInputs

import javax.xml.bind.JAXBException
import java.util.stream.Collectors

public class LibJConfigTask extends DefaultTask implements IApplicationConfiguration, IBuildManager {
	@InputFiles
	List<File> inputFiles;

//	@Input
//	List<String> inputProps;

	@OutputFiles
	List<File> outputFiles;

	@InputFile
	File jcConfig;
	File jcOutput;

	private final Library library;
	private DocumentBuilder.IOutputGroup group;

	public LibJConfigTask() throws IOException, JAXBException {
		library = new Library.Builder().addDefaultSearchpaths(getProject().file("../../").toPath()).build();
	}

	public void configure() {
		logger.info("Configuring dependencies of Verilog output")
		LibJConfigProject prj = new LibJConfigProject(jcOutput.toPath(), this);
		IDocumentHandler documentHandler = prj.getDocumentHandler();
		documentHandler.open(jcConfig.toPath());
		DocumentBuilder builder = documentHandler.getDocumentBuilder();
		group = builder.getOutputGroups(((ConfigurationDocument) documentHandler.getDocument())).get(0);
		inputFiles = group.getInputFiles().stream().map({it.toFile()}).collect(Collectors.toList());
		outputFiles = group.getOutputFiles().stream().map({it.toFile()}).collect(Collectors.toList());
//		inputProps = group.getInputProperties().entrySet().stream().map({it.getKey()+':'+it.getValue()}).sorted(String.CASE_INSENSITIVE_ORDER).collect(Collectors.toList());
//		println inputProps
	}

	@TaskAction
	public void execute(IncrementalTaskInputs inputs) throws IOException {
		boolean build = false;

		if (!inputs.isIncremental()) {
			build = true;
		} else {

			inputs.outOfDate({ change ->
				logger.info("out of date: ${change.file.name}")
				build = true;
			});

			inputs.removed({ change ->
				logger.info("removed: ${change.file.name}")
				build = true;
			});
		}

		if (build) {
			group.build()
		}
	}

	@Override
	public String getApplicationVersion() {
		return "gradle";
	}

	@Override
	public Library getLibrary() {
		return library;
	}

	@Override
	public void addLibraryReadyListener(IApplicationConfiguration.ILibraryReadyListener iLibraryReadyListener) {

	}

	@Override
	public boolean isLibraryReady() {
		return true;
	}

	@Override
	public boolean buildRequired(ConfigurationDocument configurationDocument) {
		return false;
	}

	@Override
	public boolean buildAll(ConfigurationDocument configurationDocument) {
		return false;
	}
}