#!/bin/bash

JENKINS_PREFIX="https://jenkins.rs.e-technik.tu-darmstadt.de/job/SpartanMC/job/"
JENKINS_POSTFIX="/lastSuccessfulBuild/artifact/gcc.tgz"
INSTALL_DIR="$SPARTANMC_TOOLS"
SPARTANMC_GCC="$INSTALL_DIR/bin/spartanmc-gcc"
RSYNC="rsync -abu "
TMP_FOLDER=`mktemp -d` 


progressfilt ()
{
    local flag=false c count cr=$'\r' nl=$'\n'
    while IFS='' read -d '' -rn 1 c
    do
        if $flag
        then
            printf '%c' "$c"
        else
            if [[ $c != $cr && $c != $nl ]]
            then
                count=0
            else
                ((count++))
                if ((count > 1))
                then
                    flag=true
                fi
            fi
        fi
    done
}

if [ -z "$INSTALL_DIR" ]; then
	printf "The variable SPARTANMC_TOOLS is unset!\n"
	exit 1
fi 

if [[ $SPARTANMC_TOOLS == ${SPARTANMC_ROOT}/* ]]; then
	printf "SPARTANMC_TOOLS is a subdirectory of SPARTANMC_ROOT ... this is not valid! Place SPARTANMC_TOOLS outside SPARTANMC_ROOT.\n"
	exit 1
fi

printf "\n\n>>>>>Trying to install GCC and Binutils builds from Jenkins to $INSTALL_DIR <<<<<\n\n"

if test -d $INSTALL_DIR; then
    if [ -f $SPARTANMC_GCC ]; then
        printf "GCC and Binutils are already installed at $INSTALL_DIR \n What shall be done? \n (d)elete existing directory and install \n (a)bort installation\n"
        read action
        case "$action" in
            d)      printf " -> Remove install directory\n"
                    ;;
            *)      printf " -> Aborting installation\n"
                    exit 0
                    ;;
        esac
    fi                  
fi

printf "Please enter our LDAP username for logging into Jenkins\n"
read USER
printf "Now the password\n"
read -s PASSWORD

printf "GCC branch to use: (default: master)\n"
read GCC_BRANCH

if [ -z $GCC_BRANCH ]; then
    GCC_BRANCH="master"
fi
GCC_URL="${JENKINS_PREFIX}Gcc/job/${GCC_BRANCH}${JENKINS_POSTFIX}"

printf "Starting installation: \n"

WGET="wget --progress=bar:force --auth-no-challenge --http-user=$USER --http-password=$PASSWORD -O $TMP_FOLDER/" 
printf "Loading GCC from Jenkins...\n"
eval "${WGET}gcc.tgz $GCC_URL 2>&1 | progressfilt"

GCC_ZIP="$TMP_FOLDER/gcc.tgz"

FILESIZE=$(stat -c%s "$GCC_ZIP")
if [ ! -e $GCC_ZIP -o "$FILESIZE" -eq "0" ]; then
	printf "GCC could not be downloaded. Check username/password or branch name\n"
	exit 1
fi

printf "Extracting..."
mkdir -p $TMP_FOLDER/gcc
tar -xzf $GCC_ZIP -C $TMP_FOLDER/gcc
printf "done.\n"

printf "Copy to ${INSTALL_DIR}..."
rm -rf $INSTALL_DIR/*
mkdir -p $INSTALL_DIR
$RSYNC $TMP_FOLDER/gcc/ $INSTALL_DIR
printf "done.\n"

printf "Clean temp files..."
rm -rf $TMP_FOLDER
printf "done.\n"

printf "Check if the new files are on the PATH variable..."
WHICHGCC=$(which spartanmc-gcc)
if [ -z "$WHICHGCC" ]; then
	printf "\n $INSTALL_DIR/bin seems not to be on the PATH variable. Please add this folder to PATH.\n"
elif ! [ "$SPARTANMC_GCC" -ef "$WHICHGCC" ]; then
	printf "\nThe location where the tools were copied to ($SPARTANMC_GCC) is not the location of the tools that are referred to in the PATH variable ($WHICHGCC)\n"
else
	printf "done.\n"
fi
