/**
 * _______________________________________________________________________________
 *
 * Copyright (c) 2012 TU Dresden, Chair for Embedded Systems
 * Copyright (c) 2013-2016 TU Darmstadt, Computer Systems Group
 * (http://www.rs.tu-darmstadt.de) All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement: "This product includes
 * software developed by the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group and
 * its contributors."
 *
 * 4. Neither the name of the TU Dresden Chair for Embedded Systems, TU Darmstadt Computer Systems Group nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TU DRESDEN CHAIR FOR EMBEDDED SYSTEMS, TU DARMSTADT COMPUTER SYSTEMS GROUP AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * _______________________________________________________________________________
 */
package libsim;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

public class Payload implements Collection<Object> {

	private Collection<Object> backing_store;
	private Integer sequence;
	private Integer destination;
	private Integer source;

	public Payload() {
		backing_store = new LinkedList<Object>();
		sequence = null;
		destination = null;
		source = null;
	}

	public Payload(Integer seq, Integer dst, Integer src) {
		this();
		source = src;
		destination = dst;
		sequence = seq;
	}

	public void reset() {
		backing_store.clear();
	}

	public boolean add(Object o) {
		return backing_store.add(o);
	}

	public boolean addAll(Collection<? extends Object> c) {
		return backing_store.addAll(c);
	}

	public void clear() {
		backing_store.clear();
	}

	public boolean contains(Object o) {
		return backing_store.contains(o);
	}

	public boolean containsAll(Collection<? extends Object> c) {
		return backing_store.containsAll(c);
	}

	public boolean equals(Object o) {
		return o.hashCode() == hashCode();
	}

	public boolean isEmpty() {
		return backing_store.isEmpty();
	}

	public Iterator<Object> iterator() {
		return backing_store.iterator();
	}

	public boolean remove(Object o) {
		return backing_store.remove(o);
	}

	public boolean removeAll(Collection c) {
		return backing_store.removeAll(c);
	}

	public boolean retainAll(Collection c) {
		return backing_store.retainAll(c);
	}

	public int size() {
		return backing_store.size();
	}

	public Object[] toArray() {
		return backing_store.toArray();
	}

	public <T> T[] toArray(T[] a) {
		return backing_store.toArray(a);
	}

	public void seq(Integer s) {
		if (s == null) {
			throw new NullPointerException();
		}
		sequence = s;
	}

	public void dst(Integer s) {
		if (s == null) {
			throw new NullPointerException();
		}
		destination = s;
	}

	public void src(Integer s) {
		if (s == null) {
			throw new NullPointerException();
		}
		source = s;
	}

	public int seq() {
		return sequence;
	}

	public int dst() {
		return destination;
	}

	public int src() {
		return source;
	}
}
